Package: INLA
Type: Package
Title: Full Bayesian Analysis of Latent Gaussian Models using
        Integrated Nested Laplace Approximations
Authors@R: c(
    person("Havard", "Rue", email = "hrue@r-inla.org", role = c("cre","aut")),
    person("Finn", "Lindgren", email = "finn.lindgren@gmail.com", role = "aut"),
    person("Elias", "Teixeira Krainski", email = "elias@r-inla.org", role = "aut"), 
    person("Sara", "Martino", email = "saramartino0@gmail.com", role = "ctb"),
    person("Haakon", "Bakka", email = "bakka@r-inla.org", role = "ctb"),
    person("Daniel", "Simpson", email = "dp.simpson@gmail.com", role = "ctb"),
    person("Andrea", "Riebler", email = "andrea.riebler@math.ntnu.no", role = "ctb"),
    person("Geir-Arne", "Fuglstad", email = "geirarne.fulgstad@gmail.com", role = "ctb"),
    person("Cristian", "Chiuchiolo", email = "cristian.chiuchiolo@kaust.edu.sa", role = "ctb"))
Description: Full Bayesian analysis of latent Gaussian models using Integrated Nested Laplace Approximaxion. It is a front-end to the inla-program.
Additional_repositories: http:://inlabru-org.r-universe.dev/
Depends: R (>= 3.5), Matrix (>= 1.3-0), sp (>= 1.6-0)
Imports: graphics, grDevices, fmesher (>= 0.0.9.9026), lifecycle,
        methods, parallel, rlang, splines, stats, utils, withr,
        MatrixModels
Suggests: Deriv, Ecdat, HKprocess, Rgraphviz, deldir, devtools,
        doParallel, dplyr, evd, fields, ggplot2, gsl, graph, gridExtra,
        knitr, markdown, MASS, matrixStats, mlogit, mvtnorm, numDeriv,
        pixmap, rgl, rmarkdown, sf, shiny, sn, spdep, splancs, terra,
        tidyterra, testthat, tools, INLAspacetime
VignetteBuilder: knitr
BuildVignettes: true
LazyData: true
License: GPL (>= 2) + file LICENSE
RoxygenNote: 7.3.2
Encoding: UTF-8
StagedInstall: no
Collate: '00000.R' 'INLA-package.R' 'agaussian.R' 'ar.R' 'barrier.R'
        'binary.R' 'binary.install.R' 'cgeneric.R' 'changelog.R'
        'collect.results.R' 'compare.results.R' 'control.R' 'coxph.R'
        'cpo.R' 'create.data.file.R' 'cut.R' 'debug.graph.R'
        'dev.new.R' 'display.matrix.R' 'doc.R' 'dryrun.R'
        'expand.dataframe.R' 'export-class.R' 'external-package.R'
        'f.R' 'fgn.R' 'finn.R' 'fmesher-io.R' 'fmesher-transition.R'
        'fmesher.R' 'graph.convert.R' 'graph.matrix.R' 'group.R'
        'group.cv.R' 'hyper.R' 'hyperpar.R' 'idx.R' 'iidkd.R' 'inla.R'
        'inla.call.R' 'inlaEnv.R' 'interpret.formula.R' 'jmarginal.R'
        'jp.R' 'knmodels.R' 'knmodels.sample.R' 'lattice2node.R'
        'likelihood.R' 'lincomb.R' 'link-functions.R' 'list-models.R'
        'load.R' 'marginal.R' 'mdata.R' 'merge.R' 'mesh.R'
        'mesh.components.R' 'meshassessment.R' 'meshbuilder.R'
        'model-wrapper.R' 'models-generate.R' 'models.R'
        'models_documentation.R' 'nmix.lambda.fitted.R' 'obsolete.R'
        'options.R' 'os.R' 'over_sp_mesh.R' 'pardiso.R' 'pc-alphaw.R'
        'pc-ar.R' 'pc-bym.R' 'pc-cor0.R' 'pc-cor1.R' 'pc-cormat.R'
        'pc-gamma.R' 'pc-gammacount.R' 'pc-gev.R' 'pc-multvar.R'
        'pc-prec.R' 'pc-sn.R' 'pc-t.R' 'plot.R' 'posterior.sample.R'
        'print.R' 'priors.used.R' 'prune.R' 'q.R' 'qinv.R'
        'qreordering.R' 'qsample.R' 'qsolve.R' 'quantile-regression.R'
        'read.graph.R' 'remote.R' 'reorderings.R' 'rerun.R'
        'residuals.R' 'rgeneric.R' 'rprior.R' 'sampler.R' 'sandbox.R'
        'scale.model.R' 'scopy.define.R' 'scopy.summary.R' 'sections.R'
        'set.default.arguments.R' 'sm.R' 'spde.common.R' 'spde1.R'
        'spde2.R' 'spde3.R' 'spmesh.R' 'stack.R' 'startup.R'
        'summary.R' 'surv.R' 'upgrade.R' 'utils.R' 'version.R'
Roxygen: list(markdown = TRUE)
Version: 24.09.29
NeedsCompilation: no
Packaged: 2024-09-29 18:14:30 UTC; hrue
Author: Havard Rue [cre, aut],
  Finn Lindgren [aut],
  Elias Teixeira Krainski [aut],
  Sara Martino [ctb],
  Haakon Bakka [ctb],
  Daniel Simpson [ctb],
  Andrea Riebler [ctb],
  Geir-Arne Fuglstad [ctb],
  Cristian Chiuchiolo [ctb]
Maintainer: Havard Rue <hrue@r-inla.org>
Built: R 4.4.0; ; 2024-09-29 18:22:06 UTC; unix

## Generated by R/Makefile and R/make-make-pages; do not edit by hand


## contributions from NAMESPACE.init
import(stats)
import(graphics)
import(grDevices)
import(utils)
import(methods)
import(Matrix)
import(sp)
import(splines)

# Export all names
# exportPattern(".")


## contributions from NAMESPACE.ours
# *** NAMESPACE contributions from our old system
# The functions to be exported are defined in the R-code with
# lines as:
# ## Export: func1
# ## Export: func1 func2
# ## Export: method1!class
# etc...
export(inla.agaussian)
export(inla.binary.install)
export(inla.cgeneric.define)
export(inla.cgeneric.q)
export(inla.changelog)
export(inla.collect.results)
export(inla.compare.results)
export(inla.coxph)
export(inla.rbind.data.frames)
export(inla.cpo)
export(inla.cut)
export(inla.debug.graph)
export(inla.dev.new)
export(inla.doc)
export(f)
export(inla.fgn)
export(inla.geobugs2inla)
export(inla.graph2matrix)
export(inla.spy)
export(inla.group)
export(inla.hyperpar)
export(inla.idx)
export(inla.iidkd.sample)
export(inla)
export(inla.get.inlaEnv)
S3method(summary, inla.jmarginal)
S3method(print, summary.inla.jmarginal)
S3method(print, inla.jmarginal)
export(inla.rjmarginal)
export(inla.rjmarginal.eval)
export(inla.tjmarginal)
export(inla.1djmarginal)
export(inla.jp.define)
export(inla.lattice2node.mapping)
export(inla.node2lattice.mapping)
export(inla.lattice2node)
export(inla.node2lattice)
export(inla.matrix2vector)
export(inla.vector2matrix)
export(inla.make.lincomb)
export(inla.make.lincombs)
export(inla.link.log)
export(inla.link.invlog)
export(inla.link.neglog)
export(inla.link.invneglog)
export(inla.link.logit)
export(inla.link.invlogit)
export(inla.link.probit)
export(inla.link.invprobit)
export(inla.link.cloglog)
export(inla.link.invcloglog)
export(inla.link.loglog)
export(inla.link.invloglog)
export(inla.link.tan)
export(inla.link.invtan)
export(inla.link.identity)
export(inla.link.invidentity)
export(inla.link.invalid)
export(inla.link.cauchit)
export(inla.link.invcauchit)
export(inla.link.inverse)
export(inla.link.invinverse)
export(inla.link.robit)
export(inla.link.invrobit)
export(inla.link.sn)
export(inla.link.invsn)
export(inla.list.models)
export(inla.load)
export(inla.dmarginal)
export(inla.pmarginal)
export(inla.qmarginal)
export(inla.rmarginal)
export(inla.hpdmarginal)
export(inla.smarginal)
export(inla.emarginal)
export(inla.tmarginal)
export(inla.mmarginal)
export(inla.zmarginal)
export(inla.mdata)
export(is.inla.mdata)
export(as.inla.mdata)
S3method(print, inla.mdata)
S3method(merge, inla)
export(inla.merge)
export(inla.mesh.components)
export(inla.mesh.assessment)
export(inla.models)
export(inla.nmix.lambda.fitted)
export(inla.setOption)
export(inla.getOption)
export(inla.pardiso)
export(inla.pardiso.check)
export(inla.pc.ralphaw)
export(inla.pc.dalphaw)
export(inla.pc.qalphaw)
export(inla.pc.palphaw)
export(inla.pc.ar.rpacf)
export(inla.pc.ar.dpacf)
export(inla.pc.rcor0)
export(inla.pc.dcor0)
export(inla.pc.qcor0)
export(inla.pc.pcor0)
export(inla.pc.rcor1)
export(inla.pc.dcor1)
export(inla.pc.qcor1)
export(inla.pc.pcor1)
export(inla.pc.cormat.dim2p)
export(inla.pc.cormat.p2dim)
export(inla.pc.cormat.theta2R)
export(inla.pc.cormat.R2theta)
export(inla.pc.cormat.r2R)
export(inla.pc.cormat.R2r)
export(inla.pc.cormat.r2theta)
export(inla.pc.cormat.theta2r)
export(inla.pc.cormat.permute)
export(inla.pc.cormat.rtheta)
export(inla.pc.cormat.dtheta)
export(inla.pc.rgamma)
export(inla.pc.dgamma)
export(inla.pc.qgamma)
export(inla.pc.pgamma)
export(inla.pc.rgammacount)
export(inla.pc.dgammacount)
export(inla.pc.qgammacount)
export(inla.pc.pgammacount)
export(inla.pc.rgevtail)
export(inla.pc.dgevtail)
export(inla.pc.qgevtail)
export(inla.pc.pgevtail)
export(inla.pc.multvar.h.default)
export(inla.pc.multvar.simplex.d)
export(inla.pc.multvar.simplex.r)
export(inla.pc.multvar.sphere.d)
export(inla.pc.multvar.sphere.r)
export(inla.pc.rprec)
export(inla.pc.dprec)
export(inla.pc.qprec)
export(inla.pc.pprec)
export(inla.pc.rsn)
export(inla.pc.dsn)
export(inla.pc.qsn)
export(inla.pc.psn)
export(inla.pc.ddof)
S3method(plot, inla)
export(inla.posterior.sample)
export(inla.posterior.sample.eval)
S3method(print, inla)
export(inla.priors.used)
export(inla.prune)
export(inla.qstat)
export(inla.qdel)
export(inla.qget)
export(inla.qnuke)
export(inla.qlog)
S3method(print, inla.q)
S3method(summary, inla.q)
export(inla.qinv)
export(inla.qreordering)
export(inla.qsample)
export(inla.qsolve)
export(inla.read.graph)
export(inla.write.graph)
S3method(summary, inla.graph)
S3method(plot, inla.graph)
S3method(print, inla.graph.summary)
export(inla.ssh.copy.id)
export(inla.remote)
export(inla.reorderings)
export(inla.rerun)
export(inla.ks.plot)
export(inla.rgeneric.ar1.model)
export(inla.rgeneric.ar1.model.opt)
export(inla.rgeneric.iid.model)
export(inla.rgeneric.define)
export(inla.rgeneric.wrapper)
export(inla.rgeneric.q)
export(inla.hyperpar.sample)
export(inla.scale.model)
export(inla.set.control.lincomb.default)
export(inla.set.control.update.default)
export(inla.set.control.group.default)
export(inla.set.control.mix.default)
export(inla.set.control.pom.default)
export(inla.set.control.link.default)
export(inla.set.control.expert.default)
export(inla.set.control.compute.default)
export(inla.set.control.family.default)
export(inla.set.control.fixed.default)
export(inla.set.control.inla.default)
export(inla.set.control.predictor.default)
export(inla.set.control.mode.default)
export(inla.set.control.hazard.default)
export(inla.set.control.bgev.default)
export(inla.set.control.lp.scale.default)
export(inla.set.control.pardiso.default)
export(control.lincomb)
export(control.update)
export(control.group)
export(control.mix)
export(control.pom)
export(control.link)
export(control.expert)
export(control.compute)
export(control.family)
export(control.fixed)
export(control.inla)
export(control.vb)
export(control.predictor)
export(control.mode)
export(control.hazard)
export(control.bgev)
export(control.lp.scale)
export(control.pardiso)
export(control.gcpo)
export(inla.as.sparse)
export(inla.as.dgTMatrix)
S3method(summary, inla)
S3method(print, summary.inla)
export(inla.surv)
export(is.inla.surv)
export(as.inla.surv)
S3method(plot, inla.surv)
S3method(print, inla.surv)
export(inla.upgrade)
export(inla.update)
export(inla.version)


## contributions from NAMESPACE.roxygen
# Generated by roxygen2: do not edit by hand

S3method(as.inla.mesh.segment,Line)
S3method(as.inla.mesh.segment,Lines)
S3method(as.inla.mesh.segment,Polygon)
S3method(as.inla.mesh.segment,Polygons)
S3method(as.inla.mesh.segment,SpatialLines)
S3method(as.inla.mesh.segment,SpatialLinesDataFrame)
S3method(as.inla.mesh.segment,SpatialPoints)
S3method(as.inla.mesh.segment,SpatialPointsDataFrame)
S3method(as.inla.mesh.segment,SpatialPolygons)
S3method(as.inla.mesh.segment,SpatialPolygonsDataFrame)
S3method(extract.groups,inla.mesh.segment)
S3method(inla.diameter,default)
S3method(inla.diameter,inla.mesh)
S3method(inla.diameter,inla.mesh.1d)
S3method(inla.diameter,inla.mesh.lattice)
S3method(inla.diameter,inla.mesh.segment)
S3method(inla.extract.el,data.frame)
S3method(inla.extract.el,list)
S3method(inla.extract.el,matrix)
S3method(inla.mesh.project,inla.mesh)
S3method(inla.mesh.project,inla.mesh.1d)
S3method(inla.mesh.project,inla.mesh.projector)
S3method(inla.mesh.projector,inla.mesh)
S3method(inla.mesh.projector,inla.mesh.1d)
S3method(inla.mesh.segment,default)
S3method(inla.mesh.segment,inla.mesh.segment)
S3method(inla.spTransform,SpatialPoints)
S3method(inla.spTransform,SpatialPointsDataFrame)
S3method(inla.spTransform,default)
S3method(inla.spTransform,inla.mesh)
S3method(inla.spTransform,inla.mesh.lattice)
S3method(inla.spTransform,inla.mesh.segment)
S3method(inla.spde.precision,inla.spde1)
S3method(inla.spde.precision,inla.spde2)
S3method(inla.spde.result,inla.spde1)
S3method(inla.spde.result,inla.spde2)
S3method(inla.spde.sample,default)
S3method(inla.spde.sample,inla.spde)
S3method(lines,inla.mesh.segment)
S3method(plot,CRS)
S3method(plot,inla.CRS)
S3method(plot,inla.mesh)
S3method(plot,inla.trimesh)
S3method(print,summary.inla.mesh)
S3method(summary,inla.mesh)
export(as.inla.mesh.segment)
export(extract.groups)
export(inla.CRS)
export(inla.CRSargs)
export(inla.ar.pacf2acf)
export(inla.ar.pacf2phi)
export(inla.ar.phi2acf)
export(inla.ar.phi2pacf)
export(inla.as.CRS.list)
export(inla.as.CRSargs.list)
export(inla.as.list.CRS)
export(inla.as.list.CRSargs)
export(inla.as.wkt.wkt_tree)
export(inla.as.wkt_tree.wkt)
export(inla.barrier.fem)
export(inla.barrier.pcmatern)
export(inla.barrier.polygon)
export(inla.barrier.q)
export(inla.contour.segment)
export(inla.crs_get_ellipsoid_radius)
export(inla.crs_get_lengthunit)
export(inla.crs_get_wkt)
export(inla.crs_is_geocent)
export(inla.crs_set_ellipsoid_radius)
export(inla.crs_set_lengthunit)
export(inla.dBind)
export(inla.delaunay)
export(inla.diameter)
export(inla.extract.el)
export(inla.fmesher.smorg)
export(inla.generate.colors)
export(inla.has_PROJ6)
export(inla.identical.CRS)
export(inla.knmodels)
export(inla.knmodels.sample)
export(inla.matern.cov)
export(inla.matern.cov.s2)
export(inla.matern.cov.s2.unstable)
export(inla.mesh.1d)
export(inla.mesh.1d.A)
export(inla.mesh.1d.bary)
export(inla.mesh.1d.fem)
export(inla.mesh.2d)
export(inla.mesh.assessment)
export(inla.mesh.basis)
export(inla.mesh.boundary)
export(inla.mesh.components)
export(inla.mesh.create)
export(inla.mesh.create.helper)
export(inla.mesh.deriv)
export(inla.mesh.fem)
export(inla.mesh.interior)
export(inla.mesh.lattice)
export(inla.mesh.map)
export(inla.mesh.map.lim)
export(inla.mesh.project)
export(inla.mesh.projector)
export(inla.mesh.query)
export(inla.mesh.segment)
export(inla.nonconvex.hull)
export(inla.nonconvex.hull.basic)
export(inla.over_sp_mesh)
export(inla.row.kron)
export(inla.simplify.curve)
export(inla.sp2segment)
export(inla.spTransform)
export(inla.sp_get_crs)
export(inla.spde.make.A)
export(inla.spde.make.block.A)
export(inla.spde.make.index)
export(inla.spde.models)
export(inla.spde.precision)
export(inla.spde.result)
export(inla.spde.sample)
export(inla.spde1.create)
export(inla.spde1.imatern)
export(inla.spde1.matern)
export(inla.spde1.matern.osc)
export(inla.spde1.models)
export(inla.spde1.precision)
export(inla.spde1.result)
export(inla.spde2.generic)
export(inla.spde2.matern)
export(inla.spde2.matern.sd.basis)
export(inla.spde2.models)
export(inla.spde2.pcmatern)
export(inla.spde2.precision)
export(inla.spde2.result)
export(inla.spde2.theta2phi0)
export(inla.spde2.theta2phi1)
export(inla.spde2.theta2phi2)
export(inla.stack)
export(inla.stack.A)
export(inla.stack.LHS)
export(inla.stack.RHS)
export(inla.stack.compress)
export(inla.stack.data)
export(inla.stack.index)
export(inla.stack.join)
export(inla.stack.remove.unused)
export(inla.stack.sum)
export(inla.wkt_get_ellipsoid_radius)
export(inla.wkt_get_lengthunit)
export(inla.wkt_is_geocent)
export(inla.wkt_predef)
export(inla.wkt_set_ellipsoid_radius)
export(inla.wkt_set_lengthunit)
export(inla.wkt_tree_get_item)
export(inla.wkt_tree_set_item)
export(inla.wkt_unit_params)
export(meshbuilder)
export(param2.matern.orig)
exportClasses(inla)
importFrom(methods,setOldClass)

rr = scan(system.file("demodata/Leuk.map", package="INLA"))
Leuk.newengland=vector("list",24)
idx=1
for(i in 1:51526) {
    if(is.na(rr[i]))
    {
        Leuk.newengland[[idx]]=matrix(rr[(i+2):((i+2)+2*rr[i+1]-1)],rr[i+1],2,byrow=T)
        idx=idx+1
    }
}

Leuk.names.region=scan(system.file("demodata/Leuk.names.regions", package="INLA"))


Leuk.map <- function(data, cutpoints=seq(min(data),max(data),length=256),
                           autoscale=FALSE, legend=TRUE, append=FALSE,
                           map=Leuk.newengland, id=Leuk.names.region)
{
    ## This is not clever done; I know....
    
    if (autoscale)
    {
        data = (data-min(data))/(max(data)-min(data)+1e-8)
    }
    if(length(unique(data))>1) 
        farben <- gray(1-as.numeric(cut(data,cutpoints,include.lowest=T))/length(cutpoints))
    else  farben <- rep(0,24)	

    xmin <- 1:length(map)
    xmax <- 1:length(map)
    ymin <- 1:length(map)
    ymax <- 1:length(map)

    for(i in 1:length(map))
    {
        xmin[i] <- min(map[[i]][,1],na.rm=T)
        xmax[i] <- max(map[[i]][,1],na.rm=T)
        ymin[i] <- min(map[[i]][,2],na.rm=T)
        ymax[i] <- max(map[[i]][,2],na.rm=T)
    }

    breite <- c(min(xmin),max(xmax))
    hoehe <- c(min(ymin),max(ymax))

    if (!append) plot(breite,hoehe,type="n",axes=F, xlab=" ", ylab=" ")


    for(k in length(map):1)
    {
        polygon(map[[k]][,1],map[[k]][,2],col=farben[id[k]])
    }
    
    if (legend)
    {
        for(i in 1:8)
        {
            polygon(c(0.7,0.7,0.74,0.74),c(0.73+0.03*(i-1),0.73+0.03*i,0.73+0.03*i,0.73+0.03*(i-1)),col=gray((9-i)/9))
        }
        for(i in 2:8)
        {
            text(0.79,0.73+0.03*(i-1),as.character(round(cutpoints[i/8*length(cutpoints)], 2)),cex=.7,col=rgb(0,0,0))
        }
    }
}

## ----setting,include=FALSE----------------------------------------------------
library(knitr);  library(INLA);  library(fields) 
set.seed(123)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
fig.path="figures/SPDE1d/")
knit_hooks$set(small.mar = function(before, options, envir) {
    if (before) par(mar = c(0.1, 0.1, .1, .1))  # smaller margin on top and right
})
knit_hooks$set(mar3311 = function(before, options, envir) {
    if (before) par(mar = c(3, 3, 1, 1), mgp=c(2, 0.7, 0))  # smaller margin on top and right
})

## ----url----------------------------------------------------------------------
u0 <- paste0("http://www.yr.no/place/Norway/S%C3%B8r-Tr%C3%B8ndelag/",
             "Trondheim/Trondheim/detailed_statistics.html")
### browseURL(u0) ### to visit the web page 

## ----read,eval=FALSE----------------------------------------------------------
#  d0 <- readLines(u0) ### read it as text (Done at 26 September 2016)

## ----echo=FALSE,results="hide"------------------------------------------------
load("TRD20160926weather.RData")

## ----index--------------------------------------------------------------------
i <- grep("<tr>", d0) ### index for each table line
i <- i[i>grep("<tbody>", d0)[2]] ### select those for the second table

## ----data---------------------------------------------------------------------
if (Sys.getlocale("LC_TIME")!="C")
    Sys.setlocale("LC_TIME", "C")
dates <- as.Date(d0[i+1], format="      <th>%b %d, %Y</th>")
tmed <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+4])))
(n <- length(dates)) ### it is daily over last 13 months

## ----visualize, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
pd <- pretty(c(dates, max(dates+30)), n=13)
par(mfrow=c(1,1), mar=c(3,3,0.5,2), mgp=c(2,.7,0), las=2, xaxs="i")
plot(dates, tmed, type="l", lwd=2,
     axes=FALSE, xlab="day", ylab="Temperature")
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))
box()
axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, TRUE))

## ----mesh---------------------------------------------------------------------
coo <- as.numeric(dates-min(dates)) ## have numeric temporal coordinates
mesh <- inla.mesh.1d(loc=seq(min(coo), max(coo), by=7), ## knots (7 days)
                 boundary=c("neumann", "neumann"), ## boundaries 
                 degree=2)  ### basis function degree

## ----projector, pars=list(mar=c(0,0,1,0)), fig.width=8, fig.height=2, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
A <- inla.spde.make.A( ## projector creator
    mesh=mesh, ## provide the mesh
    loc=coo) ### locations where to project the field
dim(A) ## an "n" by "m" projector matrix
summary(rowSums(A)) ### each line sums up to one
summary(colSums(A)) ### "how many" observations per knot

## ----SPDE model---------------------------------------------------------------
spde <- inla.spde2.pcmatern( ## model for the precision 
    mesh=mesh, ## mesh supplied
    alpha=2, ## smoothness parameter
    prior.range = c(1, 0.01), ## P(range < 1) = 0.01
    prior.sigma = c(1, 0.5)) ## P(sigma > 1) = 0.5

## ----datastack----------------------------------------------------------------
stk.e <- inla.stack( ## stack creator
  data=list(y=tmed),  ## response
  effects=list(## two elements:
    data.frame(b0=rep(1, n)), ## regressor part
    i=1:spde$n.spde),  ## RF index
  A=list(## projector list of each effect
    1, ## for the covariates
    A), ## for the RF
  tag="est") ## tag

## ----fitting------------------------------------------------------------------
formula <- y ~ 0 + b0 + ## fixed part
  f(i, model=spde) ## RF term
res <- inla( ## main function in INLA package
  formula, ## model formula
  data=inla.stack.data(stk.e), ## dataset
  control.predictor=list( ## inform projector needed in SPDE models
    A = inla.stack.A(stk.e), compute=TRUE)) ## projector from the stack data

## ----fixed--------------------------------------------------------------------
round(res$summary.fixed, 4) 

## ----hypeparsummary-----------------------------------------------------------
round(res$summary.hyperpar, 4)

## ----nugget-------------------------------------------------------------------
m.prec <- res$marginals.hyperpar$"Precision for the Gaussian observations" ## the marginal
post.s2e <- inla.tmarginal(## function to compute a tranformation 
  function(x) sqrt(1/x), ## inverse transformation and square root
  m.prec) ## marginal to be applied

## ----parameters, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
par(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0))
plot(post.s2e, type="l", ylab="Density", 
     xlab=expression(sigma[e]^2))
plot(res$marginals.hyperpar[[2]], type="l",
     xlab="range", ylab="Density")
plot(res$marginals.hyperpar[[3]], ty="l", 
     xlab=expression(sigma[s]), yla="Density")

## ----predicted, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
par(mfrow=c(1,1), mar=c(3,3,0.3,2), mgp=c(2,0.5,0), las=2, xaxs="i")
id <- inla.stack.index(stk.e, tag="est")$data
plot(dates, tmed, type="l", axes=FALSE, ylab="Temperature", lwd=2)
for (j in 3:5)
  lines(dates, res$summary.fitted.values[id, j], lty=2, lwd=2)
box(); axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, T))
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))

## ----other--------------------------------------------------------------------
tmax <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+2])))
tmin <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+3])))
tnormal <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+5])))
prec <- as.numeric(gsub("<td>", "", gsub("mm</td>", "", d0[i+6])))
wind <- as.numeric(gsub("<td>", "", gsub("m/s</td>", "", d0[i+10])))
wmax <- as.numeric(gsub("<td>", "", gsub("m/s</td>", "", d0[i+9])))

## ----visualize3, fig.width=9, fig.height=6, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
par(mfrow=c(3,1), mar=c(0.1,3,0.1,2), mgp=c(2,.7,0), las=2, xaxs="i")
plot(dates, tmed, type="l", ylim=range(tmin, tmax, na.rm=TRUE), 
     axes=FALSE, xlab="", ylab="Temperature", col="green")
lines(dates, tmin, col="blue")
lines(dates, tmax, col="red")
lines(dates, tnormal)
legend(dates[which.min(tmin)], par()$usr[4], c("normal", "max.", "aver.", "min."), 
      col=1:4, lty=1, ncol=2, xjust=0.5, bty="n")
abline(h=5*(-5:6), v=pd, lty=3, col=gray(.5))
box(); axis(2, 5*(-5:6)); axis(4, 5*(-5:6))

plot(dates, prec, type="l", axes=FALSE, xlab="")
box(); axis(2); axis(4)
abline(v=pd, h=10*(1:4), lty=3, col=gray(0.5))

par(mar=c(3, 3, 0.1, 2), new=FALSE)
plot(dates, wind, type="l", axes=FALSE, xlab="",
     ylim=range(wind, wmax, na.rm=TRUE))
lines(dates, wmax, col=2)
box(); axis(2); axis(4)
abline(v=pd, h=5*(1:3), lty=3, col=gray(0.5))
axis(1, pd, months(pd, TRUE))

## ----anomalia, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h"----
par(mar=c(3, 3, 0.1, 2), mgp=c(2,0.7,0), las=2, xaxs="i")
plot(dates, tmed-tnormal, type="l", axes=FALSE, 
     xlab="", ylab="Deviation from normal temperature")
box(); axis(2); axis(4)
abline(h=5*(-2:2), v=pd, lty=2, col=gray(0.5))
axis(1, pd, months(pd, TRUE))

## ----setting,include=FALSE----------------------------------------------------
set.seed(1) 
library(knitr)
knit_hooks$set(pars = function(before, options, envir) {
    if (before) graphics::par(options$pars)
})
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
               fig.path='figures/SPDEhowto-')
library(lattice);   library(gridExtra);  library(INLA);  library(plyr) 
lcall <- inla.getOption('inla.call')
##inla.setOption(inla.call='remote') 


## ----locations,tidy=FALSE-----------------------------------------------------
n <- 200; coo <- matrix(runif(2*n), n) 
s2u <- .5; k <- 10; r <- 2/k ## RF params.
R <- s2u*exp(-k*as.matrix(dist(coo))) 


## ----rmvnorm,tidy=FALSE-------------------------------------------------------
u <- drop(rnorm(n)%*%chol(R)) 


## ----noise,tidy=FALSE---------------------------------------------------------
x <- runif(n);  beta <- 1:2;  s2e <- 0.2
lin.pred <- beta[1] + beta[2]*x + u 
y <- lin.pred + rnorm(n, 0, sqrt(s2e)) 


## ----fmesh, tidy=FALSE, pars=list(mar=c(0,0,0.7,0)), out.width='0.45\\textwidth', fig.align='center'----
mesh <- inla.mesh.2d(coo, cutoff=r/10, 
  max.edge=c(r/4, r/2), offset=c(r/2, r)) 
plot(mesh, asp=1);  points(coo, col='red') 


## ----projector, tidy=FALSE----------------------------------------------------
A <- inla.spde.make.A(mesh=mesh, loc=coo)


## ----spde,tidy=FALSE----------------------------------------------------------
spde <- inla.spde2.pcmatern(
 mesh=mesh, alpha=1.5, 
 prior.range=c(0.2, 0.5),#P(range<0.2)=0.5
 prior.sigma=c(1, 0.5)) ## P(sigma>1)=0.5 


## ----stack-estimation,tidy=FALSE----------------------------------------------
stk.e <- inla.stack(tag='est', ## tag id
  data=list(y=y),  ## response
  A=list(1, A), ## two projection matrices
  effects=list(## two elements: 
    data.frame(b0=1, x=x), ## covariate
    idx.u=1:spde$n.spde)) ## RF index 


## ----fitting------------------------------------------------------------------
pcprec <- list(hyper=list(theta=list(
  prior='pc.prec',param=c(1,.1))))
mf <- y ~ 0 + b0 + x + f(idx.u, model=spde) 
res <- inla(mf, control.family=pcprec,
 data=inla.stack.data(stk.e), ## data 
 control.predictor=list(compute=TRUE, 
  A=inla.stack.A(stk.e)))# full projector


## ----fixed-summary------------------------------------------------------------
round(res$summary.fixed, 4) 


## ----sdlik--------------------------------------------------------------------
pmd.s2e <- inla.tmarginal(function(x) sqrt(1/x), ## inverse and square root
  res$marginals.hyperpar$'Precision for the Gaussian observations')


## ----rfpars, pars=list(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0)), fig.width=7.5, fig.height=2.5, out.width='0.99\\textwidth', out.height='0.33\\textwidth', fig.align='center', fig.pos='h', fig.keep='last'----
plot(pmd.s2e, type='l', ylab='Density',  xlab=expression(sigma[e]))
abline(v=sqrt(s2e), col=2) ## add the 'true' value
plot(res$marginals.hy[[3]], type='l', xlab=expression(sigma[u]), yla='Density')
abline(v=sqrt(s2u), col=2) ## add the 'true' value
plot(res$marginals.hy[[2]], type='l', xlab='range nominal', ylab='Density')
abline(v=r, col=2) ## add the 'true' value


## ----predicts, pars=list(mfrow=c(1,1), mar=c(1.5,3,0.1,0.1), mgp=c(2,0.5,0)), fig.width=10, fig.height=2.5, out.width='0.99\\textwidth', fig.align='center', fig.pos='h', fig.keep='last'----
idx.obs <- inla.stack.index(stk.e, tag='est')$data ## index in the stack 
order.eta <- order(res$summary.fitted.values$mean[idx.obs]) 
plot(y[order.eta], pch=19, ylab='y')
segments(1:n, res$summary.fitted.val$'0.025quant'[idx.obs][order.eta], 
         1:n, res$summary.fitted.val$'0.975quant'[idx.obs][order.eta]) 


## ----project-grid-------------------------------------------------------------
nxy <- c(200, 200)
gproj <- inla.mesh.projector(mesh,  xlim=0:1, ylim=0:1, dims=nxy)
g.mean <- inla.mesh.project(gproj, res$summary.random$idx.u$mean)
g.sd <- inla.mesh.project(gproj, res$summary.random$idx.u$sd)


## ----fgrid, tidy=FALSE, fig.width=9.7, fig.height=4.5, out.width='0.97\\textwidth', out.height='0.45\\textwidth', fig.pos='h'----
library(lattice);     library(gridExtra) 
trellis.par.set(regions=list(col=terrain.colors(16))) 
grid.arrange(levelplot(g.mean, scales=list(draw=F), xlab='', ylab='', main='mean'), 
             levelplot(g.sd, scal=list(draw=F), xla='', yla='', main='sd'), nrow=1)


## ----target-loc---------------------------------------------------------------
tcoo <- rbind(c(0.3,0.9), c(0.5,0.5), c(0.7,0.3))
dim(Ap <- inla.spde.make.A(mesh=mesh, loc=tcoo)) 
x0 <- c(0.5, 0.5, 0.5)


## ----prediction-stack---------------------------------------------------------
stk.pred <- inla.stack(tag='pred', A=list(Ap, 1), data=list(y=NA), ## response as NA
  effects=list(idx.u=1:spde$n.spde, data.frame(x=x0, b0=1))) ## all idx.u
stk.full <- inla.stack(stk.e, stk.pred) ## join the data and prediction scenario
p.res <- inla(mf, data=inla.stack.data(stk.full), ## supply the full data 
  control.predictor=list(compute=TRUE, A=inla.stack.A(stk.full)), ## full 
  control.mode=list(theta=res$mode$theta, restart=FALSE))## use mode already found


## ----prdind-------------------------------------------------------------------
pred.ind <- inla.stack.index(stk.full, tag='pred')$data
round(p.res$summary.fitted.val[pred.ind,], 4)


## ----ppred, tidy=FALSE, fig.width=9.9, fig.height=3.5, out.width='0.99\\textwidth', out.height='0.35\\textwidth', fig.pos='h', fig.keep='last'----
ypost <- p.res$marginals.fitted.values[pred.ind]
names(ypost) <- paste('y', seq_along(ypost), sep='_');     library(plyr) 
xyplot(y~x | .id, ldply(ypost), panel='llines', xlab='y', ylab='Density')


## ----echo=FALSE---------------------------------------------------------------
options(width=43)


## ----marginals-funcs, comment=NA----------------------------------------------
apropos('marginal')


## ----marginals-examples-------------------------------------------------------
## inla.mmarginal(ypost[[1]]) ## mode
inla.qmarginal(c(0.15, 0.7), 
               ypost[[1]]) ## quantiles
inla.pmarginal(inla.qmarginal(
    0.3, ypost[[1]]), ypost[[1]]) 


## ----configs, warning=FALSE, message=FALSE------------------------------------
res$.args$control.compute$config <- TRUE
res <- inla.rerun(res)

## ----echo=FALSE,results=FALSE-------------------------------------------------
inla.setOption(inla.call=lcall) 


## ----samples------------------------------------------------------------------
samples <- inla.posterior.sample(n=2e3, result=res, add.names=FALSE)


## ----xnames-------------------------------------------------------------------
xnames <- rownames(samples[[1]]$latent) ### collect the names
idx <- lapply(c('b0', 'x', 'idx.u'), function(nam) ## for each effect
    which(substr(xnames, 1, nchar(nam))==nam)) ## find the index


## ----samplesmat---------------------------------------------------------------
mat.samples <- sapply(samples, function(spl) 
    c(bo=spl$latent[idx[[1]]], x=spl$latent[idx[[2]]], u=spl$latent[idx[[3]]]))


## ----y3sample-----------------------------------------------------------------
eta3.sample <- as.matrix(cbind(b0=1, x=0.5, u=Ap)%*%mat.samples)


## ----comparey3, fig.width=9.9, fig.height=3.5, out.width='0.99\\textwidth', fig.pos='h', fig.keep='last'----
par(mfrow=c(1,3), mar=c(3,3,1,1), mgp=c(2,1,0))
for (j in 1:3) {
    hist(eta3.sample[j,], freq=FALSE, xlab=paste0('y',j), main='', col=gray(0.7))
    lines(ypost[[j]], lwd=2, col='blue')
}


## ----linpredsample------------------------------------------------------------
eta.g.samp <- as.matrix(cbind(b0=1, x=0.5, 
        s=gproj$proj$A)%*%mat.samples)


## ----meansdgrid,warning=FALSE,message=FALSE-----------------------------------
library(matrixStats)
ss <- list(mean=rowMeans(eta.g.samp), 
 sd=rowSds(eta.g.samp), 
 p3=rowMeans(eta.g.samp>3), 
 q95=rowQuantiles(eta.g.samp, probs=0.95))


## ----predmaps, eval=FALSE-----------------------------------------------------
## library(fields); for (s in c(1,3,4,2))
##   image.plot(list(x=gproj$x, y=gproj$y,
##       z=matrix(ss[[s]], nxy[1])), asp=1,
##     legend.mar=3, main=names(ss)[s])
## points(coo, pch=4, cex=0.5)


## ----predmapsview,echo=FALSE, pars=list(mfrow=c(4,1), mar=c(1.5,2,1,0.5), mgp=c(1,0.5,0)), fig.width=3.75, fig.height=11.7, out.width='0.49\\textwidth', fig.pos='h', fig.align='center'----
library(fields); for (s in c(1,3,4,2)) 
  image.plot(list(x=gproj$x, y=gproj$y, 
      z=matrix(ss[[s]], nxy[1])), asp=1,
    legend.mar=3, main=names(ss)[s]) 
points(coo, pch=4, cex=0.5)

## ----setup, include=FALSE-------------------------------------------
library(INLA)
inla.setOption(smtp='taucs')
inla.setOption(num.threads="1:1")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
knitr::opts_chunk$set(fig.path="figures/old-faq/")
set.seed(123)
source("render_toc.R")

## ----toc, echo=FALSE------------------------------------------------
render_toc("old-faq.Rmd")

## -------------------------------------------------------------------
inla.setOption(inla.mode="classic")

## ----eval=FALSE-----------------------------------------------------
#     expression: statement; statement; ...; return(value)

## ----eval=FALSE-----------------------------------------------------
#     prior.expression = "expression:
#         a = 1;
#         b = 0.1;
#         precision = exp(log_precision);
#         logdens = log(b^a) - lgamma(a)
#            + (a-1)*log_precision - b*precision;
#         log_jacobian = log_precision;
#         return(logdens + log_jacobian);"

## ----eval=FALSE-----------------------------------------------------
#     table: x_1 ... x_n y_1 ... y_n

## -------------------------------------------------------------------
## the loggamma-prior
prior.function = function(log_precision) {
    a = 1;
    b = 0.1;
    precision = exp(log_precision);
    logdens = log(b^a) - lgamma(a) + (a-1)*log_precision - b*precision;
    log_jacobian = log_precision;
    return(logdens + log_jacobian)
}

## implementing the loggamma-prior using "expression:"
prior.expression = "expression:
a = 1;
b = 0.1;
precision = exp(log_precision);
logdens = log(b^a) - lgamma(a)
    + (a-1)*log_precision - b*precision;
log_jacobian = log_precision;
return(logdens + log_jacobian);"

## use suitable support points x
lprec = seq(-10, 10, len=100)
## link the x and corresponding y values into a 
## string which begins with "table:""
prior.table = paste(c("table:", cbind(lprec,
                    prior.function(lprec))), collapse=" ", sep="")

# simulate some data
n = 50
y = rnorm(n)

## use the built-in loggamma prior
r1 = inla(y~1,data = data.frame(y),
control.family = list(hyper = list(prec = list(
                      prior = "loggamma", param = c(1, 0.1)))))

## use the definition using expression
r2 = inla(y~1, data = data.frame(y), 
          control.family = list(hyper = list(
		                        prec = list(prior = prior.expression))))

## use a table of x and y values representing the loggamma prior
r3 = inla(y~1, data = data.frame(y), 
          control.family = list(hyper = list(
		                        prec = list(prior = prior.table))))

print(round(c(r1$mlik[1], r2$mlik[1], r3$mlik[1]), digits=3))

## -------------------------------------------------------------------
n = 100
z = rnorm(n)
eta = 1 + 0.1*z
N = 2

p = inla.link.invlogit(eta)
y = rbinom(n,  size = N, prob = p)
r = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="logit")), 
		control.predictor = list(compute=TRUE))

p = inla.link.invprobit(eta)
y = rbinom(n,  size = N, prob = p)
rr = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="probit")), 
		control.predictor = list(compute=TRUE))

p = inla.link.invcloglog(eta)
y = rbinom(n,  size = N, prob = p)
rrr = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="cloglog")), 
		control.predictor = list(compute=TRUE))        

## -------------------------------------------------------------------
n = 100
n.pred = 10
y = arima.sim(n=n, model=list(ar=0.9))
N = n + n.pred
yy = c(y, rep(NA, n.pred))
i = 1:N
formula = yy ~ f(i, model="ar1")
r = inla(formula, data = data.frame(i,yy),
        control.family = list(initial = 10, fixed=TRUE)) ## no observational noise

## -------------------------------------------------------------------
r$summary.random$i[(n+1):N, c("mean", "sd") ]

## ----plot=TRUE------------------------------------------------------
## simple poisson regression
n = 100
x = sort(runif(n))
eta = 1 + x
lambda = exp(eta)
y = rpois(n, lambda = lambda)

## missing values:
y[1:3] = NA
y[(n-2):n] = NA

## link = 1 is a shortcut for rep(1, n) where n is the appropriate
## length. here '1' is a reference to the first 'family', ie
## 'family[1]'
r = inla(y ~ 1 + x,  family = "poisson",
        data = data.frame(y, x),
        control.predictor = list(link = 1))
plot(exp(eta),type ="l")
points(r$summary.fitted.values$mean, pch=19)

## ----plot=T---------------------------------------------------------
n2 = n %/% 2L
Y = matrix(NA, n, 2)
Y[1:n2, 1] = y[1:n2]
Y[1:n2 + n2, 2] = y[1:n2 + n2]
link = rep(NA, n)
link[which(is.na(y[1:n2]))] = 1
link[n2  + which(is.na(y[1:n2 + n2]))] = 2

r = inla(Y ~ 1 + x,  family = c("poisson", "nbinomial"),
        data = list(Y=Y, x=x),
		control.family= list(list(), list()),
        control.predictor = list(link = link))
plot(exp(eta),type ="l")
points(r$summary.fitted.values$mean, pch=19)

## -------------------------------------------------------------------
## Load the data
data(Tokyo)
summary(Tokyo)

Tokyo$y[300:366] <- NA

## Define the model
formula = y ~ f(time, model="rw2", scale.model=TRUE, 
	            constr=FALSE, cyclic=TRUE,
				hyper = list(prec=list(prior="pc.prec",
	                                   param=c(2,0.01)))) -1 

## We'll get a warning since we have not defined the link argument
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo,
	control.compute = list(return.marginals.predictor = TRUE), 
	control.predictor=list(compute=T))

## need to recompute the fitted values for those with data[i] = NA,
## as the identity link is used.
n = 366
fitted.values.mean = numeric(n)
for(i in 1:366) {
    if (is.na(Tokyo$y[i])) {
        if (FALSE) {
            ## either like this, which is slower
            marg = inla.marginal.transform(
                            function(x) exp(x)/(1+exp(x)),
                            result$marginals.fitted.values[[i]] )
            fitted.values.mean[i] = inla.emarginal(function(x) x, marg)
        } else {
            ## or like this,  which is faster
            fitted.values.mean[i] = inla.emarginal(
                            function(x) exp(x)/(1 +exp(x)),
                            result$marginals.fitted.values[[i]])
        }
    } else {
        fitted.values.mean[i] = result$summary.fitted.values[i,"mean"]
    }
}
plot(fitted.values.mean)

## ----eval=FALSE-----------------------------------------------------
# 4
# 1 2 3 4
# 2 0
# 3 1 1
# 4 1 1

## ----plot=TRUE,fig.width=2,fig.align='center'-----------------------
g = inla.read.graph("4 1 2 3 4 2 0 3 1 1 4 1 1")
if (FALSE && requireNamespace("Rgraphviz") && requireNamespace("graph")) {
  tryCatch(
    plot(g),
    error = function(e) {
      message("Rgraphviz plotting doesn't seem to work on this installation.")
    }
  )
}

## ----eval=FALSE-----------------------------------------------------
# "4 1 2 3 4  2 0 3 1 1 4 1 1"

## ----eval=FALSE-----------------------------------------------------
# formula = y ~ f(idx, model = "besag", graph = "graph.dat")

## ----eval=FALSE-----------------------------------------------------
# formula = y ~ f(idx, model = "besag", graph = "4 1 2 3 4  2 0 3 1 1 4 1 1")

## -------------------------------------------------------------------
C = matrix(c(1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1),4,4)
C

## -------------------------------------------------------------------
C.sparse= inla.as.sparse(C)
C.sparse

## -------------------------------------------------------------------
str(g)

## ----eval=FALSE-----------------------------------------------------
# formula = y ~ x + f(k, model= <some model>)

## ----eval=F---------------------------------------------------------
# control.fixed = list(expand.factor.strategy = "inla")

## -------------------------------------------------------------------
r = inla(y ~ 1 + x, data = data.frame(y=1:3, x=factor(c("a","b","c"))))
as.matrix(r$model.matrix)

## -------------------------------------------------------------------
r = inla(y ~ 1 + x, data = data.frame(y=1:3,x=factor(c("a","b","c"))), 
         control.fixed = list(expand.factor.strategy="inla"))
as.matrix(r$model.matrix)

## -------------------------------------------------------------------
r = inla(y ~ 1 + x, data = data.frame(y=1:3,x=factor(c("a","b",NA))), 
         control.fixed = list(expand.factor.strategy="inla"))
as.matrix(r$model.matrix)

## ----eval=FALSE-----------------------------------------------------
# control.compute=list(cpo=TRUE)

## ----eval=FALSE-----------------------------------------------------
# improved.result = inla.cpo(result)

## ----eval=FALSE-----------------------------------------------------
# r = inla(..., inla.call = "submit")

## ----eval=FALSE-----------------------------------------------------
# inla.qstat()

## ----eval=FALSE-----------------------------------------------------
# r = inla.qget(r)

## -------------------------------------------------------------------
## A simple model
n = 100
a = rnorm(n)
b = rnorm(n)
idx = 1:n

y = rnorm(n) + a + b
formula = y ~ 1 + a + b + f(idx, model="iid")

## assume we want to compute the posterior for
##
##  2 * beta_a + 3 * beta_b + idx[1] - idx[2]
##
## which we specify as follows (and giving it a unique name)

lc1 = inla.make.lincomb(a=2, b=3, idx = c(1,-1,rep(NA,n-2)))
names(lc1) = "lc1"

## strictly speaking, it is sufficient to use `idx = c(1,-1)', as the
## remaining idx's are not used in any case.

r = inla(formula, data = data.frame(a,b,y),
        ## add the linear combinations here
        lincomb = lc1,
        ## force noise variance to be essiatially zero
        control.family = list(initial=10, fixed=TRUE))
		
## to verify the result, we can compare the mean but the variance and
## marginal cannot be computed from the simpler marginals alone.
lc1.1 = 2 * r$summary.fixed["a", "mean"] + 3 * r$summary.fixed["b",
    "mean"] + r$summary.random$idx$mean[1] -
    r$summary.random$idx$mean[2]
lc1.2= r$summary.lincomb.derived$mean
print(round(c(lc1.1 = lc1.1, lc1.2 = lc1.2), dig=3))

## -------------------------------------------------------------------
## let wa and wb be vectors, and we want to compute the marginals for
## beta_a * wa[i] + beta_b * wb[i], for i=1..m. this is done
## conveniently as follows

m = 10
wa = runif(m)
wb = runif(m)
lc.many = inla.make.lincombs(a = wa, b=wb)

## we can give them names as well, but there are also default names, like
print(names(lc.many))
r = inla(formula, data = data.frame(a,b,y),
        lincomb = lc.many,
        control.family = list(initial=10, fixed=TRUE))
print(round(r$summary.lincomb.derived, dig=3))

## ----eval=FALSE-----------------------------------------------------
# control.inla = list(lincomb.derived.correlation.matrix = TRUE)

## ----eval=FALSE-----------------------------------------------------
# result$misc$lincomb.derived.correlation.matrix

## -------------------------------------------------------------------
n = 100
nPred = 10
phi = 0.9
x = arima.sim(n,  model = list(ar=phi)) * sqrt(1-phi^2)
y = 1 + x + rnorm(n,  sd=0.1)

time = 1:(n + nPred)
Y = c(y, rep(NA, nPred))
formula = Y ~ 1 + f(time, model="ar1")

## make linear combinations which are the nPred linear predictors
B = matrix(NA, nPred, n+nPred)
for(i in 1:nPred) {
    B[i, n+i] = 1
}
lcs = inla.make.lincombs(Predictor = B)

r = inla(formula,  data = data.frame(Y, time),
        control.predictor = list(compute=TRUE),
        lincomb = lcs,
		inla.mode="classic",
        control.inla = list(lincomb.derived.correlation.matrix=TRUE))

print(round(r$misc$lincomb.derived.correlation.matrix,dig=3))

## -------------------------------------------------------------------
n = 100
u = rnorm(n)
eta = 1 + u
p = exp(eta)/(1+exp(eta))
y = rbinom(n, size=1,  prob = p)

idx = 1:n
result = inla(y ~ 1 + f(idx, model="iid",
                        hyper = list(prec = list(prior="pc.prec",
						                         prior = c(1,0.01)))), 
              data =data.frame(y,idx), family = "binomial", 
	          Ntrials = 1)
summary(result)

## -------------------------------------------------------------------
## simple example
set.seed(12345L)
sd.data = 1
n = 2000

## fixed high and low precisions are specified with parameters
fixed.high <- 10
fixed.low <- - 6

## otherwise is the model identical to the previous FAQ-model
x1 = rnorm(n)
eta1 = 1 + x1
x2 = rnorm(n)
eta2 = 2 + 2*eta1 + 2*x2
y1 = rnorm(n, mean=eta1, sd = sd.data)
y2 = rnorm(n, mean=eta2, sd = sd.data)
## the trick is to create a vector 'u' (iid) which is equal to eta1, and then we can copy 'u' to
## create beta*u or beta*eta1. we do this by using 0 = eta1 -u + tiny.noise

formula = Y ~ -1 + intercept1 + X1 + intercept2 +
    f(u, w, model="iid", hyper = list(prec = list(initial = fixed.low, fixed=TRUE))) + 
    f(b.eta2, copy="u", hyper = list(beta = list(fixed = FALSE))) + X2
Y = matrix(NA, 3*n, 3)

## part 1: y1
intercept1 = rep(1, n)
X1 = x1
intercept2 = rep(NA, n)
u = rep(NA, n)
w = rep(NA, n)
b.eta2 = rep(NA, n)
X2 = rep(NA, n)
Y[1:n, 1] = y1
## part 2: 0 = eta1 - u + tiny.noise
intercept1 = c(intercept1,  intercept1)
X1 = c(X1,  x1)
intercept2 = c(intercept2,  rep(NA, n))
u = c(u, 1:n)
w = c(w, rep(-1, n))
b.eta2 = c(b.eta2,  rep(NA, n))
X2 = c(X2,  rep(NA, n))
Y[n + 1:n, 2] = 0
## part 3: y2
intercept1 = c(intercept1,  rep(NA, n))
X1 = c(X1,  rep(NA, n))
intercept2 = c(intercept2, rep(1, n))
u = c(u, rep(NA, n))
w = c(w, rep(NA, n))
b.eta2 = c(b.eta2, 1:n)
X2 = c(X2, x2)
Y[2*n + 1:n, 3] = y2
r = inla(formula,
    data =  list(Y=Y, intercept1=intercept1, X1=X1,
        intercept2=intercept2, u=u, w=w, b.eta2=b.eta2, X2=X2),
    family = c("gaussian", "gaussian", "gaussian"),
    control.family = list(
        list(),
        list(hyper = list(prec = list(initial = fixed.high, fixed=TRUE))),
        list()))
summary(r)

## An alternative specification of the same model applies the inla.stack() function. 
## It is called four times, one time for each submodel, one time for the "copying", 
## and one time for joining the three substacks to one.

## Part 1:
## The first model stack comes in two versions

mod1a.stack <- inla.stack(
	data = list(Y = as.matrix(cbind(y1, NA, NA))), 
	A = list(1, 1),
	effects = list(intercept1 = rep(1,n),
			x1 = x1)
)
		
mod1b.stack <- inla.stack(
	data = list(Y = as.matrix(cbind(y1, NA, NA))), 
	A = list(1),
	effects = list(u = 1:n)
)

## Part 2:		
copy.stack <- inla.stack(
	data = list(Y = as.matrix(cbind(NA, rep(0,n), NA))),
	A = list(1, 1, - 1),
	effects = list(intercept1 = rep(1,n),
			x1 = x1,
			u = 1:n)
)

## Part 3:
mod2.stack <- inla.stack(
	data = list(Y = as.matrix(cbind(NA, NA, y2))), 
	A = list(1, 1, 1),
	effects = list(intercept2 = rep(1,n),
			x2 = x2,
			b.eta2 = 1:n)
)

all.a.stack <- inla.stack(mod1a.stack, copy.stack, mod2.stack)
all.b.stack <- inla.stack(mod1b.stack, copy.stack, mod2.stack)

## The formula is slightly modified because the long vectors X1 and X2 has not been created
## and there is no weight applied as "-1" appears as the effect of u

formula = Y ~ -1 + intercept1 + x1 + intercept2 + x2 +
    f(u, model = "iid", hyper = list(prec = list(initial = fixed.low, fixed = TRUE))) + 
    f(b.eta2, copy = "u", hyper = list(beta = list(fixed = FALSE)))

## The inla() function makes use of the stack
r.new.a = inla(formula,
    data =  inla.stack.data(all.a.stack),
    control.predictor = list(A = inla.stack.A(all.a.stack)),
    family = c("gaussian", "gaussian", "gaussian"),
    control.family = list(
        list(),
        list(hyper = list(prec = list(initial = fixed.high, fixed=TRUE))),
        list()))
summary(r.new.a)

r.new.b = inla(formula,
    data =  inla.stack.data(all.b.stack),
    control.predictor = list(A = inla.stack.A(all.b.stack)),
    family = c("gaussian", "gaussian", "gaussian"),
    control.family = list(
        list(),
        list(hyper = list(prec = list(initial = fixed.high, fixed=TRUE))),
        list()))
summary(r.new.b)

## All specifications give similar and acceptable results both for fixed effects and precisions,
## The version r.new.b seems faster than r.new.a for more complex models with one or more spde-terms


## -------------------------------------------------------------------
inla.list.models()

## ----eval=FALSE-----------------------------------------------------
# y = a + b*w + ...

## ----eval=FALSE-----------------------------------------------------
# f(idx, model="iid2d", n=2*m, ...)

## ----eval=FALSE-----------------------------------------------------
# idx = 1:m
# idx.copy = m + 1:m
# formula = y ~  f(idx, model="iid2d", n=2*m) + f(idx.copy, w, copy="idx") + ....

## -------------------------------------------------------------------
n=1000
i=1:n
j = i
z = rnorm(n)
w = runif(n)
y = z  + 2*z*w + rnorm(n)
formula = y ~ f(i, model="iid",initial=0, fixed=T) +
              f(j, w, copy="i", fixed=FALSE)
r = inla(formula, data = data.frame(i,j,w,y))
summary(r)

## ----eval=FALSE-----------------------------------------------------
# beta = low + (high-low)*exp(beta.local)/(1+exp(beta.local))

## ----eval=FALSE-----------------------------------------------------
# f(idx, model = ..,  replicate = r)

## ----eval=FALSE-----------------------------------------------------
# i = 1:n
# formula = y ~  f(i, model = "iid") + ...

## ----eval=FALSE-----------------------------------------------------
# i = rep(1,n)
# r = 1:n
# formula = y ~  f(i, model="iid", replicate = r) + ...

## -------------------------------------------------------------------
n = 100
y1 = arima.sim(n=n, model=list(ar=c(0.9)))+10
y2 = arima.sim(n=n, model=list(ar=c(0.9)))+20
y3 = arima.sim(n=n, model=list(ar=c(0.9)))+30

formula = y ~ mean -1 + f(i, model="ar1", replicate=r)
y = c(y1,y2,y3)
i = rep(1:n, 3)
r = rep(1:3, each=n)
mean = as.factor(r)
result = inla(formula, family = "gaussian",
              data = data.frame(y, i, mean),
              control.family = list(initial = 12, fixed=TRUE))
summary(result)

## ----eval=FALSE-----------------------------------------------------
# y ~ a + 1

## -------------------------------------------------------------------
##  Simple linear regression with observations with two different
##  variances.
n = 100
N = 2*n
y = numeric(N)
x = rnorm(N)

y[1:n] = 1 + x[1:n] + rnorm(n, sd = 1/sqrt(1))
y[1:n + n] = 1 + x[1:n + n] + rnorm(n, sd = 1/sqrt(2))

Y = matrix(NA, N, 2)
Y[1:n, 1] = y[1:n]
Y[1:n + n, 2] = y[1:n + n]

formula = Y ~ x + 1
result = inla(
        formula,
        data = list(Y=Y, x=x),
        family = c("gaussian", "gaussian"),
        control.family = list(list(prior = "flat", param = numeric()),
                            list()))
summary(result)

## -------------------------------------------------------------------
## Simple example with two types of likelihoods
n = 10
N = 2*n

## common covariates
x = rnorm(n)

## Poisson, depends on x
E1 = runif(n)
y1 = rpois(n, lambda = E1*exp(x))

## Binomial, depends on x
size = sample(1:10, size=n, replace=TRUE)
prob = exp(x)/(1+exp(x))
y2 = rbinom(n, size= size, prob = prob)

## Join them together
Y = matrix(NA, N, 2)
Y[1:n, 1] = y1
Y[1:n + n, 2] = y2

## The E for the Poisson
E = numeric(N)
E[1:n] = E1
E[1:n + n] = NA

## Ntrials for the Binomial
Ntrials = numeric(N)
Ntrials[1:n] = NA
Ntrials[1:n + n] = size

## Duplicate the covariate which is shared
X = numeric(N)
X[1:n] = x
X[1:n + n] = x

## Formula involving Y as a matrix
formula = Y ~ X - 1

## `family' is now
result = inla(formula,
        family = c("poisson", "binomial"),
		control.family=list(list(), list()),
        data = list(Y=Y, X=X),
        E = E, Ntrials = Ntrials)
summary(result)

## ----eval=FALSE-----------------------------------------------------
# X = numeric(N)
# X[1:n] = x
# X[1:n + n] = NA
# 
# XX = numeric(N)
# XX[1:n] = NA
# XX[1:n + n] = xx
# 
# formula = Y  ~ X + XX -1

## -------------------------------------------------------------------
## An example with three independent AR(1)'s with separate means, but
## with the same hyperparameters. These are observed with three
## different likelihoods.

n = 100
x1 = arima.sim(n=n, model=list(ar=c(0.9))) + 0
x2 = arima.sim(n=n, model=list(ar=c(0.9))) + 1
x3 = arima.sim(n=n, model=list(ar=c(0.9))) + 2

## Binomial observations
Nt = 10 + rpois(n,lambda=1)
y1 = rbinom(n, size=Nt, prob = exp(x1)/(1+exp(x1)))

## Poisson observations
Ep = runif(n, min=1, max=10)
y2 = rpois(n, lambda = Ep*exp(x2))

## Gaussian observations
y3 = rnorm(n, mean=x3, sd=0.1)

## stack these in a 3-column matrix with NA's where not observed
y = matrix(NA, 3*n, 3)
y[1:n, 1] = y1
y[n + 1:n, 2] = y2
y[2*n + 1:n, 3] = y3

## define the model
r = c(rep(1,n), rep(2,n), rep(3,n))
rf = as.factor(r)
i = rep(1:n, 3)
formula = y ~ f(i, model="ar1", replicate=r, constr=TRUE) + rf -1
data = list(y=y, i=i, r=r, rf=rf)

## parameters for the binomial and the poisson
Ntrials = rep(NA, 3*n)
Ntrials[1:n] = Nt
E = rep(NA, 3*n)
E[1:n + n] = Ep

result = inla(formula, family = c("binomial", "poisson", "normal"),
              data = data, Ntrials = Ntrials, E = E,
              control.family = list(
                      list(),
                      list(),
                      list()))
summary(result)

## ----eval=FALSE-----------------------------------------------------
# y ~ 1 + z

## ----eval=FALSE-----------------------------------------------------
# eta~ = A %*% eta

## ----eval=FALSE-----------------------------------------------------
# y ~ 1 + z, with addition matrix A

## ----eval=FALSE-----------------------------------------------------
# y    ~ eta~   ## no intercept...
# eta~ = A %*% eta
# eta  = intercept + beta*z

## ----eval=FALSE-----------------------------------------------------
# control.predictor=list(A=A)

## ----eval=FALSE-----------------------------------------------------
# eta~ = A %*% eta + offset.arg
# eta = intercept + beta*z + offset.formula

## -------------------------------------------------------------------
## 'm' is the number of observations of eta*, where eta* = A eta +
## offset.arg, and A is a fixed m x n matrix, and eta has length n. An
## offset in the formula goes into 'eta' whereas an offset in the
## argument of the inla-call, goes into eta*
n = 10
m = 100
offset.formula = 10+ 1:n
offset.arg = 1 + 1:m

## a covariate
z = runif(n)

## the linear predictor eta
eta = 1 + z + offset.formula

## the linear predictor eta* = A eta + offset.arg.
A = matrix(runif(n*m), m, n);
##A = inla.as.sparse(A)  ## sparse is ok
## need 'as.vector', as 'Eta' will be a sparseMatrix if 'A' is sparse
## even if ncol(Eta) = 1
Eta = as.vector(A %*% eta) + offset.arg

s = 1e-6
Y = Eta + rnorm(m, sd=s)

## for a check, we can use several offsets. here, m1=-1 and p1=1, so
## they m1+p1 = 0.
r = inla(Y ~ 1+z + offset(offset.formula) + offset(m1) + offset(p1),
        ## The A-matrix defined here
        control.predictor = list(A = A, compute=TRUE, precision = 1e6),
        ## we need to use a list() as the different lengths of Y
        ## and z
        data = list(Y=Y, z=z,
                m1 = rep(-1, n),
                p1 = rep(1, n),
                offset.formula = offset.formula,
                offset.arg = offset.arg),
        ## this is the offset defined in the argument of inla
        offset = offset.arg,
        ##
        control.family = list(initial = log(1/s^2), fixed=TRUE))
summary(r)

## this should be a small number
print(max(abs(r$summary.linear.predictor$mean - c(Eta, eta))))

## ----eval=FALSE-----------------------------------------------------
# y = intercept + s[j] + 0.5*s[k] + noise

## ----plot=TRUE------------------------------------------------------
n = 100
s = c(-1, 0, 1)
nS = length(s)
j = sample(1L:nS, n, replace=TRUE)
k = j
k[j == 1L] = 2
k[j == 2L] = 3
k[k == 3L] = 1

noise = rnorm(n, sd=0.0001)
y = 1 + s[j] + 0.5*s[k] + noise

## build the formula such that the linear predictor is the intercept
## (index 1) and the 's' term (index 2:(n+1)). then kind of
## 'construct' the model using the A-matrix.
formula = y ~ -1 + intercept + f(idx)
A = matrix(0, n, nS+1L)
for(i in 1L:n) {
  A[i, 1L]        = 1
  A[i, 1L + j[i]] = 1
  A[i, 1L + k[i]] = 0.5
}

data = list(intercept = c(1, rep(NA, nS)), idx = c(NA, 1L:nS))
result = inla(formula, data=data, control.predictor=list(A=A))
## should be a straight line
plot(result$summary.random$idx$mean, s, pch=19)
abline(a=0,b=1)

context("test 'prior spesification (table, expression)'")

test_that("Case 1", {
    set.seed(123)
    y = rnorm(100)
    a = 1
    b = 0.1
    hyper = list(prec = list(prior = "loggamma", param = c(a, b)))
    r = inla(y ~ 1,  data = data.frame(y),
            control.family = list(hyper = hyper))
    loggamma = "expression:
            a = 1;
            b = 0.1;
            precision = exp(log_precision);
            logdens = log(b^a) - lgamma(a)
                      + (a-1)*log_precision - b*precision;
            log_jacobian = log_precision;
            return(logdens + log_jacobian);"
    hyper.new = list(prec = list(prior = loggamma))
    r.new = inla(y ~ 1,  data = data.frame(y),
            control.family = list(hyper = hyper.new), verbose=FALSE)
    expect_true(abs(r$summary.hyperpar[1,"mean"] -
                    r.new$summary.hyperpar[1,"mean"]) < 0.001)
})

test_that("Case 2", {
    prior.function = function(log_precision) {
        a = 1;
        b = 0.1;
        precision = exp(log_precision);
        logdens = log(b^a) - lgamma(a) + (a-1)*log_precision - b*precision;
        log_jacobian = log_precision;
        return(logdens + log_jacobian)
    }
    prior.expression = "expression:
            a = 1;
            b = 0.1;
            precision = exp(log_precision);
            logdens = log(b^a) - lgamma(a)
                      + (a-1)*log_precision - b*precision;
            log_jacobian = log_precision;
            return(logdens + log_jacobian);"
          
    lprec = seq(-10, 10, len=1000)
    prior.table = paste(c("table:", cbind(lprec, prior.function(lprec))),
            sep = "", collapse = " ")
          
    set.seed(123)
    n = 100
    y = rnorm(n)
    r = inla(y~1,
            data = data.frame(y),
            control.family = list(
                    hyper = list(
                            prec = list(
                                    prior = "loggamma",
                                    param = c(1, 0.1)))))
    rr = inla(y~1,
            data = data.frame(y),
            control.family = list(
                    hyper = list(
                            prec = list(
                                    prior = prior.expression))))
    rrr = inla(y~1,
            data = data.frame(y),
            control.family = list(
                    hyper = list(
                            prec = list(
                                    prior = prior.table))))
    expect_true(abs(r$mlik[1] -  rr$mlik[1]) < 0.001)
    expect_true(abs(r$mlik[1] -  rrr$mlik[1]) < 0.001)
    expect_true(abs(rr$mlik[1] -  rrr$mlik[1]) < 0.001)
})

## ----setup, include=FALSE-------------------------------------------
knitr::opts_chunk$set(echo = TRUE, cache = FALSE)
knitr::opts_chunk$set(fig.path="figures/int-design/")
Design = matrix()
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")

## -------------------------------------------------------------------
	opts = control.inla(int.strategy = "user", int.design = Design)

## ----eval=FALSE-----------------------------------------------------
#  	opts = control.inla(int.strategy = "user.std", int.design = Design)

## ----eval=TRUE, echo=TRUE-------------------------------------------
n = 100
rho = 0.9
x = scale(arima.sim(n, model = list(ar = rho)))
y = x + rnorm(n, sd = 0.1)

## -------------------------------------------------------------------
plot(y, xlab = "time", ylab = "value")
lines(x, lwd=2)

## -------------------------------------------------------------------
rho.0 = rho
to.theta = inla.models()$latent$ar1$hyper$theta2$to.theta
rho.0.internal = to.theta(rho.0)

r = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.0.internal,
		              fixed=TRUE))),
    control.inla = list(int.strategy = "grid"),
	data = data.frame(y, time = 1:n))

sd.0 = r$summary.random$time[1,"sd"]
print(sd.0)

## -------------------------------------------------------------------
summary(r)
nm <- nrow(r$summary.hyperpar)
Design = as.matrix(cbind(r$joint.hyper[, seq_len(nm)], 1))
head(Design)

## -------------------------------------------------------------------
h.rho = 0.01
rho.1.internal = to.theta(rho.0 + h.rho)
rr = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.1.internal,
		              fixed=TRUE))),
	control.mode = list(result = r, restart=FALSE),	
    data = data.frame(y, time = 1:n),
	control.inla = list(
		int.strategy = "user",
		int.design = Design))
sd.1 = rr$summary.random$time[1,"sd"]
print(sd.1)

## -------------------------------------------------------------------
deriv.1 = (sd.1 - sd.0) / h.rho
print(deriv.1)

## ----eval=FALSE-----------------------------------------------------
#  control.inla = list(int.stategy = "user.expert")

## -------------------------------------------------------------------
n = 50
x = rnorm(n)
y = 1 + x + rnorm(n, sd = 0.2)
param = c(1, 0.04)
dz = 0.1
r.std = inla(y ~ 1 + x, data = data.frame(y, x),
             control.inla = list(int.strategy = "grid",
                                 dz = dz,
                                 diff.logdens = 8), 
             control.family = list(
                 hyper = list(
                     prec = list(
                         prior = "loggamma",
                         param = param))))

s = r.std$internal.summary.hyperpar[1,"sd"]
m = r.std$internal.summary.hyperpar[1,"mean"]
theta = m + s*seq(-4, 4, by = dz)
weight = dnorm(theta,  mean = m, sd = s)

r = rep(list(list()), length(theta))
for(k in seq_along(r)) {
    r[[k]] = inla(y ~ 1 + x,
                  control.family = list(
                      hyper = list(
                          prec = list(
                              initial = theta[k],
                              fixed=TRUE))),
                  data = data.frame(y, x))
}
r.merge = inla.merge(r, prob = weight)

r.design = inla(y ~ 1 + x,
                data = data.frame(y, x),
                control.family = list(
                    hyper = list(
                        prec = list(
                            ## the prior here does not really matter, as we will override
                            ## it with the user.expert in any case.
                            prior = "pc.prec",
                            param = c(1, 0.01)))), 
                control.inla = list(int.strategy = "user.expert",
                                    int.design = cbind(theta, weight)))
for(k in 1:2) {
    plot(inla.smarginal(r.std$marginals.fixed[[k]]),
         lwd = 2, lty = 1, type = "l", 
         xlim = inla.qmarginal(c(0.0001, 0.9999), r.std$marginals.fixed[[k]]))
    lines(inla.smarginal(r.design$marginals.fixed[[k]]),
          lwd = 2, col = "blue", lty = 1)
    lines(inla.smarginal(r.merge$marginals.fixed[[k]]),
          lwd = 2, col = "yellow", lty = 1)
}

data(Germany)
g = system.file("demodata/germany.graph", package="INLA")
source(system.file("demodata/Bym-map.R", package="INLA"))
summary(Germany)

## just make a duplicated column
Germany = cbind(Germany,region.struct=Germany$region)

# standard BYM model
formula1 = Y ~ f(region.struct,model="besag",graph=g) +
               f(region,model="iid")

result1  =  inla(formula1,family="poisson",data=Germany,E=E)

# with linear covariate
formula2 = Y ~ f(region.struct,model="besag",graph=g) +
               f(region,model="iid") + x

result2 =  inla(formula2,family="poisson",data=Germany,E=E)

# with smooth covariate
formula3 = Y ~ f(region.struct,model="besag",graph=g) +
               f(region,model="iid") + f(x, model="rw2")

result3 =  inla(formula3,family="poisson",data=Germany,E=E)

dev.new()
par(mfrow=c(2,2))
Bym.map(result1$summary.random$region.struct$mean)
Bym.map(result2$summary.random$region.struct$mean)
Bym.map(result3$summary.random$region.struct$mean)

## An alternative to above is to use the combined model BYM, which
## contains both the "iid" and the "besag" model. This makes it
## possible to get the marginals of "idd" + "besag" which otherwise is
## only possible using linear-combinations. Here, I just repeat the
## first example. I also add some prior paramters and initial values
## as they are in the order ("idd", "besag"). Further, the internal
## representation is ("iid"+"besag", "besag"), which implies that the
## first 'n' elements of the mean vector (of length '2*n') is the sum
## and the remaining 'n' elements is the spatial term. If the
## constr=TRUE, the only the "besag" part satisfy the sum-to-zero
## constraint.
prior.iid = c(1,0.01)
prior.besag = c(1,0.001)
initial.iid = 4
initial.besag = 3

formula1.bym = Y ~ f(region, model = "bym", graph = g,
                     param = c(prior.iid, prior.besag),
                     initial = c(initial.iid, initial.besag))
result1.bym = inla(formula1.bym,family="poisson",data=Germany,E=E)

## ----setup, include=FALSE-------------------------------------------
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")

library(knitr)
library(formatR)
set.seed(20202)
opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/scale-model'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/scale-model/'
               , fig.pos='H')
library(spatstat)
library(mvtnorm)
library(lattice)
library(mgcv)
library(pixmap)
library(numDeriv)
library(fields)
set.seed(123)

## -------------------------------------------------------------------
formula = y ~  f(idx, model = "rw2", scale.model = TRUE, hyper=..)

## ----eval=F---------------------------------------------------------
#  f(idx, model = "..", group = g, control.group = list(model = "rw2", scale.model = TRUE))

## -------------------------------------------------------------------
# Functions to calculate the marginal standard deviations of rw1 and rw2
ss.rw1.sdref = function(values,tau) { 
    n = length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3) 
    y = NA
    
    formula = y ~ -1 + f(values, model = "rw1", constr = TRUE, 
                         values = values, diagonal = 1e-10, 
                         hyper = list(prec = list(initial = log(tau), fixed = TRUE))) 
    r = inla(formula, data = data.frame(y, values), family = "gaussian",
             control.family = list(fixed = TRUE), 
             control.compute = list(return.marginals = F), verbose = F)
    sd = r$summary.random$values$sd 
    return (sd) 
    }

ss.rw2.sdref = function(values,tau) { 
    n = length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3)
    y = NA 
    
    A1 = matrix(1, n, 1) 
    A2 = matrix(0, n, 1) 
    for(i in 1:n) A2[i, ]= i 
    
    A = rbind(c(A1), c(A2)) 
    e = rep(0, 2) 
    extraconstr = list(A = A, e= e)
    
    formula = y ~ -1 + f( values, model="rw2", constr = FALSE,
                          values = values, diagonal = 1e-10, extraconstr = extraconstr,
                          hyper = list(prec = list(initial = log(tau), fixed = TRUE)))
    
    r = inla(formula, data = data.frame(y, values), family = "gaussian", 
             control.family = list(fixed = TRUE),
             control.compute = list(return.marginals = F), verbose=F)
    
    sd = r$summary.random$values$sd 
    return (sd) 
}

# The marginal standard deviations for n nodes
n = 100 
sd.rw1 = ss.rw1.sdref(1:n, 1) 
sd.rw2 = ss.rw2.sdref(1:n, 1)

## ----plot = TRUE, fig.show = 'hold', out.width = '40%', echo = FALSE----
plot(sd.rw1, xlab = "Node i", ylab = "Marginal standard deviation") 
plot(sd.rw2, xlab = "Node i", ylab = "Marginal standard deviation") 

## ----echo=TRUE, eval=F----------------------------------------------
#  U = sqrt(b/qgamma(alpha, a, 1))

## ----echo=F---------------------------------------------------------
func.u = function(a, b, alpha, sigma.ref) { 
    upper.limit = sqrt(b)*sigma.ref/sqrt(qgamma(alpha, a, 1)) 
    return(upper.limit) 
}
a = 1 
b = 5*10^{-5} 
alpha = 0.001 
upper.limit = func.u(a, b, alpha, 1) 

## -------------------------------------------------------------------
formula = y ~  f(x, model = "rw2", scale.model = TRUE, hyper=...)


## ----echo = F-------------------------------------------------------
func <-function(x, t)
{
    f.x = x/t + sin(2*pi* x/t) - 0.5
    return(f.x)
}

f.est <- function(y, x, a, b, option)
{
    formula = y ~ 1 + f(x, model="rw2",
                    hyper = list(prec = list(prior = "loggamma", param = c(a,b))), 
                    scale.model = option)
    result = inla(formula,family = "gaussian", data = data.frame(y,x))
    f.est = result$summary.random$x$mean
    return(f.est)
}

## ----echo = F-------------------------------------------------------
### Same values of the function, same observations, but  defined on different intervals ###
set.seed(89236)
x2 = 0:n
x1 = x2/100
x3 = x2*10

sigma = 0.5
f.x = func(x2,length(x2))
y = f.x + rnorm(length(f.x), 0, sigma)

a = 1
b = 0.00005

mat1 = cbind(f.x, f.est(y, x1, a, b, T), f.est(y, x1, a, b, F))
mat2 = cbind(f.x, f.est(y, x2, a, b, T), f.est(y, x2, a, b, F))
mat3 = cbind(f.x, f.est(y, x3, a, b, T), f.est(y, x3, a, b, F))

# Generalized marginal variances  
v1 = exp(mean(log(ss.rw2.sdref(x1, 1)^2)))
v2 = exp(mean(log(ss.rw2.sdref(x2, 1)^2)))
v3 = exp(mean(log(ss.rw2.sdref(x3, 1)^2)))

u1 = func.u(a, b, alpha,sqrt(v1))
u2 = func.u(a, b, alpha,sqrt(v2))
u3 = func.u(a, b, alpha,sqrt(v3))

## ----eval=F---------------------------------------------------------
#  result = inla(formula, family = "gaussian", data = data.frame(y, x))

## ----eval=F---------------------------------------------------------
#  result$summary.random$x$mean

## ----plot=TRUE, echo = FALSE, fig.show = 'hold', out.width = '30%'----
r=range(c(mat1,mat2,mat3))
matplot(x1, mat1, type="lll", lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3) 
matplot(x2, mat2, type="lll", lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)
matplot(x3, mat3, type="lll", lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)

## ----echo=TRUE, eval=F----------------------------------------------
#  hyper.prec = list(prec = list(param = c(1, 0.01)))

## ----echo=TRUE, eval=F----------------------------------------------
#  data(Munich)
#  g = system.file("demodata/munich.graph", package="INLA")
#  
#  ## Note that here we want to have an estimator of the effect of year
#  ## also the for years where we have no observation, therefore we give a
#  ## vector with all possible values assumed by the covariate year, that
#  ## is seq(1918,2001)
#  
#  formula = rent ~ -1 +
#     f(location, model = "besag", graph = g, initial = 1, hyper = hyper.prec,
#       scale.model = option) +
#     f(year, model = "rw2", values = seq(1918, 2001), hyper = hyper.prec,
#       scale.model = option) +
#     f(floor.size, model = "rw2", hyper = hyper.prec, scale.model = option) +
#     Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
#     Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
#     zim1 + zim2 + zim3 + zim4 + zim5 + zim6
#  

## ----echo=TRUE, eval=F----------------------------------------------
#  mod = inla(formula, data = Munich, control.fixed=list(prec=1))

## ----echo=FALSE-----------------------------------------------------
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

func.munich <- function(a, b, option)
{
    hyper.prec = list(prec = list(param = c(a, b)))
    formula = rent ~ f(location, model = "besag", graph = g,initial=1, 
                       hyper = hyper.prec, scale.model = option) +
      f(year, model = "rw2", values = seq(1918, 2001), 
        hyper = hyper.prec, scale.model = option) +
      f(floor.size, model = "rw2", hyper = hyper.prec, scale.model = option) +
      Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh + 
      Kein.Badkach  + Besond.Bad + Gehobene.Kueche + 
      zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
    
    return (inla(formula, data = Munich, control.fixed = list(prec=1, prec.intercept = 1)))
}


## ----echo=FALSE-----------------------------------------------------
a = 1
b = 0.01
u.munich = func.u(a, b, alpha, 1)
mod = func.munich(a, b, FALSE)
mod.scaled = func.munich(a, b, TRUE)

## ----echo=F---------------------------------------------------------
alpha = 0.001
u.vec = c(0.001,0.1,10,30)
a.vec = c(1,1,1,1)
b.vec = u.vec^2*qgamma(alpha,a.vec,1)

option=TRUE
c1 = func.munich(a.vec[1], b.vec[1], option)
c2 = func.munich(a.vec[2], b.vec[2], option)
c3 = func.munich(a.vec[3], b.vec[3], option)
c4 = func.munich(a.vec[4], b.vec[4], option)

## ----plot=TRUE, echo = F, fig.show = 'hold', out.width = '40%'------
mat = cbind(mod$summary.random$year$mean, mod.scaled$summary.random$year$mean)
matplot(mod$summary.random$year$ID, mat, type="ll", lty=c(2,1), col=c(2,1), 
        xlab="Year of construction", ylab="", lwd=2.5)

mat = cbind(mod$summary.random$floor.size$mean, mod.scaled$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID, mat, type="ll", lty=c(2,1), col=c(2,1),
        xlab="Floor size",ylab="",lwd=2.5)

## ----plot=TRUE, fig.show = 'hold', out.width = '40%', echo = FALSE----
mat = cbind(c1$summary.random$year$mean,
            c2$summary.random$year$mean,
            c3$summary.random$year$mean,
            c4$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat, type="llll", lty=c(1,2,3,4), col=c(1,2,3,4),
        xlab="Year of construction", ylab="", lwd=2.5)
legend("topleft", paste("U= ",u.vec), lty=1:4, col=1:4, lwd=2.5)

mat = cbind(c1$summary.random$floor.size$mean,
            c2$summary.random$floor.size$mean,
            c3$summary.random$floor.size$mean,
            c4$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat, type="llll", lty=c(1,2,3,4), col=c(1,2,3,4),
        xlab="Floor size", ylab="", lwd=2.5)
legend("topright", paste("U= ",u.vec), lty=1:4, col=1:4, lwd=2.5)

## -------------------------------------------------------------------
inla.setOption(scale.model.default = TRUE)

## ----setup, include=FALSE-----------------------------------------------------
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/rgeneric/")

## ----eval=FALSE---------------------------------------------------------------
#  model = inla.rgeneric.define(rmodel, ...)

## ----eval=FALSE---------------------------------------------------------------
#  y ~ ... + f(idx, model=model, ...)

## ----eval=FALSE---------------------------------------------------------------
#  inla.rgeneric.ar1.model = function(
#          cmd = c("graph", "Q", "mu", "initial", "log.norm.const",
#                  "log.prior", "quit"),
#          theta = NULL)
#  {
#      # for reference and potential storage for objects to
#      # cache, this is the environment of this function
#      # which holds arguments passed as `...` in
#      # `inla.rgeneric.define()`.
#  	envir = parent.env(environment())
#  
#      graph = function(){ <to be completed> }
#      Q = function() { <to be completed> }
#      mu = function() { <to be completed> }
#      log.norm.const = function() { <to be completed> }
#      log.prior = function() { <to be completed> }
#      initial = function() { <to be completed> }
#      quit = function() { <to be completed> }
#  
#      # sometimes this is useful, as argument 'graph' and 'quit'
#      # will pass theta=numeric(0) (or NULL in R-3.6...) as
#  	# the values of theta are NOT
#  	# required for defining the graph. however, this statement
#      # will ensure that theta is always defined.
#      if (!length(theta)) theta = initial()
#  
#      val = do.call(match.arg(cmd), args = list())
#      return (val)
#  }

## ----eval=FALSE---------------------------------------------------------------
#  model = inla.rgeneric.define(inla.rgeneric.ar1.model, n = 100)

## ----eval=FALSE---------------------------------------------------------------
#  interpret.theta = function() {
#      return(list(prec = exp(theta[1L]),
#                  rho = 2 * exp(theta[2L])/(1 + exp(theta[2L])) - 1))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  graph = function() {
#      return (Q())
#  }

## ----eval=FALSE---------------------------------------------------------------
#  Q = function() {
#      p = interpret.theta()
#      Q = p$prec/(1 - p$rho^2) *
#  	    toeplitz(c(1 + p$rho^2, -p$rho, rep(0, n - 2L)))
#      Q[1, 1] = Q[n, n] = p$prec/(1 - p$rho^2)
#      return (inla.as.sparse(Q))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  Q = function() {
#      p = interpret.theta()
#      i = c(1L, n, 2L:(n - 1L), 1L:(n - 1L))
#      j = c(1L, n, 2L:(n - 1L), 2L:n)
#      x = p$prec/(1 - p$rho^2) *
#  	    c(1L, 1L, rep(1 + p$rho^2, n - 2L),
#  	      rep(-p$rho, n - 1L))
#       return (sparseMatrix(i = i, j = j, x = x, giveCsparse = FALSE))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  mu = function() {
#      return(numeric(0))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  log.norm.const = function() {
#     p = interpret.theta()
#     prec.i  = p$prec / (1.0 - p$rho^2)
#     val = n * (- 0.5 * log(2*pi) + 0.5 * log(prec.i)) +
#           0.5 * log(1.0 - p$rho^2)
#     return (val)
#  }

## ----eval=FALSE---------------------------------------------------------------
#  log.norm.const = function() {
#     return (numeric(0))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  log.prior = function() {
#      p = interpret.theta()
#      val = dgamma(p$prec, shape = 1, rate = 1, log=TRUE) + theta[1L] +
#            dnorm(theta[2L], mean = 0, sd = 1, log=TRUE)
#      return (val)
#  }

## ----eval=FALSE---------------------------------------------------------------
#  initial = function() {
#      return (rep(1, 2))
#  }

## ----eval=FALSE---------------------------------------------------------------
#  quit = function() {
#      return (invisible())
#  }

## -----------------------------------------------------------------------------
n = 100
rho=0.9
x = arima.sim(n, model = list(ar = rho)) * sqrt(1-rho^2)
y = x + rnorm(n, sd = 0.1)
model = inla.rgeneric.define(inla.rgeneric.ar1.model, n=n)
formula = y ~ -1 + f(idx, model=model)
r = inla(formula, data = data.frame(y, idx = 1:n))

## -----------------------------------------------------------------------------
fformula = y ~ -1 +
	f(idx, model = "ar1",
	  hyper = list(prec = list(prior = "loggamma", param = c(1,1)),
      rho = list(prior = "normal", param = c(0,1))))
rr = inla(fformula, data = data.frame(y, idx = 1:n))

## -----------------------------------------------------------------------------
plot(inla.smarginal(rr$marginals.hyperpar[[2]]), 
	 type="l", lwd=5, col="red", xlab="stdev", ylab="density")
lines(inla.tmarginal(exp, r$internal.marginals.hyperpar[[2]]), 
     col="yellow")

## -----------------------------------------------------------------------------
plot(inla.smarginal(rr$marginals.hyperpar[[3]]), 
	 type="l", lwd=5, col="red", xlab="rho", ylab="density")
lines(inla.tmarginal(function(x) 2*exp(x)/(1+exp(x))-1,
                     r$internal.marginals.hyperpar[[3]]), 
     col="yellow")

## -----------------------------------------------------------------------------
round(rbind(native = rr$cpu.used,  
            rgeneric = r$cpu.used), digits = 3)

## -----------------------------------------------------------------------------
inla.rgeneric.iid.model

## ---- eval=FALSE--------------------------------------------------------------
#  ## In this example we do linear regression using 'rgeneric'.
#  ## The regression model is y = a + b*x + noise,  and we
#  ## define 'a + b*x + tiny.noise' as a latent model.
#  ## The dimension is length(x) and number of hyperparameters
#  ## is 2 ('a' and 'b').

## -----------------------------------------------------------------------------
rgeneric.linear.regression =
    function(cmd = c("graph", "Q", "mu", "initial", "log.norm.const", 
                     "log.prior", "quit"),
             theta = NULL)
{
	envir = parent.env(environment())

    ## artifical high precision to be added to the mean-model
    prec.high = exp(15)
    
    interpret.theta = function() {
        return(list(a = theta[1L], b = theta[2L]))
    }
    
    graph = function() {
        G = Diagonal(n = length(x), x=1)
        return(G)
    } 
    
    Q = function() {
        Q = prec.high * graph()
        return(Q)
    }
    
    mu = function() {
        par = interpret.theta()
        return(par$a + par$b * x)
    }

    log.norm.const = function() {
        return(numeric(0))
    }

    log.prior = function() {
        par = interpret.theta()
        val = (dnorm(par$a, mean=0, sd=1, log=TRUE) +
               dnorm(par$b, mean=0, sd=1, log=TRUE))
        return(val)
    }

    initial = function() {
        return(rep(0, 2))
    }
   
    quit = function() {
        return(invisible())
    }

    val = do.call(match.arg(cmd), args = list())
    return(val)
}

## -----------------------------------------------------------------------------
a = 1
b = 2
n = 50
x = rnorm(n)
eta = a + b*x
s = 0.25
y = eta + rnorm(n, sd=s)

rgen = inla.rgeneric.define(model = rgeneric.linear.regression, x=x)
r = inla(y ~ -1 + f(idx, model=rgen),
         data = data.frame(y, idx = 1:n))
rr = inla(y ~ 1 + x,
          data = data.frame(y, x),
          control.fixed = list(prec.intercept = 1, prec = 1))

## -----------------------------------------------------------------------------
plot(inla.smarginal(r$marginals.hyperpar[['Theta1 for idx']]),
     type="l", lwd=5, col="red", xlab="Intercept", ylab="density")
lines(inla.smarginal(rr$marginals.fixed$'(Intercept)'), col="yellow")

## -----------------------------------------------------------------------------
plot(inla.smarginal(r$marginals.hyperpar[['Theta2 for idx']]),
     type="l", lwd=5, col="red", xlab="Slope", ylab="density")
lines(inla.smarginal(rr$marginals.fixed$x), col="yellow")

## -----------------------------------------------------------------------------
rgeneric.test = function(
    cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
    theta = NULL)
{
    envir = parent.env(environment())

    graph = function() {
        return(matrix(1, n, n))
    }

    Q = function() {
        R <- matrix(sin(1:n^2), n, n)
        R <- R %*% t(R)
        diag(R) <- diag(R)+1
        Q <- exp(theta[1]) * R
        return(Q)
    }

    mu = function() return (numeric(0))

    log.norm.const = function() {
        return (numeric(0))
    }

    log.prior = function() {
        return (dgamma(exp(theta[1]), shape = 1, rate = 1, log=TRUE) + theta[1])
    }

    initial = function() {
        return(4)
    }

    if (!length(theta)) theta = initial()
    val = do.call(match.arg(cmd), args = list())

    return (val)
}

## -----------------------------------------------------------------------------
n = 200
s = .1
Q <- rgeneric.test("Q", theta = 0)
library(mvtnorm)
S <- solve(as.matrix(Q))
S <- (S + t(S))/2
x <- drop(rmvnorm(1, sigma = S))
y <- x + rnorm(n, sd = s)
cont.family = list(hyper = list(prec = list(initial=log(1/s^2), fixed=TRUE)))

r1 = inla(y ~ -1 + f(idx, model="generic", Cmatrix = Q,
                     hyper = list(prec = list(prior = "loggamma", param = c(1, 1)))), 
          data = data.frame(y = y, idx = 1:n), control.family = cont.family)
ld <- 0.5 * log(det(as.matrix(Q)))
r1$mlik <- r1$mlik + ld ## see the documentation for why

model2 = inla.rgeneric.define(rgeneric.test, n=n, optimize = FALSE)
r2 = inla(y ~ -1 + f(idx, model=model2), 
          data = data.frame(y = y, idx = 1:n), control.family = cont.family)

## -----------------------------------------------------------------------------
r2$mlik - r1$mlik

## -----------------------------------------------------------------------------
rgeneric.test.opt.1 = function(
    cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
    theta = NULL)
{
    envir = parent.env(environment())

    if (!exists("cache.done", envir = envir)) {
        R <- matrix(sin(1:n^2), n, n)
        R <- R %*% t(R)
        diag(R) <- diag(R)+1
        R.logdet <- log(det(R))
        R <- inla.as.sparse(R)
        idx <- which(R@i <= R@j)
        R@i <- R@i[idx]
        R@j <- R@j[idx]
        R@x <- R@x[idx]
        assign("R", R, envir = envir)
        norm.const <- -n/2 * log(2*pi) + 0.5 * R.logdet
        assign("norm.const", norm.const, envir = envir)
        assign("cache.done", TRUE, envir = envir)
    }

    graph = function() {
        return (R)
    }

    Q = function() {
        return(exp(theta[1]) * R)
    }

    mu = function() return (numeric(0))

    log.norm.const = function() {
        return (norm.const + n/2 * theta[1])
    }

    log.prior = function() {
        return (dgamma(exp(theta[1]), shape = 1, rate = 1, log=TRUE) + theta[1])
    }

    initial = function() {
        return(4)
    }

    if (!length(theta)) theta = initial()
    val = do.call(match.arg(cmd), args = list())

    return (val)
}

## -----------------------------------------------------------------------------
A=matrix(1:9,3,3)
A
inla.as.sparse(A)@x

## -----------------------------------------------------------------------------
rgeneric.test.opt.2 = function(
    cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
    theta = NULL)
{
    envir = parent.env(environment())

    if (!exists("cache.done", envir = envir)) {
        R <- matrix(sin(1:n^2), n, n)
        R <- R %*% t(R)
        diag(R) <- diag(R)+1
        R.logdet <- log(det(R))
        R <- inla.as.sparse(R)
        idx <- which(R@i <= R@j)
        R@i <- R@i[idx]
        R@j <- R@j[idx]
        R@x <- R@x[idx]
        assign("R", R, envir = envir)
        norm.const <- -n/2 * log(2*pi) + 0.5 * R.logdet
        assign("norm.const", norm.const, envir = envir)
        assign("cache.done", TRUE, envir = envir)
    }

    graph = function() {
        return (R)
    }

    Q = function() {
        ## since R was created with 'inla.sparse.matrix' above, the indices are sorted in a
        ## spesific order. This ordering is REQUIRED for R@x to be interpreted correctly.
        return(exp(theta[1]) * R@x)
    }

    mu = function() return (numeric(0))

    log.norm.const = function() {
        return (norm.const + n/2 * theta[1])
    }

    log.prior = function() {
        return (dgamma(exp(theta[1]), shape = 1, rate = 1, log=TRUE) + theta[1])
    }

    initial = function() {
        return(4)
    }

    if (!length(theta)) theta = initial()
    val = do.call(match.arg(cmd), args = list())

    return (val)
}

## -----------------------------------------------------------------------------
model3 = inla.rgeneric.define(rgeneric.test.opt.1, n=n, optimize = FALSE)
r3 = inla(y ~ -1 + f(idx, model=model3), 
          data = data.frame(y = y, idx = 1:n), control.family = cont.family)

model4 = inla.rgeneric.define(rgeneric.test.opt.2, n=n, optimize = TRUE)
r4 = inla(y ~ -1 + f(idx, model=model4), 
          data = data.frame(y = y, idx = 1:n), control.family = cont.family)

## -----------------------------------------------------------------------------
print(r2$mlik - r1$mlik)
print(r3$mlik - r1$mlik)
print(r4$mlik - r1$mlik)

print(rbind(native = r1$cpu[2], 
            rgeneric.plain = r2$cpu[2], 
            rgeneric.cache = r3$cpu[2], 
            rgeneric.optimze = r4$cpu[2]))

## ----eval=FALSE---------------------------------------------------------------
#  gcc -Wall -fpic -g -O -c -o cgeneric-demo.o cgeneric-demo.c
#  gcc -shared -o cgeneric-demo.so cgeneric-demo.o

## ----eval=FALSE---------------------------------------------------------------
#  cmodel <- inla.cgeneric.define(model = "inla_cgeneric_iid_model",
#                                 shlib = "cgeneric-demo.so", n = n)

## ----eval=FALSE---------------------------------------------------------------
#  rc <- inla(
#      y ~ -1 + f(idx, model = cmodel),
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper = list(prec = list(initial = 12, fixed = TRUE))))

## ----eval=FALSE---------------------------------------------------------------
#  n <- 100
#  y <- rnorm(n)
#  
#  r <- inla(
#      y ~ -1 + f(idx, model = "iid", hyper = list(prec = list(param = c(1, 1)))),
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper = list(prec = list(initial = 12, fixed = TRUE))))
#  
#  rmodel <- inla.rgeneric.define(inla.rgeneric.iid.model, n = n)
#  rr <- inla(
#      y ~ -1 + f(idx, model = rmodel),
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper = list(prec = list(initial = 12, fixed = TRUE))))
#  
#  cmodel <- inla.cgeneric.define(model = "inla_cgeneric_iid_model",
#                                 shlib = "cgeneric-demo.so", n = n)
#  rc <- inla(
#      y ~ -1 + f(idx, model = cmodel),
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper = list(prec = list(initial = 12, fixed = TRUE))))
#  
#  print(cbind(r$mlik, rr$mlik-r$mlik, rc$mlik-r$mlik))
#  print(cbind(r$cpu[2], rr$cpu[2], rc$cpu[2]))

## ----eval=FALSE---------------------------------------------------------------
#  gcc -Wall -fpic -g3 -O   -c -o cgeneric-demo.o cgeneric-demo.c
#  gcc -shared -o cgeneric-demo.so cgeneric-demo.o
#  
#  > n <- 5
#  > cmodel <- inla.cgeneric.define(model = "inla_cgeneric_ar1_model",
#  +                                shlib = "cgeneric-demo.so", n = n)
#  > inla.cgeneric.q(cmodel)
#  $theta
#  [1] 1 1
#  
#  $graph
#  5 x 5 sparse Matrix of class "dgCMatrix"
#  
#  [1,] 1 1 . . .
#  [2,] 1 1 1 . .
#  [3,] . 1 1 1 .
#  [4,] . . 1 1 1
#  [5,] . . . 1 1
#  
#  $Q
#  5 x 5 sparse Matrix of class "dgCMatrix"
#  
#  [1,]  3.45640494 -1.59726402  .           .           .
#  [2,] -1.59726402  4.19452805 -1.59726402  .           .
#  [3,]  .          -1.59726402  4.19452805 -1.59726402  .
#  [4,]  .           .          -1.59726402  4.19452805 -1.59726402
#  [5,]  .           .           .          -1.59726402  3.45640494
#  
#  $mu
#  numeric(0)
#  
#  $log.prior
#  [1] -3.13722036
#  
#  $log.norm.const
#  [1] -1.61423464

## ----eval=FALSE---------------------------------------------------------------
#  typedef enum {
#  	INLA_CGENERIC_VOID = 0,
#  	INLA_CGENERIC_Q,
#  	INLA_CGENERIC_GRAPH,
#  	INLA_CGENERIC_MU,
#  	INLA_CGENERIC_INITIAL,
#  	INLA_CGENERIC_LOG_NORM_CONST,
#  	INLA_CGENERIC_LOG_PRIOR,
#  	INLA_CGENERIC_QUIT
#  } inla_cgeneric_cmd_tp;

## ----eval=FALSE---------------------------------------------------------------
#  #include <assert.h>
#  #if !defined(__FreeBSD__)
#  #include <malloc.h>
#  #endif
#  #include <math.h>
#  #include <stdlib.h>
#  #include <strings.h>
#  
#  #include "cgeneric.h"
#  
#  #define Calloc(n_, type_)  (type_ *)calloc((n_), sizeof(type_))
#  #define SQR(x) ((x)*(x))
#  
#  double *inla_cgeneric_iid_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cgeneric_data_tp * data)
#  {
#  	// this reimplement `inla.rgeneric.iid.model` using cgeneric
#  
#  	double *ret = NULL, prec = (theta ? exp(theta[0]) : NAN), lprec = (theta ? theta[0] : NAN);
#  
#  	assert(!strcasecmp(data->ints[0]->name, "n"));	       // this will always be the case
#  	int N = data->ints[0]->ints[0];			       // this will always be the case
#  	assert(N > 0);
#  
#  	switch (cmd) {
#  	case INLA_CGENERIC_VOID:
#  	{
#  		assert(!(cmd == INLA_CGENERIC_VOID));
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_GRAPH:
#  	{
#  		// return a vector of indices with format
#  		// c(N, M, ii, jj)
#  		// where ii<=jj, ii is non-decreasing and jj is non-decreasing for the same ii
#  		// so like the loop
#  		// for i=0, ...
#  		//     for j=i, ...
#  		//         G_ij =
#  		// and M is the total length while N is the dimension
#  
#  		int M = N;
#  		ret = Calloc(2 + 2 * N, double);
#  		ret[0] = N;				       /* dimension */
#  		ret[1] = M;				       /* number of (i <= j) */
#  		for (int i = 0; i < M; i++) {
#  			ret[2 + i] = i;			       /* i */
#  			ret[2 + N + i] = i;		       /* j */
#  		}
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_Q:
#  	{
#  		// return c(-1, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
#  		int M = N;
#  		ret = Calloc(2 + N, double);
#  		ret[0] = -1;			       /* REQUIRED! */
#  		ret[1] = M;			       /* number of (i <= j) */
#  		for (int i = 0; i < M; i++) {
#  			ret[2 + i] = prec;
#  		}
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_MU:
#  	{
#  		// return (N, mu)
#  		// if N==0 then mu is not needed as its taken to be mu[]==0
#  		ret = Calloc(1, double);
#  		ret[0] = 0;
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_INITIAL:
#  	{
#  		// return c(M, initials)
#  		// where M is the number of hyperparameters
#  		ret = Calloc(2, double);
#  		ret[0] = 1;
#  		ret[1] = 4.0;
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_LOG_NORM_CONST:
#  	{
#  		// return c(NORM_CONST) or a NULL-pointer if INLA should compute it by itself
#  		ret = Calloc(1, double);
#  		ret[0] = N * (-0.9189385332 + 0.5 * lprec);
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_LOG_PRIOR:
#  	{
#  		// return c(LOG_PRIOR)
#  		ret = Calloc(1, double);
#  		ret[0] = -prec + lprec;			       // prec ~ gamma(1,1)
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_QUIT:
#  	default:
#  		break;
#  	}
#  
#  	return (ret);
#  }

## ----eval=FALSE---------------------------------------------------------------
#  #include <assert.h>
#  #if !defined(__FreeBSD__)
#  #include <malloc.h>
#  #endif
#  #include <math.h>
#  #include <stdlib.h>
#  #include <strings.h>
#  
#  #include "cgeneric.h"
#  
#  #define Calloc(n_, type_)  (type_ *)calloc((n_), sizeof(type_))
#  #define SQR(x) ((x)*(x))
#  
#  double *inla_cgeneric_ar1_model(inla_cgeneric_cmd_tp cmd, double *theta, inla_cgeneric_data_tp * data)
#  {
#  	// this reimplement `inla.rgeneric.ar1.model` using cgeneric
#  
#  	double *ret = NULL, prec, lprec, rho, rho_intern;
#  
#  	if (theta) {
#  		lprec = theta[0];
#  		prec = exp(lprec);
#  		rho_intern = theta[1];
#  		rho = 2.0 * exp(rho_intern) / (1.0 + exp(rho_intern)) - 1.0;
#  	} else {
#  		prec = lprec = rho = rho_intern = NAN;
#  	}
#  
#  	assert(!strcasecmp(data->ints[0]->name, "n"));	       // this will always be the case
#  	int N = data->ints[0]->ints[0];			       // this will always be the case
#  	assert(N > 0);
#  
#  	switch (cmd) {
#  	case INLA_CGENERIC_VOID:
#  	{
#  		assert(!(cmd == INLA_CGENERIC_VOID));
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_GRAPH:
#  	{
#  		// return a vector of indices with format
#  		// c(N, M, ii, jj)
#  		// where ii<=jj, ii is non-decreasing and jj is non-decreasing for the same ii
#  		// so like the loop
#  		// for i=0, ...
#  		//     for j=i, ...
#  		//         G_ij =
#  		// and M is the length of ii
#  
#  		int M = N + N - 1, offset, i, k;
#  		ret = Calloc(2 + 2 * m, double);
#  
#  		offset = 2;
#  		ret[0] = N;				       /* dimension */
#  		ret[1] = M;				       /* number of (i <= j) */
#  		for (k = i = 0; i < N; i++) {
#  			ret[offset + k] = i;		       /* i */
#  			ret[offset + M + k++] = i;	       /* j */
#  			if (i < N - 1) {
#  				ret[offset + k] = i;	       /* i */
#  				ret[offset + M + k++] = i + 1; /* j */
#  			}
#  		}
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_Q:
#  	{
#  		// optimized format
#  		// return c(-1, M, Qij) in the same order as defined in INLA_CGENERIC_GRAPH
#  		// where M is the length of Qij
#  
#  		double param = prec / (1.0 - SQR(rho));
#  		int M = N + N - 1;
#  		int offset, i, k;
#  		ret = Calloc(2 + M, double);
#  
#  		offset = 2;
#  		ret[0] = -1;				       /* REQUIRED */
#  		ret[1] = M;
#  		for (i = k = 0; i < N; i++) {
#  			ret[offset + k++] = param * (i == 0 || i == N - 1 ? 1.0 : (1.0 + SQR(rho)));
#  			if (i < N - 1) {
#  				ret[offset + k++] = -param * rho;
#  			}
#  		}
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_MU:
#  	{
#  		// return (N, mu)
#  		// if N==0 then mu is not needed as its taken to be mu[]==0
#  
#  		ret = Calloc(1, double);
#  		ret[0] = 0;
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_INITIAL:
#  	{
#  		// return c(M, initials)
#  		// where M is the number of hyperparameters
#  
#  		ret = Calloc(3, double);
#  		ret[0] = 2;
#  		ret[1] = 1.0;
#  		ret[2] = 1.0;
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_LOG_NORM_CONST:
#  	{
#  		// return c(NORM_CONST) or a NULL-pointer if INLA should compute it by itself
#  
#  		double prec_innovation = prec / (1.0 - SQR(rho));
#  		ret = Calloc(1, double);
#  		ret[0] = N * (-0.5 * log(2.0 * M_PI) + 0.5 * log(prec_innovation)) + 0.5 * log(1.0 - SQR(rho));
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_LOG_PRIOR:
#  	{
#  		// return c(LOG_PRIOR)
#  
#  		ret = Calloc(1, double);
#  		ret[0] = -prec + lprec - 0.5 * log(2.0 * M_PI) - 0.5 * SQR(rho_intern);
#  		break;
#  	}
#  
#  	case INLA_CGENERIC_QUIT:
#  	default:
#  		break;
#  	}
#  
#  	return (ret);
#  }

## ----eval=FALSE---------------------------------------------------------------
#  typedef struct {
#  	int n_ints;
#  	inla_cgeneric_vec_tp **ints;
#  
#  	int n_doubles;
#  	inla_cgeneric_vec_tp **doubles;
#  
#  	int n_chars;
#  	inla_cgeneric_vec_tp **chars;
#  
#  	int n_mat;
#  	inla_cgeneric_mat_tp **mats;
#  
#  	int n_smat;
#  	inla_cgeneric_smat_tp **smats;
#  } inla_cgeneric_data_tp;

## ----eval=FALSE---------------------------------------------------------------
#  	assert(!strcasecmp(data->ints[0]->name, "n")); // this will always be the case
#  	int N = data->ints[0]->ints[0];		           // this will always be the case

## ----eval=FALSE---------------------------------------------------------------
#  typedef struct
#  {
#  	char *name;
#  	int len;
#  	int *ints;
#  	double *doubles;
#  	char *chars;
#  }
#  	inla_cgeneric_vec_tp;

## ----eval=FALSE---------------------------------------------------------------
#  /*
#   *      matrix storage is stored row by row.
#   *
#   *      In R the (default) storage is column by column, like
#   *      > matrix(1:6,2,3)
#   *      [,1] [,2] [,3]
#   *      [1,]    1    3    5
#   *      [2,]    2    4    6
#   *
#   *      hence
#   *      > c(matrix(1:6,2,3))
#   *      [1] 1 2 3 4 5 6
#   *
#   *      while in cgeneric, the matrix elements 'x', is stored as
#   *      x[] = { 1, 3, 5, 2, 4, 6 }
#   */
#  typedef struct {
#  	char *name;
#  	int nrow;
#  	int ncol;
#  	double *x;
#  } inla_cgeneric_mat_tp;

## ----eval=FALSE---------------------------------------------------------------
#  /*
#   * sparse matrix format, stored used 0-based indices.
#   * If the matrix is not a 'dgTMatrix', it is converted using
#   * 'inla.as.sparse()'
#   *
#   * the matrix is stored in the order it appears, like
#   *
#   *      > A <- inla.as.sparse(matrix(c(1,2,3,0,0,6),2,3))
#   *      > A
#   *      2 x 3 sparse Matrix of class "dgTMatrix"
#   *      [1,] 1 3 .
#   *      [2,] 2 . 6
#   *      > cbind(i=A@i, j=A@j, x=A@x)
#   *            i j x
#   *      [1,]  0 0 1
#   *      [2,]  1 0 2
#   *      [3,]  0 1 3
#   *      [4,]  1 2 6
#   *
#   *      If you want to have this matrix stored this column by column,
#   *      then swap @i and @j, and rearrange @x, before passing them into
#   *      'inla.cgeneric.define'.
#   *
#   *      For example, if we want to pass only the upper half of a
#   *      symmetric sparse matrix, stored by column, then we can do
#   *
#   *      A <- inla.as.sparse(A)
#   *      ii <- A@i
#   *      A@i <- A@j
#   *      A@j <- ii
#   *      idx <- which(A@i <= A@j)
#   *      A@i <- A@i[idx]
#   *      A@j <- A@j[idx]
#   *      A@x <- A@x[idx]
#   *
#   *      before passing it on into 'inla.cgeneric.define'
#   */
#  typedef struct {
#  	char *name;
#  	int nrow;
#  	int ncol;
#  	int n;						       /* number of triplets (i,j,x) */
#  	int *i;
#  	int *j;
#  	double *x;
#  } inla_cgeneric_smat_tp;

## ----eval=FALSE---------------------------------------------------------------
#  library(mvtnorm)
#  n <- 10
#  A <- matrix(rnorm(n^2), n, n)
#  Q <- A %*% t(A)
#  S <- solve(Q)
#  S <- S / exp(mean(log(diag(S))))
#  Q <- solve(S)
#  
#  Cmatrix <- inla.as.sparse(Q)
#  s <- 0.1
#  y <- c(rmvnorm(1, sigma = S)) + rnorm(n, sd = s)
#  
#  r <- inla(
#      y ~ -1 + f(idx, model = "generic0", Cmatrix = Cmatrix,
#                hyper = list(prec = list(prior = "loggamma",
#                                         param = c(1, 1),
#                                         initial = 4,
#                                         fixed = FALSE))),
#  	num.threads = "1:1",
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper =
#  		list(prec = list(initial = log(1/s^2), fixed = TRUE))))
#  
#  ## pass in the Cmatrix so its a good fit for
#  ## the cgeneric-implementation, so we store 'Cmatrix'
#  ## column-wise, and upper half only (diagonal included)
#  ii <- Cmatrix@i
#  Cmatrix@i <- Cmatrix@j
#  Cmatrix@j <- ii
#  idx <- which(Cmatrix@i <= Cmatrix@j)
#  Cmatrix@i <- Cmatrix@i[idx]
#  Cmatrix@j <- Cmatrix@j[idx]
#  Cmatrix@x <- Cmatrix@x[idx]
#  
#  cmodel <- inla.cgeneric.define(model = "inla_cgeneric_generic0_model",
#  	                           shlib = "cgeneric-demo.so",
#                                 n = n, Cmatrix = Cmatrix, debug = FALSE)
#  rc <- inla(
#      y ~ -1 + f(idx, model = cmodel),
#  	num.threads = "1:1",
#      data = data.frame(y, idx = 1:n),
#      control.family = list(hyper = list(
#                       prec = list(initial = log(1/s^2), fixed = TRUE))),
#      verbose = TRUE)
#  
#  print(r$mode$theta - rc$mode$theta)
#  print(r$mode$x - rc$mode$x)
#  print(r$mlik - rc$mlik)

## ----eval=FALSE---------------------------------------------------------------
#  > print(r$mode$theta - rc$mode$theta)
#  Log precision for idx
#       -8.710710942e-06
#  
#  > print(r$mode$x - rc$mode$x)
#   [1] -6.988149615e-08  9.218204538e-08  1.151077143e-07 -3.683823807e-08
#   [5] -8.793919748e-08 -9.518123112e-08  2.305469839e-08 -2.043514372e-08
#   [9] -7.478532010e-08  1.064667019e-07 -6.988848433e-08  9.219126358e-08
#  [13]  1.151192251e-07 -3.684192190e-08 -8.794799145e-08 -9.519074928e-08
#  [17]  2.305700385e-08 -2.043718722e-08 -7.479279862e-08  1.064773486e-07
#  
#  > print(r$mlik - rc$mlik)
#                                                   [,1]
#  log marginal-likelihood (integration) 2.070899825e-07
#  log marginal-likelihood (Gaussian)    1.150937410e-05

## ----setup, include=FALSE, cache=FALSE------------------------------
library(knitr)
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, error = FALSE, message = FALSE, cache =FALSE)
knitr::opts_chunk$set(fig.path="figures/a-note-on-values/")
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")

## ----eval=FALSE-----------------------------------------------------
# y ~ ... + f(year, model = "ar1", ...)

## -------------------------------------------------------------------
year <- 2000:2025

## -------------------------------------------------------------------
values <- sort(unique(year[!is.na(year)]))
values

map.year.to.t <- function(year, values) {
	a <- numeric(length(year))
	for(i in seq_along(year)) 
		a[i] = which(year[i] == values)
	return (a) 
}
yy <- c(2000, 2020, 2025)
print(cbind(year=yy, t=map.year.to.t(yy, values)))

## -------------------------------------------------------------------
year <- c(2000:2020, 2023:2025)

## -------------------------------------------------------------------
values <- sort(unique(year[!is.na(year)]))
values

## -------------------------------------------------------------------
yy <- c(2000, 2020, 2025)
print(cbind(year = yy, t = map.year.to.t(yy, values)))

## ----eval=FALSE-----------------------------------------------------
# time <- c(1.1, 2.2, 3.3, 4.4, 10.0)

## ----eval=FALSE-----------------------------------------------------
# y ~ ... + f(year, model = "ar1", values = 2000:2025)

## ----eval=FALSE-----------------------------------------------------
# y ~ ... + f(year, model = "ar1", values = 1990:2035)

## -------------------------------------------------------------------
values <- sort(unique(values[!is.na(values)]))

## ----plot=TRUE, out.width = "70%", fig.align = "center"-------------
year <- 2000:2025
n <- length(year)
phi <- 0.9
phi.intern <- inla.models()$latent$ar1$hyper$theta2$to.theta(phi)
x <- scale(arima.sim(n, model=list(ar=phi)))
s <- 0.3
y <- x + rnorm(n, sd=s)
plot(year, y, pch=19)

## ----plot=TRUE, out.width = "70%", fig.align = "center"-------------
r <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year))
plot(year, y, pch=19)
lines(year, r$summary.random$year$mean)

## -------------------------------------------------------------------
## this is correct
y[10] <- NA
r <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year))  ## missing data included

## this is wrong
rr <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year)[-10,]) ## missing data removed
## we compare the results
r$mlik - rr$mlik

## -------------------------------------------------------------------
rrr <- inla(y ~ -1 + f(year, model="ar1", 
	                  values = 2000:2025,  ## values added
	                  hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year)[-10,]) ## missing data removed
## we compare the results
r$mlik - rrr$mlik

## ----plot=TRUE, out.width = "70%", fig.align = "center"-------------
values <- 1990:2035
r <- inla(y ~ -1 + f(year, model="ar1",
	                 values = values,  ## values added
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = list(y=y, year=year, values=values))
plot(year, y, pch=19, xlim=range(values))
lines(values, r$summary.random$year$mean)

## ----setup, include=FALSE-----------------------------------------------------
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE,warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/conditional-logit/")

## ---- warning=FALSE-----------------------------------------------------------
  ### --- 0. Loading libraries --- ####
library(INLA)
library(dplyr)
library(ggplot2)
library(ggtern)

## -------------------------------------------------------------------
set.seed(201803)
inla.seed = sample.int(n=1E6, size=1)
options(width=70, digits=3)

## -------------------------------------------------------------------
### --- 1. Simulation --- ####
# Parameters for the simulation
D <- 3
N <- 1000
sigma2 <- c(0.5, 0.4)
cov_param <- 0.1
sigma_diag <- sqrt(sigma2 + cov_param)
hypers_lik <- data.frame(hypers = c(sigma2, cov_param),
                         name1 = c("sigma2.1", "sigma2.2", "gamma"))
# We create the correlation parameters based on the previous idea
# We are going to have ((D-1)^2 - (D-1))/2 rhos
rho <- diag(1/sigma_diag) %*% matrix(cov_param, D-1, D-1) %*% diag(1/sigma_diag)
diag(rho) <- 1
rho

## -------------------------------------------------------------------
x = runif(N)-0.5
# - mean 0 to not affect intercept
betas = matrix(c(-1, 3, -1, 5), nrow = D-1, byrow = TRUE)
X <- data.frame(1, x) %>% as.matrix(.)
lin.pred <- X %*% t(betas) 

## -------------------------------------------------------------------
### ------- 1.2.3. Constructing the likelihood --- ####
Sigma <- matrix(sigma_diag, ncol = 1) %*% matrix(sigma_diag, nrow = 1)
Sigma <- Sigma*rho

lin.pred %>%
  apply(., 1, function(z)
    MASS::mvrnorm( n  = 1,
             mu = z,
             Sigma = Sigma)) %>%
  t(.)-> alry

## -------------------------------------------------------------------
  y.simplex <- compositions::alrInv(alry)
  y.simplex <- as.numeric(t(y.simplex)) %>% matrix(., ncol = D, byrow = TRUE)
  colnames(y.simplex) <- paste0("y", 1:D)  
  data <- data.frame(alry, y.simplex, x)
colnames(data)[1:(D-1)] <- c(paste0("alry.", 1:(D-1)))
data %>% head(.)

## ---- fig.cap = "Simulated data in the Simplex and also using alr-coordinates in terms of x"----
### Ternary plot
ggtern::ggtern(data = data,
       aes(y1, y2, y3)) + 
   #geom_mask() +
   geom_point(aes(fill = x), shape=21, size=2) + 
   theme_bw() +
   theme_showarrows() +
   theme_clockwise() -> p_y

### Alr coordinates
data %>% 
  tidyr::pivot_longer(., cols = ,starts_with("alr"), 
                      names_to = "y.names", values_to = "y.resp") %>%
  ggplot(data = .) +
  geom_point(aes(x = x, y = y.resp, fill = x), shape = 21, size = 2) +
  ylab("alr") +
  facet_wrap(~y.names) +
  theme_bw() +
  theme(legend.position = "bottom") -> p_alr

#pdf("simulated_data.pdf", width = 8, height = 6)
grid.arrange(arrangeGrob(p_y,
                         p_alr + theme(legend.position = "none")))
#dev.off()

## -------------------------------------------------------------------
  data$id.z <- 1:dim(data)[1]

## -------------------------------------------------------------------
data_ext <- data %>%
  tidyr::pivot_longer(., cols = all_of(paste0("alry.", 1:(D-1))),
                      names_to  = "y.names",
                      values_to = "y.resp") %>%
  .[order(ordered(.$y.names)),]
data_ext$y.names <- ordered(data_ext$y.names)
head(data_ext)

## -------------------------------------------------------------------
names_y <- paste0("alry.", 1:(D-1))
1:length(names_y) %>%
  lapply(., function(i){
    data_ext %>%
      dplyr::filter(y.names == names_y[i]) -> data_comp_i
    #Response
    y_alr <- matrix(ncol = names_y %>% length(.), nrow = dim(data_comp_i)[1])
    y_alr[, i] <- data_comp_i$y.resp
  }) -> y.resp

1:length(names_y) %>%
  lapply(., function(i){
    y_aux <- data_ext %>%
      dplyr::select(y.resp, y.names) %>%
      dplyr::filter(y.names == names_y[i]) %>%
      dplyr::select(y.resp) %>%
      as.matrix(.)
    aux_vec <- rep(NA, (D-1))
    aux_vec[i] <- 1
    kronecker(aux_vec, y_aux)
  }) -> y_list

y_tot <- do.call(cbind, y_list)
y_tot %>% head(.)

## -------------------------------------------------------------------
variables <- c("intercept", data %>%
                 dplyr::select(starts_with("x")) %>%
                 colnames(.))
id.names <- paste0("id.", variables)
id.variables <- rep(data_ext$y.names %>% as.factor(.) %>% as.numeric(.), 
                    length(variables)) %>%
  matrix(., ncol = length(variables), byrow = FALSE)
colnames(id.variables) <- id.names

variables
id.variables %>% head(.)

## -------------------------------------------------------------------
stk.est <- inla.stack(data    = list(resp = y_tot),
                      A       = list(1),
                      effects = list(cbind(data_ext %>%
                                             dplyr::select(starts_with("x")),
                                           data_ext %>%
                                             dplyr::select(starts_with("id.z")),
                                           id.variables,
                                           intercept = 1)),
                      tag     = 'est')

## -------------------------------------------------------------------
  # Have different parameters for fixed effects, and do not include spatial random effects.
list_prior <- rep(list(list(prior = "pc.prec", param = c(1, 0.01))), D-1)

### Fitting the model
formula.typeII <- resp ~ -1 +
  f(id.intercept, intercept,
    model   = "iid",
    initial = log(1/1000),
    fixed   = TRUE) +
  f(id.x, x,
    model   = "iid",
    initial = log(1/1000),
    fixed   = TRUE) +
  f(id.z,
    model = "iid",
    hyper = list(prec = list(prior = "pc.prec",
                             param = c(1, 0.01))), constr = TRUE)
model.typeII <- inla(formula.typeII,
                     family         = rep("gaussian", D-1),
                     data           = inla.stack.data(stk.est),
                     control.compute = list(config = TRUE),
                     control.predictor = list(A = inla.stack.A(stk.est),
                                              compute = TRUE),
                     control.family = list_prior,
                     inla.mode = "experimental" ,
                     verbose = FALSE)

## -------------------------------------------------------------------
### Posterior distribution of the fixed effects
data_fixed <- rbind(data.frame(inla.smarginal(model.typeII$marginals.random$id.x$index.1),
                               alr = "alr(y1/y3)",
                               var = "x",
                               param = "beta1",
                               real  = betas[1,2]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.x$index.2),
                               alr = "alr(y2/y3)",
                               var = "x",
                               param = "beta1",
                               real  = betas[2,2]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.intercept$index.1),
                               alr = "alr(y1/y3)",
                               var = "intercept",
                               param = "beta0",
                               real = betas[1,1]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.intercept$index.2),
                               alr = "alr(y2/y3)",
                               var = "intercept",
                               param = "beta0",
                               real = betas[2,1]))

p_fixed <- ggplot() +
  geom_line(data = data_fixed, aes(x = x, y = y), size = 0.9) +
  #ggtitle("Effect of the covariate bio12") +
  theme_bw() +
  geom_vline(data = data_fixed, aes(xintercept = real), col = "red4") +
 # scale_color_manual(values=c("#E75F00", "#56B4E9"))+
  theme(legend.position = "bottom") +
  facet_wrap(~param + alr, ncol = D-1, scales = "free") +
  xlab(expression(beta^(d))) +
  ylab(expression(p(beta^(d) *'|'* theta))) +
  theme(legend.title = element_blank())

#pdf("posterior_fixed.pdf", width = 6, height = 5)
p_fixed
#dev.off()

## ---- fig.asp = 0.5-------------------------------------------------
### Posterior distribution of the hyperparameters
prec <- list(sigma2.1 = model.typeII$marginals.hyperpar$`Precision for the Gaussian observations`,
             sigma2.2 = model.typeII$marginals.hyperpar$`Precision for the Gaussian observations[2]`,
             gamma = model.typeII$marginals.hyper$`Precision for id.z`)

hyper <- lapply(1:length(prec),
                function(x){
                  inla.smarginal(inla.tmarginal(prec[[x]], fun = function(y)(1/y))) %>%
                    data.frame(.)
                })
names(hyper) <- names(prec)

hyper.df <- lapply(1:length(hyper),
                   function(x){
                     cbind(data.frame(hyper[[x]]), name1 = names(hyper)[x])
                   })  %>%
  do.call(rbind.data.frame, .)

hyper.df$name1 <- ordered(hyper.df$name1,
                          levels = c("sigma2.1", "sigma2.2",
                                     "gamma"))
p.hyper <- ggplot(hyper.df) +
  geom_line(aes(x = x, y = y)) +
  geom_vline(data = hypers_lik, aes(xintercept = hypers), col = "red4") +
  facet_wrap(~ name1, scales = "free") +
  theme_bw() +
  xlab(expression(theta)) +
  ylab(expression(p(theta*'|'*y)))

#pdf("marginals_hyperpar.pdf", width = 6, height = 3)
print(p.hyper)
#dev.off()

## -------------------------------------------------------------------
sim <- 1000
x.pred <- seq(-0.5, 0.5, 0.3)
n.pred <- length(x.pred)
cat("\n ----------------------------------------------- \n")
cat("Creating the data.frame for predictions \n")

data_pred <- data.frame(intercept = 1,
                        x = rep(x.pred, D-1))
id.z.pred  <- rep((N + 1):(N + n.pred), D - 1) #random effect z to model the correlation

# Category
id.cat_pred <- rep(1:(D - 1), rep(n.pred, D - 1))
#Index for covariates
variables_pred <- c("intercept", data_pred %>% 
                      dplyr::select(starts_with("x")) %>% 
                      colnames(.))
id.names_pred <- paste0("id.", variables_pred)
id.variables_pred <- rep(id.cat_pred, length(variables_pred)) %>% 
  matrix(., ncol = length(variables_pred), byrow = FALSE)
colnames(id.variables_pred) <- id.names_pred

## -------------------------------------------------------------------
stk.pred <- inla.stack(data    = list(resp = matrix(NA, ncol = D - 1, 
                                                    nrow = n.pred*(D - 1))),
                       A       = list(1),
                       effects = list(cbind(data_pred,
                                            id.z = id.z.pred,
                                            id.variables_pred)),
                       tag     = 'pred')
### --- Total stack
stk <- inla.stack(stk.est, stk.pred)

## -------------------------------------------------------------------
mod.pred <- inla(formula.typeII, 
                 family         = rep("gaussian", D - 1),
                 data              = inla.stack.data(stk), 
                 control.compute   = list(config = TRUE),
                 control.predictor = list(A = inla.stack.A(stk), compute = TRUE, link = 1),
                 control.mode      = list(theta = model.typeII$mode$theta, restart = TRUE), 
                 control.family = list_prior,   
                 num.threads       = 2,
                 inla.mode = "experimental" , 
                 verbose           = FALSE)

## -------------------------------------------------------------------
pred.values.mean <- mod.pred$summary.fitted.values$mean[inla.stack.index(stk, 'pred')$data] %>% 
  matrix(., ncol = D - 1, byrow = FALSE)

post_sim_pred <- inla.posterior.sample(n = sim, result = mod.pred)
post_sim_predictor <- inla.posterior.sample.eval(fun = function(...){
  APredictor}, post_sim_pred, return.matrix = TRUE)
post_sim_idz <- inla.posterior.sample.eval(fun = function(...){
  id.z}, post_sim_pred, return.matrix = TRUE)

ind.pred <- inla.stack.index(stk, 'pred')$data
ind.idz <- inla.stack.index(stk, 'est')$data #This is the shared random effect
ind.idz <- ind.idz[1:(length(ind.idz)/(D - 1))]

post_sim_predictor[ind.pred, ] <- post_sim_predictor[ind.pred, ]- 
  kronecker(rep(1, D-1), post_sim_idz[-ind.idz,])

post_sim_pred_alr <- post_sim_predictor[ind.pred,]

#Computing mean and sd
pred_alr_summary <- t(apply(post_sim_pred_alr, 1, function(x){c(mean(x), sd(x))}))
pred_alr_summary <- data.frame(pred_alr_summary, 
                               y.names = rep(names_y, rep(n.pred, D-1)),
                               x.pred = rep(x.pred, D-1))
colnames(pred_alr_summary)[1:2] <- c("mean", "sd")

pred_alr_summary

## -------------------------------------------------------------------
###  Prediction in the simplex --- #####
apply(post_sim_predictor[ind.pred,], 2, function(x){
  alr_pred <- matrix(x, ncol = D - 1)
  pred_simplex <- compositions::alrInv(alr_pred)
  as.numeric(t(pred_simplex)) #Byrows
}) -> post_sim_pred_simplex

#Computing credible intervals
pred_simplex_summary <- t(apply(post_sim_pred_simplex, 1, function(x){c(mean(x), sd(x))}))
pred_simplex_summary <- data.frame(pred_simplex_summary, 
                                   y.names = rep(c("y1", "y2", "y3"), n.pred),
                                   x.pred  = rep(x.pred, rep(D, n.pred)))
colnames(pred_simplex_summary)[1:2] <- c("mean", "sd")

pred_simplex_summary

## RCSId = $Id: Tokyo-compare.R,v 1.7 2009/09/28 08:36:56 hrue Exp $
##
## this example show how to compare inla-results with MCMC using the
## buildt-in sampler using the simple Tokyo-example; see demo(Tokyo)

data(Tokyo)
formula = y ~ f(time, model="rw2", cyclic=TRUE, param=c(1,0.0001)) - 1

## we need to keep the results in a known place
wd.inla = tempfile()

## get the results using some important arguments..
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo,
    ##
    ## where to place the results
    ##
    working.directory = wd.inla,
    ##
    ## we need to keep the results
    ##
    keep=TRUE,
    ##
    ## this is to prevent the results to be stored in binary-format
    ##
    inla.arg = "")

## same with mcmc-results...
wd.mcmc = tempfile()

## same as before
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo,
    ##
    ## where to place the results
    ##
    working.directory = wd.mcmc,
    ##
    ## we need to keep the results
    ##
    keep = TRUE,
    ##
    ## secret options to inla: 5000 iterations using thinning of 10
    ## and the scaling of the hyperparmeter proposal is 1.
    ##
    inla.arg = "-m mcmc -N 5000 -T 10 -S 1")

## enter the results to the comparison-routine, which interactively
## let you compare the results
inla.compare.results(wd.inla, wd.mcmc)

## remove the working-files
unlink(wd.inla, recursive = TRUE)
unlink(wd.mcmc, recursive = TRUE)

## Load the data
data(Tokyo)
summary(Tokyo)

## Define the model
formula = y ~ f(time, model="rw2", cyclic=TRUE, param=c(1,0.0001)) - 1

## The call to inla
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo)

## Once more: the call to inla in verbose mode
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo, verbose = TRUE)

## Summarise the results
summary(result)

## Plot the results
plot(result)

if (FALSE) {
    ## only for classic mode
    h = inla.hyperpar(result)
    summary(h)
    plot(h)
}

## ----setup, include=FALSE-------------------------------------------
set.seed(1234)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="classic")
if (file.exists("myinit.R")) source("myinit.R")
library(ggplot2)
library(sn)
library(knitr)
set.seed(123)
knitr::opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/jmarginal/'
               , fig.pos='H'
               , background='#ffffff'
               , results='hold'
               , eval=TRUE)

## ----eval=F---------------------------------------------------------
# inla.setOption(inla.mode="classic")

## ----eval=F---------------------------------------------------------
# inla.setOption(inla.mode="compact")

## -------------------------------------------------------------------
n = 100
x = rnorm(n, mean = 1, sd = 0.3)
xx = rnorm(n, mean = -2, sd = 0.3)
y = rpois(n, lambda = exp(x + xx))
r = inla(y ~ 1 + x + xx,
         data = data.frame(y, x, xx), 
	     family = "poisson")

## -------------------------------------------------------------------
selection = list(Predictor = 3:4, x = 1, xx = 1)

## -------------------------------------------------------------------
rs = inla(y ~ 1 + x + xx,
          data = data.frame(y, x, xx), 
	      family = "poisson",
	      inla.mode = "classic",
          control.compute = list(return.marginals.predictor = TRUE), 
	      control.predictor = list(compute = TRUE),
		  selection = selection)

## -------------------------------------------------------------------
#summary(rs$selection)
print(rs$selection)

## -------------------------------------------------------------------
ns = 10000
xs = inla.rjmarginal(ns, rs) ## or rs$selection
str(xs)

## -------------------------------------------------------------------
pairs(t(xs$samples[, 1:3000]), cex = 0.1)

## -------------------------------------------------------------------
hist(xs$samples["Predictor:3",], n = 300, prob = TRUE, 
     main = 'Histogram of Predictor:3', xlab = 'Samples 
     from the joint and linear predictor marginal (black straight line)')
lines(inla.smarginal(rs$marginals.linear.predictor[[3]]), lwd = 3)

## -------------------------------------------------------------------
dxs = inla.1djmarginal(jmarginal = rs$selection)
str(dxs)

## -------------------------------------------------------------------
ggplot(data = data.frame(y = xs$samples["Predictor:3",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`Predictor:3`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Predictor:3"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["Predictor:4",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`Predictor:4`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Predictor:4"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["x:1",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`x:1`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for x:1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["xx:1",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`xx:1`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for xx:1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red"))  

## -------------------------------------------------------------------
summary(rs$selection)

## -------------------------------------------------------------------
A = matrix(c(1,1,0,0,0,0,1,1), nrow = 2, ncol = 4, byrow = T)
rownames(A) <- c("Test1", "Test2")
A

## -------------------------------------------------------------------
m = inla.tjmarginal(jmarginal = rs, A = A)
m
class(m)


## -------------------------------------------------------------------
dxs.lin = inla.1djmarginal(jmarginal = m)
str(dxs.lin)

fun1 <- function(...) {Predictor[1]+Predictor[2]}
fun2 <- function(...) {x+xx}

xs.lin1 = inla.rjmarginal.eval(fun1, xs)
xs.lin2 = inla.rjmarginal.eval(fun2, xs)

ggplot(data = data.frame(y = xs.lin1[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs.lin$Test1), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Lin1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs.lin2[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs.lin$Test2), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Lin2"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

## -------------------------------------------------------------------
summary(m)

## -------------------------------------------------------------------
fun.exp <- function(x) exp(x)

fun5 <- function(...) {exp(x)}
fun6 <- function(...) {exp(xx)}
fun7 <- function(...) {exp(x+xx)}

tdx <- inla.tmarginal(fun = fun.exp, marginal = dxs$`x:1`)
tdxx <- inla.tmarginal(fun = fun.exp, marginal = dxs$`xx:1`)
tdx.lin <- inla.tmarginal(fun = fun.exp, marginal = dxs.lin$Test2)

tx = inla.rjmarginal.eval(fun5, xs)
txx = inla.rjmarginal.eval(fun6, xs)
tx.lin = inla.rjmarginal.eval(fun7, xs)

ggplot(data = data.frame(y = tx[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdx), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(x:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = txx[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdxx), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(xx:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = tx.lin[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdx.lin), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(x:1+xx:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

## -------------------------------------------------------------------
expx = inla.zmarginal(marginal = tdx, silent = TRUE)
expxx = inla.zmarginal(marginal = tdxx, silent = TRUE)
expx.lin = inla.zmarginal(marginal = tdx.lin, silent = TRUE)

exp.summaries = rbind(expx, expxx, expx.lin)
exp.summaries

## ----setup, include=FALSE, cache=FALSE------------------------------
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, error = FALSE, message = FALSE)

## ----libs, warning=FALSE, message=FALSE, errors=FALSE---------------
# libraries
library(maps)
library(ggplot2)
library(sf)
library(terra)
library(tidyterra) # raster plotting
library(tidyr)
library(scales)
library(inlabru)
library(INLA)
library(fmesher)
library(dplyr)
# Note: the 'splancs' package also needs to be installed,
# but doesn't need to be loaded

# set option
select <- dplyr::select
options(scipen = 99999)
options(max.print = 99999)
options(stringsAsFactors = FALSE)

## ----echo=F---------------------------------------------------------
inla.setOption(num.threads = "1:1")
inla.setOption(smtp = "taucs")
inla.setOption(inla.mode = "experimental")
if (file.exists("myinit.R")) source("myinit.R")

## ----proj, warning=F, message=FALSE---------------------------------
# define a crs
epsg6703km <- paste(
  "+proj=aea +lat_0=23 +lon_0=-96 +lat_1=29.5",
  "+lat_2=45.5 +x_0=0 +y_0=0 +datum=NAD83",
  "+units=km +no_defs"
)

# make a base map
states <- maps::map("state", plot = FALSE, fill = TRUE) %>%
  sf::st_as_sf() %>%
  filter(ID %in% c(
    "texas", "oklahoma", "kansas", "missouri",
    "arkansas", "louisiana"
  )) %>%
  sf::st_make_valid() %>%
  sf::st_transform(epsg6703km) %>%
  sf::st_make_valid()

## ----data-init, eval=T, cache=FALSE---------------------------------
# get data 
## download_dat <- read.csv(paste0(
##      "https://raw.github.com/tmeeha/inlaSVCBC",
##      "/master/code/modeling_data.csv"))
download_dat <- read.csv("data/modeling_data.csv.gz")

# select subset
count_dat <- download_dat %>%
  select(
    circle, bcr, state, year, std_yr, count, log_hrs,
    lon, lat, obs
  ) %>%
  mutate(year = year + 1899) %>%
  filter(
    state %in% c(
      "TEXAS", "OKLAHOMA", "KANSAS", "MISSOURI",
      "ARKANSAS", "LOUISIANA"
    ),
    year >= 1987
  )

## ----data-subset, eval=TRUE-----------------------------------------
count_dat <- count_dat %>%
  mutate(site_idx = as.numeric(factor(paste(circle, lon, lat)))) %>%
  group_by(site_idx) %>%
  mutate(n_years = n()) %>%
  filter(n_years >= 20) %>%
  ungroup() %>%
  mutate(
    std_yr = year - max(year),
    obs = seq_len(nrow(.)),
    site_idx = as.numeric(factor(paste(circle, lon, lat))),
    year_idx = as.numeric(factor(year)),
    site_year_idx = as.numeric(factor(paste(circle, lon, lat, year)))
  ) %>%
  st_as_sf(coords = c("lon", "lat"), crs = 4326, remove = FALSE) %>%
  st_transform(epsg6703km) %>%
  mutate(
    easting = st_coordinates(.)[, 1],
    northing = st_coordinates(.)[, 2]
  ) %>%
  arrange(circle, year)

## ----data_map, fig.pos='b', fig.align='center'----------------------
# map it
ggplot() +
  geom_sf(
    data = count_dat %>% filter(year_idx %in% seq(1, 30, 3)),
    aes(col = log(count + 1))
  ) +
  geom_sf(data = states, fill = NA) +
  coord_sf(datum = NA) +
  facet_wrap(~year) +
  scale_color_distiller(palette = "Spectral") +
  theme_bw()

## ----spat_dat-------------------------------------------------------
# make a set of distinct study sites for mapping
site_map <- count_dat %>%
  select(circle, easting, northing) %>%
  distinct() %>%
  select(circle, easting, northing)

# save coordinates as matrices for later
unique_coords <- count_dat %>%
  select(circle, easting, northing) %>%
  distinct() %>%
  select(easting, northing) %>%
  st_drop_geometry() %>%
  as.matrix()
all_coords <- as.matrix(st_coordinates(count_dat))

## ----mesh, cache=FALSE----------------------------------------------
# make a hull and mesh for spatial model
hull <- fm_nonconvex_hull(
  count_dat,
  convex = 200,
  concave = 350
)
mesh <- fm_mesh_2d(
  boundary = hull, max.edge = c(100, 600), # km inside and outside
  cutoff = 50, offset = c(100, 300),
  crs = fm_crs(count_dat)
) # cutoff is min edge

## ----mesh-plot------------------------------------------------------
# plot it
ggplot() +
  gg(data = mesh) +
  geom_sf(data = site_map, col = "darkgreen", size = 1) +
  geom_sf(data = states, fill = NA) +
  theme_bw() +
  labs(x = "", y = "")

## ----spde-----------------------------------------------------------
# make spde
spde <- inla.spde2.pcmatern(
  mesh = mesh,
  prior.range = c(500, 0.5),
  prior.sigma = c(1, 0.5)
)

## ----indices--------------------------------------------------------
# make index sets
alpha_idx <- inla.spde.make.index(name = "alpha", n.spde = mesh$n)
eps_idx <- inla.spde.make.index(name = "eps", n.spde = mesh$n)
tau_idx <- inla.spde.make.index(name = "tau", n.spde = mesh$n)

## ----projector------------------------------------------------------
# make projector matrices
A_alpha <- inla.spde.make.A(mesh = mesh, loc = count_dat)
A_eps <- inla.spde.make.A(
  mesh = mesh, loc = count_dat,
  weights = count_dat$log_hrs
) # note weights argument
A_tau <- inla.spde.make.A(
  mesh = mesh, loc = count_dat,
  weights = count_dat$std_yr
) # note weights argument

## ----stack1---------------------------------------------------------
# stack observed data
stack_fit <- inla.stack(
  tag = "obs",
  data = list(count = as.vector(count_dat$count)), # response from data frame
  effects = list(data.frame(
    intercept = 1,
    kappa = count_dat$site_idx
  ), # predictors from data frame
  alpha = alpha_idx, # or index sets if spatial
  eps = eps_idx,
  tau = tau_idx
  ),
  A = list(
    1, # a value of 1 is given for non-spatial terms
    A_alpha,
    A_eps,
    A_tau
  )
)

## ----prior----------------------------------------------------------
# iid prior
pc_prec <- list(prior = "pcprec", param = c(1, 0.1))

## ----form-----------------------------------------------------------
# formula
svc_form <- count ~ -1 +
  f(kappa, model = "iid", constr = TRUE, hyper = list(prec = pc_prec)) +
  f(alpha, model = spde) +
  f(eps, model = spde) +
  f(tau, model = spde)

## ----run, eval=T, cache=FALSE---------------------------------------
res <- inla(svc_form,
  family = "nbinomial",
  data = inla.stack.data(stack_fit), # stack the stack
  control.predictor = list(
    A = inla.stack.A(stack_fit),
    compute = FALSE
  ), # must define A
  control.compute = list(waic = TRUE, cpo = TRUE, config = TRUE),
  control.inla = list(strategy = "adaptive", int.strategy = "eb"),
  verbose = FALSE
)

## ----load_res, echo=F, eval=F---------------------------------------
# load("spatial_cbc/code/spde_vignette_v3.RData")

## ----vc-------------------------------------------------------------
# view results
res$summary.hyperpar[-1, c(1, 2)]

## ----alpha_res------------------------------------------------------
summary(exp(res$summary.random$alp$"0.5quant")) # exp(alpha) posterior median

## ----eps_res--------------------------------------------------------
summary(res$summary.random$eps$mean) # epsilon

## ----tau_res--------------------------------------------------------
summary((exp(res$summary.random$tau$"0.5quant") - 1) * 100) # (exp(tau)-1)*100

## ----map_grid-------------------------------------------------------
# get easting and northing limits
bbox <- fm_bbox(hull[[1]])
grd_dims <- round(c(x = diff(bbox[[1]]), y = diff(bbox[[2]])) / 25)

# make mesh projector to get model summaries from the mesh to the mapping grid
mesh_proj <- fm_evaluator(
  mesh,
  xlim = bbox[[1]], ylim = bbox[[2]], dims = grd_dims
)

## ----map_vals-------------------------------------------------------
# pull data
kappa <- data.frame(
  median = exp(res$summary.random$kappa$"0.5quant"),
  range95 = exp(res$summary.random$kappa$"0.975quant") -
    exp(res$summary.random$kappa$"0.025quant")
)
alph <- data.frame(
  median = exp(res$summary.random$alpha$"0.5quant"),
  range95 = exp(res$summary.random$alpha$"0.975quant") -
    exp(res$summary.random$alpha$"0.025quant")
)
epsi <- data.frame(
  median = res$summary.random$eps$"0.5quant",
  range95 = (res$summary.random$eps$"0.975quant" -
    res$summary.random$eps$"0.025quant")
)
taus <- data.frame(
  median = (exp(res$summary.random$tau$"0.5quant") - 1) * 100,
  range95 = (exp(res$summary.random$tau$"0.975quant") -
    exp(res$summary.random$tau$"0.025quant")) * 100
)

# loop to get estimates on a mapping grid
pred_grids <- lapply(
  list(alpha = alph, epsilon = epsi, tau = taus),
  function(x) as.matrix(fm_evaluate(mesh_proj, x))
)

## ----map_rast-------------------------------------------------------
# make a terra raster stack with the posterior median and range95
out_stk <- rast()
for (j in 1:3) {
  mean_j <- cbind(expand.grid(x = mesh_proj$x, y = mesh_proj$y),
    Z = c(matrix(pred_grids[[j]][, 1], grd_dims[1]))
  )
  mean_j <- rast(mean_j, crs = epsg6703km)
  range95_j <- cbind(expand.grid(X = mesh_proj$x, Y = mesh_proj$y),
    Z = c(matrix(pred_grids[[j]][, 2], grd_dims[1]))
  )
  range95_j <- rast(range95_j, crs = epsg6703km)
  out_j <- c(mean_j, range95_j)
  terra::add(out_stk) <- out_j
}
names(out_stk) <- c(
  "alpha_median", "alpha_range95", "epsilon_median",
  "epsilon_range95", "tau_median", "tau_range95"
)
out_stk <- terra::mask(
  out_stk,
  terra::vect(sf::st_union(states)),
  updatevalue = NA,
  touches = FALSE
)

## ----svc_plot_construction, warning=F, message=F--------------------
make_plot_field <- function(data_stk, scale_label) {
  ggplot(states) +
    geom_sf(fill = NA) +
    coord_sf(datum = NA) +
    geom_spatraster(data = data_stk) +
    labs(x = "", y = "") +
    scale_fill_distiller(
      scale_label,
      palette = "Spectral",
      na.value = "transparent"
    ) +
    theme_bw() +
    geom_sf(fill = NA)
}
make_plot_site <- function(data, scale_label) {
  ggplot(states) +
    geom_sf() +
    coord_sf(datum = NA) +
    geom_sf(data = data, size = 1, mapping = aes(colour = value)) +
    scale_colour_distiller(scale_label, palette = "Spectral") +
    labs(x = "", y = "") +
    theme_bw() +
    geom_sf(fill = NA)
}

# medians
# fields alpha_s, epsilon_s, tau_s
pa <- make_plot_field(
  data_stk = out_stk[["alpha_median"]],
  scale_label = "posterior\nmedian\nexp(alpha_s)"
)
pe <- make_plot_field(
  data_stk = out_stk[["epsilon_median"]],
  scale_label = "posterior\nmedian\nepsilon_s"
)
pt <- make_plot_field(
  data_stk = out_stk[["tau_median"]],
  scale_label = "posterior\nmedian\n100(exp(tau_s)-1)"
)
# sites kappa_s
ps <- make_plot_site(
  data = cbind(site_map, data.frame(value = kappa$median)),
  scale_label = "posterior\nmedian\nexp(kappa_s)"
)
# range95
# fields alpha_s, epsilon_s, tau_s
pa_range95 <- make_plot_field(
  data_stk = out_stk[["alpha_range95"]],
  scale_label = "posterior\nrange95\nexp(alpha_s)"
)
pe_range95 <- make_plot_field(
  data_stk = out_stk[["epsilon_range95"]],
  scale_label = "posterior\nrange95\nepsilon_s"
)
pt_range95 <- make_plot_field(
  data_stk = out_stk[["tau_range95"]],
  scale_label = "posterior\nrange95\n100(exp(tau_s)-1)"
)
# sites kappa_s
ps_range95 <- make_plot_site(
  data = cbind(site_map, data.frame(value = kappa$range95)),
  scale_label = "posterior\nrange95\nexp(kappa_s)"
)

## ----svc_plots, warning=F, message=F, fig.width=8, fig.align='center'----
# plot together
multiplot(ps, pa, pe, pt, cols = 2)

## ----svc_plots_range95, warning=F, message=F, fig.width=8, fig.align='center'----
# plot together
multiplot(ps_range95, pa_range95, pe_range95, pt_range95, cols = 2)

library(INLA)
library(JointModel)
data1 <- prostate
data2 <- dropout
ng <- nrow(data1)
ns <- nrow(data2)
data1 = data1[order(data1$VisitTime),] #Just for "nice" graphs

## prepare the response variable
library(INLA)
y.longPSA <- c(data1$logPSA.postRT , rep(NA,ng),rep(NA, ns))
y.etaPSA <- c(rep(NA,ng),rep(0,ng),rep(NA,ns))
y.survPSA <- inla.surv(time = c(rep(NA, ng),rep(NA,ng), data2$DropTime),
                       event = c(rep(NA, ng),rep(NA,ng),data2$Status))
Yjoint <- list(y.longPSA, y.etaPSA,y.survPSA)
N <- length(unique(prostate$ID))
linear.covariatePSA <- data.frame(mu = as.factor(c(rep(1, ng),rep(1, ng),rep(2, ns))),
                                  b13.PSAbase = c(data1$logPSA.base,data1$logPSA.base, rep(0, ns)))
random.covariatePSA <- list(V1 = c(data1$VisitTime,rep(NA,ng),rep(NA,ns)),
                            u = c(rep(NA,ng),data1$ID,rep(NA,ns)),
                            w = c(rep(NA,ng),rep(-1,ng),rep(NA,ns)),
                            b.eta = c(rep(NA,ng),rep(NA,ng),data2$ID2))
## Entire linear predictor
joint.dataPSA <- c(linear.covariatePSA, random.covariatePSA)
joint.dataPSA$Y <- Yjoint
saveRDS(joint.dataPSA,  file = "jointdataPSA.rds")

## ----setup, include=FALSE-----------------------------------------------------
library(INLA)
library(knitr)
library(formatR)
library(fastGHQuad)
library(pracma)
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")
if (file.exists("myinit.R")) source("myinit.R")
set.seed(20202)
knitr::opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/group-cv'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/group-cv/'
               , fig.pos='H'
               , background='#ffffff'
               , results='hold'
               , eval=TRUE)

# gh.x = c(-6.995680e+00,-6.275079e+00,-5.673961e+00,-5.133596e+00,-4.631560e+00,-4.156272e+00,-3.700743e+00,-3.260321e+00,-2.831680e+00,-2.412318e+00,-2.000259e+00,-1.593886e+00,-1.191827e+00,-7.928770e-01,-3.959427e-01,-2.722499e-16,3.959427e-01,7.928770e-01,1.191827e+00,1.593886e+00,2.000259e+00,2.412318e+00,2.831680e+00,3.260321e+00,3.700743e+00,4.156272e+00,4.631560e+00,5.133596e+00,5.673961e+00,6.275079e+00,6.995680e+00)
# gh.w = c(4.618968e-22,5.110609e-18,5.899556e-15,1.860374e-12,2.352492e-10,1.461199e-08,5.043713e-07,1.049860e-05,1.395209e-04,1.233683e-03,7.482800e-03,3.184723e-02,9.671795e-02,2.121328e-01,3.387727e-01,3.957786e-01,3.387727e-01,2.121328e-01,9.671795e-02,3.184723e-02,7.482800e-03,1.233683e-03,1.395209e-04,1.049860e-05,5.043713e-07,1.461199e-08,2.352492e-10,1.860374e-12,5.899556e-15,5.110609e-18,4.618968e-22)

gh.x = fastGHQuad::gaussHermiteData(51)$x
gh.w = fastGHQuad::gaussHermiteData(51)$w

inla.lik.pred_real = function(y,args = NULL,args.string,eta.mean,eta.sd,eta.skewness){
    if(is.null(args)){args = INLA:::inla.likelihood.parser(args.string)}
    args$linear.predictor = sqrt(2)*eta.sd*gh.x + eta.mean
    hfunc = inla.likelihood(type = "d",args)
    res = unlist(lapply(1:length(y),FUN = function(i){1/sqrt(pi)*sum(gh.w*hfunc(y[i]))}))
    return(res)
}

inla.lik.pred = function(y,args = NULL,args.string,eta.mean,eta.sd,eta.skewness){
    res = dnorm(y,mean = eta.mean,sd = sqrt(eta.sd^2 + 1/exp(args$theta)))
    return(res)
}

## ----message=FALSE, warning=FALSE,echo = FALSE--------------------------------
n_rat = 10
n_each = 10
n = n_rat*n_each
id = rep(1:n_rat, each = n_each)
mu_rat = rep(x = rnorm(n_rat,mean = 30,sd = 1),each = n_each)
sd_measure = .1
y = rnorm(n = n, mean = mu_rat,sd = sd_measure)
plot(id,y,main = "Data against its group id")

## ----message = FALSE, warning=FALSE-------------------------------------------
data = data.frame(y = c(y,rep(NA,n_rat+1)),id = c(id,1:n_rat,n_rat+1))
formula = y ~ 1 + f(id,model = "iid")
res = inla(formula = formula,
           data = data,
           family = "normal"
           )

## ----message = FALSE, warning=FALSE, echo = FALSE-----------------------------
res = inla(formula = formula,
           data = data.frame(y = c(y,rep(NA,n_rat+1)), id = c(id,1:n_rat,n_rat+1)),
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )
nconfig = res$misc$configs$nconfig
weights = unlist(lapply(1:nconfig,FUN = function(i){res$misc$configs$config[[i]]$log.posterior}))
weights = exp(weights)/sum(exp(weights))
eps = 0.001
q = seq(eps,1-eps,eps)
preddensity = vector(mode = "list",length = n_rat+1)
for(rat_idx in c(1,11)){
    predictor_idx = n + rat_idx
    mode_idx = which.max(weights)
    args = INLA:::inla.likelihood.parser(res$misc$configs$config[[1]]$arg.str[1])
    for(mode_idx in mode_idx){
        args$theta = res$misc$configs$config[[mode_idx]]$theta[1]
        mode_eta_mu = res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"mean"]
        mode_eta_sd = sqrt(res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"variance"])
        eta_eval = mode_eta_mu + seq(-5,5)*mode_eta_sd
        args$linear.predictor = eta_eval
        qfunc = inla.likelihood(type = "q",args)
        y_eval = c()
        for(i in 1:length(q)){
            y_eval = c(y_eval,qfunc(q[i]))
        }
    }
    y_eval = sort(y_eval)
    
    preddensity_conditional = list(mode = "list",length = nconfig)
    preddensity[[rat_idx]]$y_eval = y_eval
    preddensity[[rat_idx]]$density = numeric(length(y_eval))
    for(config_idx in 1:nconfig){
        args$theta = res$misc$configs$config[[config_idx]]$theta[1]
        preddensity_conditional[[config_idx]] = inla.lik.pred(y_eval,
                                                  args = args,
                                                  eta.mean = res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"mean"],
                                                  eta.sd = sqrt(res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"variance"])
                                                  )
        preddensity[[rat_idx]]$density  = preddensity[[rat_idx]]$density +  weights[config_idx]*preddensity_conditional[[config_idx]]
    }
}
plot(preddensity[[1]]$y_eval,preddensity[[1]]$density,
     main = "Predictive Density of A Measure from Rat 1",
     xlab = "weight",
     ylab = "density",
     type="l")
plot(preddensity[[11]]$y_eval,preddensity[[11]]$density,
     main = "Predictive Density of A Measure from An Unobserved Rat",
     xlab = "weight",
     ylab = "density",
     type="l")

## ----message = FALSE, warning=FALSE-------------------------------------------
loocv_res = inla.group.cv(result = res)
ULOOCV = mean(log(loocv_res$cv[1:n]))

groups = lapply(1:n,FUN = function(i){which(id == id[i])})
lgocv_res = inla.group.cv(result = res,groups = groups)
ULGOCV = mean(log(lgocv_res$cv[1:n]))

print(paste0("ULOOCV is ", round(ULOOCV,5),"."))
print(paste0("ULGOCV is ", round(ULGOCV,5),"."))

## ----message = FALSE, warning=FALSE-------------------------------------------
lgocv_auto_res = inla.group.cv(result = res,num.level.sets = 1)
ULGOCV_auto = mean(log(lgocv_auto_res$cv[1:n]))
print(paste0("ULGOCV_auto is ", round(ULGOCV_auto,5),"."))
print(paste0("The group for data point 1 is:"))
print(paste(lgocv_auto_res$groups[[1]]$idx,collapse=" "))

## -----------------------------------------------------------------------------
num.level.sets = 3
# We intentionally create some replicated correlations since it is not rare to have 
#|corr(eta_i,eta_j)| == |corr(eta_i,eta_k)|, where j != k.
C_i = c(1,1,0.9,0.9,0.8,0.8,-0.1,-0.1,0,0)
# L_i contains the correlation levels.
L_i = sort(unique(abs(C_i)),decreasing = T)
# Now we construct I_i
I_i = c()
for (l in 1:num.level.sets){
  I_i = c(I_i,which(abs(C_i) == L_i[l]))
}
print(paste(I_i,collapse=" "))

## ----message=FALSE, warning=FALSE,echo = FALSE--------------------------------
n_rat = 10
n_each = 10
n = n_rat*n_each
id = rep(1:n_rat, each = n_each)
x = rnorm(n)
mu_rat = rep(x = rnorm(n_rat,mean = 30,sd = 2),each = n_each) + x
sd_measure = .1
y = rnorm(n = n, mean = mu_rat,sd = sd_measure)
plot(id,y,main = "Data against its group id")
plot(x,y,main = "Data against x")

## ----message = FALSE, warning=FALSE-------------------------------------------
data = data.frame(y = y,id = id,id2 = id,x = x)
formula = y ~ 1 + x + f(id, model = "iid") + f(id2, W = x,model = "iid")
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )

## ----message = FALSE, warning=FALSE-------------------------------------------
formula_alt =  y ~ 1 + x + f(id, model = "iid")
res_alt = inla(formula = formula_alt,
           data = data,
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )

## ----message = FALSE, warning=FALSE-------------------------------------------
lgocv_res = inla.group.cv(result = res,num.level.sets = 1)
lgocv_res_alt = inla.group.cv(result = res_alt,group.cv = lgocv_res)

## ----warning = FALSE----------------------------------------------------------
lgocv_res = inla.group.cv(result = res,num.level.sets = 1)
print(paste(lgocv_res$groups[[1]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[2]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[3]]$idx,collapse=" "))

## ----warning = FALSE----------------------------------------------------------
lgocv_res = inla.group.cv(result = res,num.level.sets = 1,strategy = "prior",keep = "id")
print(paste(lgocv_res$groups[[1]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[2]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[3]]$idx,collapse=" "))

## ----warning = FALSE----------------------------------------------------------
lgocv_res = inla.group.cv(result = res,num.level.sets = 1,select = c(12,53))

## ----warning = FALSE----------------------------------------------------------
groups = vector(mode = "list",length = n)
groups[[12]] = which(id==id[12])
groups[[53]] = which(id==id[53])
lgocv_res = inla.group.cv(result = res,groups = groups)

## ----warning = FALSE----------------------------------------------------------
control.gcpo = list(enable = TRUE,num.level.sets = 1)
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(control.gcpo = control.gcpo,config = TRUE)
           )

## ----warning = FALSE----------------------------------------------------------
control.gcpo = list(enable = TRUE,num.level.sets = 1,strategy = "prior",keep = "id")
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(control.gcpo = control.gcpo,config = TRUE)
           )
# 1. choose the testing point
testing.point = 1
y_observed = y[testing.point]
# 2. get the hyperparameter density
nconfigs = res$misc$configs$nconfig
weights = unlist(lapply(X = 1:nconfigs,
	FUN = function(config.idx){
    res$misc$configs$config[[config.idx]]$log.posterior 
    + res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,3]
    }))
weights = exp(weights)/sum(exp(weights))
#3. get samples of y_i|y_{-I_i},theta
n = 1e6
n_theta = as.numeric(rmultinom(n = 1,size = n,prob = weights))
y_sample = c()
for(config.idx in 1:nconfigs){
    if(n_theta[config.idx]>0){
		eta = rnorm(n_theta[config.idx],
			mean = res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,1],
			sd = sqrt(res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,2]))
         sd = sqrt(1/exp(res$misc$configs$config[[config.idx]]$theta[1]))
         y_sample_config = rnorm(n = n_theta[config.idx],mean = eta,sd = sd)
         y_sample = c(y_sample,y_sample_config)
    }
}
#4 We can use y_sample to compute the empirical cumulative function then evaluate CRPS
F_y = function(y){sum(y_sample<=y)/n} #inefficient but straightforward...
yy = seq(min(y_sample)-10,max(y_sample)+10,0.01)
FF = unlist(lapply(yy,F_y))
CRPS = pracma::trapz(yy,(FF-(yy>=y_observed))^2)

INLA::inla.setOption(fmesher.evolution.warn = TRUE)
INLA::inla.setOption(fmesher.evolution.verbosity = "stop")

## Redo the Leuk-example using continuous domain frailty
library(INLA)

data(Leuk)
ls()

#############################################################
## model without spatial frailty
model1 <- inla.surv(Leuk$time, Leuk$cens) ~ 0 +
    age + sex + wbc + tpi 

## defining the baseline model, and its hyperparameter prior
pprior <- list(
    theta = list(
        prior = "pc.prec",
        param = c(0.5, 0.01)))
hmodel <- list(
    model = 'rw1',
    n.intervals = 50,
    constr = FALSE, 
    scale.model = TRUE,
    hyper = pprior
)

## data expansion to use it later
## expand the data (internally done in inla)
data.expanded <- inla.coxph(
    formula = model1,
    data = Leuk, 
    control.hazard = hmodel)
str(data.expanded, 1)
data.expanded$formula
names(data.expanded$data)

## fit the model
fit1 <- inla(
    formula = data.expanded$formula, 
    data = c(data.expanded$data,
             data.expanded$data.list), 
    family = "poisson",
    E = E..coxph
)

fit1$summary.hyper[, 1:2]

fit1$summary.fixed[, 1:2]

fit1$mlik

if(FALSE)
    plot(fit1)

rplot <- function(r, ...) {
    n <- nrow(r)
    y <- c(r[, 4], r[n:1, 6], r[1, 4])
    mc <- match.call()
    if(any(names(mc) == 'ylim')) {
        plot(r$ID, r$mean, type = 'l',
             ...)        
    } else { 
        plot(r$ID, r$mean, type = 'l', 
             ylim = range(y),
             ...)
    }
    polygon(r$ID[c(1:n, n:1, 1)], y, 
            col = gray(0.5, 0.5),
            border = gray(0.5, 0.5))
}

par(mfrow = c(1, 1), mar = c(3, 3, 1, 0.1), mgp = c(2, 1, 0), las = 1, bty = 'n')
rplot(fit1$summary.random$baseline, 
      main = 'Baseline with no spatial frailty',
      xlab = 'time', ylab = 'baseline')

### Model with spatial frailty

## get info from the map
library(sf)
if(!any(ls()=="NEmap")) ## older INLA versions does not contains this
    NEmap <- st_sfc(lapply(nwEngland@polygons[[1]]@Polygons, function(p)
        st_polygon(list(p@coords))))

bb <- matrix(st_bbox(NEmap), 2)
bb
rxy <- apply(bb, 1, diff)
rxy
rr <- mean(rxy)

## Create a grid to be used later
g0 <- st_make_grid(
    NEmap,
    cellsize = rr/100
)
g1 <- st_intersection(g0, st_buffer(NEmap, rr/100))

## take only the full pixels
ipoly <- which(sapply(g1, class)[2, ] == "POLYGON")
grid <- st_sf(
    data.frame(ID1 = ipoly),
    geom = st_sfc(g1[ipoly])
)

## non-convex boundaries around the map
library(fmesher)
bnd1 <- fm_nonconvex_hull_inla(
    grid,
    convex = rr * 0.05,
    concave = rr * 0.1)

## build a mesh accounting for boundaries
mesh <- fm_mesh_2d(
    boundary = bnd1, 
    max.edge = rr * c(0.05, 0.15),
    cutoff = rr * 0.02)

mesh$n

plot(mesh)
plot(st_geometry(NEmap), add = TRUE)

## SPDE random effect design matrix
Alocs <- inla.spde.make.A(
    mesh = mesh,
    loc = cbind(data.expanded$data$xcoord,
                data.expanded$data$ycoord)
)
stopifnot(sum(Alocs) == nrow(data.expanded$data))

## SPDE model and hyper-prior definition
rr
spde <- inla.spde2.pcmatern(
    mesh = mesh,
    prior.range = c(rr*0.05, 0.01),
    prior.sigma = c(0.5, 0.01)
)

## model formula
model2 <- update(
    data.expanded$formula, 
    .~. + f(spatial, model = spde, A.local = Alocs))

## fit the model 
fit2 <- inla(
    formula = model2,
    data = c(data.frame(data.expanded$data, spatial = NA),
             data.expanded$data.list), 
    family = "poisson", E = E..coxph
)

fit2$summary.hyperpar[, 1:2]
fit2$summary.fixed[, 1:2]

data.frame(m1 = fit1$mlik,
           m2 = fit2$mlik)

## plot(fit2)

par(mfrow = c(2, 1), mar = c(3, 3, 1, 0.1), mgp = c(2, 1, 0), las = 1, bty = 'n')
rplot(fit1$summary.random$baseline, ylim = c(-12, -7),
      main = 'Baseline with no spatial frailty',
      xlab = 'time', ylab = 'baseline')
rplot(fit2$summary.random$baseline, ylim = c(-12, -7),
      main = 'Baseline under spatial frailty',
      xlab = 'time', ylab = 'baseline')

## evaluate the spatial frailty at the grid locations
gproj <- fm_evaluator(mesh = mesh, loc = st_centroid(grid))

grid$frailty.mean <- fm_evaluate(gproj, field = fit2$summary.random$spatial$mean)
grid$frailty.sd <- fm_evaluate(gproj, field = fit2$summary.random$spatial$sd)

if(FALSE)
    plot(grid[, 3:4], border = 'transparent') 

library(ggplot2)
library(ggpubr)

gg0 <- ggplot(grid) + theme_minimal() 
gg1 <- geom_sf(data = NEmap, fill = 'transparent')

ggarrange(
    gg0 + geom_sf(aes(fill = frailty.mean), color = 'transparent') +
    scale_fill_distiller(
        direction = 1,
        palette = "RdBu"
    ) + gg1,
    gg0 + geom_sf(aes(fill = frailty.sd), color = 'transparent') +
    scale_fill_distiller(
        direction = 1,
        palette = "PuBu"
    ) + gg1 + geom_point(
                  aes(x = xcoord, y = ycoord),
                  data = Leuk, cex = 0.1) + xlab("") + ylab("")
)

## ----apptainer check, eval=FALSE----------------------------------------------
# $ apptainer version

## ----apptainer, eval=FALSE----------------------------------------------------
# # To pull the docker image
# $ apptainer pull your_container.sif docker://rocker/geospatial:latest
# # once the sif is downloaded, get into an interactive shell
# $ apptainer shell your_container.sif
# # Inside the interactive shell, one can install INLA on a personal library path
# $ R --verbose
# # Now, in a R environment
# > options(repos = c(
#     INLA = 'https://inla.r-inla-download.org/R/testing',
#     CRAN = 'https://cloud.r-project.org'))
# > install.packages("INLA")
# # One may be asked to create a personal library path.
# > install.packages("inlabru")
# # quit R
# > q()

## ----exec, eval=FALSE---------------------------------------------------------
# # To execute an R file with apptainer
# $ apptainer exec ./your_container.sif Rscript --no-restore --no-save --verbose file_to_run.R

## ----settings, include=FALSE---------------------------------------------
library(knitr);  library(INLA);  library(fields) 
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE, fig.align='center')
knit_hooks$set(small.mar = function(before, options, envir) {
    if (before) par(mar = c(0.1, 0.1, .1, .1))  # smaller margin on top and right
})
knit_hooks$set(mar3311 = function(before, options, envir) {
    if (before) par(mar = c(3, 3, 1, 1), mgp=c(2, 1, 0))  # smaller margin on top and right
})
set.seed(1) 

## ----locations-----------------------------------------------------------
n = 200  ## number of location points
coo = matrix(runif(2*n), n) ## location points
k <- 10;   s2s <- 0.7 ## RF parameters
R <- s2s*exp(-k*as.matrix(dist(coo))) ## covariance matrix

## ----field simulate------------------------------------------------------
s <- drop(rnorm(n)%*%chol(R)) ## one RF realization

## ----covariate and outcome-----------------------------------------------
x <- runif(n)  ## covariate
beta <- 1:2  ## regression coefficients
lin.pred <- beta[1] + beta[2]*x + s ## linear predictor
s2e <- 0.3 ## error variance (nugget)
y <- lin.pred + rnorm(n, 0, sqrt(s2e))  ## the outcome

## ----mesh, fig.width=4.5, fig.height=4.6, small.mar=TRUE-----------------
r0.1 <- sqrt(0.5 * 8)/k ## distance with correlation around 0.139
mesh <- inla.mesh.2d( ## 2D mesh creator
  loc=coo, ## provided locations 
  max.edge=c(r0.1/3, r0.1), ## maximum edge length (inner, outer): mandatory 
  offset=c(r0.1/3, r0.1*2), ## outer extension
  cutoff=r0.1/10) ## good to have >0
par(mar=c(0,0,1,0))
plot(mesh, asp=1) ## plot the mesh
points(coo, col='red') ## add the points 

## ---- projector, fig.width=8, fig.height=4.5, small.mar=TRUE-------------
image(A <- inla.spde.make.A( ## projector creator
    mesh=mesh, ## provide the mesh
    loc=coo) ### locations where to project the field
    ) ## an 'n' by 'm' projector matrix

## ----SPDE model----------------------------------------------------------
spde <- inla.spde2.matern( ## precision components creator
    mesh=mesh, ## mesh supplied
    alpha=1.5) ## smoothness parameter

## ----data stack----------------------------------------------------------
stk.e <- inla.stack( ## stack creator
  data=list(y=y),  ## response
  effects=list(## two elements:
    data.frame(b0=1, x=x), ## regressor part
    s=1:spde$n.spde),  ## RF index
  A=list(## projector list of each effect
    1, ## for the covariates
    A), ## for the RF
  tag='est') ## tag

## ----fitting-------------------------------------------------------------
formula <- y ~ 0 + b0 + x + ## fixed part
  f(s, model=spde) ## RF term
res <- inla( ## main function in INLA package
  formula, ## model formula
  data=inla.stack.data(stk.e), ## dataset
  control.predictor=list( ## inform projector needed in SPDE models
    A = inla.stack.A(stk.e))) ## projector from the stack data

## ----fixed---------------------------------------------------------------
round(res$summary.fixed, 4) 

## ----nugget, fig.width=3.5, fig.height=3, mar3311=TRUE-------------------
m.prec <- res$marginals.hyperpar$'Precision for the Gaussian observations' ## the marginal
post.s2e <- inla.tmarginal(## function to compute a tranformation 
  function(x) 1/x, ## inverse transformation
  m.prec) ## marginal to be applied
### visualize it
plot(post.s2e, type='l', ylab='Density', 
     xlab=expression(sigma[e]^2))
abline(v=s2e, col=2) ## add value used to generate the data

## ----rf------------------------------------------------------------------
rf <- inla.spde.result( ## function to compute the 'interpretable' parameters
    inla=res, ## the inla() output
    name='s', ## name of RF index set
    spde=spde, ## SPDE model object
    do.transf=TRUE) ## to user scale

## ----parameters, fig.width=10, fig.height=3------------------------------
par(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0))
plot(rf$marginals.var[[1]], ty='l', 
     xlab=expression(sigma[s]^2), yla='Density')
abline(v=s2s, col=2) ## add the true value
plot(rf$marginals.kap[[1]], type='l',
     xlab=expression(kappa), ylab='Density')
abline(v=k, col=2) ## add the true value
plot(rf$marginals.range[[1]], type='l', 
     xlab='range nominal', ylab='Density')
abline(v=sqrt(0.5 * 8)/k, col=2) ## add the 'true' value

## ----project-------------------------------------------------------------
gproj <- inla.mesh.projector( ## projector builder
  mesh, ## mesh used to define the model
  xlim=0:1, ylim=0:1, ## limits where to create the grid
  dims=c(300,300)) ## grid dimension
## project the mean and the SD
g.mean <- inla.mesh.project(gproj, res$summary.random$s$mean)
g.sd <- inla.mesh.project(gproj, res$summary.random$s$sd)

## ----visualize, fig.width=9, fig.height=5--------------------------------
par(mfrow=c(1,2), mar=c(0,0,1,0))
require(fields)
image.plot(g.mean, asp=1, main='RF posterior mean', axes=FALSE, horizontal=TRUE)
image.plot(g.sd, asp=1, main='RF posterior SD', axes=FALSE, horizontal=TRUE)

## ----prediction scenario-------------------------------------------------
tcoo <- rbind(c(0.3,0.3), c(0.5,0.5), c(0.7,0.7))
dim(Ap <- inla.spde.make.A(mesh=mesh, loc=tcoo)) 
x0 <- c(0.5, 0.5, 0.5)

## ----prediction stack----------------------------------------------------
stk.pred <- inla.stack(
  tag='pred', ## will be used to collect the posterior marginals
  data=list(y=NA), ## response set as NA
  effects=list(
    data.frame(x=x0, b0=1), ## covariate scenario
    s=1:spde$n.spde), ## same as before
  A=list(1, Ap)) ## covariate and target locations field projectors

## ----refitting-----------------------------------------------------------
stk.full <- inla.stack(stk.e, stk.pred) ## join both data
p.res <- inla(
  formula, data=inla.stack.data(stk.full), ## using the full data 
  control.predictor=list(compute=TRUE, ## compute the predictor
                         A=inla.stack.A(stk.full)), ## from full data
  control.compute = list(return.marginals.predictor = TRUE), 
  control.mode=list(theta=res$mode$theta)) ## use the mode previously found

## ----prediction index----------------------------------------------------
pred.ind <- inla.stack.index( ## stack index extractor function
  stk.full, ## the data stack to be considered
  tag='pred' ## which part of the data to look at
  )$data ## which elements to collect
ypost <- p.res$marginals.fitted.values[pred.ind]

## ----predicted, fig.width=10, fig.height=3-------------------------------
xyl <- apply(Reduce('rbind', ypost), 2, range)
par(mfrow=c(1,3), mar=c(3,3,2,1), mgp=c(2,1,0))
for (j in 1:3) 
  plot(ypost[[j]], type='l', xlim=xyl[,1], ylim=xyl[,2], 
       xlab='y', ylab='density', main=paste0('y', j))

## ----marginals-----------------------------------------------------------
apropos('marginal')

## ----marginals playing---------------------------------------------------
inla.qmarginal(c(0.15, 0.7), ypost[[1]]) ## quantiles
inla.emarginal(function(x) x^2, ypost[[1]]) - ## E(y^2) -
  inla.emarginal(function(x) x, ypost[[1]])^2 ## E(y)^2 to compute the variance
inla.pmarginal(inla.qmarginal(0.3, ypost[[1]]), ypost[[1]]) 
inla.zmarginal(ypost[[1]]) ## posterior summary

data(Leuk)

## graph file
g <- system.file("demodata/Leuk.graph", package="INLA")

## model formula
formula <- inla.surv(Leuk$time, Leuk$cens) ~ sex + age +
    f(inla.group(wbc), model="rw2")+
    f(inla.group(tpi), model="rw2")+
    f(district, model = "besag", graph = g)

## fit the model
result <- inla(formula, family="coxph", data=Leuk)
summary(result)

if(FALSE) {## old code for the map
  
  ## get the function to plot the map
  source(system.file("demodata/Leuk-map.R", package="INLA"))
  Leuk.map(result$summary.random$district$mean)
  
} 

## use the new sf map
library(ggplot2)
ggplot() + theme_minimal() + 
  geom_sf(aes(fill = result$summary.random$district$mean), 
          data = NEmap) +
    labs(fill = "log frailty")

#plot(result)

## Note: check the demo/LeukSPDE.R for continuous spatial domain frailty

## ----setup, include=FALSE-----------------------------------------------------
set.seed(123)
library(knitr)
knitr::opts_chunk$set(echo = TRUE, results = 'show', 
	fig.keep = 'high', fig.align = 'center', fig.pos = 'h')
knitr::opts_chunk$set(fig.path="figures/bgev/")

library(INLA)
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")

## ---- echo=F, message=F, warning=FALSE----------------------------------------
colFmt = function(x,color){
  outputFormat = opts_knit$get("rmarkdown.pandoc.to")
  if(outputFormat == 'latex')
    paste("\\textcolor[HTML]{",color,"}{",x,"}",sep="")
  else if(outputFormat == 'html')
    paste("<font color='",color,"'>",x,"</font>",sep="")
  else
    x
}

library(INLA)
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}

## ----pressure, echo=FALSE, fig.cap="\\label{fig:distributionH.pdf} bGEV distribution (H, black) constructed from distributions F (Frechét, red), G (Gumbel, green) and Beta weight function $p$ (purple). The shaded area is the mixing area, where F and G are merged.", out.width = '50%'----
knitr::include_graphics("fig_distribution1.png")

## -----------------------------------------------------------------------------
n = 1000
x = rnorm(n, sd=0.5) # we generate values for x from a N(0,0.5^2) dist.
eta.x = 1 + 0.4*x

## -----------------------------------------------------------------------------
spread = 0.3
tail = 0.1

## -----------------------------------------------------------------------------
p.alpha = 0.5
p.beta = 0.25

## -----------------------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = spread, alpha = p.alpha, beta = p.beta, 
                     xi = tail)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma, shape = par$xi)

## -----------------------------------------------------------------------------
hyper.spread = list(initial = 1,
                    fixed=FALSE,
                    prior = "loggamma",
                    param = c(3, 3))

## -----------------------------------------------------------------------------
tail.interval = c(0, 0.5)
tail.intern = map.tail(tail, tail.interval, inverse=TRUE)

## -----------------------------------------------------------------------------
hyper.tail = list(initial = tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= FALSE)


## -----------------------------------------------------------------------------
hyper.tail = list(initial = if (tail == 0.0) -Inf else tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= if (tail == 0.0) TRUE else FALSE)


## -----------------------------------------------------------------------------

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail)

## -----------------------------------------------------------------------------
control.bgev = list(q.location = p.alpha,
                    q.spread = p.beta,
                    # quantile levels for the mixing part
                    q.mix= c(0.05, 0.20), 
                    # the Beta(s1, s2) mixing distribution parameters. 
                    # Hard-coded for the moment: s1=s2=5
                    beta.ab = 5)

## -----------------------------------------------------------------------------
null.matrix = matrix(nrow = n, ncol= 0)
spread.x = null.matrix
tail.x = null.matrix

## -----------------------------------------------------------------------------
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x

## ---- message=F, eval = T, warning=FALSE--------------------------------------
r1 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE, safe=TRUE)

## -----------------------------------------------------------------------------
round(r1$summary.fixed,4)
round(r1$summary.hyperpar,4)

## -----------------------------------------------------------------------------
n = 1000
x1 = rnorm(n)
eta.x = 1 + 0.4*x1

## -----------------------------------------------------------------------------
x2 = rnorm(n, sd= 0.2)
s.x = exp(0.1 + 0.3*x2)
x3 = runif(n,-0.25,1)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi

## ---- eval = T----------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])

## -----------------------------------------------------------------------------
hyper.beta1 = hyper.beta2 = list(prior = "normal",
                                 param = c(0, 300),
                                 initial = 0)

## -----------------------------------------------------------------------------

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2)

## -----------------------------------------------------------------------------
spread.x = x2
tail.x = x3
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x

## -----------------------------------------------------------------------------
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)

## ---- message=F, eval = T, warning=FALSE--------------------------------------
r2 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE, safe=TRUE)

## ---- eval = T----------------------------------------------------------------
round(r2$summary.fixed,4)
round(r2$summary.hyperpar,4)

## ---- eval = T----------------------------------------------------------------
n = 1000
x = rnorm(n)
z1 = seq(0, 6, length.out = n)
z2 = 1:n
p = 2 # AR order
pacf = runif(p)
phi = inla.ar.pacf2phi(pacf)
eta.x = 1 + 0.4*x + sin(z1) + c(scale(arima.sim(n, model = list(ar = phi))))

## -----------------------------------------------------------------------------
x2 = rnorm(n, sd = 0.2)
x4 = rnorm(n, sd = 0.2)
s.x = exp(0.1 + 0.3*x2 + x4)
x3 = runif(n,-0.2, 0.2)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi

## ---- eval = T----------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])

## -----------------------------------------------------------------------------
hyper.beta1 = hyper.beta2 = hyper.beta3 = list(prior = "normal",
                                               param = c(0, 300),
                                               initial = 0)
hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2,
                  beta3 = hyper.beta3)

## ---- message=F, eval = T, warning=FALSE--------------------------------------
spread.x = x2
spread.xx = x4
tail.x = x3
# With this change of variable it is easier to keep track of the effect 
# of the covariates in each parameter, but it is not needed.
formula = inla.mdata(y, cbind(spread.x, spread.xx), tail.x) ~ -1 + intercept + x + 
	f(z1, model = "rw1", scale.model=TRUE, constr=TRUE,
		hyper = list(prec = list(prior = "pc.prec", 
		param = c(0.1, 0.01)))) +
    f(z2, model = 'ar', order = 1, 
		   hyper=list(prec=list(prior="pc.prec", constr=TRUE,
		                        param=c(0.1,0.01)),
                      pacf1=list(param=c(0.5,0.8),
                      pacf2=list(param=c(0.5,0.8)))))
data.bgev = data.frame(y = y, intercept = 1, x = x, z1 = z1, z2 = z2,
                       spread.x = spread.x, spread.xx = spread.xx, tail.x = tail.x)
r3 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000,prec.intercept=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb",
	                         cmin=0,
	                         b.strategy="keep"),
         verbose=FALSE, safe=TRUE)

## ---- eval = T----------------------------------------------------------------
round(r3$summary.fixed,4)
round(r3$summary.hyperpar,4)

## ---- eval=F------------------------------------------------------------------
#  library(INLA)
#  inla.list.models('latent')

## -----------------------------------------------------------------------------
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}


## -----------------------------------------------------------------------------
map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}

## This demo is similar to demo(Bym), but demonstrate various ways to
## spesify and deal with the 'graph'

data(Germany)
g = system.file("demodata/germany.graph", package="INLA")
source(system.file("demodata/Bym-map.R", package="INLA"))
summary(Germany)

## just make a duplicated column
Germany = cbind(Germany,region.struct=Germany$region)

## graph given as a filename

## we can view the file as a binary matrix, where G[i, j] = 1 if i~j
## and i!=j,  and G[i, i] = 1

## view it as a matrix
inla.dev.new()
inla.spy(g)

formula1 = Y ~ f(region.struct,model="besag",graph=g) +
               f(region,model="iid")
result1  =  inla(formula1,family="poisson",data=Germany,E=E)

inla.dev.new()
Bym.map(result1$summary.random$region.struct$mean)
title("Using graph from file")

## graph given as a inla.graph-object. we can create this object, by
## either reading a graph definition from file, or to specify the
## neigbours in a (sparse) symmetric matrix

go = inla.read.graph(g)
## short summary of its properties
summary(go) 
## view it as a matrix
inla.spy(go)

## view it as a graph. this require the Rgraphviz package from the
## bioconductor project.
if (require(Rgraphviz)) {
    inla.dev.new()
    plot(go)
}

# standard BYM model. the graph is spesified as a filename
formula1 = Y ~ f(region.struct,model="besag",graph=go) +
               f(region,model="iid")
result1  =  inla(formula1,family="poisson",data=Germany,E=E)

inla.dev.new()
Bym.map(result1$summary.random$region.struct$mean)
title("Using graph from an inla.graph-object")

## graph given as a binary matrix (with zero or non-zero term). we can
## also specify the graph as a binary (sparse) symmetric matrix.

## here,  we use the graph we already have
gm = inla.graph2matrix(g)
## or
gm = inla.graph2matrix(go)

## and we can view the result
inla.dev.new()
inla.spy(gm)

## 'gm' can also be a an ordinary dense matrix, but does not have to
if (FALSE)
    gm = as.matrix(gm)

# standard BYM model. the graph is spesified as a filename
formula1 = Y ~ f(region.struct,model="besag",graph=gm) +
               f(region,model="iid")
result1  =  inla(formula1,family="poisson",data=Germany,E=E)

inla.dev.new()
Bym.map(result1$summary.random$region.struct$mean)
title("Using graph from an neighbour-matrix")

## the matrix 'gm' is easier to modify manually, like adding
## neigbhours. recall that this matrix has to be symmetric.

## these two nodes are selected by 'random'. 
stopifnot(gm[1, 10] == 0)
gm[1, 10] = gm[10, 1] = 1

## to rebuild the inla.graph-object, we just re-read it from the
## matrix.
go = inla.read.graph(gm)

## write a new file
fnm = tempfile()
inla.write.graph(gm, filename = fnm)

## and we can use the modified matrix, 'gm', or the inla.graph-object,
## 'go', or the filename, 'fnm', as the as the graph-argument.
formula1 = Y ~ f(region.struct,model="besag",graph=gm) +
               f(region,model="iid")
result1  =  inla(formula1,family="poisson",data=Germany,E=E)

inla.dev.new()
Bym.map(result1$summary.random$region.struct$mean)
title("Using modified graph")

## lets make a random graph, using a dense matrix, just to illustrate...
n = dim(gm)[1L]
gr = matrix(0, n, n)
for(i in 1L:n) {
    ## just sample at random some neigbhours, can include 'i'
    ## itself. so we remove those afterwards.
    js = sample(1L:n, rpois(1, lambda = 10))
    wi = which(js == i)
    if (length(wi) > 0L) 
        js = js[-wi]
    gr[i, js] = 1
    gr[js, i] = 1
}

inla.dev.new()
inla.spy(gr)

formula1 = Y ~ f(region.struct,model="besag",graph=gr) +
               f(region,model="iid")
result1  =  inla(formula1,family="poisson",data=Germany,E=E)

## clean up
unlink(fnm)

## data
fl.dat <- system.file("demodata/Leuk.dat", package = "INLA")
if(fl.dat=="")
    fl.dat <- paste0("https://raw.githubusercontent.com/hrue/r-inla/",
                     "refs/heads/devel/r-inla.org/examples/leukemia/leuk.dat")
Leuk <- read.table(fl.dat, header = TRUE)
Leuk[1:2, ]

### map coordinates and id files
fl.map <- system.file("demodata/Leuk.map", package="INLA")
fl.id  <- system.file("demodata/Leuk.names.regions", package="INLA")

## read all coordinates
allcoo <- read.table(fl.map)

## find the split indexes
length(ii0 <- which(is.na(allcoo[,1])))

### some regions contain multiple polygons
ii1 <- split(ii0, as.integer(readLines(fl.id)))
length(unique(ii1))

library(sp)

## SpatialPolygons 
nwEngland <- SpatialPolygonsDataFrame(
    Sr = SpatialPolygons(
        lapply(1:length(ii1), function(i) {
            ii <- ii1[[i]]
            Polygons(
                lapply(ii, function(i) {
                    ii <- c(i + 1:allcoo[i, 2], i+1)
                    Polygon(cbind(allcoo[ii, 1], allcoo[ii, 2]), FALSE)
                }),
                ID = names(ii1)[i]
            )
        }
        )
    ),
    data = data.frame(ID = names(ii1))
)

## sf 
library(sf)

NEmap <- st_sf(
    ID = names(ii1), 
    geom = st_sfc(
        lapply(
            ii1, function(ii) 
                st_make_valid(                
                    st_polygon(
                        lapply(ii,
                               function(i) {
                                   ii <- c(i + 1:allcoo[i, 2], i+1)
                                   cbind(allcoo[ii, 1], allcoo[ii, 2])
                               }
                               )
                    )
                )
        )
    )
)

ddir <- here::here("data/")

if(FALSE)
    save(
        'Leuk', 'NEmap', 'nwEngland',
        file = paste0(ddir, "Leuk.rda")
    )

## ----setup, include=FALSE-----------------------------------------------------
library(INLA)
set.seed(123)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.path="figures/multinomial/")

## ---- message = FALSE, warning = FALSE----------------------------------------
# number of observations 
n = 10

x = rnorm(n, sd = 1)
eta = 1 + x
prob = exp(eta)/(1+exp(eta))

# Sizes
Ntrials = sample(100:1000, n, replace=TRUE)

# Observations
y = rbinom(n, size=Ntrials, prob = prob)

## -----------------------------------------------------------------------------
lprec = 25

## -----------------------------------------------------------------------------
sqrt(1/exp(-lprec))

## -----------------------------------------------------------------------------
mult.model = inla(y ~ -1 + intercept + x,
         data = data.frame(y, x, Ntrials, intercept = rep(1, n)),
         family = "binomial", Ntrials = Ntrials,
         control.predictor = list(hyper = list(prec = list(initial = lprec))),
         control.inla = list(tolerance = 1E-12))


## -----------------------------------------------------------------------------
# initialize
Y = intercept = xx = c()
for(i in 1:n) {
    # corresponding vector of observations
    Y = c(Y, c(y[i], Ntrials[i] - y[i]))
    # corresponding covariates
    xx = c(xx, c(x[i], 0))
    intercept = c(intercept, c(1, 0))
}


## -----------------------------------------------------------------------------
phi = rep(1:n, each=2)

## -----------------------------------------------------------------------------
pois.model = inla(Y ~ -1
          + intercept
          + f(phi, model="iid",
              hyper = list(prec = list(initial = -lprec, fixed=TRUE)))
          + xx,
          family="poisson",
          data = data.frame(Y, xx, phi , intercept), 
          control.predictor = list(hyper = list(prec = list(initial = lprec))), 
          control.inla = list(tolerance = 1E-12))


## -----------------------------------------------------------------------------
corr.factor = - sum(dnorm(pois.model$summary.random$phi$mode,
                          sd = sqrt(1/exp(-lprec)), log=TRUE)) + sum(log(Ntrials))
print(mult.model$mlik)
print(pois.model$mlik + corr.factor)

## ---- message=F---------------------------------------------------------------
library(Ecdat)
library(deldir)
library(sp)
library(rgeos)
library(mvtnorm)
library(gridExtra)
library(mlogit)

## ---- message=FALSE-----------------------------------------------------------
beta = -0.3
deltas = c(1, 4, 3)
gammas = c(0.3, 0.2, 0.4)
param = c(beta, deltas, gammas)
n = 500

set.seed(123)
# alternative specific with generic coefficient beta
X.A = rnorm(n, mean = 30, sd = 2.5)
X.B = rnorm(n, mean = 35, sd = 3.5)
X.C = rnorm(n, mean = 40, sd = 1)

# alternative specific with alternative specific coefficient delta 
W.A = abs(rnorm(n, mean = 1))
W.B = abs(rnorm(n, mean = 1))
W.C = abs(rnorm(n, mean = 1))

# individual specific with alternative specific coefficient gamma 
Z = rnorm(n, 20, 3)


## -----------------------------------------------------------------------------
Multinom.sample = function(N){
  Y = matrix(NA, ncol = 3, nrow = n)
  for(i in 1:n){
    V.A = beta*X.A[i] + deltas[1]*W.A[i] + gammas[1]*Z[i]  
    V.B = beta*X.B[i] + deltas[2]*W.B[i] + gammas[2]*Z[i] 
    V.C = beta*X.C[i] + deltas[3]*W.C[i] + gammas[3]*Z[i] 
    
    probs = c(V.A, V.B, V.C)
    probs = exp(probs)/sum(exp(probs))
    samp = rmultinom(1, N, prob = probs)
    
    Y[i,] = as.vector(samp)
  }
  colnames(Y ) = c("Y.A", "Y.B", "Y.C")
  return(Y)
}

## -----------------------------------------------------------------------------
head(Multinom.sample(1), 5)
head(Multinom.sample(100), 5)

## -----------------------------------------------------------------------------
Y = Multinom.sample(1)
df = data.frame(cbind(Y, X.A, X.B, X.C, W.A, W.B, W.C, Z))

Data.structure = function(df){
  Data = matrix(NA, ncol = 8, nrow = n*3)
  for(i in 1:n){
    # simulated variable
    Data[((i-1)*3+1):(i*3), 1] = c(df$Y.A[i], df$Y.B[i], df$Y.C[i])
    # alternative specific with generic coeff
    Data[((i-1)*3+1):(i*3), 2] = c(df$X.A[i], df$X.B[i], df$X.C[i])
    # alternative specific with alternative coeff
    Data[((i-1)*3+1):(i*3), 3:5] = diag(c(df$W.A[i], df$W.B[i], df$W.C[i]))
    # individual specific with alternative coeff
    Data[((i-1)*3+1):(i*3), 6] = rep(df$Z[i],3)
    # choice situation index
    Data[((i-1)*3+1):(i*3), 7] = rep(i,3)
    # choice alternative index
    Data[((i-1)*3+1):(i*3), 8] = c(1, 2, 3)
  }
  Data = data.frame(Data)
  names(Data) = c('Y', "X","W.A","W.B","W.C","Z",'phi','alt.idx')
  return(Data)
}

round(head(Data.structure(df)),3)

## -----------------------------------------------------------------------------
formula = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T)

## -----------------------------------------------------------------------------
Data = Data.structure(df)
model = inla(formula, data = Data, family = 'Poisson')

## -----------------------------------------------------------------------------
result = rbind(model$summary.fixed[1:5], model$summary.random$alt.idx[2:6])
result = cbind(result, true = param)
row.names(result) = c("beta","delta.A","delta.B","delta.C","gamma.A","gamma.B","gamma.C" )
round(result,3)

## -----------------------------------------------------------------------------
diff.result = 
  cbind("0.025quant"= diff(model$summary.random$alt.idx$`0.025quant`),
      "0.5quant" = diff(model$summary.random$alt.idx$`0.5quant`),
      "0.975quant" = diff(model$summary.random$alt.idx$`0.975quant`),
      "true" = diff(gammas))
row.names(diff.result) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
round(diff.result,3)

## -----------------------------------------------------------------------------
random.effect = rnorm(n)
Multinom.sample.rand = function(N, random.effect){
  Y = matrix(NA, ncol = 3, nrow = n)
  for(i in 1:n){
    V.A = beta*X.A[i] + deltas[1]*W.A[i] + gammas[1]*Z[i] + 
      random.effect[i]
    V.B = beta*X.B[i] + deltas[2]*W.B[i] + gammas[2]*Z[i] 
    V.C = beta*X.C[i] + deltas[3]*W.C[i] + gammas[3]*Z[i] 
    
    probs = c(V.A, V.B, V.C)
    probs = exp(probs)/sum(exp(probs))
    samp = rmultinom(1, N, prob = probs)
    
    Y[i,] = as.vector(samp)
  }
  colnames(Y) = c("Y.A", "Y.B", "Y.C")
  return(Y)
}

Y.rand1 = Multinom.sample.rand(100, random.effect)
df.rand1 = data.frame(cbind(Y.rand1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.rand1 = Data.structure(df.rand1)

## -----------------------------------------------------------------------------
rand.idx = rep(NA, n*3)
rand.idx[seq(1,n*3, by = 3)] = seq(1,n)
Data.rand1$rand.idx = rand.idx
round(head(Data.rand1),3)

## -----------------------------------------------------------------------------
formula.rand1 = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rand.idx, model = "iid")  #random effect

## -----------------------------------------------------------------------------
model.rand1 = inla(formula.rand1, data = Data.rand1, family = 'Poisson')
result.rand1 = rbind(model.rand1$summary.fixed[1:5])
result.rand1 = cbind(result.rand1, true = param[1:4])
row.names(result.rand1) = c("beta","delta.A","delta.B","delta.C")
round(result.rand1,3)

## -----------------------------------------------------------------------------
diff.result.rand1 = 
  cbind("0.025quant"= diff(model.rand1$summary.random$alt.idx$`0.025quant`),
      "0.5quant" = diff(model.rand1$summary.random$alt.idx$`0.5quant`),
      "0.975quant" = diff(model.rand1$summary.random$alt.idx$`0.975quant`),
      "true" = diff(gammas))
row.names(diff.result.rand1) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
round(diff.result.rand1,3)

## -----------------------------------------------------------------------------
mean((random.effect - model.rand1$summary.random$rand.idx$`0.5quant`)^2)

## ---- fig.height=2.5----------------------------------------------------------
#  Generate the random effect
random.walk1 = rep(NA, n)
random.walk1[1] = 0
for(i in 2:n){
  random.walk1[i] = rnorm(1, mean = random.walk1[i-1], sd = 0.1)
}

# Generate data
Y.rw1 = Multinom.sample.rand(100, random.walk1)
df.rw1 = data.frame(cbind(Y.rw1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.rw1 = Data.structure(df.rw1)


## -----------------------------------------------------------------------------
rw1.idx = rep(NA, n*3)
rw1.idx[seq(1,n*3, by = 3)] = seq(1,n)
Data.rw1$rw1.idx = rw1.idx

formula.rw1 = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rw1.idx, model = "rw1")  #random walk of order 1

model.rw1 = inla(formula.rw1, family = 'poisson', data = Data.rw1)

## ---- fig.cap=c("Estimate of a random walk, the black line represents the true one and the red line the estimate"), echo=FALSE, fig.height=4----
plot(random.walk1 - mean(random.walk1), type = 'l', 
     main = 'Random walk estimate', ylab = 'values', xlab = 'time')
points(model.rw1$summary.random$rw1.idx$`0.5quant`, type = 'l', col = 'red')

## -----------------------------------------------------------------------------
x.loc = runif(n)
y.loc = runif(n)
loc = cbind("x.loc" = x.loc, "y.loc" = y.loc)

## ---- message=FALSE-----------------------------------------------------------

voronoi.polygons <- function(x) {
  require(deldir)
  if (.hasSlot(x, 'coords')) {
    crds <- x@coords  
  } else crds <- x
  z <- deldir(crds[,1], crds[,2])
  w <- tile.list(z)
  polys <- vector(mode='list', length=length(w))
  require(sp)
  for (i in seq(along=polys)) {
    pcrds <- cbind(w[[i]]$x, w[[i]]$y)
    pcrds <- rbind(pcrds, pcrds[1,])
    polys[[i]] <- Polygons(list(Polygon(pcrds)), ID=as.character(i))
  }
  SP <- SpatialPolygons(polys)
  voronoi <- SpatialPolygonsDataFrame(SP, 
                                      data=data.frame(x=crds[,1],
                                                      y=crds[,2], 
                                                      row.names=sapply(slot(SP, 'polygons'), 
                                                                       function(x) slot(x, 'ID'))))
}

## ---- message=F---------------------------------------------------------------
# number of regions
n.reg = 50
# corners
boundaries = rbind(c(0,0), c(1,1), c(0,1), c(1,0))
# points
points = rbind(boundaries, cbind(runif(n.reg - 4), runif(n.reg - 4)))
# generate Voronoi polygons
vor = voronoi.polygons(points)

## -----------------------------------------------------------------------------
# Create an ID column for the regions, it will be our index
vor@data$id = seq(1,n.reg)
# Transform the locations in SpatialPoint
pp = SpatialPoints(loc)
# This vector contains the region in which each point is located
id.samples = over(pp,vor)$id
# Check the number of non epty regions 
check = as.integer(length(unique(id.samples)))
check

## ---- fig.cap=c("Partiotion of the region [0,1]x[0,1], each red point correspond to a different location"), fig.height=5----
# plot the partition and the locations
plot(vor)
points(loc, pch = 21, bg = 2, cex = 0.7)

## ---- message = F, warnings = F-----------------------------------------------
# adjacency matrix
ADJ = gTouches(vor, byid = T)
# marginal variance of each random effect component
S = diag(n.reg)
# covariance matrix
cov.matrix = solve(diag(n.reg) - ADJ)%*%S
# generate the random effect and add it to the SpatialPolygonDataFrame
vor@data$rand.eff = as.numeric(rmvnorm(1,sigma = cov.matrix))


## ---- fig.cap=c("Random effect on the partition"), fig.height=4---------------
spplot(vor, 'rand.eff', col.regions = terrain.colors(32))

## -----------------------------------------------------------------------------
# generate random effect vector
random.effect = vor@data$rand.eff[id.samples]
# generate sample
Y.spatial = Multinom.sample.rand(N = 100, random.effect)
# construct data set
df.spatial = data.frame(cbind(Y.spatial, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.spatial = Data.structure(df.spatial)

## -----------------------------------------------------------------------------
# create and add index column
rand.eff.idx = rep(NA, n*3)
rand.eff.idx[seq(1,n*3, by = 3)] = id.samples
Data.spatial$rand.eff.idx = rand.eff.idx

# Formula
formula.spatial = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) + 
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rand.eff.idx, model = 'besag', graph = ADJ) #spatial random effect

# Fit the model
model.spatial = inla(formula.spatial, family = "poisson", data = Data.spatial)

# Check the results
result.spatial = cbind(model.spatial$summary.fixed[3:5], true = param[1:4])
row.names(result.spatial) = c("beta","delta.A","delta.B","delta.C")

diff.result.spatial = 
  cbind("0.025quant"= diff(model.spatial$summary.random$alt.idx$`0.025quant`),
        "0.5quant" = diff(model.spatial$summary.random$alt.idx$`0.5quant`),
        "0.975quant" = diff(model.spatial$summary.random$alt.idx$`0.975quant`),
        "true" = diff(gammas))
row.names(diff.result.spatial) = c("gamma.B - gamma.A", "gamma.C - gamma.B")

round(rbind(result.spatial, diff.result.spatial),3)

## ---- fig.cap = c("Comparison between the true random effect and the estimated one"), fig.height=5----
# compute mean square error
mean((vor@data$rand.eff -
        model.spatial$summary.random$rand.eff.idx$`0.5quant`)^2)

# add columns to the SpatialPolygonDataFralme
vor@data$rand.eff.est = model.spatial$summary.random$rand.eff.idx$`0.5quant`
vor@data$residuals = (vor@data$rand.eff-
                        model.spatial$summary.random$rand.eff.idx$`0.5quant`)^2
# plot them
grid.arrange(spplot(vor, 'rand.eff', at = seq(-2.5, 2.5, length.out = 32),
                    col.regions = terrain.colors(32), main = "True"),
             spplot(vor, 'rand.eff.est', at = seq(-2.5,2.5,length.out = 32),
                    col.regions = terrain.colors(32), main = "Estimates"),
             spplot(vor, 'residuals', at = seq(-2.5,2.5,length.out = 32),
                    col.regions = terrain.colors(32), main = "Residuals"),
             nrow = 1)

## -----------------------------------------------------------------------------
mesh = inla.mesh.2d(loc, max.edge = c(0.02, 0.2), cutoff = 0.02)

## ---- fig.cap = c("Triangulation of the study area [0,1]x[0,1], the red dots represent the data points locations"), fig.height=5----
mesh$n
par(mfrow = c(1,1))
plot(mesh)
points(loc, pch = 21, bg = 2)

## -----------------------------------------------------------------------------
# starting parametrization
size <- min(c(diff(range(mesh$loc[, 1])), diff(range(mesh$loc[, 2]))))
range <- size/5
sigma0 = 1

# new parametrization
kappa0 <- sqrt(8)/range
tau0 <- 1/(sqrt(4 * pi) * kappa0 * sigma0)

# create an spde object
spde <- inla.spde2.matern(mesh, B.tau = cbind(log(tau0), -1, +1),
                          B.kappa = cbind(log(kappa0), 0, -1))

## -----------------------------------------------------------------------------
Q = inla.spde.precision(spde, theta = c(0,0))
# the number of elements of Q is the square of the number of triangles 
dim(Q)

## -----------------------------------------------------------------------------
sample = inla.qsample(n = 2, Q, seed = 123)

## -----------------------------------------------------------------------------
# create A
A = inla.spde.make.A(mesh = mesh,
                      loc = cbind(rep(loc[,1], each = 3), 
                                  rep(loc[,2], each = 3)))

# select rows relative to the first alternatives
A = A*rep(c(1,2,3) == 1, n)

## -----------------------------------------------------------------------------
random.effect = matrix(drop(A%*%sample[,2]), ncol = 3, byrow = T)[,1]

## ---- fig.cap=c("The right figure represents the continuos spatial effect, the left figure represents the value of the random effect on the sample locations" ), eval.after='fig.cap'----
par(mfrow = c(1,2))

# plot the continuos random effect
proj <- inla.mesh.projector(mesh, dims = c(100, 100))
sample_proj = inla.mesh.project(proj, field = sample[,2])
image(proj$x, proj$y, sample_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '')
contour(proj$x, proj$y, sample_proj, add = T)

# plot the projection on the sample locations
rbPal <- colorRampPalette(heat.colors(100))
Col <- rbPal(100)[as.numeric(cut(random.effect, breaks = 100))]
plot(loc, pch = 20, col = Col, xlab = '', ylab = '')

## -----------------------------------------------------------------------------
# generate sample
Y.spde = Multinom.sample.rand(N = 100, random.effect)
# construct data set
df.spde = data.frame(cbind(Y.spde, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.spde = Data.structure(df.spde)
round(head(Data.spde),3)

## -----------------------------------------------------------------------------
# index for the random effect
s_index <- inla.spde.make.index(name="spatial.field",
                                n.spde= spde$n.spde)

# stack object
stack = inla.stack(data = list(Y = Data.spde$Y),
                   effects = list(s_index, list(Data.spde[,2:8])),
                   A = list(A, 1))


## -----------------------------------------------------------------------------
formula.spde = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) +
  f(spatial.field, model = spde, group = spatial.field.group)


init = c(-0.008, -0.093)
model.spde <- inla(formula.spde,
                   data=inla.stack.data(stack, spde = spde),
                   family="poisson",
                   control.predictor=list(A = inla.stack.A(stack), compute=TRUE),
                   control.inla= list(int.strategy = "eb"),
                   control.mode=list(restart=T, theta=init))

round(model.spde$internal.summary.hyperpar$mode, 3)

## -----------------------------------------------------------------------------
# estimates on the mesh locations
spde.mesh.est = model.spde$summary.random$spatial.field$`0.5quant`
# estimates on the data locations
spde.loc.est = matrix(drop(A%*%spde.mesh.est),
                 ncol=3, byrow=TRUE)[,1]
# Mean Square Error
mean((spde.loc.est - random.effect)^2)

## ---- fig.cap=c("Recostruction of the spatial random field")------------------
par(mfrow = c(1,2))

# plot the estimates
output_proj = inla.mesh.project(proj, field = spde.mesh.est)
image(proj$x, proj$y, output_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '', main = "Estimate")

# plot the true one
sample_proj = inla.mesh.project(proj, field = sample[,2])
image(proj$x, proj$y, sample_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '', main = "True")


## ---- message=F---------------------------------------------------------------
data("Yogurt")
head(Yogurt)

## ---- echo = F----------------------------------------------------------------
n = nrow(Yogurt)
Data = matrix(NA, ncol = 5, nrow = n*4)
for(i in 1:n){
  choice = rep(0, 4)
  choice[Yogurt$choice[i]] = 1
  Data[((i-1)*4+1):(i*4), 1] = choice
  Data[((i-1)*4+1):(i*4), 2] = c(Yogurt$price.yoplait[i],
                                 Yogurt$price.dannon[i],
                                 Yogurt$price.hiland[i],
                                 Yogurt$price.weight[i])
  Data[((i-1)*4+1):(i*4), 3] = c(Yogurt$feat.yoplait[i],
                                 Yogurt$feat.dannon[i],
                                 Yogurt$feat.hiland[i],
                                 Yogurt$feat.weight[i])
  Data[((i-1)*4+1):(i*4), 4] = rep(Yogurt$id[i], 4)
  Data[((i-1)*4+1):(i*4), 5] = rep(i, 4)
}

Data = data.frame(Data)
names(Data) = c('Y', 'price', 'feat', 'cust.id', 'phi')
Data$price = Data$price/100 
Data$phi = as.factor(Data$phi)
Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)

## -----------------------------------------------------------------------------
head(Data)

## -----------------------------------------------------------------------------
formula = Y ~ -1 + price + feat + alpha.idx +
  f(phi, initial = -10, fixed = T) 
  

model = inla(formula, data = Data, family = 'Poisson') 

## -----------------------------------------------------------------------------
results = model$summary.fixed[,1:5]
rownames(results) = c("price", "feat", "dannon", "hiland", "weight", "yoplait")
round(results,3)

## -----------------------------------------------------------------------------
Data$price.class = inla.group(Data$price, n = 12, method = "cut")

formula.price = Y ~ -1 + feat + alpha.idx +
  f(price.class, model = "rw1") + 
  f(phi, initial = -10, fixed = T) 

model.price = inla(formula.price, data = Data, family = 'Poisson') 

res = model.price$summary.fixed[,1:5]
rownames(res) = c('feat', 'dannon','hiland','weight','yoplait')
round(model.price$summary.random$price.class[,1:5],3)

## ---- fig.cap=c("Relation between the coefficient and the price level")-------
plot(model.price$summary.random$price.class$ID,
     model.price$summary.random$price.class$mean, type = 'l', 
     ylab = expression(beta[1]),
     xlab = 'price')

## ---- echo = F----------------------------------------------------------------
Data = matrix(NA, ncol = 8, nrow = n*4)
for(i in 1:n){
  choice = rep(0, 4)
  choice[Yogurt$choice[i]] = 1
  Data[((i-1)*4+1):(i*4), 1] = choice
  Data[((i-1)*4+1):(i*4), 2] = c(Yogurt$price.yoplait[i],
                                 Yogurt$price.dannon[i],
                                 Yogurt$price.hiland[i],
                                 Yogurt$price.weight[i])
  Data[((i-1)*4+1):(i*4), 3:6] = rbind(c(Yogurt$feat.yoplait[i], NA, NA, NA),
                                       c(NA, Yogurt$feat.dannon[i], NA, NA),
                                       c(NA, NA, Yogurt$feat.hiland[i], NA),
                                       c(NA, NA, NA, Yogurt$feat.weight[i]))
                                
  Data[((i-1)*4+1):(i*4), 7] = rep(Yogurt$id[i], 4)
  Data[((i-1)*4+1):(i*4), 8] = rep(i, 4)
}


Data = data.frame(Data)
names(Data) = c('Y', 'price', 'feat.yoplait','feat.dannon', 'feat.hiland', 'feat.weight', 'cust.id', 'phi')
Data$price = Data$price/100 #(Data$price - mean(Data$price))/sd(Data$price)
Data$phi = as.factor(Data$phi)
Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)
head(Data)

## -----------------------------------------------------------------------------
formula.feat = Y ~ -1 + price + feat.yoplait + feat.dannon + 
  feat.hiland + feat.weight + alpha.idx + 
  f(phi, initial = -10, fixed = T)  
  

model.feat = inla(formula.feat, data = Data, family = 'Poisson') 

## -----------------------------------------------------------------------------
model.feat$summary.fixed[2:5, 1:5]

source(system.file("demodata/Bym.map", package="INLA"))

Bym.map = function(...)
{
    germany.map(...)
}
germany.make.graph <- function(filename)
{
    graph <- list(n=1, nbs = list(), nnbs = c(), map=germany)
    xx <- scan(filename)
    graph$n = xx[1]
    jj = 2
    for(i in 1:graph$n)
    {
        idx = xx[jj]
        nnbs = xx[jj+1]
        if (nnbs)
        {
            nbs = xx[(jj+2):(jj+1+nnbs)]
        }
        else
        {
            nbs = NULL
        }
        graph$nbs[[idx]] = nbs
        graph$nnbs[idx] = nnbs
        jj = jj + nnbs + 2
    }

    graph
}
germany.map <- function(data, cutpoints=seq(min(data),max(data),length=256), autoscale=FALSE, legend=TRUE, append=FALSE)
{
    if (autoscale)
    {
        data = (data-min(data))/(max(data)-min(data)+1e-8)
    }
    #cutpoints = c(-1e9,cutpoints, 1e9)
    
    farben <- gray(as.numeric(cut(data,cutpoints,include.lowest=T))/length(cutpoints))


    xmin <- 1:length(germany)
    xmax <- 1:length(germany)
    ymin <- 1:length(germany)
    ymax <- 1:length(germany)

    for(i in 1:length(germany))
    {
        xmin[i] <- min(germany[[i]][,2],na.rm=T)
        xmax[i] <- max(germany[[i]][,2],na.rm=T)
        ymin[i] <- min(germany[[i]][,3],na.rm=T)
        ymax[i] <- max(germany[[i]][,3],na.rm=T)
    }

    breite <- c(min(xmin),max(xmax))
    hoehe <- c(min(ymin),max(ymax))

    if (!append) plot(breite,hoehe,type="n",axes=F, xlab=" ", ylab=" ")


    for(k in length(germany):1)
    {
        polygon(germany[[k]][,2],germany[[k]][,3],col=farben[k])
    }
    
    if (legend)
    {
        for(i in 1:8)
        {
            polygon(c(5800,5800,6100,6100),c(500+350*(i-1),500+350*i,500+350*i,500+350*(i-1)),col=gray(i/9))
        }
        for(i in 2:8)
        {
            text(6375,500+350*(i-1),as.character(round(cutpoints[i/8*length(cutpoints)], 2)),cex=.7,col=rgb(0,0,0))
        }
    }
}
germany.graph.point <- function(x, y)
{
    ##
    require("splancs",quietly=TRUE)

    if (F)
    {
        list(x=mean(x[!is.na(x)]), y=mean(y[!is.na(y)]))
    }
    else
    {
        n.seg = sum(is.na(x))+1
        if (n.seg > 1) 
        {
            na.pos = c(which(is.na(x)),length(x))
        }
        else
        {
            na.pos = length(x)
        }

        pos = list(x=0,y=0,area=0)
        jj = 1
        weig = 0
        for(j in 1:n.seg)
        {
            xx = x[jj:(na.pos[j]-1)]
            yy = y[jj:(na.pos[j]-1)]
            jj = na.pos[j]+1

            area = as.double(areapl(matrix(data=c(xx,yy),length(xx),2)))
            pos$x = pos$x + area*mean(xx)
            pos$y = pos$y + area*mean(yy)
            weig = weig + area
        }
        pos$x = pos$x/weig
        pos$y = pos$y/weig
        pos$area = weig
        pos
    }
}
germany.map.graph <- function(data, cutpoints=seq(0,1,length=256), autoscale=T,
                          legend=F, append=F, pch=20, cex=1.2, ...)
{
    if (autoscale)
    {
        data = (data-min(data))/(max(data)-min(data)+1e-8)
    }
    cutpoints = c(-1e9,cutpoints, 1e9)
    
    farben <- gray(as.numeric(cut(data,cutpoints,include.lowest=T))/length(cutpoints))


    xmin <- 1:length(germany.graph$map)
    xmax <- 1:length(germany.graph$map)
    ymin <- 1:length(germany.graph$map)
    ymax <- 1:length(germany.graph$map)

    for(i in 1:length(germany.graph$map))
    {
        xmin[i] <- min(germany.graph$map[[i]][,2],na.rm=T)
        xmax[i] <- max(germany.graph$map[[i]][,2],na.rm=T)
        ymin[i] <- min(germany.graph$map[[i]][,3],na.rm=T)
        ymax[i] <- max(germany.graph$map[[i]][,3],na.rm=T)
    }

    breite <- c(min(xmin),max(xmax))
    hoehe <- c(min(ymin),max(ymax))

    if (!append) plot(breite,hoehe,type="n",axes=F, xlab=" ", ylab=" ")


    for(k in length(germany.graph$map):1)
    {
        pos = germany.graph.point(germany.graph$map[[k]][,2], germany.graph$map[[k]][,3])
        points(pos, col=farben[k], pch=pch, cex=cex, ...)
    }
    
    for(k in 1:germany.graph$n)
    {
        pos = germany.graph.point(germany.graph$map[[k]][,2],germany.graph$map[[k]][,3])

        if (length(germany.graph$nbs[[k]]) > 0)
        {
            for(kk in germany.graph$nbs[[k]])
            {
                poss = germany.graph.point(germany.graph$map[[kk]][,2], germany.graph$map[[kk]][,3])
                lines(x=c(pos$x,poss$x), y=c(pos$y, poss$y), ...)
            }
        }
    }
    
    if (legend)
    {
        for(i in 1:8)
        {
            polygon(c(5800,5800,6100,6100),c(500+350*(i-1),500+350*i,500+350*i,500+350*(i-1)),col=gray(i/9))
        }
        for(i in 2:8)
        {
            text(6375,500+350*(i-1),as.character(round(cutpoints[i/8*length(cutpoints)], 2)),cex=.7,col=rgb(0,0,0))
        }
    }
}
germany.map.add.text <- function(data)
{
    for(k in length(germany.graph$map):1)
    {
        pos = germany.graph.point(germany.graph$map[[k]][,2], germany.graph$map[[k]][,3])
        text(pos, as.character(data[k]))
    }
}

germany.graph <- germany.make.graph(system.file("demodata/germany.graph", package="INLA"))

library(INLA)
library(JMbayes)

data("aids")
data1 <- aids
data1$Time=data1$Time/max(max(data1$Time),max(data1$obstime))
data1$obstime=data1$obstime/max(max(data1$Time),max(data1$obstime))
datas <- data1[data1$obstime==0,]
datal <- data1[,c(1,4:12)]
ns <- nrow(datas)
nl <- nrow(datal)

y.long <- c(datal$CD4,rep(NA, ns))
y.surv <- inla.surv(time = c(rep(NA, nl),datas$Time ), event = c(rep(NA, nl),datas$death))
Yjoint <- list(y.long, y.surv)
N <- length(unique(data1$patient))

fixed.covariate <- data.frame(mu = as.factor(c(rep(1, nl), rep(2, ns))),
                              l.drug = as.factor(c(as.factor(datal$drug), rep(NA, ns))),
                              l.gender = as.factor(c(as.factor(datal$gender), rep(NA, ns))),
                              l.prevOI = as.factor(c(as.factor(datal$prevOI) , rep(NA, ns))),
                              l.AZT = as.factor(c(as.factor(datal$AZT) , rep(NA, ns))),
                              s.drug = as.factor(c( rep(NA, nl),as.factor(datas$drug))),
                              s.gender = as.factor(c(rep(NA, nl),as.factor(datas$gender))),
                              s.prevOI = as.factor(c(rep(NA, nl),as.factor(datas$prevOI))),
                              s.AZT = as.factor(c(rep(NA, nl),as.factor(datas$AZT))),
                              l.time=c(datal$obstime, rep(0, ns)),
                              s.time=c(rep(0, nl),datas$Time ))
random.covariate <- list(U11 = c(datal$patient,rep(NA, ns)),
                         U21 = c(datal$patient,rep(NA, ns)),
                         U12=c(rep(NA,nl),datas$patient),
                         U22=c(rep(NA,nl),datas$patient))
joint.dataCD4  <-  c(fixed.covariate,random.covariate)
joint.dataCD4$Y  <-  Yjoint #What I need to "call" is joint.dataCD4

saveRDS(joint.dataCD4, file = "jointdataCD4.rds")

## Redo the Leuk-example using more senible PC-priors
data(Leuk)

## graph file
g <- system.file("demodata/Leuk.graph", package="INLA")

## pc-prior parameters
U <- 0.3/0.31
alpha <- 0.01
pc.param <- list(prec = list(prior = "pc.prec", param = c(U, alpha)))

## model formula
formula <- inla.surv(Leuk$time, Leuk$cens) ~ sex + age +
    f(inla.group(wbc), model="rw2",
      scale.model = TRUE, hyper = pc.param)+
    f(inla.group(tpi), model="rw2",
      scale.model = TRUE, hyper = pc.param)+
    f(district,model="besag",graph = g,
      scale.model = TRUE, hyper = pc.param)

## fit the model 
result <- inla(formula, family="coxph", data=Leuk,
        control.hazard = list(scale.model = TRUE, hyper = pc.param))
summary(result)

if(FALSE) {## old code for the map
  
  ## get the function to plot the map
  source(system.file("demodata/Leuk-map.R", package="INLA"))
  Leuk.map(result$summary.random$district$mean)
  
}

## use the new sf map
library(ggplot2)

ggplot() + theme_minimal() + 
  geom_sf(aes(fill = exp(result$summary.random$district$mean)), 
          data = NEmap) +
    labs(fill = "frailty")

#plot(result)

## Note: check the demo/LeukSPDE.R for continuous spatial domain frailty

## ----setup, include=FALSE-------------------------------------------
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE,warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/conditional-logit/")

## ----formula, include=TRUE,eval=F-----------------------------------
#  fisher.dat <- readRDS(system.file("demodata/data_fisher2.rds", package
#  = "INLA"))
#  fisher.dat$id1 <- fisher.dat$id
#  fisher.dat$dist_cent <- scale(fisher.dat$dist_cent)
#  
#  formula.inla <- y ~ sex + landuse + dist_cent +
#     f(stratum,model="iid",hyper=list(theta = list(initial=log(1e-6),fixed=T))) +
#     f(id1,dist_cent, model="iid")

## ----inla_call, include=TRUE,eval=F---------------------------------
#  r.inla <- inla(formula.inla, family ="Poisson", data=fisher.dat)

## ----setup, include=FALSE-------------------------------------------
set.seed(123)
library(INLA)
##inla.setOption(num.threads="4:1")
##inla.setOption(smtp="taucs")
##if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/barrier-global/")

library(INLA)
library(shiny)

ui <- fluidPage(
    headerPanel("Two dimensional mesh demostration app"), 
    sidebarLayout(
        sidebarPanel(
            checkboxInput("domain", "Add domain: [0,1]x[0,1]", value=FALSE),
            sliderInput("N", 
                        label="Number of random locatins to drawn", 
                        min=1, max=100, value=30, step=1), 
            sliderInput("offset", 
                        label="The automatic extension distance, 1 or 2 values", 
                        min=0.1, max=2, value=c(0.5, 1), step=0.05), 
            sliderInput("n", 
                        label="Initial nodes in the automatic extensions.",
                        min=3, max=100, value=16, step=1),
            sliderInput("max.edge",
                        label="Largest allowed triangle edge length, 1 or 2 v.",      
                        min=0.05, max=1, value=c(0.3, 0.7), step=0.05),
            sliderInput("cutoff", 
                        label="The minimum allowed distance between points",
                        min=1e-10, max=1, value=0.1, step=0.01), 
            actionButton("Exit", "Exit"), 
            width=4), 
        mainPanel(plotOutput("plot"))
        )
    )

server <- function(input, output) {
    output$plot <- renderPlot({
        loc <- cbind(runif(input$N), runif(input$N))
        loc.dom <- NULL
        if(input$domain)
             loc.dom <- cbind(x=c(0,1,1,0,0), y=c(0,0,1,1,0))
        mesh <- inla.mesh.2d(loc, loc.domain=loc.dom, max.edge=input$max.edge, 
                             offset=input$offset, n=input$n, cutoff=input$cutoff)
        par(mar=c(0,0,1,0), mgp=c(1,0.5,0))
        plot(mesh, asp=1);
        points(loc, pch=19)
    }, width=500, height=500)
    observe({
        if(input$Exit > 0){
            stopApp(NULL)
        }
    })
}

runApp(shinyApp(ui=ui, server=server))

#' Render Table of Contents
#' 
#' A simple function to extract headers from an RMarkdown or Markdown document
#' and build a table of contents. Returns a markdown list with links to the 
#' headers using 
#' [pandoc header identifiers](http://pandoc.org/MANUAL.html#header-identifiers).
#' 
#' WARNING: This function only works with hash-tag headers.
#' 
#' Because this function returns only the markdown list, the header for the
#' Table of Contents itself must be manually included in the text. Use
#' `toc_header_name` to exclude the table of contents header from the TOC, or
#' set to `NULL` for it to be included.
#' 
#' @section Usage:
#' Just drop in a chunk where you want the toc to appear (set `echo=FALSE`):
#' 
#'     # Table of Contents
#' 
#'     ```{r echo=FALSE}
#'     render_toc("/path/to/the/file.Rmd")
#'     ```
#' 
#' @param filename Name of RMarkdown or Markdown document
#' @param toc_header_name The table of contents header name. If specified, any
#'   header with this format will not be included in the TOC. Set to `NULL` to
#'   include the TOC itself in the TOC (but why?).
#' @param base_level Starting level of the lowest header level. Any headers 
#'   prior to the first header at the base_level are dropped silently.
#' @param toc_depth Maximum depth for TOC, relative to base_level. Default is
#'   `toc_depth = 3`, which results in a TOC of at most 3 levels.
render_toc <- function(
  filename, 
  toc_header_name = "Table of Contents",
  base_level = NULL,
  toc_depth = 3
) {
  x <- readLines(filename, warn = FALSE)
  x <- paste(x, collapse = "\n")
  x <- paste0("\n", x, "\n")
  for (i in 5:3) {
    regex_code_fence <- paste0("\n[`]{", i, "}.+?[`]{", i, "}\n")
    x <- gsub(regex_code_fence, "", x)
  }
  x <- strsplit(x, "\n")[[1]]
  x <- x[grepl("^#+", x)]
  if (!is.null(toc_header_name)) 
    x <- x[!grepl(paste0("^#+ ", toc_header_name), x)]
  if (is.null(base_level))
    base_level <- min(sapply(gsub("(#+).+", "\\1", x), nchar))
  start_at_base_level <- FALSE
  x <- sapply(x, function(h) {
    level <- nchar(gsub("(#+).+", "\\1", h)) - base_level
    if (level < 0) {
      stop("Cannot have negative header levels. Problematic header \"", h, '" ',
           "was considered level ", level, ". Please adjust `base_level`.")
    }
    if (level > toc_depth - 1) return("")
    if (!start_at_base_level && level == 0) start_at_base_level <<- TRUE
    if (!start_at_base_level) return("")
    if (grepl("\\{#.+\\}(\\s+)?$", h)) {
      # has special header slug
      header_text <- gsub("#+ (.+)\\s+?\\{.+$", "\\1", h)
      header_slug <- gsub(".+\\{\\s?#([-_.a-zA-Z]+).+", "\\1", h)
    } else {
      header_text <- gsub("#+\\s+?", "", h)
      header_text <- gsub("\\s+?\\{.+\\}\\s*$", "", header_text) # strip { .tabset ... }
      header_text <- gsub("^[^[:alpha:]]*\\s*", "", header_text) # remove up to first alpha char
      header_slug <- paste(strsplit(header_text, " ")[[1]], collapse="-")
      header_slug <- tolower(header_slug)
    }
    paste0(strrep(" ", level * 4), "- [", header_text, "](#", header_slug, ")")
  })
  x <- x[x != ""]
  knitr::asis_output(paste(x, collapse = "\n"))
}

data(Epil)
my.center = function(x) (x - mean(x))

## make centered covariates
Epil$CTrt    = my.center(Epil$Trt)
Epil$ClBase4 = my.center(log(Epil$Base/4))
Epil$CV4     = my.center(Epil$V4)
Epil$ClAge   = my.center(log(Epil$Age))
Epil$CBT     = my.center(Epil$Trt*Epil$ClBase4)

formula = y ~ ClBase4 + CTrt + CBT+ ClAge + CV4 +
          f(Ind, model="iid") + f(rand,model="iid")

result = inla(formula,family="poisson", data = Epil)
summary(result)
plot(result)

hyper = inla.hyperpar(result)
summary(hyper)
plot(hyper)

n = 100
s = 0.1
x = rnorm(n)
y = 1 + x + rnorm(n, sd = s)

model = (inla.rgeneric.define(inla.rgeneric.iid.model, n=n, debug=FALSE))
r2 = (inla(y ~ -1 + f(idx, model=model), 
           data = data.frame(y = y, idx = 1:n),
           control.family = list(
               hyper = list(prec = list(initial = 12, fixed=TRUE)))))

r1 = (inla(y ~ -1 + f(idx, model="iid", param = c(1, 1)), 
           data = data.frame(y = y, idx = 1:n),
           control.family = list(
               hyper = list(prec = list(initial=12, fixed=TRUE)))))

print(r1$mlik - r2$mlik)

#' Check which mesh triangles are inside a polygon
#'
#' Wrapper for the [sp::over()] method to find triangle centroids
#' or vertices inside `sp` polygon objects
#'
#' @param x geometry (typically a [sp::SpatialPolygons()] object) for the queries
#' @param y an [inla.mesh()] object
#' @param type the query type; either `'centroid'` (default, for triangle centroids),
#' or `'vertex'` (for mesh vertices)
#' @param ignore.CRS logical; whether to ignore the coordinate system information in `x` and `y`
#' (default `FALSE`)
#'
#' @return A vector of triangle indices (when `type` is `'centroid'`) or
#' vertex indices (when `type` is `'vertex'`)
#'
#' @author Haakon Bakka, \email{bakka@@r-inla.org}, and Finn Lindgren \email{finn.lindgren@@gmail.com}
#'
#' @examples
#' # Create a polygon and a mesh
#' obj <- sp::SpatialPolygons(list(Polygons(list(Polygon(rbind(
#'     c(0, 0),
#'     c(50, 0),
#'     c(50, 50),
#'     c(0, 50)
#' ))),
#' ID = 1
#' )),
#' proj4string = inla.CRS("longlat_globe")
#' )
#' mesh <- inla.mesh.create(globe = 2, crs = inla.CRS("sphere"))
#'
#' ## 3 vertices found in the polygon
#' inla.over_sp_mesh(obj, mesh, type = "vertex")
#'
#' ## 3 triangles found in the polygon
#' inla.over_sp_mesh(obj, mesh)
#'
#' ## Multiple transformations can lead to slightly different results due to edge cases
#' ## 4 triangles found in the polygon
#' inla.over_sp_mesh(obj, inla.spTransform(mesh, CRSobj = inla.CRS("mollweide_norm")), ignore.CRS = FALSE)
#'
#' ## Ignoring mismatching coordinate systems is rarely useful
#' ## 20 triangles "found in" the polygon
#' inla.over_sp_mesh(obj, inla.spTransform(mesh, CRSobj = inla.CRS("mollweide_norm")), ignore.CRS = TRUE)
#' @export
inla.over_sp_mesh <- function(x, y, type = c("centroid", "vertex"), ignore.CRS = FALSE) {
    if (!inherits(y, "inla.mesh")) {
        stop(paste0(
            "'y' must be an 'inla.mesh' object, not '",
            paste0(class(y), collapse = ", "),
            "'."
        ))
    }
    type <- match.arg(type)
    if (identical(type, "centroid")) {
        ## Extract triangle centroids
        points <- (y$loc[y$graph$tv[, 1], , drop = FALSE] +
            y$loc[y$graph$tv[, 2], , drop = FALSE] +
            y$loc[y$graph$tv[, 3], , drop = FALSE]) / 3
        if (identical(y$manifold, "S2")) {
            points <- points / rowSums(points^2)^0.5
        }
    } else if (identical(type, "vertex")) {
        ## Extract vertices
        points <- y$loc
    }
    ## Convert to SpatialPoints
    if (ignore.CRS) {
        if (identical(y$manifold, "S2")) {
            stop("sp::over cannot operate on geocentric coordinates.")
        }
        ## Ignore any actual CRS information, and copy the one from x
        points <- sp::SpatialPoints(points, proj4string = inla.sp_get_crs(x))
    } else {
        ## Extract coordinate system information
        if (identical(y$manifold, "S2")) {
            crs <- inla.CRS("sphere")
        } else {
            crs <- y$crs
        }
        crs_x <- inla.sp_get_crs(x)
        ## Create SpatialPoints object and transform the coordinates.
        points <- sp::SpatialPoints(points, proj4string = crs)
        if (inla.has_PROJ6()) {
            if (!is.null(inla.crs_get_wkt(crs)) &&
                !is.null(inla.crs_get_wkt(crs_x))) {
                ## Convert to the target object CRS
                points <- inla.spTransform(points, CRSobj = crs_x)
            }
        } else {
            if (!is.na(crs) & !is.na(crs_x)) {
                ## Convert to the target object CRS
                points <- inla.spTransform(points, CRSobj = crs_x)
            }
        }
    }
    ## Find indices:
    ids <- unlist(sp::over(x, points, returnList = TRUE))

    ids
}

## Nothing to export from here

`inla.expand.dataframe.1` <- function(response, dataframe,
                                      control.hazard = inla.set.control.hazard.default())
{
    n.intervals <- control.hazard$n.intervals
    cutpoints <- control.hazard$cutpoints

    if (class(response) != "inla.surv") {
        stop("Response has to be an object of class `inla.surv'")
    }
    ## not used
    response$.special <- NULL

    class(response) <- NULL
    event <- response$event
    nn <- length(event)

    time <- numeric(nn)
    time[response$event == 1L] <- response$time[response$event == 1L]
    time[response$event == 0L] <- response$lower[response$event == 0L]

    truncation <- response$truncation

    ## create cutpoints if not provided
    if (is.null(cutpoints)) {
        cutpoints <- seq(0L, max(time), len = n.intervals + 1L)
    }

    new.data <- inla.get.poisson.data.1(time = time, truncation = truncation, event = event, cutpoints = cutpoints)
    expand.df <- table(new.data$indicator)

    if (!missing(dataframe) && prod(dim(dataframe)) > 0L) {
        new.dataframe <- as.data.frame(matrix(0.0, length(new.data$y), dim(dataframe)[2L]))
        for (i in 1L:dim(dataframe)[2L]) {
            new.dataframe[, i] <- rep(dataframe[, i], expand.df)
        }

        ## alternative code,  not faster...
        ## new.dataframe = apply(dataframe, 2, rep, times = expand.df)

        new.dataframe <- as.data.frame(new.dataframe)
        ## just give some name that we are able to recognise afterwards
        names(new.dataframe) <- paste("fake.dataframe.names", 1L:dim(new.dataframe)[2L])
    } else {
        new.dataframe <- NULL
    }

    res <- data.frame(
        y..coxph = new.data$y,
        E..coxph = new.data$E,
        expand..coxph = rep(1:nrow(dataframe), expand.df),
        baseline.hazard = cutpoints[new.data$baseline.hazard],
        baseline.hazard.idx = new.data$baseline.hazard,
        baseline.hazard.time = cutpoints[new.data$baseline.hazard],
        baseline.hazard.length = diff(cutpoints)[new.data$baseline.hazard],
        new.dataframe
    )

    names(res)[grep("fake.dataframe.names", names(res))] <- names(dataframe)

    return(list(data = res, data.list = list(baseline.hazard.values = cutpoints)))
}

`inla.get.poisson.data.1` <- function(time, truncation, event, cutpoints)
{
    if (TRUE) {
        ##
        ## this is the new code of (Jan 21st, 2021) that runs better
        ##
        data.new <- numeric(0L)
        nn <- length(event)
        start <- as.numeric(cut(truncation, cutpoints, include.lowest = FALSE))
        end <- as.numeric(cut(time, cutpoints, include.lowest = TRUE))
        ds <- diff(cutpoints)

        res <- lapply(1L:length(time),
                      function(i) ({
                          if (is.na(start[i])) {
                              if (end[i] > 1.0) {
                                  dc <- cbind(ds[1L:(end[i] - 1L)],
                                              rep(0L, (end[i] - 1L)),
                                              rep(i, (end[i] - 1L)),
                                              c(1L:(end[i] - 1L))
                                              )
                              } else {
                                  dc <- numeric(0L)
                              }
                              dc <- rbind(dc, cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i]))
                          }
                          else {
                              if (start[i] < end[i]) {
                                  dc <- cbind((cutpoints[start[i] + 1L] - truncation[i]), 0L, i, start[i])
                                  if (end[i] > (start[i] + 1L)) {
                                      dc <- rbind(dc,
                                                  cbind(ds[(start[i] + 1L):(end[i] - 1L)],
                                                        rep(0L, (end[i] - start[i] - 1L)),
                                                        rep(i, (end[i] - start[i] - 1L)),
                                                        c((start[i] + 2L):(end[i]) - 1L)))
                                  }
                                  dc <- rbind(dc, cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i]))
                              } else if (start[i] == end[i]) {
                                  dc <- cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i])
                              } else {
                                  stop("Truncation cannot be greater than time")
                              }
                          }
                          return(t(dc))
                      }))
        data.new <- matrix(unlist(res), ncol = 4, byrow = TRUE)
        ##
    } else {
        ##
        ## this is the old code that is slow for huge ammount of data
        ##
        data.new <- numeric(0L)
        nn <- length(event)
        start <- as.numeric(cut(truncation, cutpoints, include.lowest = FALSE))
        end <- as.numeric(cut(time, cutpoints, include.lowest = TRUE))
        ds <- diff(cutpoints)

        for (i in 1L:length(time)) {
            if (is.na(start[i])) {
                if (end[i] > 1.0) {
                    dc <- cbind(
                        ds[1L:(end[i] - 1L)], rep(0L, (end[i] - 1L)), rep(i, (end[i] - 1L)),
                        c(1L:(end[i] - 1L))
                    )
                } else {
                    dc <- numeric(0L)
                }
                dc <- rbind(dc, cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i]))
                data.new <- rbind(data.new, dc)
            }
            else {
                if (start[i] < end[i]) {
                    dc <- cbind((cutpoints[start[i] + 1L] - truncation[i]), 0L, i, start[i])
                    if (end[i] > (start[i] + 1L)) {
                        dc <- rbind(dc, cbind(
                                            ds[(start[i] + 1L):(end[i] - 1L)], rep(0L, (end[i] - start[i] - 1L)),
                                            rep(i, (end[i] - start[i] - 1L)),
                                            c((start[i] + 2L):(end[i]) - 1L)
                                        ))
                    }
                    dc <- rbind(dc, cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i]))
                    data.new <- rbind(data.new, dc)
                } else if (start[i] == end[i]) {
                    dc <- cbind(time[i] - (cutpoints[end[i]]), event[i], i, end[i])
                    data.new <- rbind(data.new, dc)
                } else {
                    stop("Truncation cannot be greater than time")
                }
            }
        }
    }

    data.new <- data.frame(
        E = data.new[, 1L],
        y = data.new[, 2L],
        indicator = data.new[, 3L],
        baseline.hazard = data.new[, 4L]
    )

    return(data.new)
}


`inla.expand.dataframe.2` <- function(response, dataframe,
                                      control.hazard = inla.set.control.hazard.default())
{
    n.intervals <- control.hazard$n.intervals
    cutpoints <- control.hazard$cutpoints

    if (class(response) != "inla.surv") {
        stop("Response has to be an object of class `inla.surv'")
    }
    ## not used
    response$.special <- NULL

    class(response) <- NULL
    event <- response$event
    nn <- length(event)

    ## nhpp models do not work for interval censoring
    if (sum(event == 3L) > 0L || sum(event == 2L) > 0L) {
        stop("coxph model does not work for event type 2 and 3.")
    }

    ## time, event and subjects related to response
    time <- numeric(nn)
    time[response$event == 1L] <- response$time[response$event == 1L]
    time[response$event == 0L] <- response$time[response$event == 0L]
    subject <- response$subject

    ## checking for fixed covariates:: nhpp model works for fixed covariates only
    aa1 <- which(names(dataframe) == "subject")
    aa2 <- which(names(dataframe) == "time")
    aa3 <- which(names(dataframe) == "event")
    jj <- unique(dataframe$subject)
    subject.first.line <- numeric(length(jj))
    for (i in 1L:length(jj)) {
        rows <- which(dataframe[, aa1] == i)
        sem <- dataframe[rows, -c(aa1, aa2, aa3), drop = FALSE]
        if (mode(apply(sem, 2L, unique)) == "list") {
            stop("coxph with subject only works for fixed covariates")
        }
        subject.first.line[i] <- rows[1L]
    }
    dataframe.copy <- dataframe[subject.first.line, -c(aa1, aa2, aa3), drop = FALSE]
    if (is.null(cutpoints)) {
          cutpoints <- seq(0.0, max(time), len = n.intervals + 1L)
      }

    new.data <- inla.get.poisson.data.2(time = time, subject = subject, event = event, cutpoints = cutpoints)

    ## we want to expand  only covariates
    aa <- table(new.data$indicator)
    ind <- unique(new.data$indicator)
    stopifnot(dim(dataframe)[2L] > 3L)
    new.dataframe <- as.data.frame(matrix(0.0, length(new.data$y), dim(dataframe)[2L] - 3L))

    col.data <- grep("(subject)|(time)|(event)", names(dataframe))
    if (length(col.data) != 3L) {
          stop("data.frame does not contains columns with names `subject', `time' and `event'")
      }

    ## rewriting the covariates as per new data
    for (i in 1L:dim(dataframe.copy)[2L]) {
          new.dataframe[, i] <- rep(dataframe.copy[, i], aa)
      }
    names(new.dataframe) <- names(dataframe)[-col.data]

    res <- data.frame(
        y..coxph = new.data$y,
        E..coxph = new.data$E,
        baseline.hazard = cutpoints[new.data$baseline.haz],
        baseline.hazard.idx = new.data$baseline.haz,
        baseline.hazard.time = cutpoints[new.data$baseline.haz],
        baseline.hazard.length = diff(cutpoints)[new.data$baseline.haz],
        subject = new.data$indicator,
        new.dataframe
    )

    return(list(data = res, data.list = list(baseline.hazard.values = cutpoints)))
}

`inla.get.poisson.data.2` <- function(subject, time, event, cutpoints)
{
    data.new <- numeric(0L)
    nn <- max(subject)
    ds <- diff(cutpoints)
    ris <- matrix(0.0, max(subject), (length(cutpoints) - 1L))
    dataF <- cbind(subject, time, event)
    end <- integer(0L)
    for (i in 1L:nn) {
        da <- matrix(dataF[dataF[, 1L] == i, ], ncol = 3L)
        ## to find the interval for each recurrent time
        rec <- cut(da[, 2L], cutpoints, labels = 1L:(length(cutpoints) - 1L))
        ris[i, ] <- tapply(da[, 3L], rec, sum)
        ris[i, ][is.na(ris[i, ])] <- 0
        end <- c(end, as.numeric(cut(max(time[subject == i]), cutpoints, include.lowest = TRUE)))
    }

    ## counting number of events in each interval for every subject
    totalevent <- 0
    length(totalevent) <- 0
    for (i in 1L:nn) {
        totalevent <- c(totalevent, ris[i, ])
    }

    ## checking for interval lengths
    E <- numeric(0)
    length(E) <- 0
    for (i in 1L:nn) {
        if (end[i] == 1L) {
            E <- c(E, c(max(time[subject == i]) - cutpoints[1L]), rep(0.0, length(cutpoints) - 1L - end[i]))
        } else {
            E <- c(
                E, ds[1L:end[i] - 1L], max(time[subject == i]) - cutpoints[end[i]],
                rep(0.0, length(cutpoints) - 1L - end[i])
            )
        }
    }

    ## combining the number of events, interval lengths, baseline.hazard and subject.
    index <- rep(1L:nn, each = length(cutpoints) - 1L)
    baseline.haz <- rep(1L:(length(cutpoints) - 1L), nn)
    dc <- cbind(index, E, totalevent, baseline.haz)
    data.new <- data.frame(indicator = dc[, 1L], E = dc[, 2L], y = dc[, 3L], baseline.haz = dc[, 4L]) # y= no.of events
    data.new <- data.new[data.new[, 2L] != 0L, ]

    return(data.new)
}

## Export: inla.contrib.sd

##!\name{contrib.sd}
##!\alias{contrib.sd}
##!\alias{inla.contrib.sd}
##!\title{Computes the standard deviation for the structured (random) effects in an INLA model}
##!\description{Computes the posterior distribution of the standard deviations for the structured 
##!             (random) effects in an INLA model, starting from the default output based on the precisions}
##!\usage{
##!    inla.contrib.sd(model, nsamples=1000)
##!}
##!\arguments{
##!    \item{model}{An INLA model, fitted calling the \code{inla()}-function. The formula specified for the model 
##!            should include at least one structured (random) effect in the form \code{f(variable, model="iid")}.}
##!    \item{nsamples}{The number of simulations from the posterior distribution of the standard deviations
##!                    used to compute the summary statistics}
##!}
##!\value{
##!     \code{inla.contrib.sd} returns a matrix \code{samples} including the simulated values from the
##!                            posterior distributions as well as a summary table \code{hyper} reporting 
##!                            the mean, sd and 95\% credible interval for the posterior distributions of each random effect.
##!}
##!\author{Gianluca Baio \email{gianluca@stats.ucl.ac.uk}}
##!\seealso{
##!    \code{\link{inla}}
##!}
##!\examples{
##!# Data generation
##!n=12
##!Ntrials = sample(c(80:100), size=n, replace=TRUE)
##!eta = rnorm(n,0,0.5)
##!prob = exp(eta)/(1 + exp(eta))
##!y = rbinom(n, size=Ntrials, prob = prob)
##!data=data.frame(y=y,z=1:n)
##!formula=y~f(z,model="iid")
##!m=inla(formula,data=data,family="binomial",Ntrials=Ntrials)
##!summary(m)
##!s=inla.contrib.sd(m)
##!s$hyper
##!hist(s$samples,xlab="standard deviation for z",main="")
##!}

`inla.contrib.sd` <- function(model, nsamples=1000)
{
    ## contributed by Gianluca Baio <gianluca@stats.ucl.ac.uk>

    ## Computes the sd for the random effects in an INLA model
    ## 1. Defines the precision (generates a matrix with bins and
    ## density on the precision scale)

    ## 2. Simulates replications from the posterior distributions of
    ## the quantities of interest

    ## Names of the variables associated with structured effects
    rand.effs <- names(model$marginals.hyperpar)
    for (i in 1:length(rand.effs)) {
        cmd <- paste("prec.marg.",i,"<-model$marginals.hyperpar$'",rand.effs[i],"'",sep="")
        eval(parse(text=cmd)) # marginal distribution of the precision, tau
        ## Simulation from the posterior marginal distribution for sigma = 1/sqrt(tau)
        cmd <- paste("sigma.", i,
                     "<- inla.rmarginal(nsamples,inla.marginal.transform(function(x) 1/sqrt(x), prec.marg.",
                     i,"))",sep="")
        eval(parse(text=cmd))
    }

    ## Outputs of the function
    mat <- matrix(NA, nsamples, length(rand.effs))
    for (i in 1:length(rand.effs)) {
        cmd <- paste("mat[,i] <- sigma.",i,sep="")
        eval(parse(text=cmd)) 
    }
    names2 <- gsub("Precision","sd",rand.effs)
    colnames(mat) <- names2

    tab <- matrix(NA,length(rand.effs),4)
    for (i in 1:length(rand.effs)) {
        tab[i,] <- c(mean(mat[,i]),sd(mat[,i]),quantile(mat[,i],.025),quantile(mat[,i],.975))
    }
    rownames(tab) <- names2
    colnames(tab) <- c("mean","sd","2.5%","97.5%")

    return (list(samples=mat, hyper=tab))
}

## Export: inla.posterior.sample inla.posterior.sample.eval

## ! \name{inla.sample}
## ! \alias{inla.posterior.sample}
## ! \alias{posterior.sample}
## ! \alias{inla.posterior.sample.eval}
## ! \alias{posterior.sample.eval}
## !
## ! \title{Generate samples, and functions thereof, from an approximated posterior of a fitted model}
## !
## ! \description{This function generate samples, and functions of those,
## !              from an approximated posterior of a fitted model (an inla-object)}
## ! \usage{
## !     inla.posterior.sample(n = 1L, result, selection = list(),
## !                           intern = FALSE,
## !                           use.improved.mean = TRUE, skew.corr = TRUE,
## !                           add.names = TRUE, seed = 0L, num.threads = NULL,
## !                           parallel.configs = TRUE,  verbose=FALSE)
## !     inla.posterior.sample.eval(fun, samples, return.matrix = TRUE, ...)
## ! }
## !
## ! \arguments{
## !   \item{n}{Number of samples.}
## !   \item{result}{The inla-object, ie the output from an \code{inla}-call.
## !       The \code{inla}-object must be created with
## !       \code{control.compute=list(config=TRUE)}.}
## !   \item{selection}{Select what part of the sample to return. By default, the whole sample
## !       is returned. \code{selection} is a named list with the name of the components of
## !       the sample, and what indices of them to return. Names include \code{APredictor},
## !       \code{Predictor}, \code{(Intercept)},  and otherwise names in the formula.
## !       The values of the list, is interpreted as indices. If they
## !       are negative, they are interpreted as 'not', a zero is interpreted as 'all',  and
## !       positive indices are interpreted as 'only'. The names of elements of each samples
## !       refer to the indices in the full sample. }
## !   \item{intern}{Logical. If \code{TRUE} then produce samples in the
## !        internal scale for the hyperparmater, if \code{FALSE} then produce
## !        samples in the user-scale. (For example log-precision (intern)
## !        and precision (user-scale))}
## !   \item{use.improved.mean}{Logical. If \code{TRUE} then use the
## !        marginal mean values when constructing samples. If \code{FALSE}
## !        then use the mean in the Gaussian approximations.}
## !   \item{skew.corr}{Logical. If \code{TRUE} then correct samples for skewness,
## !       if \code{FALSE},  do not correct samples for skewness (ie use the
## !       Gaussian).}
## !   \item{add.names}{Logical. If \code{TRUE} then add name for each elements of each
## !       sample. If \code{FALSE}, only add name for the first sample.
## !       (This save space.)}
## !   \item{seed}{See the same argument in \code{?inla.qsample} for further
## !               information. In order to produce reproducible results,  you
## !               ALSO need to make sure the RNG in R is in the same state,
## !               see example below.  When \code{seed} is non-zero,
## !               \code{num.threads} is forced to "1:1" and parallel.configs is
## !               set to \code{FALSE}, since parallel sampling would not produce
## !               a reproducible sequence of pseudo-random numbers.}
## !   \item{num.threads}{The number of threads to use in the format 'A:B' defining the number threads in the
## !                      outer (A) and inner (B) layer for nested parallelism. A '0' will be replaced
## !                      intelligently.
## !                      \code{seed!=0} requires serial comptuations.}
## !   \item{parallel.configs}{Logical. If \code{TRUE} and not on Windows,
## !                           then try to run each configuration in
## !                           parallel (not Windows) using \code{A} threads (see \code{num.threads}),
## !                           where each of them is using \code{B:0} threads.}
## !   \item{verbose}{Logical. Run in verbose mode or not.}
## !   \item{fun}{The function to evaluate for each sample. Upon entry, the variable names
## !              defined in the model are defined as the value of the sample.
## !              The list of names are defined in \code{result$misc$configs$contents} where
## !              \code{result} is an \code{inla}-object. This includes predefined names for
## !              for the linear predictor (\code{Predictor} and \code{APredictor}),  and the
## !              intercept (\code{(Intercept)} or \code{Intercept}).
## !              The hyperparameters are defined as \code{theta},  no matter if they are in the
## !              internal scale or not. The function \code{fun} can also return a vector.
## !              To simplify usage, \code{fun} can also be a vector character's. In this case
## !              \code{fun} it is interpreted as (strict) variable
## !              names, and a function is created that return these variables:
## !              if argument \code{fun} equals \code{c("Intercept", "a[1:2]")},  then this is equivalent to
## !              pass \code{function() return(c(get('Intercept'), get('a[1:2]')))}.}
## !   \item{samples}{\code{samples} is the output from \code{inla.posterior.sample()}}
## !   \item{return.matrix}{Logical. If \code{TRUE},  then return the samples of \code{fun}
## !                         as matrix,  otherwise,  as a list.}
## !   \item{...}{Additional arguments to \code{fun}}
## !}
## !\details{The hyperparameters are sampled from the configurations used to do the
## !       numerical integration, hence if you want a higher resolution, you need to
## !       to change the \code{int.stratey} variable and friends. The latent field is
## !       sampled from the Gaussian approximation conditioned on the hyperparameters,
## !       but with a correction for the mean (default),
## !        and optional (and by default) corrected for the estimated skewness.
## !
## !       The log.density report is only correct when there is no constraints.
## !       With constraints, it correct the Gaussian part of the sample for the constraints.
## !
## !       After the sample is (optional) skewness corrected, the log.density
## !       is is not exact for correcting for constraints, but the error is very
## !       small in most cases.
## !}
## !\value{\code{inla.posterior.sample} returns a list of the samples,
## !       where each sample is a list with
## !     names \code{hyperpar} and \code{latent}, and with their marginal
## !     densities in \code{logdens$hyperpar} and \code{logdens$latent}
## !     and the joint density is in \code{logdens$joint}.
## !     \code{inla.posterior.sample.eval} return a list or a matrix of
## !     \code{fun} applied to each sample.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org} and Cristian Chiuchiolo \email{cristian.chiuchiolo@kaust.edu.sa}}
## !
## !\examples{
## !  r = inla(y ~ 1 ,data = data.frame(y=rnorm(1)), control.compute = list(config=TRUE))
## !  samples = inla.posterior.sample(2,r)
## !
## !  ## reproducible results:
## !  inla.seed = as.integer(runif(1)*.Machine$integer.max)
## !  set.seed(12345)
## !  x = inla.posterior.sample(10, r, seed = inla.seed, num.threads="1:1")
## !  set.seed(12345)
## !  xx = inla.posterior.sample(10, r, seed = inla.seed, num.threads="1.1")
## !  all.equal(x, xx)
## !
## ! set.seed(1234)
## ! n = 25
## ! xx = rnorm(n)
## ! yy = rev(xx)
## ! z = runif(n)
## ! y = rnorm(n)
## ! r = inla(y ~ 1 + z + f(xx) + f(yy, copy="xx"),
## !         data = data.frame(y, z, xx, yy),
## !         control.compute = list(config=TRUE),
## !         family = "gaussian")
## ! r.samples = inla.posterior.sample(10, r)
## !
## ! fun = function(...) {
## !     mean(xx) - mean(yy)
## ! }
## ! f1 = inla.posterior.sample.eval(fun, r.samples)
## !
## ! fun = function(...) {
## !     c(exp(Intercept), exp(Intercept + z))
## ! }
## ! f2 = inla.posterior.sample.eval(fun, r.samples)
## !
## ! fun = function(...) {
## !     return (theta[1]/(theta[1] + theta[2]))
## ! }
## ! f3 = inla.posterior.sample.eval(fun, r.samples)
## !
## ! ## Predicting nz new observations, and
## ! ## comparing the estimated one with the true one
## ! set.seed(1234)
## ! n = 100
## ! alpha = beta = s = 1
## ! z = rnorm(n)
## ! y = alpha + beta * z + rnorm(n, sd = s)
## ! r = inla(y ~ 1 + z,
## !         data = data.frame(y, z),
## !         control.compute = list(config=TRUE),
## !         family = "gaussian")
## ! r.samples = inla.posterior.sample(10^3, r)
## !
## ! ## just return samples of the intercept
## ! intercepts = inla.posterior.sample.eval("Intercept", r.samples)
## !
## ! nz = 3
## ! znew = rnorm(nz)
## ! fun = function(zz = NA) {
## !     ## theta[1] is the precision
## !     return (Intercept + z * zz +
## !             rnorm(length(zz), sd = sqrt(1/theta[1])))
## ! }
## ! par(mfrow=c(1, nz))
## ! f1 = inla.posterior.sample.eval(fun, r.samples, zz = znew)
## ! for(i in 1:nz) {
## !     hist(f1[i, ], n = 100, prob = TRUE)
## !     m = alpha + beta * znew[i]
## !     xx = seq(m-4*s, m+4*s, by = s/100)
## !     lines(xx, dnorm(xx, mean=m, sd = s), lwd=2)
## ! }
## !
## ! ## 
## ! ## Be aware that using non-clean variable names might be a little tricky
## ! ## 
## ! n <- 100
## ! X <- matrix(rnorm(n^2), n, 2)
## ! x <- X[, 1]
## ! xx <- X[, 2]
## ! xxx <- x*xx
## ! 
## ! y <- 1 + 2*x + 3*xx + 4*xxx + rnorm(n, sd = 0.01)
## ! 
## ! r <- inla(y ~ X[, 1]*X[, 2],
## !           data = list(y = y, X = X),
## !           control.compute = list(config = TRUE))
## ! print(round(dig = 4, r$summary.fixed[,"mean"]))
## ! 
## ! sam <- inla.posterior.sample(100, r)
## ! sam.extract <- inla.posterior.sample.eval(
## !     (function(...) {
## !         beta.1 <- get("X[, 1]")
## !         beta.2 <- get("X[, 2]")
## !         beta.12 <- get("X[, 1]:X[, 2]")
## !         return(c(Intercept, beta.1, beta.2, beta.12))
## !     }), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## ! 
## ! ## a simpler form can also be used here, and in the examples below
## ! sam.extract <- inla.posterior.sample.eval(
## !                c("Intercept", "X[, 1]", "X[, 2]", "X[, 1]:X[, 2]"), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## !
## ! r <- inla(y ~ x + xx + xxx,
## !           data = list(y = y, x = x, xx = xx, xxx = xxx), 
## !           control.compute = list(config = TRUE))
## ! 
## ! sam <- inla.posterior.sample(100, r)
## ! sam.extract <- inla.posterior.sample.eval(
## !     (function(...) {
## !         return(c(Intercept, x, xx, xxx))
## !     }), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## !
## ! sam.extract <- inla.posterior.sample.eval(c("Intercept", "x", "xx", "xxx"), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## !
## ! r <- inla(y ~ x*xx,
## !           data = list(y = y, x = x, xx = xx), 
## !           control.compute = list(config = TRUE))
## ! 
## ! sam <- inla.posterior.sample(100, r)
## ! sam.extract <- inla.posterior.sample.eval(
## !     (function(...) {
## !         return(c(Intercept, x, xx, get("x:xx")))
## !     }), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## !
## ! sam.extract <- inla.posterior.sample.eval(c("Intercept", "x", "xx", "x:xx"), sam)
## ! print(round(dig = 4, rowMeans(sam.extract)))
## ! }
 
 
## Comments to the code below (contributed by CC):
##
## In order to call the right interpolating functions related to the different skewness mapping
## contained in the object "fun.splines" below we use a small trick for defining the right index order
## considering ties as well ( which is common in our skewness context). The following example should
## be clear enough:
##
## > x = sample(round(runif(3), digits = 2), size = 10, replace = T)
##
## > x
## [1] 0.88 0.88 0.16 0.16 0.16 0.53 0.53 0.53 0.16 0.53
##
## > as.numeric(factor(rank(a)))
## [1] 3 2 3 3 2 3 2 1 1 2
 
inla.create.sn.cache <- function() {
    ## Faster function for qsn,psn and dsn functions (using interpolations from the true ones)
    speed.fsn <- function(s, x, skew, fun.splines, deriv = 0) {
        ind <- order(skew)
        sss <- skew[ind]
        r <- as.numeric(factor(rank(sss)))
        skew.unique <- unique(sss)
        ## index for stored interpolated skewness by matching
        skew.ind <- findInterval(skew.unique, s)
        if (is.matrix(x)) {
            x.sample <- x[ind, , drop = FALSE]
            res.fsn <- unlist(lapply(
                seq_along(skew.unique),
                function(i) {
                lapply(t(x.sample[which(r == i), ]), fun.splines[[skew.ind[i]]])
            }
            ))
            res.fsn <- matrix(res.fsn, nrow = length(skew), ncol = ncol(x.sample), byrow = T)
            res.fsn <- res.fsn[order(ind), , drop = FALSE]
        } else {
            x.sample <- x[ind]
            res.fsn <- unlist(lapply(seq_along(skew.unique), function(i) {
                lapply(x.sample[which(r == i)], fun.splines[[skew.ind[i]]], deriv = deriv)
            }))
            res.fsn <- res.fsn[order(ind)]
        }
        return(res.fsn)
    }

    ## This code section will run only once in order to create the local object 'sn.cache'
    envir <- inla.get.inlaEnv()
    inla.require("sn", stop.on.error = TRUE)

    ## The following quantities can be changed and affect the whole code
    dig <- 2 # skewness precision (better keep this one for fast computations)
    sub <- 0.99 # highest value for the mapping function
    slb <- -sub # lowest value for the mapping function
    step <- 0.01 # step for the skewness sequence to be computed
    s <- seq(slb, sub, by = step) # overall skewness to compute
    s.pos <- seq(0, sub, by = step) # partial skewness to compute
    points <- seq(-4, 4, len = 50) # 50 points is a good deal for speed/accuracy

    ## Needed functions to store the tabulated values for 'qsn','psn' and 'dsn' functions
    ## Skewness Mapping (function)
    sn.map <- function(s, mu, sigma) {
        ## delta <- alpha/sqrt(1+alpha^2)
        s[which(is.na(s))] <- 0
        delta <- sign(s) * sqrt((pi / 2) * (abs(s)^(2 / 3) / (((4 - pi) / 2)^(2 / 3) + abs(s)^(2 / 3))))
        alpha <- delta / sqrt(1 - delta^2)
        xi <- mu - delta * sqrt((2 * sigma^2) / (pi - 2 * delta^2))
        omega <- sqrt((pi * sigma^2) / (pi - 2 * delta^2))
        return(list(alpha = alpha, xi = xi, omega = omega))
    }

    ## Needed function to construct the interpolations
    skew.spline.al <- function(s, skew, skew.store, type = "qsn") {
        res <- 0
        f <- function(x) {
            xx <- splinefun(s, x)
            return(xx)
        }
        s.fun <- apply(skew.store, 2, f)
        ## values table from "skew.points.table"
        if (type == "qsn") {
            res <- sapply(s.fun, function(f) f(abs(skew)))
            if (is.vector(res)) res <- as.matrix(t(res))
            ## Negative skewness positions computed by anti-symmetry relation
            if (any(skew < 0)) {
                neg.pos <- which(skew < 0)
                ord <- order(-res[1, ])
                res[neg.pos, ] <- -1 * res[neg.pos, ord]
            }
        } else {
            res <- sapply(s.fun, function(f) f(skew))
            if (is.vector(res)) res <- as.matrix(t(res))
        }
        return(res)
    }

    skew.points.table <- function(s.pos, s, points) {
        skew.store.qsn <- matrix(NA, nrow = length(s.pos), ncol = length(points))
        skew.store.jac <- matrix(NA, nrow = length(s), ncol = length(points))
        s.ind <- sn.map(s = s, mu = rep(0, length(s)), sigma = rep(1, length(s)))
        counter <- 1
        for (i in seq_along(s)) {
            alpha.x <- s.ind$alpha[i]
            xi.x <- s.ind$xi[i]
            omega.x <- s.ind$omega[i]
            skew.store.jac[i, ] <- sn::psn(points,
                                           xi = xi.x,
                                           omega = omega.x, alpha = alpha.x
                                           )
            if (s[i] >= 0) {
                skew.store.qsn[counter, ] <- sn::qsn(pnorm(points),
                                                     xi = xi.x,
                                                     omega = omega.x, alpha = alpha.x
                                                     )
                counter <- counter + 1
            }
        }
        return(list(skew.store.qsn = skew.store.qsn, skew.store.jac = skew.store.jac))
    }

    ## overall tabulated values for qsn,psn,dsn functions
    store.list <- skew.points.table(s.pos = s.pos, s = s, points = points)

    ## tabulated values for 'qsn' function
    store.qsn <- store.list$skew.store.qsn

    ## tabulated values for 'psn','dsn' functions
    store.jac <- store.list$skew.store.jac
    f.spline <- function(ss, type) {
        if (type != "jac") {
            val <- splinefun(points, skew.spline.al(s = s.pos, skew = ss, skew.store = store.qsn))
        } else {
            val <- splinefun(points, skew.spline.al(s = s, skew = ss, skew.store = store.jac, type = "jac"))
        }
        return(val)
    }
    ## matrix version of the skewness vector
    skew.ind <- as.matrix(s)
    ## splines for 'qsn'
    skew.splines <- apply(skew.ind, 1, f.spline, type = "fun")
    ## splines for 'psn','dsn'
    skew.splines.jac <- apply(skew.ind, 1, f.spline, type = "jac")
    ## Storing permanently the needed objects in a local INLA environment
    envir$sn.cache <- list(
        skew.splines = skew.splines, skew.splines.jac = skew.splines.jac,
        s = s, dig = dig, speed.fsn = speed.fsn
    )
    return(invisible())
}


inla.posterior.sample <- function(n = 1L, result, selection = list(),
                                  intern = FALSE,
                                  use.improved.mean = TRUE, skew.corr = TRUE,
                                  add.names = TRUE, seed = 0L, num.threads = NULL,
                                  parallel.configs = TRUE, verbose = FALSE) {
    ## New inla.posterior.sample with skewness correction. contributed by CC.

    stopifnot(!missing(result) && any(class(result) == "inla"))
    if (is.null(result$misc$configs)) {
        stop("You need an inla-object computed with option 
         'control.compute=list(config = TRUE)'.")
    }

    if (!is.null(result$misc$configs$.preopt) && result$misc$configs$.preopt) {
        rfake <- list(mlik = result$mlik, 
                      summary.hyperpar = result$summary.hyperpar,
                      internal.summary.hyperpar = result$internal.summary.hyperpar, 
                      misc = list(from.theta = result$misc$from.theta,
                                  to.theta = result$misc$to.theta,
                                  configs = result$misc$configs))
        rfake$misc$configs$.preopt <- FALSE
        class(rfake) <- "inla"

        ct <- result$misc$configs$contents
        for(nm in c("APredictor", "Predictor")) {
            if (ct$tag[1] == nm) {
                ct$tag <- ct$tag[-1]
                ct$start <- ct$start[-1] - ct$start[2] + 1
                ct$length <- ct$length[-1]
            }
        }
        rfake$misc$configs$contents <- ct

        ## yes, selection is not an argument in this call, but we account for it afterwards
        xx <- inla.posterior.sample(n, rfake, intern = intern,
                                    use.improved.mean = use.improved.mean,
                                    skew.corr = skew.corr,
                                    add.names = add.names, seed = seed,
                                    num.threads = num.threads,
                                    parallel.configs = parallel.configs, verbose = verbose)

        A <- rbind(result$misc$configs$pA %*% result$misc$configs$A,
                   result$misc$configs$A)
        sel <- inla.posterior.sample.interpret.selection(selection, result)

        if (nrow(result$misc$configs$pA) > 0) {
            pnam <- c(paste0("APredictor:", seq_len(nrow(result$misc$configs$pA))),
                      paste0("Predictor:", seq_len(nrow(result$misc$configs$A))))
        } else {
            pnam <- paste0("Predictor:", seq_len(nrow(result$misc$configs$A)))
        }
        attr(xx, ".contents") <- result$misc$configs$contents

        off <- result$misc$configs$offsets
        for(i in seq_along(xx)) {
            nam <- rownames(xx[[i]]$latent)
            xx[[i]]$latent <- c(as.numeric(A %*% xx[[i]]$latent) + off, as.numeric(xx[[i]]$latent))[sel]
            if (!is.null(nam)) {
                names(xx[[i]]$latent) <- c(pnam, nam)[sel]
            }
            xx[[i]]$latent <- as.matrix(xx[[i]]$latent, ncol = 1)
        }
        return(xx)
    }

    if (is.null(num.threads)) {
        num.threads <- inla.getOption("num.threads")
    }
    if (seed != 0L) {
        num.threads.user <- inla.parse.num.threads(num.threads)
        num.threads <- inla.parse.num.threads("1:1")
        if (num.threads != num.threads.user) {
            warning("Since 'seed!=0', parallel model is disabled and serial model is selected, num.threads='1:1'")
        }
        ## Since parallel.configs aren't used on winows anyway, warning about
        ## overriding it on other systems is overkill and doesn't really help.
        # if (!missing(parallel.configs) && !isFALSE(parallel.configs)) {
        # warning("Since 'seed!=0', parallel model is disabled and serial model is selected, parallel.configs=FALSE")
        # }
        parallel.configs <- FALSE
    } else {
        num.threads <- inla.parse.num.threads(num.threads)
    }

    if (use.improved.mean == FALSE) {
        skew.corr <- FALSE
    }
    if (skew.corr == TRUE && !exists("sn.cache", envir = inla.get.inlaEnv())) {
        ## function call for creating a local object in INLA environment
        inla.create.sn.cache()
    }

    sel <- inla.posterior.sample.interpret.selection(selection, result)
    if (sum(sel) == 0) {
        return(matrix(NA, 0, 0))
    }

    sel.n <- sum(sel)
    sel.map <- which(sel)
    stopifnot(length(sel.map) == sel.n)
    names(sel.map) <- names(sel[sel.map])

    n <- as.integer(n)
    stopifnot(is.integer(n) && n > 0L)

    ## first sample the theta(-index)
    cs <- result$misc$configs
    ld <- numeric(cs$nconfig)
    for (i in 1:cs$nconfig) {
        ld[i] <- cs$config[[i]]$log.posterior
    }
    p <- exp(ld - max(ld))
    idx <- sort(sample(1:cs$nconfig, n, prob = p, replace = TRUE))
    n.idx <- numeric(cs$nconfig)
    n.idx[] <- 0
    for (i in 1:cs$nconfig) {
        n.idx[i] <- sum(idx == i)
    }
    con <- cs$contents
    all.samples <- rep(list(c()), n)
    i.sample <- 1L

    ## it is easier to parallise here,  just doing the samples in parallel. then we process them
    ## in serial afterwards, since that is much less of an job

    if (!inla.os("windows") && parallel.configs) {
        nt <- as.numeric(strsplit(num.threads, ":")[[1]])
        ncores <- parallel::detectCores(all.tests = TRUE, logical = FALSE)
        if (nt[1] == 0) {
            nt[1] <- max(1, min(cs$nconfig, ncores))
            nt[2] <- 1
        }
        if (nt[2] == 0) {
            nt[2] <- max(1, ncores %/% nt[1])
        }
        # In parallel mode, each thread needs it's own random sequence,
        # so seed must be 0; seed != 0 is handled above to prevent parallel
        # runs with nonzero seed.
        xx.list <- parallel::mclapply(1:cs$nconfig,
            (function(k) {
                if (n.idx[k] > 0) {
                    xx <- inla.qsample(
                        n = n.idx[k],
                        Q = cs$config[[k]]$Q,
                        mu = inla.ifelse(
                            use.improved.mean,
                            cs$config[[k]]$improved.mean,
                            cs$config[[k]]$mean
                        ),
                        constr = cs$constr,
                        logdens = TRUE,
                        seed = 0L,
                        num.threads = paste0(nt[2], ":1"),
                        selection = sel.map,
                        verbose = verbose
                    )
                    return(xx)
                } else {
                    return(NA)
                }
            }),
            mc.cores = nt[1],
            mc.preschedule = TRUE
        )
    } else {
        xx.list <- NULL
    }

    for (k in 1:cs$nconfig) {
        if (n.idx[k] > 0) {
            ## then the latent field
            if (is.null(xx.list)) {
                xx <- inla.qsample(
                    n = n.idx[k], Q = cs$config[[k]]$Q,
                    mu = inla.ifelse(use.improved.mean, cs$config[[k]]$improved.mean, cs$config[[k]]$mean),
                    constr = cs$constr, logdens = TRUE, seed = seed, num.threads = num.threads,
                    selection = sel.map, verbose = verbose
                )
            }
            else {
                xx <- xx.list[[k]]
            }
            ## if user set seed,  then just continue this rng-stream
            if (seed > 0L) seed <- -1L

            ld.theta <- cs$max.log.posterior + cs$config[[k]]$log.posterior
            nm <- names(sel.map)

            theta <- cs$config[[k]]$theta
            log.J <- 0.0
            if (!is.null(theta) && !intern) {
                for (j in 1:length(theta)) {
                    theta[j] <- do.call(result$misc$from.theta[[j]],
                        args = list(theta[j])
                    )
                }
                names(theta) <- inla.transform.names(result, names(theta))

                if (TRUE) {
                    ## new fancy code using the automatic differentiation feature in R
                    for (i in 1:length(theta)) {
                        arg.val <- formals(result$misc$from.theta[[i]])
                        arg <- names(arg.val)
                        if (length(arg) == 1L) {
                            deriv.func <- inla.eval(paste("function(", arg, ") {}"))
                        } else {
                            if (length(arg) == 2L) {
                                deriv.func <- inla.eval(paste(
                                    "function(", arg[1L], ",",
                                    arg[2L], "=", arg.val[2L], ") {}"
                                ))
                            }
                            else {
                                stopifnot(length(arg) == 3L)
                                deriv.func <- inla.eval(paste(
                                    "function(", arg[1L], ",",
                                    arg[2L], "=", arg.val[2L], ",",
                                    arg[3L], "=", arg.val[3L], ") {}"
                                ))
                            }
                        }
                        temptest <- try(D(body(result$misc$from.theta[[i]]), arg[1L]),
                            silent = TRUE
                        )
                        if (!inherits(temptest, "try-error")) {
                            body(deriv.func) <- temptest
                            log.J <- log.J - log(abs(deriv.func(cs$config[[k]]$theta[i])))
                            ## Yes, it's a minus...
                        }
                        else {
                            h <- .Machine$double.eps^0.25
                            theta.1 <- do.call(result$misc$from.theta[[i]],
                                args = list(cs$config[[k]]$theta[i] - h)
                            )
                            theta.2 <- do.call(result$misc$from.theta[[i]],
                                args = list(cs$config[[k]]$theta[i] + h)
                            )
                            log.J <- log.J - log(abs((theta.2 - theta.1) / (2.0 * h)))
                            ## Yes, it's a minus...
                        }
                    }
                    ## print(paste("logJ", log.J))
                } else {
                    ## old code using numerical differentiation
                    h <- .Machine$double.eps^0.25
                    for (i in 1:length(theta)) {
                        theta.1 <- do.call(result$misc$from.theta[[i]],
                            args = list(cs$config[[k]]$theta[i] - h)
                        )
                        theta.2 <- do.call(result$misc$from.theta[[i]],
                            args = list(cs$config[[k]]$theta[i] + h)
                        )
                        log.J <- log.J - log(abs((theta.2 - theta.1) / (2.0 * h)))
                        ## Yes, it's a minus...
                    }
                    ## print(paste("logJ", log.J))
                }
            }
            if (!skew.corr) {
                ## Default for mean correction only
                sample.corr <- xx$sample
                C.th <- xx$logdens
            } else {
                ## Skewness Correction applied on the samples
                sh.cache <- get("sn.cache", envir = inla.get.inlaEnv())
                dig <- sh.cache$dig
                s <- sh.cache$s
                skew.splines <- sh.cache$skew.splines
                skew.splines.jac <- sh.cache$skew.splines.jac
                speed.fsn <- sh.cache$speed.fsn
                mean.GA <- cs$config[[k]]$mean
                mean.SN <- cs$config[[k]]$improved.mean
                sigma.th <- sqrt(diag(cs$config[[k]]$Qinv))
                val.max <- (mean.GA - mean.SN) / sigma.th
                skew.val <- cs$config[[k]]$skewness
                ## if the skewness is NA then we simply put 0
                skew.val[which(is.na(skew.val))] <- 0
                ## we round skewness values to the second digit for matching the correct interpolations
                skew.val <- round(skew.val, digits = dig)
                s.new <- round(s, digits = dig)
                skew.table.max <- max(s.new)
                if (any(abs(skew.val) > skew.table.max)) {
                    skew.val <- pmax(-skew.table.max, pmin(skew.table.max, skew.val))
                    warning(paste0("One or more marginal skewness are too high. Coerced to be ", skew.table.max))
                }
                ## Positions for skewness different from 0
                zero.not <- which(skew.val != 0)
                if (length(zero.not) == 0 || !any(sel.map %in% zero.not)) {
                    sample.corr <- xx$sample
                    C.th <- xx$logdens
                } else {
                    skew.val.not <- skew.val[zero.not]
                    val.max.not <- val.max[zero.not]
                    ## 'dsn' evaluations by interpolation
                    dsn.not <- speed.fsn(s = s.new, x = val.max.not, skew = skew.val.not, fun.splines = skew.splines.jac, deriv = 1)
                    ## 'psn' evaluations by interpolation
                    psn.not <- speed.fsn(s = s.new, x = val.max.not, skew = skew.val.not, fun.splines = skew.splines.jac)
                    dsn.new <- dnorm(val.max)
                    dsn.new[zero.not] <- dsn.not
                    psn.new <- pnorm(val.max)
                    psn.new[zero.not] <- psn.not
                    if (length(sel.map) < length(skew.val)) {
                        mean.SN <- mean.SN[sel.map]
                        sigma.th <- sigma.th[sel.map]
                        skew.val <- skew.val[sel.map]
                        zero.not <- which(skew.val != 0)
                        if (length(zero.not) != 0) {
                            skew.val.not <- skew.val[zero.not]
                        }
                    }
                    x.sample.not <- xx$sample[zero.not, , drop = FALSE]
                    mean.SN.not <- mean.SN[zero.not]
                    sigma.th.not <- sigma.th[zero.not]
                    x.val.not <- (x.sample.not - mean.SN.not) / sigma.th.not
                    ## faster vectorized 'qsn' function call
                    fast.qsn <- speed.fsn(s = s.new, x = x.val.not, skew = skew.val.not, fun.splines = skew.splines)
                    sample.corr <- xx$sample
                    sample.store <- sigma.th.not * fast.qsn + mean.SN.not
                    sample.corr[zero.not, ] <- sample.store[1:nrow(sample.store), ]
                    C.th <- xx$logdens + sum(log(dsn.new)) - sum(log(dnorm(qnorm(psn.new))))
                }
            }
            for (i in 1:n.idx[k]) {
                if (is.null(theta)) {
                    a.sample <- list(
                        hyperpar = NULL,
                        latent = sample.corr[, i, drop = FALSE],
                        logdens = list(
                            hyperpar = NULL,
                            latent = as.numeric(C.th[i]),
                            joint = as.numeric(C.th[i])
                        )
                    )
                } else {
                    ld.h <- as.numeric(ld.theta - result$mlik[1, 1] + log.J)
                    a.sample <- list(
                        hyperpar = theta,
                        latent = sample.corr[, i, drop = FALSE],
                        logdens = list(
                            hyperpar = ld.h,
                            latent = as.numeric(C.th[i]),
                            joint = as.numeric(ld.h + C.th[i])
                        )
                    )
                }
                if (add.names || i.sample == 1L) {
                    n1 <- length(nm)
                    n2 <- length(a.sample$latent)
                    stopifnot(n2 >= n1) ## this must be true. just a check
                    if (n2 > n1) {
                        ## This is the case where lincomb.derived.only = FALSE, so these are
                        ## then added to the end. Should transfer the names of them all the way
                        ## to here, but...
                        xnm <- paste0("Lincomb:", inla.num(1:(n2 - n1)))
                        nm <- c(nm, xnm)

                        if (i.sample == 1L) {
                            ## add to the contents if needed
                            con$tag <- c(con$tag, "Lincomb")
                            con$start <- c(con$start, n1 + 1)
                            con$length <- c(con$length, n2 - n1)
                        }
                    }
                    rownames(a.sample$latent) <- nm
                } else {
                    rownames(a.sample$latent) <- NULL
                }
                all.samples[[i.sample]] <- a.sample
                i.sample <- i.sample + 1L
            }
        }
    }
    if (length(selection) == 0L) {
        attr(all.samples, ".contents") <- con
    } else {
        ## we use a selection, need to build a new 'contents' list
        con <- list(tag = names(selection), start = c(), length = c())
        for (nm in names(selection)) {
            ## from Hmisc::escapeRegex. Need it for the '(Intercept)'
            re <- paste0("^", gsub("([.|()\\^{}+$*?]|\\[|\\])", "\\\\\\1", nm), ":")
            m <- grep(re, names(sel.map))
            if (length(m) > 0) {
                con$start <- c(con$start, min(m))
                con$length <- c(con$length, diff(range(m)) + 1)
            }
        }
        attr(all.samples, ".contents") <- con
    }
    return(all.samples)
}


`inla.posterior.sample.orig` <- function(n = 1, result, selection = list(), intern = FALSE,
                                         use.improved.mean = TRUE, add.names = TRUE, seed = 0L,
                                         num.threads = NULL, verbose = FALSE) {
    ## this is the original version, before the skewness correction. keep it here for completeness.

    stopifnot(!missing(result) && any(class(result) == "inla"))
    if (is.null(result$misc$configs)) {
        stop("You need an inla-object computed with option 'control.compute=list(config = TRUE)'.")
    }

    if (seed != 0L && is.null(num.threads)) {
        num.threads <- 1L
    }
    if (is.null(num.threads)) {
        num.threads <- inla.getOption("num.threads")
    }
    num.threads <- max(num.threads, 1L)
    if (num.threads > 1L && seed != 0L) {
        stop("num.threads > 1L require seed = 0L")
    }

    sel <- inla.posterior.sample.interpret.selection(selection, result)
    if (sum(sel) == 0) {
        return(matrix(NA, 0, 0))
    }

    sel.n <- sum(sel)
    sel.map <- which(sel)
    stopifnot(length(sel.map) == sel.n)
    names(sel.map) <- names(sel[sel.map])

    n <- as.integer(n)
    stopifnot(is.integer(n) && n > 0L)

    ## first sample the theta(-index)
    cs <- result$misc$configs
    ld <- numeric(cs$nconfig)
    for (i in 1:cs$nconfig) {
        ld[i] <- cs$config[[i]]$log.posterior
    }
    p <- exp(ld - max(ld))
    idx <- sort(sample(1:cs$nconfig, n, prob = p, replace = TRUE))
    n.idx <- numeric(cs$nconfig)
    n.idx[] <- 0
    for (i in 1:cs$nconfig) {
        n.idx[i] <- sum(idx == i)
    }

    con <- cs$contents
    all.samples <- rep(list(c()), n)
    i.sample <- 1L
    for (k in 1:cs$nconfig) {
        if (n.idx[k] > 0) {
            ## then the latent field
            xx <- inla.qsample(
                n = n.idx[k], Q = cs$config[[k]]$Q,
                mu = inla.ifelse(use.improved.mean, cs$config[[k]]$improved.mean, cs$config[[k]]$mean),
                constr = cs$constr, logdens = TRUE, seed = seed, num.threads = num.threads,
                selection = sel.map, verbose = verbose
            )
            ## if user set seed,  then just continue this rng-stream
            if (seed > 0L) seed <- -1L

            ld.theta <- cs$max.log.posterior + cs$config[[k]]$log.posterior
            nm <- names(sel.map)

            theta <- cs$config[[k]]$theta
            log.J <- 0.0
            if (!is.null(theta) && !intern) {
                for (j in 1:length(theta)) {
                    theta[j] <- do.call(result$misc$from.theta[[j]], args = list(theta[j]))
                }
                names(theta) <- inla.transform.names(result, names(theta))

                if (TRUE) {
                    ## new fancy code using the automatic differentiation feature in R
                    for (i in 1:length(theta)) {
                        arg.val <- formals(result$misc$from.theta[[i]])
                        arg <- names(arg.val)
                        if (length(arg) == 1L) {
                            deriv.func <- inla.eval(paste("function(", arg, ") {}"))
                        } else {
                            if (length(arg) == 2L) {
                                deriv.func <- inla.eval(paste("function(", arg[1L], ",", arg[2L], "=", arg.val[2L], ") {}"))
                            }
                            else {
                                stopifnot(length(arg) == 3L)
                                deriv.func <- inla.eval(paste("function(", arg[1L], ",", arg[2L], "=", arg.val[2L], ",", arg[3L], "=", arg.val[3L], ") {}"))
                            }
                        }
                        temptest <- try(D(body(result$misc$from.theta[[i]]), arg[1L]), silent = TRUE)
                        if (!inherits(temptest, "try-error")) {
                            body(deriv.func) <- temptest
                            log.J <- log.J - log(abs(deriv.func(cs$config[[k]]$theta[i]))) ## Yes, it's a minus...
                        }
                        else {
                            h <- .Machine$double.eps^0.25
                            theta.1 <- do.call(result$misc$from.theta[[i]], args = list(cs$config[[k]]$theta[i] - h))
                            theta.2 <- do.call(result$misc$from.theta[[i]], args = list(cs$config[[k]]$theta[i] + h))
                            log.J <- log.J - log(abs((theta.2 - theta.1) / (2.0 * h))) ## Yes, it's a minus...
                        }
                    }
                    ## print(paste("logJ", log.J))
                } else {
                    ## old code using numerical differentiation
                    h <- .Machine$double.eps^0.25
                    for (i in 1:length(theta)) {
                        theta.1 <- do.call(result$misc$from.theta[[i]], args = list(cs$config[[k]]$theta[i] - h))
                        theta.2 <- do.call(result$misc$from.theta[[i]], args = list(cs$config[[k]]$theta[i] + h))
                        log.J <- log.J - log(abs((theta.2 - theta.1) / (2.0 * h))) ## Yes, it's a minus...
                    }
                    ## print(paste("logJ", log.J))
                }
            }

            for (i in 1:n.idx[k]) {
                if (is.null(theta)) {
                    a.sample <- list(
                        hyperpar = NULL,
                        latent = xx$sample[, i, drop = FALSE],
                        logdens = list(
                            hyperpar = NULL,
                            latent = as.numeric(xx$logdens[i]),
                            joint = as.numeric(xx$logdens[i])
                        )
                    )
                } else {
                    ld.h <- as.numeric(ld.theta - result$mlik[1, 1] + log.J)
                    a.sample <- list(
                        hyperpar = theta,
                        latent = xx$sample[, i, drop = FALSE],
                        logdens = list(
                            hyperpar = ld.h,
                            latent = as.numeric(xx$logdens[i]),
                            joint = as.numeric(ld.h + xx$logdens[i])
                        )
                    )
                }
                if (add.names || i.sample == 1L) {
                    n1 <- length(nm)
                    n2 <- length(a.sample$latent)
                    stopifnot(n2 >= n1) ## this must be true. just a check
                    if (n2 > n1) {
                        ## This is the case where lincomb.derived.only = FALSE, so these are
                        ## then added to the end. Should transfer the names of them all the way
                        ## to here, but...
                        xnm <- paste0("Lincomb:", inla.num(1:(n2 - n1)))
                        nm <- c(nm, xnm)

                        if (i.sample == 1L) {
                            ## add to the contents if needed
                            con$tag <- c(con$tag, "Lincomb")
                            con$start <- c(con$start, n1 + 1)
                            con$length <- c(con$length, n2 - n1)
                        }
                    }
                    rownames(a.sample$latent) <- nm
                } else {
                    rownames(a.sample$latent) <- NULL
                }
                all.samples[[i.sample]] <- a.sample
                i.sample <- i.sample + 1L
            }
        }
    }
    if (length(selection) == 0L) {
        attr(all.samples, ".contents") <- con
    } else {
        ## we use a selection, need to build a new 'contents' list
        con <- list(tag = names(selection), start = c(), length = c())
        for (nm in names(selection)) {
            ## from Hmisc::escapeRegex. Need it for the '(Intercept)'
            re <- paste0("^", gsub("([.|()\\^{}+$*?]|\\[|\\])", "\\\\\\1", nm), ":")
            m <- grep(re, names(sel.map))
            if (length(m) > 0) {
                con$start <- c(con$start, min(m))
                con$length <- c(con$length, diff(range(m)) + 1)
            }
        }
        attr(all.samples, ".contents") <- con
    }
    return(all.samples)
}

`inla.posterior.sample.eval` <- function(fun, samples, return.matrix = TRUE, ...) {
    ## evaluate fun(...) over each sample

    contents <- attr(samples, which = ".contents", exact = TRUE)
    if (is.null(contents)) {
        stop("Argument 'samples' must be the output from 'inla.posterior.sample()'.")
    }

    ## special shorthand feature that is very useful. if this is vector of character's, then its
    ## interpreted as a function returning these names, like ....eval(c("a", "b[1:2]"), ...)
    ## will be converted into the function: function() return(c(get('a'), get('b[1:2]')))

    if (is.character(fun)) {
        arg <- NULL
        for(nm in fun) {
            arg <- paste0(if (is.null(arg)) "" else paste0(arg, ","), "get('", nm, "')")
        }
        fun <- inla.eval(paste0("function() return(c(", arg, "))"))
    } else {
        fun <- match.fun(fun)
    }

    my.fun <- function(a.sample, .contents, .fun, ...) {
        env <- new.env()
        theta <- as.vector(a.sample$hyperpar)
        assign("theta", theta, envir = env)
        for (i in seq_along(.contents$tag)) {
            assign(.contents$tag[i],
                as.vector(a.sample$latent[.contents$start[i]:(.contents$start[i] +
                    .contents$length[i] - 1), 1]),
                envir = env
            )
        }
        ## this is special
        if (exists("(Intercept)", envir = env)) {
            assign("Intercept", get("(Intercept)", envir = env), envir = env)
        }

        parent.env(env) <- .GlobalEnv
        environment(.fun) <- env
        return(.fun(...))
    }

    ret <- inla.mclapply(samples, my.fun, .fun = fun, .contents = contents, ...)
    if (return.matrix) {
        ns <- length(ret)
        nm <- names(ret[[1]])
        ret <- matrix(unlist(ret), ncol = ns)
        colnames(ret) <- paste0("sample:", 1:ns)
        rownames(ret) <- if (!is.null(nm)) nm else paste0("fun[", 1:nrow(ret), "]")
    }

    return(ret)
}

`inla.posterior.sample.interpret.selection` <- function(selection = list(), result) {
    ## this function interpret a selection, of the form of a named list,
    ## list(NAME = idx's, ...),
    ## with the standard names 'APredictor', 'Predictor', '(Intercept)' as well. the idx can
    ## contains negative numbers for which will be interpreted as 'not'. if idx=0, then this is
    ## interpreted as the whole vector. the result is a named list of vector of logicals, that
    ## described which part of the sample to select

    cs <- result$misc$configs$contents
    nc <- length(cs$tag)
    n <- sum(cs$length)
    select <- rep(FALSE, n)
    nam <- rep(NA, n)

    ## is selection is NULL or an empty list, this means select all. just make a selection that
    ## do that
    if (is.null(selection) || length(selection) == 0) {
        selection <- as.list(rep(0, nc))
        names(selection) <- cs$tag
    }

    for (k in seq_along(cs$tag)) {
        tag <- cs$tag[k]
        start <- cs$start[k]
        end <- start + cs$length[k] - 1L
        len <- cs$length[k]

        if (inla.is.element(tag, selection)) {
            idx <- which(names(selection) == tag)
            sel <- selection[[idx]]
            selection[[idx]] <- NULL

            if (all(sel == 0)) {
                sel <- 1:len
            } else if (all(sel > 0)) {
                stopifnot(all(sel <= len))
            } else if (all(sel < 0)) {
                stopifnot(all(-sel <= len))
                sel <- (1:len)[sel]
            } else {
                stop(paste("This should not happen. Something wrong with the selection for tag=", tag))
            }

            select[start + sel - 1] <- TRUE
            nam[start + sel - 1] <- paste0(tag, ":", sel)
        }
    }

    if (length(selection) > 0) {
        warning(paste0(
            "Some selections are not used: ",
            paste(names(selection), collapse = ", ", sep = "")
        ))
    }
    names(select) <- nam

    return(select)
}

`inla.selection2lincombs` <- function(selection = list(), tag.base = ".selection.") {
    ## this function convert from selection format to lincombs format.
    lc <- list()
    n.lc <- 0
    nms <- names(selection)
    nnms <- c()
    for (i in seq_along(selection)) {
        a <- selection[[i]]
        for (j in seq_along(a)) {
            n.lc <- n.lc + 1
            llc <- list()
            llc[[1]] <- list(list(idx = a[j], weight = 1))
            names(llc[[1]]) <- nms[i]
            lc[[n.lc]] <- llc
            nnms <- c(nnms, paste0(tag.base, nms[i], ":", a[j]))
        }
    }
    names(lc) <- nnms
    return(lc)
}

## Export: inla.iidkd.sample

## !\name{inla.iidkd.sample}
## !\alias{inla.iidkd.sample}
## !\alias{iidkd.sample}
## !
## !\title{Provide samples from the \code{iidkd} component (experimental)}
## !\description{
## ! This function provide samples of the \code{iidkd} component 
## ! using more interpretable parameters
## !}
## !\usage{
## !inla.iidkd.sample(n = 10^4, result, name, return.cov = FALSE)
## !}
## !
## !\arguments{
## !\item{n}{Integer Number of samples to use}
## !\item{result}{inla-object An object of class \code{inla}, ie a result of a call to \code{inla()}}
## !\item{name}{Character The name of the \code{iidkd} component}
## !\item{return.cov}{Logical Return samples of the covariance matrix instead of
## !      stdev/correlation matrix described below?}
## !}
## !\value{A list of sampled matrices, with (default) correlations on the off-diagonal and
## !standard-deviations on the diagonal}
## !\seealso{\code{inla.doc("iidkd")}}
## !\author{Havard Rue \email{hrue@r-inla.org}}

`inla.iidkd.sample` <- function(n = 10^4, result, name, return.cov = FALSE) 
{
    stopifnot(!missing(result))
    stopifnot(class(result) == "inla")
    stopifnot(!missing(name))
    stopifnot(n > 0)

    k.max <- 10
    theta.max <- (k.max * (k.max + 1L)) / 2L

    names <- paste0("Theta", 1:theta.max, " for ", name)
    idx <- which(rownames(result$summary.hyperpar) %in% names)
    if (length(idx) == 0) {
        return (list())
    }
    len <- length(idx)
    k <- 0
    for(kk in 2:k.max) {
        if (len == kk*(kk+1)/2) {
            k <- kk
            break
        }
    }
    if (k == 0) {
        stop(paste0("Cannot find correct dimension for: ", name))
    }

    theta2matrix <- function(i, thetas, dim, dim2, ret.cov) {
        theta <- thetas[i, ]
        L <- matrix(0, dim, dim)
        diag(L) <- exp(theta[1:dim])
        L[lower.tri(L)] <- theta[dim + 1:dim2]
        S <- solve(L %*% t(L))
        S <- (S + t(S))/2.0 ## force matrix to be symmetric
        if (ret.cov) {
            return (S)
        } else {
            sd <- sqrt(diag(S))
            iSigma <- diag(1/sd)
            Cor <- iSigma %*% S %*% iSigma
            diag(Cor) <- sd
            return(Cor)
        }
    }
    xx <- inla.hyperpar.sample(n, result, intern = TRUE)[, idx, drop = FALSE]
    res <- lapply(1:n, theta2matrix, dim = k, dim2 = k*(k-1)/2, thetas = xx,
                  ret.cov = return.cov)

    return (res)
}

## Export: inla.pc.ar.rpacf inla.pc.ar.dpacf

## ! \name{pc.ar}
## ! \alias{inla.pc.ar}
## ! \alias{inla.pc.ar.rpacf}
## ! \alias{ar.rpacf}
## ! \alias{inla.pc.ar.dpacf}
## ! \alias{ar.dpacf}
## !
## ! \title{Utility functions for the PC prior for a an AR(p) model}
## !
## ! \description{Functions to evaluate and sample from the
## !              PC prior for an AR(p) model}
## ! \usage{
## !    inla.pc.ar.rpacf(n=1, p, lambda = 1)
## !    inla.pc.ar.dpacf(pac, lambda = 1, log = TRUE)
## ! }
## ! \arguments{
## !   \item{p}{The order of the AR-model}
## !   \item{pac}{A vector of partial autocorrelation coefficients}
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter in the prior}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## ! }
## ! \details{
## ! }
## ! \value{%%
## !     \code{inla.pc.ar.rpac} generate samples from the prior, returning a matrix
## !     where each row is a sample of \code{theta}.
## !     \code{inla.pc.ar.dpac} evaluates the density of \code{pac}.
## !     Use \code{inla.ar.pacf2phi}, \code{inla.ar.phi2pacf},
## !     \code{inla.ar.pacf2acf} and \code{inla.ar.acf2pacf} to convert between various
## !    parameterisations.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! }

inla.pc.ar.rpacf <- function(n = 1, p, lambda = 1) {
    stopifnot(!missing(p) && p >= 1)
    stopifnot(lambda > 0)
    stopifnot(n >= 1)

    rpac <- function(n, p, lambda) {
        gamma <- inla.pc.multvar.simplex.r(n = n, b = rep(1 / 2, p), lambda = lambda)
        pac <- sqrt(1 - exp(-gamma)) * sample(c(-1, 1), size = p, replace = TRUE)
        return(pac)
    }

    x <- unlist(lapply(rep(1, n), rpac, p = p, lambda = lambda))
    x <- matrix(x, n, p, byrow = TRUE)
    return(x)
}

inla.pc.ar.dpacf <- function(pac, lambda = 1, log = TRUE) {
    p <- length(pac)
    stopifnot(p >= 1)
    gamma <- -log(1 - pac^2)
    ld <- (inla.pc.multvar.simplex.d(x = gamma, lambda = lambda, b = rep(1 / 2, p), log = TRUE)
    +
        sum(log(abs(pac / (1 - pac^2)))))
    return(if (log) ld else exp(ld))
}

## Export: inla.hyperpar

## !\name{inla.hyperpar}
## !\alias{inla.hyperpar}
## !\alias{hyperpar.inla}
## !
## !\title{Improved estimates for the hyperparameters}
## !\description{
## ! Improve the estimates of the  posterior marginals
## ! for the hyperparameters of
## ! the model using the grid integration strategy.
## !}
## !\usage{
## ! inla.hyperpar(
## !         result,
## !         skip.configurations = TRUE,
## !         verbose = FALSE,
## !         dz = 0.75,
## !         diff.logdens = 15,
## !         h = NULL,
## !         restart = FALSE,
## !         quantiles = NULL,
## !         keep = FALSE)
## !}
## !
## !\arguments{

`inla.hyperpar` <-
    function(
             ## !\item{result}{An object of class
             ## !\code{inla},  ie a result of a call to
             ## !\code{inla()}}
             result,

             ## !\item{skip.configurations}{ A boolean variable; skip
             ## !configurations if the values at the main axis are to
             ## !small. (Default TRUE)}
             skip.configurations = TRUE,

             ## !\item{verbose}{Boolean indicating wheather the inla
             ## !program should run in a verbose mode.}
             verbose = FALSE,

             ## !\item{dz}{Step length in the standardized scale used
             ## !in the construction of the grid, default 0.75.}
             dz = 0.75,

             ## !\item{diff.logdens}{The difference of the
             ## !log.density for the hyperpameters to stop numerical
             ## !integration using int.strategy='grid'.  Default 15}
             diff.logdens = 15,

             ## !\item{h}{The step-length for the gradient
             ## !calculations for the hyperparameters. Default 0.01.}
             h = NULL,

             ## !\item{restart}{A boolean defining wheather the
             ## !optimizer should start again to ind the mode or if it
             ## !should use the mode contained in the \code{object}}
             restart = FALSE,

             ## !\item{quantiles}{A vector of quantiles,
             ## !to compute for each posterior marginal.}
             quantiles = NULL,

             ## !\item{keep}{A boolean variable indicating the
             ## !working files (ini file, data files and results files)
             ## !should be kept}
             keep = FALSE) 
{
    ## !}
    ## !\value{
    ## !The object returned is the same as  \code{object} but the
    ## !estimates of the hyperparameters are replaced by improved estimates.}
    ## !\references{
    ## !See the references in \code{inla}}
    ## !\author{Havard Rue \email{hrue@r-inla.org}}
    ## !\note{This function might take a long time or if the number of
    ## !hyperparameters in the model is large. If it complains and says \code{I cannot get enough memory},
    ## !try to increase the value of the argument \code{dz} or decrease \code{diff.logdens}.}
    ## !\seealso{\code{\link{inla}}}

    result.tmp <- result
    stopifnot(class(result.tmp) == "inla")

    ## only arguments that we need to change,  is changed...
    result.tmp$.args$quantiles <- inla.ifelse(missing(quantiles) || is.null(quantiles), result.tmp$.args$quantiles, quantiles)
    result.tmp$.args$verbose <- verbose
    result.tmp$.args$only.hyperparam <- TRUE
    result.tmp$.args$keep <- keep

    result.tmp$.args$control.compute$hyperpar <- TRUE
    result.tmp$.args$control.compute$return.marginals <- TRUE
    result.tmp$.args$control.compute$dic <- FALSE
    result.tmp$.args$control.compute$mlik <- TRUE
    result.tmp$.args$control.compute$cpo <- FALSE
    result.tmp$.args$control.compute$po <- FALSE
    result.tmp$.args$control.compute$q <- FALSE
    result.tmp$.args$control.compute$graph <- FALSE

    result.tmp$.args$control.predictor$compute <- FALSE

    result.tmp$.args$control.inla$int.strategy <- "grid"
    result.tmp$.args$control.inla$strategy <- "gaussian"
    result.tmp$.args$control.inla$h <- inla.ifelse(is.null(h), result.tmp$.args$control.inla$h, h)
    result.tmp$.args$control.inla$dz <- dz
    result.tmp$.args$control.inla$diff.logdens <- diff.logdens
    result.tmp$.args$control.inla$print.joint.hyper <- TRUE
    result.tmp$.args$control.inla$force.diagonal <- TRUE
    result.tmp$.args$control.inla$adjust.weights <- FALSE

    result.tmp$.args$control.mode$result <- result.tmp
    result.tmp$.args$control.mode$restart <- restart

    ## cannot use this function with inla.call="submit", if so, then
    ## replace this into inla.call="remote"
    result.tmp$.args$inla.call <- sub("inla.submit", "inla.remote", result.tmp$.args$inla.call)

    ## call itself
    result.tmp <- inla.rerun(result.tmp, plain = TRUE)

    ## these are the entries that we want to replace
    replace.names <- c(
        "summary.hyperpar", "marginals.hyperpar", "internal.marginals.hyperpar",
        "internal.summary.hyperpar", "joint.hyper", "mlik", "version", "cpu.used"
    )

    for (nm in replace.names) {
        idx.result <- which(names(result) == nm)
        idx.result.tmp <- which(names(result.tmp) == nm)
        if (length(idx.result) > 0 && length(idx.result.tmp) > 0) {
            result[[idx.result]] <- result.tmp[[idx.result.tmp]]
        }
    }
    ## from $misc, we only want this one.
    result$misc$configs <- result.tmp$misc$configs

    return(result)
}

context("test 'copy'")

test_that("Case 1", {
    set.seed(123)
    n = 300
    rho = 0.9
    sd = 0.01
    y = arima.sim(n=n, model = list(ar = rho))*sqrt(1-rho^2) + rnorm(n, sd = sd)
    y[1:10]=NA  ## ``burn in''
    i = 1:n
    j = i
    ii = c(NA, 1:(n-1))
    ww = rep(-1,n)
    na = rep(NA,n)
    
    i2 = c(i,i)
    ii2 = c(na, ii)
    j2 = c(na, j)
    ww2 = c(na, ww)
    
    Y = matrix(NA, 2*n, 2)
    Y[1:n, 1] = y
    Y[1:n + n, 2] = 0
    
    formula = Y ~ -1 + f(i2, initial=-10, fixed=T) +
        f(ii2, ww2, copy = "i2", range = c(-1,1),
          fixed = FALSE, initial = 3, precision = 1e10,
          param = c(0,0.2), prior = "normal") +
              f(j2, initial=0, fixed=FALSE)
    
    r = inla(formula, data = list(Y=Y, i2=i2,ii2=ii2,ww2=ww2,j2=j2),
            family = c("gaussian","gaussian"),
            control.family = list(
                    list(initial = log(1/sd^2), fixed = TRUE),
                    list(initial = 13, fixed = TRUE)),
            control.predictor = list(compute=TRUE))
    
    fformula = y ~ -1 + f(i, model = "ar1")
    rr = inla(fformula, data = data.frame(i,y),
            family = "gaussian",
            control.predictor = list(compute=TRUE),
            control.family = list(initial = log(1/sd^2), fixed = TRUE))
    h = inla.hyperpar(r,  diff.logdens = 10)
    hh = inla.hyperpar(rr,  diff.logdens=10)
    
    expect_true(abs(h$summary.hyperpar[2,"mean"] -
                    hh$summary.hyperpar[2,"mean"]) < 0.02)
})


test_that("Case 2", {
    set.seed(123)
    n = 500
    N = 4*n
    time = 1:n
    x = sin(time/n*4*pi)
    age = rep(1:4, each=n)
    
    y = numeric(N)
    y[ which(age==1) ] = x
    y[ which(age==2) ] = 2*x
    y[ which(age==3) ] = 3*x
    y[ which(age==4) ] = 4*x
    
    s = 0.01
    y = y + rnorm(N, sd=s)
    
    formula = y ~ -1 + f(i, model="rw2") +
        f(j, copy="i",  fixed=FALSE) + 
            f(k, copy="i",  fixed=FALSE) + 
                f(l, copy="i",  fixed=FALSE)
    
    i = rep(NA, N)
    i[which(age == 1)] = time
    j = rep(NA, N)
    j[which(age == 2)] = time
    k = rep(NA, N)
    k[which(age == 3)] = time
    l = rep(NA, N)
    l[which(age == 4)] = time
    r = inla(formula,  data = data.frame(y, i, j, k, l),  family = "gaussian",
            control.family = list(hyper =
                    list(prec = list(initial=log(1/s^2), fixed=TRUE))))
    expect_true(abs(r$summary.hyperpar[2, "mean"] - 2) < 0.05)
    expect_true(abs(r$summary.hyperpar[3, "mean"] - 3) < 0.05)
    expect_true(abs(r$summary.hyperpar[4, "mean"] - 4) < 0.05)
}) 

## Export: inla.prune

## !\name{inla.prune}
## !\alias{inla.prune}
## !\alias{prune}
## !
## !\title{Prune the INLA-package}
## !\description{Prune the INLA-package by deleting binary files not supported by the running OS}
## !\usage{
## !inla.prune(ask = TRUE)
## !}
## !\arguments{
## !\item{ask}{Logical. If TRUE, then ask for user confirmation before deleting.
## !           If FALSE, then delete without user confirmation.}
## !}
## !\value{No value is returned.}
## !\author{Havard Rue \email{hrue@r-inla.org}}

`inla.dir.size` <- function(d) {
    ## return the disk usage of a directory, recursively, in Mb
    return(sum(file.info(list.files(d,
        all.files = TRUE,
        full.names = TRUE,
        recursive = TRUE
    ))$size, na.rm = TRUE) / 1024^2)
}

`inla.prune` <- function(ask = TRUE) {
    pkg <- installed.packages()
    if ("INLA" %in% pkg) {
        path <- pkg["INLA", "LibPath"]
    } else {
        stop("Cannot find package 'INLA' using 'installed.packages()'")
    }
    bin.path <- paste0(path, "/INLA/bin")

    arm <- paste0(bin.path, "/mac.arm64")
    dd <- arm
    for (os in c("linux", "mac", "windows")) {
        for (arch in c("32bit", "64bit", "experimental")) {
            dd <- c(dd, paste0(bin.path, "/", os, "/", arch))
        }
    }
    if (inla.os("mac.arm64")) {
        native <- arm
    } else {
        native <- paste0(bin.path, "/", inla.os.type(), "/", inla.os.32or64bit(), "bit")
    }
    dd <- setdiff(dd, native)

    size <- 0
    found <- FALSE
    for (d in dd) {
        info <- file.info(d)
        if (!is.na(info$isdir) && info$isdir) {
            found <- TRUE
            siz <- inla.dir.size(d)
            if (siz < 0.1) siz <- 0
            cat("---> Found directory (size ", format(siz, dig = 1), "Mb): ", d, "\n", sep = "")
            ans <- if (ask) askYesNo("Remove directory?", default = FALSE) else TRUE
            if (is.na(ans)) {
                cat("---> Cancel\n")
                return(invisible())
            }
            if (ans) {
                cat("---> Remove...", "\n")
                size <- size + siz
                unlink(d, recursive = TRUE, force = TRUE)
            }
        }
    }

    if (!found) {
        cat("---> Nothing to be pruned.\n")
    }
    if (size > 0) {
        cat("---> Removed ", format(size, dig = 1), "Mb in total.\n", sep = "")
    }

    return(invisible())
}

`inla.prune.check` <- function() {
    ## return the ammount that could be removed in Mb
    pkg <- installed.packages()
    if ("INLA" %in% pkg) {
        path <- pkg["INLA", "LibPath"]
    } else {
        return(0)
    }
    bin.path <- paste0(path, "/INLA/bin")
    arm <- paste0(bin.path, "/mac.arm64")
    dd <- arm
    for (os in c("linux", "mac", "windows")) {
        for (arch in c("32bit", "64bit", "experimental")) {
            dd <- c(dd, paste0(bin.path, "/", os, "/", arch))
        }
    }
    if (inla.os("mac.arm64")) {
        native <- arm
    } else {
        native <- paste0(bin.path, "/", inla.os.type(), "/", inla.os.32or64bit(), "bit")
    }
    dd <- setdiff(dd, native)

    size <- 0
    for (d in dd) {
        info <- file.info(d)
        if (!is.na(info$isdir) && info$isdir) {
            size <- size + inla.dir.size(d)
        }
    }

    return(round(dig = 1, size))
}

## Export: inla.pc.multvar.h.default
## Export: inla.pc.multvar.simplex.d
## Export: inla.pc.multvar.simplex.r
## Export: inla.pc.multvar.sphere.d
## Export: inla.pc.multvar.sphere.r

## ! \name{pc.multvar}
## ! \alias{pc.multvar}
## ! \alias{inla.pc.multvar}
## ! \alias{inla.pc.multvar.simplex}
## ! \alias{inla.pc.multvar.simplex.d}
## ! \alias{inla.pc.multvar.simplex.r}
## ! \alias{inla.pc.multvar.sphere}
## ! \alias{inla.pc.multvar.sphere.d}
## ! \alias{inla.pc.multvar.sphere.r}
## ! \alias{inla.pc.multvar.h.default}
## ! \alias{pc.multvar}
## ! \alias{pc.multvar.simplex}
## ! \alias{pc.multvar.simplex.d}
## ! \alias{pc.multvar.simplex.r}
## ! \alias{pc.multvar.sphere}
## ! \alias{pc.multvar.sphere.d}
## ! \alias{pc.multvar.sphere.r}
## ! \alias{pc.multvar.h.default}
## !
## ! \title{Multivariate PC priors}
## !
## ! \description{Functions to evaluate and simulate from
## !              multivariate PC priors: The simplex and sphere case}
## ! \usage{
## ! inla.pc.multvar.h.default(x, inverse = FALSE, derivative = FALSE)
## ! inla.pc.multvar.simplex.r(n = NULL, lambda = 1, h = inla.pc.multvar.h.default, b = NULL)
## ! inla.pc.multvar.simplex.d(x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, b = NULL)
## ! inla.pc.multvar.sphere.r(n = NULL, lambda = 1, h = inla.pc.multvar.h.default, H = NULL)
## ! inla.pc.multvar.sphere.d(x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, H = NULL)
## ! }
## ! \arguments{
## !   \item{x}{Samples to evaluate. If input is a matrix then each row is a sample. If input is
## !            a vector then this is the sample.}
## !   \item{inverse}{Compute the inverse of the h()-function.}
## !   \item{derivative}{Compute the derivative of the h()-function. (derivative of the inverse
## !                     function is not used).}
## !   \item{n}{Number of samples to generate.}
## !   \item{lambda}{The lambda-parameter in the PC-prior.}
## !   \item{log}{Evaluate the density in log-scale or ordinary scale.}
## !   \item{h}{The h()-function,  defaults to \code{inla.pc.multvar.h.default}. See that code
## !            for an example of how to write a user-spesific function.}
## !   \item{b}{The b-vector (gradient) in the expression for the simplex option,  \code{d(xi) = h(b^T xi)}}
## !   \item{H}{The H(essian)-matrix in the expression for the sphere option,  \code{d(xi) =
## !            h(1/2 *xi^T H xi)}. If \code{H} is a vector, then it is interpreted as the
## !            diagonal of a (sparse) diagonal matrix.}
## !}
## ! \details{
## !   These functions implements multivariate PC-priors of the simplex and sphere type.
## ! }
## ! \value{%%
## !     \code{inla.pc.multvar.simplex.r} generate samples from the simplex case, and
## !     \code{inla.pc.multvar.simplex.d} evaluate the density.
## !     \code{inla.pc.multvar.sphere.r} generate samples from the sphere case,  and
## !     \code{inla.pc.multvar.sphere.d} evaluate the density.
## !     \code{inla.pc.multvar.h.default} implements the default h()-function and illustrate how
## !     to code your own spesific one, if needed.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! }

inla.pc.multvar.h.default <- function(x, inverse = FALSE, derivative = FALSE) {
    ## default: h(x) = sqrt(2*x)
    if (derivative) {
        if (inverse) {
            stopifnot("Options inverse=TRUE and derivative=TRUE is not allowed.")
        } else {
            return(1 / sqrt(2 * x))
        }
    } else {
        if (inverse) {
            return(1 / 2 * x^2)
        } else {
            return(sqrt(2 * x))
        }
    }
}

inla.pc.multvar.simplex.r.core <- function(n, p, lambda = 1, h = inla.pc.multvar.h.default) {
    stopifnot(p > 0)
    stopifnot(lambda > 0)

    x <- matrix(NA, n, p)
    for (i in 1:n) {
        ## simulate z on the unit simplex
        z <- rexp(p)
        z <- z / sum(z)
        ## sample the size of the simplex
        d <- rexp(1, rate = lambda)
        r <- h(d, inverse = TRUE)
        ## and scale it
        x[i, ] <- z * r
    }
    colnames(x) <- paste0("x:", 1:p)
    rownames(x) <- paste0("sample:", 1:n)

    return(x)
}

inla.pc.multvar.simplex.d.core <- function(x, lambda = 1, log = FALSE,
                                           h = inla.pc.multvar.h.default) {
    ## evaluate the density from the pc prior where d = h(\sum x_i), x_i >=0.

    simplex.log.volume <- function(n) {
        ## volumne of the n-simplex with n+1 vertices, see
        ## https://en.wikipedia.org/wiki/Simplex#Volume
        return(-lfactorial(n))
    }

    ## each row is a sample
    if (!is.matrix(x)) {
        x <- matrix(c(x), ncol = length(x))
    }
    p <- ncol(x)
    stopifnot(p > 0)
    stopifnot(lambda > 0)

    ## compute the density
    ldens <- apply(
        x, 1,
        function(z) {
            r <- sum(z)
            d <- h(r)
            ld <- (log(lambda) - lambda * d + log(abs(h(r, derivative = TRUE)))
                - (p - 1) * log(r) - simplex.log.volume(p - 1))
            return(ld)
        }
    )

    return(if (log) ldens else exp(ldens))
}

inla.pc.multvar.simplex.r <- function(
                                      n = NULL, lambda = 1, h = inla.pc.multvar.h.default, b = NULL) {
    return(inla.pc.multvar.simplex.core(
        n = n, lambda = lambda, h = h, b = b, mode = "r"
    ))
}

inla.pc.multvar.simplex.d <- function(
                                      x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, b = NULL) {
    return(inla.pc.multvar.simplex.core(
        x = x, lambda = lambda, log = log, h = h, b = b, mode = "d"
    ))
}

inla.pc.multvar.simplex.core <- function(
                                         x = NULL, n = NULL, lambda = 1, log = FALSE,
                                         h = inla.pc.multvar.h.default, b = NULL, mode = c("r", "d")) {
    ## this is the case where d = h( b^T x)
    ## for vector b>0

    mode <- match.arg(mode)
    ## need either b or H or both
    stopifnot(!is.null(b))

    ## ensure that b is a matrix
    if (!is.null(b) && !is.matrix(b)) {
        b <- matrix(c(b), ncol = 1)
    }
    p <- nrow(b)

    ## ensure x, if given, is a matrix. this is relevant if x is just one sample which can be
    ## given as a vector
    if (!is.null(x) && !is.matrix(x)) {
        x <- matrix(c(x), ncol = length(x))
    }

    if (mode == "r") {
        X <- inla.pc.multvar.simplex.r.core(n, p = p, lambda = lambda, h = h)
        for (i in 1:n) {
            X[i, ] <- t(1 / b) * X[i, , drop = FALSE]
        }
        return(X)
    } else if (mode == "d") {
        n <- nrow(x)
        ldens.x <- numeric(n)
        for (i in 1:n) {
            theta <- c(b) * c(x[i, ])
            ldens <- inla.pc.multvar.simplex.d.core(theta, lambda = lambda, log = TRUE, h = h)
            ldens.x[i] <- ldens + sum(log(abs(b)))
        }
        return(if (log) ldens.x else exp(ldens.x))
    }

    stop("Should not happen")
    return()
}

inla.pc.multvar.sphere.r.core <- function(n, p, lambda = 1, h = inla.pc.multvar.h.default) {
    ## simulate from the pc prior where d = h(1/2 * \sum x_i^2)
    stopifnot(n > 0)
    stopifnot(p > 0)
    stopifnot(lambda > 0)

    x <- matrix(NA, n, p)
    for (i in 1:n) {
        ## simulate z on the unit sphere
        z <- rnorm(p)
        z <- z / sqrt(sum(z^2))
        ## sample the radii of the sphere
        d <- rexp(1, rate = lambda)
        r <- sqrt(2 * h(d, inverse = TRUE))
        ## and scale it
        x[i, ] <- z * r
    }
    colnames(x) <- paste0("x:", 1:p)
    rownames(x) <- paste0("sample:", 1:n)

    return(x)
}

inla.pc.multvar.sphere.d.core <- function(x, lambda = 1, log = FALSE,
                                          h = inla.pc.multvar.h.default) {
    ## evaluate the density from the pc prior where d = h(1/2 * \sum x_i^2), x_i >=0.

    sphere.log.surface <- function(n) {
        ## surface of the sphere in n-dimensions with radii=1
        ## n=0 line, n=1 circle, n=2 sphere etc
        return(log(n + 1) + (n + 1) / 2 * log(pi) - lgamma((n + 1) / 2 + 1))
    }

    ## each row is a sample
    if (!is.matrix(x)) {
        x <- matrix(c(x), ncol = length(x))
    }
    p <- ncol(x)
    stopifnot(p > 0)
    stopifnot(lambda > 0)

    ldens <- apply(
        x, 1,
        function(z) {
            r <- sqrt(sum(z^2))
            d <- h(0.5 * r^2)
            ldens <- (log(lambda) - lambda * d + log(abs(h(0.5 * r^2, derivative = TRUE)))
                - (p - 2) * log(r) - sphere.log.surface(p - 1))
            return(ldens)
        }
    )

    return(if (log) ldens else exp(ldens))
}

inla.pc.multvar.sphere.r <- function(
                                     n = NULL, lambda = 1, h = inla.pc.multvar.h.default, H = NULL) {
    return(inla.pc.multvar.sphere.core(
        n = n, lambda = lambda, h = h, H = H, mode = "r"
    ))
}

inla.pc.multvar.sphere.d <- function(
                                     x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, H = NULL) {
    return(inla.pc.multvar.sphere.core(
        x = x, lambda = lambda, log = log, h = h, H = H, mode = "d"
    ))
}


inla.pc.multvar.sphere.core <- function(
                                        x = NULL, n = NULL, lambda = 1, log = FALSE,
                                        h = inla.pc.multvar.h.default, H = NULL, mode = c("r", "d")) {
    ## this is the case where d = h(1/2 x^{T} H x)

    mode <- match.arg(mode)
    stopifnot(!is.null(H))

    ## ensure x, if given, is a matrix. this is relevant if x is just one sample which can be
    ## given as a vector
    if (!is.null(x) && !is.matrix(x)) {
        x <- matrix(c(x), ncol = length(x))
    }
    ## if the class is ddiMatrix, convert it
    if (is(H, class(Diagonal(1)))) {
        H <- diag(H)
    }

    if (is.vector(H)) {
        ## in case H is a diagonal matrix
        p <- length(H)
        d <- sort(H, decreasing = TRUE)
        ## create the reverse-diagonal, as this is how R compute the eigenvectors. In this case
        ## we will get exactly the same results as with H as a dense diagonal matrix. Copy this
        ## trick from lava::revdiag
        V <- Diagonal(p, x = rep(0, p))
        V[cbind(rev(seq(p)), seq(p))] <- rep(1, p)
        ##
        Lam.sqrt <- Diagonal(p, x = sqrt(d))
        rep.Lam.sqrt <- Diagonal(p, x = 1.0 / sqrt(d))
        eig <- list(values = d)
    } else {
        p <- nrow(H)
        ## this case involving rescaling and rotation
        eig <- eigen(H)
        V <- eig$vectors
        Lam.sqrt <- diag(sqrt(eig$values), ncol = p)
        rep.Lam.sqrt <- diag(1 / sqrt(eig$values), ncol = p)
    }

    ## x = Lam.sqrt %*% V^T z
    ## and z is such that  x^T H x = z^T z
    x.to.z <- Lam.sqrt %*% t(V)
    z.to.x <- V %*% rep.Lam.sqrt

    if (mode == "r") {
        X <- inla.pc.multvar.sphere.r.core(n, p = p, lambda = lambda, h = h)
        for (i in 1:n) {
            z <- t(X[i, , drop = FALSE])
            X[i, ] <- as.numeric(z.to.x %*% z)
        }
        return(X)
    } else if (mode == "d") {
        ljac <- 0.5 * sum(log(eig$values))
        n <- nrow(x)
        ldens.x <- numeric(n)
        for (i in 1:n) {
            z <- x.to.z %*% t(x[i, , drop = FALSE])
            ldens <- inla.pc.multvar.sphere.d.core(t(z), lambda = lambda, log = TRUE, h = h)
            ldens.x[i] <- ldens + ljac
        }
        return(if (log) ldens.x else exp(ldens.x))
    }
    stop("Should not happen")
    return()
}

## Nothing to export

## The 'fmesher'-file format is for internal use only
## but it is used in some low-level examples.

`inla.is.fmesher.file` <- function(filename) {
    ## return TRUE if file exists and is a fmesher file
    if (!is.character(filename) || !file.exists(filename)) {
          return(FALSE)
      }

    fp <- file(filename, "rb")
    len.h <- readBin(fp, what = integer(), n = 1)
    close(fp)

    ## the only test we can make now,  is if the length of the header is 8.
    if (len.h == 8L) {
        return(TRUE)
    } else {
        return(FALSE)
    }
}

`inla.read.fmesher.file` <- function(filename, verbose = FALSE, debug = FALSE) {
    ##
    ## read a binary-file from fmesher in format specified by FL.
    ##

    if (debug) {
        verbose <- TRUE
    }

    ## internal simple checking routine, which also do debug output.
    read.check <- function(x, h) {
        msg <- deparse(match.call())
        if (length(x) != h$elems) {
              stop(paste("Reading file", filename, ". Fail to read", h$elems, ", got", length(x), "."))
          }
        if (h$debug) {
              print(inla.paste(c(msg, x)))
          }

        return(invisible())
    }

    stopifnot(file.exists(filename))
    fp <- file(filename, "rb")

    if (verbose) {
          print(paste("\nOpen file", filename))
      }

    len.h <- readBin(fp, what = integer(), n = 1)
    ## currently required
    stopifnot(len.h >= 8)
    if (verbose) {
        print(paste("header is", len.h, "integers."))
    }

    h.raw <- readBin(fp, what = integer(), n = len.h)
    ## negative entries specify something different not yet defined.
    for (i in 1:len.h) {
        if (h.raw[i] < 0) {
            stop(paste("Entry ", i, " in the header of file ", filename, " is negative: ", h.raw[i],
                ". Do not know what to do.",
                sep = ""
            ))
        }
    }

    h <- list(
        filename = filename, verbose = verbose, debug = debug,
        version = h.raw[1],
        elems = h.raw[2],
        nrow = h.raw[3],
        ncol = h.raw[4],
        datatype = inla.ifelse(h.raw[5] == 0, "dense", "sparse"),
        valuetype = inla.ifelse(h.raw[6] == 0, integer(), double()),
        matrixtype = inla.ifelse(
            h.raw[7] == 0, "general",
            inla.ifelse(h.raw[7] == 1, "symmetric", "diagonal")
        ),
        storagetype = inla.ifelse(h.raw[8] == 0, "rowmajor", "columnmajor")
    )

    if (verbose) {
        print(h)
    }

    if (h$datatype == "dense") {
        ##
        ## dense matrix
        ##
        if (h$matrixtype != "general") {
              stop(paste("Read", filename, ". Type (`dense' && !`general') is not yet implemented."))
          }

        stopifnot(h$elems == h$nrow * h$ncol)
        Aelm <- readBin(fp, what = h$valuetype, n = h$elems)
        read.check(Aelm, h)
        A <- matrix(Aelm, nrow = h$nrow, ncol = h$ncol, byrow = (h$storagetype == "rowmajor"))
    } else if (h$datatype == "sparse") {
        ##
        ## sparse matrix
        ##
        if (h$storagetype == "rowmajor") {
            ##
            ## rowmajor format
            ##
            i <- c()
            j <- c()
            values <- c()
            if (h$matrixtype == "symmetric") {
                ##
                ## symmetric
                ##
                for (k in 1:h$elems) {
                    ij <- readBin(fp, what = integer(), n = 2)
                    i <- c(i, max(ij))
                    j <- c(j, min(ij))
                    values <- c(values, readBin(fp, what = h$valuetype, n = 1))
                }
                read.check(i, h)
                read.check(j, h)
                read.check(values, h)

                ## oops. Matrix adds replicated elements!!!
                if (!(all(i >= j) || all(i <= j))) {
                      stop(paste(
                          "Reading file", filename,
                          ". Both upper and lower part of symmetric sparse matrix",
                          "is specified. Do not know what to do."
                      ))
                  }
                idx <- (i != j)
                ii <- i[idx]
                jj <- j[idx]
                i <- c(i, jj)
                j <- c(j, ii)
                values <- c(values, values[idx])
            } else if (h$matrixtype == "general" || h$matrixtype == "diagonal") {
                ##
                ## general/diagonal
                ##
                for (k in 1:h$elems) {
                    ij <- readBin(fp, what = integer(), n = 2)
                    i <- c(i, ij[1])
                    j <- c(j, ij[2])
                    values <- c(values, readBin(fp, what = h$valuetype, n = 1))
                }
                read.check(i, h)
                read.check(j, h)
                read.check(values, h)

                if (h$matrixtype == "diagonal") {
                    idx <- (i == j)
                    i <- i[idx]
                    j <- j[idx]
                    values <- values[idx]
                }
            } else {
                stop("This should not happen.")
            }
        } else {
            ##
            ## columnmajor format
            ##

            ##
            ## other format: (i, j, values)
            ##
            i <- readBin(fp, what = integer(0), n = h$elems)
            j <- readBin(fp, what = integer(0), n = h$elems)
            values <- readBin(fp, what = h$valuetype, n = h$elems)
            read.check(i, h)
            read.check(j, h)
            read.check(values, h)

            if (h$matrixtype == "symmetric") {
                ##
                ## symmetric: lower or upper triangular part is given
                ##

                ## oops. Matrix adds replicated elements!!!
                if (!(all(i >= j) || all(i <= j))) {
                      stop(paste(
                          "Reading file", filename,
                          ". Both upper and lower part of symmetric sparse matrix",
                          "is specified. Do not know what to do..."
                      ))
                  }

                idx <- (i != j)
                ii <- i[idx]
                jj <- j[idx]
                ## yes, this is correct...
                i <- c(i, jj)
                j <- c(j, ii)
                values <- c(values, values[idx])
            } else if (h$matrixtype == "diagonal") {
                idx <- (i == j)
                i <- i[idx]
                j <- j[idx]
                values <- values[idx]
            }
        }
        A <- sparseMatrix(
            i = i, j = j, x = values, dims = c(h$nrow, h$ncol),
            index1 = FALSE
        )
        A <- inla.as.dgTMatrix(A)
    } else {
        stop("This should not happen.")
    }

    close(fp)

    return(A)
}

`inla.write.fmesher.file` <- function(A, filename = tempfile(), verbose = FALSE, debug = FALSE, auto.convert = FALSE) {
    ##
    ## write a binary-file from fmesher in format specified by FL.
    ##

    if (debug) {
        verbose <- TRUE
    }

    if (verbose) {
          print(paste("\nOpen file to write", filename))
      }

    version <- 0

    ## decide how to treat with almost integers...
    if (auto.convert) {
        if (is.matrix(A)) {
            A <- inla.affirm.integer(A)
        } else if (is.list(A)) {
            A$values <- inla.affirm.integer(A$values)
        } else if (is(A, "Matrix")) {
            ## Cannot store Matrix matrices as integers; do nothing.
        } else if (is.vector(A)) {
            A <- inla.affirm.integer(A)
        } else {
            stop(inla.paste(c(
                "Unknown type of matrix:",
                deparse(match.call()),
                "\n class(A) =",
                class(A)
            )))
        }
    }

    if (is.matrix(A)) {
        ##
        nrow <- dim(A)[1]
        ncol <- dim(A)[2]
        elems <- nrow * ncol
        datatype <- 0
        valuetype <- inla.ifelse(is.integer(A), integer(), double())
        matrixtype <- 0 ## general
        storagetype <- 1 ## columnmajor
    } else if (is.list(A)) {
        if (!is.null(A$dims)) {
            nrow <- A$dims[1]
            ncol <- A$dims[1]
        } else {
            nrow <- max(A$i)
            ncol <- max(A$j)
        }
        datatype <- 1 ## sparse
        valuetype <- inla.ifelse(is.integer(A$values), integer(), double())
        matrixtype <- 0 ## general
        storagetype <- 1 ## columnmajor

        i <- A$i - 1L
        j <- A$j - 1L
        values <- A$values
        elems <- length(values)
    } else if (is(A, "Matrix")) {
        ##
        A <- inla.as.dgTMatrix(A)
        nrow <- dim(A)[1]
        ncol <- dim(A)[2]
        datatype <- 1 ## sparse
        valuetype <- inla.ifelse(is.integer(A), integer(), double())
        matrixtype <- 0 ## general
        storagetype <- 1 ## columnmajor
        i <- A@i
        j <- A@j
        values <- A@x
        elems <- length(i)
    } else if (is.vector(A)) {
        ## diagonal
        nrow <- length(A)
        ncol <- length(A)
        elems <- length(A)
        datatype <- 1 ## sparse
        valuetype <- inla.ifelse(is.integer(A), integer(), double())
        matrixtype <- 2 ## diagonal
        storagetype <- 1 ## columnmajor

        i <- j <- 0:(nrow - 1)
        values <- A
    } else {
        stop(inla.paste(c(
            "Unknown type of matrix:",
            deparse(match.call()),
            "\n class(A) =",
            class(A)
        )))
    }

    h <- integer(8)
    valuetp <- inla.ifelse(identical(valuetype, integer()), 0, 1)
    h <- c(version, elems, nrow, ncol, datatype, valuetp, matrixtype, storagetype)
    if (verbose) {
          print(h)
      }
    if (debug) {
        if (!is.matrix(A)) {
            print("i (zero-based indexing)")
            print(i)
            print("j (zero-based indexing)")
            print(j)
            print("values")
            print(values)
        }
    }

    fp <- file(filename, "wb")
    writeBin(as.integer(length(h)), fp)
    writeBin(as.integer(h), fp)

    if (datatype == 0) {
        ## dense
        if (identical(valuetype, integer())) {
            writeBin(as.integer(as.vector(A)), fp)
        } else {
            writeBin(as.double(as.vector(A)), fp)
        }
    } else {
        ## sparse: columnorder
        writeBin(as.integer(i), fp)
        writeBin(as.integer(j), fp)
        if (identical(valuetype, integer())) {
            writeBin(as.integer(values), fp)
        } else {
            writeBin(as.double(values), fp)
        }
    }
    close(fp)

    return(filename)
}


inla.fmesher.make.dir <- function(dir) {
    dir.start <- dir

    ## if this already exists then create one more
    k <- 1
    while (file.exists(dir)) {
        dir <- paste(dir.start, "-", k, sep = "")
        k <- k + 1
    }

    return(dir)
}


fmesher.write <- function(m, prefix, matrixname) {
    filename <- paste(prefix, matrixname, sep = "")
    return(inla.write.fmesher.file(m, filename))
}

fmesher.read <- function(prefix, matrixname) {
    filename <- paste(prefix, matrixname, sep = "")
    if (!file.exists(filename)) {
          stop(paste("File '", filename, "' does not exist.", sep = ""))
      }
    return(inla.read.fmesher.file(filename))
}

#' @include spde.common.R
NULL

## 'spde3' model functions
## External: inla.spde.precision!inla.spde3
## External: inla.spde.result!inla.spde3 inla.spde3.generic
## External: inla.spde3.matern param2.matern.orig
## External: inla.spde3.matern.sd.basis inla.spde3.models
## External: inla.spde3.precision inla.spde3.result
## External: inla.spde3.theta2phi0 inla.spde3.theta2phi1
## External: inla.spde3.theta2phi2 inla.spde3.theta2phi3
##
## Internal: inla.internal.spde3.matern.B.tau
## Internal: inla.internal.test.spde3.sd.basis





inla.spde3.generic <-
    function(M0, M1, M2, M3, B0, B1, B2, B3, theta.mu, theta.Q,
             transform = c("identity","log","logit","shiftedlog","oldlogit"),
             theta.initial = theta.mu,
             fixed = rep(FALSE, length(theta.mu)),
             theta.fixed = theta.initial[fixed],
             BLC = cbind(0.0, diag(nrow=length(theta.mu))),
             ...)
{
    warning("The spde3 interface is still experimental and may be changed without notice.")
    transform = match.arg(transform)

    if (!inla.is.matrix(M0)) stop("M0 must be of a matrix type.")
    if (!inla.is.matrix(M1)) stop("M1 must be of a matrix type.")
    if (!inla.is.matrix(M2)) stop("M2 must be of a matrix type.")
    if (!inla.is.matrix(M3)) stop("M3 must be of a matrix type.")
    M0 = inla.as.dgTMatrix(M0)
    M1 = inla.as.dgTMatrix(M1)
    M2 = inla.as.dgTMatrix(M2)
    M3 = inla.as.dgTMatrix(M3)

    n.spde = nrow(M0)
    n.theta = length(theta.mu)

    spde = (list(model = "generic",
                 n.spde = n.spde,
                 n.theta = n.theta,
                 param.inla = list(),
                 f = list()
                 ))
    class(spde) = c("inla.spde3", "inla.spde", "inla.model.class")

    B0 = inla.spde.homogenise_B_matrix(B0, n.spde, n.theta)
    B1 = inla.spde.homogenise_B_matrix(B1, n.spde, n.theta)
    B2 = inla.spde.homogenise_B_matrix(B2, n.spde, n.theta)
    B3 = inla.spde.homogenise_B_matrix(B3, n.spde, n.theta)
    BLC = inla.spde.homogenise_B_matrix(BLC, nrow(BLC), n.theta)

    param.inla =
        list(n = n.spde,
             n.theta = n.theta,
             M0=M0, M1=M1, M2=M2, M3=M3,
             B0=B0, B1=B1, B2=B2, B3=B3, BLC=BLC,
             theta.mu=theta.mu, theta.Q=theta.Q,
             transform=transform,
             theta.initial=theta.initial,
             fixed=fixed,
             theta.fixed=theta.fixed)
    ## TODO: Check dimension consistency of inputs

    ## If any "fixed", move information from B[,1+(1:n.theta)] into
    ## B[,1], theta.mu, and theta.Q
    ## NOTE: Should this be in the f function instead?
    ##       No, the information is also needed elsewhere.
    if (any(fixed)) {
        param.inla$theta.initial = theta.initial[!fixed]
        param.inla$B0[,1] =
            B0[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B1[,1] =
            B1[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B2[,1] =
            B2[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B3[,1] =
            B3[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$BLC[,1] =
            BLC[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B0 = param.inla$B0[,c(TRUE, !fixed), drop=FALSE]
        param.inla$B1 = param.inla$B1[,c(TRUE, !fixed), drop=FALSE]
        param.inla$B2 = param.inla$B2[,c(TRUE, !fixed), drop=FALSE]
        param.inla$B3 = param.inla$B3[,c(TRUE, !fixed), drop=FALSE]
        param.inla$BLC = param.inla$BLC[,c(TRUE, !fixed), drop=FALSE]
        param.inla$theta.Q = param.inla$theta.Q[!fixed, !fixed, drop=FALSE]
        if (!all(fixed)) {
            param.inla$theta.mu =
                theta.mu - solve(param.inla$theta.Q,
                                 theta.Q[!fixed, fixed, drop=FALSE]
                                 %*% (theta.fixed[fixed]-theta.mu[fixed]))
        } else { ## All fixed.
            param.inla$theta.mu = as.vector(matrix(0.0, 0, 1))
        }
    }

    ## Remove all BLC rows that would produce point masses, which is
    ## not supported by inla:
    if (n.theta==0) {
        param.inla$BLC = matrix(0,0,1)
    } else {
        param.inla$BLC =
            param.inla$BLC[rowSums(abs(param.inla$BLC[,-1,drop=FALSE]))>0,,
                           drop=FALSE]
    }

    spde$param.inla = param.inla

    spde$f = (list(model="spde3",
                   n=n.spde,
                   spde3.transform=transform,
                   hyper.default =
                   (list(theta1 =
                         list(prior="mvnorm",
                              param=(c(param.inla$theta.mu,
                                       as.matrix(param.inla$theta.Q)))
                              )
                         ))
                   ))
    if (n.theta>0)
        for (k in 1:n.theta) {
            eval(parse(text=
                       paste("spde$f$hyper.default$theta", k,
                             "$initial = param.inla$theta.mu[k]", sep="")))
        }

    return(spde)
}









inla.internal.spde3.matern.B.tau =
    function(precision.fcn, n.theta, delta = 1e-3, ...)
{
    theta.0 = rep(0, n.theta)
    Q.0 = precision.fcn(theta.0, ...)
    logv.0 = log(diag(inla.qinv(Q.0)))

    B.tau = matrix(0, nrow(Q.0), 1L+n.theta)
    for (k in seq_len(n.theta)) {
        theta = theta.0
        theta[k] = theta[k] + delta
        Q = precision.fcn(theta, ...)
        logv = log(diag(inla.qinv(Q)))

        dlogv = (logv-logv.0)/delta

        B.tau[, k+1] = dlogv/2
    }
    B.tau[,1L] = logv.0/2

    return(B.tau)
}



inla.spde3.matern.sd.basis =
    function(mesh, B.sd, B.range, method=1,
             local.offset.compensation=FALSE,
             alpha=2, ...)
{
    p.sd = ncol(B.sd)-1L
    p.kappa = ncol(B.range)-1L

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)

    ## log(kappa) = 0.5*log(8*nu) - log(range)
    B.kappa = (cbind(log(8*nu.nominal)/2 - B.range[,1],
                     -B.range[,-1,drop=FALSE] ))

    if (method==1) {
        spde =
            inla.spde3.matern(mesh, B.tau=as.matrix(0), B.kappa=B.kappa, ...)

        B.tau =
            inla.internal.spde3.matern.B.tau(
                precision.fcn=
                function(theta, spde) {
                    return(inla.spde3.precision(spde, theta=theta))
                },
                n.theta = p.kappa,
                spde = spde)
        if (!local.offset.compensation) {
            B.tau[,1] = rep(mean(B.tau[,1]), spde$n.spde)
        }
    } else {
        spde =
            inla.spde3.matern(mesh,
                              B.tau=as.matrix(0),
                              B.kappa=B.kappa[,1,drop=FALSE],
                              alpha=alpha, ...)
        lvar.0 =
            log(diag(inla.qinv(inla.spde3.precision(spde, theta=rep(0,0)))))
        if (!local.offset.compensation) {
            lvar.0 = rep(mean(lvar.0), spde$n.spde)
        }

        B.tau = -B.kappa*nu
        B.tau[,1] = lvar.0/2
    }

    log.var.scaling = lgamma(nu.nominal)-lgamma(alpha.nominal)-log(4*pi)*d/2

    B.tau =
        cbind(B.tau[,1]-B.sd[,1],
              -B.sd[,-1,drop=FALSE],
              B.tau[,-1,drop=FALSE])
    B.kappa =
        cbind(B.kappa[,1],
              matrix(0, n.spde, p.sd),
              B.kappa[,-1,drop=FALSE])

    return(list(B.tau=B.tau, B.kappa=B.kappa))

}



inla.internal.test.spde3.sd.basis = function (k=1, dth=0.1, r=1000, globe=25, compensate=FALSE) {

    alpha=2
    mesh = inla.mesh.create(globe=globe)

    B.common = inla.mesh.basis(mesh, "b.spline", n=3)
    B.common = cbind(B.common, B.common[,1]*(mesh$loc[,3]>0))
    B.common[,1] = B.common[,1]*(mesh$loc[,3]<=0)

    B.sd = cbind(0, B.common)
    B.range = cbind(log(r/6370), B.common)

    sd.basis0 = inla.spde3.matern.sd.basis(mesh, B.sd=B.sd, B.range=B.range, method=0, alpha=alpha, local.offset.compensation=compensate)
    sd.basis1 = inla.spde3.matern.sd.basis(mesh, B.sd=B.sd, B.range=B.range, method=1, alpha=alpha, local.offset.compensation=compensate)

    spde0 =
        inla.spde3.matern(mesh, alpha=alpha,
                          B.tau=sd.basis0$B.tau,
                          B.kappa=sd.basis0$B.kappa)
    spde1 =
        inla.spde3.matern(mesh, alpha=alpha,
                          B.tau=sd.basis1$B.tau,
                          B.kappa=sd.basis1$B.kappa)

    th0 = c(0,0,0,0,0,0,0,0)
    th1 = th0
    th1[k] = th0[k]+dth
    v0.0=diag(inla.qinv(inla.spde3.precision(spde0,theta=th0)))
    v0.1=diag(inla.qinv(inla.spde3.precision(spde0,theta=th1)))
    v1.0=diag(inla.qinv(inla.spde3.precision(spde1,theta=th0)))
    v1.1=diag(inla.qinv(inla.spde3.precision(spde1,theta=th1)))

    op=par(mfrow=c(2,2))
    on.exit(par(op))
    plot(asin(mesh$loc[,3]),rowSums(sd.basis0$B.kappa[,1+k,drop=FALSE]), pch=20,
         ylim=(range(c(0, range(rowSums(sd.basis0$B.kappa[,1+k,drop=FALSE])),
                       range(-rowSums(sd.basis0$B.tau[,1+k,drop=FALSE])),
                       range(-rowSums(sd.basis1$B.tau[,1+k,drop=FALSE]))))),
         main="Basis functions")
    points(asin(mesh$loc[,3]),-rowSums(sd.basis0$B.tau[,1+k,drop=FALSE]),col=2,pch=20)
    points(asin(mesh$loc[,3]),-rowSums(sd.basis1$B.tau[,1+k,drop=FALSE]),col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1, col=2,pch=20,
         ylim=range(c(0,range(v0.0),range(v1.0),range(v0.1),range(v1.1))),
         main="Absolute variances")
    points(asin(mesh$loc[,3]),v1.1, col=4,pch=20)
    points(asin(mesh$loc[,3]),v0.0, col=2,pch=1)
    points(asin(mesh$loc[,3]),v1.0, col=4,pch=1)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1-v0.0, col=2,pch=20,
         ylim=range(c(0,range(v0.1-v0.0),range(v1.1-v1.0))),
         main="Absolute variance error")
    points(asin(mesh$loc[,3]),v1.1-v1.0, col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1/v0.0-1, col=2,pch=20,
         ylim=range(c(0,range(v0.1/v0.0-1),range(v1.1/v1.0-1))),
         main="Relative variance error")
    points(asin(mesh$loc[,3]),v1.1/v1.0-1, col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    return(environment())
}




param2.matern.orig =
    function(mesh,
             alpha=2,
             B.tau = matrix(c(0,1,0),1,3),
             B.kappa = matrix(c(0,0,1),1,3),
             prior.variance.nominal = 1,
             prior.range.nominal = NULL,
             prior.tau = NULL,
             prior.kappa = NULL,
             theta.prior.mean = NULL,
             theta.prior.prec = 0.1)
{
    ## NOTE: For d==1, degree==2, the B.* must be given per basis
    ## function, not per knot.

    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    if (is.null(B.tau))
        stop("B.tau must not be NULL.")
    if (is.null(B.kappa))
        stop("B.kappa must not be NULL.")
    is.stationary = (nrow(B.kappa)==1) && (nrow(B.tau)==1)

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)
    n.theta = ncol(B.kappa)-1L

    B.kappa = inla.spde.homogenise_B_matrix(B.kappa, n.spde, n.theta)
    B.tau = inla.spde.homogenise_B_matrix(B.tau, n.spde, n.theta)

    B.prec =
        cbind(+ lgamma(alpha.nominal) - lgamma(nu.nominal)
              + (d/2)*log(4*pi) + 2*nu.nominal*B.kappa[,1]
              + 2*B.tau[,1],
              2*nu.nominal*B.kappa[,-1,drop=FALSE]
              + 2*B.tau[,-1,drop=FALSE] )
    if (is.stationary) {
        B.tau = B.tau[1,,drop=FALSE]
        B.kappa = B.kappa[1,,drop=FALSE]
        B.prec = B.prec[1,,drop=FALSE]
    }
    B.variance = -B.prec
    B.range =
        cbind(0.5*log(8*nu.nominal)-B.kappa[,1],
              -B.kappa[,-1,drop=FALSE])

    if (n.theta>0) {
        B.theta = cbind(0,diag(1, n.theta))
        rownames(B.theta) <- rownames(B.theta, do.NULL=FALSE, prefix="theta.")
    } else {
        B.theta = NULL
    }
    rownames(B.tau) <- rownames(B.tau, do.NULL=FALSE, prefix="tau.")
    rownames(B.kappa) <- rownames(B.kappa, do.NULL=FALSE, prefix="kappa.")
    rownames(B.variance) <-
        rownames(B.variance, do.NULL=FALSE, prefix="variance.nominal.")
    rownames(B.range) <-
        rownames(B.range, do.NULL=FALSE, prefix="range.nominal.")
    BLC = rbind(B.theta, B.tau, B.kappa, B.variance, B.range)


    ## Construct priors.
    if (is.null(theta.prior.prec)) {
        theta.prior.prec = diag(0.1, n.theta, n.theta)
    } else {
        theta.prior.prec = as.matrix(theta.prior.prec)
        if (ncol(theta.prior.prec) == 1) {
            theta.prior.prec =
                diag(as.vector(theta.prior.prec), n.theta, n.theta)
        }
        if ((nrow(theta.prior.prec) != n.theta) ||
            (ncol(theta.prior.prec) != n.theta)) {
            stop(paste("Size of theta.prior.prec is (",
                       paste(dim(theta.prior.prec), collapse=",", sep=""),
                       ") but should be (",
                       paste(c(n.theta, n.theta), collapse=",", sep=""),
                       ")."))
        }
    }

    if (is.null(theta.prior.mean)) {
        if (is.null(prior.range.nominal)) {
            mesh.range =
                inla.ifelse(d==2,
                            (max(c(diff(range(mesh$loc[,1])),
                                   diff(range(mesh$loc[,2])),
                                   diff(range(mesh$loc[,3]))
                                   ))),
                            diff(mesh$interval))
            prior.range.nominal = mesh.range*0.2
        }

        if (is.null(prior.kappa)) {
            prior.kappa = sqrt(8*nu.nominal)/prior.range.nominal
        }
        if (is.null(prior.tau)) {
            prior.tau =
                sqrt(gamma(nu.nominal)/gamma(alpha.nominal)/
                     (4*pi*prior.kappa^(2*nu.nominal)*prior.variance.nominal))
        }

        if (n.theta>0) {
            theta.prior.mean =
                qr.solve(rbind(B.tau[,-1,drop=FALSE], B.kappa[,-1,drop=FALSE]),
                         c(log(prior.tau) - B.tau[,1],
                           log(prior.kappa) - B.kappa[,1]))
        } else {
            theta.prior.mean = rep(0, n.theta) ## Empty vector
        }
    }

    param =
        list(is.stationary=is.stationary,
             B.tau=B.tau, B.kappa=B.kappa, BLC=BLC,
             theta.prior.mean=theta.prior.mean,
             theta.prior.prec=theta.prior.prec)
    return(param)
}

inla.spde3.matern =
    function(mesh,
             alpha=2,
             param = NULL,
             constr = FALSE,
             extraconstr.int = NULL,
             extraconstr = NULL,
             fractional.method = c("parsimonious", "null"),
             B.tau = matrix(c(0,1,0),1,3),
             B.kappa = matrix(c(0,0,1),1,3),
             prior.variance.nominal = 1,
             prior.range.nominal = NULL,
             prior.tau = NULL,
             prior.kappa = NULL,
             theta.prior.mean = NULL,
             theta.prior.prec = 0.1)
{
    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    fractional.method = match.arg(fractional.method)

    if (is.null(param)) {
        param =
            param2.matern.orig(
                mesh, alpha,
                B.tau, B.kappa,
                prior.variance.nominal,
                prior.range.nominal,
                prior.tau,
                prior.kappa,
                theta.prior.mean,
                theta.prior.prec)
    } else {
        deprecated =
            !c(missing(B.tau), missing(B.kappa),
               missing(prior.variance.nominal),
               missing(prior.range.nominal),
               missing(prior.tau), missing(prior.kappa),
               missing(theta.prior.mean), missing(theta.prior.prec))
        deprecated =
            c("B.tau", "B.kappa",
              "prior.variance.nominal",
              "prior.range.nominal",
              "prior.tau", "prior.kappa",
              "theta.prior.mean", "theta.prior.prec")[deprecated]
        if (length(deprecated) > 0) {
            warning(paste("'param' specified;  ",
                          "Ignoring deprecated parameter(s) ",
                          paste(deprecated, collapse=", "), ".", sep=""))
        }
    }

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)
    n.theta = ncol(B.kappa)-1L

    if (d==2) {
        fem =
            inla.fmesher.smorg(mesh$loc,
                               mesh$graph$tv,
                               fem=2,
                               output=list("c0", "c1", "g1", "g2"))
    } else {
        fem = inla.mesh.1d.fem(mesh)
        if (mesh$degree==2) {
            fem$c0 = fem$c1 ## Use higher order matrix.
        }
    }

    if (alpha==2) {
        B.phi0 = param$B.tau
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0
        M1 = fem$g1
        M2 = fem$g2
    } else if (alpha==1) {
        B.phi0 = param$B.tau
        B.phi1 = param$B.kappa
        M0 = fem$c0
        M1 = fem$g1*0
        M2 = fem$g1
    } else if (!param$is.stationary) {
        stop("Non-stationary Matern with fractional alpha is not implemented.")
    } else if ((alpha<2) && (alpha>1)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,0, 1,1,0, 1,2,1),3,3) %*%
                solve(matrix(1/(c(4:2, 3:1, 2:0)+lambda), 3, 3),
                      1/(c(4:2)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha,alpha*(alpha-1)/2)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-2)*param$B.kappa
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*b[2]/2
        M2 = fem$g2*b[3]
    } else if ((alpha<1) && (alpha>0)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,1,1),2,2) %*%
                solve(matrix(1/(c(2:1, 1:0)+lambda), 2, 2),
                      1/(c(2:1)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-1)*param$B.kappa
        B.phi1 = param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*0
        M2 = fem$g1*b[2]
    } else {
        stop(paste("Unsupported alpha value (", alpha,
                   "). Supported values are 0 < alpha <= 2", sep=""))
    }

    spde =
        inla.spde3.generic(M0=M0, M1=M1, M2=M2,
                           B0=B.phi0, B1=B.phi1, B2=1,
                           theta.mu = param$theta.prior.mean,
                           theta.Q = param$theta.prior.prec,
                           transform = "identity",
                           BLC = param$BLC)
    spde$model = "matern"
    spde$BLC = param$BLC

    if (constr || !is.null(extraconstr.int)) {
        if (constr) {
            A.constr = matrix(colSums(fem$c1), 1, n.spde)
            e.constr = 0
        } else {
            A.constr = matrix(numeric(0), 0, n.spde)
            e.constr = c()
        }
        if (!is.null(extraconstr.int)) {
            A.constr =
                rbind(A.constr,
                      matrix(extraconstr.int$A %*% fem$c1,
                             nrow(extraconstr.int$A), n.spde))
            e.constr = c(e.constr, extraconstr.int$e)
        }
        if (!is.null(extraconstr)) {
            A.constr = rbind(A.constr, extraconstr$A)
            e.constr = c(e.constr, extraconstr$e)
        }

        spde$f$constr = FALSE
        spde$f$extraconstr = list(A=A.constr, e=e.constr)
    } else if (!is.null(extraconstr)) {
        spde$f$constr = FALSE
        spde$f$extraconstr = extraconstr
    }

    return(invisible(spde))
}








param3.iheat <- function(mesh.space,
                         mesh.time,
                         gamma.E = NULL,
                         alpha.E = NULL,
                         theta.prior.mean = NULL,
                         theta.prior.prec = NULL)
{
    return(list(gamma.E=gamma.E,
                alpha.E=alpha.E,
                theta.prior.mean=theta.prior.mean,
                theta.prior.prec=theta.prior.prec))
}

inla.spde3.iheat =
    function(mesh.space,
             mesh.time,
             order=2,
             param = NULL,
             constr = FALSE,
             extraconstr.int = NULL,
             extraconstr = NULL)
{
    ## order=1:
    ## gamma.t dot(u) - Laplacian (u) = E
    ##
    ## order=2:
    ## gamma.t dot(u) - Laplacian (u) = v
    ## gamma.t dot(v) - Laplacian (v) = E
    ##
    ## (1 - gamma.E Laplacian)^(alpha.E/2) E = W/sqrt(gamma.s)
    ## gamma.E and alpha.E known

    inla.require.inherits(mesh.space, c("inla.mesh", "inla.mesh.1d"),
                          "'mesh.space'")
    inla.require.inherits(mesh.time, c("inla.mesh.1d"),
                          "'mesh.time'")

  if (is.null(param)) {
    stop("Use param3.iheat() to construct the prior parameter settings")
    ##        param =
    ##            param3.iheat(
    ##                mesh.space, mesh.time,
    ##                theta.prior.mean,
    ##                theta.prior.prec)
  }

    d.space = inla.ifelse(inherits(mesh.space, "inla.mesh.1d"), 1, 2)
    d.time = 1
    n.space = inla.ifelse(d.space==2, mesh.space$n, mesh.space$m)
    n.time = mesh.time$m
    n.spde = n.space*n.time
    n.theta = 2L ## gamma.s, gamma.t

    if (d.space==2) {
        fem.space = inla.mesh.fem(mesh.space$loc, 2)
    } else {
        fem.space = inla.mesh.1d.fem(mesh.space)
        if (mesh.space$degree==2) {
            fem.space$c0 = fem.space$c1 ## Use higher order matrix.
        }
    }
    fem.time = inla.mesh.1d.fem(mesh.time)
    if (mesh.time$degree==2) {
        fem.time$c0 = fem.time$c1 ## Use higher order matrix.
    }

  ## TODO: the rest
  if (FALSE) {

    if (alpha==2) {
        B.phi0 = param$B.tau
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0
        M1 = fem$g1
        M2 = fem$g2
    } else if (alpha==1) {
        B.phi0 = param$B.tau
        B.phi1 = param$B.kappa
        M0 = fem$c0
        M1 = fem$g1*0
        M2 = fem$g1
    } else if (!param$is.stationary) {
        stop("Non-stationary Matern with fractional alpha is not implemented.")
    } else if ((alpha<2) && (alpha>1)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,0, 1,1,0, 1,2,1),3,3) %*%
                solve(matrix(1/(c(4:2, 3:1, 2:0)+lambda), 3, 3),
                      1/(c(4:2)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha,alpha*(alpha-1)/2)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-2)*param$B.kappa
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*b[2]/2
        M2 = fem$g2*b[3]
    } else if ((alpha<1) && (alpha>0)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,1,1),2,2) %*%
                solve(matrix(1/(c(2:1, 1:0)+lambda), 2, 2),
                      1/(c(2:1)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-1)*param$B.kappa
        B.phi1 = param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*0
        M2 = fem$g1*b[2]
    } else {
        stop(paste("Unsupported alpha value (", alpha,
                   "). Supported values are 0 < alpha <= 2", sep=""))
    }

    spde =
        inla.spde3.generic(M0=M0, M1=M1, M2=M2,
                           B0=B.phi0, B1=B.phi1, B2=1,
                           theta.mu = param$theta.prior.mean,
                           theta.Q = param$theta.prior.prec,
                           transform = "identity",
                           BLC = param$BLC)
    spde$model = "matern"
    spde$BLC = param$BLC

    if (constr || !is.null(extraconstr.int)) {
        if (constr) {
            A.constr = matrix(colSums(fem$c1), 1, n.spde)
            e.constr = 0
        } else {
            A.constr = matrix(numeric(0), 0, n.spde)
            e.constr = c()
        }
        if (!is.null(extraconstr.int)) {
            A.constr =
                rbind(A.constr,
                      matrix(extraconstr.int$A %*% fem$c1,
                             nrow(extraconstr.int$A), n.spde))
            e.constr = c(e.constr, extraconstr.int$e)
        }
        if (!is.null(extraconstr)) {
            A.constr = rbind(A.constr, extraconstr$A)
            e.constr = c(e.constr, extraconstr$e)
        }

        spde$f$constr = FALSE
        spde$f$extraconstr = list(A=A.constr, e=e.constr)
    } else if (!is.null(extraconstr)) {
        spde$f$constr = FALSE
        spde$f$extraconstr = extraconstr
    }

    return(invisible(spde))
  }

  invisible(NULL)
}










inla.spde3.theta2phi0 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde3", "'spde'")

    if (spde$n.theta>0)
        return(exp(spde$param.inla$B0[,1, drop=TRUE] +
                   spde$param.inla$B0[,-1, drop=FALSE] %*% theta))
    else
        return(exp(spde$param.inla$B0[,1, drop=TRUE]))
}

inla.spde3.theta2phi1 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde3", "'spde'")

    if (spde$n.theta>0)
        return(exp(spde$param.inla$B1[,1, drop=TRUE] +
                   spde$param.inla$B1[,-1, drop=FALSE] %*% theta))
    else
        return(exp(spde$param.inla$B1[,1, drop=TRUE]))
}

inla.spde3.theta2phi2 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde3", "'spde'")

    if (spde$n.theta>0) {
        phi = ((spde$param.inla$B2[,1, drop=TRUE] +
                spde$param.inla$B2[,-1, drop=FALSE] %*% theta))
    } else {
        phi = ((spde$param.inla$B2[,1, drop=TRUE]))
    }
    if (spde$param.inla$transform == "identity") {
        return(phi)
    } else if (spde$param.inla$transform == "log") {
        return(exp(phi))
    } else if (spde$param.inla$transform == "logit") {
        return((1-exp(phi))/(1+exp(phi)))
    } else if (spde$param.inla$transform == "shiftedlog") {
        return(2*exp(phi)-1)
    } else if (spde$param.inla$transform == "oldlogit") {
        return(cos(pi/(1+exp(-phi))))
    } else {
        warning(paste("Unknown link function '",
                      spde$param.inla$transform,
                      "' phi2.  Using identity link instead.", sep=""))
        return(phi)
    }
}

inla.spde3.theta2phi3 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde3", "'spde'")

    if (spde$n.theta>0)
        return(exp(spde$param.inla$B3[,1, drop=TRUE] +
                   spde$param.inla$B3[,-1, drop=FALSE] %*% theta))
    else
        return(exp(spde$param.inla$B3[,1, drop=TRUE]))
}

inla.spde3.precision =
    function(spde, theta=NULL,
             phi0=inla.spde3.theta2phi0(spde, theta),
             phi1=inla.spde3.theta2phi1(spde, theta),
             phi2=inla.spde3.theta2phi2(spde, theta),
             phi3=inla.spde3.theta2phi3(spde, theta),
             ...)
{
    inla.require.inherits(spde, "inla.spde3", "'spde'")
    if (spde$f$model!="spde3") {
        stop("'inla.spde3.precision' only supports internal inla models 'spde3'")
    }

    D0 <- Diagonal(spde$n.spde, phi0)
    D1 <- Diagonal(spde$n.spde, phi1)
    D12 <- Diagonal(spde$n.spde, phi1*phi2)
    D3 <- Diagonal(nrow(spde$param.inla$M3), phi3)
    Q <- (D0 %*% (D1 %*% spde$param.inla$M0 %*% D1 +
                  D12 %*% spde$param.inla$M1 +
                  t(spde$param.inla$M1) %*% D12 +
                  spde$param.inla$M2 +
                  t(spde$param.inla$M3) %*% D3 %*% spde$param.inla$M3)
          %*% D0)

    return(Q)
}



inla.spde3.result = function(inla, name, spde, do.transform=TRUE, ...)
{
    inla.require.inherits(inla, "inla", "'inla'")
    inla.require.inherits(spde, "inla.spde3", "'spde'")
    if (!spde$f$model=="spde3") {
        stop("'inla.spde3.result' only supports internal inla models 'spde3'")
    }

    result = list()
    ## Setup rownames
    BLC.names = rownames(spde$param.inla$BLC)

    ## Values
    result$summary.values = inla$summary.random[[name]]

    ## Marginals for values
    if (!is.null(inla$marginals.random[[name]])) {
        result$marginals.values = inla$marginals.random[[name]]
    }

    ## Theta
    result$summary.hyperpar =
        inla.extract.el(inla$summary.hyperpar,
                        paste("Theta[^ ]+ for ", name, "$", sep=""))

    ## Marginals for theta
    if (!is.null(inla$marginals$hyperpar)) {
        result$marginals.hyperpar =
            inla.extract.el(inla$marginals.hyperpar,
                            paste("Theta[^ ]+ for ", name, "$", sep=""))
    }

    ## log-tau/kappa/variance/range
    if (!is.null(inla$summary.spde3.blc[[name]])) {
        rownames(inla$summary.spde3.blc[[name]]) <- BLC.names

        result$summary.theta =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "theta\\.[^ ]+$")
        result$summary.log.tau =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "tau\\.[^ ]+$")
        result$summary.log.kappa =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$summary.log.variance.nominal =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$summary.log.range.nominal =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "range.nominal\\.[^ ]+$")
    }

    ## Marginals for log-tau/kappa/variance/range
    if (!is.null(inla$marginals.spde3.blc[[name]])) {
        names(inla$marginals.spde3.blc[[name]]) <- BLC.names

        result$marginals.theta =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "theta\\.[^ ]+$")
        result$marginals.log.tau =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "tau\\.[^ ]+$")
        result$marginals.log.kappa =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$marginals.log.variance.nominal =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$marginals.log.range.nominal =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "range.nominal\\.[^ ]+$")

        if (do.transform) {
            result$marginals.tau =
                lapply(result$marginals.log.tau,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.kappa =
                lapply(result$marginals.log.kappa,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.variance.nominal =
                lapply(result$marginals.log.variance.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.range.nominal =
                lapply(result$marginals.log.range.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
        }
    }

    return(result)
}

inla.spde3.result = function(inla, name, spde, do.transform=TRUE, ...)
{
    inla.require.inherits(inla, "inla", "'inla'")
    inla.require.inherits(spde, "inla.spde3", "'spde'")
    if (!spde$f$model=="spde3") {
        stop("'inla.spde3.result' only supports internal inla models 'spde3'")
    }

    result = list()
    ## Setup rownames
    BLC.names = rownames(spde$param.inla$BLC)

    ## Values
    result$summary.values = inla$summary.random[[name]]

    ## Marginals for values
    if (!is.null(inla$marginals.random[[name]])) {
        result$marginals.values = inla$marginals.random[[name]]
    }

    ## Theta
    result$summary.hyperpar =
        inla.extract.el(inla$summary.hyperpar,
                        paste("Theta[^ ]+ for ", name, "$", sep=""))

    ## Marginals for theta
    if (!is.null(inla$marginals$hyperpar)) {
        result$marginals.hyperpar =
            inla.extract.el(inla$marginals.hyperpar,
                            paste("Theta[^ ]+ for ", name, "$", sep=""))
    }

    ## log-tau/kappa/variance/range
    if (!is.null(inla$summary.spde3.blc[[name]])) {
        rownames(inla$summary.spde3.blc[[name]]) <- BLC.names

        result$summary.theta =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "theta\\.[^ ]+$")
        result$summary.log.tau =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "tau\\.[^ ]+$")
        result$summary.log.kappa =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$summary.log.variance.nominal =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$summary.log.range.nominal =
            inla.extract.el(inla$summary.spde3.blc[[name]],
                            "range.nominal\\.[^ ]+$")
    }

    ## Marginals for log-tau/kappa/variance/range
    if (!is.null(inla$marginals.spde3.blc[[name]])) {
        names(inla$marginals.spde3.blc[[name]]) <- BLC.names

        result$marginals.theta =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "theta\\.[^ ]+$")
        result$marginals.log.tau =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "tau\\.[^ ]+$")
        result$marginals.log.kappa =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$marginals.log.variance.nominal =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$marginals.log.range.nominal =
            inla.extract.el(inla$marginals.spde3.blc[[name]],
                            "range.nominal\\.[^ ]+$")

        if (do.transform) {
            result$marginals.tau =
                lapply(result$marginals.log.tau,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.kappa =
                lapply(result$marginals.log.kappa,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.variance.nominal =
                lapply(result$marginals.log.variance.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.range.nominal =
                lapply(result$marginals.log.range.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
        }
    }

    return(result)
}



inla.spde3.models = function()
{
    return(c("generic", "matern"))
}


## spde.common-connections:
inla.spde.precision.inla.spde3 = inla.spde3.precision
inla.spde.result.inla.spde3 = inla.spde3.result

## Export: inla.as.sparse inla.as.dgTMatrix

## !\name{inla.as.sparse}
## !\alias{inla.as.sparse}
## !\alias{inla.as.dgTMatrix}
## !
## !\title{Convert a matrix or sparse matrix into the sparse formate used by INLA}
## !
## !\description{Convert a matrix or sparse matrix into the sparse format used by INLA (dgTMatrix)}
## !
## !\usage{
## ! inla.as.sparse(...)
## ! inla.as.dgTMatrix(A, unique = TRUE, na.rm = FALSE, zeros.rm = FALSE)
## !}
## !
## !\arguments{
## !  \item{...}{The arguments. The matrix or sparse matrix,  and the additonal arguments}
## !  \item{A}{The matrix}
## !  \item{unique}{Logical. If \code{TRUE}, then ensure that the internal representation is unique and
## !                there are no duplicated entries.
## !                (Do not change this unless you know what you are doing.)}
## !  \item{na.rm}{Replace \code{NA}'s in the matrix with zeros.}
## !  \item{zeros.rm}{Remove zeros in the matrix.}
## !}
## !
## !\value{%%
## !  \code{inla.as.sparse} and \code{inla.as.dgTMatrix} is the same function.
## !  The returned value is a sparse matrix in the \code{dgTMatrix}-format.
## !}
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## ! A = matrix(1:9, 3, 3)
## ! inla.as.sparse(A)
## !}

`inla.as.sparse` <- function(...) {
    return(inla.as.dgTMatrix(...))
}

`inla.as.dgTMatrix` <- function(A, unique = TRUE, na.rm = FALSE, zeros.rm = FALSE) {
    ## convert into dgTMatrix format of Matrix. Argument A is any
    ## matrix.  make sure the representation is unique if the UNIQUE
    ## flag it TRUE. (ie no double triplets etc).
    ## if na.rm then replace NA's with zeros.
    ## if zeros.rm then remove zeros.

    if (unique) {
        ## convert through the 'dgCMatrix'-class to make it unique;
        A <- as(as(as(A, "CsparseMatrix"), "dgCMatrix"), "dgTMatrix")
    } else {
        if (!is(A, "dgTMatrix")) {
            ## Convert via virtual class TsparseMatrix; this allows more general conversions
            ## than direct conversion.
            A <- as(as(A, "TsparseMatrix"), "dgTMatrix")
        }
    }

    if (na.rm) {
        idx.na <- is.na(A@x)
        if (any(idx.na)) {
            A@x[idx.na] <- 0.0
        }
    }

    if (zeros.rm) {
        x.zero <- (A@x == 0.0)
        if (any(x.zero)) {
            idx.zero <- which(x.zero)
            A@x <- A@x[-idx.zero]
            A@i <- A@i[-idx.zero]
            A@j <- A@j[-idx.zero]
        }
    }

    return(A)
}


### Some utilities for sparse matrices using the `Matrix' library

`inla.sparse.dim` <- function(A) {
    ## return the dimension of the matrix A
    A <- inla.sparse.check(A, must.be.squared = FALSE)

    if (is.character(A)) {
        Am <- read.table(A, col.names = c("i", "j", "Aij"))
        if (min(Am$i) == 0 || min(Am$j) == 0) {
            cindex <- 1
        } else {
            cindex <- 0
        }
        return(c(max(Am$i) + cindex, max(Am$j) + cindex))
    } else {
        return(dim(A))
    }
}


`inla.sparse.check` <- function(A, must.be.squared = TRUE) {
    ## just check if matrix A exists and return either a filename or a
    ## dgTMatrix.
    if (is.character(A)) {
        if (!file.exists(A)) {
            stop(paste("File not found:", A))
        }
        return(A)
    }

    if (is.list(A)) {
          stop("Define matrix using Matrix::sparseMatrix() instead!!! The list(i=, j=, values=)-format is obsolete!")
      }

    if (must.be.squared) {
        if (dim(A)[1] != dim(A)[2]) {
            stop(paste(c("Matrix is not a square matrix:", dim(A)[1], "x", dim(A)[2])))
        }
    }

    return(inla.as.dgTMatrix(A))
}

`inla.sparse.get` <- function(A, row, col) {
    ## extract a list of the a specific row or col of a dgTMatrix
    ## A. the list returned is of type list(i=..., j=..., values=...)

    if (missing(row) && missing(col)) {
          return(list(i = numeric(0), j = numeric(0), values = numeric(0)))
      }

    if (!missing(row) && !missing(col)) {
          stop("Only one of 'row' and 'col' can be specified.")
      }

    ## need this particular format. I think this is faster for
    ## repeated use, as we do not need to duplicate the A matrix in
    ## memory. but perhaps I'm wrong...
    if (!is(A, "dgTMatrix")) {
        ## This can be slow, so its better to stop and say that the
        ## matrix has to be converted upfront.
        stop("Matrix is not of type 'dgTMatrix'; please convert it with inla.as.dgTMatrix().")
        A <- inla.as.dgTMatrix(A)
    }

    if (!missing(row)) {
        stopifnot(row >= 1 && row <= A@Dim[1])
        stopifnot(length(row) == 1)

        idx <- which(A@i == row - 1)
        return(list(i = row, j = A@j[idx] + 1, values = A@x[idx]))
    } else {
        stopifnot(col >= 1 && col <= A@Dim[2])
        stopifnot(length(col) == 1)

        idx <- which(A@j == col - 1)
        return(list(i = A@i[idx] + 1, j = col, values = A@x[idx]))
    }
}

inla.sm.write <- function(A, filename = "SparseMatrix.dat") {
    stopifnot(!missing(A))
    A <- inla.as.sparse(A)

    x.int <- as.integer(c(dim(A)[1], dim(A)[2], length(A@i), A@i + 1, A@j + 1))
    fp <- file(filename, "wb")
    writeBin(x.int, fp)
    writeBin(A@x, fp)
    close(fp)

    return(filename)
}

inla.sm.read <- function(filename = "SparseMatrix.dat") {
    fp <- file(filename, "rb")
    nn <- readBin(fp, integer(), n = 3L)
    nx <- nn[3]
    ii <- readBin(fp, integer(), n = nx)
    jj <- readBin(fp, integer(), n = nx)
    xx <- readBin(fp, double(), n = nx)
    close(fp)

    M <- inla.as.sparse(sparseMatrix(
        i = ii - 1L,
        j = jj - 1L,
        x = xx,
        dims = nn[1:2],
        index1 = FALSE,
        repr = "T"
    ))
    return(M)
}

## Export: inla.idx

## !\name{idx}
## !\alias{idx}
## !\alias{inla.idx}
## !
## !\title{Convert indexes}
## !
## !\description{Convert indexes given by to triplet `(idx, group,
## !  replicate)' to the (one-dimensional) index used in the grouped and
## !  replicated model}
## !
## !\usage{
## !inla.idx(idx, n = max(idx),
## !         group = rep(1, length(idx)), ngroup = max(group),
## !         replicate = rep(1, length(idx)),  nrep = max(replicate))
## !}
## !
## !\arguments{
## !
## !  \item{idx}{The index within the basic model. (Legal values from `1' to `n'.)}
## !  \item{n}{The length `n' of the basic model.}
## !  \item{group}{The index within group. (Legal values from `1' to `ngroup'.)}
## !  \item{ngroup}{Number of groups.}
## !  \item{replicate}{The index within replication. (Legal values from
## !  `1' to `nrep'.)}
## !  \item{nrep}{Number of replications.}
## !}
## !
## !\value{%%
## !  \code{inla.idx} returns indexes in the range `1' to `n*ngroup*nrep'
## !  representing where the triplet `(idx,group,replicate)' is stored
## !  internally in the full grouped and replicated model.
## !}
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## !##TODO
## !}


`inla.idx` <- function(idx, n = max(idx),
                       group = rep(1, length(idx)), ngroup = max(group),
                       replicate = rep(1, length(idx)), nrep = max(replicate)) {
    ## this function might be useful to convert from (idx, group, rep)
    ## to idx, in the same way as done internally in inla.R

    stopifnot(n >= 1)
    stopifnot(ngroup >= 1)
    stopifnot(nrep >= 1)
    stopifnot(all(group >= 1))
    stopifnot(all(replicate >= 1))
    stopifnot(all(idx >= 1))
    stopifnot(all(idx <= n))
    stopifnot(ngroup >= max(group))
    stopifnot(nrep >= max(replicate))

    return(idx + (group - 1) * n + (replicate - 1) * n * ngroup)
}

## Export: summary!inla print!summary.inla

## !\name{summary.inla}
## !\alias{summary.inla}
## !\alias{summary.surv.inla}
## !\alias{print.summary.inla}
## !\title{ Summary for a INLA fit }
## !\description{
## !Takes a fitted \code{inla} or \code{surv.inla} object produced by
## !\code{inla} or  \code{surv.inla} and produces
## !a summary from it.
## !
## !}
## !\usage{
## !\method{summary}{inla}(object, digits = 3L, include.lincomb = TRUE, ...)
## !\method{print}{summary.inla}(x, digits = 3L, ...)
## !}
## !%- maybe also 'usage' for other objects documented here.
## !\arguments{
## !  \item{object}{  a fitted \code{inla} object as produced by
## !    \code{inla}.}
## !  \item{x}{a \code{summary.inla} object produced by \code{summary.inla}}
## !  \item{digits}{Integer Number of digits}
## !  \item{include.lincomb}{Logcial Include the summary for the the linear combinations or not}
## !  \item{...}{ other arguments.}
## !
## !}
## !\details{
## !Posterior mean and standard deviation (together with quantiles or
## !cdf) are printed for the fixed effects in the model.
## !
## !For the random effects the function \code{summary()} prints the
## !posterior mean and standard deviations for the hyperparameters
## !
## !If the option \code{short.summary} is set to \code{TRUE} using
## !\code{inla.setOption},
## !then  a less verbose summary
## !variant will be used,  which might be
## !more suitable for Markdown documents.
## !}
## !\value{
## !  \code{summary.inla} returns an object of class \code{summary.inla}, a
## !  list of components to print.
## !}
## !\author{Sara Martino and Havard Rue}
## !\seealso{ \code{\link{inla}} }

`summary.inla` <- function(object, digits = 3L, include.lincomb = TRUE, ...) {
    `internal.summary.inla` <- function(object, digits = 3L, include.lincomb = TRUE, ...) {
        inla.eval.dots(...)
        ## provides a summary for a inla object
        ret <- list()

        maxlen <- 2048L
        if (sum(nchar(object$call)) > maxlen) {
            ret <- c(
                ret,
                list(call = paste(
                    substr(inla.paste(object$call), 1L, maxlen),
                    "<<<the rest is not shown>>>"
                ))
            )
        } else {
            ret <- c(ret, list(call = object$call))
        }

        ## might not be if using collect directly
        if (inla.is.element("cpu.used", object)) {
            ret <- c(ret, list(
                cpu.used =
                    paste0(
                        names(object$cpu.used)[1], " = ",
                        format(object$cpu.used[1], digits = digits), ", ",
                        names(object$cpu.used)[2], " = ",
                        format(object$cpu.used[2], digits = digits), ", ",
                        names(object$cpu.used)[3], " = ",
                        format(object$cpu.used[3], digits = digits), ", ",
                        names(object$cpu.used)[4], " = ",
                        format(object$cpu.used[4], digits = digits)
                    )
            ))
        } else {
            ret <- c(ret, list(cpu.used = NA))
        }

        if (!is.null(object$summary.fixed) && length(object$summary.fixed) > 0) {
              ret <- c(ret, list(fixed = round(as.matrix(object$summary.fixed), digits = digits)))
          }

        if (include.lincomb) {
            if (!is.null(object$summary.lincomb) && any(names(object) == "summary.lincomb")
            && (length(object$summary.lincomb) > 0)) {
                  ret <- c(ret, list(lincomb = round(as.matrix(object$summary.lincomb), digits = digits)))
              }
            if (!is.null(object$summary.lincomb.derived) && length(object$summary.lincomb.derived) > 0) {
                  ret <- c(ret, list(lincomb.derived = round(as.matrix(object$summary.lincomb.derived),
                      digits = digits
                  )))
              }
        } else {
            if (!is.null(object$summary.lincomb) && any(names(object) == "summary.lincomb")
            && (length(object$summary.lincomb) > 0)) {
                m <- nrow(as.matrix(object$summary.lincomb))
                ret <- c(ret, list(lincomb = paste("<", m, " lincomb not included>", sep = "")))
            }
            if (!is.null(object$summary.lincomb.derived) && length(object$summary.lincomb.derived) > 0) {
                m <- nrow(as.matrix(object$summary.lincomb.derived))
                ret <- c(ret, list(lincomb.derived = paste("<", m, " lincomb.derived not included>", sep = "")))
            }
        }

        if (!is.null(object$summary.hyperpar) && length(object$summary.hyperpar) > 0) {
              ret <- c(ret, list(hyperpar = round(object$summary.hyperpar, digits = digits)))
          }

        if (!is.null(object$summary.random) && length(object$summary.random) > 0) {
              ret <- c(ret, list(random.names = names(object$summary.random), random.model = object$model.random))
          }

        if (!is.null(object$dic)) {
            ret <- c(ret, list(dic = lapply(object$dic, round, digits = digits)))
        }

        if (!is.null(object$waic)) {
            ret <- c(ret, list(waic = lapply(object$waic, round, digits = digits)))
        }

        if (!is.null(object$mlik)) {
              ret <- c(ret, list(mlik = round(object$mlik, digits = digits)))
          }

        if (!is.null(object$cpo$cpo) && length(object$cpo$cpo) > 0L) {
            ret <- c(ret, list(cpo = lapply(object$cpo, round, digits = digits)))
        }

        if (!is.null(object$summary.linear.predictor)) {
            ret <- c(ret, list(linear.predictor = round(object$summary.linear.predictor, digits = digits)))
        }

        ret <- c(ret, list(family = object$family))
        class(ret) <- "summary.inla"

        return(ret)
    }

    ## for G & J
    if (!is.null(object$joint.model)) {
        return(summary(object$joint.model, ...))
    }

    ret <- internal.summary.inla(object,
        digits = digits,
        include.lincomb = include.lincomb, ...
    )
    ss <- inla.getOption("short.summary")
    if (!is.null(ss) && ss) {
        new.ret <- list(
            fixed = ret$fixed, hyperpar = ret$hyperpar,
            dic = ret$dic, waic = ret$waic
        )
        class(new.ret) <- class(ret)
        ret <- new.ret
    }
    return(ret)
}


`print.summary.inla` <- function(x, digits = 3L, ...) {
    form <- strwrap(inla.formula2character(x$call))
    if (!is.null(x$call)) {
        cat("\nCall:\n")
        for (i in seq_along(form)) {
            cat("  ", form[i], "\n")
        }
    }

    if (inla.is.element("cpu.used", x)) {
        cat("Time used:\n", "  ", x$cpu.used, "\n")
    }

    if (inla.is.element("fixed", x)) {
        cat("Fixed effects:\n")
        print.default(x$fixed)
        cat("\n")
    }

    if (inla.is.element("lincomb.derived", x)) {
        cat("Linear combinations (derived):\n")
        if (is.character(x$lincomb.derived)) {
            cat("\t", x$lincomb.derived)
        } else {
            print.default(x$lincomb.derived)
        }
        cat("\n")
    }

    if (inla.is.element("lincomb", x)) {
        cat("Linear combinations:\n")
        if (is.character(x$lincomb)) {
            cat("\t", x$lincomb)
        } else {
            print.default(x$lincomb)
        }
        cat("\n")
    }

    if (inla.is.element("random.names", x)) {
        cat("Random effects:\n")
        cat("  Name\t ", "Model\n ")
        for (i in 1:length(x$random.names)) {
              cat("  ", paste0(inla.nameunfix(x$random.names[i]), " ", x$random.model[i], "\n"))
          }
        cat("\n")
    }

    if (inla.is.element("hyperpar", x)) {
        cat("Model hyperparameters:\n")
        print(format(x$hyperpar, digits = digits, nsmall = 2), quote = FALSE)
        cat("\n")
    }

    if (inla.is.element("dic", x)) {
          cat(paste("Deviance Information Criterion (DIC) ...............: ",
              format(x$dic$dic, digits = digits, nsmall = 2), "\n",
              "Deviance Information Criterion (DIC, saturated) ....: ",
              format(x$dic$dic.sat, digits = digits, nsmall = 2), "\n",
              "Effective number of parameters .....................: ",
              format(x$dic$p.eff, digits = digits, nsmall = 2), "\n\n",
              sep = ""
          ))
      }

    if (inla.is.element("waic", x)) {
          cat(paste("Watanabe-Akaike information criterion (WAIC) ...: ",
              format(x$waic$waic, digits = digits, nsmall = 2), "\n",
              "Effective number of parameters .................: ",
              format(x$waic$p.eff, digits = digits, nsmall = 2), "\n\n",
              sep = ""
          ))
      }

    if (inla.is.element("mlik", x)) {
          cat(paste("Marginal log-Likelihood: ", format(x$mlik[2], digits = digits, nsmall = 2), "\n"))
      }

    if (inla.is.element("cpo", x)) {
          cat("CPO and PIT are computed\n\n")
      }

    if (inla.is.element("po", x)) {
          cat("PO is computed\n\n")
      }

    if (inla.is.element("linear.predictor", x)) {
          cat(sep = "", 
              "Posterior summaries for the linear predictor and the fitted values are computed\n",
              "(Posterior marginals needs also 'control.compute=list(return.marginals.predictor=TRUE)')\n\n")
    }
}

## Export: inla.scale.model

## ! \name{scale.model}
## ! \alias{inla.scale.model}
## ! \alias{scale.model}
## !
## ! \title{Scale an intrinsic GMRF model}
## !
## ! \description{This function scales an intrinsic GMRF model so the geometric mean of the
## !              marginal variances is one}
## !
## ! \usage{
## !     inla.scale.model(Q, constr = NULL, eps = sqrt(.Machine$double.eps))
## ! }
## !
## ! \arguments{
## !   \item{Q}{A SPD matrix,  either as a (dense) matrix or \code{sparseMatrix}}
## !   \item{constr}{Linear constraints spanning the null-space of \code{Q};
## !                 see \code{?INLA::f} and argument \code{extraconstr}}
## !   \item{eps}{A small constant added to the diagonal of \code{Q} if \code{constr}}
## !  }
## ! \value{
## !   \code{inla.scale.model} returns a \code{sparseMatrix} of type \code{dgTMatrix}
## !   scaled so the geometric mean of the marginal variances (of the possible
## !   non-singular part of \code{Q}) is one,  for each connected component of the matrix.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## ! ## Q is singular
## ! data(Germany)
## ! g = system.file("demodata/germany.graph", package="INLA")
## ! Q = -inla.graph2matrix(g)
## ! diag(Q) = 0
## ! diag(Q) = -rowSums(Q)
## ! n = dim(Q)[1]
## ! Q.scaled = inla.scale.model(Q, constr = list(A = matrix(1, 1, n), e=0))
## ! print(diag(MASS::ginv(Q.scaled)))
## !
## ! ## Q is singular with 3 connected components
## ! g = inla.read.graph("6 1 2 2 3 2 2 1 3 3 2 1 2 4 1 5 5 1 4 6 0")
## ! print(paste("Number of connected components", g$cc$n))
## ! Q = -inla.graph2matrix(g)
## ! diag(Q) = 0
## ! diag(Q) = -rowSums(Q)
## ! n = dim(Q)[1]
## ! Q.scaled = inla.scale.model(Q, constr = list(A = matrix(1, 1, n), e=0))
## ! print(diag(MASS::ginv(Q.scaled)))
## !
## ! ## Q is non-singular with 3 connected components. no constraints needed
## ! diag(Q) = diag(Q) + 1
## ! Q.scaled = inla.scale.model(Q)
## ! print(diag(MASS::ginv(Q.scaled)))
## ! }

inla.scale.model.internal <- function(Q, constr = NULL, eps = sqrt(.Machine$double.eps)) {
    ## return also the scaled marginal variances

    marg.var <- rep(0, nrow(Q))
    Q <- inla.as.sparse(Q)
    g <- inla.read.graph(Q)
    for (k in seq_len(g$cc$n)) {
        i <- g$cc$nodes[[k]]
        n <- length(i)
        QQ <- Q[i, i, drop = FALSE]
        if (n == 1) {
            QQ[1, 1] <- 1
            marg.var[i] <- 1
        } else {
            cconstr <- constr
            if (!is.null(constr)) {
                ## the GMRFLib will automatically drop duplicated constraints; how convenient...
                cconstr$A <- constr$A[, i, drop = FALSE]
                eeps <- eps
            } else {
                eeps <- 0
            }
            res <- inla.qinv(QQ + Diagonal(n) * max(diag(QQ)) * eeps, constr = cconstr)
            fac <- exp(mean(log(diag(res))))
            QQ <- fac * QQ
            marg.var[i] <- diag(res) / fac
        }
        Q[i, i] <- QQ
    }
    return(list(Q = Q, var = marg.var))
}

inla.scale.model <- function(Q, constr = NULL, eps = sqrt(.Machine$double.eps)) {
    res <- inla.scale.model.internal(Q = Q, constr = constr, eps = eps)
    return(res$Q)
}

## Export: inla.setOption inla.getOption

## ! \name{inla.option}
## ! \alias{inla.option}
## ! \alias{inla.options}
## ! \alias{inla.setOption}
## ! \alias{inla.getOption}
## !
## ! \title{Set and get global options for INLA}
## !
## ! \description{Set and get global options for INLA}
## ! \usage{
## ! inla.setOption(...)
## ! inla.getOption(option)
## ! }
## !
## ! \arguments{
## !
## !   \item{...}{Option and value,  like \code{option=value} or \code{option, value}; see the Examples}
## !   \item{option}{The option to get. If \code{option = NULL} then
## !     \code{inla.getOption} then \code{inla.getOption} will return a named list of
## !     current values, otherwise, \code{option} must be one of
## !
## !     inla.call: The path to the inla-program.
## !
## !     inla.arg: Additional arguments to \code{inla.call}
## !
## !     fmesher.call: The path to the fmesher-program
## !
## !     fmesher.arg: Additional arguments to \code{fmesher.call}
## !
## !     num.threads: Character string with the number of threads to use as \code{A:B}
## !
## !     blas.num.threads: Number of threads to use for openblas and mklblas (see \code{inla} for details)
## !
## !     smtp: Sparse matrix library to use,  one of \code{band}, \code{taucs} (\code{default}) or \code{pardiso}
## !
## !     mkl: Use binaries buildt with Intel MKL?  (If possible)
## !
## !     vecLib: This option applies to Mac only. If TRUE and mkl=FALSE, link with vecLib BLAS and LAPACK libs (if available)
## !
## !     vecLibPath: This option applies to Mac only. Path to vecLib-libraries. If empty, use default.
## !
## !     pardiso.license: The full path to the PARDISO license file
## !
## !     keep: Keep temporary files?
## !
## !     working.directory: The name of the working directory.
## !
## !     silent: Run the inla-program in a silent mode?
## !
## !     debug : Run the inla-program in a debug mode?
## !
## !     cygwin : The home of the Cygwin installation (default "C:/cygwin") [Remote computing for Windows only]
## !
## !     ssh.auth.sock: The ssh bind-adress (value of $SSH_AUTH_SOCK int the
## !     Cygwin-shell). [Remote computing for Windows only]
## !
## !     show.warning.graph.file : Give a warning for using the obsolete argument
## !                               \code{graph.file} instead of \code{graph}
## !
## !     scale.model.default : The default value of argument \code{scale.model} which
## !                           optionally scale intrinisic models to have generalized
## !                           unit average variance
## !
## !     short.summary : Use a less verbose output for \code{summary}. Useful for Markdown
## !                     documents.
## !
## !     inla.timeout : The timeout limit, in whole seconds, for calls to the
## !       inla binary. Default is 0, meaning no timeout limit.  Set to a
## !       positive integer to terminate inla calls if they run to long.
## !       Fractional seconds are
## !       rounded up to the nearest integer. This feature is EXPERIMENTAL
## !       and might change at a later stage. 
## !
## !     fmesher.timeout : The timeout limit, in whole seconds, for calls to the
## !       fmesher binary. Default is 0, meaning no timeout limit.  Set to a
## !       positive integer to terminate fmesher calls that may enter infinite
## !       loops due to special geometry regularity. Fractional seconds are
## !       rounded up to the nearest integer.
## !
## !     inla.mode : Which mode to use in INLA? Default is \code{"classic"}. Other options are
## !     (USE AT YOUR OWN RISK, THIS IS WORK IN PROGRESS!!!) options are \code{"twostage"} and
## !     \code{"experimental"}.
## !   }
## ! }
## !
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## ! ## set number of threads
## ! inla.setOption("num.threads", "4:1")
## ! ## alternative format
## ! inla.setOption(num.threads="4:1")
## ! ## check it
## ! inla.getOption("num.threads")
## !}

`inla.getOption.default` <- function() {
    ## this function is not exported. it need to be separate, to avoid infinite recursion
    return(
        list(
            inla.arg = NULL,
            fmesher.arg = "",
            num.threads = paste0(parallel::detectCores(all.tests = TRUE, logical = FALSE), ":1"),
            blas.num.threads = 0L,
            smtp = "default",
            mkl = if (inla.os("linux") || inla.os("mac")) TRUE else FALSE,
            vecLib = FALSE, 
            vecLibPath = "", 
            pardiso.license = NULL,
            keep = FALSE,
            working.directory = NULL,
            silent = TRUE,
            debug = FALSE,
            cygwin = "C:/cygwin",
            cygwin.home = paste("/home/", inla.get.USER(), sep = ""),
            ssh.auth.sock = paste("/tmp/ssh-auth-sock-", inla.get.USER(), sep = ""),
            show.warning.graph.file = TRUE,
            scale.model.default = FALSE,
            short.summary = FALSE,
            inla.timeout = 0, 
            fmesher.timeout = 0,
            inla.mode = "classic"
        )
    )
}

`inla.getOption` <- function(
                             option = c(
                                 "inla.call",
                                 "inla.arg",
                                 "fmesher.call",
                                 "fmesher.arg",
                                 "num.threads",
                                 "blas.num.threads",
                                 "smtp",
                                 "mkl",
                                 "vecLib",
                                 "vecLibPath",
                                 "pardiso.license",
                                 "keep",
                                 "working.directory",
                                 "silent",
                                 "debug",
                                 "cygwin",
                                 "ssh.auth.sock",
                                 "cygwin.home",
                                 "show.warning.graph.file",
                                 "scale.model.default",
                                 "short.summary",
                                 "inla.timeout", 
                                 "fmesher.timeout",
                                 "inla.mode"
                             )) {
    ## we 'inla.call' and 'fmesher.call' separately to avoid infinite recursion
    default.opt <- inla.getOption.default()
    default.opt$inla.call <- inla.call.builtin()
    default.opt$fmesher.call <- inla.fmesher.call.builtin()

    ## with no argument, return a named list of current values
    if (missing(option)) {
        opt.names <- names(default.opt)
        option <- opt.names
    } else {
        opt.names <- NULL
    }

    envir <- inla.get.inlaEnv()
    option <- match.arg(option, several.ok = TRUE)
    if (exists("inla.options", envir = envir)) {
        opt <- get("inla.options", envir = envir)
    } else {
        opt <- list()
    }

    if (is.null(opt$inla.call)) {
        inla.call <- inla.call.builtin()
    } else if (inla.strcasecmp(opt$inla.call, "remote") ||
        inla.strcasecmp(opt$inla.call, "inla.remote")) {
        inla.call <- gsub("\\\\", "/", system.file("bin/remote/inla.remote", package = "INLA"))
    } else {
        inla.call <- opt$inla.call
    }

    if (is.null(opt$fmesher.call)) {
        fmesher.call <- inla.fmesher.call.builtin()
    } else {
        fmesher.call <- opt$fmesher.call
    }

    res <- c()
    for (i in 1:length(option)) {
        if (inla.is.element(option[i], opt)) {
            val <- list(inla.get.element(option[i], opt))
        } else {
            val <- list(inla.get.element(option[i], default.opt))
        }
        if (!is.null(opt.names)) {
            names(val) <- opt.names[i]
        }
        res <- c(res, val)
    }

    if (is.null(opt.names)) {
        res <- unlist(res)
    }

    return(res)
}

`inla.setOption` <- function(...) {
    ## supports formats:
    ## inla.setOption("keep", TRUE)
    ## and
    ## inla.setOption(keep=TRUE)
    ## and
    ## inla.setOption(keep=TRUE, num.threads=10)

    `inla.setOption.core` <- function(
                                      option = c(
                                          "inla.call",
                                          "inla.arg",
                                          "fmesher.call",
                                          "fmesher.arg",
                                          "num.threads",
                                          "blas.num.threads",
                                          "smtp",
                                          "mkl",
                                          "vecLib",
                                          "vecLibPath",
                                          "pardiso.license",
                                          "keep",
                                          "working.directory",
                                          "silent",
                                          "debug",
                                          "cygwin",
                                          "ssh.auth.sock",
                                          "cygwin.home",
                                          "show.warning.graph.file",
                                          "scale.model.default",
                                          "short.summary",
                                          "inla.timeout", 
                                          "fmesher.timeout",
                                          "inla.mode"
                                      ), value) {
        envir <- inla.get.inlaEnv()

        option <- match.arg(option, several.ok = FALSE)
        if (!exists("inla.options", envir = envir)) {
            assign("inla.options", list(), envir = envir)
        }
        if (is.character(value)) {
            eval(parse(text = paste("inla.options$", option, "=", shQuote(value), sep = "")),
                envir = envir
            )
        } else {
            eval(parse(text = paste("inla.options$", option, "=", inla.ifelse(is.null(value), "NULL", value), sep = "")),
                envir = envir
            )
        }
        return(invisible())
    }

    called <- list(...)
    len <- length(names(called))
    if (len > 0L) {
        for (i in 1L:len) {
            do.call(inla.setOption.core, args = list(names(called)[i], called[[i]]))
        }
    } else {
        inla.setOption.core(...)
    }

    ## add checks that nothing very wrong is set
    dummy <- match.arg(inla.getOption("inla.mode"), c("classic", "twostage", "experimental"), several.ok = FALSE)

    return(invisible())
}

context("test 'control.mode'")

test_that("Case 1", {
    set.seed(123)
    n = 100
    x = arima.sim(n, model=list(ar=0.9))
    x = x/sd(x)
    y = 1 + x + rnorm(n, sd=0.01)
    
    idx = 1:n
    formula = y ~ 1 + f(idx,  model="ar1")
    
    inla.setOption(inla.call='inla.work')
    r = inla(formula, data = data.frame(y, idx),
            control.inla = list(int.strategy = "eb"))
    rr = inla(formula, data = data.frame(y, idx),
            control.mode = list(result = r,  fixed=TRUE))
    expect_true(all(abs(r$summary.random[[1]] - rr$summary.random[[1]]) < 1e-8))
    expect_true(all(abs(r$summary.fixed - rr$summary.fixed) < 1e-8))
})

## Export: inla.pc.ralphaw inla.pc.dalphaw inla.pc.qalphaw inla.pc.palphaw

## ! \name{pc.alphaw}
## ! \alias{inla.pc.alphaw}
## ! \alias{pc.alphaw}
## ! \alias{pc.ralphaw}
## ! \alias{inla.pc.ralphaw}
## ! \alias{pc.dalphaw}
## ! \alias{inla.pc.dalphaw}
## ! \alias{pc.palphaw}
## ! \alias{inla.pc.palphaw}
## ! \alias{pc.qalphaw}
## ! \alias{inla.pc.qalphaw}
## !
## ! \title{Utility functions for the PC prior for the \code{alpha} parameter in the Weibull likelihood}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the \code{alpha} parameter
## !              in the Weibull likelihood}
## ! \usage{
## ! inla.pc.ralphaw(n, lambda = 5)
## ! inla.pc.dalphaw(alpha, lambda = 5, log = FALSE)
## ! inla.pc.qalphaw(p, lambda = 5)
## ! inla.pc.palphaw(q, lambda = 5)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter in the PC-prior}
## !   \item{alpha}{Vector of evaluation points, where \code{alpha>0}.}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## ! This gives the PC prior for the \code{alpha} parameter for the Weibull likelihood,
## ! where \code{alpha=1} is the base model.
## ! }
## !\value{%%
## !  \code{inla.pc.dalphaw} gives the density,
## !  \code{inla.pc.palphaw} gives the distribution function,
## !  \code{inla.pc.qalphaw} gives the quantile function, and
## !  \code{inla.pc.ralphaw} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.alphaw")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! x = inla.pc.ralphaw(100,  lambda = 5)
## ! d = inla.pc.dalphaw(x, lambda = 5)
## ! x = inla.pc.qalphaw(0.5, lambda = 5)
## ! inla.pc.palphaw(x, lambda = 5)
## ! }

inla.pc.alphaw.cache <- function() {
    ## return the cache for these functions

    dist <- function(lalpha) {
        alpha <- exp(lalpha)
        gam <- 0.5772156649
        kld <- (gamma((1.0 + alpha) / alpha) * alpha + alpha * lalpha - alpha * gam + gam - alpha) / alpha
        return(sqrt(2.0 * kld))
    }

    tag <- "cache.pc.alphaw"
    if (!exists(tag, envir = inla.get.inlaEnv())) {
        lalphas <- seq(-4, 5, by = 0.005)
        lalphas[which.min(abs(lalphas))] <- 0 ## yes, make it exactly 0
        lalphas.pos <- lalphas[which(lalphas >= 0)]
        lalphas.neg <- lalphas[which(lalphas <= 0)]
        dist.pos <- dist(lalphas.pos)
        dist.neg <- dist(lalphas.neg)
        assign(tag,
            list(
                pos = list(
                    x = lalphas.pos,
                    dist = splinefun(lalphas.pos, dist.pos),
                    idist = splinefun(dist.pos, lalphas.pos)
                ),
                neg = list(
                    x = lalphas.neg,
                    dist = splinefun(lalphas.neg, dist.neg),
                    idist = splinefun(dist.neg, lalphas.neg)
                )
            ),
            envir = inla.get.inlaEnv()
        )
    }
    return(get(tag, envir = inla.get.inlaEnv()))
}

inla.pc.ralphaw <- function(n, lambda = 5) {
    cache <- inla.pc.alphaw.cache()
    x <- numeric(n)
    ind <- sample(c(-1, 1), n, replace = TRUE)
    idx.pos <- which(ind > 0)
    idx.neg <- which(ind < 0)
    z <- rexp(n, rate = lambda)
    if (length(idx.pos) > 0) {
        x[idx.pos] <- cache$pos$idist(z[idx.pos])
    }
    if (length(idx.neg) > 0) {
        x[idx.neg] <- cache$neg$idist(z[idx.neg])
    }
    return(exp(x))
}

inla.pc.dalphaw <- function(alpha, lambda = 5, log = FALSE) {
    cache <- inla.pc.alphaw.cache()
    d <- numeric(length(alpha))
    lalpha <- log(alpha)
    idx.pos <- which(lalpha >= 0)
    idx.neg <- which(lalpha < 0)
    if (length(idx.pos) > 0) {
        la <- lalpha[idx.pos]
        dist <- cache$pos$dist(la)
        deriv <- cache$pos$dist(la, deriv = 1)
        d[idx.pos] <- log(0.5) + log(lambda) - lambda * dist + log(abs(deriv)) - la
    }
    if (length(idx.neg) > 0) {
        la <- lalpha[idx.neg]
        dist <- cache$neg$dist(la)
        deriv <- cache$neg$dist(la, deriv = 1)
        d[idx.neg] <- log(0.5) + log(lambda) - lambda * dist + log(abs(deriv)) - la
    }
    return(if (log) d else exp(d))
}

inla.pc.qalphaw <- function(p, lambda = 5) {
    cache <- inla.pc.alphaw.cache()
    n <- length(p)
    q <- numeric(n)
    idx.pos <- which(p >= 0.5)
    idx.neg <- which(p < 0.5)
    if (length(idx.pos) > 0) {
        pp <- 2.0 * (p[idx.pos] - 0.5)
        qe <- qexp(pp, rate = lambda)
        q[idx.pos] <- cache$pos$idist(qe)
    }
    if (length(idx.neg) > 0) {
        pp <- 2.0 * (0.5 - p[idx.neg])
        qe <- qexp(pp, rate = lambda)
        q[idx.neg] <- cache$neg$idist(qe)
    }
    return(exp(q))
}

inla.pc.palphaw <- function(q, lambda = 5) {
    cache <- inla.pc.alphaw.cache()
    n <- length(q)
    p <- numeric(n)
    idx.pos <- which(q >= 1.0)
    idx.neg <- which(q < 1.0)
    if (length(idx.pos) > 0) {
        qq <- cache$pos$dist(log(q[idx.pos]))
        pp <- pexp(qq, rate = lambda)
        p[idx.pos] <- 0.5 * (1 + pp)
    }
    if (length(idx.neg) > 0) {
        qq <- cache$neg$dist(log(q[idx.neg]))
        pp <- pexp(qq, rate = lambda)
        p[idx.neg] <- 0.5 * (1 - pp)
    }
    return(p)
}

inla.pc.alphaw.test <- function(lambda = 5) {
    ## this is just an internal test, and is not exported
    n <- 10^6
    x <- inla.pc.ralphaw(n, lambda = lambda)
    x <- x[x < quantile(x, prob = 0.999)]
    alpha <- seq(min(x), max(x), by = 0.001)
    hist(x, n = 300, prob = TRUE)
    lines(alpha, inla.pc.dalphaw(alpha, lambda), lwd = 3, col = "blue")
    print(sum(alpha * inla.pc.dalphaw(alpha, lambda) * diff(alpha)[1]))
    p <- seq(0.1, 0.9, by = 0.05)
    print(cbind(p = p, q.emp = as.numeric(quantile(x, prob = p)), q.true = inla.pc.qalphaw(p, lambda)))
    q <- as.numeric(quantile(x, prob = p))
    cdf <- ecdf(x)
    print(cbind(q = q, p.emp = cdf(q), p.true = inla.pc.palphaw(q, lambda)))
}

## inla.pc.alphaw.test()

#' @include spde.common.R
NULL

#' Old SPDE model objects for INLA
#'
#' Create an `inla.spde1` model object.
#'
#' Note: This is an old spde object format retained for backwards
#' compatibility.  Please use [inla.spde2()] models for new code.
#'
#' This method constructs an object for SPDE models.  Currently implemented:
#'
#' `model="matern"`
#'
#' \deqn{(\kappa^2(u)-\Delta)^{\alpha/2}(\tau(u) }{(kappa^2(u)-Delta)^(alpha/2)
#' (tau(u) x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2(u)-Delta)^(alpha/2) (tau(u)
#' x(u)) = W(u)}
#'
#' `param`: \itemize{ \item`alpha` = 1 or 2 \item`basis.T` =
#' Matrix of basis functions for \eqn{\log\tau(u)}{log tau(u)}
#' \item`basis.K` = Matrix of basis functions for
#' \eqn{\log\kappa^2(u)}{log kappa^2(u)} }
#'
#' `model="imatern"`
#'
#' \deqn{(-\Delta)^{\alpha/2}(\tau(u) }{(-Delta)^(alpha/2) (tau(u) x(u)) =
#' W(u)}\deqn{ x(u))=W(u)}{(-Delta)^(alpha/2) (tau(u) x(u)) = W(u)}
#'
#' `param`: \itemize{ \item`alpha` = 1 or 2 \item`basis.T` =
#' Matrix of basis functions for \eqn{\log\tau(u)}{log tau(u)} }
#'
#' @aliases inla.spde1.create inla.spde1.matern
#' inla.spde1.imatern inla.spde1.matern.osc inla.spde1
#' @param mesh The mesh to build the model on, as an [inla.mesh()]
#' object.
#' @param model The name of the model.
#' @param param Model specific parameters.
#' @param ...  Additional parameters passed on to other methods.
#' @return An `inla.spde1` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde2.matern()], [inla.mesh.2d()],
#' [inla.mesh.basis()]
#' @examples
#'
#' n <- 100
#' field.fcn <- function(loc) (10 * cos(2 * pi * 2 * (loc[, 1] + loc[, 2])))
#' loc <- matrix(runif(n * 2), n, 2)
#' ## One field, 2 observations per location
#' idx.y <- rep(1:n, 2)
#' y <- field.fcn(loc[idx.y, ]) + rnorm(length(idx.y))
#'
#' mesh <- inla.mesh.create(loc, refine = list(max.edge = 0.05))
#' spde <- inla.spde1.create(mesh, model = "matern")
#' data <- list(y = y, field = mesh$idx$loc[idx.y])
#' formula <- y ~ -1 + f(field, model = spde)
#' result <- inla(formula, data = data, family = "normal")
#'
#' ## Plot the mesh structure:
#' plot(mesh)
#' \donttest{
#' if (require(rgl)) {
#'     ## Plot the posterior mean:
#'     plot(mesh,
#'         rgl = TRUE,
#'         result$summary.random$field[, "mean"],
#'         color.palette = colorRampPalette(c("blue", "green", "red"))
#'     )
#'     ## Plot residual field:
#'     plot(mesh,
#'         rgl = TRUE,
#'         result$summary.random$field[, "mean"] - field.fcn(mesh$loc),
#'         color.palette = colorRampPalette(c("blue", "green", "red"))
#'     )
#' }
#' }
#'
#' @export inla.spde1.create
inla.spde1.create <-
    function(mesh,
             model = c("matern", "imatern", "matern.osc"),
             param = NULL,
             ...) {
        inla.require.inherits(mesh, "inla.mesh", "'mesh'")

        model <- match.arg(model)
        if (is.null(param)) {
              param <- list()
          }

        spde <- (list(
            model = model,
            mesh = mesh,
            internal = list(),
            f = (list(
                model = "spde",
                n = nrow(mesh$loc)
            ))
        ))
        class(spde) <- c("inla.spde1", "inla.spde", "inla.model.class")

        if (identical(model, "matern") ||
            identical(model, "imatern") ||
            identical(model, "matern.osc")
        ) {
            if (is.null(param)) {
                  param <- list()
              }
            if (is.null(param$alpha)) {
                  param$alpha <- 2
              }
            if (is.null(param$basis.T)) {
                  param$basis.T <- matrix(1, mesh$n, 1)
              } else if (!is.matrix(param$basis.T)) {
                len <- length(as.vector(param$basis.T))
                if (len == 1L) {
                      param$basis.K <- matrix(as.vector(param$basis.T), mesh$n, 1)
                  } else {
                      param$basis.T <- as.matrix(param$basis.T)
                  }
            }
            if (nrow(param$basis.T) != mesh$n) {
                  stop(paste("'basis.T' has ", nrow(param$basis.T),
                      " rows; expected ", mesh$n, ".",
                      sep = ""
                  ))
              }
            if (identical(model, "matern") ||
                identical(model, "matern.osc")
            ) {
                if (is.null(param$basis.K)) {
                    param$basis.K <- matrix(1, mesh$n, 1)
                } else if (!is.matrix(param$basis.K)) {
                    len <- length(as.vector(param$basis.K))
                    if (len == 1L) {
                          param$basis.K <- matrix(as.vector(param$basis.K), mesh$n, 1)
                      } else {
                          param$basis.K <- as.matrix(param$basis.K)
                      }
                }
            } else {
                param$basis.K <- matrix(0, mesh$n, 1)
            }
            if (nrow(param$basis.K) != mesh$n) {
                  stop(paste("'basis.K' has ", nrow(param$basis.K),
                      " rows; expected ", mesh$n, ".",
                      sep = ""
                  ))
              }
            spde$internal <- (c(
                spde$internal,
                list(
                    alpha = param$alpha,
                    basis.T = param$basis.T,
                    basis.K = param$basis.K
                )
            ))

            mesh.range <- (max(c(
                diff(range(mesh$loc[, 1])),
                diff(range(mesh$loc[, 2])),
                diff(range(mesh$loc[, 3]))
            )))

            spde$internal <- (c(
                spde$internal,
                inla.fmesher.smorg(mesh$loc,
                    mesh$graph$tv,
                    fem = 2,
                    output = list("c0", "g1", "g2")
                )
            ))

            if (param$alpha == 2) {
                kappa0 <- sqrt(8) / (mesh.range * 0.2)
                tau0 <- 1 / sqrt(4 * pi * kappa0^2) / 1.0
            } else if (param$alpha == 1) {
                spde$internal$g2 <- spde$internal$g1
                spde$internal$g1 <- spde$internal$g1 * 0.0

                kappa0 <- sqrt(sqrt(8) / (mesh.range * 0.2))
                tau0 <- 1 / sqrt(4 * pi) / 1.0
            }

            if (identical(model, "matern")) {
                spde$f$hyper.default <- (list(
                    theta1 = (list(
                        initial = log(tau0),
                        param = c(log(tau0), 0.1)
                    )),
                    theta2 = (list(
                        initial = log(kappa0^2),
                        param = c(log(kappa0^2), 0.1)
                    )),
                    theta3 = (list(
                        initial = (log(tau0) + log(kappa0^2)) / 2,
                        param = c((log(tau0) + log(kappa0^2)) / 2, 0.1)
                    ))
                ))
            } else if (identical(model, "imatern")) {
                spde$f$hyper.default <- (list(
                    theta1 = (list(
                        initial = log(tau0),
                        param = c(log(tau0), 0.1)
                    )),
                    theta2 = (list(
                        initial = -20,
                        fixed = TRUE
                    )),
                    theta3 = (list(
                        initial = log(tau0),
                        param = c(log(tau0), 0.1)
                    ))
                ))
            } else if (identical(model, "matern.osc")) {
                spde$f$hyper.default <- (list(
                    theta1 = (list(
                        initial = log(tau0),
                        param = c(log(tau0), 0.1)
                    )),
                    theta2 = (list(
                        initial = log(kappa0^2),
                        param = c(log(kappa0^2), 0.1)
                    )),
                    theta3 = (list(
                        initial = (log(tau0) + log(kappa0^2)) / 2,
                        param = c((log(tau0) + log(kappa0^2)) / 2, 0.1)
                    )),
                    theta4 = (list(
                        fixed = FALSE,
                        initial = 0,
                        param = c(0, 0.01)
                    ))
                ))
            }
        } else {
            stop(paste("Model '", model, "' unknown or not implemented.", sep = ""))
        }

        return(invisible(spde))
    }








inla.spde1.query <- function(spde, ...) {
    inla.require.inherits(spde, "inla.spde1", "'spde'")

    not.known <- function(spde, queryname) {
        stop(paste("Query '", queryname,
            "' unknown.",
            sep = ""
        ))
    }
    not.implemented <- function(spde, queryname) {
        stop(paste("Query '", queryname,
            "' not implemented for inla.spde1 model '",
            spde$model, "'.",
            sep = ""
        ))
    }
    param.to.fcn <-
        function(basis, theta, values, n, name) {
            if (!is.null(values)) {
                if ((is.vector(values) &&
                    (length(values) == n)) ||
                    (is.matrix(values) &&
                        (nrow(values) == n))) {
                    fcn <- as.vector(values)
                } else {
                    fcn <- as.vector(values)
                }
                if (length(fcn) == 1L) {
                      fcn <- rep(fcn, n)
                  } else if (length(fcn) != n) {
                      stop(paste("Length of '", name, "' is ", length(fcn),
                          ", should be ", n, ".",
                          sep = ""
                      ))
                  }
            } else {
                fcn <- exp(basis %*% theta)
            }
            return(fcn)
        }

    result <- list()
    queries <- inla.parse.queries(...)
    if (length(queries) == 0L) {
          return(result)
      }

    for (query.idx in 1:length(queries)) {
        query <- names(queries)[query.idx]
        param <- queries[[query.idx]]
        answer <- NULL
        query <- (match.arg(query, c(
            "precision",
            "sample"
        )))
        if (identical(query, "precision")) {
            if (identical(spde$model, "matern")) {
                tau <- (param.to.fcn(
                    spde$internal$basis.T,
                    param$theta.T, param$tau,
                    spde$mesh$n, "tau"
                ))
                kappa2 <- (param.to.fcn(
                    spde$internal$basis.K,
                    param$theta.K, param$kappa2,
                    spde$mesh$n, "kappa2"
                ))
                dT <- Diagonal(spde$mesh$n, tau)
                dK2 <- Diagonal(spde$mesh$n, kappa2)
                tmp <- dK2 %*% spde$internal$g1
                answer <- (dT %*% (dK2 %*% spde$internal$c0 %*% dK2 +
                    tmp + t(tmp) +
                    spde$internal$g2) %*% dT)
            } else if (identical(spde$model, "matern.osc")) {
                tau <- (param.to.fcn(
                    spde$internal$basis.T,
                    param$theta.T, param$tau,
                    spde$mesh$n, "tau"
                ))
                kappa2 <- (param.to.fcn(
                    spde$internal$basis.K,
                    param$theta.K, param$kappa2,
                    spde$mesh$n, "kappa2"
                ))
                osc <- param$osc
                dT <- Diagonal(spde$mesh$n, tau)
                dK2 <- Diagonal(spde$mesh$n, kappa2)
                tmp <- dK2 %*% spde$internal$g1
                answer <- (dT %*% (dK2 %*% spde$internal$c0 %*% dK2 +
                    cos(pi * osc) * (tmp + t(tmp)) +
                    spde$internal$g2) %*% dT)
            } else if (identical(spde$model, "imatern")) {
                tau <- (param.to.fcn(
                    spde$internal$basis.T,
                    param$theta.T, param$tau,
                    spde$mesh$n, "tau"
                ))
                dT <- Diagonal(spde$mesh$n, tau)
                answer <- (dT %*% ((1e-10) * spde$internal$c0 +
                    spde$internal$g2) %*% dT)
            } else {
                not.implemented(spde, query)
            }
        } else if (identical(query, "sample")) {
            Q <- inla.spde1.query(spde, precision = param)$precision
            finn <- (inla.finn(Q, seed = (inla.ifelse(
                is.null(param$seed),
                0L,
                param$seed
            ))))
            answer <- finn$sample
        } else if (!identical(query, "")) {
            not.known(spde, query)
        }
        ## Expand the result list:
        result[query.idx] <- list(NULL)
        names(result)[query.idx] <- query
        ## Set the answer:
        if (!is.null(answer)) {
              result[[query.idx]] <- answer
          }
    }

    return(result)
}




#' @export
#' @rdname inla.spde.result
inla.spde1.result <- function(inla, name, spde, do.transform = TRUE, ...) {
    warning("'inla.spde1.result' is not fully implemented yet.")
    inla.require.inherits(inla, "inla", "'inla'")
    inla.require.inherits(spde, "inla.spde1", "'spde'")
    if (!spde$f$model == "spde") {
        stop("'inla.spde1.result' only supports internal inla models 'spde'")
    }

    result <- list()
    ## Setup rownames for UserFunction1
    UF.names <-
        c(
            rownames(rep(list(NULL), spde$mesh$n),
                do.NULL = FALSE, prefix = "tau."
            ),
            rownames(rep(list(NULL), spde$mesh$n),
                do.NULL = FALSE, prefix = "kappa2."
            )
        )
    rownames(inla$summary.random[["UserFunction1"]]) <- UF.names

    if (!is.null(inla$marginals.random[["UserFunction1"]])) {
        names(inla$marginals.random[["UserFunction1"]]) <-
            UF.names
    }

    ## Values
    result$summary.values <- inla$summary.random[[name]]

    ## Marginals for values
    if (!is.null(inla$marginals.random[[name]])) {
        result$marginals.values <- inla$marginals.random[[name]]
    }

    ## Theta
    result$summary.theta.tau <-
        inla.extract.el(
            inla$summary.hyperpar,
            paste("^T\\.[^ ]+ for ", name, "-basisT$",
                sep = ""
            )
        )
    result$summary.theta.kappa2 <-
        inla.extract.el(
            inla$summary.hyperpar,
            paste("^K\\.[^ ]+ for ", name, "-basisK$",
                sep = ""
            )
        )
    rownames(result$summary.theta.tau) <-
        rownames(rep(list(NULL), nrow(result$summary.theta.tau)),
            do.NULL = FALSE, prefix = "theta."
        )
    rownames(result$summary.theta.kappa2) <-
        rownames(rep(list(NULL), nrow(result$summary.theta.kappa2)),
            do.NULL = FALSE, prefix = "theta."
        )

    ## Marginals for theta
    if (!is.null(inla$marginals.hyperpar)) {
        result$marginals.theta.tau <-
            inla.extract.el(
                inla$marginals.hyperpar,
                paste("^T\\.[^ ]+ for ", name, "-basisT$",
                    sep = ""
                )
            )
        result$marginals.theta.kappa2 <-
            inla.extract.el(
                inla$marginals.hyperpar,
                paste("^K\\.[^ ]+ for ", name, "-basisK$",
                    sep = ""
                )
            )
        names(result$marginals.theta.tau) <-
            rownames(rep(list(NULL), length(result$marginals.theta.tau)),
                do.NULL = FALSE, prefix = "theta."
            )
        names(result$marginals.theta.kappa2) <-
            rownames(rep(list(NULL), length(result$marginals.theta.kappa2)),
                do.NULL = FALSE, prefix = "theta."
            )
    }

    ## log-tau/kappa2
    result$summary.log.tau <-
        inla.extract.el(
            inla$summary.random[["UserFunction1"]],
            paste("^tau\\.[^ ]+$",
                sep = ""
            )
        )
    result$summary.log.kappa2 <-
        inla.extract.el(
            inla$summary.random[["UserFunction1"]],
            paste("^kappa2\\.[^ ]+$",
                sep = ""
            )
        )

    ## Approximate log-range:
    forward.names <-
        c("mean", "0.025quant", "0.5quant", "0.975quant")
    reverse.names <-
        c("mean", "0.975quant", "0.5quant", "0.025quant")
    result$summary.log.range.nominal <- result$summary.log.kappa2
    result$summary.log.range.nominal[, forward.names] <-
        (log(sqrt(8)) -
            result$summary.log.kappa2[, reverse.names] / 2)
    result$summary.log.range.nominal[, "sd"] <-
        result$summary.log.kappa2[, "sd"] / 4

    ## Approximate log-variance:
    result$summary.log.variance.nominal <- result$summary.log.kappa2
    result$summary.log.variance.nominal[, forward.names] <-
        -(log(4 * pi) +
            result$summary.log.tau[, reverse.names] * 2 +
            result$summary.log.kappa2[, reverse.names])
    result$summary.log.range.nominal[, "sd"] <-
        sqrt(result$summary.log.tau[, "sd"]^2 * 4 +
            result$summary.log.kappa2[, "sd"]^2 +
            2 * 2 * result$summary.log.tau[, "sd"] *
                result$summary.log.kappa2[, "sd"])

    ## Marginals for log-tau/kappa2
    if (!is.null(inla$marginals.random[["UserFunction1"]])) {
        result$marginals.log.tau <-
            inla.extract.el(
                inla$marginals.random[["UserFunction1"]],
                paste("^tau\\.[^ ]+$",
                    sep = ""
                )
            )
        result$marginals.log.kappa2 <-
            inla.extract.el(
                inla$marginals.random[["UserFunction1"]],
                paste("^kappa2\\.[^ ]+$",
                    sep = ""
                )
            )

        if (do.transform) {
            result$marginals.tau <-
                lapply(
                    result$marginals.log.tau[1],
                    function(x) inla.tmarginal(function(y) exp(y), x)
                )
            result$marginals.kappa <-
                lapply(
                    result$marginals.log.kappa2[1],
                    function(x) inla.tmarginal(function(y) exp(y / 2), x)
                )
            ## result$marginals.variance.nominal =
            ## lapply(result$marginals.log.variance.nominal,
            ## function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.range.nominal <-
                lapply(
                    result$marginals.log.kappa2[1],
                    function(x) inla.tmarginal(function(y) sqrt(8) * exp(-y / 2), x)
                )
        }
    }

    return(result)
}

#' @export
#' @rdname inla.spde.precision
inla.spde1.precision <- function(spde, ...) {
    inla.require.inherits(spde, "inla.spde1", "'spde'")

    return(inla.spde1.query(spde, precision = list(...))$precision)
}


#' @export
#' @rdname inla.spde.models
inla.spde1.models <- function() {
    return(c("matern", "imatern", "matern.osc"))
}

#' @export
#' @rdname inla.spde1.create
inla.spde1.matern <- function(mesh, ...) {
    return(inla.spde1.create(mesh, model = "matern", ...))
}
#' @export
#' @rdname inla.spde1.create
inla.spde1.imatern <- function(mesh, ...) {
    return(inla.spde1.create(mesh, model = "imatern", ...))
}
#' @export
#' @rdname inla.spde1.create
inla.spde1.matern.osc <- function(mesh, ...) {
    return(inla.spde1.create(mesh, model = "matern.osc", ...))
}

## spde.common-connections:
#' @export
#' @method inla.spde.precision inla.spde1
#' @rdname inla.spde.precision
inla.spde.precision.inla.spde1 <- inla.spde1.precision
#' @export
#' @method inla.spde.result inla.spde1
#' @rdname inla.spde.result
inla.spde.result.inla.spde1 <- inla.spde1.result

## Export: inla.upgrade inla.update

## !\name{inla.upgrade}
## !\alias{inla.upgrade}
## !\alias{inla.update}
## !
## !\title{Upgrade the INLA-package}
## !
## !\description{Functions to upgrade the \code{INLA}-package to the
## !  current version.}
## !
## !\usage{
## !inla.upgrade(lib = NULL, testing=FALSE, ask = TRUE)
## !inla.update(lib = NULL, testing=FALSE, ask = TRUE)
## !}
## !
## !\arguments{
## !  \item{lib}{Location to install the library.}
## !  \item{testing}{If \code{TRUE}, then look for a test-version if the
## !    INLA-package.}
## !  \item{ask}{same argument as in \code{update.packages}}
## !}
## !\value{\code{inla.upgrade} will update the INLA package to the current version,
## ! and \code{inla.update} do the same
## !  for backward compatibility. This function is simple wrapper for
## !  \code{update.packages} using the INLA repository.}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{\code{update.packages}}

### The upgrade utility

`inla.update` <- function(lib = NULL, testing = FALSE, ask = TRUE) {
    inla.upgrade(lib = lib, testing = testing, ask = ask)
}

`inla.upgrade` <- function(lib = NULL, testing = FALSE, ask = TRUE) {

    repo <- c(CRAN = "https://cran.rstudio.com",
              INLA = paste0("https://inla.r-inla-download.org/R/",
                            if (testing) "testing" else "stable"))

    ## default timeout (60 is sometimes to low
    min.timeout <- 300
    opt <- options()
    if (is.null(opt$timeout) || opt$timeout < min.timeout) {
        options(timeout = min.timeout)
    } else {
        min.timeout <- 0
    }

    if (require("INLA", quietly = TRUE, lib.loc = lib, character.only = TRUE, warn.conflicts = FALSE)) {
        suppressWarnings(new.pack <- any(old.packages(repos = repo)[, 1] == "INLA"))
        if (new.pack) {
            if (.Platform$OS.type == "windows") {
                cat(
                    sep = "",
                    "\n *** Windows locks the INLA-package's DLL when its loaded, see",
                    "\n ***     https://cran.r-project.org/bin/windows/base/rw-FAQ.html",
                    "\n *** Section 4.8,  so you cannot update a package that is in use.",
                    "\n *** We recommend to remove the INLA-package and then reinstall, like",
                    "\n         remove.packages(\"INLA\")"
                )
                cat(paste0("\n         install.packages(\"INLA\", repos=\"", repo["INLA"], "\")"))
                cat("\n *** and then restart R.", "\n")
            } else {
                suppressWarnings(update.packages(repos = repo, oldPkgs = "INLA", ask = ask))
                cat("\n *** If the INLA-package was updated, you need to restart R and load it again.\n\n")
            }
        } else {
            cat("\n *** You already have the latest version.\n\n")
        }
    } else {
        install.packages(pkgs = "INLA", lib = lib, repos = repo, dependencies = TRUE)
        library("INLA")
    }

    if (min.timeout) {
        options(opt)
    }

    return(invisible())
}

## Export: inla.pc.rgammacount inla.pc.dgammacount inla.pc.qgammacount inla.pc.pgammacount

## ! \name{pc.gammacount}
## ! \alias{inla.pc.gammacount}
## ! \alias{pc.gammacount}
## ! \alias{pc.rgammacount}
## ! \alias{inla.pc.rgammacount}
## ! \alias{pc.dgammacount}
## ! \alias{inla.pc.dgammacount}
## ! \alias{pc.pgammacount}
## ! \alias{inla.pc.pgammacount}
## ! \alias{pc.qgammacount}
## ! \alias{inla.pc.qgammacount}
## !
## ! \title{Utility functions for the PC prior for the \code{gammacount} likelihood}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the \code{gammacount} likelihood}
## ! \usage{
## ! inla.pc.rgammacount(n, lambda = 1)
## ! inla.pc.dgammacount(x, lambda = 1, log = FALSE)
## ! inla.pc.qgammacount(p, lambda = 1)
## ! inla.pc.pgammacount(q, lambda = 1)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter (see Details)}
## !   \item{x}{Evaluation points}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## ! This gives the PC prior for the \code{gammacount} likelihood,  which is the PC prior for
## ! \code{a} in \code{Gamma(a, 1)} where \code{Gamma(1, 1)} is the base model.
## ! }
## !\value{%%
## !  \code{inla.pc.dgammacount} gives the density,
## !  \code{inla.pc.pgammacount} gives the distribution function,
## !  \code{inla.pc.qgammacount} gives the quantile function, and
## !  \code{inla.pc.rgammacount} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.gammacount")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! x = inla.pc.rgammacount(100,  lambda = 1)
## ! d = inla.pc.dgammacount(x, lambda = 1)
## ! x = inla.pc.qgammacount(0.5, lambda = 1)
## ! inla.pc.pgammacount(x, lambda = 1)
## ! }

## I think this could have be done better than using the generic routines, by tabulating the d()
## function.

inla.pc.gammacount.intern <- function(lambda = 1) {
    ## return a marginal-object in the log(x)-scale
    f <- if (lambda >= 1.0) 1.0 else 0.25
    log.x <- (seq(-10, 10, len = 4096)) / lambda^f
    idx <- which(log.x == 0.0)
    if (length(idx) > 0) {
        log.x <- log.x[-idx]
    }
    x <- exp(log.x)
    marg <- list(x = log.x, y = inla.pc.dgammacount(x, lambda = lambda, log = FALSE) * x)
    return(marg)
}

inla.pc.rgammacount <- function(n, lambda = 1) {
    log.x <- inla.rmarginal(n, inla.pc.gammacount.intern(lambda = lambda))
    return(exp(log.x))
}

inla.pc.dgammacount <- function(x, lambda = 1, log = FALSE) {
    ## > d := a -> (-2*ln(GAMMA(a)) + 2*(a - 1)*Psi(a))^(1/2);
    ## > R(log(lambda) -lambda*d(x) + log(abs(diff(d(x),x))), optimize);

    t1 <- log(lambda)
    t3 <- lgamma(x)
    t4 <- x - 1
    t5 <- digamma(x)
    t8 <- sqrt(2) * sqrt(t5 * t4 - t3)
    t12 <- trigamma(x)
    t14 <- abs(t12 * t4 / t8)
    t15 <- log(t14)
    t16 <- -t8 * lambda + t1 + t15
    ld <- t16 - log(2.0)

    return(if (log) ld else exp(ld))
}

inla.pc.qgammacount <- function(p, lambda = 1) {
    log.x <- inla.pmarginal(p, inla.pc.gammacount.intern(lambda = lambda))
    return(exp(log.x))
}

inla.pc.pgammacount <- function(q, lambda = 1) {
    p <- inla.qmarginal(log(q), inla.pc.gammacount.intern(lambda = lambda))
    return(p)
}

## Export: inla.surv is.inla.surv as.inla.surv
## Export: plot!inla.surv print!inla.surv

## !\name{inla.surv}
## !\alias{inla.surv}
## !\alias{is.inla.surv}
## !\alias{as.inla.surv}
## !\alias{plot.inla.surv}
## !\alias{print.inla.surv}
## !
## !\title{
## !Create a Survival Object for INLA
## !}
## !
## !\description{
## !Create a survival object, to be used as a response variable in a
## !model  formula for the \code{\link{inla}} function for survival models.
## !}
## !\usage{
## !inla.surv(time, event, time2, truncation, subject = NULL, .special = NULL)
## !\method{plot}{inla.surv}(x, y, ...)
## !\method{print}{inla.surv}(x, ...)
## !is.inla.surv(object)
## !as.inla.surv(object, ...)
## !}
## !
## !\arguments{
## !  \item{time}{For right censored data, this is the follow up time.  For
## !    interval data, this is the starting time for the interval. For in-interval event, this is
## !    the observed time (in the interval) for the event. For left censored data, this the
## !    censoring time. }
## !  \item{event}{The status indicator, 1=observed event, 0=right censored
## !    event, 2=left censored event, 3=interval censored event,
## !    and 4=observed event in an interval (left, right).}
## !  \item{time2}{Ending time for the interval censored data or an in-interval event.}
## !  \item{truncation}{Left truncation. If missing it is assumed to be 0. The lower limit for event=4.}
## !  \item{subject}{Patient number in multiple event data, not needed otherwise. }
## !  \item{.special}{An internal object, not for public use}
## !  \item{object}{Any \code{R}-object}
## !  \item{x}{Object to plot or print}
## !  \item{y}{Object to plot (not in use)}
## !  \item{...}{Additional argument}
## !  }
## !
## !\value{An object of class \code{inla.surv}.  There are methods for \code{print},
## !  \code{plot} for \code{inla.surv} objects.
## !
## !     \code{is.inla.surv} returns \code{TRUE} if \code{object}
## !     inherits from class \code{inla.surv}, otherwise \code{FALSE}.
## !
## !     \code{as.inla.surv} returns an object of class \code{inla.surv}
## !}
## !
## !\author{
## !Sara Martino, Rupali Akerkar and Haavard Rue
## !}
## !
## !\seealso{
## !\code{\link{inla}}
## !}
## !
## !\examples{
## !  ## First example
## !  trt = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
## !          0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
## !          1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)
## !  time = c(17,42,44,48,60,72,74,95,103, 108, 122, 144, 167, 170, 183, 185,
## !           193, 195, 197, 208, 234, 235, 254, 307, 315, 401, 445, 464, 484,  528, 542, 567,
## !           577, 580, 795, 855, 1174, 1214, 1232, 1366, 1455, 1585, 1622, 1626, 1736, 1,63,
## !           105, 125, 182, 216, 250, 262, 301, 301, 342, 354, 356, 358, 380, 383, 383, 388,
## !           394, 408, 460, 489, 499, 523, 524, 535, 562, 569, 675, 676, 748, 778, 786, 797,
## !           955, 968, 977, 1245, 1271, 1420, 1460, 1516, 1551, 1690, 1694)
## !  event = c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
## !            1,1,1,1,0,1,0,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
## !            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0,1)
## !  y = inla.surv(time, event)
## !
## !  ## Second example
## !  time = c(182,182,63,68,182,152,182,130,134,145,152,182,98,152,182,88,95,105,130,137,167,182,
## !           152,182,81,182,71,84,126,134,152,182)
## !  event = c(1,0,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0,1,0,1,0,1,1,1,1,1,0)
## !  subject = c(1,2,3,3,3,4,4,5,5,5,5,5,6,6,6,7,7,7,7,7,7,7,8,8,9,9,10,10,10,10,10,10)
## !  y = inla.surv(time, event, subject=subject)
## !}
## !
## !\keyword{Survival models}

`inla.surv` <- function(time, event, time2, truncation, subject = NULL, .special = NULL) 
{
    ret <- NULL
    if (is.null(subject)) {
        ret <- inla.surv.1(time, event, time2, truncation, .special = .special)
    } else {
        if (!missing(time2)) {
            stop("Argument 'time2' is not allowed when 'subject' is used")
        }
        if (!missing(truncation)) {
            stop("Argument 'truncation' is not allowed when 'subject' is used")
        }
        ret <- inla.surv.2(time, event, subject, .special = .special)
    }
    return(ret)
}

`plot.inla.surv` <- function(x, y, ...)
{
    ## argument y is not used
    if (is.null(x$subject)) {
        inla.plot.inla.surv.1(x, ...)
    } else {
        inla.plot.inla.surv.2(x, ...)
    }
}

`print.inla.surv` <- function(x, ...)
{
    if (is.null(x$subject)) {
        inla.print.inla.surv.1(x, ...)
    } else {
        inla.print.inla.surv.2(x, ...)
    }
}

`as.inla.surv` <- function(object, ...)
{
    ## the '...' are not used.
    if (is.null(object$subject)) {
        inla.as.inla.surv.1(object)
    } else {
        inla.as.inla.surv.2(object)
    }
}

`inla.surv.1` <- function(time, event, time2, truncation, .special = NULL)
{
    ## check that time is present
    if (missing(time)) {
        stop("Must have a 'time' argument")
    }
    if (!is.numeric(time)) {
        stop("'Time' variable is not numeric")
    }
    nn <- length(time)
    if (any(is.na(time))) {
        ## fixup the rest of the argument if some of 'time' is NA; in
        ## this case just fill in some dummy values so the rest of the
        ## routine still goes through
        idx <- is.na(time)
        if (!missing(event)) {
            event[idx] <- 1
        }
        if (!missing(time2)) {
            time2[idx] <- 0
        }
        if (!missing(truncation)) {
            truncation[idx] <- 0
        }
    }

    ## check that no time varible is negative
    if (any(time[!is.na(time)] < 0)) {
        stop("Negative times are not allowed")
    }
    if (!missing(time2)) {
        if (any(time2[!is.na(time2)] < 0)) {
            stop("Negative times2 are not allowed")
        }
    }
    if (!missing(truncation)) {
        if (any(is.na(truncation))) {
            stop("Non valid values for 'truncation")
        }
        if (any(truncation < 0)) {
            stop("Negative truncation times are not allowed")
        }
    }
    ## if event is totally missing assume that all data are failures
    if (missing(event)) {
        event <- rep(1, nn)
        warning("'event' is missing: assuming all are observed failures")
    } else if (any(is.na(event))) {
        ## if some of the element in event are missing assume that
        ## they're observed failures (give a warning)
        event[is.na(event)] <- 1
        warning("Some elements in `event' are NA: set them to observed failures.")
    }

    ## check that event is 0, 1, 2, 3, 4
    if (!all(is.element(event, 0:4))) {
        stop("Invalid value for event")
    }

    ## check that for event=3 or 4, time2 is present
    interval <- (event == 3 | event == 4)
    if (sum(interval) > 0 && missing(time2)) {
        stop("'time2' has to be present for interval censored data or in-interval events")
    }
    ## and warn that time2 is ignored for even!=3
    if (sum(interval) == 0 && !missing(time2)) {
        warning("'time2' is ignored for data that are not interval censored")
    }
    if (missing(time2)) {
        time2 <- rep(0, nn)
    }

    ## if truncation is missing set it to 0
    if (missing(truncation)) {
        truncation <- rep(0, nn)
    }
    ## check that it has the correct length
    if (length(truncation) != nn) {
        stop("'truncation' is of the wrong dimension")
    }

    surv.time <- numeric(nn)
    surv.upper <- numeric(nn)
    surv.lower <- numeric(nn)

    observed <- (event == 1)
    right <- (event == 0)
    left <- (event == 2)
    interval <- (event == 3)
    ininterval <- (event == 4)

    surv.time[observed] <- time[observed]
    surv.lower[right] <- time[right]
    surv.upper[left] <- time[left]
    surv.lower[interval] <- time[interval]
    surv.upper[interval] <- time2[interval]
    surv.time[ininterval] <- time[ininterval]
    surv.lower[ininterval] <- truncation[ininterval]
    surv.upper[ininterval] <- time2[ininterval]
    truncation[ininterval] <- 0
    ss <- list(
        time = surv.time, lower = surv.lower, upper = surv.upper,
        event = event, truncation = truncation, .special = .special
    )
    class(ss) <- "inla.surv"

    return(ss)
}

`inla.plot.inla.surv.1` <- function(object, legend = TRUE, ...) {
    time <- object$time
    upper <- object$upper
    lower <- object$lower
    event <- object$event
    truncation <- object$truncation

    nn <- length(time)

    xmax <- max(time, upper, lower, event)
    xax <- c(0, xmax + xmax / 8)
    yax <- c(0, nn)
    plot(xax, yax, type = "n", xlab = "time", ylab = "", axes = FALSE)
    axis(1)
    axis(2, labels = FALSE, tick = FALSE)
    for (i in 1:nn) {
        if (event[i] == 1) {
            lines(c(truncation[i], time[i]), c(i, i), type = "l")
            text(time[i], i, "*")
        } else if (event[i] == 0) {
            lines(c(truncation[i], lower[i]), c(i, i), type = "l")
            text(lower[i], i, ">")
        } else if (event[i] == 2) {
            lines(c(truncation[i], upper[i]), c(i, i), type = "l")
            text(upper[i], i, "<")
        } else if (event[i] == 3) {
            lines(c(lower[i], upper[i]), c(i, i), type = "l")
            text(upper[i], i, "|")
            text(lower[i], i, "|")
        } else if (event[i] == 4) {
            lines(c(lower[i], upper[i]), c(i, i), type = "l")
            text(time[i], i, "*")
        }
    }

    if (legend) {
        leg.text <- "failure"
        leg.symb <- "*"
        if (any(event == 0)) {
            leg.text <- c(leg.text, "right cens")
            leg.symb <- paste(leg.symb, ">", sep = "")
        }
        if (any(event == 2)) {
            leg.text <- c(leg.text, "left cens")
            leg.symb <- paste(leg.symb, "<", sep = "")
        }
        if (any(event == 3)) {
            leg.text <- c(leg.text, "interval cens")
            leg.symb <- paste(leg.symb, "|", sep = "")
        }
        if (any(event == 4)) {
            leg.text <- c(leg.text, "in-interval failure")
            leg.symb <- paste(leg.symb, "*", sep = "")
        }
        legend("topright", leg.text, pch = leg.symb, inset = 0.02)
    }
}

`inla.print.inla.surv.1` <- function(object, quote = FALSE, ...) 
{
    invisible(print(inla.as.character.inla.surv.1(object), quote = quote, ...))
}

`inla.as.character.inla.surv.1` <- function(x, ...)
{
    class(x) <- NULL
    nn <- length(x$event)

    out <- character(nn)
    ininterval <- (x$event == 4)
    out[ininterval] <- paste("[", x$lower[ininterval], ",", x$upper[ininterval], "]", " ", x$time[ininterval], sep = "")
    interval <- (x$event == 3)
    out[interval] <- paste("[", x$lower[interval], ",", x$upper[interval], "]", sep = "")
    right <- (x$event == 0)
    out[right] <- paste(x$lower[right], "+", sep = "")
    left <- (x$event == 2)
    out[left] <- paste(x$upper[left], "-", sep = "")
    failure <- (x$event == 1)
    out[failure] <- paste(x$time[failure], sep = "")

    return(out)
}

`is.inla.surv` <- function(object) {
    return (inherits(object, "inla.surv"))
}

`inla.as.inla.surv.1` <- function(object) 
{
    if (is.list(object)) {
        for (nm in names(object)) {
            if (!is.element(nm, c("event", "time", "lower", "upper", "truncation", ".special"))) {
                stop(paste("Wrong name:", nm))
            }
        }
        class(object) <- "inla.surv"
        return(object)
    }
    if (is.data.frame(object)) {
        return(inla.as.inla.surv.1(as.list(object)))
    }
    stop("Argument must be a list or a data.frame")
}

`inla.surv.2` <- function(time, event, subject, .special = NULL) 
{
    ## check that time is present
    if (missing(time)) {
        stop("Must have a 'time' argument")
    }
    if (!is.numeric(time)) {
        stop("'Time' variable is not numeric")
    }
    nn <- length(time)

    if (any(is.na(time))) {
        ## fixup the rest of the argument if some of 'time' is NA; in
        ## this case just fill in some dummy values so the rest of the
        ## routine still goes through
        idx <- is.na(time)
        if (!missing(event)) {
            event[idx] <- 0
        }
    }

    ## check that no time varible is negative
    if (any(time[!is.na(time)] < 0)) {
        stop("Negative times are not allowed")
    }

    ## here event is multiple events if event is totally missing
    ## assume that no tumor/event is found
    if (missing(event)) {
        event <- rep(0, nn)
        warning("'event' is missing: assuming no events")
    } else if (any(is.na(event))) {
        ## if some of the element in event are missing assume that
        ## they're no event observed (give a warning)
        event[is.na(event)] <- 0
        warning("Some elements in `event' are NA: assume no event detected for these cases.")
    }


    ## check that event is 0, 1
    if (!all(is.element(event, 0:1))) {
        stop("Invalid value for event")
    }

    if (missing(subject)) {
        stop("'subject' is missing")
    }

    surv.time <- numeric(nn)
    detect <- (event == 1)
    notdetect <- (event == 0)

    surv.time <- time
    surv.time[detect] <- time[detect]
    surv.time[notdetect] <- time[notdetect]

    ss <- list(time = surv.time, event = event, subject = subject, .special = .special)
    class(ss) <- "inla.surv"

    return(ss)
}

## plotting the time according to event detect or not detect
`inla.plot.inla.surv.2` <- function(object, legend = TRUE, ...) 
{
    time <- object$time
    event <- object$event
    subject <- object$subject

    nn <- max(subject)

    xmax <- max(time)
    xmin <- min(time)
    xax <- c(xmin, xmax + xmax / 8)
    yax <- c(0, nn + 0.5)
    plot(xax, yax, type = "n", xlab = "time", ylab = "", axes = FALSE)
    axis(1)
    axis(2, labels = FALSE, tick = FALSE)

    for (i in 1:nn) {
        lines(time[subject == i], rep(i, length(time[subject == i])), type = "l")
        points(time[subject == i & event == 1],
               rep(i, length(time[subject == i & event == 1])),
               pch = 8, col = "red"
               )
        points(time[subject == i & event == 0],
               rep(i, length(time[subject == i & event == 0])),
               pch = 10,
               col = "green"
               )
    }

    if (legend) {
        leg.text <- "detect"
        leg.symb <- "*"
        leg.col <- "red"
        if (any(event == 0)) {
            leg.text <- c(leg.text, "not detect")
            leg.symb <- paste(leg.symb, "o", sep = "")
            leg.col <- c(leg.col, "green")
        }
        legend("topright", leg.text, pch = leg.symb, col = leg.col, inset = 0.02)
    }
}

`inla.print.inla.surv.2` <- function(object, quote = FALSE, ...) {
    invisible(print(inla.as.character.inla.surv.2(object), quote = quote, ...))
}

`inla.as.character.inla.surv.2` <- function(x, ...) 
{
    class(x) <- NULL
    nn <- length(x$event)
    out <- character(nn)
    detect <- (x$event == 1)
    out[detect] <- paste(x$time[detect], sep = "")
    notdetect <- (x$event == 0)
    out[notdetect] <- paste(x$time[notdetect], "+", sep = "")

    return(out)
}

`inla.as.inla.surv.2` <- function(object) 
{
    if (is.list(object)) {
        for (nm in names(object)) {
            if (!is.element(nm, c("event", "time", "subject", ".special"))) {
                stop(paste("Wrong name:", nm))
            }
        }
        class(object) <- "inla.surv"

        return(object)
    }
    if (is.data.frame(object)) {
        return(inla.as.inla.surv.2(as.list(object)))
    }

    stop("Argument must be a list or a data.frame")
}

`inla.strata` <- function(object) 
{
    ## similar to survival::strata but with levels = 1, 2, ...
    xf <- as.factor(object)
    return(list(strata = as.numeric(xf), coding = levels(xf)))
}

context("test 'the coxph extention'")

test_that("Case 1", {
    set.seed(123)
    n = 5000
    strata = c(rep(1, n/2), rep(2, n/2))
    x = runif(n)
    lambda = exp(1+x)
    y = rexp(n, rate=lambda)
    event = rep(1,n)
    data = list(y=y, event=event, x=x)
    y.surv = inla.surv(y, event)
    intercept1 = rep(1, n)
    p = inla.coxph(y.surv ~ -1 + intercept1 + x,
            list(y.surv = y.surv,  x=x, intercept1 = intercept1, strata = strata),
            control.hazard = list(strata.name = "strata"))
    r = inla(p$formula, 
            family = p$family, 
            data=c(as.list(p$data), p$data.list), 
            E = p$E)
    expect_true(abs(r$summary.fixed[1, "mean"] - 1) < 0.05)
    expect_true(abs(r$summary.fixed[2, "mean"] - 1) < 0.05)
    ## joint model
    intercept2 = rep(1, n)
    y = 1 + x + rnorm(n, sd=0.1)
    df = data.frame(intercept2, x, y)
    ## new need to cbind the data.frames, and then add the list-part of
    ## the data
    df.joint = c(as.list(inla.cbind.data.frames(p$data, df)), p$data.list)
    df.joint$Y = cbind(df.joint$y..coxph, df.joint$y)
    ## merge the formulas, recall to add '-1' and to use the new joint
    ## reponse 'Y'
    formula = update(p$formula, Y ~ intercept2 -1 + .)
    rr = inla(formula,
            family = c(p$family, "gaussian"),
            data = df.joint,
            E = df.joint$E)
    expect_true(abs(rr$summary.fixed[1, "mean"] - 1) < 0.025)
    expect_true(abs(rr$summary.fixed[2, "mean"] - 1) < 0.025)
    expect_true(abs(rr$summary.fixed[3, "mean"] - 1) < 0.025)
})

## Export: inla.reorderings

## !\name{inla.reorderings}
## !\alias{inla.reorderings}
## !\alias{reorderings}
## !
## !\title{Reorderings methods for sparse matrices}
## !
## !\description{Provide the names of all implemented reordering schemes}
## !
## !\usage{
## ! inla.reorderings()
## !}
## !
## !\arguments{
## ! None
## !}
## !
## !\value{The names of all available reorderings}
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## !inla.reorderings()
## !}



## provide list of all reordering and map their names and code. OOPS:
## these mappings must be in sync with the ones in
## gmrflib/sparse-interface.h

`inla.reorderings.list` <- function() {
    r <- list(
        auto = -1L,
        default = 0L,
        identity = 1L,
        band = 2L,
        metis = 3L,
        genmmd = 4L,
        amd = 5L,
        amdbar = 6L,
        md = 7L,
        mmd = 8L,
        amdc = 9L,
        amdbarc = 10L,
        reverseidentity = 11L
    )
    return(r)
}

`inla.reorderings` <- function() {
    return(names(inla.reorderings.list()))
}

`inla.reorderings.name2code` <- function(name) {
    r <- inla.reorderings.list()
    name <- tolower(name)
    code <- c()
    for (nam in name) {
        idx <- which(nam == names(r))
        if (length(idx) > 0L) {
            code <- c(code, as.integer(r[idx]))
        } else {
            stop(paste("Unknown reordering:", nam))
        }
    }
    return(code)
}

`inla.reorderings.code2name` <- function(code) {
    r <- inla.reorderings.list()
    name <- c()
    for (co in code) {
        idx <- which(co == r)
        if (length(idx) > 0L) {
            name <- c(name, names(r)[idx])
        } else {
            stop(paste("Unknown code:", co))
        }
    }
    return(name)
}

## meshbuilder.R
##
## Copyright (C) 2016, 2018, Finn Lindgren
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.




meshbuilder.app <- function() {
    spatial.object.classes <- function(points) {
        if (points) {
            c(
                "matrix",
                "inla.mesh",
                "inla.mesh.segment",
                "SpatialPoints", "SpatialPointsDataFrame",
                "SpatialLines", "SpatialLinesDataFrame",
                "SpatialPolygons", "SpatialPolygonsDataFrame"
            )
        } else {
            c(
                "inla.mesh",
                "inla.mesh.segment",
                "SpatialLines", "SpatialLinesDataFrame",
                "SpatialPolygons", "SpatialPolygonsDataFrame"
            )
        }
    }

    spatial.object.choices <- function(points = TRUE) {
        obj <- with(globalenv(), ls())
        classes <- spatial.object.classes(points)
        ok <- vapply(
            obj,
            function(name) {
                cl <- class(globalenv()[[name]])
                (length(cl) > 0) && (length(intersect(cl, classes)) > 0)
            },
            TRUE
        )
        if (points) {
            c("RANDOM", obj[ok])
        } else {
            obj[ok]
        }
    }

    spatial.object.code <- function(sp, name) {
        gsub("%%%", name, attr(sp, "code"))
    }
    spatial.object.to.SpatialPoints <- function(sp) {
        code <- "%%%"
        if (length(intersect(
            class(sp),
            c("SpatialPoints", "SpatialPointsDataFrame")
        )) > 0) {
            coord <- coordinates(sp)
            crs <- inla.sp_get_crs(sp)
            if (inherits(sp, "SpatialPointsDataFrame")) {
                code <- 'as(%%%, "SpatialPoints")'
            }
        } else if (length(intersect(
            class(sp),
            c("SpatialLines", "SpatialLinesDataFrame")
        )) > 0) {
            tmp <- as.inla.mesh.segment(sp)
            coord <- tmp$loc
            crs <- inla.sp_get_crs(sp)
            if (inherits(sp, "SpatialLinesDataFrame")) {
                code <- 'as(%%%, "SpatialLines")'
            }
            code <- paste0("as(", code, ', "SpatialPoints")')
        } else if (length(intersect(
            class(sp),
            c("SpatialPolygons", "SpatialPolygonsDataFrame")
        )) > 0) {
            tmp <- as.inla.mesh.segment(sp)
            coord <- tmp$loc
            crs <- inla.sp_get_crs(sp)
            if (inherits(sp, "SpatialPolygonsDataFrame")) {
                code <- 'as(%%%, "SpatialPolygons")'
            }
            code <- paste0("as(", code, ', "SpatialPoints")')
        } else if (inherits(sp, "inla.mesh.segment") || inherits(sp, "inla.mesh")) {
            coord <- sp$loc
            if (is.null(sp$crs)) {
                crs <- sp::CRS()
                crs.code <- ""
            } else {
                crs <- inla.CRS(sp$crs)
                crs.code <- ", %%%$crs"
            }
            code <- paste0("SpatialPoints(%%%$loc", crs.code, ")")
        } else {
            coord <- as.matrix(sp)
            crs <- sp::CRS()
            code <- "SpatialPoints(as.matrix(%%%))"
        }
        out <- sp::SpatialPoints(coord, crs)
        attr(out, "code") <- code
        out
    }

    spatial.object.to.segments <- function(sp) {
        code <- "%%%"
        if (length(intersect(
            class(sp),
            c(
                "SpatialLines", "SpatialLinesDataFrame",
                "SpatialPolygons", "SpatialPolygonsDataFrame"
            )
        )) > 0) {
            out <- as.inla.mesh.segment(sp)
            code <- "as.inla.mesh.segment(%%%)"
        } else if (inherits(sp, "inla.mesh")) {
            out <- do.call(inla.mesh.segment, inla.mesh.boundary(sp))
            code <- "do.call(inla.mesh.segment, inla.mesh.boundary(%%%))"
        } else if (inherits(sp, "inla.mesh.segment")) {
            out <- sp
            code <- "%%%"
        } else {
            message(paste0(
                "Don't know how to convert (",
                paste(class(sp), collapse = ", "),
                ") to inla.mesh.segment"
            ))
            return(NULL)
        }
        attr(out, "code") <- code
        out
    }

    pretty.axis.info <- function(lim, value = NA, verbose = FALSE) {
        if (verbose) {
            message(paste(
                "pretty.axis.info <-",
                "lim =", paste(lim, collapse = ", "),
                "val = ", paste(value, collapse = ", ")
            ))
        }
        limits <- range(c(lim, value), na.rm = TRUE)
        maxi <- c(1.5, 2, 3, 4, 6, 8, 10)
        cutoff <- maxi * 9 / 10
        step <- c(1e-2, 1e-2, 1e-2, 2e-2, 2e-2, 5e-2, 5e-2)
        mini <- step
        maxi <- c(maxi, maxi * 10)
        step <- c(step, step * 10)
        mini <- c(mini, mini * 10)
        cutoff <- c(cutoff, cutoff * 10)
        level <- floor(log10(limits[2]))
        maxi <- maxi * 10^level
        mini <- mini * 10^level
        step <- step * 10^level
        cutoff <- cutoff * 10^level
        ## Find smallest k such that limits[2] < maxi[k]
        k <- min(which(limits[2] < maxi))
        ## Move to next k if limits[2] > cutoff[k]
        values <- mini[k] + round((value - mini[k]) / step[k]) * step[k]
        limits <- range(c(lim, value, values), na.rm = TRUE)
        while (limits[2] > cutoff[k]) {
            k <- k + 1
            values <- mini[k] + round((value - mini[k]) / step[k]) * step[k]
            limits <- range(c(lim, value, value), na.rm = TRUE)
        }

        if (verbose) {
            message(paste(
                "pretty.axis.info ->",
                "lim =", paste(c(mini[k], maxi[k]), collapse = ", ")
            ))
        }
        list(lim = c(mini[k], maxi[k]), step = step[k])
    }

    pretty.axis.value <- function(axis.info, value) {
        pmax(axis.info$lim[1], pmin(
            axis.info$lim[2],
            axis.info$lim[1] + round((value - axis.info$lim[1]) /
                axis.info$step) * axis.info$step
        ))
    }

    meshbuilder.ui <- shiny::fluidPage(
        shiny::sidebarLayout(
            shiny::sidebarPanel(
                shiny::h4("Settings for inla.nonconvex.hull()"),
                shiny::sliderInput(
                    inputId = "offset", label = "offsets for automatic boundaries",
                    value = c(0.1, 0.3), min = 0.05, max = 2
                ),
                shiny::h4("Settings for inla.mesh.2d()"),
                shiny::sliderInput(
                    inputId = "max.edge", label = "max.edge",
                    value = c(0.05, 0.5), min = 0.01, max = 2
                ),
                shiny::sliderInput(
                    inputId = "cutoff", label = "cutoff",
                    value = 0.01, min = 0, max = 0.2
                ),
                shiny::sliderInput(
                    inputId = "min.angle", label = "min.angle",
                    value = c(21, 30), min = 1, max = 35
                ),
                shiny::hr(),
                shiny::h4("Mesh resolution assessment"),
                shiny::fluidRow(
                    shiny::column(
                        6,
                        shiny::checkboxInput("assess", label = "Active", value = FALSE),
                        shiny::uiOutput("overlay.ui"),
                        shiny::uiOutput("assess.resolution.ui")
                    ),
                    shiny::column(
                        6,
                        shiny::checkboxInput("assess.advanced", label = "Advanced", value = FALSE),
                        shiny::uiOutput("assess.quantity.ui")
                    )
                ),
                shiny::sliderInput(
                    inputId = "corr.range", label = "Spatial correlation range",
                    value = 0.2, min = 0.01, max = 4
                ),
                shiny::sliderInput(
                    inputId = "corr.nu", label = "Matern smoothness (nu)",
                    value = 1, min = 0.01, max = 1
                ),
                shiny::hr(),
                shiny::actionButton(inputId = "plot.save", label = "Save plot"),
                shiny::verbatimTextOutput("plot.filename"),
                width = 3
            ),
            shiny::mainPanel(
                shiny::tabsetPanel(
                    shiny::tabPanel(
                        "Input",
                        shiny::fluidRow(
                            shiny::column(
                                4,
                                shiny::h4("Random seed points"),
                                shiny::fluidRow(
                                    shiny::column(
                                        8,
                                        shiny::sliderInput(
                                            inputId = "input.loc.n",
                                            label = "Seed points",
                                            value = 10, min = 0, max = 1000
                                        )
                                    ),
                                    shiny::column(
                                        4,
                                        shiny::actionButton(
                                            inputId = "update.loc",
                                            label = "Generate"
                                        )
                                    )
                                ),
                                shiny::h4("User defined points"),
                                shiny::selectInput(
                                    "boundary.loc.name",
                                    label = "Boundary domain points variable name(s)",
                                    multiple = TRUE,
                                    choices = spatial.object.choices(TRUE),
                                    selected = "RANDOM"
                                ),
                                shiny::selectInput(
                                    "mesh.loc.name",
                                    label = "Mesh vertex seed points variable name(s)",
                                    multiple = TRUE,
                                    choices = spatial.object.choices(TRUE)
                                ),
                                shiny::h4("User defined boundaries"),
                                shiny::selectInput(
                                    "boundary1.name",
                                    label = "Inner boundary variable name(s)",
                                    multiple = TRUE,
                                    choices = spatial.object.choices(FALSE),
                                    selected = c()
                                ),
                                shiny::selectInput(
                                    "boundary2.name",
                                    label = "Outer boundary variable name(s)",
                                    multiple = TRUE,
                                    choices = spatial.object.choices(FALSE),
                                    selected = c()
                                ),
                                shiny::hr(),
                                shiny::textInput("crs.mesh", label = "Mesh CRS (experimental feature)"),
                                shiny::verbatimTextOutput("crs.strings")
                            ),
                            shiny::column(
                                8,
                                shiny::plotOutput(
                                    "inputplot",
                                    height = "auto",
                                    hover = "inputplot.hover"
                                )
                            )
                        ),
                        shiny::hr(),
                        shiny::verbatimTextOutput("input.loc.coordinates")
                    ),
                    shiny::tabPanel(
                        "Display",
                        shiny::plotOutput(
                            "meshplot",
                            height = "auto",
                            click = "meshplot.click"
                        ),
                        shiny::fluidRow(
                            shiny::column(
                                3,
                                shiny::tableOutput("meshmeta")
                            ),
                            shiny::column(
                                3,
                                shiny::tableOutput("click.information")
                            ),
                            shiny::column(
                                6,
                                shiny::tableOutput("field.information")
                            )
                        ),
                        shiny::verbatimTextOutput("meshplot.loc.coordinates")
                    ),
                    shiny::tabPanel(
                        "Code",
                        shiny::verbatimTextOutput("code")
                    ),
                    shiny::tabPanel(
                        "Debug",
                        shiny::checkboxInput(
                            "debug.use.trace",
                            label = "Use trace",
                            value = FALSE
                        ),
                        shiny::checkboxInput(
                            "debug.use.plot.delay",
                            label = "Use plot.delay",
                            value = FALSE
                        ),
                        shiny::verbatimTextOutput("debug")
                    ),
                    id = "panel",
                    selected = "Input"
                )
            )
        )
    )

    meshbuilder.server <- function(input, output, session) {
        plot.filename <- shiny::eventReactive(c(input$plot.save), {
            tempfile("meshbuilder_", getwd(), ".pdf")
        })
        output$plot.filename <- shiny::renderText({
            plot.filename()
        })

        shiny::observeEvent(c(input$plot.save), {
            if (input$plot.save > 0) { ## Don't save on app initialisation!
                pdf(plot.filename(), width = 4, height = 4)
                eval(meshplot.expr)
                dev.off()
                plot.filename()
            }
        })
        loc.seed <- shiny::eventReactive(c(input$update.loc), {
            as.integer(runif(1, min = 0, max = .Machine$integer.max))
        })
        input.loc.crs <- shiny::reactive({
            sp <- c(boundary.loc.input(), mesh.loc.input())
            if (!is.null(sp) && (length(sp) > 0)) {
                stopifnot(sp::identicalCRS(sp))
                inla.sp_get_crs(sp[[1]])
            } else {
                sp::CRS()
            }
        })
        loc <- shiny::reactive({
            if (input$input.loc.n > 0) {
                set.seed(loc.seed())
                out <- sp::SpatialPoints(
                    matrix(
                        runif(2 * input$input.loc.n) *
                            c(diff(userinput.xlim()), diff(userinput.ylim())) +
                            c(userinput.xlim()[1], userinput.ylim()[1]),
                        input$input.loc.n, 2,
                        byrow = TRUE
                    ),
                    input.loc.crs()
                )
                attr(out, "code") <- paste0(
                    "set.seed(", loc.seed(), ")\n",
                    "loc <- sp::SpatialPoints(matrix(runif(2*", input$input.loc.n, "), ",
                    input$input.loc.n, ", 2, byrow=TRUE))\n"
                )
            } else {
                ## Fill in with inbetween points
                seq.to.points <- function(loc, delta, scale = c(1, 1), offset = c(0, 0)) {
                    do.call(
                        rbind,
                        lapply(
                            seq_len(nrow(loc) - 1),
                            function(k) {
                                loc1 <- loc[k, ]
                                loc2 <- loc[k + 1, ]
                                if (any(is.na(loc1)) || any(is.na(loc2))) {
                                    NULL
                                } else {
                                    n <- ceiling(sum((loc2 - loc1)^2)^0.5 / delta) + 1
                                    s <- seq(0, 1, length = n)
                                    cbind(
                                        (loc1[1] * (1 - s) + s * loc2[1]) * scale[1] + offset[1],
                                        (loc1[2] * (1 - s) + s * loc2[2]) * scale[2] + offset[2]
                                    )
                                }
                            }
                        )
                    )
                }

                ## Make "BRU" text:
                m <- 20
                seq.B <- rbind(
                    c(0, 50), c(0, 0), c(25, 0), c(40, 15), c(40, 35), c(25, 50), c(0, 50),
                    c(0, 100), c(20, 100), c(35, 85), c(35, 65), c(20, 50)
                ) / 100
                seq.R <- rbind(
                    c(0, 0), c(0, 100), c(20, 100), c(35, 85), c(35, 65), c(20, 50),
                    c(0, 50), c(50, 0)
                ) / 100
                seq.U <- rbind(c(0, 100), c(0, 15), c(15, 0), c(35, 0), c(50, 15), c(50, 100)) / 100

                delta <- 0.01
                scale <- c(2, 2)
                out <- rbind(
                    seq.to.points(seq.B, delta, scale, offset = c(0, 0) * 2),
                    seq.to.points(seq.R, delta, scale, offset = c(0.6, 0) * 2),
                    seq.to.points(seq.U, delta, scale, offset = c(1.2, 0) * 2)
                )

                out <- sp::SpatialPoints(
                    matrix(
                        as.vector(t(out)) *
                            c(diff(userinput.xlim()), diff(userinput.ylim())) +
                            c(userinput.xlim()[1], userinput.ylim()[1]),
                        nrow(out), 2,
                        byrow = TRUE
                    ),
                    input.loc.crs()
                )
                attr(out, "code") <- paste0("loc <- MAGIC\n")
            }
            out
        })

        random.loc.usage <- shiny::reactiveValues(boundary = TRUE, mesh = FALSE)
        observe({
            if (shiny::isolate(debug$trace)) {
                message(paste(
                    "  input$boundary.loc.name = (",
                    paste(input$boundary.loc.name, collapse = ", "),
                    ")",
                    sep = ""
                ))
                message(paste(
                    "current$boundary.loc.name = (",
                    paste(current$boundary.loc.name, collapse = ", "),
                    ")",
                    sep = ""
                ))
            }
        })
        observe({
            if (shiny::isolate(debug$trace)) {
                message(paste(
                    "  input$mesh.loc.name = (",
                    paste(input$mesh.loc.name, collapse = ", "),
                    ")",
                    sep = ""
                ))
                message(paste(
                    "current$mesh.loc.name = (",
                    paste(current$mesh.loc.name, collapse = ", "),
                    ")",
                    sep = ""
                ))
            }
        })
        observe({
            random.loc.usage$boundary <- length(intersect(current$boundary.loc.name, "RANDOM")) > 0
            if (shiny::isolate(debug$trace)) {
                message(paste(
                    "random.loc.usage$boundary =",
                    random.loc.usage$boundary
                ))
            }
        })
        observe({
            random.loc.usage$mesh <- length(intersect(current$mesh.loc.name, "RANDOM")) > 0
            if (shiny::isolate(debug$trace)) {
                message(paste(
                    "random.loc.usage$mesh =",
                    random.loc.usage$mesh
                ))
            }
        })

        current <- shiny::reactiveValues(
            boundary.loc.name = "RANDOM",
            mesh.loc.name = c()
        )
        shiny::observeEvent(list(input$boundary.loc.name, length(input$boundary.loc.name)), {
            tmp <- input$boundary.loc.name
            input.boundary.loc.name <- setdiff(tmp, "RANDOM")
            if (("RANDOM" %in% current$boundary.loc.name) &&
                ("RANDOM" %in% tmp) &&
                (length(input.boundary.loc.name) > 0)) {
                current$boundary.loc.name <- input.boundary.loc.name
                updateSelectInput(
                    session, "boundary.loc.name",
                    selected = current$boundary.loc.name
                )
            } else if (!("RANDOM" %in% current$boundary.loc.name) &&
                ("RANDOM" %in% tmp)) {
                current$boundary.loc.name <- "RANDOM"
                updateSelectInput(
                    session, "boundary.loc.name",
                    selected = current$boundary.loc.name
                )
            } else {
                current$boundary.loc.name <- tmp
            }
        })
        mesh.loc.name.previous <- "RANDOM"
        shiny::observeEvent(list(input$mesh.loc.name, length(input$mesh.loc.name)), {
            input.mesh.loc.name <- setdiff(input$mesh.loc.name, "RANDOM")
            if (("RANDOM" %in% current$mesh.loc.name) &&
                ("RANDOM" %in% input$mesh.loc.name) &&
                (length(input.mesh.loc.name) > 0)) {
                current$mesh.loc.name <- input.mesh.loc.name
                updateSelectInput(
                    session, "mesh.loc.name",
                    selected = current$mesh.loc.name
                )
            } else if (!("RANDOM" %in% mesh.loc.name.previous) &&
                ("RANDOM" %in% input$mesh.loc.name)) {
                current$mesh.loc.name <- "RANDOM"
                updateSelectInput(
                    session, "mesh.loc.name",
                    selected = current$mesh.loc.name
                )
            } else {
                current$mesh.loc.name <- input$mesh.loc.name
            }
        })
        convert.globalenv.to.SpatialPoints <- function(names) {
            out <- lapply(
                names,
                function(name) {
                    out <- spatial.object.to.SpatialPoints(globalenv()[[name]])
                    attr(out, "code") <- spatial.object.code(out, name)
                    out
                }
            )
            names(out) <- names
            if (length(out) > 0) {
                out <- out
            } else {
                out <- NULL
            }
            out
        }
        convert.globalenv.to.segments <- function(names) {
            out <- lapply(
                names,
                function(name) {
                    out <- spatial.object.to.segments(globalenv()[[name]])
                    attr(out, "code") <- spatial.object.code(out, name)
                    out
                }
            )
            names(out) <- names
            if (length(out) > 0) {
                out <- out
            } else {
                out <- NULL
            }
            out
        }
        boundary.loc.input <- shiny::reactive({
            names <- setdiff(current$boundary.loc.name, "RANDOM")
            convert.globalenv.to.SpatialPoints(names)
        })
        mesh.loc.input <- shiny::reactive({
            names <- setdiff(current$mesh.loc.name, "RANDOM")
            convert.globalenv.to.SpatialPoints(names)
        })
        boundary.loc.to.boundary.auto <- function(loc, offset) {
            if (is.null(loc)) {
                out <- NULL
            } else {
                out <- INLA::inla.nonconvex.hull(coordinates(loc), offset)
                attr(out, "code") <-
                    paste0(
                        "inla.nonconvex.hull(coordinates(boundary.loc), ",
                        offset, ")"
                    )
                out <- list(out)
            }
            out
        }
        boundary1.auto <- shiny::reactive({
            loc0 <- boundary.loc()
            offset0 <- input$offset[1]
            boundary.loc.to.boundary.auto(loc0, offset0)
        })
        boundary2.auto <- shiny::reactive({
            loc0 <- boundary.loc()
            offset0 <- input$offset[2]
            boundary.loc.to.boundary.auto(loc0, offset0)
        })
        boundary1.input <- shiny::reactive({
            names <- input$boundary1.name
            convert.globalenv.to.segments(names)
        })
        boundary2.input <- shiny::reactive({
            names <- input$boundary2.name
            convert.globalenv.to.segments(names)
        })

        combine.input.loc <- function(sp, use.loc) {
            out <- NULL
            code <- ""
            num <- length(sp) + (use.loc)
            if (num > 0) {
                if (num == 1) {
                    glue <- ""
                } else {
                    glue <- "rbind("
                }
                names <- names(sp)
                if (length(sp) >= 1) {
                    out <- do.call(rbind, lapply(names, function(name) sp[[name]]))
                    code <- paste0(code, glue, paste0(lapply(sp, function(x) {
                        attr(x, "code")
                    }), collapse = ",\n    "))
                    glue <- ",    \n"
                }
                if (use.loc) {
                    if (is.null(out)) {
                        out <- loc()
                    } else {
                        out <- rbind(out, loc())
                    }
                    code <- paste0(code, glue, "loc")
                }
                if (num > 1) {
                    code <- paste0(code, ")")
                }
                attr(out, "code") <- paste0(code, "\n")
            }
            out
        }
        boundary.loc <- shiny::reactive({
            sp <- boundary.loc.input()
            out <- combine.input.loc(sp, random.loc.usage$boundary)
            if (!is.null(out)) {
                attr(out, "code") <- paste0("boundary.loc <- ", attr(out, "code"))
            }
            out
        })
        mesh.loc <- shiny::reactive({
            sp <- mesh.loc.input()
            out <- combine.input.loc(sp, random.loc.usage$mesh)
            if (!is.null(out)) {
                attr(out, "code") <- paste0("mesh.loc <- ", attr(out, "code"))
            }
            out
        })
        combine.boundaries <- function(bnd1, bnd2) {
            bnd <- list()
            code <- "NULL"
            if (is.null(bnd1)) {
                n1 <- 0
            } else {
                stopifnot(inherits(bnd1, "list"))
                n1 <- length(bnd1)
            }
            if (is.null(bnd2)) {
                n2 <- 0
            } else {
                stopifnot(inherits(bnd2, "list"))
                n2 <- length(bnd2)
            }
            if (n1 + n2 == 1) {
                if (n1 == 1) {
                    bnd <- bnd1[[1]]
                    code <- attr(bnd1[[1]], "code")
                } else {
                    bnd <- bnd2[[1]]
                    code <- attr(bnd2[[1]], "code")
                }
            } else if (n1 + n2 > 1) {
                bnd <- c(bnd1, bnd2)
                code <- paste0(
                    "list(",
                    paste(
                        lapply(bnd, function(x) {
                            attr(x, "code")
                        }),
                        collapse = ",\n    "
                    ),
                    ")"
                )
            }
            attr(bnd, "code") <- code
            bnd
        }
        boundary <- shiny::reactive({
            bnd1 <- combine.boundaries(boundary1.auto(), boundary1.input())
            bnd2 <- combine.boundaries(boundary2.auto(), boundary2.input())
            if (length(bnd1) + length(bnd2) == 0) {
                bnd <- NULL
            } else {
                bnd <- list(bnd1, bnd2)
                attr(bnd, "code") <- paste0(
                    "boundary <- list(\n",
                    attr(bnd1, "code"), ",\n    ",
                    attr(bnd2, "code"), ")\n"
                )
            }
            bnd
        })
        mesh <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh")
            loc1 <- mesh.loc()
            bnd1 <- boundary()
            if (is.null(loc1)) {
                loc.code <- ""
            } else {
                loc.code <- "loc=mesh.loc,
                     "
            }
            if (is.null(bnd1)) {
                bnd.code <- ""
            } else {
                bnd.code <- "boundary=boundary,
                     "
            }
            if (is.null(loc1) && is.null(bnd1)) {
                out <- NULL
                metadata$mesh <- NULL
            } else {
                if (shiny::isolate(debug$trace)) {
                    if (inla.has_PROJ6()) {
                        try(message(paste(
                            "crs.mesh               =",
                            inla.crs_get_wkt(inla.CRS(input$crs.mesh))
                        )))
                        try(message(paste(
                            "CRS(mesh.loc)          =",
                            inla.crs_get_wkt(inla.sp_get_crs(loc1))
                        )))
                        try(message(paste(
                            "CRS(boundary[[1]]$crs) =",
                            inla.crs_get_wkt(bnd1[[1]]$crs)
                        )))
                        try(message(paste(
                            "CRS(boundary[[2]]$crs) =",
                            inla.crs_get_wkt(bnd2[[2]]$crs)
                        )))
                    } else {
                        try(message(paste(
                            "crs.mesh               =",
                            inla.CRSargs(inla.CRS(input$crs.mesh))
                        )))
                        try(message(paste("CRS(mesh.loc)          =", proj4string(loc1))))
                        try(message(paste("CRS(boundary[[1]]$crs) =", inla.CRSargs(bnd1[[1]]$crs))))
                        try(message(paste("CRS(boundary[[2]]$crs) =", inla.CRSargs(bnd1[[2]]$crs))))
                    }
                }
                out <- INLA::inla.mesh.2d(
                    loc = loc1,
                    boundary = bnd1,
                    max.edge = input$max.edge,
                    min.angle = rev(input$min.angle),
                    max.n = c(48000, 16000),
                    max.n.strict = c(128000, 128000),
                    cutoff = input$cutoff,
                    offset = input$offset,
                    crs = if (inla.has_PROJ6()) {
                        inla.CRS(input$crs.mesh)
                    } else {
                        inla.CRS(input$crs.mesh)
                    },
                    plot.delay = shiny::isolate(debug$plot.delay)
                )
                attr(out, "code") <- paste0(
                    "mesh <- inla.mesh.2d(", loc.code, bnd.code,
                    "max.edge=c(", input$max.edge[1], ", ", input$max.edge[2], "),
                     min.angle=c(", input$min.angle[2], ", ", input$min.angle[1], "),
                     max.n=c(48000, 16000), ## Safeguard against large meshes.
                     max.n.strict=c(128000, 128000), ## Don't build a huge mesh!
                     cutoff=", input$cutoff, ", ## Filter away adjacent points.
                     offset=c(", paste0(input$offset, collapse = ", "), ")) ## Offset for extra boundaries, if needed.\n"
                )
                el <- sqrt(c(
                    Matrix::rowSums((out$loc[out$graph$tv[, 2], ] -
                        out$loc[out$graph$tv[, 1], ])^2),
                    Matrix::rowSums((out$loc[out$graph$tv[, 3], ] -
                        out$loc[out$graph$tv[, 2], ])^2),
                    Matrix::rowSums((out$loc[out$graph$tv[, 1], ] -
                        out$loc[out$graph$tv[, 3], ])^2)
                ))
                metadata$mesh <- cbind(out$n, min(el), max(el))
            }
            metadata$fine <- NULL
            out
        })
        fine <- shiny::reactive({
            if (is.null(mesh())) {
                out <- NULL
                metadata$fine <- NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine")
                out <- INLA::inla.mesh.2d(
                    loc = mesh()$loc,
                    boundary = do.call(INLA::inla.mesh.segment, INLA::inla.mesh.boundary(mesh())),
                    max.edge = min(mesh.edgelengths()) / 2,
                    min.angle = 21,
                    max.n = 64000,
                    max.n.strict = 128000,
                    cutoff = 0
                )
                attr(out, "code") <- paste0(
                    "fine <- inla.mesh.2d(loc=mesh$loc,
                         boundary=do.call(inla.mesh.segment, inla.mesh.boundary(mesh)),
                         max.edge=", min(mesh.edgelengths()) / 2, ",
                         min.angle=21,
                         max.n=64000,
                         max.n.strict=128000,
                         cutoff=0)\n"
                )
                el <- sqrt(c(
                    Matrix::rowSums((out$loc[out$graph$tv[, 2], ] -
                        out$loc[out$graph$tv[, 1], ])^2),
                    Matrix::rowSums((out$loc[out$graph$tv[, 3], ] -
                        out$loc[out$graph$tv[, 2], ])^2),
                    Matrix::rowSums((out$loc[out$graph$tv[, 1], ] -
                        out$loc[out$graph$tv[, 3], ])^2)
                ))
                metadata$fine <- cbind(out$n, min(el), max(el))
            }
            out
        })

        mesh.edgelengths <- shiny::reactive({
            m <- mesh()
            if (is.null(m)) {
                c(NA)
            } else {
                sqrt(c(
                    Matrix::rowSums((m$loc[m$graph$tv[, 2], ] - m$loc[m$graph$tv[, 1], ])^2),
                    Matrix::rowSums((m$loc[m$graph$tv[, 3], ] - m$loc[m$graph$tv[, 2], ])^2),
                    Matrix::rowSums((m$loc[m$graph$tv[, 1], ] - m$loc[m$graph$tv[, 3], ])^2)
                ))
            }
        })
        fine.edgelengths <- shiny::reactive({
            m <- fine()
            if (is.null(m)) {
                c(NA)
            } else {
                sqrt(c(
                    Matrix::rowSums((m$loc[m$graph$tv[, 2], ] - m$loc[m$graph$tv[, 1], ])^2),
                    Matrix::rowSums((m$loc[m$graph$tv[, 3], ] - m$loc[m$graph$tv[, 2], ])^2),
                    Matrix::rowSums((m$loc[m$graph$tv[, 1], ] - m$loc[m$graph$tv[, 3], ])^2)
                ))
            }
        })

        mesh.proj <- shiny::reactive({
            if (is.null(mesh())) {
                NULL
            } else {
                INLA::inla.mesh.projector(mesh(), dims = c(500, 500))
            }
        })
        fine.proj <- shiny::reactive({
            if (is.null(fine())) {
                NULL
            } else {
                INLA::inla.mesh.projector(fine(), lattice = mesh.proj()$lattice)
            }
        })
        mesh.spde <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.spde")
            if (is.null(mesh())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.spde!")
                INLA::inla.spde2.pcmatern(
                    mesh(),
                    alpha = input$corr.nu + 1,
                    prior.range = c(1, 0.5),
                    prior.sigma = c(1, 0.5)
                )
            }
        })
        fine.spde <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.spde")
            if (is.null(mesh())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.spde!")
                INLA::inla.spde2.pcmatern(
                    fine(),
                    alpha = input$corr.nu + 1,
                    prior.range = c(1, 0.5),
                    prior.sigma = c(1, 0.5)
                )
            }
        })
        mesh.Q <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.Q")
            if (is.null(mesh.spde())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.Q!")
                INLA::inla.spde.precision(mesh.spde(), theta = log(c(input$corr.range, 1)))
            }
        })
        fine.Q <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.Q")
            if (is.null(fine.spde())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.Q!")
                INLA::inla.spde.precision(fine.spde(), theta = log(c(input$corr.range, 1)))
            }
        })
        mesh.S <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.S")
            if (is.null(mesh.Q())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.S!")
                INLA::inla.qinv(mesh.Q(), reordering = INLA::inla.reorderings())
            }
        })
        fine.S <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.S")
            if (is.null(fine.Q())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.S!")
                INLA::inla.qinv(fine.Q(), reordering = INLA::inla.reorderings())
            }
        })
        mesh.sd <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.sd")
            if (is.null(mesh.S())) {
                if (shiny::isolate(debug$trace)) message("mesh.sd: NULL")
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.sd!")
                proj <- mesh.proj()
                v <- Matrix::rowSums(proj$proj$A * (proj$proj$A %*% mesh.S()))
                v[!proj$proj$ok] <- NA
                matrix(v^0.5, length(proj$x), length(proj$y))
            }
        })
        fine.sd <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.sd")
            if (is.null(fine.S())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.sd!")
                proj <- fine.proj()
                v <- Matrix::rowSums(proj$proj$A * (proj$proj$A %*% fine.S()))
                v[!proj$proj$ok] <- NA
                matrix(v^0.5, length(proj$x), length(proj$y))
            }
        })
        mesh.sd.deviation.approx <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.sd.deviation.approx")
            if (is.null(mesh.sd()) || is.null(mesh.proj())) {
                if (shiny::isolate(debug$trace)) message("mesh.sd.deviation.approx: NULL")
                NULL
            } else {
                val0 <- mesh.sd()
                proj <- mesh.proj()
                val <- proj$proj$A %*% (
                    as.vector(t(proj$proj$A[proj$proj$ok, , drop = FALSE]) %*%
                        as.vector(val0)[proj$proj$ok]) /
                        colSums(proj$proj$A[proj$proj$ok, , drop = FALSE]))
                val[!proj$proj$ok] <- NA
                matrix(
                    1 + (as.vector(val0) - val),
                    length(proj$x), length(proj$y)
                )
            }
        })
        fine.sd.deviation.approx <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.sd.deviation.approx")
            if (is.null(fine.sd()) || is.null(fine.proj())) {
                if (shiny::isolate(debug$trace)) message("fine.sd.deviation.approx: NULL")
                NULL
            } else {
                val0 <- fine.sd()
                proj <- fine.proj()
                val <- proj$proj$A %*% (
                    as.vector(t(proj$proj$A[proj$proj$ok, , drop = FALSE]) %*%
                        as.vector(val0)[proj$proj$ok]) /
                        colSums(proj$proj$A[proj$proj$ok, , drop = FALSE]))
                val[!proj$proj$ok] <- NA
                matrix(
                    1 + (as.vector(val0) - val),
                    length(proj$x), length(proj$y)
                )
            }
        })

        mesh.sd.bound <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.sd.bound")
            if (is.null(mesh.S())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.sd.bound!")
                proj <- mesh.proj()
                INLA::inla.mesh.project(proj, field = diag(mesh.S())^0.5)
            }
        })
        fine.sd.bound <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.sd.bound")
            if (is.null(fine.S())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.sd.bound!")
                proj <- fine.proj()
                INLA::inla.mesh.project(proj, field = diag(fine.S())^0.5)
            }
        })
        mesh.corr <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("mesh.corr")
            if (is.null(mesh.sd()) || is.null(meshplot.A.mesh()) ||
                is.null(click.information()) || is.null(mesh.proj())) {
                if (shiny::isolate(debug$trace)) message("mesh.corr: NULL")
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("mesh.corr!")
                A <- meshplot.A.mesh()
                s <- click.information()
                proj <- mesh.proj()
                if (shiny::isolate(debug$trace)) {
                    print(str(A))
                    print(str(s))
                    print(str(proj))
                }
                corr <- proj$proj$A %*% as.vector(inla.qsolve(mesh.Q(), t(A)))
                out <- matrix(corr, length(proj$x), length(proj$y)) / mesh.sd() / s["SD", "Mesh"]
                if (shiny::isolate(debug$trace)) message("mesh.corr: end")
                out
            }
        })
        fine.corr <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("fine.corr")
            if (is.null(fine.sd()) || is.null(meshplot.A.fine()) ||
                is.null(click.information) || is.null(fine.proj())) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("fine.corr!")
                A <- meshplot.A.fine()
                s <- click.information()
                proj <- fine.proj()
                corr <- proj$proj$A %*% inla.qsolve(fine.Q(), t(A))
                matrix(corr, length(proj$x), length(proj$y)) / fine.sd() / s["SD", "Fine"]
            }
        })

        axis.infos <- shiny::reactiveValues(offset = NULL, max.edge = NULL, cutoff = NULL, corr.range = NULL)
        axis.update <- shiny::reactiveValues(offset = FALSE, max.edge = FALSE, cutoff = FALSE, corr.range = FALSE)
        shiny::observeEvent(c(input$offset, limits$input.xlim, limits$input$ylim), {
            if (shiny::isolate(debug$trace)) message("Observe offset limit recalculation")
            val <- input$offset
            new.info <- pretty.axis.info(
                range(c(
                    val,
                    diff(limits$input.xlim) / 5,
                    diff(limits$input.ylim) / 5
                )),
                val,
                shiny::isolate(debug$trace)
            )
            if (shiny::isolate(debug$trace)) {
                print(axis.infos$offset$lim)
                print(new.info$lim)
            }
            axis.update$offset <- !identical(axis.infos$offset, new.info)
            if (shiny::isolate(debug$trace)) message(paste("Update offset", axis.update$offset))
            if (axis.update$offset) {
                axis.infos$offset <- new.info
            }
        })
        shiny::observeEvent(c(input$max.edge, limits$input.xlim, limits$input.ylim), {
            if (shiny::isolate(debug$trace)) message("Observe max.edge limit recalculation")
            val <- input$max.edge
            new.info <- pretty.axis.info(
                range(c(
                    val,
                    diff(limits$input.xlim) / 5,
                    diff(limits$input.ylim) / 5
                )),
                val,
                shiny::isolate(debug$trace)
            )
            axis.update$max.edge <- !identical(axis.infos$max.edge, new.info)
            if (axis.update$max.edge) {
                axis.infos$max.edge <- new.info
            }
        })
        shiny::observeEvent(c(input$cutoff, input$max.edge), {
            if (shiny::isolate(debug$trace)) message("Observe cutoff limit recalculation")
            val <- min(input$cutoff, input$max.edge[1])
            new.info <- pretty.axis.info(
                range(c(val, input$max.edge[1])),
                val,
                shiny::isolate(debug$trace)
            )
            new.info$lim[2] <- min(new.info$lim[2], input$max.edge[1])
            axis.update$cutoff <- !identical(axis.infos$cutoff, new.info)
            if (axis.update$cutoff) {
                axis.infos$cutoff <- new.info
            }
        })
        shiny::observeEvent(c(input$corr.range, limits$input.xlim, limits$input.ylim), {
            if (shiny::isolate(debug$trace)) message("Observe corr.range limit recalculation")
            val <- input$corr.range
            new.info <- pretty.axis.info(
                range(c(
                    val,
                    diff(limits$input.xlim) / 5,
                    diff(limits$input.ylim) / 5
                )),
                val,
                shiny::isolate(debug$trace)
            )
            axis.update$corr.range <- !identical(axis.infos$corr.range, new.info)
            if (axis.update$corr.range) {
                axis.infos$corr.range <- new.info
            }
        })
        shiny::observeEvent(axis.update$offset,
            {
                if (shiny::isolate(debug$trace)) message(paste("Observe offset limit update", axis.update$offset))
                if (axis.update$offset) {
                    axis.update$offset <- FALSE
                    info <- axis.infos$offset
                    new.val <- pretty.axis.value(info, input$offset)
                    if (shiny::isolate(debug$trace)) {
                        print(paste("Old values", paste(input$offset, collapse = ", ")))
                        print(paste("New limits", paste(info$lim, collapse = ", ")))
                        print(paste("New values", paste(new.val, collapse = ", ")))
                    }
                    shiny::updateSliderInput(
                        session, "offset",
                        min = info$lim[1], max = info$lim[2],
                        step = info$step, value = new.val
                    )
                }
            },
            priority = 10
        )
        shiny::observeEvent(axis.update$max.edge,
            {
                if (shiny::isolate(debug$trace)) message(paste("Observe max.edge limit update", axis.update$max.edge))
                if (axis.update$max.edge) {
                    axis.update$max.edge <- FALSE
                    info <- axis.infos$max.edge
                    new.val <- pretty.axis.value(info, input$max.edge)
                    if (shiny::isolate(debug$trace)) {
                        print(paste("Old values", paste(input$max.edge, collapse = ", ")))
                        print(paste("New limits", paste(info$lim, collapse = ", ")))
                        print(paste("New values", paste(new.val, collapse = ", ")))
                    }
                    shiny::updateSliderInput(
                        session, "max.edge",
                        min = info$lim[1], max = info$lim[2],
                        step = info$step, value = new.val
                    )
                }
            },
            priority = 9
        )
        shiny::observeEvent(c(axis.update$cutoff, input$cutoff, input$max.edge),
            {
                if (shiny::isolate(debug$trace)) message(paste("Observe cutoff limit update", axis.update$cutoff))
                if (axis.update$cutoff || (input$cutoff > input$max.edge[1])) {
                    axis.update$cutoff <- FALSE
                    info <- axis.infos$cutoff
                    new.val <- pretty.axis.value(info, min(input$cutoff, input$max.edge[1]))
                    if (shiny::isolate(debug$trace)) {
                        print(paste("Old values", paste(input$cutoff, collapse = ", ")))
                        print(paste("New limits", paste(info$lim, collapse = ", ")))
                        print(paste("New values", paste(new.val, collapse = ", ")))
                    }
                    shiny::updateSliderInput(
                        session, "cutoff",
                        min = info$lim[1], max = info$lim[2],
                        step = info$step, value = new.val
                    )
                }
            },
            priority = 8
        )
        shiny::observeEvent(axis.update$corr.range,
            {
                if (shiny::isolate(debug$trace)) message(paste("Observe corr.range limit update", axis.update$corr.range))
                if (axis.update$corr.range) {
                    axis.update$corr.range <- FALSE
                    info <- axis.infos$corr.range
                    new.val <- pretty.axis.value(info, input$corr.range)
                    if (shiny::isolate(debug$trace)) {
                        print(paste("Old values", paste(input$corr.range, collapse = ", ")))
                        print(paste("New limits", paste(info$lim, collapse = ", ")))
                        print(paste("New values", paste(new.val, collapse = ", ")))
                    }
                    shiny::updateSliderInput(
                        session, "corr.range",
                        min = info$lim[1], max = info$lim[2],
                        step = info$step, value = new.val
                    )
                }
            },
            priority = 7
        )

        metadata <- shiny::reactiveValues(
            mesh = NULL,
            fine = NULL
        )
        meshmeta <- shiny::reactive({
            m1 <- m2 <- NULL
            if (!is.null(metadata$mesh)) {
                m1 <- rbind(metadata$mesh)
                rownames(m1) <- "Mesh"
            }
            if (!is.null(metadata$fine)) {
                m2 <- rbind(metadata$fine)
                rownames(m2) <- "Fine"
            }
            out <- rbind(m1, m2)
            colnames(out) <- c("nV", "minE", "maxE")
            out <- as.data.frame(out)
            out$nV <- as.integer(out$nV)
            out
        })

        shiny::observeEvent(c(input$assess.quantity, input$assess.resolution),
            {
                if (!is.null(input$assess.quantity)) {
                    if (input$assess.quantity == "deviation.approx") {
                        if (input$assess.resolution == "rel") {
                            shiny::updateRadioButtons(session, "assess.resolution", selected = "mesh")
                        }
                    } else
                    if (input$assess.quantity == "deviation") {
                        shiny::updateRadioButtons(session, "assess.quantity", selected = "sd")
                        shiny::updateRadioButtons(session, "assess.resolution", selected = "rel")
                    }
                }
            },
            priority = 10
        )

        output$assess.quantity.ui <- renderUI({
            sel <- shiny::isolate(input$assess.quantity)
            default <- 1
            choices <- list(
                "SD" = "sd",
                "Correlation" = "corr",
                "Approximate SD deviation" = "deviation.approx",
                "SD bound" = "sd.bound",
                "SD/bound" = "sd/bound"
            )
            if (input$assess.advanced) {
                choices <- c(choices, list("SD deviation" = "deviation"))
            }
            if (is.null(sel) || !(sel %in% choices)) {
                sel <- choices[[default]]
            }
            radioButtons(
                "assess.quantity",
                label = "Quantity",
                choices = choices, selected = sel
            )
        })
        output$overlay.ui <- renderUI({
            if (is.null(input$assess.advanced)) {
                return(NULL)
            }
            sel <- shiny::isolate(input$overlay)
            default <- 2
            if (input$assess.advanced) {
                choices <- list(
                    "None" = "none",
                    "Mesh" = "mesh",
                    "Fine" = "fine"
                )
            } else {
                choices <- list(
                    "None" = "none",
                    "Mesh" = "mesh"
                )
            }
            if (is.null(sel) || !(sel %in% choices)) {
                sel <- choices[[default]]
            }
            radioButtons(
                "overlay",
                label = "Overlay",
                choices = choices, selected = sel
            )
        })
        output$assess.resolution.ui <- renderUI({
            if (is.null(input$assess.quantity)) {
                return(NULL)
            }
            sel <- shiny::isolate(input$assess.resolution)
            default <- 1
            if (input$assess.advanced) {
                if (input$assess.quantity == "corr") {
                    choices <- list(
                        "Mesh" = "mesh",
                        "Fine" = "fine",
                        "Mesh-Fine" = "rel"
                    )
                } else {
                    choices <- list(
                        "Mesh" = "mesh",
                        "Fine" = "fine",
                        "Mesh/Fine" = "rel"
                    )
                }
            } else {
                choices <- list("Mesh" = "mesh")
            }
            if (is.null(sel) || !(sel %in% choices)) {
                sel <- choices[[default]]
            }
            radioButtons(
                "assess.resolution",
                label = "Resolution",
                choices = choices, selected = sel
            )
        })

        meshplot.expr <- quote({
            if (shiny::isolate(debug$trace)) message("meshplot.expr")
            col <- colorRampPalette(c("blue", "white", "red"))
            n.col <- 1 + 64
            if (!input$assess && !is.null(mesh())) {
                if (shiny::isolate(debug$trace)) message("meshplot.expr: basic")
                zlim <- c(0.5, 1.5)
                ## fields::image.plot(mesh.proj()$x, mesh.proj()$y,
                ## matrix(0, length(mesh.proj()$x), length(mesh.proj()$y)),
                ## zlim=zlim,
                ## xlim=xlim(), ylim=ylim(),
                ## col=col(n.col), asp=1, main="",
                ## xlab="Easting", ylab="Northing")
                fields::image.plot(
                    range(mesh.proj()$x), range(mesh.proj()$y),
                    matrix(0, 2, 2),
                    zlim = zlim,
                    xlim = xlim(), ylim = ylim(),
                    col = col(n.col), asp = 1, main = "",
                    xlab = "Easting", ylab = "Northing"
                )
                plot(mesh(), add = TRUE)
            } else if (input$assess) {
                if (shiny::isolate(debug$trace)) {
                    message("meshplot.expr: assess")
                    message(paste("meshplot.expr: assess.quantity", input$assess.quantity))
                    message(paste("meshplot.expr: assess.resolution", input$assess.resolution))
                    message(paste("meshplot.expr: overlay", input$overlay))
                }
                zlim.sd <- c(0.5, 1.5)
                zlim.sd.rel <- c(0.5, 1.5)
                zlim.corr <- c(-1, 1)
                zlim.corr.rel <- c(-1, 1) / 5
                zlim.sdbound <- c(1 / sqrt(3), 1 + (1 - 1 / sqrt(3)))
                zlim.sdbound.rel <- c(0.5, 1.5)
                ## sd approximate deviation
                if (input$assess.quantity %in% c("deviation.approx")) {
                    zlim <- zlim.sd.rel
                    if (input$assess.resolution %in% c("mesh")) {
                        val <- mesh.sd.deviation.approx()
                    } else if (input$assess.resolution %in% c("fine")) {
                        val <- fine.sd.deviation.approx()
                    } else if (input$assess.resolution %in% c("rel")) {
                        val <- mesh.sd.deviation.approx() / fine.sd.deviation.approx()
                    }
                } else
                ## sd
                if (input$assess.quantity %in% c("sd")) {
                    zlim <- zlim.sd
                    if (input$assess.resolution %in% c("mesh")) {
                        val <- mesh.sd()
                    } else if (input$assess.resolution %in% c("fine")) {
                        val <- fine.sd()
                    } else if (input$assess.resolution %in% c("rel")) {
                        val <- mesh.sd() / fine.sd()
                        zlim <- zlim.sd.rel
                    }
                } else
                ## corr
                if (input$assess.quantity %in% c("corr")) {
                    zlim <- zlim.corr
                    if (input$assess.resolution %in% c("mesh")) {
                        val <- mesh.corr()
                    } else if (input$assess.resolution %in% c("fine")) {
                        val <- fine.corr()
                    } else if (input$assess.resolution %in% c("rel")) {
                        val <- mesh.corr()
                        if (!is.null(val)) {
                            val <- val - fine.corr()
                        }
                        zlim <- zlim.corr.rel
                    }
                } else
                ## sd.bound
                if (input$assess.quantity %in% c("sd.bound")) {
                    zlim <- zlim.sd
                    if (input$assess.resolution %in% c("mesh")) {
                        val <- mesh.sd.bound()
                    } else if (input$assess.resolution %in% c("fine")) {
                        val <- fine.sd.bound()
                    } else if (input$assess.resolution %in% c("rel")) {
                        val <- mesh.sd.bound() / fine.sd.bound()
                        zlim <- zlim.sd.rel
                    }
                } else
                ## sd/bound
                if (input$assess.quantity %in% c("sd/bound")) {
                    zlim <- zlim.sdbound
                    if (input$assess.resolution %in% c("mesh")) {
                        val <- mesh.sd() / mesh.sd.bound()
                    } else if (input$assess.resolution %in% c("fine")) {
                        val <- fine.sd() / fine.sd.bound()
                    } else if (input$assess.resolution %in% c("rel")) {
                        val <- (mesh.sd() / mesh.sd.bound()) / (fine.sd() / fine.sd.bound())
                        zlim <- zlim.sdbound.rel
                    }
                } else {
                    val <- NULL
                    zlim <- c(-1, 1)
                }
                if (is.null(val)) {
                    fields::image.plot(
                        mesh.proj()$x, mesh.proj()$y,
                        matrix(0, length(mesh.proj()$x), length(mesh.proj()$y)),
                        zlim = zlim,
                        xlim = xlim(), ylim = ylim(),
                        col = col(n.col), asp = 1, main = "",
                        xlab = "Easting", ylab = "Northing"
                    )
                } else {
                    val <- matrix(
                        pmax(zlim[1], pmin(zlim[2], as.vector(val))),
                        nrow(val), ncol(val)
                    )
                    fields::image.plot(
                        mesh.proj()$x, mesh.proj()$y, val,
                        zlim = zlim,
                        xlim = xlim(), ylim = ylim(),
                        col = col(n.col), asp = 1, main = "",
                        xlab = "Easting", ylab = "Northing"
                    )
                }
                if (!is.null(clicks$meshplot.loc)) {
                    points(clicks$meshplot.loc, col = 2)
                }
                if ("mesh" %in% input$overlay) {
                    if (shiny::isolate(debug$trace)) message("meshplot.expr: mesh overlay")
                    plot(mesh(), add = TRUE)
                }
                if ("fine" %in% input$overlay) {
                    if (shiny::isolate(debug$trace)) message("meshplot.expr: fine overlay")
                    plot(fine(), add = TRUE)
                }
            }
        })

        output$meshplot <-
            renderPlot(meshplot.expr, quoted = TRUE, height = 1000, units = "px")

        userinput.xlim <- shiny::reactive({
            lim1 <- if (is.null(boundary.loc.input())) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary.loc.input(),
                        function(x) range(coordinates(x)[, 1], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim2 <- if (is.null(mesh.loc.input())) {
                NA
            } else {
                range(
                    unlist(lapply(
                        mesh.loc.input(),
                        function(x) range(coordinates(x)[, 1], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim3 <- if (length(input$boundary1.name) == 0) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary1.input(),
                        function(x) range(x$loc[, 1], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim4 <- if (length(input$boundary2.name) == 0) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary2.input(),
                        function(x) range(x$loc[, 1], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            r <- c(lim1, lim2, lim3, lim4)
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            r
        })
        userinput.ylim <- shiny::reactive({
            lim1 <- if (is.null(boundary.loc.input())) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary.loc.input(),
                        function(x) range(coordinates(x)[, 2], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim2 <- if (is.null(mesh.loc.input())) {
                NA
            } else {
                range(
                    unlist(lapply(
                        mesh.loc.input(),
                        function(x) range(coordinates(x)[, 2], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim3 <- if (length(input$boundary1.name) == 0) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary1.input(),
                        function(x) range(x$loc[, 2], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            lim4 <- if (length(input$boundary2.name) == 0) {
                NA
            } else {
                range(
                    unlist(lapply(
                        boundary2.input(),
                        function(x) range(x$loc[, 2], na.rm = TRUE)
                    )),
                    na.rm = TRUE
                )
            }
            r <- c(lim1, lim2, lim3, lim4)
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            r
        })

        limits <- shiny::reactiveValues(
            input.xlim = c(0, 1),
            input.ylim = c(0, 1),
            inputplot.xlim = c(0, 1),
            inputplot.ylim = c(0, 1)
        )
        observe({
            lim1 <- userinput.xlim()
            lim2 <- if (random.loc.usage$boundary || random.loc.usage$mesh) {
                if (is.null(loc())) NA else range(coordinates(loc())[, 1], na.rm = TRUE)
            } else {
                NA
            }
            r <- c(lim1, lim2)
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            limits$input.xlim <- r

            lim1 <- userinput.ylim()
            lim2 <- if (random.loc.usage$boundary || random.loc.usage$mesh) {
                if (is.null(loc())) NA else range(coordinates(loc())[, 2], na.rm = TRUE)
            } else {
                NA
            }
            r <- c(lim1, lim2)
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            limits$input.ylim <- r
        })
        observe({
            limits$inputplot.xlim <- limits$input.xlim + c(-1, 1) * input$offset[2]
            limits$inputplot.ylim <- limits$input.ylim + c(-1, 1) * input$offset[2]
        })

        xlim <- shiny::reactive({
            lim1 <- if (is.null(mesh())) NA else range(mesh()$loc[, 1], na.rm = TRUE)
            r <- c(lim1)
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            r
        })
        ylim <- shiny::reactive({
            lim1 <- if (is.null(mesh())) NA else range(mesh()$loc[, 2], na.rm = TRUE)
            r <- lim1
            if (all(is.na(r))) {
                r <- c(0, 1)
            } else {
                r <- range(r, na.rm = TRUE)
            }
            r
        })

        inputplot.expr <- quote({
            plot(
                NA,
                type = "n", main = "",
                xlim = limits$inputplot.xlim, ylim = limits$inputplot.ylim,
                xlab = "Easting", ylab = "Northing",
                asp = 1
            )
            points(mesh.loc(), col = 2)
            points(boundary.loc(), col = 4)
            bnd <- boundary()
            for (k in 1:2) {
                if (inherits(bnd[[k]], "inla.mesh.segment")) {
                    lines(boundary()[[k]], col = k, add = TRUE)
                } else if (inherits(bnd[[k]], "list")) {
                    lapply(boundary()[[k]], function(x) lines(x, col = k, add = TRUE))
                }
            }
        })

        output$inputplot <-
            renderPlot(inputplot.expr, quoted = TRUE, height = 800, units = "px")

        output$meshmeta <- renderTable(
            {
                meshmeta()
            },
            digits = 3,
            rownames = TRUE
        )

        clicks <- shiny::reactiveValues(meshplot.loc = NULL)
        shiny::observeEvent(input$meshplot.click, {
            if (shiny::isolate(debug$trace)) message("meshplot.loc")
            if (is.null(input$meshplot.click)) {
                clicks$meshplot.loc <- NULL
            } else {
                clicks$meshplot.loc <- cbind(input$meshplot.click$x, input$meshplot.click$y)
            }
            if (shiny::isolate(debug$trace)) {
                message(paste(
                    "meshplot.loc: meshplot.loc =",
                    paste(clicks$meshplot.loc, collapse = ", ")
                ))
            }
        })
        meshplot.A.mesh <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("meshplot.A.mesh")
            A <- NULL
            if (input$assess && !is.null(clicks$meshplot.loc) && !is.null(mesh())) {
                if (shiny::isolate(debug$trace)) {
                    message(paste(
                        "meshplot.A.mesh:",
                        paste0(
                            "meshplot.loc = ",
                            paste(clicks$meshplot.loc, collapse = ", ")
                        )
                    ))
                }
                A <- INLA::inla.spde.make.A(mesh(), clicks$meshplot.loc)
                if (!(sum(A) > 0)) {
                    A <- NULL
                }
            }
            A
        })
        meshplot.A.fine <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("meshplot.A.fine")
            A <- NULL
            if (input$assess && !is.null(clicks$meshplot.loc) && !is.null(fine())) {
                if (shiny::isolate(debug$trace)) {
                    message(paste(
                        "meshplot.A.fine:",
                        paste0(
                            "meshplot.loc = ",
                            paste(clicks$meshplot.loc, collapse = ", ")
                        )
                    ))
                }
                A <- INLA::inla.spde.make.A(fine(), clicks$meshplot.loc)
                if (!(sum(A) > 0)) {
                    A <- NULL
                }
            }
            A
        })

        click.information <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("click.information")
            if (input$assess.advanced) {
                if (is.null(clicks$meshplot.loc) ||
                    is.null(meshplot.A.mesh()) ||
                    is.null(meshplot.A.fine())) {
                    out <- rbind(cbind(NA, NA, NA), cbind(NA, NA, NA), cbind(NA, NA, NA))
                } else {
                    if (shiny::isolate(debug$trace)) message("click.information!")
                    A.mesh <- meshplot.A.mesh()
                    A.fine <- meshplot.A.fine()
                    sd.bound.mesh <- as.vector(A.mesh %*% diag(mesh.S())^0.5)
                    sd.bound.fine <- as.vector(A.fine %*% diag(fine.S())^0.5)
                    sd.mesh <- Matrix::rowSums(A.mesh * (A.mesh %*% mesh.S()))^0.5
                    sd.fine <- Matrix::rowSums(A.fine * (A.fine %*% fine.S()))^0.5
                    out <- rbind(
                        cbind(sd.mesh, sd.fine, sd.mesh / sd.fine),
                        cbind(
                            sd.bound.mesh,
                            sd.bound.fine,
                            sd.bound.mesh / sd.bound.fine
                        ),
                        cbind(
                            sd.mesh / sd.bound.mesh,
                            sd.fine / sd.bound.fine,
                            sd.mesh / sd.bound.mesh / (sd.fine / sd.bound.fine)
                        )
                    )
                }
                rownames(out) <- c("SD", "SD bound", "SD/bound")
                colnames(out) <- c("Mesh", "Fine", "Mesh/Fine")
                out <- as.data.frame(out)
            } else {
                if (is.null(clicks$meshplot.loc) ||
                    is.null(meshplot.A.mesh())) {
                    out <- rbind(NA, NA, NA)
                } else {
                    if (shiny::isolate(debug$trace)) message("click.information!")
                    A.mesh <- meshplot.A.mesh()
                    sd.bound.mesh <- as.vector(A.mesh %*% diag(mesh.S())^0.5)
                    sd.mesh <- Matrix::rowSums(A.mesh * (A.mesh %*% mesh.S()))^0.5
                    out <- rbind(sd.mesh, sd.bound.mesh, sd.mesh / sd.bound.mesh)
                }
                rownames(out) <- c("SD", "SD bound", "SD/bound")
                colnames(out) <- c("Mesh")
                out <- as.data.frame(out)
            }
            out
        })
        SD.information <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("SD.information")
            if (!input$assess) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("SD.information!")
                if (input$assess.advanced) {
                    out <- matrix(NA, 3, 5)
                } else {
                    out <- matrix(NA, 1, 5)
                }
                val <- as.vector(mesh.sd())
                if (!is.null(val)) {
                    out[1, 1] <- min(val, na.rm = TRUE)
                    out[1, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                    out[1, 5] <- max(val, na.rm = TRUE)
                    if (input$assess.advanced) {
                        val <- as.vector(fine.sd())
                        out[2, 1] <- min(val, na.rm = TRUE)
                        out[2, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                        out[2, 5] <- max(val, na.rm = TRUE)
                        val <- as.vector(mesh.sd() / fine.sd())
                        out[3, 1] <- min(val, na.rm = TRUE)
                        out[3, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                        out[3, 5] <- max(val, na.rm = TRUE)
                    }
                }
                rnames <- c(
                    "SD on Mesh",
                    "SD on Fine",
                    "SD relative deviation"
                )
                if (input$assess.advanced) {
                    rownames(out) <- rnames
                } else {
                    rownames(out) <- rnames[1]
                }
                colnames(out) <- c("min", "25%", "50%", "75%", "max")
                out <- as.data.frame(out)
                out
            }
        })
        corr.information <- shiny::reactive({
            if (shiny::isolate(debug$trace)) message("corr.information")
            if (!input$assess) {
                NULL
            } else {
                if (shiny::isolate(debug$trace)) message("corr.information!")
                if (input$assess.advanced) {
                    out <- matrix(NA, 3, 5)
                } else {
                    out <- matrix(NA, 1, 5)
                }
                val <- as.vector(mesh.corr())
                if (!is.null(val)) {
                    out[1, 1] <- min(val, na.rm = TRUE)
                    out[1, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                    out[1, 5] <- max(val, na.rm = TRUE)
                    if (input$assess.advanced) {
                        val <- as.vector(fine.corr())
                        out[2, 1] <- min(val, na.rm = TRUE)
                        out[2, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                        out[2, 5] <- max(val, na.rm = TRUE)
                        val <- as.vector(mesh.corr() - fine.corr())
                        out[3, 1] <- min(val, na.rm = TRUE)
                        out[3, 2:4] <- quantile(val, c(0.25, 0.5, 0.75), na.rm = TRUE)
                        out[3, 5] <- max(val, na.rm = TRUE)
                    }
                }
                rnames <- c(
                    "Correlation on Mesh",
                    "Correlation on Fine",
                    "Correlation deviation"
                )
                if (input$assess.advanced) {
                    rownames(out) <- rnames
                } else {
                    rownames(out) <- rnames[1]
                }
                colnames(out) <- c("min", "25%", "50%", "75%", "max")
                out <- as.data.frame(out)
                out
            }
        })

        output$click.information <- renderTable(
            {
                if (input$assess) {
                    click.information()
                }
            },
            digits = 3,
            rownames = TRUE
        )
        output$field.information <- renderTable(
            {
                if (input$assess.quantity == "corr") {
                    corr.information()
                } else {
                    SD.information()
                }
            },
            digits = 3,
            rownames = TRUE
        )

        output$code <- shiny::renderText({
            out <- "## Generated by meshbuilder()\n\n"
            spacing <- ""
            if (random.loc.usage$boundary || random.loc.usage$mesh) {
                out <- paste0(
                    out,
                    "## Prepare seed points:\n",
                    attr(loc(), "code")
                )
                spacing <- "\n"
            }
            if (!is.null(boundary())) {
                out <- paste0(
                    out, spacing,
                    "## Build boundary information:\n",
                    "## (fmesher supports SpatialPolygons, but this",
                    " app is not (yet) intelligent enough for that.)\n",
                    attr(boundary.loc(), "code"),
                    attr(boundary(), "code")
                )
                spacing <- "\n"
            }
            out <- paste0(
                out, spacing,
                "## Build the mesh:\n",
                attr(mesh.loc(), "code"),
                attr(mesh(), "code")
            )
            spacing <- "\n"
            out <- paste0(
                out, spacing,
                "## Plot the mesh:\n",
                "plot(mesh)\n"
            )
            if (input$assess) {
                if (input$assess.advanced) {
                    out <- paste0(
                        out, spacing, spacing,
                        "## Build a fine scale mesh for comparisons:\n",
                        attr(fine(), "code")
                    )
                    out <- paste0(
                        out, spacing,
                        "## Further assessment code is not generated",
                        " in this version of meshbuilder()\n"
                    )
                } else {
                    out <- paste0(
                        out, spacing, spacing,
                        "## Assessment code is not generated",
                        " in this version of meshbuilder()\n"
                    )
                }
            }
            out
        })

        output$crs.strings <- shiny::renderText({
            add_wkt <- function(crs_list, x) {
                crs <- inla.sp_get_crs(x)
                wkt <- inla.crs_get_wkt(crs)
                if (!is.null(wkt)) {
                    c(crs_list, wkt)
                } else {
                    crs_list
                }
            }
            crs <- c()
            crs <- add_wkt(crs, inla.CRS(input$crs.mesh))
            crs <- add_wkt(crs, mesh.loc())
            crs <- add_wkt(crs, boundary.loc())
            for (sp in seq_along(boundary1.input())) {
                crs <- add_wkt(crs, sp)
            }
            for (sp in seq_along(boundary2.input())) {
                crs <- add_wkt(crs, sp)
            }
            crs <- unique(crs)
            out <- paste0(crs, collapse = "\n")
            out
        })

        output$input.loc.coordinates <- shiny::renderText({
            xy_str <- function(e) {
                if (is.null(e)) {
                    "NULL\n"
                } else {
                    paste0("x=", e$x, ", y=", e$y, "\n")
                }
            }
            paste0(xy_str(input$inputplot.hover))
        })
        output$meshplot.loc.coordinates <- shiny::renderText({
            xy_str <- function(e) {
                if (is.null(e)) {
                    "NULL\n"
                } else {
                    paste0("x=", e[1, 1], ", y=", e[1, 2], "\n")
                }
            }
            paste0(xy_str(clicks$meshplot.loc))
        })




        debug <- shiny::reactiveValues(trace = FALSE, plot.delay = NULL)
        observe({
            if (input$debug.use.trace) {
                debug$trace <- TRUE
            } else {
                debug$trace <- FALSE
            }
        })
        observe({
            if (input$debug.use.plot.delay) {
                debug$plot.delay <- 0
            } else {
                debug$plot.delay <- NULL
            }
        })

        output$debug <- shiny::renderText({
            out <- paste0("boundary.loc.name: ", current$boundary.loc.name)
            out <- paste0(
                out, "\n",
                "mesh.loc.name: ", current$mesh.loc.name
            )
            out <- paste0(
                out, "\n",
                "input$boundary.loc.name: ", input$boundary.loc.name
            )
            out <- paste0(
                out, "\n",
                "input$mesh.loc.name: ", input$mesh.loc.name
            )
            out <- paste0(
                out, "\n",
                "boundary1.name: ", input$boundary1.name
            )
            out <- paste0(
                out, "\n",
                "boundary2.name: ", input$boundary2.name
            )
            out
        })
    }

    shiny::shinyApp(ui = meshbuilder.ui, server = meshbuilder.server)
}


#' @title Interactive mesh building and diagnostics
#'
#' @description Interactively design and build a triangle mesh
#' for use with SPDE models, and assess the finite element
#' approximation errors. The R code needed to recreate the mesh
#' outside the interactive Shiny app is also generated. Spatial
#' objects can be imported from the global workspace.
#'
#' @author Finn Lindgren \email{finn.lindgren@gmail.com}
#' @seealso [inla.mesh.2d()], [inla.mesh.create()]
#' @examples
#' \dontrun{
#' meshbuilder()
#' }
#' @export

meshbuilder <- function() {
    inla.require("fields", stop.on.error = TRUE)
    inla.require("shiny", stop.on.error = TRUE)

    if (inla.has_PROJ6()) {
        rgdal::set_thin_PROJ6_warnings(TRUE)
    }

    shiny::runApp(meshbuilder.app())
}

## Export: inla.qsolve

## ! \name{qsolve}
## ! \alias{inla.qsolve}
## ! \alias{qsolve}
## !
## ! \title{Solves linear SPD systems}
## !
## ! \description{This routine use the GMRFLib implementation
## !              to solve linear systems with a SPD matrix.}
## ! \usage{
## !     inla.qsolve(Q, B, reordering = inla.reorderings(), method = c("solve", "forward", "backward"))
## ! }
## !
## ! \arguments{
## !   \item{Q}{A SPD matrix,  either as a (dense) matrix or sparse-matrix}.
## !   \item{B}{The right hand side matrix, either as a (dense) matrix or  sparse-matrix.}
## !   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
## !        either one of the names listed in \code{inla.reorderings()}
## !        or the output from \code{inla.qreordering(Q)}.
## !        The default is "auto" which try several reordering
## !        algorithm and use the best one for this particular matrix (using the TAUCS library).}
## !   \item{method}{The system to solve, one of \code{"solve"},
## !                 \code{"forward"} or \code{"backward"}. Let \code{Q = L L^T},
## !                 where \code{L} is lower triangular
## !                 (the Cholesky triangle),  then \code{method="solve"} solves \code{L L^T X = B} or
## !                  equivalently \code{Q X = B},  \code{method="forward"} solves \code{L X = B},  and
## !                 \code{method="backward"} solves \code{L^T X  = B}. }
## !}
## !\value{
## !  \code{inla.qsolve} returns a matrix \code{X},
## !  which is the solution of \code{Q X = B},  \code{L X = B} or \code{L^T X = B}
## !  depending on the value of \code{method}.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## ! n = 10
## ! nb <- n-1
## ! QQ = matrix(rnorm(n^2), n, n)
## ! QQ <- QQ \%*\% t(QQ)
## !
## ! Q = inla.as.sparse(QQ)
## ! B = matrix(rnorm(n*nb), n, nb)
## !
## ! X = inla.qsolve(Q, B, method = "solve")
## ! XX = inla.qsolve(Q, B, method = "solve", reordering = inla.qreordering(Q))
## ! print(paste("err solve1", sum(abs( Q \%*\% X - B))))
## ! print(paste("err solve2", sum(abs( Q \%*\% XX - B))))
## !
## ! ## the forward and backward solve is tricky, as after permutation and with Q=LL', then L is
## ! ## lower triangular, but L in the orginal ordering is not lower triangular. if the rhs is iid
## ! ## noise, this is not important. to control the reordering, then the 'taucs' library must be
## ! ## used.
## ! inla.setOption(smtp = 'taucs')
## !
## ! ## case 1. use the matrix as is, no reordering
## ! r <- "identity"
## ! L = t(chol(Q))
## ! X = inla.qsolve(Q, B, method = "forward", reordering = r)
## ! XX = inla.qsolve(Q, B, method = "backward", reordering = r)
## ! print(paste("err forward ", sum(abs(L \%*\% X - B))))
## ! print(paste("err backward", sum(abs(t(L) \%*\% XX - B))))
## !
## ! ## case 2. use a reordering from the library
## ! r <- inla.qreordering(Q)
## ! im <- r$ireordering
## ! m <- r$reordering
## ! print(cbind(idx = 1:n, m, im) )
## ! Qr <- Q[im, im]
## ! L = t(chol(Qr))[m, m]
## !
## ! X = inla.qsolve(Q, B, method = "forward", reordering = r)
## ! XX = inla.qsolve(Q, B, method = "backward", reordering = r)
## ! print(paste("err forward ", sum(abs( L \%*\% X - B))))
## ! print(paste("err backward", sum(abs( t(L) \%*\% XX - B))))
## !}

`inla.qsolve` <- function(Q, B, reordering = inla.reorderings(),
                          method = c("solve", "forward", "backward")) {
    t.dir <- inla.tempdir()
    smtp <- match.arg(inla.getOption("smtp"), c("taucs", "band", "default", "pardiso"))
    Q <- inla.sparse.check(Q)
    if (is(Q, "dgTMatrix")) {
        Qfile <- inla.write.fmesher.file(Q, filename = inla.tempfile(tmpdir = t.dir))
    } else if (is.character(Q)) {
        Qfile <- Q
    } else {
        stop("This should not happen.")
    }

    ## ensure B is a dense matrix if its a sparse
    if (is(B, "Matrix")) {
        B <- as.matrix(B)
    }
    if (is.matrix(B)) {
        Bfile <- inla.write.fmesher.file(B, filename = inla.tempfile(tmpdir = t.dir))
    } else if (is.character(B)) {
        Bfile <- B
    } else {
        stop("This should not happen.")
    }

    if (is.list(reordering)) {
        ## argument is the output from inla.qreordering()
        reordering <- reordering$name
    }
    reordering <- match.arg(reordering)
    method <- match.arg(method)

    Xfile <- inla.tempfile(tmpdir = t.dir)
    inla.set.sparselib.env(inla.dir = t.dir)
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qsolve", "-r",
            reordering, "-S", smtp, Qfile, Xfile, Bfile, method
        ), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qsolve", "-r",
            reordering, "-S", smtp, Qfile, Xfile, Bfile, method
        ), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    X <- inla.read.fmesher.file(Xfile)
    unlink(t.dir, recursive = TRUE)

    return(X)
}

## Export: inla.ks.plot

## !\name{inla.ks.plot}
## !\alias{inla.ks.plot}
## !\alias{ks.plot}
## !\title{
## !  Kolmogorov-Smirnov Test Plots
## !}
## !\description{
## !  Illustrate a one-sample Kolmogorov-Smirnov test by plotting the
## !  empirical distribution deviation.
## !}
## !\usage{
## !inla.ks.plot(x, y, diff=TRUE, ...)
## !}
## !\arguments{
## !  \item{x}{a numeric vector of data values.}
## !  \item{y}{a cumulative distribution function such as 'pnorm'.}
## !  \item{diff}{logical, indicating if the normalised difference should be
## !    plotted.  If \code{FALSE}, the absolute distribution functions are plotted.}
## !  \item{...}{additional arguments for \code{\link{ks.test}}, ignored in
## !    the plotting.  In particular, only two-sided tests are illustrated.}
## !}
## !\details{
## !  In addition to the (normalised) empirical distribution deviation,
## !  lines for the K-S test statistic are drawn, as well as \eqn{\pm}{+/-}
## !  two standard deviations around the expectation under the null hypothesis.
## !}
## !\value{
## !  A list with class \code{"htest"}, as generated by
## !  \code{\link{ks.test}}
## !}
## !%%\references{
## !%%}
## !\author{
## !  Finn Lindgren \email{finn.lindgren@gmail.com}
## !}
## !%%\note{
## !%%}
## !
## !%% ~Make other sections like Warning with \section{Warning }{....} ~
## !
## !\seealso{
## !  \code{\link{ks.test}}
## !}
## !\examples{
## !## Check for N(0,1) data
## !data = rowSums(matrix(runif(100*12)*2-1,100,12))/2
## !inla.ks.plot(data, pnorm)
## !
## !\dontrun{
## !## Check the goodness-of-fit of cross-validated predictions
## !result = inla(..., control.predictor=list(cpo=TRUE))
## !inla.ks.plot(result$pit, punif)
## !}
## !}



## One-sample Kolmogorov-Smirnov test, plotting the normalised
## deviation between the empirical distribution function and the null
## hypothesis.
##
## Example:
## result = inla(..., control.compute=list(cpo=TRUE))
## inla.ks.plot(result$pit, punif)
inla.ks.plot <- function(x, y, diff = TRUE, ...) {
    if (any(is.na(x))) {
        x <- x[!is.na(x)]
    }
    test <- ks.test(x, y, ...)
    n <- length(x)
    Fn <- ((1:n) - 0.5) / n
    F <- y(sort(x))
    empirical.diff <- (Fn - F) * sqrt(n)
    T <- max(abs(empirical.diff))
    if (diff) {
        ylim <- c(-1, 1) * max(1, T)
        plot(F, empirical.diff,
            type = "l",
            xlim = c(0, 1),
            ylim = ylim,
            main = paste("K-S-test, p-value = ", test$p.value),
            ylab = "(Fn-F) sqrt(n)",
            xlab = "Quantile"
        )
        lines(c(0, 1), c(0, 0), type = "l")
        lines(Fn, 2 * sqrt(Fn * (1 - Fn)), type = "l")
        lines(Fn, -2 * sqrt(Fn * (1 - Fn)), type = "l")
        lines(c(0, 1, NA, 0, 1), c(T, T, NA, -T, -T), type = "l")
    } else {
        plot(F, Fn,
            type = "l",
            xlim = c(0, 1),
            ylim = c(0, 1),
            main = paste("K-S-test, p-value = ", test$p.value),
            ylab = "Fn and F",
            xlab = "Quantile"
        )
        lines(c(0, 1), c(0, 1), type = "l")
        lines(Fn, Fn + 2 * sqrt(Fn * (1 - Fn) / n), type = "l")
        lines(Fn, Fn - 2 * sqrt(Fn * (1 - Fn) / n), type = "l")
        lines(c(0, 1, NA, 0, 1), c(0, 1, NA, 0, 1) + c(T, T, NA, -T, -T) / sqrt(n), type = "l")
    }
    invisible(test)
}

## Export: inla.nmix.lambda.fitted


### The inla.nmix.lambda.fitted() function is a helper function for taking an 'nmix' or 'nmixnb'
### model, and computing expected abundance values for each site or site-by-year combination in
### the data, using the linear predictor for ## lambda (hereafter, 'fitted values').
###
### The uncertainty associated with fitted values derives from repeated sampling of INLA
### posteriors for the parameters of the linear predictor, and repeated solving of the linear
### predictor equation for each site or site-by-year combination. By default, fitted values are
### exponentiated to the count scale, and summaries of approximate fitted posteriors are returned
### by the function. Full estimated posteriors of fitted values are also available as an optional
### output.
###
### Tim Meehan <tmeehan@audubon.org>, 16 February 2018.

## !\name{inla.nmix.lambda.fitted}
## !\alias{inla.nmix.lambda.fitted}
## !\alias{nmix.lambda.fitted}
## !\alias{inla.nmix.fitted}
## !
## !\title{
## !  Estimate posterior distributions of fitted lambda values
## !}
## !
## !\description{For use with \code{'nmix'} and \code{'nmixnb'} models. This function takes the
## !    information contained in an object returned by \code{inla()} and uses the contents to create
## !    fitted lambda values using the linear predictor for log(lambda), the input covariate values,
## !    and samples from the posteriors of the model hyperparameters. Fitted values from the linear
## !    predictor are exponentiated, by default, before being returned.}
## !
## !\usage{
## !inla.nmix.lambda.fitted(result, sample.size = 1000,
## !                        return.posteriors = FALSE, scale = "exp")
## !}
## !
## !\arguments{
## !    \item{result}{The output object from a call to \code{inla()}, where the family argument has
## !  been set to \code{'nmix'} or \code{'nmixnb'}. For the function to work, the call to
## !  \code{inla()} should also include the argument \code{control.compute=list(config = TRUE))}.}
## !
## !  \item{sample.size}{The size of the sample from the posteriors of the model hyperparameters.
## !  This sample size ends up being the size of the estimated posterior for a fitted lambda value.
## !  Default is 1000. Larger values are recommended.}
## !
## !  \item{return.posterior}{A logical value for whether or not to return the full estimated
## !  posteriors for each fitted value (\code{TRUE}), or just a summary of the posteriors
## !  (\code{FALSE}). Default is \code{FALSE}.}
## !
## !  \item{scale}{A character string, where the default string, \code{"exp"}, causes values from
## !  the linear predictor to be exponentiated before being returned. The string, \code{"log"},
## !  causes values to be returned on the \code{log(lambda)} scale.}
## !}
## !
## !\value{
## !    \item{fitted.summary}{A data frame with summaries of estimated posteriors of fitted lambda
## !  values. The number of rows equals the number of rows in the data used to create the
## !  \code{'nmix'} or \code{'nmixnb'} model. There are six columns of summary statistics for each
## !  estimated posterior. Columns include an \code{index}, \code{mean.lambda}, \code{sd.lambda},
## !  \code{quant025.lambda}, \code{median.lambda}, \code{quant975.lambda}, and \code{mode.lambda}.}
## !
## !  \item{fitted.posteriors}{A data frame containing samples that comprise the full estimated
## !  posteriors of fitted values. The number of rows equals the number of rows in the data used to
## !  create the \code{'nmix'} or \code{'nmixnb'} model. The number of columns equals one plus the
## !  number of samples specified by the \code{sample.size} argument.}
## !}
## !
## !\references{
## !    See documentation for families "nmix" and "nmixmb": \code{inla.doc("nmix")}
## !}
## !\author{
## !    Tim Meehan <tmeehan@audubon.org>
## !}
## !\note{
## !    This function is experimental.
## !}
## !
## !\examples{
## !## an example analysis of an N-mixture model using simulated data
## !## set parameters
## !n <- 75                       # number of study sites
## !nrep.max <- 5                 # number of surveys per site
## !b0 <- 0.5                     # lambda intercept, expected abundance
## !b1 <- 2.0                     # effect of x1 on lambda
## !a0 <- 1.0                     # p intercept, detection probability
## !a2 <- 0.5                     # effect of x2 on p
## !size <- 3.0                   # size of theta
## !overdispersion <- 1 / size    # for negative binomial distribution
## !
## !## make empty vectors and matrix
## !x1 <- c(); x2 <- c()
## !lambdas <- c(); Ns <- c()
## !y <- matrix(NA, n, nrep.max)
## !
## !## fill vectors and matrix
## !for(i in 1:n) {
## !    x1.i <- runif(1) - 0.5
## !    lambda <- exp(b0 + b1 * x1.i)
## !    N <- rnbinom(1, mu = lambda, size = size)
## !    x2.i <- runif(1) - 0.5
## !    eta <- a0 + a2 * x2.i
## !    p <- exp(eta) / (exp(eta) + 1)
## !    nr <- sample(1:nrep.max, 1)
## !    y[i, 1:nr] <- rbinom(nr, size = N, prob = p)
## !    x1 <- c(x1, x1.i); x2 <- c(x2, x2.i)
## !    lambdas <- c(lambdas, lambda); Ns <- c(Ns, N)
## !}
## !
## !## bundle counts, lambda intercept, and lambda covariates
## !Y <- inla.mdata(y, 1, x1)
## !
## !## run inla and summarize output
## !result <- inla(Y ~ 1 + x2,
## !  data = list(Y=Y, x2=x2),
## !  family = "nmixnb",
## !  control.fixed = list(mean = 0, mean.intercept = 0, prec = 0.01,
## !                      prec.intercept = 0.01),
## !  control.family = list(hyper = list(theta1 = list(param = c(0, 0.01)),
## !                                    theta2 = list(param = c(0, 0.01)),
## !                                    theta3 = list(prior = "flat",
## !                                                 param = numeric()))),
## !  control.compute=list(config = TRUE)) # important argument
## !summary(result)
## !
## !## get and evaluate fitted values
## !lam.fits <- inla.nmix.lambda.fitted(result, 5000)$fitted.summary
## !plot(lam.fits$median.lambda, lambdas)
## !round(sum(lam.fits$median.lambda), 0); sum(Ns)
## !}


inla.nmix.lambda.fitted <- function(result, sample.size = 1000,
                                    return.posteriors = FALSE,
                                    scale = "exp") {
    fam <- result$.args$family
    if (length(grep(pattern = "nmix", x = fam)) == 0) {
        stop("This function is only for models with 'nmix' or 'nmixnb' likelihoods")
    }
    if (missing(result)) {
          stop("Please specify a model result")
      }
    s.check <- as.numeric(scale == "exp") + as.numeric(scale == "log")
    if (s.check == 0) {
          stop("Scale must be set to 'exp' or 'log'")
      }
    if (sample.size < 500) {
          warning("Please increase the sample size")
      }

    ## Get counts and lambda covariates from 'inla.mdata' object
    mdata.obj <- result$.args$data[[1]]
    counts <- as.data.frame(mdata.obj[grep(pattern = "Y", names(mdata.obj))])
    lambda.covs <- as.data.frame(mdata.obj[grep(pattern = "X", names(mdata.obj))])
    lambda.covs <- as.matrix(lambda.covs)
    n.counts <- ncol(counts)
    n.lambda.covs <- ncol(lambda.covs)
    n.data <- nrow(counts)

    ## Get samples from hyperpars
    hyperpar.samples <- inla.hyperpar.sample(sample.size, result)
    s.names <- rownames(hyperpar.samples)

    ## Discard overdispersion marginal posterior if 'nmixnb'
    if (fam == "nmixnb") {
        hyperpar.samples <- hyperpar.samples[, -(ncol(hyperpar.samples))]
    }
    n.samp.covs <- ncol(hyperpar.samples)
    if (n.lambda.covs != n.samp.covs) {
        stop("The number of hyperparameters and covariates does not match")
    }

    ## Combine lambda covariates and hyperparameter posteriors
    fitted.posteriors <- matrix(-1.0000, nrow = n.data, ncol = sample.size)
    ## For each site or site-by-year combination
    for (i in 1:n.data) {
        obs <- lambda.covs[i, ]
        ## For each sample from the hyperparameter posterior
        for (j in 1:sample.size) {
            post <- hyperpar.samples[j, ]
            fitted <- sum(obs * post)
            fitted.posteriors[i, j] <- fitted
        }
    }

    ## Clean up the resulting matrix
    index <- 1:n.data
    fitted.posteriors <- as.data.frame(fitted.posteriors)
    row.names(fitted.posteriors) <- NULL
    names(fitted.posteriors) <- s.names

    ## Create posterior summaries of fitted values
    if (scale == "exp") {
        fitted.posteriors <- exp(fitted.posteriors)
    }
    fitted.meds <- round(apply(fitted.posteriors, 1, median), 4)
    fitted.means <- round(apply(fitted.posteriors, 1, mean), 4)
    fitted.sds <- round(apply(fitted.posteriors, 1, sd), 4)
    fitted.q025 <- round(apply(fitted.posteriors, 1, quantile, probs = 0.025), 4)
    fitted.q500 <- round(apply(fitted.posteriors, 1, quantile, probs = 0.500), 4)
    fitted.q975 <- round(apply(fitted.posteriors, 1, quantile, probs = 0.975), 4)
    Mode <- function(x) {
        ux <- unique(x)
        ux[which.max(tabulate(match(x, ux)))]
    }
    fitted.modes <- round(apply(fitted.posteriors, 1, Mode), 4)
    fitted.summary <- data.frame(
        mean.lambda = fitted.means, sd.lambda = fitted.sds,
        quant025.lambda = fitted.q025,
        median.lambda = fitted.q500,
        quant975.lambda = fitted.q975,
        mode.lambda = fitted.modes
    )
    fitted.summary <- cbind(index, fitted.summary)
    fitted.posteriors <- cbind(index, fitted.posteriors)

    ## Create returned object
    if (return.posteriors == TRUE) {
        out <- list(
            fitted.summary = fitted.summary,
            fitted.posteriors = fitted.posteriors
        )
    } else {
        out <- list(fitted.summary = fitted.summary)
    }
    return(out)
}

## Export: inla.priors.used

## !\name{inla.priors.used}
## !\alias{inla.priors.used}
## !\alias{priors.used}
## !\title{Print priors used}
## !\description{Print the priors used for the hyperparameters}
## !\usage{
## !    inla.priors.used(result, digits=6L)
## !}
## !\arguments{
## !\item{result}{An \code{inla}-object, typically the output from an \code{inla()}-call}
## !\item{digits}{The \code{digits} argument to the function \code{format()}}
## !}
## !\details{
## !This function provides a more human-friendly output of \code{result$all.hyper}
## !of all the priors used for the hyperparameters.
## !Since not all information about the model is encoded in this object, more hyperparameters
## !than actually used, may be printed. In particular, \code{group.theta1} is printed
## !even though the argument \code{group} in \code{f()} is not used. Similarly for spde-models,
## !but the user should know that, for example, only the two first ones
## !are actually used.
## !Hopefully, this issue will be fixed in the future.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\examples{
## !r = inla(y ~ 1 + x, data = data.frame(y = 1:10, x = rep(1:5, 2)))
## !inla.priors.used(r)
## !}

`inla.priors.used` <- function(result, digits = 6L) {
    p2char <- function(p) {
        return(paste(as.character(format(p, digits = digits)), sep = "", collapse = ", "))
    }

    mvnorm.n <- function(len) {
        for (n in 1:300) {
            if (len == n + n^2) {
                  return(n)
              }
        }
        stop(paste0("This should not happen: Fail to find 'n' from len=", len))
    }

    mvnorm.prior.print <- function(param) {
        n <- mvnorm.n(length(param))
        m <- matrix(param[1:n], nrow = n, ncol = 1)
        P <- matrix(param[-(1:n)], nrow = n, ncol = n)
        for (i in 1:n) {
            cat("\t\t\t[", format(m[i], digits = digits), "]  [", sep = "")
            for (j in 1:n) {
                  cat(format(P[i, j], digits = digits), " ", sep = "")
              }
            cat("]\n", sep = "")
        }
    }

    h4cat <- function(h4, pre = NULL, name = "theta") {
        nh <- 0
        if (h4$fixed == FALSE) {
            nh <- 1
            cat("\t\t",
                if (!is.null(pre)) paste0(pre, ".") else "",
                name, "", j, ":", "\n",
                "\t\t\t", "parameter=[", h4$name, "]", "\n",
                "\t\t\t", "prior=[", h4$prior, "]", "\n",
                "\t\t\t", "param=[", p2char(h4$param), "]", "\n",
                sep = ""
            )
            if (FALSE) {
                if (h4$prior %in% "mvnorm") {
                      mvnorm.prior.print(h4$param)
                  }
            }
        }
        return (nh)
    }

    stopifnot(class(result) == "inla")
    h <- result$all.hyper
    ntheta <- length(result$mode$theta)

    count.theta <- 0
    for (nm in names(h)) {
        h2 <- h[[nm]]
        ## this will not print out if predictor prec is fixed=FALSE.
        if (!(nm %in% c("predictor"))) {
            cat("section=[", nm, "]", "\n", sep = "")
            for (i in seq_along(h2)) {
                h3 <- h2[[i]]
                if (is.null(h3$label)) h3$label <- h3$hyperid
                if (is.null(h3$hyperid)) h3$hyperid <- unlist(h3$label)
                cat("\t", "tag=[", h3$hyperid, "] component=[", unlist(h3$label),
                    "]", "\n",
                    sep = ""
                )
                if (nm %in% "fixed") {
                    ## this is a special case
                    h3$hyper <- list(list(
                        name = h3$label,
                        prior = "normal",
                        param = c(h3$prior.mean, h3$prior.prec),
                        fixed = FALSE
                    ))
                    h4cat(h3$hyper[[1]], name = "beta")
                } else {
                    for (j in seq_along(h3$hyper)) {
                        if (count.theta >= ntheta) break
                        count.theta <- count.theta + h4cat(h3$hyper[[j]])
                    }
                }
                for (j in seq_along(h3$link$hyper)) {
                    if (count.theta >= ntheta) break
                    count.theta <- count.theta + h4cat(h3$link$hyper[[j]], "link")
                }
                for (j in seq_along(h3$group.hyper)) {
                    if (count.theta >= ntheta) break
                    count.theta <- count.theta + h4cat(h3$group.hyper[[j]], "group")
                }
                for (j in seq_along(h3$mix$hyper)) {
                    if (count.theta >= ntheta) break
                    count.theta <- count.theta + h4cat(h3$mix$hyper[[j]], "mix")
                }
            }
        }
    }
}

#' @include spde.common.R
NULL

#' Generic spde2 model creation.
#' 
#' Creates and inla.spde2 object describing the internal structure of an
#' `'spde2'` model.
#' 
#' 
#' @aliases inla.spde2.generic inla.spde2 inla.spde2.theta2phi0
#' inla.spde2.theta2phi1 inla.spde2.theta2phi2
#' @param M0 The symmetric `M0` matrix.
#' @param M1 The square `M1` matrix.
#' @param M2 The symmetric `M2` matrix.
#' @param B0 Basis definition matrix for \eqn{\phi_0}{phi0}.
#' @param B1 Basis definition matrix for \eqn{\phi_2}{phi1}.
#' @param B2 Basis definition matrix for \eqn{\phi_2}{phi2}.
#' @param theta.mu Prior expectation for the \eqn{\theta}{theta} vector
#' @param theta.Q Prior precision for the \eqn{\theta}{theta} vector
#' @param transform Transformation link for \eqn{\phi_2}{phi2}.  Valid settings
#' are `"logit"`, `"log"`, and `"identity"`
#' @param theta.initial Initial value for the \eqn{\theta}{theta} vector.
#' Default `theta.mu`
#' @param fixed Logical vector.  For every `TRUE` value, treat the
#' corresponding `theta` value as known.
#' @param theta.fixed Vector holding the values of fixed `theta` values.
#' Default `=theta.initial[fixed]`
#' @param BLC Basis definition matrix for linear combinations of `theta`.
#' @param \dots Additional parameters, currently unused.
#' @param spde An `inla.sdpe2` object.
#' @param theta parameter values to be mapped.
#' @return For `inla.spde2.generic`, an [inla.spde2()] object.
#' 
#' For `inla.spde2.theta2phi0/1/2`, a vector of \eqn{\phi}{phi} values.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde2.models()], [inla.spde2.matern()]
#' @export inla.spde2.generic
inla.spde2.generic =
    function(M0, M1, M2, B0, B1, B2, theta.mu, theta.Q,
             transform = c("logit","log","identity"),
             theta.initial = theta.mu,
             fixed = rep(FALSE, length(theta.mu)),
             theta.fixed = theta.initial[fixed],
             BLC = cbind(0.0, diag(nrow=length(theta.mu))),
             ...)
{
    transform = match.arg(transform)

    if (!inla.is.matrix(M0)) stop("M0 must be of a matrix type.")
    if (!inla.is.matrix(M1)) stop("M1 must be of a matrix type.")
    if (!inla.is.matrix(M2)) stop("M2 must be of a matrix type.")
    M0 = inla.as.dgTMatrix(M0)
    M1 = inla.as.dgTMatrix(M1)
    M2 = inla.as.dgTMatrix(M2)

    n.spde = nrow(M0)
    n.theta = length(theta.mu)

    spde = (list(model = "generic",
                 n.spde = n.spde,
                 n.theta = n.theta,
                 param.inla = list(),
                 f = list()
                 ))
    class(spde) = c("inla.spde2", "inla.spde", "inla.model.class")

    B0 = inla.spde.homogenise_B_matrix(B0, n.spde, n.theta)
    B1 = inla.spde.homogenise_B_matrix(B1, n.spde, n.theta)
    B2 = inla.spde.homogenise_B_matrix(B2, n.spde, n.theta)
    BLC = inla.spde.homogenise_B_matrix(BLC, nrow(BLC), n.theta)

    param.inla =
        list(n = n.spde,
             n.theta = n.theta,
             M0=M0, M1=M1, M2=M2,
             B0=B0, B1=B1, B2=B2, BLC=BLC,
             theta.mu=theta.mu, theta.Q=theta.Q,
             transform=transform,
             theta.initial=theta.initial,
             fixed=fixed,
             theta.fixed=theta.fixed)
    ## TODO: Check dimension consistency of inputs

    ## If any "fixed", move information from B[,1+(1:n.theta)] into
    ## B[,1], theta.mu, and theta.Q
    ## NOTE: Should this be in the f function instead?
    ##       No, the information is also needed elsewhere.
    if (any(fixed)) {
        param.inla$theta.initial = theta.initial[!fixed]
        param.inla$B0[,1] =
            B0[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B1[,1] =
            B1[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B2[,1] =
            B2[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$BLC[,1] =
            BLC[,c(TRUE, fixed), drop=FALSE] %*% c(1.0, theta.fixed)
        param.inla$B0 = param.inla$B0[,c(TRUE, !fixed), drop=FALSE]
        param.inla$B1 = param.inla$B1[,c(TRUE, !fixed), drop=FALSE]
        param.inla$B2 = param.inla$B2[,c(TRUE, !fixed), drop=FALSE]
        param.inla$BLC = param.inla$BLC[,c(TRUE, !fixed), drop=FALSE]
        param.inla$theta.Q = param.inla$theta.Q[!fixed, !fixed, drop=FALSE]
        if (!all(fixed)) {
            param.inla$theta.mu =
                theta.mu - solve(param.inla$theta.Q,
                                 theta.Q[!fixed, fixed, drop=FALSE]
                                 %*% (theta.fixed[fixed]-theta.mu[fixed]))
        } else { ## All fixed.
            param.inla$theta.mu = as.vector(matrix(0.0, 0, 1))
        }
    }

    ## Remove all BLC rows that would produce point masses, which is
    ## not supported by inla:
    if (n.theta==0) {
        param.inla$BLC = matrix(0,0,1)
    } else {
        param.inla$BLC =
            param.inla$BLC[rowSums(abs(param.inla$BLC[,-1,drop=FALSE]))>0,,
                           drop=FALSE]
    }

    spde$param.inla = param.inla

    spde$f = (list(model="spde2",
                   n=n.spde,
                   spde2.transform=transform,
                   hyper.default =
                   (list(theta1 =
                         list(prior="mvnorm",
                              param=(c(param.inla$theta.mu,
                                       as.matrix(param.inla$theta.Q)))
                              )
                         ))
                   ))
    if (n.theta>0)
        for (k in 1:n.theta) {
            eval(parse(text=
                       paste("spde$f$hyper.default$theta", k,
                             "$initial = param.inla$theta.mu[k]", sep="")))
        }

    return(spde)
}









inla.internal.spde2.matern.B.tau =
    function(precision.fcn, n.theta, delta = 1e-3, ...)
{
    theta.0 = rep(0, n.theta)
    Q.0 = precision.fcn(theta.0, ...)
    logv.0 = log(diag(inla.qinv(Q.0)))

    B.tau = matrix(0, nrow(Q.0), 1L+n.theta)
    for (k in seq_len(n.theta)) {
        theta = theta.0
        theta[k] = theta[k] + delta
        Q = precision.fcn(theta, ...)
        logv = log(diag(inla.qinv(Q)))

        dlogv = (logv-logv.0)/delta

        B.tau[, k+1] = dlogv/2
    }
    B.tau[,1L] = logv.0/2

    return(B.tau)
}





#' Approximate variance-compensating basis functions
#' 
#' Calculates an approximate basis for `tau` and `kappa` for an
#' `inla.spde2.matern` model where `tau` is a rescaling parameter.
#' 
#' 
#' @param mesh An [inla.mesh()] object.
#' @param B.sd Desired basis for log-standard deviations.
#' @param B.range Desired basis for spatial range.
#' @param method Construction method selector.  Expert option only.
#' @param local.offset.compensation If `FALSE`, only compensate in the
#' average for the `tau` offset.
#' @param alpha The model `alpha` parameter.
#' @param \dots Additional parameters passed on to internal
#' `inla.spde2.matern` calls.
#' @return List of basis specifications \item{B.tau }{Basis for
#' `log(tau)`} \item{B.kappa }{Basis for `log(kappa)`} Intended for
#' passing on to [inla.spde2.matern()].
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde2.matern()]
#' @export inla.spde2.matern.sd.basis
inla.spde2.matern.sd.basis =
    function(mesh, B.sd, B.range, method=1,
             local.offset.compensation=FALSE,
             alpha=2, ...)
{
    p.sd = ncol(B.sd)-1L
    p.kappa = ncol(B.range)-1L

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)

    ## log(kappa) = 0.5*log(8*nu) - log(range)
    B.kappa = (cbind(log(8*nu.nominal)/2 - B.range[,1],
                     -B.range[,-1,drop=FALSE] ))

    if (method==1) {
        spde =
            inla.spde2.matern(mesh, B.tau=as.matrix(0), B.kappa=B.kappa, ...)

        B.tau =
            inla.internal.spde2.matern.B.tau(
                precision.fcn=
                function(theta, spde) {
                    return(inla.spde2.precision(spde, theta=theta))
                },
                n.theta = p.kappa,
                spde = spde)
        if (!local.offset.compensation) {
            B.tau[,1] = rep(mean(B.tau[,1]), spde$n.spde)
        }
    } else {
        spde =
            inla.spde2.matern(mesh,
                              B.tau=as.matrix(0),
                              B.kappa=B.kappa[,1,drop=FALSE],
                              alpha=alpha, ...)
        lvar.0 =
            log(diag(inla.qinv(inla.spde2.precision(spde, theta=rep(0,0)))))
        if (!local.offset.compensation) {
            lvar.0 = rep(mean(lvar.0), spde$n.spde)
        }

        B.tau = -B.kappa*nu
        B.tau[,1] = lvar.0/2
    }

    log.var.scaling = lgamma(nu.nominal)-lgamma(alpha.nominal)-log(4*pi)*d/2

    B.tau =
        cbind(B.tau[,1]-B.sd[,1],
              -B.sd[,-1,drop=FALSE],
              B.tau[,-1,drop=FALSE])
    B.kappa =
        cbind(B.kappa[,1],
              matrix(0, n.spde, p.sd),
              B.kappa[,-1,drop=FALSE])

    return(list(B.tau=B.tau, B.kappa=B.kappa))

}



inla.internal.test.spde2.sd.basis = function (k=1, dth=0.1, r=1000, globe=25, compensate=FALSE) {

    alpha=2
    mesh = inla.mesh.create(globe=globe)

    B.common = inla.mesh.basis(mesh, "b.spline", n=3)
    B.common = cbind(B.common, B.common[,1]*(mesh$loc[,3]>0))
    B.common[,1] = B.common[,1]*(mesh$loc[,3]<=0)

    B.sd = cbind(0, B.common)
    B.range = cbind(log(r/6370), B.common)

    sd.basis0 = inla.spde2.matern.sd.basis(mesh, B.sd=B.sd, B.range=B.range, method=0, alpha=alpha, local.offset.compensation=compensate)
    sd.basis1 = inla.spde2.matern.sd.basis(mesh, B.sd=B.sd, B.range=B.range, method=1, alpha=alpha, local.offset.compensation=compensate)

    spde0 =
        inla.spde2.matern(mesh, alpha=alpha,
                          B.tau=sd.basis0$B.tau,
                          B.kappa=sd.basis0$B.kappa)
    spde1 =
        inla.spde2.matern(mesh, alpha=alpha,
                          B.tau=sd.basis1$B.tau,
                          B.kappa=sd.basis1$B.kappa)

    th0 = c(0,0,0,0,0,0,0,0)
    th1 = th0
    th1[k] = th0[k]+dth
    v0.0=diag(inla.qinv(inla.spde2.precision(spde0,theta=th0)))
    v0.1=diag(inla.qinv(inla.spde2.precision(spde0,theta=th1)))
    v1.0=diag(inla.qinv(inla.spde2.precision(spde1,theta=th0)))
    v1.1=diag(inla.qinv(inla.spde2.precision(spde1,theta=th1)))

    op=par(mfrow=c(2,2))
    on.exit(par(op))
    plot(asin(mesh$loc[,3]),rowSums(sd.basis0$B.kappa[,1+k,drop=FALSE]), pch=20,
         ylim=(range(c(0, range(rowSums(sd.basis0$B.kappa[,1+k,drop=FALSE])),
                       range(-rowSums(sd.basis0$B.tau[,1+k,drop=FALSE])),
                       range(-rowSums(sd.basis1$B.tau[,1+k,drop=FALSE]))))),
         main="Basis functions")
    points(asin(mesh$loc[,3]),-rowSums(sd.basis0$B.tau[,1+k,drop=FALSE]),col=2,pch=20)
    points(asin(mesh$loc[,3]),-rowSums(sd.basis1$B.tau[,1+k,drop=FALSE]),col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1, col=2,pch=20,
         ylim=range(c(0,range(v0.0),range(v1.0),range(v0.1),range(v1.1))),
         main="Absolute variances")
    points(asin(mesh$loc[,3]),v1.1, col=4,pch=20)
    points(asin(mesh$loc[,3]),v0.0, col=2,pch=1)
    points(asin(mesh$loc[,3]),v1.0, col=4,pch=1)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1-v0.0, col=2,pch=20,
         ylim=range(c(0,range(v0.1-v0.0),range(v1.1-v1.0))),
         main="Absolute variance error")
    points(asin(mesh$loc[,3]),v1.1-v1.0, col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    plot(asin(mesh$loc[,3]),v0.1/v0.0-1, col=2,pch=20,
         ylim=range(c(0,range(v0.1/v0.0-1),range(v1.1/v1.0-1))),
         main="Relative variance error")
    points(asin(mesh$loc[,3]),v1.1/v1.0-1, col=4,pch=20)
    legend("bottomright",
           legend=paste("Method", 0:1),
           col=c(2,4), pch=20)

    return(environment())
}






#' Parameter settings for `inla.spde2.matern` models.
#' 
#' Construct parameter settings for `inla.spde2.matern` models.
#' 
#' 
#' @param mesh The mesh to build the model on, as an [inla.mesh()]
#' object.
#' @param alpha Fractional operator order, \eqn{0<\alpha\leq 2} supported.
#' (\eqn{\nu=\alpha-d/2})
#' @param B.tau Matrix with specification of log-linear model for \eqn{\tau}.
#' @param B.kappa Matrix with specification of log-linear model for
#' \eqn{\kappa}.
#' @param prior.variance.nominal Nominal prior mean for the field variance
#' @param prior.range.nominal Nominal prior mean for the spatial range
#' @param prior.tau Prior mean for tau (overrides
#' `prior.variance.nominal`)
#' @param prior.kappa Prior mean for kappa (overrides
#' `prior.range.nominal`)
#' @param theta.prior.mean (overrides `prior.*`)
#' @param theta.prior.prec Scalar, vector or matrix, specifying the joint prior
#' precision for \eqn{theta}.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde2.matern()]
#' @export param2.matern.orig
param2.matern.orig =
    function(mesh,
             alpha=2,
             B.tau = matrix(c(0,1,0),1,3),
             B.kappa = matrix(c(0,0,1),1,3),
             prior.variance.nominal = 1,
             prior.range.nominal = NULL,
             prior.tau = NULL,
             prior.kappa = NULL,
             theta.prior.mean = NULL,
             theta.prior.prec = 0.1)
{
    ## NOTE: For d==1, degree==2, the B.* must be given per basis
    ## function, not per knot.

    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    if (is.null(B.tau))
        stop("B.tau must not be NULL.")
    if (is.null(B.kappa))
        stop("B.kappa must not be NULL.")
    is.stationary = (nrow(B.kappa)==1) && (nrow(B.tau)==1)

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)
    n.theta = ncol(B.kappa)-1L

    B.kappa = inla.spde.homogenise_B_matrix(B.kappa, n.spde, n.theta)
    B.tau = inla.spde.homogenise_B_matrix(B.tau, n.spde, n.theta)

    B.prec =
        cbind(+ lgamma(alpha.nominal) - lgamma(nu.nominal)
              + (d/2)*log(4*pi) + 2*nu.nominal*B.kappa[,1]
              + 2*B.tau[,1],
              2*nu.nominal*B.kappa[,-1,drop=FALSE]
              + 2*B.tau[,-1,drop=FALSE] )
    if (is.stationary) {
        B.tau = B.tau[1,,drop=FALSE]
        B.kappa = B.kappa[1,,drop=FALSE]
        B.prec = B.prec[1,,drop=FALSE]
    }
    B.variance = -B.prec
    B.range =
        cbind(0.5*log(8*nu.nominal)-B.kappa[,1],
              -B.kappa[,-1,drop=FALSE])

    if (n.theta>0) {
        B.theta = cbind(0,diag(1, n.theta))
        rownames(B.theta) <- rownames(B.theta, do.NULL=FALSE, prefix="theta.")
    } else {
        B.theta = NULL
    }
    rownames(B.tau) <- rownames(B.tau, do.NULL=FALSE, prefix="tau.")
    rownames(B.kappa) <- rownames(B.kappa, do.NULL=FALSE, prefix="kappa.")
    rownames(B.variance) <-
        rownames(B.variance, do.NULL=FALSE, prefix="variance.nominal.")
    rownames(B.range) <-
        rownames(B.range, do.NULL=FALSE, prefix="range.nominal.")
    BLC = rbind(B.theta, B.tau, B.kappa, B.variance, B.range)


    ## Construct priors.
    if (is.null(theta.prior.prec)) {
        theta.prior.prec = diag(0.1, n.theta, n.theta)
    } else {
        theta.prior.prec = as.matrix(theta.prior.prec)
        if (ncol(theta.prior.prec) == 1) {
            theta.prior.prec =
                diag(as.vector(theta.prior.prec), n.theta, n.theta)
        }
        if ((nrow(theta.prior.prec) != n.theta) ||
            (ncol(theta.prior.prec) != n.theta)) {
            stop(paste("Size of theta.prior.prec is (",
                       paste(dim(theta.prior.prec), collapse=",", sep=""),
                       ") but should be (",
                       paste(c(n.theta, n.theta), collapse=",", sep=""),
                       ")."))
        }
    }

    if (is.null(theta.prior.mean)) {
        if (is.null(prior.range.nominal)) {
            mesh.range =
                inla.ifelse(d==2,
                            (max(c(diff(range(mesh$loc[,1])),
                                   diff(range(mesh$loc[,2])),
                                   diff(range(mesh$loc[,3]))
                                   ))),
                            diff(mesh$interval))
            prior.range.nominal = mesh.range*0.2
        } else {
          if (!is.numeric(prior.range.nominal) ||
              (length(prior.range.nominal) != 1)) {
            stop(paste0("'prior.range.nominal' must be NULL or a single scalar value.\n",
                        "Did you intend to supply 'prior.range' to inla.spde2.pcmatern instead?"))
          }
        }

        if (is.null(prior.kappa)) {
            prior.kappa = sqrt(8*nu.nominal)/prior.range.nominal
        }
        if (is.null(prior.tau)) {
            prior.tau =
                sqrt(gamma(nu.nominal)/gamma(alpha.nominal)/
                     (4*pi*prior.kappa^(2*nu.nominal)*prior.variance.nominal))
        }

        if (n.theta>0) {
            theta.prior.mean =
                qr.solve(rbind(B.tau[,-1,drop=FALSE], B.kappa[,-1,drop=FALSE]),
                         c(log(prior.tau) - B.tau[,1],
                           log(prior.kappa) - B.kappa[,1]))
        } else {
            theta.prior.mean = rep(0, n.theta) ## Empty vector
        }
    }

    param =
        list(is.stationary=is.stationary,
             B.tau=B.tau, B.kappa=B.kappa, BLC=BLC,
             theta.prior.mean=theta.prior.mean,
             theta.prior.prec=theta.prior.prec)
    return(param)
}



#' Matern SPDE model object for INLA
#' 
#' Create an `inla.spde2` model object for a Matern model. Use
#' `inla.spde2.pcmatern` instead for a PC prior for the parameters.
#' 
#' This method constructs a Matern SPDE model, with spatial scale parameter
#' \eqn{\kappa(u)} and variance rescaling parameter \eqn{\tau(u)}.
#' 
#' \deqn{(\kappa^2(u)-\Delta)^{\alpha/2}(\tau(u) }{(kappa^2(u)-Delta)^(alpha/2)
#' (tau(u) x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2(u)-Delta)^(alpha/2) (tau(u)
#' x(u)) = W(u)}
#' 
#' Stationary models are supported for \eqn{0 < \alpha \leq 2}, with spectral
#' approximation methods used for non-integer \eqn{\alpha}, with approximation
#' method determined by `fractional.method`.
#' 
#' Non-stationary models are supported for \eqn{\alpha=2} only, with \itemize{
#' \item \eqn{\log\tau(u) = B^\tau_0(u) + \sum_{k=1}^p B^\tau_k(u) }{log tau(u)
#' = B.tau_0(u) + sum_{k=1}^p B.tau_k(u) theta_k}\eqn{ \theta_k}{log tau(u) =
#' B.tau_0(u) + sum_{k=1}^p B.tau_k(u) theta_k}
#' 
#' \item \eqn{\log\kappa(u) = B^{\kappa}_0(u) + \sum_{k=1}^p B^{\kappa}_k(u)
#' }{log kappa(u) = B.kappa_0(u) + sum_{k=1}^p B.kappa_k(u) theta_k}\eqn{
#' \theta_k}{log kappa(u) = B.kappa_0(u) + sum_{k=1}^p B.kappa_k(u) theta_k}
#' 
#' }
#' 
#' The same parameterisation is used in the stationary cases, but with
#' \eqn{B^\tau_0}, \eqn{B^\tau_k}, \eqn{B^\kappa_0}, and \eqn{B^\tau_k}
#' constant across \eqn{u}.
#' 
#' Integration and other general linear constraints are supported via the
#' `constr`, `extraconstr.int`, and `extraconstr` parameters,
#' which also interact with `n.iid.group`.
#' 
#' @param mesh The mesh to build the model on, as an [inla.mesh()] or
#' [inla.mesh.1d()] object.
#' @param alpha Fractional operator order, \eqn{0<\alpha\leq 2} supported.
#' (\eqn{\nu=\alpha-d/2})
#' @param param Parameter, e.g. generated by `param2.matern.orig`
#' @param constr If `TRUE`, apply an integrate-to-zero constraint.
#' Default `FALSE`.
#' @param extraconstr.int Field integral constraints.
#' @param extraconstr Direct linear combination constraints on the basis
#' weights.
#' @param fractional.method Specifies the approximation method to use for
#' fractional (non-integer) `alpha` values. `'parsimonious'` gives an
#' overall approximate minimal covariance error, `'null'` uses
#' approximates low-order properties.
#' @param B.tau Matrix with specification of log-linear model for \eqn{\tau}.
#' @param B.kappa Matrix with specification of log-linear model for
#' \eqn{\kappa}.
#' @param prior.variance.nominal Nominal prior mean for the field variance
#' @param prior.range.nominal Nominal prior mean for the spatial range
#' @param prior.tau Prior mean for tau (overrides
#' `prior.variance.nominal`)
#' @param prior.kappa Prior mean for kappa (overrides
#' `prior.range.nominal`)
#' @param theta.prior.mean (overrides `prior.*`)
#' @param theta.prior.prec Scalar, vector or matrix, specifying the joint prior
#' precision for \eqn{theta}.
#' @param n.iid.group If greater than 1, build an explicitly iid replicated
#' model, to support constraints applied to the combined replicates, for
#' example in a time-replicated spatial model. Constraints can either be
#' specified for a single mesh, in which case it's applied to the average of
#' the replicates (`ncol(A)` should be `mesh$n` for 2D meshes,
#' `mesh$m` for 1D), or as general constraints on the collection of
#' replicates (`ncol(A)` should be `mesh$n * n.iid.group` for 2D
#' meshes, `mesh$m * n.iid.group` for 1D).
#' @param \dots Additional parameters for special uses.
#' @return An `inla.spde2` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.2d()], [inla.mesh.create()],
#' [inla.mesh.1d()], [inla.mesh.basis()],
#' [inla.spde2.pcmatern()], [inla.spde2.generic()]
#' @examples
#' 
#' n = 100
#' field.fcn = function(loc) (10*cos(2*pi*2*(loc[,1]+loc[,2])))
#' loc = matrix(runif(n*2),n,2)
#' ## One field, 2 observations per location
#' idx.y = rep(1:n,2)
#' y = field.fcn(loc[idx.y,]) + rnorm(length(idx.y))
#' 
#' mesh = inla.mesh.create(loc, refine=list(max.edge=0.05))
#' spde = inla.spde2.matern(mesh)
#' data = list(y=y, field=mesh$idx$loc[idx.y])
#' formula = y ~ -1 + f(field, model=spde)
#' result = inla(formula, data=data, family="normal")
#' 
#' ## Plot the mesh structure:
#' plot(mesh)
#' \donttest{
#' if (require(rgl)) {
#'   col.pal = colorRampPalette(c("blue","cyan","green","yellow","red"))
#'   ## Plot the posterior mean:
#'   plot(mesh, rgl=TRUE,
#'        result$summary.random$field[,"mean"],
#'        color.palette = col.pal)
#'   ## Plot residual field:
#'   plot(mesh, rgl=TRUE,
#'        result$summary.random$field[,"mean"]-field.fcn(mesh$loc),
#'        color.palette = col.pal)
#' }
#' }
#' result.field = inla.spde.result(result, "field", spde)
#' plot(result.field$marginals.range.nominal[[1]])
#' 
#' @export inla.spde2.matern
inla.spde2.matern =
    function(mesh,
             alpha=2,
             param = NULL,
             constr = FALSE,
             extraconstr.int = NULL,
             extraconstr = NULL,
             fractional.method = c("parsimonious", "null"),
             B.tau = matrix(c(0,1,0),1,3),
             B.kappa = matrix(c(0,0,1),1,3),
             prior.variance.nominal = 1,
             prior.range.nominal = NULL,
             prior.tau = NULL,
             prior.kappa = NULL,
             theta.prior.mean = NULL,
             theta.prior.prec = 0.1,
             n.iid.group = 1,
             ...)
{
  if(!is.null(list(...)[["prior.pc.rho"]]) &&
     !is.null(list(...)[["prior.pc.sig"]])){
      ## Temporary implementation of PC prior for standard deviation and range
      ##    - Changes parametrization to range and standard deviation
      ##    - Sets prior according to hyperparameters for range   : prior.pc.rho
      ##                                              and std.dev.: prior.pc.sig
      warning("You're using a deprecated experimental PC prior matern model that will be removed in a future version of the package. Use 'inla.spde2.pcmatern' instead.")
      prior.pc.rho <- list(...)[["prior.pc.rho"]]
      prior.pc.sig <- list(...)[["prior.pc.sig"]]

      ## Call inla.spde2.matern with range and standard deviation parametrization
      d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
      nu = alpha-d/2
      kappa0 = log(8*nu)/2
      tau0   = 0.5*(lgamma(nu)-lgamma(nu+d/2)-d/2*log(4*pi))-nu*kappa0
      spde   = inla.spde2.matern(mesh = mesh,
                                 B.tau   = cbind(tau0,   nu,  -1),
                                 B.kappa = cbind(kappa0, -1, 0))

      ## Change prior information
      param = c(prior.pc.rho, prior.pc.sig)
      spde$f$hyper.default$theta1$prior = "pcspdega"
      spde$f$hyper.default$theta1$param = param
      spde$f$hyper.default$theta1$initial = log(prior.pc.rho[1])+1
      spde$f$hyper.default$theta2$initial = log(prior.pc.sig[1])-1

      ## End and return
      return(invisible(spde))
    }

    ## Standard code
    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    fractional.method = match.arg(fractional.method)

    if (is.null(param)) {
        param =
            param2.matern.orig(
                mesh, alpha,
                B.tau, B.kappa,
                prior.variance.nominal,
                prior.range.nominal,
                prior.tau,
                prior.kappa,
                theta.prior.mean,
                theta.prior.prec)
    } else {
        deprecated =
            !c(missing(B.tau), missing(B.kappa),
               missing(prior.variance.nominal),
               missing(prior.range.nominal),
               missing(prior.tau), missing(prior.kappa),
               missing(theta.prior.mean), missing(theta.prior.prec))
        deprecated =
            c("B.tau", "B.kappa",
              "prior.variance.nominal",
              "prior.range.nominal",
              "prior.tau", "prior.kappa",
              "theta.prior.mean", "theta.prior.prec")[deprecated]
        if (length(deprecated) > 0) {
            warning(paste("'param' specified;  ",
                          "Ignoring deprecated parameter(s) ",
                          paste(deprecated, collapse=", "), ".", sep=""))
        }
    }

    d = inla.ifelse(inherits(mesh, "inla.mesh"), 2, 1)
    nu = alpha-d/2
    nu.nominal = max(0.5, nu)
    alpha.nominal = max(nu.nominal+d/2, alpha)

    n.spde = inla.ifelse(d==2, mesh$n, mesh$m)
    n.theta = ncol(B.kappa)-1L

    if (d==2) {
        fem =
            inla.fmesher.smorg(mesh$loc,
                               mesh$graph$tv,
                               fem=2,
                               output=list("c0", "c1", "g1", "g2"))
    } else {
        fem = inla.mesh.1d.fem(mesh)
        if (mesh$degree==2) {
            fem$c0 = fem$c1 ## Use higher order matrix.
        }
    }

    if (alpha==2) {
        B.phi0 = param$B.tau
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0
        M1 = fem$g1
        M2 = fem$g2
    } else if (alpha==1) {
        B.phi0 = param$B.tau
        B.phi1 = param$B.kappa
        M0 = fem$c0
        M1 = fem$g1*0
        M2 = fem$g1
    } else if (!param$is.stationary) {
        stop("Non-stationary Matern with fractional alpha is not implemented.")
    } else if ((alpha<2) && (alpha>1)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,0, 1,1,0, 1,2,1),3,3) %*%
                solve(matrix(1/(c(4:2, 3:1, 2:0)+lambda), 3, 3),
                      1/(c(4:2)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha,alpha*(alpha-1)/2)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-2)*param$B.kappa
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*b[2]/2
        M2 = fem$g2*b[3]
    } else if ((alpha<1) && (alpha>0)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,1,1),2,2) %*%
                solve(matrix(1/(c(2:1, 1:0)+lambda), 2, 2),
                      1/(c(2:1)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-1)*param$B.kappa
        B.phi1 = param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*0
        M2 = fem$g1*b[2]
    } else {
        stop(paste("Unsupported alpha value (", alpha,
                   "). Supported values are 0 < alpha <= 2", sep=""))
    }

    if (n.iid.group == 1) {
      spde =
        inla.spde2.generic(M0=M0, M1=M1, M2=M2,
                           B0=B.phi0, B1=B.phi1, B2=1,
                           theta.mu = param$theta.prior.mean,
                           theta.Q = param$theta.prior.prec,
                           transform = "identity",
                           BLC = param$BLC)
    } else {
      if (nrow(B.phi0) > 1) {
        B.phi0 <- kronecker(matrix(1, n.iid.group, 1), B.phi0)
      }
      if (nrow(B.phi1) > 1) {
        B.phi1 <- kronecker(matrix(1, n.iid.group, 1), B.phi1)
      }
      spde =
        inla.spde2.generic(M0=kronecker(Diagonal(n.iid.group), M0),
                           M1=kronecker(Diagonal(n.iid.group), M1),
                           M2=kronecker(Diagonal(n.iid.group), M2),
                           B0=B.phi0, B1=B.phi1, B2=1,
                           theta.mu = param$theta.prior.mean,
                           theta.Q = param$theta.prior.prec,
                           transform = "identity",
                           BLC = param$BLC)
    }
    spde$model = "matern"
    spde$BLC = param$BLC

    if (constr || !is.null(extraconstr.int) || !is.null(extraconstr)) {
      A.constr = matrix(numeric(0), 0, n.spde*n.iid.group)
      e.constr = matrix(numeric(0), 0, 1)
      if (constr) {
        A.constr <- rbind(A.constr,
                          matrix(colSums(fem$c1)/n.iid.group,
                                 1, n.spde*n.iid.group))
        e.constr <- rbind(e.constr, 0)
      }
      if (!is.null(extraconstr.int)) {
        if (ncol(extraconstr.int$A) == n.spde) {
          A.constr <-
            rbind(A.constr,
                  kronecker(matrix(1/n.iid.group, 1, n.iid.group),
                            as.matrix(extraconstr.int$A %*% fem$c1)))
        } else {
          A.constr <-
            rbind(A.constr,
                  as.matrix(extraconstr.int$A %*%
                              kronecker(Diagonal(n.iid.group),
                                        fem$c1)))
        }
        e.constr <- rbind(e.constr, as.matrix(extraconstr.int$e))
      }
      if (!is.null(extraconstr)) {
        if (ncol(extraconstr$A) == n.spde) {
          A.constr <-
            rbind(A.constr,
                  kronecker(matrix(1/n.iid.group, 1, n.iid.group),
                            as.matrix(extraconstr$A)))
        } else {
          A.constr <- rbind(A.constr, as.matrix(extraconstr$A))
        }
        e.constr <- rbind(e.constr, as.matrix(extraconstr$e))
      }

      spde$f$constr = FALSE
      spde$f$extraconstr = list(A=A.constr, e=e.constr)
    }

    ## Attach the mesh, so downstream code can have access
    spde$mesh <- mesh

    return(invisible(spde))
}







#' Matern SPDE model object with PC prior for INLA
#' 
#' Create an `inla.spde2` model object for a Matern model, using a PC
#' prior for the parameters.
#' 
#' This method constructs a Matern SPDE model, with spatial range \eqn{\rho}
#' and standard deviation parameter \eqn{\sigma}.  In the parameterisation
#' 
#' \deqn{(\kappa^2-\Delta)^{\alpha/2}(\tau }{(kappa^2-Delta)^(alpha/2) (tau
#' x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2-Delta)^(alpha/2) (tau x(u)) = W(u)}
#' 
#' the spatial scale parameter \eqn{\kappa=\sqrt{8\nu}/\rho}, where
#' \eqn{\nu=\alpha-d/2}, and \eqn{\tau} is proportional to \eqn{1/\sigma}.
#' 
#' Stationary models are supported for \eqn{0 < \alpha \leq 2}{0 < alpha <= 2},
#' with spectral approximation methods used for non-integer \eqn{\alpha}, with
#' approximation method determined by `fractional.method`.
#' 
#' Integration and other general linear constraints are supported via the
#' `constr`, `extraconstr.int`, and `extraconstr` parameters,
#' which also interact with `n.iid.group`.
#' 
#' The joint PC prior density for the spatial range, \eqn{\rho}, and the
#' marginal standard deviation, \eqn{\sigma}, and is \deqn{ }{p(rho, sigma) =
#' (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S sigma) }\deqn{ \pi(\rho,
#' \sigma) = }{p(rho, sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S
#' sigma) }\deqn{ \frac{d \lambda_\rho}{2} \rho^{-1-d/2} \exp(-\lambda_\rho
#' \rho^{-d/2}) }{p(rho, sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S
#' exp(-S sigma) }\deqn{ \lambda_\sigma\exp(-\lambda_\sigma \sigma) }{p(rho,
#' sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S sigma) } where
#' \eqn{\lambda_\rho}{R} and \eqn{\lambda_\sigma}{S} are hyperparameters that
#' must be determined by the analyst. The practical approach for this in INLA
#' is to require the user to indirectly specify these hyperparameters through
#' \deqn{P(\rho < \rho_0) = p_\rho} and \deqn{P(\sigma > \sigma_0) = p_\sigma}
#' where the user specifies the lower tail quantile and probability for the
#' range (\eqn{\rho_0} and \eqn{p_\rho}) and the upper tail quantile and
#' probability for the standard deviation (\eqn{\sigma_0} and
#' \eqn{\alpha_\sigma}).
#' 
#' This allows the user to control the priors of the parameters by supplying
#' knowledge of the scale of the problem. What is a reasonable upper magnitude
#' for the spatial effect and what is a reasonable lower scale at which the
#' spatial effect can operate? The shape of the prior was derived through a
#' construction that shrinks the spatial effect towards a base model of no
#' spatial effect in the sense of distance measured by Kullback-Leibler
#' divergence.
#' 
#' The prior is constructed in two steps, under the idea that having a spatial
#' field is an extension of not having a spatial field. First, a spatially
#' constant random effect (\eqn{\rho = \infty}) with finite variance is more
#' complex than not having a random effect (\eqn{\sigma = 0}). Second, a
#' spatial field with spatial variation (\eqn{\rho < \infty}) is more complex
#' than the random effect with no spatial variation. Each of these extensions
#' are shrunk towards the simpler model and, as a result, we shrink the spatial
#' field towards the base model of no spatial variation and zero variance
#' (\eqn{\rho = \infty} and \eqn{\sigma = 0}).
#' 
#' The details behind the construction of the prior is presented in Fuglstad,
#' et al. (2016) and is based on the PC prior framework (Simpson, et al.,
#' 2015).
#' 
#' @param mesh The mesh to build the model on, as an [inla.mesh()] or
#' [inla.mesh.1d()] object.
#' @param alpha Fractional operator order, \eqn{0<\alpha\leq 2}{0 < alpha <= 2}
#' supported, for \eqn{\nu=\alpha-d/2>0}.
#' @param param Further model parameters. Not currently used.
#' @param constr If `TRUE`, apply an integrate-to-zero constraint.
#' Default `FALSE`.
#' @param extraconstr.int Field integral constraints.
#' @param extraconstr Direct linear combination constraints on the basis
#' weights.
#' @param fractional.method Specifies the approximation method to use for
#' fractional (non-integer) `alpha` values. `'parsimonious'` gives an
#' overall approximate minimal covariance error, `'null'` uses
#' approximates low-order properties.
#' @param n.iid.group If greater than 1, build an explicitly iid replicated
#' model, to support constraints applied to the combined replicates, for
#' example in a time-replicated spatial model. Constraints can either be
#' specified for a single mesh, in which case it's applied to the average of
#' the replicates (`ncol(A)` should be `mesh$n` for 2D meshes,
#' `mesh$m` for 1D), or as general constraints on the collection of
#' replicates (`ncol(A)` should be `mesh$n * n.iid.group` for 2D
#' meshes, `mesh$m * n.iid.group` for 1D).
#' @param prior.range A length 2 vector, with `(range0,Prange)` specifying
#' that \eqn{P(\rho < \rho_0)=p_\rho}, where \eqn{\rho} is the spatial range of
#' the random field. If `Prange` is `NA`, then `range0` is used
#' as a fixed range value.
#' @param prior.sigma A length 2 vector, with `(sigma0,Psigma)` specifying
#' that \eqn{P(\sigma > \sigma_0)=p_\sigma}, where \eqn{\sigma} is the marginal
#' standard deviation of the field.  If `Psigma` is `NA`, then
#' `sigma0` is used as a fixed range value.
#' @return An `inla.spde2` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.2d()], [inla.mesh.create()],
#' [inla.mesh.1d()], [inla.mesh.basis()],
#' [inla.spde2.matern()], [inla.spde2.generic()]
#' @references Fuglstad, G.-A., Simpson, D., Lindgren, F., and Rue, H. (2016)
#' Constructing Priors that Penalize the Complexity of Gaussian Random Fields.
#' arXiv:1503.00256
#' 
#' Simpson, D., Rue, H., Martins, T., Riebler, A., and Sørbye, S. (2015)
#' Penalising model component complexity: A principled, practical approach to
#' constructing priors. arXiv:1403.4630
#' @examples
#' 
#' ## Spatial interpolation
#'   n = 100
#'   field.fcn = function(loc) (10*cos(2*pi*2*(loc[,1]+loc[,2])))
#'   loc = matrix(runif(n*2),n,2)
#'   ## One field, 2 observations per location
#'   idx.y = rep(1:n,2)
#'   y = field.fcn(loc[idx.y,]) + rnorm(length(idx.y))
#' 
#'   mesh = inla.mesh.2d(loc, max.edge=0.05, cutoff=0.01)
#'   spde = inla.spde2.pcmatern(mesh,
#'            prior.range=c(0.01,0.1), prior.sigma=c(100,0.1))
#'   data = list(y=y, field=mesh$idx$loc[idx.y])
#'   formula = y ~ -1 + f(field, model=spde)
#'   result = inla(formula, data=data, family="normal")
#' 
#'   ## Plot the mesh structure:
#'   plot(mesh)
#'   \donttest{
#'   if (require(rgl)) {
#'     col.pal = colorRampPalette(c("blue","cyan","green","yellow","red"))
#'     ## Plot the posterior mean:
#'     plot(mesh, rgl=TRUE,
#'          result$summary.random$field[,"mean"],
#'          color.palette = col.pal)
#'     ## Plot residual field:
#'     plot(mesh, rgl=TRUE,
#'          result$summary.random$field[,"mean"]-field.fcn(mesh$loc),
#'          color.palette = col.pal)
#'   }
#'   }
#' 
#'   result.field = inla.spde.result(result, "field", spde)
#'   par(mfrow=c(2,1))
#'   plot(result.field$marginals.range.nominal[[1]],
#'        type="l", main="Posterior density for range")
#'   plot(inla.tmarginal(sqrt, result.field$marginals.variance.nominal[[1]]),
#'        type="l", main="Posterior density for std.dev.")
#'   par(mfrow=c(1,1))
#' 
#' ## Spatial model
#'   set.seed(1234234)
#'   
#'   ## Generate spatial locations
#'   nObs = 200
#'   loc = matrix(runif(nObs*2), nrow = nObs, ncol = 2)
#' 
#'   ## Generate observation of spatial field
#'   nu = 1.0
#'   rhoT = 0.2
#'   kappaT = sqrt(8*nu)/rhoT
#'   sigT = 1.0
#'   Sig = sigT^2*inla.matern.cov(nu = nu,
#'                                kappa = kappaT,
#'                                x = as.matrix(dist(loc)),
#'                                d = 2,
#'                                corr = TRUE)
#'   L = t(chol(Sig))
#'   u = L %*% rnorm(nObs)
#' 
#'   ## Construct observation with nugget
#'   sigN = 0.1
#'   y = u + sigN*rnorm(nObs)
#' 
#'   ## Create the mesh and spde object
#'   mesh = inla.mesh.2d(loc,
#'                       max.edge = 0.05,
#'                       cutoff = 0.01)
#'   spde = inla.spde2.pcmatern(mesh,
#'                              prior.range = c(0.01, 0.05),
#'                              prior.sigma = c(10, 0.05))
#' 
#'   ## Create projection matrix for observations
#'   A = inla.spde.make.A(mesh = mesh,
#'                        loc = loc)
#' 
#'   ## Run model without any covariates
#'   idx = 1:spde$n.spde
#'   res = inla(y ~ f(idx, model = spde) - 1,
#'              data = list(y = y, idx = idx, spde = spde),
#'              control.predictor = list(A = A))
#' 
#' ## Re-run model with fixed range
#'   spde.fixed = inla.spde2.pcmatern(mesh,
#'                                    prior.range = c(0.2, NA),
#'                                    prior.sigma = c(10, 0.05))
#' 
#'   res.fixed = inla(y ~ f(idx, model = spde) - 1,
#'                    data = list(y = y, idx = idx, spde = spde.fixed),
#'                    control.predictor = list(A = A))
#' 
#' 
#' @export inla.spde2.pcmatern
inla.spde2.pcmatern =
    function(mesh,
             alpha = 2,
             param = NULL,
             constr = FALSE,
             extraconstr.int = NULL,
             extraconstr = NULL,
             fractional.method = c("parsimonious", "null"),
             n.iid.group = 1,
             prior.range = NULL,
             prior.sigma = NULL)
{
  ## Implementation of PC prior for standard deviation and range
  ##    - Sets the parametrization to range and standard deviation
  ##    - Sets prior according to hyperparameters for range   : prior.range
  ##                                              and std.dev.: prior.sigma
  ## Calls inla.spde2.matern to construct the object, then changes the prior
  if (inherits(mesh, "inla.mesh")) {
    d <- 2
  } else if (inherits(mesh, "inla.mesh.1d")) {
    d <- 1
  } else {
    stop(paste("Unknown mesh class '",
               paste(class(mesh), collapse=",", sep=""),
               "'.", sep=""))
  }

  if (missing(prior.range) || is.null(prior.range) ||
      !is.vector(prior.range) || (length(prior.range) != 2)) {
    stop("'prior.range' should be a length 2 vector 'c(range0,tailprob)' or a fixed range specified with 'c(range,NA)'.")
  }
  if (missing(prior.sigma) || is.null(prior.sigma) ||
      !is.vector(prior.sigma) || (length(prior.sigma) != 2)) {
    stop("'prior.sigma' should be a length 2 vector 'c(sigma0,tailprob)' or a fixed sigma specified with 'c(sigma,NA)'.")
  }
  if (prior.range[1] <= 0){
    stop("'prior.range[1]' must be a number greater than 0 specifying a spatial range")
  }
  if (prior.sigma[1] <= 0){
    stop("'prior.sigma[1]' must be a number greater than 0 specifying a standard deviation")
  }
  if (!is.na(prior.range[2]) &&
      ((prior.range[2] <= 0) || (prior.range[2] >= 1))) {
    stop("'prior.range[2]' must be a probaility strictly between 0 and 1 (or NA to specify a fixed range)")
  }
  if (!is.na(prior.sigma[2]) &&
      ((prior.sigma[2] <= 0) || (prior.sigma[2] >= 1))) {
    stop("'prior.sigma[2]' must be a probaility strictly between 0 and 1 (or NA to specify a fixed sigma)")
  }

  nu <- alpha-d/2
  if (nu <= 0) {
    stop(paste("Smoothness nu = alpha-dim/2 = ", nu,
               ", but must be > 0.", sep=""))
  }

  kappa0 <- log(8*nu)/2
  tau0   <- 0.5*(lgamma(nu)-lgamma(nu+d/2)-d/2*log(4*pi))-nu*kappa0
  spde   <- inla.spde2.matern(mesh = mesh,
                              B.tau   = cbind(tau0,   nu,  -1),
                              B.kappa = cbind(kappa0, -1, 0),
                              alpha = alpha,
                              param = NULL,
                              constr = constr,
                              extraconstr.int = extraconstr.int,
                              extraconstr = extraconstr,
                              fractional.method = fractional.method,
                              n.iid.group = n.iid.group)

  ## Calculate hyperparameters
  is.fixed.range <- is.na(prior.range[2])
  if (is.fixed.range) {
    lam1 <- 0
    initial.range <- log(prior.range[1])
  } else {
    lam1 <- -log(prior.range[2])*prior.range[1]^(d/2)
    initial.range <- log(prior.range[1]) + 1
  }

  is.fixed.sigma <- is.na(prior.sigma[2])
  if (is.fixed.sigma){
    lam2 <- 0
    initial.sigma <- log(prior.sigma[1])
  } else{
    lam2 <- -log(prior.sigma[2])/prior.sigma[1]
    initial.sigma <- log(prior.sigma[1]) - 1
  }

  pcmatern.param = c(lam1, lam2, d)

  ## Change prior information
  spde$f$hyper.default <-
    list(theta1=list(prior="pcmatern",
                     param=pcmatern.param,
                     initial=initial.range,
                     fixed=is.fixed.range),
         theta2=list(initial=initial.sigma,
                     fixed=is.fixed.sigma))

  ## Change the model descriptor
  spde$model = "pcmatern"

  invisible(spde)
}





param2.iheat <- function(mesh.space,
                         mesh.time,
                         gamma.E = NULL,
                         alpha.E = NULL,
                         theta.prior.mean = NULL,
                         theta.prior.prec = NULL)
{
    return(list(gamma.E=gamma.E,
                alpha.E=alpha.E,
                theta.prior.mean=theta.prior.mean,
                theta.prior.prec=theta.prior.prec))
}

inla.spde2.iheat =
    function(mesh.space,
             mesh.time,
             order=2,
             param = NULL,
             constr = FALSE,
             extraconstr.int = NULL,
             extraconstr = NULL)
{
    ## order=1:
    ## gamma.t dot(u) - Laplacian (u) = E
    ##
    ## order=2:
    ## gamma.t dot(u) - Laplacian (u) = v
    ## gamma.t dot(v) - Laplacian (v) = E
    ##
    ## (1 - gamma.E Laplacian)^(alpha.E/2) E = W/sqrt(gamma.s)
    ## gamma.E and alpha.E known

    inla.require.inherits(mesh.space, c("inla.mesh", "inla.mesh.1d"),
                          "'mesh.space'")
    inla.require.inherits(mesh.time, c("inla.mesh.1d"),
                          "'mesh.time'")

  if (is.null(param)) {
    stop("Use param2.iheat() to construct the prior parameter settings")
    ##        param =
    ##            param2.iheat(
    ##                mesh.space, mesh.time,
    ##                theta.prior.mean,
    ##                theta.prior.prec)
  }

    d.space = inla.ifelse(inherits(mesh.space, "inla.mesh.1d"), 1, 2)
    d.time = 1
    n.space = inla.ifelse(d.space==2, mesh.space$n, mesh.space$m)
    n.time = mesh.time$m
    n.spde = n.space*n.time
    n.theta = 2L ## gamma.s, gamma.t

    if (d.space==2) {
        fem.space = inla.mesh.fem(mesh.space$loc, 2)
    } else {
        fem.space = inla.mesh.1d.fem(mesh.space)
        if (mesh.space$degree==2) {
            fem.space$c0 = fem.space$c1 ## Use higher order matrix.
        }
    }
    fem.time = inla.mesh.1d.fem(mesh.time)
    if (mesh.time$degree==2) {
        fem.time$c0 = fem.time$c1 ## Use higher order matrix.
    }

  ## TODO: the rest
  if (FALSE) {

    if (alpha==2) {
        B.phi0 = param$B.tau
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0
        M1 = fem$g1
        M2 = fem$g2
    } else if (alpha==1) {
        B.phi0 = param$B.tau
        B.phi1 = param$B.kappa
        M0 = fem$c0
        M1 = fem$g1*0
        M2 = fem$g1
    } else if (!param$is.stationary) {
        stop("Non-stationary Matern with fractional alpha is not implemented.")
    } else if ((alpha<2) && (alpha>1)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,0, 1,1,0, 1,2,1),3,3) %*%
                solve(matrix(1/(c(4:2, 3:1, 2:0)+lambda), 3, 3),
                      1/(c(4:2)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha,alpha*(alpha-1)/2)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-2)*param$B.kappa
        B.phi1 = 2*param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*b[2]/2
        M2 = fem$g2*b[3]
    } else if ((alpha<1) && (alpha>0)) {
        if (fractional.method == "parsimonious") {
            lambda = alpha-floor(alpha)
            b = matrix(c(1,0,1,1),2,2) %*%
                solve(matrix(1/(c(2:1, 1:0)+lambda), 2, 2),
                      1/(c(2:1)+lambda-alpha))
        } else if (fractional.method == "null") {
            b = c(1,alpha)
        } else {
            stop(paste("Unknown fractional.method '", fractional.method,
                       "'.", sep=""))
        }
        B.phi0 = param$B.tau + (alpha-1)*param$B.kappa
        B.phi1 = param$B.kappa
        M0 = fem$c0*b[1]
        M1 = fem$g1*0
        M2 = fem$g1*b[2]
    } else {
        stop(paste("Unsupported alpha value (", alpha,
                   "). Supported values are 0 < alpha <= 2", sep=""))
    }

    spde =
        inla.spde2.generic(M0=M0, M1=M1, M2=M2,
                           B0=B.phi0, B1=B.phi1, B2=1,
                           theta.mu = param$theta.prior.mean,
                           theta.Q = param$theta.prior.prec,
                           transform = "identity",
                           BLC = param$BLC)
    spde$model = "matern"
    spde$BLC = param$BLC

    if (constr || !is.null(extraconstr.int)) {
        if (constr) {
            A.constr = matrix(colSums(fem$c1), 1, n.spde)
            e.constr = 0
        } else {
            A.constr = matrix(numeric(0), 0, n.spde)
            e.constr = c()
        }
        if (!is.null(extraconstr.int)) {
            A.constr =
                rbind(A.constr,
                      matrix(extraconstr.int$A %*% fem$c1,
                             nrow(extraconstr.int$A), n.spde))
            e.constr = c(e.constr, extraconstr.int$e)
        }
        if (!is.null(extraconstr)) {
            A.constr = rbind(A.constr, extraconstr$A)
            e.constr = c(e.constr, extraconstr$e)
        }

        spde$f$constr = FALSE
        spde$f$extraconstr = list(A=A.constr, e=e.constr)
    } else if (!is.null(extraconstr)) {
        spde$f$constr = FALSE
        spde$f$extraconstr = extraconstr
    }

    return(invisible(spde))
  }

  invisible(NULL)
}








#' @export
#' @describeIn inla.spde2.matern Convert from theta vector to phi0 values in
#' the internal spde2 model representation
inla.spde2.theta2phi0 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde2", "'spde'")

    if (spde$n.theta>0)
        return(exp(spde$param.inla$B0[,1, drop=TRUE] +
                   spde$param.inla$B0[,-1, drop=FALSE] %*% theta))
    else
        return(exp(spde$param.inla$B0[,1, drop=TRUE]))
}

#' @export
#' @describeIn inla.spde2.matern Convert from theta vector to phi1 values in
#' the internal spde2 model representation
inla.spde2.theta2phi1 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde2", "'spde'")

    if (spde$n.theta>0)
        return(exp(spde$param.inla$B1[,1, drop=TRUE] +
                   spde$param.inla$B1[,-1, drop=FALSE] %*% theta))
    else
        return(exp(spde$param.inla$B1[,1, drop=TRUE]))
}

#' @export
#' @describeIn inla.spde2.matern Convert from theta vector to phi2 values in
#' the internal spde2 model representation
inla.spde2.theta2phi2 = function(spde, theta)
{
    inla.require.inherits(spde, "inla.spde2", "'spde'")

    if (spde$n.theta>0) {
        phi = ((spde$param.inla$B2[,1, drop=TRUE] +
                spde$param.inla$B2[,-1, drop=FALSE] %*% theta))
    } else {
        phi = ((spde$param.inla$B2[,1, drop=TRUE]))
    }
    if (spde$param.inla$transform == "identity") {
        return(phi)
    } else if (spde$param.inla$transform == "logit") {
        return(cos(pi/(1+exp(-phi))))
    } else if (spde$param.inla$transform == "log") {
        return(2*exp(phi)-1)
    } else {
        warning(paste("Unknown link function '",
                      spde$param.inla$transform,
                      "' phi2.  Using identity link instead.", sep=""))
        return(phi)
    }
}

#' @export
#' @rdname inla.spde.precision
inla.spde2.precision =
    function(spde, theta=NULL,
             phi0=inla.spde2.theta2phi0(spde, theta),
             phi1=inla.spde2.theta2phi1(spde, theta),
             phi2=inla.spde2.theta2phi2(spde, theta),
             ...)
{
    inla.require.inherits(spde, "inla.spde2", "'spde'")
    if (spde$f$model!="spde2") {
        stop("'inla.spde2.precision' only supports internal inla models 'spde2'")
    }

    D0 = Diagonal(spde$n.spde, phi0)
    D1 = Diagonal(spde$n.spde, phi1)
    D12 = Diagonal(spde$n.spde, phi1*phi2)
    Q = (D0 %*% (D1 %*% spde$param.inla$M0 %*% D1 +
                 D12 %*% spde$param.inla$M1 +
                 t(spde$param.inla$M1) %*% D12 +
                 spde$param.inla$M2) %*% D0)

    return(Q)
}


#' @export
#' @rdname inla.spde.result
inla.spde2.result = function(inla, name, spde, do.transform=TRUE, ...)
{
    inla.require.inherits(inla, "inla", "'inla'")
    inla.require.inherits(spde, "inla.spde2", "'spde'")
    if (!spde$f$model=="spde2") {
        stop("'inla.spde2.result' only supports internal inla models 'spde2'")
    }

    result = list()
    ## Setup rownames
    BLC.names = rownames(spde$param.inla$BLC)

    ## Values
    result$summary.values = inla$summary.random[[name]]

    ## Marginals for values
    if (!is.null(inla$marginals.random[[name]])) {
        result$marginals.values = inla$marginals.random[[name]]
    }

    ## Theta
    result$summary.hyperpar =
        inla.extract.el(inla$summary.hyperpar,
                        paste("Theta[^ ]+ for ", name, "$", sep=""))

    ## Marginals for theta
    if (!is.null(inla$marginals$hyperpar)) {
        result$marginals.hyperpar =
            inla.extract.el(inla$marginals.hyperpar,
                            paste("Theta[^ ]+ for ", name, "$", sep=""))
    }

    ## log-tau/kappa/variance/range
    if (!is.null(inla$summary.spde2.blc[[name]])) {
        rownames(inla$summary.spde2.blc[[name]]) <- BLC.names

        result$summary.theta =
            inla.extract.el(inla$summary.spde2.blc[[name]],
                            "theta\\.[^ ]+$")
        result$summary.log.tau =
            inla.extract.el(inla$summary.spde2.blc[[name]],
                            "tau\\.[^ ]+$")
        result$summary.log.kappa =
            inla.extract.el(inla$summary.spde2.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$summary.log.variance.nominal =
            inla.extract.el(inla$summary.spde2.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$summary.log.range.nominal =
            inla.extract.el(inla$summary.spde2.blc[[name]],
                            "range.nominal\\.[^ ]+$")
    }

    ## Marginals for log-tau/kappa/variance/range
    if (!is.null(inla$marginals.spde2.blc[[name]])) {
        names(inla$marginals.spde2.blc[[name]]) <- BLC.names

        result$marginals.theta =
            inla.extract.el(inla$marginals.spde2.blc[[name]],
                            "theta\\.[^ ]+$")
        result$marginals.log.tau =
            inla.extract.el(inla$marginals.spde2.blc[[name]],
                            "tau\\.[^ ]+$")
        result$marginals.log.kappa =
            inla.extract.el(inla$marginals.spde2.blc[[name]],
                            "kappa\\.[^ ]+$")
        result$marginals.log.variance.nominal =
            inla.extract.el(inla$marginals.spde2.blc[[name]],
                            "variance.nominal\\.[^ ]+$")
        result$marginals.log.range.nominal =
            inla.extract.el(inla$marginals.spde2.blc[[name]],
                            "range.nominal\\.[^ ]+$")

        if (do.transform) {
            result$marginals.tau =
                lapply(result$marginals.log.tau,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.kappa =
                lapply(result$marginals.log.kappa,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.variance.nominal =
                lapply(result$marginals.log.variance.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
            result$marginals.range.nominal =
                lapply(result$marginals.log.range.nominal,
                       function(x) inla.tmarginal(function(y) exp(y), x))
        }
    }

    return(result)
}


#' @export
#' @rdname inla.spde.models
inla.spde2.models = function()
{
    return(c("generic", "matern", "pcmatern"))
}


## spde.common-connections:
#' @export
#' @method inla.spde.precision inla.spde2
#' @rdname inla.spde.precision
inla.spde.precision.inla.spde2 = inla.spde2.precision
#' @export
#' @method inla.spde.result inla.spde2
#' @rdname inla.spde.result
inla.spde.result.inla.spde2 = inla.spde2.result


## Deprecated:
##inla.spde.generic2 = inla.spde2.generic

## Export: inla.read.graph inla.write.graph
## Export: summary!inla.graph
## Export: plot!inla.graph
## Export: print!inla.graph.summary

## !\name{read.graph}
## !\alias{read.graph}
## !\alias{write.graph}
## !\alias{inla.read.graph}
## !\alias{inla.write.graph}
## !\alias{inla.graph}
## !\alias{summary.inla.graph}
## !\alias{plot.inla.graph}
## !\alias{print.inla.graph.summary}
## !\title{Read and write a graph-object}
## !\description{Construct a graph-object from a file or a matrix; write graph-object to file}
## !\usage{
## !inla.read.graph(..., size.only = FALSE)
## !inla.write.graph(graph, filename = "graph.dat", mode = c("binary", "ascii"), ...)
## !
## !\method{summary}{inla.graph}(object, ...)
## !\method{plot}{inla.graph}(x, y, ...)
## !\method{print}{inla.graph.summary}(x, ...)
## !}
## !\arguments{
## !    \item{filename}{The filename of the graph.}
## !    \item{graph}{An \code{inla.graph}-object, a (sparse) symmetric matrix, a filename containing the graph,
## !                 a list or collection of characters and/or numbers defining the graph,
## !                 or a neighbours list with class \code{nb} (see \code{spdep::card} and
## !                 \code{spdep::poly2nb} for for details of \code{nb} and an example a function
## !                 returning an \code{nb} object}
## !    \item{mode}{The mode of the file; ascii-file or a (gzip-compressed) binary.}
## !    \item{object}{An \code{inla.graph} -object}
## !    \item{x}{An \code{inla.graph} -object}
## !    \item{y}{Not used}
## !    \item{size.only}{Only read the size of the graph}
## !    \item{...}{Additional arguments. In \code{inla.read.graph},
## !               then it is the graph definition (object, matrix, character, filename),  plus extra arguments.
## !               In \code{inla.write.graph} it is extra arguments to \code{inla.read.graph}.}
## !}
## !\value{
## !    The output of \code{inla.read.graph}, is an \code{inla.graph} object, with elements
## !    \item{n}{is the size of the graph}
## !    \item{nnbs}{is a vector with the number of neigbours}
## !    \item{nbs}{is a list-list with the neigbours}
## !    \item{cc}{list with connected component information
## !        \itemize{
## !            \item{\code{id}}{is a vector with the connected component id for each node (starting from 1)}
## !            \item{\code{n}}{is the number of connected components}
## !            \item{\code{nodes}}{is a list-list of nodes belonging to each connected component}
## !        }
## !    }
## !    Methods implemented for \code{inla.graph} are \code{summary} and \code{plot}.
## !    The method \code{plot} require the libraries \code{Rgraphviz} and \code{graph} from the Bioconductor-project,
## !    see \url{https://www.bioconductor.org}.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{
## !    \code{\link{inla.spy}}
## !}
## !\examples{
## !## a graph from a file
## !cat("3 1 1 2 2 1 1 3 0\n", file="g.dat")
## !g = inla.read.graph("g.dat")
## !## writing an inla.graph-object to file
## !g.file = inla.write.graph(g, mode="binary")
## !## re-reading it from that file
## !gg = inla.read.graph(g.file)
## !summary(g)
## !##
## !Not run:
## !plot(g)
## !inla.spy(g)
## !## when defining the graph directly in the call,
## !## we can use a mix of character and numbers
## !g = inla.read.graph(c(3, 1, "1 2 2 1 1 3", 0))
## !inla.spy(c(3, 1, "1 2 2 1 1 3 0"))
## !inla.spy(c(3, 1, "1 2 2 1 1 3 0"),  reordering=3:1)
## !inla.write.graph(c(3, 1, "1 2 2 1 1 3 0"))
## !
## !## building a graph from adjacency matrix
## !adjacent = matrix(0, nrow = 4, ncol = 4)
## !adjacent[1,4] = adjacent[4,1] = 1
## !adjacent[2,4] = adjacent[4,2] = 1
## !adjacent[2,3] = adjacent[3,2] = 1
## !adjacent[3,4] = adjacent[4,3] = 1
## !g = inla.read.graph(adjacent)
## !plot(g)
## !summary(g)
## !End(Not run)
## !}

`inla.graph.binary.file.magic` <- function() {
    ## the value of the first integer (read binary) in a binary
    ## filename. this value must be the same as
    ## 'GMRFLib_BINARY_GRAPH_FILE_MAGIC' in GMRFLib/graph.h

    return(-1L)
}

`inla.add.graph.cc` <- function(...) {
    ## add the cc information to a graph

    args <- list(...)
    if (length(args) == 0L) {
        return(NULL)
    }

    ## need this test to avoid infinite recursion, as inla.read.graph also call inla.add.graph.cc!
    if (class(args[[1L]]) == "inla.graph") {
        graph <- args[[1L]]
    } else {
        graph <- inla.read.graph(...)
    }

    cc <- list(id = NA, n = NA, nodes = NA)
    n <- graph$n

    if (TRUE) {
        ## need these {...} as we want to define 's' globally inside
        ## these, to prevent copying the object all the time duing the
        ## recursions.

        s <- integer(n)
        s[] <- 0L
        k <- 1L

        do.visit <- function(x) x ## just to avoid warning for missing
        ## function 'do.visit' during compile
        do.visit <- inla.cmpfun(function(idxs) {
            if (any(s[idxs] == 0L)) {
                which.idxs <- idxs[which(s[idxs] == 0L)]
                s[which.idxs] <<- k
                ## its ok to refer to 'graph' here:
                visit.nodes <- unique(unlist(lapply(which.idxs, function(x) graph$nbs[[x]])))

                ## check which of the visit.nodes that needs to be
                ## visited. although this is done already in the
                ## beginning of this routine, but we do that also here
                ## to reduce the depth of the recursive call
                visit.nodes <- visit.nodes[which(s[visit.nodes] == 0L)]
                if (length(visit.nodes) > 0L) {
                    do.visit(visit.nodes)
                }
            }
            return(invisible())
        })

        ## need to allow for larger recursion depth,  temporary
        ex.save <- getOption("expressions")
        options(expressions = 500000L)
        for (i in 1L:n) {
            if (s[i] == 0L) {
                do.visit(i)
                k <- k + 1L
            }
        }
        options(paste("expressions=", ex.save, sep = ""))

        cc$id <- s
        cc$n <- max(s)
        cc$nodes <- lapply(1L:cc$n, function(cc.id, cs) sort(which(cc.id == cs)), cs = s)
    }

    graph$cc <- cc
    return(graph)
}

`inla.graph.size` <- function(...) {
    return(inla.read.graph(..., size.only = TRUE))
}


`inla.read.graph` <- function(..., size.only = FALSE) {
    ## graph is either a filename, a graph-object, a (sparse) matrix,
    ## or a list of integers or strings defining the graph.

    `inla.read.graph.ascii.internal` <- function(filename, offset = 0L, size.only = FALSE) {
        ## offset it needed if the graph is zero-based, then offset is
        ## set to 1.
        stopifnot(file.exists(filename))

        s <- readLines(filename)
        if (length(s) == 0L) {
            return(NULL)
        }

        ## remove comment lines
        s <- sapply(s, function(x) {
            return(gsub("#.*$", "", x))
        }) #
        ## convert "1 2 3" into 1 2 3
        s <- as.integer(unlist(sapply(s, function(x) strsplit(x, "[ \t]+"))))
        ## remove possibe NA's that might appear due to spaces at the end of the file
        s <- s[!is.na(s)]

        n <- s[1L]
        if (size.only) {
            return(n)
        }

        g <- list(n = n, nnbs = numeric(n), nbs = rep(list(numeric()), n))

        k <- 2L
        for (i in 1L:n) {
            if (s[k] + offset == 0L) {
                ## this is a zero-based graph
                return(inla.read.graph.ascii.internal(filename, offset = 1L))
            }

            stopifnot(s[k] + offset >= 1L && s[k] + offset <= n)
            idx <- s[k] + offset
            k <- k + 1L

            g$nnbs[idx] <- s[k]
            k <- k + 1L

            if (g$nnbs[idx] > 0L) {
                g$nbs[[idx]] <- s[k:(k + g$nnbs[idx] - 1L)] + offset
                k <- k + g$nnbs[idx]
            }
        }

        stopifnot(k - 1L == length(s))
        class(g) <- "inla.graph"
        if (length(g$nbs) < g$n) {
            g$nbs <- c(g$nbs, rep(list(numeric()), g$n - length(g$nbs)))
        }
        g <- inla.add.graph.cc(g)

        return(g)
    }

    `inla.read.graph.binary.internal` <- function(filename, offset = 0L, size.only = FALSE) {
        ## offset it needed if the graph is zero-based, then offset is
        ## set to 1.

        ## read the binary filename, which is the output from inla().
        stopifnot(file.exists(filename))

        ## read the first int,  and check that its the key.
        fp <- gzfile(filename, "rb")
        s <- as.integer(readBin(fp, integer(), n = 1L))
        close(fp)
        if (length(s) == 0L || s[1L] != inla.graph.binary.file.magic()) {
            ## then its not a binary filename
            return(NULL)
        }

        ## since we're using gzfiles (as GMRFLib do that), we don't know
        ## how many elements this file contains from looking at the
        ## size. so we got to try to read to many simply...
        n.try <- 2^12
        while (TRUE) {
            fp <- gzfile(filename, "rb")
            s <- as.integer(readBin(fp, integer(), n = n.try))
            close(fp)
            if (length(s) < n.try) {
                break
            } else {
                n.try <- n.try * 4L
            }
        }

        ## remove the key
        s <- s[-1L]

        ## then the rest is the graph
        n <- s[1L]
        if (size.only) {
            return(n)
        }
        g <- list(n = n, nnbs = numeric(n), nbs = rep(list(numeric()), n))

        ## graphs are always 1-based by definition
        k <- 2L
        for (i in 1L:n) {
            if (s[k] + offset == 0L) {
                ## this is a zero-based graph
                return(inla.read.graph.binary.internal(filename, offset = 1L))
            }

            stopifnot(s[k] + offset >= 1L && s[k] + offset <= n)
            idx <- s[k] + offset
            k <- k + 1L

            g$nnbs[idx] <- s[k]
            k <- k + 1L

            if (g$nnbs[idx] > 0L) {
                g$nbs[[idx]] <- s[k:(k + g$nnbs[idx] - 1L)] + offset
                k <- k + g$nnbs[idx]
            }
        }
        stopifnot(k - 1L == length(s))
        class(g) <- "inla.graph"
        if (length(g$nbs) < g$n) {
            g$nbs <- c(g$nbs, rep(list(numeric()), g$n - length(g$nbs)))
        }
        g <- inla.add.graph.cc(g)

        return(g)
    }

    `inla.matrix2graph.internal` <- function(Q, size.only = FALSE) {
        if (missing(Q)) {
            return(NULL)
        }

        n <- dim(Q)
        if (size.only) {
            return(n[1L])
        }
        Q <- inla.as.dgTMatrix(Q)

        if (n[1] != n[2]) {
            stop(paste("Matrix must be a square matrix, dim(Q) =", dim(Q)))
        }

        n <- dim(Q)[1]
        g <- list(n = n, nnbs = numeric(n), nbs = rep(list(numeric()), n), graph.file = NA)

        if (TRUE) {
            diag(Q) <- 1
            Q <- inla.as.sparse(Q) ## to avoid possible duplicates
            ord <- order(Q@i)
            Q@i <- Q@i[ord]
            Q@j <- Q@j[ord]
            Q@x <- Q@x[ord]
            hash.len <- table(Q@i)
            hash.idx <- c(1L, 1L + cumsum(hash.len))
            stopifnot(length(hash.len) == ncol(Q))

            for (i in 1L:n) {
                if (hash.len[i] > 1L) {
                    idx <- hash.idx[i]:(hash.idx[i] + hash.len[i] - 1L)
                    j <- Q@j[idx] + 1L
                    x <- Q@x[idx]
                    j <- j[(x != 0.0) & (j != i)]
                } else {
                    j <- NULL
                }
                g$nbs[[i]] <- j
                g$nnbs[i] <- length(j)
            }
        } else {
            if (TRUE) {
                ## new improved version, using apply. DO NOT PASS 'Q' as argument,  slower...
                g$nbs <- lapply(
                    1L:n,
                    inla.cmpfun(function(i) {
                        ## inline: row = inla.sparse.get(Q, row = i)
                        idx <- which(Q@i == i - 1L)
                        row <- list(i = i, j = Q@j[idx] + 1L, values = Q@x[idx])
                        if (length(row$j) > 0) {
                            row$j <- row$j[(row$values != 0.0) & (row$j != i)]
                        }
                        return(row$j)
                    })
                )
                g$nnbs <- sapply(g$nbs, length)
            } else {
                ## keep old version...
                for (i in 1L:n) {
                    row <- inla.sparse.get(Q, row = i)
                    nb <- length(row$j)
                    if (nb > 0) {
                        ## setting elements of a sparse-matrix to 0 does not
                        ## necessarily remove that entry.
                        row$j <- row$j[(row$values != 0.0) & (row$j != i)]
                        nb <- length(row$j)
                    }
                    g$nnbs[i] <- nb
                    if (g$nnbs[i] > 0L) {
                        g$nbs[[i]] <- row$j
                    }
                }
            }
        }
        class(g) <- "inla.graph"
        if (length(g$nbs) < g$n) {
            g$nbs <- c(g$nbs, rep(list(numeric()), g$n - length(g$nbs)))
        }
        g <- inla.add.graph.cc(g)
        return(g)
    }

    ##
    ## code starts here, really...
    ##

    args <- list(...)
    graph <- args[[1L]]

    if (is.character(graph) || length(args) > 1L ||
        (is.numeric(graph) && !(is.matrix(graph) || is(graph, "Matrix")))) {
        graph <- paste(as.character(graph))

        ## if the file exists, its a file
        if (length(graph) == 1L && file.exists(graph)) {
            ## try binary first, if it fail, try ascii
            g <- inla.read.graph.binary.internal(..., size.only = size.only)
            if (is.null(g)) {
                g <- inla.read.graph.ascii.internal(..., size.only = size.only)
            }
            return(g)
        } else {
            ## otherwise, its the definition itself
            tfile <- tempfile()
            cat(unlist(args), sep = "\n", file = tfile, append = FALSE)
            ## recursive call
            g <- inla.read.graph(tfile, size.only = size.only)
            unlink(tfile)
            return(g)
        }
    } else if (inherits(graph, "inla.graph")) {
        ## no need to do anything.
        if (size.only) {
            return(graph$n)
        } else {
            return(graph)
        }
    } else if (inherits(graph, "nb")) {
        ## a neigbour-graph from spdep with class="nb".
        ## this can replace spdep::nb2INLA.
        ## call spdep::nb2listw and use spdep coercion.
        inla.require("spdep", stop.on.error = TRUE)
        Q <- spdep::nb2listw(graph, style = "B", zero.policy = TRUE)
        Q <- inla.as.sparse(as(Q, "symmetricMatrix"))
        return(inla.matrix2graph.internal(Q, size.only = size.only))
    } else {
        return(inla.matrix2graph.internal(..., size.only = size.only))
    }

    stopifnot(FALSE)
    return(NULL)
}

`inla.write.graph` <- function(graph, filename = "graph.dat", mode = c("binary", "ascii"), ...) {
    `inla.write.graph.ascii.internal` <- function(graph, filename = "graph.dat") {
        ## write a graph read from inla.read.graph, or in that format, to
        ## file.
        fd <- file(filename, "w")
        cat(graph$n, "\n", file = fd)
        for (i in 1:graph$n) {
            cat(i, graph$nnbs[i], graph$nbs[[i]], "\n", file = fd)
        }
        close(fd)
        return(filename)
    }

    `inla.write.graph.binary.internal` <- function(graph, filename = "graph.dat") {
        ## write a graph to file,  1-based binary format.
        fd <- file(filename, "wb")
        writeBin(as.integer(inla.graph.binary.file.magic()), fd)
        writeBin(as.integer(graph$n), fd)
        if (graph$n > 0L) {
            for (i in 1:graph$n) {
                writeBin(as.integer(i), fd)
                writeBin(as.integer(graph$nnbs[i]), fd)
                if (graph$nnbs[i] > 0L) {
                    writeBin(as.integer(graph$nbs[[i]]), fd)
                }
            }
        }
        close(fd)
        return(filename)
    }

    ##
    ## code starts here
    ##
    if (missing(mode)) {
        mode <- "ascii"
    }
    mode <- match.arg(mode)
    g <- inla.read.graph(graph, ...)

    if (mode == "binary") {
        return(invisible(inla.write.graph.binary.internal(g, filename)))
    } else if (mode == "ascii") {
        return(invisible(inla.write.graph.ascii.internal(g, filename)))
    } else {
        stopifnot(FALSE)
    }
}

`plot.inla.graph` <- function(x, y, ...) {
    ## these are default options to plot for class inla.graph
    filter <- filter.args <- c("neato", "dot", "fdp", "twopi")
    attrs <- NULL
    scale <- 0.5
    node.names <- NULL
    ## we evaluate them here,  as they are set in '...'
    inla.eval.dots(...)

    if (!inla.require("Rgraphviz")) {
        stop("Package 'Rgraphiviz' is required but not installed. Please install from 'https://www.bioconductor.org'.")
    }
    if (!inla.require("graph")) {
        stop("Package 'graph' is required but not installed. Please install from 'https://www.bioconductor.org'.")
    }

    filter <- match.arg(filter, filter.args)
    if (is.null(attrs)) {
        attrs <- Rgraphviz::getDefaultAttrs(layoutType = filter)
    }
    if (!is.null(node.names)) {
        stopifnot(length(node.names) == x$n)
    } else {
        node.names <- as.character(1:x$n)
    }
    g <- graph::graphNEL(nodes = node.names, edgemode = "undirected")
    for (i in 1L:x$n) {
        if (x$nnbs[i] > 0L) {
            j <- x$nbs[[i]]
            j <- j[j > i]
            if (length(j) > 0L) {
                g <- graph::addEdge(node.names[i], node.names[j], g)
            }
        }
    }
    attrs$node$height <- as.numeric(attrs$node$height) * scale
    attrs$node$width <- as.numeric(attrs$node$width) * scale
    plot(g, filter, attrs = attrs, ...)
}

`summary.inla.graph` <- function(object, ...) {
    ret <- list()
    ret <- c(ret, list(n = object$n))
    if (!is.null(object$cc)) {
        ret <- c(ret, list(ncc = object$cc$n))
    } else {
        ret <- c(ret, list(ncc = NA))
    }
    ret <- c(ret, list(nnbs = table(object$nnbs)))

    class(ret) <- "inla.graph.summary"
    return(ret)
}

`print.inla.graph.summary` <- function(x, ...) {
    cat(paste("\tn = ", x$n, "\n"))
    cat(paste("\tncc = ", x$ncc, "\n"))
    w <- max(nchar(names(x$nnbs)))
    cat(inla.paste(c("\tnnbs = (names) ", format(names(x$nnbs), width = w, justify = "right"), "\n")))
    cat(inla.paste(c("\t       (count) ", format(x$nnbs, width = w, justify = "right"), "\n")))
    return(invisible())
}

context("test 'latent besagproper'")

test_that("Case 1", {
    set.seed(12)
    ## pick a graph
    graph.file = system.file("demodata/germany.graph", package="INLA")
    g = inla.read.graph(graph.file)

    ## we will use replicated samples in our testing
    nrep = 5

    ## make life easy; use dense matrix algebra
    d = 1.0
    tau = 1.0
    Q = matrix(0, g$n, g$n)
    diag(Q) = tau * (d + g$nnbs)
    for(i in 1:g$n) {
        if (g$nnbs[i] > 0) {
            Q[i, g$nbs[[i]]] = -tau
            Q[g$nbs[[i]], i] = -tau
        }
    }
    R = chol(Q) ## 'chol' returns the upper triangular
    ## simulate data with replications
    y = c()
    for(i in 1:nrep) {
        y = c(y, backsolve(R, rnorm(g$n)))
    }
    i = rep(1:g$n, nrep)
    replicate = rep(1:nrep, each = g$n)
    formula = y ~ f(i, model="besagproper",  graph = graph.file,
            replicate=replicate,
            hyper = list(diag = list(param = c(1, 1)))) -1
    ## use 'exact' observations, so we fix the noise precisin to a high
    ## value
    r = inla(formula,
            data = data.frame(y, i, replicate),
            family = "gaussian",
            control.family = list(
                    hyper = list(
                            prec = list(
                                    initial = 10,
                                    fixed=TRUE))))
    expect_true(abs(r$summary.hyperpar[1, "mean"] < d) < 0.05)
})

context("test 'latent bym2'")

test_that("Case 1", {
    set.seed(123)
    library(MASS)
    data(Germany)
    g = inla.read.graph(system.file("demodata/germany.graph", package="INLA"))
    Germany = cbind(Germany,region.struct=Germany$region)
    Q = -inla.graph2matrix(g)
    diag(Q) = g$nnbs
    Q = Q * exp(mean(log(diag(ginv(as.matrix(Q))))))
    tau = 1
    rho = 0.3
    n = dim(Q)[1]
    u = inla.qsample(n = 1,  Q = Q + 1e-8 * Diagonal(n),
            constr=list(A = matrix(1, 1, n), e=0), seed=1234L)
    v = rnorm(n)
    x = 1/sqrt(tau) * (sqrt(1-rho) * v + sqrt(rho) * u)
    intercept = log(Germany$E)
    y = rpois(n, lambda = exp(intercept + x))
    
    formula = y ~ 1 + f(region,
            model = "bym2", graph = g, constr=TRUE)
    r = inla(formula, family="poisson",
            data=data.frame(y, region=Germany$region))
    expect_true(abs(r$summary.hyperpar[1,"mean"] - tau) < 0.4)
    expect_true(abs(r$summary.hyperpar[2,"mean"] - rho) < 0.175)
})

context("test 'latent besagproper2'")

test_that("Case 1", {
    set.seed(1234)
    ## pick a graph
    graph.file = system.file("demodata/germany.graph", package="INLA")
    g = inla.read.graph(graph.file)
    ## we will use replicated samples in our testing
    nrep = 5
    tau = 10.0
    lambda = 0.3
    R = -inla.graph2matrix(g)
    diag(R) = g$nnbs
    n = g$n
    Q = tau * ( (1-lambda) * diag(n) + lambda * R)
    y = c(inla.qsample(nrep, Q))
    i = rep(1:g$n, nrep)
    replicate = rep(1:nrep, each = g$n)
    formula = y ~ f(i, model="besagproper2",  graph = g,
            replicate=replicate) - 1
    r = inla(formula,
            data = data.frame(y, i, replicate),
            family = "gaussian",
            control.family = list(
                    hyper = list(
                            prec = list(
                                    initial = 10,
                                    fixed=TRUE))))
    expect_true(abs(r$summary.hyperpar[2, "mean"] - lambda) < 0.05)
})

## Export: inla.pardiso inla.pardiso.check

## !\name{inla.pardiso}
## !\alias{inla.pardiso}
## !\alias{inla.pardiso.check}
## !\title{PARDISO support in R-INLA}
## !\description{
## !Describe and check the PARDISO support in R-INLA
## !}
## !\usage{
## !inla.pardiso()
## !inla.pardiso.check()
## !}
## !\arguments{
## !}
## !\details{
## !\code{inla.pardiso()} describes the \code{PARDISO} support in R-INLA, how to get the license
## !key and enable it in the \code{R-INLA} package. \code{inla.pardiso.check()} check if the
## !\code{PARDISO} support is working.
## !}
## !\value{
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}

`inla.pardiso` <- function() {
    browseURL("https://pardiso-project.org/r-inla/")
    cat(
        "\n\n",
        "The R-INLA page at the PARDISO project should open in your browser, \n",
        "which gives more information (https://pardiso-project.org/r-inla)\n",
        "\n",
        "The PARDISO package is a thread-safe, high-performance, and easy\n",
        "to use software for solving sparse symmetric that arises in the\n",
        "R-INLA approach to Bayesian inference within the R project for \n",
        "Statistical Computing.\n",
        "R-INLA supports PARDISO for MacOSX and Linux.\n",
        "Windows is not supported for the moment.\n",
        "\n",
        "To improve R-INLA within parallel and high-performance computing, \n",
        "we need a modern sparse-matrix library to build upon, and\n",
        "and for this, PARDISO, is the very best choice in our opinion.\n",
        "\n",
        "The PARDISO library is not enabled by default\n",
        "when calling inla(). You can you can enable it, by\n",
        "\n",
        "1. Obtaining a license key from pardiso-project.org/r-inla\n",
        "   The licenses is tied up to the username, so the same license\n",
        "   can be used one all computers where you have the same username.\n",
        "   (You can also collect various licenses in the same file, each\n",
        "   connected with different usernames.)\n",
        "2. Add the license key to R-INLA, by assigning the full path\n",
        "   of the license-file to variable 'pardiso.license', for example,\n",
        "     library(INLA)\n",
        "     inla.setOption(pardiso.license=\"~/sys/licenses/pardiso.lic\")\n",
        "3. That is it!\n",
        "4. You can check that PARDISO is enabled and is working, by\n",
        "     inla.pardiso.check()\n",
        "\n",
        "Using remote computing with inla.call=\"remote\", you may need to add\n",
        "the full path to the license file (on the remote host) in the\n",
        "~/.inlarc file:\n",
        "     PardisoLicenseFile=\"FullPathToTheLicenseFile\"\n",
        "Otherwise, your local license file will be used if its available.\n",
        "\n",
        "You can also request PARDISO explicitely with \n",
        "     control.compute=list(openmp.strategy=\"pardiso\"), \n",
        "Nested parallelism, is controlled by the \"num.threads\" argument\n",
        "     num.threads=\"A:B\" \n",
        "with A threads in the outer layer and B threads in the inner.\n",
        "(Think of this as dealing with A matrices at the same time, and \n",
        "using B threads on each of them.)\n",
        "Good choices are\n",
        "-  A=number.of.hyperparameters+1 or A=2*number.of.hyperparameters\n",
        "-  B=a low number like 1, 2, 4, ... higher for huge models.\n",
        "You also need to take the number of cores/threads available into account.\n",
        "A zero can be used for both A and B and their values will be set by the program.\n",
        "\n",
        "PARDISO support is still under development, and \n",
        "changes in the interface/behaviour might occour.\n",
        "\n",
        "Please report problems to help@r-inla.org\n",
        "\n",
        "Havard Rue & Olaf Schenk\n",
        "May 2018\n",
        "Updated August 2020\n",
        "\n"
    )
}

`inla.pardiso.check` <- function() {
    t.dir <- inla.tempdir()
    inla.set.sparselib.env(inla.dir = t.dir)
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        ret <- system(paste(shQuote(inla.call.no.remote()), "-m pardiso"), intern = TRUE)
    } else if (inla.os("windows")) {
        ret <- system(paste(shQuote(inla.call.no.remote()), "-m pardiso"), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }
    unlink(t.dir, recursive = TRUE)

    return(ret)
}

## Export: f

## !\name{f}
## !\alias{f}
## !\title{Define general Gaussian models in the INLA formula }
## !\description{
## !
## !  Function used for defining of smooth and spatial terms within \code{inla} model
## !  formulae. The function does not evaluate anything - it
## !  exists purely to help set up a model. The function specifies one
## !  smooth function in the linear predictor (see \code{\link{inla.list.models}}) as
## !  \deqn{w\ f(x)}{weight*f(var)}
## !
## !}
## !\usage{
## !    f(...,
## !         model = "iid",
## !         copy=NULL,
## !         same.as = NULL,
## !         n=NULL,
## !         nrep = NULL,
## !         replicate = NULL,
## !         ngroup = NULL,
## !         group = NULL,
## !         control.group = inla.set.control.group.default(),
## !         hyper = NULL,
## !         initial=NULL,
## !         prior=NULL,
## !         param = NULL,
## !         fixed = NULL,
## !         season.length=NULL,
## !         constr = NULL,
## !         extraconstr=list(A=NULL, e=NULL),
## !         values=NULL,
## !         cyclic = NULL,
## !         diagonal = NULL,
## !         graph=NULL,
## !         graph.file=NULL,
## !         cdf=NULL,
## !         quantiles=NULL,
## !         Cmatrix=NULL,
## !         rankdef=NULL,
## !         Z = NULL,
## !         nrow = NULL,
## !         ncol = NULL,
## !         nu = NULL,
## !         bvalue = NULL,
## !         spde.prefix = NULL,
## !         spde2.prefix = NULL,
## !         spde2.transform = c("logit", "log", "identity"),
## !         spde3.prefix = NULL,
## !         spde3.transform = c("logit", "log", "identity"),
## !         mean.linear = inla.set.control.fixed.default()$mean,
## !         prec.linear = inla.set.control.fixed.default()$prec,
## !         compute = TRUE,
## !         of=NULL,
## !         precision = exp(14),
## !         range = NULL,
## !         adjust.for.con.comp = TRUE,
## !         order = NULL,
## !         scale = NULL,
## !         strata = NULL,
## !         rgeneric = NULL,
## !         scale.model = NULL,
## !         args.slm = list(rho.min = NULL, rho.max = NULL,
## !                         X = NULL, W = NULL, Q.beta = NULL),
## !         args.ar1c = list(Z = NULL, Q.beta = NULL),
## !         args.intslope = list(subject = NULL, strata = NULL, covariates = NULL),
## !         vb.correct = NULL,
## !         locations = NULL,
## !         debug = FALSE)
## !}
## !\arguments{
`f` <- function(
                ## !\item{...}{ Name of the covariate and, possibly of the
                ## !weights vector. NB: order counts!!!! The first specified
                ## !term is the covariate and the second one is the vector of
                ## !weights (which can be negative).}
                ...,

                ## !\item{model}{ A string indicating the choosen model. The
                ## ! default is \code{iid}. See
                ## ! \code{names(inla.models()$latent)} for a list of possible
                ## ! alternatives and \code{\link{inla.doc}} for detailed docs.}
                model = "iid",

                ## !\item{copy}{TODO}
                copy = NULL,

                ## !\item{same.as}{TODO}
                same.as = NULL,

                ## !\item{n}{An optional argument which defines the dimension
                ## !of the model if this is different from
                ## !\code{length(sort(unique(covariate)))}}
                n = NULL,

                ## !\item{nrep}{TODO}

                nrep = NULL,

                ## !\item{replicate}{We need to write documentation here}
                replicate = NULL,

                ## !\item{ngroup}{TODO}
                ngroup = NULL,

                ## !\item{group}{TODO}
                group = NULL,

                ## !\item{control.group}{TODO}
                control.group = inla.set.control.group.default(),

                ## !\item{hyper}{Specification of the hyperparameter, fixed or
                ## !random, initial values, priors and its parameters. See
                ## !\code{?inla.models} for the list of hyparameters for each
                ## !model and its default options or
                ## !use \code{inla.doc()} for
                ## !detailed info on the family and
                ## !supported prior distributions.}
                hyper = NULL,

                ## !\item{initial}{THIS OPTION IS OBSOLETE; use
                ## !\code{hyper}!!! Vector indicating the starting values for
                ## !the optimization algorithm. The length of the vector
                ## !depends on the number of hyperparamters in the choosen
                ## !\code{model}. If \code{fixed=T} the value at which the
                ## !parameters are fixed is determines through \code{initial}.
                ## !See \code{inla.models()$latent$'model name'} to have info
                ## !about the choosen model.}
                initial = NULL,

                ## !\item{prior}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
                ## !Prior distribution(s) for the hyperparameters of the
                ## !!random model. The default value depends on the type of
                ## !model, see !\url{www.r-inla.org} for a detailed
                ## !description of the models. See
                ## !\code{names(inla.models()$priors)} for possible prior
                ## !choices}
                prior = NULL,

                ## !\item{param}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
                ## !Vector indicating the parameters \eqn{a}{a} and \eqn{b}{b}
                ## !of the prior distribution for the hyperparameters. The
                ## !length of the vector depends on the choosen \code{model}.
                ## !See \code{inla.models()$latent$'model name'} to have info
                ## !about the choosen model.}
                param = NULL,

                ## !\item{fixed}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
                ## !Vector of boolean variables indicating wheater the
                ## !hyperparameters of the model are fixed or random. The
                ## !length of the vector depends on the choosen \code{model}
                ## !See \code{inla.models()$latent$'model name'} to have info
                ## !about the choosen model.}
                fixed = NULL,

                ## !\item{season.length}{Lenght of the seasonal compoment
                ## !(ONLY if \code{model="seasonal"}) }
                season.length = NULL,

                ## !\item{constr}{A boolean variable indicating whater to set
                ## !a sum to 0 constraint on the term. By default the sum to 0
                ## !constraint is imposed on all intrinsic models
                ## !("iid","rw1","rw1","besag", etc..).}
                constr = NULL,

                ## !\item{extraconstr}{This argument defines extra linear
                ## !constraints. The argument is a list with two elements, a
                ## !matrix \code{A} and a vector \code{e}, which defines the
                ## !extra constraint \code{Ax = e}; for example
                ## !\code{extraconstr = list(A = A, e=e)}. The number of
                ## !columns of \code{A} must correspond to the length of this
                ## !\code{f}-model.  Note that this constraint comes
                ## !additional to the sum-to-zero constraint defined if
                ## !\code{constr = TRUE}.}
                extraconstr = list(A = NULL, e = NULL),

                ## !\item{values}{An optional vector giving all values
                ## !assumed by the covariate for which we want estimated the
                ## !effect. It must be a numeric vector, a vector of factors
                ## !or \code{NULL}.}
                values = NULL,

                ## !\item{cyclic}{A boolean specifying wheather the model is
                ## !cyclical. Only valid for "rw1" and "rw2" models, is
                ## !cyclic=T then the sum to 0 constraint is removed. For the
                ## !correct form of the grah file see \cite{Martino and Rue
                ## !(2008)}.}
                cyclic = NULL,

                ## !\item{diagonal}{An extra constant added to the diagonal of
                ## !the precision matrix.}
                diagonal = NULL,

                ## !\item{graph}{Defines the graph-object either as a file with
                ## !a graph-description, an \code{inla.graph}-object, or as a
                ## !(sparse) symmetric matrix.}
                graph = NULL,

                ## !\item{graph.file}{THIS OPTION IS OBSOLETE AND REPLACED BY
                ## !THE MORE GENERAL ARGUMENT \code{graph}. PLEASE CHANGE YOUR
                ## !CODE.
                ## !Name of the file containing the graph
                ## !of the model; see
                ## !\url{www.r-inla.org/faq}.}
                graph.file = NULL,

                ## !\item{cdf}{A vector of maximum 10 values between 0 and 1
                ## !\eqn{x(0), x(1),\ldots}{x(0), x(1),\ldots}. The function
                ## !returns, for each posterior marginal the probabilities
                ## !\deqn{\mbox{Prob}(X<x(p))}{Prob(X<x(p))} }
                cdf = NULL,

                ## !\item{quantiles}{A vector of maximum 10 quantiles,
                ## !\eqn{p(0), p(1),\dots}{p(0), p(1),\ldots} to compute for
                ## !each posterior marginal. The function returns, for each
                ## !posterior marginal, the values
                ## !\eqn{x(0), x(1),\dots}{x(0), x(1),\ldots} such that
                ## !\deqn{\mbox{Prob}(X<x(p))=p}{Prob(X<x)=p}}
                quantiles = NULL,

                ## !\item{Cmatrix}{The specification of the precision matrix
                ## !for the generic,  generic3 or z models (up to a scaling constant).
                ## !\code{Cmatrix} is either a
                ## !(dense) matrix, a matrix created using
                ## !\code{Matrix::sparseMatrix()}, or a filename which stores the
                ## !non-zero elements of \code{Cmatrix}, in three columns:
                ## !\code{i}, \code{j} and \code{Qij}. In case of the generic3 model,
                ## !it is a list of such specifications.}
                Cmatrix = NULL,

                ## !\item{rankdef}{A number \bold{defining} the rank
                ## !deficiency of the model, with sum-to-zero constraint and
                ## !possible extra-constraints taken into account. See
                ## !details.}
                rankdef = NULL,

                ## !\item{Z}{The matrix for the z-model}
                Z = NULL,

                ## !\item{nrow}{Number of rows for 2d-models}
                nrow = NULL,

                ## !\item{ncol}{Number of columns for 2d-models}
                ncol = NULL,

                ## !\item{nu}{Smoothing parameter for the Matern2d-model,
                ## !possible values are \code{c(0, 1, 2, 3)}}
                nu = NULL,

                ## !\item{bvalue}{TODO}
                bvalue = NULL,

                ## !\item{spde.prefix}{TODO}
                spde.prefix = NULL,

                ## !\item{spde2.prefix}{TODO}
                spde2.prefix = NULL,

                ## !\item{spde2.transform}{TODO}
                spde2.transform = c("logit", "log", "identity"),

                ## !\item{spde3.prefix}{TODO}
                spde3.prefix = NULL,

                ## !\item{spde3.transform}{TODO}
                spde3.transform = c("logit", "log", "identity"),

                ## !\item{mean.linear}{Prior mean for the linear component,
                ## !only used if \code{model="linear"}}
                mean.linear = inla.set.control.fixed.default()$mean,

                ## !\item{prec.linear}{Prior precision for the linear
                ## !component, only used if \code{model="linear"}}
                prec.linear = inla.set.control.fixed.default()$prec,

                ## !\item{compute}{ A boolean variable indicating wheather the
                ## ! marginal posterior distribution for the nodes in the
                ## ! \code{f()} model should be computed or not. This is
                ## ! usefull for large models where we are only interested in
                ## ! some posterior marginals.}
                compute = TRUE,

                ## !\item{of}{TODO}
                of = NULL,

                ## !\item{precision}{The precision for the artifical noise added when creating a copy of a model and others.}
                precision = exp(13),

                ## !\item{range}{A vector of size two giving the lower and
                ## !upper range for the scaling parameter \code{beta} in the
                ## !model \code{COPY}, \code{CLINEAR}, \code{MEC} and \code{MEB}.
                ## !If \code{low = high} then the identity mapping
                ## !is used.}
                range = NULL,

                ## !\item{adjust.for.con.comp}{If TRUE (default), adjust some
                ## !of the models (currently: besag, bym, bym2 and besag2) if the
                ## !number of connected components in graph is larger than
                ## !1. If FALSE, do nothing.}
                adjust.for.con.comp = TRUE,

                ## !\item{order}{Defines the \code{order} of the model: for
                ## !model \code{ar} this defines the order p, in AR(p). Not
                ## !used for other models at the time being.}
                order = NULL,

                ## !\item{scale}{A scaling vector. Its meaning depends on the model.}
                scale = NULL,

                ## !\item{strata}{Currently not in use}
                strata = NULL,

                ## !\item{rgeneric}{A object of class \code{inla.rgeneric} which defines the model. (EXPERIMENTAL!)}
                rgeneric = NULL,

                ## !\item{scale.model}{Logical. If \code{TRUE} then scale the RW1 and RW2 and BESAG and BYM and BESAG2 and RW2D models so the their (generlized) variance is 1. Default value is \code{inla.getOption("scale.model.default")}}
                scale.model = NULL,

                ## !\item{args.slm}{Required arguments to the model="slm"; see the documentation for further details.},
                args.slm = list(rho.min = NULL, rho.max = NULL, X = NULL, W = NULL, Q.beta = NULL),

                ## !\item{args.ar1c}{Required arguments to the model="ar1c"; see the documentation for further details.},
                args.ar1c = list(Z = NULL, Q.beta = NULL),

                ## !\item{args.intslope}{A list with the \code{subject} (factor),  \code{strata} (factor) and \code{covariates} (numeric) for the \code{intslope} model; see the documentation for further details.},
                args.intslope = list(subject = NULL, strata = NULL, covariates = NULL),

                ## !\item{vb.correct}{Add this model component to the list of variables to be used for the vb corrected Laplace approximation? If \code{NULL} use default choice,  otherwise correct if \code{TRUE} and do not if \code{FALSE}. (expert option)},
                vb.correct = NULL,

                ## !\item{locations}{A matrix with locations for the model \code{dmatern}. This also defines \code{n}.}
                locations = NULL,

                ## !\item{debug}{Enable local debug output}
                debug = FALSE) {
    ## !}
    ## !\value{TODO}

    ## !\details{There is no default value for \code{rankdef}, if it
    ## !is not defined by the user then it is computed by the rank
    ## !deficiency of the prior model (for the generic model, the
    ## !default is zero), plus 1 for the sum-to-zero constraint if the
    ## !prior model is proper, plus the number of extra
    ## !constraints. \bold{Oops:} This can be wrong, and then the user
    ## !must define the \code{rankdef} explicitely.}
    ## !\author{Havard Rue \email{hrue@r-inla.org}}
    ## !\seealso{\code{\link{inla}}, \code{\link{hyperpar.inla}}}

    ## this is required. the hyper.defaults can only be changed in the
    ## model=model.object
    hyper.default <- NULL

    empty.extraconstr <- function(ec) {
        return(is.null(ec) || all(sapply(ec, is.null)))
    }

    ## if model is a particular class, then use this to set default
    ## arguments to all names(formals(f)) (except the "...").
    ## THIS FEATURE IS EXPERIMENTAL FOR THE MOMENT!  OOPS: the classes
    ## are defined in the function inla.model.object.classes() as this
    ## is also used in the inla() function itself.
    if (any(inherits(model, inla.model.object.classes()))) {
        if (any(inherits(model, inla.spde.object.classes()))) {
            ## Write specific spde models to disk. These files are
            ## removed later in inla.copy.dir.for.section.spde, in section.R
            my.spde.prefix <- inla.fmesher.make.prefix(NULL, NULL)
            if (any(inherits(model, "inla.spde1"))) {
                spde.prefix <- my.spde.prefix
                ## inla checks PREFIX valididy by looking for "s"
                model$param.inla <- list(
                    s = model$mesh$loc,
                    c0 = model$internal$c0,
                    g1 = model$internal$g1,
                    g2 = model$internal$g2,
                    basis.T = model$internal$basis.T,
                    basis.K = model$internal$basis.K
                )
                spde.matrices <- names(model$param.inla)
            } else if (any(inherits(model, "inla.spde2"))) {
                spde2.prefix <- my.spde.prefix
                spde.matrices <- c(
                    "M0", "M1", "M2",
                    "B0", "B1", "B2", "BLC"
                )
            } else if (any(inherits(model, "inla.spde3"))) {
                spde3.prefix <- my.spde.prefix
                spde.matrices <- c(
                    "M0", "M1", "M2", "M3",
                    "B0", "B1", "B2", "B3", "BLC"
                )
            }
            for (spde.matrix.name in spde.matrices) {
                ## Only write matrix if it is non-empty (only happens for BLC)
                if (nrow(model$param.inla[[spde.matrix.name]]) > 0) {
                    fmesher.write(
                        inla.affirm.double(
                            model$param.inla[[spde.matrix.name]]
                        ),
                        my.spde.prefix, spde.matrix.name
                    )
                }
            }
        }

        atmp <- paste(unlist(lapply(as.list(formals(INLA::f)), function(x) names(x))))
        arguments <- unique(sort(c(names(formals(INLA::f)), atmp[-which(nchar(atmp) == 0L)])))
        arguments <- arguments[-grep("^[.][.][.]$", arguments)]
        ## add this one manually
        arguments <- c(arguments, "hyper.default")

        ## evaluate arguments in 'model' and set those that are in
        ## 'arguments'. However, due to the 'missing || is.null', the
        ## argument 'model' does not pass the test and must be set
        ## manually afterwards.
        for (nm in names(model$f)) {
            if (inla.one.of(nm, arguments)) {
                inla.eval(paste("if (missing(", nm, ") || is.null(", nm, ")) ",
                    nm, "=", "model$f$", nm,
                    sep = ""
                ))
            } else {
                ## this should not happen
                stop(paste("Argument `", nm, "' is not an argument in f().",
                    " This is likely not what you want!",
                    sep = ""
                ))
            }
        }

        ## the 'model' argument is required!
        stopifnot(is.character(model$f$model))
        model <- model$f$model
    }

    ## this is a nice trick
    if (!is.null(copy)) {
        if (!missing(model)) {
            warning(paste("Ignored argument model=`", model,
                "' in f() due to copy=`", copy, "'",
                sep = ""
            ))
        }
        if (!is.null(of)) {
            stop("Argument `of=NULL' is required when `copy=...' is used.")
        }
        model <- "copy"
        of <- copy
        copy <- NULL
    }

    if (is.null(model)) {
        stop("No model is specified.")
    }
    inla.is.model(model, "latent", stop.on.error = TRUE)

    if (!is.null(constr)) {
        if (inla.one.of(model, c("spde3", "spde2", "spde")) && constr) {
            stop("Option 'constr=TRUE' is disabled for model='spde2' and 'spde' and 'spde3'; please refer to the spde-tutorial.")
        }
    }

    ## in ... is the name of the covariate  and possibly the location of the weights
    ## like f(covariate, weights)
    vars <- as.list(substitute(list(...)))[-1]
    d <- length(vars)
    if (d == 0L) {
          stop(paste("Missing covariate in f(...) for model=", model))
      }
    term <- deparse(vars[[1]], backtick = TRUE, width.cutoff = 500)
    if (debug) {
        print(vars)
    }
    ## the second term in ... is the (possible) weights for the selected covariate!
    if (d == 1) {
        weights <- NULL
    } else if (d == 2) {
        weights <- deparse(vars[[2]], backtick = TRUE, width.cutoff = 500)
    } else if (d > 2) {
        stop(paste("To many variables included in f():", inla.paste(vars)))
    } else if (d == 0) {
        stop("At least one variable in f() needs to be defined")
    }
    ## get the weights
    term <- attr(terms(reformulate(term)), "term.labels")
    if (d > 1) {
        weigths <- attr(terms(reformulate(weights)), "weights.labels")
    }

    ## set the hyperparameters
    hyper <- inla.set.hyper(
        model = model, section = "latent",
        hyper = hyper, hyper.default = hyper.default,
        initial = initial, fixed = fixed, prior = prior, param = param
    )

    ## for model = copy, its is not allowed to define constr or extraconstr
    if (inla.one.of(model, "copy")) {
        stopifnot(missing(constr))
        stopifnot(missing(extraconstr))
    }

    if (!inla.one.of(model, c("copy", "clinear", "mec", "meb"))) {
        stopifnot(missing(range))
    }

    if (!missing(same.as) && !is.null(same.as) && model != "copy") {
        stop(paste("Argument 'same.as' must be NULL is model != 'copy' :", model))
    }

    inla.check.control(control.group)
    cont.group <- inla.set.control.group.default()
    cont.group[(namc <- names(control.group))] <- control.group
    cont.group$hyper <- inla.set.hyper(
        cont.group$model, "group", cont.group$hyper,
        cont.group$initial, cont.group$fixed, cont.group$prior, cont.group$param
    )

    ## CHECK ARGUMENTS.
    ## This is a bit tricky. We want to check if there are arguments
    ## in `...' which is of type `name = value' which are invalid. If
    ## so, this lead to an obscoure error later on...
    ##
    ## we first collect all arguments of type `name = value'
    if (TRUE) {
        ## New code
        args.eq <- names(match.call(expand.dots = TRUE))
        args.eq <- args.eq[args.eq != ""]
    } else {
        args.eq <- c()
        for (arg in unlist(strsplit(as.character(as.expression(match.call(expand.dots = TRUE))), ","))) {
            if (length(grep("=", arg)) > 0) {
                args.eq <- c(args.eq, gsub(" ", "", unlist(strsplit(arg, "="))[1]))
            }
        }
    }

    ## then we compare these with the legal ones in f(), and
    ## flag an error its not among the legal ones.
    atmp <- paste(unlist(lapply(as.list(formals(INLA::f)), function(x) names(x))))
    arguments <- unique(sort(c(names(formals(INLA::f)), atmp[-which(nchar(atmp) == 0L)])))
    arguments <- arguments[-grep("^[.][.][.]$", arguments)]
    for (elm in args.eq) {
        if (!is.element(elm, arguments)) {
            f.call <- as.character(as.expression(match.call(expand.dots = TRUE)))
            valid.args <- inla.paste(sort(arguments), sep = "\n\t")
            stop(paste("Argument `", elm, "' in formula specification\n\n\t\t",
                f.call, "\n\n  is invalid. Valid arguments are:\n\n\t", valid.args,
                sep = ""
            ))
        }
    }

    ## check that 'order' is defined
    if (inla.one.of(model, c("ar"))) {
        if (is.null(order) || missing(order) || order < 1L) {
            stop("Model 'ar' needs 'order' to be defined as an integer > 0.")
        }
        order <- as.integer(order)
        max.order <- length(inla.models()$latent$ar$hyper) - 1L
        if (order > max.order) {
            stop(paste("Model 'ar': order=", order, ", is to large. max.order =", max.order, sep = ""))
        }
    }
    if (inla.one.of(model, c("fgn", "fgn2"))) {
        if (is.null(order) || missing(order)) {
            order <- inla.models()$latent$fgn$order.default
        } else {
            order <- as.integer(order)
        }
        stopifnot(any(order == inla.models()$latent$fgn$order.defined))
    }
    if (inla.one.of(model, c("iidkd"))) {
        if (is.null(order) || missing(order)) {
            stop("Model 'iidkd' needs 'order' to be defined")
        } else {
            order <- as.integer(order)
            if (!(order >= 2 && order <= 10)) {
                stop("Model 'iidkd' needs 'order' to be in the interval 2 to 10")
            }
        }
    }

    ## Check that the Cmatrix is defined for those models needing it, and oposite.
    if (!inla.one.of(model, "z")) {
        if (inla.one.of(model, c("generic", "generic0", "generic1", "generic2"))) {
            if (is.null(Cmatrix)) {
                stop("For generic models the Cmatrix has to be provided")
            }
            Cmatrix <- inla.sparse.check(Cmatrix)
            if (is.null(n)) {
                n <- inla.sparse.dim(Cmatrix)[1]
            }
        } else if (inla.one.of(model, "generic3")) {
            if (!is.list(Cmatrix) || (is.list(Cmatrix) && length(Cmatrix) == 0L)) {
                stop("For model='generic3' then argument 'Cmatrix' must be a non-empty list of matrices.")
            }
            stopifnot(length(Cmatrix) > 0L)
            ## check each matrix
            Cdim <- numeric(length(Cmatrix))
            for (i in 1:length(Cmatrix)) {
                Cmatrix[[i]] <- inla.sparse.check(Cmatrix[[i]], must.be.squared = TRUE)
                Cdim[i] <- inla.sparse.dim(Cmatrix[[i]])[1L]
            }
            stopifnot(all(Cdim == Cdim[1L]))
            if (is.null(n)) {
                n <- Cdim[1L]
            }
        } else {
            if (!is.null(Cmatrix)) {
                stop(paste("Cmatrix is not used for this model:", model))
            }
        }
    } else {
        ## for the z-model, then Cmatrix is required but Cmatrix=NULL is allowed; so we do not
        ## need to do anything at this stage.
    }

    if (!is.null(graph) && !is.null(graph.file)) {
        stop("Don't know what to do, as 'graph' and 'graph.file' is set; please use only argument 'graph'.")
    }
    if (is.null(graph) && !is.null(graph.file)) {
        ## then it is easy and we can just go ahead with 'graph'
        graph <- graph.file

        ## show a warning,  if this hasn't been shown before.
        if (inla.getOption("show.warning.graph.file")) {
            warning("Argument 'graph.file' in 'f()' is obsolete; please use the more general argument 'graph' instead.")
            ## disable this warning from now on.
            inla.setOption(show.warning.graph.file = FALSE)
        }
    }

    ## chech that the graph is provided, if required. Set 'n' from the graph.
    if (inla.one.of(model, c("besag", "bym", "bym2", "besagproper", "besagproper2"))) {
        if (is.null(graph)) {
            stop(paste("The 'graph' has to be provided for model", model))
        }
        n.from.graph <- inla.graph.size(graph)
        if (n.from.graph <= 0) {
            stop(paste("Argument 'n from graph' is void:", n.from.graph))
        }
        if (!is.null(n) && n != n.from.graph) {
            stop(paste("Argument 'n' and 'n from graph' does not match", n, n.from.graph))
        }
        n <- n.from.graph
    }
    if (inla.one.of(model, c("besag2"))) {
        if (is.null(graph)) {
            stop(paste("The graph has to be provided for model", model))
        }
        ## read n from the graph
        nn <- inla.graph.size(graph)
        n.from.graph <- 2L * nn
        if (n.from.graph <= 0) {
            stop(paste("Argument 'n from graph' is void:", n.from.graph))
        }
        if (!is.null(n) && n != n.from.graph) {
            stop(paste("Argument 'n' and 2*'n from graph' does not match", n, n.from.graph))
        }
        n <- n.from.graph

        ## this is a special case
        if (is.null(constr) || constr) {
            constr <- FALSE
            con <- list(
                A = rbind(
                    rep(1:0, each = nn),
                    rep(0:1, each = nn)
                ),
                e = c(0, 0)
            )
            extraconstr$A <- rbind(extraconstr$A, con$A)
            extraconstr$e <- c(extraconstr$e, con$e)
            if (missing(rankdef) || is.null(rankdef)) {
                rankdef <- nrow(extraconstr$A) %/% 2L
            }
        }
        if (is.null(diagonal)) {
            ## need a larger value...
            diagonal <- inla.set.f.default()$diagonal * 10
        }
    }

    if (inla.one.of(model, c("z"))) {
        if (is.null(Z)) {
            stop("With model [z] then covariate-matrix Z is required. Example: f(ind, Z=Z, model=\"z\")")
        }
        if (!is.null(rankdef) && rankdef != 0) {
            stop("Option rankdef!=0, is not allowed for model='z'; use 'constr' or 'extraconstr' to define intrinsic models from proper ones.")
        }
        if (is.null(n)) {
            n <- sum(dim(Z))
        }
        if (!is.null(constr) && constr == TRUE) {
            ## let constr=TRUE be defined as sum(z)=0 only.
            constr <- FALSE
            zn <- dim(Z)[1L]
            zm <- dim(Z)[2L]
            z.row <- c(rep(0, zn), rep(1, zm))
            if (empty.extraconstr(extraconstr)) {
                extraconstr <- list(A = matrix(z.row, 1, zn + zm), e = 0)
            } else {
                extraconstr$A <- rbind(extraconstr$A, z.row)
                extraconstr$e <- c(extraconstr$e, 0)
            }
        }
    }

    if (inla.one.of(model, c("slm"))) {
        stopifnot(!is.null(args.slm))
        stopifnot(!is.null(args.slm$rho.min) && length(args.slm$rho.min) == 1L)
        stopifnot(!is.null(args.slm$rho.max) && length(args.slm$rho.max) == 1L)
        stopifnot(args.slm$rho.max > args.slm$rho.min)
        stopifnot(!is.null(args.slm$X) && inla.is.matrix(args.slm$X))
        stopifnot(!is.null(args.slm$W) && inla.is.matrix(args.slm$W))
        stopifnot(!is.null(args.slm$Q.beta) && inla.is.matrix(args.slm$Q.beta))

        ## make sure they are sparse
        args.slm$X <- inla.as.sparse(args.slm$X)
        args.slm$W <- inla.as.sparse(args.slm$W)
        args.slm$Q.beta <- inla.as.sparse(args.slm$Q.beta)

        slm.n <- dim(args.slm$X)[1L]
        slm.m <- dim(args.slm$X)[2L]
        stopifnot(all(dim(args.slm$X) == c(slm.n, slm.m)))
        stopifnot(all(dim(args.slm$W) == c(slm.n, slm.n)))
        stopifnot(all(dim(args.slm$Q.beta) == c(slm.m, slm.m)))

        if (missing(n) || is.null(n)) {
            n <- slm.n + slm.m
        } else {
            stopifnot(n == slm.n + slm.m)
        }
    }

    if (inla.one.of(model, c("ar1c"))) {
        stopifnot(!is.null(args.ar1c))
        stopifnot(!is.null(args.ar1c$Z) && inla.is.matrix(args.ar1c$Z))
        stopifnot(!is.null(args.ar1c$Q.beta) && inla.is.matrix(args.ar1c$Q.beta))

        args.ar1c$Z <- as.matrix(args.ar1c$Z) ## is dense
        args.ar1c$Q.beta <- as.matrix(args.ar1c$Q.beta) ## is dense

        ar1c.n <- dim(args.ar1c$Z)[1L]
        ar1c.m <- dim(args.ar1c$Z)[2L]
        stopifnot(all(dim(args.ar1c$Z) == c(ar1c.n, ar1c.m)))
        stopifnot(all(dim(args.ar1c$Q.beta) == c(ar1c.m, ar1c.m)))

        if (missing(n) || is.null(n)) {
            n <- ar1c.n + ar1c.m
        } else {
            stopifnot(n == ar1c.n + ar1c.m)
        }
    }

    if (inla.one.of(model, c("dmatern"))) {
        stopifnot(!missing(locations) && !is.null(locations))
        if (is.vector(locations)) {
            locations <- matrix(locations, ncol = 1)
        }
        stopifnot(is.matrix(locations))
        stopifnot(nrow(locations) > 1)
        stopifnot(!any(is.na(locations)))
        n <- nrow(locations)
    } else {
        stopifnot(missing(locations) || is.null(locations))
    }

    if (inla.one.of(model, "intslope")) {
        stopifnot(!is.null(args.intslope$subject))
        stopifnot(!is.null(args.intslope$strata))
        stopifnot(!is.null(args.intslope$covariates))
        zz <- args.intslope$covariates
        zz[is.na(zz)] <- 0
        args.intslope <- list(
            subject = as.numeric(as.factor(args.intslope$subject)),
            strata = as.numeric(as.factor(args.intslope$strata)),
            covariates = zz
        )
        if (is.null(n)) {
            n <- length(args.intslope$subject)
        } else {
            stopifnot(n == length(args.intslope$subject))
        }
        stopifnot(length(args.intslope$subject) == length(args.intslope$strata))
        stopifnot(length(args.intslope$subject) == length(args.intslope$covariates))
    }

    ## is N required?
    if (is.null(n) && (!is.null(inla.model.properties(model, "latent")$n.required)
        && inla.model.properties(model, "latent")$n.required)) {
        stop(paste("Argument `n' in f() is required for model:", model))
    }

    ## special N required?
    if (inla.one.of(model, "iidkd")) {
        n.div.by <- order
        if (!inla.divisible(n, n.div.by)) {
            stop(paste("Argument `n'", n, "is not divisible by", n.div.by))
        }
    } else if ((!is.null(inla.model.properties(model, "latent")$n.div.by)
        && inla.model.properties(model, "latent")$n.div.by) && !is.null(n)) {
        if (!inla.divisible(n, inla.model.properties(model, "latent")$n.div.by)) {
            stop(paste("Argument `n'", n, "is not divisible by", inla.model.properties(model, "latent")$n.div.by))
        }
    }

    ## set default 'values'?  Do the check for values for nrow.ncol
    ## models further below.
    if (!is.null(n) && is.null(values) &&
        (!is.null(inla.model.properties(model, "latent")$set.default.values)
        && inla.model.properties(model, "latent")$set.default.values)) {
        values <- 1:n
    }

    if (inla.one.of(model, "linear")) {
        if (d != 1L) {
            stop("Model = 'linear' do not accept weights. Just set 'z.new = z * weights' as the covariates.")
        }
        if (!is.null(group)) {
            stop("Model = 'linear' do not accept argument 'group'. Please use another model component.")
        }
        if (!is.null(replicate)) {
            stop("Model = 'linear' do not accept argument 'replicate'. Please use another model component.")
        }
        ret <- list(
            d = d, term = term, model = model, mean.linear = mean.linear, prec.linear = prec.linear, label = term,
            cdf = cdf, quantiles = quantiles, compute = compute
        )
        ## return here!
        return(ret)
    }
    if (!missing(prec.linear) || !missing(mean.linear)) {
        stop("Arguments 'mean.linear' and 'prec.linear' are defined only for model='linear'.")
    }

    if (inla.one.of(model, c("spde"))) {
        if (is.null(spde.prefix)) {
            stop("Argument spde.prefix=NULL is required for model = spde")
        }
        ## file ``PREFIXs'' must exists... test this one
        if (!(file.exists(paste(spde.prefix, "s", sep = "")))) {
            stop(paste("Argument spde.prefix=", spde.prefix, "does not seems to be valid (no file `PREFIXs')"))
        }
    }

    if (inla.one.of(model, c("spde2"))) {
        ## If spde2.prefix is given, use it.
        if (is.null(spde2.prefix)) {
            stop("Argument spde2.prefix=NULL is required for model = spde2")
        }
    }
    spde2.transform <- match.arg(spde2.transform, several.ok = FALSE)

    if (inla.one.of(model, c("spde3"))) {
        if (is.null(spde3.prefix)) {
            stop("Argument spde3.prefix=NULL is required for model = spde3")
        }
    }
    spde3.transform <- match.arg(spde3.transform, several.ok = FALSE)

    if (inla.one.of(model, "seasonal") &&
        is.null(season.length)) {
        stop("The length of the season has to be provided in season.length")
    }

    ## cyclic is only valid for rw1, rw2 and rw2d-models
    if (!is.null(cyclic) && cyclic &&
        !inla.one.of(model, c("rw1", "rw2", "rw2d", "rw2diid", "ar1"))) {
        stop("Cyclic defined only for rw1, rw1c2, rw2, rw2d, rw2diid and ar1 models")
    }

    need.nrow.ncol <- inla.model.properties(model, "latent")$nrow.ncol
    ## nrow/ncol
    if ((!is.null(nrow) || !is.null(ncol)) && !need.nrow.ncol) {
        stop(paste("'nrow' and 'ncol' are not needed for model = ", model))
    }
    if (need.nrow.ncol) {
        if (is.null(nrow) || is.null(ncol)) {
            stop(paste("'nrow' and 'ncol' must be specified for model", model))
        }
        if (nrow <= 0 || ncol <= 0 || trunc(nrow) != nrow || trunc(ncol) != ncol) {
            stop("'nrow' and 'ncol' must be positive intergers.")
        }

        ## set n as well, makes it easier.
        if (!missing(n)) {
            stopifnot(n == nrow * ncol)
        } else {
            n <- nrow * ncol
        }

        ## and set default values, if required
        if (inla.model.properties(model, "latent")$set.default.values) {
            if (missing(values)) {
                values <- 1:n
            } else {
                stopifnot(length(values) == n)
            }
        }
    }

    ## check the NU parameter
    if (model != "matern2d" && model != "matern2dx2part0" && model != "matern2dx2p1") {
        if (!is.null(nu)) {
            stop("Argument NU is only used for matern2d/matern2dx2(part0/p1)-model.")
        }
    } else {
        if (!is.null(nu) && !is.element(nu, c(0, 1, 2, 3))) {
            stop("For matern2d/matern2dx2(part0/p1)-model, the NU-parameter must be 0, 1, 2 or 3.")
        }
    }

    ## for all instrinsic model the constraint has to be default ON
    if (is.null(constr)) {
        constr <- inla.model.properties(model, "latent")$constr
    }

    ## if diagonal is not set, the set this depending on the constr
    if (is.null(diagonal)) {
        if (constr || !empty.extraconstr(extraconstr)) {
            diagonal <- inla.set.f.default()$diagonal
        } else {
            diagonal <- 0.0
        }
    }

    if (!empty.extraconstr(extraconstr)) {
        ## check
        A <- extraconstr$A
        e <- extraconstr$e
        if (!is.matrix(A)) {
            stop("A(extraconstr) has to be a matrix")
        } else {
            if (nrow(A) != length(e)) {
                stop("Dimension of A and e do not correspond")
            }
        }
    }

    if (!missing(scale.model) && !inla.one.of(model, c(
        "rw1", "rw2", "besag", "bym",
        "bym2", "besag2", "rw2d",
        "rw2diid", "seasonal"
    ))) {
        stop(paste("Option 'scale.model' is not used for model:", model))
    }
    if (missing(scale.model) || is.null(scale.model)) {
        ## must doit like this otherwise we run into problems when
        ## compiling the package
        scale.model <- inla.getOption("scale.model.default")
        if (inla.one.of(model, c("bym2", "rw2diid"))) {
            scale.model <- TRUE
        }
    }
    if (inla.one.of(model, c("bym2", "rw2diid")) && !scale.model) {
        stop("Model 'bym2' and 'rw2diid' require scale.model=TRUE or 'missing(scale.model)'")
    }

    if (inla.one.of(model, c("besag", "besag2", "bym", "bym2"))) {
        ## this is a somewhat complicated case
        g <- inla.read.graph(graph)
        cc.n <- sapply(g$cc$nodes, length)
        cc.n1 <- sum(cc.n == 1L)
        cc.n2 <- sum(cc.n >= 2L)

        if (is.null(rankdef)) {
            if (scale.model) {
                rankdef <- cc.n2
            } else {
                rankdef <- cc.n2 + cc.n1
            }
            if (!(empty.extraconstr(extraconstr))) {
                rankdef <- rankdef + dim(extraconstr$A)[1]
            }
        }

        if (adjust.for.con.comp) {
            ## we need to check if the graph contains more than 1 connected components.
            if (g$cc$n == 1) {
                ## nothing to do
            } else {
                if (debug) {
                    print(paste("modify model", model))
                    print(paste("number of connected components", inla.paste(cc.n)))
                    print(paste("number of connected components of size >= 2L", cc.n2))
                }
                if (constr) {
                    ## need to redefine the meaning of constr = TRUE to mean
                    ## constr=TRUE for all connected components with size >= 2L

                    ## like bym/bym2 place the constr on the second half
                    m <- inla.model.properties(model, "latent")
                    if (m$augmented) {
                        N <- m$aug.factor * n
                        offset <- (m$aug.constr - 1L) * n
                        stopifnot(length(offset) == 1)
                    } else {
                        N <- n
                        offset <- 0
                    }
                    constr <- FALSE
                    AA <- matrix(0, cc.n2, N)
                    ee <- rep(0, cc.n2)

                    if (debug) {
                        print(paste("add new extraconstr, dim = ", cc.n2, "x", n))
                    }

                    k <- 1L
                    for (i in 1L:length(cc.n)) {
                        if (cc.n[i] >= 2L) {
                            AA[k, offset + g$cc$nodes[[i]]] <- 1
                            k <- k + 1L
                        }
                    }
                    stopifnot(k - 1L == cc.n2)

                    if (!empty.extraconstr(extraconstr)) {
                        extraconstr$A <- rbind(AA, extraconstr$A)
                        extraconstr$e <- c(ee, extraconstr$e)
                    } else {
                        extraconstr <- list(A = AA, e = ee)
                    }

                    if (debug) {
                        print(AA)
                        print(ee)
                    }
                } else {
                    if (!missing(adjust.for.con.comp)) {
                        stop("The option 'adjust.for.con.comp' is only used for models 'besag', 'besag2', 'bym' and 'bym2'.")
                    }
                }
                if (is.null(diagonal)) {
                    diagonal <- inla.set.f.default()$diagonal
                    if (debug) {
                        print(paste("set diagonal = ", diagonal))
                    }
                }
            }
        } else {
            ## nothing to do
        }
    }

    if (model %in% "rgeneric") {
        if (inla.is.element("f", rgeneric) && inla.is.element("rgeneric", rgeneric$f)) {
            rgeneric <- rgeneric$f$rgeneric
        }
        R.init <- rgeneric$R.init
        stopifnot(inherits(rgeneric, "inla.rgeneric"))
        ## add an 'Id' so we know who we are
        rgeneric <- list(model = rgeneric, Id = vars[[1]], R.init = R.init)
    }

    ret <- list(
        Cmatrix = Cmatrix,
        Z = Z,
        bvalue = bvalue,
        cdf = cdf,
        compute = compute,
        constr = constr,
        control.group = cont.group,
        cyclic = cyclic,
        d = d,
        diagonal = diagonal,
        extraconstr = inla.ifelse(empty.extraconstr(extraconstr), NULL, extraconstr),
        graph = graph,
        group = group,
        hyper = hyper,
        label = term,
        model = model,
        n = n,
        ncol = ncol,
        ngroup = ngroup,
        nrep = nrep,
        nrow = nrow,
        nu = nu,
        of = of,
        precision = precision,
        quantiles = quantiles,
        range = range,
        rankdef = rankdef,
        replicate = replicate,
        same.as = same.as,
        season.length = season.length,
        spde.prefix = spde.prefix,
        spde2.prefix = spde2.prefix,
        spde2.transform = spde2.transform,
        spde3.prefix = spde3.prefix,
        spde3.transform = spde3.transform,
        term = term,
        values = values,
        order = order,
        weights = weights,
        scale = scale,
        strata = strata,
        rgeneric = rgeneric,
        scale.model = as.logical(scale.model),
        adjust.for.con.comp = as.logical(adjust.for.con.comp),
        args.intslope = args.intslope,
        args.slm = args.slm,
        args.ar1c = args.ar1c,
        vb.correct = vb.correct,
        locations = locations
    )

    if (debug) print(ret)
    return(ret)
}

## "inla.model.class" is a generic model class that can be inherited
## from to mark a class as an inla model object class.
`inla.model.object.classes` <- function() {
    return(c(
        "inla.model.class", "inla.wrapper.model",
        "inla.spde", "inla.spde1", "inla.spde2", "inla.spde3",
        "inla.rgeneric"
    ))
}

## List spde classes that f() can write to disk when needed.
`inla.spde.object.classes` <- function() {
    return(c("inla.spde1", "inla.spde2", "inla.spde3"))
}

## Export: inla.make.lincomb inla.make.lincombs

## !\name{make.lincomb}
## !\alias{make.lincomb}
## !\alias{make.lincombs}
## !\alias{inla.make.lincomb}
## !\alias{inla.make.lincombs}
## !\title{Create linear combinations}
## !\description{Create a linear combination or several
## !linear combinations, as input to \code{inla(...,  lincomb = <lincomb>)}}
## !\usage{
## !inla.make.lincomb(...)
## !inla.make.lincombs(...)
## !}
## !\arguments{
## !   \item{...}{Arguments; see examples}
## !}
## !\value{
## ! A structure to be passed on to \code{\link{inla}} argument \code{lincomb}
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{TODO}
## !\examples{
## !##See the worked out examples and description in the FAQ
## !##section on {www.r-inla.org}
## !}

`inla.make.lincomb` <- function(...) {
    arg <- match.call(expand.dots = TRUE)

    ## we might need to expand arguments?
    if (all(is.null(names(arg)))) {
        arg <- eval(parse(text = arg[2]), envir = parent.frame(), enclos = environment(list(...)))
    }

    f.arg <- list()
    f.arg[[length(arg)]] <- list
    for (k in 1:length(arg)) {
        var <- names(arg)[k]
        if (var != "") {
            value <- eval(arg[[k]], envir = parent.frame(), enclos = environment(list(...)))

            if (length(value) == 1) {
                ff.arg <- list(list(weight = value))
            } else {
                ii <- which(!is.na(value) & (value != 0))
                ff.arg <- list(list(idx = ii, weight = value[ii]))
            }

            names(ff.arg) <- var
            f.arg[[k]] <- ff.arg
        }
    }
    if (is.null(f.arg[[1]][[1]])) {
          f.arg[[1]] <- NULL
      }

    lc <- list(f.arg)
    names(lc) <- "lc"

    return(lc)
}

`inla.make.lincombs` <- function(...) {
    ## this is the more general version, which constructs one lincomb
    ## for each row

    arg <- match.call(expand.dots = TRUE)

    ## we might need to expand arguments?
    if (all(is.null(names(arg)))) {
        arg <- eval(parse(text = arg[2]),
            envir = parent.frame(),
            enclos = environment(list(...))
        )
    }

    ## check that all 'rows' has the same lengths. store the evalued
    ## args in values[[.]]
    values <- list()
    is.m <- numeric(length(arg))
    n <- -1
    for (k in 1:length(arg)) {
        if (names(arg)[k] != "") {
            values[[k]] <- eval(arg[[k]], envir = parent.frame(), enclos = environment(list(...)))
            if (is.matrix(values[[k]]) || is(values[[k]], "Matrix")) {
                if (is(values[[k]], "Matrix")) {
                    ## Make sure we have a dgTMatrix with unique representation:
                    values[[k]] <- inla.as.dgTMatrix(values[[k]])
                }
                stopifnot(nrow(values[[k]]) == n || n < 0)
                n <- nrow(values[[k]])
                is.m[k] <- TRUE
            } else {
                stopifnot(length(values[[k]]) == n || n < 0)
                values[[k]][is.na(values[[k]])] <- 0
                n <- length(values[[k]])
                is.m[k] <- FALSE
            }
        }
    }
    ## make the nameing so it is the correct order, like lc001, lc002,
    ## ... etc.
    name <- paste("lc", inla.num(1:n), sep = "")

    ## create one lincomb for each row. faster if dynamic arrays are
    ## allocated upfront.
    lc <- list()
    lc[[n]] <- list

    if (TRUE) {

        ## this is the fast version, which is kind of ugly, therefore
        ## the ``slow'' code is below. we simply pass all additional
        ## arguments in the reminder of lapply. There is obviously
        ## something in the memory management of R that I don't
        ## understand....
        lc <- lapply(1:n,
            function(idx, ...) {
                ## preallocate
                f.arg <- list()
                f.arg[[length(arg)]] <- list
                for (k in 1:length(arg)) {
                    if (names(arg)[k] != "") {
                        var <- names(arg)[k]
                        if (is.m[k]) {
                            if (is(values[[k]], "dgTMatrix")) {
                                row <- inla.sparse.get(values[[k]], row = idx)
                                ff.arg <- list(list(idx = row$j, weight = row$values))
                            } else {
                                value <- values[[k]][idx, ]
                                ii <- which(!is.na(value) & (value != 0))
                                ff.arg <- list(list(idx = ii, weight = value[ii]))
                            }
                        } else {
                            ff.arg <- list(list(weight = values[[k]][idx]))
                        }
                        names(ff.arg) <- var
                        f.arg[[k]] <- ff.arg
                    }
                }
                ## the first might or might not be relevant
                if (is.null(f.arg[[1]][[1]])) {
                      f.arg[[1]] <- NULL
                  }

                return(f.arg)
            },
            arg = arg, is.m = is.m, values = values
        )
        names(lc) <- name
    } else {

        ## this is the slow version for which the lapply-version is
        ## buildt upon.

        for (idx in 1:n) {
            f.arg <- list()
            for (k in 1:length(arg)) {
                if (names(arg)[k] != "") {
                    var <- names(arg)[k]
                    if (is.m[k]) {
                        value <- values[[k]][idx, ]
                        ii <- which(!is.na(value) & (value != 0))
                        ff.arg <- list(list(idx = ii, weight = value[ii]))
                    } else {
                        ff.arg <- list(list(weight = values[[k]][idx]))
                    }
                    names(ff.arg) <- var
                    f.arg <- c(f.arg, ff.arg)
                }
                if (is.null(f.arg[[1]][[1]])) {
                      f.arg[[1]] <- NULL
                  }
            }
            lc[[idx]] <- f.arg
            names(lc)[idx] <- name[idx]
        }
    }

    return(lc)
}

`inla.uncbind` <- function(A, name.prefix = "col") {
    ## given a matrix with colnames, expand this into
    ## list(a=c1, b=c2, ....)

    if (!is.matrix(A)) {
          return(NULL)
      }

    result <- list()

    ## it's easier to check names of the matrix upfront...
    rownames(A) <- NULL
    if (is.null(colnames(A))) {
        colnames(A) <-
            paste(name.prefix,
                inla.num(1:ncol(A), width = inla.numlen(ncol(A))),
                sep = ""
            )
    } else {
        ii <- which(is.na(colnames(A)))
        if (length(ii) > 0) {
            colnames(A)[ii] <- paste(name.prefix,
                inla.num(ii, width = inla.numlen(ncol(A))),
                sep = ""
            )
        }
    }

    if (TRUE) {
        ## this is the fast version
        tmp.result <- apply(A, 2, function(x) list(x))
        result <- sapply(tmp.result, function(x) c(x))
    } else {
        ## and this is the slow one
        for (icol in 1:ncol(A)) {
            ## extract each column and give it its name or 'column001' if
            ## its NULL.
            x <- list(A[, icol])
            name <- colnames(A)[icol]
            names(x) <- name
            result <- c(result, x)
        }
    }

    return(result)
}

`inla.unrbind` <- function(A, name.prefix = "row") {
    return(inla.uncbind(t(A), name.prefix = name.prefix))
}

## Export: inla.qsample

## ! \name{qsample}
## ! \alias{inla.qsample}
## ! \alias{qsample}
## !
## ! \title{Generate samples from a GMRF using the GMRFLib implementation}
## !
## ! \description{This function generate samples from a GMRF using the GMRFLib implementation}
## ! \usage{
## ! inla.qsample(
## !        n = 1L,
## !        Q,
## !        b,
## !        mu,
## !        sample,
## !        constr,
## !        reordering = INLA::inla.reorderings(),
## !        seed = 0L,
## !        logdens = ifelse(missing(sample), FALSE, TRUE),
## !        compute.mean = ifelse(missing(sample), FALSE, TRUE),
## !        num.threads = if (seed == 0L) "0:0" else NULL,
## !        selection = NULL, verbose = FALSE)
## ! }
## !
## ! \arguments{
## !   \item{n}{Number of samples. Only used if \code{missing(sample)}}
## !   \item{Q}{The precision matrix or a filename containing it.}
## !   \item{b}{The linear term}
## !   \item{mu}{The mu term}
## !   \item{sample}{A matrix of optional samples where each column is a sample. If set, then evaluate the log-density for each sample only.}
## !   \item{constr}{Optional linear constraints; see \code{?INLA::f} and argument \code{extraconstr}}
## !   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
## !        either one of the names listed in \code{inla.reorderings()}
## !        or the output from \code{inla.qreordering(Q)}.
## !        The default is "auto" which try several reordering algorithm and use the best one for this particular matrix.}
## !   \item{seed}{Control the RNG. If \code{seed=0L} then GMRFLib will set the seed
## !               intelligently/at 'random',  and this is and should be the default behaviour.
## !               If \code{seed < 0L}  then the saved state of the RNG will be reused if possible, otherwise,
## !               GMRFLib will set the seed intelligently/at 'random'.
## !               If \code{seed > 0L} then this value is used as the seed for the RNG.
## !
## !               PLEASE NOTE1: If \code{seed!=0} then the computations will run in serial mode,  over-riding
## !               whatever is set in \code{num.threads} (a warning might be issued).
## !
## !               PLEASE NOTE2: If the PARDISO sparse matrix library is used, continuity of the samples
## !               with respect to small changes in the precision matrix, can be expected but is
## !               not guaranteed. If this feature is required, please use the TAUCS sparse matrix library.}
## !   \item{logdens}{If \code{TRUE}, compute also the log-density of each sample. Note that the output format then change.}
## !   \item{compute.mean}{If \code{TRUE}, compute also the (constrained) mean. Note that the output format then change.}
## !   \item{num.threads}{Maximum number of threads the
## !                      \code{inla}-program will use, or as 'A:B' defining the number threads in the
## !                      outer (A) and inner (B) layer for nested parallelism.
## !                      \code{seed!=0} requires serial comptuations.}
## !   \item{selection}{A vector of indices of each sample to
## !                    return. \code{NULL} means return the whole sample.
## !                    (Note that the log-density retured,  is for the whole sample.)
## !                    The use of \code{selection} cannot be combined with the use of \code{sample}.}
## !   \item{verbose}{Logical. Run in verbose mode or not.}
## ! }
## !\value{
## !      The log-density has form {-1/2(x-mu)^T Q (x-mu) + b^T x}
## !
## !  If \code{logdens} is \code{FALSE},  then \code{inla.qsample} returns
## !  the samples in a matrix,  where each column is a sample.
## !  If \code{logdens} or \code{compute.mean} is \code{TRUE}, then a list
## !   with names \code{sample},
## !  \code{logdens} and \code{mean} is returned. The samples are stored in the matrix
## !  \code{sample} where each column is a sample, and the log
## !  densities of each sample are stored as the vector \code{logdens}.
## !  The mean (include corrections for the constraints,  if any) is store in
## !  the vector \code{mean}.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## ! g = system.file("demodata/germany.graph", package="INLA")
## ! Q = inla.graph2matrix(g)
## ! diag(Q) = dim(Q)[1L]
## ! x = inla.qsample(10, Q)
## ! \dontrun{matplot(x)}
## ! x = inla.qsample(10, Q, logdens=TRUE)
## ! \dontrun{matplot(x$sample)}
## !
## ! n = 3
## ! Q = diag(n)
## ! ns = 2
## !
## ! ## sample and evaluate a sample
## ! x = inla.qsample(n, Q=Q, logdens=TRUE)
## ! xx = inla.qsample(Q=Q,  sample = x$sample)
## ! print(x$logdens - xx$logdens)
## !
## ! ## the use of a constraint
## ! constr = list(A = matrix(rep(1, n), 1, n), e = 0)
## ! x = inla.qsample(n, Q=Q, constr=constr)
## ! print(constr$A \%*\% x)
## !
## ! ## control the RNG (require serial mode)
## ! x = inla.qsample(n, Q=Q, seed = 123, num.threads="1:1")
## ! ## restart from same seed,  only sample 1
## ! xx = inla.qsample(n=1, Q=Q, seed = 123, num.threads="1:1")
## ! ## continue from the save state, sample the remaining 2
## ! xxx = inla.qsample(n=n-1, Q=Q, seed = -1, num.threads="1:1")
## ! ## should be 0
## ! print(x - cbind(xx, xxx))
## !}

`inla.qsample` <- function(
                           n = 1L,
                           Q,
                           b,
                           mu,
                           sample,
                           constr,
                           reordering = INLA::inla.reorderings(),
                           seed = 0L,
                           logdens = ifelse(missing(sample), FALSE, TRUE),
                           compute.mean = ifelse(missing(sample), FALSE, TRUE),
                           num.threads = if (seed == 0L) "0:0" else NULL,
                           selection = NULL,
                           verbose = FALSE) {
    t.dir <- inla.tempdir()
    smtp <- match.arg(inla.getOption("smtp"), c("taucs", "band", "default", "pardiso"))
    stopifnot(!missing(Q))
    stopifnot(n >= 1L)

    if (is.null(num.threads)) {
        num.threads <- inla.getOption("num.threads")
    }
    if (seed != 0L) {
        num.threads.user <- inla.parse.num.threads(num.threads)
        num.threads <- inla.parse.num.threads("1:1")
        if (num.threads != num.threads.user) {
            warning("Since 'seed!=0', parallel model is disabled and serial model is selected")
        }
    } else {
        num.threads <- inla.parse.num.threads(num.threads)
    }

    if (is.list(reordering)) {
        ## argument is the output from inla.qreordering()
        reordering <- reordering$name
    }
    reordering <- match.arg(reordering)

    Q <- inla.sparse.check(Q)
    if (is(Q, "dgTMatrix")) {
        Q.file <- inla.write.fmesher.file(Q, filename = inla.tempfile(tmpdir = t.dir))
    } else if (is.character(Q)) {
        Q.file <- Q
    } else {
        stop("This should not happen.")
    }

    b.file <- inla.tempfile(tmpdir = t.dir)
    mu.file <- inla.tempfile(tmpdir = t.dir)
    constr.file <- inla.tempfile(tmpdir = t.dir)
    x.file <- inla.tempfile(tmpdir = t.dir)
    sample.file <- inla.tempfile(tmpdir = t.dir)
    rng.file <- inla.tempfile(tmpdir = t.dir)
    cmean.file <- inla.tempfile(tmpdir = t.dir)
    selection.file <- inla.tempfile(tmpdir = t.dir)

    if (!missing(b)) {
        stopifnot(length(b) == nrow(Q))
        b <- matrix(b, nrow(Q), 1)
        inla.write.fmesher.file(b, filename = b.file)
    }

    if (!missing(mu)) {
        stopifnot(length(mu) == nrow(Q))
        mu <- matrix(mu, nrow(Q), 1)
        inla.write.fmesher.file(mu, filename = mu.file)
    }

    if (!missing(constr) && !is.null(constr)) {
        stopifnot(is.list(constr))
        A <- as.matrix(constr$A)
        e <- as.numeric(constr$e)
        stopifnot(ncol(A) == ncol(Q))
        stopifnot(nrow(A) == length(e))
        xx <- matrix(c(nrow(A), c(A), c(e)), ncol = 1)
        inla.write.fmesher.file(xx, filename = constr.file)
    }

    if (!missing(sample) && !is.null(sample)) {
        sample <- as.matrix(sample)
        stopifnot(nrow(sample) == nrow(Q))
        stopifnot(ncol(sample) > 0L)
        inla.write.fmesher.file(sample, filename = sample.file)
        n <- ncol(sample) ## redefine n here
    }

    if (!missing(selection) && !is.null(selection)) {
        if (!missing(sample)) {
            stop("Cannot use 'selection' and 'sample' at the same time")
        }
        selection <- as.matrix(selection, ncol = 1)
        stopifnot(nrow(selection) <= nrow(Q))
        inla.write.fmesher.file(selection - 1, filename = selection.file)
    } else {
        ## make the code easier below
        selection <- 1:nrow(Q)
    }

    envir <- inla.get.inlaEnv()
    if (seed < 0L) {
        if (!exists("GMRFLib.rng.state", envir = envir)) {
            seed <- 0L
        } else {
            rng.state <- get("GMRFLib.rng.state", envir = envir)
            fp <- file(rng.file, "wb")
            writeBin(as.raw(rng.state), fp)
            close(fp)
        }
    }

    inla.set.sparselib.env(inla.dir = t.dir)
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qsample",
            paste0("-t", num.threads), "-r", reordering, "-z", seed, "-S", smtp,
            if (verbose) "-v" else "",
            Q.file, x.file, as.integer(n), rng.file,
            sample.file, b.file, mu.file, constr.file, cmean.file, selection.file
        ), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qsample",
            paste0("-t", num.threads), "-r", reordering, "-z", seed, "-S", smtp,
            if (verbose) "-v" else "",
            Q.file, x.file, as.integer(n), rng.file,
            sample.file, b.file, mu.file, constr.file, cmean.file, selection.file
        ), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    fp <- file(rng.file, "rb")
    siz <- file.info(rng.file)$size
    rng.state <- readBin(fp, raw(), siz)
    close(fp)
    assign("GMRFLib.rng.state", rng.state, envir = envir)

    x <- inla.read.fmesher.file(x.file)
    cmean <- inla.read.fmesher.file(cmean.file)

    nx <- dim(x)[1L] - 1L
    samples <- matrix(x[-(nx + 1L), , drop = FALSE], nx, n)
    colnames(samples) <- paste0("sample:", 1L:n)
    rownames(samples) <- paste0("x:", selection)
    ld <- c(x[nx + 1L, ])
    names(ld) <- paste("logdens", 1L:n, sep = "")

    unlink(t.dir, recursive = TRUE)

    if (logdens || compute.mean) {
        return(list(sample = samples, logdens = ld, mean = c(cmean)))
    } else {
        return(samples)
    }
}

## Nothing to Export

`inla.fmesher.call` <- function(
                                fmesher.call = inla.getOption("fmesher.call"),
                                all.args, prefix,
                                timeout = inla.getOption("fmesher.timeout")) {
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        echoc <- system(paste(shQuote(fmesher.call), all.args, shQuote(prefix)),
            timeout = ceiling(timeout)
        )
    }
    else if (inla.os("windows")) {
        if (TRUE) {
            echoc <- try(system(paste(
                shQuote(fmesher.call),
                all.args,
                shQuote(prefix)
            ),
            timeout = ceiling(timeout)
            ),
            silent = TRUE
            )
            echoc <- 0
        } else {
            ## we might need it if we want one day to make the fmesher program run remotely
            stop("This code is not in use.")
            ##
            echoc <- try(inla.cygwin.run.command(
                paste(
                    inla.cygwin.map.filename(fmesher.call),
                    all.args,
                    inla.cygwin.map.filename(prefix)
                )
            ), silent = TRUE)
            echoc <- 0
        }
    }
    else {
          stop("\n\tNot supported architecture.")
      }
}

`inla.fmesher.make.prefix` <- function(dir = tempdir(), prefix = NULL) {
    if (is.null(prefix)) {
        if (is.null(dir)) {
              prefix <- paste(tempfile(), ".", sep = "")
          } else {
            inla.dir.create(dir)
            prefix <- paste(tempfile(tmpdir = dir), ".", sep = "")
        }
    } else {
        inla.dir.create(inla.dirname(prefix))
    }
    return(prefix)
}

## Nothing to export

## These functions are all just internal functions for building the
## documentation,  and generates Rd and tex for the models

`inla.models.generate.Rd` <- function(file = NULL) {
    ## this function is used to generate the man-page for the models

    if (!is.null(file)) {
          sink(file)
      }

    tab <- "\t"
    tab2 <- paste(tab, "\t", sep = "")
    tab3 <- paste(tab2, "\t", sep = "")
    tab4 <- paste(tab3, "\t", sep = "")
    tab5 <- paste(tab4, "\t", sep = "")
    tab6 <- paste(tab5, "\t", sep = "")
    tab7 <- paste(tab6, "\t", sep = "")
    tab8 <- paste(tab7, "\t", sep = "")


    cat("
%% DO NOT EDIT!
%% This file is generated automatically from models.R
\\name{inla.models}
\\alias{inla.models}
\\title{Valid models in INLA}
\\description{
This page describe the models implemented in \\code{inla}, divided into sections: ")
    cat(inla.paste(names(inla.models()), sep = ", "), ".\n}\n")

    cat("
\\usage{
inla.models()
}\n")

    cat("\\value{
", tab, "Valid sections are: ")
    cat(inla.paste(names(inla.models()), sep = ", "), "\n")
    cat("
", tab, "\\describe{
")

    for (section in names(inla.models())) {
        cat(tab2, "\\item{Section `", section, "'.}{\n", sep = "")
        cat(tab3, "Valid models in this section are:\n")
        cat(tab3, "\\describe{\n")

        for (model in names(inla.models()[[section]])) {
            cat(tab4, "\\item{Model `", model, "'.}{", sep = "")

            if (section != "prior") {
                nhyper <- length(inla.models()[[section]][[model]]$hyper)
                cat("Number of hyperparmeters are ", nhyper, ".\n", sep = "")
                if (nhyper > 0) {
                    cat(tab5, "\\describe{\n")

                    h <- inla.models()[[section]][[model]]$hyper
                    nhyper <- length(h)
                    if (nhyper > 0) {
                        for (nh in 1:nhyper) {
                            nm <- names(h)[nh]
                            cat(tab6, "\\item{Hyperparameter `", nm, "'}{\n", sep = "")
                            cat(tab7, "\\describe{\n")

                            for (m in names(h[[nm]])) {
                                mval <- h[[nm]][[m]]
                                if (is.null(mval)) {
                                      mval <- "NULL"
                                  }
                                if (is.function(mval)) {
                                    mval.src <- attr(mval, "srcref")
                                    if (is.null(mval.src)) {
                                        mval.src <- inla.paste(deparse(mval, control = "keepInteger"))
                                    }
                                    mval <- paste("\\code{", mval.src, "}", sep = "")
                                }
                                cat(tab7, "\\item{", m, " = }{`", inla.paste(mval), "'}\n", sep = "")
                            }

                            cat(tab7, "}\n", sep = "")
                            cat(tab6, "}\n", sep = "")
                        }
                        cat(tab5, "}\n")
                    }

                    h <- inla.models()[[section]][[model]]
                    ms <- names(h)
                    ms <- ms[ms != "hyper"]

                    cat(tab5, "\\describe{\n")
                    cat(tab6, "\\item{Properties:}{\n", sep = "")
                    cat(tab7, "\\describe{\n", sep = "")

                    for (m in ms) {
                        mval <- h[[m]]
                        if (is.null(mval)) {
                              mval <- "NULL"
                          }
                        cat(tab8, "\\item{", m, " = }{`", inla.paste(mval), "'}\n", sep = "")
                    }

                    cat(tab7, "}\n")
                    cat(tab6, "}\n")
                    cat(tab5, "}\n")
                }
            } else if (section == "prior") {
                np <- inla.models()[[section]][[model]]$nparameters
                cat(tab5, "Number of parameters in the prior =", np, "\n")
            } else {
                stop("Should not happen.")
            }

            cat(tab4, "}\n")
        }
        cat(tab3, "}\n")
        cat(tab2, "}\n")
    }

    cat(tab, "} \n")
    cat("} \n")

    cat("\\examples{
## How to set hyperparameters to pass as the argument 'hyper'. This
## format is compatible with the old style (using 'initial', 'fixed',
## 'prior', 'param'), but the new style using 'hyper' take preceedence
## over the old style. The two styles can also be mixed. The old style
## might be removed from the code in the future...

## Only a subset need to be given
   hyper = list(theta = list(initial = 2))
## The `name' can be used instead of 'theta', or 'theta1', 'theta2',...
   hyper = list(precision = list(initial = 2))
   hyper = list(precision = list(prior = \"flat\", param = numeric(0)))
   hyper = list(theta2 = list(initial=3), theta1 = list(prior = \"gaussian\"))
## The 'short.name' can be used instead of 'name'
   hyper = list(rho = list(param = c(0,1)))
}\n")
    if (!is.null(file)) {
          sink()
      }
}

`inla.models.generate.tex` <- function(a.list = inla.models(), file = NULL) {
    my.doit.recursively <- function(a.list, level = 0L) {
        if (level == 0L) {
            cat("%% DO NOT EDIT!\n")
            cat("%% This file is generated automatically from models.R\n")
            cat("\\begin{description}\n")
        }

        tab <- inla.paste(rep("\t", level + 1L))

        for (nm in names(a.list)) {
            if (is.list(a.list[[nm]])) {
                if (length(a.list[[nm]]) > 0) {
                    cat(tab, "\\item[", nm, "]\\ ", "\n", sep = "")
                    cat(tab, "\\begin{description}\n")
                    my.doit.recursively(a.list[[nm]], level + 1L)
                    cat(tab, "\\end{description}\n")
                } else {
                    cat(tab, "\\item[", nm, "]\\ ", "\n", sep = "")
                }
            } else {
                val <- a.list[[nm]]
                if (is.function(val)) {
                    val.src <- attr(val, "srcref")
                    if (is.null(val.src)) {
                        val.src <- inla.paste(deparse(val, control = "keepInteger"))
                    }
                    val <- paste("\\verb!", val.src, "!", sep = "")
                }
                cat(tab, "\\item[", nm, "]", " ", inla.paste(val), "\n", sep = "")
            }
        }

        if (level == 0L) {
              cat("\\end{description}\n")
          }
    }

    ##
    ##

    if (!is.null(file)) {
          sink(file)
      }

    my.doit.recursively(a.list)

    if (!is.null(file)) {
          sink()
      }
}






`inla.models.generate.roxygen` <- function(file = NULL) {
    ## this function is used to generate the man-page for the models

    if (!is.null(file)) {
        sink(file)
        on.exit(sink())
    }
    on.exit(cat("NULL\n"), add = TRUE, after = FALSE)


    tab <- "#' "
    tab2 <- paste(tab, "  ", sep = "")
    tab3 <- paste(tab2, "  ", sep = "")
    tab4 <- paste(tab3, "  ", sep = "")
    tab5 <- paste(tab4, "  ", sep = "")
    tab6 <- paste(tab5, "  ", sep = "")
    tab7 <- paste(tab6, "  ", sep = "")
    tab8 <- paste(tab7, "  ", sep = "")


    cat("## DO NOT EDIT!\n",
        "## This file is generated automatically from models.R\n",
        "#' @title Valid models in INLA\n#'\n",
        "#' @name inla.models_doc\n",
        "#' @rdname inla.models_doc\n",
        "#' @aliases inla.models_doc\n",
        "#'\n",
        "#' @description\n",
        "#' This page describe the models implemented in \`inla`, divided into sections:\n",
        "#' ", paste0(names(inla.models()), collapse = ", "), ".\n#'\n",
        sep = ""
    )

    cat("#' @usage\n",
        "#' inla.models()\n",
        "#'\n",
        sep = ""
    )

    cat("#' @return\n",
        "#' Valid sections are:\n",
        "#' ", paste0(names(inla.models()), collapse = ", "), ".\n",
        sep = ""
    )

    for (section in names(inla.models())) {
        cat("#' ", "@section '", section, "':\n#'\n", sep = "")
        cat(tab2, "Valid models in this section are:\n")
        cat(tab3, "\\describe{\n")

        for (model in names(inla.models()[[section]])) {
            cat(tab4, "\\item{Model `", model, "'.}{", sep = "")

            if (section != "prior") {
                nhyper <- length(inla.models()[[section]][[model]]$hyper)
                cat("Number of hyperparmeters are ", nhyper, ".\n", sep = "")
                if (nhyper > 0) {
                    cat(tab5, "\\describe{\n")

                    h <- inla.models()[[section]][[model]]$hyper
                    nhyper <- length(h)
                    if (nhyper > 0) {
                        for (nh in 1:nhyper) {
                            nm <- names(h)[nh]
                            cat(tab6, "\\item{Hyperparameter `", nm, "'}{\n", sep = "")
                            cat(tab7, "\\describe{\n")

                            for (m in names(h[[nm]])) {
                                mval <- h[[nm]][[m]]
                                if (is.null(mval)) {
                                      mval <- "NULL"
                                  }
                                if (is.function(mval)) {
                                    mval.src <- attr(mval, "srcref")
                                    if (is.null(mval.src)) {
                                        mval.src <- inla.paste(deparse(mval, control = "keepInteger"))
                                    }
                                    mval <- paste("\\code{", mval.src, "}", sep = "")
                                }
                                cat(tab7, "\\item{", m, " = }{`", inla.paste(mval), "'}\n", sep = "")
                            }

                            cat(tab7, "}\n", sep = "")
                            cat(tab6, "}\n", sep = "")
                        }
                        cat(tab5, "}\n")
                    }

                    h <- inla.models()[[section]][[model]]
                    ms <- names(h)
                    ms <- ms[ms != "hyper"]

                    cat(tab5, "\\describe{\n")
                    cat(tab6, "\\item{Properties:}{\n", sep = "")
                    cat(tab7, "\\describe{\n", sep = "")

                    for (m in ms) {
                        mval <- h[[m]]
                        if (is.null(mval)) {
                              mval <- "NULL"
                          }
                        cat(tab8, "\\item{", m, " = }{`", inla.paste(mval), "'}\n", sep = "")
                    }

                    cat(tab7, "}\n")
                    cat(tab6, "}\n")
                    cat(tab5, "}\n")
                }
            } else if (section == "prior") {
                np <- inla.models()[[section]][[model]]$nparameters
                cat(tab5, "Number of parameters in the prior =", np, "\n")
            } else {
                stop("Should not happen.")
            }

            cat(tab4, "}\n")
        }
        cat(tab3, "}\n")
        # cat(tab2, "}\n")
    }

    # cat(tab, "} \n")
    # cat("#' } \n")

    cat("#' @examples\n",
        "#' ## How to set hyperparameters to pass as the argument 'hyper'. This\n",
        "#' ## format is compatible with the old style (using 'initial', 'fixed',\n",
        "#' ## 'prior', 'param'), but the new style using 'hyper' take preceedence\n",
        "#' ## over the old style. The two styles can also be mixed. The old style\n",
        "#' ## might be removed from the code in the future...\n",
        "#' \n",
        "#' ## Only a subset need to be given\n",
        "#' hyper = list(theta = list(initial = 2))\n",
        "#' ## The `name' can be used instead of 'theta', or 'theta1', 'theta2',...\n",
        "#' hyper = list(precision = list(initial = 2))\n",
        "#' hyper = list(precision = list(prior = \"flat\", param = numeric(0)))\n",
        "#' hyper = list(theta2 = list(initial=3), theta1 = list(prior = \"gaussian\"))\n",
        "#' ## The 'short.name' can be used instead of 'name'\n",
        "#' hyper = list(rho = list(param = c(0,1)))\n",
        sep = ""
    )
}

## Export: inla.hyperpar.sample

## !\name{inla.hyperpar.sample}
## !\alias{inla.hyperpar.sample}
## !\alias{inla.hyperpar.sampler}
## !\alias{hyperpar.sample}
## !\alias{hyperpar.sampler}
## !
## !\title{Produce samples from the approximated joint posterior for the hyperparameters}
## !
## !\description{Produce samples from the approximated joint posterior for the hyperparameters}
## !
## !\usage{
## !inla.hyperpar.sample(n, result, intern=FALSE, improve.marginals = FALSE)
## !}
## !
## !\arguments{
## !  \item{n}{Integer. Number of samples required.}
## !  \item{result}{An \code{inla}-object,  f.ex the output from an \code{inla}-call.}
## !  \item{intern}{Logical. If \code{TRUE} then produce samples in the
## !        internal scale for the hyperparmater, if \code{FALSE} then produce
## !        samples in the user-scale. (For example log-precision (intern)
## !        and precision (user-scale))}
## !  \item{improve.marginals}{Logical. If \code{TRUE}, then improve the samples taking into account
## !        possible better marginal estimates for the hyperparameters in \code{result}.}
## !}
## !
## !\value{%%
## ! A matrix where each sample is a row. The contents of the column
## ! is described in the rownames.
## !}
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## !n = 100
## !r = inla(y ~ 1 + f(idx), data = data.frame(y=rnorm(n), idx = 1:n))
## !ns = 500
## !x = inla.hyperpar.sample(ns, r)
## !
## !rr = inla.hyperpar(r)
## !xx = inla.hyperpar.sample(ns, rr, improve.marginals=TRUE)
## !}


`inla.hyperpar.sample` <- function(n, result, intern = FALSE, improve.marginals = FALSE) {
    ## generate 'n' samples from the joint for the hyperparameters
    ## computed using the CCD approach. if intern==TRUE, then generate
    ## the variables in their intern scale otherwise in the user
    ## scale.

    stopifnot(!is.null(result))
    stopifnot(any(class(result) == "inla"))
    stopifnot(n > 0)

    if (improve.marginals) {
        ns <- max(n, 300L)
    } else {
        ns <- n
    }

    sigma <- result$misc$cov.intern
    if (dim(sigma)[1L] == 0L) {
        return(NULL)
    }

    p <- dim(sigma)[1L]
    sd.plus <- result$misc$stdev.corr.positive
    sd.neg <- result$misc$stdev.corr.negative

    if (is.null(sd.plus) || is.null(sd.neg)) {
        sd.plus <- rep(1.0, p)
        sd.neg <- rep(1.0, p)
    }

    ## do each column  dim(z) = ns x p,  each row is one sample
    z <- matrix(NA, ns, p)
    for (i in 1L:p) {
        prob <- c(sd.plus[i], sd.neg[i])
        direction <- sample.int(2L, ns, prob = prob, replace = TRUE)
        s <- c(sd.plus[i], -sd.neg[i])
        z[, i] <- s[direction] * abs(rnorm(ns))
    }
    A <- result$misc$cov.intern.eigenvectors %*%
        diag(sqrt(result$misc$cov.intern.eigenvalues), nrow = p, ncol = p)
    theta <- apply(z, 1L,
        function(x, A, m) A %*% x + m,
        A = A, m = result$misc$theta.mode
    )
    if (p > 1L) {
        theta <- t(theta)
    } else {
        theta <- matrix(theta, ns, p)
    }

    if (improve.marginals) {
        for (i in 1L:p) {
            cdf.old <- ecdf(theta[, i])
            theta[, i] <- inla.qmarginal(
                (ns / (ns + 1.0)) * cdf.old(theta[, i]),
                result$internal.marginals.hyperpar[[i]]
            )
        }
    }

    theta <- theta[1:n, , drop = FALSE]
    if (!intern) {
        for (i in 1L:p) {
            theta[, i] <- result$misc$from.theta[[i]](theta[, i])
        }
    }

    if (intern) {
        colnames(theta) <- names(result$misc$to.theta)
    } else {
        colnames(theta) <- inla.transform.names(result, names(result$misc$to.theta))
    }
    rownames(theta) <- paste0("sample:", 1L:n)

    return(theta)
}

inla.transform.names <- function(res, nms) {
    ## enter name of a hyperpar, return the corresponding name in either in the internal or user
    ## scale
    ret <- c()
    for (nm in nms) {
        def <- paste(nm, "-- unknown")
        if (is.null(res$internal.summary.hyperpar) || is.null(res$summary.hyperpar)) {
            ret <- c(ret, def)
        } else {
            i <- which(nm == rownames(res$internal.summary.hyperpar))
            if (length(i) == 1) {
                ret <- c(ret, rownames(res$summary.hyperpar)[i])
            } else {
                i <- which(nm == rownames(res$summary.hyperpar))
                if (length(i) == 1) {
                    ret <- c(ret, rownames(res$internal.summary.hyperpar)[i])
                } else {
                    ret <- c(ret, def)
                }
            }
        }
    }
    return(ret)
}

# For development, run with devtools::test() from the rinla/ directory
# Requires the binaries to be locally available via the relevant
#   rinla/inst/bin/... directory

library(testthat)
library(INLA)

test_check("INLA")

## Export: inla.knmodels.sample

## ! \name{inla.knmodels.sample}
## ! \alias{inla.knmodels.sample}
## ! \title{Spacetime interaction models sampler function}
## ! \description{
## !    It implements the sampling method for the models
## !    in Knorr-Held, L. (2000) considering the algorithm
## !    3.1 in Rue & Held (2005) book.
## ! }
## ! \usage{
## !inla.knmodels.sample(
## !  graph,
## !  m,
## !  type=4,
## !  intercept=0,
## !  tau.t=1,
## !  phi.t=0.7,
## !  tau.s=1,
## !  phi.s=0.7,
## !  tau.st=1,
## !  ev.t=NULL,
## !  ev.s=NULL)
## !}
## ! \arguments{
## ! \item{graph}{}
## ! \item{m}{Time dimention.}
## ! \item{type}{Integer from 1 to 4 to identify one
## !    of the four interaction type.}
## ! \item{intercept}{A constant to be added to the linear predictor}
## ! \item{tau.t}{Precision parameter for the main temporal effect.}
## ! \item{phi.t}{Mixing parameter in the \code{bym2} model
## !    assumed for the main temporal effect.}
## ! \item{tau.s}{Precision parameter for the main spatial effect.}
## ! \item{phi.s}{Mixing parameter in the \code{bym2} model
## !    assumed for the main spatial effect.}
## ! \item{tau.st}{Precision parameter for the spacetime effect.}
## ! \item{ev.t}{Eigenvalues and eigenvectors of the temporal
## !    precision matrix structure.}
## ! \item{ev.s}{Eigenvalues and eigenvectors of the spatial
## !    precision matrix structure.}
## ! }
## ! \value{
## !  A list with the following elements
## !   \item{time}{The time index for each obervation,
## !       with length equals m*n.}
## !   \item{space}{The spatial index for each obervation,
## !       with length equals m*n.}
## !   \item{spacetime}{The spacetime index for each obervation,
## !       with length equals m*n.}
## !   \item{x}{A list with the following elements}
## !   \item{t.iid}{The unstructured main temporal effect part.}
## !   \item{t.str}{The structured main temporal effect part.}
## !   \item{t}{The main temporal effect with length equals 2m.}
## !   \item{s.iid}{The unstructured main spatial effect part.}
## !   \item{s.str}{The structured main spatial effect part.}
## !   \item{s}{The main spatial effect with length equals 2n.}
## !   \item{st}{The spacetime interaction effect with length equals m*n.}
## !   \item{eta}{The linear predictor with length equals n*m.}
## ! }
## ! \author{Elias T. Krainski}
## ! \seealso{
## !     \code{\link{inla.knmodels}} for model fitting
## ! }
`inla.knmodels.sample` <- function(graph,
                                   m,
                                   type = 4,
                                   intercept = 0,
                                   tau.t = 1,
                                   phi.t = 0.7,
                                   tau.s = 1,
                                   phi.s = 0.7,
                                   tau.st = 1,
                                   ev.t = NULL,
                                   ev.s = NULL) {
    type <- pmatch(type, 1:4)
    if (!any(type == (1:4))) stop("'type' must be 1, 2, 3 or 4!")
    qsample <- function(q, ev = NULL, verbose = FALSE) {
        ## algorithm 3.1 to sample from a precision matrix
        ## or from the eigenvalue/vector pairs when suplied
        if (is.null(ev)) {
              ev <- eigen(as.matrix(q))
          }
        n <- length(ev$values)
        jj <- which(ev$values > sqrt(.Machine$double.eps))
        k <- n - length(jj)
        if (verbose) cat("rankdef =", k, "\n")
        y <- rnorm(n - k, 0, 1 / sqrt(ev$values[jj]))
        return(colSums(t(ev$vectors[, jj]) * y))
    }
    if (missingArg(ev.t)) {
        R.t <- crossprod(diff(
            Diagonal(m),
            lag = 1, differences = 1))
        R.t <- inla.scale.model(R.t, 
				constr=list(A=matrix(1, 1, m), e=0))
	ev.t <- eigen(as.matrix(R.t))
    } else {
        m <- length(ev.t$values)
    }
    if (missingArg(ev.s)) {
        if (missingArg(graph)) {
              stop("'graph' or 'ev.s' must be provided!")
          }
        graph <- inla.graph2matrix(graph)
        n <- nrow(graph)
        R.s <- inla.scale.model(Diagonal(n, colSums(graph)) - graph,
            constr = list(A = matrix(1, 1, n), e = 0)
        )
        ev.s <- eigen(as.matrix(R.s))
    } else {
        n <- length(ev.s$values)
    }
    dat <- list(time = rep(1:m, each = n), space = rep(1:n, m), spacetime = 1:(m * n), x = list())
    ### AR(1), as used before:
    ## dat$x$t <- arima.sim(model=list(ar=rho), n=m, ### sample with marginal variance = 1/tau.t
    ## rand.gen=function(leng) rnorm(leng, 0, sqrt((1-rho^2)/tau.t)))
    dat$x$t.str <- qsample(ev = ev.t)
    dat$x$t.iid <- rnorm(m, 0.0, 1.0)
    dat$x$time <- c(
        (sqrt(phi.t) * dat$x$t.str + sqrt(1 - phi.t) * dat$x$t.iid) / sqrt(tau.t),
        dat$x$t.str
    )
    dat$x$s.iid <- rnorm(length(ev.s$value), 0, 1)
    dat$x$s.str <- qsample(ev = ev.s)
    dat$x$space <- c(
        (sqrt(phi.s) * dat$x$s.str + sqrt(1 - phi.s) * dat$x$s.iid) / sqrt(tau.s),
        dat$x$s.str
    )
    if (type == 1) {
          ev.st <- list(values = rep(1, m * n), vectors = diag(m * n))
      }
    if (type == 2) {
          ev.st <- list(
              values = rep(ev.t$values, each = n),
              vectors = kronecker(ev.t$vectors, diag(n))
          )
      }
    if (type == 3) {
          ev.st <- list(
              values = rep(ev.s$values, m),
              vectors = kronecker(diag(m), ev.s$vectors)
          )
      }
    if (type == 4) {
          ev.st <- list(
              values = rep(ev.t$values, each = n) * rep(ev.s$values, m),
              vectors = kronecker(ev.t$vectors, ev.s$vectors)
          )
      }
    dat$x$spacetime <- qsample(ev = ev.st) / sqrt(tau.st)
    dat$x$eta <- intercept + dat$x$time[dat$time] +
        dat$x$space[dat$space] + dat$x$spacetime[dat$spacetime]
    return(dat)
}

## Export: summary!inla.jmarginal
## Export: print!summary.inla.jmarginal
## Export: print!inla.jmarginal

## Export: inla.rjmarginal inla.rjmarginal.eval inla.tjmarginal inla.1djmarginal

## ! \name{joint.marginal}
## ! \alias{inla.joint.marginal}
## ! \alias{inla.joint.marginal.eval}
## ! \alias{joint.marginal}
## ! \alias{joint.marginal.eval}
## ! \alias{rjmarginal}
## ! \alias{rjmarginal.eval}
## ! \alias{inla.rjmarginal}
## ! \alias{inla.rjmarginal.eval}
## ! \alias{inla.tjmarginal}
## ! \alias{tjmarginal}
## ! \alias{inla.1djmarginal}
## ! \alias{1djmarginal}
## !
## ! \title{Sample, transform and evaluate from a joint marginal approximation}
## !
## ! \description{Sample, transform and evalue from from a joint marginal approximation
## ! as returned using argument \code{selection} in \code{inla}.
## ! }
## !
## ! \usage{
## ! inla.rjmarginal(n, jmarginal, constr)
## ! inla.rjmarginal.eval(fun, samples, ...)
## ! inla.tjmarginal(jmarginal, A)
## ! inla.1djmarginal(jmarginal)
## ! }
## ! \arguments{
## !   \item{n}{The number of samples}
## !   \item{jmarginal}{A marginal object given  either by a \code{inla} object
## !     or \code{result$selection}}
## !   \item{constr}{Optional linear constraints;
## !                 see \code{?INLA::f} and argument \code{extraconstr}}
## !   \item{fun}{A function which is evaluated for each sample, similar to
## !              \code{inla.posterior.sample.eval}: please see the documentation
## !              for this functions for details. }
## !   \item{samples}{The samples, as in the form of the output from \code{inla.rjmarginal}}
## !   \item{A}{A matrix used for the linear combination}
## ! }
## !
## ! \value{%%
## ! THESE FUNCTIONS ARE EXPERIMENTAL FOR THE MOMENT (JULY 2020)
## !
## !
## ! \code{inla.rjmarginal} returns a list with the samples in \code{samples}
## ! (matrix) and the corresponding log-densities
## ! in \code{log.density} (vector). Each column in \code{samples} contains one sample.
## !
## ! \code{inla.rjmarginal.eval} returns a matrix, where each row is the (vector) function
## ! evaluated at each sample.
## !
## ! \code{inla.tjmarginal} returns a \code{inla.jmarginal}-object of the linear combination
## ! defined by the matrix \code{A}.
## !
## ! \code{inla.1djmarginal} return the marginal densities from a joint approximation.
## ! }
## !
## ! \author{Cristian Chiuchiolo and Havard Rue \email{hrue@r-inla.org}}
## ! \seealso{\code{\link{inla}}}
## !
## ! \examples{
## ! n = 10
## ! x = 1+rnorm(n)
## ! xx = 3 + rnorm(n)
## ! y = 1 + x + xx + rnorm(n)
## ! selection = list(xx=1, Predictor = 3:4, x=1)
## ! r = inla(y ~ 1 + x + xx,
## !          data = data.frame(y, x, xx),
## !          selection = selection)
## ! ns = 100
## ! xx = inla.rjmarginal(ns, r)
## !
## ! print(cbind(mean = r$selection$mean, sample.mean = rowMeans(xx$samples)))
## ! print("cov matrix")
## ! print(round(r$selection$cov.matrix, dig=3))
## ! print("sample cov matrix")
## ! print(round(cov(t(xx$samples)), dig=3))
## !
## ! skew = function(z) mean((z-mean(z))^3)/var(z)^1.5
## ! print(round(cbind(skew = r$selection$skewness,
## !                   sample.skew = apply(xx$sample, 1, skew)), dig=3))
## !
## ! ## illustrating the eval function
## ! n = 10
## ! x = rnorm(n)
## ! eta = 1 + x
## ! y = eta + rnorm(n, sd=0.1)
## ! selection = list(x = 1, Predictor = c(1, 2, 4, 5),  '(Intercept)' = 1)
## ! r = inla(y ~ 1 + x,
## !          data = data.frame(y, x),
## !          selection = selection)
## ! xx = inla.rjmarginal(100,  r)
## ! xx.eval = inla.rjmarginal.eval(function() c(x, Predictor, Intercept),  xx)
## ! print(cbind(xx$samples[, 1]))
## ! print(cbind(xx.eval[, 1]))
## !
## ! constr <- list(A = matrix(1, ncol = n, nrow = 1), e = 1)
## ! x <- inla.rjmarginal(10, r, constr = constr)
## !
## ! A <- matrix(rnorm(n^2), n, n)
## ! b <- inla.tjmarginal(r, A)
## ! b.marg <- inla.1djmarginal(b)
## !}

`inla.rjmarginal` <- function(n, jmarginal, constr) {
    if (missing(jmarginal) || missing(n) || n <= 0) {
        return(list(samples = matrix(ncol = 0, nrow = 0), log.density = numeric(0)))
    }

    if (inherits(jmarginal, "inla")) {
        jmarginal <- jmarginal$selection
    } else if (inherits(jmarginal, "inla.jmarginal")) {
        ## ok
    } else {
        stop("Unknown object: argument 'jmarginal'")
    }

    ## the setup depends on 'constr' or not
    if (missing(constr)) {
        constr <- NULL
        Q <- solve(jmarginal$cov.matrix)
        Qi <- jmarginal$cov.matrix
        mm <- jmarginal$mean
    } else {
        constr$nc <- nrow(constr$A) ## this is required in the 'configs'
        Q <- solve(jmarginal$cov.matrix)
        Qi <- jmarginal$cov.matrix
        QiA <- Qi %*% t(constr$A)
        SS <- solve(constr$A %*% QiA)
        Qi <- Qi - QiA %*% SS %*% t(QiA)
        mm <- jmarginal$mean - QiA %*% SS %*% (constr$A %*% jmarginal$mean - constr$e)
    }

    ## build a fake 'inla'-object, so we can feed that into 'inla.posterior.sample'
    m <- length(jmarginal$mean)
    r <- list(
        misc = list(
            configs = list(
                n = m,
                nconfig = 1,
                max.log.posterior = 0,
                nz = m * (m + 1) / 2,
                contents = list(
                    tag = jmarginal$names,
                    start = 1:m,
                    length = rep(1, m)
                ),
                ntheta = 0,
                constr = constr,
                config = list(list(
                    theta = NULL,
                    Q = inla.as.sparse(Q),
                    Qinv = inla.as.sparse(Qi),
                    mean = mm,
                    improved.mean = mm,
                    skewness = jmarginal$skewness,
                    log.posterior = 0,
                    log.posterior.orig = 0
                ))
            )
        )
    )
    class(r) <- "inla"

    x <- inla.posterior.sample(n, r,
                               use.improved.mean = TRUE, skew.corr = TRUE,
                               add.names = FALSE
                               )
    xx <- matrix(unlist(lapply(x, function(z) z$latent)), ncol = n)
    log.dens <- unlist(lapply(x, function(z) z$logdens$joint))
    rownames(xx) <- jmarginal$names
    colnames(xx) <- paste0("sample:", 1:n)
    names(log.dens) <- colnames(xx)

    return(list(samples = xx, log.density = log.dens))
}

`inla.rjmarginal.eval` <- function(fun, samples, ...) {
    stopifnot(all(names(samples) == c("samples", "log.density")))

    var <- "inla.jmarginal.eval.warning.given"
    if (!(exists(var, envir = inla.get.inlaEnv()) &&
          get(var, envir = inla.get.inlaEnv()) == TRUE)) {
        warning("Function 'inla.rjmarginal.eval()' is experimental.")
        assign(var, TRUE, envir = inla.get.inlaEnv())
    }

    nm.split <- strsplit(rownames(samples$samples), ":")
    ## determine the names
    nms <- unique(unlist(lapply(nm.split, function(x) x[1])))
    contents <- rep(list(list()), length(nms))
    for (i in seq_along(nms)) {
        contents[[i]]$tag <- nms[i]
        contents[[i]]$start <- min(which(nms[i] == lapply(nm.split, function(x) x[1])))
        contents[[i]]$sample.idx <- which(nms[i] == lapply(nm.split, function(x) x[1]))
    }

    my.fun <- function(a.sample, .contents, .fun, ...) {
        env <- new.env()
        for (i in seq_along(.contents)) {
            .tmp <- a.sample[contents[[i]]$sample.idx]
            assign(.contents[[i]]$tag, .tmp, envir = env)
        }
        if (exists("(Intercept)", envir = env)) {
            assign("Intercept", get("(Intercept)", envir = env),
                   envir = env
                   )
        }
        parent.env(env) <- .GlobalEnv
        environment(.fun) <- env
        return(.fun(...))
    }

    ret <- apply(samples$samples, 2, my.fun, .fun = fun, .contents = contents, ...)
    ret <- matrix(ret, ncol = ncol(samples$samples))
    colnames(ret) <- paste0("sample:", 1:ncol(ret))
    rownames(ret) <- paste0("fun[", 1:nrow(ret), "]")

    return(ret)
}

`print.inla.jmarginal` <- function(x) {
    x$.private <- NULL
    class(x) <- class(list())
    print(x)
}

`summary.inla.jmarginal` <- function(object) {
    inla.require("sn", stop.on.error = TRUE)
    mode.sn <- function(xi, omega, alpha) {
        med <- sn::qsn(0.5, xi, omega, alpha)
        res <- optimize(
            f = dsn, interval = c(med - omega, med + omega), maximum = TRUE,
            ## arguments to 'dsn'
            log = TRUE, xi = xi, omega = omega, alpha = alpha
        )
        return(res$maximum)
    }

    n.sel <- object$names
    mu <- object$mean
    std <- sqrt(diag(object$cov.matrix))
    dsn.xi <- object$marginal.sn.par$xi
    dsn.omega <- object$marginal.sn.par$omega
    dsn.alpha <- object$marginal.sn.par$alpha
    prob <- c(0.025, 0.5, 0.975)
    mode <- c()
    qsn.eval <- matrix(NA, nrow = length(n.sel), ncol = length(prob))
    for (i in seq_along(n.sel)) {
        qsn.eval[i, ] <- sn::qsn(prob, xi = dsn.xi[i], omega = dsn.omega[i], alpha = dsn.alpha[i])
        mode[i] <- mode.sn(xi = dsn.xi[i], omega = dsn.omega[i], alpha = dsn.alpha[i])
    }
    obj <- cbind(mu, std, qsn.eval, mode)
    rownames(obj) <- n.sel
    colnames(obj) <- c("mean", "sd", "0.025quant", "0.5quant", "0.975quant", "mode")
    ret <- list(msg = "Joint marginal is computed for: ", matrix = obj)
    class(ret) <- "summary.inla.jmarginal"

    return(ret)
}

`print.summary.inla.jmarginal` <- function(x) {
    print(x$msg)
    print(as.matrix(x$matrix))
    return(invisible())
}

`inla.tjmarginal` <- function(jmarginal, A) {
    inla.require("mpoly", stop.on.error = TRUE)
    inla.require("symmoments", stop.on.error = TRUE)

    if (inherits(jmarginal, "inla")) {
        jmarginal <- jmarginal$selection
    } else if (inherits(jmarginal, "inla.jmarginal")) {
        ## ok
    } else {
        stop("Unknown object: argument 'jmarginal'")
    }
    stopifnot(is.matrix(A))

    sel.len <- length(jmarginal$names)
    if (ncol(A) != sel.len){
        stop("Incorrect dimension for the selected components")
    }

    if (is.null(rownames(A))) {
        names.sel <- sapply(1:nrow(A), function(x) paste0("Lin:", x))
    } else {
        names.sel <- rownames(A)
    }

    ## create global environment symmoments to store all moments
    if (!exists("symmoments")) {
        symmoments <<- new.env()
    }

    skew.max <- 0.99
    moments <- jmarginal$.private$moments
    mom1 <- moments[[1]]
    mom2 <- moments[[2]]
    mom3 <- moments[[3]]
    S <- jmarginal$cov.matrix
    mu.tjoint <- A %*% mom1
    S.tjoint <- A %*% S %*% t(A)
    m2.tjoint <- diag(S.tjoint) + mu.tjoint^2
    m3.tjoint <- skew.tjoint <- c()
    for (lc in 1:nrow(A)) {
        x <- A[lc, ]
        lc.ind <- which(x != 0 & !is.na(x))
        coef <- A[lc, lc.ind]
        n <- length(lc.ind)
        m1.lin <- mom1[lc.ind]
        m3.lin <- mom3[lc.ind]
        S.lin <- S[lc.ind, lc.ind]
        S.str <- S.lin[lower.tri(S.lin, diag = TRUE)]
        m <- rep(2, n)
        names(m) <- sapply(1:n, function(x) paste0("x", x))
        names(coef) <- rep("coef", n)
        bicross <- tricross <- NULL
        for (i in 1:(n - 1)) {
            j <- i + 1
            while ((i < j) && (j <= n)) {
                mom.eval <- rep(0, n)
                bicross <- c(
                    bicross, list(c(m[i], m[j] / 2, 3 * coef[i]^2, coef[j])),
                    list(c(m[i] / 2, m[j], 3 * coef[i], coef[j]^2))
                )
                mom.eval[i] <- 1
                mom.eval[j] <- 2
                symmoments::make.all.moments(moment = mom.eval,
                                             verbose = FALSE)
                mom.eval[i] <- 2
                mom.eval[j] <- 1
                symmoments::make.all.moments(moment = mom.eval,
                                             verbose = FALSE)
                j <- j + 1
            }
        }

        m[1:n] <- 3
        if (n > 2) {
            for (i in 1:n) {
                j <- 1
                while (j < i) {
                    k <- 1
                    while (k < j) {
                        mom.eval <- rep(0, n)
                        tricross <- c(tricross, list(c(m[i] / 3, m[j] / 3, m[k] / 3, 6 * coef[i], coef[j], coef[k])))
                        mom.eval[i] <- mom.eval[j] <- mom.eval[k] <- 1
                        symmoments::make.all.moments(moment = mom.eval, 
                                                     verbose = FALSE)
                        k <- k + 1
                    }
                    j <- j + 1
                }
            }
        }

        all3 <- c(bicross, tricross)
        poly3 <- mpoly::mpoly(all3)
        mom1.m <- mu.tjoint[lc]
        mom2.m <- m2.tjoint[lc]
        mom3.m <- sum(coef^3 * m3.lin) +
            symmoments::evaluate_expected.polynomial(poly = poly3, mu = m1.lin, sigma = S.str)
        skew.m <- (mom3.m - 3 * mom2.m * mom1.m + 2 * mom1.m^3) * ((mom2.m - mom1.m^2)^(-1.5)) # global skewnesses
        m3.tjoint[lc] <- mom3.m
        if (any(abs(skew.m) > skew.max)) {
            skew.m <- pmax(-skew.max, pmin(skew.max, skew.m))
            warning(paste0("One or more marginal skewness are too high. Coerced to be ", skew.max))
        }
        skew.tjoint[lc] <- skew.m
    }
    sn.par <- inla.sn.reparam(moments = list(mean = as.numeric(mu.tjoint),
                                          variance = diag(S.tjoint),
                                          skewness = skew.tjoint))
    output <- list()
    output$names <- names.sel
    output$mean <- mu.tjoint
    output$cov.matrix <- S.tjoint
    output$skewness <- skew.tjoint
    output$marginal.sn.par$xi <- sn.par$xi
    output$marginal.sn.par$omega <- sn.par$omega
    output$marginal.sn.par$alpha <- sn.par$alpha
    output$.private$moments <- list(as.numeric(mu.tjoint), as.numeric(m2.tjoint), m3.tjoint)
    names(output$.private$moments[[1]]) <- names.sel
    names(output$.private$moments[[2]]) <- names.sel
    names(output$.private$moments[[3]]) <- names.sel
    class(output) <- "inla.jmarginal"

    return(output)
}

`inla.1djmarginal` <- function(jmarginal) {
    inla.require("sn", stop.on.error = TRUE)
    if (inherits(jmarginal, "inla")) {
        jmarginal <- jmarginal$selection
    } else if (inherits(jmarginal, "inla.jmarginal")) {
        ## ok
    } else {
        stop("Unknown object: argument 'jmarginal'")
    }

    n.sel <- jmarginal$names
    mu <- jmarginal$mean
    std <- sqrt(diag(jmarginal$cov.matrix))
    dsn.xi <- jmarginal$marginal.sn.par$xi
    dsn.omega <- jmarginal$marginal.sn.par$omega
    dsn.alpha <- jmarginal$marginal.sn.par$alpha
    obj <- vector("list", length(n.sel))
    names(obj) <- n.sel

    ## copy from density.c
    q.many <- c(
        0.0000001, 0.000001, 0.00001, 0.0001, 0.0005, 0.001, 0.005, 0.01, 0.025,
        0.05, 0.075, 0.10, 0.125, 0.15, 0.175, 0.2, 0.225, 0.25, 0.275, 0.30,
        0.325, 0.35, 0.375, 0.40, 0.425, 0.45, 0.46, 0.47, 0.475, 0.48, 0.49,
        0.50, 0.51, 0.52, 0.525, 0.53, 0.54, 0.55, 0.575, 0.60, 0.625, 0.65,
        0.675, 0.70, 0.725, 0.75, 0.775, 0.80, 0.825, 0.85, 0.875, 0.9, 0.925,
        0.95, 0.975, 0.99, 0.995, 0.999, 0.9995, 0.9999, 0.99999, 0.999999, 0.9999999
    )

    for (i in seq_along(n.sel)) {
        val <- sn::qsn(q.many, xi = dsn.xi[i], omega = dsn.omega[i], alpha = dsn.alpha[i])
        dsn.eval <- sn::dsn(val, xi = dsn.xi[i], omega = dsn.omega[i], alpha = dsn.alpha[i])
        obj[[i]] <- cbind(val, dsn.eval)
        colnames(obj[[i]]) <- c("x", "y")
    }

    return(obj)
}

## Export: print!inla

## !\name{print.inla}
## !\alias{print.inla}
## !\title{Print a INLA fit }
## !\description{
## !Print a INLA fit
## !}
## !\usage{
## !\method{print}{inla}(x, digits = 3L, ...)
## !}
## !\arguments{
## !  \item{x}{An inla-object (output from an \code{\link{inla}}-call).}
## !  \item{digits}{Number of digits to print}
## !  \item{...}{ other arguments.}
## !}
## !\details{
## ! None
## !}
## !\value{
## ! None
## !}
## !\author{Havard Rue}
## !
## !\seealso{ \code{\link{inla}} }
## !\examples{
## !## None
## !}

`print.inla` <- function(x, digits = 3L, ...) {
    nsmall <- 2L
    form <- strwrap(inla.formula2character(x$call))
    cat("Call:\n")
    for (i in seq_along(form)) {
        cat("  ", form[i], "\n")
    }

    if (inla.is.element("cpu.used", x)) {
        cat("Time used:\n  ")
        cat(
            sep = "",
            names(x$cpu.used)[1], " = ",
            format(x$cpu.used[1], digits = digits), ", ",
            names(x$cpu.used)[2], " = ",
            format(x$cpu.used[2], digits = digits), ", ",
            names(x$cpu.used)[3], " = ",
            format(x$cpu.used[3], digits = digits), ", ",
            names(x$cpu.used)[4], " = ",
            format(x$cpu.used[4], digits = digits), "\n"
        )
    }

    return(invisible())
}

## Nothing to Export.

## this is a bit messy. (model, section) picks out the right model in the correct section. this
## model has a default setting for 'hyper'. simply replace these entries with given ones, first
## in 'hyper', then in 'initial', 'fixed', 'prior' and 'param'. somewhat tricky is that the
## length in 'param' depends on the argument(s) of 'prior' as various priors can have different
## number of parameters. also, if NA or NULL is given, then we use the default ones.

## I think its a better strategy to take initial, fixed, prior, param and make a hyper-similar
## structure and then merge these two. the current code is messy but does not have to be.

inla.set.hyper <- function(
                           model = NULL,
                           section = NULL,
                           hyper = NULL,
                           initial = NULL,
                           fixed = NULL,
                           prior = NULL,
                           param = NULL,
                           debug = FALSE,
                           hyper.default = NULL) {
    ## name of the list can be in any CASE, and a longer name is ok,
    ## like 'list(precision=list(...))' where the 'name="prec"'.
    skip.final.check <- FALSE

    if (is.null(section)) {
        stop("No section given; please fix...")
    }
    mm <- inla.models()
    section <- match.arg(section, names(mm))

    if (is.null(hyper.default)) {
        ## default values are given in the inla.models()
        hyper.new <- inla.model.properties(model, section)$hyper
    } else {
        ## default values are given by the user. process these
        hyper.new <- inla.set.hyper(model, section, hyper = hyper.default, debug = debug)
    }
    nhyper <- length(hyper.new)

    if (debug) {
        cat(paste("* Get default hyper from model", model, "in section", section), "\n")
    }

    if (nhyper == 0L) {
        if (!is.null(hyper) && !identical(hyper, list())) {
            stop(inla.paste(c(
                "Model", model, "[", section,
                "], has none hyperparameters, but 'hyper' is ",
                inla.paste(list(hyper))
            ), sep = " "))
        }
        if (!is.null(initial) && length(initial) > 0L) {
            stop(inla.paste(c(
                "Model", model, "[", section, "], has none hyperparameters, but 'initial' is ",
                initial
            ), sep = " "))
        }
        if (!is.null(fixed) && length(fixed) > 0L) {
            stop(inla.paste(c(
                "Model", model, "[", section, "], has none hyperparameters, but 'fixed' is",
                fixed
            ), sep = " "))
        }
        if (!is.null(param) && length(param) > 0L) {
            stop(inla.paste(c(
                "Model", model, "[", section, "], has none hyperparameters, but 'param' is",
                param
            ), sep = " "))
        }
        if (!is.null(prior) && length(prior) > 0L) {
            stop(inla.paste(c(
                "Model", model, "[", section, "], has none hyperparameters, but 'prior' is",
                param
            ), sep = " "))
        }
        return(hyper.new)
    }

    if (!is.null(hyper)) {
        for (nm in names(hyper)) {
            if (debug) {
                cat(paste("* Check contents of given hyper$", nm, sep = ""), "\n")
            }
            valid.keywords <- c(
                "theta",
                paste("theta", 1L:nhyper, sep = ""),
                as.character(sapply(hyper.new, function(x) x$short.name)),
                as.character(sapply(hyper.new, function(x) x$name))
            )

            if (is.null(nm)) {
                stop(paste("Missing name/keyword in `hyper'; must be one of ", inla.paste(valid.keywords), ".",
                    sep = " "
                ))
            }
            if (!any(inla.strcasecmp(nm, valid.keywords))) {
                stop(paste("Unknown keyword in `hyper' `", nm,
                    "'. Must be one of ", inla.paste(valid.keywords), ".",
                    sep = " "
                ))
            }
        }
    }

    ## need `prior' to be before `param'!
    keywords <- c("initial", "fixed", "prior", "param")
    off <- list(0L, 0L, 0L, 0L)
    names(off) <- keywords

    ## do each hyperparameter one by time. fill into `hyper.new'
    for (ih in 1L:nhyper) {
        if (debug) {
            cat(paste("** Check hyperparameter", ih, "with name", names(hyper.new)[ih]), "\n")
        }

        ## find idx in the new one
        idx.new <- which(names(hyper.new) == paste("theta", ih, sep = ""))
        if (ih == 1L && length(idx.new) == 0L) {
            idx.new <- which(names(hyper.new) == "theta")
        }
        stopifnot(length(idx.new) > 0L)
        name <- hyper.new[[idx.new]]$name ## full name
        short.name <- hyper.new[[idx.new]]$short.name ## short name
        stopifnot(!is.null(short.name))

        if (debug) {
            cat(paste("** idx.new =", idx.new), "\n")
        }

        if (!is.null(hyper)) {
            ## find idx in the given one
            idx <- which(names(hyper) == paste("theta", ih, sep = ""))
            if (length(idx) == 0L && nhyper == 1L) {
                idx <- which(names(hyper) == "theta")
            }
            if (length(idx) == 0L) {
                idx <- which(inla.strcasecmp(names(hyper), name))
            }
            if (length(idx) == 0L) {
                idx <- which(inla.strcasecmp(names(hyper), short.name))
            }

            if (length(idx) == 0L) {
                h <- NULL
                idx <- NULL
            } else {
                h <- hyper[[idx]]
                if (!is.list(h)) {
                    stop(paste("Argument `hyper$theta", ih, "' or `", short.name,
                        "' or `", name, "', is not a list: ", h,
                        sep = ""
                    ))
                }
            }

            if (debug) {
                cat(paste("** Corresponding idx in given hyper =", idx), "\n")
                if (is.null(h)) {
                    cat(paste("*** h is NULL"), "\n")
                } else {
                    cat(paste("*** h is !NULL"), "\n")
                }
            }
        } else {
            h <- NULL
            idx <- NULL
        }

        for (key in keywords) {
            key.val <- inla.eval(key)
            if (debug) {
                cat(paste("*** Process key =", key), "\n")
                cat(inla.paste("*** key =", inla.ifelse(
                    is.null(key.val) || (is.numeric(key.val) && length(key.val) == 0),
                    "NULL", as.character(key.val)
                )), "\n")
            }

            ## special case. if attribute is set to 'read only' then refuse to change value
            read.only <- attr(hyper.new[[idx.new]][[key]], "inla.read.only")
            if (!is.null(read.only) && read.only == TRUE) {
                if (!is.null(h[[key]]) && h[[key]] != hyper.new[[idx.new]][[key]]) {
                      stop(paste("Setting hyperparameter `", name,
                          "'. Key `", key, "' = '", hyper.new[[idx.new]][[key]],
                          "' is 'read-only', cannot change to `", h[[key]], "'.",
                          sep = ""
                      ))
                  }
                ii <- idx.new + off[[key]]
                if (!is.null(key.val) && !is.na(key.val[ii])) {
                    if (key.val[ii] != hyper.new[[idx.new]][[key]]) {
                        stop(paste("Setting hyperparameter `", name,
                            "'. Key `", key, "' = '", hyper.new[[idx.new]][[key]],
                            "' is 'read-only', cannot change to `", key.val[ii], "'.",
                            sep = ""
                        ))
                    }
                }
            }

            ## start of cmd is here
            if (!is.null(key.val)) {
                ## known length
                if (key == "param") {
                    len <- inla.model.properties(hyper.new[[idx.new]]$prior, "prior")$nparameters
                    if (len < 0L) {
                        ## this is a special case, where the number of
                        ## parameters in the prior vary. first example
                        ## is the spde2/spde3 model. in this case, we just
                        ## eat all the remaining parameters.
                        skip.final.check <- TRUE
                    } else {
                        ## this is the normal case
                        if (len <= length(hyper.new[[idx.new]][[key]])) {
                            if (len > 0L) {
                                hyper.new[[idx.new]][[key]] <- hyper.new[[idx.new]][[key]][1L:len]
                            } else if (len == 0L) {
                                hyper.new[[idx.new]][[key]] <- numeric(0L)
                            } else {
                                stop("SHOULD NOT HAPPEN")
                            }
                        } else if (len > length(hyper.new[[idx.new]][[key]])) {
                            hyper.new[[idx.new]][[key]] <- c(
                                hyper.new[[idx.new]][[key]],
                                rep(NA, len - length(hyper.new[[idx.new]][[key]]))
                            )
                        } else {
                            stop("SHOULD NOT HAPPEN")
                        }
                    }
                } else {
                    len <- length(hyper.new[[idx.new]][[key]])
                }
                ## given length
                llen <- length(key.val) - off[[key]]

                if (llen < len) {
                    key.val <- c(key.val, rep(NA, len - llen))
                }

                if (llen >= 0L) {
                    if (len < 0L) {
                        if (debug) {
                            cat(inla.paste(c("*** Replace hyper.new|", idx.new, "|", key, "|", idxx, " with ", key.val)), "\n")
                        }
                        hyper.new[[idx.new]][[key]] <- key.val
                    } else {
                        if (len > 0L) {
                            ## set those NA's to the default ones
                            ii <- key.val[off[[key]] + 1L:len]
                            idxx <- which(!(is.na(ii) | is.null(ii)))
                            if (length(idxx) > 0L) {
                                if (debug) {
                                    cat(inla.paste(c("*** Replace hyper.new|", idx.new, "|", key, "|", idxx, " with ", ii[idxx])), "\n")
                                }
                                hyper.new[[idx.new]][[key]][idxx] <- ii[idxx]
                            }
                        }
                    }
                    off[[key]] <- off[[key]] + len
                }
            }

            test.val <- (!is.null(h) && !is.null(h[[key]]) && any(!(is.na(h[[key]]))))

            if (debug) {
                cat(paste("*** test.val =", test.val), "\n")
            }

            if (is.na(test.val) || test.val) {
                ## known length
                if (key == "param") {
                    len <- inla.model.properties(hyper.new[[idx.new]]$prior, "prior")$nparameters
                    if (debug) {
                        cat(paste("*** nparam =", len), "\n")
                    }
                    if (len < 0L) {
                        ## see explanation above
                        skip.final.check <- TRUE
                        hyper.new[[idx.new]][[key]] <- h[[key]]
                    } else {
                        ## normal case
                        if (len < length(hyper.new[[idx.new]][[key]]) && len > 0L) {
                            hyper.new[[idx.new]][[key]] <- hyper.new[[idx.new]][[key]][1L:len]
                        } else if (len < length(hyper.new[[idx.new]][[key]]) && len == 0L) {
                            hyper.new[[idx.new]][[key]] <- NULL
                        } else if (len > length(hyper.new[[idx.new]][[key]])) {
                            hyper.new[[idx.new]][[key]] <- c(
                                hyper.new[[idx.new]][[key]],
                                rep(NA, len - length(hyper.new[[idx.new]][[key]]))
                            )
                        }
                    }
                } else {
                    len <- length(hyper.new[[idx.new]][[key]])
                }
                ## given length
                llen <- length(h[[key]])

                if (len >= 0L) {
                    if (llen > len) {
                        stop(paste("model", model, "[", section, "], hyperparam", ih, ", length(hyper[[key]]) =",
                            llen, ">", len,
                            sep = " "
                        ))
                    } else if (llen < len) {
                        h[[key]] <- c(h[[key]], rep(NA, len - llen))
                    }
                }
                if (len != 0L) {
                    ## set those NA's to the default ones
                    idxx <- which(!is.na(h[[key]]) & !is.null(h[[key]]))
                    if (length(idxx) > 0L) {
                        if (debug) {
                            cat(inla.paste(c(
                                "*** Replace hyper.new|", idx.new, "|", key, "|", idxx, " with h|", key, "|", idxx,
                                "=", h[[key]][idxx]
                            )), "\n")
                        }
                        hyper.new[[idx.new]][[key]][idxx] <- h[[key]][idxx]
                    }
                }
            }

            if (key == "param") {
                ans <- inla.model.properties(hyper.new[[idx.new]]$prior, "prior", stop.on.error = TRUE)
                if (ans$nparameters >= 0L) {
                    if (length(hyper.new[[idx.new]]$param) != ans$nparameters) {
                        stop(paste("Wrong length of prior-parameters, prior `", hyper.new[[idx.new]]$prior, "' needs ",
                            ans$nparameters, " parameters, you have ",
                            length(hyper.new[[idx.new]]$param), ".",
                            sep = ""
                        ))
                    }
                }
            }
        }
    }

    for (key in keywords) {
        key.val <- inla.eval(key)
        if (key != "param" || (key == "param" && !skip.final.check)) {
            if (!is.null(key.val) && (length(key.val) > off[[key]])) {
                stop(paste("Length of argument `", key, "' is ", length(key.val),
                    ", does not match the total length of `", key, "' in `hyper' which is ", off[[key]],
                    sep = ""
                ))
            }
        }
    }

    return(hyper.new)
}

## nothing to export


`inla.os` <- function(type = c("linux", "mac", "mac.arm64", "windows", "else")) {
    if (missing(type)) {
        stop("Type of OS is required.")
    }
    type <- match.arg(type)

    if (type == "windows") {
        return(.Platform$OS.type == "windows")
    } else if (type == "mac.arm64") {
        result <- (file.info("/Library")$isdir && file.info("/Applications")$isdir)
        if (is.na(result)) {
            result <- FALSE
        }
        result <- result && (Sys.info()[["machine"]] == "arm64")
        return(result)
    } else if (type == "mac") {
        result <- (file.info("/Library")$isdir && file.info("/Applications")$isdir)
        if (is.na(result)) {
            result <- FALSE
        }
        if (result) {
            ## check that the version is at least the one use to build the binaries.
            s <- system("sw_vers -productVersion", intern = T)
            vers <- as.integer(strsplit(s, ".", fixed = TRUE)[[1]])
            ver <- vers[1] + vers[2] / 10
            s.req <- 10.15 ## @@@HARDCODED@@@
            if (ver < s.req) {
                warning(paste0("Your version, ",
                               s,
                               ", of MacOS might be to old for R-INLA which is built on MacOS ",
                               as.character(s.req)))
            }
        }
        result <- result && (Sys.info()[["machine"]] != "arm64")
        return(result)
    } else if (type == "linux") {
        return((.Platform$OS.type == "unix") && !inla.os("mac") && !inla.os("mac.arm64"))
    } else if (type == "else") {
        return(TRUE)
    } else {
        stop("This shouldn't happen.")
    }
}
`inla.os.type` <- function() {
    for (os in c("windows", "mac", "mac.arm64", "linux", "else")) {
        if (inla.os(os)) {
            return(os)
        }
    }
    stop("This shouldn't happen.")
}

## this seems be to a nice way to test 32/64 bits architecture.
`inla.os.32or64bit` <- function() {
    return(ifelse(.Machine$sizeof.pointer == 4, "32", "64"))
}
`inla.os.is.32bit` <- function() {
    return(inla.os.32or64bit() == "32")
}
`inla.os.is.64bit` <- function() {
    return(inla.os.32or64bit() == "64")
}

#' Convert between parameterizations for the AR(p) model
#'
#' These functions convert between the AR(p) coefficients `phi`, the
#' partial autorcorrelation coefficients `pacf` and the autocorrelation
#' function `acf`.  The `phi`-parameterization is the same as used
#' for `arima`-models in `R`; see `?arima` and the
#' parameter-vector `a` in `Details`.
#'
#'
#' @aliases inla.ar.pacf2phi ar.pacf2phi pacf2phi inla.ar.phi2pacf ar.phi2pacf
#' phi2pacf inla.ar.phi2acf ar.phi2acf phi2acf inla.ar.pacf2acf ar.pacf2acf
#' pacf2acf
#' @param pac The partial autorcorrelation coefficients
#' @param phi The AR(p) parameters `phi`
#' @param lag.max The maximum lag to compute the ACF for
#' @return
#' * `inla.ar.pacf2phi` returns `phi` for given `pacf`.
#' * `inla.ar.phi2pacf` returns `pac` for given `phi`.
#' * `inla.ar.phi2acf` returns `acf` for given `phi`.
#' * `inla.ar.pacf2acf` returns `acf` for given `pacf`.
#' @author Havard Rue \email{hrue@@r-inla.org}
#' @examples
#'
#' pac <- runif(5)
#' phi <- inla.ar.pacf2phi(pac)
#' pac2 <- inla.ar.phi2pacf(phi)
#' print(paste("Error:", max(abs(pac2 - pac))))
#' print("Correlation matrix (from pac)")
#' print(toeplitz(inla.ar.pacf2acf(pac)))
#' print("Correlation matrix (from phi)")
#' print(toeplitz(inla.ar.phi2acf(phi)))
#' @export
#' @rdname ar

## functions for the AR model, same as its C-versions in ar.c
inla.ar.pacf2phi <- function(pac) {
    ## I know, the R-coding is a bit weird as these are translated
    ## from arima.c in R...

    ## run the Durbin-Levinson recursions to find phi_{j.}, ( j = 2,
    ## ..., p and phi_{p.} are the autoregression coefficients
    p <- length(pac)
    stopifnot(p > 0)
    phi <- work <- pac
    if (p > 1L) {
        for (j in 1L:(p - 1L)) {
            a <- phi[j + 1L]
            phi[1L:j] <- work[1L:j] <- work[1L:j] - a * phi[j:1L]
        }
    }

    return(phi)
}
#' @export
#' @rdname ar
inla.ar.phi2pacf <- function(phi) {
    ## I know, the R-coding is a bit weird as these are translated
    ## from arima.c in R...

    ## Run the Durbin-Levinson recursions backwards to find the PACF
    ## phi_{j.} from the autoregression coefficients
    p <- length(phi)
    stopifnot(p > 0)
    work <- pac <- phi

    if (p > 1L) {
        for (j in (p - 1L):1L) {
            a <- pac[j + 1L]
            pac[1L:j] <- work[1L:j] <- (pac[1L:j] + a * pac[j:1L]) / (1.0 - a^2)
        }
    }

    return(pac)
}
#' @export
#' @rdname ar
inla.ar.phi2acf <- function(phi, lag.max = length(phi)) {
    ## return acf for given phi
    p <- length(phi)
    stopifnot(p > 0)

    A <- matrix(0, p, p)
    b <- numeric(p)

    for (i in 1:p) {
        for (j in 1:p) {
            if (i == j) {
                A[i, j] <- -1.0
            } else {
                lag <- abs(i - j)
                A[i, lag] <- phi[j] + A[i, lag]
            }
        }
        b[i] <- -phi[i]
    }
    r <- try(solve(A, b), silent = TRUE)
    ## if the model is singular,  then return nothing
    if (inherits(r, "try-error")) {
        return(numeric(0))
    }
    r <- pmax(-1, pmin(1, r)) ## known to be true
    r <- c(1, r)
    if (lag.max > p) {
        r <- c(r, rep(0, lag.max - p))
        for (i in (p + 1):(lag.max + 1)) {
            r[i] <- sum(phi * r[(i - 1):(i - 1 - p + 1)])
        }
        r <- pmax(-1, pmin(1, r)) ## known to be true
    }

    return(r)
}
#' @export
#' @rdname ar
inla.ar.pacf2acf <- function(pac, lag.max = length(pac)) {
    return(inla.ar.phi2acf(inla.ar.pacf2phi(pac), lag.max = lag.max))
}

context("test 'latent ar'")

test_that("Case 1", {
    for(i in 1:20) {
        pac = runif(i)
        phi = inla.ar.pacf2phi(pac)
        pac2 = inla.ar.phi2pacf(phi)
        expect_true( mean(abs(pac2-pac)) < 0.00001)
    }
})

test_that("Case 2", {
    set.seed(123)
    n = 300
    phi = 0.9
    y = arima.sim(n, model = list(ar = phi))
    y = scale(y)*2
    idx = 1:n
    param.phi = c(0, 1)
    param.prec = c(1, 0.01)
    r1 = inla(y ~ -1 + f(idx, model='ar1',
            hyper = list(
                    prec = list(initial = 1, fixed=FALSE, param = param.prec),
                    rho = list(initial = 3, fixed=FALSE, param = param.phi))),
            family = "gaussian", 
            control.family = list(initial = 5, fixed=TRUE), 
            data = data.frame(y, idx))

    r = inla(y ~ -1 + f(idx, model='ar',
            order = 1, 
            hyper = list(
                    prec = list(initial = 1, fixed=FALSE, param = param.prec),
                    theta2 = list(initial = 3, fixed=FALSE, param = param.phi))), 
            family = "gaussian", 
            control.family = list(initial = 5, fixed=TRUE), 
            data = data.frame(y, idx)) 
    expect_true(all(abs(r1$summary.hyperpar[, "mean"] - r$summary.hyperpar[, "mean"]) < 0.001))
})
    
test_that("Case 3", {
    set.seed(1234)
    n = 1000
    p = 3
    pacf = runif(p)
    phi = inla.ar.pacf2phi(pacf)
    y = arima.sim(n, model = list(ar = phi))
    two = 2
    y = scale(y)*two
    idx = 1:n
    param.phi = c(rep(0, p),  diag(p))
    param.prec = c(1, 0.01)
    r = inla(y ~ -1 + f(idx, model='ar', order = p), 
            family = "gaussian", 
            control.family = list(initial = 8, fixed=TRUE), 
            data = data.frame(y, idx))
    expect_true(abs(r$summary.hyperpar[1, "mode"] - 1/two^2) < 0.01)
    expect_true(all(abs(r$summary.hyperpar[-1, "mode"] - pacf) < 0.1))
})

                
                

## Export: inla.pc.rgamma inla.pc.dgamma inla.pc.qgamma inla.pc.pgamma

## ! \name{pc.gamma}
## ! \alias{inla.pc.gamma}
## ! \alias{pc.gamma}
## ! \alias{pc.rgamma}
## ! \alias{inla.pc.rgamma}
## ! \alias{pc.dgamma}
## ! \alias{inla.pc.dgamma}
## ! \alias{pc.pgamma}
## ! \alias{inla.pc.pgamma}
## ! \alias{pc.qgamma}
## ! \alias{inla.pc.qgamma}
## !
## ! \title{Utility functions for the PC prior for \code{Gamma(1/a, 1/a)}}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for \code{Gamma(1/a, 1/a)}}
## ! \usage{
## ! inla.pc.rgamma(n, lambda = 1)
## ! inla.pc.dgamma(x, lambda = 1, log = FALSE)
## ! inla.pc.qgamma(p, lambda = 1)
## ! inla.pc.pgamma(q, lambda = 1)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter (see Details)}
## !   \item{x}{Evaluation points}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## ! This gives the PC prior for the \code{Gamma(1/a, 1/a)} case,  where \code{a=0} is the
## ! base model.
## ! }
## !\value{%%
## !  \code{inla.pc.dgamma} gives the density,
## !  \code{inla.pc.pgamma} gives the distribution function,
## !  \code{inla.pc.qgamma} gives the quantile function, and
## !  \code{inla.pc.rgamma} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.gamma")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! x = inla.pc.rgamma(100,  lambda = 1)
## ! d = inla.pc.dgamma(x, lambda = 1)
## ! x = inla.pc.qgamma(0.5, lambda = 1)
## ! inla.pc.pgamma(x, lambda = 1)
## ! }

## I think this could have be done better than using the generic routines, by tabulating the d()
## function.

inla.pc.gamma.intern <- function(lambda = 1) {
    ## return a marginal-object in the log(x)-scale. the important variable is x*lambda,
    log.x <- seq(-20, 10, len = 4096) - log(lambda)
    x <- exp(log.x)
    marg <- list(x = log.x, y = inla.pc.dgamma(x, lambda = lambda, log = FALSE) * x)
    return(marg)
}

inla.pc.rgamma <- function(n, lambda = 1) {
    log.x <- inla.rmarginal(n, inla.pc.gamma.intern(lambda = lambda))
    return(exp(log.x))
}

inla.pc.dgamma <- function(x, lambda = 1, log = FALSE) {
    inv.x <- 1 / x
    d <- sqrt(2 * (log(inv.x) - psigamma(inv.x)))
    ld <- -lambda * d - log(d) + log(lambda) - 2 * log(x) +
        log(psigamma(inv.x, deriv = 1) - x)
    return(if (log) ld else exp(ld))
}

inla.pc.qgamma <- function(p, lambda = 1) {
    log.x <- inla.pmarginal(p, inla.pc.gamma.intern(lambda = lambda))
    return(exp(log.x))
}

inla.pc.pgamma <- function(q, lambda = 1) {
    p <- inla.qmarginal(log(q), inla.pc.gamma.intern(lambda = lambda))
    return(p)
}

## Internal: inla.spde.homogenise_B_matrix inla.regex.match



#' Build a block-diagonal sparse matrix.
#'
#' Build a block-diagonal sparse matrix.  Obsolete wrapper for `bdiag()`.
#'
#'
#' @aliases inla.dBind inla.dBind-deprecated
#' @param \dots A list of square or rectangular matrices.
#' @return A sparse matrix.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @keywords internal
#' @examples
#'
#' inla.dBind(Matrix(1, 2, 1), Matrix(2, 1, 2))
#' @export inla.dBind
inla.dBind <- function(...) {
    .Deprecated("Matrix::bdiag")
    return(bdiag(...))
}



#' Extract elements by matching name from container objects.
#'
#' Extract elements by wildcard name matching from a `data.frame`,
#' `list`, or `matrix`.
#'
#'
#' @aliases inla.extract.el inla.extract.el.data.frame inla.extract.el.list
#' inla.extract.el.matrix
#' @param M A container object.
#' @param match A regex defining the matching criterion.
#' @param by.row If `TRUE`, extract data by row, otherwise by column.
#' @param \dots Additional arguments, not used.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.extract.el
inla.extract.el <- function(M, ...) {
    if (is.null(M)) {
          return(NULL)
      }
    UseMethod("inla.extract.el", M)
}

inla.regex.match <- function(x, match) {
    return(strsplit(x, match)[[1]][1] == "")
}

#' @export
#' @rdname inla.extract.el
inla.extract.el.matrix <- function(M, match, by.row = TRUE, ...) {
    if (by.row) {
        return(M[sapply(rownames(M), inla.regex.match, match = match), , drop = FALSE])
    } else {
        return(M[, sapply(colnames(M), inla.regex.match, match = match), drop = FALSE])
    }
}

#' @export
#' @rdname inla.extract.el
inla.extract.el.data.frame <- function(M, match, by.row = TRUE, ...) {
    if (by.row) {
        return(M[sapply(rownames(M), inla.regex.match, match = match), , drop = FALSE])
    } else {
        return(M[, sapply(colnames(M), inla.regex.match, match = match), drop = FALSE])
    }
}

#' @export
#' @rdname inla.extract.el
inla.extract.el.list <- function(M, match, ...) {
    return(M[sapply(names(M), inla.regex.match, match = match)])
}



inla.spde.homogenise_B_matrix <- function(B, n.spde, n.theta) {
    if (!is.numeric(B)) {
          stop("B matrix must be numeric.")
      }
    if (is.matrix(B)) {
        if ((nrow(B) != 1) && (nrow(B) != n.spde)) {
            stop(inla.paste(list("B matrix has",
                as.character(nrow(B)),
                "rows but should have 1 or",
                as.character(n.spde),
                sep = " "
            )))
        }
        if ((ncol(B) != 1) && (ncol(B) != 1 + n.theta)) {
            stop(inla.paste(list("B matrix has",
                as.character(ncol(B)),
                "columns but should have 1 or",
                as.character(1 + n.theta),
                sep = " "
            )))
        }
        if (ncol(B) == 1) {
            return(cbind(as.vector(B), matrix(0.0, n.spde, n.theta)))
        } else if (ncol(B) == 1 + n.theta) {
            if (nrow(B) == 1) {
                return(matrix(as.vector(B), n.spde, 1 + n.theta, byrow = TRUE))
            } else if (nrow(B) == n.spde) {
                return(B)
            }
        }
    } else { ## !is.matrix(B)
        if ((length(B) == 1) || (length(B) == n.spde)) {
            return(cbind(B, matrix(0.0, n.spde, n.theta)))
        } else if (length(B) == 1 + n.theta) {
            return(matrix(B, n.spde, 1 + n.theta, byrow = TRUE))
        } else {
            stop(inla.paste(list(
                "Length of B vector is",
                as.character(length(B)),
                "but should be 1,",
                as.character(1 + n.theta), "or",
                as.character(n.spde)
            ),
            sep = " "
            ))
        }
    }
    stop(inla.paste(list("Unrecognised structure for B matrix"),
        sep = " "
    ))
}





#' Numerical evaluation of Matern and related covariance functions.
#'
#' Calculates covariance and correlation functions for Matern models and
#' related oscillating SPDE models, on \eqn{R^d}{R^d} and on the sphere,
#' \eqn{S^2}{S^2}.
#'
#' On \eqn{R^d}{R^d}, the models are *defined* by the spectral density
#' given by
#' \deqn{S(w) = \frac{1}{(2\pi)^d (\kappa^4 + 2 \kappa^2 \cos(\pi
#' \theta) |w|^2 + |w|^4)^{(\nu + d/2)/2}}
#' }{S(w) = 1 / ( (2\pi)^d * (kappa^4 + 2 kappa^2 * cos(pi * theta) * |w|^2 +
#' |w|^4)^((nu + d/2)/2) )}
#'
#' On \eqn{S^2}{S^2}, the models are *defined* by the spectral
#' coefficients
#' \deqn{S(k) = \frac{2k+1}{4\pi (\kappa^4 + 2 \kappa^2 \cos(\pi
#' \theta) k(k+1) + k^2(k+1)^2)^{(\nu +
#' 1)/2}}
#' }{S(k) = (2k+1) / (4 pi (kappa^4 + 2 kappa^2 cos(pi theta) k(k+1) +
#' k^2(k+1)^2)^((\nu + 1)/2) )}
#'
#' @aliases inla.matern.cov inla.matern.cov.s2
#' @param nu The Matern smoothness parameter.
#' @param kappa The spatial scale parameter.
#' @param x Distance values.
#' @param d Space dimension; the domain is \eqn{R^d}{R^d}.
#' @param corr If `TRUE`, calculate correlations, otherwise calculate
#' covariances.  Only used for pure Matern models (i.e. with
#' \eqn{\theta=0}{theta=0}).
#' @param norm.corr If `TRUE`, normalise by the estimated variance, giving
#' approximate correlations.
#' @param theta Oscillation strength parameter.
#' @param epsilon Tolerance for detecting points close to distance zero.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.matern.cov
inla.matern.cov <- function(nu, kappa, x, d = 1, corr = FALSE, norm.corr = FALSE, theta, epsilon = 1e-8) {
    if (missing(theta)) { ## Ordinary Matern
        y <- kappa * abs(x)
        if (corr) {
            ok <- (y >= epsilon)
            if (nu <= 0) {
                covariance <- y * 0
                covariance[!ok] <- 1 - y / epsilon
            } else {
                covariance <- y * 0
                covariance[ok] <-
                    2^(1 - nu) / gamma(nu) * (y[ok])^nu * besselK(y[ok], nu)
                if (any(!ok)) {
                    scale <- 2^(1 - nu) / gamma(nu)
                    ## corr = scale y^nu K_nu(y)
                    ## = 1 - b y^(2 nu) + o(y^(2 nu), 0 < \nu < 1
                    ## = 1 - b y^2 + o(y^2), 1 <= \nu
                    ## (1-corr(eps)/vari)/eps^(2nu) = b
                    if (nu < 1) {
                        exponent <- 2 * nu
                    } else {
                        exponent <- 2
                    }
                    corr.eps <-
                        scale * epsilon^nu * besselK(epsilon, nu)
                    b <- (1 - corr.eps) / epsilon^exponent
                    covariance[!ok] <- (1 - b * y[!ok]^exponent)
                }
            }
            return(covariance)
        } else {
            ok <- (y >= epsilon)
            covariance <- y * 0
            covariance[ok] <-
                2^(1 - nu) / gamma(nu + d / 2) / (4 * pi)^(d / 2) / kappa^(2 * nu) *
                    (y[ok])^nu * besselK(y[ok], nu)
            if (any(!ok)) {
                if (nu > 0) { ## Regular Matern case
                    vari <- gamma(nu) / gamma(nu + d / 2) / (4 * pi)^(d / 2) / kappa^(2 * nu)
                    scale <- 2^(1 - nu) / gamma(nu)
                    ## corr = scale y^nu K_nu(y)
                    ## = 1 - b y^(2 nu) + o(y^(2 nu), 0 < \nu < 1
                    ## = 1 - b y^2 + o(y^2), 1 <= \nu
                    ## (1-corr(eps)/vari)/eps^(2nu) = b
                    if (nu < 1) {
                        exponent <- 2 * nu
                    } else {
                        exponent <- 2
                    }
                    corr.eps <-
                        scale * epsilon^nu * besselK(epsilon, nu)
                    b <- (1 - corr.eps) / epsilon^exponent
                    covariance[!ok] <- vari * (1 - b * y[!ok]^exponent)
                } else if (nu == 0) { ## Limiting Matern case
                    g <- 0.577215664901484 ## Euler's constant
                    covariance[!ok] <-
                        2 / gamma(d / 2) / (4 * pi)^(d / 2) *
                            (-log(y[!ok] / 2) - g)
                } else { ## (nu<0)
                    ## TODO: check this...
                    covariance[!ok] <-
                        ((2^(1 - nu) / gamma(nu + d / 2) / (4 * pi)^(d / 2) / kappa^(2 * nu) *
                            gamma(nu) * 2^(nu - 1)) * (1 - (y[!ok] / epsilon)) +
                            (2^(1 - nu) / gamma(nu + d / 2) / (4 * pi)^(d / 2) / kappa^(2 * nu) *
                                epsilon^nu * besselK(epsilon, nu)) * (y[!ok] / epsilon))
                }
            }
            return(covariance)
        }
    } else { ## Oscillating covariances
        y <- abs(x)
        if (d > 2L) {
            warning("Dimension > 2 not implemented for oscillating models.")
        }
        freq.max <- 1000 / max(y)
        freq.n <- 10000
        w <- seq(0, freq.max, length.out = freq.n)
        dw <- w[2] - w[1]
        spec <- 1 / (2 * pi)^d / (kappa^4 + 2 * kappa^2 * cos(pi * theta) * w^2 + w^4)^((nu + d / 2) / 2)
        if (d == 1L) {
            covariance <- y * 0 + spec[1] * dw
        } else {
            covariance <- y * 0
        }
        for (k in 2:freq.n) {
            if (d == 1L) {
                covariance <- covariance + 2 * cos(y * w[k]) * spec[k] * dw
            } else {
                covariance <- covariance + w[k] * besselJ(y * w[k], 0) * spec[k] * dw
            }
        }

        if (norm.corr) {
            noise.variance <- 1 / covariance[1]
        } else {
            noise.variance <- 1
        }

        return(covariance * noise.variance)
    }
}

#' @export
#' @rdname inla.matern.cov
inla.matern.cov.s2 <- function(nu, kappa, x, norm.corr = FALSE, theta = 0) {
    inla.require("orthopolynom", stop.on.error = TRUE)
    y <- cos(abs(x))

    freq.max <- 40L
    freq.n <- freq.max + 1L
    w <- 0L:freq.max
    spec <- 1 / (kappa^4 + 2 * kappa^2 * cos(pi * theta) * w * (w + 1) + w^2 * (w + 1)^2)^((nu + 1) / 2)
    leg <- orthopolynom::legendre.polynomials(freq.max)
    covariance <- y * 0
    for (k in 1:freq.n) {
        covariance <- (covariance + (2 * w[k] + 1) / (4 * pi) * spec[k] *
            orthopolynom::polynomial.values(leg[k], y)[[1]])
    }

    if (norm.corr) {
        noise.variance <- 1 / covariance[1]
    } else {
        noise.variance <- 1
    }

    return(covariance * noise.variance)
}





#' List SPDE models supported by inla.spde objects
#'
#' List SPDE models supported by inla.spde objects
#'
#' Returns a list of available SPDE model type name lists, one for each
#' inla.spde model class (currently [inla.spde1()] and
#' [inla.spde2()]).
#'
#' @aliases inla.spde.models inla.spde1.models inla.spde2.models
#' @param function.names If `FALSE`, return list model name lists.  If
#' `TRUE`, return list of model object constructor function names.
#' @return List of available SPDE model type name lists.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @examples
#'
#' ## Display help for each supported inla.spde2 model:
#' for (model in inla.spde2.models()) {
#'       print(help(paste("inla.spde2.", model, sep = "")))
#'   }
#'
#' ## Display help for each supported inla.spde* model:
#' models <- inla.spde.models()
#' for (type in names(models)) {
#'       for (model in models[[type]]) {
#'             print(help(paste("inla.", type, ".", model, sep = "")))
#'         }
#'   }
#'
#' ## Display help for each supported inla.spde* model (equivalent to above):
#' for (model in inla.spde.models(function.names = TRUE)) {
#'       print(help(model))
#'   }
#' @export inla.spde.models
inla.spde.models <- function(function.names = FALSE) {
    types <- c("spde1", "spde2")
    models <- list()
    for (t in types) {
        models[[t]] <-
            do.call(
                what = paste("inla.", t, ".models", sep = ""),
                args = list()
            )
        if (function.names) {
            models[[t]] <- paste("inla.", t, ".", models[[t]], sep = "")
        }
    }

    if (function.names) {
        models <- as.vector(do.call(c, models))
    }

    return(models)
}




#' Sample from SPDE models
#'
#' Old methods fo sampling from a SPDE model.  For new code, use
#' [inla.spde.precision()] and [inla.qsample()] instead.
#'
#'
#' @aliases inla.spde.sample inla.spde.sample.default
#' inla.spde.sample.inla.spde
#' @param precision A precision matrix.
#' @param seed The seed for the pseudo-random generator.
#' @param spde An `inla.spde` object.
#' @param \dots Parameters passed on to other methods.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.precision()], [inla.qsample()]
#' @export inla.spde.sample
inla.spde.sample <- function(...) {
    warning("inla.spde.sample is deprecated.  Please use inla.qsample() in combination with inla.spde.precision() instead.")
    UseMethod("inla.spde.sample")
}

#' @export
#' @rdname inla.spde.sample
inla.spde.sample.default <-
    function(precision, seed = NULL, ...) {
        return(inla.finn(precision,
            seed = (inla.ifelse(
                is.null(seed),
                0L,
                seed
            ))
        )$sample)
    }

#' @export
#' @rdname inla.spde.sample
inla.spde.sample.inla.spde <-
    function(spde, seed = NULL, ...) {
        precision <- inla.spde.precision(spde, ...)
        return(inla.spde.sample(precision, seed = seed))
    }





#' @title Precision matrices for SPDE models
#'
#' Calculates the precision matrix for given parameter values based on an
#' `inla.spde` model object.
#'
#'
#' @aliases inla.spde.precision inla.spde1.precision inla.spde2.precision
#' inla.spde.precision.inla.spde1 inla.spde.precision.inla.spde2
#' @param spde An `inla.spde` object.
#' @param theta The parameter vector.
#' @param phi0 Internal parameter for a generic model.  Expert option only.
#' @param phi1 Internal parameter for a generic model.  Expert option only.
#' @param phi2 Internal parameter for a generic model.  Expert option only.
#' @param \dots Additional parameters passed on to other methods.
#' @return A sparse precision matrix.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.models()], [inla.spde2.generic()],
#' [inla.spde2.theta2phi0()], [inla.spde2.theta2phi1()],
#' [inla.spde2.theta2phi2()]
#' @export
#' @rdname inla.spde.precision
inla.spde.precision <- function(...) {
    UseMethod("inla.spde.precision")
}



#' SPDE result extraction from INLA estimation results
#'
#' Exctract field and parameter values and distributions for an
#' `inla.spde` SPDE effect from an inla result object.
#'
#'
#' @aliases inla.spde.result inla.spde.result.inla.spde1
#' inla.spde.result.inla.spde2 inla.spde1.result inla.spde2.result
#' @param inla An `inla` object obtained from a call to [inla()]
#' @param name A character string with the name of the SPDE effect in the inla
#' formula.
#' @param spde The `inla.spde` object used for the effect in the inla
#' formula. (Note: this could have been stored in the inla output, but isn't.)
#' Usually the result of a call to [inla.spde2.matern()].
#' @param do.transform If `TRUE`, also calculate marginals transformed to
#' user-scale.  Setting to `FALSE` is useful for large non-stationary
#' models, as transforming many marginal densities is time-consuming.
#' @param \dots Further arguments passed to and from other methods.
#' @return For `inla.spde2` models, a list, where the nominal range and
#' variance are defined as the values that would have been obtained with a
#' stationary model and no boundary effects: \item{marginals.kappa }{Marginal
#' densities for kappa} \item{marginals.log.kappa }{Marginal densities for
#' log(kappa)} \item{marginals.log.range.nominal }{Marginal densities for
#' log(range)} \item{marginals.log.tau }{Marginal densities for log(tau)}
#' \item{marginals.log.variance.nominal }{Marginal densities for log(variance)}
#' \item{marginals.range.nominal }{Marginal densities for range}
#' \item{marginals.tau }{Marginal densities for tau} \item{marginals.theta
#' }{Marginal densities for the theta parameters} \item{marginals.values
#' }{Marginal densities for the field values} \item{marginals.variance.nominal
#' }{Marginal densities for variance} \item{summary.hyperpar }{The SPDE related
#' part of the inla hyperpar output summary} \item{summary.log.kappa }{Summary
#' statistics for log(kappa)} \item{summary.log.range.nominal }{Summary
#' statistics for log(range)} \item{summary.log.tau }{Summary statistics for
#' log(tau)} \item{summary.log.variance.nominal }{Summary statistics for
#' log(kappa)} \item{summary.theta }{Summary statistics for the theta
#' parameters} \item{summary.values }{Summary statistics for the field values}
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.models()], [inla.spde2.matern()]
#' @examples
#'
#' loc <- matrix(runif(100 * 2), 100, 2)
#' mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(0.1, 0.5))
#' spde <- inla.spde2.matern(mesh)
#' index <- inla.spde.make.index("spatial", mesh$n, n.repl = 2)
#' spatial.A <- inla.spde.make.A(mesh, loc,
#'     index = rep(1:nrow(loc), 2),
#'     repl = rep(1:2, each = nrow(loc))
#' )
#' ## Toy example with no spatial correlation (range=zero)
#' y <- 10 + rnorm(100 * 2)
#' stack <- inla.stack(
#'     data = list(y = y),
#'     A = list(spatial.A),
#'     effects = list(c(index, list(intercept = 1))),
#'     tag = "tag"
#' )
#' data <- inla.stack.data(stack, spde = spde)
#' formula <- y ~ -1 + intercept + f(spatial,
#'     model = spde,
#'     replicate = spatial.repl
#' )
#' result <- inla(formula,
#'     family = "gaussian", data = data,
#'     control.predictor = list(A = inla.stack.A(stack))
#' )
#' spde.result <- inla.spde.result(result, "spatial", spde)
#' plot(spde.result$marginals.range.nominal[[1]], type = "l")
#' @export inla.spde.result
inla.spde.result <- function(...) {
    inla.require.inherits(list(...)[[1]], "inla", "First parameter")
    inla.require.inherits(list(...)[[2]], "character", "Second parameter")
    UseMethod("inla.spde.result", list(...)[[3]])
}









#' SPDE model index vector generation
#'
#' Generates a list of named index vectors for an SPDE model.
#'
#'
#' @param name A character string with the base name of the effect.
#' @param n.spde The size of the model, typically from `spde$n.spde`.
#' @param n.group The size of the `group` model.
#' @param n.repl The number of model replicates.
#' @param ...  Additional parameters.  Currently unused.
#' @return A list of named index vectors. \item{name }{Indices into the vector
#' of latent variables} \item{name.group }{'group' indices} \item{name.repl
#' }{Indices for replicates}
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.make.A()], [inla.spde2.result()]
#' @examples
#'
#' loc <- matrix(runif(100 * 2), 100, 2)
#' mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(0.1, 0.5))
#' spde <- inla.spde2.matern(mesh)
#' index <- inla.spde.make.index("spatial", spde$n.spde, n.repl = 2)
#' spatial.A <- inla.spde.make.A(mesh, loc,
#'     index = rep(1:nrow(loc), 2),
#'     repl = rep(1:2, each = nrow(loc))
#' )
#' y <- 10 + rnorm(100 * 2)
#' stack <- inla.stack(
#'     data = list(y = y),
#'     A = list(spatial.A),
#'     effects = list(c(index, list(intercept = 1))),
#'     tag = "tag"
#' )
#' data <- inla.stack.data(stack, spde = spde)
#' formula <- y ~ -1 + intercept + f(spatial,
#'     model = spde,
#'     replicate = spatial.repl
#' )
#' result <- inla(formula,
#'     family = "gaussian", data = data,
#'     control.predictor = list(A = inla.stack.A(stack))
#' )
#' spde.result <- inla.spde2.result(result, "spatial", spde)
#' @export inla.spde.make.index
inla.spde.make.index <- function(name, n.spde, n.group = 1, n.repl = 1, ...) {
    if ("n.mesh" %in% names(list(...))) {
        warning("'n.mesh' is deprecated, please use 'n.spde' instead.")
        if (missing(n.spde) || is.null(n.spde)) {
              n.spde <- list(...)$n.mesh
          }
    }

    name.group <- paste(name, ".group", sep = "")
    name.repl <- paste(name, ".repl", sep = "")
    out <- list()
    out[[name]] <- rep(rep(1:n.spde, times = n.group), times = n.repl)
    out[[name.group]] <- rep(rep(1:n.group, each = n.spde), times = n.repl)
    out[[name.repl]] <- rep(1:n.repl, each = n.spde * n.group)
    return(out)
}







#' Row-wise Kronecker products
#'
#' Takes two Matrices and computes the row-wise Kronecker product.  Optionally
#' applies row-wise weights and/or applies an additional 0/1 row-wise Kronecker
#' matrix product, as needed by [inla.spde.make.A()].
#'
#'
#' @param M1 A matrix that can be transformed into a sparse Matrix.
#' @param M2 A matrix that can be transformed into a sparse Matrix.
#' @param repl An optional index vector.  For each entry, specifies which
#' replicate the row belongs to, in the sense used in
#' [inla.spde.make.A()].
#' @param n.repl The maximum replicate index, in the sense used in
#' [inla.spde.make.A()].
#' @param weights Optional scaling weights to be applied row-wise to the
#' resulting matrix.
#' @return A `sparseMatrix` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.make.A()]
#' @export inla.row.kron
inla.row.kron <- function(M1, M2, repl = NULL, n.repl = NULL, weights = NULL) {
    M1 <- inla.as.dgTMatrix(M1)
    M2 <- inla.as.dgTMatrix(M2)
    n <- nrow(M1)
    if (is.null(repl)) {
        repl <- rep(1L, n)
    }
    if (is.null(n.repl)) {
        n.repl <- max(repl)
    }
    if (is.null(weights)) {
        weights <- rep(1, n)
    } else if (length(weights) == 1L) {
        weights <- rep(weights[1], n)
    }

    if (FALSE) {
        ## Slow version:
        print(system.time({
            M <- (sparseMatrix(
                i = numeric(0), j = numeric(0), x = integer(0),
                dims = c(n, ncol(M1) * ncol(M2))
            ))
            for (k in seq_len(n)) {
                M[k, ] <- kronecker(M1[k, , drop = FALSE], M2[k, , drop = FALSE])
            }
            M <- inla.as.dgTMatrix(M)
            weights.ii <- weights[1L + M@i]
            M <- (sparseMatrix(
                i = (1L + M@i),
                j = (1L + M@j + ncol(M) * (repl[M@i + 1L] - 1L)),
                x = weights.ii * M@x,
                dims = c(n, n.repl * ncol(M))
            ))
        }))
        M.slow <- M
    }

    ## Fast version:
    ## TODO: Check robustness for all-zero rows.
    ## TODO: Maybe move big sparseMatrix call outside the loop.
    ## TODO: Automatically choose M1 or M2 for looping.

    ## print(system.time({
    n1 <- (as.vector(sparseMatrix(
        i = 1L + M1@i, j = rep(1L, length(M1@i)),
        x = 1L, dims = c(n, 1)
    )))
    n2 <- (as.vector(sparseMatrix(
        i = 1L + M2@i, j = rep(1L, length(M2@i)),
        x = 1L, dims = c(n, 1)
    )))

    M <- (sparseMatrix(
        i = integer(0), j = integer(0), x = numeric(0),
        dims = c(n, ncol(M2) * ncol(M1) * n.repl)
    ))
    n1 <- n1[1L + M1@i]
    for (k in unique(n1)) {
        sub <- which(n1 == k)
        n.sub <- length(sub)

        i.sub <- 1L + M1@i[sub]
        j.sub <- 1L + M1@j[sub]
        o1 <- order(i.sub, j.sub)
        jj <- rep(seq_len(k), times = n.sub / k)

        i.sub <- i.sub[o1]
        j.sub <- (sparseMatrix(
            i = i.sub,
            j = jj,
            x = j.sub[o1],
            dims = c(n, k)
        ))
        x.sub <- (sparseMatrix(
            i = i.sub,
            j = jj,
            x = weights[i.sub] * M1@x[sub][o1],
            dims = c(n, k)
        ))
        sub2 <- which(is.element(1L + M2@i, i.sub))

        if (length(sub2) > 0) {
            i <- 1L + M2@i[sub2]
            ii <- rep(i, times = k)
            repl.i <- repl[ii]

            M <- (M +
                sparseMatrix(
                    i = ii,
                    j = (1L + rep(M2@j[sub2], times = k) +
                        ncol(M2) * (as.vector(j.sub[i, ]) - 1L) +
                        ncol(M2) * ncol(M1) * (repl.i - 1L)),
                    x = (rep(M2@x[sub2], times = k) *
                        as.vector(x.sub[i, ])),
                    dims = c(n, ncol(M2) * ncol(M1) * n.repl)
                ))
        }
    }
    ## }))

    ## For debugging:
    ## print(max(abs(M-M.slow)))

    ## o2 = order(n2[1L+M2@i], M2@i, M2@j)

    return(M)
}






#' Observation matrices for mesh models.
#'
#' Constructs observation/prediction weight matrices for numerical integration
#' schemes for regional data problems.  Primarily intended for internal use by
#' [inla.spde.make.A()].
#'
#'
#' @param A A precomputed observation/prediction matrix for locations that are
#' to be joined.
#' @param block Indices specifying block groupings: Entries with the same
#' `block` value are joined into a single row in the resulting matrix, and
#' the `block` values are the row indices.
#' @param n.block The number of blocks.
#' @param weights Optional scaling weights to be applied row-wise to the input
#' `A` matrix.
#' @param rescale Specifies what scaling method should be used when joining the
#' rows of the `A` matrix as grouped by the `block` specification.
#' \itemize{ \item `'none'`: Straight sum, no rescaling.  \item
#' `'count'`: Divide by the number of entries in the block.  \item
#' `'weights'`: Divide by the sum of the `weight` values within each
#' block.  \item `'sum'`: Divide by the resulting row sums.  }
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.make.A()]
#' @export inla.spde.make.block.A
inla.spde.make.block.A <-
    function(A,
             block,
             n.block = max(block),
             weights = NULL,
             rescale = c("none", "count", "weights", "sum")) {
        ## Add A-matrix rows belonging to the same "block" with optional
        ## weights and optional rescaling, by dividing row i by a_i, for |a_i|>0:
        ## B(i) = {j; block(i)==block(j) and sum_k |A_jk| > 0 }
        ## "count":   a_i = #{j in B(i)} }
        ## "weights": a_i = \sum_{j in B(i)} weights_j
        ## "sum":     a_i = \sum_{j in B(i)} \sum_k A_jk weights_j
        ##
        ## This function makes use of the feature of sparseMatrix to sum all
        ## values for multiple instances of the same (i,j) pair.

        A <- inla.as.dgTMatrix(A)
        N <- nrow(A)
        ## length(block) should be == N or 1
        if (length(block) == 1L) {
            block <- rep(block, N)
        }
        if (is.null(weights)) {
            weights <- rep(1, N)
        }

        rescale <- match.arg(rescale)

        if (!(rescale == "none")) {
            if (rescale == "count") {
                ## Count the non-zero rows within each block
                sums <- (sparseMatrix(
                    i = block,
                    j = rep(1L, N),
                    x = (rowSums(abs(A)) > 0) * 1.0,
                    dims = c(n.block, 1L)
                ))[block]
            } else if (rescale == "weights") {
                ## Sum the weights within each block
                sums <- (sparseMatrix(
                    i = block,
                    j = rep(1L, N),
                    x = (rowSums(abs(A)) > 0) * weights,
                    dims = c(n.block, 1L)
                ))[block]
            } else { ## (rescale == "sum"){
                ## Sum the weighted values within each block
                sums <- (sparseMatrix(
                    i = block,
                    j = rep(1L, N),
                    x = rowSums(A) * weights,
                    dims = c(n.block, 1L)
                ))[block]
            }
            ## Normalise:
            ok <- (abs(sums) > 0)
            weights[ok] <- weights[ok] / sums[ok]
        }

        return(inla.as.dgTMatrix(sparseMatrix(
            i = block[1L + A@i],
            j = 1L + A@j,
            x = A@x * weights[1L + A@i],
            dims = c(n.block, ncol(A))
        )))
    }





#' Observation/prediction matrices for mesh models.
#'
#' Constructs observation/prediction weight matrices for models based on
#' [inla.mesh()] and [inla.mesh.1d()] objects.
#'
#'
#' @param mesh An [inla.mesh()] or [inla.mesh.1d()] object
#' specifying a function basis on a mesh domain.  Alternatively, an
#' `inla.spde` object that includes a mesh (e.g. from
#' [inla.spde2.matern()]).
#' @param loc Observation/prediction coordinates.  `mesh` and `loc`
#' defines a matrix `A.loc` of mapping weights between basis function
#' weights and field values.  If `loc` is `NULL`, `A.loc` is
#' defined as `Diagonal(n.spde, 1)`.
#' @param index For each observation/prediction value, an index into
#' `loc`.  Default is `seq_len(nrow(A.loc))`.
#' @param group For each observation/prediction value, an index into the group
#' model.
#' @param repl For each observation/prediction value, the replicate index.
#' @param n.spde The number of basis functions in the mesh model. (Note: may be
#' different than the number of mesh vertices/nodes/knots.)
#' @param n.group The size of the group model.
#' @param n.repl The total number of replicates.
#' @param group.mesh An optional [inla.mesh.1d()] object for the
#' group model.
#' @param weights Optional scaling weights to be applied row-wise to the
#' resulting matrix.
#' @param A.loc Optional precomputed observation/prediction matrix.
#' `A.loc` can be specified instead of `mesh`+`loc`, optionally
#' with `index` supplied.
#' @param A.group Optional precomputed observation/prediction matrix for the
#' group model. `A.group` can be specified instead of `group` and/or
#' `group.mesh`, optionally with `group.index` supplied.
#' @param group.index For each observation/prediction value, an index into the
#' rows of `A.group`.
#' @param block Optional indices specifying block groupings: Entries with the
#' same `block` value are joined into a single row in the resulting
#' matrix, and the `block` values are the row indices.  This is intended
#' for construction of approximate integration schemes for regional data
#' problems. See [inla.spde.make.block.A()] for details.
#' @param n.block The number of blocks.
#' @param block.rescale Specifies what scaling method should be used when
#' joining entries as grouped by a `block` specification.  See
#' [inla.spde.make.block.A()] for details.
#' @param \dots Additional parameters.  Currently unused.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.spde.make.index()]
#' @examples
#'
#' loc <- matrix(runif(10000 * 2) * 1000, 10000, 2)
#' mesh <- inla.mesh.2d(
#'     loc = loc,
#'     cutoff = 50,
#'     max.edge = c(50, 500)
#' )
#' A <- inla.spde.make.A(mesh, loc = loc)
#' @export inla.spde.make.A
inla.spde.make.A <-
    function(mesh = NULL,
             loc = NULL,
             index = NULL,
             group = NULL,
             repl = 1L,
             n.spde = NULL,
             n.group = NULL,
             n.repl = NULL,
             group.mesh = NULL,
             weights = NULL,
             A.loc = NULL,
             A.group = NULL,
             group.index = NULL,
             block = NULL,
             n.block = NULL,
             block.rescale = c("none", "count", "weights", "sum"),
             ...)
    ## Deprecated/obsolete parameters: n.mesh, group.method
    {
        ## A.loc can be specified instead of mesh+loc, optionally with
        ## index supplied.
        ## A.group can be specified instead of group and/or group.mesh,
        ## optionally with group.index supplied.

        if ("n.mesh" %in% names(list(...))) {
            warning("'n.mesh' is deprecated, use 'n.spde' instead.")
            n.spde <- list(...)$n.mesh
        }
        if ("group.method" %in% names(list(...))) {
            group.method <-
                match.arg(list(...)$group.method, c("nearest", "S0", "S1"))
            warning(paste("'group.method=", group.method,
                "' is deprecated.  Specify 'degree=",
                switch(group.method, nearest = "0", S0 = "0", S1 = "1"),
                "' in inla.mesh.1d() instead.",
                sep = ""
            ))
        }

        if (is.null(mesh)) {
            if (is.null(A.loc) && is.null(n.spde)) {
                  stop("At least one of 'mesh', 'n.spde', and 'A.loc' must be specified.")
              }
            if (!is.null(A.loc)) {
                n.spde <- ncol(A.loc)
            }
        } else if (inherits(mesh, "inla.spde")) {
            spde <- mesh
            mesh <- spde$mesh
            inla.require.inherits(spde$mesh, c("inla.mesh", "inla.mesh.1d"), "'spde$mesh'")
            n.spde <- spde$n.spde
        } else {
            inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
            if (inherits(mesh, "inla.mesh.1d")) {
                n.spde <- mesh$m
            } else {
                n.spde <- mesh$n
            }
        }
        if (!is.null(group.mesh)) {
            inla.require.inherits(group.mesh, "inla.mesh.1d", "'mesh'")
        }

        n.group <-
            ifelse(!is.null(n.group),
                n.group,
                ifelse(!is.null(A.group),
                    nrow(A.group),
                    ifelse(!is.null(group.mesh),
                        group.mesh$m,
                        max(
                            1,
                            ifelse(is.null(group),
                                1,
                                ifelse(length(group) == 0,
                                    1,
                                    max(group)
                                )
                            )
                        )
                    )
                )
            )
        n.repl <-
            ifelse(!is.null(n.repl),
                n.repl,
                max(1, ifelse(is.null(repl),
                    1,
                    ifelse(length(repl) == 0,
                        1,
                        max(repl)
                    )
                ))
            )


        ## Handle loc and index input semantics:
        if (is.null(loc)) {
            if (is.null(A.loc)) {
                A.loc <- Diagonal(n.spde, 1)
            }
        } else {
            if (is.null(mesh)) {
                  stop("'loc' specified but 'mesh' is NULL.")
              }

            ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
            if (inherits(loc, "SpatialPoints") ||
                inherits(loc, "SpatialPointsDataFrame")) {
                loc <- inla.spTransform(coordinates(loc),
                    inla.sp_get_crs(loc),
                    mesh$crs,
                    passthrough = TRUE
                )
            }

            A.loc <- inla.mesh.project(mesh, loc = loc)$A
        }
        if (is.null(index)) {
            index <- seq_len(nrow(A.loc))
        }
        ## Now 'index' points into the rows of 'A.loc'

        if (is.null(n.block)) {
            n.block <- ifelse(is.null(block), length(index), max(block))
        }
        block.rescale <- match.arg(block.rescale)

        ## Handle group semantics:
        ## TODO: FIXME!!! group, group.index, group.mesh, A.group, etc
        if (!is.null(A.group)) {
            if (!is.null(group) || !is.null(group.mesh)) {
                warning("'A.group' has been specified; ignoring non-NULL 'group' or 'group.mesh'.")
            }
        } else if (!is.null(group.mesh)) {
            if (is.null(group)) {
                group <- rep(group.mesh$mid[1], length(index))
            }
        } else if (is.null(group)) {
            group <- rep(1L, length(index))
        } else if (length(group) == 1) {
            group <- rep(group, length(index))
        }
        if (is.null(group.index)) {
            group.index <- seq_len(length(group))
        }
        ## Now 'group.index' points into the rows of 'A.group' or 'group'
        if (length(group.index) != length(index)) {
            stop(paste("length(group.index) != length(index): ",
                length(group.index), " != ", length(index),
                sep = ""
            ))
        }

        if (!is.null(group.mesh) && is.null(A.group)) {
            A.group <- inla.mesh.1d.A(group.mesh, loc = group)
        }
        ## Now 'group.index' points into the rows of 'A.group' or 'group'

        ## Handle repl semantics:
        if (is.null(repl)) {
            repl <- rep(1, length(index))
        } else if (length(repl) == 1) {
            repl <- rep(repl, length(index))
        } else if (length(repl) != length(index)) {
            stop(paste("length(repl) != length(index): ",
                length(repl), " != ", length(index),
                sep = ""
            ))
        }

        if (length(index) > 0L) {
            A.loc <- inla.as.dgTMatrix(A.loc[index, , drop = FALSE])

            if (length(A.loc@i) > 0L) {
                if (is.null(weights)) {
                    weights <- rep(1, length(index))
                } else if (length(weights) == 1L) {
                    weights <- rep(weights[1], length(index))
                }
                if (!is.null(block)) {
                    ## Leave the rescaling until the block phase,
                    ## so that the proper rescaling can be determined.
                    block.weights <- weights
                    weights <- rep(1, length(index))
                }

                if (!is.null(A.group)) {
                    A.group <- inla.as.dgTMatrix(A.group[group.index, , drop = FALSE])
                    A <- (inla.row.kron(A.group, A.loc,
                        repl = repl, n.repl = n.repl,
                        weights = weights
                    ))
                    ## More general version:
                    ## A = inla.row.kron(A.repl,
                    ## inla.row.kron(A.group, A.loc),
                    ## weights=weights))
                } else {
                    i <- 1L + A.loc@i
                    group.i <- group[group.index[i]]
                    repl.i <- repl[i]
                    weights.i <- weights[i]
                    A <- (sparseMatrix(
                        i = i,
                        j = (1L + A.loc@j +
                            n.spde * (group.i - 1L) +
                            n.spde * n.group * (repl.i - 1L)),
                        x = weights.i * A.loc@x,
                        dims = (c(
                            length(index),
                            n.spde * n.group * n.repl
                        ))
                    ))
                }
                if (!is.null(block)) {
                    A <- (inla.spde.make.block.A(
                        A = A,
                        block = block,
                        n.block = n.block,
                        weights = block.weights,
                        rescale = block.rescale
                    ))
                }
            } else {
                A <- (sparseMatrix(
                    i = integer(0),
                    j = integer(0),
                    x = numeric(0),
                    dims = c(n.block, n.spde * n.group * n.repl)
                ))
            }
        } else {
            A <- (sparseMatrix(
                i = integer(0),
                j = integer(0),
                x = numeric(0),
                dims = c(0L, n.spde * n.group * n.repl)
            ))
        }
        return(A)
    }











# Internal function for merging raw stack information
rbind.inla.data.stack.info <- function(...) {
    l <- list(...)
    names(l) <- NULL
    names.tmp <- do.call(c, lapply(l, function(x) x$names))
    ncol.tmp <- do.call(c, lapply(l, function(x) x$ncol))

    ncol <- c()
    names <- list()
    for (k in 1:length(names.tmp)) {
        name <- names(names.tmp)[k]
        if (!is.null(names[[name]])) {
            if (!identical(
                names[[name]],
                names.tmp[[k]]
            )) {
                stop("Name mismatch.")
            }
        }
        names[[name]] <- names.tmp[[k]]

        if (!is.null(as.list(ncol)[[name]])) {
            if (ncol[name] != ncol.tmp[[k]]) {
                stop("ncol mismatch.")
            }
        }
        ncol[name] <- ncol.tmp[[k]]
    }

    external.names <- names(names)
    internal.names <- do.call(c, names)

    factors <- rep(FALSE, length(internal.names))
    names(factors) <- internal.names
    factor.names <-
        lapply(l, function(x) {
            do.call(
                c,
                x$names[do.call(
                    c,
                    lapply(
                        x$data,
                        is.factor
                    )
                )]
            )
        })
    for (factor.loop in seq_along(l)) {
        factors[factor.names[[factor.loop]]] <- TRUE
    }

    handle.missing.columns <- function(x) {
        missing.names <-
            setdiff(
                internal.names,
                do.call(c, x$names)
            )
        if (length(missing.names) > 0) {
            df <- c(
                rep(
                    list(rep(NA, x$nrow)),
                    sum(!factors[missing.names])
                ),
                rep(
                    list(rep(as.factor(NA), x$nrow)),
                    sum(factors[missing.names])
                )
            )
            names(df) <- c(
                missing.names[!factors[missing.names]],
                missing.names[factors[missing.names]]
            )
            df <- as.data.frame(df)
            return(cbind(x$data, df))
        } else {
            return(x$data)
        }
    }

    data <- do.call(rbind, lapply(l, handle.missing.columns))

    offset <- 0
    index <- list()
    for (k in 1:length(l)) {
        for (j in 1:length(l[[k]]$index)) {
            if (is.null(index[[names(l[[k]]$index)[j]]])) {
                index[[names(l[[k]]$index)[j]]] <- l[[k]]$index[[j]] + offset
            } else {
                index[[names(l[[k]]$index)[j]]] <-
                    c(
                        index[[names(l[[k]]$index)[j]]],
                        l[[k]]$index[[j]] + offset
                    )
            }
        }
        offset <- offset + l[[k]]$nrow
    }

    info <-
        list(
            data = data,
            nrow = nrow(data),
            ncol = ncol,
            names = names,
            index = index
        )
    class(info) <- "inla.data.stack.info"

    return(info)
}

#' @describeIn inla.stack Remove unused entries from an existing stack
#'
#' @export
inla.stack.remove.unused <- function(stack) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    if (stack$effects$nrow < 2) {
        return(stack)
    }

    ## Remove components with no effect:
    remove <- rep(FALSE, stack$effects$nrow)
    remove.unused.indices <-
        which(colSums(abs(stack$A[, , drop = FALSE])) == 0)
    remove[remove.unused.indices] <- TRUE

    index.new <- rep(as.integer(NA), stack$effect$nrow)

    ncol.A <- sum(!remove)
    if (ncol.A > 0) {
          index.new[!remove] <- 1:ncol.A
      }
    index.new[remove] <- index.new[index.new[remove]]

    for (k in 1:length(stack$effects$index)) {
        stack$effects$index[[k]] <- index.new[stack$effects$index[[k]]]
    }

    A <- inla.as.dgTMatrix(stack$A)
    j.new <- index.new[A@j + 1L]
    ## Check for any zero-elements in remove.unused-columns:
    ok <- !is.na(j.new)
    stack$A <-
        sparseMatrix(
            i = A@i[ok] + 1L,
            j = j.new[ok],
            x = A@x[ok],
            dims = c(nrow(A), ncol.A)
        )

    stack$effects$data <- stack$effects$data[!remove, , drop = FALSE]
    stack$effects$nrow <- ncol.A

    return(stack)
}

#' @describeIn inla.stack Compress an existing stack by removing duplicates
#'
#' @export
inla.stack.compress <- function(stack, remove.unused = TRUE) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    if (stack$effects$nrow < 2) {
        return(stack)
    }

    ii <- do.call(order, as.list(stack$effects$data))
    jj.dupl <-
        which(1L ==
            diff(c(
                duplicated(stack$effects$data[ii, , drop = FALSE]),
                FALSE
            )))
    kk.dupl <-
        which(-1L ==
            diff(c(
                duplicated(stack$effects$data[ii, , drop = FALSE]),
                FALSE
            )))
    ## ii[jj.dupl] are the rows that have duplicates.
    ## ii[(jj.dupl[k]+1):kk.dupl[k]] are the duplicate rows for each k

    remove <- rep(FALSE, stack$effects$nrow)
    index.new <- rep(as.integer(NA), stack$effect$nrow)

    if (length(jj.dupl) > 0) {
        for (k in 1:length(jj.dupl)) {
            i <- ii[jj.dupl[k]]
            j <- ii[(jj.dupl[k] + 1):kk.dupl[k]]

            remove[j] <- TRUE
            index.new[j] <- i
        }
    }

    ncol.A <- sum(!remove)
    if (ncol.A > 0) {
          index.new[!remove] <- 1:ncol.A
      }
    index.new[remove] <- index.new[index.new[remove]]

    for (k in 1:length(stack$effects$index)) {
        stack$effects$index[[k]] <- index.new[stack$effects$index[[k]]]
    }

    A <- inla.as.dgTMatrix(stack$A)
    j.new <- index.new[A@j + 1L]
    ## Check for any zero-elements in remove.unused-columns:
    ok <- !is.na(j.new)
    stack$A <-
        sparseMatrix(
            i = A@i[ok] + 1L,
            j = j.new[ok],
            x = A@x[ok],
            dims = c(nrow(A), ncol.A)
        )

    stack$effects$data <- stack$effects$data[!remove, , drop = FALSE]
    stack$effects$nrow <- ncol.A

    if (remove.unused) {
        return(inla.stack.remove.unused(stack))
    } else {
        return(stack)
    }
}





#' Data stacking for advanced INLA models
#'
#' Functions for combining data, effects and observation matrices into
#' `inla.stack` objects, and extracting information from such objects.
#'
#' For models with a single effects collection, the outer list container for
#' `A` and `effects` may be omitted.
#'
#' Component size definitions:
#' * \eqn{n_l}{n_l} effect blocks
#' * \eqn{n_k}{n_k} effects
#' * \eqn{n_i}{n_i} data values
#' * \eqn{n_{j,l}}{n_jl} effect size for block \eqn{l}{l}
#' * \eqn{n_j}{n_j} \eqn{= \sum_{l=1}^{n_l} n_{j,l}}{sum_l n_jl} total
#' effect size
#'
#' Input: \describe{
#' \item{`data`}{\eqn{(y^1, \ldots, y^p)}{(y1,\dots,y2)} \eqn{p}{p}
#' vectors, each of length \eqn{n_i}{n_i}}
#' \item{`A`}{\eqn{(A^1, \ldots, A^{n_l})}{(A1,\dots,A2)} matrices of size
#' \eqn{n_i \times n_{j,l}}{n_i by n_jl}}
#' \item{`effects`}{\eqn{\left((x^{1,1},\ldots,x^{n_k,1}), \ldots,
#' (x^{1,n_l},\ldots,x^{n_k,n_l})\right)}{((x_[1,1],\dots,x_[n_k,1]),\dots(x_[1,n_l],\dots,x_[n_k,n_l]))}
#' collections of effect
#' vectors of length \eqn{n_{j,l}}{n_jl} }
#' }
#'
#' \deqn{\mbox{predictor}(y^1, \ldots, y^p) \sim
#' \sum_{l=1}^{n_l} A^l \sum_{k=1}^{n_k} g(k, x^{k,l})
#' = \tilde{A} \sum_{k=1}^{n_k} g(k, \tilde{x}^k)
#' }{ predictor(y^1, \ldots, y^p) ~ sum_{l=1}^{n_l} A^l sum_{k=1}^{n_k}
#' g(k, x^{k,l}) = tilde{A} sum_{k=1}^{n_k} g(k, tilde{x}^k) }
#' where
#' \deqn{\tilde{A} = \mbox{cbind}\left( A^1, \ldots, A^{n_l} \right)
#' }{ tilde{A} = cbind( A^1, ..., A^{n_l} ) }
#' and
#' \deqn{\tilde{x}^k = \mbox{rbind}\left( x^{k,1}, \ldots, x^{k,n_l} \right)
#' }{ tilde{x}^k = rbind( x^{k,1}, ..., x^{k,n_l} ) }
#' and for each block \eqn{l}{l}, any missing
#' \eqn{x^{k,l}} is replaced by an `NA` vector.
#'
#' @aliases inla.stack inla.stack.remove.unused inla.stack.compress
#' inla.stack.sum inla.stack.join inla.stack.index inla.stack.LHS
#' inla.stack.RHS inla.stack.data inla.stack.A
#' @param stack A `inla.data.stack` object, created by a call to
#' `inla.stack`, `inla.stack.sum`, or `inla.stack.join`.
#' @param remove.unused If `TRUE`, compress the model by removing rows of
#' effects corresponding to all-zero columns in the `A` matrix (and
#' removing those columns).
#' @param ... For `inla.stack.join`, two or more data stacks of class
#' `inla.data.stack`, created by a call to `inla.stack`,
#' `inla.stack.sum`, or `inla.stack.join`. For
#' `inla.stack.data`, a list of variables to be joined with the data list.
#' @param compress If `TRUE`, compress the model by removing duplicated
#' rows of effects, replacing the corresponding A-matrix columns with a single
#' column containing the sum.
#' @param data A list or codedata.frame of named data vectors. Scalars are
#' expanded to match the number of rows in the A matrices, or any non-scalar
#' data vectors. An error is given if the input is inconsistent.
#' @param A A list of observation matrices. Scalars are expanded to diagonal
#' matrices matching the effect vector lengths. An error is given if the input
#' is inconsistent or ambiguous.
#' @param effects A collection of effects/predictors.  Each list element
#' corresponds to an observation matrix, and must either be a single vector, a
#' list of vectors, or a `data.frame`. Single-element effect vectors are
#' expanded to vectors matching the number of columns in the corresponding A
#' matrix.  An error is given if the input is inconsistent or ombiguous.
#' @param tag A string specifying a tag for later identification.
#' @return A data stack of class `inla.data.stack`.
#' Elements: \itemize{
#' \item`data` \eqn{=(y^1, \ldots, y^p, \tilde{x}^1, \ldots,
#' \tilde{x}^{n_k})}{=(y^1, \dots, y^p, tilde{x}^1, \dots,
#' tilde{x}^{n_k})}
#' \item`A` \eqn{=\tilde{A}}{=tilde{A}}
#' \item`data.names` List
#' of data names, length \eqn{p}
#' \item`effect.names` List of effect names,
#' length \eqn{n_k}
#' \item`n.data` Data length, \eqn{n_i}
#' \item`index`
#' List indexed by `tag`s, each element indexing into \eqn{i=1, \ldots,
#' n_i} }
#' @section Functions: \itemize{
#' \item `inla.stack.remove.unused`: Remove
#' unused entries from an existing stack
#'
#' \item `inla.stack.compress`: Compress an existing stack by removing
#' duplicates
#'
#' \item `inla.stack`: Shorthand for inla.stack.join and inla.stack.sum
#'
#' \item `inla.stack.sum`: Create data stack as a sum of predictors
#'
#' \item `inla.stack.join`: Join two or more data stacks
#'
#' \item `inla.stack.index`: Extract tagged indices
#'
#' \item `inla.stack.LHS`: Extract data associated with the "left hand
#' side" of the model (e.g. the data itself, `Ntrials`, `link`,
#' `E`)
#'
#' \item `inla.stack.RHS`: Extract data associated with the "right hand
#' side" of the model (all the covariates/predictors)
#'
#' \item `inla.stack.data`: Extract data for an inla call, and optionally
#' join with other variables
#'
#' \item `inla.stack.A`: Extract the "A matrix" for control.predictor }
#' @seealso [inla.spde.make.A()], [inla.spde.make.index()]
#' @keywords fmesher
#' @examples
#'
#' n <- 200
#' loc <- matrix(runif(n * 2), n, 2)
#' mesh <- inla.mesh.2d(
#'     loc.domain = loc,
#'     max.edge = c(0.05, 0.2)
#' )
#' proj.obs <- inla.mesh.projector(mesh, loc = loc)
#' proj.pred <- inla.mesh.projector(mesh, loc = mesh$loc)
#' spde <- inla.spde2.pcmatern(mesh,
#'     prior.range = c(0.01, 0.01),
#'     prior.sigma = c(10, 0.01)
#' )
#'
#' covar <- rnorm(n)
#' field <- inla.qsample(n = 1, Q = inla.spde.precision(spde, theta = log(c(0.5, 1))))[, 1]
#' y <- 2 * covar + inla.mesh.project(proj.obs, field)
#'
#' A.obs <- inla.spde.make.A(mesh, loc = loc)
#' A.pred <- inla.spde.make.A(mesh, loc = proj.pred$loc)
#' stack.obs <-
#'     inla.stack(
#'         data = list(y = y),
#'         A = list(A.obs, 1),
#'         effects = list(c(
#'             list(Intercept = 1),
#'             inla.spde.make.index("spatial", spde$n.spde)
#'         ),
#'         covar = covar
#'         ),
#'         tag = "obs"
#'     )
#' stack.pred <-
#'     inla.stack(
#'         data = list(y = NA),
#'         A = list(A.pred),
#'         effects = list(c(
#'             list(Intercept = 1),
#'             inla.spde.make.index("spatial", mesh$n)
#'         )),
#'         tag = "pred"
#'     )
#' stack <- inla.stack(stack.obs, stack.pred)
#'
#' formula <- y ~ -1 + Intercept + covar + f(spatial, model = spde)
#' result1 <- inla(formula,
#'     data = inla.stack.data(stack.obs, spde = spde),
#'     family = "gaussian",
#'     control.predictor = list(
#'         A = inla.stack.A(stack.obs),
#'         compute = TRUE
#'     )
#' )
#'
#' plot(y, result1$summary.fitted.values[inla.stack.index(stack.obs, "obs")$data, "mean"],
#'     main = "Observations vs posterior predicted values at the data locations"
#' )
#'
#' result2 <- inla(formula,
#'     data = inla.stack.data(stack, spde = spde),
#'     family = "gaussian",
#'     control.predictor = list(
#'         A = inla.stack.A(stack),
#'         compute = TRUE
#'     )
#' )
#'
#' field.pred <- inla.mesh.project(
#'     proj.pred,
#'     result2$summary.fitted.values[inla.stack.index(stack, "pred")$data, "mean"]
#' )
#' field.pred.sd <- inla.mesh.project(
#'     proj.pred,
#'     result2$summary.fitted.values[inla.stack.index(stack, "pred")$data, "sd"]
#' )
#'
#' plot(field, field.pred, main = "True vs predicted field")
#' abline(0, 1)
#' image(inla.mesh.project(mesh,
#'     field = field,
#'     dims = c(200, 200)
#' ),
#' main = "True field"
#' )
#' image(inla.mesh.project(mesh,
#'     field = field.pred,
#'     dims = c(200, 200)
#' ),
#' main = "Posterior field mean"
#' )
#' image(inla.mesh.project(mesh,
#'     field = field.pred.sd,
#'     dims = c(200, 200)
#' ),
#' main = "Prediction standard deviation"
#' )
#' plot(field, (field.pred - field) / 1,
#'     main = "True field vs standardised prediction residuals"
#' )
#' @export inla.stack
inla.stack <- function(..., compress = TRUE, remove.unused = TRUE) {
    if (all(sapply(list(...), function(x) inherits(x, "inla.data.stack")))) {
        return(do.call(
            inla.stack.join,
            c(list(...),
                compress = compress,
                remove.unused = remove.unused
            )
        ))
    } else {
        return(do.call(
            inla.stack.sum,
            c(list(...),
                compress = compress,
                remove.unused = remove.unused
            )
        ))
    }
}


#' @describeIn inla.stack Create data stack as a sum of predictors
#'
#' @export
inla.stack.sum <- function(data, A, effects,
                           tag = "",
                           compress = TRUE,
                           remove.unused = TRUE) {
    input.nrow <- function(x) {
        return(inla.ifelse(
            is.matrix(x) || is(x, "Matrix"),
            nrow(x),
            inla.ifelse(
                is.data.frame(x),
                rep(nrow(x), ncol(x)),
                length(x)
            )
        ))
    }
    input.ncol <- function(x) {
        return(inla.ifelse(
            is.matrix(x) || is(x, "Matrix"),
            ncol(x),
            inla.ifelse(
                is.data.frame(x),
                rep(1L, ncol(x)),
                1L
            )
        ))
    }

    input.list.nrow <- function(l) {
        if (is.data.frame(l)) {
              return(input.nrow(l))
          }
        return(do.call(c, lapply(l, input.nrow)))
    }
    input.list.ncol <- function(l) {
        if (is.data.frame(l)) {
              return(input.ncol(l))
          }
        return(do.call(c, lapply(l, input.ncol)))
    }
    input.list.names <- function(l) {
        if (is.data.frame(l)) {
              return(colnames(l))
          }
        is.df <- sapply(l, is.data.frame)
        name <- vector("list", length(l))
        if (!is.null(names(l))) {
              name[!is.df] <-
                  lapply(
                      names(l)[!is.df],
                      function(x) list(x)
                  )
          } else {
              name[!is.df] <- ""
          }
        name[is.df] <-
            lapply(
                l[is.df],
                function(x) as.list(colnames(x))
            )

        return(do.call(c, name))
    }


    parse.input.list <- function(l, n.A, error.tag, tag = "", n.A.strict = FALSE) {
        ncol <- input.list.ncol(l)
        nrow <- input.list.nrow(l)
        names <- input.list.names(l)
        if ((n.A > 1) && any(nrow == 1)) {
            for (k in which(nrow == 1)) {
                if (ncol[k] == 1) {
                    l[[k]] <- rep(l[[k]], n.A)
                    nrow[k] <- n.A
                } else {
                    stop(paste(error.tag,
                        "Automatic expansion only available for scalars.",
                        sep = ""
                    ))
                }
            }
        }

        if (length(unique(c(names, ""))) < length(c(names, ""))) {
            stop(paste(error.tag,
                "All variables must have unique names\n",
                "Names: ('",
                paste(names, collapse = "', '", sep = ""),
                "')",
                sep = ""
            ))
        }

        for (k in 1:length(names)) {
            if (ncol[k] == 1) {
                names(names)[k] <- names[[k]][[1]]
                names[[k]] <- c(names[[k]][[1]])
            } else {
                names(names)[k] <- names[[k]][[1]]
                names[[k]] <- paste(names[[k]][[1]], ".", 1:ncol[k], sep = "")
            }
        }

        names(nrow) <- names(names)
        names(ncol) <- names(names)

        ## data = as.data.frame(do.call(cbind, l))
        data <- as.data.frame(l)
        names(data) <- do.call(c, names)
        nrow <- nrow(data)
        if ((n.A > 1 || n.A.strict) && (nrow != n.A)) {
            stop(paste(error.tag,
                "Mismatching row sizes: ",
                paste(nrow, collapse = ",", sep = ""),
                ", n.A=", n.A,
                sep = ""
            ))
        }

        index <- list(1:nrow)
        if (!is.null(tag)) {
            names(index) <- tag
        }

        info <- list(data = data, nrow = nrow, ncol = ncol, names = names, index = index)
        class(info) <- "inla.data.stack.info"

        return(info)
    }

    if (is.null(tag)) {
          stop("'tag' must not be 'NULL'")
      }

    ## Check if only a single block was specified.
    if (!is.list(A)) {
        A <- list(A)
        effects <- list(effects)
    }
    if (length(A) != length(effects)) {
          stop(paste("length(A)=", length(A),
              " should be equal to length(effects)=", length(effects),
              sep = ""
          ))
      }

    n.effects <- length(effects)

    eff <- list()
    for (k in 1:n.effects) {
        if (is.data.frame(effects[[k]])) {
            eff[[k]] <-
                parse.input.list(
                    list(effects[[k]]),
                    input.ncol(A[[k]]),
                    paste("Effect block ", k, ":\n", sep = ""),
                    tag
                )
        } else {
            if (!is.list(effects[[k]])) {
                tmp <-
                    inla.ifelse(
                        is.null(names(effects)[k]),
                        "",
                        names(effects)[k]
                    )
                effects[[k]] <- list(effects[[k]])
                names(effects[[k]]) <- tmp
            }
            eff[[k]] <-
                parse.input.list(
                    effects[[k]],
                    input.ncol(A[[k]]),
                    paste("Effect block ", k, ":\n", sep = ""),
                    tag
                )
        }
    }

    for (k in 1:n.effects) {
        if (is.vector(A[[k]])) {
            A[[k]] <- Matrix(A[[k]], input.nrow(A[[k]]), 1)
        }
        if ((input.ncol(A[[k]]) == 1) && (eff[[k]]$nrow > 1)) {
            if (input.nrow(A[[k]]) != 1) {
                  stop(paste("ncol(A) does not match nrow(effect) for block ",
                      k, ": ",
                      input.ncol(A[[k]]), " != ", eff[[k]]$nrow,
                      sep = ""
                  ))
              }
            A[[k]] <- Diagonal(eff[[k]]$nrow, A[[k]][1, 1])
        } else if (input.ncol(A[[k]]) != eff[[k]]$nrow) {
            stop(paste("ncol(A) does not match nrow(effect) for block ",
                k, ": ",
                input.ncol(A[[k]]), " != ", eff[[k]]$nrow,
                sep = ""
            ))
        }
    }
    if (length(unique(input.list.nrow(A))) > 1) {
        stop(paste("Row count mismatch for A: ",
            paste(input.list.nrow(A), collapse = ",", sep = ""),
            sep = ""
        ))
    }
    A.nrow <- nrow(A[[1]])
    A.ncol <- input.list.ncol(A)

    data <-
        parse.input.list(inla.ifelse(
            is.data.frame(data),
            as.list(data),
            data
        ),
        A.nrow,
        paste("Data block:\n", sep = ""),
        tag,
        n.A.strict = TRUE
        )

    effects <- do.call(rbind.inla.data.stack.info, eff)

    A.matrix <- do.call(cbind, A)
    A.nrow <- nrow(A.matrix)
    A.ncol <- ncol(A.matrix)

    if (length(unique(c(names(data$names), names(effects$names)))) <
        length(c(names(data$names), names(effects$names)))) {
        stop(paste("Names for data and effects must not coincide.\n",
            "Data names:   ",
            paste(names(data$names), collapse = ", ", sep = ""),
            "\n",
            "Effect names: ",
            paste(names(effects$names), collapse = ", ", sep = ""),
            sep = ""
        ))
    }

    stack <- list(A = A.matrix, data = data, effects = effects)
    class(stack) <- "inla.data.stack"

    if (compress) {
        return(inla.stack.compress(stack, remove.unused = remove.unused))
    } else if (remove.unused) {
        return(inla.stack.remove.unused(stack))
    } else {
        return(stack)
    }
}

#' @describeIn inla.stack Join two or more data stacks
#'
#' @export
inla.stack.join <- function(..., compress = TRUE, remove.unused = TRUE) {
    S.input <- list(...)

    data <- do.call(
        rbind.inla.data.stack.info,
        lapply(S.input, function(x) x$data)
    )
    effects <- do.call(
        rbind.inla.data.stack.info,
        lapply(S.input, function(x) x$effects)
    )
    ## The .bdiag form of bdiag takes a list as input.
    A <- .bdiag(lapply(S.input, function(x) x$A))

    S.output <- list(A = A, data = data, effects = effects)
    class(S.output) <- "inla.data.stack"

    if (length(unique(c(names(data$names), names(effects$names)))) <
        length(c(names(data$names), names(effects$names)))) {
        stop(paste("Names for data and effects must not coincide.\n",
            "Data names:   ",
            paste(names(data$names), collapse = ", ", sep = ""),
            "\n",
            "Effect names: ",
            paste(names(effects$names), collapse = ", ", sep = ""),
            sep = ""
        ))
    }

    if (compress) {
        return(inla.stack.compress(S.output, remove.unused = remove.unused))
    } else if (remove.unused) {
        return(inla.stack.remove.unused(S.output))
    } else {
        return(S.output)
    }
}





#' @describeIn inla.stack Extract tagged indices
#'
#' @export
inla.stack.index <- function(stack, tag) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    if (is.null(tag)) {
        return(list(
            data = as.vector(do.call(c, stack$data$index)),
            effects = (stack$data$nrow +
                as.vector(do.call(c, stack$effects$index)))
        ))
    } else {
        return(list(
            data = as.vector(do.call(c, stack$data$index[tag])),
            effects = (stack$data$nrow +
                as.vector(do.call(c, stack$effects$index[tag])))
        ))
    }
}

## Extract an object list from an inla.stack internal structure
inla.stack.do.extract <- function(dat) {
    inla.require.inherits(dat, "inla.data.stack.info", "'dat'")

    handle.entry <- function(x) {
        if (dat$ncol[[x]] > 1) {
            return(matrix(
                do.call(
                    c,
                    dat$data[dat$names[[x]]]
                ),
                dat$nrow,
                dat$ncol[[x]]
            ))
        } else if (is.factor(dat$data[[dat$names[[x]]]])) {
            return(dat$data[[dat$names[[x]]]])
        }
        return(as.vector(dat$data[[dat$names[[x]]]]))
    }

    out <- lapply(names(dat$names), handle.entry)
    names(out) <- names(dat$names)

    return(out)
}


#' @describeIn inla.stack Extract data associated with the "left hand side" of the model
#' (e.g. the data itself, `Ntrials`, `link`, `E`)
#'
#' @export
inla.stack.LHS <- function(stack) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    return(inla.stack.do.extract(stack$data))
}

#' @describeIn inla.stack Extract data associated with the "right hand side" of the model
#' (all the covariates/predictors)
#'
#' @export
inla.stack.RHS <- function(stack) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    return(inla.stack.do.extract(stack$effects))
}

#' @describeIn inla.stack Extract data for an inla call, and optionally join with other variables
#'
#' @export
inla.stack.data <- function(stack, ...) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")

    return(c(
        inla.stack.do.extract(stack$data),
        inla.stack.do.extract(stack$effects),
        list(...)
    ))
}

#' @describeIn inla.stack Extract the "A matrix" for control.predictor
#'
#' @export
inla.stack.A <- function(stack) {
    inla.require.inherits(stack, "inla.data.stack", "'stack'")
    return(stack$A)
}

## Export: inla.fgn

## ! \name{fgn}
## ! \alias{inla.fgn}
## ! \alias{fgn}
## !
## ! \title{Return the coefficients in the 3-component AR(1) mixture representing FGN(H)}
## !
## ! \description{This function will return the coefficients in the 3-component AR(1)
## !              mixture representing FGN(H)}
## !
## ! \usage{
## !     inla.fgn(H, K=4L, lag.max = NULL, approx = TRUE)
## ! }
## !
## ! \arguments{
## !   \item{H}{The Hurst coeffcient (0<H<1),  or a vector of those}
## !   \item{K}{The number of components in representation,  must be 3L or 4L}
## !   \item{lag.max}{Integer. If positive integer, return the coeffcients implicitely as the ACF
## !                  from 0 to \code{lag.max}}
## !   \item{approx}{Logical. If \code{lag.max} is an positive integer and \code{approx} is \code{FALSE},
## !                   then return the true ACF instead of the approximated one.}
## !  }
## ! \value{
## !  \code{inla.fgn} returns a named matrix.
## !  If \code{is.null(lag.max)},  then
## !  first column is \code{H},
## !  columns \code{1+1:K} are lag one correlations (or phi's),
## !  and columns \code{1+K+1:K} are the weights.
## !  If \code{lag.max > 0},  then return the ACFs in columns \code{2+(0:lag.max)},
## !  for the H in column 1,  either the approximated ones or the the true ones.
## !
## !  This function is EXPERIMENTAL!!!
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## !     r = c(inla.fgn(0.7))
## !     r_m = inla.fgn(seq(0.6, 0.8, by=0.01))
## ! }

`inla.fgn` <- function(H, K = 4L, lag.max = NULL, approx = TRUE) {
    if (!any(K == c(3L, 4L))) {
        stop(paste0("Number of components 'K' must be 3 or 4,  not ", K))
    }

    t.dir <- inla.tempdir()
    in.file <- inla.tempfile(tmpdir = t.dir)
    out.file <- inla.tempfile(tmpdir = t.dir)
    inla.write.fmesher.file(matrix(c(K, as.numeric(H)), ncol = 1), file = in.file)
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(shQuote(inla.call.no.remote()), "-s -m fgn", in.file, out.file), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(shQuote(inla.call.no.remote()), "-s -m fgn", in.file, out.file), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    res <- inla.read.fmesher.file(out.file)
    stopifnot(ncol(res) == 2 * K + 1)
    colnames(res) <- c("H", paste0("phi", 1:K), paste0("weight", 1:K))

    unlink(t.dir, recursive = TRUE)

    if (!is.null(lag.max)) {
        lag.max <- as.integer(lag.max)
        stopifnot(lag.max > 0)
        m <- length(H)
        n <- lag.max + 1
        ACF <- matrix(NA, m, n)
        for (i in 1:m) {
            if (approx) {
                phi <- res[i, 1 + 1:K]
                w <- res[i, 1 + K + 1:K]
                a <- rep(0, n)
                for (j in 1:K) {
                    a <- a + w[j] * phi[j]^(0:(n - 1))
                }
            } else {
                a <- inla.acvfFGN(H[i], n - 1)
            }
            ACF[i, ] <- a
        }
        ACF[, 1] <- 1.0 ## just to make sure
        colnames(ACF) <- paste0("acf", (1:ncol(ACF)) - 1)
        return(cbind(H = H, ACF))
    } else {
        return(res)
    }
}

`inla.acvfFGN` <- function(H, maxlag) {
    ## a copy of function FGN::acvfFGN as library FGN in not there for R-3.5
    h2 <- 2 * H
    k <- 1:maxlag
    return(c(1, 0.5 * ((k + 1)^h2 - 2 * k^h2 + (k - 1)^h2)))
}

## Export: merge!inla
## Export: inla.merge

## ! \name{inla.merge}
## ! \alias{inla.merge}
## ! \alias{merge.inla}
## ! \title{Merge a mixture of \code{inla}-objects}
## ! \description{Merge a mixture of \code{inla}-objects}
## ! \usage{
## !     \method{merge}{inla}(x, y, ..., prob = rep(1,  length(loo)),
## !                          mc.cores = NULL, verbose = FALSE)
## !     inla.merge(loo, prob = rep(1,  length(loo)), verbose = FALSE)
## ! }
## ! \arguments{
## !   \item{x}{An \code{inla}-object to be merged}
## !   \item{y}{An \code{inla}-object to be merged}
## !   \item{...}{Additional \code{inla}-objects to be merged}
## !   \item{loo}{List of \code{inla}-objects to be merged}
## !   \item{prob}{The mixture of (possibly unnormalized) probabilities}
## !   \item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
## !                   \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
## !                   and \code{inla.getOption("num.threads")} in that order.}
## !   \item{verbose}{Turn on verbose-output or not}
## !  }
## ! \value{
## !   A merged \code{inla}-object.
## ! }
## ! \details{
## !    The function \code{merge.inla} implements method \code{merge} for
## !    \code{inla}-objects. \code{merge.inla} is a wrapper for the function
## !    \code{inla.merge}. The interface is slightly different, \code{merge.inla}
## !    is more tailored for interactive use, whereas \code{inla.merge} is better
## !    in general code.
## !
## !    \code{inla.merge} is intented for merging a mixture of \code{inla}-objects,
## !    each run with the same formula and settings, except for a set of
## !    hyperparameters that are fixed to different values. Using this function,
## !    we can then integrate over these hyperparameters using (unnormalized)
## !    integration weights
## !    \code{prob}. The main objects to be merged,  are the summary statistics
## !    and marginal densities (like for hyperparameters, fixed,  random,  etc).
## !    Not all entries in the object can be merged, and by default these
## !    are inheritated from the first object in the list,
## !    while some are just set to  \code{NULL}.
## !    Those objectes that are merged,
## !    will be listed if run with option \code{verbose=TRUE}.
## !
## !    Note that merging hyperparameter in the user-scale is prone to
## !    discretization error in general, so it is more stable to convert
## !    the marginal of the hyperparameter from the merged internal scale
## !    to the user-scale. (This is not done by this function.)
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! set.seed(123)
## ! n = 100
## ! y = rnorm(n)
## ! y[1:10] = NA
## ! x = rnorm(n)
## ! z1 = runif(n)
## ! z2 = runif(n)*n
## ! idx = 1:n
## ! idx2 = 1:n
## ! lc1 = inla.make.lincomb(idx = c(1, 2, 3))
## ! names(lc1) = "lc1"
## ! lc2 = inla.make.lincomb(idx = c(0, 1, 2, 3))
## ! names(lc2) = "lc2"
## ! lc3 = inla.make.lincomb(idx = c(0, 0, 1, 2, 3))
## ! names(lc3) = "lc3"
## ! lc = c(lc1, lc2, lc3)
## ! rr = list()
## ! for (logprec in c(0, 1, 2))
## !     rr[[length(rr)+1]] = inla(y ~ 1 + x + f(idx, z1) + f(idx2, z2),
## !              lincomb = lc,
## !              control.family = list(hyper = list(prec = list(initial = logprec))),
## !              control.predictor = list(compute = TRUE, link = 1),
## !              data = data.frame(y, x, idx, idx2, z1, z2))
## ! r = inla.merge(rr, prob = seq_along(rr), verbose=TRUE)
## ! summary(r)
## !}

`merge.inla` <- function(x, y, ..., prob = rep(1, length(loo)), verbose = FALSE) {
    return(inla.merge(loo = list(x, y, ...), prob = prob, verbose = verbose))
}

`inla.merge` <- function(loo, prob = rep(1, length(loo)), mc.cores = NULL, verbose = FALSE) {
    nm <- "disable.inla.merge.warning"
    if (!exists(nm, envir = inla.get.inlaEnv())) {
        warning("This function is experimental.", immediate. = TRUE)
        assign(nm, TRUE, envir = inla.get.inlaEnv())
    }

    verboze <- function(..., sep = "") {
        if (verbose) {
            cat("inla.merge: ", ..., "\n", sep = sep)
        }
    }

    merge.marginals <- function(lom, prob, nx = 128, eps.y = 1e-6) {
        ## compute the (min, median, max) for each marginal, and then the min, median, max of
        ## those again.
        xs <- matrix(unlist(lapply(
            lom,
            function(m) {
                xx <- m[, "x"]
                return(c(min(xx), median(xx), max(xx)))
            }
        )), nrow = 3)
        x.min <- min(xs[1, ])
        x.med <- median(xs[2, ])
        x.max <- max(xs[3, ])
        ## we guess the transformation... In any case, it should not to be to bad.
        if (x.min > 0 && x.max < 1) {
            ## probability
            m1 <- function(x) 1.0 / (1.0 + exp(-x))
            m1i <- function(x) log(x / (1.0 - x))
        } else if (x.min > 0) {
            ## this is a positive marginal. Choose the scale that make is more symmetric
            if (abs((x.med - x.min) / (x.max - x.min) - 0.5) <
                abs((log(x.med) - log(x.min)) / (log(x.max) - log(x.min)) - 0.5)) {
                ## linear
                m1 <- m1i <- function(x) x
            } else {
                ## log
                m1 <- function(x) exp(x)
                m1i <- function(x) log(x)
            }
        } else {
            ## use a linear mapping
            m1 <- m1i <- function(x) x
        }
        xx <- m1(seq(m1i(x.min), m1i(x.max), len = nx))
        yy <- rep(0, nx)
        for (i in seq_along(lom)) {
            yy <- yy + prob[i] * inla.dmarginal(xx, lom[[i]])
        }
        ## remove points not needed
        idx.keep <- (yy > eps.y * max(yy))
        marg <- inla.smarginal(cbind(x = xx[idx.keep], y = yy[idx.keep]), factor = 2L, keep.type = TRUE)
        return(marg)
    }

    merge.summary <- function(los, prob) {
        n <- length(los)
        stopifnot(length(prob) == n)
        m <- nrow(los[[1]])
        m1 <- numeric(m)
        m2 <- numeric(m)
        prob <- prob / sum(prob)
        for (i in 1:n) {
            m1 <- m1 + prob[i] * los[[i]][, "mean"]
            m2 <- m2 + prob[i] * (los[[i]][, "sd"]^2 + los[[i]][, "mean"]^2)
        }
        df <- data.frame(
            mean = m1,
            sd = sqrt(pmax(.Machine$double.eps, m2 - m1^2))
        )
        if (!is.null(los[[1]]$ID)) {
            df$ID <- los[[1]]$ID
        }
        rownames(df) <- rownames(los[[1]])
        return(df)
    }

    merge.cpo <- function(cpos, prob) {
        n <- length(cpos)
        stopifnot(length(prob) == n)
        m <- length(cpos[[1]]$cpo)
        res <- matrix(0, m, 3)
        if (m > 0) {
            prob <- prob / sum(prob)
            for (i in 1:n) {
                res[, 1] <- res[, 1] + prob[i] * cpos[[i]]$cpo
                res[, 2] <- res[, 2] + prob[i] * cpos[[i]]$pit
                res[, 3] <- res[, 3] + prob[i] * cpos[[i]]$failure
            }
            res[, 3] <- as.numeric(res[, 3] > 0)
        }
        return(list(cpo = res[, 1], pit = res[, 2], failure = res[, 3]))
    }

    merge.po <- function(pos, prob) {
        n <- length(pos)
        stopifnot(length(prob) == n)
        m <- length(pos[[1]]$po)
        res <- numeric(m)
        if (m > 0) {
            prob <- prob / sum(prob)
            for (i in 1:n) {
                res <- res + prob[i] * pos[[i]]$po
            }
        }
        return(list(po = res))
    }

    stopifnot(length(prob) == length(loo))
    stopifnot(all(prob > 0))
    prob <- prob / sum(prob)
    m <- length(loo)
    res <- loo[[1]]

    verboze("Enter with prob = [", round(prob, dig = 3), "], and", m, "models", sep = " ")

    ## list
    marginals <- c(
        paste0("marginals.", c("hyperpar", "fixed", "lincomb", "lincomb.derived", "linear.predictor")),
        "internal.marginals.hyperpar"
    )
    ## list of list
    marginals2 <- paste0("marginals.", c("random", "spde2.blc", "spde3.blc"))
    ## data.frame
    summaries <- c(
        paste0("summary.", c("hyperpar", "fixed", "lincomb", "lincomb.derived", "linear.predictor")),
        "internal.summary.hyperpar"
    )
    ## list of data.frame
    summaries2 <- paste0("summary.", c("random", "spde2.blc", "spde3.blc"))
    ## items to remove
    remove <- c(
        "marginals.fitted.values", "summary.fitted.values", "dic", "waic",
        "neffp", "mode", ".args", "model.matrix"
    )
    ## items to remove in $misc
    misc.remove <- c(
        "cov.intern", "cov.intern.eigenvalues", "cov.intern.eigenvectors",
        "lincomb.derived.correlation.matrix", "lincomb.derived.covariance.matrix",
        "log.posterior.mode", "stdev.corr.negative", "stdev.corr.negative",
        "stdev.corr.positive", "theta.mode"
    )

    for (nm in marginals) {
        idx <- which(names(res) == nm)
        if (length(idx) > 0) {
            verboze("Merge '$", nm, "'")
            nm <- names(res[[idx]])
            res[[idx]] <- (inla.mclapply(
                seq_along(res[[idx]]),
                function(k) {
                    margs <- lapply(loo, function(x, kk) x[[idx]][[kk]], kk = k)
                    return(merge.marginals(margs, prob))
                },
                mc.cores = mc.cores
            ))
            names(res[[idx]]) <- nm
        }
    }

    for (nm in marginals2) {
        idx <- which(names(res) == nm)
        if (length(idx) > 0) {
            verboze("Merge '$", nm, "'")
            for (k in seq_along(res[[idx]])) {
                verboze("      '$", nm, "$", names(res[[idx]])[k], "'")
                nm <- names(res[[idx]][[k]])
                res[[idx]][[k]] <- (inla.mclapply(
                    seq_along(res[[idx]][[k]]),
                    function(kk) {
                        margs <- lapply(loo, function(x, k, kkk) x[[idx]][[k]][[kkk]], k = k, kkk = kk)
                        return(merge.marginals(margs, prob))
                    },
                    mc.cores = mc.cores
                ))
                names(res[[idx]][[k]]) <- nm
            }
        }
    }

    for (nm in summaries) {
        idx <- which(names(res) == nm)
        if (length(idx) > 0) {
            verboze("Merge '$", nm, "'")
            if (!is.null(res[[idx]]) && nrow(res[[idx]]) > 0) {
                zum <- lapply(loo, function(x) x[[idx]])
                res[[idx]] <- merge.summary(zum, prob)
            }
        }
    }

    if (!is.null(res$cpo)) {
        verboze(paste0("Merge '$cpo'"))
        nm <- "disable.inla.merge.cpo.warning"
        if (!exists(nm, envir = inla.get.inlaEnv())) {
            warning("Merging 'cpo' and 'pit'-results are/can be, approximate only")
            assign(nm, TRUE, envir = inla.get.inlaEnv())
        }
        cpo <- lapply(loo, function(x) x$cpo)
        res$cpo <- merge.cpo(cpo, prob)
    }

    if (!is.null(res$po)) {
        verboze(paste0("Merge '$po'"))
        po <- lapply(loo, function(x) x$po)
        res$po <- merge.po(po, prob)
    }

    for (nm in summaries2) {
        idx <- which(names(res) == nm)
        if (length(idx) > 0) {
            for (k in seq_along(res[[idx]])) {
                verboze(paste0("      '$", nm, "$", names(res[[idx]])[k], "'"))
                zum <- lapply(loo, function(x) x[[idx]][[k]])
                res[[idx]][[k]] <- merge.summary(zum, prob)
            }
        }
    }


    ## merge configs, if they are there
    if (!is.null(res$misc$configs)) {
        verboze("Merge '$misc$configs'")
        res$misc$configs$nconfig <- m * res$misc$configs$nconfig
        res$misc$configs$config <- as.list(seq_len(res$misc$configs$nconfig)) ## create the list to be filled
        res$misc$configs$max.log.posterior <- NA ## to be computer later
        count <- 1
        for (k in seq_along(loo)) {
            conf <- loo[[k]]$misc$configs
            for (kk in seq_len(conf$nconfig)) {
                conf$config[[kk]]$log.posterior <- conf$config[[kk]]$log.posterior +
                    conf$max.log.posterior + log(prob[k])
                conf$config[[kk]]$log.posterior.orig <- conf$config[[kk]]$log.posterior.orig +
                    conf$max.log.posterior + log(prob[k])
                res$misc$configs$config[[count]] <- conf$config[[kk]]
                count <- count + 1
            }
        }
        ## rescale the log.posterior's, similar code as in 'inla.collect.misc()'
        res$misc$configs$max.log.posterior <- max(sapply(res$misc$configs$config, function(x) x$log.posterior.orig))
        for (k in seq_len(res$misc$configs$nconfig)) {
            res$misc$configs$config[[k]]$log.posterior <- res$misc$configs$config[[k]]$log.posterior -
                res$misc$configs$max.log.posterior
            res$misc$configs$config[[k]]$log.posterior.orig <- res$misc$configs$config[[k]]$log.posterior.orig -
                res$misc$configs$max.log.posterior
        }
    }

    verboze(paste0("Merge '$misc$nfunc"))
    res$misc$nfunc <- sum(unlist(lapply(loo, function(x) x$misc$nfunc)))

    verboze(paste0("Merge '$cpu.used"))
    res$cpu.used <- rowSums(sapply(loo, function(x) x$cpu.used))

    verboze(paste0("Merge '$logfile"))
    res$logfile <- c()
    for (k in seq_along(loo)) {
        res$logfile <- c(
            res$logfile,
            "###",
            paste0("### CONFIGURATION number ", k, ", prob = ", round(prob[k], dig = 5)),
            "###",
            loo[[k]]$logfile
        )
    }

    if (!is.null(res$joint.hyper)) {
        verboze(paste0("Merge '$joint.hyper"))
        res$joint.hyper <- data.frame()
        for (k in seq_along(loo)) {
            tmp <- loo[[k]]$joint.hyper
            tmp[, ncol(tmp)] <- tmp[, ncol(tmp)] + log(prob[k])
            res$joint.hyper <- rbind(res$joint.hyper, tmp)
        }
    }

    for (nm in remove) {
        verboze(paste0("Remove '$", nm, "'"))
        idx <- which(names(res) == nm)
        if (length(idx) > 0) {
            res[[idx]] <- NULL
        }
    }

    for (nm in misc.remove) {
        verboze(paste0("Remove '$misc$", nm, "'"))
        idx <- which(names(res$misc) == nm)
        if (length(idx) > 0) {
            res$misc[[idx]] <- NULL
        }
    }

    return(res)
}

## Export: inla.link.log inla.link.invlog
## Export: inla.link.neglog inla.link.invneglog
## Export: inla.link.logit inla.link.invlogit inla.link.probit
## Export: inla.link.invprobit inla.link.cloglog inla.link.invcloglog
## Export: inla.link.loglog inla.link.invloglog inla.link.tan inla.link.invtan
## Export: inla.link.identity inla.link.invidentity inla.link.invalid
## Export: inla.link.cauchit inla.link.invcauchit
## Export: inla.link.inverse inla.link.invinverse
## Export: inla.link.robit inla.link.invrobit
## Export: inla.link.sn inla.link.invsn

## ! \name{link}
## ! \alias{link}
## ! \alias{inla.link}
## ! \alias{inla.link.log}
## ! \alias{inla.link.invlog}
## ! \alias{inla.link.neglog}
## ! \alias{inla.link.invneglog}
## ! \alias{inla.link.logit}
## ! \alias{inla.link.invlogit}
## ! \alias{inla.link.probit}
## ! \alias{inla.link.invprobit}
## ! \alias{inla.link.cloglog}
## ! \alias{inla.link.invcloglog}
## ! \alias{inla.link.loglog}
## ! \alias{inla.link.invloglog}
## ! \alias{inla.link.tan}
## ! \alias{inla.link.invtan}
## ! \alias{inla.link.identity}
## ! \alias{inla.link.invidentity}
## ! \alias{inla.link.invalid}
## ! \alias{inla.link.invinvalid}
## ! \alias{inla.link.cauchit}
## ! \alias{inla.link.invcauchit}
## ! \alias{inla.link.inverse}
## ! \alias{inla.link.invinverse}
## ! \alias{inla.link.robit}
## ! \alias{inla.link.invrobit}
## ! \alias{inla.link.sn}
## ! \alias{inla.link.invsn}
## ! \alias{inla.link.invqpoisson}
## !
## ! \title{Link functions in INLA}
## !
## ! \description{Define link-functions and its inverse}
## !
## ! \usage{
## ! inla.link.log(x, inverse=FALSE)
## ! inla.link.invlog(x, inverse=FALSE)
## ! inla.link.neglog(x, inverse=FALSE)
## ! inla.link.invneglog(x, inverse=FALSE)
## ! inla.link.logit(x, inverse=FALSE)
## ! inla.link.invlogit(x, inverse=FALSE)
## ! inla.link.probit(x, inverse=FALSE)
## ! inla.link.invprobit(x, inverse=FALSE)
## ! inla.link.cloglog(x, inverse=FALSE)
## ! inla.link.invcloglog(x, inverse=FALSE)
## ! inla.link.loglog(x, inverse=FALSE)
## ! inla.link.invloglog(x, inverse=FALSE)
## ! inla.link.tan(x, inverse=FALSE)
## ! inla.link.invtan(x, inverse=FALSE)
## ! inla.link.cauchit(x, inverse=FALSE)
## ! inla.link.invcauchit(x, inverse=FALSE)
## ! inla.link.identity(x, inverse=FALSE)
## ! inla.link.invidentity(x, inverse=FALSE)
## ! inla.link.inverse(x, inverse=FALSE)
## ! inla.link.invinverse(x, inverse=FALSE)
## ! inla.link.robit(x, df=7, inverse=FALSE)
## ! inla.link.invrobit(x, df=7, inverse=FALSE)
## ! inla.link.sn(x, intercept=0.5, skew=0, a=0, inverse=FALSE)
## ! inla.link.invsn(x, intercept=0.5, skew=0, a=0, inverse=FALSE)
## ! inla.link.invalid(x, inverse=FALSE)
## ! inla.link.invinvalid(x, inverse=FALSE)
## ! inla.link.invqpoisson(x, inverse = FALSE, quantile = 0.5)
## ! }
## !
## ! \arguments{
## !     \item{x}{The argument. A numeric vector.}
## !     \item{df}{The degrees of freedom for the Student-t}
## !     \item{inverse}{Logical. Use the link (\code{inverse=FALSE})
## !                    or its inverse (\code{inverse=TRUE})}
## !     \item{intercept}{The quantile level for the intercept in the Skew-Normal link}
## !     \item{skew}{The skewness in the Skew-Normal.
## !                 Not both of \code{skew} and \code{a} can be given.}
## !     \item{a}{The \code{a}-paraeter in the Skew-Normal.
## !              Not both of \code{skew} and \code{a} can be given.}
## !     \item{quantile}{The quantile level for quantile links}
## !}
## !
## ! \value{Return the values of the link-function or its inverse.}
## ! \note{The \code{inv}-functions are redundant,  as
## !       \code{inla.link.invlog(x) = inla.link.log(x, inverse=TRUE)}
## !       and so on,  but they are simpler to use a arguments
## !       to other functions.}
## ! \author{Havard Rue \email{hrue@r-inla.org}}

`inla.link.cauchit` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(tan(pi * (x - 0.5)))
    } else {
        return(1.0 / pi * atan(x) + 0.5)
    }
}
`inla.link.invcauchit` <- function(x, inverse = FALSE) {
    return(inla.link.cauchit(x, inverse = !inverse))
}


`inla.link.log` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(log(x))
    } else {
        return(exp(x))
    }
}
`inla.link.invlog` <- function(x, inverse = FALSE) {
    return(inla.link.log(x, inverse = !inverse))
}

`inla.link.neglog` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(-log(x))
    } else {
        return(exp(-x))
    }
}
`inla.link.invneglog` <- function(x, inverse = FALSE) {
    return(inla.link.neglog(x, inverse = !inverse))
}

`inla.link.logit` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(log(x / (1.0 - x)))
    } else {
        return(1.0 / (1.0 + exp(-x)))
    }
}
`inla.link.invlogit` <- function(x, inverse = FALSE) {
    return(inla.link.logit(x, inverse = !inverse))
}

`inla.link.probit` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(qnorm(x))
    } else {
        return(pnorm(x))
    }
}
`inla.link.invprobit` <- function(x, inverse = FALSE) {
    return(inla.link.probit(x, inverse = !inverse))
}

`inla.link.robit` <- function(x, df = 7, inverse = FALSE) {
    s <- sqrt(df / (df - 2.0))
    if (!inverse) {
        return(qt(x, df = df) * s)
    } else {
        return(pt(x / s, df = df))
    }
}
`inla.link.invrobit` <- function(x, df = 7, inverse = FALSE) {
    return(inla.link.robit(x, df = df, inverse = !inverse))
}

`inla.link.loglog` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(-log(-log(x)))
    } else {
        return(exp(-exp(-x)))
    }
}
`inla.link.invloglog` <- function(x, inverse = FALSE) {
    return(inla.link.loglog(x, inverse = !inverse))
}

`inla.link.cloglog` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(log(-log(1 - x)))
    } else {
        return(1.0 - exp(-exp(x)))
    }
}
`inla.link.invcloglog` <- function(x, inverse = FALSE) {
    return(inla.link.cloglog(x, inverse = !inverse))
}

`inla.link.tan` <- function(x, inverse = FALSE) {
    if (!inverse) {
        return(tan(x / 2.0))
    } else {
        return(2.0 * atan(x))
    }
}
`inla.link.invtan` <- function(x, inverse = FALSE) {
    return(inla.link.tan(x, inverse = !inverse))
}

`inla.link.identity` <- function(x, inverse = FALSE) {
    return(x)
}
`inla.link.invidentity` <- function(x, inverse = FALSE) {
    return(inla.link.identity(x, inverse = !inverse))
}

`inla.link.inverse` <- function(x, inverse = FALSE) {
    return(1 / x)
}
`inla.link.invinverse` <- function(x, inverse = FALSE) {
    return(inla.link.inverse(x, inverse = !inverse))
}

`inla.link.invqpoisson` <- function(x, inverse = FALSE, quantile = 0.5) {
    stopifnot(inverse == FALSE)
    return (qgamma(1.0 - quantile, shape = exp(x)+1.0, rate=1))
}

`inla.link.sn` <- function(x, intercept = 0.5, skew = 0, a = NULL, inverse = FALSE) {
    inla.require("sn", stop.on.error = TRUE)

    if (is.null(a)) {
        cache <- inla.pc.sn.cache()
        a <- sign(skew) * cache$pos$a(abs(skew))
    } else {
        stopifnot(all(skew == 0))
    }

    delta <- a / sqrt(1 + a^2)
    omega <- 1 / sqrt(1 - 2 * delta^2 / pi)
    xi <- -omega * delta * sqrt(2 / pi)

    ## skewness
    ## print((4-pi)/2 * (delta*sqrt(2/pi))^3 / ((1-2*delta^2/pi)^(3/2)))

    stopifnot(length(intercept) == 1)
    ## can turn this feature off...
    if (intercept > 0 && intercept < 1.0) {
        intcept <- sn::qsn(intercept, xi = xi, omega = omega, alpha = a)
    } else {
        intcept <- 0
    }

    ## the sn's qsn and psn handle 'a' in a strange way
    if (length(a) == 1) {
        ## then it vectorize...
        if (!inverse) {
            ret <- sn::qsn(x, xi = xi, omega = omega, alpha = a) - intcept
        } else {
            ret <- sn::psn(x + intcept, xi = xi, omega = omega, alpha = a)
        }
    } else {
        ## then it does not vectorize... but does not give any error/warning!
        X <- cbind(x = x, xi = xi, omega = omega, alpha = a)
        ret <- numeric(nrow(X))
        if (!inverse) {
            for (i in 1:nrow(X)) {
                ret[i] <- sn::qsn(X[i, "x"],
                    xi = X[i, "xi"],
                    omega = X[i, "omega"], alpha = X[i, "alpha"]
                ) -
                    intcept
            }
        } else {
            for (i in 1:nrow(X)) {
                ret[i] <- sn::psn(X[i, "x"] + intcept,
                    xi = X[i, "xi"],
                    omega = X[i, "omega"], alpha = X[i, "alpha"]
                )
            }
        }
    }
    return(ret)
}
`inla.link.invsn` <- function(x, intercept = 0.5, skew = 0, a = NULL, inverse = FALSE) {
    return(inla.link.sn(x, intercept = intercept, skew = skew, a = a, inverse = !inverse))
}


## These are the invalid ones
`inla.link.invalid` <- function(x, inverse = FALSE) {
    stop("The invalid link-function is used.")
}
`inla.link.invinvalid` <- function(x, inverse = FALSE) {
    stop("The invinvalid link-function is used.")
}

`inla.link.sn.test` <- function() {
    n <- 4
    a <- runif(n, min = -10, max = 10)
    intercept <- runif(n)
    x <- rnorm(n)

    for (i in 1:n) {
        ans <- c(
            x = x[i], intercept = intercept[i], a = a[i],
            prob = inla.link.sn(x[i], a = a[i], intercept = intercept[i], inverse = TRUE),
            prob.inv = inla.link.sn(inla.link.sn(x[i], a = a[i], intercept = intercept[i], inverse = TRUE),
                a = a[i], intercept = intercept[i], inverse = FALSE
            ),
            "is 0?" = inla.link.sn(inla.link.sn(x[i], a = a[i], intercept = intercept[i], inverse = TRUE),
                a = a[i], intercept = intercept[i], inverse = FALSE
            ) - x[i]
        )
        print(round(dig = 3, ans))
    }

    cat("\n\n")
    n <- 4
    s <- runif(n, min = -0.98, max = 0.98)
    intercept <- runif(n)
    x <- rnorm(n)

    for (i in 1:n) {
        ans <- c(
            x = x[i], intercept = intercept[i], skew = s[i],
            prob = inla.link.sn(x[i], skew = s[i], intercept = intercept[i], inverse = TRUE),
            prob.inv = inla.link.sn(inla.link.sn(x[i], skew = s[i], intercept = intercept[i], inverse = TRUE),
                skew = s[i], intercept = intercept[i], inverse = FALSE
            ),
            "is 0?" = inla.link.sn(inla.link.sn(x[i], skew = s[i], intercept = intercept[i], inverse = TRUE),
                skew = s[i], intercept = intercept[i], inverse = FALSE
            ) - x[i]
        )
        print(round(dig = 3, ans))
    }
}

context("test 'likelihood circular'")

test_that("Case 1", {
    set.seed(123)
    n = 10000
    z = rnorm(n, sd=0.3)
    eta = 1 + z
    y.pred = inla.link.invtan(eta)
    
    kappa = 10
    x = seq(-pi, pi, len = 10000)
    d = exp(kappa*cos(x))
    dd = cumsum(d)
    dd = dd /max(dd)
    cn.icdf.func = splinefun(dd, x, method = "monoH.FC")
    rcn = function(n) cn.icdf.func(runif(n))
    y = y.pred + rcn(n)
    
    formula = y ~ 1 + z
    r = inla(formula,  data = data.frame(y, z),
            family = "circularnormal",
            control.inla = list(cmin = -Inf))
    expect_true(all(abs(r$summary.fixed[, "mean"] - c(1, 1)) < 0.05))
    expect_true(all(abs(r$summary.hyperpar[1, "mean"] - kappa) < 0.1))
})

## Export: inla.group

## !\name{inla.group}
## !\alias{inla.group}
## !
## !\title{Group or cluster covariates}
## !
## !\description{\code{inla.group} group or cluster covariates so to reduce
## !  the number of unique values}
## !\usage{
## !inla.group(x, n = 25, method = c("cut", "quantile"), idx.only = FALSE)
## !}
## !\arguments{
## !
## !  \item{x}{The vector of covariates to group.}
## !  \item{n}{Number of classes or bins to group into.}
## !  \item{method}{Group either using bins with equal length intervals
## !    (\code{method = "cut"}), or equal distance in the `probability'
## !    scale using the quantiles (\code{method = "quantile"}).}
## !  \item{idx.only}{Option to return the index only and not the
## !    \code{method}.}
## !}
## !
## !
## !\value{
## !  \code{inla.group} return the new grouped covariates where the classes
## !  are set to the median of all the covariates belonging to that group.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{\code{\link{f}}}
## !\examples{
## !## this gives groups 3 and 8
## !x = 1:10
## !x.group = inla.group(x, n = 2)
## !
## !## this is the intended use, to reduce the number of unique values in
## !## the of first argument of f()
## !n = 100
## !x = rnorm(n)
## !y = x + rnorm(n)
## !result = inla(y ~ f(inla.group(x, n = 20), model = "iid"), data=data.frame(y=y,x=x))
## !}

`inla.group` <- function(x, n = 25, method = c("cut", "quantile"), idx.only = FALSE) {
    `inla.group.core` <- function(x, n = 25, method = c("cut", "quantile"), idx.only) {
        ## group covariates into N groups using method "quantile" or
        ## "cut", i.e., the functions quantile() or cut().  the cut
        ## use 'even length' wheras the 'quantile' use even length
        ## quantiles.

        ## I make the "cut" default, as then we have control over the
        ## minimum distance between each cell, whereas the "quantile"
        ## approach, we do not have a such control.

        if (n < 1) {
              stop("Number of groups must be > 0")
          }

        if (n == 1) {
              return(rep(median(x), length(x)))
          }

        method <- match.arg(method)
        if (method == "cut") {
            ## use equal length
            a <- cut(x, n)
        } else {
            ## use break-points corresponding to the quantiles
            aq <- unique(quantile(x, probs = c(0, ppoints(n - 1), 1)))
            a <- cut(x, breaks = as.numeric(aq), include.lowest = TRUE)
        }
        ## the rest is then the same
        nlev <- nlevels(a)
        xx <- list()
        for (i in 1:nlev) {
              xx[[i]] <- list()
          }

        for (i in 1:length(x)) {
              xx[[as.numeric(a[i])]] <- c(unlist(xx[[as.numeric(a[i])]]), x[i])
          }
        values <- numeric(nlev)

        ff.local <- function(xx) {
            if (length(xx) > 0) {
                  return(median(xx))
              } else {
                  return(NA)
              }
        }
        if (!idx.only) {
            values <- unlist(sapply(xx, ff.local))
            return(as.numeric(values[as.numeric(a)]))
        } else {
            return(as.numeric(a))
        }
    }

    if (missing(x)) {
          return(NULL)
      }

    if (any(is.na(x))) {
        idx.ok <- !is.na(x)
        x[idx.ok] <- inla.group.core(x[idx.ok], n, method, idx.only)
        return(x)
    } else {
        return(inla.group.core(x, n, method, idx.only))
    }
}

`inla.group.dist` <- function(x, dist.min = NULL, dist.min.rel = NULL) {
    ## form groups based on the distance such that min.distance > dist.min
    if (missing(x)) {
          return(NULL)
      }

    if (any(is.na(x))) {
        idx.ok <- !is.na(x)
        x[idx.ok] <- inla.group.dist(x[idx.ok], dist.min, dist.min.rel)
        return(x)
    }

    if ((is.null(dist.min) && is.null(dist.min.rel)) || (length(x) == 1L)) {
        return(x)
    }
    dist <- if (!is.null(dist.min)) dist.min else diff(range(x)) * dist.min.rel
    x.sort <- order(x)
    n <- length(x)
    xx <- x[x.sort]

    ## this is potentially slow...
    i <- 1L
    while (i <= n - 1) {
        if (xx[i + 1L] - xx[i] > 0.0 && xx[i + 1L] - xx[i] < 2 * dist) {
            k <- 2L
            while (i + k <= n && xx[i + k] - xx[i] < 2 * dist) {
                k <- k + 1L
            }
            idx <- i:(i + k - 1L)
            xx[idx] <- mean(range(xx[idx]))
            i <- i + k
        } else {
            i <- i + 1L
        }
    }
    xx[x.sort] <- xx

    return(xx)
}

## Export: inla.geobugs2inla

## !\name{geobugs2inla}
## !\alias{geobugs2inla}
## !\alias{inla.geobugs2inla}
## !\keyword{graph}
## !
## !\title{INLA utility functions}
## !
## !\description{Various utility functions for INLA}
## !
## !\usage{
## !     inla.geobugs2inla(adj, num, graph.file="graph.dat")
## !}
## !
## !\arguments{
## !
## !  \item{adj}{A vector listing the ID numbers of the adjacent areas for
## !  each area. This is a sparse representation of the full adjacency
## !  matrix for the study region, and can be generated using the Adjacency
## !  Tool from the Map menu in GeoBUGS.}
## !
## !  \item{num}{A vector of length N (the total number of areas) giving the
## !  number of neighbours n.i for each area.}
## !
## !  \item{graph.file}{Name of the file of the new graph in the INLA
## !    format.}
## !
## !}
## !
## !
## !\value{The return value is the name of the graph-file created.}
## !
## !\note{These are all the same function,  and the two different names are due to backward-compatibility}
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\seealso{\code{\link{inla}}, \code{\link{inla.surv}}, \code{\link{hyperpar.inla}}}
## !
## !%%\examples{}

## various graph convert utilities that I got from someone. I don't
## know how useful they are really...

`inla.geobugs2inla` <- function(adj, num, graph.file = "graph.dat") {
    return(inla.graph.convert.2(adj, num, graph.file))
}

`inla.graph.convert.2` <- function(adj, num, graph.file = "graph.dat") {
    ## A function for converting GeoBUGS adjacency data into the INLA
    ## graph format. Kindly provided by Aki Havunlinna tkk.fi; thanks.

    fd <- file(graph.file, "w")
    len <- length(num)
    cat(len, "\n", file = fd)
    k <- 1L
    for (i in 1L:len) {
        if (num[i] > 0) {
            cat(i, num[i], adj[k:(k + num[i] - 1L)], "\n", file = fd)
        } else {
            cat(i, num[i], "\n", file = fd)
        }
        k <- k + num[i]
    }
    close(fd)
    return(graph.file)
}

## Export: inla

## ! \name{inla}
## ! \alias{inla}
## !
## ! \title{Bayesian analysis of
## ! structured additive models} \description{\code{inla} performs a
## ! full Bayesian analysis of additive models using Integrated Nested
## ! Laplace approximation
## ! }
## !
## ! \usage{
## ! inla(
## !    formula,
## !    family = "gaussian",
## !    contrasts = NULL,
## !    data,
## !    quantiles=c(0.025, 0.5, 0.975),
## !    E = NULL,
## !    offset=NULL,
## !    scale = NULL,
## !    weights = NULL,
## !    Ntrials = NULL,
## !    strata = NULL,
## !    link.covariates = NULL,
## !    verbose = FALSE,
## !    lincomb = NULL,
## !    selection = NULL,
## !    control.compute = list(),
## !    control.predictor = list(),
## !    control.family = list(),
## !    control.inla = list(),
## !    control.fixed = list(),
## !    control.mode = list(),
## !    control.expert = list(),
## !    control.hazard = list(),
## !    control.lincomb = list(),
## !    control.update = list(),
## !    control.pardiso = list(),
## !    only.hyperparam = FALSE,
## !    inla.call = inla.getOption("inla.call"),
## !    inla.arg = inla.getOption("inla.arg"),
## !    num.threads = inla.getOption("num.threads"),
## !    blas.num.threads = inla.getOption("blas.num.threads"),
## !    keep = inla.getOption("keep"),
## !    working.directory = inla.getOption("working.directory"),
## !    silent = inla.getOption("silent"),
## !    inla.mode = c("classic", "twostage", "experimental"), 
## !    debug = inla.getOption("debug"),
## !    .parent.frame = parent.frame()
## !    )
## ! }
## ! \arguments{
## ! \item{formula}{ A \code{inla} formula like \code{y
## ! ~1 + z + f(ind, model="iid")} + f(ind2,
## ! weights, model="ar1") This is much like the formula
## ! for a \code{glm} except that smooth or spatial terms
## ! can be added to the right hand side of the formula.
## ! See \code{\link{f}} for full details and the web site
## ! \url{www.r-inla.org} for several worked out
## ! examples. Each smooth or spatial term specified
## ! through \code{f} should correspond to separate column
## ! of the data frame \code{data}.
## ! The response
## ! variable, \code{y} can be a univariate response
## ! variable, a list or the output of the function
## ! \code{inla.surf} for survival analysis models.}

## ! \item{family}{ A string indicating the likelihood
## ! family. The default is \code{gaussian} with identity
## ! link. See \code{names(inla.models()$likelihood)} for a
## ! list of possible alternatives and use \code{\link{inla.doc}}
## ! for detailed docs for individual families.}

## ! \item{contrasts}{Optional contrasts for the fixed
## ! effects; see \code{?lm} or \code{?glm} for details.}

## ! \item{data}{ A data frame or list containing the
## ! variables in the model.  The data frame MUST be
## ! provided}

## ! \item{quantiles}{ A vector of quantiles,
## ! \eqn{p(0), p(1),\dots}{p(0), p(1),\ldots} to compute
## ! for each posterior marginal. The function returns,
## ! for each posterior marginal, the values
## ! \eqn{x(0), x(1),\dots}{x(0), x(1),\ldots} such that
## ! \deqn{\mbox{Prob}(X<x(p))=p}{Prob(X<x)=p} }

## ! \item{E}{ Known component in the mean for the Poisson
## ! likelihoods defined as \deqn{E_i\exp(\eta_i)}{E
## ! exp(eta)} where \deqn{\eta_i}{eta} is the linear
## ! predictor. If not provided it is set to \code{rep(1, n.data)}.}

## ! \item{offset}{This argument is used to specify an
## ! a-priori known and fixed component to be included in
## ! the linear predictor during fitting.  This should be
## ! \code{NULL} or a numeric vector of length either one
## ! or equal to the number of cases. One or more
## ! \code{offset()} terms can be included in the formula
## ! instead or as well, and if both are used, they are
## ! combined into a common offset.  If the
## ! \code{A}-matrix is used in the linear predictor
## ! statement \code{control.predictor}, then the
## ! \code{offset} given in this argument is added to
## ! \code{eta*}, the linear predictor related to the
## ! observations, as \code{eta* = A eta + offset},
## ! whereas an offset in the formula is added to
## ! \code{eta}, the linear predictor related to the
## ! formula, as \code{eta = ... + offset.formula}. So in
## ! this case, the offset defined here and in the formula
## ! has a different meaning and usage.}

## ! \item{scale}{ Fixed (optional) scale parameters of
## ! the precision for Gaussian and Student-T response
## ! models. Default value is rep(1, n.data).}

## ! \item{weights}{ Fixed (optional) weights parameters of
## ! the likelihood, so the log-likelihood[i] is changed into
## ! weights[i]*log-likelihood[i]. Default value is rep(1,
## ! n.data). 
## ! WARNING: The normalizing constant for the likelihood is NOT recomputed, so
## ! ALL marginals (and the marginal likelihood) must be interpreted with great care.}

## ! \item{Ntrials}{A vector containing the number of trials for the \code{binomial}
## ! likelihood and variantes, or the number of required successes for the
## ! \code{nbinomial2} likelihood. Default value is \code{rep(1, n.data)}.}

## ! \item{strata}{Fixed (optional) strata indicators
## ! for tstrata likelihood model.}

## ! \item{link.covariates}{A vector or matrix with covariates for link functions}

## ! \item{verbose}{
## ! Boolean indicating if the \code{inla}-program should
## ! run in a verbose mode (default \code{FALSE}).}

## ! \item{lincomb}{Used to define linear combination of
## ! nodes in the latent field. The posterior distribution
## ! of such linear combination is computed by the
## ! \code{inla} function. See
## ! \url{www.r-inla.org/faq} for examples of
## ! how to define such linear combinations.}

## ! \item{selection}{ This is a similar argument to the one in
## ! \code{inla.posterior.sample}  and follow the same format.
## ! This argument allows to define a subset of the latent field
## ! for which to compute an approximated joint
## ! distribution. It will appear in \code{result$selection}.
## ! See also \code{?inla.rjmarginal}. }

## ! \item{control.compute}{ See \code{?control.compute}}

## ! \item{control.predictor}{ See
## ! \code{?control.predictor}}

## ! \item{control.family}{ See \code{?control.family}}

## ! \item{control.inla}{ See \code{?control.inla}}

## ! \item{control.fixed}{ See \code{?control.fixed}}

## ! \item{control.mode}{ See \code{?control.mode}}

## ! \item{control.expert}{ See \code{?control.expert}}

## ! \item{control.hazard}{ See \code{?control.hazard}}

## ! \item{control.lincomb}{ See \code{?control.lincomb}}

## ! \item{control.update}{ See \code{?control.update}}

## ! \item{control.pardiso}{ See \code{?control.pardiso}}

## ! \item{only.hyperparam}{ A boolean variable saying if
## ! only the hyperparameters should be computed. This option is mainly used
## ! internally. (TODO: This option should not be located here,  change it!)}

## ! \item{inla.call}{ The path to, or the name of, the
## ! \code{inla}-program. This is program is installed
## ! together with the \code{R}-package, but, for example,
## ! a native compiled version can be used instead to
## ! improve the performance.}

## ! \item{inla.arg}{ A string indicating ALL arguments to
## ! the 'inla' program and do not include default
## ! arguments. (OOPS: This is an expert option!)}

## ! \item{num.threads}{ Maximum number of threads the
## ! \code{inla}-program will use, or as 'A:B' defining the number threads in the
## ! outer (A) and inner (B) layer for nested parallelism. }

## ! \item{blas.num.threads}{The absolute value of \code{blas.num.threads} is the maximum
## ! number of threads the the \code{openblas}/\code{mklblas} will use (if
## ! available). Value is ignored if \code{<=0} (then environment variables are used).}

## ! \item{keep}{ A boolean variable indicating that the
## ! working files (ini file, data files and results
## ! files) should be kept. If TRUE and no
## ! \code{working.directory} is specified the working
## ! files are stored in a directory called "inla".  }

## ! \item{working.directory}{ A string giving the name
## ! of an non-existing directory where to store the
## ! working files.}

## ! \item{silent}{If equal to 1L or TRUE, then the
## ! \code{inla}-program would be ``silent''. If equal to
## ! 2L, then supress also error messages from the
## ! \code{inla}-program.}

## ! \item{inla.mode}{Run \code{inla} in \code{classic}-mode,
## !             \code{twostage}-mode or the new \code{experimental}-mode?
## !             Not all features are available int the \code{experimental}-mode and this mode
## !             is really work-in-progress at the moment!!!!
## !             Default is to use the 
## !             mode set by \code{inla.getOption("inla.mode")} which is default
## !             \code{classic}-mode. }

## ! \item{debug}{ If \code{TRUE}, then enable some debug
## ! output.  }

## ! \item{.parent.frame}{Internal use only}
## ! }

## ! \value{%%

## ! \code{inla} returns an object of class \code{"inla"}. This is a list
## ! containing at least the following arguments:

## ! \item{summary.fixed}{Matrix containing the mean and standard
## ! deviation (plus, possibly quantiles and cdf) of the the fixed
## ! effects of the model.}

## ! \item{marginals.fixed}{
## ! A list containing the posterior marginal
## ! densities of the fixed effects of the model.}

## ! \item{summary.random}{List of matrices containing the mean and
## ! standard deviation (plus, possibly quantiles and cdf) of the
## ! the smooth or spatial effects defined through \code{f()}.}

## ! \item{marginals.random}{A list containing the
## ! posterior marginal densities of the random effects defined
## ! through \code{f}.}

## ! \item{summary.hyperpar}{
## ! A matrix containing the mean and sd
## ! (plus, possibly quantiles and cdf) of the hyperparameters of
## ! the model }

## ! \item{marginals.hyperpar}{
## ! A list containing the posterior marginal
## ! densities of the hyperparameters of the model.}

## ! \item{summary.linear.predictor}{
## ! A matrix containing the mean and sd
## ! (plus, possibly quantiles and cdf) of the linear predictors
## ! \eqn{\eta} in
## ! the model
## ! }

## ! \item{marginals.linear.predictor}{
## ! If \code{compute=TRUE} in
## ! \code{control.predictor}, a list containing the posterior
## ! marginals of the linear predictors \eqn{\eta} in the model.
## ! }

## ! \item{summary.fitted.values}{ A matrix containing the mean and
## ! sd (plus, possibly quantiles and cdf) of the fitted values
## ! \eqn{g^{-1}(\eta)} obtained by transforming the linear
## ! predictors by the inverse of the link function. This quantity
## ! is only computed if \code{marginals.fitted.values} is
## ! computed. Note that if an observation is \code{NA} then the
## ! identity link is used. You can manually transform a marginal
## ! using \code{inla.marginal.transform()} or set the argument
## ! \code{link} in the \code{control.predictor}-list;
## ! see \code{?control.predictor}
## ! }

## ! \item{marginals.fitted.values}{
## ! If \code{compute=TRUE} in
## ! \code{control.predictor}, a list containing the posterior
## ! marginals  of the fitted values
## ! \eqn{g^{-1}(\eta)} obtained by
## ! transforming the linear predictors by the inverse of the link
## ! function.
## ! Note that if an observation is \code{NA} then the
## ! identity link is used. You can manually transform a marginal
## ! using \code{inla.marginal.transform()} or set the argument
## ! \code{link} in the \code{control.predictor}-list;
## ! see \code{?control.predictor}
## ! }

## ! \item{summary.lincomb}{
## ! If \code{lincomb != NULL} a list of
## ! matrices containing the mean and sd (plus, possibly quantiles
## ! and cdf) of all linear combinations defined.  }

## ! \item{marginals.lincomb}{
## ! If \code{lincomb != NULL} a list of
## ! posterior marginals of all linear combinations defined.  }

## ! \item{selection}{Provide the approximated joint
## ! distribution for the \code{selection}}

## ! \item{dic}{
## ! If \code{dic}=\code{TRUE} in \code{control.compute}, the
## ! deviance information criteria and effective number of parameters,
## ! otherwise \code{NULL}
## ! }

## ! \item{cpo}{
## ! If \code{cpo}=\code{TRUE} in \code{control.compute}, a list
## ! of three elements: \code{cpo$cpo} are the values of the conditional
## ! predictive ordinate (CPO), \code{cpo$pit} are the values of the
## ! probability integral transform (PIT) and \code{cpo$failure}
## ! indicates whether some assumptions are violated. In short, if
## ! cpo$failure[i] > 0 then some assumption is violated, the higher the
## ! value (maximum 1) the more seriously.
## ! }

## ! \item{po}{
## ! If \code{po}=\code{TRUE} in \code{control.compute}, a list
## ! of one elements: \code{po$po} are the values of the
## ! predictive ordinate (CPO) (\code{pi(yi|y)})
## ! }

## ! \item{waic}{
## ! If \code{waic}=\code{TRUE} in \code{control.compute}, a list
## ! of two elements: \code{waic$waic} is the Watanabe-Akaike information criteria,  and
## ! \code{waic$p.eff} is the estimated effective number of parameters
## ! }

## ! \item{mlik}{
## ! If \code{mlik}=\code{TRUE} in \code{control.compute}, the
## ! log marginal likelihood of the model (using two different estimates), otherwise \code{NULL}
## ! }

## ! \item{neffp}{
## ! Expected effective number of parameters in the model. The
## ! standard deviation of the expected number of parameters and the
## ! number of replicas for parameter are also returned}

## ! \item{mode}{
## ! A list of two elements: \code{mode$theta} is the
## ! computed mode of the hyperparameters and \code{mode$x} is the
## ! mode of the latent field given the modal value of the
## ! hyperparamters.
## ! }

## ! \item{call}{
## ! The matched call.}

## ! \item{formula}{
## ! The formula supplied}

## ! \item{nhyper}{
## ! The number of hyperparameters in the model}

## ! \item{cpu.used}{
## ! The cpu time used by the \code{inla} function}
## ! }

## ! \references{
## ! Rue, H. and Martino, S. and Chopin, N. (2009)
## ! \emph{Approximate Bayesian Inference for latent Gaussian models
## ! using Integrated Nested Laplace Approximations, JRSS-series B
## ! (with discussion)}, vol 71, no 2, pp 319-392.
## ! Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields
## ! - Theory and Applications} Chapman and Hall}
## ! \author{Havard Rue \email{hrue@r-inla.org} and Sara Martino}
## ! \seealso{\code{\link{f}},
## ! \code{\link{inla.hyperpar}} }
## ! \examples{
## ! \dontrun{
## ! ##See the web page \url{www.r-inla.org} for a series of worked out examples
## ! }
## ! }

`inla` <- function(formula,
                   family = "gaussian",
                   contrasts = NULL,
                   data,
                   quantiles = c(0.025, 0.5, 0.975),
                   E = NULL,
                   offset = NULL,
                   scale = NULL,
                   weights = NULL,
                   Ntrials = NULL,
                   strata = NULL,
                   link.covariates = NULL,
                   verbose = FALSE,
                   lincomb = NULL,
                   selection = NULL,
                   control.compute = list(),
                   control.predictor = list(),
                   control.family = list(),
                   control.inla = list(),
                   control.fixed = list(),
                   control.mode = list(),
                   control.expert = list(),
                   control.hazard = list(),
                   control.lincomb = list(),
                   control.update = list(),
                   control.pardiso = list(),
                   only.hyperparam = FALSE,
                   inla.call = inla.getOption("inla.call"),
                   inla.arg = inla.getOption("inla.arg"),
                   num.threads = inla.getOption("num.threads"),
                   blas.num.threads = inla.getOption("blas.num.threads"),
                   keep = inla.getOption("keep"),
                   working.directory = inla.getOption("working.directory"),
                   silent = inla.getOption("silent"),
                   inla.mode = c("classic", "twostage", "experimental"), 
                   debug = inla.getOption("debug"),
                   .parent.frame = parent.frame()) {

    ## This will prevent values of 'OutDec' not '.' to cause error, as we create the Model.ini
    ## file with cat().
    old.options <- options()
    on.exit(options(old.options))
    options(OutDec = ".")

    my.time.used <- numeric(4)
    my.time.used[1] <- Sys.time()

    ## keep track of all priors and keep the original data
    all.hyper <- list()
    data.orig <- data

    if (missing(formula)) {
        stop("Usage: inla(formula, family, data, ...); see ?inla\n")
    }

    if (missing(data)) {
        stop("Missing data.frame/list `data'. Leaving `data' empty might lead to\n\t\tuncontrolled behaviour, therefore is it required.")
    }
    if (!is.data.frame(data) && !is.list(data)) {
        stop("\n\tArgument `data' must be a data.frame or a list.")
    }

    ## replace alias's
    family.alias <- list(
        list(from = "normal", to = "gaussian")
    )
    for (i in seq_along(family.alias)) {
        family[which(inla.trim.family(family) %in% family.alias[[i]]$from)] <- family.alias[[i]]$to
    }

    ## if data is a list, then it can contain elements that defines a
    ## model, like f(idx, model = model.objects). These objects crash
    ## the formula routines in R since then the data cannot be cast
    ## into a data.frame. to solve this, we remove such objects from
    ## data, and create a second data-object, data.model, which hold
    ## these.
    data.model <- NULL
    if (is.list(data) && length(data) > 0L) {

        ## first check if all elements have names
        if (any(nchar(names(data)) == 0L)) {
            stop(paste("Elements in the 'data'-list has no name: data[[k]], for k=c(", inla.paste(which(nchar(names(data)) == 0L), sep = ","), ")."))
        }

        i.remove <- c()
        for (i in 1:length(data)) {
            ## these are the objects which we want to remove:
            ## inla.model.object.classes()
            if (any(inherits(data[[i]], inla.model.object.classes()))) {
                add.to <- list(data[[i]])
                names(add.to) <- names(data)[i]
                data.model <- c(data.model, add.to)
                i.remove <- c(i.remove, i)
            }
        }
        names.remove <- names(data)[i.remove]
        for (nm in names.remove) {
            idx <- which(names(data) == nm)
            data[[idx]] <- NULL
        }
    }

    if (missing(inla.mode)) {
        inla.mode <- inla.getOption("inla.mode")
    }
    inla.mode <- match.arg(inla.mode)

    ## check all control.xx arguments here. do the assign as variable
    ## expansion might occur.
    control.compute <- inla.check.control(control.compute, data)
    control.predictor <- inla.check.control(control.predictor, data)
    ## I need to check for NA's already here.
    if (!is.null(control.predictor$A)) {
        control.predictor$A <- inla.as.sparse(control.predictor$A, na.rm = TRUE, zeros.rm = TRUE)
    }
    ## do not check control.family here, as we need to know n.family
    control.inla <- inla.check.control(control.inla, data)
    control.fixed <- inla.check.control(control.fixed, data)
    control.mode <- inla.check.control(control.mode, data)
    control.expert <- inla.check.control(control.expert, data)
    control.hazard <- inla.check.control(control.hazard, data)
    control.lincomb <- inla.check.control(control.lincomb, data)
    control.update <- inla.check.control(control.update, data)
    control.pardiso <- inla.check.control(control.pardiso, data)

    n.family <- length(family)
    for (i in 1:n.family) {
        family[i] <- inla.trim.family(family[i])
        ## check if this family is legal.
        if (is.null(inla.model.properties(family[i], "likelihood"))) {
            stop(paste("Unknown family: ", family[i], ". Do `names(inla.models()$likelihood)' to list available families.", sep = ""))
        }
    }

    ## if the user specify inla.call="remote" or "inla.remote" then
    ## use the internal inlaprogram
    remote <- FALSE
    submit <- FALSE
    ownfun <- FALSE
    submit.id <- ""

    if (is.function(inla.call)) {
        ## in this case, the responsibility is with the user
        ownfun <- TRUE
    } else if (inla.strcasecmp(inla.call, "remote") ||
        inla.strcasecmp(inla.call, "inla.remote") ||
        length(grep("/inla.remote$", inla.call)) > 0 ||
        length(grep("/inla.remote.cygwin$", inla.call)) > 0) {
        remote <- TRUE
        inla.call <- system.file("bin/remote/inla.remote", package = "INLA")
        if (inla.os("windows")) {
            inla.call <- paste(inla.call, ".cygwin", sep = "")
        }
    } else if (inla.strcasecmp(inla.call, "submit") ||
        inla.strcasecmp(inla.call, "inla.submit") ||
        length(grep("/inla.submit$", inla.call)) > 0) {
        remote <- TRUE
        submit <- TRUE
        submit.id <- paste(gsub("[ :]", "-", date()), "---", as.integer(runif(1, min = 1E8, max = 1E9 - 1)), sep = "")
        inla.call <- system.file("bin/remote/inla.submit", package = "INLA")
        if (inla.os("windows")) {
            inla.call <- paste(inla.call, ".cygwin", sep = "")
        }
    }

    ## Need to do this here.
    cont.fixed <- inla.set.control.fixed.default()
    cont.fixed[names(control.fixed)] <- control.fixed

    ##
    ## check for survival model with a baseline-hazard. if so, then
    ## expand the data-frame and call inla() again.
    ##
    have.surv <- FALSE
    cont.hazard <- NULL
    for (i in 1:n.family) {
        have.surv <- have.surv || inla.model.properties(family[i], "likelihood")$survival
    }

    if (have.surv && (inla.one.of(family, c("coxph")))) {
        ## This is not supported yet.
        stopifnot(is.null(control.predictor$A))

        cph <- inla.coxph(formula, data, control.hazard, debug = debug)
        result <- inla(
            cph$formula,
            family = cph$family,
            data = c(as.list(cph$data), cph$data.list),
            contrasts = contrasts,
            quantiles = quantiles,
            E = cph$E,
            offset = offset,
            scale = scale,
            weights = weights,
            Ntrials = NULL, # Not used for the poisson
            strata = NULL, # Not used for the poisson
            lincomb = lincomb,
            selection = selection,
            verbose = verbose,
            control.compute = control.compute,
            control.predictor = control.predictor,
            control.family = control.family,
            control.inla = control.inla,
            control.fixed = control.fixed,
            control.mode = control.mode,
            control.expert = control.expert,
            control.hazard = control.hazard,
            control.lincomb = control.lincomb,
            control.update = control.update,
            control.pardiso = control.pardiso,
            only.hyperparam = only.hyperparam,
            inla.call = inla.call,
            inla.arg = inla.arg,
            num.threads = num.threads,
            blas.num.threads = blas.num.threads,
            keep = keep,
            working.directory = working.directory,
            silent = silent,
            inla.mode = inla.mode, 
            debug = debug
        )

        ## replace the argument so it can be reused, if...
        result$call.orig <- deparse(match.call())
        return(result)
    }

    ## this is nice hack ;-) we keep the original response. then we
    ## delete it from 'data' keeping a copy of the original one
    y...orig <- eval(parse(text = formula[2]), data)

    if (n.family > 1) {
        y...orig <- inla.as.list.of.lists(y...orig)
        ny <- max(sapply(y...orig, function(xx) if (is.list(xx)) max(sapply(xx, length)) else length(xx)))
        nc <- length(y...orig)
        if (n.family != nc) {
            stop(paste(
                "Number of families", n.family,
                "does not match number of response variables", nc
            ))
        }
    } else {
        nc <- NULL ## not in use
        if (inherits(y...orig, "inla.surv")) {
            class(y...orig) <- NULL
            ## this one is not passed along
            y...orig$.special <- NULL
            ny <- max(sapply(y...orig, length))
        } else if (inherits(y...orig, "inla.mdata")) {
            class(y...orig) <- NULL
            ny <- max(sapply(y...orig, length))
        } else {
            if (length(dim(y...orig)) == 2) {
                ## some matrix type, could be a sparse matrix
                stopifnot(dim(y...orig)[2] == 1)
                y...orig <- c(drop(as.matrix(y...orig)))
            }
            ny <- length(y...orig)
        }
    }

    data <- inla.remove(as.character(formula[2]), data)
    if (!is.null(control.predictor$A)) {
        MN <- inla.sparse.dim(control.predictor$A)
        MPredictor <- MN[1]
        NPredictor <- MN[2]
        stopifnot(MPredictor == ny)
    } else {
        MPredictor <- 0L
        NPredictor <- ny
    }
    NData <- inla.ifelse(MPredictor > 0, MPredictor, NPredictor)
    stopifnot(NData > 0)
    stopifnot(NPredictor > 0)
    if (debug) {
        print(paste("MPredictor", MPredictor))
        print(paste("NPredictor", NPredictor))
        print(paste("NData", NPredictor))
    }

    if (n.family == 1) {
        y...fake <- c(rep(Inf, NData))
        if (debug) {
            print(paste("y...fake has length", length(y...fake)))
        }
    } else {
        y...fake <- c(rep(Inf, NData))
        if (debug) {
            print(paste("y...fake has length", length(y...fake)))
        }
    }

    if (FALSE) {
        if (n.family != 1) {
            stop(paste("length(family) are", n.family, "but the response has only one column!"))
        }

        y...fake <- c(y...orig)
        y...fake[is.na(y...fake)] <- Inf ## otherwise model.matrix() fails below
    }
    if (debug) {
        cat("n.family", n.family, "\n")
    }

    ## ...and then we add the fake to the data-frame after removing the original response
    if (is.data.frame(data)) {
        if (MPredictor > 0) {
            if (MPredictor != NPredictor) {
                stop(paste(
                    "It can be ``dangerous'' to use a 'data.frame' as data, when the\n",
                    "\tA-matrix is not a square matrix; please pass the data\n",
                    "\tas a list: data = list(...)."
                ))
            }
        }
        data <- as.data.frame(c(as.data.frame(data), list(y...fake = y...fake)))
    } else {
        data <- c(data, list(y...fake = y...fake))
        if (MPredictor > 0 && (MPredictor > NPredictor)) {
            data$y...fake <- data$y...fake[1:NPredictor]
        }
        if (MPredictor > 0 && (MPredictor < NPredictor)) {
            data$y...fake <- c(data$y...fake, rep(Inf, NPredictor - MPredictor))
        }
    }

    data.same.len <- inla.fix.data(data, NPredictor)
    if (debug) {
        print(c("Entries with same length:", names(data.same.len)))
    }

    ## creates a new version of ``formula'' and keep the old one for reference
    formula.orig <- formula
    ## inla.eval(paste("formula = y...fake ~ ", inla.formula2character(formula.orig[3])))
    formula <- update.formula(formula, y...fake ~ .)
    ## parse the formula
    gp <- inla.interpret.formula(formula,
        data.same.len = data.same.len, data = data,
        data.model = data.model, parent.frame = .parent.frame
    )
    call <- deparse(match.call())

    ## issue a warning if the intercept is spesified while the
    ## control.predictor A matrix is used.
    if (length(grep("\\+ ?1($| )", gp$fixf)) && !is.null(control.predictor$A)) {
        warning("The A-matrix in the predictor (see ?control.predictor) is specified
  but an intercept is in the formula. This will likely result
  in the intercept being applied multiple times in the model, and is likely
  not what you want. See ?inla.stack for more information.
  You can remove the intercept adding ``-1'' to the formula.")
    }

    if (gp$n.fix > 0) {
        ## use y...fake also here (which is the same as in gp$fixf[2])
        ## and construct the model.matrix().
        new.fix.formula <- gp$fixf
        ## inla.eval(paste("new.fix.formula = y...fake ~ ", inla.formula2character(gp$fixf[3])))
        new.fix.formula <- update.formula(new.fix.formula, y...fake ~ .)

        ## replace NA's in covariates with 0. Ignore factors. NA's in
        ## factors is done afterwards.
        inla.na.action <- function(x, ...) {
            for (k in seq_along(x)) {
                if ((is.numeric(x[[k]]) || inla.is.matrix(x[[k]])) && !is.factor(x[[k]])) {
                    x[[k]][is.na(x[[k]])] <- 0
                }
            }
            return(na.pass(x))
        }
        if (inla.one.of(cont.fixed$expand.factor.strategy, "inla")) {
            ## expand all factors as matrices, exclude possible
            ## factors in the f()'s. it is ok to include all terms in
            ## the f()'s as duplicated names are not allowed.
            exclude.names <- c()
            for (k in seq_along(gp$random.spec)) {
                exclude.names <- c(exclude.names, gp$random.spec[[k]]$label)
            }
            data.same.len <- inla.expand.factors(data.same.len, exclude.names)
        } else if (inla.one.of(cont.fixed$expand.factor.strategy, "model.matrix")) {
            ## do nothing
        } else {
            stop(paste(
                "Unknown value for flag 'expand.factor.strategy' in 'control.fixed':",
                cont.fixed$expand.factor.strategy
            ))
        }
        if (inla.require("MatrixModels")) {
            gp$model.matrix <- MatrixModels::model.Matrix(
                new.fix.formula,
                data = model.frame(new.fix.formula, data.same.len, na.action = inla.na.action),
                contrasts.arg = contrasts, sparse = TRUE
            )
        } else {
            gp$model.matrix <- model.matrix(
                new.fix.formula,
                data = model.frame(new.fix.formula, data.same.len, na.action = inla.na.action),
                contrasts.arg = contrasts
            )
        }
        ## as NA's in factors are not set to zero in
        ## 'inla.na.action'. Do that here if the strategy is 'inla',
        ## otherwise signal an error.
        if (any(is.na(gp$model.matrix))) {
            if (inla.one.of(cont.fixed$expand.factor.strategy, "inla")) {
                gp$model.matrix[is.na(gp$model.matrix)] <- 0
            } else {
                stop(paste(
                    "With control.fixed = list(expand.factor.strategy='model.matrix'),",
                    "then NA's in factor are not allowd. Please use strategy 'inla' instead."
                ))
            }
        }

        ## remove names from the model.matrix. This is useful if we need a model without an
        ## intercept as we might have the intercept in the link-model f.ex. In this case we want
        ## to do the expansion of factors as we have an intercept and then remove it afterwards.
        if (!is.null(cont.fixed$remove.names)) {
            rm.cols <- which(cont.fixed$remove.names %in% colnames(gp$model.matrix))
            if (length(rm.cols) > 0) {
                gp$model.matrix <- gp$model.matrix[, -rm.cols, drop = FALSE]
            }
            if (length(cont.fixed$remove.names) > length(rm.cols)) {
                warning(paste0(
                    "You requested to remove ", length(cont.fixed$remove.names),
                    " names in 'data', but only ", length(rm.cols), " names were removed."
                ))
            }
        }

        ## this have to match
        stopifnot(dim(gp$model.matrix)[1L] == NPredictor)

        if (any(is.infinite(gp$model.matrix))) {
            n.infs <- sum(is.infinite(gp$model.matrix))
            stop(paste("There are", n.infs, "Inf's in the model.matrix. This is not allowed."))
        }

        ## n.fix can have been changed here due to a `-1'
        gp$n.fix <- dim(gp$model.matrix)[2L]

        ## if reqested, then setup the fixed effects as linear
        ## combinations, so we can compute their posterior correlation
        ## matrix. so the lincombs are 'inla.make.lincomb(z=1)'
        ## etc. we need the quotes \"..\" for the "(Intercept)".
        if (!is.null(control.fixed$correlation.matrix) &&
            control.fixed$correlation.matrix && !is.null(gp$model.matrix)) {
            fix.names <- colnames(gp$model.matrix)
            lc.all.fix <- c()
            for (fix.name in fix.names) {
                lc.fix <- inla.eval(paste("inla.make.lincomb(\"", fix.name, "\"=1)", sep = ""))
                names(lc.fix) <- fix.name
                lc.all.fix <- c(lc.all.fix, lc.fix)
            }
            ## if we have defined lincomb's also in the inla call,
            ## then simply append the fixed-effects at the end,
            ## otherwise set the 'lincomb' argument. however. for this
            ## to work with repeated calls (like inla.hyperpar()) we
            ## have to turn off control.fixed$correlation.matrix as we
            ## have moved that part into 'lincomb'. also, we need to
            ## make sure that we actually compute the correlation
            ## matrix for the linear combinations in control.inla$...
            lincomb <- c(lincomb, lc.all.fix)
            control.fixed$correlation.matrix <- FALSE
            control.inla$lincomb.derived.correlation.matrix <- TRUE
        }
    } else {
        gp$model.matrix <- NULL
    }

    if (!is.null(selection)) {
        ## convert the selection into lincomb's.
        lincomb.sel <- inla.selection2lincombs(selection)
        lincomb <- c(lincomb, lincomb.sel)
        ## both these are required.
        control.inla$lincomb.derived.correlation.matrix <- TRUE
        control.compute$config <- TRUE
    }

    ## control what should be computed
    cont.compute <- inla.set.control.compute.default()
    cont.compute$dic <- cont.compute$cpo <- cont.compute$po <- cont.compute$waic <- FALSE
    cont.compute[names(control.compute)] <- control.compute
    if (only.hyperparam) {
        cont.compute$hyperpar <- TRUE
        cont.compute$dic <- cont.compute$cpo <- cont.compute$po <- cont.compute$waic <- FALSE
    }

    ## control inla
    cont.inla <- cont.inla.def <- inla.set.control.inla.default(family = family)
    cont.inla[names(control.inla)] <- control.inla
    ## because we have 'control' within a 'control', we have to process them spesifically
    cont.inla$control.vb <- cont.inla.def$control.vb
    cont.inla$control.vb[names(control.inla$control.vb)] <- control.inla$control.vb

    ## control predictor section
    cont.predictor <- inla.set.control.predictor.default()
    cont.predictor[names(control.predictor)] <- control.predictor
    cont.predictor$hyper <- inla.set.hyper(
        "predictor", "predictor",
        cont.predictor$hyper, cont.predictor$initial,
        cont.predictor$fixed, cont.predictor$prior, cont.predictor$param
    )
    all.hyper$predictor$hyper <- cont.predictor$hyper
    if (cont.compute$cpo || cont.compute$dic || cont.compute$po || cont.compute$waic ||
        !is.null(cont.predictor$link) ||
        (is.character(cont.inla$control.vb$enable) || cont.inla$control.vb$enable)) {
        cont.predictor$compute <- TRUE
    }
    if (only.hyperparam) {
        cont.predictor$compute <- FALSE
        cont.predictor$return.marginals <- FALSE
        cont.predictor$return.marginals.predictor <- FALSE
        cont.predictor$cdf <- cont.predictor$quantiles <- NULL
    }

    ## control.family
    control.family.orig <- control.family
    if (n.family == 1) {
        if (!missing(control.family) && (inla.is.list.of.lists(control.family) && length(control.family) > 1L)) {
            stop(paste("Argument 'control.family' does not match length(family)=", n.family))
        }
    } else {
        if (!missing(control.family) && !(inla.is.list.of.lists(control.family) && length(control.family) == n.family)) {
            stop(paste("Argument 'control.family' does not match length(family)=", n.family))
        }
    }

    if (missing(control.family)) {
        tt <- list(list())
        for (i in 1:n.family) {
            tt[[i]] <- control.family
        }
        control.family <- tt
    } else if (n.family == 1) {
        control.family <- list(control.family)
    }

    if (length(control.family) == 1 && n.family > 1) {
        stop(paste("length(control.family) = 1 while length(family) > 1."))
    }

    ## finally, do the check. Note that the name of the argument is
    ## also used in this function, so we need to borrow the name.
    control.family.save <- control.family
    for (ii in 1:n.family) {
        control.family <- control.family.save[[ii]]
        ## need to be able to say when n.family =  1
        ## control.family = list(
        ## list(hyper = list(), control.link = list())))
        while (is.list(control.family) &&
            length(control.family) > 0 &&
            is.null(names(control.family))) {
            control.family <- control.family[[1]]
        }
        control.family.save[[ii]] <- inla.check.control(control.family, data)
    }
    control.family <- control.family.save
    cont.family <- list(list())
    for (i.family in 1:n.family) {
        cont.family[[i.family]] <- inla.set.control.family.default()
        cont.family[[i.family]]$control.mix <- inla.set.control.mix.default()
        cont.family[[i.family]]$control.link <- inla.set.control.link.default()
        cont.family[[i.family]]$control.bgev <- inla.set.control.bgev.default()

        ## need to take option 'control.mix' and 'control.link' out and process it seperately
        c.mix <- control.family[[i.family]]$control.mix
        c.link <- control.family[[i.family]]$control.link
        c.bgev <- control.family[[i.family]]$control.bgev
        control.family[[i.family]]$control.mix <- NULL
        control.family[[i.family]]$control.link <- NULL
        control.family[[i.family]]$control.bgev <- NULL

        cont.family[[i.family]][names(control.family[[i.family]])] <- control.family[[i.family]]
        cont.family[[i.family]]$hyper <- inla.set.hyper(
            family[i.family],
            "likelihood",
            cont.family[[i.family]]$hyper,
            cont.family[[i.family]]$initial,
            cont.family[[i.family]]$fixed,
            cont.family[[i.family]]$prior,
            cont.family[[i.family]]$param
        )
        all.hyper$family[[i.family]] <- list(
            hyperid = paste("INLA.Data", i.family, sep = ""),
            label = family[i.family],
            hyper = cont.family[[i.family]]$hyper
        )

        cont.family[[i.family]]$control.mix[names(c.mix)] <- c.mix
        cont.family[[i.family]]$control.link[names(c.link)] <- c.link
        cont.family[[i.family]]$control.bgev[names(c.bgev)] <- c.bgev

        if (!is.null(cont.family[[i.family]]$control.mix$model)) {
            cont.family[[i.family]]$control.mix$hyper <- inla.set.hyper(
                cont.family[[i.family]]$control.mix$model,
                "mix",
                cont.family[[i.family]]$control.mix$hyper,
                cont.family[[i.family]]$control.mix$initial,
                cont.family[[i.family]]$control.mix$fixed,
                cont.family[[i.family]]$control.mix$prior,
                cont.family[[i.family]]$control.mix$param
            )
            all.hyper$family[[i.family]]$mix$hyper <- cont.family[[i.family]]$control.mix$hyper
        }
        cont.family[[i.family]]$control.link$hyper <- inla.set.hyper(
            cont.family[[i.family]]$control.link$model,
            "link",
            cont.family[[i.family]]$control.link$hyper,
            cont.family[[i.family]]$control.link$initial,
            cont.family[[i.family]]$control.link$fixed,
            cont.family[[i.family]]$control.link$prior,
            cont.family[[i.family]]$control.link$param
        )
        all.hyper$family[[i.family]]$link$hyper <- cont.family[[i.family]]$control.link$hyper
    }

    ## Create the directory where to store Model.ini and data.files
    ## and results.file
    if (!is.null(working.directory)) {
        keep <- TRUE
    }
    if (keep) {
        ## create the directory locally or whereever specified by the user
        if (is.null(working.directory)) {
            working.directory <- "inla.model"
            working.directory.start <- "inla.model"
        } else {
            working.directory.start <- working.directory
        }

        ## if this directory already exists, try the numbered versions
        kk <- 1
        ans <- file.exists(working.directory)
        while (ans) {
            working.directory <- paste(working.directory.start, "-", kk, sep = "")
            kk <- kk + 1
            ans <- file.exists(working.directory)
        }
        inla.dir <- working.directory
        tmp <- inla.dir.create(inla.dir, StopOnError = FALSE) ## return NULL if fail
        if (is.null(tmp)) {
            ## this is the failsafe-option
            inla.dir <- paste(inla.dir, "-", substring(as.character(runif(1)), 3), sep = "")
            tmp <- inla.dir.create(inla.dir, StopOnError = FALSE)
            if (is.null(tmp)) {
                stop(paste("Cannot create directory [", inla.dir,
                    "] even after trying a random dirname. I give up.",
                    sep = ""
                ))
            }
        }
        if (verbose) {
            cat("Model and results are stored in working directory [", inla.dir, "]\n", sep = "")
        }
    } else {
        ## create a temporary directory
        inla.dir <- inla.tempfile()
        inla.dir <- gsub("\\\\", "/", inla.dir)
        inla.dir.create(inla.dir)
    }
    ## Create a directory where to store data and results
    inla.dir <- normalizePath(inla.dir)
    data.dir <- paste(inla.dir, "/data.files", sep = "")
    results.dir <- paste(inla.dir, "/results.files", sep = "")
    inla.dir.create(data.dir)

    ## create the .file.ini and make the problem.section
    file.ini <- paste(inla.dir, "/Model.ini", sep = "")
    file.log <- paste(inla.dir, "/Logfile.txt", sep = "")
    file.log2 <- paste(inla.dir, "/Logfile2.txt", sep = "")

    ## problem section
    if (debug) {
        print("prepare problem section")
    }

    inla.problem.section(
        file = file.ini, data.dir = data.dir, result.dir = results.dir,
        hyperpar = cont.compute$hyperpar,
        return.marginals = cont.compute$return.marginals,
        return.marginals.predictor = cont.compute$return.marginals.predictor,
        dic = cont.compute$dic, mlik = cont.compute$mlik,
        cpo = cont.compute$cpo,
        ## these two are merged together as they are compute together
        po = (cont.compute$po || cont.compute$waic),
        quantiles = quantiles, smtp = cont.compute$smtp, q = cont.compute$q,
        openmp.strategy = cont.compute$openmp.strategy, graph = cont.compute$graph,
        config = cont.compute$config
    )

    ## PREPARE RESPONSE AND FIXED EFFECTS
    if (debug) {
        cat("Prepare inla file.....")
    }

    ## copy the argument-lists
    mf <- match.call(expand.dots = FALSE)
    mf$family <- NULL
    mf$quantiles <- NULL
    mf$verbose <- NULL
    mf$control.compute <- NULL
    mf$control.predictor <- NULL
    mf$silent <- NULL
    mf$mode <- NULL
    mf$control.hazard <- NULL
    mf$control.family <- NULL
    mf$control.update <- NULL
    mf$control.pardiso <- NULL
    mf$control.inla <- NULL
    mf$control.fixed <- NULL
    mf$control.lincomb <- NULL
    mf$control.mode <- NULL
    mf$control.expert <- NULL
    mf$inla.call <- NULL
    mf$num.threads <- NULL
    mf$blas.num.threads <- NULL
    mf$keep <- NULL
    mf$working.directory <- NULL
    mf$only.hyperparam <- NULL
    mf$debug <- NULL
    mf$contrasts <- NULL
    mf$inla.arg <- NULL
    mf$lincomb <- NULL
    mf$selection <- NULL
    mf$inla.mode <- NULL
    mf$.parent.frame <- NULL
    mf$data <- data.same.len

    if (gp$n.fix > 0) {
        mf$formula <- gp$fixf
    } else if (gp$n.random > 0) {
        mf$formula <- gp$randf
    } else {
        mf$formula <- y...fake ~ 1
    }

    ## these we need
    mf$na.action <- na.pass
    mf[[1]] <- as.name("model.frame")

    if (gp$n.random > 0) {
        rf <- mf ## for later use
        rf$weights <- rf$scale <- rf$Ntrials <- rf$offset <- rf$E <- rf$strata <- rf$link.covariates <- NULL ## these we do not need
        rf$formula <- gp$randf
        rf$data <- data.same.len
        rf <- eval.parent(rf)
    } else {
        rf <- NULL
    }

    if (gp$n.weights > 0) {
        wf <- mf
        wf$weights <- wf$scale <- wf$Ntrials <- wf$offset <- wf$E <- wf$strata <- wf$link.covariates <- NULL ## these we do not need
        wf$formula <- gp$weightf
        wf$data <- data.same.len
        wf <- eval.parent(wf)
    } else {
        wf <- NULL
    }

    ## We set these here, instead of::
    ## scale = model.extract(mf, "scale")
    ## Ntrials = model.extract(mf, "Ntrials")
    ## E = model.extract(mf, "E")
    ## offset = as.vector(model.extract(mf, "offset"))

    for (nm in c("scale", "weights", "Ntrials", "offset", "E", "strata", "link.covariates")) {
        inla.eval(paste("tmp = try(eval(mf$", nm, ", data, enclos = parent.frame()), silent=TRUE)", sep = ""))
        if (!is.null(tmp) && !inherits(tmp, "try-error")) {
            inla.eval(paste("mf$", nm, " = NULL", sep = ""))
            inla.eval(paste(nm, " = tmp"))
        } else {
            inla.eval(paste("tmp = try(eval.parent(mf$", nm, "), silent=TRUE)", sep = ""))
            if (!is.null(tmp) && !inherits(tmp, "try-error")) {
                inla.eval(paste("mf$", nm, " = NULL", sep = ""))
                inla.eval(paste(nm, " = tmp"))
            } else {
                ## this *is* defined by default,  as all variables are default NULL
                inla.eval(paste("mf$", nm, " = NULL", sep = ""))
                inla.eval(paste(nm, "= inla.eval(nm)", sep = ""))
            }
        }
    }

    ## as there are functions as well with this name....
    if (is.function(offset)) {
        offset <- NULL
    }
    if (is.function(scale)) {
        scale <- NULL
    }

    ## ## ## ## ## ## ## ## ## ## ## ##
    ## ## ## ## ## ## ## ## ## ## ## ##

    mf <- eval.parent(mf)
    indN <- seq(0L, NPredictor - 1L)
    indM <- seq(0L, MPredictor - 1L)
    indD <- seq(0L, NData - 1)

    ## this takes care of the offset: `offset' is the argument,
    ## `offset.formula' is in the formula. Note that 'offset' goes
    ## into eta* = A %*% eta + offset, whereas, eta = .... +
    ## offset.formula
    if (!is.null(gp$offset)) {
        ## there can be more offsets
        offset.formula <- 0
        for (i in 1:length(gp$offset)) {
            offset.formula <- offset.formula + as.vector(eval(parse(text = gp$offset[i]), data))
        }
    } else {
        offset.formula <- rep(0, NPredictor)
    }

    offset.len <- inla.ifelse(MPredictor > 0, MPredictor, NPredictor)
    offset.formula.len <- NPredictor
    if (is.null(offset)) {
        offset <- 0
    }
    if (length(offset) == 1L) {
        offset <- rep(offset, offset.len)
    }
    if (length(offset) != offset.len) {
        stop(paste("Length of argument 'offset' is wrong:", length(offset), "!=", offset.len))
    }
    if (length(offset.formula) != offset.formula.len) {
        stop(paste("Length of 'offset(...)' in the formula is wrong:", length(offset.formula), "!=", offset.formula.len))
    }

    if (length(family) == 1) {
        family <- rep(family, n.family)
    }

    ## Create a file with the response, each for each 'family'
    for (i.family in 1:n.family) {
        if (n.family == 1) {
            yy <- y...orig
        } else {
            yy <- y...orig[[i.family]]
        }

        if (MPredictor > 0) {
            if (is.list(yy)) {
                stopifnot(max(sapply(yy, length)) == MPredictor)
            } else if (inla.is.matrix(yy)) {
                stopifnot(dim(yy)[1L] == MPredictor)
            } else {
                stopifnot(length(yy) == MPredictor)
            }
        }

        if (!is.null(yy) && !(is.numeric(yy) || is.list(yy) || is.matrix(yy) || inla.is.matrix(yy) || all(is.na(yy)))) {
            stop(paste("The response for family[", i.family, "] is not of type 'numeric|list|matrix'; don't know what to do.", sep = ""))
        }

        ## we can check for 'survial' here, either if data is and family isn't,  or the oposite
        if (is.list(yy) &&
            all(is.element(names(yy), names(inla.surv(1, 1))))) {
            ## check if family is of survival type. if not, check if appending 'surv' is.
            if (!inla.model.properties(family[i.family], "likelihood")$survival) {
                new.fam <- paste0(family[i.family], "surv")
                ok <- inla.model.properties(new.fam, "likelihood", stop.on.error = FALSE)$survival
                if (!is.null(ok) && ok) {
                    warning(paste0(
                        "*** WARNING *** Input family is '", family[i.family],
                        "' but input data is of 'inla.surv(...)' type\n",
                        "  *** WARNING *** Do you ment to use family '", new.fam,
                        "' ?"
                    ))
                }
            }
        }
        if (inla.model.properties(family[i.family], "likelihood")$survival && !is.list(yy)) {
            ## check if removing 'surv' makes a valid family
            new.fam <- gsub("surv$", "", family[i.family])
            ok <- inla.model.properties(new.fam, "likelihood",
                stop.on.error = FALSE
            )$survival
            if (!is.null(ok) && !ok) {
                warning(paste0(
                    "*** WARNING *** Input family is '", family[i.family],
                    "' and require input of type 'inla.surv(...)'\n",
                    "  *** WARNING *** Do you ment to use family '", new.fam,
                    "' ?"
                ))
            }
        }

        files <- inla.create.data.file(
            y.orig = yy, mf = mf, E = E, scale = scale,
            weights = weights, Ntrials = Ntrials, strata = strata,
            family = family[i.family], data.dir = data.dir, file = file.ini, debug = debug
        )

        ## add a section to the file.ini
        prop <- inla.model.properties(family[i.family], "likelihood", stop.on.error = TRUE)

        if (debug) {
            print("prepare data section")
        }

        ## ....then create the new section
        inla.family.section(
            file = file.ini, family = family[[i.family]], file.data = files$file.data, file.weights = files$file.weights,
            file.attr = files$file.attr,
            control = cont.family[[i.family]], i.family = i.family, link.covariates = link.covariates, data.dir = data.dir
        )
    }

    ## create the PREDICTOR section.
    if (debug) {
        print("prepare predictor section")
    }

    stopifnot(!is.null(offset.formula) && !is.null(offset)) ## must be zeros if not used. this makes it easier
    offset.formula[is.na(offset.formula)] <- 0
    offset[is.na(offset)] <- 0
    if (!is.null(control.predictor$A)) {
        off <- cbind(c(indM, MPredictor + indN), c(as.vector(control.predictor$A %*% offset.formula + offset), offset.formula))
    } else {
        off <- cbind(indN, offset + offset.formula)
    }
    file.offset <- inla.tempfile(tmpdir = data.dir)
    inla.write.fmesher.file(as.matrix(off), filename = file.offset, debug = debug)
    file.offset <- gsub(data.dir, "$inladatadir", file.offset, fixed = TRUE)

    if (!is.null(cont.predictor$link)) {
        not.na <- which(!is.na(cont.predictor$link))
        if (any(cont.predictor$link[not.na] != as.integer(cont.predictor$link[not.na])) || !all(cont.predictor$link[not.na] %in% 1L:n.family)) {
            stop(paste("Argument 'control.predictor$link' must consist of integers from the set 1:", n.family, " or NA's.", sep = ""))
        }
        if (!all(is.na(cont.predictor$link))) {
            if (max(cont.predictor$link[not.na]) > n.family) {
                stop(paste("n.family =", n.family, "while argument 'control.predictor$link' has max=", max(cont.predictor$link[not.na])))
            }
        }
        if (!is.null(control.predictor$A)) {
            if (length(cont.predictor$link) == 1L) {
                ## shortcut
                cont.predictor$link <- rep(cont.predictor$link, MPredictor)
            }
            if (!(length(cont.predictor$link) == MPredictor || length(cont.predictor$link) == MPredictor + NPredictor)) {
                stop(paste("Length of argument 'control.predictor$link' is wrong: length(link) = ",
                    length(cont.predictor$link), ". Length must be equal to ",
                    "length(A%*%eta)=", MPredictor, " or length(c(A%*%eta,eta))=", MPredictor + NPredictor,
                    ".",
                    sep = ""
                ))
            }
            if (length(cont.predictor$link) == MPredictor) {
                tlink <- cbind(0L:(MPredictor + NPredictor - 1L), c(cont.predictor$link - 1L, rep(NA, NPredictor)))
            } else {
                tlink <- cbind(0L:(MPredictor + NPredictor - 1L), cont.predictor$link - 1L)
            }
        } else {
            if (length(cont.predictor$link) == 1L) {
                ## shortcut
                cont.predictor$link <- rep(cont.predictor$link, NPredictor)
            }
            if (!(length(cont.predictor$link) == NPredictor)) {
                stop(paste("Length of argument 'control.predictor$link' is wrong: length(link)=", length(control.predictor$link),
                    ". Length must be equal to ",
                    "length(eta)=", NPredictor, ".",
                    sep = ""
                ))
            }
            stopifnot(length(cont.predictor$link) == NPredictor)
            tlink <- cbind(indN, as.numeric(cont.predictor$link) - 1L)
        }
        file.link.fitted.values <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(as.matrix(tlink), filename = file.link.fitted.values, debug = debug)
        file.link.fitted.values <- gsub(data.dir, "$inladatadir", file.link.fitted.values, fixed = TRUE)
    } else {
        file.link.fitted.values <- NULL
    }

    inla.predictor.section(
        file = file.ini, n = NPredictor, m = MPredictor,
        predictor.spec = cont.predictor, file.offset = file.offset, data.dir = data.dir,
        file.link.fitted.values = file.link.fitted.values
    )

    all.labels <- character(0)
    if (!is.null(cont.predictor$compute) && cont.predictor$compute) {
        all.labels <- c(all.labels, "predictor")
    }

    ## FIXED EFFECTS
    if (gp$n.fix > 0) {
        nc <- ncol(gp$model.matrix)
        labels <- colnames(gp$model.matrix)
        for (i in 1:nc) {
            if (debug) {
                cat("write label[", labels[i], "]\n")
            }

            fixed.eff <- cbind(indN, as.numeric(gp$model.matrix[, i]))
            ## remove lines with NA
            fixed.eff <- fixed.eff[!is.na(fixed.eff[, 2]), , drop = FALSE]

            file.fixed <- inla.tempfile(tmpdir = data.dir)
            inla.write.fmesher.file(as.matrix(fixed.eff), filename = file.fixed, debug = debug)
            file.fixed <- gsub(data.dir, "$inladatadir", file.fixed, fixed = TRUE)

            if (debug) {
                cat("file fixed", file.fixed, "\n")
            }

            all.hyper$fixed[[i]] <-
                inla.linear.section(
                    file = file.ini, file.fixed = file.fixed, label = labels[i],
                    results.dir = paste("fixed.effect", inla.num(i), sep = ""),
                    control.fixed = cont.fixed, only.hyperparam = only.hyperparam
                )
        }
    }

    ## RANDOM EFFECT OR LINEAR EFFCT DEFINED VIA f()
    nr <- gp$n.random
    n.weights <- 0
    j <- 0
    extra.fixed <- 0

    if (nr > 0) {
        name.random.dir <- c()
        if (nr != (ncol(rf) - 1)) {
            stop("\n\tSOMETHING STRANGE: You probably used one variable as a covariate more than once.")
        }

        ## make the covariate numeric if its all NA (for which it is of type 'logical')
        stopifnot(ncol(rf) >= 2L)
        for (r in 1:(ncol(rf) - 1)) { ## YES, '-1' is correct
            idx <- rf[, r + 1]
            if (is.logical(idx) && all(is.na(idx))) {
                rf[, r + 1] <- idx <- as.numeric(idx)
            }
            if (all(is.na(idx))) {
                ## check that either 'n' or 'values' are given, to prevent errors from happening
                ## below.
                if (is.null(gp$random.spec[[r]]$values) && is.null(gp$random.spec[[r]]$n)) {
                    stop(paste("Model component 'f(",
                        gp$random.spec[[r]]$term,
                        ", ...)' have only NA values in '",
                        gp$random.spec[[r]]$term, "'. ",
                        "In this case, ",
                        "either argument 'n' or 'values' must be spesified.",
                        sep = ""
                    ))
                }
            }
        }

        location <- list()
        covariate <- list()

        count.linear <- 0
        count.random <- 0

        ## sort the models so copies are at the end
        if (nr > 1) {
            swapped <- TRUE
            while (swapped) {
                swapped <- FALSE
                for (r in 1:(nr - 1)) {
                    if (
                        ##
                        ## not both are copy, then copy is at the end
                        ##
                        (gp$random.spec[[r]]$model == "copy" && gp$random.spec[[r + 1]]$model != "copy") ||
                            ##
                            ## both are copy, then 'same.as' is at the end
                            ##
                            (gp$random.spec[[r]]$model == "copy" && gp$random.spec[[r + 1]]$model == "copy" &&
                                !is.null(gp$random.spec[[r]]$same.as) && is.null(gp$random.spec[[r + 1]]$same.as))) {
                        tmp <- gp$random.spec[[r]]
                        gp$random.spec[[r]] <- gp$random.spec[[r + 1]]
                        gp$random.spec[[r + 1]] <- tmp

                        tmp <- rf[, r + 1]
                        rf[, r + 1] <- rf[, r + 2]
                        rf[, r + 2] <- tmp

                        swapped <- TRUE
                    }
                }
            }
        }
        if (debug) {
            for (r in 1:nr) {
                cat(paste(r, gp$random.spec[[r]]$model, gp$random.spec[[r]]$of, gp$random.spec[[r]]$same.as), "\n")
            }
        }

        all.terms <- c()
        for (r in 1:nr) {
            all.terms <- c(all.terms, gp$random.spec[[r]]$term)
        }

        for (r in 1:nr) {
            n <- nrep <- ngroup <- N <- NULL

            if (gp$random.spec[[r]]$model != "linear") {
                ## in this case we have to add a FFIELD section.........
                count.random <- count.random + 1
                xx <- rf[, r + 1]

                ## Do some checking for 'copy', which might lead
                ## to some strange errors if not met.  Its OK to
                ## check these here as we already have sorted the
                ## fields.

                ## if copy, then make sure that some critical variables are the same
                if (gp$random.spec[[r]]$model == "copy") {
                    r.idx <- which(all.terms == gp$random.spec[[r]]$of)
                    if (length(r.idx) == 0) {
                        stop(paste("Something wrong: cannot find the match of copy='", gp$random.spec[[r]]$of, "'", sep = ""))
                    }

                    gp$random.spec[[r]]$ngroup <- gp$random.spec[[r.idx]]$ngroup
                    gp$random.spec[[r]]$nrep <- gp$random.spec[[r.idx]]$nrep
                    gp$random.spec[[r]]$n <- gp$random.spec[[r.idx]]$n
                    gp$random.spec[[r]]$values <- gp$random.spec[[r.idx]]$values
                }

                if (!is.null(gp$random.spec[[r]]$nrep)) {
                    nrep <- as.integer(gp$random.spec[[r]]$nrep)
                } else {
                    if (!is.null(gp$random.spec[[r]]$replicate)) {
                        nrep <- as.integer(max(gp$random.spec[[r]]$replicate, na.rm = TRUE))
                    } else {
                        nrep <- 1
                    }
                }

                if (!is.null(gp$random.spec[[r]]$ngroup)) {
                    ngroup <- as.integer(gp$random.spec[[r]]$ngroup)
                } else {
                    if (!is.null(gp$random.spec[[r]]$group)) {
                        ngroup <- as.integer(max(gp$random.spec[[r]]$group, na.rm = TRUE))
                    } else {
                        ngroup <- 1
                    }
                }

                if (nrep > 1 || ngroup > 1) {
                    if (is.null(gp$random.spec[[r]]$n)) {
                        if (is.factor(xx)) {
                            n <- length(unique(xx[!is.na(xx)]))
                        } else {
                            if (!is.null(gp$random.spec[[r]]$values)) {
                                n <- length(unique(gp$random.spec[[r]]$values[!is.na(gp$random.spec[[r]]$values)]))
                            } else {
                                n <- length(unique(xx[!is.na(xx)]))
                            }
                        }
                        gp$random.spec[[r]]$n <- n
                    } else {
                        n <- gp$random.spec[[r]]$n
                    }
                    ## warning(paste("inla: I do not think `n' is required to give...; compute n=", n))

                    N <- inla.model.properties(gp$random.spec[[r]]$model, "latent")$aug.factor * n
                    replicate <- gp$random.spec[[r]]$replicate
                    if (is.null(replicate)) {
                        replicate <- rep(1, length(xx))
                    }

                    group <- gp$random.spec[[r]]$group
                    if (is.null(group)) {
                        group <- rep(1, length(xx))
                    }

                    if (nrep > 1) {
                        ## if replicate is a single number, then reuse
                        if (length(replicate) == 1) {
                            replicate <- rep(replicate, length(xx))
                        }

                        if (length(replicate) != length(xx)) {
                            stop(paste("length(replicate) != length(xx)", length(replicate), "!=", length(xx)))
                        }
                        if (!all(is.element(replicate, 1:nrep) | is.na(replicate))) {
                            stop(paste("Error in the values of `replicate'; not in [1,...,", nrep, "]", sep = ""))
                        }
                        if (any(is.na(replicate[!is.na(xx)]))) {
                            stop(paste("There are one or more NA's in 'replicate' where 'idx' in f(idx,...) is not NA: idx = \'",
                                gp$random.spec[[r]]$term, "\'",
                                sep = ""
                            ))
                        }
                        replicate[is.na(xx)] <- 1
                    }

                    if (ngroup > 1) {
                        ## if group is a single number, then reuse
                        if (length(group) == 1) {
                            group <- rep(group, length(xx))
                        }

                        if (length(group) != length(xx)) {
                            stop(paste("length(group) != length(xx)", length(group), "!=", length(xx)))
                        }
                        if (!all(is.element(group, 1:ngroup) | is.na(group))) {
                            stop(paste("Error in the values of `group'; not in [1,...,", ngroup, "]", sep = ""))
                        }
                        if (any(is.na(group[!is.na(xx)]))) {
                            stop(paste("There are one or more NA's in 'group' where 'idx' in f(idx,...) is not NA: idx = \'",
                                gp$random.spec[[r]]$term, "\'",
                                sep = ""
                            ))
                        }
                        group[is.na(xx)] <- 1

                        ## issue a WARNING,  if there are to many unused groups
                        g.used <- unique(sort(group))
                        g.unused <- setdiff(1:ngroup, g.used)
                        ng.used <- length(g.used)
                        ng.unused <- length(g.unused)

                        txt <- paste("f(", gp$random.spec[[r]]$term, ", ...)", sep = "")
                        if (!is.element(1, g.used)) {
                            warning(paste(
                                txt, ": ",
                                "There is no indices where group[]=1, this is *usually* a misspesification"
                            ),
                            immediate. = TRUE
                            )
                        }
                        if (ng.unused >= ng.used) {
                            warning(paste(txt, ": ",
                                "Number of unused groups >= the number of groups used: ", ng.unused, " >= ", ng.used,
                                ", this is *usually* a misspesification",
                                sep = ""
                            ),
                            immediate. = TRUE
                            )
                        }
                    }
                } else {
                    N <- NULL
                }
                NG <- N * ngroup

                ## might be used for plotting etc...
                gp$random.spec[[r]]$N <- N
                gp$random.spec[[r]]$NG <- NG
                gp$random.spec[[r]]$NREP <- NREP <- nrep

                ## make sure these are set so they will be used by the `copy'-feature
                if (is.null(gp$random.spec[[r]]$n)) {
                    gp$random.spec[[r]]$n <- n
                }
                gp$random.spec[[r]]$ngroup <- ngroup
                gp$random.spec[[r]]$nrep <- nrep

                if (nrep > 1 || ngroup > 1) {
                    if (is.null(replicate)) {
                        stop("replicate is NULL")
                    }
                    if (is.null(group)) {
                        stop("group is NULL")
                    }
                }

                if (!is.null(gp$random.spec[[r]]$values)) {

                    ## in any case they must be unique:
                    if (length(gp$random.spec[[r]]$values) != length(unique(gp$random.spec[[r]]$values))) {
                        stop(paste("The 'values' given in f(",
                            gp$random.spec[[r]]$term, ",..., values=...) are not unique:\n",
                            "    length(values)=", length(gp$random.spec[[r]]$values),
                            " but length(unique(values))=", length(unique(gp$random.spec[[r]]$values)),
                            sep = ""
                        ))
                    }

                    ## values are given. then either both must be numeric or a factor,  or factor x character

                    if (is.numeric(xx) && is.numeric(gp$random.spec[[r]]$values)) {
                        ## case 1: both numeric

                        ## no sort for values here, since they are given as they should be !!!!
                        location[[r]] <- unique(gp$random.spec[[r]]$values)
                        cov <- match(xx, location[[r]]) - 1L + inla.ifelse(nrep > 1L || ngroup > 1L, (replicate - 1L) * NG + (group - 1) * N, 0L)

                        ## check that not do anything we will regret
                        i.cov <- is.na(match(xx, location[[r]]))
                        i.xx <- is.na(xx)
                        if (!all(i.cov == i.xx)) {
                            stop(paste("f(", gp$random.spec[[r]]$term, "). Covariate does not match 'values' ",
                                sum(i.cov != i.xx), " times. Indexes for mismatch:", inla.paste(which(i.cov != i.xx)),
                                ". This is not what you want. Use NA values in the covariate!",
                                sep = ""
                            ))
                        }
                        cov[is.na(cov)] <- -1L
                        covariate[[r]] <- cov
                    } else if ((is.factor(xx) || is.character(xx))
                    &&
                        (is.factor(gp$random.spec[[r]]$values) || is.character(gp$random.spec[[r]]$values))) {
                        ## case 2: both factors or character's

                        if (is.character(xx)) {
                            ## convert to factor (easier)
                            xx <- factor(xx, unique(xx))
                        }
                        if (is.character(gp$random.spec[[r]]$values)) {
                            ## convert it to a factor
                            gp$random.spec[[r]]$values <- factor(gp$random.spec[[r]]$values, gp$random.spec[[r]]$values)
                        }

                        gp$random.spec[[r]]$id.names <- levels(gp$random.spec[[r]]$values)
                        location[[r]] <- 1L:length(levels(gp$random.spec[[r]]$values))

                        ## let us first check if there are levels in
                        ## xx that is not present in values. this
                        ## should not happen as NA's should be used
                        ## instead

                        if (length(setdiff(levels(xx), gp$random.spec[[r]]$values)) != 0L) {
                            dif <- setdiff(levels(xx), levels(gp$random.spec[[r]]$values))
                            stop(paste("In f(", gp$random.spec[[r]]$term, "): Covariate does not match 'values' ",
                                length(dif), " times. Levels that mismatch:", inla.paste(c(dif), sep = " "),
                                ".\n  This is not what you want. Use NA values in the covariate!",
                                sep = ""
                            ))
                        }

                        cov <- match(xx, levels(gp$random.spec[[r]]$values)) - 1L
                        cov[is.na(cov)] <- -1L
                        covariate[[r]] <- cov
                    } else {
                        ## all combinations not covered above is not allowed.
                        stop(paste("In f(", gp$random.spec[[r]]$term, "): 'covariate' must match 'values'",
                            ",  and both must either be 'numeric', or 'factor'/'character'.",
                            sep = ""
                        ))
                    }
                } else {
                    ## values are not given. then 'values' depends on the type of the covariate.
                    if (is.factor(xx)) {
                        gp$random.spec[[r]]$id.names <- levels(xx)
                        location[[r]] <- 1L:length(levels(xx))
                        xx <- as.numeric(xx)
                    } else if (is.character(xx)) {
                        gp$random.spec[[r]]$id.names <- levels(as.factor(xx))
                        location[[r]] <- 1L:length(levels(as.factor(xx)))
                        xx <- as.numeric(sapply(xx, function(xx.i, id.names) which(xx.i == id.names), id.names = gp$random.spec[[r]]$id.names))
                    } else if (is.numeric(xx)) {
                        gp$random.spec[[r]]$id.names <- NULL
                        location[[r]] <- sort(unique(xx))
                        ## need to store the mapping for later use
                    } else {
                        stop(paste("f(", gp$random.spec[[r]]$term, "). Covariate is not of type 'factor', 'character' or 'numeric'.", sep = ""))
                    }

                    cov <- match(xx, location[[r]]) - 1L + inla.ifelse(nrep > 1L || ngroup > 1L, (replicate - 1L) * NG + (group - 1L) * N, 0L)

                    ## check that we do not do anything we will regret
                    i.cov <- is.na(match(xx, location[[r]]))
                    i.xx <- is.na(xx)
                    if (!all(i.cov == i.xx)) {
                        stop(paste("f(", gp$random.spec[[r]]$term, "). Covariate does not match 'values' ",
                            sum(i.cov != i.xx), " times. Indexes for mismatch:", inla.paste(which(i.cov != i.xx)),
                            ". This is not what you want. Use NA values in the covariate!",
                            sep = ""
                        ))
                    }

                    cov[is.na(cov)] <- -1L
                    covariate[[r]] <- cov
                }

                ## just make sure this is all set
                if (is.null(gp$random.spec[[r]]$values)) {
                    gp$random.spec[[r]]$values <- location[[r]]
                }

                ## create a location and covariate file
                file.loc <- inla.tempfile(tmpdir = data.dir)
                inla.write.fmesher.file(as.matrix(as.numeric(location[[r]]), ncol = 1), filename = file.loc, debug = debug)
                ## prevent some numerical instabilities for models rw1, rw2, crw2, etc...
                inla.check.location(location[[r]],
                                    term = gp$random.spec[[r]]$term,
                                    model = gp$random.spec[[r]]$model, section = "latent")
                file.loc <- gsub(data.dir, "$inladatadir", file.loc, fixed = TRUE)

                ## this have to match
                stopifnot(length(covariate[[r]]) == NPredictor)
                file.cov <- inla.tempfile(tmpdir = data.dir)
                inla.write.fmesher.file(as.matrix(cbind(indN, covariate[[r]])), filename = file.cov, debug = debug)
                file.cov <- gsub(data.dir, "$inladatadir", file.cov, fixed = TRUE)

                ## name of the 'names of the values'
                if (!is.null(gp$random.spec[[r]]$id.names)) {
                    file.id.names <- inla.tempfile(tmpdir = data.dir)
                    inla.writeLines(file.id.names, gp$random.spec[[r]]$id.names)
                    file.id.names <- gsub(data.dir, "$inladatadir", file.id.names, fixed = TRUE)
                } else {
                    file.id.names <- NULL
                }

                file.cov <- inla.tempfile(tmpdir = data.dir)
                inla.write.fmesher.file(as.matrix(cbind(indN, covariate[[r]])), filename = file.cov, debug = debug)
                file.cov <- gsub(data.dir, "$inladatadir", file.cov, fixed = TRUE)

                if (nrep == 1 && ngroup == 1) {
                    n <- inla.ifelse(
                        is.null(gp$random.spec[[r]]$n), length(location[[r]][!is.na(location[[r]])]),
                        gp$random.spec[[r]]$n
                    )
                } else {
                    if (is.null(n)) {
                        stop("n is NULL!!!!")
                    }
                }
                if (is.null(gp$random.spec[[r]]$n)) {
                    gp$random.spec[[r]]$n <- n
                }

                if (debug) {
                    print(paste("n", n))
                    print(paste("nrep", nrep))
                }

                n.div.by <- inla.model.properties(gp$random.spec[[r]]$model, "latent")$n.div.by
                if (!is.null(n.div.by)) {
                    if (!inla.one.of(gp$random.spec[[r]]$model, "iidkd") && !inla.divisible(n, by = n.div.by)) {
                        stop(paste("Model [", gp$random.spec[[r]]$model, "] require `n'", n, "divisible by", n.div.by))
                    }
                }

                if (nrep < 1 || nrep != round(nrep)) {
                    stop(paste("\nArgument NREP is void:", nrep))
                }
                if (ngroup < 1 || ngroup != round(ngroup)) {
                    stop(paste("\nArgument NGROUP is void:", ngroup))
                }

                ## for the CRW2/BYM (or agumented) models, we have to
                ## implement the constr=TRUE using the argument
                ## EXTRACONSTR

                if (inla.model.properties(gp$random.spec[[r]]$model, "latent")$augmented) {
                    fac <- inla.model.properties(gp$random.spec[[r]]$model, "latent")$aug.factor
                    con <- inla.model.properties(gp$random.spec[[r]]$model, "latent")$aug.constr

                    ## this is a very special case that we need to handle here, as 'aug.constr'
                    ## is defined for all possible model, and we need to restrict it to what we
                    ## actually have.
                    if (inla.one.of(gp$random.spec[[r]]$model, "iidkd")) {
                        con <- con[1:gp$random.spec[[r]]$order]
                    }

                    if (fac > 1) {
                        if ((max(con) > fac || min(con) < 1) || is.null(con)) {
                            stop(paste("INTERNAL ERROR: aug.constr = ", con))
                        }

                        ## this case is a very special case
                        if (gp$random.spec[[r]]$constr) {
                            if (is.null(gp$random.spec[[r]]$extraconstr)) {
                                A <- matrix(0, 1, fac * n)
                                for (con.elm in con) {
                                    A[1, (con.elm - 1) * n + 1:n] <- 1
                                }
                                e <- 0
                            } else {
                                nrow <- dim(gp$random.spec[[r]]$extraconstr$A)[1]
                                ncol <- dim(gp$random.spec[[r]]$extraconstr$A)[2]

                                if (ncol != fac * n) {
                                    stop(paste("Wrong dimension for the extraconstr: ncol", ncol, "n", n))
                                }

                                A <- matrix(0, nrow + 1, ncol)
                                e <- c(gp$random.spec[[r]]$extraconstr$e, 0)
                                A[1:nrow, 1:ncol] <- gp$random.spec[[r]]$extraconstr$A

                                for (con.elm in con) {
                                    A[nrow + 1, (con.elm - 1) * n + 1:n] <- 1
                                }
                            }
                            gp$random.spec[[r]]$extraconstr <- list(A = A, e = e)
                            gp$random.spec[[r]]$constr <- FALSE
                        }
                    } else {
                        stopifnot(fac == 1)

                        ## this is the case fac = 1. this case is a
                        ## very special case, for iid2d, iid3d, etc

                        n.small <- n %/% max(con)

                        if (gp$random.spec[[r]]$constr) {
                            if (is.null(gp$random.spec[[r]]$extraconstr)) {
                                A <- matrix(0, length(con), n)
                                k <- 1
                                for (con.elm in con) {
                                    A[k, (con.elm - 1) * n.small + 1:n.small] <- 1
                                    k <- k + 1
                                }
                                e <- rep(0, length(con))
                            } else {
                                nrow <- dim(gp$random.spec[[r]]$extraconstr$A)[1]
                                ncol <- dim(gp$random.spec[[r]]$extraconstr$A)[2]

                                if (ncol != n) {
                                    stop(paste("Wrong dimension for the extraconstr: ncol", ncol, "n", n))
                                }

                                A <- matrix(0, nrow + length(con), ncol)
                                e <- c(gp$random.spec[[r]]$extraconstr$e, rep(0, length(con)))
                                A[1:nrow, 1:ncol] <- gp$random.spec[[r]]$extraconstr$A

                                k <- 1
                                for (con.elm in con) {
                                    A[nrow + k, (con.elm - 1) * n.small + 1:n.small] <- 1
                                    k <- k + 1
                                }
                            }
                            gp$random.spec[[r]]$extraconstr <- list(A = A, e = e)
                            gp$random.spec[[r]]$constr <- FALSE
                        }
                    }
                }

                ## print(gp$random.spec[[r]]$extraconstr$A)
                ## print(gp$random.spec[[r]]$extraconstr$e)

                ## and in case a file for the extraconstr
                if (!is.null(gp$random.spec[[r]]$extraconstr)) {
                    A <- gp$random.spec[[r]]$extraconstr$A
                    e <- gp$random.spec[[r]]$extraconstr$e

                    if ((gp$random.spec[[r]]$model != "rgeneric") &&
                        (ncol(A) != inla.model.properties(gp$random.spec[[r]]$model, "latent")$aug.factor * n)) {
                        stop(paste(
                            "\n\tncol in matrix A(extraconstr) does not correspont to the length of f:",
                            ncol(A),
                            inla.model.properties(gp$random.spec[[r]]$model, "latent")$aug.factor * n
                        ))
                    }

                    file.extraconstr <- inla.tempfile(tmpdir = data.dir)
                    inla.write.fmesher.file(as.matrix(c(as.vector(t(A)), e), ncol = 1), filename = file.extraconstr, debug = debug)
                    file.extraconstr <- gsub(data.dir, "$inladatadir", file.extraconstr, fixed = TRUE)
                } else {
                    file.extraconstr <- NULL
                }

                ## ....also if necessary a file for the weights (not to be confused with argument 'weights' in the inla() call...)
                www <- NULL
                if (!is.null(gp$random.spec[[r]]$weights)) {
                    ## $weights is the name
                    www <- wf[, gp$random.spec[[r]]$weights]
                    www[is.na(www)] <- 0

                    ## create a file for the weights
                    file.weights <- inla.tempfile(tmpdir = data.dir)
                    inla.write.fmesher.file(as.matrix(cbind(indN, www)), filename = file.weights, debug = debug)
                    file.weights <- gsub(data.dir, "$inladatadir", file.weights, fixed = TRUE)
                    n.weights <- n.weights + 1
                }

                ## for some models, the priors are computed in this function. in these cases, we
                ## need to updated all.hyper with these priors, not the ones that goes into this
                ## function...
                rs.updated <- (inla.ffield.section(
                    file = file.ini,
                    file.loc = file.loc, file.cov = file.cov,
                    file.id.names = file.id.names,
                    file.extraconstr = file.extraconstr,
                    file.weights = file.weights, n = n, nrep = nrep, ngroup = ngroup,
                    random.spec = gp$random.spec[[r]],
                    results.dir = paste("random.effect", inla.num(count.random), sep = ""),
                    only.hyperparam = only.hyperparam,
                    data.dir = data.dir
                ))
                all.hyper$random[[r]] <- (list(
                    hyperid = inla.namefix(gp$random.spec[[r]]$term),
                    hyper = rs.updated$hyper,
                    group.hyper = rs.updated$control.group$hyper
                ))
            } else if (inla.one.of(gp$random.spec[[r]]$model, "linear")) {
                ## ....while here we have to add a LINEAR section
                count.linear <- count.linear + 1
                xx <- rf[, r + 1]
                xx[is.na(xx)] <- 0
                file.linear <- inla.tempfile(tmpdir = data.dir)
                inla.write.fmesher.file(as.matrix(cbind(indN, xx)), filename = file.linear, debug = debug)
                file.linear <- gsub(data.dir, "$inladatadir", file.linear, fixed = TRUE)

                cont <- list(
                    cdf = gp$random.spec[[r]]$cdf,
                    quantiles = gp$random.spec[[r]]$quantiles,
                    prec = gp$random.spec[[r]]$prec.linear,
                    mean = gp$random.spec[[r]]$mean.linear,
                    compute = gp$random.spec[[r]]$compute
                )

                if (is.null(all.hyper$linear)) {
                    lin.count <- 1L
                } else {
                    lin.count <- length(all.hyper$linear) + 1L
                }
                all.hyper$linear[[lin.count]] <- inla.linear.section(
                    file = file.ini, file.fixed = file.linear, label = gp$random.spec[[r]]$term,
                    results.dir = paste("fixed.effect", inla.num(gp$n.fix + count.linear), sep = ""),
                    control.fixed = cont, only.hyperparam = only.hyperparam
                )
            } else {
                stop("This should not happen.")
            }
        }
    }

    if (n.weights != gp$n.weights) {
        stop("\n\tSomething strange with weights in the covariate...")
    }

    ## the inla section
    inla.inla.section(file = file.ini, inla.spec = cont.inla, data.dir, inla.mode)

    ## create mode section
    cont.mode <- inla.set.control.mode.default()
    cont.mode[names(control.mode)] <- control.mode
    if (!is.null(cont.mode$result)) {
        ## Reduce the size of 'result' stored in 'r$.args'. If this is stored directly it
        ## can/will require lots of storage. We do this by creating a stripped object with only
        ## what is needed and pass that one along, with the expected classical
        cont.mode$result <- list(mode = list(x = cont.mode$result$mode$x,
                                             theta = cont.mode$result$mode$theta))
        class(cont.mode$result) <- "inla" ## in case there are checks on
    }
    inla.mode.section(file = file.ini, cont.mode, data.dir)

    ## create expert section. the 'preopt' option is processed here and not in the expert.section
    cont.expert <- inla.set.control.expert.default()
    cont.expert[names(control.expert)] <- control.expert
    inla.expert.section(file = file.ini, cont.expert, data.dir = data.dir)

    ## create lincomb section
    cont.lincomb <- inla.set.control.lincomb.default()
    cont.lincomb[names(control.lincomb)] <- control.lincomb
    inla.lincomb.section(file = file.ini, data.dir = data.dir, contr = cont.lincomb, lincomb = lincomb)

    ## create update section
    cont.update <- inla.set.control.update.default()
    cont.update[names(control.update)] <- control.update
    inla.update.section(file = file.ini, data.dir = data.dir, contr = cont.update)

    ## create pardiso section
    cont.pardiso <- inla.set.control.pardiso.default()
    cont.pardiso[names(control.pardiso)] <- control.pardiso
    inla.pardiso.section(file = file.ini, data.dir = data.dir, contr = cont.pardiso)

    ## now, do the job
    if (debug) {
        cat("...done\n")
        cat("Run inla...")
    }

    if (is.null(inla.arg)) {
        arg.arg <- ""

        num.threads <- inla.parse.num.threads(num.threads)
        arg.nt <- paste0(" -t", num.threads, " -B", blas.num.threads, " ")

        ## due to the weird behaviour,  we will do the verbose-mode differently now
        if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
            arg.v <- inla.ifelse(verbose, "-v", "-v")
        } else {
            arg.v <- inla.ifelse(verbose, "-v", "-v")
        }

        arg.s <- inla.ifelse(silent, "-s", "")
    } else {
        arg.arg <- inla.arg
        arg.nt <- ""
        arg.v <- ""
        arg.s <- ""
    }

    if ((inla.os("mac") || inla.os("mac.arm64")) && inla.getOption("vecLib") && !inla.getOption("mkl")) {
        arg.vecLib <- "-L"
    } else {
        arg.vecLib <- ""
    }

    if (inla.mode %in% "classic") {
        arg.P <- "-P classic"
    } else if (inla.mode %in% "twostage") {
        arg.P <- "-P twostage"
    } else if (inla.mode %in% "experimental") {
        arg.P <- "-P experimental"
    } else {
        stop("Unknown 'inla.mode'")
    }

    ## collect all. we might add '-p' later if inla.call="submit"
    all.args <- paste(arg.arg, arg.s, arg.v, arg.nt, arg.vecLib, arg.P, sep = " ")

    ## define some environment variables for remote computing
    vars <- list(
        INLA_PATH = system.file("bin", package = "INLA"),
        INLA_OS = inla.os.type(),
        INLA_VERSION = inla.version("version"),
        INLA_RVERSION = paste0(
            R.Version()$major, ".",
            strsplit(R.Version()$minor, "[.]")[[1]][1]
        ),
        INLA_RHOME = Sys.getenv("R_HOME"),
        INLA_VECLIB_PATH = inla.getOption("vecLibPath")
    )
    do.call("Sys.setenv", vars)
    inla.set.sparselib.env(inla.dir, blas.num.threads = blas.num.threads)

    if (debug) {
        print(paste("all.args: ", all.args))
    }

   vars <- NULL
    if (debug) {
        vars <- c(vars, INLA_DEBUG = 1)
    }
    if (remote || submit) {
        if (submit) {
            all.args <- paste(all.args, "-p") ## need this option
            vars <- c(vars,
                INLA_SUBMIT_ID = submit.id
            )
        }
        if (inla.os("windows")) {
            vars <- c(vars,
                INLA_SSH_AUTH_SOCK = inla.getOption("ssh.auth.sock"),
                INLA_CYGWIN_HOME = inla.getOption("cygwin.home"),
                INLA_HOME = inla.cygwin.map.filename(gsub("\\\\", "/", inla.get.HOME()))
            )
        } else {
            vars <- c(vars,
                INLA_HOME = inla.get.HOME()
            )
            if (Sys.getenv("SSH_AUTH_SOCK") == "") {
                vars <- c(vars,
                    INLA_SSH_AUTH_SOCK = inla.getOption("ssh.auth.sock")
                )
            }
        }
    }
    if (!is.null(vars)) {
        do.call("Sys.setenv", as.list(vars))
    }

    ## write the list of environment variables set, so they can be reset if needed
    env <- Sys.getenv()
    env.n <- names(env)
    idx <- grep("^(INLA_|(OPENBLAS|MKL)_NUM_THREADS|PARDISO)", env.n)
    env.list <- env[idx]
    file.env <- paste0(inla.dir, "/environment")
    cat(file = file.env)
    for (i in seq_along(env.list)) {
        cat(names(env.list[i]), "=\"", env.list[i], "\"\n", sep = "", file = file.env, append = TRUE)
    }

    timeout <- inla.getOption("inla.timeout")
    timeout <- if (!is.numeric(timeout) || timeout < 0) 0 else ceiling(timeout)
    timeout.used <- Sys.time()
    
    my.time.used[2] <- Sys.time()
    ## ...meaning that if inla.call = "" then just build the files (optionally...)
    if (ownfun || nchar(inla.call) > 0) {
        if (ownfun) {
            ## undocumented feature for PB
            echoc <- inla.call(
                file.ini = file.ini,
                file.log = if (verbose) NULL else file.log,
                results.dir = results.dir,
                inla.call.args = all.args
            )
        } else if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
            if (verbose) {
                echoc <- system(paste(shQuote(inla.call), all.args, shQuote(file.ini)), timeout = timeout)
            } else {
                echoc <- system(paste(
                    shQuote(inla.call), all.args, shQuote(file.ini), " > ", shQuote(file.log),
                    inla.ifelse(silent == 2L, " 2>/dev/null", "")
                ), timeout = timeout)
            }
            timeout.used <- Sys.time() - timeout.used
            inla.inlaprogram.timeout(timeout.used, timeout)
        } else if (inla.os("windows")) {
            if (!remote && !submit) {
                if (verbose) {
                    echoc <- try(system2(inla.call, args = paste(all.args, shQuote(file.ini)), stdout = "", stderr = "", wait = TRUE, timeout = timeout))
                } else {
                    if (FALSE) {
                        ## old .bat-solution
                        bat.file <- paste(tempfile(), ".BAT", sep = "")
                        cat("@echo off\n", file = bat.file, append = FALSE)
                        cat(paste(
                            shQuote(inla.call), all.args, "-v", shQuote(file.ini), ">", shQuote(file.log),
                            inla.ifelse(silent == 2L, "2>NUL", "")
                        ), file = bat.file, append = TRUE)
                        echoc <- try(system2(bat.file, wait = TRUE, timeout = timeout), silent = FALSE)
                        unlink(bat.file)
                    } else {
                        ## new try
                        echoc <- try(system2(inla.call,
                                             args = paste(all.args, shQuote(file.ini)),
                                             stdout = file.log, stderr = file.log2,
                                             wait = TRUE, timeout = timeout))
                    }
                }
                timeout.used <- Sys.time() - timeout.used
                inla.inlaprogram.timeout(timeout.used, timeout)
                if (echoc != 0L) {
                    if (!verbose && (silent != 2L)) {
                        inla.inlaprogram.has.crashed()
                    }
                }
            } else {
                ## remote || submit
                echoc <- try(inla.cygwin.run.command(
                    paste(
                        inla.cygwin.map.filename(inla.call),
                        all.args,
                        inla.cygwin.map.filename(file.ini)
                    ),
                    file.log = inla.ifelse(verbose, NULL, inla.cygwin.map.filename(file.log))
                ), silent = TRUE)
                ## echoc = 0L
            }
        } else {
            stop("\n\tNot supported architecture.")
        }

        if (debug) {
            cat("..done\n")
        }

        my.time.used[3] <- Sys.time()

        if (echoc == 0L) {
            if (!submit) {
                ret <- try(inla.collect.results(results.dir,
                    only.hyperparam = only.hyperparam, file.log = file.log, file.log2 = file.log2
                ), silent = FALSE)
                if (!is.list(ret)) {
                    ret <- list()
                }
            } else {
                ret <- list()
            }

            my.time.used[4] <- Sys.time()
            cpu.used <- c(
                "Pre" = diff(my.time.used)[1],
                "Running" = diff(my.time.used)[2],
                "Post" = diff(my.time.used)[3],
                "Total" = my.time.used[4] - my.time.used[1]
            )

            ret$cpu.used <- cpu.used
            ## store all arguments; replacing 'control.xxx' with 'cont.xxx'
            the.args <- list()
            for (nm in names(formals(inla))) {
                nnm <- nm
                nnm <- gsub("^control\\.", "cont.", nnm) ## these are the processed ones
                nnm <- gsub("^data$", "data.orig", nnm)
                nnm <- gsub("^formula$", "formula.orig", nnm)
                ## maybe comment out this one so we use the processed one? I cannot recall the
                ## argument for doing like this. It does not make sense now.
                ## nnm = gsub("^cont(rol)?\\.family$", "control.family.orig", nnm)
                inla.eval(paste("the.args$", nm, " = ", nnm, sep = ""))
            }
            ## remove the .Evironment attribute, as it will fail if
            ## its rerun if .Environment is not there.
            attr(the.args$formula, ".Environment") <- NULL

            ## further fix for $control.family until the c(param = numeric(0)) issue is solved.
            if (n.family > 1L) {
                ## we need to fix the case where there this option is
                ## not set. otherwise, we will get list(), instead of
                ## list(list(), list()), say, for n.family=2
                if (is.list(the.args$control.family) && length(the.args$control.family) == 0L) {
                    the.args$control.family <- lapply(1:n.family, function(x) list())
                }
            }
            ## OLD CODE: if (n.family == 1) the.args$control.family = the.args$control.family[[1L]]
            ret$all.hyper <- all.hyper
            ret$.args <- the.args
            ret$call <- call
            ret$model.matrix <- gp$model.matrix
            class(ret) <- "inla"
        } else {
            ## do this instead
            if (silent != 2L) {
                inla.inlaprogram.has.crashed()
            } else {
                ## with a crash, try to collect the logfile only
                ret1 <- try(inla.collect.logfile(file.log, debug), silent = TRUE)
                ret2 <- try(inla.collect.logfile(file.log2, debug), silent = TRUE)
                if (inherits(ret1, "try-error")) { ## yes,  its 'ret1'
                    ret <- NULL
                } else {
                    ret <- list(logfile = c(
                        ret1$logfile,
                        "", paste(rep("*", 72), sep = "", collapse = ""), "",
                        ret2$logfile
                    ))
                    class(ret) <- "inla"
                }
            }
        }

        if (debug && !keep) {
            cat("clean up\n")
        }
        if (!keep && !submit) {
            try(unlink(inla.dir, recursive = TRUE), silent = TRUE)
        }
    } else {
        ret <- NULL
    }

    ##
    if (submit) {
        ret$misc$inla.dir <- inla.dir
        ret.sub <- list(ret = ret, id = submit.id)
    } else {

        ## post-processign part. first the 'selection'. add the missing part and remove the
        ## 'selection' results from the other results
        if (!is.null(selection)) {
            tag.base <- gsub("\\.", "\\\\.", formals(inla.selection2lincombs)$tag.base)
            cmat <- ret$misc$lincomb.derived.covariance.matrix
            idx <- grep(paste0("^", tag.base, "..*:[0-9]+$"), colnames(cmat))
            snames <- gsub(paste0("^", tag.base), "", colnames(cmat)[idx])
            covmat <- cmat[idx, idx, drop = FALSE]
            colnames(covmat) <- rownames(covmat) <- NULL
            mean <- ret$summary.lincomb.derived[idx, "mean"]

            ## note that this ordering might be different than in the selection above, which
            ## depends on the ordering of the lincomb. so we need to make sure they are aligned!

            if (inla.mode %in% "experimental") {
                ct <- ret$misc$configs$contents
                for(nm in c("APredictor", "Predictor")) {
                    if (ct$tag[1] == nm) {
                        ct$tag <- ct$tag[-1]
                        ct$start <- ct$start[-1] - ct$start[2] + 1
                        ct$length <- ct$length[-1]
                    }
                }
                rfake <- list(misc = list(configs = list(contents = ct)))
                sel.idx <- which(inla.posterior.sample.interpret.selection(selection, rfake))
            } else {
                sel.idx <- which(inla.posterior.sample.interpret.selection(selection, ret))
            }
            nc <- ret$misc$configs$nconfig
            ns <- length(idx)
            m <- list()

            m[[1]] <- matrix(unlist(lapply(ret$misc$configs$config,
                function(x, sel.idx) {
                    return(x$improved.mean[sel.idx])
                },
                sel.idx = sel.idx
            )), nrow = ns, ncol = nc)

            m[[2]] <- (matrix(unlist(lapply(ret$misc$configs$config,
                function(x, sel.idx) {
                    return(diag(x$Qinv)[sel.idx])
                },
                sel.idx = sel.idx
            )), nrow = ns, ncol = nc)
            + m[[1]][, , drop = FALSE]^2)

            m[[3]] <- (matrix(unlist(lapply(ret$misc$configs$config,
                (function(x, sel.idx) {
                    skew <- x$skewness[sel.idx]
                    skew[is.na(skew)] <- 0
                    return(skew * diag(x$Qinv)[sel.idx]^1.5)
                }),
                sel.idx = sel.idx
            )), nrow = ns, ncol = nc)
            + 3 * m[[2]][, , drop = FALSE] * m[[1]][, , drop = FALSE] - 2 * m[[1]][, , drop = FALSE]^3)

            for (i in seq_along(m)) {
                rownames(m[[i]]) <- names(sel.idx)
                m[[i]] <- m[[i]][snames, , drop = FALSE] ## put them in the correct order
            }
            prob <- exp(unlist(lapply(ret$misc$configs$config, function(x) x$log.posterior)))
            prob <- prob / sum(prob)
            mm <- list()
            for (i in seq_along(m)) {
                mm[[i]] <- numeric(nrow(m[[i]]))
            }
            for (i in seq_len(ncol(m[[3]]))) {
                for (j in seq_along(m)) {
                    mm[[j]] <- mm[[j]] + prob[i] * m[[j]][, i]
                }
            }
            skewness <- as.numeric((mm[[3]] - 3 * mm[[2]] * mm[[1]] + 2 * mm[[1]]^3)
            / (mm[[2]] - mm[[1]]^2)^1.5)
            ret$selection <- list(
                names = snames,
                mean = mean,
                cov.matrix = covmat,
                skewness = skewness,
                marginal.sn.par = inla.sn.par(mean, diag(covmat), skewness),
                .private = list(moments = mm, selection = selection)
            )
            class(ret$selection) <- "inla.jmarginal"

            ## remove the 'selection' results from other places
            if (!is.null(ret$marginals.lincomb.derived)) {
                mnames <- names(ret$marginals.lincomb.derived)
                new.names <- c()
                new <- list()
                j <- 1
                for (i in seq_along(ret$marginals.lincomb.derived)) {
                    if (!(mnames[i] %in% snames)) {
                        new[[j]] <- ret$marginals.lincomb.derived[[i]]
                        new.names <- c(new.names, mnames[i])
                        j <- j + 1
                    }
                }
                names(new) <- new.names
                ret$marginals.lincomb.derived <- new
            }

            ret$summary.lincomb.derived <- ret$summary.lincomb.derived[-idx, , drop = FALSE]
            ret$misc$lincomb.derived.covariance.matrix <-
                ret$misc$lincomb.derived.covariance.matrix[-idx, -idx, drop = FALSE]
            ret$misc$lincomb.derived.correlation.matrix <-
                ret$misc$lincomb.derived.correlation.matrix[-idx, -idx, drop = FALSE]
            if (all(dim(ret$misc$lincomb.derived.correlation.matrix)) == 0) {
                ret$misc$lincomb.derived.covariance.matrix <- NULL
                ret$misc$lincomb.derived.correlation.matrix <- NULL
                ret$summary.lincomb.derived <- data.frame()
            }
        }

        return(ret)
    }
}

`inla.fix.data` <- function(data, n, revert = FALSE) {
    ## extract all entries in 'data' with length='n'. if 'revert',  do the oposite
    if (is.data.frame(data)) {
        if (dim(data)[1L] == n) {
            return(inla.ifelse(revert, data.frame(), data))
        } else {
            return(inla.ifelse(!revert, data.frame(), data))
        }
    }
    if (is.list(data) && length(data) > 0L) {
        idx <- which(sapply(
            data,
            function(a, n) {
                if (inla.is.matrix(a) && (length(dim(a)) > 1L) && (dim(a)[1L] == n)) {
                    return(TRUE)
                } else if (length(a) != n || inla.is.matrix(a)) {
                    return(FALSE)
                } else if (length(a) == n) {
                    return(TRUE)
                } else {
                    stop("This should not happen.")
                }
            },
            n = n
        ))
        if (revert) {
            idx <- (1:length(data))[-idx]
        }
        if (length(idx) > 0L) {
            return(data[idx])
        } else {
            return(inla.ifelse(revert || is.list(data), list(), data.frame()))
        }
    } else {
        return(data)
    }
    stop("Should not happen.")
}

`inla.expand.factors` <- function(data, exclude.names = c()) {
    ## replace factors in 'data' with their expanded version we get
    ## from model.matrix without any intercept.
    for (k in seq_along(data)) {
        if (is.factor(data[[k]])) {
            if (!(names(data)[k] %in% exclude.names)) {
                formula <- as.formula(paste("~ -1 + ", names(data)[k]))
                tmp <- model.matrix(formula, model.frame(formula, data, na.action = na.pass))
                colnames(tmp) <- paste0(names(data)[k], levels(data[[k]]))
                data[[k]] <- tmp
            }
        }
    }
    return(data)
}

`inla.set.sparselib.env` <- function(inla.dir = NULL, blas.num.threads = 1L) {
    ## environment variables for sparse libraries
    if (is.null(inla.dir)) {
        inla.dir <- inla.tempdir()
    }

    lic.filename <- "pardiso.lic" ## do not change
    lic.filename.dir <- paste0(inla.dir, "/", lic.filename)
    file.create(lic.filename.dir)

    if (!is.null(inla.getOption("pardiso.license"))) {
        lic.file <- normalizePath(inla.getOption("pardiso.license"))
        lic.path <- NA
        if (file.exists(lic.file)) {
            info <- file.info(lic.file)
            if (!is.na(info$isdir)) {
                if (info$isdir) {
                    lic.path <- lic.file
                } else if (!is.null(inla.dir)) {
                    file.copy(lic.file, lic.filename.dir, overwrite = TRUE)
                    lic.path <- inla.dir
                } else {
                    stop("This should not happen")
                }
            } else {
                lic.path <- lic.file
            }
        } else {
            lic.path <- lic.file
        }
        do.call("Sys.setenv", list(
            PARDISO_LIC_PATH = normalizePath(lic.path),
            INLA_LOAD_PARDISO = 1
        ))
    } else {
        Sys.unsetenv("INLA_LOAD_PARDISO")
    }

    if (Sys.getenv("PARDISOLICMESSAGE") == "") {
          Sys.setenv(PARDISOLICMESSAGE = 1)
      }

    return(invisible())
}

`inla.parse.num.threads` <- function(num.threads) {
    ## it is easier to do the parsing of 'num.threads' here. use '0' to represent a value to
    ## be decided by the inla-program.

    if (is.null(num.threads)) {
        num.threads <- "0:0"
    }
    num.threads <- as.character(num.threads)
    num.threads <- gsub("L", "", num.threads)
    ## remove tabs and spaces and the end,  and the interior as a ':'
    num.threads <- gsub("^[ \t]+", "", num.threads)
    num.threads <- gsub("[ \t]+$", "", num.threads)
    num.threads <- gsub("[ \t:,]+", ":", num.threads)

    ## if '' -> '0:0'
    if (nchar(num.threads) == 0) {
        num.threads <- "0:0"
        return(num.threads)
    }

    ## we take this a request for serial mode:
    ## if 'N' -> 'N:1',  where 'N' is any integer
    xx <- suppressWarnings(as.integer(num.threads))
    if (!is.na(xx)) {
        num.threads <- paste0(max(0, xx), ":1")
        return(num.threads)
    }

    ## if ':' then '0:0'
    if (length(grep("^:$", num.threads)) > 0) {
        num.threads <- "0:0"
        return(num.threads)
    }

    ## if 'A:'  then  'A:0'
    if (length(grep("^[0-9]+:$", num.threads)) > 0) {
        num.threads <- paste0(num.threads, "0")
        return(num.threads)
    }

    ## if 'A' then 'A:0'
    if (length(grep("^[0-9]+$", num.threads)) > 0) {
        num.threads <- paste0(num.threads, ":0")
        return(num.threads)
    }

    ## if :B then 0:B
    if (length(grep("^:[0-9]+$", num.threads)) > 0) {
        num.threads <- paste0("0", num.threads)
        return(num.threads)
    }

    return(num.threads)
}

context("test 'latent generic'")

test_that("Case 1", {
    set.seed(1234)
    n = 100
    Q = INLA:::inla.rw1(n)
    i = 1:n
    y = (i-n/2)^2 + rnorm(n)
    
    r = inla(y ~ f(i, model="rw1", param = c(1, .001), constr=FALSE) - 1,
            data = data.frame(i,y))
    rr = inla(y ~ f(i, model="generic", Cmatrix=Q,
            rankdef=1, param=c(1, .001), constr=FALSE) - 1,
            data = data.frame(i,y))
    expect_true(all(abs(rr$summary.random[[1]] -
                        r$summary.random[[1]]) < 0.0001))
})

context("test 'config'")

test_that("Case 1", {
    set.seed(123)
    n = 100
    rho = 0.9
    x = arima.sim(n=n, model = list(ar = rho))
    x = x - mean(x)
    z = rnorm(n)
    y = 1 + z + x + rnorm(n, sd=0.01)
    
    idx = 1:n
    formula = y ~ 1 + z + f(idx, model="ar1", constr=TRUE)
    r = inla(formula,  data = data.frame(y, z, idx),
            control.compute = list(config = TRUE),
            control.inla = list(int.strategy = "grid", diff.logdens = 5, dz=.75))
    cs = r$misc$configs
    for (i in 1:length(cs$contents$tag)) {
        idx = seq(cs$contents$start[i],  len = cs$contents$len[i])
        mean = 0
        psum = 0
        for(k in 1:cs$nconfig) {
            mean = mean + exp(cs$config[[k]]$log.posterior) * cs$config[[k]]$mean[idx]
            psum = psum + exp(cs$config[[k]]$log.posterior)
        }
        mean = mean/psum
        desc = cs$contents$tag[i]
        if (desc == "z" || desc == "(Intercept)") {
            ##print(abs(mean - r$summary.fixed[desc, "mean"]))
            expect_true(abs(mean - r$summary.fixed[desc, "mean"]) < 0.001)
        }
    }
})

context("test 'likelihood betabinomial'")

test_that("Case 1", {
    set.seed(123)
    ## overdispersion parameter in the betabinomial
    rho = 0.1
    pzero = 0.2
    type=0
    n = 1000
    z = rnorm(n, sd=0.2)
    Ntrials = sample(1:20, n, replace=TRUE)
    eta = 1 + z
    p.eta = exp(eta)/(1+exp(eta))
    a = p.eta * (1-rho)/rho
    b = (p.eta * rho - p.eta - rho + 1)/rho
    p = rbeta(n, a, b)
    
    y = numeric(n)
    for(i in 1:n) {
        pp = dbinom(0:Ntrials[i], size = Ntrials[i], prob = p[i])
        if (type == 0) {
            pp[-1] = (1-pzero) * pp[-1]/(1-pp[1])
            pp[1] = pzero
        } else {
            pp = pp * (1-pzero)
            pp[1] = pp[1] + pzero
        }
        y[i] = sample(0:Ntrials[i], 1, prob = pp)
    }
    formula = y ~ 1 + z
    data = data.frame(y, z)
    r = inla(formula, data = data,
            family = paste("zeroinflatedbetabinomial", type, sep=""),
            Ntrials=Ntrials, 
            control.compute=list(cpo=TRUE))

    expect_true(abs(r$summary.hyperpar[1, "mean"] - rho) < 0.05)
    expect_true(abs(r$summary.hyperpar[2, "mean"] - pzero) < 0.025)
})

context("test 'latent besag2'")

test_that("Case 1", {
    data(Oral)
    g = system.file("demodata/germany.graph", package="INLA")
    ## use data Oral to estimate a spatial field in order to simulate a
    ## 'realistic' dataset.
    formula = Y ~ f(region, model="bym", graph=g)
    result = inla(formula, data = Oral, family = "poisson", E = E)
    x = result$summary.random$region$mean
    n = length(x)/2
    ## simulate two new datasets. 'a' is the weighting between the
    ## log.rel.risk:
    a = 2
    xx = x[1:n]+1
    x = c(a*xx, xx/a)
    E = c(Oral$E, Oral$E)
    N = 2*n
    y = rpois(N, lambda = E*exp(x))
    
    ## model='besag2' defines a model with length N = 2*graph->n, the
    ## first half is weighted with 'a' the other half is weighted with
    ## 1/a. here there is no unstructed terms.
    i = 1:N
    std = TRUE
    formula = y ~ f(i, model="besag2", graph=g, scale.model=std) -1
    r = inla(formula, family = "poisson", data = data.frame(E,y,i), E=E)
    expect_true(abs(r$summary.hyperpar[2, "mean"] - a) < 0.1)
})

context("test 'the A-matrix'")

test_that("Case 1", {
    set.seed(123)
    eps = 0.025
    n = 300
    z = rnorm(n)
    zz = 1:n
    y = z + zz + rnorm(n, sd = 0.1)
    r = sample(1:n, n)
    Y = y[r]
    A=sparseMatrix(i=1:n, j=r, x=rep(1,n))
    AA = as.matrix(A)
    formula = Y ~ -1 + z + zz
    r = inla(formula, data = data.frame(Y,z,zz),
            control.predictor = list(compute=T, A = A))
    rr = inla(formula, data = data.frame(Y,z,zz),
            control.predictor = list(compute=T, A = AA))
    
    expect_true(all(abs(r$summary.fixed[, "mean"] - 1) < eps))
    expect_true(all(abs(rr$summary.fixed[, "mean"] - 1) < eps))
    expect_true(all(abs(r$summary.fixed- rr$summary.fixed) < eps))
})

test_that("Case 2", {
    set.seed(1234)
    eps = 0.05
    n = 100
    z = rnorm(n)
    zz = 1:n
    lin.pred = z + zz
    y = numeric(n)
    lin.pred.new = numeric(n)
    A=list(i=c(), j=c(), values=c())
    for(i in 1:n) {
        lin.pred.new[i] = mean(lin.pred[1:i])
        y[i] = lin.pred.new[i] + rnorm(1, sd=0.1)
        A$i = c(A$i, rep(i, i))
        A$j = c(A$j, 1:i)
        A$values = c(A$values, rep(1/i, i))
    }
    Am = sparseMatrix(i = A$i, j = A$j, x = A$values)
    formula = y ~ -1 + z + zz
    r = inla(formula, data = data.frame(y,z,zz),
            control.predictor = list(compute=T, A = Am))
    expect_true(all(abs(r$summary.linear.predictor$mean[1:n] - lin.pred.new) < eps))
    expect_true(all(abs(r$summary.linear.predictor$mean[1:n + n] -  lin.pred) < eps))
})


test_that("Case 3", {
    set.seed(12345)
    s = 0.00001
    n = 100
    fac = 2
    m = n*fac
    z = runif(n)
    zz = rep(z, fac)
    
    Y = rep(NA, m)
    A = matrix(0,m,m)
    for(i in 1:m) {
        ## just make some random selection
        A[i, sample(1:m, n %/% 10)] = 1
        Y[i] =  sum(A[i,]*(1+zz)) + rnorm(1, sd=s)
    }
    rr = inla(Y ~ -1 + zz, control.predictor = list(compute=TRUE,A=A),
            data = data.frame(z,Y),
            control.family = list(initial = log(1/s^2), fixed=T))
    expect_true(all(abs(rr$summary.linear.predictor$mean[1:n] - Y[1:n]) < sqrt(s)))
})

test_that("Case 4", {
    set.seed(123)
    n = 30
    m = 10
    off = 10+runif(m)
    z = runif(n)
    eta = 0 + z
    A = matrix(runif(n*m),m,n)
    s = 0.01
    Eta = A %*% eta + off
    Y = Eta +  rnorm(m, sd=s)
    
    r = inla(Y ~ -1+z, control.predictor = list(A=A,compute=TRUE,
                                            precision = 1e6),
            data = list(Y=Y, z=z, off = off),
            offset = off, 
            control.compute=list(cpo=T),
            control.family = list(initial = log(1/s^2), fixed=TRUE))
    expect_true(all(abs(Eta - r$summary.linear.predictor[1:m,"mean"]) < sqrt(s)))
    expect_true(all(abs(eta - r$summary.linear.predictor[m+1:n,"mean"]) < sqrt(s)))
})

context("test 'argument link'")

test_that("Case 1", {
    set.seed(12345)
    ## simple poisson regression
    n = 100
    x = runif(n)
    eta = 1 + x
    lambda = exp(eta)
    y = rpois(n, lambda = lambda)
    
    ## missing values:
    y[1:3] = NA
    y[(n-2):n] = NA
    
    ## link = 1 is a shortcut for rep(1, <n>) where <n> is the appropriate
    ## length. here '1' is a reference to the first 'family', ie
    ## 'family[1]'
    r = inla(y ~ 1 + x,  family = "poisson",
            data = data.frame(y, x), 
            control.predictor = list(compute = TRUE, link = 1))
    expect_true(all(abs(r$summary.fitted.values$mean - lambda) < 0.6))
    
    ## this is the formally correct way, defining 'link' only where there
    ## are missing values. entries in 'link' for which the observation is
    ## not NA, is currently ignored.
    link = rep(NA, n)
    link[which(is.na(y))] = 1
    r = inla(y ~ 1 + x,  family = "poisson",
            data = data.frame(y, x), 
            control.predictor = list(compute = TRUE, link = link))
    expect_true(all(abs(r$summary.fitted.values$mean - lambda) < 0.6))

    
    ## for more than one likelihood, use '2' to refer to the second
    ## likelihood. here we just split the data in two, and assign the
    ## second half the nbinomial distribution.
    n2 = n %/% 2L
    Y = matrix(NA, n, 2)
    Y[1:n2, 1] = y[1:n2]
    Y[1:n2 + n2, 2] = y[1:n2 + n2]
    link = rep(NA, n)
    link[which(is.na(y[1:n2]))] = 1
    link[n2  + which(is.na(y[1:n2 + n2]))] = 2
    
    r = inla(Y ~ 1 + x,  family = c("poisson", "nbinomial"), 
            data = list(Y=Y, x=x), 
            control.predictor = list(compute = TRUE, link = link))
    expect_true(all(abs(r$summary.fitted.values$mean - lambda) < 0.8))
})

context("test 'likelihood cbinomial'")

test_that("Case 1", {
    set.seed(123)
    N=10000
    a = -1
    b = 1
    z = rnorm(N, sd=0.1)
    eta = a + b*z
    n = sample(c(1, 2, 3, 4), size=N, replace=TRUE)
    p = exp(eta)/(1 + exp(eta))
    prob = 1.0 - (1-p)^n
    k = sample(c(1,5,10,15), size=N, replace=TRUE)
    y = rbinom(N, size=k, prob = prob)
    data = list(y=y,z=z)
    formula = y ~ 1+z
    result = inla(formula, family = "cbinomial", data = data,
            Ntrials=cbind(k, n))
    expect_true(abs(result$summary.fixed[1,"mean"] -a) < 0.05)
    expect_true(abs(result$summary.fixed[2,"mean"] -b) < 0.05)
})   

context("test 'cpo-po-mlik'")

test_that("Case 1", {
    set.seed(12345)
    library(INLA)
    library(mvtnorm)
    N = rpois(1, lambda = 100)
    rho = runif(1)
    sigma.eps = runif(1)
    tau.eps=1/sigma.eps^2
    sigma.x = runif(1) ## marginal stdev!
    tau.x=1/sigma.x^2  ## marginal prec!
    C0 = matrix(0, nrow=N, ncol=N)
    for(i in 1:(N-1))
    {
        C0[i,i+1]=-rho
        C0[i+1,i]=-rho
    }
    diag(C0) = 1+rho^2
    C0[1,1]=1
    C0[N,N]=1
    ## Cov for y
    Sigma = solve(tau.x * C0/(1-rho^2)) + 1/tau.eps * diag(N)

    x = rep(NA,N)
    x[1] = rnorm(1, mean = 0, sd = sigma.x*sqrt(1-rho^2))
    for(i in 2:N) {
        x[i] = rho*x[i-1] + sigma.x*rnorm(1)
    }
    y = x + sigma.eps*rnorm(N)

    d = data.frame(id=1:N, y=y)
    fit = inla(y ~ -1 +
            f(id,
              model="generic0",
              Cmatrix=C0/(1-rho^2),
              hyper=list(prec=list(initial=log(tau.x),fixed=TRUE))),
            data=d,
            control.family=list(
                    hyper=list(
                            prec=list(initial=log(tau.eps),fixed=TRUE))),
            control.compute=list(cpo=TRUE, po=TRUE))

    theta1=log(tau.x)
    theta2=log((1+rho)/(1-rho))
    fit2 = inla(y~ -1 +
            f(id,
              model="ar1",
              hyper=list(
                      theta1=list(initial=theta1,fixed=TRUE),
                      theta2=list(initial=theta2,fixed=TRUE))),
            data=d,
            control.family=list(
                    hyper=list(
                            prec=list(initial=log(tau.eps),fixed=TRUE))),
            control.compute=list(cpo=TRUE, po=TRUE))

    ##Checking marginal likelihood, seems ok
    dens = dmvnorm(y,rep(0,N),Sigma,log=TRUE)
    err.mlik = sum(abs(c(fit$mlik[1]+
            0.5*determinant(C0/(1-rho^2),log=TRUE)$mod,
            fit2$mlik[2]) -  dens))

    ##Checking cpo
    cpo = numeric(N)
    for(i in 1:N) {
        D = diag(N)
        D[i, i] = 0
        QQ = tau.x * C0/(1-rho^2) + tau.eps * D
        bb = tau.eps * y
        bb[i] = 0
        sQQ = solve(QQ)
        mu = (sQQ %*% bb)[i]
        var = sQQ[i, i] + 1/tau.eps
        cpo[i] = dnorm(y[i], mean = mu, sd = sqrt(var))
    }

    ##po
    D = diag(N)
    QQ = tau.x * C0/(1-rho^2) + tau.eps * D
    bb = tau.eps * y
    sQQ = solve(QQ)
    mu = c(sQQ %*% bb)
    var = diag(sQQ) + 1/tau.eps
    po = dnorm(y, mean = mu, sd = sqrt(var))

    res.po = cbind(fit = fit$po$po, fit2 = fit2$po$po)
    err.po = mean(abs(res.po - po))
    res = cbind(fit = fit$cpo$cpo, fit2 = fit2$cpo$cpo)
    err.cpo = mean(abs(res -cpo))

    expect_true(err.mlik < 0.001)
    expect_true(err.po < 0.00001)
    expect_true(err.cpo < 0.00001)
})

context("test 'latent bym'")

test_that("Case 1", {
    data(Germany)
    g = system.file("demodata/germany.graph", package="INLA")
    Germany = cbind(Germany,region.struct=Germany$region)
    prior.iid = c(1,0.1)
    prior.besag = c(1,0.001)
    formula1 = Y ~ 1 + f(region.struct,
            model="besag",graph=g, constr=TRUE,
            hyper = list(prec = list(param = prior.besag))) +
                f(region,
                  model="iid", hyper = list(prec = list(param = prior.iid)))
    result1  =  inla(formula1,family="poisson",data=Germany,E=E,
            control.predictor = list(compute=TRUE))
    
    formula1.bym = Y ~ 1 + f(region,
            model = "bym", graph = g, constr=TRUE, 
            hyper = list(
                    prec.unstruct = list(param = prior.iid),
                    prec.spatial = list(param = prior.besag)))
    
    result1.bym = inla(formula1.bym,family="poisson",data=Germany,E=E, 
            control.predictor = list(compute=TRUE))
    
    expect_true(mean(abs(result1$summary.linear.predictor$mean -
                         result1.bym$summary.linear.predictor$mean)) < 1e-4)
})

context("test 'expand-factor-strategy'")

test_that("Case 1", {
    r = inla(y ~ 1 + x + xx,
            data = data.frame(
                    y=1:4,
                    x=factor(c("a","b","c", NA)), 
                    xx=factor(c("A","B","C", NA))), 
            control.fixed = list(expand.factor.strategy="inla"))

    expect_true("xa" %in% colnames(r$model.matrix))
    expect_true("xb" %in% colnames(r$model.matrix))
    expect_true("xc" %in% colnames(r$model.matrix))
    expect_true("xxA" %in% colnames(r$model.matrix))
    expect_true("xxB" %in% colnames(r$model.matrix))
    expect_true("xxC" %in% colnames(r$model.matrix))
})

context("test 'DIC'")

test_that("Case 1", {
    set.seed(123)
    n = 20
    y = rnorm(n)
    yy = rnorm(n)
    k=3
    y[1:k] = NA
    yy[n:n] = NA
    
    Y = matrix(NA, 2*n, 2)
    Y[1:n, 1] = y
    Y[n + 1:n, 2] = yy
    
    r = inla(Y ~1,  data = list(Y=Y),  family = rep("gaussian", 2),
            control.compute = list(dic = TRUE))

    expect_true(all(r$dic$family[(k+1):n] == 1))
    expect_true(all(is.na(r$dic$family[1:k])))
    expect_true(all(r$dic$family[n + 1:(n-1)] == 2))
    expect_true(all(is.na(r$dic$family[2*n])))

    expect_true(all(is.na(r$dic$local.dic[1:k])))
    expect_true(all(is.na(r$dic$local.p.eff[2*n])))
})

context("test 'likelihood beta'")

test_that("Case 1", {
    set.seed(123)
    ## the precision parameter in the beta distribution
    phi = 5
    
    ## generate simulated data
    n = 1000
    z = rnorm(n, sd=0.2)
    eta = 1 + z
    mu = exp(eta)/(1+exp(eta))
    a = mu * phi
    b = -mu * phi + phi
    y = rbeta(n, a, b)
    
    ## estimate the model
    formula = y ~ 1 + z
    r = inla(formula, data = data.frame(y, z), family = "beta",
            control.compute = list(cpo=TRUE))

    expect_true(abs(r$summary.hyperpar[1, "mean"] - phi) < 0.5)
})

context("test 'likelihood gamma'")

test_that("Case 1", {
    set.seed(123)
    n = 1000
    x = rnorm(n)
    eta = 1 + x
    mu = exp(eta)
    prec.scale = runif(n, min = 0.5, max = 2)
    prec.par = 1.2
    a = prec.par * prec.scale
    b = mu / (prec.par * prec.scale)
    y = rgamma(n, shape = a, scale = b)
    r = inla(y ~ 1 + x,  data = data.frame(y, x),
            scale = prec.scale, family = "gamma")

    expect_true(abs(r$summary.fixed[1, "mean"] - 1) < 0.01)
    expect_true(abs(r$summary.fixed[2, "mean"] - 1) < 0.01)
    expect_true(abs(r$summary.hyperpar[1, "mean"] - prec.par) < 0.1)
})

## NOT-YET-Export:


inla.bym.constr.internal <- function(Q, adjust.for.con.comp = TRUE) {
    ## return the rankdef and the constr for the BYM model given in Q

    n <- dim(Q)[1]
    g <- inla.read.graph(Q)
    cc.n <- sapply(g$cc$nodes, length)
    cc.n1 <- sum(cc.n == 1L)
    cc.n2 <- sum(cc.n >= 2L)

    if (adjust.for.con.comp) {
        constr <- list(A = matrix(0, cc.n2, n), e = rep(0, cc.n2))
        kk <- 0
        for (k in which(cc.n >= 2L)) {
            kk <- kk + 1
            constr$A[kk, g$cc$nodes[[k]]] <- 1
        }
        stopifnot(kk == cc.n2)
        rankdef <- cc.n2
    } else {
        nc <- 1
        constr <- list(A = matrix(1, nc, n), e = rep(1, nc))
        rankdef <- cc.n1 + 1
    }

    return(list(
        rankdef = nrow(constr$A),
        constr = constr,
        rankdef = rankdef,
        cc.n = cc.n,
        cc.n1 = cc.n1,
        cc.n2 = cc.n2
    ))
}

inla.sparse.det.bym <- function(Q,
                                rankdef,
                                adjust.for.con.comp = TRUE,
                                log = TRUE,
                                constr = NULL,
                                eps = 0.01 * sqrt(.Machine$double.eps)) {
    ## compute the (log-)determinant. Assume a sum to zero constraint on each connected
    ## component >1. if 'constr' is given,  it is supposed to be the 'constr' that is
    ## constructed below.

    stopifnot(adjust.for.con.comp == TRUE)
    Q <- inla.as.sparse(Q)
    n <- dim(Q)[1]
    res <- NULL

    if (is.null(constr)) {
        res <- inla.bym.constr.internal(Q, adjust.for.con.comp = adjust.for.con.comp)
        constr <- res$constr
    }
    if (missing(rankdef)) {
        if (!is.null(constr)) {
            rankdef <- nrow(constr$A)
        } else {
            rankdef <- 0
        }
    }
    d <- diag(Q)
    diag(Q) <- d + mean(d) * eps
    res <- inla.qsample(n = 1, Q = Q, sample = matrix(0, n, 1), constr = constr)
    logdet <- 2.0 * (res$logdens + (n - rankdef) / 2.0 * log(2.0 * pi))
    return(if (log) logdet else exp(logdet))
}

inla.scale.model.bym <- function(Q, eps = sqrt(.Machine$double.eps), adjust.for.con.comp = TRUE) {
    ## the old version
    res <- inla.scale.model.bym.internal(Q = Q, adjust.for.con.comp = adjust.for.con.comp, eps = eps)
    return(res$Q)
}

inla.scale.model.bym.internal <- function(Q, eps = sqrt(.Machine$double.eps), adjust.for.con.comp = TRUE) {
    ## the new one which also return the (scaled) marginal variances
    n <- dim(Q)[1]
    constr <- list(A = matrix(1, nrow = 1, ncol = n), e = 0)

    if (adjust.for.con.comp) {
        res <- inla.scale.model.internal(Q, constr = constr, eps = eps)
    } else {
        mvar <- rep(NA, n)
        idx <- which(diag(Q) > 0)
        QQ <- Q[idx, idx, drop = FALSE]
        QQ <- inla.as.sparse(QQ)
        n <- dim(QQ)[1]
        constr <- list(A = matrix(1, nrow = 1, ncol = n), e = 0)
        res <- inla.qinv(QQ + Diagonal(n) * mean(diag(QQ)) * eps, constr = constr)
        fac <- exp(mean(log(diag(res))))
        QQ <- fac * QQ
        Q[idx, idx] <- QQ
        mvar[idx] <- diag(res) / fac
        res <- list(Q = Q, var = mvar)
    }
    return(res)
}

inla.pc.bym.Q <- function(graph) {
    Q <- -inla.graph2matrix(graph)
    diag(Q) <- 0
    diag(Q) <- -rowSums(Q)
    return(Q)
}

## also used for the rw2d+iid model (eigenvalues/marginal.variances arguments...)
inla.pc.bym.phi <- function(graph,
                            Q,
                            eigenvalues = NULL,
                            marginal.variances = NULL,
                            rankdef,
                            alpha,
                            u = 1 / 2,
                            lambda,
                            scale.model = TRUE,
                            return.as.table = FALSE,
                            adjust.for.con.comp = TRUE,
                            ## where to switch to alternative strategy
                            eps = sqrt(.Machine$double.eps),
                            debug = FALSE) {
    my.debug <- function(...) if (debug) cat("*** debug *** inla.pc.bym.phi: ", ..., "\n")

    ## I must assume this!!!
    stopifnot(scale.model == TRUE)
    stopifnot(adjust.for.con.comp == TRUE)
    res <- NULL

    if (missing(eigenvalues) && missing(marginal.variances)) {
        ## computes the prior for the mixing parameter `phi'.
        if (missing(graph) && !missing(Q)) {
            Q <- inla.as.sparse(Q)
        } else if (!missing(graph) && missing(Q)) {
            Q <- inla.pc.bym.Q(graph)
        } else if (missing(graph) && missing(Q)) {
            stop("Either <Q> or <graph> must be given.")
        } else {
            stop("Only one of <Q> and <graph> can be given.")
        }

        res <- inla.bym.constr.internal(Q, adjust.for.con.comp = adjust.for.con.comp)
        if (missing(rankdef)) {
            rankdef <- res$rankdef
        }

        n <- dim(Q)[1]
        res <- inla.scale.model.bym.internal(Q, adjust.for.con.comp = adjust.for.con.comp)
        Q <- res$Q
        f <- mean(res$var, na.rm = TRUE) - 1.0
        use.eigenvalues <- FALSE
    } else {
        n <- length(eigenvalues)
        eigenvalues <- pmax(0, sort(eigenvalues, decreasing = TRUE))
        gamma.invm1 <- c(1 / eigenvalues[1:(n - rankdef)], rep(0, rankdef)) - 1.0
        f <- mean(marginal.variances) - 1.0
        use.eigenvalues <- TRUE
    }

    if (use.eigenvalues) {
        ## this is fast for low dimension where we can compute the
        ## eigenvalues
        phi.s <- 1 / (1 + exp(-seq(-15, 12, len = 1000)))
        d <- numeric(length(phi.s))
        k <- 1
        for (phi in phi.s) {
            aa <- n * phi * f
            ## bb = sum(log(1-phi + phi*gamma.inv))
            bb <- sum(log1p(phi * gamma.invm1))
            ## bb = sum(log(1+phi*gamma.invm1))
            ## sometimes we *might* experience numerical problems, so we need
            ## to remove (if any) small phi's.
            d[k] <- (if (aa >= bb) sqrt(aa - bb) else NA)
            k <- k + 1
        }
    } else {
        ## alternative strategy for larger matrices
        phi.s <- 1 / (1 + exp(-c(seq(-15, 0, len = 40), 1:12)))
        d <- numeric(length(phi.s))
        log.q1.det <- inla.sparse.det.bym(Q,
            adjust.for.con.comp = adjust.for.con.comp,
            constr = res$constr, rankdef = rankdef
        )
        d <- c(unlist(
            inla.mclapply(
                phi.s,
                function(phi) {
                    aa <- n * phi * f
                    bb <- (n * log((1.0 - phi) / phi) +
                        inla.sparse.det.bym(Q + phi / (1 - phi) * Diagonal(n),
                            adjust.for.con.comp = adjust.for.con.comp,
                            constr = res$constr, rankdef = rankdef
                        ) -
                        (log.q1.det - n * log(phi)))
                    my.debug("aa=", aa, " bb=", bb, " sqrt(", aa - bb, ")")
                    return(if (aa >= bb) sqrt(aa - bb) else NA)
                }
            )
        ))
    }
    names(d) <- NULL

    ## remove phi's that failed, if any
    remove <- is.na(d)
    my.debug(paste(sum(remove), "out of", length(remove), "removed"))
    d <- d[!remove]
    phi.s <- phi.s[!remove]
    remove <- 1:6
    d <- d[-remove]
    phi.s <- phi.s[-remove]
    phi.intern <- log(phi.s / (1 - phi.s))
    ff.d <- splinefun(phi.intern, log(d))
    phi.intern <- seq(min(phi.intern) - 0, max(phi.intern), len = 10000)
    phi.s <- 1 / (1 + exp(-phi.intern))
    d <- exp(ff.d(phi.intern))

    ## ff.d:  return distance as a function of phi.intern
    ff.d.core <- splinefun(phi.intern, log(d))
    ff.d <- function(phi.intern, deriv = 0) {
        if (deriv == 0) {
            return(exp(ff.d.core(phi.intern)))
        } else if (deriv == 1) {
            return(exp(ff.d.core(phi.intern)) * ff.d.core(phi.intern, deriv = 1))
        } else {
            stop("ERROR")
        }
    }
    ## f.d: return distance as a function of phi
    f.d <- function(phi.s) ff.d(log(phi.s / (1 - phi.s)))
    d <- f.d(phi.s)
    if (missing(lambda)) {
        ## Prob(phi < u) = alpha gives an analytical solution
        stopifnot(alpha > 0.0 && alpha < 1.0 && u > 0)
        lambda <- -log(1 - alpha) / f.d(u)
    }

    ## this is the log.prior wrt phi.intern
    log.jac <- log(abs(ff.d(phi.intern, deriv = 1)))
    log.prior <- log(lambda) - lambda * d + log.jac
    ssum <- sum(exp(log.prior) * (c(0, diff(phi.intern)) + c(diff(phi.intern), 0))) / 2.0
    my.debug("empirical integral of the prior = ", ssum, ". correct it.")
    log.prior <- log.prior - log(ssum)

    if (return.as.table) {
        ## return this prior as a "table:" converted to the intern scale for phi.
        my.debug("return prior as a table on phi.intern")
        theta.prior.table <- paste(c("table:", cbind(phi.intern, log.prior)),
            sep = "", collapse = " "
        )
        return(theta.prior.table)
    } else {
        ## return a function which evaluates the log-prior as a function of phi.
        my.debug("return prior as a function of phi")
        ## do the interpolation in the intern scale
        prior.theta.1 <- splinefun(phi.intern, log.prior + phi.intern + 2 * log(1 + exp(-phi.intern)))
        prior.phi <- function(phi) prior.theta.1(log(phi / (1.0 - phi)))
        return(prior.phi)
    }
}

inla.pc.rw2diid.phi <- function(
                                size,
                                alpha,
                                u = 1 / 2,
                                lambda,
                                return.as.table = FALSE,
                                debug = FALSE) {
    ## not all functions here are used, but...
    DFT2 <- function(x) {
        fft(x, inverse = FALSE) / sqrt(length(x))
    }

    IDFT2 <- function(x) {
        fft(x, inverse = TRUE) / sqrt(length(x))
    }

    make.base.1 <- function(size, delta = 0) {
        if (length(size) == 1) size <- c(size, size)
        base <- matrix(data = 0, nrow = size[1], ncol = size[2])
        base[1, 1] <- 4 + delta
        base[1, 2] <- base[2, 1] <- base[size[1], 1] <- base[1, size[2]] <- -1
        return(base)
    }

    eigenvalues <- function(base) {
        return(Re(sqrt(length(base)) * DFT2(base)))
    }

    eigenvalues.rw2d <- function(base) {
        n <- dim(base)[1]
        outer(
            0:(n - 1), 0:(n - 1),
            function(i, j) {
                two.pi <- 2 * pi
                return(base[1, 1]
                + base[2, 1] * cos(two.pi * i / n)
                    + base[1, 2] * cos(two.pi * j / n)
                    + base[n, n] * cos(two.pi * ((n - 1) * i / n + (n - 1) * j / n))
                    + base[n, 1] * cos(two.pi * (n - 1) * i / n)
                    + base[1, n] * cos(two.pi * (n - 1) * j / n))
            }
        )^2
    }

    make.base <- function(size, delta = 0) {
        base <- make.base.1(size, delta)
        base2 <- Re(IDFT2(DFT2(base)^2)) * sqrt(length(base))
        return(base2)
    }

    inverse <- function(base, rankdef, factor = 100) {
        eig <- Re(DFT2(base))
        eig.max <- max(eig)
        if (missing(rankdef)) {
            pos <- eig > eig.max * factor * .Machine$double.eps
        } else {
            eig.sort <- sort(eig)
            if (rankdef > 0) {
                pos <- eig > eig.sort[rankdef]
            } else {
                pos <- eig > 0
            }
        }
        eig[pos] <- 1 / eig[pos]
        eig[!pos] <- 0
        base.inv <- Re(IDFT2(eig)) / length(base)
        return(base.inv)
    }

    tozero <- function(base, factor = 100) {
        elm.max <- max(abs(base))
        base[abs(base) < factor * .Machine$double.eps * elm.max] <- 0
        return(base)
    }

    mult <- function(base, bbase) {
        a <- Re(DFT2(base))
        b <- Re(DFT2(bbase))
        ab <- a * b
        xx <- Re(IDFT2(ab)) * sqrt(length(base))
        return(xx)
    }

    check.inla <- function(size) {
        if (length(size) == 1) size <- c(size, size)
        n <- prod(size)
        y <- numeric(n)
        y[] <- 0
        idx <- 1:n
        r <- inla(y ~ -1 + f(idx, model = "rw2d", nrow = size[1], ncol = size[2], scale.model = TRUE, fixed = TRUE),
            data = data.frame(y, idx),
            family = "gaussian",
            control.family = list(hyper = list(prec = list(initial = 8, fixed = TRUE)))
        )
        return(r$logfile[grep(" prec_scale", r$logfile)])
    }

    if (length(size) == 1) {
        size <- rep(size, 2)
    }
    size <- size * 1.55

    ## numbers of the form 2^i*3^j*5^k
    good.size <- c(
        8, 9, 10, 12, 15, 16, 18, 20, 24, 25, 27, 30, 32,
        36, 40, 45, 48, 50, 54, 60, 64, 72, 75, 80, 81, 90, 96,
        100, 108, 120, 125, 128, 135, 144, 150, 160, 162, 180,
        192, 200, 216, 225, 240, 243, 250, 256, 270, 288, 300,
        320, 324, 360, 375, 384, 400, 405, 432, 450, 480, 486,
        500, 512, 540, 576, 600, 625, 640, 648, 675, 720, 729,
        750, 768, 800, 810, 864, 900, 960, 972, 1000, 1024, 1080,
        1125, 1152, 1200, 1215, 1250, 1280, 1296, 1350, 1440,
        1458, 1500, 1536, 1600, 1620, 1728, 1800, 1875, 1920,
        1944, 2000, 2025, 2048, 2160, 2187, 2250, 2304, 2400,
        2430, 2500, 2560, 2592, 2700, 2880, 2916, 3000, 3072,
        3125, 3200, 3240, 3375, 3456, 3600, 3645, 3750, 3840,
        3888, 4000, 4050, 4096
    )

    for (k in 1:length(size)) {
        size[k] <- good.size[min(which(size[k] <= good.size))]
    }

    base <- make.base(size)
    base <- base * inverse(base, rankdef = 1L)[1, 1]
    marg.var <- 1.0
    e.values <- pmax(0, eigenvalues(base))

    return(inla.pc.bym.phi(
        eigenvalues = c(e.values),
        marginal.variances = marg.var,
        return.as.table = return.as.table,
        debug = debug,
        alpha = alpha,
        u = u,
        lambda = lambda,
        rankdef = 1L
    ))
}

## Export: inla.pc.cormat.dim2p inla.pc.cormat.p2dim inla.pc.cormat.theta2R
## Export: inla.pc.cormat.R2theta inla.pc.cormat.r2R inla.pc.cormat.R2r
## Export: inla.pc.cormat.r2theta inla.pc.cormat.theta2r inla.pc.cormat.permute
## Export: inla.pc.cormat.rtheta inla.pc.cormat.dtheta

## ! \name{pc.cormat}
## ! \alias{inla.pc.cormat}
## ! \alias{inla.pc.cormat.dim2p}
## ! \alias{cormat.dim2p}
## ! \alias{inla.pc.cormat.p2dim}
## ! \alias{cormat.p2dim}
## ! \alias{inla.pc.cormat.theta2R}
## ! \alias{cormat.theta2R}
## ! \alias{inla.pc.cormat.R2theta}
## ! \alias{cormat.R2theta}
## ! \alias{inla.pc.cormat.r2R}
## ! \alias{cormat.r2R}
## ! \alias{inla.pc.cormat.R2r}
## ! \alias{cormat.R2r}
## ! \alias{inla.pc.cormat.r2theta}
## ! \alias{cormat.r2theta}
## ! \alias{inla.pc.cormat.theta2r}
## ! \alias{cormat.theta2r}
## ! \alias{inla.pc.cormat.permute}
## ! \alias{cormat.permute}
## ! \alias{inla.pc.cormat.rtheta}
## ! \alias{cormat.rtheta}
## ! \alias{inla.pc.cormat.dtheta}
## ! \alias{cormat.dtheta}
## !
## ! \title{Utility functions for the PC prior for a correlation matrix}
## !
## ! \description{Functions to evaluate and sample from the
## !              PC prior for a correlation matrix.}
## ! \usage{
## !    inla.pc.cormat.dim2p(dim)
## !    inla.pc.cormat.p2dim(p)
## !    inla.pc.cormat.theta2R(theta)
## !    inla.pc.cormat.R2theta(R)
## !    inla.pc.cormat.r2R(r)
## !    inla.pc.cormat.R2r(R)
## !    inla.pc.cormat.r2theta(r)
## !    inla.pc.cormat.theta2r(theta)
## !    inla.pc.cormat.permute(R)
## !    inla.pc.cormat.rtheta(n=1, p, lambda = 1)
## !    inla.pc.cormat.dtheta(theta, lambda = 1, log = FALSE)
## ! }
## ! \arguments{
## !   \item{dim}{The dimension of \code{theta}, the parameterisatin of the correlation matrix}
## !   \item{p}{The dimension the correlation matrix}
## !   \item{theta}{A vector of parameters for the correlation matrix}
## !   \item{r}{The off diagonal elements of a correlation matrix}
## !   \item{R}{A correlation matrix}
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter in the prior}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## ! }
## ! \details{
## !    The parameterisation of a correlation matrix of dimension \code{p} has \code{dim}
## !    parameters: \code{theta} which are in the interval -pi to pi.
## !    The alternative parameterisation is through the off-diagonal elements \code{r} of the
## !    correlation matrix \code{R}. The functions \code{inla.pc.cormat.<A>2<B>} convert between
## !    parameterisations \code{<A>} to parameterisations \code{<B>},  where both
## !    \code{<A>} and \code{<B>} are one of \code{theta},  \code{r} and \code{R},
## !    and \code{p} and \code{dim}.
## ! }
## ! \value{%%
## !     \code{inla.pc.cormat.rtheta} generate samples from the prior,  returning a matrix
## !     where each row is a sample of \code{theta}.
## !     \code{inla.pc.cormat.dtheta} evaluates the density of \code{theta}.
## !     \code{inla.pc.cormat.permute} randomly permutes a correlation matrix,
## !     which is useful if an exchangable sample of a correlation matrix is required.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## !  p = 4
## !  print(paste("theta has length", inla.pc.cormat.p2dim(p)))
## !  theta = inla.pc.cormat.rtheta(n=1, p=4, lambda = 1)
## !  print("sample theta:")
## !  print(theta)
## !  print(paste("log.dens", inla.pc.cormat.dtheta(theta, log=TRUE)))
## !  print("r:")
## !  r = inla.pc.cormat.theta2r(theta)
## !  print(r)
## !  print("A sample from the non-exchangable prior, R:")
## !  R = inla.pc.cormat.r2R(r)
## !  print(R)
## !  print("A sample from the exchangable prior, R:")
## !  R = inla.pc.cormat.permute(R)
## !  print(R)
## ! }

inla.pc.cormat.dim2p <- function(dim) {
    p <- round(1 / 2 + 1 / 2 * sqrt(1 + 4L * dim * 2L))
    stopifnot(abs(dim - inla.pc.cormat.p2dim(p)) < sqrt(.Machine$double.eps))
    return(p)
}

inla.pc.cormat.p2dim <- function(p) {
    return(p * (p - 1L) / 2L)
}

inla.pc.cormat.theta2R <- function(theta) {
    p <- inla.pc.cormat.dim2p(length(theta))
    theta.m <- matrix(NA, p, p)
    theta.m[lower.tri(theta.m)] <- theta

    B <- matrix(0, p, p)
    B[1, 1] <- 1.0
    for (i in 2L:p) {
        for (j in 1L:i) {
            if (j == 1) {
                B[i, j] <- cos(theta.m[i, j])
            } else if (j >= 2L && j <= i - 1) {
                B[i, j] <- cos(theta.m[i, j]) * prod(sin(theta.m[i, 1:(j - 1)]))
            } else if (j == i) {
                B[i, j] <- prod(sin(theta.m[i, 1:(j - 1)]))
            } else {
                stop("This should not happen.")
            }
        }
    }
    R <- B %*% t(B)
    diag(R) <- 1.0 ## so that its exactly 1, not just numerically 1

    return(R)
}

inla.pc.cormat.R2theta <- function(R) {
    L <- t(chol(R))
    p <- dim(R)[1]

    ## overwrite L with theta
    for (i in 2:p) {
        if (i == 2) {
            L[2, 1] <- acos(L[2, 1])
        } else {
            for (j in 1:(i - 1)) {
                if (j == 1) {
                    L[i, j] <- acos(L[i, j])
                } else {
                    L[i, j] <- acos(L[i, j] / prod(sin(L[i, 1:(j - 1)])))
                }
            }
        }
    }
    theta <- L[lower.tri(L)]

    return(theta)
}

inla.pc.cormat.r2R <- function(r) {
    p <- inla.pc.cormat.dim2p(length(r))
    R <- matrix(1, p, p)
    R[lower.tri(R)] <- r
    R <- t(R)
    R[lower.tri(R)] <- r
    diag(R) <- 1.0

    return(R)
}

inla.pc.cormat.R2r <- function(R) {
    return(R[lower.tri(R)])
}

inla.pc.cormat.r2theta <- function(r) {
    R <- inla.pc.cormat.r2R(r)
    theta <- inla.pc.cormat.R2theta(R)

    return(theta)
}

inla.pc.cormat.theta2r <- function(theta) {
    R <- inla.pc.cormat.theta2R(theta)
    r <- inla.pc.cormat.R2r(R)

    return(r)
}

inla.pc.cormat.permute <- function(R) {
    r <- inla.pc.cormat.R2r(R)
    r <- r[order(runif(length(r)))]
    R <- inla.pc.cormat.r2R(r)

    return(R)
}

inla.pc.cormat.rtheta <- function(n = 1, p, lambda = 1) {
    stopifnot(!missing(p) && p > 1)
    stopifnot(lambda > 0)
    stopifnot(n >= 1)

    rcormat <- function(n, p, lambda) {
        q <- inla.pc.cormat.p2dim(p)
        ## sample a point on the simplex
        r <- rexp(1, rate = lambda)
        gamma <- rexp(q)
        gamma <- (gamma / sum(gamma)) * r^2 / 2
        theta <- asin(exp(-gamma))
        ## sample branch by random
        branch <- sample(c(0, 1), size = q, replace = TRUE)
        theta <- branch * theta + (1 - branch) * (pi - theta)
        return(theta)
    }

    x <- unlist(lapply(rep(1, n), rcormat, p = p, lambda = lambda))
    x <- matrix(x, n, inla.pc.cormat.p2dim(p), byrow = TRUE)
    return(x)
}

inla.pc.cormat.dtheta <- function(theta, lambda = 1, log = FALSE) {
    ## reimplementation using the simplex function
    p <- length(theta)
    gamma <- -log(sin(theta))
    ldens <- (inla.pc.multvar.simplex.d(gamma, lambda = lambda, log = TRUE, b = rep(1, p))
    + sum(log(abs(1 / tan(theta)))) - p * log(2))

    return(if (log) ldens else exp(ldens))
}

## Nothing to export

## Various utility functions

`inla.numlen` <- function(n) {
    ## number of digits required to represent a specific (integer)
    ## numb{e{r
    return(floor(log10(max(abs(n)))) + 1L)
}

`inla.num` <- function(x, width = if (length(x) > 1L) inla.numlen(x) else 8L, digits = max(4L, width)) {
    ## format numbers using preceeding zeros.
    ## > inla.num(sqrt(2))
    ## [1] "0001.414"
    ## > inla.num(2)
    ## [1] "00000002"

    ## for a sequence of numbers, the width is calculated
    ## automatically.> inla.num(1:10)
    ## > inla.num(1:10)
    ## [1] "01" "02" "03" "04" "05" "06" "07" "08" "09" "10"

    return(formatC(x, format = "g", width = width, flag = "0", digits = digits))
}


`inla.trim` <- function(string) {
    ## trim leading and trailing whitespaces. there is a function in R.oo called `trim' that do
    ## this, but I don't want INLA to be dependent on R.oo. This function also works the string
    ## is a list of strings.

    string <- gsub("^[ \t]+", "", string)
    string <- gsub("[ \t]+$", "", string)
    return(string)
}

`inla.namefix` <- function(string) {
    ## must be the same as in iniparser.h
    re <- "[$]"
    re.to <- "|S|"
    old.string <- inla.trim(string)
    ## special characters, need to do something
    while (TRUE) {
        string <- gsub(re, re.to, old.string)
        if (string == old.string) break
        old.string <- string
    }
    return(string)
}

`inla.nameunfix` <- function(string) {
    ## makes a nice printable version of STRING whic is a `name'
    if (FALSE) {
        string <- gsub("[ .]+", " ", string)
        string <- gsub("([0-9]) ([0-9])", "\\1.\\2", string)
    }
    return(string)
}

`inla.strcmp` <- function(s, ss) {
    ## compare two strings
    return(s == ss)
}

`inla.strncmp` <- function(s, ss) {
    ## compare two strings
    if (length(s) == 1L && length(ss) > 1L) {
        ans <- NULL
        for (i in 1L:length(ss)) {
            ans <- c(ans, inla.strncmp(s, ss[i]))
        }
        return(ans)
    } else if (length(s) > 1L && length(ss) > 1L) {
        stop("length(s) > 1 && length(ss) > 1: not allowed.")
    } else {
        return(substr(s, 1L, nchar(ss)) == ss)
    }
}

`inla.strcasecmp` <- function(s, ss) {
    ## compare two strings, ignore case
    return(tolower(s) == tolower(ss))
}

`inla.strncasecmp` <- function(s, ss) {
    ## compare two strings, ignore case
    if (length(s) == 1L && length(ss) > 1L) {
        ans <- NULL
        for (i in 1L:length(ss)) {
            ans <- c(ans, inla.strncasecmp(s, ss[i]))
        }
        return(ans)
    } else if (length(s) > 1L && length(ss) > 1L) {
        stop("length(s) > 1 && length(ss) > 1: not allowed.")
    } else {
        return(substr(tolower(s), 1L, nchar(ss)) == tolower(ss))
    }
}

`inla.pause` <- function(msg = NULL) {
    ## just print a msg and wait for the next return
    if (!is.null(msg)) {
          cat(msg, "\n")
      }
    scan(quiet = TRUE, multi.line = TRUE, what = character(0L))
}

`inla.call.builtin` <- function() {
    ## cannot call inla.getOption() here as it leads to an infinite recursive call. do this
    ## manually instead.
    opt.default <- inla.getOption.default()
    if (exists("inla.options", env = inla.get.inlaEnv())) {
        opt <- get("inla.options", env = inla.get.inlaEnv())
        mkl <- if (!is.null(opt$mkl)) opt$mkl else opt.default$mkl
        lic <- (!is.null(opt$pardiso.license) && nchar(opt$pardiso.license) > 0L)
    } else {
        mkl <- opt.default$mkl
        lic <- FALSE
    }
    mkl <- if (mkl) "mkl." else ""

    if (inla.os("mac")) {
        if (nchar(mkl) > 0L && !lic) mkl <- ""
        fnm <- system.file(paste("bin/mac/", inla.os.32or64bit(), "bit/inla.", mkl, "run", sep = ""), package = "INLA")
    } else if (inla.os("mac.arm64")) {
        fnm <- system.file("bin/mac.arm64/inla.run", package = "INLA")
    } else if (inla.os("linux")) {
        fnm <- system.file(paste("bin/linux/", inla.os.32or64bit(), "bit/inla.", mkl, "run", sep = ""), package = "INLA")
    } else if (inla.os("windows")) {
        fnm <- system.file(paste("bin/windows/", inla.os.32or64bit(), "bit/inla.exe", sep = ""), package = "INLA")
    } else {
        stop("Unknown OS")
    }

    if (file.exists(fnm)) {
        return(fnm)
    } else {
        stop(paste("INLA installation error; no such file", fnm))
    }
}

`inla.call.no.remote` <- function() {
    ## return what is defined in options$inla.call except for 'remote', for which we revert back
    ## to the builtin one
    inla.call <- inla.getOption("inla.call")
    if (is.null(inla.call) || any(inla.strcasecmp(inla.call, c("remote", "inla.remote")))) {
        inla.call <- inla.call.builtin()
    }
    return(inla.call)
}

`inla.fmesher.call.builtin` <- function() {
    if (inla.os("mac")) {
        fnm <- system.file(paste("bin/mac/", inla.os.32or64bit(), "bit/fmesher.run", sep = ""),
                           package = "INLA")
    } else if (inla.os("mac.arm64")) {
        fnm <- system.file("bin/mac.arm64/fmesher.run", package = "INLA")
    } else if (inla.os("linux")) {
        fnm <- system.file(paste("bin/linux/", inla.os.32or64bit(), "bit/fmesher.run", sep = ""), package = "INLA")
    } else if (inla.os("windows")) {
        fnm <- system.file(paste("bin/windows/", inla.os.32or64bit(), "bit/fmesher.exe", sep = ""), package = "INLA")
    } else {
        stop("Unknown OS")
    }

    if (file.exists(fnm)) {
        return(fnm)
    } else {
        stop(paste("INLA installation error; no such file", fnm))
    }
}

`inla.paste` <- function(strings, sep = " ") {
    return(paste(strings, collapse = sep, sep = ""))
}

`inla.only.for.developers` <- function(msg = "<<<ADD MESSAGE HERE>>>", strict = FALSE) {
    if (!(Sys.getenv("USER") %in%
        c("hrue", "rueh", "finnkrl", "finn", "cristian", "chiuchc", "elias", "vanniej"))) {
        if (strict) {
            stop(paste0("This function/feature [", msg, "] is for developers only."))
        } else {
            warning(paste0("This function/feature [", msg, "] is for developers only."))
        }
    }
    return(invisible())
}

`inla.my.update` <- function(dir, binaries = FALSE, ignore.regexp = NULL) {
    ## Set binaries=TRUE to set the inla.call and fmesher.call options
    ## To override the default binaries path, set binaries="/the/path/bin"

    inla.only.for.developers(msg = "inla.my.update", strict = TRUE)

    a <- inla.models()
    rm(a)
        
    if (Sys.getenv("USER") %in% "hrue") {
        dir.default <- "~/p/inla/r-inla/rinla/R"
        bin.default <- "~/bin/inla.debug"
    } else if (Sys.getenv("USER") %in% "rueh") {
        dir.default <- "~/build64/r-inla/rinla/R"
        bin.default <- "~/build64/local/bin"
    } else if (Sys.getenv("USER") %in% "elias") {
        dir.default <- "~/inla-project/source/inla/rinla/R"
        bin.default <- "~/inla-project/compile/local/bin"
    } else if (Sys.getenv("USER") %in% "fuglstad") {
        dir.default <- "~/build64/rinla/R"
        bin.default <- "~/build64/local/bin"
    } else if (Sys.getenv("USER") %in% c("rieblera", "ariebler")) {
        dir.default <- "~/inla/rinla/R"
        bin.default <- "~/local/bin"
    } else if (Sys.getenv("USER") %in% c("cristian", "chiuchc")) {
        dir.default <- "~/Desktop/r-inla/rinla/R"
        bin.default <- "~/local/bin"
    } else {
        dir.default <- "~/github/r-inla/rinla/R"
        bin.default <- "~/local/bin"
    }
    if (!missing(binaries)) {
        if (is.character(binaries)) {
            bin.default <- binaries
        }
        binaries <- TRUE
    } else {
        binaries <- FALSE
    }
    if (missing(dir)) {
        dir <- path.expand(dir.default)
    }
    if (binaries) {
        bin.path <- path.expand(bin.default)
    }

    files <- dir(dir, pattern = "[.][Rr]$")
    ## remove files matching 'ignore.regexp'
    if (!is.null(ignore.regexp)) {
        idx <- grep(ignore.regexp, files)
        files <- files[-idx]
    }

    ## source the files in a temporary environment and not the globalenv
    tmp.env <- new.env()
    for (ff in files) {
        fff <- paste(dir, "/", ff, sep = "")
        res <- try(local(
            {
                source(fff, local = TRUE)
            },
            envir = tmp.env
        ))
        if (class(res) %in% "try-error") {
              warning(paste0("Got an error while sourcing file ", fff))
          }
    }

    ## replace the ones in the INLA-namespace
    funcs <- ls(tmp.env)
    env <- as.environment("package:INLA")
    nfuncs <- 0L
    for (func in funcs) {
        if (existsFunction(f = func, where = tmp.env)) {
            locked <- try(bindingIsLocked(func, env), silent = TRUE)
            if (class(locked) %in% "try-error") {
                ## then this function does not exists in package:INLA,
                ## so we assign it in the globalenv()
                assign(func, get(func, envir = tmp.env), envir = globalenv())
            } else {
                ## otherwise, we change it in the environment and
                ## namespace of INLA.
                if (locked) {
                    unlockBinding(func, env)
                }
                ## 'assignInNamespace' might not be 'allowed' in the future.
                try(assignInNamespace(func, get(func, envir = tmp.env), ns = "INLA", envir = env), silent = TRUE)
                assign(func, get(func, envir = tmp.env), envir = env)
                if (locked) {
                    lockBinding(func, env)
                }
            }
            nfuncs <- nfuncs + 1L
        }
    }

    cat("Source files in ", dir, ". Loaded ", length(files), " files and replaced ", nfuncs, " functions.\n", sep = "")

    if (binaries) {
        inla.setOption("inla.call", path.expand(paste(bin.path, "/",
            if (inla.getOption("mkl")) "inla.mkl.run" else "inla.run",
            sep = ""
        )))
        inla.setOption("fmesher.call", path.expand(paste(bin.path, "/", "fmesher.run", sep = "")))
        cat("Define new values for 'inla.call' and 'fmesher.call'\n", sep = "")
    }

    ## hash the models again
    rm("inla.models", envir = inla.get.inlaEnv())
    rm("rinla.version", envir = inla.get.inlaEnv())
    cat("Reset stored 'inla.models()' in .inlaEnv\n")
    m <- inla.models()

    return(invisible())
}

`inla.remove` <- function(name, from) {
    ## remove NAME FROM. Works for both lists and data.frames

    if (FALSE) {
        ## OLD CODE
        if (is.list(from) || is.data.frame(from)) {
              for (nm in name) {
                    if (length(grep(nm, names(from))) > 0L) {
                          inla.eval(paste("from$", nm, "=NULL", sep = ""))
                      }
                }
          }
    } else {
        ## NEW CODE
        if (is.list(from) || is.data.frame(from)) {
              for (nm in name) {
                    from[which(nm == names(from))] <- NULL
                }
          }
    }

    return(from)
}

`inla.ifelse` <- function(test, yes, no) {
    if (length(test) > 1L) {
          stop("oops: len(test) > 1")
      }

    if (test) {
          return(yes)
      } else {
          return(no)
      }
}


`inla.sparse.matrix.pattern` <- function(A, factor = 1.0, size = NULL, reordering = NULL,
                                         binary.pattern = TRUE) {
    ## Calculate sparse matrix pattern, with optional resolution reduction
    A <- inla.as.dgTMatrix(A)
    n <- dim(A)
    if (is.null(reordering)) {
        AA <- list(i = A@i + 1L, j = A@j + 1L, values = A@x)
    } else {
        if (is.numeric(reordering)) {
            AA <- list(i = reordering[A@i + 1L], j = reordering[A@j + 1L], values = A@x)
        }
        else if (inla.is.element("reordering", reordering)) {
            AA <- list(i = reordering$reordering[A@i + 1L], j = reordering$reordering[A@j + 1L], values = A@x)
        } else {
            stop("This should not happen.")
        }
    }

    if (is.null(size)) {
        ## Resize by factor:
        size <- ceiling(n * factor)
    } else if (length(size) == 1L) {
        ## Keep aspect ratio:
        size <- c(size, size / n[1L] * n[2L])
    }
    fac <- size / n

    ## duplicated entries will simply add up, so we need to truncate
    M <- inla.as.dgTMatrix(sparseMatrix(
        i = pmin(size[1L], ceiling(AA$i * fac[1L])), j = pmin(size[2L], ceiling(AA$j * fac[2L])), x = 1.0,
        dims = size
    ))
    if (binary.pattern) {
          M@x[] <- 1.0
    }

    return(M)
}



`inla.scale` <- function(x) {
    ## didn't knew about this one...
    return(scale(x))
}

`inla.trim.family` <- function(family) {
    ## remove `_' and `.' and space and tabs, and then convert to
    ## lowercase. remove also everything after a `:', so that "name:
    ## a" is "name:".
    family <- gsub(":.*$", ":", family)
    family <- tolower(gsub("[_ \t.]+", "", family))
    return(family)
}

`inla.is.list.of.lists` <- function(a.list) {
    ## return TRUE if `a.list' is a list of lists, otherwise FALSE
    if (length(a.list) == 0L) {
        return(FALSE)
    } else {
        return(is.null(names(a.list)) && all(sapply(a.list, is.list)))
    }
}

`inla.as.list.of.lists` <- function(a) {
    if (is.matrix(a) || is.data.frame(a) || length(dim(a)) == 2L) {
        return(as.list(as.data.frame(as.matrix(a))))
    } else if (inla.is.list.of.lists(a) || TRUE) {
        return(a)
    } else {
        stop("Argument if of unknown type; do not know what to do...")
    }
}

`inla.replicate.list` <- function(a.list, nrep) {
    ## make a list() into a list of lists.  example: list(a=1) =>
    ## list(list(a=1), list(a=1), ...)

    if (nrep <= 0L) {
          stop("nrep must be > 0")
      }

    return(rep(list(a.list), nrep))
}

`inla.tictac` <- function(num, elms = "|/-\\|/-\\") {
    len <- nchar(elms)
    i <- (num %% len) + 1L
    return(substr(elms, i, i))
}

`inla.2list` <- function(x = NULL) {
    ## convert a vector `x' or string 'x' into the string ``c(x[1L],
    ## x[2L], x[3L])'' so it can be evaluated using
    ## inla.eval()

    if (is.numeric(x)) {
        return(as.character(enquote(as.numeric(x)))[2L])
    } else {
        return(as.character(enquote(x))[2L])
    }

    ## example:
    ## > x=1:3
    ## > inla.2list(x)
    ## [1] "c(1, 2, 3)"

    if (is.null(x)) {
          return(NULL)
    }

    if (!is.character(x)) {
        return(paste("c(", paste(x, collapse = ","), ")"))
    } else {
        if (length(x) == 0L) {
              return("numeric(0)")
          }
        s <- inla.paste(as.character(x))
        s <- gsub("^[ ]+", "", s)
        s <- gsub("[ ]+$", "", s)
        s <- gsub("c[ ]*[(][ ]*", "", s)
        s <- gsub("[ ]*[)][ ]*", "", s)
        return(paste("c(", gsub("[, ]+", ",", s, ")"), ")", sep = ""))
    }
}

`inla.even` <- function(n) {
    return(inla.divisible(n, by = 2L))
}

`inla.odd` <- function(n) {
    return(inla.divisible(n, by = -2L))
}

`inla.divisible` <- function(n, by = 2L) {
    ### if by>0, return TRUE if `n' is divisible by `by', and if by<0,
    ### return TRUE if `n' is not divisible by `-by'. if by==0, return
    ### TRUE.
    if (by == 0L) {
          return(rep(TRUE, length(n)))
      }
    if (by > 0L) {
        return((n %% by) == 0L)
    } else {
        return((n %% (-by)) != 0L)
    }
}

`inla.one.of` <- function(family, candidates) {
    if (is.null(candidates) || length(candidates) == 0L) {
        return(FALSE)
    }

    ## check if family is one of the canidates
    return(any(inla.trim.family(family) == inla.trim.family(candidates)))
}

`inla.get.HOME` <- function() {
    return(as.character(inla.ifelse(inla.os("windows"), gsub("\\\\", "/", Sys.getenv("USERPROFILE")), Sys.getenv("HOME"))))
}

`inla.get.USER` <- function() {
    u <- ""
    for (U in c("USER", "USERNAME", "LOGNAME")) {
        u <- Sys.getenv(U)
        if (u != "") {
            break
        }
    }
    if (u == "") {
        u <- "UnknownUserName"
    }

    return(as.character(u))
}


`inla.eval` <- function(command,
                        envir = parent.frame(),
                        enclos = if (is.list(envir) || is.pairlist(envir)) {
                              parent.frame()
                          } else {
                            baseenv()
                        }) {
    return(eval(parse(text = command), envir, enclos))
}

`inla.tempfile` <- function(pattern = "file", tmpdir = tempdir()) {
    ## just replace \ in Windows with /
    return(gsub("\\\\", "/", tempfile(pattern, tmpdir)))
}

`inla.tempdir` <- function() {
    ## just replace \ in Windows with /
    t.dir <- tempfile()
    inla.dir.create(t.dir)
    return(gsub("\\\\", "/", t.dir))
}


`inla.formula2character` <- function(formula) {
    ## convert a formula to characters without the 500 character
    ## constraint. Remove the ``()'' at the end.
    return(gsub("[(][)]$", "", inla.paste(deparse(formula))))
}

`inla.unique.rows` <- function(A) {
    ## return the unique rows and index-list how to map rows of A.

    ## Example:
    ## > A
    ## [, 1] [, 2]
    ## [1,]    1    1
    ## [2,]    2    2
    ## [3,]    3    3
    ## [4,]    1    1
    ## [5,]    2    2
    ## [6,]    3    3
    ## [7,]    4    4
    ## [8,]    4    4
    ## [9,]    1    1

    ## > inla.unique.rows(A)
    ## $rows
    ## [, 1] [, 2]
    ## [1,]    1    1
    ## [2,]    2    2
    ## [3,]    3    3
    ## [4,]    4    4
    ##
    ## $idx
    ## [1] 1 2 3 1 2 3 4 4 1


    stopifnot(is.matrix(A))

    ## we will use the builtin function 'duplicated', but since we
    ## have a matrix, we form a vector where each element is the
    ## character string of that column.

    n <- dim(A)[1L]
    ncol <- dim(A)[2L]
    a <- apply(A, 1L, function(x) inla.paste(c(as.character(x)), sep = "<|>"))

    ## then we know which columns that are duplicated. the next step
    ## is to make an index-table over unique rows, and if duplicated,
    ## the index to the first equal unique row.

    dup <- as.integer(duplicated(a))
    nu <- sum(!dup)
    uA <- matrix(NA, nu, ncol)
    k <- 1L
    for (i in 1L:n) {
        if (dup[i] == 1L) {
            idx <- which(a[i] == a[1L:(i - 1L)])[1L]
            dup[i] <- dup[idx]
        } else {
            dup[i] <- k
            uA[k, ] <- A[i, ]
            k <- k + 1L
        }
    }

    ## we need both the unique ones and the mapping

    return(list(rows = uA, idx = dup))
}

`inla.is.dir` <- function(dir) {
    return(!is.na(file.info(dir)$isdir) && file.info(dir)$isdir)
}

`inla.dirname` <- function(path) {
    if (identical(substr(path, nchar(path), nchar(path)), .Platform$file.sep)) {
        return(substr(path, 1L, nchar(path) - 1L))
    } else {
        return(dirname(path))
    }
}

`inla.affirm.integer` <- function(A, ...) {
    is.wholenumber <- function(x, tol = .Machine$double.eps * 2.0) {
        return(abs(x - round(x)) <= tol)
    }
    is.integer.values <- function(A, ...) {
        return(is.integer(A) || all(is.wholenumber(A, ...)))
    }
    if (is.integer.values(A, ...)) {
        A <- round(A)
        storage.mode(A) <- "integer"
    }
    return(A)
}

`inla.affirm.double` <- function(A, ...) {
    if (!is(A, "Matrix")) {
        storage.mode(A) <- "double"
    }
    return(A)
}

`inla.matrix2list` <- function(A, byrow = FALSE) {
    ## convert a matrix to a list of list, with columns as the first
    ## index (byrow=FALSE) with with rows as the first index
    ## (byrow=TRUE).

    if (byrow) {
        return(inla.matrix2list(t(A)))
    }

    stopifnot(is.matrix(A))
    a <- NULL
    for (i in 1L:nrow(A)) {
        b <- list(as.list(A[i, ]))
        names(b) <- rownames(A)[i]
        a <- c(a, b)
    }
    return(a)
}

`inla.dir.create` <- function(dir, showWarnings = TRUE, recursive = TRUE, mode = "0777", StopOnError = TRUE) {
    if (inla.is.dir(dir)) {
        return(dir)
    }

    result <- try(dir.create(dir, showWarnings = showWarnings, recursive = recursive, mode = mode))
    if ((inherits(result, "try-error") || !result)) {
        if (StopOnError) {
            stop(paste("Fail to create directory [", dir, "]. Stop.", sep = ""))
        }
        result <- NULL
    }

    return(result)
}

`inla.is.element` <- function(name, alist) {
    ## return TRUE if element with name NAME is a member of LIST and
    ## the value is non null and not NA.
    if (any(names(alist) == name)) {
        idx <- which(names(alist) == name)
        if (length(alist[[idx]]) > 0L && !is.null(alist[[idx]]) && !is.na(alist[[idx]])) {
            return(TRUE)
        } else {
            return(FALSE)
        }
    } else {
        return(FALSE)
    }
}
`inla.get.element` <- function(name, alist) {
    if (inla.is.element(name, alist)) {
        return(alist[[which(names(alist) == name)]])
    } else {
        return(NULL)
    }
}
`inla.require.inherits` <- function(x, what, name = "Object") {
    if (!inherits(x, what)) {
        n.what <- length(as.list(what))
        stop(paste(name, " must inherit from class ",
            inla.ifelse(
                n.what == 1L,
                paste("\"", what, "\".", sep = ""),
                paste("\"",
                    inla.paste(
                        what[1L:(n.what - 1L)],
                        "\", \""
                    ),
                    "\"",
                    inla.ifelse(n.what == 2L, "", ","),
                    " or \"",
                    what[n.what],
                    "\".",
                    sep = ""
                )
            ),
            sep = ""
        ))
    }
    return(invisible())
}


`inla.function2source` <- function(the.function, newline = "<<NEWLINE>>") {
    ## take a function and return the souce with 'newline' as the
    ## newline
    attributes(the.function) <- NULL
    return(paste(deparse(the.function), collapse = newline))
}

`inla.source2function` <- function(the.source, newline = "<<NEWLINE>>") {
    ## take function source, output from inla.function2source(), and
    ## return the function, using 'newline' as newline.
    return(inla.eval(strsplit(the.source, split = newline)[[1L]]))
}

`inla.writeLines` <- function(filename, lines) {
    ## write a sequence of lines to file in binary format. need to do
    ## it like this in order to solve the unix->windows encoding
    ## issue, reading and writing this same file on different
    ## platforms.

    fp <- file(filename, "wb")
    len <- length(lines)
    writeBin(len, fp)

    for (i in 1L:len) {
        nc <- nchar(lines[i])
        writeBin(nc, fp)
        writeChar(lines[i], fp, nchars = nchar(lines[i]), eos = NULL)
    }
    close(fp)
}

`inla.readLines` <- function(filename) {
    ## read a sequence of lines to file in binary format. need to do
    ## it like this in order to solve the unix->windows encoding
    ## issue, reading and writing this same file on different
    ## platforms.

    if (!file.exists(filename)) {
        return(NULL)
    }

    fp <- file(filename, "rb")
    len <- readBin(fp, integer(), n = 1L)
    lines <- character(len)

    for (i in 1L:len) {
        nc <- readBin(fp, integer(), n = 1L)
        lines[i] <- readChar(fp, nc)
    }
    close(fp)

    return(lines)
}

`inla.is.matrix` <- function(A) {
    ## return TRUE if A is a 'matrix' neglecting possible formats
    ## (dense, sparse, etc...)
    if (is.matrix(A) || is(A, "Matrix")) {
        ## the common cases
        return(TRUE)
    } else {
        ## then its something else
        d <- dim(A)
        if (is.null(d)) {
            return(FALSE)
        } else {
            return(length(d) == 2L)
        }
    }
}


`inla.factor2matrix` <- function(f, sparse = FALSE) {
    ok <- !is.na(f)
    levels <- levels(f)
    if (sparse) {
        factor.matrix <- sparseMatrix(
            i = which(ok), j = as.integer(f[ok]), x = 1.0,
            dims = c(length(f), nlevels(f))
        )
    } else {
        factor.matrix <- matrix(0.0, length(f), nlevels(f))
        for (k in 1L:nlevels(f)) {
            factor.matrix[ok, k] <- (f[ok] == levels[k])
        }
    }
    colnames(factor.matrix) <- levels(f)
    rownames(factor.matrix) <- names(f)
    return(factor.matrix)
}

`inla.require` <- function(pkg, stop.on.error = FALSE, quietly = TRUE, ...) {
    ## follow the `new' standard...
    ret <- requireNamespace(pkg, quietly = quietly, ...)
    if (!ret && stop.on.error) {
        stop(paste0("Package '", pkg, "' is required to proceed, but is not installed. Please install."))
    } else {
        return (ret)
    }
}

`inla.inlaprogram.has.crashed` <- function() {
    stop("The inla-program exited with an error. Unless you interupted it yourself, please rerun with verbose=TRUE and check the output carefully.\n  If this does not help, please contact the developers at <help@r-inla.org>.")
}

`inla.inlaprogram.timeout` <- function(timeused, timeout) {
    if (timeout > 0 && timeused > timeout) {
        stop(paste0(" *** Interupted after ", round(dig = 1, timeused),
                    " seconds due to timeout = ", round(dig = 1, timeout), " seconds"))
    }
}

`inla.eval.dots` <- function(..., stop.if.no.name = TRUE, allowed.names = NULL) {
    ## evaluate named argument in the parent frame. allowed.names can
    ## be a list of allowed names, or if NULL then all names are
    ## allowed. this function will give an error if one argument has
    ## no name and stop.of.no.name is TRUE
    if (!length(list(...))) {
        return(invisible())
    }
    dots <- lapply(match.call(), eval, envir = parent.frame())[-1L]
    for (i in seq_along(dots)) {
        nm <- names(dots)[i]
        if (!is.null(nm)) {
            if (!is.null(allowed.names)) {
                if (!(nm %in% allowed.names)) {
                    stop(paste("This argument is not allowed:", nm))
                }
            }
            assign(nm, dots[[i]], envir = parent.frame())
        } else {
            if (stop.if.no.name) {
                stop(paste("The", i, "th argument has no name."))
            }
        }
    }
    return(invisible())
}


`match.arg.vector` <- function(arg = NULL,
                               choices,
                               length = NULL) {
    ## Like match.arg, but for a vector of options 'arg'
    if (is.null(length)) {
        length <- inla.ifelse(is.null(arg), 1L, length(arg))
    }
    if (is.null(arg)) {
        arg <- match.arg(arg, choices)
    } else {
        for (k in seq_along(arg)) {
            arg[k] <- match.arg(arg[k], choices)
        }
    }
    if (length(arg) < length) {
        arg <- c(arg, rep(arg, length - length(arg)))
    } else if (length(arg) > length) {
        stop("Option list too long.")
    }
    return(arg)
}

`inla.get.var` <- function(var, data = NULL) {
    if (is.null(data)) {
        return(get(var))
    } else {
        return(get(var, envir = as.environment(data)))
    }
}

`inla.ginv` <- function(x, tol = sqrt(.Machine$double.eps), rankdef = NULL) {
    ## from MASS::ginv, but with added option 'rankdef'.
    if (!is.matrix(x)) {
        x <- as.matrix(x)
    }
    if (length(dim(x)) > 2L || !(is.numeric(x) || is.complex(x))) {
        stop("'x' must be a numeric or complex matrix")
    }

    xsvd <- svd(x)
    if (is.complex(x)) {
        xsvd$u <- Conj(xsvd$u)
    }

    if (is.null(rankdef) || rankdef == 0L) {
        Positive <- xsvd$d > max(tol * xsvd$d[1L], 0.0)
    } else {
        n <- length(xsvd$d)
        stopifnot(rankdef >= 1L && rankdef <= n)
        Positive <- c(rep(TRUE, n - rankdef), rep(FALSE, rankdef))
    }

    if (all(Positive)) {
        xsvd$v %*% (1.0 / xsvd$d * t(xsvd$u))
    } else if (!any(Positive)) {
        array(0.0, dim(x)[2L:1L])
    } else {
        xsvd$v[, Positive, drop = FALSE] %*% ((1.0 / xsvd$d[Positive]) *
            t(xsvd$u[, Positive, drop = FALSE]))
    }
}
`inla.gdet` <- function(x, tol = sqrt(.Machine$double.eps), rankdef = NULL, log = TRUE) {
    x <- as.matrix(x)
    lambda <- eigen(x, only.values = TRUE)$values
    if (is.null(rankdef) || rankdef == 0L) {
        non.zero <- (lambda > max(tol * max(lambda), 0.0))
        lambda <- lambda[non.zero]
    } else {
        if (rankdef > 0L) {
            lambda <- sort(lambda)
            lambda <- lambda[-(1L:rankdef)]
        }
    }

    if (log) {
        return(sum(log(lambda)))
    } else {
        return(prod(lambda))
    }
}

## nice to have these around
`inla.rw1` <- function(n, ...) {
    inla.rw(n, order = 1L, ...)
}

`inla.rw2` <- function(n, ...) {
    inla.rw(n, order = 2L, ...)
}

`inla.rw` <- function(n, order = 1L, sparse = TRUE, scale.model = FALSE, cyclic = FALSE) {
    stopifnot(n >= 1L + 2L * order)
    if (scale.model) {
        if (!cyclic) {
            rd <- order
        } else {
            if (order == 1L) {
                rd <- 1L
            } else {
                rd <- order - 1L
            }
        }
        Q <- inla.rw(n, order = order, sparse = sparse, scale.model = FALSE, cyclic = cyclic)
        fac <- exp(mean(log(diag(inla.ginv(as.matrix(Q), rankdef = rd)))))
        Q <- fac * Q
    } else {
        if (!cyclic) {
            U <- diff(diag(n), diff = order)
            Q <- t(U) %*% U
        } else {
            m <- 2L * order + 1L
            k <- 1L + order
            U <- diff(diag(m), diff = order)
            U <- t(U) %*% U
            Q <- toeplitz(c(U[k, k:m], rep(0.0, n - m), U[k, m:(k + 1L)]))
        }
    }
    return(if (sparse) inla.as.sparse(Q) else Q)
}

##
`inla.mclapply` <- function(..., mc.cores = NULL, parallel = TRUE) {
    if (parallel && !inla.os("windows")) {
        ## slightly different default 'mc.cores'
        if (is.null(mc.cores)) {
            mc.cores <- getOption("mc.cores", NULL)
            if (is.null(mc.cores)) {
                num.threads <- inla.getOption("num.threads")
                num.threads <- inla.parse.num.threads(num.threads)
                nt <- as.numeric(strsplit(num.threads, ":")[[1L]])
                if (nt[1L] == 0L) {
                    mc.cores <- parallel::detectCores(all.tests = TRUE, logical = FALSE)
                } else {
                    mc.cores <- nt[1L]
                }
            }
        }
        return(parallel::mclapply(..., mc.cores = mc.cores))
    } else {
        return(lapply(...))
    }
}

`inla.cmpfun` <- function(fun, options = list(optimize = 3L, suppressUndefined = TRUE)) {
    if (inla.require("compiler")) {
        return(compiler::cmpfun(fun, options = options))
    } else {
        return(fun)
    }
}

`inla.sn.reparam` <- function(moments, param) {
    stopifnot((!missing(moments) && missing(param)) ||
        (missing(moments) && !missing(param)))

    if (!missing(param)) {
        if (is.list(param)) {
            xi <- param$xi
            omega <- param$omega
            alpha <- param$alpha
        } else {
            xi <- param[1L]
            omega <- param[2L]
            alpha <- param[3L]
        }
        delta <- alpha / sqrt(1 + alpha^2L)
        mean <- xi + omega * delta * sqrt(2.0 / pi)
        variance <- omega^2L * (1.0 - 2.0 * delta^2L / pi)
        skewness <- (4.0 - pi) / 2.0 * (delta * sqrt(2.0 / pi))^3L / (1.0 - 2.0 * delta^2L / pi)^(3.0 / 2.0)

        return(list(mean = mean, variance = variance, skewness = skewness))
    } else {
        if (is.list(moments)) {
            mean <- moments$mean
            variance <- moments$variance
            skewness <- moments$skewness
        } else {
            mean <- moments[1L]
            variance <- moments[2L]
            skewness <- moments[3L]
        }
        delta <- sqrt(pi / 2.0 * abs(skewness)^(2.0 / 3.0) /
            (abs(skewness)^(2.0 / 3.0) + ((4.0 - pi) / 2.0)^(2.0 / 3.0)))
        delta <- delta * sign(skewness)
        alpha <- delta / sqrt(1.0 - delta^2L)
        omega <- sqrt(variance / (1.0 - 2.0 * delta^2L / pi))
        xi <- mean - omega * delta * sqrt(2.0 / pi)

        return(list(xi = xi, omega = omega, alpha = alpha))
    }
}

`inla.runjags2dataframe` <- function(runjags.object) {
    ## convert from runjags-output to a data.frame
    inla.require("runjags", stop.on.error = TRUE)
    return(as.data.frame(runjags::combine.mcmc(runjags.object, collapse.chains = TRUE)))
}

`inla.check.location` <- function(loc, term, model, section = "latent") {
    if (is.null(loc) || length(loc) <= 1L) {
        return(invisible())
    }
    lim <- inla.model.properties(model, section)$min.diff
    if (is.null(lim)) {
          return(invisible())
      }

    min.diff <- min(diff(sort(loc))) / diff(range(loc))
    if (min.diff < lim) {
        stop(paste(
            sep = "",
            "Locations are too close for f(",
            term, ", model=\"",
            model, "\", ...): ",
            " min(diff(sort(x)))/diff(range(x)) = ",
            format(min.diff, scientific = TRUE, digits = 4L),
            " < ", format(lim, scientific = TRUE, digits = 4L), "\n",
            "  You can fix this by some kind of binning, see ?inla.group", "\n",
            "  If you want/need to bypass this check at your own risk, do", "\n",
            "\t> m = get(\"inla.models\", inla.get.inlaEnv())\n",
            "\t> m$", section, "$", model, "$min.diff = NULL\n",
            "\t> assign(\"inla.models\", m, inla.get.inlaEnv())"
        ))
    }
    return(invisible())
}

`inla.dynload.workaround` <- function() {
    stop("This function is replaced by: inla.binary.install()")
    return(invisible())
}

`inla.matern.cf` <- function(dist, range = 1.0, nu = 0.5) {
    ## the matern correlation function, with parameter 'range' as defined for the SPDE models.
    ## this function can vectorize over 'dist'
    kappa <- sqrt(8.0 * nu) / range ## this the definition used
    d <- kappa * dist
    res <- numeric(length(dist))
    is.zero <- (dist == 0.0)
    d <- d[!is.zero]
    res[is.zero] <- 1.0
    res[!is.zero] <- 1.0 / 2.0^(nu - 1.0) / gamma(nu) * d^nu * besselK(d, nu)
    return(res)
}


`inla.sn.par` <- function(mean, variance, skew) {
    ## return the parameters in the skew-normal for the given three moments. the parameters are
    ## like those given in the sn-package. based on code written by CC.

    sn.map <- function(mu, variance, skew) {
        delta <- sign(skew) * sqrt((pi / 2.0) * (abs(skew)^(2.0 / 3.0) / (((4.0 - pi) / 2.0)^(2.0 / 3.0) + abs(skew)^(2.0 / 3.0))))
        alpha <- delta / sqrt(1.0 - delta^2L)
        xi <- mu - delta * sqrt((2.0 * variance) / (pi - 2.0 * delta^2L))
        omega <- sqrt((pi * variance) / (pi - 2.0 * delta^2L))
        return(list(xi = xi, omega = omega, alpha = alpha))
    }

    skew.max <- 0.99
    skew[which(is.na(skew))] <- 0.0
    if (any(abs(skew) > skew.max)) {
        skew <- pmax(-skew.max, pmin(skew.max, skew))
        warning(paste0("One or more abs(skewness) are too high. Coerced to be ", skew.max))
    }
    return(sn.map(mean, variance, skew))
}

## Export: inla.agaussian

## ! \name{aggregate.gaussian}
## ! \alias{inla.agaussian}
## ! \alias{aggregate.gaussian}
## ! \alias{agaussian}
## !
## ! \title{Aggregate Gaussian into an equivalent observation}
## !
## ! \description{Aggreagate Gaussians observerd with the same mean and precision,
## !              into an eqv triplet, for use with \code{family="agaussian"}}
## !
## ! \usage{
## !     inla.agaussian(y, s = NULL)
## ! }
## !
## ! \arguments{
## !   \item{y}{Repeated observations. If \code{y} is a matrix,  then each row represents
## !            repeated observations. if \code{y} is a list,  then each element of the list is
## !            a vector of repeated observations. If \code{y} is a vector,  then the whole vector
## !            represents repeated observations. The optional scaling \code{s},  must have 
## !            the same format as \code{y},  ie \code{matrix} or \code{vector}.
## !            \code{NA}'s in \code{y} (and \code{s}) are removed and not used or counted.
## !            If \code{s} is given, then the \code{NA}-pattern in \code{y} and \code{s} must be the same.}
## !   \item{s}{Optinonal fixed scaling of the precisions. Must be in the same format as \code{y},  and
## !            have the same \code{NA}-pattern. See the documentation for details.}
## !  }
## !
## ! \value{
## !   The output is a \code{inla.mdata}-object ready for use
## !   with \code{family="agaussian"}. See the example in the documentation.
## ! }
## !
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## !  A = matrix(1:25,5,5)
## !  inla.agaussian(A)
## !
## !  A[1,-1] = NA
## !  A[2,-(2:3)] = NA
## !  inla.agaussian(A)
## ! }

`inla.agaussian` <- function(y, s = NULL) {

    agg <- function(i, yy, ss) {
        if (is.matrix(yy)) {
            y <- yy[i, ]
            nn <- length(y)
            s <- if (is.null(ss)) rep(1, nn) else ss[i, ]
        } else if (is.list(yy)) {
            y <- yy[[i]]
            nn <- length(y)
            s <- if (is.null(ss)) rep(1, nn) else ss[[i]]
        } else {
            y <- yy
            nn <- length(y)
            s <- if (is.null(ss)) rep(1, nn) else ss
        }
        
        idx <- !is.na(y)
        y <- y[idx]
        s <- s[idx]
        n <- length(y)
        m <- sum(s)
        y.bar <- sum(s*y)/m
        ldet.s <- 0.5 * sum(log(s))
        v <- sum(s * y^2) / m - y.bar^2
        
        return (list(v, ldet.s, m, n, y.bar))
    }
    
    if (is.matrix(y)) {
        return (inla.mdata(matrix(unlist(lapply(seq_len(nrow(y)), agg, yy = y, ss = s)),
                                  ncol = 5L, byrow = TRUE)))
    } else if (is.list(y)) {
        return (inla.mdata(matrix(unlist(lapply(seq_along(y), agg, yy = y, ss = s)),
                                  ncol = 5L, byrow = TRUE)))
    } else {
        return (inla.mdata(agg(yy = y, ss = s)))
    }
}
        
`inla.agaussian.test` <- function() {
    for(i in 1:10) {
        n <- sample(1:5, 1)
        s <- runif(n)
        y <- rnorm(n)
        mu <- rnorm(1)
        tau <- exp(rnorm(1))
        d <- inla.agaussian(y, s)
        v1 <- sum(dnorm(y, mean = mu, sd = 1/sqrt(s*tau), log = TRUE))
        v2 <- -d$Y4/2*log(2*pi) + 0.5 * d$Y4 * log(tau) + d$Y2 -
            0.5 * tau * d$Y3 * ( (d$Y5 - mu)^2 + d$Y1 )
        cat("Test ", i, "n=", n, "value=", v1, " error=", v2-v1, "\n")
    }
}

# Save the num.threads option so it can be restored in teardown-INLA.R
testthat_inla_num_threads <- INLA::inla.getOption("num.threads")
# Set num.threads = "1:1" to ensure within-system repeatability
INLA::inla.setOption(num.threads = "1:1")

## Nothing to export

### Functions to write the different sections in the .ini-file

`inla.text2vector` <- function(text) {
    ## > as.numeric(unlist(strsplit("1,2,3 5", "[ ,\t]")))
    ## [1] 1 2 3 5
    if (is.character(text)) {
        return(as.numeric(unlist(strsplit(text, "[ ,\t]+"))))
    } else {
        return(text)
    }
}

`inla.secsep` <- function(secname) {
    sep <- "!"
    if (missing(secname)) {
        return(sep)
    } else {
        return(paste0(sep, inla.namefix(secname), sep))
    }
}

`inla.write.hyper` <- function(hyper, file, prefix = "", data.dir, ngroup = -1L, low = -Inf, high = Inf) {
    stopifnot(!missing(hyper))
    stopifnot(!missing(file))

    if (is.null(hyper) || length(hyper) == 0L) {
        return()
    }
    len <- length(hyper)
    for (k in 1L:len) {
        if (len == 1L) {
            suff <- ""
        } else {
            suff <- as.character(k - 1L)
        }
        cat(prefix, "initial", suff, " = ", hyper[[k]]$initial, "\n", file = file, append = TRUE, sep = "")
        cat(prefix, "fixed", suff, " = ", as.numeric(hyper[[k]]$fixed), "\n", file = file, append = TRUE, sep = "")
        cat(prefix, "hyperid", suff, " = ", as.numeric(hyper[[k]]$hyperid), "\n", file = file, append = TRUE, sep = "")

        ## these are for "expression:"...
        ## if there are newlines,  remove them
        tmp.prior <- gsub("\n", "", hyper[[k]]$prior)
        ## remove preceding spaces
        tmp.prior <- gsub("^[ \t]+", "", tmp.prior)
        ## if the expression ends with a ";" with or without spaces, remove it
        tmp.prior <- gsub(";*[ \t]*$", "", tmp.prior)
        ## if there is a 'return (' replace it with 'return('
        tmp.prior <- gsub("return[ \t]+\\(", "return(", tmp.prior)
        ## for all priors except the "expression:" one,  then trim the name
        if (length(grep("^(expression|table)[ \t]*:", tolower(tmp.prior))) == 0L) {
            tmp.prior <- inla.trim.family(tmp.prior)
        }

        ## table: is now stored in a file
        if (length(grep("^table:", tmp.prior)) > 0) {
            tab <- substr(tmp.prior, nchar("table:") + 1, nchar(tmp.prior))
            xy <- as.numeric(unlist(strsplit(tab, "[ \t\n\r]+")))
            xy <- xy[!is.na(xy)]
            nxy <- length(xy) %/% 2L
            xx <- xy[1:nxy]
            yy <- xy[1:nxy + nxy]
            xy <- cbind(xx, yy)
            file.xy <- inla.tempfile(tmpdir = data.dir)
            inla.write.fmesher.file(xy, filename = file.xy)
            file.xy <- gsub(data.dir, "$inladatadir", file.xy, fixed = TRUE)
            cat(prefix, "prior", suff, " = table: ", file.xy, "\n", append = TRUE, sep = "", file = file)
        } else {
            cat(prefix, "prior", suff, " = ", tmp.prior, "\n", file = file, append = TRUE, sep = "")
        }

        cat(prefix, "parameters", suff, " = ", inla.paste(hyper[[k]]$param), "\n", file = file, append = TRUE, sep = "")

        ## the PCGEVTAIL prior is a special case, as the (low, high) is given as parameters 2
        ## and 3, in the prior. So we need to extract those, and make sure they are set to
        ## (low, high), so they will be replaced
        if (tmp.prior == "pcgevtail") {
            low <- hyper[[k]]$param[2]
            high <- hyper[[k]]$param[3]
        }

        to.t <- gsub("REPLACE.ME.ngroup", paste("ngroup=", as.integer(ngroup), sep = ""), inla.function2source(hyper[[k]]$to.theta))
        from.t <- gsub("REPLACE.ME.ngroup", paste("ngroup=", as.integer(ngroup), sep = ""), inla.function2source(hyper[[k]]$from.theta))
        to.t <- gsub("REPLACE.ME.low", paste("low=", as.numeric(low), sep = ""), to.t)
        from.t <- gsub("REPLACE.ME.low", paste("low=", as.numeric(low), sep = ""), from.t)
        to.t <- gsub("REPLACE.ME.high", paste("high=", as.numeric(high), sep = ""), to.t)
        from.t <- gsub("REPLACE.ME.high", paste("high=", as.numeric(high), sep = ""), from.t)
        cat(prefix, "to.theta", suff, " = ", to.t, "\n", file = file, append = TRUE, sep = "")
        cat(prefix, "from.theta", suff, " = ", from.t, "\n", file = file, append = TRUE, sep = "")

        ## do a second replacement so that we replace the functions with actual functions after
        ## the replacement of REPLACE.ME.....
        hyper[[k]]$from.theta <- eval(parse(text = gsub(
            "REPLACE.ME.ngroup", paste("ngroup=", as.integer(ngroup), sep = ""),
            inla.function2source(hyper[[k]]$from.theta, newline = "
")
        )))
        hyper[[k]]$from.theta <- eval(parse(text = gsub(
            "REPLACE.ME.low", paste("low=", as.numeric(low), sep = ""),
            inla.function2source(hyper[[k]]$from.theta, newline = "
")
        )))
        hyper[[k]]$from.theta <- eval(parse(text = gsub(
            "REPLACE.ME.high", paste("high=", as.numeric(high), sep = ""),
            inla.function2source(hyper[[k]]$from.theta, newline = "
")
        )))
        hyper[[k]]$to.theta <- eval(parse(text = gsub(
            "REPLACE.ME.low", paste("low=", as.numeric(low), sep = ""),
            inla.function2source(hyper[[k]]$to.theta, newline = "
")
        )))
        hyper[[k]]$to.theta <- eval(parse(text = gsub(
            "REPLACE.ME.high", paste("high=", as.numeric(high), sep = ""),
            inla.function2source(hyper[[k]]$to.theta, newline = "
")
        )))
        hyper[[k]]$to.theta <- eval(parse(text = gsub(
            "REPLACE.ME.ngroup", paste("ngroup=", as.integer(ngroup), sep = ""),
            inla.function2source(hyper[[k]]$to.theta, newline = "
")
        )))
    }

    return(hyper)
}

`inla.write.boolean.field` <- function(tag, val, file) {
    ## write tag = 1 or tag = 0 depending on val. if val is NULL do not write
    if (!is.null(val)) {
        if (val) {
            cat(tag, " = 1\n", sep = " ", file = file, append = TRUE)
        } else {
            cat(tag, " = 0\n", sep = " ", file = file, append = TRUE)
        }
    }
}

`inla.family.section` <- function(...) {
    ## this is just a wrapper to make the naming better
    return(inla.data.section(...))
}

`inla.data.section` <- function(
                                file, family, file.data, file.weights, file.attr, control, i.family = "",
                                link.covariates = link.covariates, data.dir) {
    ## this function is called from 'inla.family.section' only.
    cat(inla.secsep(), "INLA.Data", i.family, inla.secsep(), "\n", sep = "", file = file, append = TRUE)
    cat("type = data\n", sep = " ", file = file, append = TRUE)
    cat("likelihood = ", family, "\n", sep = " ", file = file, append = TRUE)
    cat("filename = ", file.data, "\n", sep = " ", file = file, append = TRUE)
    cat("weights = ", file.weights, "\n", sep = " ", file = file, append = TRUE)
    cat("attributes = ", file.attr, "\n", sep = " ", file = file, append = TRUE)

    cat("variant = ",
        inla.ifelse(is.null(control$variant), 0L, as.integer(control$variant)),
        "\n",
        file = file, append = TRUE
    )

    if (inla.one.of(family, c("cenpoisson", "zeroinflatedcenpoisson0", "zeroinflatedcenpoisson1"))) {
        if (is.null(control$cenpoisson.I)) {
            interval <- inla.set.control.family.default()$cenpoisson.I
        } else {
            ## must be integers
            interval <- round(control$cenpoisson.I)
        }
        if (length(interval) != 2L) {
            stop(paste("cenpoisson.I: Must be a vector of length 2.", length(interval)))
        }
        interval <- sort(pmax(0, interval))
        stopifnot(all(is.infinite(interval) == FALSE))
        cat("cenpoisson.I = ", interval[1], " ", interval[2], "\n", sep = "", file = file, append = TRUE)
    }

    if (inla.one.of(family, c("beta"))) {
        c.val <- control$beta.censor.value
        if (c.val < 0 || c.val >= 0.5) {
            stop(paste("beta.censor.value: Must be in the intervel 0 to 1/2", c.val))
        }
        cat("beta.censor.value = ", c.val, "\n", sep = "", file = file, append = TRUE)
    }

    if (TRUE) {
        if (!is.null(control$quantile)) {
              stop("control.family=list(quantile=...) is disabled. Use control.family=list(control.link=list(quantile=...)) instead")
          }
        quantile <- control$control.link$quantile
        if (is.numeric(quantile)) {
            if ((quantile <= 0.0) || (quantile >= 1.0)) {
                stop(paste("quantile: Must be a numeric in the interval (0, 1)"))
            }
        } else {
            quantile <- -1 ## so we get an error if used.
        }
        cat("quantile = ", quantile, "\n", sep = "", file = file, append = TRUE)
    }

    if (inla.one.of(family, "gev")) {
        cat("gev.scale.xi = ", inla.ifelse(is.null(control$gev.scale.xi), 0.01, control$gev.scale.xi), "\n",
            sep = "", file = file, append = TRUE
        )
    }

    if (inla.one.of(family, "bgev")) {
        c.bgev <- control$control.bgev
        nms <- names(c.bgev)
        for (i in seq_along(c.bgev)) {
            ## need this, as the xi.range is a vector of length 2
            cat("bgev.", nms[i], " = ", paste(as.numeric(c.bgev[[i]]), collapse = " "),
                "\n",
                sep = "", file = file, append = TRUE
            )
        }
    }

    inla.write.hyper(control$hyper, file, data.dir = data.dir)

    ## the link-part. first make it backward-compatible...
    if (!(is.null(control$link) || inla.strcasecmp(control$link, "default"))) {
        ## control$link is set,  use that if not control.link$model is set
        if (!(is.null(control$control.link$model) || inla.strcasecmp(control$control.link$model, "default"))) {
            stop("Both control.family$link (OBSOLETE) and control.family$control.link$model is set. Please use the latter only.")
        }
        control$control.link$model <- control$link
    }
    lmod <- control$control.link$model <- inla.model.validate.link.function(family, control$control.link$model)
    ord <- control$control.link$order
    cat("link.model = ", lmod, "\n", file = file, append = TRUE)
    if (inla.one.of(lmod, c("special1"))) {
        ## for these models, the argument order is required
        if (is.null(ord)) {
            stop(paste("For link-model:", lmod, ", 'order' must be spesified."))
        }
        cat("link.order = ", ord, "\n", file = file, append = TRUE)
        ## special
        if (ord > 1L && length(control$control.link$hyper$theta2$param) == 2L) {
            par <- control$control.link$hyper$theta2$param
            control$control.link$hyper$theta2$param <- c(rep(par[1], ord), c(diag(rep(par[2], ord))))
        }
    } else {
        if (!is.null(ord)) {
            stop(paste("For link-model:", lmod, ", the argument 'order' is not used and must be NULL."))
        }
    }

    variant <- control$control.link$variant
    if (inla.one.of(lmod, c("logoffset"))) {
        ## for these models, the argument 'variant' is required
        if ( ## general
            is.null(variant) ||
                ## model-spesific
                (inla.one.of(lmod, c("logoffset")) && all(variant != c(0, 1)))) {
            stop(paste("For link-model:", lmod, ", the argument variant must be 0 or 1,  not", variant))
        }
        cat("link.variant = ", variant, "\n", file = file, append = TRUE)
    } else {
        if (!is.null(variant)) {
            stop(paste("For link-model:", lmod, ", the argument 'variant' is not used and must be NULL."))
        }
    }

    a <- control$control.link$a
    if (inla.one.of(lmod, c("loga"))) {
        if (is.null(a) || !is.numeric(a) || (a <= 0 || a > 1)) {
              stop(paste("For link-model:", lmod, ", the argument 'a' must be given, and 0 < a <= 1"))
          }
        cat("link.a = ", a, "\n", file = file, append = TRUE)
    }

    inla.write.hyper(control$control.link$hyper, file, prefix = "link.", data.dir = dirname(file))
    if (!is.null(link.covariates)) {
        if (!is.matrix(link.covariates)) {
            link.covariates <- matrix(c(link.covariates), ncol = 1)
        }
        file.link.cov <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(link.covariates, filename = file.link.cov)
        file.link.cov <- gsub(data.dir, "$inladatadir", file.link.cov, fixed = TRUE)
        cat("link.covariates = ", file.link.cov, "\n", append = TRUE, sep = " ", file = file)
    }

    ## the mix-part
    inla.write.boolean.field("mix.use", !is.null(control$control.mix$model), file)
    if (!is.null(control$control.mix$model)) {
        cat("mix.model = ", control$control.mix$model, "\n", sep = "", file = file, append = TRUE)
        npoints <- as.integer(control$control.mix$npoints)
        stopifnot(npoints >= 5L)
        cat("mix.npoints = ", npoints, "\n", sep = "", file = file, append = TRUE)
        integrator <- match.arg(control$control.mix$integrator, c("default", "quadrature", "simpson"))
        cat("mix.integrator = ", integrator, "\n", sep = "", file = file, append = TRUE)
        inla.write.hyper(control$control.mix$hyper, file, prefix = "mix.", data.dir = dirname(file))
    }

    cat("\n", sep = " ", file = file, append = TRUE)
}

`inla.ffield.section` <- function(file, file.loc, file.cov, file.id.names = NULL, n, nrep, ngroup,
                                  file.extraconstr, file.weights, random.spec, results.dir, only.hyperparam, data.dir) {
    label <- random.spec$term
    prop <- inla.model.properties(random.spec$model, "latent", stop.on.error = TRUE)

    cat(inla.secsep(label), "\n", sep = "", file = file, append = TRUE)
    cat("dir = ", results.dir, "\n", sep = " ", file = file, append = TRUE)
    cat("type = ffield\n", sep = " ", file = file, append = TRUE)
    cat("model = ", random.spec$model, "\n", sep = " ", file = file, append = TRUE)
    if (!is.null(random.spec$same.as)) {
        cat("same.as = ", random.spec$same.as, "\n", sep = " ", file = file, append = TRUE)
    }
    cat("covariates = ", file.cov, "\n", sep = " ", file = file, append = TRUE)
    if (!is.null(file.id.names)) {
        cat("id.names =", file.id.names, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$diagonal)) {
        cat("diagonal =", random.spec$diagonal, "\n", sep = " ", file = file, append = TRUE)
    }
    inla.write.boolean.field("constraint", random.spec$constr, file)
    inla.write.boolean.field("si", random.spec$si, file)
    if (!is.null(file.extraconstr)) {
        cat("extraconstraint =", file.extraconstr, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$weights)) {
        cat("weights =", file.weights, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$spde.prefix)) {
        ## need a special one, as spde.prefix is not a file or a directory...
        fnm <- inla.copy.dir.for.section.spde(random.spec$spde.prefix, data.dir)
        cat("spde.prefix =", fnm, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$spde2.prefix)) {
        ## need a special one, as spde2.prefix is not a file or a directory...
        fnm <- inla.copy.dir.for.section.spde(random.spec$spde2.prefix, data.dir)
        cat("spde2.prefix =", fnm, "\n", sep = " ", file = file, append = TRUE)
        cat("spde2.transform =", random.spec$spde2.transform, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$spde3.prefix)) {
        ## need a special one, as spde3.prefix is not a file or a directory...
        fnm <- inla.copy.dir.for.section.spde(random.spec$spde3.prefix, data.dir)
        cat("spde3.prefix =", fnm, "\n", sep = " ", file = file, append = TRUE)
        cat("spde3.transform =", random.spec$spde3.transform, "\n", sep = " ", file = file, append = TRUE)
    }
    if (inla.one.of(random.spec$model, "copy")) {
        if (!is.null(random.spec$of)) {
            cat("of =", random.spec$of, "\n", sep = " ", file = file, append = TRUE)
        }
    }
    if (inla.one.of(random.spec$model, c("copy", "sigm", "revsigm", "log1exp", "fgn", "intslope"))) {
        if (!is.null(random.spec$precision)) {
            cat("precision =", random.spec$precision, "\n", sep = " ", file = file, append = TRUE)
        }
    }

    if (inla.one.of(random.spec$model, c("clinear", "copy", "mec", "meb"))) {
        if (is.null(random.spec$range)) {
            ## default is the identity mapping
            random.spec$range <- c(0, 0)
        }
        cat("range.low  =", random.spec$range[1], "\n", sep = " ", file = file, append = TRUE)
        cat("range.high =", random.spec$range[2], "\n", sep = " ", file = file, append = TRUE)
    }
    if (inla.one.of(random.spec$model, c("rw1", "rw2", "besag", "bym", "bym2", "besag2", "rw2d", "rw2diid", "seasonal"))) {
        if (is.null(random.spec$scale.model)) {
            random.spec$scale.model <- inla.getOption("scale.model.default")
        }
        cat("scale.model = ", as.numeric(random.spec$scale.model), "\n", sep = " ", file = file, append = TRUE)
    }
    if (inla.one.of(random.spec$model, c("besag", "bym", "bym2", "besag2"))) {
        cat("adjust.for.con.comp = ", as.numeric(random.spec$adjust.for.con.comp), "\n", sep = " ", file = file, append = TRUE)
    }

    if (FALSE) {
        ## this is only for the mvnorm prior,  which we do not use anymore (hrue/16/02/2015)
        if (inla.one.of(random.spec$model, "ar")) {
            ## set a default prior for order > 1 if the param is given only for p=1
            par <- random.spec$hyper$theta2$param
            if (length(par) == 2L && random.spec$order > 1L) {
                random.spec$hyper$theta2$param <- c(rep(par[1], random.spec$order), par[2] * diag(random.spec$order))
            }
        }
    }

    ## possible adaptive priors
    if (inla.one.of(random.spec$model, c("bym2", "rw2diid"))) {
        stopifnot(random.spec$hyper$theta2$short.name == "phi") ## just a check...
        if (random.spec$hyper$theta2$prior == "pc") {
            ## this is the pc-prior which is adaptive for this
            ## model. compute the prior here.
            if (inla.one.of(random.spec$model, "bym2")) {
                random.spec$hyper$theta2$prior <- inla.pc.bym.phi(
                    graph = random.spec$graph,
                    ## have to do this automatic
                    ## rankdef = random.spec$rankdef,
                    u = random.spec$hyper$theta2$param[1L],
                    alpha = random.spec$hyper$theta2$param[2L],
                    scale.model = TRUE,
                    return.as.table = TRUE,
                    adjust.for.con.comp = as.numeric(random.spec$adjust.for.con.comp)
                )
                random.spec$hyper$theta2$param <- numeric(0)
            } else if (inla.one.of(random.spec$model, "rw2diid")) {
                random.spec$hyper$theta2$prior <- inla.pc.rw2diid.phi(
                    size = c(random.spec$nrow, random.spec$ncol),
                    u = random.spec$hyper$theta2$param[1L],
                    alpha = random.spec$hyper$theta2$param[2L],
                    return.as.table = TRUE
                )
                random.spec$hyper$theta2$param <- numeric(0)
            } else {
                stop("This should not happpen.")
            }
        }
    }

    if (is.null(random.spec$range)) {
        low <- -Inf
        high <- Inf
    } else {
        low <- random.spec$range[1]
        high <- random.spec$range[2]
    }
    random.spec$hyper <- inla.write.hyper(random.spec$hyper, file, data.dir = data.dir, ngroup = ngroup, low = low, high = high)

    if (inla.model.properties(random.spec$model, "latent")$nrow.ncol) {
        cat("nrow = ", random.spec$nrow, "\n", sep = " ", file = file, append = TRUE)
        cat("ncol = ", random.spec$ncol, "\n", sep = " ", file = file, append = TRUE)

        if (!is.null(random.spec$bvalue)) {
            cat("bvalue = ", random.spec$bvalue, "\n", sep = " ", file = file, append = TRUE)
        }
        if (inla.one.of(random.spec$model, c("matern2d", "matern2dx2part0", "matern2dx2p1"))) {
            if (!is.null(random.spec$nu)) {
                cat("nu = ", random.spec$nu, "\n", sep = " ", file = file, append = TRUE)
            }
        }
    } else {
        cat("n = ", n, "\n", sep = " ", file = file, append = TRUE)
    }
    cat("nrep = ", inla.ifelse(is.null(nrep), 1, nrep), "\n", sep = " ", file = file, append = TRUE)

    if (!is.null(ngroup) && ngroup > 1) {
        cat("ngroup = ", ngroup, "\n", sep = " ", file = file, append = TRUE)

        if (!inla.one.of(random.spec$model, "copy")) {
            cat("group.model = ", random.spec$control.group$model, "\n", sep = " ", file = file, append = TRUE)
            inla.write.boolean.field("group.cyclic", random.spec$control.group$cyclic, file)
            if (is.null(random.spec$control.group$scale.model)) {
                random.spec$control.group$scale.model <- inla.getOption("scale.model.default")
            }
            inla.write.boolean.field("group.scale.model", random.spec$control.group$scale.model, file)
            inla.write.boolean.field("group.adjust.for.con.comp", random.spec$control.group$adjust.for.con.comp, file)
            if (inla.one.of(random.spec$control.group$model, "ar")) {
                ## 'order' is only used for model=ar
                p <- inla.ifelse(is.null(random.spec$control.group$order), 0, as.integer(random.spec$control.group$order))
                cat("group.order = ", p, "\n", sep = " ", file = file, append = TRUE)
            }
            if (inla.one.of(random.spec$control.group$model, "besag")) {
                stopifnot(!is.null(random.spec$control.group$graph))
                gfile <- inla.write.graph(random.spec$control.group$graph, filename = inla.tempfile())
                fnm <- inla.copy.file.for.section(gfile, data.dir)
                unlink(gfile)
                cat("group.graph = ", fnm, "\n", sep = " ", file = file, append = TRUE)
            } else {
                stopifnot(is.null(random.spec$control.group$graph))
            }
            random.spec$control.group$hyper <- (inla.write.hyper(random.spec$control.group$hyper,
                file = file, prefix = "group.",
                data.dir = data.dir, ngroup = ngroup
            ))
        }
    }

    if (!is.null(random.spec$cyclic)) {
        cat("cyclic = ", as.numeric(random.spec$cyclic), "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$season.length)) {
        cat("season = ", random.spec$season.length, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$graph)) {
        gfile <- inla.write.graph(random.spec$graph, filename = inla.tempfile())
        fnm <- inla.copy.file.for.section(gfile, data.dir)
        unlink(gfile)
        cat("graph = ", fnm, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(file.loc)) {
        cat("locations = ", file.loc, "\n", sep = " ", file = file, append = TRUE)
    }

    if (inla.one.of(random.spec$model, "z")) {
        ## This is for the random-effect Z*z, where Z is a n x m
        ## matrix and z are N_m(0, prec*C). We rewrite this as a model
        ## for zz = (v, z), where v is N(Z*z, high.precision). This
        ## gives the precision matrix for zz as A+prec*B, where A and
        ## B are defined below.
        Z <- inla.as.sparse(random.spec$Z)
        tZ <- t(Z)
        Z.n <- dim(Z)[1]
        Z.m <- dim(Z)[2]
        A <- inla.as.sparse(random.spec$precision * cbind(rbind(Diagonal(Z.n), -tZ), rbind(-Z, tZ %*% Z)))
        if (is.null(random.spec$Cmatrix)) {
            Cm <- inla.as.sparse(Diagonal(Z.m))
        } else {
            Cm <- inla.as.sparse(random.spec$Cmatrix)
        }
        stopifnot(all(Z.m == dim(Cm)))
        B <- inla.as.sparse(cbind(
            rbind(
                Diagonal(Z.n, 0.0), # n x n zero-matrix
                sparseMatrix(dims = c(Z.m, Z.n), i = 1, j = 1, x = 0)
            ), # m x n zero-matrix
            rbind(
                sparseMatrix(dims = c(Z.n, Z.m), i = 1, j = 1, x = 0), # n x m zero-matrix
                Cm
            )
        ))

        ## dimensions
        cat("z.n = ", Z.n, "\n", append = TRUE, sep = " ", file = file)
        cat("z.m = ", Z.m, "\n", append = TRUE, sep = " ", file = file)
        ## matrix A
        file.A <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(A, filename = file.A)
        file.A <- gsub(data.dir, "$inladatadir", file.A, fixed = TRUE)
        cat("z.Amatrix = ", file.A, "\n", append = TRUE, sep = " ", file = file)
        ## matrix B
        file.B <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(B, filename = file.B)
        file.B <- gsub(data.dir, "$inladatadir", file.B, fixed = TRUE)
        cat("z.Bmatrix = ", file.B, "\n", append = TRUE, sep = " ", file = file)
    }

    if (inla.one.of(random.spec$model, "dmatern")) {
        ## need the matrix of locations
        file.loc <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(random.spec$locations, filename = file.loc)
        file.loc <- gsub(data.dir, "$inladatadir", file.loc, fixed = TRUE)
        cat("dmatern.locations = ", file.loc, "\n", append = TRUE, sep = " ", file = file)
    }

    if (inla.one.of(random.spec$model, "generic3")) {
        ## For this model, Cmatrix is a list of matrices. Error checking have be done already in
        ## f()
        nC <- length(random.spec$Cmatrix)
        stopifnot(nC > 0L)
        cat("generic3.n = ", random.spec$n, "\n", append = TRUE, sep = "", file = file)
        cat("generic3.m = ", nC, "\n", append = TRUE, sep = "", file = file)
        for (k in 1L:nC) {
            file.A <- inla.tempfile(tmpdir = data.dir)
            inla.write.fmesher.file(inla.as.sparse(random.spec$Cmatrix[[k]]), filename = file.A)
            file.A <- gsub(data.dir, "$inladatadir", file.A, fixed = TRUE)
            cat("generic3.Cmatrix.", as.integer(k - 1L), " = ", file.A, "\n", append = TRUE, sep = "", file = file)
        }
    }

    ## if the Cmatrix is defined we need to process it except if its
    ## the z/generic3-model for which this has already been done.
    if (!inla.one.of(random.spec$model, c("z", "generic3")) && !is.null(random.spec$Cmatrix)) {
        if (is.character(random.spec$Cmatrix)) {
            fnm <- inla.copy.file.for.section(random.spec$Cmatrix, data.dir)
            cat("Cmatrix = ", fnm, "\n", append = TRUE, sep = " ", file = file)
        } else {
            file.C <- inla.tempfile(tmpdir = data.dir)
            inla.write.fmesher.file(random.spec$Cmatrix, filename = file.C)
            file.C <- gsub(data.dir, "$inladatadir", file.C, fixed = TRUE)
            cat("Cmatrix = ", file.C, "\n", append = TRUE, sep = " ", file = file)
        }
    }

    if (inla.one.of(random.spec$model, "slm")) {
        ## This is the spatial-lag-model (SLM),  with
        ## N_C(b,  Q = kappa*A1 + A2 + rho*kappa*B + rho^2*kappa*C)
        X <- random.spec$args.slm$X
        W <- random.spec$args.slm$W
        Q <- random.spec$args.slm$Q.beta
        slm.n <- dim(X)[1L]
        slm.m <- dim(X)[2L]

        cat("slm.n = ", slm.n, "\n", append = TRUE, sep = " ", file = file)
        cat("slm.m = ", slm.m, "\n", append = TRUE, sep = " ", file = file)
        cat("slm.rho.min = ", random.spec$args.slm$rho.min, "\n", append = TRUE, sep = " ", file = file)
        cat("slm.rho.max = ", random.spec$args.slm$rho.max, "\n", append = TRUE, sep = " ", file = file)

        ## matrix A1
        A1 <- cbind(
            rbind(
                Diagonal(slm.n),
                -t(X)
            ),
            rbind(
                -X,
                t(X) %*% X
            )
        )
        file.A1 <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(A1, filename = file.A1)
        file.A1 <- gsub(data.dir, "$inladatadir", file.A1, fixed = TRUE)
        cat("slm.A1matrix = ", file.A1, "\n", append = TRUE, sep = " ", file = file)

        ## matrix A2
        A2 <- cbind(
            rbind(
                Matrix(0, slm.n, slm.n),
                Matrix(0, slm.m, slm.n)
            ),
            rbind(
                Matrix(0, slm.n, slm.m),
                Q
            )
        )
        file.A2 <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(A2, filename = file.A2)
        file.A2 <- gsub(data.dir, "$inladatadir", file.A2, fixed = TRUE)
        cat("slm.A2matrix = ", file.A2, "\n", append = TRUE, sep = " ", file = file)

        ## matrix B
        B <- cbind(
            rbind(
                -(t(W) + W),
                t(X) %*% W
            ),
            rbind(
                t(W) %*% X,
                Matrix(0, slm.m, slm.m)
            )
        )
        file.B <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(B, filename = file.B)
        file.B <- gsub(data.dir, "$inladatadir", file.B, fixed = TRUE)
        cat("slm.Bmatrix = ", file.B, "\n", append = TRUE, sep = " ", file = file)

        ## matrix C
        C <- cbind(
            rbind(
                t(W) %*% W,
                Matrix(0, slm.m, slm.n)
            ),
            rbind(
                Matrix(0, slm.n, slm.m),
                Matrix(0, slm.m, slm.m)
            )
        )
        file.C <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(C, filename = file.C)
        file.C <- gsub(data.dir, "$inladatadir", file.C, fixed = TRUE)
        cat("slm.Cmatrix = ", file.C, "\n", append = TRUE, sep = " ", file = file)
    }

    if (inla.one.of(random.spec$model, "ar1c")) {
        Z <- random.spec$args.ar1c$Z
        Q.beta <- random.spec$args.ar1c$Q.beta
        ar1c.n <- dim(Z)[1L]
        ar1c.m <- dim(Z)[2L]

        cat("ar1c.n = ", ar1c.n, "\n", append = TRUE, sep = " ", file = file)
        cat("ar1c.m = ", ar1c.m, "\n", append = TRUE, sep = " ", file = file)

        ## matrix Z
        Z[ar1c.n, ] <- 0 ## not used
        file.Z <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(Z, filename = file.Z)
        file.Z <- gsub(data.dir, "$inladatadir", file.Z, fixed = TRUE)
        cat("ar1c.Z = ", file.Z, "\n", append = TRUE, sep = " ", file = file)

        ## matrix t(Z) %*% Z (makes my life simpler)
        ZZ <- t(Z) %*% Z
        file.ZZ <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(ZZ, filename = file.ZZ)
        file.ZZ <- gsub(data.dir, "$inladatadir", file.ZZ, fixed = TRUE)
        cat("ar1c.ZZ = ", file.ZZ, "\n", append = TRUE, sep = " ", file = file)

        file.Qbeta <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(Q.beta, filename = file.Qbeta)
        file.Qbeta <- gsub(data.dir, "$inladatadir", file.Qbeta, fixed = TRUE)
        cat("ar1c.Qbeta = ", file.Qbeta, "\n", append = TRUE, sep = " ", file = file)
    }

    ## the z-model for which this has already been done.
    if (!inla.one.of(random.spec$model, c("z", "generic3")) && !is.null(random.spec$Cmatrix)) {
        if (is.character(random.spec$Cmatrix)) {
            fnm <- inla.copy.file.for.section(random.spec$Cmatrix, data.dir)
            cat("Cmatrix = ", fnm, "\n", append = TRUE, sep = " ", file = file)
        } else {
            file.C <- inla.tempfile(tmpdir = data.dir)
            inla.write.fmesher.file(random.spec$Cmatrix, filename = file.C)
            file.C <- gsub(data.dir, "$inladatadir", file.C, fixed = TRUE)
            cat("Cmatrix = ", file.C, "\n", append = TRUE, sep = " ", file = file)
        }
    }

    if (inla.one.of(random.spec$model, "intslope")) {
        stopifnot(!is.null(random.spec$args.intslope))
        M.matrix <- cbind(
            random.spec$args.intslope$subject - 1L,
            random.spec$args.intslope$strata - 1L,
            random.spec$args.intslope$covariates
        )
        file.M <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(M.matrix, filename = file.M)
        file.M <- gsub(data.dir, "$inladatadir", file.M, fixed = TRUE)
        cat("intslope.def = ", file.M, "\n", append = TRUE, sep = " ", file = file)
        cat("intslope.nsubject = ", max(random.spec$args.intslope$subject), "\n", append = TRUE, sep = " ", file = file)
        cat("intslope.nstrata = ", max(random.spec$args.intslope$strata), "\n", append = TRUE, sep = " ", file = file)
    }

    if (!is.null(random.spec$rankdef)) {
        cat("rankdef = ", random.spec$rankdef, "\n", append = TRUE, sep = " ", file = file)
    }
    if (!is.null(random.spec$cdf)) {
        cat("cdf = ", random.spec$cdf, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(random.spec$quantiles)) {
        cat("quantiles = ", random.spec$quantiles, "\n", sep = " ", file = file, append = TRUE)
    }
    if (only.hyperparam || !random.spec$compute) {
        cat("compute = 0\n", sep = " ", file = file, append = TRUE)
    } else {
        cat("compute = 1\n", sep = " ", file = file, append = TRUE)
    }

    if (inla.one.of(random.spec$model, c("mec", "meb", "iid"))) {
        ## possible scale-variable
        if (!is.null(random.spec$scale)) {
            file.scale <- inla.tempfile(tmpdir = data.dir)
            ns <- length(random.spec$scale)
            if (is.null(random.spec$values)) {
                idxs <- 1:ns
            } else {
                idxs <- 1:ns
                stopifnot(length(random.spec$values) == ns)
                stopifnot(!is.null(idxs))
            }
            inla.write.fmesher.file(as.matrix(cbind(idxs - 1L, random.spec$scale)), filename = file.scale, debug = FALSE)
            ## print(cbind(idxs, random.spec$scale))
            file.scale <- gsub(data.dir, "$inladatadir", file.scale, fixed = TRUE)
            cat("scale =", file.scale, "\n", sep = " ", file = file, append = TRUE)
        }
    } else {
        if (!is.null(random.spec$scale)) {
            stop(paste("Section [", label, "]: option 'scale' is not NULL but not used.", sep = ""))
        }
    }

    if (random.spec$model == "rgeneric") {
        file.rgeneric <- inla.tempfile(tmpdir = data.dir)
        ## this must be the same name as R_GENERIC_MODEL in inla.h
        model <- paste(".inla.rgeneric.model", ".", random.spec$rgeneric$Id, sep = "")
        assign(model, random.spec$rgeneric$model)
        ## save model, or the object that 'model' expands to
        inla.eval(paste("save(", model,
            ", file = ", "\"", file.rgeneric, "\"",
            ", ascii = FALSE, compress = TRUE)",
            sep = ""
        ))
        fnm <- gsub(data.dir, "$inladatadir", file.rgeneric, fixed = TRUE)
        cat("rgeneric.file =", fnm, "\n", file = file, append = TRUE)
        cat("rgeneric.model =", model, "\n", file = file, append = TRUE)
        rm(model) ## do not need it anymore
    }

    if (inla.one.of(random.spec$model, c("ar", "fgn", "fgn2", "iidkd"))) {
        cat("order = ", random.spec$order, "\n", append = TRUE, sep = " ", file = file)
    }

    if (is.null(random.spec$vb.correct)) {
        random.spec$vb.correct <- -1L ## code for ``make the default choice''
    }
    cat("vb.correct = ", as.numeric(random.spec$vb.correct), "\n", append = TRUE, sep = "", file = file)
    cat("\n", sep = " ", file = file, append = TRUE)

    ## need to store the updated one
    return(random.spec)
}

`inla.inla.section` <- function(file, inla.spec, data.dir, inla.mode) {
    cat(inla.secsep("INLA.Parameters"), "\n", sep = " ", file = file, append = TRUE)
    cat("type = inla\n", sep = " ", file = file, append = TRUE)

    if (!is.null(inla.spec$int.strategy)) {
        cat("int.strategy = ", inla.spec$int.strategy, "\n", sep = " ", file = file, append = TRUE)
    }

    if (inla.one.of(inla.spec$int.strategy, c("user", "user.std", "user.expert"))) {
        if (is.null(inla.spec$int.design)) {
            stop(paste0("int.strategy = 'user' or 'user.std' or 'user.expert' require the integration design in 'int.design'"))
        }
        file.A <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(as.matrix(inla.spec$int.design), filename = file.A)
        file.A <- gsub(data.dir, "$inladatadir", file.A, fixed = TRUE)
        cat("int.design = ", file.A, "\n", sep = " ", file = file, append = TRUE)
    }

    if (!is.null(inla.spec$strategy)) {
        cat("strategy = ", inla.spec$strategy, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$adaptive.max)) {
        cat("adaptive.max = ", as.integer(inla.spec$adaptive.max), "\n", sep = " ", file = file, append = TRUE)
    }
    inla.write.boolean.field("fast", inla.spec$fast, file)
    if (!is.null(inla.spec$linear.correction)) {
        cat("linear.correction = ", inla.spec$linear.correction, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$h)) {
        cat("h = ", inla.spec$h, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$dz)) {
        cat("dz = ", inla.spec$dz, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$interpolator)) {
        cat("interpolator = ", inla.spec$interpolator, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$diff.logdens)) {
        cat("diff.log.dens = ", inla.spec$diff.logdens, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$print.joint.hyper)) {
        cat("fp.hyperparam = $inlaresdir/joint.dat\n", sep = "", file = file, append = TRUE)
    }

    if (is.null(inla.spec$tolerance) || is.na(inla.spec$tolerance)) {
        ## we need a default value
        inla.spec$tolerance <- 0.005
    }

    if (is.null(inla.spec$tolerance.f) || is.na(inla.spec$tolerance.f)) {
        inla.spec$tolerance.f <- inla.spec$tolerance * 2.0
    }
    cat("tolerance.f = ", inla.spec$tolerance.f, "\n", sep = " ", file = file, append = TRUE)

    if (is.null(inla.spec$tolerance.g) || is.na(inla.spec$tolerance.g)) {
        inla.spec$tolerance.g <- inla.spec$tolerance
    }
    cat("tolerance.g = ", inla.spec$tolerance.g, "\n", sep = " ", file = file, append = TRUE)

    if (is.null(inla.spec$tolerance.x) || is.na(inla.spec$tolerance.x)) {
        inla.spec$tolerance.x <- inla.spec$tolerance
    }
    cat("tolerance.x = ", inla.spec$tolerance.x, "\n", sep = " ", file = file, append = TRUE)

    if (!(is.null(inla.spec$tolerance.step) || is.na(inla.spec$tolerance.step))) {
        cat("tolerance.step = ", inla.spec$tolerance.step, "\n", sep = " ", file = file, append = TRUE)
    }

    inla.write.boolean.field("hessian.force.diagonal", inla.spec$force.diagonal, file)
    inla.write.boolean.field("skip.configurations", inla.spec$skip.configurations, file)
    inla.write.boolean.field("mode.known", inla.spec$mode.known.conf, file)
    inla.write.boolean.field("adjust.weights", inla.spec$adjust.weights, file)
    inla.write.boolean.field("lincomb.derived.correlation.matrix", inla.spec$lincomb.derived.correlation.matrix, file)

    if (!is.null(inla.spec$lincomb.derived.only)) {
        stop("Option 'control.inla$lincomb.derived.only' is disabled. Please fix.")
    }

    if (!is.null(inla.spec$restart) && inla.spec$restart >= 0) {
        cat("restart = ", as.integer(inla.spec$restart), "\n", file = file, sep = " ", append = TRUE)
    }

    if (!is.null(inla.spec$optimiser)) {
        cat("optimiser = ", inla.spec$optimiser, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$verbose) && inla.spec$verbose) {
        cat("optpar.fp = stdout\n", sep = " ", file = file, append = TRUE)
    }

    if (!is.null(inla.spec$reordering)) {
        ## reordering could be a number -1, 0, ....  or a string,  or the output from inla.qreordering()
        r <- inla.spec$reordering
        if (is.list(r)) {
            ## output from inla.qreordering
            r <- r$name
        }
        if (is.character(r)) {
            r.code <- inla.reorderings.name2code(r)
        } else if (is.integer(r)) {
            ## this will fail is code is wrong
            dummy <- inla.reorderings.code2name(r)
            r.code <- r
        } else {
            stop("This should not happen.")
        }
        cat("reordering = ", r.code, "\n", sep = " ", file = file, append = TRUE)
    }

    if (!is.null(inla.spec$cpo.diff)) {
        cat("cpo.diff = ", inla.spec$cpo.diff, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$npoints)) {
        cat("n.points = ", inla.spec$npoints, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$cutoff)) {
        cat("cutoff = ", inla.spec$cutoff, "\n", sep = " ", file = file, append = TRUE)
    }
    inla.write.boolean.field("adapt.hessian.mode", inla.spec$adapt.hessian.mode, file)

    if (!is.null(inla.spec$adapt.hessian.max.trials) && inla.spec$adapt.hessian.max.trials >= 0) {
        cat("adapt.hessian.max.trials = ", inla.spec$adapt.hessian.max.trials, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$adapt.hessian.scale) && inla.spec$adapt.hessian.scale >= 1) {
        cat("adapt.hessian.scale = ", inla.spec$adapt.hessian.scale, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$step.len)) {
        cat("step.len = ", inla.spec$step.len, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$stencil)) {
        stopifnot(inla.spec$stencil %in% c(3, 5, 7, 9))
        cat("stencil = ", inla.spec$stencil, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$diagonal) && inla.spec$diagonal >= 0.0) {
        cat("diagonal = ", inla.spec$diagonal, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$numint.maxfeval)) {
        cat("numint.maxfeval = ", as.integer(inla.spec$numint.maxfeval), "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$numint.relerr)) {
        cat("numint.relerr = ", inla.spec$numint.relerr, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$numint.abserr)) {
        cat("numint.abserr = ", inla.spec$numint.abserr, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$cmin)) {
        cat("cmin = ", inla.spec$cmin, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$b.strategy)) {
        b <- inla.spec$b.strategy
        if (is.character(b)) {
            ## use the same enum mapping as in GMRFLibP.h for INLA_B_STRATEGY_SKIP and _KEEP
            if (inla.strcasecmp(b, "skip")) {
                b <- 0
            } else if (inla.strcasecmp(b, "keep")) {
                b <- 1
            } else {
                stop("control.inla$b.strategy must be 'keep' or 'skip'.")
            }
        } else {
            b <- as.integer(b)
            stopifnot(any(b == 0:1))
        }
        cat("b.strategy = ", b, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$step.factor)) {
        cat("nr.step.factor = ", inla.spec$step.factor, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$global.node.factor)) {
        cat("global.node.factor = ", inla.spec$global.node.factor, "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$global.node.degree)) {
        cat("global.node.degree = ", inla.spec$global.node.degree, "\n", file = file, append = TRUE)
    }

    ## options related to 'stupid search'.
    inla.write.boolean.field("stupid.search", inla.spec$stupid.search, file)
    if (!is.null(inla.spec$stupid.search.max.iter)) {
        cat("stupid.search.max.iter = ", as.integer(inla.spec$stupid.search.max.iter), "\n", file = file, append = TRUE)
    }
    if (!is.null(inla.spec$stupid.search.factor)) {
        fac <- as.numeric(inla.spec$stupid.search.factor)
        stopifnot(fac >= 1.0)
        cat("stupid.search.factor = ", fac, "\n", file = file, append = TRUE)
    }

    ## this covers ... == "auto"  (or whaterever is given)
    if (is.character(inla.spec$control.vb$enable)) {
        if (inla.mode == "experimental") {
            inla.spec$control.vb$enable <- TRUE
        } else {
            inla.spec$control.vb$enable <- FALSE
        }
    }
    inla.write.boolean.field("control.vb.enable", inla.spec$control.vb$enable, file)
    inla.write.boolean.field("control.vb.verbose", inla.spec$control.vb$verbose, file)
    inla.write.boolean.field("control.vb.hyperpar.correct", inla.spec$control.vb$hyperpar.correct, file)
    cat("control.vb.strategy = ", inla.spec$control.vb$strategy, "\n", file = file, append = TRUE)
    cat("control.vb.refinement = ", inla.spec$control.vb$refinement, "\n", file = file, append = TRUE)
    cat("control.vb.max.correct = ", inla.spec$control.vb$max.correct, "\n", file = file, append = TRUE)
    cat("control.vb.enable.limit = ", inla.spec$control.vb$enable.limit, "\n", file = file, append = TRUE)

    num.gradient <- match.arg(tolower(inla.spec$num.gradient), c("central", "forward"))
    num.hessian <- match.arg(tolower(inla.spec$num.hessian), c("central", "forward"))
    optimise.strategy <- match.arg(tolower(inla.spec$optimise.strategy), c("plain", "smart"))
    cat("num.gradient = ", num.gradient, "\n", sep = " ", file = file, append = TRUE)
    cat("num.hessian = ", num.hessian, "\n", sep = " ", file = file, append = TRUE)
    cat("optimise.strategy = ", optimise.strategy, "\n", sep = " ", file = file, append = TRUE)

    ud <- inla.spec$use.directions
    if (is.null(ud)) ud <- FALSE
    ud.val <- if (is.logical(ud) && (ud == FALSE)) FALSE else TRUE
    inla.write.boolean.field("use.directions", ud.val, file)
    if (ud.val && is.matrix(ud)) {
        file.directions <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(as.matrix(ud), filename = file.directions)
        fnm <- gsub(data.dir, "$inladatadir", file.directions, fixed = TRUE)
        cat("use.directions.matrix =", fnm, "\n", file = file, append = TRUE)
    }

    if (inla.spec$constr.marginal.diagonal > 0.0) {
        cat("constr.marginal.diagonal = ", inla.spec$constr.marginal.diagonal, "\n", sep = " ", file = file, append = TRUE)
    }

    inla.write.boolean.field("improved.simplified.laplace", inla.spec$improved.simplified.laplace, file)

    cat("\n", sep = " ", file = file, append = TRUE)
}

`inla.predictor.section` <- function(file, n, m, predictor.spec, file.offset, data.dir, file.link.fitted.values) {
    ## n = NPredictor
    ## m = MPredictor

    cat(inla.secsep("Predictor"), "\n", sep = " ", file = file, append = TRUE)
    cat("type = predictor\n", sep = " ", file = file, append = TRUE)
    cat("dir = predictor\n", sep = " ", file = file, append = TRUE)
    cat("n = ", n, "\n", sep = " ", file = file, append = TRUE)
    cat("m = ", m, "\n", sep = " ", file = file, append = TRUE)

    inla.write.boolean.field("fixed", predictor.spec$fixed, file)
    inla.write.boolean.field("compute", predictor.spec$compute, file)

    if (!is.null(predictor.spec$cdf)) {
        cat("cdf = ", predictor.spec$cdf, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(predictor.spec$quantiles)) {
        cat("quantiles = ", predictor.spec$quantiles, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(file.offset)) {
        cat("offset = ", file.offset, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(file.link.fitted.values)) {
        cat("link.fitted.values = ", file.link.fitted.values, "\n", sep = " ", file = file, append = TRUE)
    }

    inla.write.hyper(predictor.spec$hyper, file, data.dir = data.dir)

    if (!is.null(predictor.spec$cross) && length(predictor.spec$cross) > 0) {
        if (length(predictor.spec$cross) != n + m) {
            stop(paste("Length of cross does not match the total length of predictor", length(predictor.spec$cross), "!=", n + m))
        }
        file.cross <- inla.tempfile(tmpdir = data.dir)
        ## better to go through factor to get levels 1...ncross.
        cross <- as.factor(predictor.spec$cross)
        cross <- as.integer(cross)
        cross[is.na(cross)] <- 0L ## means not in use
        inla.write.fmesher.file(as.matrix(cross, ncol = 1L), filename = file.cross)
        fnm <- gsub(data.dir, "$inladatadir", file.cross, fixed = TRUE)
        cat("cross.constraint =", fnm, "\n", file = file, append = TRUE)
    }

    if (!is.null(predictor.spec$A)) {

        ## Now we will build the extended Matrix, which is
        ##
        ## Aextended = [ I, -A; -A^T, A^T A ] ((n+m) x (n+m))
        ##
        ## This matrix is the one that is needed for input to inla.
        if (is.character(predictor.spec$A)) {
            A <- read.table(predictor.spec$A, col.names = c("i", "j", "x"))
            A <- sparseMatrix(i = A$i, j = A$j, x = A$x, index1 = TRUE)
        } else {
            A <- predictor.spec$A
        }
        A <- inla.sparse.check(A, must.be.squared = FALSE)

        ## check dimensions
        stopifnot(dim(A)[1] == m)
        stopifnot(dim(A)[2] == n)

        file.A <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(A, filename = file.A)
        file.A <- gsub(data.dir, "$inladatadir", file.A, fixed = TRUE)
        cat("A = ", file.A, "\n", append = TRUE, sep = " ", file = file)
        Aij <- NULL

        ## Aext = [ I, -A; -A^T, A^T A ] ((n+m) x (n+m))
        Aext <- rbind(cbind(Diagonal(m), -A), cbind(-t(A), t(A) %*% A))
        stopifnot(dim(Aext)[1] == m + n)
        stopifnot(dim(Aext)[2] == m + n)

        file.A <- inla.tempfile(tmpdir = data.dir)
        inla.write.fmesher.file(Aext, filename = file.A)
        file.A <- gsub(data.dir, "$inladatadir", file.A, fixed = TRUE)
        cat("Aext = ", file.A, "\n", append = TRUE, sep = " ", file = file)
        cat("AextPrecision = ", predictor.spec$precision, "\n", append = TRUE, sep = " ", file = file)
    }

    inla.write.boolean.field("vb.correct", predictor.spec$vb.correct, file)

    cat("\n", sep = " ", file = file, append = TRUE)
}

`inla.problem.section` <- function(file, data.dir, result.dir, hyperpar, return.marginals, return.marginals.predictor, dic,
                                   cpo, po, mlik, quantiles, smtp, q, openmp.strategy, graph, config) {
    cat("", sep = "", file = file, append = FALSE)
    cat("###  ", inla.version("version"), "\n", sep = "", file = file, append = TRUE)
    cat("###  ", inla.paste(Sys.info()), "\n", sep = "", file = file, append = TRUE)
    cat("###  ", inla.os.type(), "-", inla.os.32or64bit(), "bit", " ", date(), "\n", sep = "", file = file, append = TRUE)

    cat("\n### [[[start of output from sessionInfo()]]]", "\n", file = file, append = TRUE)
    s <- paste("###   ", capture.output(sessionInfo()))
    for (i in seq_along(s)) {
        cat(s[i], "\n", file = file, append = TRUE)
    }
    cat("### [[[end of output from sessionInfo()]]]\n", "\n", file = file, append = TRUE)

    cat("inladatadir = ", data.dir, "\n", sep = "", file = file, append = TRUE)
    cat("inlaresdir = ", result.dir, "\n", sep = "", file = file, append = TRUE)
    cat("##inladatadir = ", gsub("^.*/", "", data.dir), "\n", sep = "", file = file, append = TRUE) #
    cat("##inlaresdir = ", gsub("^.*/", "", result.dir), "-%d\n", sep = "", file = file, append = TRUE) #

    ## libR-stuff
    cat("\n", sep = " ", file = file, append = TRUE)
    cat(inla.secsep("INLA.libR"), "\n", sep = " ", file = file, append = TRUE)
    cat("type = libR\n", sep = " ", file = file, append = TRUE)
    cat("R_HOME = ", Sys.getenv("R_HOME"), "\n", sep = "", file = file, append = TRUE)

    cat("\n", sep = " ", file = file, append = TRUE)
    cat(inla.secsep("INLA.Model"), "\n", sep = " ", file = file, append = TRUE)
    cat("type = problem\n", sep = " ", file = file, append = TRUE)
    cat("dir = $inlaresdir\n", sep = " ", file = file, append = TRUE)
    cat("rinla.version = ", inla.version("version"), "\n", file = file, append = TRUE)
    cat("rinla.bdate = ", inla.version("date"), "\n", file = file, append = TRUE)
    inla.write.boolean.field("return.marginals", return.marginals, file)
    inla.write.boolean.field("return.marginals.predictor", return.marginals.predictor, file)
    inla.write.boolean.field("hyperparameters", hyperpar, file)
    inla.write.boolean.field("cpo", cpo, file)
    inla.write.boolean.field("po", po, file)
    inla.write.boolean.field("dic", dic, file)
    inla.write.boolean.field("mlik", mlik, file)
    inla.write.boolean.field("q", q, file)
    inla.write.boolean.field("graph", graph, file)
    inla.write.boolean.field("config", config, file)

    if (is.null(smtp) || !(is.character(smtp) && (nchar(smtp) > 0))) {
        smtp <- inla.getOption("smtp")
    }
    smtp <- match.arg(tolower(smtp), c("band", "taucs", "pardiso", "default"))
    cat("smtp = ", smtp, "\n", sep = " ", file = file, append = TRUE)

    if (is.null(openmp.strategy) || !(is.character(openmp.strategy) && (nchar(openmp.strategy) > 0))) {
        openmp.strategy <- "default"
    }
    openmp.strategy <- match.arg(
        tolower(openmp.strategy),
        c(
            "default", "small", "medium", "large", "huge",
            "pardiso.serial", "pardiso.parallel", "pardiso.nested", "pardiso"
        )
    )
    if (inla.one.of(openmp.strategy, c("pardiso.serial", "pardiso.parallel", "pardiso.nested"))) {
        ## they are all the same now. this is for backward compatibility
        warning(paste0(
            "openmp.strategy='pardiso.serial', 'pardiso.parallel', or 'pardiso.nested', is the same as openmp.strategy='pardiso', ",
            "\nplease update your code. You define how the parallelisation is done using argument 'inla(...,  num.threads='A:B')'",
            "\nwhere A are the number of threads in the outer layer, and B in the inner layer."
        ))
        openmp.strategy <- "pardiso"
    }
    cat("openmp.strategy = ", openmp.strategy, "\n", sep = " ", file = file, append = TRUE)

    if (!is.null(quantiles)) {
        cat("quantiles = ", quantiles, "\n", sep = " ", file = file, append = TRUE)
    }

    cat("\n", sep = " ", file = file, append = TRUE)
}

`inla.parse.fixed.prior` <- function(name, prior) {
    if (is.null(prior)) {
        return(NULL)
    } else if (is.numeric(prior)) {
        return(prior)
    } else if (any(name == names(prior))) {
        return(prior[[which(name == names(prior))]])
    } else if (any("default" == names(prior))) {
        return(prior[[which("default" == names(prior))]])
    }
    return(NULL)
}

`inla.linear.section` <- function(file, file.fixed, label, results.dir, control.fixed, only.hyperparam) {
    cat(inla.secsep(label), "\n", sep = "", file = file, append = TRUE)
    cat("dir = ", results.dir, "\n", sep = " ", file = file, append = TRUE)
    cat("type = linear\n", sep = " ", file = file, append = TRUE)
    cat("covariates = ", file.fixed, "\n", sep = " ", file = file, append = TRUE)
    if (only.hyperparam || !control.fixed$compute) {
        cat("compute = 0\n", sep = " ", file = file, append = TRUE)
    } else {
        cat("compute = 1\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(control.fixed$cdf)) {
        cat("cdf = ", control.fixed$cdf, "\n", sep = " ", file = file, append = TRUE)
    }
    if (!is.null(control.fixed$quantiles)) {
        cat("quantiles = ", control.fixed$quantiles, "\n", sep = " ", file = file, append = TRUE)
    }
    if (length(grep("^[(]Intercept[)]$", inla.trim(label))) == 1) {
        prec <- control.fixed$prec.intercept
        mean <- control.fixed$mean.intercept
        if (is.null(mean)) {
            mean <- inla.set.control.fixed.default()$mean.intercept
        }
        if (is.null(prec)) {
            prec <- inla.set.control.fixed.default()$prec.intercept
        }
        stopifnot(!is.null(mean) || !is.null(prec))
    } else {
        mean <- inla.parse.fixed.prior(label, control.fixed$mean)
        prec <- inla.parse.fixed.prior(label, control.fixed$prec)
        if (is.null(mean)) {
            mean <- inla.set.control.fixed.default()$mean
        }
        if (is.null(prec)) {
            prec <- inla.set.control.fixed.default()$prec
        }
        stopifnot(!is.null(mean) || !is.null(prec))
    }
    cat("mean = ", mean, "\n", sep = " ", file = file, append = TRUE)
    cat("precision = ", prec, "\n", sep = " ", file = file, append = TRUE)
    cat("\n", sep = " ", file = file, append = TRUE)

    return(list(label = label, prior.mean = mean, prior.prec = prec))
}

`inla.mode.section` <- function(file, args, data.dir) {
    if (!is.null(args$result) || !is.null(args$theta) || !is.null(args$x)) {
        cat(inla.secsep("INLA.Control.Mode"), "\n", sep = " ", file = file, append = TRUE)
        cat("type = mode\n", sep = " ", file = file, append = TRUE)

        ## use default the mode in result if given
        if (is.null(args$theta) && !is.null(args$result)) {
            args$theta <- args$result$mode$theta
        }

        if (!is.null(args$theta)) {
            args$theta <- inla.text2vector(args$theta)
            ## set non-finite's to 0
            args$theta[!is.finite(args$theta)] <- 0
            file.theta <- inla.tempfile(tmpdir = data.dir)
            ## cat("theta = ", inla.paste(as.character(args$theta)), "\n", sep = " ", file = file,  append = TRUE)
            fp.binary <- file(file.theta, "wb")
            ## convert from character to a vector of doubles
            if (is.character(args$theta)) {
                args$theta <- as.numeric(strsplit(args$theta, "[ \t]+")[[1]])
            }
            args$theta <- args$theta[!is.na(args$theta)]
            writeBin(as.integer(length(args$theta)), fp.binary)
            writeBin(args$theta, fp.binary)
            close(fp.binary)
            fnm <- gsub(data.dir, "$inladatadir", file.theta, fixed = TRUE)
            cat("theta =", fnm, "\n", file = file, append = TRUE)
        }
        ## use default the mode in result if given
        if (is.null(args$x) && !is.null(args$result)) {
            args$x <- args$result$mode$x
        }
        if (!is.null(args$x)) {
            args$x <- inla.text2vector(args$x)
            ## set non-finite's to 0
            args$x[!is.finite(args$x)] <- 0
            file.x <- inla.tempfile(tmpdir = data.dir)
            ## write(args$x, ncol=1, file=file.x)
            fp.binary <- file(file.x, "wb")
            writeBin(as.integer(length(args$x)), fp.binary)
            writeBin(args$x, fp.binary)
            close(fp.binary)
            fnm <- gsub(data.dir, "$inladatadir", file.x, fixed = TRUE)
            cat("x =", fnm, "\n", file = file, append = TRUE)
        }
        inla.write.boolean.field("restart", args$restart, file)
        inla.write.boolean.field("fixed", args$fixed, file)
    }
}

`inla.expert.section` <- function(file, args, data.dir) {
    cat(inla.secsep("INLA.Expert"), "\n", sep = " ", file = file, append = TRUE)
    cat("type = expert\n", sep = " ", file = file, append = TRUE)

    if (!is.null(args$cpo.manual) && args$cpo.manual) {
        inla.write.boolean.field("cpo.manual", args$cpo.manual, file)
        ## recall to convert to 0-based index'ing
        cat("cpo.idx = ", args$cpo.idx - 1, "\n", sep = " ", file = file, append = TRUE)
    }

    if (!is.null(args$jp)) {
        stopifnot(inherits(args$jp, "inla.jp"))
        file.jp <- inla.tempfile(tmpdir = data.dir)
        ## this must be the same name as R_JP_MODEL in inla.h
        model <- ".inla.jp.model"
        assign(model, args$jp$model)
        ## save model, or the object that 'model' expands to
        inla.eval(paste("save(", model,
            ", file = ", "\"", file.jp, "\"",
            ", ascii = FALSE, compress = TRUE)",
            sep = ""
        ))
        fnm <- gsub(data.dir, "$inladatadir", file.jp, fixed = TRUE)
        cat("jp.file =", fnm, "\n", file = file, append = TRUE)
        cat("jp.model =", model, "\n", file = file, append = TRUE)
        rm(model) ## do not need it anymore
    }

    if (is.null(args$disable.gaussian.check)) {
        args$disable.gaussian.check <- FALSE
    }
    inla.write.boolean.field("DISABLE.GAUSSIAN.CHECK", args$disable.gaussian.check, file)
}

`inla.update.section` <- function(file, data.dir, contr) {
    if (!is.null(contr$result) && length(contr$result$mode$theta) > 0L) {
        file.update <- inla.tempfile(tmpdir = data.dir)
        cat(inla.secsep("INLA.update"), "\n", sep = " ", file = file, append = TRUE)
        cat("type = update\n", sep = " ", file = file, append = TRUE)
        x <- c(
            length(contr$result$mode$theta),
            contr$result$mode$theta,
            contr$result$misc$stdev.corr.positive,
            contr$result$misc$stdev.corr.negative,
            1 / sqrt(contr$result$misc$cov.intern.eigenvalues), ## this is what is required
            c(contr$result$misc$cov.intern.eigenvectors) ## column-wise storage
        )
        x <- as.matrix(x, ncol = 1L)
        inla.write.fmesher.file(x, filename = file.update)
        file.update <- gsub(data.dir, "$inladatadir", file.update, fixed = TRUE)
        cat("filename = ", file.update, "\n", sep = " ", file = file, append = TRUE)
    }
}

`inla.pardiso.section` <- function(file, data.dir, contr) {
    cat("\n", inla.secsep("INLA.pardiso"), "\n", sep = "", file = file, append = TRUE)
    cat("type = pardiso\n", sep = " ", file = file, append = TRUE)

    cat("verbose = ", if (contr$verbose) 1 else 0, "\n", sep = " ", file = file, append = TRUE)
    cat("debug = ", if (contr$debug) 1 else 0, "\n", sep = " ", file = file, append = TRUE)
    cat("parallel.reordering = ", if (contr$parallel.reordering) 1 else 0, "\n", sep = " ", file = file, append = TRUE)
    cat("nrhs = ", contr$nrhs, "\n", sep = " ", file = file, append = TRUE)
}

`inla.lincomb.section` <- function(file, data.dir, contr, lincomb) {
    ## this one write binary format files...

    ## format is either
    ##
    ## list("lc1" = "a 1 1 b 2 1 3 2...", ...)
    ##
    ## or
    ##
    ## list("lc1" = list( "a" = list(idx=1, weight=1), "b" = list(idx=c(2, 3), weight = c(1, 2)), ...), ...)
    ##
    ## use the functions 'inla.make.lincomb()' and 'inla.make.lincombs()'

    ## if use.one.file = TRUE, then use one file for all lincombs and
    ## the 'ENTRY keyword', otherwise, use one file for each lincomb.

    if (!is.null(lincomb)) {
        fnm <- inla.tempfile(tmpdir = data.dir)
        file.create(fnm)
        fp.binary <- file(fnm, "wb")
        stopifnot(!is.null(fnm))
        stopifnot(!is.null(fp.binary))

        numlen <- inla.numlen(length(lincomb))
        prev.secnames <- c()

        for (i in 1:length(lincomb)) {
            nam <- names(lincomb[i])
            if (is.null(nam) || is.na(nam)) {
                secname <- paste("lincomb.", inla.num(i, width = numlen), sep = "")
                lc <- lincomb[[i]]
            } else if (nam == "") {
                secname <- paste("lincomb.", inla.num(i, width = numlen), sep = "")
                lc <- lincomb[[i]]
            } else {
                secname <- paste("lincomb.", nam[1], sep = "")
                lc <- lincomb[[i]]
            }

            ## check if the name is used previously, if so, stop.
            if (secname %in% prev.secnames) {
                stop(paste("Duplicated name [", secname, "] in 'lincomb'; need unique names or NA or ''.",
                    sep = ""
                ))
            } else {
                prev.secnames <- c(secname, prev.secnames)
            }

            cat("\n", inla.secsep(secname), "\n", sep = "", file = file, append = TRUE)
            cat("type = lincomb\n", sep = " ", file = file, append = TRUE)
            cat("lincomb.order = ", i, "\n", sep = " ", file = file, append = TRUE)
            inla.write.boolean.field("verbose", contr$verbose, file)
            cat("file.offset = ", as.integer(seek(fp.binary, where = NA)), "\n", sep = "", file = file, append = TRUE)

            ## number of entries
            writeBin(as.integer(length(lc)), fp.binary)

            for (j in 1:length(lc)) {
                lc.j.name <- as.character(names(lc[[j]]))
                lc.j <- lc[[j]][[1]]

                ## if $idx is not there, its default 1.
                if (is.null(lc.j$idx) && length(lc.j$weight) == 1) {
                    lc.j$idx <- 1
                }

                ## NA's are allowed; but we just remove them.
                idx <- lc.j$idx[!is.na(lc.j$idx)]
                weight <- lc.j$weight[!is.na(lc.j$weight)]
                if (length(idx) == 0 || length(weight) == 0) {
                      stop(paste("lincomb", secname, "has only zero entries. This is not allowed"))
                  }
                stopifnot(length(idx) == length(weight))

                ## this the old code:
                ## cat(c( lc.j.name, c(rbind( idx, weight))), "\n", file=fnm, append=TRUE)
                writeBin(as.integer(nchar(lc.j.name)), fp.binary)
                writeBin(as.character(lc.j.name), fp.binary)
                writeBin(as.integer(length(idx)), fp.binary) ## number of pairs
                writeBin(as.integer(idx), fp.binary)
                writeBin(as.double(weight), fp.binary)

                ## print(paste(" stop writing at position ", as.integer(seek(fp.binary, where=NA))))
            }

            fnm.new <- gsub(data.dir, "$inladatadir", fnm, fixed = TRUE)
            cat("filename = ", fnm.new, "\n", sep = " ", file = file, append = TRUE)
        }

        close(fp.binary)
    }
}

`inla.copy.file.for.section` <- function(filename, data.dir) {
    if (missing(filename)) {
        return(NULL)
    }
    if (missing(data.dir)) {
        stop("data.dir required")
    }

    fnm <- inla.tempfile(tmpdir = data.dir)
    file.copy(filename, fnm, overwrite = TRUE)
    return(gsub(data.dir, "$inladatadir", fnm, fixed = TRUE))
}

`inla.copy.dir.for.section` <- function(dir.name, data.dir) {
    d.fnm <- inla.tempfile(tmpdir = data.dir)
    inla.dir.create(d.fnm)
    files.to.copy <- paste(dir.name, "/", dir(dir.name, recursive = TRUE), sep = "")
    file.copy(files.to.copy, d.fnm, recursive = TRUE)
    return(gsub(data.dir, "$inladatadir", d.fnm, fixed = TRUE))
}

`inla.copy.dir.for.section.spde` <- function(prefix, data.dir) {
    dir.name <- dirname(prefix)
    file.prefix <- basename(prefix)

    d.fnm <- inla.tempfile(tmpdir = data.dir)
    inla.dir.create(d.fnm)
    files.to.copy <- paste(dir.name, "/",
        dir(dir.name, pattern = paste("^", file.prefix, sep = ""), recursive = TRUE),
        sep = ""
    )
    file.copy(files.to.copy, d.fnm, recursive = TRUE)
    rdir <- gsub(data.dir, "$inladatadir", d.fnm, fixed = TRUE)
    rprefix <- paste(rdir, "/", file.prefix, sep = "")
    ## They original files were created by f(), so remove them:
    unlink(files.to.copy)
    return(rprefix)
}

## Nothing to export

## obsolete functions goes here.

`inla.obsolete` <- function(old, new) {
    stop(inla.paste(c(
        "INLA-function `", as.character(old)[1], "' is obsolete. Use function `",
        as.character(new), "' instead."
    ), sep = ""))
}

## Export: inla.cpo

## !\name{inla.cpo}
## !\alias{inla.cpo}
## !\alias{cpo.inla}
## !
## !\title{Improved estimates for the CPO/PIT-values}
## !\description{
## ! Improve the estimates of the  CPO/PIT-values
## ! be recomputing the model-fit by removing data-points.
## !}
## !\usage{
## !inla.cpo(result,
## !         force = FALSE,
## !         mc.cores = NULL,
## !         verbose = TRUE,
## !         recompute.mode = TRUE)
## !}
## !
## !\arguments{
`inla.cpo` <- function(
                       ## !\item{result}{An object of class \code{inla}, ie a result
                       ## !of a call to \code{inla()}}
                       result,

                       ## !\item{force}{If TRUE,  then recompute all CPO/PIT values and not just
                       ## !those with \code{result$cpo$failure > 0}.}
                       force = FALSE,

                       ## !\item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
                       ## !                \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
                       ## !                and \code{inla.getOption("num.threads")} in that order.}
                       mc.cores = NULL,

                       ## !\item{verbose}{Run in verbose mode?}
                       verbose = TRUE,

                       ## !\item{recompute.mode}{Should be mode (and the integration points) be recomputed when a data-point is removed or not?}
                       recompute.mode = TRUE) {
    ## !}
    ## !\value{
    ## !The object returned is the same as  \code{result} but the
    ## !new improved estimates of the CPO/PIT values replaced.}
    ## !\author{Havard Rue \email{hrue@r-inla.org}}
    ## !\examples{
    ## !n = 10
    ## !y = rnorm(n)
    ## !r = inla(y ~ 1, data = data.frame(y), control.compute = list(cpo=TRUE))
    ## !\donttest{
    ## !rr = inla.cpo(r, force=TRUE)
    ## !}
    ## !}
    ## !\seealso{\code{\link{inla}}}

    stopifnot(!missing(result))
    if (class(result) != "inla") {
        warning("The function 'inla.cpo()' has changed; see ?inla.cpo for futher details.\n  The old version is available as 'inla.cpo.OLD()' for a while.")
    }
    stopifnot(class(result) == "inla")

    ## if there is no cpo, then done
    if (is.null(result$cpo)) {
          return(result)
      }

    ## loop over those with failure > 0
    if (!force) {
        idx.fail <- which(result$cpo$failure > 0)
    } else {
        idx.fail <- 1:length(result$cpo$failure)
    }

    result.new <- result
    if (length(idx.fail) > 0L) {
        k <- 1L
        cpo.old <- result$cpo$cpo[idx.fail]
        pit.old <- result$cpo$pit[idx.fail]

        if (verbose) {
            cat("Compute new CPO/PIT values manually, for", length(idx.fail), "cases...\n")
        }

        nt <- as.numeric(strsplit(result$.args$num.threads, ":")[[1]])
        res <- inla.mclapply(
            idx.fail,
            function(idx, result) {
                result$.args$control.expert <- list(cpo.manual = TRUE, cpo.idx = idx)
                result$.args$control.mode <- list(
                    result = NULL,
                    fixed = result$.args$control.mode$fixed,
                    theta = result$mode$theta,
                    x = result$mode$x,
                    restart = recompute.mode
                )
                result$.args$control.compute$dic <- result$.args$control.compute$waic <- FALSE
                result$.args$num.threads <- paste0(nt[2], ":1")
                rr <- inla.rerun(result, plain = TRUE)$cpo
                return(list(cpo = rr$cpo[idx], pit = rr$pit[idx], failure = rr$failure[idx]))
            },
            result = result, mc.cores = mc.cores
        )

        result.new$cpo$cpo[idx.fail] <- unlist(lapply(res, function(xx) {
            return(xx$cpo)
        }))
        result.new$cpo$pit[idx.fail] <- unlist(lapply(res, function(xx) {
            return(xx$pit)
        }))
        result.new$cpo$failure[idx.fail] <- unlist(lapply(res, function(xx) {
            return(xx$failure)
        }))

        if (verbose) {
            dig <- getOption("digits")
            options(digits = 6)
            print(cbind(
                index = idx.fail,
                cpo.old = cpo.old, cpo.new = result.new$cpo$cpo[idx.fail],
                pit.old = pit.old, pit.new = result.new$cpo$pit[idx.fail]
            ))
            cat("\nThe retured result contain the new values.\n")
            options(digits = dig)
        }
    }

    return(result.new)
}

## copied from the mgcv-code...

## this does not work...
## invisible(utils::globalVariables(c("low", "high", "spde", "internal")))
## invisible(utils::suppressForeignCheck(c("low", "high", "spde", "internal")))


inla.print.version <- function() {
    info <- library(help = INLA)$info[[1]]
    if (!is.null(info)) {
        version <- info[pmatch("Version", info)]
        built <- strsplit(info[pmatch("Built", info)], "; ")[[1]][3]
        date <- info[pmatch("Date", info)]
        um <- strsplit(version, " ")[[1]]
        version <- um[nchar(um) > 0][2]
        um <- strsplit(date, " ")[[1]]
        date <- um[nchar(um) > 0]

        hello <- paste0(
            "This is INLA_", version,
            " built ", built, ".", "\n",
            " - See www.r-inla.org/contact-us for how to get help."
        )
        if (!inla.os("windows")) {
            hello <- paste0(
                hello,
                "\n",
                " - To enable PARDISO sparse library; see inla.pardiso()"
            )
        }
        s <- inla.prune.check()
        if (s > 0) {
            hello <- paste0(
                hello,
                "\n",
                paste0(
                    " - Save ", s,
                    "Mb of storage running 'inla.prune()'"
                )
            )
        }
        packageStartupMessage(hello)
        if (!inla.os("windows") && inla.os.is.32bit()) {
            warning(paste0(
                "INLA_", version,
                ": 32bit binaries are no longer supported!"
            ))
        }
    }
}

.onLoad <- function(...) {
    ## nothing for the moment
}

.onAttach <- function(...) {
    inla.print.version()
}

.onUnload <- function(libpath) {
    ## library.dynam.unload("INLA", libpath)
}

## Export: inla.load

## ! \name{inla.load}
## ! \alias{inla.load}
## ! \title{Load or source a file}
## ! \description{Load or source a file: (internal use)}
## ! \usage{
## !     inla.load(filename, debug = TRUE)
## ! }
## ! \arguments{
## !   \item{filename}{The name of the file to be loaded, alternatively, sourced. }
## !   \item{debug}{Logical. Turn on/off debug information.}
## !  }
## ! \value{
## !   None
## ! }
## ! \details{
## !   Try to \code{load} the file into the global environment,
## !   if that fail,  try to \code{source} the file into the
## !   global environment.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}

`inla.load` <- function(filename, debug = TRUE) {
    msg <- function(...) {
        if (debug) {
            cat("inla.load: ", ..., "\n", sep = "")
        }
    }

    w <- getOption("warn")
    options(warn = -1L)
    val <- try(load(filename, envir = globalenv()), silent = TRUE)
    options(warn = w)

    if (inherits(val, "try-error")) {
        msg("source file [", filename, "] in the global environment")
        source(filename, echo = TRUE, local = FALSE)
    } else {
        msg("load file [", filename, "] in the global environment")
    }

    return(invisible())
}

## Nothing to export

`inla.cygwin.check.path` <- function(path = inla.getOption("cygwin")) {
    return(file.exists(path) && file.info(path)$isdir)
}

`inla.cygwin.protect.special.chars` <- function(cmd) {
    ## cmd = gsub(" ", "\\\\ ", cmd)
    return(cmd)
}

`inla.cygwin.run.command` <- function(command, file.log = NULL, ...) {
    if (inla.cygwin.check.path()) {
        cmd <- paste(inla.getOption("cygwin"),
            "/bin/bash.exe -c ",
            shQuote(paste("export PATH=/bin:/usr/bin:$PATH;",
                inla.cygwin.protect.special.chars(command),
                inla.ifelse(is.null(file.log), "", paste(" > ", file.log)),
                sep = ""
            ), type = "cmd"),
            sep = ""
        )
        system(cmd, ...)
    } else {
        stop(paste("Cannot find the CYGWIN installation:", inla.getOption("cygwin")))
    }
}

`inla.cygwin.map.filename` <- function(filename, windows2cygwin = TRUE) {
    if (windows2cygwin) {
        fnm <- inla.paste(inla.cygwin.run.command(paste(
            "cygpath -u ",
            inla.cygwin.protect.special.chars(filename)
        ),
        intern = TRUE, ignore.stderr = TRUE
        ))
    } else {
        fnm <- inla.paste(inla.cygwin.run.command(paste(
            "cygpath -w -m -s ",
            inla.cygwin.protect.special.chars(filename)
        ),
        intern = TRUE, ignore.stderr = TRUE
        ))
    }
    return(fnm)
}

## NoExport: inla.sens
## NoExport: inla.sens.distance
## NoExport: inla.sens.distance.skew
## NoExport: inla.sens.skewMap
## NoExport: inla.sens.post

## !\name{inla.sens}
## !
## !\title{Calculate sensitivity measurements}
## !
## !\description{TODO}
## !\usage{
## !inla.sens(inlaObj, lambda = 0.3, nThreads = NULL, seed = NULL,
## !          nGrid = 1e4, nSamples = 2e4, nIntGrid = 1e4, useSkew = FALSE,
## !          calcPriorSens = FALSE, makePlots = TRUE)
## !}
## !
## !\arguments{
## !  \item{inlaRes}{Object returned by \code{inla} function.}
## !  \item{lambda}{TODO}
## !  \item{nThreads}{TODO}
## !  \item{seed}{TODO}
## !  \item{nGrid}{TODO}
## !  \item{nSamples}{TODO}
## !  \item{nIntGrid}{TODO}
## !  \item{useSkew}{TODO}
## !  \item{calcPriorSens}{TODO}
## !  \item{makePlots}{TODO}
## !}
## !\value{
## !  \code{inla.sens}  plots robustness and returns object with different robustnesses
## !}
## !\author{Geir-Arne Fuglstad \email{geirarne.fuglstad@gmail.com}}
## !\examples{
## ! TODO
## !}

inla.sens <- function(inlaObj, lambda = 0.3, nThreads = NULL, seed = NULL,
                      nGrid = 1e4, nSamples = 2e4, nIntGrid = 1e4, useSkew = FALSE,
                      calcPriorSens = FALSE, makePlots = TRUE) {
    ## Ensure reproducability
    if (!is.null(seed)) {
          set.seed(seed)
      }

    ## Ensure that $misc$configs information is available
    if (!inlaObj$.args$control.compute$config) {
        ## Turn on storage of x|theta distributions
        inlaObj$.args$control.compute$config <- TRUE

        ## Use the previously calculated mode
        inlaObj$.args$control.mode$result <- NULL
        inlaObj$.args$control.mode$restart <- FALSE
        inlaObj$.args$control.mode$theta <- inlaObj$mode$theta
        inlaObj$.args$control.mode$x <- inlaObj$mode$x

        ## Run INLA with all the settings
        inlaObj <- do.call("inla", args = inlaObj$.args)
    }

    ## Re-run with all data set to NA to get prior-level
    if (calcPriorSens) {
        inlaObjRef <- inlaObj
        inlaObjRef$.args$data$y <- NA * inlaObjRef$.args$data$y
        inlaObjRef <- do.call("inla", args = inlaObjRef$.args)
        priorSens <- inla.sens(inlaObjRef,
            lambda = lambda, nThreads = nThreads,
            seed = seed, nGrid = nGrid, nSamples = nSamples,
            nIntGrid = nIntGrid, useSkew = useSkew, calcPriorSens = FALSE
        )
    }

    ## Number of different theta values
    nTheta <- inlaObj$misc$configs$nconfig

    ## Number of latent components
    nLatent <- length(inlaObj$misc$configs$config[[1]]$improved.mean)

    ## Store parameters of distributions in matrices
    muMarg <- matrix(0, nrow = nTheta, ncol = nLatent)
    sdMarg <- muMarg
    skMarg <- muMarg
    prob <- vector(mode = "numeric", length = nTheta)
    for (idxC in 1:nTheta) {
        muMarg[idxC, ] <- inlaObj$misc$configs$config[[idxC]]$improved.mean
        sdMarg[idxC, ] <- sqrt(diag(inlaObj$misc$configs$config[[idxC]]$Qinv))
        skMarg[idxC, ] <- inlaObj$misc$configs$config[[idxC]]$skewness
        prob[idxC] <- exp(inlaObj$misc$configs$config[[idxC]]$log.posterior)
    }
    prob <- prob / sum(prob)

    ## Pre-compute robustification (Could be tabulated)
    ## Using the exponential distribution too easily leads to a distribution for the standard deviations
    ## that has no mean

    ## Correct lambda for number of parameters
    ## nPar = nLatent + nLatent*(nLatent+1)/2
    ## lambda = lambda/sqrt(nPar)

    ## ## Make sure mean and variance of standard deviations will exist


    ## Sample distances
    ## R = rexp(n = nSamples, rate = lambda)

    ## With the Gaussian distribution we can ensure the correct behaviour
    ## Scale variance according to number of samples
    nPar <- nLatent + nLatent * (nLatent + 1) / 2
    sigDist <- lambda * sqrt(nPar - 3)

    ## Simulate distances
    R <- abs(rnorm(nSamples, sd = sigDist))

    ## Approximate the draws from the ellipsiod with draws from n-sphere
    z12 <- matrix(rnorm(n = 2 * nSamples), ncol = 2)
    zRest2 <- rchisq(n = nSamples, df = nPar - 2)
    sphereRadius <- sqrt(rowSums(z12^2) + zRest2)
    sphereRadius <- cbind(sphereRadius, sphereRadius)
    z12 <- z12 / sphereRadius

    ## Scale with distances
    z12 <- z12 * R

    ## Correct for different semi-axis
    z12[, 1] <- z12[, 1] / 1
    z12[, 2] <- z12[, 2] / 2

    ## Extract mean and standard deviations
    muRobust <- z12[, 1]
    sdRobust <- exp(z12[, 2])

    ## Choose interval to compute robustification of Gaussian on
    len <- sqrt(var(muRobust) + mean(sdRobust^2))

    ## Calculate distribution
    xR <- seq(-20 * len, 20 * len, length.out = nGrid)
    yR <- vector(mode = "numeric", length = nGrid)
    for (idxS in 1:nSamples) {
        yR <- yR + dnorm(xR, mean = muRobust[idxS], sd = sdRobust[idxS])
    }
    yR <- log(yR) - log(nSamples)
    robMarg <- list(x = xR, y = yR)

    ## Calculate distance between original marginal posterior and
    ## posterior with uncertainty added in each conditional density
    ## x_i | \theta, y
    inla.require("doParallel", stop.on.error = TRUE)
    if (is.null(nThreads)) {
        doParallel::registerDoParallel()
    } else {
        doParallel::registerDoParallel(nThreads)
    }
    inla.require("foreach", stop.on.error = TRUE)
    nWorkers <- foreach::getDoParWorkers()
    breaks <- floor(seq(1, nLatent + 1, length.out = nWorkers + 1))
    ds <- foreach::foreach(idxW = 1:nWorkers, .combine = "c") %dopar% {
        tmpRes <- vector(mode = "numeric", length = breaks[idxW + 1] - breaks[idxW])
        for (idx in breaks[idxW]:(breaks[idxW + 1] - 1)) {
            if (useSkew) {
                tmpRes[idx - breaks[idxW] + 1] <- inla.sens.distance.skew(
                    muMarg[, idx], sdMarg[, idx], skMarg[, idx], prob, robMarg, nIntGrid
                )
            } else {
                tmpRes[idx - breaks[idxW] + 1] <- inla.sens.distance(
                    muMarg[, idx], sdMarg[, idx], skMarg[, idx], prob, robMarg, nIntGrid
                )
            }
        }

        tmpRes
    }

    ## Calculate max distance
    dMax <- inla.sens.distance(0, 1, 0, 1, robMarg, nIntGrid)

    ## Standardize against max distance
    stdDist <- (dMax - ds) / dMax

    ## Store results in groups
    res <- list()

    ## Make one plot for each group of variables
    groups <- inlaObj$misc$configs$contents
    nGroups <- length(groups$tag)
    xLab <- c()
    val <- c()
    cex.names <- 1.2
    cex.axis <- 1.2
    lwd <- 1.2
    for (idxP in 1:nGroups) {
        sIdx <- groups$start[idxP]
        eIdx <- sIdx + groups$length[idxP] - 1
        if (groups$length[idxP] == 1) {
            xLab <- c(xLab, groups$tag[idxP])
            val <- c(val, stdDist[sIdx])
        } else {
            if (makePlots) {
                inla.dev.new()
                barplot(stdDist[sIdx:eIdx],
                    space = 2,
                    ylim = c(0, 1),
                    main = groups$tag[idxP],
                    names.arg = 1:(eIdx - sIdx + 1),
                    xlab = "Index",
                    ylab = "Uncertainty",
                    cex.names = cex.names,
                    cex.axis = cex.axis,
                    lwd = lwd
                )

                ## Add prior level to plot if calculated
                if (calcPriorSens) {
                    barplot(priorSens[[length(res) + 1]]$val,
                        add = TRUE,
                        col = rgb(1, 0, 0, alpha = .20),
                        space = 2,
                        cex.axis = cex.axis
                    )
                }
            }

            ## Add groups into result object
            res <- c(res, list(list(
                tag = groups$tag[idxP],
                val = stdDist[sIdx:eIdx]
            )))
        }
    }
    if (length(val) >= 1) {
        if (makePlots) {
            inla.dev.new()
            barplot(val,
                space = 2,
                ylim = c(0, 1),
                main = "Fixed effects",
                names.arg = xLab,
                ylab = "Uncertainty",
                cex.names = cex.names,
                cex.axis = cex.axis,
                lwd = lwd
            )

            ## Add prior level to plot if calculated
            if (calcPriorSens) {
                barplot(priorSens[[length(res) + 1]]$val,
                    add = TRUE,
                    col = rgb(1, 0, 0, alpha = .20),
                    space = 2,
                    cex.axis = cex.axis
                )
            }
        }

        ## Add fixed effects to result object
        res <- c(res, list(list(
            tag = "Fixed effects",
            val = val,
            names = xLab
        )))
    }

    return(res)
}

inla.sens.distance <- function(muMarg, sdMarg, skMarg, prob, robMarg, nGrid, extraLen = 20) {
    ## Estimate required integration grid
    sdMax <- max(sdMarg)
    xs <- seq(-1, 1, length.out = nGrid) * sdMax * extraLen + mean(muMarg)

    ## Iterate through \theta values
    yO <- vector(mode = "numeric", length = nGrid)
    yR <- yO
    for (idxT in 1:length(muMarg)) {
        ## Use precomputed table of standard robust distribution
        xx <- (xs - muMarg[idxT]) / sdMarg[idxT]
        yy <- exp(spline(x = robMarg$x, y = robMarg$y, xout = xx)$y) / sdMarg[idxT]

        ## Remove the extrapolated values
        yy[(xx < min(robMarg$x)) | (xx > max(robMarg$x))] <- 0

        ## Add original and robust to their respective mixtures
        yO <- yO + prob[idxT] * dnorm(xs, mean = muMarg[idxT], sd = sdMarg[idxT])
        yR <- yR + prob[idxT] * yy
    }

    ## Calculate KLD between original and added uncertainty
    intG <- yR * log(yR / yO)
    intG[yR == 0] <- 0
    KLD <- sum((intG[-nGrid] + intG[-1]) * (xs[2] - xs[1]) / 2)

    ## Convert to distance and return value
    return(sqrt(2 * KLD))
}

inla.sens.distance.skew <- function(muMarg, sdMarg, skMarg, prob, robMarg, nGrid, extraLen = 20) {
    inla.require("sn", stop.on.error = TRUE)
    ## Estimate required integration grid
    sdMax <- max(sdMarg)
    xs <- seq(-1, 1, length.out = nGrid) * sdMax * extraLen + mean(muMarg)

    ## Iterate through \theta values
    yO <- vector(mode = "numeric", length = nGrid)
    yR <- yO
    for (idxT in 1:length(muMarg)) {
        ## Extract parameters of skew normal marginal
        mu <- muMarg[idxT]
        sig <- sdMarg[idxT]
        gamma <- skMarg[idxT]
        if (is.nan(gamma)) {
              gamma <- 0
          }

        ## Convert to standard parametrization of skew normal
        p <- inla.sens.skewMap(c(mu, sig, gamma))

        ## Map to standard distribution
        sIdx <- 1
        mIdx <- floor(nGrid / 2)
        eIdx <- nGrid
        tmpXX1 <- sn::psn(xs[sIdx:(mIdx - 1)], xi = p[1], omega = p[2], alpha = p[3])
        tmpXX2 <- sn::psn(-xs[mIdx:eIdx], xi = -p[1], omega = p[2], alpha = -p[3])
        xx <- qnorm(tmpXX1)
        xx2 <- -qnorm(tmpXX2)
        xx <- c(xx, xx2)

        ## Use precomputed table of standard robust distribution
        yy <- exp(spline(x = robMarg$x, y = robMarg$y, xout = xx)$y)

        ## Correct for transformation
        yy <- yy * exp(sn::dsn(xs, xi = p[1], omega = p[2], alpha = p[3], log = TRUE) - dnorm(xx, log = TRUE))

        ## Remove the extrapolated values
        yy[(xx < min(robMarg$x)) | (xx > max(robMarg$x))] <- 0

        ## Add original and robust to their respective mixtures
        yO <- yO + prob[idxT] * sn::dsn(xs, xi = p[1], omega = p[2], alpha = p[3])
        yR <- yR + prob[idxT] * yy
    }

    ## Calculate KLD between original and added uncertainty
    intG <- yR * log(yR / yO)
    intG[yR == 0] <- 0
    KLD <- sum((intG[-nGrid] + intG[-1]) * (xs[2] - xs[1]) / 2)

    ## Convert to distance and return value
    return(sqrt(2 * KLD))
}

inla.sens.skewMap <- function(x) {
    ## Extract desired parameters of skew normal
    mu <- x[1]
    s <- x[2]
    g1 <- x[3]

    ## Transform to usual parametrization of skew normal
    d <- sign(g1) * sqrt(abs(g1)^(2 / 3) / (2 / pi * (((4 - pi) / 2)^(2 / 3) + abs(g1)^(2 / 3))))
    alpha <- d / sqrt(1 - d^2)
    w <- s * (1 - 2 * d^2 / pi)^(-0.5)
    ksi <- mu - w * d * sqrt(2 / pi)

    return(c(ksi, w, alpha))
}

inla.sens.post <- function(res.sens, rIdx, X) {
    ## Number of components
    nC <- length(res.sens)
    nP <- length(res.sens[[1]]$val)

    ## Iterate through each posterior and plot
    par(ask = TRUE)
    for (idxP in 1:nP) {
        ## Get values and names
        val <- NULL
        nam <- NULL
        for (idxC in 1:nC) {
            if (res.sens[[idxC]]$tag == "Fixed effects") {
                tmpIdx <- (X[idxP, ] != 0)
                val <- c(val, res.sens[[idxC]]$val[tmpIdx])
                nam <- c(nam, res.sens[[idxC]]$names[tmpIdx])
            } else {
                if (!is.na(rIdx[idxP, idxC])) {
                    val <- c(val, res.sens[[idxC]]$val[rIdx[idxP, idxC]])
                    nam <- c(nam, res.sens[[idxC]]$tag)
                }
            }
        }

        ## Standardize with linear predictor
        val <- val / val[1]
        inla.dev.new()
        cex.names <- 1.5
        cex.axis <- 1.5
        lwd <- 1.5
        barplot(val,
            space = 2,
            ylim = c(1 / exp(3), exp(3)),
            main = paste("Linear predicter", idxP),
            names.arg = nam,
            xlab = "Index",
            ylab = "Resiliency",
            cex.names = cex.names,
            cex.axis = cex.axis,
            lwd = lwd,
            log = c("y")
        )
    }
    par(ask = FALSE)
}

## Export:  inla.ssh.copy.id  inla.remote

## !\name{inla.ssh.copy.id}
## !\alias{inla.ssh.copy.id}
## !\alias{ssh.copy.id}
## !\alias{inla.remote}
## !
## !\title{Setup remote computing}
## !
## !\description{Initialize the definition file and print the path to the internal script to transfer ssh-keys}
## !
## !\usage{
## !inla.remote()
## !inla.ssh.copy.id()
## !}
## !\arguments{
## ! None
## !}
## !
## !\value{%%
## ! \code{inla.remote} is used once to setup the remote host information
## ! file (definition file) in the users home directory; see the FAQ entry on this
## ! issue for more information.
## ! \code{inla.ssh.copy.id} will return the path to the internal script to transfer ssh-keys.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\examples{
## !##See the FAQ entry on this issue on r-inla.org.
## !}


`inla.ssh.copy.id` <- function() {
    ## print the path to the copy-id script

    f <- system.file("bin/remote/ssh-copy-id", package = "INLA")
    if (inla.os("windows")) {
          return(inla.cygwin.map.filename(f))
      } else {
          return(f)
      }
}

`inla.remote` <- function() {
    if (inla.os("windows") && !inla.cygwin.check.path()) {
        cat("\n\n\tRemote computing in INLA from Windows require CYGWIN.\n")
        cat("\tThe CYGWIN-installation is not found. Please install it from www.cygwin.com\n")
        cat("\tMake sure to install also all `ssh' components; search for it, select them and install.\n")
        cat("\tIts easiest to use the standard installation path; c:/cygwin, otherwise give the new path using\n")
        cat("\t\tinla.setOption(\"cygwin\", \"NEWPATH\")\n")
        cat("\n")
        return(NULL)
    }

    inla <- system.file("bin/remote/inla.remote", package = "INLA")
    inla.setOption("inla.call", inla)

    f <- paste(inla.get.HOME(), "/.inlarc", sep = "")
    if (!file.exists(f)) {
        if (inla.os("windows")) {
              ini <- system.file("bin/remote/dot.inlarc-example-windows", package = "INLA")
          } else {
              ini <- system.file("bin/remote/dot.inlarc-example", package = "INLA")
          }

        if (!file.exists(ini)) {
              stop(paste("Missing file in installation:", ini, "This should not happen..."))
          }
        file.copy(ini, f)
        cat(paste("Remote computing: Please edit remote-host details in\n\t\t", f, "\n"))
    } else {
        cat(paste("Remote computing is defined in\n\t\t", f, "\n"))
    }
    if (inla.os("windows")) {
        cat("\n\tIf you need to copy ssh-keys, you can do that using this script from within a cygwin-terminal\n")
        cat(paste("\t\t", inla.cygwin.map.filename(system.file("bin/remote/ssh-copy-id", package = "INLA")), "\n\n"))
    }

    return(invisible())
}

## ----setup, include=FALSE-----------------------------------------------------
set.seed(123)
library(knitr)
knitr::opts_chunk$set(echo = TRUE, results = 'show', 
	fig.keep = 'high', fig.align = 'center', fig.pos = 'h')
knitr::opts_chunk$set(fig.path="figures/bgev/")

library(INLA)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")

## ---- echo=F, message=F, warning=FALSE----------------------------------------
colFmt = function(x,color){
  outputFormat = opts_knit$get("rmarkdown.pandoc.to")
  if(outputFormat == 'latex')
    paste("\\textcolor[HTML]{",color,"}{",x,"}",sep="")
  else if(outputFormat == 'html')
    paste("<font color='",color,"'>",x,"</font>",sep="")
  else
    x
}

library(INLA)
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}

## ----pressure, echo=FALSE, fig.cap="\\label{fig:distributionH.pdf} bGEV distribution (H, black) constructed from distributions F (Frechét, red), G (Gumbel, green) and Beta weight function $p$ (purple). The shaded area is the mixing area, where F and G are merged.", out.width = '50%'----
knitr::include_graphics("fig_distribution1.png")

## -----------------------------------------------------------------------------
n = 1000
x = rnorm(n) # we generate values for x from a N(0,1) dist.
eta.x = 1 + 0.4*x

## -----------------------------------------------------------------------------
spread = 0.3
tail = 0.1

## -----------------------------------------------------------------------------
p.alpha = 0.5
p.beta = 0.25

## -----------------------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = spread, alpha = p.alpha, beta = p.beta, 
                     xi = tail)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma, shape = par$xi)

## -----------------------------------------------------------------------------
hyper.spread = list(initial = 1,
                    fixed=FALSE,
                    prior = "loggamma",
                    param = c(3, 3))

## -----------------------------------------------------------------------------
tail.interval = c(0, 0.5)
tail.intern = map.tail(tail, tail.interval, inverse=TRUE)

## -----------------------------------------------------------------------------
hyper.tail = list(initial = tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= FALSE)


## -----------------------------------------------------------------------------
hyper.tail = list(initial = if (tail == 0.0) -Inf else tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= if (tail == 0.0) TRUE else FALSE)


## -----------------------------------------------------------------------------

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail)

## -----------------------------------------------------------------------------
control.bgev = list(q.location = p.alpha,
                    q.spread = p.beta,
                    # quantile levels for the mixing part
                    q.mix= c(0.05, 0.20), 
                    # the Beta(s1, s2) mixing distribution parameters. 
                    # Hard-coded for the moment: s1=s2=5
                    beta.ab = 5)

## -----------------------------------------------------------------------------
null.matrix = matrix(nrow = n, ncol= 0)
spread.x = null.matrix
tail.x = null.matrix

## -----------------------------------------------------------------------------
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x

## ---- message=F, eval = T, warning=FALSE--------------------------------------
r1 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE)

## -----------------------------------------------------------------------------
round(r1$summary.fixed,4)
round(r1$summary.hyperpar,4)

## -----------------------------------------------------------------------------
n = 1000
x1 = rnorm(n)
eta.x = 1 + 0.4*x1

## -----------------------------------------------------------------------------
x2 = rnorm(n)
s.x = exp(0.1 + 0.3*x2)
x3 = runif(n,-0.25,1)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi

## ---- eval = T----------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])

## -----------------------------------------------------------------------------
hyper.beta1 = hyper.beta2 = list(prior = "normal",
                                 param = c(0, 300),
                                 initial = 0)

## -----------------------------------------------------------------------------

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2)

## -----------------------------------------------------------------------------
spread.x = x2
tail.x = x3
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x

## -----------------------------------------------------------------------------
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)

## ---- message=F, eval = T, warning=FALSE--------------------------------------
r2 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE)


## ---- eval = T----------------------------------------------------------------
round(r2$summary.fixed,4)
round(r2$summary.hyperpar,4)

## ---- eval = T----------------------------------------------------------------
n = 1000
x1 = rnorm(n)
z1 = seq(0, 6, length.out = n)
z2 = 1:n
p = 2 # AR order
pacf = runif(p)
phi = inla.ar.pacf2phi(pacf)
eta.x = 1 + 0.4*x + sin(z1) + c(scale(arima.sim(n, model = list(ar = phi))))

## -----------------------------------------------------------------------------
x2 = rnorm(n)
x4 = rnorm(n)
s.x = exp(0.1 + 0.3*x2 + x4)
x3 = runif(n,-0.5,2)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi

## ---- eval = T----------------------------------------------------------------
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])

## -----------------------------------------------------------------------------
hyper.beta1 = hyper.beta2 = hyper.beta3 = list(prior = "normal",
                                               param = c(0, 300),
                                               initial = 0)
hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2,
                  beta3 = hyper.beta3)

## ---- message=F, eval = T, warning=FALSE--------------------------------------
spread.x = x2
spread.xx = x4
tail.x = x3
# With this change of variable it is easier to keep track of the effect 
# of the covariates in each parameter, but it is not needed.
formula = inla.mdata(y, cbind(spread.x, spread.xx), tail.x) ~ -1 + intercept + x + 
	f(z1, model = "rw1", scale.model=TRUE, constr=TRUE,
		hyper = list(prec = list(prior = "pc.prec", 
		param = c(0.1, 0.01)))) +
		f(z2, model = 'ar', order = 2, 
		   hyper=list(prec=list(prior="pc.prec", constr=TRUE,
		                        param=c(0.1,0.01)),
                      pacf1=list(param=c(0.5,0.8),
                      pacf2=list(param=c(0.5,0.8)))))
data.bgev = data.frame(y = y, intercept = 1, x = x, z1 = z1, z2 = z2,
                       spread.x = spread.x, spread.xx = spread.xx, tail.x = tail.x)
r3 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100,prec.intercept=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb",cmin=0),
         verbose=FALSE)

## ---- eval = T----------------------------------------------------------------
round(r3$summary.fixed,4)
round(r3$summary.hyperpar,4)

## ---- eval=F------------------------------------------------------------------
#  library(INLA)
#  inla.list.models('latent')

## -----------------------------------------------------------------------------
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}


## -----------------------------------------------------------------------------
map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}

## meshassessment.R
##
## Copyright (C) 2016, 2018, Finn Lindgren
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Only one function is exported:
## Export: inla.mesh.assessment

## !\name{inla.mesh.assessment}
## !\alias{inla.mesh.assessment}
## !\title{Interactive mesh building and diagnostics}
## !\description{Assess the finite element
## !  approximation errors in a mesh for interactive R sessions.
## !  More detailed assessment tools are in \code{\link{meshbuilder}}.
## !}
## !\usage{
## !inla.mesh.assessment(mesh)
## !}
## !\author{Finn Lindgren \email{finn.lindgren@gmail.com}}
## !\seealso{inla.mesh.2d, inla.mesh.create, meshbuilder}
## !\examples{
## !bnd <- inla.mesh.segment(cbind(c(0, 10, 10, 0, 0),
## !                               c(0, 0, 10, 10, 0)), bnd = TRUE)
## !mesh <- inla.mesh.2d(boundary = bnd, max.edge = 1)
## !out <- inla.mesh.assessment(mesh, spatial.range = 3, alpha = 2)
## !}





#' Interactive mesh building and diagnostics
#'
#' Assess the finite element approximation errors in a mesh for interactive R
#' sessions.  More detailed assessment tools are in [meshbuilder()].
#'
#'
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso inla.mesh.2d, inla.mesh.create, meshbuilder
#' @examples
#'
#' bnd <- inla.mesh.segment(cbind(
#'     c(0, 10, 10, 0, 0),
#'     c(0, 0, 10, 10, 0)
#' ), bnd = TRUE)
#' mesh <- inla.mesh.2d(boundary = bnd, max.edge = 1)
#' out <- inla.mesh.assessment(mesh, spatial.range = 3, alpha = 2)
#' @export inla.mesh.assessment
inla.mesh.assessment <- function(mesh, spatial.range, alpha = 2,
                                 dims = c(500, 500)) {
    mesh.edgelengths <- function(mesh, proj) {
        i <- c()
        val <- c()
        num <- c()
        tri_idx <- rbind(c(1, 2), c(2, 3), c(3, 1))
        for (k in 1:3) {
            ti1 <- tri_idx[k, 1]
            ti2 <- tri_idx[k, 2]
            len <- Matrix::rowSums((mesh$loc[mesh$graph$tv[, ti2], ] -
                mesh$loc[mesh$graph$tv[, ti1], ])^2)^0.5
            i <- c(i, mesh$graph$tv[, ti1], mesh$graph$tv[, ti2])
            val <- c(val, rep(as.vector(len), times = 2))
            num <- c(num, rep(1, 2 * length(as.vector(len))))
        }
        avg_len <-
            as.vector(sparseMatrix(i = i, j = rep(1, length(i)), x = val)) /
                as.vector(sparseMatrix(i = i, j = rep(1, length(i)), x = num))

        proj_len <- as.vector(proj$proj$A %*% avg_len)
        proj_len[!proj$proj$ok] <- NA
        proj_len
    }
    mesh.proj <- function(mesh, dims) {
        INLA::inla.mesh.projector(mesh, dims = dims)
    }
    mesh.spde <- function(mesh, alpha) {
        INLA::inla.spde2.pcmatern(mesh,
            alpha = alpha,
            prior.range = c(1, 0.5),
            prior.sigma = c(1, 0.5)
        )
    }
    mesh.Q <- function(spde, spatial.range) {
        INLA::inla.spde.precision(spde, theta = log(c(spatial.range, 1)))
    }
    mesh.S <- function(Q) {
        INLA::inla.qinv(Q, reordering = INLA::inla.reorderings())
    }
    mesh.sd <- function(proj, S) {
        v <- Matrix::rowSums(proj$proj$A * (proj$proj$A %*% S))
        v[!proj$proj$ok] <- NA
        matrix(v^0.5, length(proj$x), length(proj$y))
    }
    mesh.sd.deviation.approx <- function(proj, S, sd0) {
        val <- proj$proj$A %*% (
            as.vector(t(proj$proj$A[proj$proj$ok, , drop = FALSE]) %*%
                as.vector(sd0)[proj$proj$ok]) /
                colSums(proj$proj$A[proj$proj$ok, , drop = FALSE]))
        val[!proj$proj$ok] <- NA
        matrix(
            1 + (as.vector(sd0) - val),
            length(proj$x), length(proj$y)
        )
    }
    mesh.sd.bound <- function(proj, S) {
        INLA::inla.mesh.project(proj, field = diag(mesh.S())^0.5)
    }


    if (mesh$manifold != "R2") {
        warning("inla.mesh.assessment has only been tested on flat 2D manifolds.")
    }

    spde <- mesh.spde(mesh = mesh, alpha = alpha)
    Q <- mesh.Q(spde = spde, spatial.range = spatial.range)
    S <- mesh.S(Q = Q)
    proj <- mesh.proj(mesh = mesh, dims = dims)
    sd0 <- mesh.sd(proj = proj, S = S)
    sd.deviation <- mesh.sd.deviation.approx(proj = proj, S = S, sd0 = sd0)
    edgelengths <- mesh.edgelengths(mesh, proj)

    out <- data.frame(
        x = proj$lattice$loc[, 1],
        y = proj$lattice$loc[, 2],
        sd = as.vector(sd0),
        sd.dev = as.vector(sd.deviation),
        edge.len = edgelengths
    )
    coordinates(out) <- ~ x + y
    out
}

## Export: inla.collect.results

## ! \name{inla.collect.results}
## ! \alias{inla.collect.results}
## ! \alias{collect.results}
## ! \title{Collect results from a inla-call}
## ! \description{\code{inla.collect.results} collect results  from a inla-call}
## ! \usage{
## ! inla.collect.results(
## !              results.dir,
## !              debug=FALSE,
## !              only.hyperparam=FALSE,
## !              file.log = NULL,
## !              file.log2 = NULL)
## !}
## ! \arguments{

`inla.collect.results` <-
    function(
             ## ! \item{results.dir}{The directory where the results of the inla run are stored}
             results.dir,

             ## ! \item{debug}{Logical. If \code{TRUE} some debugging information are printed}
             debug = FALSE,

             ## ! \item{only.hyperparam}{Binary variable indicating wheather only the
             ## ! results for the hyperparameters should be collected}
             only.hyperparam = FALSE,

             ## ! \item{file.log}{Character. The filename, if any, of the logfile for
             ## ! the internal calculations}
             file.log = NULL,

             ## ! \item{file.log2}{Character. The filename, if any, of the logfile2 for
             ## ! the internal calculations}
             file.log2 = NULL)
{
    ## ! }
    ## ! \value{ The function returns an object of class \code{"inla"}, see the
    ## ! help file for \code{inla} for details.}
    ## !
    ## ! \details{This function is mainly used inside \code{inla}
    ## !  to collect results after running the inla
    ## !  function. It can also be used to collect results into R after having
    ## !  runned a inla section outside R.  }

    if (is.na(file.info(results.dir)$isdir) ||
        !file.info(results.dir)$isdir) {
        stop(paste("This is not a directory: ", results.dir, "\n"))
    }

    filename <- paste(results.dir, "/.ok", sep = "")
    res.ok <- file.exists(filename)
    if (!res.ok) {
        ## try this one instead
        results.dir.new <- paste(results.dir, "/results.files", sep = "")
        filename <- paste(results.dir.new, "/.ok", sep = "")
        res.ok <- file.exists(filename)
        if (res.ok) {
            if (debug) {
                cat(paste("inla.collect.results: retry with directory", results.dir.new, "\n"))
            }
            return(inla.collect.results(results.dir.new,
                                        debug = debug,
                                        only.hyperparam = only.hyperparam,
                                        file.log = file.log,
                                        file.log2 = file.log2
                                        ))
        } else {
            ## neither directories contain the file /.ok, then we
            ## assume the inla-program has crashed
            inla.inlaprogram.has.crashed()
        }
    }

    if (!only.hyperparam) {
        res.fixed <- inla.collect.fixed(results.dir, debug)
        res.lincomb <- inla.collect.lincomb(results.dir, debug, derived = FALSE)
        res.lincomb.derived <- inla.collect.lincomb(results.dir, debug, derived = TRUE)
        res.dic <- inla.collect.dic(results.dir, debug)
        res.cpo.pit <- inla.collect.cpo(results.dir, debug)
        res.po <- inla.collect.po(results.dir, debug)
        res.waic <- inla.collect.waic(results.dir, debug)
        res.random <- inla.collect.random(results.dir, debug)
        res.predictor <- inla.collect.predictor(results.dir, debug)
        res.spde2.blc <- inla.collect.spde2.blc(results.dir, debug)
        res.spde3.blc <- inla.collect.spde3.blc(results.dir, debug)
    } else {
        res.fixed <- NULL
        res.lincomb <- NULL
        res.lincomb.derived <- NULL
        res.dic <- NULL
        res.cpo.pit <- NULL
        res.po <- NULL
        res.waic <- NULL
        res.random <- NULL
        res.predictor <- NULL
        res.spde2.blc <- NULL
        res.spde3.blc <- NULL
    }
    res.mlik <- inla.collect.mlik(results.dir, debug)
    res.q <- inla.collect.q(results.dir, debug)
    res.graph <- inla.collect.graph(results.dir, debug)
    res.offset <- inla.collect.offset.linear.predictor(results.dir, debug)

    ## get the hyperparameters
    theta.mode <- inla.read.binary.file(paste(results.dir, .Platform$file.sep, ".theta_mode", sep = ""))[-1]
    x.mode <- inla.read.binary.file(paste(results.dir, .Platform$file.sep, ".x_mode", sep = ""))[-1]
    gitid <- readLines(paste(results.dir, .Platform$file.sep, ".gitid", sep = ""))

    lfn.fnm <- paste(results.dir, .Platform$file.sep, "linkfunctions.names", sep = "")
    if (file.exists(lfn.fnm)) {
        linkfunctions.names <- readLines(lfn.fnm)
        fp <- file(paste(results.dir, .Platform$file.sep, "linkfunctions.link", sep = ""), "rb")
        n <- readBin(fp, integer(), 1)
        idx <- readBin(fp, double(), n)
        ok <- which(!is.nan(idx))
        idx[ok] <- idx[ok] + 1
        close(fp)
        linkfunctions <- list(names = linkfunctions.names, link = as.integer(idx))
    } else {
        linkfunctions <- NULL
    }

    if (length(theta.mode) > 0) {
        res.hyper <- inla.collect.hyperpar(results.dir, debug)

        ## get the joint (if printed)
        alldir <- dir(results.dir)
        if (length(grep("joint.dat", alldir)) == 1) {
            if (debug) {
                print("inla.collect.joint hyperpar")
            }
            fnm <- paste(results.dir, "/joint.dat", sep = "")
            if (file.info(fnm)$size > 0) {
                joint.hyper <- read.table(fnm)
            } else {
                joint.hyper <- NULL
            }
        } else {
            joint.hyper <- NULL
        }
    } else {
        res.hyper <- NULL
        joint.hyper <- NULL
    }

    logfile <- list(logfile = c(
                        inla.collect.logfile(file.log, debug)$logfile,
                        "", paste(rep("*", 72), sep = "", collapse = ""), "",
                        inla.collect.logfile(file.log2, debug)$logfile
                    ))
    misc <- inla.collect.misc(results.dir, debug)
    theta.tags <- NULL
    mode.status <- NA
    if (!is.null(misc)) {
        ## put also theta.mode in here
        misc$theta.mode <- theta.mode
        ## we need theta.tags for later usage
        if (!is.null(misc$theta.tags)) {
            theta.tags <- misc$theta.tags
        }
        mode.status <- misc$mode.status

        if (!is.null(misc$lincomb.derived.correlation.matrix)) {
            if (!is.null(res.lincomb.derived)) {
                id <- res.lincomb.derived$summary.lincomb.derived$ID
                tag <- rownames(res.lincomb.derived$summary.lincomb.derived)
                R <- misc$lincomb.derived.correlation.matrix
                rownames(R) <- colnames(R) <- tag[id]
                misc$lincomb.derived.correlation.matrix <- R
            } else {
                misc$lincomb.derived.correlation.matrix <- NULL
            }
        }
        if (!is.null(misc$lincomb.derived.covariance.matrix)) {
            if (!is.null(res.lincomb.derived)) {
                id <- res.lincomb.derived$summary.lincomb.derived$ID
                tag <- rownames(res.lincomb.derived$summary.lincomb.derived)
                R <- misc$lincomb.derived.covariance.matrix
                rownames(R) <- colnames(R) <- tag[id]
                misc$lincomb.derived.covariance.matrix <- R
            } else {
                misc$lincomb.derived.covariance.matrix <- NULL
            }
        }

        ## also put the linkfunctions here
        misc$linkfunctions <- linkfunctions
        if (!is.null(linkfunctions)) {
            ## a better name
            misc$family <- linkfunctions$link
        }
    }

    ## add the names of the theta's here, as they are available.
    if (!is.null(misc) && !is.null(joint.hyper)) {
        colnames(joint.hyper) <- c(misc$theta.tags,
                                   "Log posterior density",
                                   "Total integration weight (log.dens included)")
    }

    names(theta.mode) <- theta.tags
    res <- c(res.fixed, res.lincomb, res.lincomb.derived, res.mlik,
             list(cpo = res.cpo.pit), list(po = res.po), list(waic = res.waic),
             res.random, res.predictor, res.hyper,
             res.offset, res.spde2.blc, res.spde3.blc, logfile,
             list(
                 misc = misc,
                 dic = res.dic, mode = list(
                                    theta = theta.mode, x = x.mode,
                                    theta.tags = theta.tags, mode.status = mode.status,
                                    log.posterior.mode = misc$log.posterior.mode
                                ),
                 joint.hyper = joint.hyper, nhyper = length(theta.mode),
                 version = list(inla.call = gitid, R.INLA = inla.version("version"))
             ),
             list(Q = res.q),
             res.graph,
             ok = res.ok
             )
    class(res) <- "inla"
    return(res)
}

`inla.collect.misc` <- function(dir, debug = FALSE) 
{
    d <- paste(dir, "/misc", sep = "")
    d.info <- file.info(d)$isdir

    if (debug) {
        print(paste("collect misc from", d))
    }

    if (is.na(d.info) || (d.info == FALSE)) {
        return(NULL)
    }

    fnm <- paste(d, "/theta-tags", sep = "")
    if (file.exists(fnm)) {
        tags <- readLines(fnm)
    } else {
        tags <- NULL
    }

    fnm <- paste(d, "/theta-from", sep = "")
    if (file.exists(fnm)) {
        theta.from <- readLines(fnm)
        ## evaluate these as functions
        theta.from <- lapply(theta.from, inla.source2function)
        if (!is.null(tags)) {
            names(theta.from) <- tags
        }
    } else {
        theta.from <- NULL
    }

    fnm <- paste(d, "/theta-to", sep = "")
    if (file.exists(fnm)) {
        theta.to <- readLines(fnm)
        ## evaluate these as functions
        theta.to <- lapply(theta.to, inla.source2function)
        if (!is.null(tags)) {
            names(theta.to) <- tags
        }
    } else {
        theta.to <- NULL
    }

    fnm <- paste(d, "/covmat-hyper-internal.dat", sep = "")
    if (file.exists(fnm)) {
        siz <- inla.read.binary.file(fnm)
        n <- siz[1L]
        stopifnot(length(siz) == n^2L + 1L)

        cov.intern <- matrix(siz[-1L], n, n)
        dd <- diag(cov.intern)
        s <- matrix(0.0, n, n)
        diag(s) <- 1.0 / sqrt(dd)
        cor.intern <- s %*% cov.intern %*% s
        diag(cor.intern) <- 1.0
    } else {
        cov.intern <- NULL
        cor.intern <- NULL
    }

    fnm <- paste(d, "/covmat-eigenvectors.dat", sep = "")
    if (file.exists(fnm)) {
        siz <- inla.read.binary.file(fnm)
        n <- siz[1L]
        stopifnot(length(siz) == n^2L + 1L)
        cov.intern.eigenvectors <- matrix(siz[-1L], n, n)
    } else {
        cov.intern.eigenvectors <- NULL
    }

    fnm <- paste(d, "/covmat-eigenvalues.dat", sep = "")
    if (file.exists(fnm)) {
        siz <- inla.read.binary.file(fnm)
        n <- siz[1L]
        stopifnot(length(siz) == n + 1L)
        cov.intern.eigenvalues <- siz[-1L]
    } else {
        cov.intern.eigenvalues <- NULL
    }

    fnm <- paste(d, "/reordering.dat", sep = "")
    if (file.exists(fnm)) {
        r <- as.integer(inla.read.binary.file(fnm))
    } else {
        r <- NULL
    }

    fnm <- paste(d, "/stdev_corr_pos.dat", sep = "")
    if (file.exists(fnm)) {
        stdev.corr.positive <- as.numeric(inla.read.fmesher.file(fnm))
    } else {
        stdev.corr.positive <- NULL
    }

    fnm <- paste(d, "/stdev_corr_neg.dat", sep = "")
    if (file.exists(fnm)) {
        stdev.corr.negative <- as.numeric(inla.read.fmesher.file(fnm))
    } else {
        stdev.corr.negative <- NULL
    }

    fnm <- paste(d, "/lincomb_derived_correlation_matrix.dat", sep = "")
    if (file.exists(fnm)) {
        lincomb.derived.correlation.matrix <- inla.read.fmesher.file(fnm)
    } else {
        lincomb.derived.correlation.matrix <- NULL
    }

    fnm <- paste(d, "/lincomb_derived_covariance_matrix.dat", sep = "")
    if (file.exists(fnm)) {
        lincomb.derived.covariance.matrix <- inla.read.fmesher.file(fnm)
    } else {
        lincomb.derived.covariance.matrix <- NULL
    }

    fnm <- paste(d, "/opt_directions.dat", sep = "")
    if (file.exists(fnm)) {
        opt.directions <- inla.read.fmesher.file(fnm)
        n <- dim(opt.directions)[1]
        colnames(opt.directions) <- paste0("dir:", 1:n)
        rownames(opt.directions) <- paste0("theta:", 1:n)
    } else {
        opt.directions <- NULL
    }

    fnm <- paste(d, "/mode-status.dat", sep = "")
    if (file.exists(fnm)) {
        mode.status <- scan(fnm, quiet = TRUE)
    } else {
        mode.status <- NA
    }

    fnm <- paste(d, "/nfunc.dat", sep = "")
    if (file.exists(fnm)) {
        nfunc <- as.numeric(scan(fnm, quiet = TRUE))
    } else {
        nfunc <- NA
    }

    fnm <- paste(d, "/log-posterior-mode.dat", sep = "")
    if (file.exists(fnm)) {
        lpm <- scan(fnm, quiet = TRUE)
    } else {
        lpm <- NA
    }

    fnm <- paste(d, "/config/configs.dat", sep = "")
    if (file.exists(fnm)) {
        fp <- file(fnm, "rb")
        iarr <- readBin(fp, integer(), 3)
        configs <- list(
            .preopt = FALSE, 
            n = iarr[1],
            nz = iarr[2],
            ntheta = iarr[3]
        )
        configs.i <- readBin(fp, integer(), configs$nz) ## 0-based
        configs.j <- readBin(fp, integer(), configs$nz) ## 0-based
        configs$nconfig <- readBin(fp, integer(), 1)

        nc <- readBin(fp, integer(), 1)
        if (nc > 0) {
            A <- readBin(fp, numeric(), configs$n * nc)
            e <- readBin(fp, numeric(), nc)
            configs$constr <- list(
                nc = nc,
                A = matrix(A, nc, configs$n),
                e = e
            )
        } else {
            configs$constr <- NULL
        }

        theta.tag <- readLines(paste(d, "/config/theta-tag.dat", sep = ""))
        configs$contents <- list(
            tag = readLines(paste(d, "/config/tag.dat", sep = "")),
            start = as.integer(readLines(paste(d, "/config/start.dat", sep = ""))) + 1L,
            length = as.integer(readLines(paste(d, "/config/n.dat", sep = "")))
        )

        if (configs$nconfig > 0L) {
            configs$config[[configs$nconfig]] <- list()
            for (k in 1L:configs$nconfig) {
                log.post <- readBin(fp, numeric(), 1)
                log.post.orig <- readBin(fp, numeric(), 1)
                if (configs$ntheta > 0L) {
                    theta <- readBin(fp, numeric(), configs$ntheta)
                    names(theta) <- theta.tag
                } else {
                    theta <- NULL
                }
                mean <- readBin(fp, numeric(), configs$n)
                improved.mean <- readBin(fp, numeric(), configs$n)
                skewness <- readBin(fp, numeric(), configs$n)
                ## add the offsets to the mean here
                offsets <- readBin(fp, numeric(), configs$n)
                mean <- mean + offsets
                improved.mean <- improved.mean + offsets

                Q <- readBin(fp, numeric(), configs$nz)
                Qinv <- readBin(fp, numeric(), configs$nz)
                Qprior <- readBin(fp, numeric(), configs$n)
                dif <- which(configs$i != configs$j)
                if (length(dif) > 0L) {
                    iadd <- configs.j[dif] ## yes, its the transpose part
                    jadd <- configs.i[dif] ## yes, its the transpose part
                    Qadd <- Q[dif]
                    Qinvadd <- Qinv[dif]
                } else {
                    iadd <- c()
                    jadd <- c()
                    Qadd <- c()
                    Qinvadd <- c()
                }
                configs$config[[k]] <- list(
                    .preopt = FALSE, 
                    theta = theta,
                    log.posterior = log.post,
                    log.posterior.orig = log.post.orig,
                    mean = mean,
                    improved.mean = improved.mean,
                    skewness = skewness,
                    Q = sparseMatrix(
                        i = c(configs.i, iadd),
                        j = c(configs.j, jadd),
                        x = c(Q, Qadd),
                        dims = c(configs$n, configs$n),
                        index1 = FALSE,
                        repr = "C"
                    ),
                    Qinv = sparseMatrix(
                        i = c(configs.i, iadd),
                        j = c(configs.j, jadd),
                        x = c(Qinv, Qinvadd),
                        dims = c(configs$n, configs$n),
                        index1 = FALSE,
                        repr = "C"
                    ),
                    Qprior.diag = Qprior
                )
            }

            ## rescale the log.posteriors
            configs$max.log.posterior <- max(sapply(configs$config, function(x) x$log.posterior.orig))
            max.log.posterior <- max(sapply(configs$config, function(x) x$log.posterior))
            for (k in 1L:configs$nconfig) {
                ## with integration weight
                configs$config[[k]]$log.posterior <- configs$config[[k]]$log.posterior - max.log.posterior
                ## without integration weights
                configs$config[[k]]$log.posterior.orig <- configs$config[[k]]$log.posterior.orig - configs$max.log.posterior
            }
        } else {
            configs$config <- NULL
        }
        close(fp)
    } else {
        configs <- NULL
    }


    fnm <- paste(d, "/config_preopt/configs.dat", sep = "")
    if (file.exists(fnm)) {
        fp <- file(fnm, "rb")
        iarr <- readBin(fp, integer(), 6)
        configs <- list(
            .preopt = TRUE, 
            mnpred = iarr[1], 
            Npred = iarr[2], 
            n = iarr[3], 
            nz = iarr[4], 
            prior_nz = iarr[5],
            ntheta = iarr[6])
        configs.i <- readBin(fp, integer(), configs$nz) ## 0-based
        configs.j <- readBin(fp, integer(), configs$nz) ## 0-based
        configs.iprior <- readBin(fp, integer(), configs$prior_nz) ## 0-based
        configs.jprior <- readBin(fp, integer(), configs$prior_nz) ## 0-based
        configs$nconfig <- readBin(fp, integer(), 1)

        nc <- readBin(fp, integer(), 1)
        if (nc > 0) {
            A <- readBin(fp, numeric(), configs$n * nc)
            e <- readBin(fp, numeric(), nc)
            configs$constr <- list(
                nc = nc,
                A = matrix(A, nc, configs$n),
                e = e
            )
        } else {
            configs$constr <- NULL
        }
        configs$offsets <- readBin(fp, numeric(), configs$mnpred)

        theta.tag <- readLines(paste(d, "/config_preopt/theta-tag.dat", sep = ""))
        configs$contents <- list(
            tag = readLines(paste(d, "/config_preopt/tag.dat", sep = "")),
            start = as.integer(readLines(paste(d, "/config_preopt/start.dat", sep = ""))) + 1L,
            length = as.integer(readLines(paste(d, "/config_preopt/n.dat", sep = "")))
        )

        configs$A <- inla.read.fmesher.file(paste0(d, "/config_preopt/A.dat"))
        fnm <- paste0(d, "/config_preopt/pA.dat")
        if (file.exists(fnm)) {
            configs$pA <- inla.read.fmesher.file(fnm)
        } else {
            configs$pA <- matrix(nrow = 0, ncol = nrow(configs$A))
        }

        if (configs$nconfig > 0L) {
            configs$config[[configs$nconfig]] <- list()
            for (k in 1L:configs$nconfig) {
                log.post <- readBin(fp, numeric(), 1)
                log.post.orig <- readBin(fp, numeric(), 1)
                if (configs$ntheta > 0L) {
                    theta <- readBin(fp, numeric(), configs$ntheta)
                    names(theta) <- theta.tag
                } else {
                    theta <- NULL
                }
                mean <- readBin(fp, numeric(), configs$n)
                improved.mean <- readBin(fp, numeric(), configs$n)

                Q <- readBin(fp, numeric(), configs$nz)
                Qinv <- readBin(fp, numeric(), configs$nz)
                Qprior <- readBin(fp, numeric(), configs$prior_nz)
                cpodens.moments <- matrix(readBin(fp, numeric(), configs$Npred * 3), ncol = 3, byrow = TRUE)
                colnames(cpodens.moments) <- c("mean", "variance", "skewness")
                dif <- which(configs$i != configs$j)
                if (length(dif) > 0L) {
                    iadd <- configs.j[dif] ## yes, its the transpose part
                    jadd <- configs.i[dif] ## yes, its the transpose part
                    Qadd <- Q[dif]
                    Qinvadd <- Qinv[dif]
                } else {
                    iadd <- c()
                    jadd <- c()
                    Qadd <- c()
                    Qinvadd <- c()
                }
                dif <- which(configs$iprior != configs$jprior)
                if (length(dif) > 0L) {
                    iprioradd <- configs.jprior[dif] ## yes, its the transpose part
                    jprioradd <- configs.iprior[dif] ## yes, its the transpose part
                    Qprioradd <- Qprior[dif]
                } else {
                    iprioradd <- c()
                    jprioradd <- c()
                    Qprioradd <- c()
                }

                configs$config[[k]] <- list(
                    theta = theta,
                    log.posterior = log.post,
                    log.posterior.orig = log.post.orig,
                    mean = mean,
                    improved.mean = improved.mean,
                    skewness = rep(NA, configs$n), 
                    Q = sparseMatrix(
                        i = c(configs.i, iadd),
                        j = c(configs.j, jadd),
                        x = c(Q, Qadd),
                        dims = c(configs$n, configs$n),
                        index1 = FALSE,
                        repr = "C"
                    ),
                    Qinv = sparseMatrix(
                        i = c(configs.i, iadd),
                        j = c(configs.j, jadd),
                        x = c(Qinv, Qinvadd),
                        dims = c(configs$n, configs$n),
                        index1 = FALSE,
                        repr = "C"
                    ),
                    Qprior = sparseMatrix(
                        i = c(configs.iprior, iprioradd),
                        j = c(configs.jprior, jprioradd),
                        x = c(Qprior, Qprioradd),
                        dims = c(configs$n, configs$n),
                        index1 = FALSE,
                        repr = "C"
                    ),
                    cpodens.moments = cpodens.moments
                )
            }

            ## rescale the log.posteriors
            configs$max.log.posterior <- max(sapply(configs$config, function(x) x$log.posterior.orig))
            for (k in 1L:configs$nconfig) {
                configs$config[[k]]$log.posterior <- configs$config[[k]]$log.posterior - configs$max.log.posterior
                configs$config[[k]]$log.posterior.orig <- configs$config[[k]]$log.posterior.orig - configs$max.log.posterior
            }
        } else {
            configs$config <- NULL
        }
        close(fp)
    }

    if (debug) {
        print(paste("collect misc from", d, "...done"))
    }

    return(list(
        cov.intern = cov.intern, cor.intern = cor.intern,
        cov.intern.eigenvalues = cov.intern.eigenvalues, cov.intern.eigenvectors = cov.intern.eigenvectors,
        reordering = r, theta.tags = tags, log.posterior.mode = lpm,
        stdev.corr.negative = stdev.corr.negative, stdev.corr.positive = stdev.corr.positive,
        to.theta = theta.to, from.theta = theta.from, mode.status = mode.status,
        lincomb.derived.correlation.matrix = lincomb.derived.correlation.matrix,
        lincomb.derived.covariance.matrix = lincomb.derived.covariance.matrix,
        opt.directions = opt.directions,
        configs = configs, nfunc = nfunc
    ))
}

`inla.collect.logfile` <- function(file.log = NULL, debug = FALSE)
{
    if (is.null(file.log)) {
        return(list(logfile = NULL))
    }
    if (debug) {
        print(paste("Read logfile", file.log))
    }
    if (file.exists(file.log)) {
        ## replace tab with spaces
        logfile = gsub("\t", "        ", readLines(file.log))
        logfile <- logfile[-which(nchar(logfile) == 0)]
        return(list(logfile = logfile))
    } else {
        return(list(logfile = NULL))
    }
}

`inla.collect.size` <- function(dir, debug = FALSE)
{
    fnm <- paste(dir, "/size.dat", sep = "")
    siz <- inla.read.binary.file(fnm)
    if (length(siz) != 5L) {
        return(rep(0L, 5))
        ## stop(paste("length of siz is not 5L: fnm=", fnm))
    }
    if (is.na(siz[1L]) || siz[1L] < 0L) stop("siz[1L] = NA")
    if (is.na(siz[2L]) || siz[2L] <= 0L) siz[2L] <- siz[1L]
    if (is.na(siz[3L]) || siz[3L] <= 0L) siz[3L] <- siz[2L]
    if (is.na(siz[4L]) || siz[4L] <= 0L) siz[4L] <- 1L
    if (is.na(siz[5L]) || siz[5L] <= 0L) siz[5L] <- 1L

    return(list(n = siz[1L], N = siz[2L], Ntotal = siz[3L], ngroup = siz[4L], nrep = siz[5L]))
}

`inla.collect.hyperid` <- function(dir, debug = FALSE)
{
    fnm <- paste(dir, "/hyperid.dat", sep = "")
    id <- readLines(fnm)
    return(id)
}

`inla.collect.fixed` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    if (debug) {
        print("collect fixed effects")
    }

    ## read FIXED EFFECTS
    fix <- alldir[grep("^fixed.effect", alldir)]
    fix <- c(fix, alldir[grep("^intercept$", alldir)])
    n.fix <- length(fix)

    ## read the names of the fixed effects
    if (n.fix > 0L) {
        names.fixed <- inla.trim(character(n.fix))
        for (i in 1L:n.fix) {
            tag <- paste(results.dir, .Platform$file.sep, fix[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.fixed[i] <- "NameMissing"
            } else {
                names.fixed[i] <- readLines(tag, n = 1L)
            }
        }
        ## read summary the fixed effects
        if (debug) {
            print(names.fixed)
        }

        summary.fixed <- numeric()
        marginals.fixed <- list()
        marginals.fixed[[n.fix]] <- NA

        for (i in 1L:n.fix) {
            first.time <- (i == 1L)
            file <- paste(results.dir, .Platform$file.sep, fix[i], sep = "")
            dir.fix <- dir(file)
            if (length(dir.fix) > 3L) {
                summ <- inla.read.binary.file(paste(file, .Platform$file.sep, "summary.dat", sep = ""))[-1L]
                if (first.time) {
                    col.nam <- c("mean", "sd")
                }

                ## read quantiles if existing
                if (length(grep("^quantiles.dat$", dir.fix)) > 0L) {
                    qq <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "quantiles.dat", sep = "")),
                                                debug = debug
                                                )
                    summ <- c(summ, qq[, 2L])
                    if (first.time) {
                        col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "quant", sep = ""))
                    }
                }

                ## read mode if existing
                if (length(grep("^mode.dat$", dir.fix)) > 0L) {
                    mm <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "mode.dat", sep = "")),
                                                debug = debug
                                                )
                    summ <- c(summ, mm[, 2L])
                    if (first.time) {
                        col.nam <- c(col.nam, "mode")
                    }
                }

                if (length(grep("^cdf.dat$", dir.fix)) > 0L) {
                    qq <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "cdf.dat", sep = "")),
                                                debug = debug
                                                )
                    summ <- c(summ, qq[, 2L])
                    if (first.time) {
                        col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "cdf", sep = ""))
                    }
                }

                ## read also kld distance
                kld.fixed <- inla.read.binary.file(paste(file, .Platform$file.sep, "symmetric-kld.dat", sep = ""))[-1L]
                summ <- c(summ, kld.fixed)
                if (first.time) {
                    col.nam <- c(col.nam, "kld")
                }
                summary.fixed <- rbind(summary.fixed, summ)

                ## read the marginals
                xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "marginal-densities.dat", sep = "")),
                                            debug = debug
                                            )
                if (is.null(xx)) {
                    xx <- cbind(c(NA, NA, NA), c(NA, NA, NA))
                }
                colnames(xx) <- c("x", "y")
                marginals.fixed[[i]] <- xx
            } else {
                if (first.time) {
                    col.nam <- c("mean", "sd", "kld")
                }
                summary.fixed <- rbind(summary.fixed, c(NA, NA, NA))
                xx <- cbind(c(NA, NA, NA), c(NA, NA, NA))
                colnames(xx) <- c("x", "y")
                marginals.fixed[[i]] <- xx
            }
        }
        rownames(summary.fixed) <- names.fixed
        colnames(summary.fixed) <- col.nam
        if (length(marginals.fixed) > 0L) {
            names(marginals.fixed) <- names.fixed
        }
    } else {
        if (debug) {
            print("No fixed effects")
        }
        names.fixed <- NULL
        summary.fixed <- NULL
        marginals.fixed <- NULL
    }

    ret <- list(
        names.fixed = names.fixed,
        summary.fixed = as.data.frame(summary.fixed),
        marginals.fixed = marginals.fixed
    )
    return(ret)
}

`inla.collect.lincomb` <- function(results.dir, debug = FALSE, derived = TRUE)
{
    ## rewrite from collect.random
    alldir <- dir(results.dir)
    if (derived) {
        lincomb <- alldir[grep("^lincomb.*derived[.]all", alldir)]
    } else {
        lincomb1 <- alldir[grep("^lincomb.*derived[.]all", alldir)]
        lincomb2 <- alldir[grep("^lincomb", alldir)]
        lincomb <- setdiff(lincomb2, lincomb1)
        if (debug) {
            print(paste("lincomb", lincomb))
        }
    }
    n.lincomb <- length(lincomb)
    if (debug) {
        print("collect lincombs")
    }

    ## read the names and model of the lincomb effects
    if (n.lincomb > 0L) {
        names.lincomb <- character(n.lincomb)
        model.lincomb <- inla.trim(character(n.lincomb))

        summary.lincomb <- list()
        summary.lincomb[[n.lincomb]] <- NA
        marginals.lincomb <- list()
        marginals.lincomb[[n.lincomb]] <- NA
        size.lincomb <- list()
        size.lincomb[[n.lincomb]] <- NA

        for (i in 1L:n.lincomb) {
            if (debug) {
                print(paste("read lincomb ", i, " of ", n.lincomb))
            }

            ## read the summary
            file <- paste(results.dir, .Platform$file.sep, lincomb[i], sep = "")
            dir.lincomb <- dir(file)

            if (debug) {
                print(paste("read from dir ", file))
            }

            if (length(dir.lincomb) > 4L) {
                dd <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "summary.dat", sep = "")),
                             ncol = 3L, byrow = TRUE
                             )
                col.nam <- c("ID", "mean", "sd")

                ## read quantiles if existing
                if (debug) {
                    cat("...quantiles.dat if any\n")
                }
                if (length(grep("^quantiles.dat$", dir.lincomb)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep,
                                                                            "quantiles.dat",
                                                                            sep = ""
                                                                            )), debug = debug)
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
                    dd <- cbind(dd, t(qq))
                }

                ## read mode if existing
                if (length(grep("^mode.dat$", dir.lincomb)) > 0L) {
                    mm <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "mode.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(mm)[2L]
                    qq <- mm[, seq(2L, len, by = 2L), drop = FALSE]
                    dd <- cbind(dd, t(qq))
                    col.nam <- c(col.nam, "mode")
                }

                ## read cdf if existing
                if (debug) {
                    cat("...cdf.dat if any\n")
                }
                if (length(grep("^cdf.dat$", dir.lincomb)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "cdf.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
                    dd <- cbind(dd, t(qq))
                }

                if (debug) {
                    cat("...NAMES if any\n")
                }
                if (length(grep("^NAMES$", dir.lincomb)) == 1L) {
                    row.names <- readLines(paste(file, .Platform$file.sep, "NAMES", sep = ""))
                    ## remove the prefix 'lincomb.' as we do not need it in the names.
                    row.names <- sapply(row.names, function(x) gsub("^lincomb[.]", "", x))
                    names(row.names) <- NULL
                } else {
                    row.names <- NULL
                }

                ## read kld
                if (debug) {
                    cat("...kld\n")
                }
                kld1 <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "symmetric-kld.dat", sep = "")),
                               ncol = 2L, byrow = TRUE
                               )
                qq <- kld1[, 2L, drop = FALSE]
                dd <- cbind(dd, qq)
                if (debug) {
                    cat("...kld done\n")
                }

                col.nam <- c(col.nam, "kld")
                colnames(dd) <- col.nam
                summary.lincomb[[i]] <- as.data.frame(dd)
                if (!is.null(row.names)) {
                    rownames(summary.lincomb[[i]]) <- row.names
                }

                xx <- inla.read.binary.file(paste(file, .Platform$file.sep, "marginal-densities.dat", sep = ""))
                rr <- inla.interpret.vector.list(xx, debug = debug)
                rm(xx)
                if (!is.null(rr)) {
                    nd <- length(rr)
                    names(rr) <- paste("index.", as.character(1L:nd), sep = "")
                    for (j in 1L:nd) {
                        colnames(rr[[j]]) <- c("x", "y")
                    }
                }
                marginals.lincomb[[i]] <- rr

                if (!is.null(row.names) && (length(marginals.lincomb) > 0L)) {
                    names(marginals.lincomb[[i]]) <- row.names
                }
            } else {
                N.file <- paste(file, .Platform$file.sep, "N", sep = "")
                if (!file.exists(N.file)) {
                    N <- 0L
                } else {
                    N <- scan(file = N.file, what = numeric(0L), quiet = TRUE)
                }
                summary.lincomb[[i]] <- data.frame("mean" = rep(NA, N), "sd" = rep(NA, N), "kld" = rep(NA, N))
                marginals.lincomb <- NULL
            }
            size.lincomb[[i]] <- inla.collect.size(file)
        }
        names(summary.lincomb) <- names.lincomb

        ## could be that marginals.lincomb is a list of lists of NULL
        if (!is.null(marginals.lincomb)) {
            if (all(sapply(marginals.lincomb, is.null))) {
                marginals.lincomb <- NULL
            }
        }

        if (!is.null(marginals.lincomb) && (length(marginals.lincomb) > 0L)) {
            names(marginals.lincomb) <- names.lincomb
        }
    } else {
        if (debug) {
            cat("No lincomb effets\n")
        }
        summary.lincomb <- NULL
        marginals.lincomb <- NULL
        size.lincomb <- NULL
    }

    if (derived) {
        res <- list(
            summary.lincomb.derived = as.data.frame(summary.lincomb[[1L]]),
            marginals.lincomb.derived = inla.ifelse(length(marginals.lincomb) > 0L, marginals.lincomb[[1L]], NULL),
            size.lincomb.derived = size.lincomb[[1L]]
        )
    } else {
        res <- list(
            summary.lincomb = as.data.frame(summary.lincomb[[1L]]),
            marginals.lincomb = inla.ifelse(length(marginals.lincomb) > 0L, marginals.lincomb[[1L]], NULL),
            size.lincomb = size.lincomb[[1L]]
        )
    }
    return(res)
}

`inla.collect.cpo` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    if (length(grep("^cpo$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect cpo\n", sep = ""))
        }

        xx <- inla.read.binary.file(file = paste(results.dir, .Platform$file.sep, "cpo", .Platform$file.sep, "cpo.dat", sep = ""))
        n <- xx[1L]
        xx <- xx[-1L]
        len <- length(xx)
        cpo.res <- numeric(n)
        cpo.res[1L:n] <- NA
        cpo.res[xx[seq(1L, len, by = 2L)] + 1L] <- xx[seq(2L, len, by = 2L)]

        xx <- inla.read.binary.file(file = paste(results.dir, .Platform$file.sep, "cpo", .Platform$file.sep, "pit.dat", sep = ""))
        n <- xx[1L]
        xx <- xx[-1L]
        len <- length(xx)
        pit.res <- numeric(n)
        pit.res[1L:n] <- NA
        pit.res[xx[seq(1L, len, by = 2L)] + 1L] <- xx[seq(2L, len, by = 2L)]

        fnm <- paste(results.dir, .Platform$file.sep, "cpo", .Platform$file.sep, "failure.dat", sep = "")
        if (file.exists(fnm)) {
            xx <- inla.read.binary.file(fnm)
            n <- xx[1L]
            xx <- xx[-1L]
            len <- length(xx)
            failure.res <- numeric(n)
            failure.res[1L:n] <- NA
            failure.res[xx[seq(1L, len, by = 2L)] + 1L] <- xx[seq(2L, len, by = 2L)]
        }
        else {
            failure.res <- NULL
        }
        rm(xx)
    } else {
        cpo.res <- NULL
        pit.res <- NULL
        failure.res <- NULL
    }

    ## want NA not NaN
    cpo.res[is.nan(cpo.res)] <- NA
    pit.res[is.nan(pit.res)] <- NA
    failure.res[is.nan(failure.res)] <- NA

    return(list(cpo = cpo.res, pit = pit.res, failure = failure.res))
}

`inla.collect.po` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    if (length(grep("^po$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect po\n", sep = ""))
        }

        xx <- inla.read.binary.file(file = paste(results.dir, .Platform$file.sep, "po", .Platform$file.sep, "po.dat", sep = ""))
        n <- xx[1L]
        xx <- xx[-1L]
        xx <- xx[-seq(3, length(xx), by = 3L)] ## skip entry 3, 6, 9, ...
        len <- length(xx)
        po.res <- numeric(n)
        po.res[1L:n] <- NA
        po.res[xx[seq(1L, len, by = 2L)] + 1L] <- xx[seq(2L, len, by = 2L)]
    } else {
        po.res <- NULL
    }

    ## want NA not NaN
    po.res[is.nan(po.res)] <- NA

    return(list(po = po.res))
}

`inla.collect.waic` <- function(results.dir, debug = FALSE)
{
    ## yes, here we use the po-results!!!!
    alldir <- dir(results.dir)
    if (length(grep("^po$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect waic from po-results\n", sep = ""))
        }

        xx <- inla.read.binary.file(file = paste(results.dir, .Platform$file.sep, "po", .Platform$file.sep, "po.dat", sep = ""))
        n <- xx[1L]
        xx <- xx[-1L]
        len <- length(xx)
        po.res <- numeric(n)
        po2.res <- numeric(n)
        po.res[1L:n] <- NA
        po.res[xx[seq(1L, len, by = 3L)] + 1L] <- xx[seq(2L, len, by = 3L)]
        po2.res[1L:n] <- NA
        po2.res[xx[seq(1L, len, by = 3L)] + 1L] <- xx[seq(3L, len, by = 3L)]

        ## want NA not NaN
        po.res[is.nan(po.res)] <- NA
        po2.res[is.nan(po2.res)] <- NA

        ## compute waic
        return(list(
            waic = -2 * (sum(log(po.res), na.rm = TRUE) - sum(po2.res, na.rm = TRUE)),
            p.eff = sum(po2.res, na.rm = TRUE),
            local.waic = -2 * (log(po.res) - po2.res),
            local.p.eff = po2.res
        ))
    } else {
        return(NULL)
    }
}

`inla.collect.dic` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    ## get dic (if exists)
    if (length(grep("^dic$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect dic\n", sep = ""))
        }
        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "dic.dat", sep = "")
        dic.values <- inla.read.binary.file(file)

        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "deviance_e.dat", sep = "")
        if (inla.is.fmesher.file(file)) {
            dev.e <- c(inla.read.fmesher.file(file))
            dev.e[is.nan(dev.e)] <- NA
        } else {
            dev.e <- NULL
        }

        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "deviance_e_sat.dat", sep = "")
        if (inla.is.fmesher.file(file)) {
            dev.e.sat <- c(inla.read.fmesher.file(file))
            dev.e.sat[is.nan(dev.e.sat)] <- NA
        } else {
            dev.e.sat <- NULL
        }

        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "e_deviance.dat", sep = "")
        if (inla.is.fmesher.file(file)) {
            e.dev <- c(inla.read.fmesher.file(file))
            e.dev[is.nan(e.dev)] <- NA
        } else {
            e.dev <- NULL
        }

        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "e_deviance_sat.dat", sep = "")
        if (inla.is.fmesher.file(file)) {
            e.dev.sat <- c(inla.read.fmesher.file(file))
            e.dev.sat[is.nan(e.dev.sat)] <- NA
        } else {
            e.dev.sat <- NULL
        }

        f.idx <- NULL
        file <- paste(results.dir, .Platform$file.sep, "dic", .Platform$file.sep, "family_idx.dat", sep = "")
        if (inla.is.fmesher.file(file)) {
            f.idx <- c(inla.read.fmesher.file(file)) + 1L ## convert to R-indexing
            f.idx[is.nan(f.idx)] <- NA
        }

        ## if there there is no data at all, then all dic'values are
        ## NA. the returned values are 0, so we override them here.
        if (!is.null(f.idx) && all(is.na(f.idx))) {
            dic.values[] <- NA
        }

        local.dic <- 2.0 * e.dev - dev.e
        local.dic.sat <- 2.0 * e.dev.sat - dev.e.sat
        local.p.eff <- e.dev - dev.e
        fam.dic <- dic.values[4L]
        fam.p.eff <- dic.values[3L]

        if (!is.null(f.idx) && !all(is.na(f.idx))) {
            n.fam <- max(f.idx, na.rm = TRUE)
            fam.dic <- numeric(n.fam)
            fam.dic.sat <- numeric(n.fam)
            fam.p.eff <- numeric(n.fam)
            for (i in 1:n.fam) {
                idx <- which(f.idx == i)
                fam.dic[i] <- sum(local.dic[idx])
                fam.dic.sat[i] <- sum(local.dic.sat[idx])
                fam.p.eff[i] <- sum(local.p.eff[idx])
            }
        }

        dic <- list(
            "dic" = dic.values[4L],
            "p.eff" = dic.values[3L],
            "mean.deviance" = dic.values[1L],
            "deviance.mean" = dic.values[2L],
            "dic.sat" = dic.values[4L + 4L],
            "mean.deviance.sat" = dic.values[4L + 1L],
            "deviance.mean.sat" = dic.values[4L + 2L],
            "family.dic" = fam.dic,
            "family.dic.sat" = fam.dic.sat,
            "family.p.eff" = fam.p.eff,
            "family" = f.idx,
            "local.dic" = local.dic,
            "local.dic.sat" = local.dic.sat,
            "local.p.eff" = local.p.eff
        )
    } else {
        dic <- NULL
    }
    return(dic)
}

`inla.collect.q` <- function(results.dir, debug = FALSE)
{
    my.read.pnm <- function(...) {
        args <- list(...)
        filename <- args[[1]]
        inla.require("pixmap", stop.on.error = TRUE)
        if (file.exists(filename)) {
            ## disable warnings
            warn <- getOption("warn")
            options(warn = -1L) ## disable...
            ret <- pixmap::read.pnm(...)
            do.call("options", args = list(warn = warn))
        } else {
            ret <- NULL
        }
        return(ret)
    }

    alldir <- dir(results.dir)
    if (length(grep("^Q$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect q\n", sep = ""))
        }

        file <- paste(results.dir, .Platform$file.sep, "Q/precision-matrix.pbm", sep = "")
        Q.matrix <- my.read.pnm(file)

        file <- paste(results.dir, .Platform$file.sep, "Q/precision-matrix-reordered.pbm", sep = "")
        Q.matrix.reorder <- my.read.pnm(file)

        file <- paste(results.dir, .Platform$file.sep, "Q/precision-matrix_L.pbm", sep = "")
        L <- my.read.pnm(file)

        if (is.null(Q.matrix) && is.null(Q.matrix.reorder) && is.null(L)) {
            q <- NULL
        } else {
            q <- list(Q = Q.matrix, Q.reorder = Q.matrix.reorder, L = L)
        }
    } else {
        q <- NULL
    }

    return(q)
}

`inla.collect.graph` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    if (length(grep("^graph.dat$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect graph\n", sep = ""))
        }
        file <- paste(results.dir, .Platform$file.sep, "graph.dat", sep = "")
        g <- inla.read.graph(file)
    } else {
        g <- NULL
    }

    return(list(graph = g))
}

`inla.collect.hyperpar` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    all.hyper <- alldir[grep("^hyperparameter", alldir)]
    hyper <- all.hyper[grep("user-scale$", all.hyper)]
    n.hyper <- length(hyper)
    if (n.hyper > 0L) {
        ## get names for hyperpar
        names.hyper <- character(n.hyper)
        for (i in 1L:n.hyper) {
            tag <- paste(results.dir, .Platform$file.sep, hyper[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.hyper[i] <- "missing NAME"
            } else {
                names.hyper[i] <- readLines(tag, n = 1L)
            }
        }

        ## get summary and marginals
        summary.hyper <- numeric()
        marginal.hyper <- list()
        marginal.hyper[[n.hyper]] <- NA

        for (i in 1L:n.hyper) {
            first.time <- (i == 1L)
            dir.hyper <- paste(results.dir, .Platform$file.sep, hyper[i], sep = "")
            file <- paste(dir.hyper, .Platform$file.sep, "summary.dat", sep = "")
            hyperid <- inla.collect.hyperid(dir.hyper)
            dd <- inla.read.binary.file(file)[-1L]
            summ <- dd
            if (first.time) {
                col.nam <- c("mean", "sd")
            }
            if (length(grep("^quantiles.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "quantiles.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "quant", sep = ""))
                }
            }
            if (length(grep("^mode.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "mode.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, "mode")
                }
            }

            if (length(grep("^cdf.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "cdf.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "cdf", sep = ""))
                }
            }
            summary.hyper <- rbind(summary.hyper, summ)
            file <- paste(results.dir, .Platform$file.sep, hyper[i], .Platform$file.sep, "marginal-densities.dat", sep = "")
            xx <- inla.read.binary.file(file)
            marg1 <- inla.interpret.vector(xx, debug = debug)
            attr(marg1, "hyperid") <- hyperid
            rm(xx)
            if (!is.null(marg1)) {
                colnames(marg1) <- c("x", "y")
            }
            marginal.hyper[[i]] <- marg1
        }
        names(marginal.hyper) <- names.hyper
        rownames(summary.hyper) <- names.hyper
        colnames(summary.hyper) <- col.nam
    } else {
        marginal.hyper <- NULL
        summary.hyper <- NULL
    }

    ## collect also the hyperparameters in the internal scale
    all.hyper <- alldir[grep("^hyperparameter", alldir)]
    hyper <- all.hyper[-grep("user-scale$", all.hyper)]
    n.hyper <- length(hyper)
    if (n.hyper > 0L) {
        ## get names for hyperpar
        names.hyper <- character(n.hyper)
        for (i in 1L:n.hyper) {
            tag <- paste(results.dir, .Platform$file.sep, hyper[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.hyper[i] <- "missing NAME"
            } else {
                names.hyper[i] <- readLines(tag, n = 1L)
            }
        }

        ## get summary and marginals
        internal.summary.hyper <- numeric()
        internal.marginal.hyper <- list()
        internal.marginal.hyper[[n.hyper]] <- NA
        for (i in 1L:n.hyper) {
            first.time <- (i == 1L)
            dir.hyper <- paste(results.dir, .Platform$file.sep, hyper[i], sep = "")
            file <- paste(dir.hyper, .Platform$file.sep, "summary.dat", sep = "")
            hyperid <- inla.collect.hyperid(dir.hyper)
            dd <- inla.read.binary.file(file)[-1L]
            summ <- dd
            if (first.time) {
                col.nam <- c("mean", "sd")
            }
            if (length(grep("^quantiles.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "quantiles.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "quant", sep = ""))
                }
            }
            if (length(grep("^mode.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "mode.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, "mode")
                }
            }
            if (length(grep("^cdf.dat$", dir(dir.hyper))) > 0L) {
                qq <- inla.interpret.vector(inla.read.binary.file(paste(dir.hyper, .Platform$file.sep, "cdf.dat", sep = "")),
                                            debug = debug
                                            )
                summ <- c(summ, qq[, 2L])
                if (first.time) {
                    col.nam <- c(col.nam, paste(as.character(qq[, 1L]), "cdf", sep = ""))
                }
            }
            if (first.time) {
                internal.summary.hyper <- matrix(NA, n.hyper, length(summ))
            }
            internal.summary.hyper[i, ] <- summ
            file <- paste(results.dir, .Platform$file.sep, hyper[i], .Platform$file.sep, "marginal-densities.dat", sep = "")
            xx <- inla.read.binary.file(file)
            marg1 <- inla.interpret.vector(xx, debug = debug)
            attr(marg1, "hyperid") <- hyperid
            rm(xx)
            if (!is.null(marg1)) {
                colnames(marg1) <- c("x", "y")
            }

            internal.marginal.hyper[[i]] <- marg1
        }
        names(internal.marginal.hyper) <- names.hyper
        rownames(internal.summary.hyper) <- names.hyper
        colnames(internal.summary.hyper) <- col.nam
    } else {
        internal.summary.hyper <- NULL
        internal.marginal.hyper <- NULL
    }

    ret <- list(
        summary.hyperpar = as.data.frame(summary.hyper),
        marginals.hyperpar = marginal.hyper,
        internal.summary.hyperpar = as.data.frame(internal.summary.hyper),
        internal.marginals.hyperpar = internal.marginal.hyper
    )
    return(ret)
}

`inla.collect.mlik` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)
    if (length(grep("^marginal-likelihood$", alldir)) == 1L) {
        if (debug) {
            cat(paste("collect mlik\n", sep = ""))
        }
        file <- paste(results.dir, .Platform$file.sep, "marginal-likelihood",
                      .Platform$file.sep, "marginal-likelihood.dat",
                      sep = ""
                      )
        mlik.res <- matrix(inla.read.binary.file(file), 2L, 1L)
        rownames(mlik.res) <- c(
            "log marginal-likelihood (integration)",
            "log marginal-likelihood (Gaussian)"
        )
    }
    else {
        mlik.res <- NULL
    }

    return(list(mlik = mlik.res))
}

`inla.collect.predictor` <- function(results.dir, debug = FALSE)
{
    alldir <- dir(results.dir)

    ## FIRST: get the linear predictor
    subdir <- paste(results.dir, .Platform$file.sep, "predictor", sep = "")

    if (length(dir(subdir)) > 3L) {
        if (debug) {
            cat(paste("collect linear predictor\n", sep = ""))
        }

        if (debug) {
            cat("...read summary.dat\n")
        }
        file <- paste(subdir, .Platform$file.sep, "summary.dat", sep = "")
        dd <- matrix(inla.read.binary.file(file = file), ncol = 3L, byrow = TRUE)[, -1L, drop = FALSE]
        col.nam <- c("mean", "sd")

        ## info about size
        size.info <- inla.collect.size(subdir)
        if (!is.null(size.info)) {
            A <- (size.info$nrep == 2)
            n <- size.info$n
            nA <- size.info$Ntotal - size.info$n
        } else {
            ## should not happen
            stop("This should not happen")
        }

        ## get quantiles if computed
        if (length(grep("^quantiles.dat$", dir(subdir))) == 1L) {
            if (debug) {
                cat("...read quantiles.dat\n")
            }
            file <- paste(subdir, .Platform$file.sep, "quantiles.dat", sep = "")
            xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
            len <- dim(xx)[2L]
            qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
            col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
            dd <- cbind(dd, t(qq))
            rm(xx)
        }

        if (length(grep("^mode.dat$", dir(subdir))) == 1L) {
            if (debug) {
                cat("...read mode.dat\n")
            }
            file <- paste(subdir, .Platform$file.sep, "mode.dat", sep = "")
            xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
            len <- dim(xx)[2L]
            qq <- xx[, seq(2L, len, by = 2L)]
            col.nam <- c(col.nam, "mode")
            dd <- cbind(dd, qq)
            rm(xx)
        }

        ## get cdf if computed
        if (length(grep("^cdf.dat$", dir(subdir))) == 1L) {
            if (debug) {
                cat("...read cdf.dat\n")
            }
            file <- paste(subdir, .Platform$file.sep, "cdf.dat", sep = "")
            xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
            len <- dim(xx)[2L]
            qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
            col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
            dd <- cbind(dd, t(qq))
            rm(xx)
        } else {
            if (debug) {
                cat("... no cdf.dat\n")
            }
        }

        ## get kld
        if (debug) {
            cat("...read kld\n")
        }
        kld <- matrix(inla.read.binary.file(file = paste(subdir, .Platform$file.sep, "symmetric-kld.dat", sep = "")),
                      ncol = 2L, byrow = TRUE
                      )
        dd <- cbind(dd, kld[, 2L, drop = FALSE])
        col.nam <- c(col.nam, "kld")
        colnames(dd) <- col.nam
        summary.linear.predictor <- as.data.frame(dd)

        if (A) {
            rownames(summary.linear.predictor) <- c(
                paste("APredictor.", inla.num(1L:nA), sep = ""),
                paste("Predictor.", inla.num(1:n), sep = "")
            )
        } else {
            rownames(summary.linear.predictor) <- paste("Predictor.", inla.num(1L:size.info$Ntotal), sep = "")
        }

        if (debug) {
            cat("...read marginal-densities.dat\n")
        }
        file <- paste(subdir, .Platform$file.sep, "marginal-densities.dat", sep = "")
        xx <- inla.read.binary.file(file)
        rr <- inla.interpret.vector.list(xx, debug = debug)
        rm(xx)
        if (!is.null(rr)) {
            if (A) {
                names(rr) <- c(
                    paste("APredictor.", inla.num(1L:nA), sep = ""),
                    paste("Predictor.", inla.num(1L:n), sep = "")
                )
            } else {
                names(rr) <- paste("Predictor.", as.character(1L:length(rr)), sep = "")
            }
            names.rr <- names(rr)
            for (i in 1L:length(rr)) {
                colnames(rr[[i]]) <- c("x", "y")
            }
        }
        marginals.linear.predictor <- rr
    } else {
        summary.linear.predictor <- NULL
        marginals.linear.predictor <- NULL
        size.info <- NULL
    }

    ## SECOND: get the inverse linear predictor(if computed)
    if (length(grep("^predictor-user-scale$", alldir)) == 1L) {
        subdir <- paste(results.dir, .Platform$file.sep, "predictor-user-scale", sep = "")
        if (length(dir(subdir)) > 3L) {
            if (debug) {
                cat(paste("collect fitted values\n", sep = ""))
            }

            file <- paste(subdir, .Platform$file.sep, "summary.dat", sep = "")
            dd <- matrix(inla.read.binary.file(file = file), ncol = 3L, byrow = TRUE)[, -1L, drop = FALSE]
            col.nam <- c("mean", "sd")

            ## get quantiles if computed
            if (length(grep("^quantiles.dat$", dir(subdir))) == 1L) {
                file <- paste(subdir, .Platform$file.sep, "quantiles.dat", sep = "")
                xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
                len <- dim(xx)[2L]
                qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
                dd <- cbind(dd, t(qq))
                rm(xx)
            }
            if (length(grep("^mode.dat$", dir(subdir))) == 1L) {
                file <- paste(subdir, .Platform$file.sep, "mode.dat", sep = "")
                xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
                len <- dim(xx)[2L]
                qq <- xx[, seq(2L, len, by = 2L)]
                col.nam <- c(col.nam, "mode")
                dd <- cbind(dd, qq)
                rm(xx)
            }

            ## get cdf if computed
            if (length(grep("^cdf.dat$", dir(subdir))) == 1L) {
                file <- paste(subdir, .Platform$file.sep, "cdf.dat", sep = "")
                xx <- inla.interpret.vector(inla.read.binary.file(file), debug = debug)
                len <- dim(xx)[2L]
                qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
                dd <- cbind(dd, t(qq))
                rm(xx)
            }

            colnames(dd) <- col.nam
            if (A) {
                rownames(dd) <- c(
                    paste("fitted.APredictor.", inla.num(1L:nA), sep = ""),
                    paste("fitted.Predictor.", inla.num(1L:n), sep = "")
                )
            } else {
                rownames(dd) <- paste("fitted.Predictor.", inla.num(1L:n), sep = "")
            }
            summary.fitted.values <- as.data.frame(dd)

            file <- paste(subdir, .Platform$file.sep, "marginal-densities.dat", sep = "")
            xx <- inla.read.binary.file(file)
            rr <- inla.interpret.vector.list(xx, debug = debug)
            rm(xx)
            if (!is.null(rr)) {
                if (A) {
                    names(rr) <- c(
                        paste("fitted.APredictor.", inla.num(1L:nA), sep = ""),
                        paste("fitted.Predictor.", inla.num(1:n), sep = "")
                        )
                } else {
                    names(rr) <- paste("fitted.Predictor.", inla.num(1L:length(rr)), sep = "")
                }
                names.rr <- names(rr)
                for (i in 1L:length(rr)) {
                    colnames(rr[[i]]) <- c("x", "y")
                    }
            }
            marginals.fitted.values <- rr
        }
    } else {
        summary.fitted.values <- NULL
        marginals.fitted.values <- NULL
    }


    res <- list(
        summary.linear.predictor = as.data.frame(summary.linear.predictor),
        marginals.linear.predictor = marginals.linear.predictor,
        summary.fitted.values = as.data.frame(summary.fitted.values),
        marginals.fitted.values = marginals.fitted.values,
        size.linear.predictor = size.info
    )

    return(res)
}

`inla.collect.random` <- function(results.dir, debug = FALSE) 
{
    alldir <- dir(results.dir)
    random <- alldir[grep("^random.effect", alldir)]
    n.random <- length(random)
    if (debug) {
        print("collect random effects")
    }

    ## read the names and model of the random effects
    if (n.random > 0L) {
        names.random <- character(n.random)
        model.random <- inla.trim(character(n.random))
        for (i in 1L:n.random) {
            tag <- paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.random[i] <- "missing NAME"
            } else {
                names.random[i] <- readLines(tag, n = 1L)
            }
            modelname <- inla.trim(paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "MODEL", sep = ""))
            if (!file.exists(modelname)) {
                model.random[i] <- "NoModelName"
            } else {
                model.random[i] <- inla.trim(readLines(modelname, n = 1L))
            }
        }

        summary.random <- list()
        summary.random[[n.random]] <- NA
        size.random <- list()
        size.random[[n.random]] <- NA

        marginals.random <- list()
        marginals.random[[n.random]] <- NA

        for (i in 1L:n.random) {
            if (debug) {
                print(paste("read random ", i, " of ", n.random))
            }
            ## read the summary
            file <- paste(results.dir, .Platform$file.sep, random[i], sep = "")
            dir.random <- dir(file)

            if (length(dir.random) > 5L) {
                dd <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "summary.dat", sep = "")), ncol = 3L, byrow = TRUE)
                col.nam <- c("ID", "mean", "sd")
                ## read quantiles if existing
                if (debug) {
                    cat("...quantiles.dat if any\n")
                }
                if (length(grep("^quantiles.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "quantiles.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
                    dd <- cbind(dd, t(qq))
                }
                if (length(grep("^mode.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "mode.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L)]
                    col.nam <- c(col.nam, "mode")
                    dd <- cbind(dd, qq)
                }

                ## read cdf if existing
                if (debug) {
                    cat("...cdf.dat if any\n")
                }
                if (length(grep("^cdf.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "cdf.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
                    dd <- cbind(dd, t(qq))
                }

                ## read kld
                if (debug) {
                    cat("...kld\n")
                }
                kld1 <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "symmetric-kld.dat", sep = "")),
                               ncol = 2L, byrow = TRUE
                               )
                qq <- kld1[, 2L, drop = FALSE]
                dd <- cbind(dd, qq)
                if (debug) {
                    cat("...kld done\n")
                }


                col.nam <- c(col.nam, "kld")
                colnames(dd) <- col.nam
                summary.random[[i]] <- as.data.frame(dd)

                xx <- inla.read.binary.file(paste(file, .Platform$file.sep, "marginal-densities.dat", sep = ""))
                rr <- inla.interpret.vector.list(xx, debug = debug)
                rm(xx)
                if (!is.null(rr)) {
                    nd <- length(rr)
                    names(rr) <- paste("index.", as.character(1L:nd), sep = "")
                    names.rr <- names(rr)
                    for (j in 1L:nd) {
                        colnames(rr[[j]]) <- c("x", "y")
                        }
                }
                marginals.random[[i]] <- if (is.null(rr)) NA else rr

                ## if id.names are present,  override the default names
                id.names <- inla.readLines(paste(file, .Platform$file.sep, "id-names.dat", sep = ""))
                if (!is.null(id.names)) {
                    len.id.names <- length(id.names)
                    summary.random[[i]]$ID[1L:len.id.names] <- id.names
                    if (length(marginals.random) >= i && !is.na(marginals.random[[i]])) {
                        names(marginals.random[[i]][1L:len.id.names]) <- id.names
                    }
                }
            } else {
                N.file <- paste(file, .Platform$file.sep, "N", sep = "")
                if (!file.exists(N.file)) {
                    N <- 0L
                } else {
                    N <- scan(file = N.file, what = numeric(0L), quiet = TRUE)
                }
                summary.random[[i]] <- data.frame("mean" = rep(NA, N), "sd" = rep(NA, N), "kld" = rep(NA, N))
                marginals.random <- NULL
            }

            size.random[[i]] <- inla.collect.size(file)
        }
        names(summary.random) <- names.random

        ## could be that marginals.random is a list of lists of NULL or NA
        if (!is.null(marginals.random)) {
            if (all(sapply(marginals.random, function(x) (is.null(x) || is.na(x))))) {
                marginals.random <- NULL
            }
        }

        if (!is.null(marginals.random) && (length(marginals.random) > 0L)) {
            names(marginals.random) <- names.random
        }
    } else {
        if (debug) {
            cat("No random effets\n")
        }
        model.random <- NULL
        summary.random <- NULL
        marginals.random <- NULL
        size.random <- NULL
    }

    res <- list(
        model.random = model.random,
        summary.random = lapply(summary.random, as.data.frame),
        marginals.random = marginals.random,
        size.random = size.random
    )
    return(res)
}

`inla.collect.spde2.blc` <- function(results.dir, debug = FALSE) 
{
    ## a copy from collect.random
    alldir <- dir(results.dir)
    random <- alldir[grep("^spde2.blc", alldir)]
    n.random <- length(random)
    if (debug) {
        print("collect random effects")
    }

    ## read the names and model of the random effects
    if (n.random > 0L) {
        names.random <- character(n.random)
        model.random <- inla.trim(character(n.random))
        for (i in 1L:n.random) {
            tag <- paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.random[i] <- "missing NAME"
            } else {
                names.random[i] <- readLines(tag, n = 1L)
            }
            modelname <- inla.trim(paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "MODEL", sep = ""))
            if (!file.exists(modelname)) {
                model.random[i] <- "NoModelName"
            } else {
                model.random[i] <- inla.trim(readLines(modelname, n = 1L))
            }
        }


        summary.random <- list()
        summary.random[[n.random]] <- NA
        size.random <- list()
        size.random[[n.random]] <- NA

        marginals.random <- list()
        marginals.random[[n.random]] <- NA

        for (i in 1L:n.random) {
            if (debug) {
                print(paste("read random ", i, " of ", n.random))
            }
            ## read the summary
            file <- paste(results.dir, .Platform$file.sep, random[i], sep = "")
            dir.random <- dir(file)

            if (length(dir.random) > 4L) {
                dd <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "summary.dat", sep = "")), ncol = 3L, byrow = TRUE)
                col.nam <- c("ID", "mean", "sd")
                ## read quantiles if existing
                if (debug) {
                    cat("...quantiles.dat if any\n")
                }
                if (length(grep("^quantiles.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "quantiles.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
                    dd <- cbind(dd, t(qq))
                }
                if (length(grep("^mode.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "mode.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, "mode")
                    dd <- cbind(dd, t(qq))
                }

                ## read cdf if existing
                if (debug) {
                    cat("...cdf.dat if any\n")
                }
                if (length(grep("^cdf.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "cdf.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
                    dd <- cbind(dd, t(qq))
                }

                ## read kld
                if (debug) {
                    cat("...kld\n")
                }
                kld1 <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "symmetric-kld.dat", sep = "")),
                               ncol = 2L, byrow = TRUE
                               )
                qq <- kld1[, 2L, drop = FALSE]
                dd <- cbind(dd, qq)
                if (debug) {
                    cat("...kld done\n")
                }


                col.nam <- c(col.nam, "kld")
                colnames(dd) <- col.nam
                summary.random[[i]] <- as.data.frame(dd)

                xx <- inla.read.binary.file(paste(file, .Platform$file.sep, "marginal-densities.dat", sep = ""))
                rr <- inla.interpret.vector.list(xx, debug = debug)
                rm(xx)
                if (!is.null(rr)) {
                    nd <- length(rr)
                    names(rr) <- paste("index.", as.character(1L:nd), sep = "")
                    names.rr <- names(rr)
                    for (j in 1L:nd) {
                        colnames(rr[[j]]) <- c("x", "y")
                        }
                }
                marginals.random[[i]] <- rr
            } else {
                N.file <- paste(file, .Platform$file.sep, "N", sep = "")
                if (!file.exists(N.file)) {
                    N <- 0L
                } else {
                    N <- scan(file = N.file, what = numeric(0L), quiet = TRUE)
                }
                summary.random[[i]] <- data.frame("mean" = rep(NA, N), "sd" = rep(NA, N), "kld" = rep(NA, N))
                marginals.random <- NULL
            }

            size.random[[i]] <- inla.collect.size(file)
        }
        names(summary.random) <- names.random

        ## could be that marginals.random is a list of lists of NULL or NA
        if (!is.null(marginals.random)) {
            if (all(sapply(marginals.random, function(x) (is.null(x) || is.na(x))))) {
                marginals.random <- NULL
            }
        }

        if (!is.null(marginals.random) && (length(marginals.random) > 0L)) {
            names(marginals.random) <- names.random
        }
    } else {
        if (debug) {
            cat("No random effets\n")
        }
        model.random <- NULL
        summary.random <- NULL
        marginals.random <- NULL
        size.random <- NULL
    }

    res <- list(
        model.spde2.blc = model.random,
        summary.spde2.blc = lapply(summary.random, as.data.frame),
        marginals.spde2.blc = marginals.random,
        size.spde2.blc = size.random
    )
    return(res)
}

`inla.collect.spde3.blc` <- function(results.dir, debug = FALSE)
{
    ## a copy from collect.random
    alldir <- dir(results.dir)
    random <- alldir[grep("^spde3.blc", alldir)]
    n.random <- length(random)
    if (debug) {
        print("collect random effects")
    }

    ## read the names and model of the random effects
    if (n.random > 0L) {
        names.random <- character(n.random)
        model.random <- inla.trim(character(n.random))
        for (i in 1L:n.random) {
            tag <- paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "TAG", sep = "")
            if (!file.exists(tag)) {
                names.random[i] <- "missing NAME"
            } else {
                names.random[i] <- readLines(tag, n = 1L)
            }
            modelname <- inla.trim(paste(results.dir, .Platform$file.sep, random[i], .Platform$file.sep, "MODEL", sep = ""))
            if (!file.exists(modelname)) {
                model.random[i] <- "NoModelName"
            } else {
                model.random[i] <- inla.trim(readLines(modelname, n = 1L))
            }
        }


        summary.random <- list()
        summary.random[[n.random]] <- NA
        size.random <- list()
        size.random[[n.random]] <- NA

        marginals.random <- list()
        marginals.random[[n.random]] <- NA

        for (i in 1L:n.random) {
            if (debug) {
                print(paste("read random ", i, " of ", n.random))
            }
            ## read the summary
            file <- paste(results.dir, .Platform$file.sep, random[i], sep = "")
            dir.random <- dir(file)

            if (length(dir.random) > 4L) {
                dd <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "summary.dat", sep = "")), ncol = 3L, byrow = TRUE)
                col.nam <- c("ID", "mean", "sd")
                ## read quantiles if existing
                if (debug) {
                    cat("...quantiles.dat if any\n")
                }
                if (length(grep("^quantiles.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "quantiles.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), "quant", sep = ""))
                    dd <- cbind(dd, t(qq))
                }
                if (length(grep("^mode.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "mode.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, "mode")
                    dd <- cbind(dd, t(qq))
                }

                ## read cdf if existing
                if (debug) {
                    cat("...cdf.dat if any\n")
                }
                if (length(grep("^cdf.dat$", dir.random)) == 1L) {
                    xx <- inla.interpret.vector(inla.read.binary.file(paste(file, .Platform$file.sep, "cdf.dat", sep = "")),
                                                debug = debug
                                                )
                    len <- dim(xx)[2L]
                    qq <- xx[, seq(2L, len, by = 2L), drop = FALSE]
                    col.nam <- c(col.nam, paste(as.character(xx[, 1L]), " cdf", sep = ""))
                    dd <- cbind(dd, t(qq))
                }

                ## read kld
                if (debug) {
                    cat("...kld\n")
                }
                kld1 <- matrix(inla.read.binary.file(file = paste(file, .Platform$file.sep, "symmetric-kld.dat", sep = "")),
                               ncol = 2L, byrow = TRUE
                               )
                qq <- kld1[, 2L, drop = FALSE]
                dd <- cbind(dd, qq)
                if (debug) {
                    cat("...kld done\n")
                }

                col.nam <- c(col.nam, "kld")
                colnames(dd) <- col.nam
                summary.random[[i]] <- as.data.frame(dd)

                xx <- inla.read.binary.file(paste(file, .Platform$file.sep, "marginal-densities.dat", sep = ""))
                rr <- inla.interpret.vector.list(xx, debug = debug)
                rm(xx)
                if (!is.null(rr)) {
                    nd <- length(rr)
                    names(rr) <- paste("index.", as.character(1L:nd), sep = "")
                    names.rr <- names(rr)
                    for (j in 1L:nd) {
                        colnames(rr[[j]]) <- c("x", "y")
                        }
                }
                marginals.random[[i]] <- rr
            } else {
                N.file <- paste(file, .Platform$file.sep, "N", sep = "")
                if (!file.exists(N.file)) {
                    N <- 0L
                } else {
                    N <- scan(file = N.file, what = numeric(0L), quiet = TRUE)
                }
                summary.random[[i]] <- data.frame("mean" = rep(NA, N), "sd" = rep(NA, N), "kld" = rep(NA, N))
                marginals.random <- NULL
            }

            size.random[[i]] <- inla.collect.size(file)
        }
        names(summary.random) <- names.random

        ## could be that marginals.random is a list of lists of NULL or NA
        if (!is.null(marginals.random)) {
            if (all(sapply(marginals.random, function(x) (is.null(x) || is.na(x))))) {
                marginals.random <- NULL
            }
        }

        if (!is.null(marginals.random) && (length(marginals.random) > 0L)) {
            names(marginals.random) <- names.random
        }
    } else {
        if (debug) {
            cat("No random effets\n")
        }
        model.random <- NULL
        summary.random <- NULL
        marginals.random <- NULL
        size.random <- NULL
    }

    res <- list(
        model.spde3.blc = model.random,
        summary.spde3.blc = lapply(summary.random, as.data.frame),
        marginals.spde3.blc = marginals.random,
        size.spde3.blc = size.random
    )
    return(res)
}

`inla.image.reduce` <- function(im, image.dim = 512)
{
    ## reduce image IM to image.dim IMAGE.DIM and return the image as a matrix.
    ## order the indices so the output can be plotted by image()

    if ((class(im) != "pixmapGrey") || (im@size[1L] != im@size[2L])) {
        return(im)
    } else {
        return(im@grey)
    }

    ## do not need this anymore as we do this in GMRFLib.
    if (FALSE) {
        if (image.dim >= im@size[1L]) {
            n <- as.integer(im@size[1L])
            x <- matrix(NA, n, n)
            for (j in 1L:n) {
                x[j, n - (1L:n) + 1L] <- im@grey[1L:n, j]
            }
            return(x)
        }
        block <- ceiling(im@size[1L] / image.dim)
        n <- floor(im@size[1L] / block)
        ii <- jj <- 0L
        x <- matrix(NA, n, n)
        for (i in seq(1L, im@size[1L] - block + 1L, by = block)) {
            ii <- ii + 1L
            jj <- 0L
            for (j in seq(1L, im@size[1L] - block + 1L, by = block)) {
                jj <- jj + 1L
                x[jj, n - ii + 1L] <- min(im@grey[i:(i + block - 1L), j:(j + block - 1L)])
            }
        }
        return(x)
    }
}

`inla.collect.offset.linear.predictor` <- function(results.dir, debug = FALSE)
{
    filename <- paste(results.dir, "/totaloffset/totaloffset.dat", sep = "")
    stopifnot(file.exists(filename))

    xx <- inla.read.binary.file(filename)
    return(list(offset.linear.predictor = xx))
}

## Export: inla.pc.rsn inla.pc.dsn inla.pc.qsn inla.pc.psn

## ! \name{pc.sn}
## ! \alias{inla.pc.sn}
## ! \alias{pc.sn}
## ! \alias{pc.rsn}
## ! \alias{inla.pc.rsn}
## ! \alias{pc.dsn}
## ! \alias{inla.pc.dsn}
## ! \alias{pc.psn}
## ! \alias{inla.pc.psn}
## ! \alias{pc.qsn}
## ! \alias{inla.pc.qsn}
## !
## ! \title{Utility functions for the PC prior for \code{skewness} in the skew-normal
## ! linkfunction and likelihood}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the \code{skewness}
## !              in the skew-normal link-function and likelihood}
## ! \usage{
## ! inla.pc.rsn(n, lambda = 40)
## ! inla.pc.dsn(skew, lambda = 40, log = FALSE)
## ! inla.pc.qsn(p, lambda = 40)
## ! inla.pc.psn(q, lambda = 40)
## ! }
## ! \arguments{
## !   \item{n}{number of observations}
## !   \item{lambda}{the rate parameter in the PC prior}
## !   \item{skew}{vector of evaluation points}
## !   \item{log}{logical. return the density in natural or log-scale.}
## !   \item{p}{vector of probabilities}
## !   \item{q}{vector of quantiles}
## ! }
## ! \details{
## ! Defines the PC prior for the \code{skewness} for the skew-normal linkfunction
## ! and likelihood, where \code{skew=0} is the base model. The skewness range from
## ! -0.99527... to 0.99527....  ca.
## ! }
## !\value{%%
## !  \code{inla.pc.dsn} gives the density,
## !  \code{inla.pc.psn} gives the distribution function,
## !  \code{inla.pc.qsn} gives the quantile function, and
## !  \code{inla.pc.rsn} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.sn")}
## ! \author{havard rue \email{hrue@r-inla.org}}
## ! \examples{
## ! x = inla.pc.rsn(100,  lambda = 40)
## ! d = inla.pc.dsn(x, lambda = 40)
## ! x = inla.pc.qsn(0.5, lambda = 40)
## ! inla.pc.psn(x, lambda = 40)
## ! }

inla.pc.sn.cache <- function(force = FALSE, write.files = FALSE) {
    sn.skew <- function(alpha) {
        delta <- alpha / sqrt(1 + alpha^2)
        skew <- (4 - pi) / 2 * (delta * sqrt(2 / pi))^3 / (1 - 2 * delta^2 / pi)^(3 / 2)
        return(skew)
    }

    sn <- function(x, alpha) {
        delta <- alpha / sqrt(1 + alpha^2)
        omega <- 1 / sqrt(1 - 2 * delta^2 / pi)
        xi <- -omega * delta * sqrt(2 / pi)
        xx <- (x - xi) / omega
        return(2 / omega * dnorm(xx) * pnorm(alpha * xx))
    }

    kld.value <- function(x, alpha) {
        dsn <- sn(x, alpha)
        idx <- which(dsn > 0.0)
        dsn <- dsn[idx]
        x <- x[idx]
        if (length(x) %% 2 == 0) {
            x <- x[-length(x)]
            dsn <- dsn[-length(dsn)]
        }
        w <- c(1, rep(c(4, 2), (length(x) - 3) %/% 2), 4, 1) / 3
        dn <- dnorm(x)
        dsn <- dsn / sum(w * dsn)
        dn <- dn / sum(w * dn)
        return(sum(w * dsn * log(dsn / dn)))
    }

    kld.t <- function(alpha) {
        t1 <- alpha * alpha
        t8 <- t1 * t1
        t10 <- t8 * t1 * (0.3960827996046746E-2 + t1 * (-0.4043436622715706E-2 + t1 * (
            0.205737163227263E-2 + 0.2400653839003813E-4 * t1)))
        return(t10)
    }

    kld <- function(alpha) {
        ## 0.4 is decided empirically, so I do not see any discontinuities
        if (alpha < 0.4) {
            return(kld.t(alpha))
        } else {
            dx <- 0.005
            ran <- 8
            x <- seq(-ran, ran, by = dx)
            return(kld.value(x, alpha))
        }
    }

    dist <- function(alpha) {
        d <- numeric(length(alpha))
        for (i in seq_along(alpha)) {
            d[i] <- sqrt(2.0 * max(0, kld(alpha[i])))
        }
        idx <- which(!(is.na(d) | is.infinite(d)))
        return(data.frame(alpha = alpha[idx], dist = d[idx]))
    }

    tag <- "cache.pc.sn"
    if (force || !exists(tag, envir = inla.get.inlaEnv())) {
        ran <- 25
        alphas <- seq(0, ran, len = 500)[-1]
        alphas <- c(-rev(alphas), 0, alphas)
        alphas.pos <- alphas[which(alphas >= 0)]
        alphas.neg <- alphas[which(alphas <= 0)]
        dist.pos <- dist(alphas.pos)$dist
        dist.neg <- rev(dist.pos)

        skews <- sn.skew(alphas)
        skews.pos <- sn.skew(alphas.pos)
        skews.neg <- sn.skew(alphas.neg)

        if (write.files) {
            print("write file [s-sn-pc-prior.dat]")
            write(skews.pos, ncol = 1, file = "s-sn-pc-prior.dat")
            print("write file [d-sn-pc-prior.dat]")
            write(dist.pos, ncol = 1, file = "d-sn-pc-prior.dat")
        }

        ## we could improve here, as d'(s) = d*'(s*) d(s) / s, where d* = log(d), and s* =
        ## log(s), but since we're using the Taylor expansion near zero, it is ok not to.

        m <- "monoH.FC"
        assign(tag,
            list(
                skews.range = range(skews),
                dist.max = max(dist.pos),
                pos = list(
                    x = skews.pos,
                    a = splinefun(skews.pos, alphas.pos, method = m),
                    dist = splinefun(skews.pos, dist.pos, method = m),
                    idist = splinefun(dist.pos, skews.pos, method = m)
                ),
                neg = list(
                    x = skews.neg,
                    a = splinefun(skews.neg, alphas.neg, method = m),
                    dist = splinefun(skews.neg, dist.neg, method = m),
                    idist = splinefun(dist.neg, skews.neg, method = m)
                )
            ),
            envir = inla.get.inlaEnv()
        )
    }
    return(get(tag, envir = inla.get.inlaEnv()))
}

inla.pc.rsn <- function(n, lambda = 40) {
    cache <- inla.pc.sn.cache()
    fac <- 1.0 - exp(-lambda * cache$dist.max)
    x <- numeric(n)
    ind <- sample(c(-1, 1), n, replace = TRUE)
    idx.pos <- which(ind > 0)
    idx.neg <- which(ind < 0)
    z <- -1.0 / lambda * log(1.0 - runif(n) * fac)
    if (length(idx.pos) > 0) {
        x[idx.pos] <- cache$pos$idist(z[idx.pos])
    }
    if (length(idx.neg) > 0) {
        x[idx.neg] <- cache$neg$idist(z[idx.neg])
    }
    return(x)
}

inla.pc.dsn <- function(skew, lambda = 40, log = FALSE) {
    cache <- inla.pc.sn.cache()
    fac <- 1.0 - exp(-lambda * cache$dist.max)
    d <- numeric(length(skew))
    idx.pos <- which(skew >= 0)
    idx.neg <- which(skew < 0)
    if (length(idx.pos) > 0) {
        la <- skew[idx.pos]
        dist <- cache$pos$dist(la)
        deriv <- cache$pos$dist(la, deriv = 1)
        d[idx.pos] <- log(0.5) + log(lambda) - lambda * dist + log(abs(deriv)) - log(fac)
    }
    if (length(idx.neg) > 0) {
        la <- skew[idx.neg]
        dist <- cache$neg$dist(la)
        deriv <- cache$neg$dist(la, deriv = 1)
        d[idx.neg] <- log(0.5) + log(lambda) - lambda * dist + log(abs(deriv)) - log(fac)
    }
    return(if (log) d else exp(d))
}

inla.pc.qsn <- function(p, lambda = 40) {
    cache <- inla.pc.sn.cache()
    fac <- 1.0 - exp(-lambda * cache$dist.max)
    n <- length(p)
    q <- numeric(n)
    idx.pos <- which(p >= 0.5)
    idx.neg <- which(p < 0.5)
    if (length(idx.pos) > 0) {
        pp <- 2.0 * (p[idx.pos] - 0.5) * fac
        qe <- qexp(pp, rate = lambda)
        q[idx.pos] <- cache$pos$idist(qe)
    }
    if (length(idx.neg) > 0) {
        pp <- 2.0 * (0.5 - p[idx.neg]) * fac
        qe <- qexp(pp, rate = lambda)
        q[idx.neg] <- cache$neg$idist(qe)
    }
    return(q)
}

inla.pc.psn <- function(q, lambda = 40) {
    cache <- inla.pc.sn.cache()
    fac <- 1.0 - exp(-lambda * cache$dist.max)
    n <- length(q)
    p <- numeric(n)
    idx.pos <- which(q >= 0.0)
    idx.neg <- which(q < 0.0)
    if (length(idx.pos) > 0) {
        qq <- cache$pos$dist(q[idx.pos])
        pp <- pexp(qq, rate = lambda) / fac
        p[idx.pos] <- (1.0 + pp) / 2.0
    }
    if (length(idx.neg) > 0) {
        qq <- cache$neg$dist(q[idx.neg])
        pp <- pexp(qq, rate = lambda) / fac
        p[idx.neg] <- (1.0 - pp) / 2.0
    }
    return(p)
}

inla.pc.sn.test <- function(lambda = 40, n = 10^6) {
    ## this is for testing only
    f <- function(...) format(..., digits = 3)
    x <- inla.pc.rsn(n, lambda = lambda)
    skew <- seq(min(x), max(x), by = 0.01)
    hist(x, n = 3000, prob = TRUE, xlim = quantile(x, c(0.001, 0.999)))
    lines(skew, inla.pc.dsn(skew, lambda), lwd = 3, col = "blue")
    p <- c(0.001, 0.01, seq(0.05, 0.95, by = 0.1), 0.99, 0.999)
    print(cbind(
        p = f(p), q.emp = f(as.numeric(quantile(x, prob = p))),
        q.true = f(inla.pc.qsn(p, lambda))
    ))
    q <- as.numeric(quantile(x, prob = p))
    cdf <- ecdf(x)
    print(cbind(q = f(q), p.emp = f(cdf(q)), p.true = f(inla.pc.psn(q, lambda))))
}

inla.pc.sn.test2 <- function() {
    fun <- function(intern.intercept, skew) {
        cache <- inla.pc.sn.cache()
        alpha <- sign(skew) * cache$pos$a(abs(skew))
        delta <- alpha / sqrt(1 + alpha^2)
        omega <- 1 / sqrt(1 - 2 * delta^2 / pi)
        xi <- -omega * delta * sqrt(2 / pi)
        print(c(skew = skew, xi = xi, omega = omega, alpha = alpha))
        return(qsn(intern.intercept, xi = xi, omega = omega, alpha = alpha))
    }

    print(fun(0.43, 0.123))
    print(fun(0.823, -0.123))
}

if (FALSE) {
    inla.get.inlaEnv <- function(...) inla.get.inlaEnv(...)
}

## Export: inla.mesh.components

#' Compute connected mesh subsets
#'
#' Compute subsets of vertices and triangles in an inla.mesh object that are
#' connected by edges.
#'
#'
#' @return A list with elements `vertex` and `triangle`, vectors of
#' integer labels for which connected component they belong, and `info`, a
#' `data.frame` with columns
#' \item{component}{Connected component integer label.}
#' \item{nV}{The number of vertices in the component.}
#' \item{nT}{The number of triangles in the component.}
#' \item{area}{The surface area associated with the component.
#' Component lables are not comparable across
#' different meshes, but some ordering stability is guaranteed by initiating
#' each component from the lowest numbered triangle whenever a new component is
#' initiated.}
#'
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.2d()], [inla.mesh.create()]
#' @examples
#'
#' # Construct two simple meshes:
#' loc <- matrix(c(0, 1, 0, 1), 2, 2)
#' mesh1 <- inla.mesh.2d(loc = loc, max.edge = 0.1)
#' bnd <- inla.nonconvex.hull(loc, 0.3)
#' mesh2 <- inla.mesh.2d(boundary = bnd, max.edge = 0.1)
#'
#' # Compute connectivity information:
#' conn1 <- inla.mesh.components(mesh1)
#' conn2 <- inla.mesh.components(mesh2)
#' # One component, simply connected mesh
#' conn1$info
#' # Two disconnected components
#' conn2$info
#'
#' # Extract the subset mesh for the largest component:
#' # (Note: some information is lost, such as fixed segments,
#' # and boundary edge labels.)
#' maxi <- conn2$info$component[which.max(conn2$info$area)]
#' mesh3 <- inla.mesh.create(
#'     loc = mesh2$loc,
#'     tv = mesh2$graph$tv[conn2$triangle == maxi, , drop = FALSE]
#' )
#' @export
inla.mesh.components <- function(mesh) {
    vertex <- integer(mesh$n)
    Nt <- nrow(mesh$graph$tv)
    triangle <- integer(Nt)
    ok <- !is.na(mesh$graph$tt)
    tt <- Matrix::sparseMatrix(
        i = c(
            which(ok[, 1]),
            which(ok[, 2]),
            which(ok[, 3])
        ),
        j = c(
            mesh$graph$tt[ok[, 1], 1],
            mesh$graph$tt[ok[, 2], 2],
            mesh$graph$tt[ok[, 3], 3]
        ),
        x = rep(1, sum(ok)),
        dims = c(Nt, Nt)
    )
    component <- 0
    while (any(triangle == 0)) {
        component <- component + 1
        tri <- integer(Nt)
        tri[min(which(triangle == 0))] <- 1
        tri_prev <- integer(Nt)
        while (any(tri_prev != tri)) {
            tri_prev <- tri
            tri <- tri | as.vector(tt %*% tri > 0)
        }
        triangle[tri > 0] <- component
        vtx <- sort(unique(as.vector(mesh$graph$tv[tri > 0, ])))
        if (any(vertex[vtx] > 0)) {
            warning(paste0(
                "Corner-only connected triangles detected.\n",
                "  Vertices = ", paste0(vtx[vertex[vtx] > 0], collapse = ", "), "\n",
                "  Components = ", paste0(vertex[vtx[vertex[vtx] > 0]], collapse = ", "), "\n",
                "  New component = ", component, "\n",
                "  Vertex component information will be inconsistent.", "\n",
                "  Triangle component information will ignore corner-only connections."
            ))
        }
        vertex[vtx] <- component
    }

    if (component == 0) {
        info <- data.frame(component = integer(0), nV = integer(0), nT = integer(0), area = numeric(0))
    } else {
        fem <- inla.mesh.fem(mesh, order = 1)
        info <- do.call(
            rbind,
            lapply(
                sort(unique(triangle)),
                function(x) {
                    data.frame(
                        component = x,
                        nV = sum(vertex == x),
                        nT = sum(triangle == x),
                        area = sum(fem$ta[triangle == x])
                    )
                }
            )
        )
    }

    list(
        vertex = vertex,
        triangle = triangle,
        info = info
    )
}

## Nothing to export

inla.extract.Q <- function(what, result) {
    stopifnot(!missing(what))
    stopifnot(!missing(result))

    conf <- result$misc$configs
    cont <- conf$contents
    i <- which(cont$tag == what)
    if (length(i) != 1L) {
        stop(paste0("Cannot find '", what, "'"))
    }
    p.len <- cont$length[1]
    i.start <- cont$start[i]
    i.len <- cont$length[i]

    k <- NA
    for (j in seq_along(conf$config)) {
        if (conf$config[[j]]$log.posterior.orig == 0.0) {
            k <- j
            break
        }
    }
    stopifnot(!is.na(k))
    Q <- conf$config[[k]]$Q
    diag(Q) <- conf$config[[k]]$Qprior.diag
    d <- diag(Q)
    log.prec <- inla.models()$predictor$predictor$hyper$theta$initial
    browser()
    for (i in i.start:(i.start + i.len - 1L)) {
        Q[i, i] <- Q[i, i] - sum(Q[1:p.len, i]^2 / (d[1:p.len] - exp(log.prec)))
    }
    Q <- Q[i.start:(i.start + i.len - 1), i.start:(i.start + i.len - 1)]
    Q <- Q + t(Q)
    diag(Q) <- diag(Q) / 2.0

    return(Q)
}

## Export: inla.mdata is.inla.mdata as.inla.mdata
## Export: print!inla.mdata

## !\name{inla.mdata}
## !\alias{inla.mdata}
## !\alias{is.inla.mdata}
## !\alias{as.inla.mdata}
## !\alias{print.inla.mdata}
## !
## !\title{
## !Create an mdata-object for INLA
## !}
## !
## !\description{
## !This defines an mdata-object for matrix valued response-families
## !}
## !\usage{
## !inla.mdata(y, ...)
## !is.inla.mdata(object)
## !as.inla.mdata(object)
## !}
## !
## !\arguments{
## !  \item{y}{The response vector/matrix}
## !  \item{...}{Additional vectors/matrics of same length as \code{y}}
## !  \item{object}{Any \code{R}-object}
## !  \item{x}{An mdata object}
## !  }
## !
## !\value{
## !     An object of class \code{inla.mdata}.  There is method for \code{print}.
## !
## !     \code{is.inla.mdata} returns \code{TRUE} if \code{object}
## !     inherits from class \code{inla.mdata}, otherwise \code{FALSE}.
## !
## !     \code{as.inla.mdata} returns an object of class \code{inla.mdata}
## !}
## !\note{It is often required to set \code{Y=inla.mdata(...)} and then
## !      define the formula as \code{Y~...},  especially when used with
## !      \code{inla.stack}.}
## !\author{
## ! Havard Rue
## !}
## !
## !\seealso{
## !\code{\link{inla}}
## !}

`inla.mdata` <- function(y, ...) {
    y.obj <- as.list(as.data.frame(list(y)))
    x.obj <- as.list(as.data.frame(list(...)))
    if (length(list(...)) == 0) {
        ncols <- ncol(as.data.frame(list(y)))
    } else {
        ncols <- c(
            ncol(as.data.frame(list(y))),
            unlist(lapply(
                list(...),
                function(x) if (is.null(ncol(x))) 1 else ncol(x)
            ))
        )
    }
    names(y.obj) <- paste("Y", 1:length(y.obj), sep = "")
    if (length(x.obj) > 0) {
        names(x.obj) <- paste("X", 1:length(x.obj), sep = "")
        obj <- c(y.obj, x.obj)
    } else {
        obj <- y.obj
    }
    attr(obj, "inla.ncols") <- c(length(ncols), ncols)
    class(obj) <- "inla.mdata"
    return(obj)
}

`print.inla.mdata` <- function(object, ...) {
    cat("inla.cols = ", attr(object, "inla.ncols", exact = TRUE), "\n")
    print(as.data.frame(unclass(object)), ...)
}

`as.inla.mdata` <- function(object) {
    if (is.inla.mdata(object)) {
        return(object)
    }
    object <- as.list(as.data.frame(object))
    if (length(object) == 1) {
        names(object) <- "Y1"
        ncols <- c(1, 1)
    } else {
        names(object) <- c("Y1", paste("X", 1:(length(object) - 1), sep = ""))
        ncols <- c(2, 1, length(object) - 1)
    }
    warning("Guess that ncol(response) == 1. Otherwise,  please modify 'names(object)'.")
    attr(object, "inla.ncols") <- ncols
    class(object) <- "inla.mdata"
    return(object)
}

`is.inla.mdata` <- function(object) {
    return(!is.null(attr(object, "inla.ncols", exact = TRUE)) ||
        inherits(object, "inla.mdata"))
}

## Export: inla.rerun

## ! \name{inla.rerun}
## ! \alias{inla.rerun}
## ! \alias{rerun}
## !
## ! \title{Rerun an analysis}
## ! \description{Rerun \code{\link{inla}} on an
## !              inla-object (output from \code{link{inla}})}
## ! \usage{
## ! inla.rerun(object, plain=FALSE)
## ! }
## ! \arguments{
## !
`inla.rerun` <- function(
                         ## !\item{object}{An \code{inla}-object, ie the output from an \code{inla}-call}
                         object,
                         ## !\item{plain}{Logical. If \code{FALSE} (default), then make
                         ## !changes in \code{object} to improve the performance}
                         plain = FALSE)
                         ## !}
                         ## !\value{This function will take the result in \code{object},
                         ## !       and rerun \code{inla} again.
                         ## !       If \code{plain}  is \code{FALSE},  start  the optimization
                         ## !       from the mode in \code{object} so that
                         ## !       we can obtain an  improvement the mode for the hyperparameters.
                         ## !       Otherwise,  start from the same configuration
                         ## !       as for \code{object}.
                         ## !       The returned value is an \code{inla}-object.}
                         ## !\seealso{\code{\link{inla}}}
                         ## !\examples{
                         ## !r = inla(y ~ 1,  data = data.frame(y=1:10))
                         ## !r = inla.rerun(r)
## !}
{
    stopifnot(any(inherits(object, "inla")))

    ## need to do this, as if its true it will regenerate the linear combinations
    object$.args$control.fixed$correlation.matrix <- FALSE

    if (!plain) {
        object$.args$control.mode$result <- NULL
        object$.args$control.mode$restart <- TRUE
        object$.args$control.mode$theta <- object$mode$theta
        object$.args$control.mode$x <- object$mode$x
        ## to do a proper restart, we should keep the direction is any. if they are not used,
        ## they will be NULL, which is the same as FALSE.
        object$.args$control.inla$use.directions <- object$misc$opt.directions
    }

    new.obj <- do.call("inla", args = object$.args)
    ## revert, as variables are expanded...
    new.obj$call <- object$call

    return(new.obj)
}

## Export: inla.lattice2node.mapping inla.node2lattice.mapping inla.lattice2node inla.node2lattice inla.matrix2vector inla.vector2matrix

## !\name{lattice2node}
## !\alias{lattice2node}
## !\alias{inla.lattice2node}
## !\alias{node2lattice}
## !\alias{inla.node2lattice}
## !\alias{node2lattice.mapping}
## !\alias{inla.node2lattice.mapping}
## !\alias{lattice2node.mapping}
## !\alias{inla.lattice2node.mapping}
## !\alias{matrix2vector}
## !\alias{vector2matrix}
## !\alias{inla.matrix2vector}
## !\alias{inla.vector2matrix}
## !
## !\title{Functions to define mapping between a lattice and nodes}
## !
## !\description{These functions define mapping in between two-dimensional
## !  indices on a lattice and the one-dimensional node representation used
## !  in \code{inla}.
## !
## !  The mapping from node to lattice follows the default \code{R}
## !  behaviour (which is column based storage), and \code{as.vector(A)} and
## !  \code{matrix(a, nrow, ncol)} can be used instead of
## !  \code{inla.matrix2vector} and \code{inla.vector2matrix}.
## !}
## !
## !\usage{
## !inla.lattice2node.mapping(nrow, ncol)
## !inla.node2lattice.mapping(nrow, ncol)
## !inla.lattice2node(irow, icol, nrow, ncol)
## !inla.node2lattice(node, nrow, ncol)
## !inla.matrix2vector(a.matrix)
## !inla.vector2matrix(a.vector, nrow, ncol)
## !}
## !
## !\arguments{
## !
## !  \item{nrow}{Number of rows in the lattice.}
## !
## !  \item{ncol}{Number of columns in the lattice.}
## !
## !  \item{irow}{Lattice row index, between \code{1} and \code{nrow}}
## !
## !  \item{icol}{Lattice column index, between \code{1} and \code{ncol}}
## !
## !  \item{node}{The node index, between \code{1} and \code{ncol*nrow}}
## !
## !  \item{a.matrix}{is a matrix to be mapped to a vector using internal
## !  representation defined by \code{inla.lattice2node}}
## !
## !  \item{a.vector}{is a vector to be mapped into a matrix using the
## !    internal representation defined by \code{inla.node2lattice}}
## !}
## !
## !\value{\code{inla.lattice2node.mapping} returns the hole mapping as a
## !  matrix, and \code{inla.node2lattice.mapping} returns the hole mapping
## !  as \code{list(irow=..., icol=...)}. \code{inla.lattice2node} and
## !  \code{inla.node2lattice} provide the mapping for a given set of
## !  lattice indices and nodes. \code{inla.matrix2vector} provide the
## !  mapped vector from a matrix, and \code{inla.vector2matrix} provide the
## !  inverse mapped matrix from vector.}
## !
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\seealso{\link{inla}}
## !
## !\examples{
## !## write out the mapping using the two alternatives
## !nrow = 2
## !ncol = 3
## !mapping = inla.lattice2node.mapping(nrow,ncol)
## !
## !for (i in 1:nrow){
## !    for(j in 1:ncol){
## !        print(paste("Alt.1: lattice index [", i,",", j,"] corresponds",
## !                    "to node [", mapping[i,j],"]", sep=""))
## !    }
## !}
## !
## !for (i in 1:nrow){
## !    for(j in 1:ncol){
## !        print(paste("Alt.2: lattice index [", i,",", j,"] corresponds to node [",
## !                    inla.lattice2node(i,j,nrow,ncol), "]", sep=""))
## !    }
## !}
## !
## !inv.mapping = inla.node2lattice.mapping(nrow,ncol)
## !for(node in 1:(nrow*ncol))
## !   print(paste("Alt.1: node [", node, "] corresponds to lattice index [",
## !               inv.mapping$irow[node], ",",
## !               inv.mapping$icol[node],"]", sep=""))
## !
## !for(node in 1:(nrow*ncol))
## !   print(paste("Alt.2: node [", node, "] corresponds to lattice index [",
## !               inla.node2lattice(node,nrow,ncol)$irow[1], ",",
## !               inla.node2lattice(node,nrow,ncol)$icol[1],"]", sep=""))
## !
## !## apply the mapping from matrix to vector and back
## !n = nrow*ncol
## !z = matrix(1:n,nrow,ncol)
## !z.vector = inla.matrix2vector(z)  # as.vector(z) could also be used
## !print(mapping)
## !print(z)
## !print(z.vector)
## !
## !## the vector2matrix is the inverse, and should give us the z-matrix
## !## back. matrix(z.vector, nrow, ncol) could also be used here.
## !z.matrix = inla.vector2matrix(z.vector, nrow, ncol)
## !print(z.matrix)
## !}

`inla.lattice2node.mapping` <- function(nrow, ncol) {
    ## return a matrix with the mapping

    stopifnot(nrow > 0 && ncol > 0)

    mapping <- matrix(NA, nrow = nrow, ncol = ncol)
    for (i in 1:nrow) {
        j <- 1:ncol
        mapping[i, j] <- inla.lattice2node(i, j, nrow, ncol)
    }
    return(mapping)
}

`inla.node2lattice.mapping` <- function(nrow, ncol) {
    stopifnot(nrow > 0 && ncol > 0)

    return(inla.node2lattice(1:(nrow * ncol), nrow, ncol))
}

`inla.lattice2node` <- function(irow, icol, nrow, ncol) {
    ## convert from a lattice point (irow, icol) to a node-number in
    ## the graph; similar to the GMRFLib_lattice2node()-function in
    ## GMRFLib.  Indices goes from irow=1...nrow, to icol=1...ncol and
    ## node=1.....nrow*ncol.

    stopifnot(nrow > 0 && ncol > 0)

    if (length(irow) == length(icol) && length(irow) > 1) {
        ## this makes it kind of 'vectorize' for two arguments...
        n <- length(irow)
        k <- 1:n
        return(sapply(k,
            function(irow, icol, nrow, ncol, k) {
                return(inla.lattice2node(irow[k], icol[k], nrow, ncol))
            },
            irow = irow, icol = icol, nrow = nrow, ncol = ncol
        ))
    } else {
        return((irow - 1) + (icol - 1) * nrow + 1)
    }
}

`inla.node2lattice` <- function(node, nrow, ncol) {
    ## convert from a node-number in the graph to a lattice point
    ## (irow, icol); similar to the GMRFLib_node2lattice()-function in
    ## GMRFLib.  Indices goes from irow=1...nrow, to icol=1...ncol and
    ## node=1.....nrow*ncol.

    stopifnot(nrow > 0 && ncol > 0)

    icol <- (node - 1) %/% nrow
    irow <- (node - 1) %% nrow

    return(list(irow = irow + 1, icol = icol + 1))
}

`inla.matrix2vector` <- function(a.matrix) {
    ## utility function for mapping a matrix to inla's internal `node'
    ## representation by inla.lattice2node() and inla.node2lattice()

    if (!is.matrix(a.matrix)) {
          stop("Argument must be a matrix")
      }

    return(as.vector(a.matrix))
}
`inla.vector2matrix` <- function(a.vector, nrow, ncol) {
    ## utility function for mapping from inla's internal `node'
    ## representation, inla.lattice2node() and inla.node2lattice(),
    ## and to a matrix

    n <- length(a.vector)
    if (missing(nrow) && !missing(ncol)) {
          nrow <- n %/% ncol
      }
    if (!missing(nrow) && missing(ncol)) {
          ncol <- n %/% nrow
      }
    if (n != nrow * ncol) {
          stop(paste("Length of vector", n, "does not equal to nrow*ncol", nrow * ncol))
      }

    return(matrix(a.vector, nrow, ncol))
}

## utility function for quantile-regression.
## Continous Poisson and quantile regression for Poisson


inla.incGamma <- function(x, lambda = 0, log = FALSE) {
    ## library(gsl); return (gamma_inc(x, lambda))
    lres <- lgamma(x) + pgamma(lambda, x, lower = FALSE, log.p = TRUE)
    return(if (log) lres else exp(lres))
}

inla.qcontpoisson <- function(p, lambda, print.level = 0) {
    inla.qcontpoisson.core <- function(p, lambda, print.level = 0) {
        fun.min <- function(x, lambda, prob) {
            log.x <- log(x + 1)
            p.est <- inla.pcontpoisson(exp(log.x) - 1, lambda)
            eps <- 1E-12
            p.est <- max(eps, min(p.est, 1 - eps))
            return((inla.link.logit(p.est) - inla.link.logit(prob))^2)
        }
        initial.value <- qpois(p, lambda) - 0.49
        return((nlm(fun.min,
            p = initial.value, print.level = print.level,
            lambda = lambda, prob = p
        )$estimate))
    }

    fun <- Vectorize(inla.qcontpoisson.core, c("p", "lambda", "print.level"))
    return(fun(p, lambda, print.level))
}

inla.pcontpoisson <- function(x, lambda, log = FALSE, deriv = 0) {
    inla.pcontpoisson.core <- function(x, lambda, log = FALSE, deriv = 0) {
        ## > F := (x, lambda) -> GAMMA(x, lambda) / GAMMA(x);
        ## GAMMA(x, lambda)
        ## F := (x, lambda) -> ----------------
        ## GAMMA(x)
        ##
        ## > simplify(diff(F(x, lambda), lambda));
        ## (x - 1)
        ## lambda        exp(-lambda)
        ## - --------------------------
        ## GAMMA(x)
        ##
        ## > simplify(diff(F(x, lambda), lambda$2));
        ## (x - 2)
        ## lambda        exp(-lambda) (-x + 1 + lambda)
        ## --------------------------------------------
        ## GAMMA(x)
        ##

        x <- x + 1
        if (deriv == 0) {
            ## can use gsl::gamma_inc_Q() instead
            lres <- inla.incGamma(x, lambda, log = TRUE) - inla.incGamma(x, log = TRUE)
            return(if (log) lres else exp(lres))
        } else if (deriv == 1) {
            stopifnot(!log)
            return(-exp((x - 1) * log(lambda) - lambda - inla.incGamma(x, log = TRUE)))
        } else if (deriv == 2) {
            stopifnot(!log)
            return((lambda - x + 1) * exp((x - 2) * log(lambda) - lambda - inla.incGamma(x, log = TRUE)))
        } else {
            stop("deriv != 0, 1, 2")
        }
    }

    fun <- Vectorize(inla.pcontpoisson.core, c("x", "lambda", "log", "deriv"))
    return(fun(x, lambda, log, deriv))
}

inla.pcontpoisson.eta <- function(x, eta, deriv = 0, log = FALSE) {
    inla.pcontpoisson.eta.core <- function(x, eta, deriv = 0, log = FALSE) {
        ## the cdf of the contpoisson parameterised by the linear predictor and the log-link
        lambda <- exp(eta)
        if (deriv == 0) {
            return(inla.pcontpoisson(x, lambda, log = log))
        } else if (deriv == 1) {
            stopifnot(!log)
            return(inla.pcontpoisson(x, lambda, deriv = 1) * lambda)
        } else if (deriv == 2) {
            stopifnot(!log)
            return(lambda * (inla.pcontpoisson(x, lambda, deriv = 1) +
                inla.pcontpoisson(x, lambda, deriv = 2) * lambda))
        } else {
            stop("deriv != 0, 1, 2")
        }
    }
    fun <- Vectorize(inla.pcontpoisson.eta.core, c("x", "eta", "deriv", "log"))
    return(fun(x, eta, deriv, log))
}

inla.contpoisson.solve.lambda <- function(quantile, alpha, iter.max = 1000, max.step = 3,
                                          tol = sqrt(.Machine$double.eps), verbose = FALSE) {
    ## solve quantile=inla.pcontpoisson(lambda, alpha),  for lambda
    ## stopifnot(length(quantile) == 1 && length(alpha) == 1)
    return(exp(inla.contpoisson.solve.eta(quantile, alpha, iter.max, max.step, tol, verbose)))
}

inla.contpoisson.solve.eta <- function(quantile, alpha, iter.max = 1000, max.step = 3,
                                       tol = sqrt(.Machine$double.eps), verbose = FALSE) {
    inla.contpoisson.solve.eta.core <- function(quantile, alpha, iter.max = 1000, max.step = 3,
                                                tol = sqrt(.Machine$double.eps), verbose = FALSE) {
        ## solve quantile=inla.pcontpoisson(lambda=exp(eta), alpha),  for eta
        stopifnot(length(quantile) == 1 && length(alpha) == 1)
        eta.0 <- log((sqrt(quantile) - qnorm(alpha, sd = 0.5))^2)
        for (i in 1:iter.max) {
            f <- inla.pcontpoisson(quantile, lambda = exp(eta.0)) - alpha
            fd <- inla.pcontpoisson.eta(quantile, eta.0, deriv = 1)
            d <- -min(max.step, max(-max.step, f / fd))
            eta <- eta.0 <- eta.0 + d
            if (verbose) {
                  print(round(c(iter = i, eta = eta, f = f, f.deriv = fd, err = d), digits = 6))
              }
            if (abs(d) < tol) {
                  return(eta)
              }
        }
        stop(paste("FAILURE", quantile, alpha))
    }
    fun <- Vectorize(
        inla.contpoisson.solve.eta.core,
        c("quantile", "alpha", "iter.max", "max.step", "tol")
    )
    return(fun(quantile, alpha, iter.max, max.step, tol))
}

## Export: inla.knmodels

## ! \name{inla.knmodels}
## ! \alias{inla.knmodels}
## ! \alias{knmodels}
## ! \title{Spacetime interaction models}
## ! \description{
## !    It implements the models in Knorr-Held, L. (2000)
## !    with three different constraint approaches:
## !    sum-to-zero, contrast or diagonal add.
## ! }
## ! \usage{
## !inla.knmodels(
## ! formula,
## ! data,
## ! progress=FALSE,
## ! control.st=list(
## !   t=NULL,
## !   s=NULL,
## !   st=NULL,
## !   graph=NULL,
## !   type=c(paste(1:4), paste0(2:4, 'c'), paste0(2:4, 'd')),
## !   diagonal=1e-5,
## !   ...)
## ! )
## !}
## ! \arguments{
## ! \item{formula}{The formula specifying the other
## !   model components, without the spacetime
## !   interaction term. The spacetime interaction term
## !   will be added accordly to the specification in
## !   the \code{control.st} argument. See \code{inla}}

## ! \item{progress}{If it is to be shown the model
## !   fitting progress. Useful if more than one
## !   interaction type is being fitted.}

## ! \item{control.st}{Named list of arguments to control
## !   the spacetime interaction. It should contain:
## ! \describe{

## !  \item{time}{to be used as the index set for the
## !   main temporal effect which will be considered
## !   for the constraints when it is the case.}

## !  \item{space}{to be used as the index set for the
## !   main spatial effect which will be considered
## !   for the constraints when it is the case.}

## !  \item{spacetime}{to be the index set for the
## !   spacetime interaction effect.}

## ! \item{graph}{to be the graph for the spatial neighbor
## !   structure to be used in a \code{\link{f}} term
## !   for the main spatial random effect term or for
## !   building the spacetime interaction model.}

## ! \item{type}{to specify the spacetime interaction type.
## !   \code{1} to \code{4} corresponds to the four
## !   interaction types in Knorr-Held, L. (2000) with
## !   all the needed sum-to-zero constraints.
## !   \code{2c}, \code{3c} and \code{4c} are
## !   the contrast version considering the first time
## !   or space constrained to be equal to zero.
## !   \code{2d}, \code{3d} and \code{4d} are the
## !   corresponding versions when considering the
## !   diagonal add approach.}

## ! \item{diagonal}{to be the value to be added to the
## !   diagonal when using the diagonal add approach.}

## ! \item{timeref}{to specify the time point to be the
## !   reference time in the contrast parametrization.}

## ! \item{spaceref}{to specify the area to be the
## !   reference for the contrast parametrization.}

## !  \item{...}{where additional arguments can be
## !   passed to \code{\link{f}} function.
## !   Specification of the hyperparameter,
## !   fixed or random, initial value, prior and its
## !   parameters for the spacetime interaction. See
## !   \code{?inla.models} and look for \code{generic0}.
## !   By default we scale it and use the PC-prior to set
## !   the prior using the \code{pc.prec} prior with
## !   \code{param = c(0.5, 0.5)}. See documentation with
## !   \code{?inla.doc("pc.prec")}.}
## ! }
## ! }
## ! \item{...}{Arguments to be passed to the
## !   \code{\link{inla}} function.}
## !}
## ! \value{
## !  \code{inla.knmodels} returns an object of class \code{"inla"}.
## !    or a list of objects of this class if it is asked to compute
## !    more than one interaction type at once.
## ! Note: when the model type is 2c, 3c, 4c, 2d, 3d or 4d, it also
## !   includes linear combinations summary.
## ! }
## ! \author{Elias T. Krainski}
## ! \seealso{
## !     \code{\link{inla.knmodels.sample}} to sample from
## ! }
## ! \examples{
## !### define space domain as a grid
## !grid <- SpatialGrid(GridTopology(c(0,0), c(1, 1), c(4, 5)))
## !(n <- nrow(xy <- coordinates(grid)))
## !
## !### build a spatial neighborhood list
## !jj <- lapply(1:n, function(i)
## !    which(sqrt((xy[i,1]-xy[,1])^2 + (xy[i,2]-xy[,2])^2)==1))
## !
## !### build the spatial adjacency matrix
## !graph <- sparseMatrix(rep(1:n, sapply(jj, length)),
## !                      unlist(jj), x=1, dims=c(n, n))
## !
## !### some random data at 10 time point
## !dat <- inla.knmodels.sample(graph, m=10, tau.t=2, tau.s=2, tau.st=3)
## !str(dat)
## !sapply(dat$x, summary)
## !
## !nd <- length(dat$x$eta)
## !dat$e <- runif(nd, 0.9, 1.1)*rgamma(n, 40, 2)
## !dat$y <- rpois(nd, dat$e*exp(dat$x$eta-3))
## !summary(dat$y)
## !
## !### fit the type 4 considering three different approaches
## !tgraph <- sparseMatrix(i=c(2:10, 1:9), j=c(1:9, 2:10), x=1)
## !res <- inla.knmodels(y ~ f(time, model='bym2', graph=tgraph) +
## !     f(space, model='bym2', graph=graph),
## !     data=dat, family='poisson', E=dat$E, progress=TRUE,
## !     control.st=list(time=time, space=space,
## !        spacetime=spacetime, graph=graph, type=c(4, '4c', '4d')),
## !     control.compute=list(dic=TRUE, waic=TRUE, cpo=TRUE))
## !sapply(res, function(x)
## !       c(dic=x$dic$dic, waic=x$waic$waic, cpo=-sum(log(x$cpo$cpo))))
## !}


`inla.knmodels` <- function(formula,
                            progress = FALSE,
                            control.st = list(
                                time,
                                space,
                                spacetime,
                                graph,
                                type = c(paste(1:4), paste0(2:4, "c"), paste0(2:4, "d")),
                                diagonal = 1e-5,
                                timeref = 1,
                                spaceref = 1,
                                ...
                            ),
                            ...) {
    mcall <- match.call(expand.dots = TRUE)
    ft <- paste("~", mcall$control.st$time)
    if (ft == "~ ") {
        time <- tname <- NULL
    } else {
        tname <- substring(ft, 3)
        time <- model.frame(as.formula(ft), data = eval(mcall$data))[, 1]
        timeref <- unique(eval(mcall$control.st$timeref))
        if (length(timeref) > 1) {
            stop("length(timeref)>1")
        }
        if (length(timeref) == 0) timeref <- 1
    }
    fs <- paste("~", mcall$control.st$space)
    if (fs == "~ ") {
        space <- sname <- NULL
    } else {
        sname <- substring(fs, 3)
        space <- model.frame(as.formula(fs), data = eval(mcall$data))[, 1]
        spaceref <- 1
        spaceref <- unique(eval(mcall$control.st$spaceref))
        if (length(spaceref) > 1) {
            stop("length(spaceref)>1")
        }
        if (length(spaceref) == 0) spaceref <- 1
    }
    fst <- paste("~", mcall$control.st$spacetime)
    if (fst == "~ ") {
        spacetime <- NULL
    } else {
        stname <- substring(fst, 3)
        spacetime <- model.frame(as.formula(fst), data = eval(mcall$data))[, 1]
    }
    ## cat('tname =', tname, ', sname =', sname, ', stname =', stname, '\n')
    type <- as.character(unique(eval(mcall$control.st$type)))
    types <- c(1:4, paste0(2:4, "c"), paste0(2:4, "d"))
    type <- if (length(type) == 0) types else types[match(type, types)]
    ## cat('type =', type, '\n')
    if (length(type) == 0) {
        return(NULL)
    } else {
        res <- list()
    }
    if (length(mcall$control.st$diagonal) == 0) diagonal <- 1e-5
    ## cat('diagonal =', diagonal, '\n')
    m <- n <- NULL
    nst <- length(unique(spacetime))
    if (!is.null(mcall$control.st$graph)) {
        graph <- eval(mcall$control.st$graph)
        n <- nrow(graph <- inla.graph2matrix(graph))
        R.s <- inla.scale.model(Diagonal(n, colSums(graph)) - graph,
            constr = list(A = matrix(1, 1, n), e = 0)
        )
    } else {
        if (any(substr(type, 1, 1) %in% c("3", "4"))) {
            stop("'graph' must be provided to build the spacetime interaction model!")
        }
    }
    if (!is.null(space)) {
        if (!is.null(mcall$control.st$graph)) {
            if (n != length(unique(space))) {
                stop("Size of 'space' is not equal to the size of 'graph'!")
            }
        }
        n <- length(unique(space))
    }
    if (!is.null(time)) {
        m <- length(unique(time))
        if (any(substr(type, 1, 1) %in% c("2", "4"))) {
            R.t <- inla.scale.model(crossprod(diff(Diagonal(m))),
                constr = list(A = matrix(1, 1, m), e = 0)
            )
        }
    }
    if (is.null(m)) m <- nst / n
    if (is.null(n)) n <- nst / m
    ## cat('m = ', m, ', n = ', n, ', nst = ', nst, '\n', sep='')
    if (TRUE) { ## working in progress: identify need of constraints from the formula
        etemp <- inla.interpret.formula(formula, data, debug = FALSE)
        rterms <- attr(terms(etemp[[1]]), "term.labels")
        id.r <- which(sapply(etemp$random.spec, function(x) is.null(x$weights)))
        if (length(id.r) > 0) {
            r.rankdef <- which(sapply(etemp$random.spec[id.r], function(x) is.null(x$rankdef)))
            if (length(r.rankdef) > 0) {
                r.size <- sapply(etemp$random.spec[id.r[r.rankdef]], function(x) x$n)
                j.s <- which(r.size == n)
                j.t <- which(r.size == m)
                if (length(j.s) > 1) {
                    stop("Too many spatial effects with rank deficiency.")
                }
                if (length(j.t) > 1) {
                    stop("Too many temporal effects with rank deficiency.")
                }
            }
        }
        lc2.on <- any(rterms == sname)
        lc3.on <- any(rterms == tname)
    }
    ## cat('lc2 =', lc2.on, ' and lc3 =', lc3.on, '\n')
    M2 <- kronecker(matrix(1 / m, 1, m), diag(n))
    M3 <- kronecker(diag(m), matrix(1 / n, 1, n))
    dotdot <- mcall$control.st[which(!is.element(
        names(mcall$control.st),
        c(
            "time", "space", "graph", "type",
            "timeref", "spaceref"
        )
    ))]

    add0 <- ""
    if (length(names(dotdot)) > 2) {
        add0 <- paste0(", ", names(dotdot)[3], "=", dotdot[3])
    }
    if (any(type %in% "1")) {
        add1 <- paste0("f(", stname, ', model="iid"', add0, ")")
        res$"1" <- inla(update(formula, paste(".~.+", add1)), ...)
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "1") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "2")) {
        add2 <- paste0(
            "f(", stname, ', model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(R.t, Diagonal(n)), ",
            "extraconstr=list(A=M2, e=rep(0,n))", add0, ")"
        )
        res$"2" <- inla(update(formula, paste(".~.+", add2)), ...)
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "2") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "3")) {
        add3 <- paste0(
            "f(", stname, ', model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(Diagonal(m), R.s), ",
            "extraconstr=list(A=M3, e=rep(0,m))", add0, ")"
        )
        res$"3" <- inla(update(formula, paste(".~.+", add3)), ...)
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "3") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "4")) {
        add4 <- paste0(
            "f(", stname, ', model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(R.t, R.s), ",
            "extraconstr=list(A=rbind(M2,M3), e=rep(0,n+m))", add0, ")"
        )
        res$"4" <- inla(update(formula, paste(".~.+", add4)), ...)
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "4") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "2c")) {
        add2c <- paste0(
            'f(st2, model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(R.t, Diagonal(n)), ",
            "extraconstr=list(A=M2, e=rep(0,n))", add0, ")"
        )
        if (is.null(time)) {
            st2 <- spacetime
        } else {
            st2 <- ifelse(time == timeref, NA, spacetime - cumsum(time == timeref))
        }
        id2 <- which(!is.na(st2))
        lc2 <- inla.make.lincombs(
            st2 = Diagonal(n * m)[, id2] - M2[space, id2]
        )
        names(lc2) <- gsub("lc", "st", names(lc2))
        if (lc2.on) {
            lc2args <- list(cbind(Diagonal(n), Diagonal(n, 0)), st2 = M2[, id2] - 1 / (n * m))
            names(lc2args)[1] <- sname
            lcc2 <- do.call("inla.make.lincombs", lc2args)
            names(lcc2) <- gsub("lc", "s", names(lcc2))
            lc2 <- c(lcc2, lc2)
        }
        res$"2c" <- inla(update(formula, paste(".~.+", add2c)),
            lincomb = lc2, ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "2c") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "3c")) {
        add3c <- paste0(
            'f(st3, model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(Diagonal(m), R.s), ",
            "extraconstr=list(A=M3, e=rep(0,m))", add0, ")"
        )
        if (is.null(space)) {
            st3 <- spacetime
        } else {
            st3 <- ifelse(space == spaceref, NA, spacetime - cumsum(space == spaceref))
        }
        id3 <- which(!is.na(st3))
        lc3 <- inla.make.lincombs(
            st3 = Diagonal(n * m)[, id3] - M3[time, id3]
        )
        names(lc3) <- gsub("lc", "st", names(lc3))
        if (lc3.on) {
            lc3args <- list(Diagonal(m), st3 = M3[, id3] - 1 / (n * m))
            names(lc3args)[1] <- tname
            lcc3 <- do.call("inla.make.lincombs", lc3args)
            names(lcc3) <- gsub("lc", "t", names(lcc3))
            lc3 <- c(lcc3, lc3)
        }
        res$"3c" <- inla(update(formula, paste(".~.+", add3c)),
            lincomb = lc3, ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "3c") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "4c")) {
        add4c <- paste0(
            'f(st4, model="generic0", constr=FALSE, ',
            "Cmatrix=kronecker(R.t, R.s), ",
            "extraconstr=list(A=rbind(M2, M3), e=rep(0,n+m))", add0, ")"
        )
        aux4 <- logical(n * m)
        if (is.null(time)) {
            aux4 <- aux4 | (time == timeref)
        }
        if (is.null(space)) {
            aux4 <- aux4 | (space == spaceref)
        }
        st4 <- ifelse(aux4, NA, spacetime - cumsum(aux4))
        id4 <- which(!is.na(st4))
        lc4 <- inla.make.lincombs(
            st4 = (Diagonal(n * m)[, id4] - M2[space, id4] - M3[time, id4] + 1 / (n * m))
        )
        names(lc4) <- gsub("lc", "st", names(lc4))
        if (lc3.on) {
            lc3args <- list(Diagonal(m), st4 = M3[, id4] - 1 / (n * m))
            names(lc3args)[1] <- tname
            lcc3 <- do.call("inla.make.lincombs", lc3args)
            names(lcc3) <- gsub("lc", "t", names(lcc3))
            lc4 <- c(lcc3, lc4)
        }
        if (lc2.on) {
            lc2args <- list(cbind(Diagonal(n), Diagonal(n, 0)),
                st4 = M2[, id4] - 1 / (n * m)
            )
            names(lc2args)[1] <- sname
            lcc2 <- do.call("inla.make.lincombs", lc2args)
            names(lcc2) <- gsub("lc", "s", names(lcc2))
            lc4 <- c(lcc2, lc4)
        }
        res$"4c" <- inla(update(formula, paste(".~.+", add4c)),
            lincomb = lc4, ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "4c") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% c("2d", "3d", "4d"))) {
        lcd2 <- lcd3 <- NULL
        if (lc2.on) {
            lc2args <- list(cbind(Diagonal(n), Diagonal(n, 0)), M2)
            names(lc2args) <- c(sname, stname)
            lcd2 <- do.call("inla.make.lincombs", lc2args)
        }
        if (lc3.on) {
            lc3args <- list(Diagonal(m), M3)
            names(lc3args) <- c(tname, stname)
            lcd3 <- do.call("inla.make.lincombs", lc3args)
        }
        dd <- Diagonal(m * n, diagonal)
    }
    if (any(type %in% "2d")) {
        lcd2args <- list(Diagonal(n * m) - M2[space, ])
        names(lcd2args) <- stname
        lcd <- do.call("inla.make.lincombs", lcd2args)
        add2d <- paste0(
            "f(", stname, ', model="generic0", ',
            "constr=TRUE, rankdef=n, ",
            "Cmatrix=kronecker(R.t, Diagonal(n)) + dd", add0, ")"
        )
        res$"2d" <- inla(update(formula, paste(".~.+", add2d)),
            lincomb = c(lcd2, lcd), ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "2d") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "3d")) {
        lcd3args <- list(Diagonal(n * m) - M3[time, ])
        names(lcd3args) <- stname
        lcd <- do.call("inla.make.lincombs", lcd3args)
        names(lcd) <- gsub("lc", "st", names(lcd))
        add3d <- paste0(
            "f(", stname, ', model="generic0", ',
            "constr=TRUE, rankdef=m, ",
            "Cmatrix=kronecker(Diagonal(m), R.s) + dd", add0, ")"
        )
        res$"3d" <- inla(update(formula, paste(".~.+", add3d)),
            lincomb = c(lcd3, lcd), ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "3d") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (any(type %in% "4d")) {
        lcd3args <- lcd2args <- NULL
        if (lc2.on) {
            lcd2args <- list(cbind(Diagonal(n), Diagonal(n, 0)), M2)
            names(lcd2args) <- c(sname, stname)
            lcd2 <- do.call("inla.make.lincombs", lcd2args)
            names(lcd2) <- gsub("lc", "s", names(lcd2))
        }
        if (lc3.on) {
            lcd3args <- list(Diagonal(m), M3)
            names(lcd3args) <- c(tname, stname)
            lcd3 <- do.call("inla.make.lincombs", lcd3args)
            names(lcd3) <- gsub("lc", "t", names(lcd3))
        }
        lcdargs <- list(Diagonal(n * m) - M2[space, ] - M3[time, ] + 1 / (n * m))
        names(lcdargs) <- stname
        lcd <- do.call("inla.make.lincombs", lcdargs)
        names(lcd) <- gsub("lc", "st", names(lcd))
        add4d <- paste0(
            "f(", stname, ', model="generic0", ',
            "constr=TRUE, rankdef=n+m, ",
            "Cmatrix=kronecker(R.t, R.s) + dd", add0, ")"
        )
        res$"4d" <- inla(update(formula, paste(".~.+", add4d)),
            lincomb = c(lcd2, lcd3, lcd), ...
        )
    }
    if (progress && (length(res) > 0) && tail(names(res), 1) == "4d") {
        cat("type = ", tail(names(res), 1), ", cpu = ", res[[length(res)]]$cpu[4], "\n", sep = "")
    }
    if (length(res) == 1) {
        return(res[[1]])
    }
    return(res)
}

## Export: inla.qstat inla.qdel inla.qget inla.qnuke inla.qlog
## Export: print!inla.q summary!inla.q

## !\name{inla.qstat}
## !\alias{inla.qstat}
## !\alias{inla.q}
## !\alias{inla.qget}
## !\alias{inla.qdel}
## !\alias{inla.qnuke}
## !\alias{inla.qlog}
## !\alias{summary.inla.q}
## !\alias{print.inla.q}
## !\title{Control and view a remote inla-queue}
## !\description{
## !Control and view a remote inla-queue of submitted jobs
## !}
## !\usage{
## !inla.qget(id, remove = TRUE)
## !inla.qdel(id)
## !inla.qstat(id)
## !inla.qlog(id)
## !inla.qnuke()
## !\method{summary}{inla.q}(object,...)
## !\method{print}{inla.q}(x,...)
## !}
## !\arguments{
## !  \item{id}{The job-id which is the output from \code{inla} when the job is submitted,  the
## !            job-number or job-name. For \code{inla.qstat}, \code{id} is optional and if omitted
## !            all the jobs will be listed.}
## !  \item{remove}{Logical If FALSE, leave the job on the server after retrival,
## !                otherwise remove it (default).}
## !  \item{x}{An \code{inla.q}-object which is the output from \code{inla.qstat}}
## !  \item{object}{An \code{inla.q}-object  which is the output from \code{inla.qstat}}
## !  \item{...}{ other arguments.}
## !}
## !\details{
## !\code{inla.qstat} show job(s) on the server,
## !\code{inla.qget} fetch the results (and by default remove
## !the files on the server),  \code{inla.qdel} removes
## !a job on the server and \code{inla.qnuke} remove all jobs on the server.
## !\code{inla.qlog} fetches the logfile only.
## !
## !The recommended procedure is to use \code{r=inla(...,
## !inla.call="submit")} and then do \code{r=inla.qget(r)} at a later
## !stage.  If the job is not finished, then \code{r} will not be
## !overwritten and this step can be repeated.  The reason for this
## !procedure, is that some information usually stored in the result
## !object does not go through the remote server, hence have to be
## !appended to the results that are retrieved from the server. Hence
## !doing \code{r=inla(..., inla.call="submit")} and then later retrive
## !it using \code{r=inla.qget(1)}, say, then \code{r} does not contain
## !all the usual information.  All the main results are there, but
## !administrative information which is required to call
## !\code{inla.hyperpar} or \code{inla.rerun} are not there.
## !}
## !\value{
## !  \code{inla.qstat} returns an \code{inla.q}-object with information about current jobs.
## !}
## !\author{Havard Rue}
## !\seealso{ \code{\link{inla}} }
## !\examples{
## !\dontrun{
## !r = inla(y~1, data = data.frame(y=rnorm(10)), inla.call="submit")
## !inla.qstat()
## !r = inla.qget(r, remove=FALSE)
## !inla.qdel(1)
## !inla.qnuke()
## !}
## !}

`summary.inla.q` <- function(object, ...) {
    print(object, ...)
}

`print.inla.q` <- function(x, ...) {
    for (k in seq_along(x)) {
        cat("Job:", x[[k]]$no, "  Id:", x[[k]]$id, "  Size:", x[[k]]$size, "  Status:", x[[k]]$status, "\n")
    }
    return(invisible(x))
}

`inla.qget` <- function(id, remove = TRUE) {
    return(inla.q(cmd = "get", id = id, remove = remove))
}

`inla.qdel` <- function(id) {
    return(inla.q(cmd = "del", id = id))
}

`inla.qstat` <- function(id) {
    return(inla.q(cmd = "stat", id = id))
}

`inla.qlog` <- function(id) {
    return(inla.q(cmd = "log", id = id))
}

`inla.qnuke` <- function() {
    return(inla.q(cmd = "nuke"))
}

`inla.q` <- function(cmd = c("get", "del", "stat", "log", "nuke"), id, remove = TRUE) {
    cmd <- match.arg(cmd)

    ## do a quick return here if possible
    if (cmd == "log" && missing(id)) {
          return(NULL)
      }

    ## define some environment variables for remote computing
    vars <- c(
        INLA_PATH = system.file("bin", package = "INLA"),
        INLA_OS = inla.os.type(),
        INLA_VERSION = inla.version("version")
    )
    if (inla.os("windows")) {
        vars <- c(vars,
            INLA_SSH_AUTH_SOCK = inla.getOption("ssh.auth.sock"),
            INLA_CYGWIN_HOME = inla.getOption("cygwin.home"),
            INLA_HOME = inla.cygwin.map.filename(gsub("\\\\", "/", inla.get.HOME()))
        )
    } else {
        vars <- c(vars,
            INLA_HOME = inla.get.HOME()
        )
        if (Sys.getenv("SSH_AUTH_SOCK") == "") {
            vars <- c(vars,
                INLA_SSH_AUTH_SOCK = inla.getOption("ssh.auth.sock")
            )
        }
    }
    do.call("Sys.setenv", as.list(vars))

    inla.call <- system.file("bin/remote/inla.q", package = "INLA")
    if (inla.os("windows")) {
        inla.call <- paste(inla.call, ".cygwin", sep = "")
    }

    if (!missing(id) && is.list(id)) {
        ret <- id$ret
        id <- id$id
    } else {
        ret <- NULL
    }

    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        output <- system(paste(shQuote(inla.call), cmd, inla.ifelse(missing(id) || is.null(id), "NULL", id), as.numeric(remove)), intern = TRUE)
    } else {
        bat.file <- paste(tempfile(), ".BAT", sep = "")
        cat("@echo off\n", file = bat.file, append = FALSE)
        cat(paste(shQuote(inla.call), cmd, inla.ifelse(missing(id) || is.null(id), "NULL", id), as.numeric(remove)))
        output <- try(shell(paste("@", shQuote(bat.file), sep = ""), wait = TRUE, intern = TRUE))
        unlink(bat.file)
    }

    if (length(grep("^ERROR", output)) > 0) {
        output <- gsub("^ERROR *", "", output)
        stop(output)
    } else if (length(grep("^DELETE", output)) > 0) {
        output <- gsub("^DELETE *", "", output)
        cat("Delete job", output, "\n")
    } else if (length(grep("^NUKE", output)) > 0) {
        output <- gsub("^NUKE *", "", output)
    } else if (length(grep("^LOG", output)) > 0) {
        output <- gsub("^LOG *", "", output)
        if (file.exists(output)) {
            logfile <- list(logfile = gsub("\t", "        ", readLines(output)))
            try(unlink(output, recursive = TRUE), silent = TRUE)
        } else {
            logfile <- NULL
        }
        return(logfile)
    } else if (length(output) > 0 && length(strsplit(output, " ")[[1]]) == 1) {
        r <- inla.collect.results(output, file.log = paste(output, "/results.files/Logfile.txt", sep = ""))
        rr <- c(r, ret)
        class(rr) <- class(r)
        if (!is.null(ret) && ret$.args$keep == TRUE && !is.null(ret$.args$working.directory)) {
            ## copy the files to the target directory
            d.fnm <- paste(ret$.args$working.directory, "/results.files", sep = "")
            inla.dir.create(d.fnm)
            files.to.copy <- paste(output, "/", dir(output, recursive = TRUE), sep = "")
            file.copy(files.to.copy, d.fnm, recursive = TRUE)
        } else {
            try(unlink(output, recursive = TRUE), silent = TRUE)
        }
        if (!is.null(ret) && ret$.args$keep == FALSE && is.null(ret$.args$working.directory)) {
            try(unlink(ret$misc$inla.dir, recursive = TRUE), silent = TRUE)
        }
        return(rr)
    } else {
        if (length(output) >= 1 && nchar(output[1]) > 0) {
            output <- lapply(
                strsplit(output, " +"),
                function(a) {
                    names(a) <- c("id", "no", "pid", "status", "size")
                    return(as.list(a))
                }
            )
        } else {
            output <- list()
        }
        class(output) <- "inla.q"
        return(output)
    }
}

## Export: inla.get.inlaEnv

## !\name{inla.get.inlaEnv}
## !\alias{inla.get.inlaEnv}
## !\alias{get.inlaEnv}
## !\title{Return the internal environment used by INLA}
## !\description{A function which return the internal environment used by INLA}
## !\usage{
## !inla.get.inlaEnv()
## !}
## !\arguments{
## !}
## !
## !\value{%%
## !  This function returns the internal environment used by INLA to
## !  keep internal variables.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}


## Define the environment for INLA used to store options and the
## model-list. Reuse the environment if it is there already.

if (!(exists(".inlaEnv") && is.environment(.inlaEnv))) {
    .inlaEnv <- new.env()
}

`inla.get.inlaEnv` <- function() {
    if (exists(".inlaEnv") && is.environment(.inlaEnv)) {
          return(.inlaEnv)
      }
    stop("Environment '.inlaEnv' does not exists and is required for INLA to work. Restart 'R'.")
}

## Export: inla.pc.rcor0 inla.pc.dcor0 inla.pc.qcor0 inla.pc.pcor0

## ! \name{pc.cor0}
## ! \alias{inla.pc.cor0}
## ! \alias{pc.cor0}
## ! \alias{pc.rcor0}
## ! \alias{inla.pc.rcor0}
## ! \alias{pc.dcor0}
## ! \alias{inla.pc.dcor0}
## ! \alias{pc.pcor0}
## ! \alias{inla.pc.pcor0}
## ! \alias{pc.qcor0}
## ! \alias{inla.pc.qcor0}
## !
## ! \title{Utility functions for the PC prior for correlation in AR(1)}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the correlation
## !              in the Gaussian AR(1) model where the base-model
## !              is zero correlation.}
## ! \usage{
## ! inla.pc.rcor0(n, u, alpha, lambda)
## ! inla.pc.dcor0(cor, u, alpha, lambda, log = FALSE)
## ! inla.pc.qcor0(p, u, alpha, lambda)
## ! inla.pc.pcor0(q, u, alpha, lambda)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{u}{The upper limit (see Details)}
## !   \item{alpha}{The probability going above the upper limit (see Details)}
## !   \item{lambda}{The rate parameter (see Details)}
## !   \item{cor}{Vector of correlations}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## !     The statement \code{Prob(|cor| > u) = alpha} is used to
## !     determine \code{lambda} unless \code{lambda} is given.
## !     Either \code{lambda}  must be given,  or
## !     \code{u} AND \code{alpha}. The density is symmetric around zero.
## ! }
## !\value{%%
## !  \code{inla.pc.dcor0} gives the density,
## !  \code{inla.pc.pcor0} gives the distribution function,
## !  \code{inla.pc.qcor0} gives the quantile function, and
## !  \code{inla.pc.rcor0} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.rho0")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! cor = inla.pc.rcor0(100,  lambda = 1)
## ! d = inla.pc.dcor0(cor, lambda = 1)
## ! cor = inla.pc.qcor0(c(0.3, 0.7), u = 0.5, alpha=0.01)
## ! inla.pc.pcor0(cor, u = 0.5, alpha=0.01)
## ! }

inla.pc.cor0.lambda <- function(u, alpha, lambda) {
    if (missing(lambda)) {
        stopifnot(!missing(u) && !missing(alpha))
        lambda <- -log(alpha) / sqrt(-log(1 - u^2))
    }
    return(lambda)
}

inla.pc.rcor0 <- function(n, u, alpha, lambda) {
    lambda <- inla.pc.cor0.lambda(u, alpha, lambda)
    d <- rexp(n, rate = lambda)
    sign <- sample(c(-1, 1), size = n, replace = TRUE)
    cor <- sqrt(1 - exp(-d^2)) * sign
    return(cor)
}

inla.pc.dcor0 <- function(cor, u, alpha, lambda, log = FALSE) {
    lambda <- inla.pc.cor0.lambda(u, alpha, lambda)
    mu <- sqrt(-log(1 - cor^2))
    jac <- abs(cor) / mu / (1 - cor^2)
    if (log) {
        return(dexp(mu, rate = lambda, log = TRUE) + log(jac / 2.0))
    } else {
        return(dexp(mu, rate = lambda) * jac / 2.0)
    }
}

inla.pc.qcor0 <- function(p, u, alpha, lambda) {
    lambda <- inla.pc.cor0.lambda(u, alpha, lambda)
    sign <- (p >= 0.5)
    pp <- sign * p + (1 - sign) * (1 - p)
    pp <- 2 * pp - 1
    qq <- qexp(pp, rate = lambda)
    q <- sqrt(1 - exp(-qq^2))
    q <- sign * q + (1 - sign) * (-q)
    return(q)
}

inla.pc.pcor0 <- function(q, u, alpha, lambda) {
    lambda <- inla.pc.cor0.lambda(u, alpha, lambda)
    sign <- (q >= 0)
    qq <- sign * q + (1 - sign) * (-q)
    pp <- sqrt(-log(1 - qq^2))
    pp <- pexp(pp, rate = lambda)
    p <- sign * (1 / 2 + 1 / 2 * pp) + (1 - sign) * (1 / 2 - 1 / 2 * pp)
    return(p)
}

## Nothing to export.

## this function interface with mode = finn in the inlaprogram, which
## computes whatever Finn wants...

`inla.finn` <- function(C, reordering = inla.reorderings(), seed = 0L) {
    reordering <- match.arg(reordering)

    C <- inla.sparse.check(C)

    if (is(C, "dgTMatrix")) {
        finn.file <- inla.write.fmesher.file(C)
        remove <- TRUE
    } else if (is.character(C)) {
        finn.file <- C
        remove <- FALSE
    } else {
        stop("This should not happen.")
    }

    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m finn",
            "-r", reordering, "-z", seed, finn.file
        ), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m finn",
            "-r", reordering, "-z", seed, finn.file
        ), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    if (remove) {
        unlink(finn.file)
    }

    s <- as.numeric(s)
    n <- length(s) %/% 2L

    r <- s[n + (1L:n)] + 1L
    ir <- r
    ir[r] <- 1:length(r)

    return(list(sample = s[1L:n], reordering = r, ireordering = ir))
}

## Export: inla.version

## !\name{inla.version}
## !\alias{version}
## !\alias{inla.version}
## !
## !\title{Show the version of the INLA-package}
## !
## !\description{Show the version of the INLA-package}
## !
## !\usage{
## !inla.version(what = c("default", "version", "date"))
## !}
## !
## !\arguments{
## !  \item{what}{What to show version of}
## !}
## !
## !\value{%%
## !  \code{inla.version} display the current version information using \code{cat}
## !  with
## !  \code{default} or \code{info},  or return
## !  other spesific requests through the call.
## !}
## !%%
## !
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## !## Summary of all
## !inla.version()
## !## The building date
## !inla.version("date")
## !}

`inla.version` <- function(what = c("default", "version", "date")) {

    `trim` <- function(string) {
        string <- gsub("^[^:]*:", "", string)
        string <- gsub("^[ \t]+", "", string)
        string <- gsub("[ \t]+$", "", string)
        return(string)
    }

    desc <- readLines(system.file("DESCRIPTION", package="INLA"))
    version <- trim(desc[grep("^Version:", desc)])
    date <- trim(desc[grep("^Date:", desc)])
    what <- match.arg(what)

    if (what %in% "default") {
        cat(paste("\tR-INLA version ..........: ", version, "\n", sep = ""))
        cat(paste("\tDate ....................: ", date, "\n", sep = ""))
        cat("\tMaintainers .............: Havard Rue <hrue@r-inla.org>\n")
        cat("\t                         : Finn Lindgren <finn.lindgren@gmail.com>\n")
        cat("\t                         : Elias Teixeira Krainski <elias@r-inla.org>\n")
        cat("\tMain web-page ...........: www.r-inla.org\n")
        cat("\tDownload-page ...........: inla.r-inla-download.org\n")
        cat("\tRepository ..............: github.com/hrue/r-inla\n")
        cat("\tEmail support ...........: help@r-inla.org\n")
        cat("\t                         : r-inla-discussion-group@googlegroups.com\n")
        return(invisible())
    } else if (what %in% "date") {
        return(date)
    } else if (what %in% "version") {
        return(version)
    }

    stop("This should not happen.")
}

#' Integrated Nested Laplace Approximation
#'
#' Package to perform full Bayesian analysis on generalised additive mixed
#' models using Integrated Nested Laplace Approximations.
#'
#' \describe{
#' \item{Package}{INLA}
#' \item{Version}{Currently, this package uses a YY.MM.DD versioning
#' system, and is in heavy development.
#' See <https://github.com/hrue/r-inla/> and
#' <https://www.r-inla.org>}
#' \item{License}{GPL2}
#' }
#'
#' See the web-site <https://www.r-inla.org> for further details.
#'
#' @name INLA-package
#' @aliases INLA-package INLA
#' @docType package
#' @author Havard Rue, Sara Martino, Finn Lindgren, Daniel Simpson and Andrea
#' Riebler
NULL





#' Bivariate Meta Analysis
#'
#' Data are taken from a meta-analysis to compare the utility of three types of
#' diagnostic imaging - lymphangiography (LAG), computed tomography (CT) and
#' magnetic resonance (MR) - to detect lymph node metastases in patients with
#' cervical cancer. The dataset consists of a total of 46 studies: the first 17
#' for LAG, the following 19 for CT and the last 10 for MR.
#'
#' @format A data frame with 92 observations on the following 9 variables.
#' \describe{
#' \item{N}{a numeric vector}
#' \item{Y}{a numeric vector}
#' \item{diid}{a numeric vector}
#' \item{lag.tp}{a numeric vector}
#' \item{lag.tn}{a numeric vector}
#' \item{ct.tp}{a numeric vector}
#' \item{ct.tn}{a numeric vector}
#' \item{mr.tp}{a numeric vector}
#' \item{mr.tn}{a numeric vector}
#' }
#' @references J. Scheidler and H. Hricak and K. K. Yu and L. Subak and M. R.
#' Segal,"Radiological evaluation of lymph node metastases in patients with
#' cervical cancer: a meta-analysis",JAMA 1997
#' @keywords datasets
#' @docType data
#' @examples
#'
#' data(BivMetaAnalysis)
"BivMetaAnalysis"





#' ~~ data name/kind ... ~~
#'
#' ~~ A concise (1-5 lines) description of the dataset. ~~
#'
#'
#' @name Cancer
#' @docType data
#' @format A data frame with 6690 observations on the following 4 variables.
#' \describe{
#' \item{Y}{Number of cases}
#' \item{N}{a numeric vector}
#' \item{Age}{a numeric vector}
#' \item{region}{a numeric vector}
#' }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @keywords datasets
NULL





#' Time series with seasonal effect
#'
#' Montly total of car drivers killed or several injuried in England from
#' January 1969 to December 1984
#'
#' NB: The last 12 lines of the data set have the first column set to
#' `NULL` since these data where not observed but we want to predict them.
#'
#'
#' @name Drivers
#' @docType data
#' @format A data frame with 204 observations on the following 4 variables.
#' \describe{
#' \item{y}{Number of deaths}
#' \item{belt}{Indicator of weather the belt was compulsory to use
#' (1) or not (0)}
#' \item{trend}{time (in months)}
#' \item{seasonal}{time (in months)}
#' }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @keywords datasets
#' @examples
#'
#' data(Drivers)
NULL





#' Repeated measures on Poisson counts
#'
#' Seizure counts in a randomised trial of anti-convulsant therpay in epilepsy
#' for 59 patients.
#'
#'
#' @name Epil
#' @docType data
#' @format A data frame with 236 observations on the following 7 variables.
#' \describe{
#' \item{y}{Number of seizures}
#' \item{Trt}{indicator for the presence of treatment}
#' \item{Base}{8-week baseline seizure counts}
#' \item{Age}{Age of the patient}
#' \item{V4}{indicator variable for the 4th visit.}
#' \item{rand}{a numeric vector}
#' \item{Ind}{indicator for the specific patient}
#' }
#' @source WinBUGS/OpenBUGS Manual Examples Vol I
#' @keywords datasets
#' @examples
#'
#' data(Epil)
NULL





#' Disease Mapping
#'
#' Cases of Oral cavity cancer in Germany from 1986-1990
#'
#'
#' @name Germany
#' @docType data
#' @format A data frame with 544 observations on the following 4 variables.
#' \describe{
#' \item{region}{Region of Germany}
#' \item{E}{Fixed quantity which accounts for number of people
#' in the district (offset)}
#' \item{Y}{Number of cases}
#' \item{x}{covariate measuring smoking consumption}
#' }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @keywords datasets
#' @examples
#'
#' data(Germany)
NULL










#' Kidney infection data
#'
#' Times of infection from the time to insertion of the catheter for 38 kindey
#' patients using portable dialysis equipment
#'
#'
#' @name Kidney
#' @docType data
#' @format A data frame with 76 observations on the following 9 variables.
#' \describe{
#' \item{time}{a numeric vector. Time to infection from the
#' insertion of catheter}
#' \item{event}{a numeric vector. 1: time of
#' infection 0: time of censuring }
#' \item{age}{a numeric vector. Age of
#' the patient at the time of infection}
#' \item{sex}{a numeric vector.
#' Sex of the patient 0: male 1:female}
#' \item{disease}{a numeric
#' vector. Type of disease}
#' \item{dis1}{a numeric vector. Dummy
#' variable to codify the disease type.}
#' \item{dis2}{a numeric vector.
#' Dummy variable to codify the disease type.}
#' \item{dis3}{a numeric
#' vector. Dummy variable to codify the disease type.}
#' \item{ID}{a
#' numeric vector. Patient code.}
#' }
#' @references McGilchrist and C.W. Aisbett (1991), Regression with frailty in
#' survival analysis, Biometrics,vol.47,pages 461--166.
#'
#' D.J. Spiegelhalter and A. Thomas and N.G. Best and W.R. Gilks (1995) BUGS:
#' Bayesian Inference Using Gibbs sampling, Version 0.50., MRC Biostatistics
#' Unit, Cambridre, England.
#' @keywords datasets
NULL





#' The Leukemia data
#'
#' This the Leukemia data from Henderson et al (2003); see source.
#'
#'
#' @name Leuk
#' @aliases Leukemia Leuk
#' @docType data
#' @format A data frame with 1043 observations on the following 9 variables.
#' \describe{
#' \item{time}{TODO}
#' \item{cens}{TODO}
#' \item{xcoord}{TODO}
#' \item{ycoord}{TODO}
#' \item{age}{TODO}
#' \item{sex}{TODO}
#' \item{wbc}{TODO}
#' \item{tpi}{TODO}
#' \item{district}{TODO}
#' }
#' @source This is the dataset from
#'
#' Henderson, R. and Shimakura, S. and Gorst, D., 2002, Modeling spatial
#' variation in leukemia survival data, JASA, 97, 460, 965--972.
#' @keywords datasets
#' @examples
#'
#' data(Leuk)
NULL





#' The Munich rent data
#'
#' The Munich rent data
#'
#'
#' @name Munich
#' @docType data
#' @format A data frame with 2035 observations on the following 17 variables.
#' \describe{
#'
#' \item{"rent}{Net rent per square meter.}
#' \item{"floor.size}{Size of the flat in square meters.}
#' \item{"year}{Year of construction of the building in which the flat
#' is located.} \item{location}{Location index (in terms of
#' subquarters).} \item{Gute.Wohnlage}{Dummy variable for good
#' locations / good neighborhoods.} \item{Beste.Wohnlage}{Dummy
#' variable for very good locations / very good neighborhoods.}
#' \item{Keine.Wwv}{Dummy for absence of warm water supply.}
#' \item{Keine.Zh}{Dummy for absence of central heating system.}
#' \item{Kein.Badkach}{Dummy for absence of flagging in the bathroom.}
#' \item{Besond.Bad}{Dummy for special features of the bathroom.}
#' \item{Gehobene.Kueche}{Dummy for more refined kitchen equipment.}
#' \item{zim1}{Dummy for a flat with 1 room.} \item{zim2}{Dummy
#' for a flat with 2 rooms.} \item{zim3}{Dummy for a flat with 3
#' rooms.} \item{zim4}{Dummy for a flat with 4 rooms.}
#' \item{zim5}{Dummy for a flat with 5 rooms.}
#' \item{zim6}{Dummy for a flat with 6 rooms.}
#'
#' }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @source See Rue and Held (2005), Chapter 4.
#' @keywords datasets
NULL





#' The New England map
#'
#' This map is used in association to the Leukemia data from Henderson et al
#' (2003); see source.
#'
#'
#' @name nwEngland
#' @aliases nwEngland NewEngland
#' @docType data
#' @format A SpatialPolygons object.
#' @source This map are used to analyse the Leukaemia dataset from
#'
#' Henderson, R. and Shimakura, S. and Gorst, D., 2002, Modeling spatial
#' variation in leukemia survival data, JASA, 97, 460, 965--972.
#' @keywords datasets
#' @examples
#'
#' data(Leuk)
#' plot(nwEngland)
NULL





#' ~~ data name/kind ... ~~
#'
#' ~~ A concise (1-5 lines) description of the dataset. ~~
#'
#'
#' @name Oral
#' @docType data
#' @format A data frame with 544 observations on the following 3 variables.
#' \describe{ \item{region}{a numeric vector} \item{E}{a
#' numeric vector} \item{Y}{a numeric vector} }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @keywords datasets
NULL





#' The PRborder data
#'
#' A data matrix with Longitude and Latitude coordinates for the boundary of
#' Parana State.
#'
#'
#' @name PRborder
#' @aliases PRborder PRborder
#' @docType data
#' @format \describe{ \item{Longtiude}{The Longtiude coordinate}
#' \item{Latitude}{The Latitude coordinate} }
#' @seealso PRprec
#' @keywords datasets
NULL





#' The PRprec data
#'
#' A data frame with daily rainfall in the Parana State.
#'
#'
#' @name PRprec
#' @aliases PRprec PRprec
#' @docType data
#' @format A data frame .... TODO
#' \describe{
#' \item{Altitude}{TODO}
#' \item{Latitude}{TODO}
#' \item{Longitude}{TODO}
#' \item{d0101}{Daily rainfall at day "mmdd"}
#' \item{d0102}{Daily rainfall at day "mmdd"}
#' \item{d0103}{Daily rainfall at day "mmdd"}
#' \item{d0104}{Daily rainfall at day "mmdd"}
#' \item{d0105}{Daily rainfall at day "mmdd"}
#' \item{d0106}{Daily rainfall at day "mmdd"}
#' \item{d0107}{Daily rainfall at day "mmdd"} \item{d0108}{Daily
#' rainfall at day "mmdd"} \item{d0109}{Daily rainfall at day "mmdd"}
#' \item{d0110}{Daily rainfall at day "mmdd"} \item{d0111}{Daily rainfall at
#' day "mmdd"} \item{d0112}{Daily rainfall at day "mmdd"} \item{d0113}{Daily
#' rainfall at day "mmdd"} \item{d0114}{Daily rainfall at day "mmdd"}
#' \item{d0115}{Daily rainfall at day "mmdd"} \item{d0116}{Daily rainfall at
#' day "mmdd"} \item{d0117}{Daily rainfall at day "mmdd"} \item{d0118}{Daily
#' rainfall at day "mmdd"} \item{d0119}{Daily rainfall at day "mmdd"}
#' \item{d0120}{Daily rainfall at day "mmdd"} \item{d0121}{Daily rainfall at
#' day "mmdd"} \item{d0122}{Daily rainfall at day "mmdd"} \item{d0123}{Daily
#' rainfall at day "mmdd"} \item{d0124}{Daily rainfall at day "mmdd"}
#' \item{d0125}{Daily rainfall at day "mmdd"} \item{d0126}{Daily rainfall at
#' day "mmdd"} \item{d0127}{Daily rainfall at day "mmdd"} \item{d0128}{Daily
#' rainfall at day "mmdd"} \item{d0129}{Daily rainfall at day "mmdd"}
#' \item{d0130}{Daily rainfall at day "mmdd"} \item{d0131}{Daily rainfall at
#' day "mmdd"} \item{d0201}{Daily rainfall at day "mmdd"} \item{d0202}{Daily
#' rainfall at day "mmdd"} \item{d0203}{Daily rainfall at day "mmdd"}
#' \item{d0204}{Daily rainfall at day "mmdd"} \item{d0205}{Daily rainfall at
#' day "mmdd"} \item{d0206}{Daily rainfall at day "mmdd"} \item{d0207}{Daily
#' rainfall at day "mmdd"} \item{d0208}{Daily rainfall at day "mmdd"}
#' \item{d0209}{Daily rainfall at day "mmdd"} \item{d0210}{Daily rainfall at
#' day "mmdd"} \item{d0211}{Daily rainfall at day "mmdd"} \item{d0212}{Daily
#' rainfall at day "mmdd"} \item{d0213}{Daily rainfall at day "mmdd"}
#' \item{d0214}{Daily rainfall at day "mmdd"} \item{d0215}{Daily rainfall at
#' day "mmdd"} \item{d0216}{Daily rainfall at day "mmdd"} \item{d0217}{Daily
#' rainfall at day "mmdd"} \item{d0218}{Daily rainfall at day "mmdd"}
#' \item{d0219}{Daily rainfall at day "mmdd"} \item{d0220}{Daily rainfall at
#' day "mmdd"} \item{d0221}{Daily rainfall at day "mmdd"} \item{d0222}{Daily
#' rainfall at day "mmdd"} \item{d0223}{Daily rainfall at day "mmdd"}
#' \item{d0224}{Daily rainfall at day "mmdd"} \item{d0225}{Daily rainfall at
#' day "mmdd"} \item{d0226}{Daily rainfall at day "mmdd"} \item{d0227}{Daily
#' rainfall at day "mmdd"} \item{d0228}{Daily rainfall at day "mmdd"}
#' \item{d0301}{Daily rainfall at day "mmdd"} \item{d0302}{Daily rainfall at
#' day "mmdd"} \item{d0303}{Daily rainfall at day "mmdd"} \item{d0304}{Daily
#' rainfall at day "mmdd"} \item{d0305}{Daily rainfall at day "mmdd"}
#' \item{d0306}{Daily rainfall at day "mmdd"} \item{d0307}{Daily rainfall at
#' day "mmdd"} \item{d0308}{Daily rainfall at day "mmdd"} \item{d0309}{Daily
#' rainfall at day "mmdd"} \item{d0310}{Daily rainfall at day "mmdd"}
#' \item{d0311}{Daily rainfall at day "mmdd"} \item{d0312}{Daily rainfall at
#' day "mmdd"} \item{d0313}{Daily rainfall at day "mmdd"} \item{d0314}{Daily
#' rainfall at day "mmdd"} \item{d0315}{Daily rainfall at day "mmdd"}
#' \item{d0316}{Daily rainfall at day "mmdd"} \item{d0317}{Daily rainfall at
#' day "mmdd"} \item{d0318}{Daily rainfall at day "mmdd"} \item{d0319}{Daily
#' rainfall at day "mmdd"} \item{d0320}{Daily rainfall at day "mmdd"}
#' \item{d0321}{Daily rainfall at day "mmdd"} \item{d0322}{Daily rainfall at
#' day "mmdd"} \item{d0323}{Daily rainfall at day "mmdd"} \item{d0324}{Daily
#' rainfall at day "mmdd"} \item{d0325}{Daily rainfall at day "mmdd"}
#' \item{d0326}{Daily rainfall at day "mmdd"} \item{d0327}{Daily rainfall at
#' day "mmdd"} \item{d0328}{Daily rainfall at day "mmdd"} \item{d0329}{Daily
#' rainfall at day "mmdd"} \item{d0330}{Daily rainfall at day "mmdd"}
#' \item{d0331}{Daily rainfall at day "mmdd"} \item{d0401}{Daily rainfall at
#' day "mmdd"} \item{d0402}{Daily rainfall at day "mmdd"} \item{d0403}{Daily
#' rainfall at day "mmdd"} \item{d0404}{Daily rainfall at day "mmdd"}
#' \item{d0405}{Daily rainfall at day "mmdd"} \item{d0406}{Daily rainfall at
#' day "mmdd"} \item{d0407}{Daily rainfall at day "mmdd"} \item{d0408}{Daily
#' rainfall at day "mmdd"} \item{d0409}{Daily rainfall at day "mmdd"}
#' \item{d0410}{Daily rainfall at day "mmdd"} \item{d0411}{Daily rainfall at
#' day "mmdd"} \item{d0412}{Daily rainfall at day "mmdd"} \item{d0413}{Daily
#' rainfall at day "mmdd"} \item{d0414}{Daily rainfall at day "mmdd"}
#' \item{d0415}{Daily rainfall at day "mmdd"} \item{d0416}{Daily rainfall at
#' day "mmdd"} \item{d0417}{Daily rainfall at day "mmdd"} \item{d0418}{Daily
#' rainfall at day "mmdd"} \item{d0419}{Daily rainfall at day "mmdd"}
#' \item{d0420}{Daily rainfall at day "mmdd"} \item{d0421}{Daily rainfall at
#' day "mmdd"} \item{d0422}{Daily rainfall at day "mmdd"} \item{d0423}{Daily
#' rainfall at day "mmdd"} \item{d0424}{Daily rainfall at day "mmdd"}
#' \item{d0425}{Daily rainfall at day "mmdd"} \item{d0426}{Daily rainfall at
#' day "mmdd"} \item{d0427}{Daily rainfall at day "mmdd"} \item{d0428}{Daily
#' rainfall at day "mmdd"} \item{d0429}{Daily rainfall at day "mmdd"}
#' \item{d0430}{Daily rainfall at day "mmdd"} \item{d0501}{Daily rainfall at
#' day "mmdd"} \item{d0502}{Daily rainfall at day "mmdd"} \item{d0503}{Daily
#' rainfall at day "mmdd"} \item{d0504}{Daily rainfall at day "mmdd"}
#' \item{d0505}{Daily rainfall at day "mmdd"} \item{d0506}{Daily rainfall at
#' day "mmdd"} \item{d0507}{Daily rainfall at day "mmdd"} \item{d0508}{Daily
#' rainfall at day "mmdd"} \item{d0509}{Daily rainfall at day "mmdd"}
#' \item{d0510}{Daily rainfall at day "mmdd"} \item{d0511}{Daily rainfall at
#' day "mmdd"} \item{d0512}{Daily rainfall at day "mmdd"} \item{d0513}{Daily
#' rainfall at day "mmdd"} \item{d0514}{Daily rainfall at day "mmdd"}
#' \item{d0515}{Daily rainfall at day "mmdd"} \item{d0516}{Daily rainfall at
#' day "mmdd"} \item{d0517}{Daily rainfall at day "mmdd"} \item{d0518}{Daily
#' rainfall at day "mmdd"} \item{d0519}{Daily rainfall at day "mmdd"}
#' \item{d0520}{Daily rainfall at day "mmdd"} \item{d0521}{Daily rainfall at
#' day "mmdd"} \item{d0522}{Daily rainfall at day "mmdd"} \item{d0523}{Daily
#' rainfall at day "mmdd"} \item{d0524}{Daily rainfall at day "mmdd"}
#' \item{d0525}{Daily rainfall at day "mmdd"} \item{d0526}{Daily rainfall at
#' day "mmdd"} \item{d0527}{Daily rainfall at day "mmdd"} \item{d0528}{Daily
#' rainfall at day "mmdd"} \item{d0529}{Daily rainfall at day "mmdd"}
#' \item{d0530}{Daily rainfall at day "mmdd"} \item{d0531}{Daily rainfall at
#' day "mmdd"} \item{d0601}{Daily rainfall at day "mmdd"} \item{d0602}{Daily
#' rainfall at day "mmdd"} \item{d0603}{Daily rainfall at day "mmdd"}
#' \item{d0604}{Daily rainfall at day "mmdd"} \item{d0605}{Daily rainfall at
#' day "mmdd"} \item{d0606}{Daily rainfall at day "mmdd"} \item{d0607}{Daily
#' rainfall at day "mmdd"} \item{d0608}{Daily rainfall at day "mmdd"}
#' \item{d0609}{Daily rainfall at day "mmdd"} \item{d0610}{Daily rainfall at
#' day "mmdd"} \item{d0611}{Daily rainfall at day "mmdd"} \item{d0612}{Daily
#' rainfall at day "mmdd"} \item{d0613}{Daily rainfall at day "mmdd"}
#' \item{d0614}{Daily rainfall at day "mmdd"} \item{d0615}{Daily rainfall at
#' day "mmdd"} \item{d0616}{Daily rainfall at day "mmdd"} \item{d0617}{Daily
#' rainfall at day "mmdd"} \item{d0618}{Daily rainfall at day "mmdd"}
#' \item{d0619}{Daily rainfall at day "mmdd"} \item{d0620}{Daily rainfall at
#' day "mmdd"} \item{d0621}{Daily rainfall at day "mmdd"} \item{d0622}{Daily
#' rainfall at day "mmdd"} \item{d0623}{Daily rainfall at day "mmdd"}
#' \item{d0624}{Daily rainfall at day "mmdd"} \item{d0625}{Daily rainfall at
#' day "mmdd"} \item{d0626}{Daily rainfall at day "mmdd"} \item{d0627}{Daily
#' rainfall at day "mmdd"} \item{d0628}{Daily rainfall at day "mmdd"}
#' \item{d0629}{Daily rainfall at day "mmdd"} \item{d0630}{Daily rainfall at
#' day "mmdd"} \item{d0701}{Daily rainfall at day "mmdd"} \item{d0702}{Daily
#' rainfall at day "mmdd"} \item{d0703}{Daily rainfall at day "mmdd"}
#' \item{d0704}{Daily rainfall at day "mmdd"} \item{d0705}{Daily rainfall at
#' day "mmdd"} \item{d0706}{Daily rainfall at day "mmdd"} \item{d0707}{Daily
#' rainfall at day "mmdd"} \item{d0708}{Daily rainfall at day "mmdd"}
#' \item{d0709}{Daily rainfall at day "mmdd"} \item{d0710}{Daily rainfall at
#' day "mmdd"} \item{d0711}{Daily rainfall at day "mmdd"} \item{d0712}{Daily
#' rainfall at day "mmdd"} \item{d0713}{Daily rainfall at day "mmdd"}
#' \item{d0714}{Daily rainfall at day "mmdd"} \item{d0715}{Daily rainfall at
#' day "mmdd"} \item{d0716}{Daily rainfall at day "mmdd"} \item{d0717}{Daily
#' rainfall at day "mmdd"} \item{d0718}{Daily rainfall at day "mmdd"}
#' \item{d0719}{Daily rainfall at day "mmdd"} \item{d0720}{Daily rainfall at
#' day "mmdd"} \item{d0721}{Daily rainfall at day "mmdd"} \item{d0722}{Daily
#' rainfall at day "mmdd"} \item{d0723}{Daily rainfall at day "mmdd"}
#' \item{d0724}{Daily rainfall at day "mmdd"} \item{d0725}{Daily rainfall at
#' day "mmdd"} \item{d0726}{Daily rainfall at day "mmdd"} \item{d0727}{Daily
#' rainfall at day "mmdd"} \item{d0728}{Daily rainfall at day "mmdd"}
#' \item{d0729}{Daily rainfall at day "mmdd"} \item{d0730}{Daily rainfall at
#' day "mmdd"} \item{d0731}{Daily rainfall at day "mmdd"} \item{d0801}{Daily
#' rainfall at day "mmdd"} \item{d0802}{Daily rainfall at day "mmdd"}
#' \item{d0803}{Daily rainfall at day "mmdd"} \item{d0804}{Daily rainfall at
#' day "mmdd"} \item{d0805}{Daily rainfall at day "mmdd"} \item{d0806}{Daily
#' rainfall at day "mmdd"} \item{d0807}{Daily rainfall at day "mmdd"}
#' \item{d0808}{Daily rainfall at day "mmdd"} \item{d0809}{Daily rainfall at
#' day "mmdd"} \item{d0810}{Daily rainfall at day "mmdd"} \item{d0811}{Daily
#' rainfall at day "mmdd"} \item{d0812}{Daily rainfall at day "mmdd"}
#' \item{d0813}{Daily rainfall at day "mmdd"} \item{d0814}{Daily rainfall at
#' day "mmdd"} \item{d0815}{Daily rainfall at day "mmdd"} \item{d0816}{Daily
#' rainfall at day "mmdd"} \item{d0817}{Daily rainfall at day "mmdd"}
#' \item{d0818}{Daily rainfall at day "mmdd"} \item{d0819}{Daily rainfall at
#' day "mmdd"} \item{d0820}{Daily rainfall at day "mmdd"} \item{d0821}{Daily
#' rainfall at day "mmdd"} \item{d0822}{Daily rainfall at day "mmdd"}
#' \item{d0823}{Daily rainfall at day "mmdd"} \item{d0824}{Daily rainfall at
#' day "mmdd"} \item{d0825}{Daily rainfall at day "mmdd"} \item{d0826}{Daily
#' rainfall at day "mmdd"} \item{d0827}{Daily rainfall at day "mmdd"}
#' \item{d0828}{Daily rainfall at day "mmdd"} \item{d0829}{Daily rainfall at
#' day "mmdd"} \item{d0830}{Daily rainfall at day "mmdd"} \item{d0831}{Daily
#' rainfall at day "mmdd"} \item{d0901}{Daily rainfall at day "mmdd"}
#' \item{d0902}{Daily rainfall at day "mmdd"} \item{d0903}{Daily rainfall at
#' day "mmdd"} \item{d0904}{Daily rainfall at day "mmdd"} \item{d0905}{Daily
#' rainfall at day "mmdd"} \item{d0906}{Daily rainfall at day "mmdd"}
#' \item{d0907}{Daily rainfall at day "mmdd"} \item{d0908}{Daily rainfall at
#' day "mmdd"} \item{d0909}{Daily rainfall at day "mmdd"} \item{d0910}{Daily
#' rainfall at day "mmdd"} \item{d0911}{Daily rainfall at day "mmdd"}
#' \item{d0912}{Daily rainfall at day "mmdd"} \item{d0913}{Daily rainfall at
#' day "mmdd"} \item{d0914}{Daily rainfall at day "mmdd"} \item{d0915}{Daily
#' rainfall at day "mmdd"} \item{d0916}{Daily rainfall at day "mmdd"}
#' \item{d0917}{Daily rainfall at day "mmdd"} \item{d0918}{Daily rainfall at
#' day "mmdd"} \item{d0919}{Daily rainfall at day "mmdd"} \item{d0920}{Daily
#' rainfall at day "mmdd"} \item{d0921}{Daily rainfall at day "mmdd"}
#' \item{d0922}{Daily rainfall at day "mmdd"} \item{d0923}{Daily rainfall at
#' day "mmdd"} \item{d0924}{Daily rainfall at day "mmdd"} \item{d0925}{Daily
#' rainfall at day "mmdd"} \item{d0926}{Daily rainfall at day "mmdd"}
#' \item{d0927}{Daily rainfall at day "mmdd"} \item{d0928}{Daily rainfall at
#' day "mmdd"} \item{d0929}{Daily rainfall at day "mmdd"} \item{d0930}{Daily
#' rainfall at day "mmdd"} \item{d1001}{Daily rainfall at day "mmdd"}
#' \item{d1002}{Daily rainfall at day "mmdd"} \item{d1003}{Daily rainfall at
#' day "mmdd"} \item{d1004}{Daily rainfall at day "mmdd"} \item{d1005}{Daily
#' rainfall at day "mmdd"} \item{d1006}{Daily rainfall at day "mmdd"}
#' \item{d1007}{Daily rainfall at day "mmdd"} \item{d1008}{Daily rainfall at
#' day "mmdd"} \item{d1009}{Daily rainfall at day "mmdd"} \item{d1010}{Daily
#' rainfall at day "mmdd"} \item{d1011}{Daily rainfall at day "mmdd"}
#' \item{d1012}{Daily rainfall at day "mmdd"} \item{d1013}{Daily rainfall at
#' day "mmdd"} \item{d1014}{Daily rainfall at day "mmdd"} \item{d1015}{Daily
#' rainfall at day "mmdd"} \item{d1016}{Daily rainfall at day "mmdd"}
#' \item{d1017}{Daily rainfall at day "mmdd"} \item{d1018}{Daily rainfall at
#' day "mmdd"} \item{d1019}{Daily rainfall at day "mmdd"} \item{d1020}{Daily
#' rainfall at day "mmdd"} \item{d1021}{Daily rainfall at day "mmdd"}
#' \item{d1022}{Daily rainfall at day "mmdd"} \item{d1023}{Daily rainfall at
#' day "mmdd"} \item{d1024}{Daily rainfall at day "mmdd"} \item{d1025}{Daily
#' rainfall at day "mmdd"} \item{d1026}{Daily rainfall at day "mmdd"}
#' \item{d1027}{Daily rainfall at day "mmdd"} \item{d1028}{Daily rainfall at
#' day "mmdd"} \item{d1029}{Daily rainfall at day "mmdd"} \item{d1030}{Daily
#' rainfall at day "mmdd"} \item{d1031}{Daily rainfall at day "mmdd"}
#' \item{d1101}{Daily rainfall at day "mmdd"} \item{d1102}{Daily rainfall at
#' day "mmdd"} \item{d1103}{Daily rainfall at day "mmdd"} \item{d1104}{Daily
#' rainfall at day "mmdd"} \item{d1105}{Daily rainfall at day "mmdd"}
#' \item{d1106}{Daily rainfall at day "mmdd"} \item{d1107}{Daily rainfall at
#' day "mmdd"} \item{d1108}{Daily rainfall at day "mmdd"} \item{d1109}{Daily
#' rainfall at day "mmdd"} \item{d1110}{Daily rainfall at day "mmdd"}
#' \item{d1111}{Daily rainfall at day "mmdd"} \item{d1112}{Daily rainfall at
#' day "mmdd"} \item{d1113}{Daily rainfall at day "mmdd"} \item{d1114}{Daily
#' rainfall at day "mmdd"} \item{d1115}{Daily rainfall at day "mmdd"}
#' \item{d1116}{Daily rainfall at day "mmdd"} \item{d1117}{Daily rainfall at
#' day "mmdd"} \item{d1118}{Daily rainfall at day "mmdd"} \item{d1119}{Daily
#' rainfall at day "mmdd"} \item{d1120}{Daily rainfall at day "mmdd"}
#' \item{d1121}{Daily rainfall at day "mmdd"} \item{d1122}{Daily rainfall at
#' day "mmdd"} \item{d1123}{Daily rainfall at day "mmdd"} \item{d1124}{Daily
#' rainfall at day "mmdd"} \item{d1125}{Daily rainfall at day "mmdd"}
#' \item{d1126}{Daily rainfall at day "mmdd"} \item{d1127}{Daily rainfall at
#' day "mmdd"} \item{d1128}{Daily rainfall at day "mmdd"} \item{d1129}{Daily
#' rainfall at day "mmdd"} \item{d1130}{Daily rainfall at day "mmdd"}
#' \item{d1201}{Daily rainfall at day "mmdd"} \item{d1202}{Daily rainfall at
#' day "mmdd"} \item{d1203}{Daily rainfall at day "mmdd"} \item{d1204}{Daily
#' rainfall at day "mmdd"} \item{d1205}{Daily rainfall at day "mmdd"}
#' \item{d1206}{Daily rainfall at day "mmdd"} \item{d1207}{Daily rainfall at
#' day "mmdd"} \item{d1208}{Daily rainfall at day "mmdd"} \item{d1209}{Daily
#' rainfall at day "mmdd"} \item{d1210}{Daily rainfall at day "mmdd"}
#' \item{d1211}{Daily rainfall at day "mmdd"} \item{d1212}{Daily rainfall at
#' day "mmdd"} \item{d1213}{Daily rainfall at day "mmdd"} \item{d1214}{Daily
#' rainfall at day "mmdd"} \item{d1215}{Daily rainfall at day "mmdd"}
#' \item{d1216}{Daily rainfall at day "mmdd"} \item{d1217}{Daily rainfall at
#' day "mmdd"} \item{d1218}{Daily rainfall at day "mmdd"} \item{d1219}{Daily
#' rainfall at day "mmdd"} \item{d1220}{Daily rainfall at day "mmdd"}
#' \item{d1221}{Daily rainfall at day "mmdd"} \item{d1222}{Daily rainfall at
#' day "mmdd"} \item{d1223}{Daily rainfall at day "mmdd"} \item{d1224}{Daily
#' rainfall at day "mmdd"} \item{d1225}{Daily rainfall at day "mmdd"}
#' \item{d1226}{Daily rainfall at day "mmdd"} \item{d1227}{Daily rainfall at
#' day "mmdd"} \item{d1228}{Daily rainfall at day "mmdd"} \item{d1229}{Daily
#' rainfall at day "mmdd"} \item{d1230}{Daily rainfall at day "mmdd"}
#' \item{d1231}{Daily rainfall at day "mmdd"} }
#' @seealso PRborder
#' @keywords datasets
NULL





#' Extra-Poisson variation in dose-response study
#'
#' Breslow (1984) analyses some mutagenicity assay data (shown below) on
#' salmonella in which three plates have been processed at each dose i of
#' quinoline and the number of revertant colonies of TA98 Salmonella measured
#'
#'
#' @name Salm
#' @docType data
#' @format A data frame with 18 observations on the following 3 variables.
#' \describe{ \item{y}{number of salmonella bacteria}
#' \item{dose}{dose of quinoline (mg per plate)}
#' \item{rand}{indicator} }
#' @source WinBUGS/OpenBUGS manual Examples VOl.I
#' @keywords datasets
#' @examples
#'
#' data(Salm)
NULL





#' Conditional Autoregressive (CAR) model for disease mapping
#'
#'
#' The rate of lip cancer in 56 counties in Scotland is recorder. The data set
#' includes the observed and expected cases (based on the population and its
#' age and sex distribution in the country), a covariate measuring the
#' percentage of the population engaged in agricolture, fishing or forestry and
#' the "position" of each county expressed as a list of adjacent counties
#'
#'
#' @name Scotland
#' @docType data
#' @format A data frame with 56 observations on the following 4 variables.
#' \describe{ \item{Counts}{The number of lip cancer registered}
#' \item{E}{The expected number of lip cancer } \item{X}{The
#' percentage of the population engaged in agricolture, fishing or forestry }
#' \item{Region}{The county} }
#' @references
#'
#' Clayton and Kaldor (1987) and Breslow and Clayton (1993)
#' @source OpenBUGS Example manual, GeoBUGS
#' @keywords datasets
#' @examples
#'
#' data(Scotland)
NULL





#' Factorial design
#'
#'
#' Proportion of seeds that germinated on each of 21 plates arranged according
#' to a 2 by 2 factorial layout by seed and type of root extract
#'
#'
#' @name Seeds
#' @docType data
#' @format A data frame with 21 observations on the following 5 variables.
#' \describe{ \item{r}{number of germinated seeds per plate}
#' \item{n}{number of total seeds per plate} \item{x1}{seed
#' type} \item{x2}{root extracted} \item{plate}{indicator for
#' the plate} }
#' @source WinBUGS/OpenBUGS Manual Example, Vol. I
#' @keywords datasets
#' @examples
#'
#' data(Seeds)
NULL





#' toy simulated data set for the SPDE tutorial
#'
#' Simulated data set on 200 location points.  The simulation process is made
#' at the introduction of the SPDE tutorial.
#'
#'
#' @name SPDEtoy
#' @docType data
#' @format A data frame with 200 observations on the following 3 variables.
#' \describe{ \item{s1}{First element of the coordinates}
#' \item{s2}{Second element of the coordinates} \item{y}{data
#' simulated at the locations} }
#' @source SPDE tutorial
#' @keywords datasets
#' @examples
#'
#' data(SPDEtoy)
NULL





#' Surgical: Institutional ranking
#'
#' This example considers mortality rates in 12 hospitals performing cardiac
#' surgery in babies
#'
#'
#' @name Surg
#' @docType data
#' @format A data frame with 12 observations on the following 3 variables.
#' \describe{ \item{n}{Number of deaths} \item{r}{Total number
#' of cases} \item{hospital}{a factor with levels `A` `B`
#' `C` `D` `E` `F` `G` `H` `I` `J`
#' `K` `L`} }
#' @source WinBUGS/OpenBUGS Manual Examples Vol. I
#' @keywords datasets
#' @examples
#'
#' data(Surg)
NULL





#' Survival data
#'
#' Simulated data set for Weibull survival model
#'
#'
#' @name SurvSim
#' @docType data
#' @format A data frame with 100 observations on the following 3 variables.
#' \describe{ \item{y}{a numeric vector of survival times}
#' \item{cens}{a numeric vector of event indicator (0=censured
#' 1=failure)} \item{x}{a numeric vector of covariate } }
#' @keywords datasets
NULL





#' Binomial time series
#'
#' Recorded days of rain above 1 mm in Tokyo for 2 years, 1983:84
#'
#'
#' @name Tokyo
#' @docType data
#' @format A data frame with 366 observations on the following 3 variables.
#' \describe{
#' \item{y}{number of days with rain}
#' \item{n}{total number of days}
#' \item{time}{day of the year}
#' }
#' @references Rue, H and Held, L. (2005) *Gaussian Markov Random Fields -
#' Theory and Applications* Chapman and Hall
#' @source
#' <http://www.math.ntnu.no/~hrue/GMRF-book/tokyo.rainfall.data.dat>
#' @keywords datasets
#' @examples
#'
#' data(Tokyo)
NULL





#' Semiparametric regression
#'
#' Undernutrition of children in each region of Zambia is measured through a
#' score computed on the basis of some anthropometric measures. The data set
#' contains also other infomation about each child.
#'
#'
#' @name Zambia
#' @docType data
#' @format A data frame with 4847 observations on the following 10 variables.
#' \describe{ \item{hazstd}{standardised Z score of stunting}
#' \item{bmi}{body mass index of the mother} \item{agc}{age of
#' the child in months} \item{district}{district where the child lives}
#' \item{rcw}{mother employment status with categories "working" (1)
#' and "not working" (-1)} \item{edu1}{mother's educations status with
#' categories "complete primary but incomplete secondary " (`edu1=1)`,
#' "complete secondary or higher" (`edu2=1`) and "no education or
#' incomplete primary" (`edu1=edu2=-1`)} \item{edu2}{see above}
#' \item{tpr}{locality of the domicile with categories "urban" (1) and
#' "rural" (-1)} \item{sex}{gender of the child with categories "male"
#' (1) and "female" (-1)} \item{edu}{DO NOT KNOW; check source} }
#' @source BayesX Manual
#' <http://www.stat.uni-muenchen.de/~bayesx/bayesx.html>
#' @keywords datasets
#' @examples
#'
#' data(Zambia)
NULL

## Export: inla.coxph
## Export: inla.rbind.data.frames

## !\name{inla.coxph}
## !\alias{inla.coxph}
## !\alias{inla.rbind.data.frames}
## !\alias{coxph}
## !\alias{cbind.data.frames}
## !\title{Convert a Cox proportional hazard model into Poisson regression}
## !\description{Tools to convert a Cox proportional hazard model into Poisson regression}
## !\usage{
## !    inla.coxph(formula, data, control.hazard = list(), tag="", debug=FALSE)
## !    inla.rbind.data.frames(...)
## !}
## !\arguments{
## !  \item{formula}{The formula for the coxph model where the reponse must be a \code{inla.surv}-object.}
## !  \item{data}{All the data used in the formula,  as a list.}
## !  \item{control.hazard}{Control the model for the baseline-hazard; see \code{?control.hazard}.}
## !  \item{tag}{An optional tag added to the names of the new variables created (to make them
## !             unique when combined with several calls of \code{inla.coxph}. Note that 
## !             \code{E..coxph} is not included,  as its usually merged into one 
## !             vector over different expansions.}
## !  \item{debug}{Print debug-information}
## !  \item{...}{Data.frames to be \code{rbind}-ed,  padding with \code{NA}.}
## !}
## !\value{
## !      \code{inla.coxph} returns a list of new expanded variables to be used in the \code{inla}-call.
## !      Note that element \code{data} and \code{data.list} needs to be merged into
## !      a \code{list} to be passed as the \code{data} argument. See the example for details.
## !
## !      \code{inla.rbind.data.frames} returns the rbinded data.frames padded with NAs.
## !      There is a better
## !      implementation in \code{dplyr::bind_rows}, which is used if package \code{dplyr} is
## !      installed.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\examples{
## !## How the cbind.data.frames works:
## !df1 = data.frame(x=1:2, y=2:3, z=3:4)
## !df2 = data.frame(x=3:4, yy=4:5, zz=5:6)
## !inla.rbind.data.frames(df1, df2)
## !
## !## Standard example of how to convert a coxph into a Poisson regression
## !n = 1000
## !x = runif(n)
## !lambda = exp(1+x)
## !y = rexp(n, rate=lambda)
## !event = rep(1,n)
## !data = list(y=y, event=event, x=x)
## !y.surv = inla.surv(y, event)
## !intercept1 = rep(1, n)
## !p = inla.coxph(y.surv ~ -1 + intercept1 + x,
## !               list(y.surv = y.surv,  x=x, intercept1 = intercept1))
## !
## !r = inla(p$formula,
## !        family = p$family,
## !        data=c(as.list(p$data), p$data.list),
## !        E = p$E)
## !summary(r)
## !
## !## How to use this in a joint model
## !intercept2 = rep(1, n)
## !y = 1 + x + rnorm(n, sd=0.1)
## !df = data.frame(intercept2, x, y)
## !
## !## new need to cbind the data.frames, and then add the list-part of
## !## the data
## !df.joint = c(as.list(inla.rbind.data.frames(p$data, df)), p$data.list)
## !df.joint$Y = cbind(df.joint$y..coxph, df.joint$y)
## !
## !## merge the formulas, recall to add '-1' and to use the new joint
## !## reponse 'Y'
## !formula = update(p$formula, Y ~ intercept2 -1 + .)
## !
## !rr = inla(formula,
## !        family = c(p$family, "gaussian"),
## !        data = df.joint,
## !        E = df.joint$E..coxph)
## !}

`inla.coxph` <- function(formula, data, control.hazard = list(), debug = FALSE, tag = "") 
{
    ## convert a coxph-model into poisson-regression and return a new
    ## data-list with the expand variables and new variables to use in
    ## the poisson regression

    if (is.data.frame(data)) {
        data <- as.list(data)
    }

    name.y <- inla.formula2character(formula[2])
    tmp <- (names(data) %in% name.y)
    y.surv <- NULL
    if (any(tmp)) {
        if (sum(tmp) > 1) {
            stop(paste(c("Several entries in 'data' match the name of the response:",
                         "response=", name.y, ", matches=", sum(tmp), ".")))
        }
        y.surv <- data[[which(tmp)]]
        data[[which(tmp)]] <- NULL
    } else {
        try.res <- try(inla.eval(paste(c("y.surv = with(data,", name.y, ")"))), silent = TRUE)
        if (inherits(try.res, "try-error")) {
            stop(paste(c("The reponse '", name.y, "' is not in 'data' and trying to expand it, failed.")))
        }
    }

    ## this one is not used in this function
    y.surv$.special <- NULL

    len.y.surv <- max(sapply(y.surv, length))
    data.f <- inla.fix.data(data, len.y.surv, revert = FALSE)
    data.l <- inla.fix.data(data, len.y.surv, revert = TRUE)
    data.f <- try(as.data.frame(data.f), silent = TRUE)
    if (inherits(data.f, "try-error")) {
        stop("Fail to convert 'data' into a 'data.frame' even after 'inla.fix.data'.")
    }

    if (class(y.surv) != "inla.surv") {
        stop(paste0("For survival models, then the reponse has to be of class `inla.surv'; you have `",
                    class(y.surv), "'"))
    }
    control.hazard <- inla.check.control(control.hazard, data.f)
    cont.hazard <- inla.set.control.hazard.default()
    cont.hazard[names(control.hazard)] <- control.hazard
    cont.hazard$hyper <- inla.set.hyper(
        model = cont.hazard$model,
        section = "hazard",
        hyper = cont.hazard$hyper,
        initial = cont.hazard$initial,
        fixed = cont.hazard$fixed,
        prior = cont.hazard$prior,
        param = cont.hazard$param,
        debug = FALSE
    )
    if (is.null(y.surv$subject)) {
        res <- inla.expand.dataframe.1(y.surv, data.f, control.hazard = cont.hazard)
    } else {
        res <- inla.expand.dataframe.2(y.surv, data.f, control.hazard = cont.hazard)
    }

    strata.var <- NULL
    strata.tmp <- NULL
    if (!is.null(cont.hazard$strata.name)) {
        if (is.character(cont.hazard$strata.name) && length(cont.hazard$strata.name) == 1) {
            ## strata = "x"
            strata.var <- cont.hazard$strata
        } else {
            stop("Argument to `strata.name' must be the name of a variable in the data.frame.")
        }
    }
    if (!is.null(strata.var)) {
        if (!is.element(strata.var, names(data.f))) {
            stop(inla.paste(c("Variable `", strata.var,
                              "' in control.hazard=list(strata=...) needs to be in the data.frame: names(data) = ",
                              names(data.f))))
        }
        if (debug) print("apply inla.strata() on strata.var")
        ## cleaner code...
        strata.tmp <- inla.strata(inla.get.var(strata.var, data.f))
        data.f[[strata.var]] <- strata.tmp$strata
        data.l$baseline.hazard.strata.coding <- strata.tmp$coding
        ## old code:
        ## inla.eval(paste("strata.tmp = inla.strata(data.f$", strata.var, ")", sep=""))
        ## inla.eval(paste("data.f$", strata.var, " = strata.tmp$strata"))
        ## inla.eval(paste("data.l$baseline.hazard.strata.coding = strata.tmp$coding"))
    }

    ## if -1 the intercept is not included
    intercept <- inla.ifelse(attr(terms(formula), "intercept") == 0, FALSE, TRUE)

    ## we have to check if we have rw1/2,  or the iid
    if (inla.one.of(cont.hazard$model, c("rw1", "rw2"))) {
        d <- inla.ifelse((is.null(cont.hazard$diagonal) && cont.hazard$constr),
                         inla.set.f.default()$diagonal, cont.hazard$diagonal)
        sc <- paste0(", scale.model = ", inla.ifelse(
                                             is.null(cont.hazard$scale.model),
                                             inla.getOption("scale.model.default"),
                                             cont.hazard$scale.model
                                         ))
    } else {
        sc <- ""
        d <- if (is.null(cont.hazard$diagonal)) 0.0 else cont.hazard$diagonal
    }

    f.hazard <- paste(
        "~",
        inla.ifelse(intercept, "1 +", "-1 +"),
        ". + f(baseline.hazard, model=\"", cont.hazard$model, "\"",
        ", values = baseline.hazard.values",
        ", hyper = list(prec=", as.character(cont.hazard$hyper), ")",
        ", constr = ", cont.hazard$constr,
        ", diagonal = ", d,
        sc,
        inla.ifelse(is.null(strata.var), "", paste(", replicate=", strata.var)),
        ")",
        sep = ""
    )

    new.formula <- update(update(formula, as.formula(f.hazard)), y..coxph ~ .)

    data.list <- c(res$data.list, data.l)
    if (!is.null(strata.tmp)) {
        data.list <- c(data.list, list(baseline.hazard.strata.coding = strata.tmp$coding))
    }

    stopifnot(is.character(tag))
    if (nchar(tag) > 0) {
        old.names <- c(
            "y..coxph",
            "expand..coxph",
            "baseline.hazard", 
            "baseline.hazard.idx", 
            "baseline.hazard.time", 
            "baseline.hazard.length"
        )
        new.names <- c(
            paste0("y", tag, "..coxph"),
            paste0("expand", tag, "..coxph"),
            paste0("baseline", tag, ".hazard"), 
            paste0("baseline", tag, ".hazard.idx"), 
            paste0("baseline", tag, ".hazard.time"), 
            paste0("baseline", tag, ".hazard.length")
        )
        ## formula
        nf.text <- as.character(new.formula)
        nf.text <- paste(nf.text[2], nf.text[1], nf.text[3])
        for (i in seq_along(old.names)) {
            nf.text <- gsub(old.names[i], new.names[i], nf.text, fixed = TRUE)
        }
        new.formula <- as.formula(nf.text)

        ## data
        rd.names <- names(res$data)
        for (i in seq_along(old.names)) {
            rd.names <- gsub(old.names[i], new.names[i], rd.names, fixed = TRUE)
        }
        names(res$data) <- rd.names

        ## data.list
        dl.names <- names(data.list)
        for (i in seq_along(old.names)) {
            dl.names <- gsub(old.names[i], new.names[i], dl.names, fixed = TRUE)
        }
        names(data.list) <- dl.names
    }

    expand..coxph.idx <- which(names(res$data) == paste0("expand", tag, "..coxph"))

    return(list(
        formula = new.formula,
        data = res$data,
        data.list = data.list,
        family = "poisson",
        E = res$data[, "E..coxph"],
        expand.df = res$data[, expand..coxph.idx],
        control.hazard = cont.hazard
    ))
}

`inla.rbind.data.frames` <- function(...)
{
    ## rbind data.frames padding with NA

    if (inla.require("dplyr")) {
        ## a better implementation is \code{dplyr::bind_rows})
        return(dplyr::bind_rows(...))
    } else {
        rbind.two.data.frames <- function(df1, df2) {
            df <- NULL
            nr1 <- nrow(df1)
            nr2 <- nrow(df2)
            nams <- sort(unique(c(names(df1), names(df2))), decreasing = TRUE)

            for (nam in nams) {
                if (nam %in% names(df1)) {
                    idx <- which(names(df1) %in% nam)
                    if (nam %in% names(df)) {
                        idxx <- which(names(df) %in% nam)
                        df[1:nr1, idxx] <- df1[1:nr1, idx]
                    } else {
                        if (!is.null(df)) {
                            df <- data.frame(c(df1[1:nr1, idx], rep(NA, nr2)), df)
                        } else {
                            df <- data.frame(c(df1[1:nr1, idx], rep(NA, nr2)))
                        }
                        names(df)[1] <- nam
                    }
                }
                if (nam %in% names(df2)) {
                    idx <- which(names(df2) %in% nam)
                    if (nam %in% names(df)) {
                        idxx <- which(names(df) %in% nam)
                        df[nr1 + 1:nr2, idxx] <- df2[1:nr2, idx]
                    } else {
                        if (!is.null(df)) {
                            df <- data.frame(c(rep(NA, nr1), df2[1:nr2, idx]), df)
                        } else {
                            df <- data.frame(c(rep(NA, nr1), df2[1:nr2, idx]))
                        }
                        names(df)[1] <- nam
                    }
                }
            }

            return(df)
        }

        args <- list(...)
        stopifnot(length(args) >= 2L)

        for (i in 2L:length(args)) {
            if (i == 2L) {
                df <- rbind.two.data.frames(args[[1]], args[[2]])
            } else {
                df <- rbind.two.data.frames(df, args[[i]])
            }
        }

        return(df)
    }
}

## Nothing to Export

### Binary I/O and interpretation of vectors

`inla.interpret.vector` <- function(xx, debug = FALSE)
{
    if (is.null(xx)) {
        return(NULL)
    }

    if (debug) {
        cat("\tEnter inla.interpret.vector ... length(xx) = ", length(xx), "\n")
    }

    len <- length(xx)
    np <- as.integer(xx[2L])
    stopifnot((len %% (2L * np + 2L)) == 0L)
    res.ncol <- (2L * len) %/% (2L * np + 2L)
    res.ncol2 <- res.ncol %/% 2L
    m <- 1L
    ii <- 3L
    np2 <- 2L * np + 2L
    res <- matrix(NA, np, res.ncol)

    if (FALSE) {
        ## this loops over columns
        for (j in 1L:(res.ncol %/% 2L)) {
            idx <- seq.int(ii, by = 2L, length.out = np)
            res[, m] <- xx[idx]
            res[, m + 1L] <- xx[idx + 1L]
            m <- m + 2L
            ii <- ii + np2
        }
        stopifnot(res.ncol == m - 1L) ## since we start from 1L
        stopifnot(len == ii - 3L)
    } else {
        ## this loops over rows,  better
        idx <- seq.int(from = 3L, by = np2, len = res.ncol2)
        idx2 <- seq.int(1L, by = 2L, len = res.ncol2)
        for(i in 1L:np) {
            res[i, idx2] <- xx[idx]
            res[i, idx2 + 1L] <- xx[idx + 1L]
            idx <- idx + 2L
        }
    }

    if (debug) {
        cat("\tLeave inla.interpret.vector ...\n")
    }

    return(res)
}

`inla.interpret.vector.list` <- function(xx, debug = FALSE)
{
    if (is.null(xx)) {
        return(NULL)
    }

    if (debug) {
        cat("\tEnter inla.interpret.vector.list ... length(xx) = ", length(xx), "\n")
    }

    ## this gives approximately the correct length, most likely, an
    ## upper bound.
    m.approx <- length(xx) %/% 54L
    nps <- integer(m.approx) ## all lengths
    inds <- integer(m.approx) ## all offsets where the density starts

    ## need to get the lengths and the offsets as they vary.
    count <- 0L
    ind <- 2L
    while (ind < length(xx)) {
        count <- count + 1L
        np <- as.integer(xx[ind])
        inds[count] <- ind + 1L
        nps[count] <- np
        ind <- ind + 2L * np + 2L
    }

    fun <- function(m, xx, inds, nps) {
        ii <- seq.int(inds[m], length.out = nps[m], by = 2L)
        return(cbind(xx[ii], xx[ii + 1L]))
    }
    res <- lapply(1L:count, fun, xx, inds, nps)

    if (debug) {
        cat("\tLeave inla.interpret.vector.list ...\n")
    }

    return(res)
}

`inla.read.binary.file` <- function(file, silent = TRUE)
{
    if (file.exists(file)) {
        fp <- file(file, "rb")
        len <- file.info(file)$size
        xx <- readBin(fp, numeric(), n = len)
        close(fp)
    } else {
        if (!silent) {
            warning(paste("File does not exits: ", file))
        }
        xx <- NULL
    }

    return(xx)
}

## Internal: inla.mesh.filter.locations
## Internal: inla.mesh.parse.segm.input inla.mesh.extract.segments


#' @title Constraint segments for inla.mesh
#'
#' Constructs `inla.mesh.segment` objects that can be used to specify
#' boundary and interior constraint edges in calls to [inla.mesh()].
#'
#'
#' @param loc Matrix of point locations.
#' @param idx Segment index sequence vector or index pair matrix.  The indices
#' refer to the rows of `loc`.  If `loc==NULL`, the indices will be
#' interpreted as indices into the point specification supplied to
#' [inla.mesh.create()].  If `is.bnd==TRUE`, defaults to linking
#' all the points in `loc`, as `c(1:nrow(loc),1L)`, otherwise
#' `1:nrow(loc)`.
#' @param grp Vector of group labels for each segment.  Set to `NULL` to
#' let the labels be chosen automatically in a call to
#' [inla.mesh.create()].
#' @param is.bnd `TRUE` if the segments are boundary segments, otherwise
#' `FALSE`.
#' @param grp.default When joining segments, use this group label for segments
#' that have `grp=NULL`.
#' @param x,y,z,nlevels,levels Parameters specifying a set of surface contours,
#' with syntax described in [contour()].
#' @param groups Vector of group ID:s, one for each contour level.
#' @param positive `TRUE` if the contours should encircle positive level
#' excursions in a counter clockwise direction.
#' @param eps Tolerance for [inla.simplify.curve()].
#' @param crs An optional `CRS` or `inla.CRS` object
#' @param ...  Additional parameters.  When joining segments, a list of
#' `inla.mesh.segment` objects.
#' @return An `inla.mesh.segment` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.create()], [inla.mesh.2d()]
#' @examples
#'
#' ## Create a square boundary and a diagonal interior segment
#' loc.bnd <- matrix(c(0, 0, 1, 0, 1, 1, 0, 1), 4, 2, byrow = TRUE)
#' loc.int <- matrix(c(0.9, 0.1, 0.1, 0.6), 2, 2, byrow = TRUE)
#' segm.bnd <- inla.mesh.segment(loc.bnd)
#' segm.int <- inla.mesh.segment(loc.int, is.bnd = FALSE)
#'
#' ## Points to be meshed
#' loc <- matrix(runif(10 * 2), 10, 2) * 0.9 + 0.05
#' mesh <- inla.mesh.create(loc,
#'     boundary = segm.bnd,
#'     interior = segm.int,
#'     refine = list()
#' )
#' plot(mesh)
#' \dontrun{
#' mesh <- inla.mesh.create(loc, interior = list(segm.bnd, segm.int))
#' plot(mesh)
#' }
#'
#' @export inla.mesh.segment
inla.mesh.segment <- function(...) {
    UseMethod("inla.mesh.segment")
}

#' @export
#' @rdname inla.mesh.segment
inla.mesh.segment.default <- function(loc = NULL, idx = NULL, grp = NULL, is.bnd = TRUE,
                                      crs = NULL, ...) {
    if ((missing(loc) || is.null(loc)) &&
        (missing(idx) || is.null(idx))) {
        stop("At most one of 'loc' and 'idx' may be missing or null.")
    }
    if (!missing(loc) && !is.null(loc)) {
        ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
        if (inherits(loc, "SpatialPoints") ||
            inherits(loc, "SpatialPointsDataFrame")) {
            loc <- inla.spTransform(coordinates(loc),
                inla.sp_get_crs(loc),
                crs,
                passthrough = TRUE
            )
        }

        if (!is.matrix(loc)) {
            loc <- as.matrix(loc)
        }
        if (!is.double(loc)) {
            storage.mode(loc) <- "double"
        }
        if (missing(idx) || is.null(idx)) {
            idx <- (inla.ifelse(
                is.bnd,
                c(1:nrow(loc), 1),
                c(1:nrow(loc))
            ))
        }
    } else {
        loc <- NULL
    }

    if (!missing(idx) && !is.null(idx)) {
        if (!is.vector(idx) && !is.matrix(idx)) {
            stop("'idx' must be a vector or a matrix")
        }
        if (is.vector(idx)) {
            idx <- as.matrix(idx, nrow = length(idx), ncol = 1)
        }
        if (ncol(idx) == 1) {
            if (nrow(idx) < 2) {
                if (nrow(idx) == 1) {
                    warning("Segment specification must have at least 2, or 0, indices.")
                }
                idx <- matrix(0L, 0, 2)
            } else {
                idx <- matrix(c(idx[-nrow(idx)], idx[-1]), nrow(idx) - 1, 2)
            }
        }
        storage.mode(idx) <- "integer"
        if (!is.null(loc) &&
            (nrow(idx) > 0) &&
            (max(idx, na.rm = TRUE) > nrow(loc))) {
            warning(
                "Segment indices (max=", max(idx, na.rm = TRUE),
                ") exceed specified location list length (",
                nrow(loc), ")."
            )
        }
    }

    if (!missing(grp) && !is.null(grp)) {
        if (!is.vector(grp) && !is.matrix(grp)) {
            stop("'grp' must be a vector or a matrix")
        }
        grp <- matrix(grp, min(length(grp), nrow(idx)), 1)
        if (nrow(grp) < nrow(idx)) {
            grp <- (matrix(
                c(
                    as.vector(grp),
                    rep(grp[nrow(grp)], nrow(idx) - length(grp))
                ),
                nrow(idx), 1
            ))
        }
        storage.mode(grp) <- "integer"
    } else {
        grp <- NULL
    }

    ## Filter away NAs in loc and idx
    if (!is.null(loc)) {
        idx[is.na(idx)] <- 0L ## Avoid R annoyances with logical+NA indexing
        while (sum(is.na(loc)) > 0) {
            i <- min(which(rowSums(is.na(loc)) > 0))
            loc <- loc[-i, , drop = FALSE]
            idx[idx == i] <- 0L
            idx[idx > i] <- idx[idx > i] - 1L
        }
        idx[idx == 0L] <- NA
    }
    while (sum(is.na(idx)) > 0) {
        i <- min(which(rowSums(is.na(idx)) > 0))
        idx <- idx[-i, , drop = FALSE]
        if (!is.null(grp)) {
            grp <- grp[-i, , drop = FALSE]
        }
    }

    if (!is.null(loc)) {
        ## Identify unused locations and remap indices accordingly.
        idx.new <- rep(0L, nrow(loc))
        idx.new[as.vector(idx)] <- 1L
        loc <- loc[idx.new == 1L, , drop = FALSE]
        idx.new[idx.new == 1L] <- seq_len(sum(idx.new))
        idx <- (matrix(idx.new[as.vector(idx)],
            nrow = nrow(idx),
            ncol = ncol(idx)
        ))
    }

    ret <- list(loc = loc, idx = idx, grp = grp, is.bnd = is.bnd, crs = crs)
    class(ret) <- "inla.mesh.segment"
    return(invisible(ret))
}

#' @export
#' @rdname inla.mesh.segment
inla.mesh.segment.inla.mesh.segment <- function(..., grp.default = 0) {
    segm <- list(...)
    if (!all(unlist(lapply(
        segm,
        function(x) inherits(x, "inla.mesh.segment")
    )))) {
        stop("All objects must be of class 'inla.mesh.segment'.")
    }

    Nloc <- unlist(lapply(segm, function(x) nrow(x$loc)))
    cumNloc <- c(0, cumsum(Nloc))
    Nidx <- unlist(lapply(segm, function(x) nrow(x$idx)))

    loc <- do.call(rbind, lapply(segm, function(x) x$loc))
    idx <- do.call(rbind, lapply(
        seq_along(segm),
        function(x) segm[[x]]$idx + cumNloc[x]
    ))
    grp <- unlist(lapply(
        seq_along(segm),
        function(x) {
            if (is.null(segm[[x]]$grp)) {
                rep(grp.default, Nidx[x])
            } else {
                segm[[x]]$grp
            }
        }
    ))
    is.bnd <- unlist(lapply(segm, function(x) x$is.bnd))
    if (!all(is.bnd) || (any(!is.bnd) && !all(!is.bnd))) {
        warning("Inconsistent 'is.bnd' attributes.  Setting 'is.bnd=FALSE'.")
        is.bnd <- FALSE
    } else {
        is.bnd <- all(is.bnd)
    }

    crs <- lapply(segm, function(x) x$crs)
    if (!is.null(crs)) {
        crs <- crs[unlist(lapply(
            crs,
            function(x) !is.null(x)
        ))]
        if (length(crs) > 0) {
            if (!all(unlist(lapply(
                crs,
                function(x) identical(crs[[1]], x)
            )))) {
                lapply(crs, function(x) show(x))
                stop("Inconsistent 'crs' attributes.")
            } else {
                crs <- crs[[1]]
            }
        } else {
            crs <- NULL
        }
    }

    invisible(inla.mesh.segment(
        loc = loc, idx = idx, grp = grp, is.bnd = is.bnd,
        crs = crs
    ))
}


#' Draw `inla.mesh.segment` objects.
#'
#' Draws a [inla.mesh.segment()] object with generic or `rgl`
#' graphics.
#'
#'
#' @param x An [inla.mesh.segment()] object.
#' @param loc Point locations to be used if `x$loc` is `NULL`.
#' @param col Segment color specification.
#' @param colors Colors to cycle through if `col` is `NULL`.
#' @param add If `TRUE`, add to the current plot, otherwise start a new
#' plot.
#' @param xlim X axis limits for a new plot.
#' @param ylim Y axis limits for a new plot.
#' @param rgl If `TRUE`, use `rgl` for plotting.
#' @param \dots Additional parameters, passed on to graphics methods.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.segment()]
#' @export
lines.inla.mesh.segment <- function(x, loc = NULL, col = NULL,
                                    colors = c("black", "blue", "red", "green"),
                                    add = TRUE, xlim = NULL, ylim = NULL,
                                    rgl = FALSE, ...) {
    segm <- x
    if (!is.null(segm$loc)) {
          loc <- segm$loc
      }
    stopifnot(!is.null(loc), ncol(loc) >= 2)
    if (ncol(loc) < 3) {
        loc <- cbind(loc, 0.0)
    }
    color <- col
    if (rgl) {
        inla.require("rgl", stop.on.error = TRUE)
        if (!add) {
            dev <- rgl::open3d()
            rgl::view3d(0, 0, fov = 0)
        } else {
            dev <- NULL
        }
    } else if (!add) {
        idx <- unique(as.vector(segm$idx))
        if (is.null(xlim)) {
              xlim <- range(loc[idx, 1])
          }
        if (is.null(ylim)) {
              ylim <- range(loc[idx, 2])
          }
        plot.new()
        plot.window(xlim = xlim, ylim = ylim, ...)
    }

    grps <- inla.ifelse(is.null(segm$grp), rep(0L, nrow(segm$idx)), segm$grp)
    for (grp in unique(grps)) {
        idx <- which(grps == grp)
        if (is.null(col)) {
            color <- colors[1 + (grp %% length(colors))]
        }
        if (rgl) {
            rgl::segments3d(loc[as.vector(t(segm$idx[idx, , drop = FALSE])), , drop = FALSE],
                color = color,
                ...
            )
        } else {
            lines(loc[t(cbind(segm$idx[idx, , drop = FALSE], NA)), 1],
                loc[t(cbind(segm$idx[idx, , drop = FALSE], NA)), 2],
                col = color,
                ...
            )
        }
    }
}

#' Generate text RGB color specifications.
#'
#' Generates a tex RGB color specification matrix based on a color palette.
#'
#'
#' @param color `character`, `matrix` or `vector`
#' @param color.axis The min/max limit values for the color mapping.
#' @param color.n The number of colors to use in the color palette.
#' @param color.palette A color palette function.
#' @param color.truncate If `TRUE`, truncate the colors at the color axis
#' limits.
#' @param alpha Transparency/opaqueness values.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.generate.colors
inla.generate.colors <- function(color,
                                 color.axis = NULL,
                                 color.n = 512,
                                 color.palette = cm.colors,
                                 color.truncate = FALSE,
                                 alpha = NULL) {
    if (is.character(color)) {
        colors <- color
    } else if (is.vector(color) || (is.matrix(color) && (ncol(color) == 1))) {
        if (is.null(color.axis)) {
              color.axis <- c(min(color, na.rm = TRUE), max(color, na.rm = TRUE))
          }
        if (color.truncate) {
            not.ok <- ((color < color.axis[1]) |
                (color > color.axis[2]))
        } else {
            not.ok <- rep(FALSE, length(color))
        }
        cs <- (pmax(color.axis[1],
            pmin(color.axis[2], color, na.rm = TRUE),
            na.rm = TRUE
        ))
        cs <- (cs - color.axis[1]) / (color.axis[2] - color.axis[1])
        not.ok <- not.ok | is.na(cs)
        cs[not.ok] <- 0.5
        if (is.null(alpha)) {
            alpha <- as.numeric(!not.ok)
        } else {
            alpha[not.ok] <- 0
        }

        ics <- (as.numeric(cut(cs, seq(0, 1, length.out = color.n + 1),
            include.lowest = TRUE
        )))
        colors <- color.palette(color.n)[ics]

        ## Todo: handle alpha, combining "input alpha" with "not.ok-alpha"
    } else if (is.matrix(color) && (ncol(color) == 3)) {
        if (is.null(color.axis)) {
              color.axis <- c(min(color, na.rm = TRUE), max(color, na.rm = TRUE))
          }
        if (color.truncate) {
            not.ok <- ((color[, 1] < color.axis[1]) |
                (color[, 2] < color.axis[1]) |
                (color[, 3] < color.axis[1]) |
                (color[, 1] > color.axis[2]) |
                (color[, 2] > color.axis[2]) |
                (color[, 3] > color.axis[2]))
        } else {
            not.ok <- rep(FALSE, nrow(color))
        }
        cs <- matrix(
            pmax(color.axis[1],
                pmin(color.axis[2], color, na.rm = TRUE),
                na.rm = TRUE
            ), dim(color)
        )
        cs <- (cs - color.axis[1]) / (color.axis[2] - color.axis[1])
        not.ok <- not.ok | is.na(cs[, 1]) | is.na(cs[, 2]) | is.na(cs[, 3])
        cs[not.ok, ] <- c(0.5, 0.5, 0.5)
        if (is.null(alpha)) {
            alpha <- as.numeric(!not.ok)
        } else {
            alpha[not.ok] <- 0
        }
        colors <- rgb(cs[, 1], cs[, 2], cs[, 3])
    } else {
        stop("color specification must be character, matrix, or vector.")
    }

    return(list(colors = colors, alpha = alpha))
}


## Export as plot!inla.trimesh as well as plot.inla.trimesh,
## even though it's not actually an S3 method.


#' Low level triangulation mesh plotting
#'
#' Plots a triangulation mesh using `rgl`.
#'
#'
#' @param x A 3-column triangle-to-vertex index map matrix.
#' @param S A 3-column vertex coordinate matrix.
#' @param color Color specification.  A single named color, a vector of scalar
#' values, or a matrix of RGB values.
#' @param color.axis The min/max limit values for the color mapping.
#' @param color.n The number of colors to use in the color palette.
#' @param color.palette A color palette function.
#' @param color.truncate If `TRUE`, truncate the colors at the color axis
#' limits.
#' @param alpha Transparency/opaqueness values. See `rgl.material`.
#' @param lwd Line width for edges. See `rgl.material`.
#' @param specular Specular color. See `rgl.material`.
#' @param draw.vertices If `TRUE`, draw triangle vertices.
#' @param draw.edges If `TRUE`, draw triangle edges.
#' @param edge.color Edge color specification.
#' @param \dots Additional parameters passed to and from other methods.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [plot.inla.mesh()]
#' @method plot inla.trimesh
#' @export
plot.inla.trimesh <- function(x, S, color = NULL, color.axis = NULL,
                              color.n = 512, color.palette = cm.colors,
                              color.truncate = FALSE, alpha = NULL,
                              lwd = 1, specular = "black",
                              draw.vertices = TRUE, draw.edges = TRUE,
                              edge.color = rgb(0.3, 0.3, 0.3), ...) {
    TV <- x
    inla.require("rgl", stop.on.error = TRUE)

    ## Make indices 1 based.  Deprecated and is deactivated.
    if (min(TV) == 0) {
        stop("Zero-based indices in TV are not supported.")
    }

    colors <- (inla.generate.colors(
        color, color.axis, color.n,
        color.palette, color.truncate, alpha
    ))

    tTV <- t(TV)
    tETV <- t(TV[, c(1, 2, 3, 1, NA)])
    Tx <- S[tTV, 1]
    Ty <- S[tTV, 2]
    Tz <- S[tTV, 3]
    if (length(colors$colors) == 1) {
        ## One color
        Tcol <- colors$colors
        Talpha <- colors$alpha
    } else if (length(colors$colors) == nrow(S)) {
        ## One color per vertex
        Tcol <- colors$colors[tTV]
        Talpha <- colors$alpha[tTV]
    } else {
        ## One color per triangle
        stopifnot(length(colors$colors) == nrow(TV))
        Tcol <- colors$colors[t(matrix(rep(1:nrow(TV), 3), dim(TV)))]
        Talpha <- colors$alpha[t(matrix(rep(1:nrow(TV), 3), dim(TV)))]
    }
    Ex <- S[tETV, 1]
    Ey <- S[tETV, 2]
    Ez <- S[tETV, 3]
    Ecol <- edge.color
    if (draw.vertices) {
        rgl::points3d(S, color = "black", ...)
    }
    if (draw.edges) {
        rgl::lines3d(Ex, Ey, Ez, color = Ecol, lwd = lwd, ...)
    }
    rgl::triangles3d(Tx, Ty, Tz, color = Tcol, specular = specular, alpha = Talpha, ...)

    return(invisible())
}

## library(geometry)
## S = cbind(x=rnorm(30), y=rnorm(30), z=0)
## TV = delaunayn(S[, 1:2]) # NOTE: inconsistent triangle orders, only for test.
## trimesh(TV, S)
##
## colors = rgb(runif(30), runif(30), runif(30))
## rgl.viewpoint(0, 0, fov=20)
## plot.inla.trimesh(TV, S, colors)

## Ecol = col2rgb(color)/256
## Ecol = Ecol*0.5+(1-0.5)*0 # Rescale towards black
## Ecol = 1-Ecol # Invert
## Ecol = Ecol[, c(2, 3, 1)] # Permute
## Ecol = rgb(Ecol[1,], Ecol[2,], Ecol[3,], maxColorValue = 1)
## Ecol = Ecol[tETV]


#' Draw a triangulation mesh object
#'
#' Plots an [inla.mesh()] object using either standard graphics or
#' with `rgl`.
#'
#'
#' @param x An [inla.mesh()] object.
#' @param col Color specification.  A single named color, a vector of scalar
#' values, or a matrix of RGB values.  Requires `rgl=TRUE`.
#' @param t.sub Optional triangle index subset to be drawn.
#' @param add If `TRUE`, adds to the current plot instead of starting a
#' new one.
#' @param lwd Line width for triangle edges.
#' @param xlim X-axis limits.
#' @param ylim Y-axis limits.
#' @param main The main plot title.  If not specified, a default title is
#' generated based on the mesh type.
#' @param rgl When `TRUE`, generates an `rgl` plot instead of a
#' generic graphics plot.  Allows 3D plotting and color surface plotting.
#' @param size Size of vertex points in `rgl` plotting. See
#' `rgl.material`.
#' @param draw.vertices If `TRUE`, draw triengle vertices.
#' @param vertex.color Color specification for all vertices.
#' @param draw.edges If `TRUE`, draw triangle edges.
#' @param edge.color Color specification for all edges.
#' @param draw.segments If `TRUE`, draw boundary and interior constraint
#' edges more prominently.
#' @param \dots Further graphics parameters, interpreted by the respective
#' plotting systems.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [plot.inla.trimesh()]
#' @examples
#'
#' mesh <- inla.mesh.create(globe = 10)
#' plot(mesh)
#' \donttest{
#' if (require(rgl)) {
#'     plot(mesh, rgl = TRUE, col = mesh$loc[, 1])
#' }
#' }
#'
#' @method plot inla.mesh
#' @export
plot.inla.mesh <- function(x,
                           col = "white",
                           t.sub = 1:nrow(mesh$graph$tv),
                           add = FALSE,
                           lwd = 1,
                           xlim = range(mesh$loc[, 1]),
                           ylim = range(mesh$loc[, 2]),
                           main = NULL,
                           rgl = FALSE,
                           size = 2,
                           draw.vertices = FALSE,
                           vertex.color = "black",
                           draw.edges = TRUE,
                           edge.color = rgb(0.3, 0.3, 0.3),
                           draw.segments = draw.edges,
                           ...) {
    inla.require.inherits(x, "inla.mesh", "'mesh'")
    mesh <- x

    if (rgl) {
        inla.require("rgl", stop.on.error = TRUE)
        if (!add) {
            dev <- rgl::open3d()
            rgl::view3d(0, 0, fov = 0)
        } else {
            dev <- NULL
        }
        tv <- mesh$graph$tv[t.sub, , drop = FALSE]
        if (draw.vertices) {
            idx <- intersect(unique(as.vector(tv)), mesh$idx$loc)
            rgl::points3d(mesh$loc[idx, , drop = FALSE],
                size = 2 * size, lwd = lwd, color = "blue", ...
            )
        }
        if (draw.segments) {
            if (!is.null(mesh$segm$bnd)) {
                  lines(mesh$segm$bnd, mesh$loc,
                      lwd = lwd + 1,
                      rgl = TRUE, add = TRUE, ...
                  )
              }
            if (!is.null(mesh$segm$int)) {
                  lines(mesh$segm$int, mesh$loc,
                      lwd = lwd + 1,
                      rgl = TRUE, add = TRUE, ...
                  )
              }
        }
        plot.inla.trimesh(tv, mesh$loc,
            color = col,
            size = size, lwd = lwd,
            draw.vertices = draw.vertices,
            vertex.color = vertex.color,
            draw.edges = draw.edges,
            edge.color = edge.color,
            ...
        )
        return(invisible(dev))
    } else {
        idx <- cbind(mesh$graph$tv[t.sub, c(1:3, 1), drop = FALSE], NA)
        x <- mesh$loc[t(idx), 1]
        y <- mesh$loc[t(idx), 2]

        if (!add) {
            plot.new()
            plot.window(xlim = xlim, ylim = ylim, ...)
        }
        if (draw.edges) {
            lines(x, y, type = "l", col = edge.color, lwd = lwd)
        }
        tv <- mesh$graph$tv[t.sub, , drop = FALSE]
        if (draw.vertices) {
            idx <- unique(as.vector(tv))
            points(mesh$loc[idx, , drop = FALSE],
                pch = 20, col = vertex.color, ...
            )
            idx <- intersect(idx, mesh$idx$loc)
            points(mesh$loc[idx, , drop = FALSE],
                pch = 20, col = "blue", ...
            )
        }
        if (draw.segments) {
            if (!is.null(mesh$segm$bnd)) {
                  lines(mesh$segm$bnd, mesh$loc, lwd = lwd + 1, ...)
              }
            if (!is.null(mesh$segm$int)) {
                  lines(mesh$segm$int, mesh$loc, lwd = lwd + 1, ...)
              }
        }
        if (!add && missing(main)) {
            if (mesh$meta$is.refined) {
                title("Constrained refined Delaunay triangulation", ...)
            } else {
                title("Constrained Delaunay triangulation", ...)
            }
        }
        return(invisible())
    }
}


#' @export
#' @describeIn inla.mesh.map Projection extent limit calculations
inla.mesh.map.lim <- function(loc = NULL,
                              projection =
                                  c("default", "longlat", "longsinlat", "mollweide")) {
    projection <- match.arg(projection)
    if (identical(projection, "default")) {
        if (is.null(loc)) {
            lim <- list(xlim = c(0, 1), ylim = c(0, 1))
        } else {
            lim <-
                list(
                    xlim = range(loc[, 1], na.rm = TRUE),
                    ylim = range(loc[, 2], na.rm = TRUE)
                )
        }
    } else if (identical(projection, "longlat")) {
        lim <- list(xlim = c(-180, 180), ylim = c(-90, 90))
    } else if (identical(projection, "longsinlat")) {
        lim <- list(xlim = c(-180, 180), ylim = c(-1, 1))
    } else if (identical(projection, "mollweide")) {
        lim <- list(xlim = c(-2, 2), ylim = c(-1, 1))
    } else {
        stop(paste("Unknown projection '", projection, "'.", sep = ""))
    }
    return(lim)
}


#' Coordinate mappings for `inla.mesh` projections.
#'
#' Calculates coordinate mappings for `inla.mesh` projections.
#'
#'
#' @aliases inla.mesh.map inla.mesh.map.lim
#' @param loc Coordinates to be mapped.
#' @param projection The projection type.
#' @param inverse If `TRUE`, `loc` are map coordinates and
#' coordinates in the mesh domain are calculated.  If `FALSE`, `loc`
#' are coordinates in the mesh domain and the forward map projection is
#' calculated.
#' @return For `inla.mesh.map.lim`, a list: \item{xlim }{X axis limits in
#' the map domain} \item{ylim }{Y axis limits in the map domain} No attempt is
#' made to find minimal limits for partial spherical domains.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.project()]
#' @export inla.mesh.map
inla.mesh.map <- function(loc,
                          projection =
                              c("default", "longlat", "longsinlat", "mollweide"),
                          inverse = TRUE) {
    projection <- match.arg(projection)
    if (identical(projection, "default")) {
        return(loc)
    } else if (identical(projection, "longlat")) {
        if (inverse) {
            proj <-
                cbind(
                    cos(loc[, 1] * pi / 180) * cos(loc[, 2] * pi / 180),
                    sin(loc[, 1] * pi / 180) * cos(loc[, 2] * pi / 180),
                    sin(loc[, 2] * pi / 180)
                )
        } else {
            proj <-
                cbind(
                    atan2(loc[, 2], loc[, 1]) * 180 / pi,
                    asin(pmax(-1, pmin(+1, loc[, 3]))) * 180 / pi
                )
        }
    } else if (identical(projection, "longsinlat")) {
        if (inverse) {
            coslat <- sqrt(pmax(0, 1 - loc[, 2]^2))
            proj <-
                cbind(
                    cos(loc[, 1] * pi / 180) * coslat,
                    sin(loc[, 1] * pi / 180) * coslat,
                    loc[, 2]
                )
        } else {
            proj <-
                cbind(
                    atan2(loc[, 2], loc[, 1]) * 180 / pi,
                    loc[, 3]
                )
        }
    } else if (identical(projection, "mollweide")) {
        if (inverse) {
            ok <- ((loc[, 1]^2 + 4 * loc[, 2]^2) <= 4)
            cos.theta <- sqrt(pmax(0, 1 - loc[ok, 2]^2))
            theta <- atan2(loc[ok, 2], cos.theta)
            sin.lat <- (2 * theta + sin(2 * theta)) / pi
            cos.lat <- sqrt(pmax(0, 1 - sin.lat^2))
            lon <- loc[ok, 1] * pi / 2 / (cos.theta + (cos.theta == 0))
            lon[cos.theta == 0] <- pi / 2 * sign(theta[cos.theta == 0])
            proj <- matrix(NA, nrow(loc), 3)
            proj[ok, ] <- cbind(cos(lon) * cos.lat, sin(lon) * cos.lat, sin.lat)
        } else {
            lon <- atan2(loc[, 2], loc[, 1])
            z <- pmin(1, pmax(-1, loc[, 3]))
            sin.theta <- z
            cos.theta <- sqrt(pmax(0, 1 - sin.theta^2))
            ## NR-solver for sin.theta.
            ## Typically finishes after at most 7 iterations.
            ## When cos.theta=0, sin.theta is already correct, +/- 1.
            nook <- (cos.theta > 0)
            for (k in 1:20) {
                if (any(nook)) {
                    delta <-
                        (atan2(sin.theta[nook], cos.theta[nook]) +
                            sin.theta[nook] * cos.theta[nook] - pi / 2 * z[nook]) /
                            (2 * cos.theta[nook])
                    sin.theta[nook] <- sin.theta[nook] - delta
                    cos.theta[nook] <- sqrt(1 - sin.theta[nook]^2)
                    nook[nook] <- (abs(delta) > 1e-14)
                }
            }
            proj <- cbind(2 * lon / pi * cos.theta, sin.theta)
        }
    } else {
        stop(paste("Unknown projection '", projection, "'.", sep = ""))
    }
    return(proj)
}



#' Lattice grids for inla.mesh
#'
#' Construct a lattice grid for [inla.mesh()]
#'
#'
#' @param x vector or grid matrix of x-values
#' @param y vector of grid matrix of y-values
#' @param z if x is a matrix, a grid matrix of z-values
#' @param dims the size of the grid, length 2 vector
#' @param units One of `c("default", "longlat", "longsinlat")`.
#' @param crs An optional `CRS` or `inla.CRS` object
#' @return An `inla.mesh.lattice` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh()]
#' @examples
#'
#' lattice <- inla.mesh.lattice(seq(0, 1, length.out = 17), seq(0, 1, length.out = 10))
#'
#' ## Use the lattice "as-is", without refinement:
#' mesh <- inla.mesh.create(lattice = lattice, boundary = lattice$segm)
#' mesh <- inla.mesh.create(lattice = lattice, extend = FALSE)
#' plot(mesh)
#'
#' ## Refine the triangulation, with limits on triangle angles and edges:
#' mesh <- inla.mesh.create(
#'     lattice = lattice,
#'     refine = list(max.edge = 0.08),
#'     extend = FALSE
#' )
#' plot(mesh)
#'
#' ## Add an extension around the lattice, but maintain the lattice edges:
#' mesh <- inla.mesh.create(
#'     lattice = lattice,
#'     refine = list(max.edge = 0.08),
#'     interior = lattice$segm
#' )
#' plot(mesh)
#'
#' ## Only add extension:
#' mesh <- inla.mesh.create(lattice = lattice, refine = list(max.edge = 0.08))
#' plot(mesh)
#' @export inla.mesh.lattice
inla.mesh.lattice <- function(x = seq(0, 1, length.out = 2),
                              y = seq(0, 1, length.out = 2),
                              z = NULL,
                              dims =
                                  if (is.matrix(x)) {
                                      dim(x)
                                  } else {
                                      c(length(x), length(y))
                                  },
                              units = NULL,
                              crs = NULL) {
    if (is.null(crs)) {
        units <- match.arg(units, c("default", "longlat", "longsinlat", "mollweide"))

        lim <- inla.mesh.map.lim(projection = units)
        xlim <- lim$xlim
        ylim <- lim$ylim
    } else { ## !is.null(crs)
        if (!is.null(units)) {
            stop("Only one of 'units' and 'crs' can be non-null.")
        }
        bounds <- inla.crs.bounds(crs)
        xlim <- bounds$xlim
        ylim <- bounds$ylim
    }

    if (missing(x) && !missing(dims)) {
        x <- seq(xlim[1], xlim[2], length.out = dims[1])
    }
    if (missing(y) && !missing(dims)) {
        y <- seq(ylim[1], ylim[2], length.out = dims[2])
    }
    dims <- as.integer(dims)

    if (is.matrix(x)) {
        if (!identical(dims, dim(x)) ||
            !identical(dims, dim(y)) ||
            (is.matrix(z) && !identical(dims, dim(z)))) {
              stop("The size of matrices 'x', 'y', and 'z' must match 'dims'.")
          }
        loc <- cbind(as.vector(x), as.vector(y), as.vector(z))
        x <- NULL
        y <- NULL
    } else {
        if (!identical(dims[1], length(x)) ||
            !identical(dims[2], length(y))) {
              stop(paste("The lengths of vectors 'x' and 'y' (",
                  length(x), ",", length(y),
                  ") must match 'dims' (", dims[1], ",", dims[2], ").",
                  sep = ""
              ))
          }
        loc <- (cbind(
            rep(x, times = dims[2]),
            rep(y, each = dims[1])
        ))
    }
    if (!is.double(loc)) {
          storage.mode(loc) <- "double"
      }

    if (is.null(crs)) {
        loc <- inla.mesh.map(loc = loc, projection = units, inverse = TRUE)
    }

    ## Construct lattice boundary
    segm.idx <- (c(
        1:(dims[1] - 1),
        dims[1] * (1:(dims[2] - 1)),
        dims[1] * dims[2] - (0:(dims[1] - 2)),
        dims[1] * ((dims[2] - 1):1) + 1
    ))
    segm.grp <- (c(
        rep(1L, dims[1] - 1),
        rep(2L, dims[2] - 1),
        rep(3L, dims[1] - 1),
        rep(4L, dims[2] - 1)
    ))

    segm <- (inla.mesh.segment(
        loc = loc[segm.idx, , drop = FALSE],
        grp = segm.grp,
        is.bnd = TRUE,
        crs = crs
    ))

    lattice <- list(dims = dims, x = x, y = y, loc = loc, segm = segm, crs = crs)
    class(lattice) <- "inla.mesh.lattice"
    return(lattice)
}


#' Extract tagged boundary/internal segments.
#'
#' Extract boundary or internal segments tagged by group id:s.
#'
#'
#' @aliases extract.groups extract.groups.inla.mesh.segment
#' @param segm An [inla.mesh.segment()] object.
#' @param groups The segment groups id:s to extract.
#' @param groups.new Optional vector of group id remapping; `groups[k]` in
#' the input will be replaced by `groups.new[k]` in the output.
#' @param \dots Additional arguments, passed on to other methods.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.segment()]
#' @export extract.groups
extract.groups <- function(...) {
    UseMethod("extract.groups")
}

#' @export
#' @rdname extract.groups
extract.groups.inla.mesh.segment <- function(segm,
                                             groups,
                                             groups.new = groups,
                                             ...) {
    inla.require.inherits(segm, "inla.mesh.segment", "'segm'")

    if (length(groups.new) == 1L) {
        groups.new <- rep(groups.new, length(groups))
    }
    if (length(groups.new) != length(groups)) {
        stop(
            "Length of 'groups.new' (", length(groups.new),
            ") does not match length of 'groups' (", length(groups), ")"
        )
    }

    idx <- c()
    segm.grp <- c()
    for (k in 1:length(groups)) {
        extract.idx <- which(segm$grp == groups[k])
        idx <- c(idx, extract.idx)
        segm.grp <- c(segm.grp, rep(groups.new[k], length(extract.idx)))
    }
    segm.idx <- segm$idx[idx, , drop = FALSE]

    return(inla.mesh.segment(
        loc = segm$loc,
        idx = segm.idx,
        grp = segm.grp,
        segm$is.bnd,
        crs = segm$crs
    ))
}

inla.mesh.parse.segm.input <- function(boundary = NULL,
                                       interior = NULL,
                                       segm.offset = 0L,
                                       loc.offset = 0L,
                                       crs = NULL) {
    homogenise.segm.input <- function(x, is.bnd, crs = NULL) {
        if (is.matrix(x) || is.vector(x)) { ## Coordinates or indices
            x <- (inla.ifelse(
                is.matrix(x), x,
                as.matrix(x, nrow = length(x), ncol = 1)
            ))
            if (is.integer(x)) { ## Indices
                ret <- inla.mesh.segment(NULL, x, NULL, is.bnd, crs = crs)
            } else if (is.numeric(x)) { ## Coordinates
                ret <- inla.mesh.segment(x, NULL, NULL, is.bnd, crs = crs)
            } else {
                stop("Segment info matrix must be numeric or integer.")
            }
        } else if (inherits(x, "inla.mesh.segment")) {
            ## Override x$is.bnd:
            ret <- inla.mesh.segment(x$loc, x$idx, x$grp, is.bnd, x$crs)
        } else if (inherits(x, c(
            "SpatialPoints", "SpatialPointsDataFrame",
            "Line", "Lines",
            "SpatialLines", "SpatialLinesDataFrame",
            "Polygon", "Polygons",
            "SpatialPolygons",
            "SpatialPolygonsDataFrame"
        ))) {
            x <- as.inla.mesh.segment(x)
            ## Override x$is.bnd:
            ret <- inla.mesh.segment(x$loc, x$idx, x$grp, is.bnd, x$crs)
        } else if (!is.null(x)) {
            inla.require.inherits(
                NULL,
                c("matrix", "inla.mesh.segment"),
                "Segment info"
            )
        } else {
            ret <- NULL
        }
        ret
    }

    homogenise.segm.grp <- function(input) {
        grp.idx <- 0L
        for (k in 1:length(input)) {
            if (!is.null(input[[k]])) {
                inla.require.inherits(
                    input[[k]],
                    "inla.mesh.segment",
                    "Segment info list members "
                )
                if (is.null(input[[k]]$grp)) {
                    grp.idx <- grp.idx + 1L
                    input[[k]] <- (inla.mesh.segment(
                        input[[k]]$loc,
                        input[[k]]$idx,
                        grp.idx,
                        input[[k]]$is.bnd,
                        input[[k]]$crs
                    ))
                } else {
                    grp.idx <- max(grp.idx, input[[k]]$grp, na.rm = TRUE)
                }
            }
        }
        return(input)
    }

    join.segm.input <- function(segm, segm.offset = 0L, loc.offset = 0L,
                                crs = NULL) {
        loc <- NULL
        bnd <- list(loc = NULL, idx = matrix(, 0, 2), grp = matrix(, 0, 1))
        int <- list(loc = NULL, idx = matrix(, 0, 2), grp = matrix(, 0, 1))
        storage.mode(bnd$idx) <- "integer"
        storage.mode(bnd$grp) <- "integer"
        storage.mode(int$idx) <- "integer"
        storage.mode(int$grp) <- "integer"
        for (k in seq_along(segm)) {
            if (!is.null(segm[[k]])) {
                inla.require.inherits(
                    segm[[k]],
                    "inla.mesh.segment",
                    "Segment info list members "
                )
                if (!is.null(segm[[k]]$loc)) {
                    local.loc <- inla.spTransform(segm[[k]], crs,
                        passthrough = TRUE
                    )$loc
                    extra.loc.n <- nrow(local.loc)
                    idx.offset <- segm.offset
                    segm.offset <- segm.offset + extra.loc.n
                    loc <- (inla.ifelse(
                        is.null(loc),
                        local.loc,
                        rbind(loc, local.loc)
                    ))
                } else {
                    idx.offset <- loc.offset
                }
                if (segm[[k]]$is.bnd) {
                    bnd$idx <- rbind(bnd$idx, segm[[k]]$idx + idx.offset)
                    bnd$grp <- rbind(bnd$grp, segm[[k]]$grp)
                } else {
                    int$idx <- rbind(int$idx, segm[[k]]$idx + idx.offset)
                    int$grp <- rbind(int$grp, segm[[k]]$grp)
                }
            }
        }
        if (nrow(bnd$idx) == 0) {
              bnd <- NULL
          }
        if (nrow(int$idx) == 0) {
              int <- NULL
          }
        return(list(
            loc = loc,
            bnd = (inla.ifelse(
                is.null(bnd),
                NULL,
                inla.mesh.segment(
                    bnd$loc,
                    bnd$idx,
                    bnd$grp,
                    TRUE,
                    crs
                )
            )),
            int = (inla.ifelse(
                is.null(int),
                NULL,
                inla.mesh.segment(
                    int$loc,
                    int$idx,
                    int$grp,
                    FALSE,
                    crs
                )
            ))
        ))
    }

    segm <- (c(
        lapply(
            inla.ifelse(
                inherits(boundary, "list"),
                boundary, list(boundary)
            ),
            function(x) {
                homogenise.segm.input(x, TRUE, crs = crs)
            }
        ),
        lapply(
            inla.ifelse(
                inherits(interior, "list"),
                interior, list(interior)
            ),
            function(x) {
                homogenise.segm.input(x, FALSE, crs = crs)
            }
        )
    ))
    segm <- homogenise.segm.grp(segm)

    join.segm.input(
        segm = segm, segm.offset = segm.offset, loc.offset = loc.offset,
        crs = crs
    )
}


##
## Old code.  Filtering is now done in fmesher itself.
## Retained for now so that we can check if the results are the same.
##
inla.mesh.filter.locations <- function(loc, cutoff) {
    ## Map locations to nodes, avoiding near-duplicates.
    loc.n <- nrow(loc)
    loc.dim <- ncol(loc)

    loc.is.na <- (rowSums(is.na(loc)) > 0)
    if (sum(loc.is.na) > 0) {
          stop("NAs in locations not yet supported.")
      }

    node.coord <- matrix(nrow = loc.n, ncol = loc.dim)
    map.loc.to.node <- rep(0L, nrow = loc.n)
    excluded <- c()
    loc.i <- 1L
    node.i.max <- 1L
    node.coord[node.i.max, ] <- loc[loc.i, ]
    map.loc.to.node[[loc.i]] <- node.i.max
    for (loc.i in 2:loc.n) {
        loc.to.node.dist <-
            sqrt(rowSums((as.matrix(rep(1, node.i.max)) %*%
                loc[loc.i, , drop = FALSE] -
                node.coord[1:node.i.max, , drop = FALSE])^2))
        if (min(loc.to.node.dist) > cutoff) {
            node.i.max <- node.i.max + 1L
            node.coord[node.i.max, ] <- loc[loc.i, , drop = FALSE]
            map.loc.to.node[[loc.i]] <- node.i.max
        } else {
            excluded <- c(excluded, loc.i)
        }
    }
    ## Remove excess nodes.
    node.coord <- node.coord[1:node.i.max, ]

    ## Identify nearest nodes for excluded locations.
    for (loc.i in excluded) {
        loc.to.node.dist <-
            sqrt(rowSums((as.matrix(rep(1, node.i.max)) %*%
                loc[loc.i, , drop = FALSE] -
                node.coord)^2))
        node.i <- which.min(loc.to.node.dist)
        map.loc.to.node[[loc.i]] <- node.i
    }

    return(list(loc = node.coord, node.idx = map.loc.to.node))
}





# Mesh creation ----

inla.mesh <- function(...) {
    args <- list(...)
    if (length(args) > 0) {
        if (inherits(args[[1]], "inla.mesh")) {
            warning("'inla.mesh(mesh, ...)' is deprecated.  Use 'inla.mesh.query(mesh, ...)' instead.")
            return(inla.mesh.query(...))
        }
    }
    warning("'inla.mesh(...)' is deprecated.  Use 'inla.mesh.create(...)' instead.")
    return(inla.mesh.create(...))
}




#' Low level function for high-quality triangulations
#'
#' Create a constrained refined Delaunay triangulation (CRDT) for a set of
#' spatial locations.
#'
#' `inla.mesh.create` generates triangular meshes on subsets of
#' \eqn{R^2}{R^2} and \eqn{S^2}{S^2}.  Use the higher level wrapper function
#' [inla.mesh.2d()] for greater control over mesh resolution and
#' coarser domain extensions.
#'
#' `inla.delaunay` is a wrapper function for obtaining the convex hull of
#' a point set and calling `inla.mesh.create` to generate the classical
#' Delaunay tringulation.
#'
#' @aliases inla.mesh inla.mesh.create inla.delaunay
#' @param loc Matrix of point locations.  Can alternatively be a
#' `SpatialPoints` or `SpatialPointsDataFrame` object.
#' @param tv A triangle-vertex index matrix, specifying an existing
#' triangulation.
#' @param boundary A list of `inla.mesh.segment` objects, generated by
#' [inla.mesh.segment()], specifying boundary constraint segments.
#' @param interior A list of `inla.mesh.segment` objects, generated by
#' [inla.mesh.segment()], specifying interior constraint segments.
#' @param extend `logical` or `list` specifying whether to extend the
#' data region, with parameters \describe{ \item{list("n")}{the number of edges
#' in the extended boundary (default=8)} \item{list("offset")}{the extension
#' distance.  If negative, interpreted as a factor relative to the approximate
#' data diameter (default=-0.10)} } Setting to `FALSE` is only useful in
#' combination `lattice` or `boundary`.
#' @param refine `logical` or `list` specifying whether to refine the
#' triangulation, with parameters \describe{ \item{list("min.angle")}{the
#' minimum allowed interior angle in any triangle.  The algorithm is guaranteed
#' to converge for `min.angle` at most 21 (default=`21`)}
#' \item{list("max.edge")}{the maximum allowed edge length in any triangle.  If
#' negative, interpreted as a relative factor in an ad hoc formula depending on
#' the data density (default=`Inf`)} \item{list("max.n.strict")}{the
#' maximum number of vertices allowed, overriding `min.angle` and
#' `max.edge` (default=-1, meaning no limit)} \item{list("max.n")}{the
#' maximum number of vertices allowed, overriding `max.edge` only
#' (default=-1, meaning no limit)} }
#' @param lattice An `inla.mesh.lattice` object, generated by
#' [inla.mesh.lattice()], specifying points on a regular lattice.
#' @param globe Subdivision resolution for a semi-regular spherical
#' triangulation with equidistant points along equidistant latitude bands.
#' @param cutoff The minimum allowed distance between points.  Point at most as
#' far apart as this are replaced by a single vertex prior to the mesh
#' refinement step.
#' @param plot.delay On Linux (and Mac if appropriate X11 libraries are
#' installed), specifying a numeric value activates a rudimentary plotting
#' system in the underlying `fmesher` program, showing the triangulation
#' algorithm at work.
#' @param data.dir Where to store the `fmesher` data files.  Defaults to
#' `tempdir()` if `keep` is `FALSE`, otherwise
#' `"inla.mesh.data"`.
#' @param keep `TRUE` if the data files should be kept in `data.dir`
#' or deleted afterwards.  Defaults to true if `data.dir` is specified,
#' otherwise false.  Warning: If `keep` is false, `data.dir` and its
#' contents will be deleted (unless set to `tempdir()`).
#' @param timings If `TRUE`, obtain timings for the mesh construction.
#' @param quality.spec List of vectors of per vertex `max.edge` target
#' specification for each location in `loc`, `boundary/interior`
#' (`segm`), and `lattice`.  Only used if refining the mesh.
#' @param crs An optional `CRS` or `inla.CRS` object
#' @param ...  Optional parameters passed on to `inla.mesh.create`.
#' @return An `inla.mesh` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.2d()], [inla.mesh.1d()],
#' [inla.mesh.segment()], [inla.mesh.lattice()],
#' [inla.mesh.query()]
#' @examples
#'
#' loc <- matrix(runif(10 * 2), 10, 2)
#'
#' mesh <- inla.delaunay(loc)
#' plot(mesh)
#'
#' mesh <- inla.mesh.create(loc,
#'     interior = inla.mesh.segment(idx = 1:2),
#'     extend = TRUE,
#'     refine = list(max.edge = 0.1)
#' )
#' plot(mesh)
#'
#' loc2 <- matrix(c(0, 1, 1, 0, 0, 0, 1, 1), 4, 2)
#' mesh2 <- inla.mesh.create(
#'     loc = loc,
#'     boundary = inla.mesh.segment(loc2),
#'     interior = inla.mesh.segment(idx = 1:2),
#'     quality.spec = list(segm = 0.2, loc = 0.05),
#'     refine = list(min.angle = 26)
#' )
#' plot(mesh2)
#' @export inla.mesh.create
inla.mesh.create <- function(loc = NULL, tv = NULL,
                             boundary = NULL, interior = NULL,
                             extend = (missing(tv) || is.null(tv)),
                             refine = FALSE,
                             lattice = NULL,
                             globe = NULL,
                             cutoff = 1e-12,
                             plot.delay = NULL,
                             data.dir,
                             keep = (!missing(data.dir) && !is.null(data.dir)),
                             timings = FALSE,
                             quality.spec = NULL,
                             crs = NULL) {
    if (!timings) {
        system.time <- function(expr) {
            expr
            structure(c(0, 0, 0, 0, 0), class = "proc_time")
        }
    }

    time.pre <- system.time({ ## Pre-processing timing start

        crs.target <- crs
        if (!is.null(crs) &&
            inla.crs_is_geocent(crs)) {
            ## Build all geocentric meshes on a sphere, and transform afterwards,
            ## to allow general geoids.
            crs <- inla.CRS("sphere")
        }

        if (!(missing(loc) || is.null(loc))) {
            ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
            if (inherits(loc, "SpatialPoints") ||
                inherits(loc, "SpatialPointsDataFrame")) {
                loc <- inla.spTransform(coordinates(loc),
                    inla.sp_get_crs(loc),
                    crs,
                    passthrough = TRUE
                )
            }

            if (!is.matrix(loc)) {
                loc <- as.matrix(loc)
            }
            if (!is.double(loc)) {
                storage.mode(loc) <- "double"
            }
        }
        loc.n <- max(0L, nrow(loc))

        if (is.logical(extend) && extend) extend <- list()
        if (is.logical(refine) && refine) refine <- list()

        if (missing(lattice) || is.null(lattice)) {
            lattice <- list(loc = NULL, segm = NULL)
            lattice.n <- 0L
        } else {
            inla.require.inherits(lattice, "inla.mesh.lattice", "'lattice'")
            if (!is.null(tv)) {
                warning("Both 'lattice' and 'tv' specified.  Ignoring 'tv'.")
                tv <- NULL
            }
            if (!inherits(extend, "list")) {
                boundary <- (c(
                    inla.ifelse(
                        inherits(boundary, "list"),
                        boundary, list(boundary)
                    ),
                    list(lattice$segm)
                ))
            }

            if (!is.null(lattice[["crs"]])) {
                lattice$loc <- inla.spTransform(lattice$loc,
                    lattice$crs,
                    crs,
                    passthrough = TRUE
                )
            }

            if (!is.matrix(lattice$loc)) {
                lattice$loc <- as.matrix(lattice$loc)
            }
            if (!is.double(lattice$loc)) {
                storage.mode(lattice$loc) <- "double"
            }
        }
        lattice.n <- max(0L, nrow(lattice$loc))

        segm <- (inla.mesh.parse.segm.input(boundary,
            interior,
            loc.n,
            0L,
            crs = crs
        ))
        segm.n <- max(0, nrow(segm$loc))
        ## Run parse again now that we know where the indices should point:
        segm <- (inla.mesh.parse.segm.input(boundary,
            interior,
            0L,
            segm.n + lattice.n,
            crs = crs
        ))

        loc0 <- rbind(segm$loc, lattice$loc, loc)
        if ((!is.null(loc0)) && (nrow(loc0) > 0)) {
            idx0 <- seq_len(nrow(loc0))
            if (any(rowSums(!is.finite(loc0)) > 0)) {
                stop("Non-finite values (NA, NaN, Inf, or similar) detected in location inputs.")
            }
        } else {
            idx0 <- c()
        }

        if (is.null(quality.spec)) {
            quality <- NULL
        } else {
            quality <- rep(NA, segm.n + lattice.n + loc.n)
            ## Order must be same as for loc0 above.
            if (!is.null(quality.spec$segm)) {
                quality[seq_len(segm.n)] <- quality.spec$segm
            }
            if (!is.null(quality.spec$lattice)) {
                quality[segm.n + seq_len(lattice.n)] <- quality.spec$lattice
            }
            if (!is.null(quality.spec$loc)) {
                quality[segm.n + lattice.n + seq_len(loc.n)] <- quality.spec$loc
            }
            ## NA:s will be replaced with max.edge settings below.
        }

        ## Where to put the files?
        if (keep) {
            if (missing(data.dir)) {
                data.dir <- inla.fmesher.make.dir("inla.mesh.data")
            }
            prefix <- paste(data.dir, "/mesh.", sep = "")
            keep.dir <- TRUE
        } else {
            if (missing(data.dir)) {
                prefix <- inla.tempfile(pattern = "fmesher", tmpdir = tempdir())
                prefix <- paste(prefix, ".", sep = "")
                keep.dir <- TRUE
            } else {
                data.dir <- inla.fmesher.make.dir(data.dir)
                prefix <- paste(data.dir, "/mesh.", sep = "")
                ## We should not try to delete the session tempdir...
                keep.dir <- identical(tempdir(), dirname(prefix))
            }
        }
        prefix <- inla.fmesher.make.prefix(NULL, prefix)

        if (is.null(loc0)) {
            all.args <- ""
        } else {
            loc.file <- fmesher.write(inla.affirm.double(loc0), prefix, "input.s")
            all.args <- "--input=input.s"

            if (!is.null(tv)) {
                fmesher.write(inla.affirm.integer(tv) - 1L, prefix, "input.tv")
                all.args <- paste(all.args, ",input.tv", sep = "")
            }
        }
        if (!missing(cutoff)) {
            all.args <- paste(all.args, " --cutoff=", cutoff, sep = "")
        }
        if (!is.null(segm$bnd)) {
            fmesher.write(inla.affirm.integer(segm$bnd$idx) - 1L, prefix, "input.segm.bnd.idx")
            fmesher.write(inla.affirm.integer(segm$bnd$grp), prefix, "input.segm.bnd.grp")
            all.args <- paste(all.args, " --boundary=input.segm.bnd.idx")
            all.args <- paste(all.args, " --boundarygrp=input.segm.bnd.grp")
        }
        if (!is.null(segm$int)) {
            fmesher.write(inla.affirm.integer(segm$int$idx) - 1L, prefix, "input.segm.int.idx")
            fmesher.write(inla.affirm.integer(segm$int$grp), prefix, "input.segm.int.grp")
            all.args <- paste(all.args, " --interior=input.segm.int.idx")
            all.args <- paste(all.args, " --interiorgrp=input.segm.int.grp")
        }
        if (!missing(globe) && !is.null(globe)) {
            all.args <- paste(all.args, " --globe=", globe, sep = "")
        }


        if (inherits(extend, "list")) {
            cet <- c(0, 0)
            cet[1] <- inla.ifelse(is.null(extend$n), 16, extend$n)
            cet[2] <- inla.ifelse(is.null(extend$offset), -0.1, extend$offset)
            all.args <- (paste(all.args, " --cet=",
                cet[1], ",", cet[2],
                sep = ""
            ))
        }
        if (inherits(refine, "list")) {
            rcdt <- c(0, 0, 0)
            if (!missing(globe) && !is.null(globe)) {
                max.edge.default <- pi
            } else {
                ## Multiply by to ensure cover S2 domains
                max.edge.default <- inla.diameter(loc0) * 2
            }
            if ((inherits(extend, "list")) && (!is.null(extend$offset))) {
                max.edge.default <- (max.edge.default +
                    max(0, 2 * extend$offset))
                max.edge.default <- (max.edge.default *
                    (1 + max(0, -2 * extend$offset)))
            }
            rcdt[1] <- inla.ifelse(is.null(refine$min.angle), 21, refine$min.angle)
            rcdt[2] <- (inla.ifelse(
                is.null(refine$max.edge) ||
                    is.na(refine$max.edge),
                max.edge.default,
                refine$max.edge
            ))
            rcdt[3] <- (inla.ifelse(
                is.null(refine$max.edge) ||
                    is.na(refine$max.edge),
                max.edge.default,
                refine$max.edge
            ))
            rcdt[2] <- (inla.ifelse(
                is.null(refine$max.edge.extra) ||
                    is.na(refine$max.edge.extra),
                rcdt[2], refine$max.edge.extra
            ))
            rcdt[3] <- (inla.ifelse(
                is.null(refine$max.edge.data) ||
                    is.na(refine$max.edge.date),
                rcdt[3], refine$max.edge.data
            ))
            all.args <- (paste(all.args, " --rcdt=",
                rcdt[1], ",", rcdt[2], ",", rcdt[3],
                sep = ""
            ))

            if (!is.null(refine[["max.n.strict"]]) &&
                !is.na(refine$max.n.strict)) {
                rcdt_max_n0 <- refine$max.n.strict
            } else {
                rcdt_max_n0 <- -1
            }
            if (!is.null(refine[["max.n"]]) &&
                !is.na(refine$max.n)) {
                rcdt_max_n1 <- refine$max.n
            } else {
                rcdt_max_n1 <- -1
            }
            all.args <- (paste(all.args,
                " --max_n0=", rcdt_max_n0,
                " --max_n1=", rcdt_max_n1,
                sep = ""
            ))
            is.refined <- TRUE

            if (!is.null(quality)) {
                quality[is.na(quality)] <- rcdt[3]
                quality.file <-
                    fmesher.write(
                        inla.affirm.double(as.matrix(quality)),
                        prefix, "input.quality"
                    )
                all.args <- (paste(all.args,
                    " --input=input.quality",
                    " --quality=input.quality",
                    sep = ""
                ))
            }
        } else {
            is.refined <- FALSE
        }

        if (!is.null(plot.delay)) {
            all.args <- paste(all.args, " --x11=", plot.delay, sep = "")
        }
        all.args <- paste(all.args, inla.getOption("fmesher.arg"))
    }) ## Pre-processing timing end

    ## Call fmesher:
    time.fmesher <- system.time({
        echoc <- (inla.fmesher.call(
            all.args = all.args,
            prefix = prefix
        ))
    })

    time.post <- system.time({ ## Post-processing timing start

        ## Read the mesh:
        manifold <- 1L + fmesher.read(prefix, "manifold")
        manifold <- list("M", "R2", "S2")[[manifold]]

        loc <- fmesher.read(prefix, "s")
        graph <- (list(
            tv = 1L + fmesher.read(prefix, "tv"),
            vt = 1L + fmesher.read(prefix, "vt"),
            tt = 1L + fmesher.read(prefix, "tt"),
            tti = 1L + fmesher.read(prefix, "tti"),
            vv = fmesher.read(prefix, "vv")
        ))
        graph$tv[graph$tv == 0L] <- NA
        graph$vt[graph$vt == 0L] <- NA
        graph$tt[graph$tt == 0L] <- NA
        graph$tti[graph$tti == 0L] <- NA

        ## Read the vertex input/output mapping:
        idx.all <- 1L + fmesher.read(prefix, "idx")
        idx <- (list(
            loc = (inla.ifelse(
                loc.n > 0,
                idx.all[idx0[segm.n + lattice.n + (1:loc.n)]],
                NULL
            )),
            lattice = (inla.ifelse(
                lattice.n > 0,
                idx.all[idx0[segm.n + (1:lattice.n)]],
                NULL
            )),
            segm = (inla.ifelse(
                segm.n > 0,
                idx.all[idx0[(1:segm.n)]],
                NULL
            ))
        ))
        if (!is.null(idx$loc)) idx$loc[idx$loc == 0L] <- NA
        if (!is.null(idx$lattice)) idx$lattice[idx$lattice == 0L] <- NA
        if (!is.null(idx$segm)) idx$segm[idx$segm == 0L] <- NA

        ## Read constraint segment information:
        segm.bnd <- (inla.mesh.segment(
            NULL,
            1L + fmesher.read(prefix, "segm.bnd.idx"),
            fmesher.read(prefix, "segm.bnd.grp"),
            TRUE
        ))
        segm.int <- (inla.mesh.segment(
            NULL,
            1L + fmesher.read(prefix, "segm.int.idx"),
            fmesher.read(prefix, "segm.int.grp"),
            FALSE
        ))

        ## Remap indices to remove unused vertices
        used <- !is.na(graph$vt)
        if (!all(used)) {
            used <- which(used)
            idx.map <- rep(NA, nrow(loc))
            idx.map[used] <- seq_len(length(used))
            loc <- loc[used, , drop = FALSE]
            graph$tv <- matrix(idx.map[as.vector(graph$tv)], nrow(graph$tv), 3)
            graph$vt <- graph$vt[used, , drop = FALSE]
            ## graph$tt  ## No change needed
            ## graph$tti ## No change needed
            graph$vv <- graph$vv[used, used, drop = FALSE]
            if (!is.null(idx$loc)) idx$loc <- idx.map[idx$loc]
            if (!is.null(idx$lattice)) idx$lattice <- idx.map[idx$lattice]
            if (!is.null(idx$segm)) idx$segm <- idx.map[idx$segm]
            segm.bnd$idx <- matrix(idx.map[segm.bnd$idx], nrow(segm.bnd$idx), 2)
            segm.int$idx <- matrix(idx.map[segm.int$idx], nrow(segm.int$idx), 2)
            segm.bnd$idx[segm.bnd$idx == 0L] <- NA
            segm.int$idx[segm.int$idx == 0L] <- NA
        }


        if (!keep) {
              unlink(paste(prefix, "*", sep = ""), recursive = FALSE)
          }
        if (!keep.dir) {
            unlink(dirname(prefix), recursive = TRUE)
        }

        if (!is.null(crs) &&
            !inla.identical.CRS(crs, crs.target)) {
            ## Target is a non-spherical geoid
            loc <- inla.spTransform(loc, crs, crs.target)
            crs <- crs.target
        }
    }) ## Post-processing timing end

    time.object <- system.time({ ## Object construction timing start
        mesh <- (list(
            meta = (list(
                call = match.call(),
                fmesher.args = all.args,
                time = (rbind(
                    pre = time.pre,
                    fmesher = time.fmesher,
                    post = time.post
                )),
                prefix = prefix,
                is.refined = is.refined
            )),
            manifold = manifold,
            n = nrow(loc),
            loc = loc,
            graph = graph,
            segm = list(bnd = segm.bnd, int = segm.int),
            idx = idx,
            crs = crs
        ))
        class(mesh) <- "inla.mesh"
    }) ## Object construction timing end

    ## Entire function timing
    time.total <- time.pre + time.fmesher + time.post + time.object

    mesh$meta$time <- (rbind(mesh$meta$time,
        object = time.object,
        total = time.total
    ))
    return(invisible(mesh))
}


inla.mesh.extract.segments <- function(mesh.loc,
                                       mesh.idx,
                                       mesh.grp,
                                       grp = NULL,
                                       is.bnd,
                                       crs = NULL) {
    segments <- list()
    if (nrow(mesh.idx) > 0) {
        if (is.null(grp)) {
            grp <- unique(sort(mesh.grp))
        }
        for (g in grp) {
            extract <- (mesh.grp == g)
            segments <-
                c(
                    segments,
                    list(inla.mesh.segment(mesh.loc,
                        idx = mesh.idx[extract, , drop = FALSE],
                        grp = mesh.grp[extract, drop = FALSE],
                        is.bnd = is.bnd,
                        crs = crs
                    ))
                )
        }
    }
    if (length(segments) > 0) {
          return(segments)
      } else {
          return(NULL)
      }
}



#' Constraint segment extraction for inla.mesh
#'
#' Constructs an list of `inla.mesh.segment` object from boundary or
#' interior constraint information in an [inla.mesh()] object.
#'
#'
#' @aliases inla.mesh.boundary inla.mesh.interior
#' @param mesh An `inla.mesh` object.
#' @param grp Group indices to extract.  If `NULL`, all boundary/interior
#' constrain groups are extracted.
#' @return A list of `inla.mesh.segment` objects.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.segment()], [inla.mesh.create()],
#' [inla.mesh.create.helper()]
#' @examples
#'
#' loc <- matrix(runif(100 * 2) * 1000, 100, 2)
#' mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(50, 500))
#' boundary <- inla.mesh.boundary(mesh)
#' interior <- inla.mesh.interior(mesh)
#' @export
#' @rdname inla.mesh.boundary
inla.mesh.boundary <- function(mesh, grp = NULL) {
    inla.require.inherits(mesh, "inla.mesh", "'mesh'")

    return(inla.mesh.extract.segments(
        mesh$loc,
        mesh$segm$bnd$idx,
        mesh$segm$bnd$grp,
        grp,
        TRUE,
        mesh$crs
    ))
}

#' @export
#' @rdname inla.mesh.boundary
inla.mesh.interior <- function(mesh, grp = NULL) {
    inla.require.inherits(mesh, "inla.mesh", "'mesh'")

    return(inla.mesh.extract.segments(
        mesh$loc,
        mesh$segm$int$idx,
        mesh$segm$int$grp,
        grp,
        FALSE,
        mesh$crs
    ))
}


## Generate nice triangulation, with an inner domain strictly enclosed
## by an outer domain
## The inner and outer domains can have different quality parameters
## At least one of loc, loc.domain, boundary[[1]], boundary[[2]], interior
## must be non-NULL
## For more complicated multi-step meshings, study the code and write your own.


#' High-quality triangulations
#'
#' Create a triangle mesh based on initial point locations, specified or
#' automatic boundaries, and mesh quality parameters.
#'
#'
#' @param loc Matrix of point locations to be used as initial triangulation
#' nodes.  Can alternatively be a `SpatialPoints` or
#' `SpatialPointsDataFrame` object.
#' @param loc.domain Matrix of point locations used to determine the domain
#' extent.  Can alternatively be a `SpatialPoints` or
#' `SpatialPointsDataFrame` object.
#' @param offset The automatic extension distance.  One or two values, for an
#' inner and an optional outer extension.  If negative, interpreted as a factor
#' relative to the approximate data diameter (default=-0.10???)
#' @param n The number of initial nodes in the automatic extensions
#' (default=16)
#' @param boundary A list of one or two [inla.mesh.segment()] objects
#' describing domain boundaries.
#' @param interior An [inla.mesh.segment()] object describing desired
#' interior edges.
#' @param max.edge The largest allowed triangle edge length.  One or two
#' values.
#' @param min.angle The smallest allowed triangle angle.  One or two values.
#' (Default=21)
#' @param cutoff The minimum allowed distance between points.  Point at most as
#' far apart as this are replaced by a single vertex prior to the mesh
#' refinement step.
#' @param max.n.strict The maximum number of vertices allowed, overriding
#' `min.angle` and `max.edge` (default=-1, meaning no limit).  One or
#' two values, where the second value gives the number of additional vertices
#' allowed for the extension.
#' @param max.n The maximum number of vertices allowed, overriding
#' `max.edge` only (default=-1, meaning no limit).  One or two values,
#' where the second value gives the number of additional vertices allowed for
#' the extension.
#' @param plot.delay On Linux (and Mac if appropriate X11 libraries are
#' installed), specifying a nonnegative numeric value activates a rudimentary
#' plotting system in the underlying `fmesher` program, showing the
#' triangulation algorithm at work, with waiting time factor `plot.delay`
#' between each step.
#'
#' On all systems, specifying any negative value activates displaying the
#' result after each step of the multi-step domain extension algorithm.
#' @param crs An optional `CRS` or `inla.CRS` object
#' @return An `inla.mesh` object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.create()], [inla.delaunay()],
#' [inla.nonconvex.hull()]
#' @examples
#'
#' loc <- matrix(runif(10 * 2), 10, 2)
#'
#' if (require("splancs")) {
#'     boundary <- list(
#'         inla.nonconvex.hull(loc, 0.1, 0.15),
#'         inla.nonconvex.hull(loc, 0.2, 0.2)
#'     )
#'     offset <- NULL
#' } else {
#'     boundary <- NULL
#'     offset <- c(0.1, 0.2)
#' }
#' mesh <- inla.mesh.2d(loc, boundary = boundary, offset = offset, max.edge = c(0.05, 0.1))
#'
#' plot(mesh)
#' @export inla.mesh.2d
inla.mesh.2d <- function(loc = NULL, ## Points to include in final triangulation
                         loc.domain = NULL, ## Points that determine the automatic domain
                         offset = NULL, ## Size of automatic extensions
                         n = NULL, ## Sides of automatic extension polygons
                         boundary = NULL, ## User-specified domains (list of length 2)
                         interior = NULL, ## User-specified constraints for the inner domain
                         max.edge = NULL,
                         min.angle = NULL, ## Angle constraint for the entire domain
                         cutoff = 1e-12, ## Only add input points further apart than this
                         max.n.strict = NULL,
                         max.n = NULL,
                         plot.delay = NULL,
                         crs = NULL) {
    ## plot.delay: Do plotting.
    ## NULL --> No plotting
    ## <0  --> Intermediate meshes displayed at the end
    ## >0   --> Dynamical fmesher plotting

    unify.one.segm <- function(segm, crs = NULL) {
        if (inherits(segm, "inla.mesh.segment")) {
            segm <- inla.spTransform(segm, crs, passthrough = TRUE)
        } else if (inherits(segm, "matrix")) {
            segm <- inla.mesh.segment(loc = segm, crs = crs)
        } else {
            segm <- inla.spTransform(as.inla.mesh.segment(segm), crs,
                passthrough = TRUE
            )
        }
        if (ncol(segm$loc) == 2) {
            segm$loc <- cbind(segm$loc, 0.0)
        }
        segm
    }
    unify.segm.coords <- function(segm, crs = NULL) {
        if (!is.null(segm)) {
            if (inherits(segm, "list")) {
                for (j in seq_along(segm)) {
                    segm[[j]] <- unify.one.segm(segm[[j]], crs = crs)
                }
            } else {
                segm <- unify.one.segm(segm, crs = crs)
            }
        }
        segm
    }


    if ((missing(max.edge) || is.null(max.edge)) &&
        (missing(max.n.strict) || is.null(max.n.strict)) &&
        (missing(max.n) || is.null(max.n))) {
        stop("At least one of max.edge, max.n.strict, and max.n must be specified")
    }

    if (!is.null(crs)) {
        issphere <- inla.identical.CRS(crs, inla.CRS("sphere"))
        isgeocentric <- inla.crs_is_geocent(crs)
        if (isgeocentric) {
            crs.target <- crs
            crs <- inla.CRS("sphere")
        }
    }

    ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
    if (!(missing(loc) || is.null(loc)) &&
        (inherits(loc, "SpatialPoints") ||
            inherits(loc, "SpatialPointsDataFrame"))) {
        loc <- inla.spTransform(coordinates(loc),
            inla.sp_get_crs(loc),
            crs,
            passthrough = TRUE
        )
    }
    if (!(missing(loc.domain) || is.null(loc.domain)) &&
        (inherits(loc.domain, "SpatialPoints") ||
            inherits(loc.domain, "SpatialPointsDataFrame"))) {
        loc.domain <- inla.spTransform(coordinates(loc.domain),
            inla.sp_get_crs(loc.domain),
            crs,
            passthrough = TRUE
        )
    }

    if (missing(loc) || is.null(loc)) {
        loc <- matrix(c(0.0), 0, 3)
    } else if (!is.matrix(loc)) {
        loc <- as.matrix(loc)
    }
    if (missing(loc.domain) || is.null(loc.domain)) {
        loc.domain <- loc
    } else if (!is.matrix(loc.domain)) {
        loc.domain <- as.matrix(loc.domain)
    }
    if (missing(boundary)) {
        boundary <- list(NULL)
    } else {
        if (!inherits(boundary, "list")) {
            boundary <- list(boundary)
        }
    }
    if (missing(interior)) {
        interior <- NULL
    }
    if (missing(offset) || is.null(offset)) {
        if (length(boundary) < 2) {
            offset <- -0.05
        } else {
            offset <- c(-0.05, -0.15)
        }
    }
    if (missing(n) || is.null(n)) {
        n <- c(8)
    }
    if (missing(max.edge) || is.null(max.edge)) {
        max.edge <- c(NA)
    }
    if (missing(min.angle) || is.null(min.angle)) {
        min.angle <- c(21)
    }
    if (missing(max.n.strict) || is.null(max.n.strict)) {
        max.n.strict <- c(NA)
    }
    if (missing(max.n) || is.null(max.n)) {
        max.n <- c(NA)
    }
    if (missing(cutoff) || is.null(cutoff)) {
        cutoff <- 1e-12
    }
    if (missing(plot.delay) || is.null(plot.delay)) {
        plot.delay <- NULL
    }

    num.layers <-
        max(c(
            length(boundary), length(offset), length(n),
            length(min.angle), length(max.edge),
            length(max.n.strict), length(max.n)
        ))
    if (num.layers > 2) {
        warning(paste("num.layers=", num.layers, " > 2 detected.  ",
            "Excess information ignored.",
            sep = ""
        ))
        num.layers <- 2
    }

    if (length(boundary) < num.layers) {
        boundary <- c(boundary, list(NULL))
    }
    if (length(min.angle) < num.layers) {
        min.angle <- c(min.angle, min.angle)
    }
    if (length(max.n.strict) < num.layers) {
        max.n0 <- c(max.n.strict, max.n.strict)
    }
    if (length(max.n) < num.layers) {
        max.n <- c(max.n, max.n)
    }
    if (length(max.edge) < num.layers) {
        max.edge <- c(max.edge, max.edge)
    }
    if (length(offset) < num.layers) {
        offset <- c(offset, -0.15)
    }
    if (length(n) < num.layers) {
        n <- c(n, 16)
    }
    if (length(n) < num.layers) {
        n <- c(n, 16)
    }

    ## Unify the dimensionality of the point input.
    if (!is.null(loc) && (ncol(loc) == 2)) {
        loc <- cbind(loc, 0.0)
    }
    if (!is.null(loc.domain) && (ncol(loc.domain) == 2)) {
        loc.domain <- cbind(loc.domain, 0.0)
    }
    ## Unify the dimensionality of the boundary&interior segments input
    ## and optionally transform coordinates.
    for (k in seq_len(num.layers)) {
        if (!is.null(boundary[[k]])) {
            boundary[[k]] <- unify.segm.coords(boundary[[k]], crs = crs)
        }
    }
    interior <- unify.segm.coords(interior, crs = crs)

    ## Triangulate to get inner domain boundary
    ## Constraints included only to get proper domain extent
    ## First, attach the loc points to the domain definition set
    if (!is.null(loc) && !is.null(loc.domain)) {
        loc.domain <- rbind(loc.domain, loc)
    }
    mesh1 <-
        inla.mesh.create(
            loc = loc.domain,
            boundary = boundary[[1]],
            interior = interior,
            cutoff = cutoff,
            extend = list(n = n[1], offset = offset[1]),
            refine = FALSE,
            plot.delay = plot.delay,
            crs = crs
        )

    ## Save the resulting boundary
    boundary1 <- inla.mesh.boundary(mesh1)
    interior1 <- inla.mesh.interior(mesh1)

    if (!is.null(plot.delay) && (plot.delay < 0)) {
        inla.dev.new()
        plot(mesh1)
    }

    ## Triangulate inner domain
    mesh2 <-
        inla.mesh.create(
            loc = loc,
            boundary = boundary1,
            interior = interior1,
            cutoff = cutoff,
            extend = FALSE, ## Should have no effect
            refine =
                list(
                    min.angle = min.angle[1],
                    max.edge = max.edge[1],
                    max.edge.extra = max.edge[1],
                    max.n.strict = max.n.strict[1],
                    max.n = max.n[1]
                ),
            plot.delay = plot.delay,
            crs = crs
        )

    boundary2 <- inla.mesh.boundary(mesh2)
    interior2 <- inla.mesh.interior(mesh2)

    if (!is.null(plot.delay) && (plot.delay < 0)) {
        inla.dev.new()
        plot(mesh2)
    }

    if (num.layers == 1) {
        if (!is.null(crs) && isgeocentric && !issphere) {
            mesh2$loc <- inla.spTransform(mesh2$loc, mesh2$crs, crs.target)
            mesh2$crs <- crs.target
        }
        return(invisible(mesh2))
    }

    ## Triangulate inner+outer domain
    mesh3 <-
        inla.mesh.create(
            loc = rbind(loc, mesh2$loc),
            boundary = boundary[[2]],
            interior = c(boundary2, interior2),
            cutoff = cutoff,
            extend = list(n = n[2], offset = offset[2]),
            refine =
                list(
                    min.angle = min.angle[2],
                    max.edge = max.edge[2],
                    max.edge.extra = max.edge[2],
                    max.n.strict = mesh2$n + max.n.strict[2],
                    max.n = mesh2$n + max.n[2]
                ),
            plot.delay = plot.delay,
            crs = crs
        )

    ## Hide generated points, to match regular inla.mesh.create output
    mesh3$idx$loc <- mesh3$idx$loc[seq_len(nrow(loc))]

    ## Obtain the corresponding segm indices.
    segm.loc <- matrix(0.0, 0, 3)
    for (k in seq_along(boundary)) {
        if (!is.null(boundary[[k]])) {
            segm.loc <- rbind(segm.loc, boundary[[k]]$loc)
        }
    }
    for (k in seq_along(interior)) {
        if (!is.null(interior[[k]])) {
            segm.loc <- rbind(segm.loc, interior[[k]]$loc)
        }
    }
    if (nrow(segm.loc) > 0) {
        proj <- inla.mesh.project(mesh3, loc = segm.loc)
        mesh3$idx$segm <- rep(NA, nrow(segm.loc))
        if (any(proj$ok)) {
            t.idx <- proj$t[proj$ok]
            tv.idx <- max.col(proj$bary[proj$ok, , drop = FALSE],
                ties.method = "first"
            )
            mesh3$idx$segm[proj$ok] <-
                mesh3$graph$tv[t.idx + nrow(mesh3$graph$tv) * (tv.idx - 1)]
        }
    } else {
        mesh3$idx$segm <- NULL
    }

    if (!is.null(crs) && isgeocentric && !issphere) {
        mesh3$loc <- inla.spTransform(mesh3$loc, mesh3$crs, crs.target)
        mesh3$crs <- crs.target
    }

    if (!is.null(plot.delay) && (plot.delay < 0)) {
        inla.dev.new()
        plot(mesh3)
    }

    return(invisible(mesh3))
}

## Support for legacy code:


#' High-quality triangulations
#'
#' Create a triangle mesh based on initial point locations, specified or
#' automatic boundaries, and mesh quality parameters.
#'
#'
#' @param points Matrix of point locations to be used as initial triangulation
#' nodes.
#' @param points.domain Matrix of point locations used to determine the domain
#' extent.
#' @param \dots Additional arguments passed on to [inla.mesh.2d()].
#' @note Obsolete function kept for backwards compatibility.  Use
#' [inla.mesh.2d()] instead.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.2d()]
#' @keywords internal
#' @export inla.mesh.create.helper
inla.mesh.create.helper <- function(points = NULL, points.domain = NULL, ...) {
    return(invisible(inla.mesh.2d(loc = points, loc.domain = points.domain, ...)))
}

## Example:
## if (FALSE) {
## mesh =
## inla.mesh.create.helper(loc=matrix(runif(20),10,2)*200,
## n=c(8,16),
## offset=c(10,140),
## max.edge=c(25,1000),
## min.angle=26,
## cutoff=0,
## plot.delay=-1
## )
## }
##

#' @export
#' @rdname inla.mesh.create
inla.delaunay <- function(loc, ...) {
    ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
    if (!(missing(loc) || is.null(loc)) &&
        (inherits(loc, "SpatialPoints") ||
            inherits(loc, "SpatialPointsDataFrame"))) {
        crs <- inla.sp_get_crs(loc)
        loc <- coordinates(loc)
    } else {
        crs <- NULL
    }

    hull <- chull(loc[, 1], loc[, 2])
    bnd <- inla.mesh.segment(loc = loc[hull[length(hull):1], ], is.bnd = TRUE)
    mesh <-
        inla.mesh.create(
            loc = loc,
            boundary = bnd,
            extend = list(n = 3),
            refine = FALSE,
            crs = crs,
            ...
        )

    return(invisible(mesh))
}





# Queries ----


#' High-quality triangulations
#'
#' Query information about an inla.mesh object.
#'
#'
#' @param mesh An `inla.mesh` object.
#' @param \dots Query arguments.
#' \itemize{
#' \item tt.neighbours Compute
#' neighbour triangles for triangles; list of vectors: list(triangles, orders)
#'
#' \item vt.neighbours Compute neighbour triangles for vertices; list of
#' vectors: list(vertices, orders)
#' }
#' @return A list of query results.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.create()], [inla.mesh.segment()],
#' [inla.mesh.lattice()]
#' @examples
#'
#' loc <- matrix(c(0.1, 0.15), 1, 2)
#' lattice <- inla.mesh.lattice(dims = c(10, 10))
#' mesh <- inla.mesh.create(loc = loc, lattice = lattice, extend = FALSE)
#'
#' vt <- which(inla.mesh.query(mesh,
#'     vt.neighbours = list(
#'         mesh$idx$loc,
#'         4:6
#'     )
#' )$vt.neighbours)
#'
#' mesh2 <- inla.mesh.create(mesh$loc,
#'     tv = mesh$graph$tv[vt, , drop = FALSE],
#'     refine = FALSE, extend = FALSE
#' )
#' @export
inla.mesh.query <- function(mesh, ...) {
    inla.require.inherits(mesh, "inla.mesh", "'mesh'")

    not.known <- function(mesh, queryname) {
        stop(paste("Query '", queryname,
            "' unknown.",
            sep = ""
        ))
    }
    not.implemented <- function(mesh, queryname) {
        stop(paste("Query '", queryname,
            "' not implemented for inla.mesh.",
            sep = ""
        ))
        ## stop(paste("Query '", queryname,
        ## "' not implemented for inla.mesh for mesh type '",
        ## model$type, "'.", sep=""))
    }

    result <- list()
    queries <- inla.parse.queries(...)
    if (length(queries) == 0L) {
          return(result)
      }

    for (query.idx in 1:length(queries)) {
        query <- names(queries)[query.idx]
        param <- queries[[query.idx]]
        answer <- NULL
        query <- (match.arg(query, c(
            "tt.neighbours",
            "vt.neighbours"
        )))
        if (identical(query, "tt.neighbours")) {
            if (is.null(param)) {
                  param <- list(c(1))
              }
            if (length(param) < 2) {
                  param <- c(as.list(param), list(c(1)))
              }
            nT <- nrow(mesh$graph$tv)
            i <- rep(1:nT, 3)
            j <- as.vector(mesh$graph$tt)
            i <- i[!is.na(j)]
            j <- j[!is.na(j)]
            tt <- sparseMatrix(i = i, j = j, x = rep(1, length(i)), dims = c(nT, nT))
            answer <- (sparseMatrix(
                i = param[[1]],
                j = rep(1, length(param[[1]])),
                x = 1,
                dims = c(nT, 1)
            ))
            tt0 <- (answer == 0.5) * 1
            tt1 <- answer
            order <- 0
            while (order < min(param[[2]])) {
                order <- order + 1
                tt0 <- tt1
                tt1 <- answer
                answer <- ((((tt %*% answer) > 0) - tt1 - tt0) > 0)
            }
            while (order < max(param[[2]])) {
                order <- order + 1
                answer <- ((((tt %*% answer) > 0) - tt1 - tt0) > 0)
            }

            ## not.implemented(mesh,query)
        } else if (identical(query, "vt.neighbours")) {
            if (is.null(param)) {
                  param <- list(1)
              }
            if (length(param) < 2) {
                  param <- c(as.list(param), list(c(1)))
              }
            nV <- nrow(mesh$loc)
            nT <- nrow(mesh$graph$tv)
            i <- rep(1:nT, 3)
            j <- as.vector(mesh$graph$tv)
            # i = i[!is.na(j)]
            # j = j[!is.na(j)]
            tv <- sparseMatrix(i = i, j = j, x = rep(1, length(i)), dims = c(nT, nV))
            vv <- (sparseMatrix(
                i = param[[1]],
                j = rep(1, length(param[[1]])),
                x = 1,
                dims = c(nV, 1)
            ))
            vt <- (tv %*% vv) > 0
            vv0 <- (vv == 0.5) * 1
            vv1 <- vv
            vt0 <- (tv %*% vv0) > 0
            vt1 <- vt
            order <- 0
            while (order < min(param[[2]])) {
                order <- order + 1
                vv0 <- vv1
                vv1 <- vv
                vv <- ((((mesh$graph$vv %*% vv) > 0) - vv1 - vv0) > 0)
                vt0 <- vt1
                vt1 <- vt
                vt <- (((tv %*% vv) > 0) - vt1 - vt0) > 0
            }
            while (order < max(param[[2]])) {
                order <- order + 1
                vv <- ((((mesh$graph$vv %*% vv) > 0) - vv1 - vv0) > 0)
                vt <- (((((tv %*% vv) > 0) - vt1 - vt0) > 0) + vt) > 0
            }
            answer <- vt

            ## not.implemented(mesh,query)
        } else if (!identical(query, "")) {
            not.known(mesh, query)
        }
        ## Expand the result list:
        result[query.idx] <- list(NULL)
        names(result)[query.idx] <- query
        ## Set the answer:
        if (!is.null(answer)) {
              result[[query.idx]] <- answer
          }
    }

    return(result)
}



#' Summarizing triangular mesh objects
#'
#' Construct and print `inla.mesh` object summaries
#'
#'
#' @aliases summary.inla.mesh print.summary.inla.mesh
#' @param object an object of class `"inla.mesh"`, usually a result of a
#' call to [inla.mesh.create()] or [inla.mesh.2d()].
#' @param x an object of class `"summary.inla.mesh"`, usually a result of
#' a call to [summary.inla.mesh()].
#' @param verbose If `TRUE`, produce a more detailed output.
#' @param \dots further arguments passed to or from other methods.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @method summary inla.mesh
#' @export
summary.inla.mesh <- function(object, verbose = FALSE, ...) {
    x <- object
    ## provides a summary for a mesh object
    inla.require.inherits(x, "inla.mesh", "'x'")

    ret <- list(verbose = verbose)
    if (verbose) {
        ret <- (c(ret, list(
            call = x$meta$call,
            fmesher.args = x$meta$fmesher.args,
            prefix = x$meta$prefix,
            time = x$meta$time,
            is.refined = x$meta$is.refined
        )))
    }
    ret <- (c(ret, list(
        manifold = x$manifold,
        nV = x$n,
        nT = nrow(x$graph$tv),
        xlim = range(x$loc[, 1]),
        ylim = range(x$loc[, 2]),
        zlim = range(x$loc[, 3])
    )))
    if (inla.has_PROJ6()) {
        if (is.null(x$crs) || is.null(inla.crs_get_wkt(x$crs))) {
            ret <- c(ret, list(crs = "N/A", crs_proj4 = "N/A"))
        } else {
            ret <- c(ret, list(crs = inla.crs_get_wkt(x$crs)))
            ret <- c(ret, list(crs_proj4 = rgdal::showP4(ret$crs)))
        }
    } else {
        if (is.na(inla.CRSargs(x$crs))) {
            ret <- c(ret, list(crs = "N/A"))
        } else {
            ret <- c(ret, list(crs = inla.CRSargs(x$crs)))
        }
    }

    my.segm <- function(x) {
        if (is.null(x)) {
              return(list(n = 0, grps = NULL))
          }
        n <- max(0, nrow(x$idx))
        if (max(0, length(unique(x$grp))) > 0) {
            grps <- unique(x$grp)
        } else {
            grps <- NULL
        }
        return(list(n = n, grps = grps))
    }
    if (!is.null(x$segm)) {
        ret <- (c(ret, list(
            segm.bnd = my.segm(x$segm$bnd),
            segm.int = my.segm(x$segm$int)
        )))
    } else {
        ret <- (c(ret, list(
            segm.bnd = my.segm(NULL),
            segm.int = my.segm(NULL)
        )))
    }

    class(ret) <- "summary.inla.mesh"
    return(ret)
}

#' @export
#' @method print summary.inla.mesh
#' @rdname summary.inla.mesh
print.summary.inla.mesh <- function(x, ...) {
    my.print.proc_time <- function(x, ...) {
        if (!is.matrix(x)) {
            y <- matrix(x, 1, 5)
        } else {
            y <- x
        }
        for (k in 1:nrow(x)) {
            if (!is.na(y[k, 4L])) {
                  y[k, 1L] <- y[k, 1L] + y[k, 4L]
              }
            if (!is.na(y[k, 5L])) {
                  y[k, 2L] <- y[k, 2L] + y[k, 5L]
              }
        }
        y <- y[, 1L:3L]
        colnames(y) <- c(gettext("user"), gettext("system"), gettext("elapsed"))
        print(y, ...)
        invisible(x)
    }


    inla.require.inherits(x, "summary.inla.mesh", "'x'")

    if (x$verbose) {
        cat("\nCall:\n")
        print(x$call)

        cat("\nfmesher:\t", x$fmesher.args, "\n", sep = "")
        cat("prefix:\t\t", x$prefix, "\n", sep = "")

        cat("\nTimings:\n")
        my.print.proc_time(x$time)
    }

    cat("\nManifold:\t", x$manifold, "\n", sep = "")
    if (inla.has_PROJ6()) {
        cat("CRS/LegacyPROJ4:\t", x$crs_proj4, "\n", sep = "")
        if (x$verbose) {
            cat("CRS/WKT:\n", x$crs, "\n", sep = "")
        } else {
            cat("CRS/WKT: (only shown with verbose = TRUE)", "\n", sep = "")
        }
    } else {
        cat("CRS/PROJ4:\t", x$crs, "\n", sep = "")
    }
    if (x$verbose) {
        cat("Refined:\t", x$is.refined, "\n", sep = "")
    }
    cat("Vertices:\t", as.character(x$nV), "\n", sep = "")
    cat("Triangles:\t", as.character(x$nT), "\n", sep = "")

    my.print.segm <- function(x) {
        cat(as.character(x$n))
        if (!is.null(x$grps)) {
            n <- length(x$grps)
            cat(" (", n, " group", inla.ifelse(n == 1, "", "s"), sep = "")
            if (n <= 10) {
                cat(":", x$grps, sep = " ")
            } else {
                cat(":", x$grps[1:10], "...", sep = " ")
            }
            cat(")")
        }
        cat("\n", sep = "")
        return(invisible())
    }
    cat("Boundary segm.:\t")
    my.print.segm(x$segm.bnd)
    cat("Interior segm.:\t")
    my.print.segm(x$segm.int)
    cat("xlim:\t", x$xlim[1], " ", x$xlim[2], "\n", sep = "")
    cat("ylim:\t", x$ylim[1], " ", x$ylim[2], "\n", sep = "")
    cat("zlim:\t", x$zlim[1], " ", x$zlim[2], "\n", sep = "")
    cat("\n")

    invisible(x)
}





#' Methods for projecting to/from an inla.mesh
#'
#' Calculate a lattice projection to/from an [inla.mesh()]
#'
#' The call `inla.mesh.project(mesh, loc, field=..., ...)`, is a shortcut
#' to inla.mesh.project(inla.mesh.projector(mesh, loc), field).
#'
#' @aliases inla.mesh.project inla.mesh.projector inla.mesh.projector.inla.mesh
#' inla.mesh.project.inla.mesh inla.mesh.project.inla.mesh.projector
#' inla.mesh.project.inla.mesh.1d inla.mesh.projector.inla.mesh.1d
#' @param mesh An [inla.mesh()] or [inla.mesh.1d()] object.
#' @param loc Projection locations.  Can be a matrix or a `SpatialPoints`
#' or a `SpatialPointsDataFrame` object.
#' @param lattice An [inla.mesh.lattice()] object.
#' @param xlim X-axis limits for a lattice. For R2 meshes, defaults to covering
#' the domain.
#' @param ylim Y-axis limits for a lattice. For R2 meshes, defaults to covering
#' the domain.
#' @param dims Lattice dimensions.
#' @param projector An `inla.mesh.projector` object.
#' @param field Basis function weights, one per mesh basis function, describing
#' the function to be avaluated at the projection locationssFunction values for
#' on the mesh
#' @param projection One of `c("default", "longlat", "longsinlat",
#' "mollweide")`.
#' @param crs An optional CRS or inla.CRS object associated with `loc`
#' and/or `lattice`.
#' @param \dots Additional arguments passed on to methods.
#' @return For `inla.mesh.project(mesh, ...)`, a list with projection
#' information.  For `inla.mesh.projector(mesh, ...)`, an
#' `inla.mesh.projector` object.  For `inla.mesh.project(projector,
#' field, ...)`, a field projected from the mesh onto the locations given by
#' the projector object.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh()], [inla.mesh.1d()],
#' [inla.mesh.lattice()]
#' @examples
#'
#' n <- 20
#' loc <- matrix(runif(n * 2), n, 2)
#' mesh <- inla.mesh.create(loc, refine = list(max.edge = 0.05))
#' proj <- inla.mesh.projector(mesh)
#' field <- cos(mesh$loc[, 1] * 2 * pi * 3) * sin(mesh$loc[, 2] * 2 * pi * 7)
#' image(proj$x, proj$y, inla.mesh.project(proj, field))
#' \donttest{
#' if (require(rgl)) {
#'     plot(mesh, rgl = TRUE, col = field, draw.edges = FALSE, draw.vertices = FALSE)
#' }
#' }
#'
#' @export inla.mesh.project
inla.mesh.project <- function(...) {
    UseMethod("inla.mesh.project")
}

#' @export
#' @rdname inla.mesh.project
inla.mesh.project.inla.mesh <- function(mesh, loc = NULL, field = NULL,
                                        crs = NULL, ...) {
    inla.require.inherits(mesh, "inla.mesh", "'mesh'")

    if (!is.null(mesh$crs) &&
        inla.crs_is_geocent(mesh$crs)) {
        crs.sphere <- inla.CRS("sphere")
        if (!inla.identical.CRS(mesh$crs, crs.sphere)) {
            ## Convert the mesh to a perfect sphere.
            mesh$loc <- inla.spTransform(mesh$loc, mesh$crs, crs.sphere)
            mesh$manifold <- "S2"
            mesh$crs <- crs.sphere
        }
    }

    ## Handle loc given as SpatialPoints or SpatialPointsDataFrame object
    ## or an explicit crs
    if (!(missing(loc) || is.null(loc))) {
        if (inherits(loc, "SpatialPoints") ||
            inherits(loc, "SpatialPointsDataFrame")) {
            if (is.null(mesh$crs)) {
                loc <- coordinates(loc)
            } else {
                loc <- inla.spTransform(coordinates(loc),
                    inla.sp_get_crs(loc),
                    mesh$crs,
                    passthrough = FALSE
                )
            }
        } else if (!is.null(crs)) {
            if (!is.null(mesh$crs)) {
                loc <- inla.spTransform(loc,
                    crs,
                    mesh$crs,
                    passthrough = FALSE
                )
            }
        }
    }

    if (!missing(field) && !is.null(field)) {
        proj <- inla.mesh.projector(mesh, loc = loc, ...)
        return(inla.mesh.project(proj, field = field))
    }

    jj <-
        which(rowSums(matrix(is.na(as.vector(loc)),
            nrow = nrow(loc),
            ncol = ncol(loc)
        )) == 0)
    smorg <- (inla.fmesher.smorg(mesh$loc,
        mesh$graph$tv,
        points2mesh = loc[jj, , drop = FALSE]
    ))
    ti <- matrix(0L, nrow(loc), 1)
    b <- matrix(0, nrow(loc), 3)
    ti[jj, 1L] <- smorg$p2m.t
    b[jj, ] <- smorg$p2m.b

    ok <- (ti[, 1L] > 0L)
    ti[ti[, 1L] == 0L, 1L] <- NA

    ii <- which(ok)
    A <- (sparseMatrix(
        dims = c(nrow(loc), mesh$n),
        i = rep(ii, 3),
        j = as.vector(mesh$graph$tv[ti[ii, 1L], ]),
        x = as.vector(b[ii, ])
    ))

    list(t = ti, bary = b, A = A, ok = ok)
}

#' @export
#' @rdname inla.mesh.project
inla.mesh.project.inla.mesh.1d <- function(mesh, loc, field = NULL, ...) {
    inla.require.inherits(mesh, "inla.mesh.1d", "'mesh'")

    if (!missing(field) && !is.null(field)) {
        proj <- inla.mesh.projector(mesh, loc, ...)
        return(inla.mesh.project(proj, field))
    }

    A <- inla.mesh.1d.A(mesh, loc = loc)

    return(list(
        A = A,
        ok = (loc >= mesh$interval[1]) & (loc <= mesh$interval[2])
    ))
}


#' @export
#' @rdname inla.mesh.project
inla.mesh.project.inla.mesh.projector <-
    function(projector, field, ...) {
        inla.require.inherits(projector, "inla.mesh.projector", "'projector'")

        if (is.data.frame(field)) {
            field <- as.matrix(field)
        }

        if (is.null(dim(field))) {
            if (is.null(projector$lattice)) {
                data <- as.vector(projector$proj$A %*% as.vector(field))
                data[!projector$proj$ok] <- NA
                return(data)
            } else {
                data <- as.vector(projector$proj$A %*% as.vector(field))
                data[!projector$proj$ok] <- NA
                return(matrix(
                    data,
                    projector$lattice$dims[1],
                    projector$lattice$dims[2]
                ))
            }
        } else {
            data <- projector$proj$A %*% field
            data[!projector$proj$ok, ] <- NA
            return(data)
        }
    }


#' @export
#' @rdname inla.mesh.project
inla.mesh.projector <- function(...) {
    UseMethod("inla.mesh.projector")
}

#' @export
#' @rdname inla.mesh.project
inla.mesh.projector.inla.mesh <- function(mesh,
                                          loc = NULL,
                                          lattice = NULL,
                                          xlim = NULL,
                                          ylim = NULL,
                                          dims = c(100, 100),
                                          projection = NULL,
                                          crs = NULL,
                                          ...) {
    inla.require.inherits(mesh, "inla.mesh", "'mesh'")

    if (missing(loc) || is.null(loc)) {
        if (missing(lattice) || is.null(lattice)) {
            if (identical(mesh$manifold, "R2") &&
                (is.null(mesh$crs) || is.null(crs))) {
                units <- "default"
                lim <- list(
                    xlim = range(mesh$loc[, 1]),
                    ylim = range(mesh$loc[, 2])
                )
            } else if (identical(mesh$manifold, "S2") &&
                (is.null(mesh$crs) || is.null(crs))) {
                projection <-
                    match.arg(projection, c(
                        "longlat", "longsinlat",
                        "mollweide"
                    ))
                units <- projection
                lim <- inla.mesh.map.lim(loc = mesh$loc, projection = projection)
            } else {
                lim <- inla.crs.bounds(crs)
                if (identical(mesh$manifold, "R2")) {
                    lim0 <- list(
                        xlim = range(mesh$loc[, 1]),
                        ylim = range(mesh$loc[, 2])
                    )
                    lim$xlim[1] <- max(lim$xlim[1], lim0$xlim[1])
                    lim$xlim[2] <- min(lim$xlim[2], lim0$xlim[2])
                    lim$ylim[1] <- max(lim$ylim[1], lim0$ylim[1])
                    lim$ylim[2] <- min(lim$ylim[2], lim0$ylim[2])
                }
            }
            if (missing(xlim) && is.null(xlim)) {
                xlim <- lim$xlim
            }
            if (missing(ylim) && is.null(ylim)) {
                ylim <- lim$ylim
            }
            x <- seq(xlim[1], xlim[2], length.out = dims[1])
            y <- seq(ylim[1], ylim[2], length.out = dims[2])
            if (is.null(mesh$crs) || is.null(crs)) {
                lattice <- inla.mesh.lattice(x = x, y = y, units = units)
            } else {
                lattice <- inla.mesh.lattice(x = x, y = y, crs = crs)
            }
        } else {
            dims <- lattice$dims
            x <- lattice$x
            y <- lattice$y
        }

        if (is.null(mesh$crs) || is.null(lattice$crs)) {
            proj <- inla.mesh.project(mesh, lattice$loc)
        } else {
            proj <- inla.mesh.project(mesh,
                loc = lattice$loc,
                crs = lattice$crs
            )
        }
        projector <- list(x = x, y = y, lattice = lattice, loc = NULL, proj = proj, crs = crs)
        class(projector) <- "inla.mesh.projector"
    } else {
        proj <- inla.mesh.project(mesh, loc = loc, crs = crs)
        projector <- list(x = NULL, y = NULL, lattice = NULL, loc = loc, proj = proj, crs = crs)
        class(projector) <- "inla.mesh.projector"
    }

    return(projector)
}


#' @export
#' @rdname inla.mesh.project
inla.mesh.projector.inla.mesh.1d <- function(mesh,
                                             loc = NULL,
                                             xlim = mesh$interval,
                                             dims = 100,
                                             ...) {
    inla.require.inherits(mesh, "inla.mesh.1d", "'mesh'")

    if (missing(loc) || is.null(loc)) {
        loc <- seq(xlim[1], xlim[2], length.out = dims[1])
    }

    proj <- inla.mesh.project(mesh, loc)
    projector <- list(x = loc, lattice = NULL, loc = loc, proj = proj)
    class(projector) <- "inla.mesh.projector"

    return(projector)
}




inla.internal.make.spline.mesh <- function(interval, m, degree, boundary, free.clamped) {
    boundary <-
        match.arg(
            boundary,
            c("neumann", "dirichlet", "free", "cyclic")
        )
    if (degree <= 1) {
        n <- (switch(boundary,
            neumann = m,
            dirichlet = m + 2,
            free = m,
            cyclic = m + 1
        ))
        if (n < 2) {
            n <- 2
            degree <- 0
            boundary <- "c"
        }
    } else {
        stopifnot(degree == 2)
        n <- (switch(boundary,
            neumann = m + 1,
            dirichlet = m + 1,
            free = m - 1,
            cyclic = m
        ))
        if (boundary == "free") {
            if (m <= 1) {
                n <- 2
                degree <- 0
                boundary <- "c"
            } else if (m == 2) {
                n <- 2
                degree <- 1
            }
        } else if (boundary == "cyclic") {
            if (m <= 1) {
                n <- 2
                degree <- 0
            }
        }
    }
    return(inla.mesh.1d(seq(interval[1], interval[2], length = n),
        degree = degree,
        boundary = boundary,
        free.clamped = free.clamped
    ))
}




#' Basis functions for inla.mesh
#'
#' Calculate basis functions on a 1d or 2d [inla.mesh()]
#'
#'
#' @param mesh An `inla.mesh.1d` or `inla.mesh` object.
#' @param type `b.spline` (default) for B-spline basis functions,
#' `sph.harm` for spherical harmonics (available opnly for meshes on the
#' sphere)
#' @param n For B-splines, the number of basis functions in each direction (for
#' 1d meshes `n` must be a scalar, and for planar 2d meshes a 2-vector).
#' For spherical harmonics, `n` is the maximal harmonic order.
#' @param degree Degree of B-spline polynomials.  See
#' [inla.mesh.1d()].
#' @param knot.placement For B-splines on the sphere, controls the latitudinal
#' placements of knots. `"uniform.area"` (default) gives uniform spacing
#' in sin(latitude), `"uniform.latitude"` gives uniform spacing in
#' latitudes.
#' @param rot.inv For spherical harmonics on a sphere, `rot.inv=TRUE`
#' gives the rotationally invariant subset of basis functions.
#' @param boundary Boundary specification, default is free boundaries.  See
#' [inla.mesh.1d()] for more information.
#' @param free.clamped If `TRUE` and `boundary` is `"free"`, the
#' boundary basis functions are clamped to 0/1 at the interval boundary by
#' repeating the boundary knots.
#' @param ...
#'
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.1d()] [inla.mesh.2d()]
#' @examples
#'
#' n <- 100
#' loc <- matrix(runif(n * 2), n, 2)
#' mesh <- inla.mesh.2d(loc, max.edge = 0.05)
#' basis <- inla.mesh.basis(mesh, n = c(4, 5))
#'
#' proj <- inla.mesh.projector(mesh)
#' image(proj$x, proj$y, inla.mesh.project(proj, basis[, 7]))
#' \donttest{
#' if (require(rgl)) {
#'     plot(mesh, rgl = TRUE, col = basis[, 7], draw.edges = FALSE, draw.vertices = FALSE)
#' }
#' }
#'
#' @export inla.mesh.basis
inla.mesh.basis <- function(mesh,
                            type = "b.spline",
                            n = 3,
                            degree = 2,
                            knot.placement = "uniform.area",
                            rot.inv = TRUE,
                            boundary = "free",
                            free.clamped = TRUE,
                            ...) {
    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")

    type <- match.arg(type, c("b.spline", "sph.harm"))
    knot.placement <- (match.arg(
        knot.placement,
        c(
            "uniform.area",
            "uniform.latitude"
        )
    ))

    if (identical(type, "b.spline")) {
        if (identical(mesh$manifold, "R1") || identical(mesh$manifold, "S1")) {
            mesh1 <-
                inla.internal.make.spline.mesh(
                    mesh$interval, n, degree,
                    boundary, free.clamped
                )
            basis <- inla.mesh.1d.A(mesh1, mesh$loc)
        } else if (identical(mesh$manifold, "R2")) {
            if (length(n) == 2) {
                if (length(degree) == 1) {
                    degree <- rep(degree, 2)
                }
                if (length(boundary) == 1) {
                    boundary <- rep(boundary, 2)
                }
                mesh1x <-
                    inla.internal.make.spline.mesh(
                        range(mesh$loc[, 1]),
                        n[1], degree[1],
                        boundary[1], free.clamped
                    )
                mesh1y <-
                    inla.internal.make.spline.mesh(
                        range(mesh$loc[, 2]),
                        n[2], degree[2],
                        boundary[2], free.clamped
                    )
                basis <-
                    inla.row.kron(
                        inla.mesh.1d.A(mesh1y, mesh$loc[, 2]),
                        inla.mesh.1d.A(mesh1x, mesh$loc[, 1])
                    )
            } else {
                warning("Old style call for an R2 mesh detected.\n  Please supply a 2-element n-vector instead.")
                long <- ((mesh$loc[, 1] - min(mesh$loc[, 1])) /
                    diff(range(mesh$loc[, 1]))) * 90 * pi / 180
                sinlat <- (((mesh$loc[, 2] - min(mesh$loc[, 2])) /
                    diff(range(mesh$loc[, 2]))) * 2 - 1)
                coslat <- sapply(sinlat, function(x) sqrt(max(0.0, 1.0 - x^2)))
                loc <- (matrix(c(
                    cos(long) * coslat,
                    sin(long) * coslat,
                    sinlat
                ), mesh$n, 3))
                knots <- 0
                degree <- max(0L, min(n - 1L, degree))
                basis <- (inla.fmesher.smorg(loc,
                    mesh$graph$tv,
                    bspline = c(n, degree, knots),
                    fem = -1
                )$bspline)
            }
        } else if (identical(mesh$manifold, "S2")) {
            loc <- mesh$loc
            knots <- identical(knot.placement, "uniform.latitude")
            degree <- max(0L, min(n - 1L, degree))
            basis <- (inla.fmesher.smorg(loc,
                mesh$graph$tv,
                bspline = c(n, degree, knots),
                fem = -1
            )$bspline)
            if (!rot.inv) {
                warning("Currently only 'rot.inv=TRUE' is supported for B-splines.")
            }
        } else {
            stop("Only know how to make B-splines on R2 and S2.")
        }
    } else if (identical(type, "sph.harm")) {
        if (!identical(mesh$manifold, "S2")) {
            stop("Only know how to make spherical harmonics on S2.")
        }
        if (rot.inv) {
            basis <- (inla.fmesher.smorg(mesh$loc,
                mesh$graph$tv,
                sph0 = n
            )$sph0)
        } else {
            basis <- (inla.fmesher.smorg(mesh$loc,
                mesh$graph$tv,
                sph = n
            )$sph)
        }
    }

    return(basis)
}


inla.parse.queries <- function(...) {
    queries <- list(...)
    if (length(queries) == 0) {
          return(queries)
      }

    ## Make sure that we have a list of names, empty or not:
    if (is.null(names(queries))) {
        q.names <- rep("", length(queries))
    } else {
        q.names <- names(queries)
    }

    ## All nameless entries must be strings with query names.  Replace
    ## empty names with those names, and set those entries to NULL.
    for (query.idx in 1:length(queries)) {
        if (q.names[[query.idx]] == "") {
            if (is.character(queries[[query.idx]])) {
                names(queries)[query.idx] <- queries[[query.idx]]
                queries[query.idx] <- list(NULL)
            } else {
                queries[query.idx] <- list(NULL)
                warning(paste("Unnamed query ignored.  Check query #",
                    query.idx,
                    sep = ""
                ))
            }
        }
    }

    return(queries)
}


#' Compute various mesh related quantities.
#'
#' Low level function for computing finite element matrices, spherical
#' harmonics, B-splines, and point mappings with barycentric triangle
#' coordinates.
#'
#'
#' @param loc 3-column triangle vertex coordinate matrix.
#' @param tv 3-column triangle vertex index matrix.
#' @param fem Maximum finite element matrix order to be computed.
#' @param aniso A two-element list with \eqn{\gamma}{gamma} and \eqn{v}{v} for
#' an anisotropic operator \eqn{\nabla\cdot H \nabla}{div H grad}, where
#' \eqn{H=\gamma I + v v^\top}{H = gamma I + v v'}
#' @param gradients When `TRUE`, calculate derivative operator matrices
#' `dx`, dy, and dz.
#' @param sph0 Maximal order of rotationally invariant spherical harmonics.
#' @param sph Maximal order of general spherical harmonics.
#' @param bspline Rotationally invariant B-splines on a sphere.  3-vector with
#' number of basis functions `n`, basis degree `degree`, and a
#' logical; `TRUE` uniform knot angles, `FALSE` for uniform spacing
#' in \eqn{\sin(latitude)}{sin(latitude)}.
#' @param points2mesh 3-column matrix with points to be located in the mesh.
#' @param splitlines A list with elements `loc` (3-column coordinate
#' matrix) and `idx` (2-column index matrix) describing line segments that
#' are to be split into sub-segments at triangle boundaries.
#' @param output Names of objects to be included in the output, if different
#' from defaults.
#' @param keep When `TRUE`, for debugging purposes keep the `fmesher`
#' I/O files on disk.
#' @return A list of generated named quantities.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.fmesher.smorg
`inla.fmesher.smorg` <- function(loc, tv,
                                 fem = NULL,
                                 aniso = NULL,
                                 gradients = FALSE,
                                 sph0 = NULL,
                                 sph = NULL,
                                 bspline = NULL,
                                 points2mesh = NULL,
                                 splitlines = NULL,
                                 output = NULL,
                                 keep = FALSE) {
    prefix <- inla.fmesher.make.prefix(NULL, NULL)

    n <- nrow(loc)
    s.dim <- ncol(loc)

    if (s.dim == 1) {
          stop("1-D models not implemented yet.")
      }
    stopifnot(s.dim >= 2, s.dim <= 3)

    if (missing(output) || is.null(output)) {
        output.given <- FALSE
        output <- NULL
    } else {
        output.given <- TRUE
    }
    output.fem <- list("c0", "g1", "g2")
    output.aniso <- list("g1aniso", "g2aniso")
    output.gradients <- list("dx", "dy", "dz")
    output.sph0 <- list("sph0")
    output.sph <- list("sph")
    output.bspline <- list("bspline")
    output.p2m <- list("p2m.t", "p2m.b")
    output.splitlines <- list(
        "split.loc", "split.idx", "split.origin",
        "split.t", "split.b1", "split.b2"
    )

    ## Outputs that need +1L index adjustment
    indexoutput <- list("split.idx", "split.t", "split.origin")

    fmesher.write(inla.affirm.double(loc), prefix, "s")
    fmesher.write(inla.affirm.integer(tv) - 1L, prefix, "tv")
    all.args <- "--smorg --input=s,tv"

    ## additional arguments
    if (!is.null(fem)) {
        all.args <- paste(all.args, " --fem=", max(2, fem), sep = "")
        if (!output.given) output <- c(output, output.fem)
    }
    if (!is.null(aniso)) {
        fmesher.write(inla.affirm.double(aniso[[1]]), prefix, "aniso.gamma")
        fmesher.write(inla.affirm.double(aniso[[2]]), prefix, "aniso.vec")

        all.args <- paste(all.args, " --aniso=aniso.gamma,aniso.vec", sep = "")
        if (!output.given) output <- c(output, output.aniso)
    }
    if (!is.null(gradients) && gradients) {
        all.args <- paste(all.args, " --grad", sep = "")
        if (!output.given) output <- c(output, output.gradients)
    }
    if (!is.null(sph0)) {
        all.args <- paste(all.args, " --sph0=", sph0, sep = "")
        if (!output.given) output <- c(output, output.sph0)
    }
    if (!is.null(sph)) {
        all.args <- paste(all.args, " --sph=", sph, sep = "")
        if (!output.given) output <- c(output, output.sph)
    }
    if (!is.null(bspline)) {
        all.args <- (paste(all.args, " --bspline=",
            bspline[1], ",", bspline[2], ",", bspline[3],
            sep = ""
        ))
        if (!output.given) output <- c(output, output.bspline)
    }
    if (!is.null(points2mesh)) {
        fmesher.write(inla.affirm.double(points2mesh), prefix, "p2m")

        all.args <- paste(all.args, " --points2mesh=p2m", sep = "")
        if (!output.given) output <- c(output, output.p2m)
    }
    if (!is.null(splitlines)) {
        fmesher.write(inla.affirm.double(splitlines$loc), prefix, "splitloc")
        fmesher.write(inla.affirm.integer(splitlines$idx) - 1L, prefix, "splitidx")

        all.args <- paste(all.args, " --splitlines=splitloc,splitidx", sep = "")
        if (!output.given) output <- c(output, output.splitlines)
    }
    all.args <- paste(all.args, inla.getOption("fmesher.arg"))

    echoc <- inla.fmesher.call(all.args = all.args, prefix = prefix)

    result <- list()
    for (name in output) {
        if (identical(name, "p2m.t")) {
              if (!file.exists(paste(prefix, name, sep = ""))) {
                    result[[name]] <- fmesher.read(prefix, "points2mesh.t") + 1L
                } else {
                    result[[name]] <- fmesher.read(prefix, name) + 1L
                }
          } else if (identical(name, "p2m.b")) {
              if (!file.exists(paste(prefix, name, sep = ""))) {
                    result[[name]] <- fmesher.read(prefix, "points2mesh.b")
                } else {
                    result[[name]] <- fmesher.read(prefix, name)
                }
          } else if (name %in% indexoutput) {
              result[[name]] <- fmesher.read(prefix, name) + 1L
          } else {
              result[[name]] <- fmesher.read(prefix, name)
          }
    }

    if (!keep) {
          unlink(paste(prefix, "*", sep = ""), recursive = FALSE)
      }

    return(result)
}


# 1D mesh creation ----


#' Function space definition objects for 1D SPDE models.
#'
#' Create a 1D mesh specification `inla.mesh.1d` object, that defines a
#' function space for 1D SPDE models.
#'
#'
#' @param loc B-spline knot locations.
#' @param interval Interval domain endpoints.
#' @param boundary Boundary condition specification.  Valid conditions are
#' `c('neumann', 'dirichlet', 'free', 'cyclic')`.  Two separate values can
#' be specified, one applied to each endpoint.
#' @param degree The B-spline basis degree.  Supported values are 0, 1, and 2.
#' @param free.clamped If `TRUE`, for `'free'` boundaries, clamp the
#' basis functions to the interval endpoints.
#' @param \dots Additional option, currently unused.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.mesh.1d
inla.mesh.1d <- function(loc,
                         interval = range(loc),
                         boundary = NULL,
                         degree = 1,
                         free.clamped = FALSE,
                         ...) {
    ## Note: do not change the order of these options without also
    ## changing 'basis.reduction' below.
    boundary.options <- c("neumann", "dirichlet", "free", "cyclic")

    if ("cyclic" %in% names(list(...))) {
        warning(paste("Option 'cyclic' is deprecated.",
            "  Use 'boundary=\"cyclic\"' instead.",
            sep = ""
        ))
        if (cyclic) {
            if (!missing(boundary)) {
                boundary <- match.arg.vector(boundary, boundary.options, length = 2)
                warning(paste("'cyclic=TRUE' overrides 'boundary=c(\"",
                    paste(boundary, collapse = "\",\""), "\")'.",
                    sep = ""
                ))
            }
            boundary <- "cyclic"
        }
    }
    boundary <- match.arg.vector(boundary, boundary.options, length = 2)
    cyclic <- !is.na(pmatch(boundary[1], "cyclic"))
    if (cyclic && is.na(pmatch(boundary[2], "cyclic"))) {
        stop("Inconsistent boundary specification 'boundary=c(",
            paste(boundary, collapse = ","), ")'.",
            sep = ""
        )
    }

    loc.orig <- loc
    if (cyclic) {
          loc <-
              (sort(unique(c(0, loc - interval[1]) %% diff(interval))) +
                  interval[1])
      } else {
          loc <-
              (sort(unique(c(
                  interval,
                  pmax(interval[1], pmin(interval[2], loc))
              ))))
      }

    n <- length(loc)

    if (loc[1] < interval[1]) {
          stop("All 'loc' must be >= interval[1].")
      }
    if (loc[n] > interval[2]) {
          stop("All 'loc' must be <= interval[2].")
      }

    if ((degree < 0) || (degree > 2)) {
        stop(paste("'degree' must be 0, 1, or 2.  'degree=",
            degree,
            "' is not supported.",
            sep = ""
        ))
    }

    if (length(free.clamped) == 1L) {
        free.clamped <- rep(free.clamped, 2)
    }


    ## Number of basis functions
    if (degree == 0) {
        basis.reduction <- c(0, 1, 0, 1 / 2) ## neu, dir, free, cyclic
    } else if (degree == 1) {
        basis.reduction <- c(0, 1, 0, 1 / 2) ## neu, dir, free, cyclic
    } else {
        basis.reduction <- c(1, 1, 0, 1)
    }
    m <- (n + cyclic + (degree == 2) * 1
        - basis.reduction[pmatch(boundary[1], boundary.options)]
        - basis.reduction[pmatch(boundary[2], boundary.options)])
    ## if (m < 1+max(1,degree)) {
    if (m < 1L) {
        stop("Degree ", degree,
            " meshes must have at least ", 1L,
            " basis functions, not 'm=", m, "'.",
            sep = ""
        )
    }

    ## Compute representative basis midpoints.
    if ((degree == 0) || (degree == 1)) {
        mid <- loc
        if (boundary[1] == "dirichlet") {
            mid <- mid[-1]
        }
        if (boundary[2] == "dirichlet") {
            mid <- mid[-(m + 1)]
        }
    } else { ## degree==2
        if (cyclic) {
            mid <- (loc + c(loc[-1], interval[2])) / 2
        } else {
            mid <- c(loc[1], (loc[-n] + loc[-1]) / 2, loc[n])
            mid <-
                switch(boundary[1],
                    neumann = mid[-1],
                    dirichlet = mid[-1],
                    free = mid
                )
            mid <-
                switch(boundary[2],
                    neumann = mid[-(m + 1)],
                    dirichlet = mid[-(m + 1)],
                    free = mid
                )
        }
    }

    mesh <-
        list(
            n = n,
            m = m,
            loc = loc,
            mid = mid,
            interval = interval,
            boundary = boundary,
            cyclic = cyclic,
            manifold = inla.ifelse(cyclic, "S1", "R1"),
            degree = degree,
            free.clamped = free.clamped,
            idx = list(loc = NULL)
        )
    class(mesh) <- "inla.mesh.1d"

    if (degree < 2) {
        mesh$idx$loc <-
            inla.mesh.1d.bary(mesh, loc.orig, method = "nearest")$index[, 1]
    } else {
        if (length(mid) >= 2) {
            mesh$idx$loc <-
                inla.mesh.1d.bary(inla.mesh.1d(mid, degree = 0),
                    loc.orig,
                    method = "nearest"
                )$index[, 1]
        } else {
            mesh$idx$loc <- rep(1, length(loc.orig))
        }
    }

    return(invisible(mesh))
}


# 1D mesh queries ----

#' @export inla.mesh.1d.bary
#' @rdname inla.mesh.1d.A
inla.mesh.1d.bary <- function(mesh, loc, method = c("linear", "nearest")) {
    inla.require.inherits(mesh, "inla.mesh.1d", "'mesh'")
    method <- match.arg(method)

    if (method == "linear") {
        if (mesh$cyclic) {
            mloc <- c(mesh$loc - mesh$loc[1], diff(mesh$interval))
            loc <- (loc - mesh$loc[1]) %% diff(mesh$interval)
        } else {
            mloc <- c(mesh$loc - mesh$loc[1], diff(mesh$interval))
            loc <- pmax(0, pmin(diff(mesh$interval), loc - mesh$loc[1]))
        }
    } else {
        if (mesh$cyclic) {
            mloc <-
                c(
                    mesh$loc[mesh$n] - diff(mesh$interval),
                    mesh$loc,
                    diff(mesh$interval)
                )
            mloc <- (mloc[-(mesh$n + 2)] + mloc[-1]) / 2
            loc <- (loc - mloc[1]) %% diff(mesh$interval)
            mloc <- mloc - mloc[1]
        } else {
            mloc <-
                c(
                    0,
                    (mesh$loc[1:(mesh$n - 1L)] +
                        mesh$loc[2:mesh$n]) / 2 - mesh$loc[1],
                    diff(mesh$interval)
                )
            loc <- pmax(0, pmin(diff(mesh$interval), loc - mesh$loc[1]))
        }
    }

    ## Binary split method:
    do.the.split <- function(knots, loc) {
        n <- length(knots)
        if (n <= 2L) {
            return(rep(1L, length(loc)))
        }
        split <- 1L + (n - 1L) %/% 2L ## Split point
        upper <- (loc >= knots[split])
        idx <- rep(0, length(loc))
        idx[!upper] <- do.the.split(knots[1:split], loc[!upper])
        idx[upper] <- split - 1L + do.the.split(knots[split:n], loc[upper])
        return(idx)
    }

    idx <- do.the.split(mloc, loc)

    if (method == "nearest") {
        u <- rep(0, length(loc))
        if (mesh$cyclic) {
            found <- which(idx == (mesh$n + 1L))
            idx[found] <- 1L
        }
    } else { ## (method=="linear") {
        u <- pmax(0, pmin(1, (loc - mloc[idx]) / (mloc[idx + 1L] - mloc[idx])))
        if (!mesh$cyclic) {
            found <- which(idx == mesh$n)
            idx[found] <- mesh$n - 1L
            u[found] <- 1
        }
    }

    if (mesh$cyclic) {
        index <- matrix(c(idx, (idx %% mesh$n) + 1L), length(idx), 2)
        bary <- matrix(c(1 - u, u), length(idx), 2)
    } else {
        index <- matrix(c(idx, idx + 1L), length(idx), 2)
        bary <- matrix(c(1 - u, u), length(idx), 2)
    }

    return(list(index = index, bary = bary))
}



#' Mapping matrix for 1D meshes
#'
#' Calculates barycentric coordinates and weight matrices for
#' [inla.mesh.1d()] objects.
#'
#'
#' @aliases inla.mesh.1d.A inla.mesh.1d.bary
#' @param mesh An [inla.mesh.1d()] object.
#' @param loc Coordinate values.
#' @param weights Weights to be applied to the `A` matrix rows.
#' @param derivatives If `TRUE`, also compute derivative weight matrices
#' `dA` and `d2A`.
#' @param method Interpolation method.  If not specified for
#' `inla.mesh.1d.A` (recommended), it is determined by the `mesh`
#' basis function properties.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.mesh.1d.A
inla.mesh.1d.A <- function(mesh, loc,
                           weights = NULL,
                           derivatives = NULL,
                           method = c("linear", "nearest", "quadratic")) {
    inla.require.inherits(mesh, "inla.mesh.1d", "'mesh'")
    if (missing(method)) {
        ## Compute basis based on mesh$degree and mesh$boundary
        if (mesh$degree == 0) {
            info <- (inla.mesh.1d.A(mesh, loc,
                method = "nearest",
                weights = weights,
                derivatives =
                    inla.ifelse(
                        is.null(derivatives),
                        FALSE, TRUE
                    )
            ))
            if (mesh$boundary[1] == "dirichlet") {
                info$A <- info$A[, -1, drop = FALSE]
                if (!is.null(derivatives) && derivatives) {
                      info$dA <- info$dA[, -1, drop = FALSE]
                  }
            }
            if (mesh$boundary[2] == "dirichlet") {
                info$A <- info$A[, -(mesh$m + 1), drop = FALSE]
                if (!is.null(derivatives) && derivatives) {
                      info$dA <- info$dA[, -(mesh$m + 1), drop = FALSE]
                  }
            }
            if (!is.null(derivatives) && derivatives) {
                  return(info)
              } else {
                  return(info$A)
              }
        } else if (mesh$degree == 1) {
            info <- (inla.mesh.1d.A(mesh, loc,
                method = "linear",
                weights = weights,
                derivatives =
                    inla.ifelse(
                        is.null(derivatives),
                        FALSE, TRUE
                    )
            ))
            if (mesh$boundary[1] == "dirichlet") {
                info$A <- info$A[, -1, drop = FALSE]
                if (!is.null(derivatives) && derivatives) {
                      info$dA <- info$dA[, -1, drop = FALSE]
                  }
            }
            if (mesh$boundary[2] == "dirichlet") {
                info$A <- info$A[, -(mesh$m + 1), drop = FALSE]
                if (!is.null(derivatives) && derivatives) {
                      info$dA <- info$dA[, -(mesh$m + 1), drop = FALSE]
                  }
            }
            if (!is.null(derivatives) && derivatives) {
                  return(info)
              } else {
                  return(info$A)
              }
        } else if (mesh$degree == 2) {
            info <-
                inla.mesh.1d.A(mesh, loc,
                    method = "quadratic",
                    weights = weights,
                    derivatives =
                        inla.ifelse(
                            is.null(derivatives),
                            FALSE, TRUE
                        )
                )

            adjust.matrix <- function(mesh, A) {
                A <- inla.as.dgTMatrix(A)
                i <- A@i + 1L
                j <- A@j + 1L
                x <- A@x
                if (mesh$cyclic) {
                    j[j == 1L] <- mesh$m + 1L
                    j[j == mesh$m + 2L] <- 2L
                    j <- j - 1L
                } else {
                    if (mesh$boundary[1] == "neumann") {
                        j[j == 1L] <- 2L
                        j <- j - 1L
                    } else if (mesh$boundary[1] == "dirichlet") {
                        x[j == 1L] <- -x[j == 1L]
                        j[j == 1L] <- 2L
                        j <- j - 1L
                    } else if ((mesh$boundary[1] == "free") &&
                        mesh$free.clamped[1]) {
                        j1 <- which(j == 1L)
                        i <- c(i, i[j1])
                        j <- c(j, j[j1] + 1L)
                        x <- c(x, -x[j1])
                        x[j1] <- 2 * x[j1]
                    }
                    if (mesh$boundary[2] == "neumann") {
                        j[j == (mesh$m + 1L)] <- mesh$m
                    } else if (mesh$boundary[2] == "dirichlet") {
                        x[j == (mesh$m + 1L)] <- -x[j == (mesh$m + 1L)]
                        j[j == (mesh$m + 1L)] <- mesh$m
                    } else if ((mesh$boundary[2] == "free") &&
                        mesh$free.clamped[2]) {
                        j1 <- which(j == (mesh$m))
                        i <- c(i, i[j1])
                        j <- c(j, j[j1] - 1L)
                        x <- c(x, -x[j1])
                        x[j1] <- 2 * x[j1]
                    }
                }
                return(sparseMatrix(
                    i = i, j = j, x = x,
                    dims = c(length(loc), mesh$m)
                ))
            }

            if (!is.null(derivatives) && derivatives) {
                return(list(
                    A = adjust.matrix(mesh, info$A),
                    dA = adjust.matrix(mesh, info$dA),
                    d2A = adjust.matrix(mesh, info$d2A)
                ))
            } else {
                return(adjust.matrix(mesh, info$A))
            }
        } else {
            stop(paste("'degree' must be 0, 1, or 2.  'degree=",
                mesh$degree,
                "' is not supported.",
                sep = ""
            ))
        }
    } else {
        method <- match.arg(method)
        if (missing(weights) || is.null(weights)) {
            weights <- rep(1, length(loc))
        }

        if (!is.na(pmatch(method, c("linear", "nearest")))) {
            idx <- inla.mesh.1d.bary(mesh, loc, method)
            dA <- NULL

            if (method == "linear") {
                ## Compute the n 1st order B-splines.
                i <- rep(1:length(loc), times = 2)
                weights.i <- weights[i]
                A <- (sparseMatrix(
                    i = i,
                    j = as.vector(idx$index),
                    x = weights.i * as.vector(idx$bary),
                    dims = c(length(loc), mesh$n)
                ))
                if (!is.null(derivatives) && derivatives) {
                    if (mesh$cyclic) {
                        d <- (c(mesh$loc[-1], mesh$interval[2]) -
                            mesh$loc)
                    } else {
                        d <- (mesh$loc[-1] - mesh$loc[-mesh$n])
                    }
                    dA <- (sparseMatrix(
                        i = i,
                        j = as.vector(idx$index),
                        x = (weights.i * c(
                            -1 / d[idx$index[, 1]],
                            1 / d[idx$index[, 1]]
                        )),
                        dims = c(length(loc), mesh$n)
                    ))
                }
            } else {
                ## Nearest neighbours.
                A <- (sparseMatrix(
                    i = 1:length(loc),
                    j = idx$index[, 1],
                    x = weights * idx$bary[, 1],
                    dims = c(length(loc), mesh$n)
                ))
            }

            if (missing(derivatives) || is.null(derivatives)) {
                return(A)
            } else {
                return(list(A = A, dA = dA))
            }
        } else { ## Quadratic
            ## Compute the n+1 2nd order B-splines.

            if (mesh$cyclic) {
                Boundary.knots <-
                    (c(mesh$loc, mesh$interval[2])[c(mesh$n, 2)] +
                        diff(mesh$interval) * c(-1, 1))
                knots <- c(mesh$loc, mesh$interval[2])
            } else {
                Boundary.knots <-
                    c(
                        mesh$loc[1] - (mesh$loc[2] - mesh$loc[1]),
                        mesh$loc[mesh$n] + (mesh$loc[mesh$n] - mesh$loc[mesh$n - 1])
                    )
                knots <- mesh$loc
            }

            if (FALSE) { ## Only for debugging.
                ## Using bs():
                ## Note: Intermediate step constructs dense matrix.
                bsobj <-
                    bs(
                        x = loc,
                        knots = knots,
                        degree = 2,
                        Boundary.knots = Boundary.knots
                    )
                bsobj <-
                    Matrix(
                        as.vector(bsobj[, 1:(mesh$n + mesh$cyclic + 1)]),
                        nrow(bsobj), mesh$n + mesh$cyclic + 1
                    )
            }

            ## Direct calculation:
            knots <- c(Boundary.knots[1], knots)
            if (mesh$cyclic) {
                idx <-
                    inla.mesh.1d.bary(inla.mesh.1d(knots, boundary = "free"),
                        (loc - mesh$interval[1]) %%
                            diff(mesh$interval) + mesh$interval[1],
                        method = "linear"
                    )
            } else {
                idx <-
                    inla.mesh.1d.bary(inla.mesh.1d(knots, boundary = "free"),
                        loc,
                        method = "linear"
                    )
            }
            knots <- c(knots, Boundary.knots[2])
            idx$index <- idx$index[, 1] - 1L ## Indices into mesh intervals.

            d <- knots[2:length(knots)] - knots[1:(length(knots) - 1)]
            d2 <- knots[3:length(knots)] - knots[1:(length(knots) - 2)]

            ## Left intervals for each basis function:
            i.l <- 1:length(idx$index)
            j.l <- idx$index + 2L
            x.l <- (idx$bary[, 2] * d[idx$index + 1] / d2[idx$index + 1] * idx$bary[, 2])
            ## Right intervals for each basis function:
            i.r <- 1:length(idx$index)
            j.r <- idx$index
            x.r <- (idx$bary[, 1] * d[idx$index + 1] / d2[idx$index] * idx$bary[, 1])
            ## Middle intervals for each basis function:
            i.m <- 1:length(idx$index)
            j.m <- idx$index + 1L
            x.m <- (1 - (idx$bary[, 1] * d[idx$index + 1] / d2[idx$index] * idx$bary[, 1] +
                idx$bary[, 2] * d[idx$index + 1] / d2[idx$index + 1] * idx$bary[, 2]
            ))

            i <- c(i.l, i.r, i.m)
            j <- c(j.l, j.r, j.m)
            weights.i <- weights[i]

            A <- (sparseMatrix(
                i = i, j = j,
                x = weights.i * c(x.l, x.r, x.m),
                dims = c(length(idx$index), mesh$n + mesh$cyclic + 1L)
            ))

            dA <- NULL
            d2A <- NULL
            if (!is.null(derivatives) && derivatives) {
                ## dA:
                ## Left, right, middle intervals for each basis function:
                x.l <- (2 / d2[idx$index + 1] * idx$bary[, 2])
                x.r <- (-2 / d2[idx$index] * idx$bary[, 1])
                x.m <- (-(-2 / d2[idx$index] * idx$bary[, 1] +
                    2 / d2[idx$index + 1] * idx$bary[, 2]
                ))
                dA <- (sparseMatrix(
                    i = i, j = j,
                    x = weights.i * c(x.l, x.r, x.m),
                    dims = (c(
                        length(idx$index),
                        mesh$n + mesh$cyclic + 1L
                    ))
                ))

                ## d2A:
                ## Left, right, middle intervals for each basis function:
                x.l <- (2 / d[idx$index + 1] / d2[idx$index + 1])
                x.r <- (2 / d[idx$index + 1] / d2[idx$index])
                x.m <- (-(2 / d[idx$index + 1] / d2[idx$index] +
                    2 / d[idx$index + 1] / d2[idx$index + 1]))
                d2A <- (sparseMatrix(
                    i = i, j = j,
                    x = weights.i * c(x.l, x.r, x.m),
                    dims = (c(
                        length(idx$index),
                        mesh$n + mesh$cyclic + 1L
                    ))
                ))
            }

            if (missing(derivatives) || is.null(derivatives)) {
                return(A)
            } else {
                return(list(A = A, dA = dA, d2A = d2A))
            }
        }
    }
}

#' @param mesh An inla.mesh.1d object
#' @export
#' @rdname inla.mesh.1d
inla.mesh.1d.fem <- function(mesh) {
    inla.require.inherits(mesh, "inla.mesh.1d", "'mesh'")

    ## Use the same matrices for degree 0 as for degree 1
    if ((mesh$degree == 0) || (mesh$degree == 1)) {
        if (mesh$cyclic) {
            loc <-
                c(
                    mesh$loc[mesh$n] - diff(mesh$interval),
                    mesh$loc,
                    mesh$loc[1] + diff(mesh$interval)
                )
            c0 <- (loc[3:length(loc)] - loc[1:(length(loc) - 2)]) / 2
            c1.l <- (loc[2:(length(loc) - 1)] - loc[1:(length(loc) - 2)]) / 6
            c1.r <- (loc[3:length(loc)] - loc[2:(length(loc) - 1)]) / 6
            c1.0 <- (c1.l + c1.r) * 2
            g1.l <- -1 / (loc[2:(length(loc) - 1)] - loc[1:(length(loc) - 2)])
            g1.r <- -1 / (loc[3:length(loc)] - loc[2:(length(loc) - 1)])
            g1.0 <- -g1.l - g1.r
            i.l <- 1:mesh$n
            i.r <- 1:mesh$n
            i.0 <- 1:mesh$n
            if (mesh$n > 1) {
                j.l <- c(mesh$n, 1:(mesh$n - 1))
                j.r <- c(2:mesh$n, 1)
                j.0 <- 1:mesh$n
            } else {
                j.l <- 1L
                j.r <- 1L
                j.0 <- 1L
            }
        } else {
            c0 <-
                c(
                    (mesh$loc[2] - mesh$loc[1]) / 2,
                    (mesh$loc[mesh$n] - mesh$loc[mesh$n - 1]) / 2
                )
            if (mesh$n > 2) {
                c0 <-
                    c(
                        c0[1],
                        (mesh$loc[3:mesh$n] - mesh$loc[1:(mesh$n - 2)]) / 2,
                        c0[2]
                    )
            }
            c1.l <- (mesh$loc[2:mesh$n] - mesh$loc[1:(mesh$n - 1)]) / 6
            c1.r <- c1.l
            c1.0 <- (c(0, c1.l) + c(c1.r, 0)) * 2
            g1.l <- -1 / (mesh$loc[2:mesh$n] - mesh$loc[1:(mesh$n - 1)])
            g1.r <- g1.l
            g1.0 <- -c(0, g1.l) - c(g1.r, 0)
            i.l <- 2:mesh$n
            i.r <- 1:(mesh$n - 1)
            i.0 <- 1:mesh$n
            j.l <- 1:(mesh$n - 1)
            j.r <- 2:mesh$n
            j.0 <- 1:mesh$n

            if (mesh$boundary[1] == "dirichlet") {
                g1.0 <- g1.0[-1]
                g1.l <- g1.l[-1]
                g1.r <- g1.r[-1]
                c1.0 <- c1.0[-1]
                c1.l <- c1.l[-1]
                c1.r <- c1.r[-1]
                c0 <- c0[-1]
                i.l <- i.l[-1] - 1
                i.r <- i.r[-1] - 1
                i.0 <- i.0[-1] - 1
                j.l <- j.l[-1] - 1
                j.r <- j.r[-1] - 1
                j.0 <- j.0[-1] - 1
            } else if (mesh$boundary[1] == "free") {
                g1.0[1] <- 0
                g1.r[1] <- 0
            }
            if (mesh$boundary[2] == "dirichlet") {
                m <- mesh$m
                g1.0 <- g1.0[-(m + 1)]
                g1.l <- g1.l[-m]
                g1.r <- g1.r[-m]
                c1.0 <- c1.0[-(m + 1)]
                c1.l <- c1.l[-m]
                c1.r <- c1.r[-m]
                c0 <- c0[-(m + 1)]
                i.l <- i.l[-m]
                i.r <- i.r[-m]
                i.0 <- i.0[-(m + 1)]
                j.l <- j.l[-m]
                j.r <- j.r[-m]
                j.0 <- j.0[-(m + 1)]
            } else if (mesh$boundary[2] == "free") {
                g1.0[mesh$m] <- 0
                g1.l[mesh$m - 1] <- 0
            }
        }

        g1 <-
            sparseMatrix(
                i = c(i.l, i.r, i.0),
                j = c(j.l, j.r, j.0),
                x = c(g1.l, g1.r, g1.0),
                dims = c(mesh$m, mesh$m)
            )
        c1 <-
            sparseMatrix(
                i = c(i.l, i.r, i.0),
                j = c(j.l, j.r, j.0),
                x = c(c1.l, c1.r, c1.0),
                dims = c(mesh$m, mesh$m)
            )
        g2 <- t(g1) %*% Diagonal(mesh$m, 1 / c0) %*% g1
        c0 <- Diagonal(mesh$m, c0)
    } else if (mesh$degree == 2) {
        if (mesh$cyclic) {
            knots1 <- mesh$loc
            knots2 <- c(mesh$loc[-1], mesh$interval[2])
        } else {
            knots1 <- mesh$loc[-mesh$n]
            knots2 <- mesh$loc[-1]
        }
        knots.m <- (knots1 + knots2) / 2
        knots.d <- (knots2 - knots1) / 2
        ## 3-point Gaussian quadrature
        info <-
            inla.mesh.1d.A(mesh,
                loc = (c(
                    knots.m,
                    knots.m - knots.d * sqrt(3 / 5),
                    knots.m + knots.d * sqrt(3 / 5)
                )),
                weights =
                    c(knots.d * 8 / 9, knots.d * 5 / 9, knots.d * 5 / 9)^0.5,
                derivatives = TRUE
            )
        c1 <- t(info$A) %*% info$A
        g1 <- t(info$dA) %*% info$dA
        g2 <- t(info$d2A) %*% info$d2A

        g01 <- t(info$A) %*% info$dA
        g02 <- t(info$A) %*% info$d2A
        g12 <- t(info$dA) %*% info$d2A

        c0 <- Diagonal(nrow(c1), rowSums(c1))

        return(list(c0 = c0, c1 = c1, g1 = g1, g2 = g2, g01 = g01, g02 = g02, g12 = g12))
    } else {
        stop(paste("Mesh basis degree=", mesh$degree,
            " is not supported.",
            sep = ""
        ))
    }

    return(list(c0 = c0, c1 = c1, g1 = g1, g2 = g2))
}


# Queries ----

#' Diameter of a point set
#'
#' Find an upper bound to the convex hull of a point set
#'
#'
#' @aliases inla.diameter inla.diameter.default inla.diameter.inla.mesh
#' inla.diameter.inla.mesh.segment inla.diameter.inla.mesh.lattice
#' inla.diameter.inla.mesh.1d
#' @param x A point set as an \eqn{n\times d}{n x d} matrix, or an
#' `inla.mesh` related object.
#' @param manifold Character string specifying the manifold type. Default is to
#' treat the point set with Euclidean \eqn{R^d} metrics. Use
#' `manifold="S2"` for great circle distances on the unit sphere (this is
#' set automatically for `inla.mesh` objects).
#' @param \dots Additional parameters passed on to other methods.
#' @return A scalar, upper bound for the diameter of the convex hull of the
#' point set.
#' @author Finn Lindgren <finn.lindgren@@gmail.com>
#' @examples
#'
#' inla.diameter(matrix(c(0, 1, 1, 0, 0, 0, 1, 1), 4, 2))
#' @export inla.diameter
inla.diameter <- function(x, ...) {
    UseMethod("inla.diameter")
}

#' @details * `inla.diameter.default`
#' Calculate upper bound for the diameter of a point set,
#' by encapsulating in a circular domain.
#' @export
#' @rdname inla.diameter
inla.diameter.default <- function(x, manifold = "", ...) {
    if (nrow(x) <= 1) {
        0
    } else {
        if (identical(manifold, "S2")) {
            distance <- function(u, v) {
                2 * asin(pmin(
                    1,
                    ((u[1] - v[, 1])^2 + (u[2] - v[, 2])^2 + (u[3] - v[, 3])^2)^0.5 / 2
                ))
            }
            center <- colMeans(x)
            tmp <- sqrt(sum(center^2))
            if (tmp < 1e-6) {
                pi
            } else {
                center <- center / tmp
                min(pi, 2 * max(distance(center, x)))
            }
        } else {
            distance <- function(u, v) {
                d <- 0
                for (k in seq_len(ncol(v))) {
                    d <- d + (u[k] - v[, k])^2
                }
                d^0.5
            }
            center <- rep(0, ncol(x))
            for (k in seq_len(ncol(x))) {
                center[k] <- mean(range(x[, k]))
            }
            2 * max(distance(center, x))
        }
    }
}

#' @export
#' @rdname inla.diameter
inla.diameter.inla.mesh.1d <- function(x, ...) {
    diff(x$interval)
}

#' @export
#' @rdname inla.diameter
inla.diameter.inla.mesh <- function(x, ...) {
    inla.diameter(x$loc, manifold = x$manifold, ...)
}

#' @export
#' @rdname inla.diameter
inla.diameter.inla.mesh.segment <- function(x, ...) {
    inla.diameter(x$loc, ...)
}

#' @export
#' @rdname inla.diameter
inla.diameter.inla.mesh.lattice <- function(x, ...) {
    inla.diameter(x$loc, ...)
}




#' Finite element matrices
#'
#' Constructs finite element matrices for [inla.mesh()] and
#' [inla.mesh.1d()] objects.
#'
#'
#' @aliases inla.mesh.fem inla.mesh.1d.fem
#' @param mesh An [inla.mesh()] or [inla.mesh.1d()] object.
#' @param order The model order.
#' @return A list of sparse matrices based on basis functions `psi_i`:
#' \item{c0 }{`c0[i,j] = < psi_i, 1 >`} \item{c1 }{`c1[i,j] = <
#' psi_i, psi_j >`} \item{g1 }{`g1[i,j] = < grad psi_i, grad psi_j >`}
#' \item{g2 }{`g2 = g1 * c0^-1 * g1`} \item{gk }{`gk = g1 * (c0^-1 *
#' g1)^(k-1)`, up to and including `k=order`}
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.mesh.fem
inla.mesh.fem <- function(mesh, order = 2) {
    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    if (inherits(mesh, "inla.mesh.1d")) {
        if (order > 2) {
            stop("order>2 not supported for 1d meshes.")
            ## TODO: Compute higher order matrices based on order 2 output.
        }
        return(inla.mesh.1d.fem(mesh))
    } else {
        ## output name list:
        if (order > 0) {
            output <- paste("g", seq_len(order), sep = "")
        } else {
            output <- NULL
        }
        output <- c(
            "c0", "c1", output,
            "va", "ta"
        )
        return(inla.fmesher.smorg(mesh$loc, mesh$graph$tv,
            fem = order, output = output
        ))
    }
}





#' Directional derivative matrices for functions on meshes.
#'
#' Calculates directional derivative matrices for functions on
#' [inla.mesh()] objects.
#'
#'
#' @param mesh An [inla.mesh()] object.
#' @param loc Coordinates where the derivatives should be evaluated.
#' @return \item{A }{The projection matrix, `u(loc_i)=sum_j A_ij w_i`}
#' \item{dx, dy, dz }{Derivative weight matrices, `du/dx(loc_i)=sum_j
#' dx_ij w_i`, etc.}
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @export inla.mesh.deriv
inla.mesh.deriv <- function(mesh, loc) {
    inla.require.inherits(mesh, c("inla.mesh", "inla.mesh.1d"), "'mesh'")
    if (inherits(mesh, "inla.mesh.1d")) {
        stop("1d derivatives are not implemented.")
    }
    n.mesh <- mesh$n

    info <- inla.mesh.project(mesh, loc = loc)

    ii <- which(info$ok)
    n.ok <- sum(info$ok)
    tv <- mesh$graph$tv[info$t[ii, 1L], ]
    e1 <- mesh$loc[tv[, 3], ] - mesh$loc[tv[, 2], ]
    e2 <- mesh$loc[tv[, 1], ] - mesh$loc[tv[, 3], ]
    e3 <- mesh$loc[tv[, 2], ] - mesh$loc[tv[, 1], ]
    n1 <- e2 - e1 * matrix(rowSums(e1 * e2) / rowSums(e1 * e1), n.ok, 3)
    n2 <- e3 - e2 * matrix(rowSums(e2 * e3) / rowSums(e2 * e2), n.ok, 3)
    n3 <- e1 - e3 * matrix(rowSums(e3 * e1) / rowSums(e3 * e3), n.ok, 3)
    g1 <- n1 / matrix(rowSums(n1 * n1), n.ok, 3)
    g2 <- n2 / matrix(rowSums(n2 * n2), n.ok, 3)
    g3 <- n3 / matrix(rowSums(n3 * n3), n.ok, 3)
    x <- cbind(g1[, 1], g2[, 1], g3[, 1])
    y <- cbind(g1[, 2], g2[, 2], g3[, 2])
    z <- cbind(g1[, 3], g2[, 3], g3[, 3])
    dx <- (sparseMatrix(
        dims = c(nrow(loc), n.mesh),
        i = rep(ii, 3),
        j = as.vector(tv),
        x = as.vector(x)
    ))
    dy <- (sparseMatrix(
        dims = c(nrow(loc), n.mesh),
        i = rep(ii, 3),
        j = as.vector(tv),
        x = as.vector(y)
    ))
    dz <- (sparseMatrix(
        dims = c(nrow(loc), n.mesh),
        i = rep(ii, 3),
        j = as.vector(tv),
        x = as.vector(z)
    ))

    return(list(A = info$A, dx = dx, dy = dy, dz = dz))
}



#' Recursive curve simplification.
#'
#' Attempts to simplify a polygonal curve by joining nearly colinear segments.
#'
#' Uses a variation of the binary splitting Ramer-Douglas-Peucker algorithm,
#' with a width `eps` ellipse instead of a rectangle, motivated by
#' prediction ellipse for Brownian bridge.
#'
#' @param loc Coordinate matrix.
#' @param idx Index vector into `loc` specifying a polygonal curve.
#' @param eps Straightness tolerance.
#' @return An index vector into `loc` specifying the simplified polygonal
#' curve.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @examples
#'
#' theta <- seq(0, 2 * pi, length = 1000)
#' loc <- cbind(cos(theta), sin(theta))
#' idx <- inla.simplify.curve(loc = loc, idx = 1:nrow(loc), eps = 0.01)
#' print(c(nrow(loc), length(idx)))
#' plot(loc, type = "l")
#' lines(loc[idx, ], col = "red")
#' @export inla.simplify.curve
inla.simplify.curve <- function(loc, idx, eps) {
    ## Variation of Ramer-Douglas-Peucker
    ## Uses width epsilon ellipse instead of rectangle,
    ## motivated by prediction ellipse for Brownian bridge
    n <- length(idx)
    if ((n == 2) || (eps == 0)) {
        return(idx)
    }
    segm <- loc[idx[n], ] - loc[idx[1], ]
    segm.len <- sum(segm^2)^0.5
    if (segm.len <= 1e-12) {
        ## End point same as start; closed curve.  Split.
        len2 <- ((loc[idx[2:(n - 1)], 1] - loc[idx[1], 1])^2 +
            (loc[idx[2:(n - 1)], 2] - loc[idx[1], 2])^2)
        split <- which.max(len2) + 1L
    } else {
        segm.mid <- (loc[idx[n], ] + loc[idx[1], ]) / 2
        segm <- segm / segm.len
        segm.perp <- c(-segm[2], segm[1])
        vec <- (cbind(
            loc[idx[2:(n - 1)], 1] - segm.mid[1],
            loc[idx[2:(n - 1)], 2] - segm.mid[2]
        ))
        ## Always split if any point is outside the circle
        epsi <- min(c(eps, segm.len / 2))
        dist1 <- abs(vec[, 1] * segm[1] + vec[, 2] * segm[2]) / (segm.len / 2) * epsi
        dist2 <- abs(vec[, 1] * segm.perp[1] + vec[, 2] * segm.perp[2])
        dist <- (dist1^2 + dist2^2)^0.5

        ## Find the furthest point, in the ellipse metric, and
        ## check if it inside the radius (radius=segm.len/2)
        split <- which.max(dist) + 1L
        if (dist[split - 1L] < epsi) {
            ## Flat segment, eliminate.
            return(idx[c(1, n)])
        }
        ## Split at the furthest point.
        split <- which.max(dist) + 1L
    }

    ## Do the split recursively:
    return(c(
        inla.simplify.curve(loc, idx[1L:split], eps),
        inla.simplify.curve(loc, idx[split:n], eps)[-1L]
    ))
}


#' @export
#' @rdname inla.mesh.segment
inla.contour.segment <- function(x = seq(0, 1, length.out = nrow(z)),
                                 y = seq(0, 1, length.out = ncol(z)),
                                 z, nlevels = 10,
                                 levels = pretty(range(z, na.rm = TRUE), nlevels),
                                 groups = seq_len(length(levels)),
                                 positive = TRUE,
                                 eps = NULL,
                                 crs = NULL) {
    ## Input checking from contourLines:
    if (missing(z)) {
        if (!missing(x)) {
            if (is.list(x)) {
                z <- x$z
                y <- x$y
                x <- x$x
            }
            else {
                z <- x
                x <- seq.int(0, 1, length.out = nrow(z))
            }
        }
        else {
            stop("no 'z' matrix specified")
        }
    }
    else if (is.list(x)) {
        y <- x$y
        x <- x$x
    }

    if (is.null(eps)) {
        eps <- min(c(min(diff(x)), min(diff(y)))) / 8
    }
    ## End of input checking.

    ## Get contour pieces
    curves <- contourLines(x, y, z, levels = levels)

    ## Make a mesh for easy gradient interpolation:
    latt <- inla.mesh.lattice(x, y)
    mesh <-
        inla.mesh.create(
            lattice = latt,
            boundary = latt$segm,
            extend = (list(
                n = 3,
                offset = (max(
                    diff(range(x)),
                    diff(range(y))
                ) * 0.1)
            ))
        )
    ## Map function values to mesh indexing:
    zz <- rep(0, mesh$n)
    ## 2020-07-15:
    ## Bug in fmesher may lead to some lattice points missing. Ignore those.
    lattice_points_ok <- !is.na(mesh$idx$lattice)
    zz[mesh$idx$lattice[lattice_points_ok]] <- as.vector(z)[lattice_points_ok]

    ## Mapping from level to group value:
    level2grp <- function(level) {
        if (length(groups) == 1) {
            return(groups)
        }
        for (k in seq_along(groups)) {
            if (levels[k] == level) {
                return(groups[k])
            }
        }
        return(0)
    }

    ## Join all contour pieces into a single mesh.segment
    ## Different levels can later be identified via the grp indices.
    loc <- matrix(0, 0, 2)
    idx <- matrix(0, 0, 2)
    grp <- c()
    for (k in seq_len(length(curves))) {
        curve.loc <- cbind(curves[[k]]$x, curves[[k]]$y)
        curve.n <- nrow(curve.loc)

        ## Extract the rotated gradients along the curve
        curve.mid <- (curve.loc[1:(curve.n - 1), ] + curve.loc[2:curve.n, ]) / 2
        A <- inla.mesh.deriv(mesh, loc = curve.mid)
        ## Gradients rotated 90 degrees CW, i.e. to the direction
        ## of CCW curves around positive excursions:
        grid.diff <- cbind(A$dy %*% zz, -A$dx %*% zz)

        ## Determine the CCW/CW orientation
        curve.diff <- diff(curve.loc)
        ccw <- (sum(curve.diff * grid.diff) >= 0) ## True if in CCW direction
        if ((ccw && positive) | (!ccw && !positive)) {
            curve.idx <- 1:curve.n
        } else {
            curve.idx <- curve.n:1
        }

        ## Filter short line segments:
        curve.idx <-
            inla.simplify.curve(curve.loc,
                curve.idx,
                eps = eps
            )

        ## Reorder, making sure any unused points are removed:
        curve.loc <- curve.loc[curve.idx, , drop = FALSE]
        curve.n <- nrow(curve.loc)
        curve.idx <- cbind(1L:(curve.n - 1L), 2L:curve.n)

        ## Check if the curve is closed, and adjust if it is:
        if (max(abs(curve.loc[1, ] - curve.loc[curve.n, ])) < 1e-12) {
            curve.loc <- curve.loc[-curve.n, , drop = FALSE]
            curve.n <- nrow(curve.loc)
            curve.idx <- cbind(1L:curve.n, c(2L:curve.n, 1L))
        }

        ## Add the curve:
        offset <- nrow(loc)
        loc <- rbind(loc, curve.loc)
        idx <- rbind(idx, curve.idx + offset)
        grp <- c(grp, rep(level2grp(curves[[k]]$level), curve.n - 1L))
    }

    inla.mesh.segment(loc = loc, idx = idx, grp = grp, is.bnd = FALSE, crs = crs)
}




## Morphological dilation by "convex",
## followed by closing by "concave", with
## minimum concave curvature radius "concave".
## If the dilated set has no gaps of width between
## 2*convex*(sqrt(1+2*concave/convex) - 1)
## and
## 2*concave,
## then the minimum convex curvature radius is "convex".
## Default is concave=convex
## Special case concave=0 delegates to inla.nonconvex.hull.basic()
##
## The implementation is based on the identity
## dilation(a) & closing(b) = dilation(a+b) & erosion(b)
## where all operations are with respect to disks with the specified radii.


#' Nonconvex set extensions.
#'
#' Constructs a nonconvex boundary for a point set using morphological
#' operations.
#'
#' Morphological dilation by `convex`, followed by closing by
#' `concave`, with minimum concave curvature radius `concave`.  If
#' the dilated set has no gaps of width between \deqn{2 convex (\sqrt{1+2
#' concave/convex} - 1)}{2*convex*(sqrt(1+2*concave/convex) - 1)} and \eqn{2
#' concave}{2*concave}, then the minimum convex curvature radius is
#' `convex`.  Special case `concave=0` delegates to
#' `inla.nonconvex.hull.basic`
#'
#' The implementation is based on the identity \deqn{dilation(a) \& closing(b)
#' = dilation(a+b) \& erosion(b)}{dilation(a) & closing(b) = dilation(a+b) &
#' erosion(b)} where all operations are with respect to disks with the
#' specified radii.
#'
#' @aliases inla.nonconvex.hull inla.nonconvex.hull.basic
#' @param points 2D point coordinates (2-column matrix).  Can alternatively be
#' a `SpatialPoints` or `SpatialPointsDataFrame` object.
#' @param convex The desired extension radius.  Also determines the smallest
#' allowed convex curvature radius.  Negative values are interpreted as
#' fractions of the approximate initial set diameter.
#' @param concave The desired minimal concave curvature radius. Default is
#' `concave=convex`.
#' @param resolution The internal computation resolution.  A warning will be
#' issued when this needs to be increased for higher accuracy, with the
#' required resolution stated.
#' @param eps The polygonal curve simplification tolerance used for simplifying
#' the resulting boundary curve.  See [inla.simplify.curve()] for
#' details.
#' @param crs An optional `CRS` or `inla.CRS` object
#' @return An [inla.mesh.segment()] object.
#' @note Requires `nndistF` from the `splancs` package.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @examples
#'
#' if (require(splancs)) {
#'     loc <- matrix(runif(20), 10, 2)
#'     boundary <- inla.nonconvex.hull(loc, convex = 0.2)
#'     lines(boundary, add = FALSE)
#'     points(loc)
#' }
#' @export inla.nonconvex.hull
inla.nonconvex.hull <- function(points, convex = -0.15, concave = convex, resolution = 40, eps = NULL,
                                crs = NULL) {
    if (!(missing(points) || is.null(points)) &&
        (inherits(points, "SpatialPoints") ||
            inherits(points, "SpatialPointsDataFrame"))) {
        points <- inla.spTransform(coordinates(points),
            inla.sp_get_crs(points),
            crs,
            passthrough = TRUE
        )
    }

    if (length(resolution) == 1) {
        resolution <- rep(resolution, 2)
    }
    lim <- rbind(range(points[, 1]), range(points[, 2]))

    approx.diam <- max(diff(lim[1, ]), diff(lim[2, ]))
    if (convex < 0) {
        convex <- -convex * approx.diam
    }
    if (concave < 0) {
        concave <- -concave * approx.diam
    }
    if (concave == 0) {
        return(inla.nonconvex.hull.basic(points, convex, resolution, eps,
            crs = crs
        ))
    }

    ex <- convex + concave
    domain <- c(diff(lim[1, ]), diff(lim[2, ])) + 2 * ex
    dif <- domain / (resolution - 1)
    if (max(dif) > min(convex, concave)) {
        req.res <- ceiling(domain / min(convex, concave) + 1)
        warning(paste("Resolution (",
            paste(resolution, collapse = ","),
            ") too small for convex/concave radius (",
            convex, ",", concave,
            ").\n",
            "Resolution >=(",
            paste(req.res, collapse = ","),
            ") required for more accurate results.",
            sep = ""
        ))
    }
    ax <-
        list(
            seq(lim[1, 1] - ex, lim[1, 2] + ex, length = resolution[1]),
            seq(lim[2, 1] - ex, lim[2, 2] + ex, length = resolution[2])
        )
    xy <- as.matrix(expand.grid(ax[[1]], ax[[2]]))

    inla.require("splancs", stop.on.error = TRUE)
    z <- (matrix(
        splancs::nndistF(points, xy),
        resolution[1], resolution[2]
    ))
    segm.dilation <-
        inla.contour.segment(ax[[1]], ax[[2]], z,
            levels = c(convex + concave),
            positive = TRUE,
            eps = 0
        ) ## Don't simplify curve at this stage
    mesh.dilation <-
        inla.mesh.create(
            loc = xy,
            boundary = segm.dilation,
            extend = (list(
                n = 3,
                offset = (max(
                    diff(ax[[1]]),
                    diff(ax[[2]])
                ) * 0.1)
            ))
        )

    ## This filtering is not necessary; the inla.mesh.create() should
    ## have removed all unused points. 2013-02-24 /FL
    ## points.dilation =
    ## mesh.dilation$loc[unique(as.vector(mesh.dilation$graph$tv)),]

    z <- (matrix(
        splancs::nndistF(mesh.dilation$loc, xy),
        resolution[1], resolution[2]
    ))
    segm.closing <-
        inla.contour.segment(ax[[1]], ax[[2]], z,
            levels = c(concave),
            positive = TRUE,
            eps = eps
        )

    segm.closing$crs <- crs

    segm.closing
}

#' @export
#' @rdname inla.nonconvex.hull
## Based on an idea from Elias Teixeira Krainski
## Requires  splancs::nndistF
inla.nonconvex.hull.basic <- function(points, convex = -0.15, resolution = 40, eps = NULL, crs = NULL) {
    if (!(missing(points) || is.null(points)) &&
        (inherits(points, "SpatialPoints") ||
            inherits(points, "SpatialPointsDataFrame"))) {
        points <- inla.spTransform(coordinates(points),
            inla.sp_get_crs(points),
            crs,
            passthrough = TRUE
        )
    }

    if (length(convex) == 1) {
        convex <- rep(convex, 2)
    }
    if (length(resolution) == 1) {
        resolution <- rep(resolution, 2)
    }

    lim <- rbind(range(points[, 1]), range(points[, 2]))
    ex <- convex
    if (convex[1] < 0) {
        ex[1] <- -convex[1] * diff(lim[1, ])
    }
    if (convex[2] < 0) {
        ex[2] <- -convex[2] * diff(lim[2, ])
    }

    domain <- c(diff(lim[1, ]), diff(lim[2, ])) + 2 * ex
    dif <- domain / (resolution - 1)
    if (any(dif > min(convex))) {
        req.res <- ceiling(domain / convex + 1)
        warning(paste("Resolution (",
            paste(resolution, collapse = ","),
            ") too small for convex (",
            paste(convex, collapse = ","),
            ").\n",
            "Resolution >=(",
            paste(req.res, collapse = ","),
            ") required for more accurate results.",
            sep = ""
        ))
    }

    ax <- list(
        seq(lim[1, 1] - ex[1], lim[1, 2] + ex[1], length = resolution[1]),
        seq(lim[2, 1] - ex[2], lim[2, 2] + ex[2], length = resolution[2])
    )
    xy <- as.matrix(expand.grid(ax[[1]], ax[[2]]))
    tr <- diag(c(1 / ex[1], 1 / ex[2]))

    inla.require("splancs", stop.on.error = TRUE)
    z <- (matrix(
        splancs::nndistF(points %*% tr, xy %*% tr),
        resolution[1], resolution[2]
    ))
    segm <- inla.contour.segment(ax[[1]], ax[[2]], z,
        levels = c(1), positive = FALSE, eps = eps, crs = crs
    )
    return(segm)
}

## Export: inla.pc.rgevtail inla.pc.dgevtail inla.pc.qgevtail inla.pc.pgevtail

## ! \name{pc.gevtail}
## ! \alias{inla.pc.gevtail}
## ! \alias{pc.gevtail}
## ! \alias{pc.rgevtail}
## ! \alias{inla.pc.rgevtail}
## ! \alias{pc.dgevtail}
## ! \alias{inla.pc.dgevtail}
## ! \alias{pc.pgevtail}
## ! \alias{inla.pc.pgevtail}
## ! \alias{pc.qgevtail}
## ! \alias{inla.pc.qgevtail}
## !
## ! \title{Utility functions for the PC prior for the \code{tail} parameter in the GEV likelihood}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the \code{tail} parameter
## !              in the GEV likelihood}
## ! \usage{
## ! inla.pc.rgevtail(n, lambda = 7)
## ! inla.pc.dgevtail(xi, lambda = 7, log = FALSE)
## ! inla.pc.qgevtail(p, lambda = 7)
## ! inla.pc.pgevtail(q, lambda = 7)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{lambda}{The rate parameter in the PC-prior}
## !   \item{xi}{Vector of evaluation points, where \code{1>xi>0}.}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## ! This gives the PC prior for the \code{tail} parameter for the GEV likelihood,
## ! where \code{xi=0} is the base model.
## ! }
## !\value{%%
## !  \code{inla.pc.dgevtail} gives the density,
## !  \code{inla.pc.pgevtail} gives the distribution function,
## !  \code{inla.pc.qgevtail} gives the quantile function, and
## !  \code{inla.pc.rgevtail} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.gevtail")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! xi = inla.pc.rgevtail(100,  lambda = 7)
## ! d = inla.pc.dgevtail(xi, lambda = 7)
## ! xi = inla.pc.qgevtail(0.5, lambda = 7)
## ! inla.pc.pgevtail(xi, lambda = 7)
## ! }

inla.pc.gevtail.map <- function(x, inverse = FALSE, deriv = 0) {
    ## map from R to (0, 1)
    if (!inverse) {
        ex <- exp(x)
        if (deriv == 0) {
            return(ex / (1 + ex))
        } else if (deriv == 1) {
            return(ex / (1 + ex)^2)
        } else {
            stop("error")
        }
    } else {
        if (deriv == 0) {
            return(log(x / (1.0 - x)))
        } else {
            stop("error")
        }
    }
}
inla.pc.gevtail.cache <- function() {
    ## return the cache for these functions. even though we could do all this analytic, the
    ## simplicity of just reusing the code was to tempting...

    dist <- function(lxi) {
        xi <- inla.pc.gevtail.map(lxi)
        kld <- xi^2 / (1 - xi)
        return(sqrt(2.0 * kld))
    }

    tag <- "cache.pc.gevtail"
    if (!exists(tag, envir = inla.get.inlaEnv())) {
        lxis <- seq(-15, 10, by = 0.01)
        dist <- dist(lxis)
        assign(tag,
            list(
                x = lxis,
                dist = splinefun(lxis, dist),
                idist = splinefun(dist, lxis)
            ),
            envir = inla.get.inlaEnv()
        )
    }
    return(get(tag, envir = inla.get.inlaEnv()))
}

inla.pc.rgevtail <- function(n, lambda = 7) {
    cache <- inla.pc.gevtail.cache()
    x <- numeric(n)
    z <- rexp(n, rate = lambda)
    x <- cache$idist(z)
    return(inla.pc.gevtail.map(x))
}

inla.pc.dgevtail <- function(xi, lambda = 7, log = FALSE) {
    cache <- inla.pc.gevtail.cache()
    d <- numeric(length(xi))
    lxi <- inla.pc.gevtail.map(xi, inverse = TRUE)
    dist <- cache$dist(lxi)
    deriv <- cache$dist(lxi, deriv = 1)
    d <- log(lambda) - lambda * dist + log(abs(deriv)) - log(abs(inla.pc.gevtail.map(lxi, deriv = 1)))
    return(if (log) d else exp(d))
}

inla.pc.qgevtail <- function(p, lambda = 7) {
    cache <- inla.pc.gevtail.cache()
    n <- length(p)
    q <- numeric(n)
    qe <- qexp(p, rate = lambda)
    q <- cache$idist(qe)
    return(inla.pc.gevtail.map(q))
}

inla.pc.pgevtail <- function(q, lambda = 7) {
    cache <- inla.pc.gevtail.cache()
    n <- length(q)
    p <- numeric(n)
    qq <- cache$dist(inla.pc.gevtail.map(q, inverse = TRUE))
    p <- pexp(qq, rate = lambda)
    return(p)
}

inla.pc.gevtail.test <- function(lambda = 7) {
    ## this is just an internal test, and is not exported
    n <- 10^6
    x <- inla.pc.rgevtail(n, lambda = lambda)
    x <- x[x < quantile(x, prob = 0.999)]
    xi <- seq(min(x), max(x), by = 0.001)
    hist(x, n = 300, prob = TRUE)
    lines(xi, inla.pc.dgevtail(xi, lambda), lwd = 3, col = "blue")
    p <- seq(0.1, 0.9, by = 0.05)
    print(cbind(p = p, q.emp = as.numeric(quantile(x, prob = p)), q.true = inla.pc.qgevtail(p, lambda)))
    q <- as.numeric(quantile(x, prob = p))
    cdf <- ecdf(x)
    print(cbind(q = q, p.emp = cdf(q), p.true = inla.pc.pgevtail(q, lambda)))
}

## How to run test outside the library:
## inla.get.inlaEnv = function(...) inla.get.inlaEnv(...)
## inla.pc.gevtail.test(lambda = .01)

## Export: inla.debug.graph

## !\name{debug.graph}
## !\alias{debug.graph}
## !\alias{inla.debug.graph}
## !\title{Debug a graph-file}
## !\description{Debug a graph specification on file (ascii-mode only), by checking the specification along the way.}
## !\usage{inla.debug.graph(graph.file)}
## !\arguments{
## !    \item{graph.file}{The filename of the graph (ascii-mode)}
## !}
## !\value{
## ! If an error is found, then an error message is shows, otherwise the graph-object returned by
## ! \code{inla.read.graph()} is returned.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{inla.read.graph}
## !\examples{
## !## Not run:
## !## cat("3\n 1 1 2n\ 2 1 1\n 3 4\n", file="g.dat")
## !## g = inla.debug.graph("g.dat")
## !## End(Not run)
## !}

`inla.debug.graph` <- function(graph.file) {
    ## read a graph with verbose output and try to detect any errors
    ## in the specification along the way. This is ment as a tool to
    ## detect errors in the graph specification only.

    stopifnot(file.exists(graph.file))

    xx <- readLines(graph.file, encoding = "utf-8")

    cat("\n")
    cat("* File [", graph.file, "] consists of ", length(xx), " lines.\n", sep = "")

    ## remove lines starting with '#'
    for (i in 1:length(xx)) {
        xx[i] <- gsub("[ \t]+", " ", xx[i])
        xx[i] <- gsub("#.*$", "", xx[i]) #
        if (length(grep("^[ \t]*$", xx[i])) > 0) {
            xx[i] <- NA
        }
    }
    xx <- xx[!is.na(xx)]
    cat("* Number of lines left after removing empty lines:", length(xx), "\n")

    to.ints <- function(text) {
        xx <- as.integer(unlist(sapply(text, function(x) strsplit(x, " "))))
        return(xx[!is.na(xx)])
    }

    N <- to.ints(xx[1])
    stopifnot(N > 0)

    cat("* Size of the graph is N=", N, "\n")

    for (i in 2:length(xx)) {
        cat("* Read line", i, "...")

        x <- to.ints(xx[i])
        cat("node =", x[1], " number.of.neigbours=", x[2])
        check <- (x[2] + 2L == length(x))
        if (!check) {
            cat("\n*** ERROR IN THIS LINE: x = ", x, "\n")
            stop("Number of neigbours  (x[2]) does not correspond to the number of elements in that line")
        }

        check <- all(sapply(x[-2], function(ii, N) {
            return(ii >= 1L & ii <= N)
        }, N = N))
        if (!check) {
            cat("\n*** ERROR IN THIS LINE: x = ", x, "\n")
            if (min(x[-2]) == 0L) {
                stop("Minimum node is 0, which is not/no longer, allowed.")
            } else {
                stop("One of more of the nodes are (x[-2]) outside the legal range: [1, ...,", N, "]\n")
            }
        }
        if (x[2L] > 0L) {
            check <- all(x[1L] != x[-c(1L, 2L)])
            if (!check) {
                cat("\n*** ERROR IN THIS LINE: x = ", x, "\n")
                stop("Node is defined as neighbour to itsef!")
            }
        }
        if (x[2L] > 0L) {
            check <- (length(x) == length(unique(x[-c(1L, 2L)])) + 2L)
            if (!check) {
                cat("\n*** ERROR IN THIS LINE: x = ", x, "\n")
                stop("Some of the neighbours are duplicated!")
            }
        }
        cat("  ok\n")
    }

    cat("\n")
    cat("\n")
    cat("* I will now try to read the graph properly using inla.read.graph().\n")
    cat("* If there are any errors in the following, then recall\n")
    cat("* that the numbering for lines and nodes, are 0-based (and NOT 1-based)!\n")

    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(shQuote(inla.call.builtin()), "-s -m graph", shQuote(graph.file)), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(shQuote(inla.call.builtin()), "-s -m graph", shQuote(graph.file)), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    return(inla.read.graph(graph.file))
}

## Nothing to export

## I am not sure if this is a good idea after all...

`inla.model.wrapper` <- function(
                                 covariates = NULL,
                                 model = c("joint"),
                                 data = NULL,
                                 Q = NULL,
                                 precision = 1.0E8,
                                 hyper = NULL,
                                 initial = NULL,
                                 prior = NULL,
                                 param = NULL,
                                 fixed = NULL,
                                 ...) {
    model <- match.arg(model, several.ok = FALSE)

    if (inla.one.of(model, "joint")) {
        res <- list()
        res$model <- "generic0"

        if (is.character(covariates)) {
            if (is.null(data)) {
                covariates <- eval.parent(parse(text = covariates))
            } else {
                covariates <- eval(parse(text = covariates), data)
            }
        }
        covariates <- as.matrix(covariates)
        n <- dim(covariates)[1]
        nb <- dim(covariates)[2]
        N <- n + nb

        if (is.null(Q) || any(dim(Q) != nb)) {
            stop("Matrix Q is either null or has wrong dimension")
        }

        QQ <- sparseMatrix(i = 1:n, j = 1:n, x = rep(precision, n), dims = c(N, N))
        for (i in 1:n) {
            cov.row <- -precision * covariates[i, 1:nb, drop = FALSE]
            QQ[(n + 1):N, i] <- cov.row
            QQ[i, (n + 1):N] <- t(cov.row)
        }
        QQ[(n + 1):N, (n + 1):N] <- precision * t(covariates) %*% covariates + Q

        hyper <- inla.set.hyper(
            model, "wrapper", hyper,
            initial, fixed, prior, param
        )

        ret <- list(f = list(
            Cmatrix = QQ, hyper.default = hyper,
            model = "generic", n = N, values = 1:N, rankdef = n
        ))
        class(ret) <- "inla.wrapper.model"

        return(ret)
    } else {
        stop("This should not happen.")
    }
}

## Nothing to Export.

`inla.display.matrix` <- function(x, wrap = TRUE, xaxt = FALSE, yaxt = FALSE, col = gray(seq(0, 1, len = 256)), ...) {
    ## display a matrix as an image with correct layout and autoscaling

    x <- as.matrix(x)
    n <- dim(x)
    y <- x
    if (wrap) {
        ii <- 1L:n[1L]
        jj <- n[1L]:1L
        for (j in 1L:n[2L]) {
              y[ii, j] <- x[jj, j]
          }
    }

    ## use the image.plot-function in package fields; its much better...
    inla.squishplot(c(0, 1), c(0, 1), n[1] / n[2])
    inla.require("fields", stop.on.error = TRUE)
    fields::image.plot(t(y), col = col, bty = "n", xaxt = "n", yaxt = "n", ...)

    box()
    if (xaxt) {
        title(xlab = "")
        nn <- (n[2] + 1) %/% 2
        axis(1, at = c(0, (nn - 1) / (n[2] - 1), 1), labels = as.character(c(1, nn, n[2])))
    }
    if (yaxt) {
        title(ylab = "")
        nn <- (n[1] + 1) %/% 2
        axis(2, at = c(0, (nn - 1) / (n[1] - 1), 1), labels = as.character(c(1, nn, n[1])))
    }
}

`inla.squishplot` <- function(xlim, ylim, asp = 1, newplot = TRUE) {
    ## This function is a copy from package TeachingDemos

    if (length(xlim) < 2) {
          stop("xlim must be a vector of length 2")
      }
    if (length(ylim) < 2) {
          stop("ylim must be a vector of length 2")
      }
    if (newplot) {
          plot.new()
      }
    tmp <- par(c("plt", "pin", "xaxs", "yaxs"))
    if (tmp$xaxs == "i") {
        xlim <- range(xlim)
    } else {
        tmp.r <- diff(range(xlim))
        xlim <- range(xlim) + c(-1, 1) * 0.04 * tmp.r
    }
    if (tmp$yaxs == "i") {
        ylim <- range(ylim)
    } else {
        tmp.r <- diff(range(ylim))
        ylim <- range(ylim) + c(-1, 1) * 0.04 * tmp.r
    }
    tmp2 <- (ylim[2] - ylim[1]) / (xlim[2] - xlim[1])
    tmp.y <- tmp$pin[1] * tmp2 * asp
    if (tmp.y < tmp$pin[2]) {
        par(pin = c(tmp$pin[1], tmp.y))
        par(plt = c(tmp$plt[1:2], par("plt")[3:4]))
    } else {
        tmp.x <- tmp$pin[2] / tmp2 / asp
        par(pin = c(tmp.x, tmp$pin[2]))
        par(plt = c(par("plt")[1:2], tmp$plt[3:4]))
    }
    return(invisible(tmp["plt"]))
}

#' Functions for defining the Barrier models
#'
#' Functions for defining Barrier models as an `inla rgeneric` model
#'
#' This model is described in the ArXiv preprint arXiv:1608.03787.  For
#' examples, see <https://haakonbakkagit.github.io/btopic128.html>
#'
#' @aliases inla.barrier barrier inla.barrier.pcmatern barrier.pcmatern
#' inla.barrier.polygon barrier.polygon inla.barrier.q barrier.q
#' inla.barrier.fem barrier.fem
#' @param mesh The mesh to build the model on, from inla.mesh.2d
#' @param barrier.triangles The numerical ids of the triangles that make up the
#' barrier area
#' @param prior.range 2 parameters `(range0,Prange)` for the prior spatial
#' range.  If `Prange` is `NA`, then `range0` is used as a fixed
#' range value (not tested).
#' @param prior.sigma 2 parameters `(sig0,Psig)` for the prior marginal
#' standard deviation sigma.  If `Psig` is `NA`, then `sig0` is
#' used as a fixed sigma value (not tested).
#' @param range.fraction The length of the spatial range inside the barrier
#' area, as a fraction of the range parameter.
#' @param Omega Advanced option for creating a set of permeable barriers (not
#' documented)
#' @return
#' * `inla.barrier.pcmatern` gives the (rgeneric) model object
#' for fitting the model in INLA
#' * `inla.barrier.polygon` gives the polygon
#' around the barrier (mainly for plotting)
#' * `inla.barrier.q` is an
#' internal method producing the Q matrix from a result of inla.barrier.fem,
#' * `inla.barrier.fem` is an internal method producing the Finite Element
#' matrices.
#' @author Haakon Bakka \email{bakka@@r-inla.org}
#' @seealso inla.spde2.pcmatern
#'
#' @export
#' @rdname inla.barrier
#' @details * `inla.barrier.pcmatern`
#' This function creates the model component used in inla(...)
`inla.barrier.pcmatern` <- function(mesh, barrier.triangles, prior.range, prior.sigma, range.fraction = 0.2) {
    ## Give default values if absolutely needed
    if (missing(prior.range)) {
        warning("Arbitrary prior values chosen automatically. This may suffice for a first attempt, 
            but should be changed in any serious analysis.")
        prior.range <- c(diff(range(mesh$loc[, 1])) / 5, 0.5)
    }

    if (missing(prior.sigma)) {
        prior.sigma <- c(1, 0.5)
    }

    ## Input verification
    stopifnot(class(mesh) == "inla.mesh")
    stopifnot(range.fraction > 0.000001)

    ## ## ## FUNCTIONS FOR RGENERIC MODEL SETUP ## ## ##

    ## This function is the model component definition in the rgeneric inla framework
    ## See inla.doc('rgeneric')
    barrier.rgeneric.model <- function(cmd = c(
                                           "graph", "Q", "mu", "initial",
                                           "log.norm.const", "log.prior", "quit"
                                       ),
                                       theta = NULL) {

        envir = parent.env(environment())

        ## Dynamic input
        ## theta = log(sigma), log(range1), log(range2), ...

        ## Static input
        prior.sigma <- obj$prior.sigma
        prior.range <- obj$prior.range

        ## The finite element matrices
        fem <- obj$fem

        ## The function for creating the precision matrix
        inla.barrier.q <- obj$inla.barrier.q

        ## The fraction used for the barrier model
        range.fraction <- obj$range.fraction

        ## Initial values on the internal scale
        ## Can be overrridden in inla(...)
        ## Warning: Not implemented for multiple ranges
        initial <- function(theta) {
            initial.theta <- c()
            if (!is.na(prior.sigma[2])) {
                initial.theta <- c(0, initial.theta)
            }
            if (!is.na(prior.range[2])) {
                initial.theta <- c(initial.theta, 0)
            }

            return(initial.theta)
        }

        ## This is the log-prior for the internal parametrisation theta
        ## Log of probability for log transform of exponential distribution
        log.prior <- function(theta) {
            ## Lambdas:
            ## The prior parameters are the lambdas in the exponential
            ## priors for standard deviation and inverse-range
            ## - the first is log(prob)/exceed, the second log(prob)*exceed
            ## - the second is exponential for inverse range, therefore multiplication!

            val <- 0
            ## If the sigma is not fixed
            if (!is.na(prior.sigma[2])) {
                lambda0 <- -log(prior.sigma[2]) / prior.sigma[1]
                ## Prior for standard deviation
                val <- val + log(lambda0) - lambda0 * exp(theta[1]) + theta[1]
                ## Which of the thetas are ranges ...
                ## ... not the first
                theta.ran <- theta[-1]
            } else {
                ## ... all of them
                theta.ran <- theta
            }

            ## If the range(s) is/are not fixed
            if (!is.na(prior.range[2])) {
                lambda1 <- -log(prior.range[2]) * prior.range[1]

                ## Prior for range(s)
                ## All are forced to have the same prior (if there are more than 1)
                for (logrange in theta.ran) {
                    val <- val + log(lambda1) - lambda1 * exp(-logrange) + -logrange
                }
            }
            return(val)
        }

        Q <- function(theta) {
            ## Make a theta that contains all variables
            theta.full <- theta
            ## If there is no theta param for sigma
            ## add the fixed sigma value
            if (is.na(prior.sigma[2])) {
                theta.full <- c(log(prior.sigma[1]), theta.full)
            }
            ## If there is no theta param for sigma
            ## add the fixed range value
            if (is.na(prior.range[2])) {
                theta.full <- c(theta.full, log(prior.range[1]))
                ## How it would be for multiple ranges (not barrier model)
                                        # all.ranges = rep(prior.range[1], fem$hdim)
                                        # theta.full = c(theta.full, log(all.ranges))
            }

            ## Not implemented for multiple ranges
            ## Hence, we should always have 2 theta param at this stage
            stopifnot(length(theta.full) == 2)

            ## Construct the precision matrix
            ## Not from the raw theta, but using both parameters
            Q <- inla.barrier.q(fem = fem, ranges = exp(theta.full[2]) * c(1, range.fraction),
                                sigma = exp(theta.full[1]), envir = envir)
            return(Q)
        }

        ## Here we create the graph by just calling the Q function a few times
        ## on some arbitrary inputs (several: to be sure we do not get accidental 0s)
        graph <- function(theta) {
            require(methods)
            ntheta <- 2 # only for barrier model
            theta.full <- (1:ntheta) / 3.217233456
            G1 <- inla.barrier.q(fem = fem, ranges = exp(theta.full[2]) * c(1, range.fraction), sigma = exp(theta.full[1]))
            G1[G1 != 0] <- 1
            theta.full <- (1:ntheta)^2 / 12.1543534
            G2 <- inla.barrier.q(fem = fem, ranges = exp(theta.full[2]) * c(1, range.fraction), sigma = exp(theta.full[1]))
            G2[G2 != 0] <- 1

            return(G1 + G2)
        }

        ## These can always be like this for all rgeneric models
        mu <- function(theta) numeric(0)
        log.norm.const <- function(theta) numeric(0)
        quit <- function(theta) invisible()

        val <- do.call(match.arg(cmd), args = list(theta))
        return(val)
    }

    ## Create a valid Omega from the barrier.triangles
    barrier.triangles <- unique(barrier.triangles)

    ## Create the barrier model
    obj <- list()
    obj$prior.sigma <- prior.sigma
    obj$prior.range <- prior.range
    obj$range.fraction <- range.fraction
    obj$inla.barrier.q <- inla.barrier.q

    obj$fem <- inla.barrier.fem(mesh, barrier.triangles = barrier.triangles)
    barrier.model <- inla.rgeneric.define(model = barrier.rgeneric.model, optimize = TRUE, obj = obj)

    if (!is.na(prior.sigma[2]) && !is.na(prior.range[2])) {
        ## All ok
    } else {
        warning("Not properly tested, let us know if you have problems.")
    }

    return(barrier.model)
}

#' @export
#' @rdname inla.barrier
#' @details * `inla.barrier.polygon` This function constructs SpatialPolygons for the different subdomains (areas)
`inla.barrier.polygon` <- function(mesh, barrier.triangles, Omega = NULL) {
    ## Requires an inla mesh to work
    stopifnot(class(mesh) == "inla.mesh")
    ## Requires rgeos for combining polygons
    inla.require("rgeos", stop.on.error = TRUE)

    if (missing(barrier.triangles)) {
        ## Use Omega
    } else {
        ## Create a valid Omega
        barrier.triangles <- unique(barrier.triangles)
        ## Number of triangles in total
        t <- length(mesh$graph$tv[, 1])
        remaining <- setdiff(1:t, barrier.triangles)
        if (!is.null(Omega)) warning("Omega is replaced by barrier.triangles")
        Omega <- list(remaining, barrier.triangles)
    }

    Omega.SP.list <- list()
    for (j in 1:length(Omega)) {
        poly.list <- list()
        for (tri in Omega[[j]]) {
            px <- mesh$graph$tv[tri, ]
            temp <- mesh$loc[px, ] # is a 3 by 3 matrix of node locations
            poly.list <- c(poly.list, Polygon(rbind(temp[, 1:2], temp[1, 1:2]), hole = F))
        }
        mesh.polys <- SpatialPolygons(list(Polygons(poly.list, ID = "noid")))
        Omega.SP.list[[j]] <- rgeos::gUnaryUnion(mesh.polys)
    }

    if (missing(barrier.triangles)) {
        return(Omega.SP.list)
    } else {
        ## Only return the polygon describing the barrier
        return(Omega.SP.list[[2]])
    }
}

### PRECISION MATRIX FUNCTIONS ###
## I.e. Solve the differential equation (the SPDE)

#' @details * `inla.barrier.q`: This function computes a specific precision matrix
#' @param fem represents the Barrier model or the Different Terrains (DT) model,
#' by containing all the needed matrices to solve the SPDE
#' @param ranges,sigma the hyperparameters that determine Q
#' @param envir the environment used for caching (with optimize=TRUE), if any
#' @export
#' @rdname inla.barrier
`inla.barrier.q` <- function(fem, ranges, sigma = 1, envir = NULL) {
    if (is.null(ranges)) stop("ranges cannot be NULL")
    if (any(is.na(ranges))) stop("No range can be NA")

    xi <- length(ranges)
    if (xi != length(fem$D)) {
        print("inla.barrier.q has encountered an error. Will stop.")
        stop("Ranges do no correspond to fem")
    }
    if (any(ranges < 0.001)) {
        warning("This hyper parameter value may fail. A very small maximum edge length needed in the mesh.")
    }

    Cdiag <- ranges[1]^2 * fem$C[[1]] # already raised to power 2
    if (xi > 1) {
        for (k in 2:xi) {
            Cdiag <- Cdiag + ranges[k]^2 * fem$C[[k]]
        }
    }
    N <- length(Cdiag)
    Cinv <- sparseMatrix(i = 1:N, j = 1:N, x = 1.0 / Cdiag, dims = c(N, N), repr = "T")

    A <- fem$I
    for (k in 1:xi) {
        A <- A + (ranges[k]^2 / 8.0) * fem$D[[k]]
    }

    ## Build the precision matrix
    ## The last multiplication factor is because the C matrix
    ## is different by a factor of 3 compared to the corresponding matrix
    ## in the stationary spde approach
    Q <- inla.as.sparse(t(A) %*% Cinv %*% A * (1 / sigma^2) / pi * 2 * 3)

    ## if it exists, then this is the environment used for caching and optimize=TRUE.
    ## otherwise, use optimize=FALSE format
    if (is.environment(envir)) {
        if (!exists("cache.done", envir = envir)) {
            Qx.idx <- which(Q@i <= Q@j)
            assign("Qx.idx", Qx.idx, envir = envir)
            assign("cache.done", TRUE, envir = envir)
        } else {
            Qx.idx <- get("Qx.idx", envir = envir)
        }
        return(Q@x[Qx.idx])
    } else {
        return (Q)
    }
}

#' @details * `inla.barrier.fem` This function computes the Finite Element
#' matrices that are needed to compute the precision matrix Q later
#' @export
#' @rdname inla.barrier
`inla.barrier.fem` <- function(mesh, barrier.triangles, Omega = NULL) {
    stopifnot(class(mesh) == "inla.mesh")

    if (missing(barrier.triangles) && is.null(Omega)) stop("Input barrier triangles")

    if (missing(barrier.triangles)) {
        ## Use Omega
    } else {
        ## Create a valid Omega
        barrier.triangles <- unique(barrier.triangles)
        t <- length(mesh$graph$tv[, 1])
        ## - all triangles
        remaining <- setdiff(1:t, barrier.triangles)
        if (!is.null(Omega)) warning("Omega is replaced by barrier.triangles")
        Omega <- list(remaining, barrier.triangles)
    }

    ## This function computes the Finite Element matrix of the white noise
    ## on one subdomain (area)
    ## Returns a diagonal matrix, represented as a vector
    dt.fem.white <- function(mesh, subdomain) {

        ## Pre-allocation
        Ck <- rep(0, mesh$n)

        for (t in subdomain) {
            ## Node indexes for triangle t:
            px <- mesh$graph$tv[t, ]
            temp <- mesh$loc[px, ] # is a 3 by 3 matrix of node locations
            p1 <- t(t(temp[1, c(1, 2)]))
            p2 <- t(t(temp[2, c(1, 2)]))
            p3 <- t(t(temp[3, c(1, 2)]))
            Ts <- cbind(p2 - p1, p3 - p1) # is the transformation to reference triangle
            area <- abs(det(Ts)) * 0.5
            for (i in 1:3) {
                Ck[px[i]] <- Ck[px[i]] + area
            }
        }
        return(Ck)
    }

    ## This function computes the Finite Element matrix for the '1' in
    ## the SPDE (the identity operator) <phi, phi>
    ## Result does not depend on the subdomains
    dt.fem.identity <- function(mesh) {
        ## Preallocation
        len <- length(mesh$graph$tv[, 1])
        index.i <- rep(0, len * 6)
        index.j <- rep(0, len * 6)
        Aij <- rep(0, len * 6) # matrix values
        counter <- 1

        for (t in 1:len) {
            ## Node indexes for triangle t:
            px <- mesh$graph$tv[t, ]
            temp <- mesh$loc[px, ] # is a 3 by 3 matrix of node locations
            p1 <- t(t(temp[1, c(1, 2)]))
            p2 <- t(t(temp[2, c(1, 2)]))
            p3 <- t(t(temp[3, c(1, 2)]))
            Ts <- cbind(p2 - p1, p3 - p1) # is the transformation to reference triangle
            twiceArea <- abs(det(Ts))

            for (i in 1:3) {
                index.i[counter] <- px[i]
                index.j[counter] <- px[i] # same
                Aij[counter] <- (twiceArea) * 1 / 12
                counter <- counter + 1
            }

            for (i in 1:2) {
                for (j in (i + 1):3) {
                    index.i[counter] <- px[i]
                    index.j[counter] <- px[j]
                    Aij[counter] <- (twiceArea) * 1 / 24
                    counter <- counter + 1
                    ## symmetry:
                    index.i[counter] <- px[j]
                    index.j[counter] <- px[i]
                    Aij[counter] <- (twiceArea) * 1 / 24
                    counter <- counter + 1
                }
            }
        }

        I <- sparseMatrix(i = index.i, j = index.j, x = Aij,
                          dims = c(mesh$n, mesh$n), repr = "T")
        return(I)
    }

    ## This function computes the Finite Element matrix of the laplace operator
    ## on one subdomain (area)
    ## The resulting matrix is very sparse
    dt.fem.laplace <- function(mesh, subdomain) {
        ## The nabla phi's
        Nphix <- rbind(c(-1, -1), c(1, 0), c(0, 1))
        len <- length(subdomain)
        index.i <- rep(0, len * 9)
        index.j <- rep(0, len * 9)
        Aij <- rep(0, len * 9) # matrix values
        counter <- 1

        for (tri in subdomain) {
            px <- mesh$graph$tv[tri, ]
            temp <- mesh$loc[px, ] # is a 3 by 3 matrix of node locations
            p1 <- t(t(temp[1, c(1, 2)]))
            p2 <- t(t(temp[2, c(1, 2)]))
            p3 <- t(t(temp[3, c(1, 2)]))
            Ts <- cbind(p2 - p1, p3 - p1) # is the transformation to reference triangle
            TTTinv <- solve(t(Ts) %*% Ts)
            area <- abs(det(Ts)) * 0.5

            for (k in 1:3) {
                for (m in 1:3) {
                    tmp <- (3 * m + k - 4) * length(subdomain)
                    index.i[(tmp + counter)] <- px[k]
                    index.j[(tmp + counter)] <- px[m]
                    Aij[(tmp + counter)] <- area * Nphix[k, c(1, 2)] %*% TTTinv %*% as.matrix(Nphix[m, c(1, 2)])
                }
            }
            counter <- counter + 1
        }

        Dk <- sparseMatrix(i = index.i, j = index.j, x = Aij, dims = c(mesh$n, mesh$n), repr = "T")
        return(Dk)
    }

    ## Call all the functions to get the resulting list of fem matrices

    xi <- length(Omega)
    fem <- list()
    fem$I <- dt.fem.identity(mesh)
    fem$D <- list()
    fem$C <- list()
    for (k in 1:xi) {
        fem$D[[k]] <- dt.fem.laplace(mesh, Omega[[k]])
    }
    for (k in 1:xi) {
        fem$C[[k]] <- dt.fem.white(mesh, Omega[[k]])
    }
    fem$hdim <- xi

    return(fem)
}

## Export: inla.dev.new

## ! \name{inla.dev.new}
## ! \alias{inla.dev.new}
## !
## ! \title{Opens a new device}
## !
## ! \description{Open a new device using \code{\link{dev.new}} unless using RStudio}
## ! \usage{
## !     inla.dev.new(...)
## ! }
## ! \arguments{
## !   \item{...}{Optional arguments to \code{\link{dev.new}}}
## !}
## ! \value{
## !   The value of \code{\link{dev.new}} if not running RStudio, otherwise \code{NULL}
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}

`inla.dev.new` <- function(...) {
    ## If running in RStudio then don't open a new device,  otherwise,  do.
    dev <- getOption("device")
    if (is.character(dev) && inla.strncasecmp(dev, "RStudioGD")) {
        ret <- NULL
    } else {
        ret <- dev.new(...)
    }
    if (exists("inla.dev.new.hook") && is.function(inla.dev.new.hook)) {
        do.call("inla.dev.new.hook")
    }
    return(invisible(ret))
}

## Export: inla.graph2matrix inla.spy

## !\name{graph2matrix}
## !\alias{graph2matrix}
## !\alias{inla.graph2matrix}
## !\alias{spy}
## !\alias{inla.spy}
## !\title{Construct a neighbour-matrix from a \code{graph}}
## !\description{Construct a neighbour-matrix from a \code{graph} and disaply it}
## !\usage{
## !    inla.graph2matrix(graph, ...)
## !    inla.spy(graph, ..., reordering = NULL, factor = 1.0, max.dim = NULL)
## !}
## !\arguments{
## !    \item{graph}{An \code{inla.graph}-object, a (sparse) symmetric matrix, a filename containing the graph,
## !                 or a list or collection of characters and/or numbers defining the graph.}
## !    \item{reordering}{A possible reordering. Typical the one obtained from a \code{inla}-call,  \code{result$misc$reordering},
## !                      or the result of \code{inla.qreordering}.}
## !    \item{factor}{A scaling of the \code{inla.graph}-object to reduce the size.}
## !    \item{max.dim}{Maximum dimension of the \code{inla.graph}-object plotted;
## !                   if \code{missing(factor)} and \code{max.dim} is set,  then \code{factor}
## !                   is computed automatically to give the given \code{max.dim}.}
## !    \item{...}{Additional arguments to \code{inla.read.graph()}}
## !}
## !\value{
## !     \code{inla.graph2matrix} returns a sparse symmetric matrix where the non-zero pattern is defined by the \code{graph}.
## !     The \code{inla.spy} function, plots a binary image of a \code{graph}. The \code{reordering} argument
## !     is typically the reordering used by \code{inla}, found in \code{result$misc$reordering}.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{
## !    \code{\link{inla.read.graph}}, \code{inla.qreordering}
## !}
## !\examples{
## !n = 50
## !Q = matrix(0, n, n)
## !idx = sample(1:n, 2*n, replace=TRUE)
## !Q[idx, idx] = 1
## !diag(Q) = 1
## !g = inla.read.graph(Q)
## !QQ = inla.graph2matrix(g)
## !inla.spy(QQ)
## !print(all.equal(as.matrix(Q), as.matrix(QQ)))
## !
## !g.file = inla.write.graph(g)
## !inla.dev.new()
## !inla.spy(g.file)
## !inla.spy(g.file,  reordering = inla.qreordering(g))
## !
## !g = inla.read.graph(g.file)
## !inla.dev.new()
## !inla.spy(g)
## !
## !inla.dev.new()
## !inla.spy(3, 1, "1 2 2 1 1 3 0")
## !inla.dev.new()
## !inla.spy(3, 1, "1 2 2 1 1 3 0", reordering = 3:1)
## !}

`inla.matrix2graph` <- function(graph, ...) {
    ## this function is not really needed, but is included for
    ## backwards compatibility
    return(inla.read.graph(graph, ...))
}

`inla.graph2matrix` <- function(graph, ...) {
    g <- inla.read.graph(graph, ...)
    i <- rep(1:g$n, 1L + g$nnbs)
    j <- unlist(sapply(1:g$n, function(i, g) {
        return(c(i, g$nbs[[i]]))
    }, g))
    stopifnot(length(i) == length(j))
    Q <- inla.as.dgTMatrix(sparseMatrix(i = i, j = j, x = rep(1, length(i))))

    return(Q)
}

`inla.spy` <- function(graph, ..., reordering = NULL, factor = 1.0, max.dim = NULL) {
    ## add this test here, as otherwise, this can be very inefficient
    ## for large matrices. this is because we convert it into a graph
    ## and then back to a matrix.
    M <- try(inla.as.dgTMatrix(graph, ...), silent = TRUE)
    if (inherits(M, "try-error")) {
        M <- inla.graph2matrix(graph, ...)
    }

    ## if max.dim is set, compute the corresponding factor
    if (missing(factor) && !is.null(max.dim) && max.dim > 0) {
        d.max <- max(dim(M))
        factor <- min(1.0, max.dim / d.max)
    }

    M <- inla.sparse.matrix.pattern(M, factor = factor, reordering = reordering)

    ## plot -M,  to make the background white
    inla.display.matrix(-M, nlevel = 2)
}

## Export: inla.binary.install

## ! \name{inla.binary.install}
## ! \alias{inla.binary.install}
## ! \alias{binary.install}
## ! \title{Install alternative binary builds}
## ! \description{
## !    Install alternative binary builds.
## ! }
## ! \usage{
## !    inla.binary.install(os = c("CentOS Linux-6", "CentOS Linux-7", "CentOS Linux-8",
## !                               "CentOS Stream-8", "Fedora-33", "Fedora-34", "Manjaro Linux",
## !                               "Ubuntu-16.04", "Ubuntu-18.04", "Ubuntu-20.04"), 
## !                        verbose = TRUE, md5.check = TRUE)
## ! }
## ! \arguments{
## !    \item{os}{If \code{os} is given, install binary build for this \code{os}.
## !              If \code{os} is not given, chose \code{os} interactively
## !              among available builds.}
## !    \item{verbose}{Logical. Verbose output if \code{TRUE}}
## !    \item{md5.check}{Logical. If \code{TRUE}, stop if md5-checksum-file is not present
## !                     or md5-checksum fail. If \code{FALSE}, ignore md5-checksum check.}
## ! }
## ! \details{
## ! Install a new binary for \code{os} unless
## ! \code{missing(os)}, for which the \code{os} is chosen
## ! interactively among the available builds.
## ! }
## ! \value{
## ! Return \code{TRUE} if installation was sucessful and \code{FALSE} if not.
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## !   \dontrun{
## !     inla.binary.install()
## !     inla.binary.install(os = "CentOS Linux-7")
## !   }
## ! }

`inla.binary.install` <- function(os = c("CentOS Linux-6",
                                         "CentOS Linux-7", 
                                         "CentOS Linux-8", 
                                         "CentOS Stream-8", 
                                         "Fedora-33", 
                                         "Fedora-34", 
                                         "Manjaro Linux", 
                                         "Ubuntu-16.04", 
                                         "Ubuntu-18.04", 
                                         "Ubuntu-20.04"),
                                  verbose = TRUE,
                                  md5.check = TRUE) {
    show <- function(...) {
        if (verbose) {
            msg <- paste(unlist(list(...)), sep = "", collapse = "")
            cat("* ", msg, "\n", sep = "")
        }
    }

    map.filename <- function(fnm) {
        return(gsub(" ", "%20", fnm))
    }

    random.num <- gsub("\\.", "", as.character(abs(rnorm(1))))
    os <- if (missing(os)) NULL else match.arg(os)
    stopifnot(inla.os.type() == "linux")
    version <- paste("Version_", inla.version("version"), sep = "")
    show("Looking for ", version, " and os=", if (!is.null(os)) os else "'<choose interactively>'")

    address <- "https://inla.r-inla-download.org/Linux-builds"
    Files <- paste0(address, "/FILES")
    fp <- url(Files, open = "r")
    ff <- readLines(fp)
    close(fp)
    ff <- ff[grep(version, ff)]
    nf <- length(ff)

    if (is.null(os)) {
        cat("  Available alternatives:\n")
        for (i in seq_len(nf)) {
            cat("  \t", paste0("Alternative ", i), " is ", ff[i], "\n")
        }
        cat("  ", "Chose alternative [", 1, ":", nf, "]", sep = "", "\n\t")
        ans <- scan(file = "", what = integer(), n = 1, quiet = TRUE)
        if (length(ans) == 0) {
            return (invisible(FALSE))
        }
    } else {
        ans <- grep(os, ff)
        if (length(ans) == 0) {
            stop(paste0("Sorry, os=", os, " is not available for ", version))
        }
        if (length(ans) > 1) {
            stop("Internal error. Please report to <help@r-inla.org>")
        }
    }
    if (!(ans %in% seq_len(nf))) stop("Not a valid choice. Exit.")

    fnm <- paste0(address, "/", ff[ans])
    show("Install file [", fnm, "]")
    pa <- searchpaths()
    pa <- pa[grep("/INLA$", pa)]
    if (length(pa) == 0) {
        stop("I cannot find '/INLA' in the searchpath(), so please retry after doing library('INLA')")
    }
    stopifnot(file.info(pa)$isdir)
    show("INLA is installed in [", pa, "]")
    pa <- paste0(pa, "/bin/linux")

    show("Checking for write access...")
    test.fnm <- paste0(pa, "/test-file---", random.num, ".txt")
    test.result <- file.create(test.fnm, showWarnings = FALSE)
    if (test.result) {
        unlink(test.fnm, force = TRUE)
    } else {
        show(paste0("ERROR: No write access to [", pa, "]"))
    }

    show("Download file, please wait...")
    to.file <- paste0(pa, "/64bit-download-", random.num, ".tgz")
    ret <- download.file(map.filename(fnm), to.file, quiet = TRUE)
    
    if (md5.check) {
        fnm.md5 <- gsub("/64bit.tgz", "/md5sum.txt", fnm)
        md5.file <- paste0(pa, "/64bit-download-md5sum-", random.num, ".txt")
        ret.md5 <- try(download.file(map.filename(fnm.md5), md5.file, quiet = TRUE),  silent = TRUE)
        if (!inherits(ret.md5, "try-error")) {
            md5.checksum <- scan(file=md5.file, what=character(), n=1, quiet = TRUE)
            if (md5.checksum == tools::md5sum(to.file)) {
                show("md5-checksum [", md5.checksum, "] OK.")
            } else {
                stop(paste0("md5-checksum [", md5.checksum, "] FAILED. Stop."))
            }
        } else {
            stop("No md5-checksum found. Run with 'md5.check=FALSE' to force install.")
        }
    }

    if (ret == 0) {
        ##
    } else {
        unlink(to.file, force = TRUE)
        if (md5.check) {
            unlink(md5.file, force = TRUE)
        }
        stop("Error downloading file. Abort.")
    }

    my.restore <- function() {
        show("Error. Will try to restore old configuration.")
        show("If unsuccessful, then reinstall R-INLA.")
        unlink(from.dir, recursive = TRUE, force = TRUE)
        unlink(to.file, force = TRUE)
        file.rename(to.dir, from.dir)
    }

    show("Rename old 64bit directory...")
    from.dir <- paste0(pa, "/64bit")
    to.dir <- paste0(pa, "/64bit-", random.num)
    ret <- file.rename(from.dir, to.dir)
    if (ret == TRUE) {
        ##
    } else {
        my.restore()
        stop("Error renaming old 64bit directory. Abort.")
    }

    show("Unpack file...")
    ret <- untar(to.file, exdir = dirname(to.file), verbose = FALSE)
    if (ret == 0) {
        ##
    } else {
        my.restore()
        stop("Error unpacking file. Abort.")
    }

    show("Remove temporary file...")
    unlink(to.file, force = TRUE)
    if (md5.check) {
        unlink(md5.file, force = TRUE)
    }
    show("Remove old 64bit directory...")
    unlink(to.dir, recursive = TRUE, force = TRUE)
    show("Done!")

    return(invisible(TRUE))
}

## Export: inla.jp.define

## !\name{jp.define}
## !\alias{jp}
## !\alias{jp.define}
## !\alias{inla.jp.define}
## !\title{Joint-prior models}
## !\description{A framework for defining joint priors in R}
## !
## !\usage{
## !    inla.jp.define(jp = NULL, ...)
## !}
## !
## !\arguments{
## !  \item{jp}{The \code{jp}-function which returns the joint log-prior as a function of
## !            argument \code{theta}. There is an optional second argument that is a
## !            vector of \code{theta}-names. If
## !            second argument is not present, argument \code{.theta.desc}
## !            will be added.}
## !  \item{...}{Named list of variables that defines the environment of \code{jp}}
## !}
## !
## !\value{%%
## !  This allows joint priors to be defined in \code{R}.
## !
## !  This function is for internal use only.}
## !\author{Havard Rue \email{hrue@r-inla.org}}


`inla.jp.define` <- function(jp = NULL, ...) {
    stopifnot(!missing(jp))
    stopifnot(is.function(jp))
    args <- list(...)
    if (any(names(args) == "")) {
        stop("The '...' argument in 'inla.jp.define()' needs *named* arguments.")
    }

    a <- names(formals(jp))
    if (length(a) == 1) {
        ## add second argument
        fun <- inla.eval(paste0("function(", a[1], ", .theta.desc = NULL) NULL"))
        body(fun) <- body(jp)
    } else if (length(a) == 2) {
        fun <- jp
    } else {
        stop("Number of arguments in 'jp'-function must be 1 or 2.")
    }

    env <- if (length(args) > 0) as.environment(args) else new.env()
    parent.env(env) <- .GlobalEnv
    environment(fun) <- env

    fun.c <- inla.cmpfun(fun)
    rjp <- list(model = fun.c) ## maybe we need something additional later
    class(rjp) <- "inla.jp"

    return(rjp)
}

## Export: inla.compare.results

## !\name{inla.compare.results}
## !\alias{inla.compare.results}
## !\alias{compare.results}
## !
## !\title{Compare INLA and MCMC results}
## !
## !\description{A small utility to compare INLA and MCMC results (OBSOLETE)}
## !\usage{
## ! inla.compare.results(dir.inla = NULL, dir.mcmc = NULL)
## !}
## !
## !\arguments{
## !  \item{dir.inla}{The directory with the INLA results}
## !  \item{dir.mcmc}{The directory with the MCMC results}
## !}
## !
## !\value{%%
## !  Return nothing. This is an interactive function.
## !
## !  This function is OBSOLETE
## !}
## !%%
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\examples{
## !## See demo("Tokyo-compare")
## !}


`inla.compare.results` <- function(dir.inla = NULL, dir.mcmc = NULL) {
    ## This function is useful for comparing the results of INLA with
    ## those obtained from a MCMC run (ie running `inla -m mcmc ...')

    if (is.null(dir.inla)) {
        print("Enter directory for INLA results ")
        dir.inla <- scan(nmax = 1, what = character(0))
    }
    if (!file.exists(dir.inla)) {
          stop(paste(" *** ERROR *** No such directory ", dir.inla))
      }
    if (file.exists(paste(dir.inla, "/", "results.files", sep = ""))) {
          dir.inla <- paste(dir.inla, "/", "results.files", sep = "")
      }

    if (is.null(dir.mcmc)) {
        print("Enter directory for MCMC (from ``inla -m mcmc'') results ")
        dir.mcmc <- scan(nmax = 1, what = character(0))
    }
    if (!file.exists(dir.mcmc)) {
          stop(paste(" *** ERROR *** No such directory ", dir.mcmc))
      }
    if (file.exists(paste(dir.mcmc, "/", "results.files", sep = ""))) {
          dir.mcmc <- paste(dir.mcmc, "/", "results.files", sep = "")
      }

    options(device.ask.default = TRUE)
    inla.dev.new()

    dd <- c()
    for (d in dir(dir.inla)) {
          if (file.exists(paste(dir.inla, "/", d, "/marginal-densities.dat", sep = ""))) {
                dd <- c(dd, d)
            }
      }

    quit <- FALSE
    while (TRUE) {
        cat("\nChose directory to view\n")
        i <- 1
        for (a in dd) {
            cat("[", i, "] ", a, "\n", sep = "")
            i <- i + 1
        }
        cat("\n(<0: histogram, >0: density: q:quit) : ")

        ok <- FALSE
        while (!ok) {
            id <- readline("")
            if (id == "q" || id == "quit") {
                ok <- TRUE
                quit <- TRUE
            } else {
                id <- as.integer(id)
                if (!is.na(id)) {
                      ok <- TRUE
                  } else {
                      cat("***Input error.\n")
                  }
            }
        }

        if (quit) {
              return(invisible())
          }

        if (id < 0) {
            hist <- TRUE
            id <- abs(id)
        } else {
            hist <- FALSE
        }

        d.inla <- paste(dir.inla, "/", dd[id], sep = "")
        d.mcmc <- paste(dir.mcmc, "/", dd[id], sep = "")

        fnm <- paste(d.inla, "/", "marginal-densities.dat", sep = "")
        f <- paste(d.inla, "/", "TAG", sep = "")
        TAG <- inla.ifelse(file.exists(f), inla.paste(scan(paste(d.inla, "/", "TAG", sep = ""),
            what = as.character()
        )), "TAG")
        if (file.exists(fnm)) {
            ## print(paste("read marginals from ", fnm))
            marg <- read.table(fnm)

            n.marg <- dim(marg)[1]
            n.points <- dim(marg)[2]

            fnm <- paste(d.mcmc, "/", "trace.dat", sep = "")
            if (file.exists(fnm)) {
                samples <- read.table(fnm)
                di <- dim(samples)
                samples <- samples[-dim(samples)[1], ]
                ## with ncol=1, the previous line remove dimension attributes
                samples <- as.matrix(samples, nrow = di[1] - 1, ncol = di[2])
                n.samples <- dim(samples)[1]

                if (dim(samples)[2] == n.marg) {
                    if (n.marg <= 4) {
                          par(mfrow = c(n.marg, 1))
                      } else {
                          par(mfrow = c(4, 3))
                      }

                    for (i in 1:n.marg) {
                        x.values <- as.double(marg[i, seq(2, n.points, by = 2)])
                        f.values <- as.double(marg[i, seq(3, n.points, by = 2)])
                        plot(x.values, f.values,
                            type = "l", lty = 1, lwd = 2,
                            main = paste("Marginal for ", marg[i, 1], " ", TAG, sep = ""),
                            sub = fnm, frame.plot = FALSE
                        )
                        if (hist) {
                              hist(samples[, i], n = 50, add = TRUE, probability = TRUE)
                          } else {
                              lines(density(samples[, i]), lty = 3, lwd = 2)
                          }
                    }
                }
                else {
                      print("n.marg != columns(samples) ", n.marg, dim(samples)[2])
                  }
            }
            else {
                  print(paste(" *** ERROR *** file does not exists: ", fnm))
              }

            rm(marg)
            rm(samples)
        }
        else {
              print(paste(" *** ERROR *** file does not exits: ", fnm))
          }
    }
}

## Export: inla.qinv

## ! \name{qinv}
## ! \alias{inla.qinv}
## ! \alias{qinv}
## !
## ! \title{Computes (parts of) the inverse of a SPD sparse matrix}
## !
## ! \description{This routine use the GMRFLib implementation
## !              which compute parts of the inverse of a SPD sparse matrix.
## !              The diagonal and values for the neighbours in the inverse, are provided.}
## !
## ! \usage{
## !     inla.qinv(Q, constr, reordering = INLA::inla.reorderings(), num.threads = NULL)
## ! }
## !
## ! \arguments{
## !
## !   \item{Q}{A SPD matrix,  either as a (dense) matrix or sparseMatrix.}
## !   \item{constr}{Optional linear constraints;
## !                 see \code{?INLA::f} and argument \code{extraconstr}}
## !   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
## !        either one of the names listed in \code{inla.reorderings()}
## !        or the output from \code{inla.qreordering(Q)}.
## !        The default is "auto" which try several reordering algorithm and use the best one for this particular matrix.}
## !   \item{num.threads}{Maximum number of threads the
## !                      \code{inla}-program will use, or as 'A:B' defining the number threads in the
## !                      outer (A) and inner (B) layer for nested parallelism. }
## !  }
## ! \value{
## !   \code{inla.qinv} returns a \code{sparseMatrix} of type \code{dgTMatrix} with the
## !   diagonal and values for the neigbours in the inverse. Note that the full inverse is NOT provided!
## ! }
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## !
## ! ## dense matrix example
## ! n = 10
## ! A = matrix(runif(n^2), n, n)
## ! Q = A \%*\% t(A)
## ! print(mean(abs(inla.qinv(Q) - solve(Q))))
## !
## ! ## sparse matrix example
## ! rho = 0.9
## ! Q = toeplitz(c(1+rho^2, -rho,  rep(0, n-3), -rho)) / (1-rho^2)
## ! Q = inla.as.dgTMatrix(Q)
## ! Q.inv = inla.qinv(Q)
## !
## ! ## compute the marginal variances as a vector from a precision matrix
## ! marginal.variances = diag(inla.qinv(Q))
## !
## ! ## read the sparse matrix from a file in the 'i, j, value' format
## ! filename = tempfile()
## ! write(t(cbind(Q@i+1L,  Q@j+1L,  Q@x)), ncol=3, file=filename)
## ! Qinv = inla.qinv(filename)
## ! unlink(filename)
## ! }

`inla.qinv` <- function(Q, constr, reordering = INLA::inla.reorderings(),
                        num.threads = NULL) {
    t.dir <- inla.tempdir()
    smtp <- match.arg(inla.getOption("smtp"), c("taucs", "band", "default", "pardiso"))
    if (is.null(num.threads)) {
        num.threads <- inla.getOption("num.threads")
    }
    num.threads <- inla.parse.num.threads(num.threads)

    Q <- inla.sparse.check(Q)
    if (is(Q, "dgTMatrix")) {
        qinv.file <- inla.write.fmesher.file(Q, filename = inla.tempfile(tmpdir = t.dir))
    } else if (is.character(Q)) {
        qinv.file <- Q
    } else {
        stop("This chould not happen.")
    }

    constr.file <- inla.tempfile(tmpdir = t.dir)
    if (!missing(constr) && !is.null(constr)) {
        stopifnot(is.list(constr))
        A <- as.matrix(constr$A)
        e <- as.numeric(constr$e)
        stopifnot(ncol(A) == ncol(Q))
        stopifnot(nrow(A) == length(e))
        xx <- matrix(c(nrow(A), c(A), c(e)), ncol = 1)
        inla.write.fmesher.file(xx, filename = constr.file)
    }

    if (is.list(reordering)) {
        ## argument is the output from inla.qreordering()
        reordering <- reordering$name
    }
    reordering <- match.arg(reordering)

    out.file <- inla.tempfile(tmpdir = t.dir)
    inla.set.sparselib.env(inla.dir = t.dir)
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qinv",
            "-r", reordering, "-S", smtp, paste0("-t", num.threads),
            qinv.file, constr.file, out.file
        ), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qinv",
            "-r", reordering, "-S", smtp, paste0("-t", num.threads),
            qinv.file, constr.file, out.file
        ), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    Qinv <- inla.read.fmesher.file(out.file)
    unlink(t.dir, recursive = TRUE)

    return(Qinv)
}

## Export: plot!inla

## ! \name{plot.inla}
## ! \alias{plot.inla}
## ! \alias{inla.plot}
## ! \title{Default INLA plotting}
## ! \description{
## !   Takes am \code{inla} object produced by \code{inla} and plot the results
## ! }
## ! \usage{
## ! \method{plot}{inla}(x,
## !              plot.fixed.effects = TRUE,
## !              plot.lincomb = TRUE,
## !              plot.random.effects = TRUE,
## !              plot.hyperparameters = TRUE,
## !              plot.predictor = TRUE,
## !              plot.q = TRUE,
## !              plot.cpo = TRUE,
## !              plot.prior = FALSE,
## !              single = FALSE,
## !              postscript = FALSE,
## !              pdf = FALSE,
## !              prefix = "inla.plots/figure-",
## !              intern = FALSE,
## !              debug = FALSE,
## !              cex = 1.75,
## !              ...)
## ! }
## ! \arguments{
## !   \item{x}{A fitted  \code{inla} object produced by \code{inla} }
## !   \item{plot.fixed.effects}{Boolean indicating if posterior marginals
## !     for the fixed effects in the model should be plotted }
## !   \item{plot.lincomb}{Boolean indicating if posterior marginals
## !     for the linear combinations should be plotted }
## !   \item{plot.random.effects}{Boolean indicating if posterior mean and quantiles
## !     for the random effects in the model should be plotted  }
## !   \item{plot.hyperparameters}{Boolean indicating if posterior marginals
## !     for the hyperparameters in the model should be plotted }
## !   \item{plot.predictor}{Boolean indicating if posterior mean and quantiles
## !     for the linear predictor in the model should be plotted }
## !   \item{plot.q}{Boolean indicating if precision matrix should be displayed}
## !   \item{plot.cpo}{Boolean indicating if CPO/PIT valuesshould be plotted}
## !   \item{plot.prior}{Plot also the prior density for the hyperparameters}
## !   \item{single}{Boolean indicating if there should be more than one plot per page
## !                 (FALSE) or just one (TRUE)}
## !   \item{postscript}{Boolean indicating if postscript files should be produced instead}
## !   \item{pdf}{Boolean indicating if PDF files should be produced instead}
## !   \item{prefix}{The prefix for the created files. Additional numbering and suffix is added.}
## !   \item{intern}{Plot also the hyperparameters in its internal scale.}
## !   \item{debug}{Write some debug information}
## !   \item{cex}{The \code{cex} parameter in \code{par()}. If set to \code{NULL} or \code{0},
## !              then default values will be used for graphics parameters}
## !   \item{...}{Additional arguments to \code{postscript()}, \code{pdf()} or \code{dev.new()}.}
## ! }
## ! \value{The return value is a list of the files created (if any).}
## ! \author{Havard Rue \email{hrue@r-inla.org} }
## ! \seealso{\code{\link{inla}}}
## ! \examples{
## !\dontrun{
## !result = inla(...)
## !plot(result)
## !plot(result, single = TRUE, plot.prior = TRUE)
## !plot(result, single = TRUE, pdf = TRUE, paper = "a4")
## !   }
## ! }
## ! \keyword{plot}

`plot.inla` <-
    function(x,
             plot.fixed.effects = TRUE,
             plot.lincomb = TRUE,
             plot.random.effects = TRUE,
             plot.hyperparameters = TRUE,
             plot.predictor = TRUE,
             plot.q = TRUE,
             plot.cpo = TRUE,
             plot.prior = FALSE,
             single = FALSE,
             postscript = FALSE,
             pdf = FALSE,
             prefix = "inla.plots/figure-",
             intern = FALSE,
             debug = FALSE,
             cex = 1.75,
             ...) {
        ## for G & J
        if (!is.null(x$joint.model)) {
            return(plot(x$joint.model, ...))
        }

        ## if cex is ok, then use that one, otherwise, fall-back to plain defaults
        if (is.numeric(cex) && cex > 0) {
            cex <- cex
            cex.axis <- cex
            cex.lab <- cex
            cex.main <- cex
            cex.sub <- cex
            lwd <- 2
            bty <- "l"
            my.plot <- function(...) {
                plot(...,
                    cex = cex, cex.axis = cex.axis, cex.main = cex.main,
                    cex.sub = cex.sub, bty = "l", lwd = lwd
                )
            }
            my.lines <- function(...) {
                lines(...,
                    cex = cex, cex.axis = cex.axis, cex.main = cex.main,
                    cex.sub = cex.sub, bty = "l", lwd = lwd
                )
            }
            my.points <- function(...) {
                points(...,
                    cex = cex, cex.axis = cex.axis, cex.main = cex.main,
                    cex.sub = cex.sub, bty = "l", lwd = lwd
                )
            }
            my.title <- function(...) {
                title(...,
                    cex = cex, cex.axis = cex.axis, cex.main = cex.main,
                    cex.sub = cex.sub, bty = "l", lwd = lwd
                )
            }
        } else {
            my.plot <- function(...) plot(...)
            my.lines <- function(...) lines(...)
            my.points <- function(...) points(...)
            my.title <- function(...) title(...)
        }

        figure.count <- 1L
        figures <- c()

        if (plot.prior) {
            all.hyper <- inla.all.hyper.postprocess(x$all.hyper)
        }

        if (postscript && pdf) {
            stop("Only one of 'postscript' and 'pdf' can be generated at the time.")
        }

        initiate.plot <- function(...) {
            if (!postscript && !pdf) {
                inla.dev.new(...)
            } else {
                dir <- dirname(prefix)
                if (!file.exists(dir) && nchar(dir) > 0L) {
                    dir.create(dir, recursive = TRUE)
                } else {
                    stopifnot(file.info(dir)$isdir)
                }
            }
            return(invisible())
        }

        new.plot <- function(...) {
            if (!postscript && !pdf) {
                inla.dev.new(...)
            } else {
                found <- FALSE
                while (!found) {
                    filename <- paste(prefix,
                        inla.ifelse(regexpr("/$", prefix), "", "/"),
                        figure.count,
                        inla.ifelse(postscript, ".eps", ".pdf"),
                        sep = ""
                    )
                    if (file.exists(filename)) {
                        figure.count <<- figure.count + 1L ## YES
                    } else {
                        found <- TRUE
                    }
                }
                if (postscript) {
                    postscript(file = filename, ...)
                } else if (pdf) {
                    pdf(file = filename, ...)
                } else {
                    stop("This should not happen")
                }
                figures <<- c(figures, filename)
            }
            return(invisible())
        }

        close.plot <- function(...) {
            if (postscript || pdf) {
                if (names(dev.cur()) != "null device") {
                    dev.off()
                }
            }
            return(invisible())
        }

        close.and.new.plot <- function(...) {
            close.plot(...)
            new.plot(...)
            return(invisible())
        }

        ##
        initiate.plot(...)

        if (plot.fixed.effects) {
            ## plot marginals for the fixed effects
            fix <- x$marginals.fixed
            labels.fix <- names(x$marginals.fixed)
            nf <- length(labels.fix)
            if (nf > 0) {
                if (nf == 1 || single) {
                    plot.layout <- c(1, 1)
                } else if (nf == 2) {
                    plot.layout <- c(2, 1)
                } else {
                    plot.layout <- c(3, 3)
                }
                np <- prod(plot.layout)
                ip <- 0
                for (i in 1:nf) {
                    if (ip %% np == 0) {
                        close.and.new.plot(...)
                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                    }
                    ip <- ip + 1

                    if (!all(is.na(fix[[i]]))) {
                        ss <- x$summary.fixed[i, ]
                        sub <- paste("Mean = ", round(ss[names(ss) == "mean"], 3), " SD = ", round(ss[names(ss) == "sd"], 3), sep = "")
                        m <- inla.smarginal(fix[[i]])
                        my.plot(m,
                            type = "l", main = paste("PostDens [", inla.nameunfix(labels.fix[i]), "]", sep = ""),
                            sub = sub, xlab = "", ylab = ""
                        )

                        if (plot.prior) {
                            xy <- (inla.get.prior.xy(
                                section = "fixed", hyperid = labels.fix[i],
                                all.hyper = all.hyper, range = range(m$x), debug = debug
                            ))
                            my.lines(xy, col = "blue")
                        }
                    }
                }
            }
        }

        if (plot.lincomb) {
            ## plot marginals for the derived lincombs
            fix <- x$marginals.lincomb.derived
            labels.fix <- names(x$marginals.lincomb.derived)
            nf <- length(labels.fix)
            if (nf > 0) {
                if (nf == 1 || single) {
                    plot.layout <- c(1, 1)
                } else if (nf == 2) {
                    plot.layout <- c(2, 1)
                } else {
                    plot.layout <- c(3, 3)
                }
                np <- prod(plot.layout)
                ip <- 0
                for (i in 1:nf) {
                    if (ip %% np == 0) {
                        close.and.new.plot(...)
                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                    }
                    ip <- ip + 1

                    if (!all(is.na(fix[[i]]))) {
                        ss <- x$summary.lincomb.derived[i, ]
                        sub <- paste("Mean = ", round(ss[names(ss) == "mean"], 3), " SD = ", round(ss[names(ss) == "sd"], 3), sep = "")
                        my.plot(inla.smarginal(fix[[i]]),
                            type = "l",
                            main = paste("PostDens [", inla.nameunfix(labels.fix[i]), "] (derived)", sep = ""),
                            sub = sub, xlab = "", ylab = ""
                        )
                    }
                }
            }
        }

        if (plot.lincomb) {
            ## plot marginals for the lincombs
            if (inla.is.element("marginals.lincomb", x)) {
                fix <- x$marginals.lincomb
                labels.fix <- names(x$marginals.lincomb)
            } else {
                fix <- NULL
                labels.fix <- NULL
            }
            nf <- length(labels.fix)
            if (nf > 0) {
                if (nf == 1 || single) {
                    plot.layout <- c(1, 1)
                } else if (nf == 2) {
                    plot.layout <- c(2, 1)
                } else {
                    plot.layout <- c(3, 3)
                }
                np <- prod(plot.layout)
                ip <- 0
                for (i in 1:nf) {
                    if (ip %% np == 0) {
                        close.and.new.plot(...)
                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                    }
                    ip <- ip + 1

                    if (!all(is.na(fix[[i]]))) {
                        ss <- x$summary.lincomb[i, ]
                        sub <- paste("Mean = ", round(ss[names(ss) == "mean"], 3), " SD = ", round(ss[names(ss) == "sd"], 3), sep = "")
                        my.plot(inla.smarginal(fix[[i]]),
                            type = "l",
                            main = paste("PostDens [", inla.nameunfix(labels.fix[i]), "]", sep = ""),
                            sub = sub, xlab = "", ylab = ""
                        )
                    }
                }
            }
        }

        if (plot.random.effects) {
            rand <- x$summary.random
            labels.random <- names(x$summary.random)
            nr <- length(labels.random)
            if (nr > 0) {
                for (i in 1:nr) {
                    if (!all(is.na(rand[[i]]))) {
                        rr <- rand[[i]]
                        dim1 <- dim(rr)[1]
                        dim2 <- dim(rr)[2]
                        tp <- ifelse(labels.random[i] == "baseline.hazard", "s", "l")

                        r.n <- x$size.random[[i]]$n
                        r.N <- x$size.random[[i]]$N
                        r.Ntotal <- x$size.random[[i]]$Ntotal
                        nrep <- x$size.random[[i]]$nrep
                        ngroup <- x$size.random[[i]]$ngroup
                        r.n.orig <- r.n %/% ngroup
                        r.N.orig <- r.N %/% ngroup

                        ## determine the plot-layout here
                        nrr <- ngroup * nrep
                        if (nrr == 1 || single) {
                            plot.layout <- c(1, 1)
                        } else if (nrr == 2) {
                            plot.layout <- c(2, 1)
                        } else {
                            plot.layout <- c(3, 3)
                        }
                        np <- prod(plot.layout)
                        ip <- 0

                        if (r.n > 1) {
                            for (r.rep in 1:nrep) {
                                for (r.group in 1:ngroup) {
                                    for (ii in 1:inla.ifelse(r.N > r.n, r.N %/% r.n, 1)) {
                                        if (ip %% np == 0) {
                                            close.and.new.plot(...)
                                            par(mfrow = c(plot.layout[1], plot.layout[2]))
                                        }
                                        ip <- ip + 1

                                        rep.txt <- ""
                                        if (nrep > 1) {
                                            rep.txt <- paste(rep.txt, " rep:", r.rep, sep = "")
                                        }
                                        if (ngroup > 1) {
                                            rep.txt <- paste(rep.txt, " group:", r.group, sep = "")
                                        }
                                        if (r.N > r.n) {
                                            rep.txt <- paste(rep.txt, " part:", ii, sep = "")
                                        }

                                        idx <- (r.rep - 1) * r.N + (r.group - 1) * r.N.orig + (ii - 1) * r.n.orig + (1:r.n.orig)

                                        ## if the dimension is > 1, then plot the means++
                                        xval <- NULL
                                        if (tp == "s") {
                                            ## baseline.hazard
                                            xval <- rr[, colnames(rr) == "ID"][idx]
                                            yval <- rr[, colnames(rr) == "mean"][idx]
                                            my.plot(xval, yval,
                                                ylim = range(rr[, setdiff(colnames(rr), c("ID", "sd", "kld"))]),
                                                xlim = range(xval),
                                                axes = TRUE, ylab = "", xlab = "", type = tp
                                            )
                                            if (!is.null(x$.args$data$baseline.hazard.strata.coding)) {
                                                rep.txt <- inla.paste(c(
                                                    rep.txt, "[",
                                                    x$.args$data$baseline.hazard.strata.coding[r.rep], "]"
                                                ), sep = "")
                                            }
                                        } else {
                                            xval <- rr[, colnames(rr) == "ID"][idx]
                                            yval <- rr[, colnames(rr) == "mean"][idx]
                                            if (is.numeric(xval)) {
                                                my.plot(xval, yval,
                                                    ylim = range(rr[, setdiff(colnames(rr), c("ID", "sd", "kld"))]),
                                                    xlim = range(xval),
                                                    ylab = "", xlab = "", type = tp
                                                )
                                            } else {
                                                my.plot(as.factor(xval), yval,
                                                    ylim = range(rr[, setdiff(colnames(rr), c("ID", "sd", "kld"))]),
                                                    axes = TRUE, ylab = "", xlab = "", type = tp
                                                )
                                            }
                                        }

                                        lq <- grep("quan", colnames(rr))
                                        main <- inla.nameunfix(labels.random[i])

                                        if (length(lq) > 0) {
                                            qq <- rr[, lq]
                                            dq <- dim(qq)[2]
                                            sub <- paste("PostMean ")
                                            for (j in 1:dq) {
                                                yval <- qq[, j][idx]
                                                ## this is the baseline.hazard case
                                                if (length(yval) + 1 == length(xval)) {
                                                    yval <- c(yval, yval[length(yval)])
                                                }
                                                if (is.numeric(xval)) {
                                                    my.points(xval, yval, type = tp, lty = 2)
                                                } else {
                                                    my.points(as.factor(xval), yval, pch = 19)
                                                }
                                                sub <- gsub("quant", "%", paste(sub, colnames(qq)[j]))
                                            }
                                            my.title(main = inla.nameunfix(main), sub = paste(inla.nameunfix(sub), rep.txt))
                                        }
                                        else {
                                              my.title(main = inla.nameunfix(main), sub = paste("Posterior mean", rep.txt))
                                          }
                                    }
                                }
                            }
                        } else {
                            for (r.rep in 1:nrep) {
                                for (r.group in 1:ngroup) {
                                    if (ip %% np == 0) {
                                        close.and.new.plot(...)
                                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                                    }
                                    ip <- ip + 1

                                    rep.txt <- ""
                                    if (nrep > 1) {
                                        rep.txt <- paste(rep.txt, ", replicate", r.rep)
                                    }
                                    if (ngroup > 1) {
                                        rep.txt <- paste(rep.txt, ", group", r.group)
                                    }

                                    idx <- (r.rep - 1) * ngroup + r.group

                                    ## if the dimension is 1, the plot the marginals
                                    ##
                                    if (!is.null(x$marginals.random[[i]])) {
                                        zz <- x$marginals.random[[i]][[r.rep]]
                                        my.plot(inla.smarginal(zz),
                                            type = "l",
                                            main = paste("PostDens [", inla.nameunfix(labels.random[i]), "]", " ", rep.txt, sep = ""),
                                            xlab = inla.nameunfix(labels.random[i]), ylab = ""
                                        )
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        if (plot.hyperparameters) {
            hyper <- x$marginals.hyperpar
            if (!is.null(hyper)) {
                nhyper <- length(hyper)

                if (nhyper == 1 || single) {
                    plot.layout <- c(1, 1)
                } else if (nhyper == 2) {
                    plot.layout <- c(2, 1)
                } else {
                    plot.layout <- c(3, 3)
                }
                np <- prod(plot.layout)

                ip <- 0
                for (i in 1:nhyper) {
                    if (ip %% np == 0) {
                        close.and.new.plot(...)
                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                    }
                    ip <- ip + 1

                    hh <- hyper[[i]]
                    if (!is.null(hh)) {
                        label <- inla.nameunfix(names(hyper)[i])
                        m <- inla.smarginal(hh)
                        my.plot(m, type = "l", ylab = "", xlab = "")
                        my.title(main = paste("PostDens [", label, "]", sep = ""))

                        if (plot.prior) {
                            id <- unlist(strsplit(attr(hyper[[i]], "hyperid"), "\\|"))
                            if (length(id) > 0) {
                                xy <- (inla.get.prior.xy(
                                    section = tolower(id[2]), hyperid = id[1],
                                    all.hyper = all.hyper, range = range(m$x), intern = FALSE,
                                    debug = debug
                                ))
                                my.lines(xy, col = "blue")
                            }
                        }
                    }
                }
            }
        }

        if (plot.hyperparameters && intern) {
            hyper <- x$internal.marginals.hyperpar
            if (!is.null(hyper)) {
                nhyper <- length(hyper)

                if (nhyper == 1 || single) {
                    plot.layout <- c(1, 1)
                } else if (nhyper == 2) {
                    plot.layout <- c(2, 1)
                } else {
                    plot.layout <- c(3, 3)
                }
                np <- prod(plot.layout)

                ip <- 0
                for (i in 1:nhyper) {
                    if (ip %% np == 0) {
                        close.and.new.plot(...)
                        par(mfrow = c(plot.layout[1], plot.layout[2]))
                    }
                    ip <- ip + 1

                    hh <- hyper[[i]]
                    if (!is.null(hh)) {
                        label <- inla.nameunfix(names(hyper)[i])
                        m <- inla.smarginal(hh)
                        my.plot(m, type = "l", ylab = "", xlab = "")
                        my.title(main = paste("PostDens [", label, "]", sep = ""))

                        if (plot.prior) {
                            id <- unlist(strsplit(attr(hyper[[i]], "hyperid"), "\\|"))
                            if (length(id) > 0) {
                                xy <- (inla.get.prior.xy(
                                    section = tolower(id[2]), hyperid = id[1],
                                    all.hyper = all.hyper, range = range(m$x), intern = TRUE,
                                    debug = debug
                                ))
                                my.lines(xy, col = "blue")
                            }
                        }
                    }
                }
            }
        }

        if (plot.predictor) {
            ## linear predictor and fitted values
            n <- x$size.linear.predictor$n
            if (!is.null(n)) {
                lp <- x$summary.linear.predictor
                fv <- x$summary.fitted.values
                ## remove the 'kld' column, we do not need it
                lp <- lp[, names(lp) != "kld"]
                fv <- fv[, names(fv) != "kld"]
                A <- (x$size.linear.predictor$nrep == 2)

                if (A) {
                    nm <- dim(lp)[1] - n
                } else {
                    nm <- n
                }

                for (m in inla.ifelse(A, 1:2, 1)) {
                    if (m == 1) {
                        idx <- 1:nm
                        plot.idx <- idx
                    } else if (m == 2) {
                        idx <- 1:n + nm
                        plot.idx <- 1:n
                    } else {
                        stop("This should not happen")
                    }
                    if (A) {
                        if (m == 1) {
                            msg <- inla.ifelse(A, "(A*lin.pred)", "")
                        } else {
                            msg <- "(orig.)"
                        }
                    } else {
                        msg <- ""
                    }

                    if (!is.null(lp)) {
                        close.and.new.plot(...)
                        if (!is.null(fv)) {
                            if (single) {
                                par(mfrow = c(1, 1))
                            } else {
                                par(mfrow = c(2, 1))
                            }
                        }
                        my.plot(plot.idx, lp[idx, colnames(lp) == "mean"],
                            ylim = range(lp[idx, names(lp) != "sd"]), ylab = "", xlab = "Index", type = "l", ...
                        )
                        lq <- grep("quan", colnames(lp))
                        if (length(lq) > 0) {
                            qq <- lp[, lq]
                            dq <- dim(qq)[2]
                            sub <- paste("Posterior mean together with ")
                            for (j in 1:dq) {
                                my.points(plot.idx, qq[idx, j], type = "l", lty = 2)
                                sub <- paste(sub, colnames(qq)[j])
                            }
                            my.title(main = paste("Linear Predictor", msg), sub = inla.nameunfix(sub))
                        }
                        else {
                              my.title(main = paste("Linear Predictor ", msg, inla.nameunfix(labels.random[i])), sub = "Posterior mean")
                          }

                        if (single) {
                            close.and.new.plot()
                        }

                        if (!is.null(fv)) {
                            my.plot(plot.idx, fv[idx, colnames(fv) == "mean"],
                                ylim = range(fv[idx, names(fv) != "sd"]), ylab = "", xlab = "Index", type = "l", ...
                            )
                            lq <- grep("quan", colnames(fv))
                            if (length(lq) > 0) {
                                qq <- fv[, lq]
                                dq <- dim(qq)[2]
                                sub <- paste("Posterior mean together with ")
                                for (j in 1:dq) {
                                    my.points(plot.idx, qq[idx, j], type = "l", lty = 2)
                                    sub <- paste(sub, colnames(qq)[j])
                                }
                                my.title(main = paste("Fitted values (inv.link(lin.pred))", msg), sub = inla.nameunfix(sub))
                            }
                            else {
                                  my.title(main = paste("Fitted values (inv.link(lin.pred))", msg, inla.nameunfix(labels.random[i])))
                              }
                        }
                    }
                }
            }
        }

        if (plot.q && !is.null(x$Q)) {
            close.and.new.plot(...)
            if (single) {
                par(mfrow = c(1, 1))
            } else {
                par(mfrow = c(2, 2))
            }
            if (!is.null(x$Q$Q)) {
                my.plot(x$Q$Q, main = "The precision matrix", ...)
                nx <- x$Q$Q@size[1L]
                my.lines(c(0, nx, nx, 0, 0), c(0, 0, nx, nx, 0))
            }
            if (!is.null(x$Q$Q.reorder)) {
                my.plot(x$Q$Q.reorder, main = "The reordered precision matrix", ...)
                nx <- x$Q$Q.reorder@size[1L]
                my.lines(c(0, nx, nx, 0, 0), c(0, 0, nx, nx, 0))
            }
            if (!is.null(x$Q$L)) {
                my.plot(x$Q$L, main = "The Cholesky triangle", ...)
                nx <- x$Q$L@size[1L]
                my.lines(c(0, nx, nx, 0, 0), c(0, 0, nx, nx, 0))
            }
        }

        if (plot.cpo && !is.null(x$cpo)) {
            if (!(is.null(x$cpo$pit) || length(x$cpo$pit) == 0L) || !(is.null(x$cpo$cpo) || length(x$cpo$cpo) == 0L)) {

                ## do one plot for each likelihood
                fam.names <- x$.args$family
                fam <- sort(unique(x$misc$family))
                len <- length(x$cpo$cpo)

                for (ifam in fam) {
                    close.and.new.plot(...)
                    if (single) {
                        par(mfrow = c(1, 1))
                    } else {
                        par(mfrow = c(2, 2))
                    }

                    fam.name <- fam.names[ifam]
                    idx <- which(x$misc$family == ifam)
                    ok <- (!is.na(x$cpo$failure[idx]) & !is.na(x$cpo$pit[idx]) & !is.na(x$cpo$cpo[idx]))
                    failure <- (x$cpo$failure[idx])[ok]
                    pit <- (x$cpo$pit[idx])[ok]
                    cpo <- (x$cpo$cpo[idx])[ok]
                    fail <- (x$cpo$fail[idx])[ok]
                    ii <- 1:len
                    ii <- (ii[idx])[ok]

                    if (!(is.null(pit) || length(pit) == 0L)) {
                        ## if the observational model is discrete then do some
                        ## adjustments: define the modified pit as Prob(y < y_obs)
                        ## + 0.5*Prob(y = y_obs).
                        n.fail <- sum(failure > 0.0)
                        if (inla.model.properties(fam.name, section = "likelihood")$discrete) {
                            discrete <- TRUE
                            m <- "Modified PIT-values"
                            p <- pit - 0.5 * cpo
                        } else {
                            discrete <- FALSE
                            m <- "PIT-values"
                            p <- pit
                        }

                        pp <- rep(NA, len)
                        pp[ii] <- p
                        main.txt <- paste(m, ", nfail", n.fail, ", ", fam.name, "[", ifam, "]", sep = "")
                        my.plot(pp, main = main.txt, ylab = "Probability", xlab = "index", ylim = c(0, 1), ...)
                        if (n.fail > 0) {
                            p.fail <- p
                            p.fail[failure == 0] <- NA
                            pp[ii] <- p.fail
                            my.points(pp, pch = 20)
                        }
                        hist(p,
                            main = main.txt, xlab = "Probability",
                            n = max(20, min(round(length(pit) / 10), 100))
                        )
                    }

                    if (!(is.null(cpo) || length(cpo) == 0L)) {
                        n.fail <- sum(failure != 0.0)
                        main.txt <- paste("CPO-values", ", nfail", n.fail, ", ", fam.name, "[", ifam, "]", sep = "")
                        ccpo <- rep(NA, len)
                        ccpo[ii] <- cpo
                        if (discrete) {
                            my.plot(ccpo, main = main.txt, ylab = "Probability", xlab = "index", ylim = c(0, 1), ...)
                        } else {
                            my.plot(ccpo, main = main.txt, ylab = "Probability", xlab = "index", ...)
                        }
                        if (n.fail > 0) {
                            cpo.fail <- cpo
                            cpo.fail[failure == 0] <- NA
                            ccpo[ii] <- cpo.fail
                            my.points(ccpo, pch = 20)
                        }
                        hist(cpo,
                            main = main.txt, xlab = "Probability",
                            n = max(20L, min(round(length(pit) / 10L), 100L))
                        )
                    }
                }
            }
        }

        close.plot(...)
        return(invisible(figures))
    }

inla.extract.prior <- function(section = NULL, hyperid = NULL, all.hyper, debug = FALSE) {
    str.trunc <- function(..., max.len = 32) {
        str <- paste(..., sep = "", collapse = " ")
        str <- substr(str, 1, min(max.len, nchar(str)))
        return(str)
    }

    output <- function(..., warning = FALSE, force = FALSE) {
        msg <- paste("*** inla.extract.prior: ", ..., sep = "", collapse = " ")
        if (warning) warning(msg)
        if (debug || force) print(msg)
        return(invisible())
    }

    if (section == "fixed") {
        ## hyperid = name of fixed effect
        output("enter section [fixed]")
        output("searching for hyperid = ", hyperid)
        h <- all.hyper$fixed
        found <- FALSE
        for (i in seq_along(h)) {
            ## output("search for label = ", h[[i]]$label)
            if (inla.strcasecmp(h[[i]]$label, hyperid)) {
                found <- TRUE
                break
            }
        }
        if (found) {
            prior <- "gaussian"
            param <- c(h[[i]]$prior.mean, h[[i]]$prior.prec)
            from.theta <- function(x) x
            to.theta <- function(x) x
        } else {
            ## try section 'linear' instead
            output("enter section [linear]")
            output("searching for hyperid = ", hyperid)
            h <- all.hyper$linear
            found <- FALSE
            for (i in seq_along(h)) {
                ## output("search for label = ", h[[i]]$label)
                if (inla.strcasecmp(h[[i]]$label, hyperid)) {
                    found <- TRUE
                    break
                }
            }
            if (!found) {
                output(paste("cannot find hyperid '", hyperid,
                    "' in sections 'fixed' and 'linear'.",
                    sep = ""
                ), warning = TRUE)
                return(NA)
            }
            prior <- "gaussian"
            param <- c(h[[i]]$prior.mean, h[[i]]$prior.prec)
            from.theta <- function(x) x
            to.theta <- function(x) x
        }
    } else if (section == "predictor") {
        output("section predictor")
        h <- all.hyper$predictor
        stopifnot(length(h) == 1)
        stopifnot(inla.strcasecmp(as.character(h[[1]]$theta$hyperid), hyperid))
        prior <- h[[1]]$theta$prior
        param <- h[[1]]$theta$param
        from.theta <- h[[1]]$theta$from.theta
        to.theta <- h[[1]]$theta$to.theta
    } else if (length(grep("^inla.data[0-9]+$", section)) > 0) {
        ## likelihood
        output("request for likelihood ", section, " with hyperid ", hyperid)
        h <- all.hyper$family
        found <- FALSE
        for (idx.family in seq_along(h)) {
            if (inla.strcasecmp(section, h[[idx.family]]$hyperid)) {
                found <- TRUE
                break
            }
        }
        if (!found) {
            output("likelihood ", section, " with hyperid ", hyperid, " is not found", warning = TRUE)
            return(NA)
        } else {
            output("likelihood ", section, " with hyperid ", hyperid, " is found with idx.family=", idx.family)
        }

        ## now we need to find the theta
        found <- FALSE
        for (idx.theta in seq_along(h[[idx.family]]$hyper)) {
            if (inla.strcasecmp(hyperid, h[[idx.family]]$hyper[[idx.theta]]$hyperid)) {
                found <- TRUE
                break
            }
        }
        if (found) {
            output("likelihood ", section, " with hyperid ", hyperid, ". theta is found with idx.theta=", idx.theta)
            prior <- h[[idx.family]]$hyper[[idx.theta]]$prior
            param <- h[[idx.family]]$hyper[[idx.theta]]$param
            from.theta <- h[[idx.family]]$hyper[[idx.theta]]$from.theta
            to.theta <- h[[idx.family]]$hyper[[idx.theta]]$to.theta
        } else {
            ## look in the link-section for theta
            found <- FALSE
            for (idx.theta in seq_along(h[[idx.family]]$link$hyper)) {
                if (inla.strcasecmp(hyperid, h[[idx.family]]$link$hyper[[idx.theta]]$hyperid)) {
                    found <- TRUE
                    break
                }
            }
            if (found) {
                output("likelihood ", section, " with hyperid ", hyperid, ". theta is found with idx.theta=", idx.theta)
                prior <- h[[idx.family]]$link$hyper[[idx.theta]]$prior
                param <- h[[idx.family]]$link$hyper[[idx.theta]]$param
                from.theta <- h[[idx.family]]$link$hyper[[idx.theta]]$from.theta
                to.theta <- h[[idx.family]]$link$hyper[[idx.theta]]$to.theta
            } else {
                ## look in the mix-section for theta
                found <- FALSE
                for (idx.theta in seq_along(h[[idx.family]]$mix$hyper)) {
                    if (inla.strcasecmp(hyperid, h[[idx.family]]$mix$hyper[[idx.theta]]$hyperid)) {
                        found <- TRUE
                        break
                    }
                }
                if (found) {
                    output("likelihood ", section, " with hyperid ", hyperid, ". theta is found with idx.theta=", idx.theta)
                    prior <- h[[idx.family]]$mix$hyper[[idx.theta]]$prior
                    param <- h[[idx.family]]$mix$hyper[[idx.theta]]$param
                    from.theta <- h[[idx.family]]$mix$hyper[[idx.theta]]$from.theta
                    to.theta <- h[[idx.family]]$mix$hyper[[idx.theta]]$to.theta
                } else {
                    output("likelihood ", section, " with hyperid ", hyperid, ". theta is not found",
                        warning = TRUE
                    )
                    return(NA)
                }
            }
        }
    } else {
        ## random
        output("request for random ", section, " with hyperid ", hyperid)
        h <- all.hyper$random
        found <- FALSE
        for (idx.random in seq_along(h)) {
            if (inla.strcasecmp(section, h[[idx.random]]$hyperid)) {
                found <- TRUE
                break
            }
        }
        if (!found) {
            output("random ", section, " with hyperid ", hyperid, " is not found", warning = TRUE)
            return(NA)
        } else {
            output("random ", section, " with hyperid ", hyperid, " is found with idx.random=", idx.random)
        }

        ## now we need to find the theta
        found <- FALSE
        for (idx.theta in seq_along(h[[idx.random]]$hyper)) {
            if (inla.strcasecmp(hyperid, h[[idx.random]]$hyper[[idx.theta]]$hyperid)) {
                found <- TRUE
                break
            }
        }
        hyper <- h[[idx.random]]$hyper
        if (!found) {
            for (idx.theta in seq_along(h[[idx.random]]$group.hyper)) {
                if (inla.strcasecmp(hyperid, h[[idx.random]]$group.hyper[[idx.theta]]$hyperid)) {
                    found <- TRUE
                    break
                }
            }
            hyper <- h[[idx.random]]$group.hyper
            if (!found) {
                output("random ", section, " with hyperid ", hyperid, ". theta is not found",
                    warning = TRUE
                )
                return(NA)
            } else {
                output("random ", section, " with hyperid ", hyperid, ". theta is found with (group) idx.theta=", idx.theta)
            }
        } else {
            output("random ", section, " with hyperid ", hyperid, ". theta is found with idx.theta=", idx.theta)
        }
        prior <- hyper[[idx.theta]]$prior
        param <- hyper[[idx.theta]]$param
        from.theta <- hyper[[idx.theta]]$from.theta
        to.theta <- hyper[[idx.theta]]$to.theta
    }

    output("prior ", str.trunc(prior), " param ", str.trunc(param))
    return(list(prior = prior, param = param, from.theta = from.theta, to.theta = to.theta))
}

inla.get.prior.xy <- function(section = NULL, hyperid = NULL, all.hyper, debug = FALSE,
                              len = 10000, range, intern = FALSE) {
    str.trunc <- function(..., max.len = 32) {
        str <- paste(..., sep = "", collapse = " ")
        str <- substr(str, 1, min(max.len, nchar(str)))
        return(str)
    }

    output <- function(..., stop = FALSE, force = FALSE) {
        msg <- paste("*** inla.get.prior.xy: ", ..., sep = "", collapse = " ")
        if (stop) stop(msg)
        if (debug || force) print(msg)
        return(invisible())
    }

    map.interval <- function(x, param, deriv = 0) {
        low <- param[1]
        high <- param[2]
        ex <- exp(x)
        if (deriv == 0) {
            return(low + (high - low) * ex / (1.0 + ex))
        } else {
            return((high - low) * ex / (1.0 + ex)^2)
        }
    }

    log.jac.fun <- function(x, y) {
        ## return a function that returns func(x) = log(abs(f(x, deriv=1))),  where y = f(x)
        fun <- function(x) x
        sf <- splinefun(x, y)
        body(fun) <- log(abs(sf(x, deriv = 1)))
        return(fun)
    }

    log.jac <- function(x, y) {
        ## evaluate log.jac.fun in x
        fun <- log.jac.fun(x, y)
        return(-fun(x))
    }

    ## add priors here. the format is
    ##
    ## my.<NameOfPrior> = function(theta, param, log=FALSE)
    ##
    ## and should return the density (log=F) or log-density (log=T) for the internal parameter
    ## THETA (which is a vector), and the prior has parameters PARAM (which is the same ones as
    ## specified from within the R-interface and argument 'hyper'. the conversion to the prior
    ## density for the user-scale is done automatically.

    my.pc.gevtail <- function(theta, param, log = FALSE) {
        dist <- function(xi, deriv = 0) {
            if (deriv == 0) {
                return(xi * sqrt(2.0 / (1.0 - xi)))
            } else {
                return((2.0 - xi) * sqrt(2.0) / 2.0 / sqrt(1.0 / (1.0 - xi)) / (1.0 - xi)^2)
            }
        }

        lambda <- param[1]
        interval <- param[c(2, 3)]
        xi <- map.interval(theta, interval)
        xi.deriv <- map.interval(theta, interval, deriv = 1)

        if (interval[1] == 0.0) {
            p.low <- 0.0
        } else {
            p.low <- 1.0 - exp(-lambda * dist(interval[1]))
        }
        if (interval[2] == 1.0) {
            p.high <- 1.0
        } else {
            p.high <- 1.0 - exp(-lambda * dist(interval[2]))
        }

        d <- dist(xi)
        d.deriv <- dist(xi, deriv = 1)

        return(-log(p.high - p.low) + log(lambda) - lambda * d + log(abs(d.deriv)) +
            log(abs(xi.deriv)))
    }

    my.pc.dof <- function(theta, param, log = FALSE) {
        dof <- inla.models()$likelihood$t$hyper$theta2$from.theta(theta)
        ld <- inla.pc.ddof(dof, lambda = param[1], log = TRUE) + log.jac(dof, theta)
        return(if (log) ld else exp(ld))
    }

    my.pc.alphaw <- function(theta, param, log = FALSE) {
        ## this one comes with a scale
        sc <- log(inla.models()$likelihood$weibull$hyper$theta$from.theta(1))
        alpha <- inla.models()$likelihood$weibull$hyper$theta$from.theta(theta)
        ld <- inla.pc.dalphaw(alpha, lambda = param[1], log = TRUE) + log.jac(alpha, theta)
        return(if (log) ld else exp(ld))
    }

    my.pc.sn <- function(theta, param, log = FALSE) {
        fun <- inla.models()$link$sn$hyper$theta1$from.theta
        ## make a function that extracts 'skew.max'
        fun.skew.max <- args(fun)
        body(fun.skew.max) <- expression(skew.max)
        skew <- fun(theta)
        skew.max <- fun.skew.max()
        integral <- inla.pc.psn(skew.max, lambda = param[1]) - inla.pc.psn(-skew.max, lambda = param[1])
        ld <- inla.pc.dsn(skew, lambda = param[1], log = TRUE) - log(integral) + log.jac(skew, theta)
        return(if (log) ld else exp(ld))
    }

    my.pc.gamma <- function(theta, param, log = FALSE) {
        ## see ?inla.pc.dgamma. this is the same prior, but for theta where x=exp(theta)
        x <- exp(theta)
        ld <- inla.pc.dgamma(x, lambda = param[1], log = TRUE) + theta
        return(if (log) ld else exp(ld))
    }

    my.pc.mgamma <- function(theta, param, log = FALSE) {
        ## see ?inla.pc.dgamma. this is the same prior, but for theta where x=exp(-theta)
        return(my.pc.gamma(-theta, param, log = log))
    }

    my.pc.gammacount <- function(theta, param, log = FALSE) {
        ## see ?inla.pc.dgammacount. this is the same prior, but for theta where x=exp(theta)
        x <- exp(theta)
        ld <- inla.pc.dgammacount(x, lambda = param[1], log = TRUE) + theta
        return(if (log) ld else exp(ld))
    }

    my.pc.cor0 <- function(theta, param, log = FALSE) {
        e.theta <- exp(theta)
        rho <- 2.0 * e.theta / (1 + e.theta) - 1.0
        ld <- (inla.pc.dcor0(rho, u = param[1], alpha = param[2], log = TRUE) +
            log(2) + theta - 2.0 * log(1 + e.theta))
        return(if (log) ld else exp(ld))
    }

    my.pc.rho0 <- function(theta, param, log = FALSE) {
        return(my.pc.cor0(theta, param, log))
    }

    my.pc.cor1 <- function(theta, param, log = FALSE) {
        e.theta <- exp(theta)
        rho <- 2.0 * e.theta / (1 + e.theta) - 1.0
        ld <- (inla.pc.dcor1(rho, u = param[1], alpha = param[2], log = TRUE) +
            log(2) + theta - 2.0 * log(1 + e.theta))
        return(if (log) ld else exp(ld))
    }

    my.pc.rho1 <- function(theta, param, log = FALSE) {
        return(my.pc.cor1(theta, param, log))
    }

    my.logitbeta <- function(theta, param, log = FALSE) {
        e.theta <- exp(theta)
        p <- e.theta / (1.0 + e.theta)
        ld <- (dbeta(p, shape1 = param[1], shape2 = param[2], log = TRUE) +
            theta - 2.0 * log(1 + e.theta))
        return(if (log) ld else exp(ld))
    }

    my.betacorrelation <- function(theta, param, log = FALSE) {
        print(param)
        e.theta <- exp(theta)
        p <- e.theta / (1 + e.theta)
        ld <- dbeta(p, shape1 = param[1], shape2 = param[2], log = TRUE) +
            theta - 2.0 * log(1.0 + e.theta)
        return(if (log) ld else exp(ld))
    }

    my.pcfgnh <- function(theta, param, log = FALSE) {
        ## needed to compute the PC-prior on the fly.
        inla.require("HKprocess", stop.on.error = TRUE)

        to.theta <- inla.models()$latent$fgn$hyper$theta2$to.theta
        from.theta <- inla.models()$latent$fgn$hyper$theta2$from.theta

        logdet.FGN <- function(H, n) {
            ans <- c()
            Hseq <- H
            for (H in Hseq) {
                r <- inla.acvfFGN(H, n - 1)
                res <- HKprocess::ltzc(r, rep(0, n))
                ans <- c(ans, as.numeric(res[2]))
            }
            return(ans)
        }
        d <- function(H, n) {
            return(sqrt(-logdet.FGN(H, n)))
        }
        H.intern <- seq(-10, 19, by = 0.1)
        dist <- d(from.theta(H.intern), 100)
        log.d <- log(dist / max(dist))
        res <- cbind(H.intern = H.intern, log.d = log.d)
        log.d.spline <- splinefun(res[, 1], res[, 2])
        d.spline <- function(H.intern, ...) {
            return(exp(log.d.spline(H.intern, ...)))
        }
        d.spline.deriv <- function(H.intern, ...) {
            return(d.spline(H.intern, ...) * log.d.spline(H.intern, deriv = 1L))
        }

        U <- param[1]
        alpha <- param[2]
        lambda <- -log(alpha) / d.spline(to.theta(U))
        ld <- dexp(d.spline(theta), rate = lambda, log = TRUE) + log(abs(d.spline.deriv(theta)))
        ## print(cbind(theta = theta,  ld = ld))

        return(if (log) ld else exp(ld))
    }

    my.pc.prec <- function(theta, param, log = FALSE) {
        ## sigma = exp(-theta/2)
        ## pi(theta) = lambda * exp(-lambda*sigma) * sigma/2
        ## param = c(U, alpha)
        lambda <- -log(param[2]) / param[1]
        sigma <- exp(-theta / 2.0)
        ld <- log(lambda) + (-theta / 2.0) - log(2.0) - lambda * sigma
        return(if (log) ld else exp(ld))
    }

    my.loggamma <- function(theta, param, log = FALSE) {
        ## the log-of-a-gamma(a, b) density
        ld <- dgamma(exp(theta), shape = param[1], rate = param[2], log = TRUE) + theta
        return(if (log) ld else exp(ld))
    }

    my.gaussian <- function(theta, param, log = FALSE) {
        if (param[2] == 0) {
            ld <- 0.0
        } else {
            ld <- dnorm(theta, mean = param[1], sd = sqrt(1 / param[2]), log = TRUE)
        }
        return(if (log) ld else exp(ld))
    }

    my.normal <- function(theta, param, log = FALSE) {
        return(my.gaussian(theta, param, log))
    }

    my.table <- function(theta, param, log = FALSE) {
        fun <- splinefun(param[, 1], param[, 2])
        ld <- fun(theta)
        return(if (log) ld else exp(ld))
    }

    my.linksnintercept <- function(theta, param, log = FALSE) {
        mu.fun <- function(theta) qnorm(1.0 / (1 + exp(-theta)))
        theta.fun <- function(mu) {
            xx <- pnorm(mu)
            return(log(xx / (1.0 - xx)))
        }
        step <- 1.0e-4
        mu <- mu.fun(theta)
        deriv <- 1.0 / ((theta.fun(mu + step) - theta.fun(mu - step)) / (2.0 * step))
        ld <- log(abs(deriv)) + my.gaussian(mu, param, log = TRUE)
        return(if (log) ld else exp(ld))
    }

    ## end of prior functions

    stopifnot(!missing(range))
    prior <- inla.extract.prior(section, hyperid, all.hyper, debug)

    if (length(prior) == 1 && is.na(prior)) {
        return(list(x = NA, y = NA))
    }

    if (length(grep("table:", prior$prior)) > 0) {
        tab <- substr(prior$prior, nchar("table:") + 1, nchar(prior$prior))
        xy <- as.numeric(unlist(strsplit(tab, "[ \t\n\r]+")))
        xy <- xy[!is.na(xy)]
        nxy <- length(xy) %/% 2L
        xx <- xy[1:nxy]
        yy <- xy[1:nxy + nxy]
        xy <- cbind(xx, yy)
        prior$param <- xy
        prior$prior <- "table"
    }
    if (length(grep("expression:", prior$prior)) > 0) {
        ## not available yet.
        return(list(x = NA, y = NA))
    }

    myp <- paste("my.", prior$prior, sep = "")
    if (!exists(myp) || !is.function(eval(parse(text = myp)))) {
        output("internal prior-function not found: ", myp, ", NEEDS TO BE IMPLEMENTED", force = TRUE)
        return(list(x = NA, y = NA))
    }
    output("prior: ", str.trunc(prior$prior), " param: ", str.trunc(prior$param))
    if (intern) {
        ## use a linear scale. 'x' is in the linear scale
        x <- seq(range[1], range[2], len = len)
        y <- do.call(myp, list(theta = x, param = prior$param))
    } else {
        ## 'x' is in the user-scale.

        ## this is a special case, need to extract the interval and use that as the correct
        ## argument
        if (prior$prior == "pc.gevtail") {
            interval <- prior$param[c(2, 3)]
            range.theta <- prior$to.theta(range, interval = interval)
            theta <- seq(range.theta[1], range.theta[2], len = len)
            x <- prior$from.theta(theta, interval = interval)
        } else {
            range.theta <- prior$to.theta(range)
            theta <- seq(range.theta[1], range.theta[2], len = len)
            x <- prior$from.theta(theta)
        }
        ld <- do.call(myp, args = list(theta = theta, param = prior$param, log = TRUE))
        fun <- splinefun(x, theta)
        y <- exp(ld + log(abs(fun(x, deriv = 1))))
    }
    return(list(x = x, y = y))
}

`inla.all.hyper.postprocess` <- function(all.hyper) {
    ## postprocess all.hyper, by converting and replacing prior = 'mvnorm' into its p
    ## marginals. this is for the spde-models

    len.n <- function(param, max.dim = 10000) {
        len <- function(n) {
            return(n + n^2)
        }

        len.target <- length(param)
        for (n in 1:max.dim) {
            if (len(n) == len.target) {
                return(n)
            }
        }
        stop(paste("length(param) is wrong:", len.target))
    }

    get.mvnorm.marginals <- function(param) {
        n <- len.n(param)
        mu <- param[1:n]
        Q <- matrix(param[-(1:n)], n, n)
        Sigma <- solve((Q + t(Q)) / 2.)
        return(list(mean = mu, prec = 1 / diag(Sigma)))
    }

    for (i in seq_along(all.hyper$random)) {
        for (j in seq_along(all.hyper$random[[i]]$hyper)) {
            if (all.hyper$random[[i]]$hyper[[j]]$prior == "mvnorm") {
                ## replace this one, and the p-following ones, with its marginals
                m <- get.mvnorm.marginals(all.hyper$random[[i]]$hyper[[j]]$param)
                for (k in 1:length(m$mean)) {
                    kk <- j + k - 1
                    all.hyper$random[[i]]$hyper[[kk]]$prior <- "normal"
                    all.hyper$random[[i]]$hyper[[kk]]$param <- c(m$mean[k], m$prec[k])
                }
            }
        }
    }

    return(all.hyper)
}

## nothing to export

`inla.interpret.formula` <- function(
                                     gf,
                                     debug = FALSE,
                                     data.same.len = NULL,
                                     data = NULL,
                                     data.model = NULL,
                                     parent.frame = NULL) {
    ## use a copy if data.model is !NULL, as we have to assign an
    ## entry to it. otherwise, just use the
    if (!is.null(data.model)) {
        ## make a copy of the environment
        p.env <- new.env(hash = TRUE, parent = environment(gf))
        ## then add the entries in data.model to that environment
        for (nm in names(data.model)) {
            idx <- which(names(data.model) == nm)
            assign(nm, data.model[[idx]], envir = p.env)
        }
    } else {
        ## otherwise, just use (for read-only) this environment
        if (missing(parent.frame)) {
            p.env <- environment(gf)
        } else {
            p.env <- parent.frame
        }
    }

    ## argument data is here only used for infereing '.'
    tf <- terms.formula(gf, specials = c("f"), data = NULL)
    terms <- attr(tf, "term.labels")
    nt <- length(terms)

    if (attr(tf, "response") > 0) {
        ## fixf formula with ONLY fixed effects.  randf formula with
        ## ONLY random effect.  weightf formula where are the names of
        ## the (possible) weigths for the covariates
        response <- as.character(attr(tf, "variables")[2])
        fixf <- randf <- weightf <- paste(response, "~", sep = "")
    } else {
        stop("\n\tA response variable has to be present")
    }

    if (length(attr(tf, "offset")) > 0) {
        i <- grep("^offset[(]", attr(tf, "variables"))
        offset <- as.character(attr(tf, "variables")[i])
        if (debug) {
              cat("found offset\n")
          }
    } else {
        offset <- NULL
        if (debug) {
              cat("no offset\n")
          }
    }

    rt <- attr(tf, "specials")$f
    vtab <- attr(tf, "factors")
    if (length(rt) > 0) {
        for (i in 1:length(rt)) {
            ind <- (1:nt)[as.logical(vtab[rt[i], ])]
            rt[i] <- ind
        }
    }

    k <- ks <- kp <- 1
    len.rt <- length(rt)
    random.spec <- list()
    if (nt > 0) {
        for (i in 1:nt) {
            if (k <= len.rt && ((ks <= len.rt && rt[ks] == i))) {
                st <- eval(parse(text = gsub("^f\\(", "INLA::f(", terms[i])), envir = data, enclos = p.env)
                random.spec[[k]] <- st
                if (ks <= len.rt && rt[ks] == i) {
                    ks <- ks + 1
                } else {
                    kt <- kt + 1
                }
                k <- k + 1
            } else {
                if (kp > 1) {
                    fixf <- paste(fixf, " + ", terms[i], sep = "")
                } else {
                    fixf <- paste(fixf, terms[i], sep = "")
                }
                kp <- kp + 1
            }
        }
    }

    ## if -1 the intercept is not included
    if (attr(tf, "intercept") == 0) {
        intercept <- FALSE
    } else {
        intercept <- TRUE
    }

    if (length(random.spec) > 0) {
        ## check that we are not using the same covariate more than
        ## one time.
        all.terms <- c()
        for (i in 1:length(random.spec)) {
            all.terms <- c(all.terms, random.spec[[i]]$term)
        }
        for (i in 1:length(random.spec)) {
            if (sum(random.spec[[i]]$term == all.terms) > 1) {
                ## duplicate names!
                stop(inla.paste(c(
                    "\n",
                    "The covariate `", random.spec[[i]]$term, "' are used in the following f()-terms: ",
                    inla.paste(which(random.spec[[i]]$term == all.terms), sep = ", "),
                    "\n",
                    "Only one is allowed since the naming convention of the f()-terms depends on unique names of the covariates",
                    "\n\n",
                    "You can resolve this as follows:",
                    "\n",
                    "Not allowed: formula = y ~ f(x, <etc>) + f(x, <etc>) + <etc>",
                    "\n",
                    "Fix: data$xx = data$x; formula = y ~ f(x, <etc>) + f(xx, <etc>) + <etc>",
                    "\n\n"
                ), sep = ""))
            }
        }
    }

    ## number of covariate which have weights attached!
    n.weights <- 0
    if (length(random.spec) > 0) {
          for (i in 1:length(random.spec)) {
              ff1 <- paste(random.spec[[i]]$term, collapse = "+")

              if (!is.null(random.spec[[i]]$weights)) {
                  ww1 <- paste(random.spec[[i]]$weights, collapse = "+")
                  n.weights <- n.weights + 1
              }
              if (i == 1) {
                  randf <- paste(randf, ff1)
                  if (!is.null(random.spec[[i]]$weights)) {
                        weightf <- paste(weightf, ww1)
                    }
              } else {
                  randf <- paste(randf, "+", ff1)
                  if (!is.null(random.spec[[i]]$weights)) {
                        weightf <- paste(weightf, "+", ww1)
                    }
              }
          }
      }

    if (len.rt > 0) {
        randf <- as.formula(randf, p.env)
    } else {
        randf <- NULL
    }

    if (intercept) {
        fixf <- paste(fixf, " +1", sep = "")
        nt <- nt + 1
    } else {
        fixf <- paste(fixf, " -1", sep = "")
        nt <- nt + 1
    }

    if ((nt - len.rt) > 0) {
        fixf <- as.formula(fixf, p.env)
    } else {
        fixf <- NULL
    }

    if (n.weights > 0) {
        weightf <- as.formula(weightf, p.env)
    } else {
        weightf <- NULL
    }

    ret <- list(
        randf = randf,
        random.spec = random.spec, n.random = len.rt,
        fixf = fixf, n.fix = (nt - len.rt),
        weightf = weightf, n.weights = n.weights,
        offset = offset, response = response
    )

    return(ret)
}

## Export: inla.pc.rprec inla.pc.dprec inla.pc.qprec inla.pc.pprec

## ! \name{pc.prec}
## ! \alias{inla.pc.prec}
## ! \alias{pc.prec}
## ! \alias{pc.rprec}
## ! \alias{inla.pc.rprec}
## ! \alias{pc.dprec}
## ! \alias{inla.pc.dprec}
## ! \alias{pc.pprec}
## ! \alias{inla.pc.pprec}
## ! \alias{pc.qprec}
## ! \alias{inla.pc.qprec}
## !
## ! \title{Utility functions for the PC prior for the precision}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the precision
## !              in the Gaussian distribution.}
## ! \usage{
## ! inla.pc.rprec(n, u, alpha, lambda)
## ! inla.pc.dprec(prec, u, alpha, lambda, log = FALSE)
## ! inla.pc.qprec(p, u, alpha, lambda)
## ! inla.pc.pprec(q, u, alpha, lambda)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{u}{The upper limit (see Details)}
## !   \item{alpha}{The probability going above the upper limit (see Details)}
## !   \item{lambda}{The rate parameter (see Details)}
## !   \item{prec}{Vector of precisions}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## !     The statement \code{Prob(1/sqrt(prec) > u) = alpha} is used to
## !     determine \code{lambda} unless \code{lambda} is given.
## !     Either \code{lambda}  must be given,  or
## !     \code{u} AND \code{alpha}.
## ! }
## !\value{%%
## !  \code{inla.pc.dprec} gives the density,
## !  \code{inla.pc.pprec} gives the distribution function,
## !  \code{inla.pc.qprec} gives the quantile function, and
## !  \code{inla.pc.rprec} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.prec")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! prec = inla.pc.rprec(100,  lambda = 1)
## ! d = inla.pc.dprec(prec, lambda = 1)
## ! prec = inla.pc.qprec(0.5, u = 1, alpha=0.01)
## ! inla.pc.pprec(prec, u = 1, alpha=0.01)
## ! }

inla.pc.prec.lambda <- function(u, alpha, lambda) {
    if (missing(lambda)) {
        stopifnot(!missing(u) && !missing(alpha))
        lambda <- -log(alpha) / u
    }
    return(lambda)
}

inla.pc.rprec <- function(n, u, alpha, lambda) {
    lambda <- inla.pc.prec.lambda(u, alpha, lambda)
    s <- rexp(n, rate = lambda)
    prec <- 1 / s^2
    return(prec)
}

inla.pc.dprec <- function(prec, u, alpha, lambda, log = FALSE) {
    lambda <- inla.pc.prec.lambda(u, alpha, lambda)
    s <- 1 / sqrt(prec)
    d <- dexp(s, rate = lambda, log = log)
    log.jac <- -log(2) - 3 / 2 * log(prec)
    if (log) {
        return(d + log.jac)
    } else {
        return(d * exp(log.jac))
    }
}

inla.pc.qprec <- function(p, u, alpha, lambda) {
    lambda <- inla.pc.prec.lambda(u, alpha, lambda)
    q <- qexp(1 - p, rate = lambda)
    qprec <- 1 / q^2
    return(qprec)
}

inla.pc.pprec <- function(q, u, alpha, lambda) {
    lambda <- inla.pc.prec.lambda(u, alpha, lambda)
    p <- pexp(1 / sqrt(q), rate = lambda)
    return(1 - p)
}

## Export: inla.doc

## !\name{inla.doc}
## !\alias{inla.doc}
## !\alias{inla.doc}
## !\alias{inla.inla.doc}
## !\title{View documentation}
## !\description{View documentation of latent, prior and likelihood models.}
## !\usage{
## !inla.doc(what, section, verbose=FALSE)
## !}
## !\arguments{
## !\item{what}{What to view documentation about;
## !            name of latent model,  name of prior,  etc. (A regular expression.)}
## !\item{section}{An optional section, like \code{names(inla.models())},
## !               to look for the documentation. If missing, all sections are used.}
## !\item{verbose}{Logical If \code{TRUE} then run in verbose mode}
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !\seealso{\code{www.r-inla.org}}
## !\examples{
## !\dontrun{inla.doc("rw2")}
## !\dontrun{inla.doc("gaussian", section = "prior")}
## !}

`inla.doc` <- function(what, section, verbose = FALSE) {
    what <- inla.trim.family(what)
    sections <- names(inla.models())
    m <- inla.models()

    for (sec in 1:length(m)) {
        if (missing(section) || sections[sec] == section) {
            if (verbose) {
                cat(paste("Search in section:", sections[sec], "\n"))
                cat(paste("\t", "Names in section:", names(m[[sec]]), "\n"))
            }
            x <- inla.trim.family(names(m[[sec]]))
            if (length(x) > 0L) {
                idx <- grep(what, x)
                if (length(idx) > 0L) {
                    for (i in idx) {
                        pdf <- m[[sec]][[i]]$pdf
                        if (!is.null(pdf) && !is.na(pdf)) {
                            filename <- paste("documentation/", sections[[sec]], "/", pdf, ".pdf", sep = "")
                            filename.full <- system.file(filename, package = "INLA")
                            if (verbose) {
                                cat("\n")
                                cat(paste("\t\t", "Found name:", names(m[[sec]])[i], "\n"))
                                cat(paste("\t\t", " ", "pdf           : ", pdf, ".pdf", sep = "", "\n"))
                                cat(paste("\t\t", " ", "filename      : ", filename, sep = "", "\n"))
                                cat(paste("\t\t", " ", "filename.full : ", filename.full, sep = "", "\n"))
                            }
                            if (file.exists(filename.full)) {
                                RShowDoc(
                                    paste("documentation/", sections[[sec]], "/", pdf, sep = ""),
                                    "pdf", "INLA"
                                )
                            }
                        }
                    }
                }
            }
        } else {
            if (verbose) {
                cat(paste("Skip section:", sections[sec], "\n"))
            }
        }
    }
}

## Export: inla.set.control.lincomb.default
## Export: inla.set.control.update.default
## Export: inla.set.control.group.default
## Export: inla.set.control.mix.default
## Export: inla.set.control.link.default
## Export: inla.set.control.expert.default
## Export: inla.set.control.compute.default
## Export: inla.set.control.family.default
## Export: inla.set.control.fixed.default
## Export: inla.set.control.inla.default
## Export: inla.set.control.predictor.default
## Export: inla.set.control.mode.default
## Export: inla.set.control.hazard.default
## Export: inla.set.control.bgev.default
## Export: inla.set.control.pardiso.default

## Export: control.lincomb
## Export: control.update
## Export: control.group
## Export: control.mix
## Export: control.link
## Export: control.expert
## Export: control.compute
## Export: control.family
## Export: control.fixed
## Export: control.inla
## Export: control.vb
## Export: control.predictor
## Export: control.mode
## Export: control.hazard
## Export: control.bgev
## Export: control.pardiso


### Defines default arguments

`inla.set.control.update.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.update
        list(
            ## :ARGUMENT: result Update the joint posterior for the hyperparameters from result
            result = NULL
        )

        ## :SEEALSO: inla
    }

`inla.set.control.lincomb.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.lincomb
        list(
            ## :ARGUMENT: verbose Use verbose mode for linear combinations if verbose model is set globally. (Default TRUE)
            verbose = FALSE
        )

        ## :SEEALSO: inla
    }

`inla.set.control.group.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.group
        list(
            ## :ARGUMENT: model Group model (one of 'exchangable', 'exchangablepos', 'ar1',  'ar', 'rw1', 'rw2', 'besag', or 'iid')
            model = "exchangeable",

            ## :ARGUMENT: order Defines the \code{order} of the model: for model \code{ar} this defines the order p, in AR(p). Not used for other models at the time being.
            order = NULL,

            ## :ARGUMENT: cyclic Make the group model cyclic? (Only applies to models 'ar1',  'rw1' and 'rw2')
            cyclic = FALSE,

            ## :ARGUMENT: graph The graph spesification (Only applies to model 'besag')
            graph = NULL,

            ## :ARGUMENT: scale.model Scale the intrinsic model (RW1, RW2, BESAG) so the generalized variance is 1. (Default \code{TRUE})
            scale.model = TRUE,

            ## :ARGUMENT: adjust.for.con.comp Adjust for connected components when \code{scale.model=TRUE}? (default \code{TRUE})
            adjust.for.con.comp = TRUE,

            ## :ARGUMENT: hyper Definition of the hyperparameter(s)
            hyper = NULL,

            ## :ARGUMENT: initial (OBSOLETE!) The initial value for the group correlation or precision in the internal scale.
            initial = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) A boolean variable if the group correction or precision is assumed to be fixed or random.
            fixed = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The name of the prior distribution for the group correlation or precision in the internal scale
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) Prior parameters
            param = NULL
        )

        ## :SEEALSO: inla
    }


`inla.set.control.mix.default` <-
    function(...) {
        ## :EXTRA: The \code{control.mix} -list is set within the corresponding \code{control.family}-list a the mixture of the likelihood is likelihood spesific. (This option is EXPERIMENTAL.)
        ## :NAME: control.mix
        list(
            ## :ARGUMENT: model The model for the random effect. Currently, only \code{model='gaussian'} is implemented
            model = NULL,

            ## :ARGUMENT: hyper Definition of the hyperparameter(s) for the random effect model chosen
            hyper = NULL,

            ## :ARGUMENT: initial (OBSOLETE!) The initial value(s) for the hyperparameter(s)
            initial = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) A boolean variable if hyperparmater(s) is/are fixed or random
            fixed = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The name of the prior distribution(s) for the hyperparmater(s)
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) The parameters for the prior distribution(s) for the hyperparmater(s)
            param = NULL,

            ## :ARGUMENT: npoints Number of points used to do the numerical integration (default 101)
            npoints = 101,

            ## :ARGUMENT: integrator The integration scheme to use (\code{default}, \code{quadrature}, \code{simpson})
            integrator = "default"
        )

        ## :SEEALSO: inla
    }

`inla.set.control.link.default` <-
    function(...) {
        ## :EXTRA: The \code{control.link}-list is set within the corresponding \code{control.family}-list as the link is likelihood-familiy spesific.
        ## :NAME: control.link
        list(
            ## :ARGUMENT: model The name of the link function/model
            model = "default",

            ## :ARGUMENT: order The \code{order} of the link function, where the interpretation of \code{order} is model-dependent.
            order = NULL,

            ## :ARGUMENT: variant The \code{variant} of the link function, where the interpretation of \code{variant} is model-dependent.
            variant = NULL,

            ## :ARGUMENT: hyper Definition of the hyperparameter(s) for the link model chosen
            hyper = NULL,

            ## :ARGUMENT: quantile The quantile for quantile link function
            quantile = NULL,

            ## :ARGUMENT: a The parameter \code{a} in the LOGa link
            a = 1.0,

            ## :ARGUMENT: initial (OBSOLETE!) The initial value(s) for the hyperparameter(s)
            initial = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) A boolean variable if hyperparmater(s) is/are fixed or random
            fixed = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The name of the prior distribution(s) for the hyperparmater(s)
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) The parameters for the prior distribution(s) for the hyperparmater(s)
            param = NULL
        )

        ## :SEEALSO: inla
    }


`inla.set.f.default` <-
    function(...) {
        list(diagonal = 1.0E-4)
    }


`inla.set.control.expert.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.expert
        list(
            ## :ARGUMENT: cpo.manual A boolean variable to decide if the inla-program is to be runned in a manual-cpo-mode. (EXPERT OPTION: DO NOT USE)
            cpo.manual = FALSE,

            ## :ARGUMENT: cpo.idx  The index/indices of the data point(s) to remove. (EXPERT OPTION: DO NOT USE)
            cpo.idx = -1,

            ## :ARGUMENT: disable.gaussian.check Disable the check for fast computations with a Gaussian likelihood and identity link (default \code{FALSE})
            disable.gaussian.check = FALSE,

            ## :ARGUMENT: jp An object of class \code{inla.jp} defining a joint prior
            jp = NULL
        )

        ## :SEEALSO: inla
    }

`inla.set.control.compute.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.compute
        list(
            ## :ARGUMENT: openmp.strategy The computational strategy to use: 'small', 'medium', 'large', 'huge', 'default' and 'pardiso'.
            openmp.strategy = "default",

            ## :ARGUMENT: hyperpar A boolean variable if the marginal for the hyperparameters should be computed. Default TRUE.
            hyperpar = TRUE,

            ## :ARGUMENT: return.marginals A boolean variable if the marginals for the latent field should be returned (although it is computed). Default TRUE
            return.marginals = TRUE,

            ## :ARGUMENT: return.marginals.predictor A boolean variable if the marginals for the linear predictor should be returned (although it is computed). Default FALSE
            return.marginals.predictor = FALSE,

            ## :ARGUMENT: dic A boolean variable if the DIC-value should be computed. Default FALSE.
            dic = FALSE,

            ## :ARGUMENT: mlik A boolean variable if the marginal likelihood should be computed. Default \code{TRUE}.
            mlik = TRUE,

            ## :ARGUMENT: cpo A boolean variable if the cross-validated predictive measures (cpo, pit) should be computed (default \code{FALSE})
            cpo = FALSE,

            ## :ARGUMENT: po A boolean variable if the predictive ordinate should be computed (default \code{FALSE})
            po = FALSE,

            ## :ARGUMENT: waic A boolean variable if the Watanabe-Akaike information criteria should be computed (default \code{FALSE})
            waic = FALSE,

            ## :ARGUMENT: q A boolean variable if binary images of the precision matrix, the reordered precision matrix and the Cholesky triangle should be generated. (Default FALSE.)
            q = FALSE,

            ## :ARGUMENT: config A boolean variable if the internal GMRF approximations be stored. (Default FALSE. EXPERIMENTAL)
            config = FALSE,

            ## :ARGUMENT: smtp The sparse-matrix solver, one of 'default', 'taucs', 'band' or 'pardiso' (default \code{inla.getoption("smtp")}). \code{smtp='pardiso'} implies \code{openmp.strategy='pardiso'}.
            smtp = NULL,

            ## :ARGUMENT: graph A boolean variable if the graph itself should be returned. (Default FALSE.)
            graph = FALSE
        )

        ## :SEEALSO: inla
    }

`inla.set.control.pardiso.default` <-
    function(...) {
        ## :EXTRA: Extra options controlling the PARDISO library
        ## :NAME: control.pardiso.default
        list(
            ## :ARGUMENT: verbose Show detailed output (default FALSE)
            verbose = FALSE,

            ## :ARGUMENT: debug Show internal debug output (default FALSE)
            debug = FALSE,

            ## :ARGUMENT: parallel.reordering Do reordering in parallel (default TRUE)
            parallel.reordering = TRUE,

            ## :ARGUMENT: nrhs Number of right-hand sides to solve for in parallel (default depends on the argument \code{num.threads} and \code{-1} will not change it)
            nrhs = -1
        )

        ## :SEEALSO: inla
    }

`inla.set.control.bgev.default` <-
    function(...) {
        ## :EXTRA: The \code{control.bgev}-list is set within the corresponding \code{control.family}-list as control parameters to the \code{family="bgev"}
        ## :NAME: control.bgev.default
        list(
            ## :ARGUMENT: q.location The quantile level for the location parameter
            q.location = 0.5,

            ## :ARGUMENT: q.spread The quantile level for the spread parameter (must be < 0.5)
            q.spread = 0.25,

            ## :ARGUMENT: q.mix The lower and upper quantile level for the mixing function
            q.mix = c(0.10, 0.20),

            ## :ARGUMENT: beta.ab The parameters a and b in the Beta mixing function
            beta.ab = 5L
        )

        ## :SEEALSO: inla
    }

`inla.set.control.family.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.family
        list(
            ## :ARGUMENT: dummy A dummy argument that can be used as a workaround
            dummy = 0,

            ## :ARGUMENT: hyper Definition of the hyperparameters
            hyper = NULL,

            ## :ARGUMENT: initial (OBSOLETE!) Initial value for the hyperparameter(s) of the likelihood in the internal scale.
            initial = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The name of the prior distribution(s) for othe hyperparameter(s).
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) The parameters for the prior distribution
            param = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) Boolean variable(s) to say if the hyperparameter(s) is fixed or random.
            fixed = NULL,

            ## :ARGUMENT: link (OBSOLETE! Use \code{control.link=list(model=)} instead.) The link function to use.
            link = "default",

            ## :ARGUMENT: sn.shape.max Maximum value for the shape-parameter for Skew Normal observations (default 5.0)
            sn.shape.max = 5.0,

            ## :ARGUMENT: gev.scale.xi (Expert option, do not use unless you know what you are doing.) The internal scaling of the shape-parameter for the GEV distribution. (default 0.1)
            gev.scale.xi = 0.1,

            ## :ARGUMENT: control.bgev  See \code{?control.bgev}
            control.bgev = NULL,

            ## :ARGUMENT: cenpoisson.I The censoring interval for the censored Poisson
            cenpoisson.I = c(-1L, -1L),

            ## :ARGUMENT: beta.censor.value The censor value for the Beta-likelihood \code{(0 <= beta.censor.value < 1/2)}
            beta.censor.value = 0.0,

            ## :ARGUMENT: variant This variable is used to give options for various variants of the likelihood,  like chosing different parameterisations for example. See the relevant likelihood documentations for options (does only apply to some likelihoods).
            variant = 0L,

            ## :ARGUMENT: control.mix See \code{?control.mix}
            control.mix = NULL,

            ## :ARGUMENT: control.link See \code{?control.link}
            control.link = NULL
        )

        ## :SEEALSO: inla
    }

`inla.set.control.fixed.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.fixed
        list(
            ## :ARGUMENT: cdf  A list of values to compute the CDF for, for all fixed effects
            cdf = NULL,

            ## :ARGUMENT: quantiles  A list of quantiles to compute for all fixed effects
            quantiles = NULL,

            ## :ARGUMENT: expand.factor.strategy The strategy used to expand factors into fixed effects based on their levels. The default strategy is us use the \code{model.matrix}-function for which NA's are not allowed (\code{expand.factor.strategy="model.matrix"}) and levels are possible removed. The alternative option (\code{expand.factor.strategy="inla"}) use an \code{inla}-spesific expansion which expand a factor into one fixed effects for each level, do allow for NA's and all levels are present in the model. In this case, factors MUST BE factors in the data.frame/list and NOT added as \code{.+factor(x1)+.} in the formula only.
            expand.factor.strategy = "model.matrix",

            ## :ARGUMENT: mean Prior mean for all fixed effects except the intercept. Alternatively, a named list with specific means where name=default applies to unmatched names. For example \code{control.fixed=list(mean=list(a=1, b=2, default=0))} assign 'mean=1' to fixed effect 'a' , 'mean=2' to effect 'b' and 'mean=0' to all others. (default 0.0)
            mean = 0.0,

            ## :ARGUMENT: mean.intercept Prior mean for the intercept (default 0.0)
            mean.intercept = 0.0,

            ## :ARGUMENT: prec  Default precision for all fixed effects except the intercept. Alternatively, a named list with specific means where name=default applies to unmatched names.  For example \code{control.fixed=list(prec=list(a=1, b=2, default=0.01))} assign 'prec=1' to fixed effect 'a' , 'prec=2' to effect 'b' and 'prec=0.01' to all others. (default 0.001)
            prec = 0.001,

            ## :ARGUMENT: prec.intercept  Default precision the intercept (default 0.0)
            prec.intercept = 0.0,

            ## :ARGUMENT: compute Compute marginals for the fixed effects ? (default TRUE)
            compute = TRUE,

            ## :ARGUMENT: correlation.matrix Compute the posterior correlation matrix for all fixed effects? (default FALSE) OOPS: This option will set up appropriate linear combinations and the results are shown as the posterior correlation matrix of the linear combinations. This option will imply \code{control.inla=list(lincomb.derived.correlation.matrix=TRUE)}.

            correlation.matrix = FALSE,

            ## :ARGUMENT: remove.names A vector of names of expanded fixed effects to remove from the model-matrix. This is an expert option, and should only be used if you know what you are doing.
            remove.names = NULL
        )
        ## :SEEALSO: inla
    }

`inla.set.control.inla.default` <-
    function(...) {
        family <- "gaussian"
        xx <- list(...)[1]
        if (!is.null(xx$family)) {
            family <- xx$family
        }

        ## :EXTRA:
        ## :NAME: control.inla
        ans <- list(
            ## :ARGUMENT: strategy  Character The strategy to use for the approximations; one of 'auto' (default), 'gaussian', 'simplified.laplace', 'laplace' or 'adaptive'
            strategy = "auto",

            ## :ARGUMENT: int.strategy  Character The integration strategy to use; one of 'auto' (default),  'ccd', 'grid', 'eb' (empirical bayes),  'user' or 'user.std'
            int.strategy = "auto",

            ## :ARGUMENT: int.design  Matrix Matrix of user-defined integration points and weights. Each row consists theta values and the integration weight. (EXPERIMENTAL!)
            int.design = NULL,

            ## :ARGUMENT: interpolator  Character The interpolator used to compute the marginals for the hyperparameters. One of 'auto', 'nearest', 'quadratic', 'weighted.distance', 'ccd', 'ccdintegrate', 'gridsum', 'gaussian'. Default is 'auto'.
            interpolator = "auto",

            ## :ARGUMENT: fast Logical If TRUE, then replace conditional modes in the Laplace approximation with conditional expectation (default TRUE)
            fast = TRUE,

            ## :ARGUMENT: linear.correction Logical Default TRUE for the 'strategy = laplace' option.
            linear.correction = NULL,

            ## :ARGUMENT: h Numerical The step-length for the gradient calculations for the hyperparameters. Default 0.005.
            h = 0.005,

            ## :ARGUMENT: dz Numerical The step-length in the standarised scale for the integration of the hyperparameters. Default 0.75.
            dz = 0.75,

            ## :ARGUMENT: diff.logdens Numerical The difference of the log.density for the hyperpameters to stop numerical integration using int.strategy='grid'. Default 6.
            diff.logdens = 6,

            ## :ARGUMENT: print.joint.hyper Logical If TRUE, the store also the joint distribution of the hyperparameters (without any costs). Default TRUE.
            print.joint.hyper = TRUE,

            ## :ARGUMENT: force.diagonal Logical If TRUE, then force the Hessian to be diagonal. (Default \code{FALSE})
            force.diagonal = FALSE,

            ## :ARGUMENT: skip.configurations Logical Skip configurations if the values at the main axis are to small. (Default \code{TRUE})
            skip.configurations = TRUE,

            ## :ARGUMENT: mode.known Logical If TRUE then no optimisation is done. (Default FALSE.)
            mode.known = FALSE,

            ## :ARGUMENT: adjust.weights Logical If TRUE then just more accurate integration weights. (Default TRUE.)
            adjust.weights = TRUE,

            ## :ARGUMENT: tolerance Numerical The tolerance for the optimisation of the hyperparameters. If set, this is the default value for for 'tolerance.f/2',  'tolerance.g' and  'tolerance.x'; see below.
            tolerance = 0.005,

            ## :ARGUMENT: tolerance.f Numerical The tolerance for the absolute change in the log posterior in the optimisation of the hyperparameters.
            tolerance.f = NULL,

            ## :ARGUMENT: tolerance.g Numerical The tolerance for the absolute change in the gradient of the log posterior in the optimisation of the hyperparameters.
            tolerance.g = NULL,

            ## :ARGUMENT: tolerance.x Numerical The tolerance for the change in the hyperparameters (root-mean-square) in the optimisation of the hyperparameters.
            tolerance.x = NULL,

            ## :ARGUMENT: tolerance.step Numerical The tolerance for the change in root-mean_squre in the inner Newton-like optimisation of the latent field.
            tolerance.step = 0.0005,

            ## :ARGUMENT: restart Numerical To improve the optimisation, the optimiser is restarted at the found optimum 'restart' number of times.
            restart = 0L,

            ## :ARGUMENT: optimiser Character The optimiser to use; one of 'gsl' or 'default'.
            optimiser = "default",

            ## :ARGUMENT: verbose Logical Run in verbose mode? (Default FALSE)
            verbose = NULL,

            ## :ARGUMENT: reordering Character Type of reordering to use. (EXPERT OPTION; one of "AUTO", "DEFAULT", "IDENTITY", "REVERSEIDENTITY",  "BAND", "METIS", "GENMMD", "AMD", "MD", "MMD", "AMDBAR", "AMDC", "AMDBARC",  or the output from \code{inla.qreordering}. Default is 'auto'.)
            reordering = "auto",

            ## :ARGUMENT: cpo.diff Numerical Threshold to define when the cpo-calculations are inaccurate. (EXPERT OPTION.)
            cpo.diff = NULL,

            ## :ARGUMENT: npoints Numerical Number of points to use in the 'stratey=laplace' approximation (default 9)
            npoints = 9,

            ## :ARGUMENT: cutoff Numerical The cutoff used in the 'stratey=laplace' approximation. (Smaller value is more accurate and more slow.) (default 1e-4)
            cutoff = 1e-4,

            ## :ARGUMENT: adapt.hessian.mode Logical Should optimisation be continued if the Hessian estimate is void? (Default TRUE)
            adapt.hessian.mode = NULL,

            ## :ARGUMENT: adapt.hessian.max.trials Numerical Number of steps in the adaptive Hessian optimisation
            adapt.hessian.max.trials = NULL,

            ## :ARGUMENT: adapt.hessian.scale Numerical The scaling of the 'h' after each trial.
            adapt.hessian.scale = NULL,

            ## :ARGUMENT: adaptive.max Selecting \code{strategy="adaptive"} will chose the default strategy for all fixed effects and model components with length less or equal to \code{adaptive.max}, for others, the gaussian strategy will be applied.
            adaptive.max = 25L,

            ## :ARGUMENT: huge Logical If TRUE then try to do some of the internal parallisations differently. Hopefully this will be of benefite for 'HUGE' models. (Default FALSE.) [THIS OPTION IS OBSOLETE AND NOT USED!]
            huge = FALSE,

            ## :ARGUMENT: step.len Numerical The step-length used to compute numerical derivaties of the log-likelihood
            step.len = .Machine$double.eps^(1.0 / 3.9134),

            ## :ARGUMENT: stencil Numerical Number of points in the stencil used to compute the numerical derivaties of the log-likelihood (3, 5, 7 or 9). (default 5)
            stencil = 5L,

            ## :ARGUMENT: lincomb.derived.correlation.matrix Logical If TRUE compute also the correlations for the derived linear combinations, if FALSE do not (Default FALSE)
            lincomb.derived.correlation.matrix = FALSE,

            ## :ARGUMENT: diagonal Numerical Expert use only! Add a this value on the diagonal of the joint precision matrix. (default 0.0)
            diagonal = 0.0,

            ## :ARGUMENT: numint.maxfeval Numerical Maximum number of function evaluations in the the numerical integration for the hyperparameters. (Default 100000.)
            numint.maxfeval = 100000,

            ## :ARGUMENT: numint.relerr Numerical Relative error requirement in the the numerical integration for the hyperparameters. (Default 1e-5)
            numint.relerr = 1e-5,

            ## :ARGUMENT: numint.abserr Numerical Absolute error requirement in the the numerical integration for the hyperparameters. (Default 1e-6)
            numint.abserr = 1e-6,

            ## :ARGUMENT: cmin Numerical The minimum value for the negative Hessian from the likelihood. Increasing this value will stabalise the optimisation but can introduce bias.  (Default -Inf)
            cmin = -Inf,

            ## :ARGUMENT: b.strategy Character If \code{cmin} is used, either keep the linear term (with \code{b.strategy="keep"}) or skip the contribution by setting the linear term to zero (\code{b.strategy="skip"}). The default value is \code{"skip"}
            b.strategy = "skip",

            ## :ARGUMENT: step.factor Numerical The step factor in the Newton-Raphson algorithm saying how large step to take (Default 1.0)
            ## YES! setting this to a negative values means = 1,  EXCEPT the first time (for each thread) where |step.factor| is used.
            ## This is an hidden option.
            step.factor = -0.1,

            ## :ARGUMENT: global.node.factor Numerical The factor which defines the degree ## ## required (how many neighbors), as a fraction of n-1, that is required to be ## ## classified as a global node and numbered last (whatever the reordering routine ## ## says). Here, n, is the size of the graph. (Disabled if larger than 1.) (default ## ## 2.0)
            global.node.factor = 2.0,

            ## :ARGUMENT: global.node.degree Numerical The degree required (number of neighbors) to be classified as a global node and numbered last (whatever the reordering routine says). (default \code{.Machine$integer.max})
            global.node.degree = .Machine$integer.max,

            ## :ARGUMENT: stupid.search Logical Enable or disable the stupid-search-algorithm, if the Hessian calculations reveals that the mode is not found. (Default \code{TRUE}.)
            stupid.search = TRUE,

            ## :ARGUMENT: stupid.search.max.iter Numerical Maximum number of iterations allowed for the stupid-search-algorithm. (default 1000)
            stupid.search.max.iter = 1000L,

            ## :ARGUMENT: stupid.search.factor Numerical Factor (>=1) to increase the step-length with after each new interation. (default 1.05)
            stupid.search.factor = 1.05,

            ## :ARGUMENT: control.vb List of arguments for various VB corrections. \code{enable} ## ## ## Logical/Character Use this feature? If \code{"auto"} this will be selected ## automatically. \code{strategy} Charactor What to correct, one of ## ## "mean", ## "more.to.come.later...". \code{verbose} Logical Be verbose or not. ## ## ## \code{refinement} Integer Number of extra possible refinement iterations. ## ## \code{max.correct} Numerical Bound the allowed correction ## ## (\code{strategy="mean"}: \code{|diff.mean/stdev| < max.correct}). ## ## \code{hyperpar.correct} Logical Correct the marginal posterior for the ## ## hyperparameters? \code{f.enable.limit} The size limit to correct for a \code{f()} ## ## model component.
            control.vb = list(enable = "auto", strategy = "mean", verbose = TRUE, refinement = 9, max.correct = 0.5, hyperpar.correct = FALSE, enable.limit = 25),

            ## :ARGUMENT: num.gradient Character Set the numerical scheme to compute the gradient,  one of \code{"forward"} or \code{"central"} (default).
            num.gradient = "central",

            ## :ARGUMENT: num.hessian Character Set the numerical scheme to compute the Hessian,  one of \code{"forward"} or \code{"central"} (default).
            num.hessian = "central",

            ## :ARGUMENT: optimise.strategy Character THIS OPTION IS EXPERIMENTAL. Chose the optimiser strategy,  one of \code{"plain"} or \code{"smart"} (default)
            optimise.strategy = "smart",

            ## :ARGUMENT: use.directions THIS OPTION IS EXPERIMENTAL. Unless \code{FALSE} or \code{NULL},  use directions for computing gradient and Hessian, initialised with \code{use.directions} if a matrix.
            use.directions = TRUE,

            ## :ARGUMENT: constr.marginal.diagonal Add stability to \code{AQ^-1A^T} by adding a small diagonal term. (default \code{epsilon^0.5})
            constr.marginal.diagonal = sqrt(.Machine$double.eps),

            ## :ARGUMENT: improved.simplified.laplace If \code{TRUE} use an experimental improved variant, otherwise, use the standard one.
            improved.simplified.laplace = FALSE,

            ## :ARGUMENT: fast If \code{stage1only} then try to do all calculations during stage1 by ## enabling Gaussian strategy with VB correction enabled.
            control.twostage = list(stage1only = TRUE)
        )

        ## :SEEALSO: inla

        return(ans)
    }


`inla.set.control.predictor.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.predictor
        list(
            ## :ARGUMENT: hyper Definition of the hyperparameters.
            hyper = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) If the precision for the artificial noise is fixed or not (defualt TRUE)
            fixed = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The prior for the artificial noise
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) Prior parameters for the artificial noise
            param = NULL,

            ## :ARGUMENT: initial (OBSOLETE!) The value of the log precision of the artificial noise
            initial = NULL,

            ## :ARGUMENT: compute A boolean variable; should the marginals for the linear predictor be computed? (Default FALSE.)
            compute = FALSE,

            ## :ARGUMENT: cdf A list of values to compute the CDF for the linear predictor
            cdf = NULL,

            ## :ARGUMENT: quantiles A list of quantiles to compute for the linear predictor
            quantiles = NULL,

            ## :ARGUMENT: cross Cross-sum-to-zero constraints with the linear predictor. All linear predictors with the same level of 'cross' are constrained to have sum zero. Use 'NA' for no contribution. 'Cross' has the same length as the linear predictor (including the 'A' matrix extention). (THIS IS AN EXPERIMENTAL OPTION, CHANGES MAY APPEAR.)
            cross = NULL,

            ## :ARGUMENT: A The observation matrix (matrix or Matrix::sparseMatrix).
            A = NULL,

            ## :ARGUMENT: precision The precision for eta* - A*eta, (default \code{exp(15)})
            precision = exp(15),

            ## :ARGUMENT: link Define the family-connection for unobserved observations (\code{NA}). \code{link} is integer values which defines the family connection; \code{family[link[idx]]} unless \code{is.na(link[idx])} for which the identity-link is used. The \code{link}-argument only influence the \code{fitted.values} in the \code{result}-object. If \code{is.null(link)} (default) then the identity-link is used for all missing observations. If the length of \code{link} is 1, then this value is replicated with the length of the responce vector. If an element of the responce vector is \code{!NA} then the corresponding entry in \code{link} is not used (but must still be a legal value). Setting this variable implies \code{compute=TRUE}.
            link = NULL,

            ## :ARGUMENT: vb.correct Logical Add this model component to the list of variables to be used for the vb corrected Laplace approximation? If \code{NULL} use default choice,  otherwise correct if \code{TRUE} and do not if \code{FALSE} (expert option)
            vb.correct = NULL
        )

        ## :SEEALSO: inla
    }

`inla.set.control.mode.default` <-
    function(...) {
        ## this is internal use only...
        ## :EXTRA:
        ## :NAME: control.mode
        list(
            ## :ARGUMENT: result Prevous result from inla(). Use the theta- and x-mode from this run.
            result = NULL,

            ## :ARGUMENT: theta The theta-mode/initial values for theta. This option has preference over result$mode$theta.
            theta = NULL,

            ## :ARGUMENT: x The x-mode/intitial values for x. This option has preference over result$mode$x.
            x = NULL,

            ## :ARGUMENT: restart A boolean variable; should we restart the optimisation from this configuration or fix the mode at this configuration? (Default FALSE.)
            restart = FALSE,

            ## :ARGUMENT: fixed A boolean variable. If TRUE then treat all thetas as known and fixed, and if FALSE then treat all thetas as unknown and random (default).
            fixed = FALSE
        )
        ## :SEEALSO: inla
    }

`inla.set.control.hazard.default` <-
    function(...) {
        ## :EXTRA:
        ## :NAME: control.hazard
        list(
            ## :ARGUMENT: model The model for the baseline hazard model. One of 'rw1', 'rw2' or 'iid'. (Default 'rw1'.)
            model = "rw1",

            ## :ARGUMENT: hyper The definition of the hyperparameters.
            hyper = NULL,

            ## :ARGUMENT: fixed (OBSOLETE!) A boolean variable; is the precision for 'model' fixed? (Default FALSE.)
            fixed = FALSE,

            ## :ARGUMENT: initial (OBSOLETE!) The initial value for the precision.
            initial = NULL,

            ## :ARGUMENT: prior (OBSOLETE!) The prior distribution for the precision for 'model'
            prior = NULL,

            ## :ARGUMENT: param (OBSOLETE!) The parameters in the prior distribution
            param = NULL,

            ## :ARGUMENT: constr A boolean variable; shall the  'model' be constrained to sum to zero?
            constr = TRUE,

            ## :ARGUMENT: diagonal An extra constant added to the diagonal of the precision matrix
            diagonal = NULL,

            ## :ARGUMENT: n.intervals Number of intervals in the baseline hazard. (Default 15)
            n.intervals = 15,

            ## :ARGUMENT: cutpoints The cutpoints to use. If not specified the they are compute from 'n.intervals' and the maximum length of the interval. (Default NULL)
            cutpoints = NULL,

            ## :ARGUMENT: strata.name The name of the stratefication variable for the baseline hazard in the data.frame
            strata.name = NULL,

            ## :ARGUMENT: scale.model Scale the baseline hazard model (RW1, RW2) so the generalized variance is 1. (Default \code{inla.getOption("scale.model.default")}.)
            scale.model = NULL
        )
        ## :SEEALSO: inla
    }

## check control-arguments

`inla.check.control` <- function(contr, data = NULL) {
    ## This function will signal an error if the arguments in CONTR
    ## does not match the ones in the corresponding
    ## `inla.set.XX.default()' routine.  EXAMPLE: contr is
    ## `control.inla' and default arguments is found in
    ## `inla.set.control.inla.default()'

    ## Will expand unexpanded names from the names in 'data' first
    contr <- local({
        name <- paste("inla.tmp.env", as.character(runif(1)), sep = "")
        attach(data, name = name, warn.conflicts = FALSE)
        ccontr <- contr
        detach(name, character.only = TRUE)
        ccontr
    })

    stopifnot(!missing(contr))
    stopifnot(is.list(contr))
    if (length(contr) == 0) {
        return(contr)
    }

    nm <- paste(sys.call()[2])
    f <- paste("inla.set.", nm, ".default()", sep = "")
    elms <- names(inla.eval(f))

    if (is.null(names(contr))) {
        stop(inla.paste(c(
            "Named elements in in control-argument `", nm, "', is required: ", contr,
            "\n\n  Valid ones are:\n\t",
            inla.paste(sort(elms), sep = "\n\t")
        ), sep = ""))
    }

    for (elm in names(contr)) {
        if (!is.element(elm, elms)) {
            stop(inla.paste(c(
                "Name `", elm, "' in control-argument `", nm, "', is void.\n\n  Valid ones are:\n\t",
                inla.paste(sort(elms), sep = "\n\t")
            ), sep = ""))
        }
    }

    return(contr)
}



## test-implementation
## `control.lincomb` = function(precision, verbose)
## {
## aa = match.call()[-1]
## ret = list()
## for(a in names(aa)) {
## if (!missing(a)) {
## xx = get(a)
## names(xx) = a
## ret = c(ret, xx)
## }
## }
## return (ret)
## }

inla.make.completion.function <- function(...) {
    my.eval <- function(command, envir = parent.frame(),
                        enclos = if (is.list(envir) || is.pairlist(envir)) parent.frame() else baseenv()) {
        return(eval(parse(text = command), envir, enclos))
    }

    xx <- sort(list(...)[[1L]])
    my.eval(paste("function(", paste(xx, sep = "", collapse = ", "), ") {
    aa = match.call()[-1L]
    ret = list()
    for(a in names(aa)) {
        if (!missing(a)) {
            xx = get(a)
            names(xx) = a
            ret = c(ret, xx)
        }
    }
    return (ret)
}"))
}

control.update <- inla.make.completion.function(names(inla.set.control.update.default()))
control.lincomb <- inla.make.completion.function(names(inla.set.control.lincomb.default()))
control.group <- inla.make.completion.function(names(inla.set.control.group.default()))
control.mix <- inla.make.completion.function(names(inla.set.control.mix.default()))
control.link <- inla.make.completion.function(names(inla.set.control.link.default()))
control.expert <- inla.make.completion.function(names(inla.set.control.expert.default()))
control.compute <- inla.make.completion.function(names(inla.set.control.compute.default()))
control.family <- inla.make.completion.function(names(inla.set.control.family.default()))
control.fixed <- inla.make.completion.function(names(inla.set.control.fixed.default()))
control.inla <- inla.make.completion.function(names(inla.set.control.inla.default()))
control.vb <- inla.make.completion.function(names(inla.set.control.inla.default()$control.vb))
control.predictor <- inla.make.completion.function(names(inla.set.control.predictor.default()))
control.mode <- inla.make.completion.function(names(inla.set.control.mode.default()))
control.hazard <- inla.make.completion.function(names(inla.set.control.hazard.default()))
control.bgev <- inla.make.completion.function(names(inla.set.control.bgev.default()))
control.pardiso <- inla.make.completion.function(names(inla.set.control.pardiso.default()))

## Export: inla.rgeneric.ar1.model
## Export: inla.rgeneric.ar1.model.opt
## Export: inla.rgeneric.iid.model
## Export: inla.rgeneric.define
## Export: inla.rgeneric.wrapper
## Export: inla.rgeneric.q

## !\name{rgeneric.define}
## !\alias{rgeneric}
## !\alias{rgeneric.define}
## !\alias{inla.rgeneric.define}
## !\alias{rgeneric.ar1.model}
## !\alias{inla.rgeneric.ar1.model}
## !\alias{rgeneric.iid.model}
## !\alias{inla.rgeneric.iid.model}
## !\alias{rgeneric.wrapper}
## !\alias{inla.rgeneric.wrapper}
## !\alias{rgeneric.q}
## !\alias{inla.rgeneric.q}
## !
## !\title{rgeneric models}
## !
## !\description{A framework for defining latent models in R}
## !
## !\usage{
## !inla.rgeneric.define(model = NULL, debug = FALSE, compile = TRUE, optimize = FALSE, ...)
## !inla.rgeneric.iid.model(
## !        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
## !        theta = NULL)
## !inla.rgeneric.ar1.model(
## !        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
## !        theta = NULL)
## !inla.rgeneric.ar1.model.opt(
## !        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
## !        theta = NULL)
## !inla.rgeneric.wrapper(
## !        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
## !        model, theta = NULL)
## !inla.rgeneric.q(
## !        rmodel,
## !        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
## !        theta = NULL)
## !}
## !
## !\arguments{
## !  \item{model}{The definition of the model; see \code{inla.rgeneric.ar1.model}}
## !  \item{rmodel}{The rgeneric model-object, the output of \code{inla.rgeneric.define}}
## !  \item{debug}{Logical. Turn on/off debugging}
## !  \item{compile}{Logical. Compile the definition of the model or not.}
## !  \item{optimze}{Logical. With this option \code{TRUE}, then \code{model}
## !                 pass only the values of \code{Q} and not the whole matrix.
## !                 Please see the vignette for details and
## !                 \code{inla.rgeneric.ar1.model.opt} for an example.}
## !  \item{cmd}{An allowed request}
## !  \item{theta}{Values of theta}
## !  \item{...}{Named list of variables that defines the environment of \code{model}}
## !  \item{debug}{Logical. Enable debug output}
## !}
## !
## !\value{%%
## !  This allows a latent model to be
## !  defined in \code{R}.
## !  See \code{inla.rgeneric.ar1.model} and
## !  \code{inla.rgeneric.iid.model} and the documentation for
## !  worked out examples of how to define latent models in  this way.
## !  This will be somewhat slow and is intended for special cases and
## !  protyping. The function \code{inla.rgeneric.wrapper} is for
## !  internal use only.}
## !\author{Havard Rue \email{hrue@r-inla.org}}


`inla.rgeneric.ar1.model` <- function(
                                      cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
                                      theta = NULL) {
    ## this is an example of the 'rgeneric' model. here we implement
    ## the AR-1 model as described in inla.doc("ar1"), where 'rho' is
    ## the lag-1 correlation and 'prec' is the *marginal* (not
    ## conditional) precision.

    ## variables defined the in the define-call, are stored here
    ## (which is in the path)
    envir <- parent.env(environment())

    interpret.theta <- function() {
        ## internal helper-function to map the parameters from the internal-scale to the
        ## user-scale
        return(list(
            prec = exp(theta[1L]),
            rho = 2 * exp(theta[2L]) / (1 + exp(theta[2L])) - 1.0
        ))
    }

    graph <- function() {
        if (TRUE) {
            ## we can also use this easy solution, if we know that Q[i, j] is not 0 by
            ## accident... this require that 'theta' is set; see 'theta = initial()' below.
            G <- Q()
        } else {
            ## return the graph of the model. the values of Q is only interpreted as zero or
            ## non-zero. return a sparse.matrix
            if (FALSE) {
                ## slow and easy: dense-matrices
                G <- toeplitz(c(1, 1, rep(0, n - 2L)))
                G <- inla.as.sparse(G)
            } else {
                ## faster. we only need to define the upper-triangular of G
                i <- c(
                    ## diagonal
                    1L, n, 2L:(n - 1L),
                    ## off-diagonal
                    1L:(n - 1L)
                )
                j <- c(
                    ## diagonal
                    1L, n, 2L:(n - 1L),
                    ## off-diagonal
                    2L:n
                )
                x <- 1 ## meaning that all are 1
                G <- sparseMatrix(i = i, j = j, x = x, repr = "T")
            }
        }
        return(G)
    }

    Q <- function() {
        ## returns the precision matrix for given parameters
        param <- interpret.theta()
        if (FALSE) {
            ## slow and easy: dense-matrices
            Q <- param$prec / (1 - param$rho^2) * toeplitz(c(1 + param$rho^2, -param$rho, rep(0, n - 2L)))
            Q[1, 1] <- Q[n, n] <- param$prec / (1 - param$rho^2)
            Q <- inla.as.sparse(Q)
        } else {
            ## faster. we only need to define the upper-triangular Q!
            i <- c(
                ## diagonal
                1L, n, 2L:(n - 1L),
                ## off-diagonal
                1L:(n - 1L)
            )
            j <- c(
                ## diagonal
                1L, n, 2L:(n - 1L),
                ## off-diagonal
                2L:n
            )
            x <- param$prec / (1 - param$rho^2) *
                c( ## diagonal
                    1L, 1L, rep(1 + param$rho^2, n - 2L),
                    ## off-diagonal
                    rep(-param$rho, n - 1L)
                )
            Q <- sparseMatrix(i = i, j = j, x = x, repr = "T")
        }
        return(Q)
    }

    mu <- function() {
        return(numeric(0))
    }

    log.norm.const <- function() {
        ## return the log(normalising constant) for the model
        param <- interpret.theta()
        prec.innovation <- param$prec / (1.0 - param$rho^2)
        val <- n * (-0.5 * log(2 * pi) + 0.5 * log(prec.innovation)) + 0.5 * log(1.0 - param$rho^2)
        return(val)
    }

    log.prior <- function() {
        ## return the log-prior for the hyperparameters. the '+theta[1L]' is the log(Jacobian)
        ## for having a gamma prior on the precision and convert it into the prior for the
        ## log(precision).
        param <- interpret.theta()
        val <- (dgamma(param$prec, shape = 1, rate = 1, log = TRUE) + theta[1L] +
            dnorm(theta[2L], mean = 0, sd = 1, log = TRUE))
        return(val)
    }

    initial <- function() {
        ## return initial values. second argument cannot be 0 otherwise the graph is diagonal
        ## and everything breaks down.
        return(rep(1, 2))
    }

    quit <- function() {
        return(invisible())
    }

    ## if theta is not required, it is not set. we set it here, for convenience.
    ## (see the graph() function)
    if (!length(theta)) {
          theta <- initial()
      }

    val <- do.call(match.arg(cmd), args = list())
    return(val)
}

`inla.rgeneric.ar1.model.opt` <- function(
                                          cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
                                          theta = NULL) {
    ## this is an example of the optimzed 'rgeneric' model

    ## variables defined the in the define-call, are stored here
    ## (which is in the path)
    envir <- parent.env(environment())

    interpret.theta <- function() {
        ## internal helper-function to map the parameters from the internal-scale to the
        ## user-scale
        return(list(
            prec = exp(theta[1L]),
            rho = 2 * exp(theta[2L]) / (1 + exp(theta[2L])) - 1.0
        ))
    }

    graph <- function() {
        i <- c(
            ## diagonal
            1L, n, 2L:(n - 1L),
            ## off-diagonal
            1L:(n - 1L)
        )
        j <- c(
            ## diagonal
            1L, n, 2L:(n - 1L),
            ## off-diagonal
            2L:n
        )
        G <- inla.as.sparse(sparseMatrix(i = i, j = j, x = 1, repr = "T"))
        return(G)
    }

    Q <- function() {
        param <- interpret.theta()
        a <- 1 + param$rho^2
        b <- -param$rho
        x <- param$prec / (1 - param$rho^2) * c(1, b, rep(c(a, b), n - 2), 1)
        return(x)
    }

    mu <- function() {
        return(numeric(0))
    }

    log.norm.const <- function() {
        ## return the log(normalising constant) for the model
        param <- interpret.theta()
        prec.innovation <- param$prec / (1.0 - param$rho^2)
        val <- n * (-0.5 * log(2 * pi) + 0.5 * log(prec.innovation)) + 0.5 * log(1.0 - param$rho^2)
        return(val)
    }

    log.prior <- function() {
        ## return the log-prior for the hyperparameters. the '+theta[1L]' is the log(Jacobian)
        ## for having a gamma prior on the precision and convert it into the prior for the
        ## log(precision).
        param <- interpret.theta()
        val <- (dgamma(param$prec, shape = 1, rate = 1, log = TRUE) + theta[1L] +
            dnorm(theta[2L], mean = 0, sd = 1, log = TRUE))
        return(val)
    }

    initial <- function() {
        ## return initial values. second argument cannot be 0 otherwise the graph is diagonal
        ## and everything breaks down.
        return(rep(1, 2))
    }

    quit <- function() {
        return(invisible())
    }

    ## if theta is not required, it is not set. we set it here, for convenience.
    ## (see the graph() function)
    if (!length(theta)) {
          theta <- initial()
      }

    val <- do.call(match.arg(cmd), args = list())
    return(val)
}

`inla.rgeneric.iid.model` <- function(
                                      cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
                                      theta = NULL) {
    ## this is an example of the 'rgeneric' model. here we implement the iid model as described
    ## in inla.doc("iid"), without the scaling-option

    ## variables defined the in the define-call, are stored here
    ## (which is in the path)
    envir <- parent.env(environment())

    interpret.theta <- function() {
        return(list(prec = exp(theta[1L])))
    }

    graph <- function() {
        G <- Diagonal(n, x = rep(1, n))
        return(G)
    }

    Q <- function() {
        prec <- interpret.theta()$prec
        Q <- Diagonal(n, x = rep(prec, n))
        return(Q)
    }

    mu <- function() {
        return(numeric(0))
    }

    log.norm.const <- function() {
        prec <- interpret.theta()$prec
        val <- sum(dnorm(rep(0, n), sd = 1 / sqrt(prec), log = TRUE))
        return(val)
    }

    log.prior <- function() {
        prec <- interpret.theta()$prec
        val <- dgamma(prec, shape = 1, rate = 1, log = TRUE) + theta[1L]
        return(val)
    }

    initial <- function() {
        ntheta <- 1
        return(rep(1, ntheta))
    }

    quit <- function() {
        return(invisible())
    }

    ## if theta is not required, it is not set. we set it here, for convenience.
    if (!length(theta)) {
          theta <- initial()
      }

    val <- do.call(match.arg(cmd), args = list())
    return(val)
}

`inla.rgeneric.define` <- function(model = NULL, debug = FALSE, compile = TRUE, optimize = FALSE, ...) {
    stopifnot(!missing(model))
    args <- list(...)
    if (any(names(args) == "")) {
        stop("The '...' argument in 'inla.rgeneric.define()' needs *named* arguments.")
    }
    env <- if (length(args) > 0) as.environment(args) else new.env()
    parent.env(env) <- .GlobalEnv
    environment(model) <- env

    rmodel <- list(
        f = list(
            model = "rgeneric",
            n = dim(model(cmd = "graph", theta = NULL))[1],
            rgeneric = list(
                definition = if (!compile) {
                                 model
                             } else {
                                 inla.require("compiler", stop.on.error = TRUE)
                                 compiler::cmpfun(model,
                                                  options = list(optimize = 3L,
                                                                 suppressUndefined = TRUE))
                             },
                debug = debug,
                optimize = optimize
            )
        )
    )
    class(rmodel) <- "inla.rgeneric"
    class(rmodel$f$rgeneric) <- "inla.rgeneric"
    return(rmodel)
}

`inla.rgeneric.wrapper` <- function(
                                    cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
                                    model, theta = NULL) {
    debug.cat <- function(...) {
        if (debug) {
              cat("Rgeneric: ", ..., "\n", file = stderr())
          }
    }

    model.orig <- model
    if (is.character(model)) {
        model <- get(model, envir = parent.frame())
    }
    stopifnot(inherits(model, "inla.rgeneric"))

    debug <- ifelse(is.null(model$debug) || !model$debug, FALSE, TRUE)
    if (is.character(model.orig)) {
        debug.cat("Enter with cmd=", cmd, ", model=", model.orig, "theta=", theta)
    } else {
        debug.cat("Enter with cmd=", cmd, ", theta=", theta)
    }

    result <- NULL
    cmd <- match.arg(cmd)
    res <- do.call(model$definition, args = list(cmd = cmd, theta = theta))
    time.ref <- proc.time()[3]

    if (cmd %in% "Q") {
        if (model$optimize) {
            ## pass only Q@x, using the ordering after applying 'inla.sparse.matrix()' to 'Q'
            ## and (of course) only the upper triangular part of Q
            len <- length(res)
            debug.cat("length(Q@x)", len)
            result <- c(-1, len, res) ## yes, this is the code that we have optimized Q-output
        } else {
            Q <- inla.as.sparse(res)
            debug.cat("dim(Q)", dim(Q))
            n <- dim(Q)[1L]
            stopifnot(dim(Q)[1L] == dim(Q)[2L])
            idx <- which(Q@i <= Q@j)
            len <- length(Q@i[idx])
            result <- c(n, len, Q@i[idx], Q@j[idx], Q@x[idx])
            Q <- NULL
        }
    } else if (cmd %in% "graph") {
        diag(res) <- 1
        G <- inla.as.sparse(res)
        stopifnot(dim(G)[1L] == dim(G)[2L])
        n <- dim(G)[1L]
        idx <- which(G@i <= G@j)
        len <- length(G@i[idx])
        debug.cat("n", n, "len", len)
        result <- c(n, len, G@i[idx], G@j[idx])
        G <- NULL
    } else if (cmd %in% "mu") {
        mu <- res
        debug.cat("length(mu)", length(mu))
        result <- c(length(mu), mu)
    } else if (cmd %in% "initial") {
        init <- res
        debug.cat("initial", init)
        result <- c(length(init), init)
    } else if (cmd %in% "log.norm.const") {
        lnc <- res
        debug.cat("log.norm.const", lnc)
        result <- c(lnc)
    } else if (cmd %in% "log.prior") {
        lp <- res
        debug.cat("log.prior", lp)
        result <- c(lp)
    } else if (cmd %in% c("quit", "exit")) {
        ## nothing for the moment
        result <- NULL
    } else {
        stop(paste("Unknown command", cmd))
    }
    res <- NULL

    return(as.numeric(result))
}

`inla.rgeneric.q` <- function(rmodel,
                              cmd = c(
                                  "graph", "Q", "mu", "initial", "log.norm.const",
                                  "log.prior", "quit"
                              ),
                              theta = NULL) {
    if (missing(cmd)) {
        stop("A value for argument 'cmd' is required.")
    }
    cmd <- match.arg(cmd)
    rmodel.orig <- rmodel
    if (is.character(rmodel)) {
        rmodel <- get(rmodel, envir = parent.frame())
    }
    if (!inherits(rmodel, "inla.rgeneric")) {
        stop("Argument 'rmodel' is not of class 'inla.rgeneric' (usually the output of 'inla.rgeneric.define')")
    }
    func <- rmodel$f$rgeneric$definition

    if (cmd %in% c("Q", "mu", "log.norm.const", "log.prior")) {
        ## for these we need values of 'theta': check that the length is correct
        initial <- do.call(what = func, args = list(cmd = "initial", theta = NULL))
        if (length(initial) != length(theta)) {
            stop(paste0(
                "Length of argument theta: ", length(theta),
                ", does not match the length of the initial values in 'rmodel': ", length(initial)
            ))
        }
        ## just to make sure nothing else of length zero is passed
        if (length(initial) == 0) {
              theta <- NULL
          }
    } else {
        theta <- NULL
    }

    res <- do.call(what = func, args = list(cmd = cmd, theta = theta))
    if (cmd %in% c("Q", "graph")) {

        ## in the case of optimized output of Q
        if (!(is.matrix(res) || is(res, "Matrix"))) {
            ## optimized output
            len.x <- length(res)
            ## need the graph to interpret the output
            graph <- do.call(what = func, args = list(cmd = "graph"))
            diag(graph) <- 1
            graph <- inla.as.sparse(graph)
            idx <- which(graph@i <= graph@j)
            graph@i <- graph@i[idx]
            graph@j <- graph@j[idx]
            graph@x <- graph@x[idx]
            graph@x[] <- 1
            stopifnot(length(graph@x) == len.x)
            Q <- inla.as.sparse(graph)
            graph <- NULL
            Q@x <- res
        } else {
            ## since only the upper triangular matrix (diagonal included) is required return from
            ## 'do.call', then make sure its symmetric and that diag(Graph) = 1
            if (cmd %in% "Q") {
                Q <- inla.as.sparse(res)
            } else {
                diag(res) <- 1
                Q <- inla.as.sparse(res, na.rm = TRUE, zeros.rm = TRUE)
                Q[Q != 0] <- 1
            }
        }
        n <- dim(Q)[1]
        idx.eq <- which(Q@i == Q@j)
        idx.gt <- which(Q@i < Q@j)
        Q <- sparseMatrix(
            i = c(Q@i[idx.eq], Q@i[idx.gt], Q@j[idx.gt]),
            j = c(Q@j[idx.eq], Q@j[idx.gt], Q@i[idx.gt]),
            x = c(Q@x[idx.eq], Q@x[idx.gt], Q@x[idx.gt]),
            index1 = FALSE,
            dims = c(n, n),
            repr = "T"
        )
        Q <- inla.as.sparse(Q)
        return(Q)
    } else if (cmd %in% c("mu", "initial", "log.norm.const", "log.prior")) {
        return(c(as.numeric(res)))
    } else if (cmd %in% "quit") {
        return(NULL)
    } else {
        stop(paste("Unknown command", cmd))
    }
}

## Export: inla.qreordering

## ! \name{qreordering}
## ! \alias{inla.qreordering}
## ! \alias{qreordering}
## !
## ! \title{Compute the reordering using the GMRFLib implementation}
## !
## ! \description{This function compute the reordering (or find the best reordering)
## !              using the GMRFLib implementation}
## ! \usage{
## !     inla.qreordering(graph, reordering = inla.reorderings())
## ! }
## !
## ! \arguments{
## !   \item{graph}{A \code{(inla-)graph} object}
## !   \item{reordering}{The name of the reordering algorithm to be used; either
## !                     one of the names listed in \code{inla.reorderings()}.
## !                     The default is "auto" which try several reordering algorithm
## !                     and use the best one for this particular matrix.}
## !}
## !\value{
## !  \code{inla.qreordering} returns a list with the name of the reordering algorithm used or
## !  found, the reordering code for the reordering algorithm,
## !  the actual reordering and its inverse.
## !}
## !\author{Havard Rue \email{hrue@r-inla.org}}
## !
## !\examples{
## ! g = system.file("demodata/germany.graph", package="INLA")
## ! r = inla.qreordering(g)
## ! m = inla.graph2matrix(g)
## ! r = inla.qreordering(m)
## !}

`inla.qreordering` <- function(graph, reordering = inla.reorderings()) {
    reordering <- match.arg(reordering)

    if (inla.is.matrix(graph)) {
        graph <- inla.sparse.check(graph)
        g.file <- inla.write.fmesher.file(graph)
        g.remove <- TRUE
    } else if (inla.is.fmesher.file(graph)) {
        g.file <- graph
        g.remove <- FALSE
    } else {
        g <- inla.read.graph(graph)
        g.file <- inla.write.graph(g)
        g.remove <- FALSE
    }

    ## smtp must be taucs
    if (inla.os("linux") || inla.os("mac") || inla.os("mac.arm64")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qreordering",
            "-r", reordering, "-S", "taucs", g.file
        ), intern = TRUE)
    } else if (inla.os("windows")) {
        s <- system(paste(
            shQuote(inla.call.no.remote()), "-s -m qreordering",
            "-r", reordering, "-S", "taucs", g.file
        ), intern = TRUE)
    } else {
        stop("\n\tNot supported architecture.")
    }

    if (g.remove) {
        unlink(g.file)
    }

    for (i in 1:length(s)) {
        ## the output containts this code, as there may be some other output before this
        if (as.character(s[i]) == "QREORDERING") {
              break
          }
    }
    s <- s[-(1:i)]

    nm <- as.character(s[1L])
    code <- as.numeric(s[2L])
    s <- as.numeric(s[-c(1L, 2L)])
    r <- s + 1L
    ir <- r
    ir[r] <- 1L:length(r)

    return(list(name = nm, code = code, reordering = r, ireordering = ir))
}

## Export: inla.dmarginal inla.pmarginal inla.qmarginal inla.rmarginal inla.hpdmarginal
## Export: inla.smarginal inla.emarginal inla.tmarginal inla.mmarginal inla.zmarginal

## NOT-YET-IN-USE-Export: summary!inla.marginal plot!inla.marginal

## ! \name{marginal}
## ! \alias{inla.marginal}
## ! \alias{marginal}
## ! \alias{pmarginal}
## ! \alias{inla.pmarginal}
## ! \alias{qmarginal}
## ! \alias{inla.qmarginal}
## ! \alias{dmarginal}
## ! \alias{inla.dmarginal}
## ! \alias{rmarginal}
## ! \alias{inla.rmarginal}
## ! \alias{inla.hpdmarginal}
## ! \alias{hpdmarginal}
## ! \alias{inla.emarginal}
## ! \alias{emarginal}
## ! \alias{inla.smarginal}
## ! \alias{smarginal}
## ! \alias{inla.tmarginal}
## ! \alias{inla.mmarginal}
## ! \alias{mmarginal}
## ! \alias{inla.zmarginal}
## ! \alias{zmarginal}
## !
## ! \title{Functions which operates on marginals}
## !
## ! \description{Density, distribution function, quantile function, random
## !      generation, hpd-interval, interpolation, expectations, mode and transformations of
## !      marginals obtained by \code{inla} or \code{inla.hyperpar()}.
## ! These functions computes the density (inla.dmarginal),
## ! the distribution function (inla.pmarginal),
## ! the quantile function (inla.qmarginal),
## ! random generation (inla.rmarginal),
## ! spline smoothing (inla.smarginal),
## ! computes expected values (inla.emarginal),
## ! computes the mode (inla.mmarginal),
## ! transforms the marginal (inla.tmarginal), and provide summary statistics (inla.zmarginal).
## ! }
## !
## ! \usage{
## ! inla.dmarginal(x, marginal, log = FALSE)
## ! inla.pmarginal(q, marginal, normalize = TRUE, len = 2048L)
## ! inla.qmarginal(p, marginal, len = 2048L)
## ! inla.rmarginal(n, marginal)
## ! inla.hpdmarginal(p, marginal, len = 2048L)
## ! inla.smarginal(marginal, log = FALSE, extrapolate = 0.0, keep.type = FALSE, factor=15L)
## ! inla.emarginal(fun, marginal, ...)
## ! inla.mmarginal(marginal)
## ! inla.tmarginal(fun, marginal, n=2048L, h.diff = .Machine[["double.eps"]]^(1.0/3.0),
## !                method = c("quantile", "linear"))
## ! inla.zmarginal(marginal, silent = FALSE)
## ! }
## ! \arguments{
## !
## !   \item{marginal}{A marginal object from either \code{inla} or
## !     \code{inla.hyperpar()}, which is either \code{list(x=c(), y=c())}
## !     with density values \code{y} at locations \code{x}, or a
## !     \code{matrix(,n,2)} for which the density values are the second
## !     column and the locations in the first column.
## !     The\code{inla.hpdmarginal()}-function
## !     assumes a unimodal density.}
## !
## !   \item{fun}{A (vectorised) function like \code{function(x) exp(x)} to
## !     compute the expectation against, or which define the transformation
## !     new = fun(old)}
## !
## !   \item{x}{Evaluation points}
## !
## !   \item{q}{Quantiles}
## !
## !   \item{p}{Probabilities}
## !
## !   \item{n}{The number of observations. If \code{length(n) > 1}, the
## !     length is taken to be the number required.}
## !
## !   \item{h.diff}{The step-length for the numerical differeniation inside \code{inla.tmarginal}}
## !
## !   \item{...}{Further arguments to be passed to function which
## !     expectation is to be computed.}
## !
## !   \item{log}{Return density or interpolated density in log-scale?}
## !
## !   \item{normalize}{Renormalise the density after interpolation?}
## !   \item{len}{Number of locations used to interpolate the distribution
## !   function.}
## !
## !   \item{keep.type}{If \code{FALSE} then return a \code{list(x=, y=)},  otherwise if \code{TRUE},
## !                   then return a matrix if the input is a matrix}
## !
## !   \item{extrapolate}{How much to extrapolate on each side when computing the
## !   interpolation. In fraction of the range.}
## !
## !   \item{factor}{The number of points after interpolation is \code{factor} times the original number of points;
## !                 which is argument \code{n} in \code{spline}}
## !
## !    \item{method}{Which method should be used to layout points for where the transformation is computed.}
## !
## !    \item{silent}{Output the result visually (TRUE) or just through the call.}
## ! }
## !
## ! \value{%%
## !   \code{inla.smarginal} returns \code{list=c(x=c(), y=c())} of
## !   interpolated values do extrapolation using the factor given,
## !   and the remaining function returns what they say they should do.  }
## ! %%
## !
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## !
## ! \seealso{\code{\link{inla}}, \code{\link{inla.hyperpar}}}
## !
## ! \examples{
## ! ## a simple linear regression example
## ! n = 10
## ! x = rnorm(n)
## ! sd = 0.1
## ! y = 1+x + rnorm(n,sd=sd)
## ! res = inla(y ~ 1 + x, data = data.frame(x,y),
## !            control.family=list(initial = log(1/sd^2L),fixed=TRUE))
## !
## ! ## chose a marginal and compare the with the results computed by the
## ! ## inla-program
## ! r = res$summary.fixed["x",]
## ! m = res$marginals.fixed$x
## !
## ! ## compute the 95% HPD interval
## ! inla.hpdmarginal(0.95, m)
## !
## ! x = seq(-6, 6, len = 1000)
## ! y = dnorm(x)
## ! inla.hpdmarginal(0.95, list(x=x, y=y))
## !
## ! ## compute the the density for exp(r), version 1
## ! r.exp = inla.tmarginal(exp, m)
## ! ## or version 2
## ! r.exp = inla.tmarginal(function(x) exp(x), m)
## !
## ! ## to plot the marginal, we use the inla.smarginal, which interpolates (in
## ! ## log-scale). Compare with some samples.
## ! plot(inla.smarginal(m), type="l")
## ! s = inla.rmarginal(1000, m)
## ! hist(inla.rmarginal(1000, m), add=TRUE, prob=TRUE)
## ! lines(density(s), lty=2)
## !
## ! m1 = inla.emarginal(function(x) x, m)
## ! m2 = inla.emarginal(function(x) x^2L, m)
## ! stdev = sqrt(m2 - m1^2L)
## ! q = inla.qmarginal(c(0.025,0.975), m)
## !
## ! ## inla-program results
## ! print(r)
## !
## ! ## inla.marginal-results (they shouldn't be perfect!)
## ! print(c(mean=m1, sd=stdev, "0.025quant" = q[1], "0.975quant" = q[2L]))
## ! ## using the buildt-in function
## ! inla.zmarginal(m)
## ! }
## !

### functions to work with the marginal, either defined as a matrix
### x[2, n], or a list(x=, y=).  NOTE:: there are NO EXTRAPOLATION, so
### the marginal is defined FROM...TO!

`inla.marginal.fix` <- function(marginal) {
    ## just remove points where the density is <= 0 or where the
    ## density is to small compared to the maximum density. (othewise
    ## we can get trouble with the spline interpolation). same with
    ## 'x'? No...
    eps <- .Machine[["double.eps"]] * 1000.0
    ## marginal = spline(marginal)
    if (is.matrix(marginal)) {
        if (any(is.na(marginal[, 2L]))) {
            idx <- which(is.na(marginal[, 2L]))
            marginal <- marginal[-idx, ]
        }
        i <- (marginal[, 2L] > 0.0) & (abs(marginal[, 2L] / max(marginal[, 2L])) > eps)
        m <- list(x = marginal[i, 1L], y = marginal[i, 2L])
        ## i = c(diff(marginal[, 1L]) > eps, TRUE)
        ## m = list(x=marginal[i, 1L], y=marginal[i, 2L])
    } else {
        if (any(is.na(marginal[["y"]]))) {
            idx <- which(is.na(marginal[["y"]]))
            marginal[["x"]] <- marginal[["x"]][-idx]
            marginal[["y"]] <- marginal[["y"]][-idx]
        }
        i <- (marginal[["y"]] > 0.0) & (abs(marginal[["y"]] / max(marginal[["y"]])) > eps)
        m <- list(x = marginal[["x"]][i], y = marginal[["y"]][i])
        ## i = c(diff(marginal[["x"]]) > eps, TRUE)
        ## m = list(x = marginal[["x"]][i], y = marginal[["y"]][i])
    }

    return(m)
}

`inla.spline` <- function(x, ...) {
    s <- spline(x, ...)
    if (is.matrix(x)) {
        m <- cbind(x = s[["x"]], y = s[["y"]])
    } else if (is.list(x)) {
        m <- list(x = s[["x"]], y = s[["y"]])
    } else {
        m <- s
    }
    return(m)
}

`inla.smarginal` <- function(marginal, log = FALSE, extrapolate = 0.0, keep.type = FALSE, factor = 15L) {
    ## for marginal in matrix MARGINAL, which is a marginal density,
    ## return the nice interpolated (x, y) where the interpolation is
    ## done in log(y)

    is.mat <- is.matrix(marginal)
    m <- inla.marginal.fix(marginal)
    r <- diff(range(m[["x"]]))
    xmin <- min(m[["x"]]) - extrapolate * r
    xmax <- max(m[["x"]]) + extrapolate * r
    n <- factor * length(m[["x"]])
    xx <- seq(xmin, xmax, len = n)
    if (extrapolate) {
        xx <- c(xmin, m[["x"]], xmax)
    } else {
        xx <- m[["x"]]
    }
    nx <- length(xx)
    dx <- nx * diff(xx) / mean(diff(xx))
    xnew <- c(0.0, cumsum(sqrt(dx)))
    xnew <- xmin + (xmax - xmin) * ((xnew - min(xnew)) / (max(xnew) - min(xnew)))
    fun <- splinefun(xnew, xx, method = "hyman")
    fun.inv <- splinefun(xx, xnew, method = "hyman")
    ans <- spline(fun.inv(m[["x"]]), log(m[["y"]]), xmin = fun.inv(xmin), xmax = fun.inv(xmax), n = n, method = "fmm")
    ans[["x"]] <- fun(ans[["x"]])
    if (!log) {
        ans[["y"]] <- exp(ans[["y"]])
        ans <- inla.marginal.fix(ans)
    }
    if (is.mat && keep.type) {
        return(cbind(x = ans[["x"]], y = ans[["y"]]))
    } else {
        return(ans)
    }
}

`inla.sfmarginal` <- function(marginal) {
    ## for marginal in matrix MARGINAL, which is a marginal density,
    ## return the spline-function which returns the log-density

    m <- inla.marginal.fix(marginal)
    r <- range(m[["x"]])
    return(list(range = r, fun = splinefun(m[["x"]], log(m[["y"]]))))
}

`inla.emarginal` <- function(fun, marginal, ...) {
    ## compute E(FUN(x)), where the marginal of x is given in
    ## `marginal'; see inla.smarginal(). Also work for FUN(x)
    ## returning a vector.

    xx <- inla.smarginal(marginal)
    n <- length(xx[["x"]])
    if (n %% 2L == 0L) {
        n <- n - 1L
        xx[["x"]] <- xx[["x"]][1L:n]
        xx[["y"]] <- xx[["y"]][1L:n]
    }

    ## use Simpsons integration rule
    i.0 <- c(1L, n)
    i.4 <- seq(2L, n - 1L, by = 2L)
    i.2 <- seq(3L, n - 2L, by = 2L)

    dx <- diff(xx[["x"]])
    dx <- 0.5 * (c(dx, 0.0) + c(0.0, dx))
    fun <- match.fun(fun)
    ff <- fun(xx[["x"]][1L:n], ...) * xx[["y"]][1L:n] * dx
    nf <- length(ff) %/% n
    e <- numeric(nf)
    off <- 0L
    for (i in 1L:nf) {
        e[i] <- sum(sum(ff[i.0 + off]) + 4.0 * sum(ff[i.4 + off]) + 2.0 * sum(ff[i.2 + off]))
        off <- off + n
    }

    ## normalise, so that E(1) = 1
    ff <- dx * xx[["y"]][1L:n]
    e.1 <- sum(sum(ff[i.0]) + 4.0 * sum(ff[i.4]) + 2.0 * sum(ff[i.2]))

    return(e / e.1)
}

`inla.dmarginal` <- function(x, marginal, log = FALSE) {
    ## return the density of the marginal. if LOG, in log-scale.  this
    ## density is not renormalized (but should be already normalized
    ## in some scale).

    ## return list(range=, fun=..)
    f <- inla.sfmarginal(inla.smarginal(marginal))

    n <- length(x)
    d <- numeric(n)

    for (i in 1L:n) {
        if (x[i] >= f[["range"]][[1L]] && x[i] <= f[["range"]][[2L]]) {
            if (log) {
                d[i] <- f[["fun"]](x[i])
            } else {
                d[i] <- exp(f[["fun"]](x[i]))
            }
        } else {
            if (log) {
                d[i] <- -Inf
            } else {
                d[i] <- 0.0
            }
        }
    }
    return(d)
}

`inla.pmarginal` <- function(q, marginal, normalize = TRUE, len = 2048L) {
    f <- inla.sfmarginal(inla.smarginal(marginal))
    xx <- seq(f[["range"]][[1L]], f[["range"]][[2L]], length = len)
    d <- cumsum(exp(f[["fun"]](xx)))
    d <- d / d[length(d)]

    ## just spline-interpolate the mapping
    fq <- splinefun(xx, d, method = "hyman")

    ## just make sure the p's are in [0, 1]
    n <- length(q)
    xx <- pmin(pmax(q, rep(f[["range"]][[1L]], n)), rep(f[["range"]][[2L]], n))

    return(fq(xx))
}

`inla.qmarginal` <- function(p, marginal, len = 2048L) {
    f <- inla.sfmarginal(inla.smarginal(marginal))
    xx <- seq(f[["range"]][[1L]], f[["range"]][[2L]], length = len)
    d <- cumsum(exp(f[["fun"]](xx)))
    d <- d / d[length(d)]

    ## for the moment, we remove only duplicated zero's and one's.
    eps <- .Machine[["double.eps"]] * 1000.0
    for (val in c(0.0, 1.0)) {
        is.val <- which(abs(d - val) <= eps)
        if (length(is.val) > 1L) {
            ## keep the first, ie remove it from the list to the removed
            is.val <- is.val[-1L]
            ## and remove the rest
            d <- d[-is.val]
            xx <- xx[-is.val]
        }
    }

    ## just spline-interpolate the inverse mapping
    fq <- splinefun(d, xx, method = "hyman")

    ## just make sure the p's are in [0, 1]
    n <- length(p)
    pp <- pmin(pmax(p, rep(0.0, n)), rep(1.0, n))

    return(fq(pp))
}
`inla.hpdmarginal` <- function(p, marginal, len = 2048L) {
    sm <- inla.smarginal(marginal, keep.type = FALSE)
    f <- inla.sfmarginal(sm)
    x.range <- f[["range"]]
    xx <- seq(x.range[[1L]], x.range[[2L]], length = len)
    ## simply add 0 density outside the range.
    dx <- diff(xx)[[1L]]
    ef <- exp(f[["fun"]](xx))
    xx <- c(min(xx) - dx, xx, max(xx) + dx)
    d <- c(0.0, ef, 0.0)
    d <- cumsum(d)
    d <- d / d[length(d)]

    ## for the moment, we remove only duplicated zero's and one's.
    eps <- .Machine[["double.eps"]] * 1000.0
    for (val in c(0.0, 1.0)) {
        is.val <- which(abs(d - val) <= eps)
        if (length(is.val) > 1L) {
            ## keep the first, ie remove it from the list to the removed
            is.val <- is.val[-1L]
            ## and remove the rest
            d <- d[-is.val]
            xx <- xx[-is.val]
        }
    }
    ## just spline-interpolate the inverse mapping
    fq <- splinefun(d, xx, method = "hyman")

    ## just make sure the p's are in [0, 1]
    np <- length(p)
    pp <- 1.0 - pmin(pmax(p, rep(0.0, np)), rep(1.0, np))

    ## parts of this code below is taken from TeachingDemo::hpd
    f <- function(x, posterior.icdf, conf) {
        return(posterior.icdf(1.0 - conf + x) - posterior.icdf(x))
    }

    tol <- sqrt(.Machine[["double.eps"]])
    tol <- 1E-6
    result <- matrix(NA, np, 2L)
    for (i in 1L:np) {
        out <- optimize(f, c(0.0, pp[i]), posterior.icdf = fq, conf = pp[i], tol = tol)
        result[i, ] <- c(fq(out[["minimum"]]), fq(1.0 - pp[i] + out[["minimum"]]))
        ## make sure its in the range of the marginal itself as we expanded the domain
        result[i, ] <- c(
            min(x.range[[2L]], max(result[i, 1L], x.range[[1L]])),
            min(x.range[[2L]], max(result[i, 2L], x.range[[1L]]))
        )
    }
    colnames(result) <- c("low", "high")
    rownames(result) <- paste("level:", format(1.0 - pp, digits = 6L, justify = "left", trim = TRUE), sep = "")

    return(result)
}
`inla.rmarginal` <- function(n, marginal) {
    return(inla.qmarginal(runif(n), marginal))
}

`inla.marginal.transform` <- function(fun, marginal, n = 2048L, h.diff =
                                          .Machine[["double.eps"]]^(1.0 / 3.0),
                                      method = c("quantile", "linear")) {
    return(inla.tmarginal(fun, marginal, n, h.diff, method = method))
}

`inla.deriv.func` <- function(fun,
                              step.size = .Machine[["double.eps"]]^(1.0 / 4.0)) {
    ## return a function computing the derivatives to 'fun = function(x)' which must
    ## vectorize
    func <- match.fun(fun)
    if (inla.require("Deriv")) {
        fd <- try(Deriv::Deriv(func, names(formals(func))[[1L]]), silent = TRUE)
        if (!inherits(fd, "try-error")) {
            return(fd)
        }
    }
    if (inla.require("numDeriv")) {
        fd <- function(x) unlist(lapply(x, function(xi) numDeriv::grad(func, xi)))
    } else {
        fd <- function(x) {
            ((-func(x + 2.0 * step.size) + 8.0 * func(x + step.size) -
                8.0 * func(x - step.size) + func(x - 2.0 * step.size)) / (12.0 * step.size))
        }
    }

    return(fd)
}

`inla.tmarginal` <- function(fun, marginal, n = 2048L,
                             h.diff = .Machine[["double.eps"]]^(1.0 / 3.0),
                             method = c("quantile", "linear")) {
    ff <- match.fun(fun)

    is.mat <- is.matrix(marginal)
    m <- inla.smarginal(marginal)
    r <- range(m[["x"]])

    method <- match.arg(method)
    if (inla.strcasecmp(method, "quantile")) {
        x <- inla.qmarginal((1L:n) / (n + 1.0), marginal)
    } else if (inla.strcasecmp(method, "linear")) {
        x <- seq(r[[1L]], r[[2L]], length = n)
    } else {
        stop("unknown method")
    }
    xx <- ff(x)

    fd <- inla.deriv.func(ff)
    log.dens <- inla.dmarginal(x, marginal, log = FALSE) / abs(fd(x))

    ## if decreasing function, then reverse
    if (xx[[1L]] > xx[n]) {
        xx[1L:n] <- xx[n:1L]
        log.dens[1L:n] <- log.dens[n:1L]
    }

    if (is.mat) {
        ret <- cbind(x = xx, y = log.dens)
    } else {
        ret <- list(x = xx, y = log.dens)
    }

    if (FALSE) {
        class(ret) <- "inla.marginal"
        attr(ret, "inla.tag") <- paste(attr(marginal, "inla.tag"), "transformed")
    }

    return(ret)
}

`inla.mmarginal` <- function(marginal) {
    p <- seq(0.01, 0.99, by = 0.01)
    n <- length(p)
    x <- inla.qmarginal(p, marginal)
    d <- inla.dmarginal(x, marginal, log = TRUE)
    idx <- which.max(d)
    res <- optimize(
        f = inla.dmarginal,
        interval = c(x[max(1L, idx - 1L)], x[min(n, idx + 1L)]),
        maximum = TRUE,
        ## arguments to inla.dmarginal
        marginal = marginal, log = TRUE
    )
    return(res[["maximum"]])
}

`inla.zmarginal` <- function(marginal, silent = FALSE) {
    stopifnot(inla.is.marginal(marginal))

    m <- inla.emarginal(function(xx) c(xx, xx^2L), marginal)
    q <- inla.qmarginal(c(0.025, 0.25, 0.5, 0.75, 0.975), marginal)
    s <- sqrt(max(0.0, m[[2L]] - m[[1L]]^2L))

    if (!silent) {
        ## cat(as.character(match.call()[-1L]), "\n")
        ## cat("+--------+-----+------------\n")
        cat("Mean           ", format(m[[1L]], digits = 6L), "\n")
        cat("Stdev          ", format(s, digits = 6L), "\n")
        cat("Quantile  0.025", format(q[[1L]], digits = 6L), "\n")
        cat("Quantile  0.25 ", format(q[[2L]], digits = 6L), "\n")
        cat("Quantile  0.5  ", format(q[[3L]], digits = 6L), "\n")
        cat("Quantile  0.75 ", format(q[[4L]], digits = 6L), "\n")
        cat("Quantile  0.975", format(q[[5L]], digits = 6L), "\n")
    }
    return(invisible(list(
        mean = m[[1L]], sd = s, "quant0.025" = q[[1L]],
        "quant0.25" = q[[2L]], "quant0.5" = q[[3L]], "quant0.75" = q[[4L]], "quant0.975" = q[[5L]]
    )))
}

`inla.is.marginal` <- function(m) {
    return((is.matrix(m) && ncol(m) == 2L && nrow(m) > 2L) ||
        (is.list(m) && all(names(m) == c("x", "y"))))
}


## 'plot' and 'summary'-methods for marginals
plot.inla.marginal <- function(x, y, ...) {
    m <- inla.emarginal(function(xx) c(xx, xx^2L), x)
    xlab <- paste(
        "x (mean", format(m[[1L]], digits = 4L),
        ", stdev", format(sqrt(max(0.0, m[[2L]] - m[[1L]]^2L)), digits = 4L), ")"
    )
    plot(inla.smarginal(x, extrapolate = 0.0), type = "l", xlab = xlab, ylab = "marginal density", ...)

    return(invisible())
}
summary.inla.marginal <- function(object, ...) {
    ## option
    silent <- FALSE
    inla.eval.dots(...)

    m <- inla.emarginal(function(xx) c(xx, xx^2L), object)
    q <- inla.qmarginal(c(0.025, 0.25, 0.5, 0.75, 0.975), object)
    s <- sqrt(max(0.0, m[[2L]] - m[[1L]]^2L))

    if (!silent) {
        cat("Properties of: ", "FIXME", "\n")
        cat("+--------------+------------------\n")
        cat("Mean           ", format(m[[1L]], digits = 6L), "\n")
        cat("Stdev          ", format(s, digits = 6L), "\n")
        cat("Quantile  0.025", format(q[[1L]], digits = 6L), "\n")
        cat("Quantile  0.25 ", format(q[[2L]], digits = 6L), "\n")
        cat("Quantile  0.5  ", format(q[[3L]], digits = 6L), "\n")
        cat("Quantile  0.75 ", format(q[[4L]], digits = 6L), "\n")
        cat("Quantile  0.975", format(q[[5L]], digits = 6L), "\n")
    }
    return(invisible(list(
        mean = m[[1L]], sd = s, "quant0.025" = q[[1L]],
        "quant0.25" = q[[2L]], "quant0.5" = q[[3L]], "quant0.75" = q[[4L]], "quant0.975" = q[[5L]]
    )))
}

`inla.plot.inla.marginals` <- function(x, ...) {
    stop("NOT IN USE FOR THE MOMENT")
}

context("test 'likelihood poisson'")

test_that("Case 1", {
    set.seed(123)
    y <- 4
    Expect <- 0.25
    a <- 0; stopifnot(a==0)
    b <- 1.38
    pdat <- list(y = y, E = Expect)
    mod.pois <- inla(y ~ 1, 
                     family='poisson',
                     E=E,
                     data=pdat,
                     control.compute=list(mlik=T),
                     control.fixed=list(mean.intercept=a, prec.intercept=1/b^2),
                     control.inla = list(strategy = "laplace"))
    m <- mod.pois$marginals.fixed[[1]]
    t12 <- inla.emarginal(function(x) c(x, x^2), m)
    t1 <- t12[1]
    v1 <- t12[2]-t12[1]^2
    fac = 10
    xx = seq(t1 - fac*sqrt(v1), t1 + fac*sqrt(v1), length = 10000)
    dx = diff(xx)[1]
    ff = dpois(y, lambda = Expect * exp(xx)) * dnorm(xx, mean = a,  sd = b)
    ff = ff/max(ff)
    ff = ff / sum(ff) / dx
    mm = inla.smarginal(list(x = xx, y = ff))
    tt12 <- inla.emarginal(function(x) c(x, x^2), mm)
    tt1 <- tt12[1]
    vv1 <- tt12[2]-tt12[1]^2

    expect_true(abs(t1 - tt1) < 0.01)
    expect_true(abs(v1 - vv1) < 0.01)
    
    integral = integrate(
            function(x, y, Expect, a, b)
            dnorm(x, mean=a, sd = b) * dpois(y, lambda = Expect*exp(x)),
            lower = t1 - fac*sqrt(v1), upper = t1 + fac*sqrt(v1),
            y=y, Expect=Expect, a=a, b=b)
    true.mlik = log(integral$value)

    expect_true(abs(true.mlik - mod.pois$mlik[[1]]) < 0.02)
})

## Export: inla.list.models

## !\name{inla.list.models}
## !\alias{inla.list.models}
## !\alias{list.models}
## !\title{List available model components, likelihoods,  priors,  etc}
## !\description{
## !    List available model components, likelihoods,  priors,  etc.
## !    To read specific documentation for the individual elements, use \code{\link{inla.doc}}.
## !}
## !\usage{
## !    inla.list.models(section = names(inla.models()), ...)
## !}
## !\arguments{
## !   \item{section}{The section(s) to list, missing \code{section} will list all sections.
## !                  \code{names(inla.models())} lists available sections.}
## !   \item{...}{Additional argument to \code{cat}}
## !}
## !\details{
## !   The list is \code{cat}'ed with \code{...} arguments.
## !
## !   This function is EXPERIMENTAL.
## !}
## !\value{
## !   Nothing is returned
## !}
## !\author{Havard Rue}
## !\examples{
## !\dontrun{
## !inla.list.models("likelihood")
## !inla.list.models(c("prior", "group"))
## !inla.list.models(file=file("everything.txt"))
## !
## !#Show detailed doc for a specific prior/likelihood/latent model
## !inla.doc("binomial")
## !}
## !}


`inla.list.models` <- function(section = names(inla.models()), ...) {
    section <- sort(match.arg(section, several.ok = TRUE))
    res.tot <- NULL
    prefix <- "    "
    for (sec in section) {
        res.tot <- c(res.tot, paste0("Section [", sec, "]\n"))
        res <- NULL
        a <- inla.models()[[sec]]
        nm <- names(a)
        for (i in seq_along(a)) {
            res <- c(res, sprintf("%s%-30s%-40s\n", prefix, nm[i], a[[i]]$doc))
        }
        res.tot <- c(res.tot, res[order(nm)])
    }
    cat(res.tot, ...)
    return(invisible())
}

## Export: inla.pc.rcor1 inla.pc.dcor1 inla.pc.qcor1 inla.pc.pcor1

## ! \name{pc.cor1}
## ! \alias{inla.pc.cor1}
## ! \alias{pc.cor1}
## ! \alias{pc.rcor1}
## ! \alias{inla.pc.rcor1}
## ! \alias{pc.dcor1}
## ! \alias{inla.pc.dcor1}
## ! \alias{pc.pcor1}
## ! \alias{inla.pc.pcor1}
## ! \alias{pc.qcor1}
## ! \alias{inla.pc.qcor1}
## !
## ! \title{Utility functions for the PC prior for correlation in AR(1)}
## !
## ! \description{Functions to evaluate, sample, compute quantiles and
## !              percentiles of the PC prior for the correlation
## !              in the Gaussian AR(1) model where the base-model
## !              is correlation one.}
## ! \usage{
## ! inla.pc.rcor1(n, u, alpha, lambda)
## ! inla.pc.dcor1(cor, u, alpha, lambda, log = FALSE)
## ! inla.pc.qcor1(p, u, alpha, lambda)
## ! inla.pc.pcor1(q, u, alpha, lambda)
## ! }
## ! \arguments{
## !   \item{n}{Number of observations}
## !   \item{u}{The upper limit (see Details)}
## !   \item{alpha}{The probability going above the upper limit (see Details)}
## !   \item{lambda}{The rate parameter (see Details)}
## !   \item{cor}{Vector of correlations}
## !   \item{log}{Logical. Return the density in natural or log-scale.}
## !   \item{p}{Vector of probabilities}
## !   \item{q}{Vector of quantiles}
## ! }
## ! \details{
## !     The statement \code{Prob(cor > u) = alpha} is used to
## !     determine \code{lambda} unless \code{lambda} is given.
## !     Either \code{lambda}  must be given,  or
## !     \code{u} AND \code{alpha}.
## ! }
## !\value{%%
## !  \code{inla.pc.dcor1} gives the density,
## !  \code{inla.pc.pcor1} gives the distribution function,
## !  \code{inla.pc.qcor1} gives the quantile function, and
## !  \code{inla.pc.rcor1} generates random deviates.
## ! }
## ! \seealso{inla.doc("pc.rho1")}
## ! \author{Havard Rue \email{hrue@r-inla.org}}
## ! \examples{
## ! cor = inla.pc.rcor1(100,  lambda = 1)
## ! d = inla.pc.dcor1(cor, lambda = 1)
## ! cor = inla.pc.qcor1(c(0.3, 0.7), u = 0.5, alpha=0.75)
## ! inla.pc.pcor1(cor, u = 0.5, alpha=0.75)
## ! }

inla.pc.cor1.lambda <- function(u, alpha, lambda) {
    if (missing(lambda)) {
        stopifnot(!missing(u) && !missing(alpha))
        alpha.min <- sqrt((1 - u) / 2)
        if (!(alpha > alpha.min)) {
              stop(paste("inla.pc.cor1.lambda: alpha >", alpha.min))
          }
        fun <- function(lam, u, alpha) {
            F <- (1 - exp(-lam * sqrt(1 - u))) / (1 - exp(-lam * sqrt(2)))
            return((F - alpha)^2)
        }
        lambdas <- unique(c(seq(0.0000001, 10, len = 100), seq(10, 100, len = 10)))
        idx <- which.min(fun(lambdas, u, alpha))
        stopifnot(idx > 1 && idx < length(lambdas))
        lambda <- optimize(fun,
            interval = lambdas[c(idx - 1, idx + 1)],
            maximum = FALSE, u = u, alpha = alpha
        )$minimum
        stopifnot(fun(lambda, u, alpha) < 1e-8)
    }
    return(lambda)
}

inla.pc.rcor1 <- function(n, u, alpha, lambda) {
    lambda <- inla.pc.cor1.lambda(u, alpha, lambda)
    u <- runif(n)
    cor <- -1 / lambda * log(1 - u * (1 - exp(-sqrt(2) * lambda)))
    cor <- 1 - cor^2
    return(cor)
}

inla.pc.dcor1 <- function(cor, u, alpha, lambda, log = FALSE) {
    lambda <- inla.pc.cor1.lambda(u, alpha, lambda)
    log.dens <- log(lambda) - lambda * sqrt(1 - cor) -
        log(1 - exp(-sqrt(2) * lambda)) - log(2 * sqrt(1 - cor))
    return(inla.ifelse(log, log.dens, exp(log.dens)))
}

inla.pc.qcor1 <- function(p, u, alpha, lambda) {
    lambda <- inla.pc.cor1.lambda(u, alpha, lambda)
    pp <- -1 / lambda * log(1 - (1 - p) * (1 - exp(-lambda * sqrt(2))))
    q <- 1 - pp^2
    return(q)
}

inla.pc.pcor1 <- function(q, u, alpha, lambda) {
    lambda <- inla.pc.cor1.lambda(u, alpha, lambda)
    qq <- (1 - exp(-lambda * sqrt(1 - q))) / (1 - exp(-lambda * sqrt(2)))
    return(1 - qq)
}

## ----echo=FALSE,eval=TRUE,message=FALSE----
library(spatstat)
library(mvtnorm)
library(lattice)
library(mgcv)
library(pixmap)
library(numDeriv)
library(fields)
library(INLA)
set.seed(123)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
               fig.path='figures/scale-model/')

## ----echo=TRUE,eval=FALSE----------------
#  formula = y ~  f(idx, model = "rw2", scale.model = TRUE, hyper=..)

## ----echo=TRUE,eval=FALSE----------------
#  f(idx, model="..", group=g, control.group = list(model="rw2", scale.model=TRUE))

## ----echo=F,eval=TRUE--------------------
ss.rw1.sdref = function(values,tau) { 
    n = length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3) 
    y = NA
    
    formula = y ~ -1 + f(values, 
            model="rw1", 
            constr =TRUE, 
            values = values, 
            diagonal = 1e-10, 
            hyper = list(prec = list(
                                 initial = log(tau), 
                                 fixed=TRUE))) 
    r =inla(formula,data=data.frame(y,values),family = "gaussian",
            control.family = list(fixed =
                    TRUE), 
            control.compute=list(return.marginals=F),verbose=F)
    sd=r$summary.random$values$sd 
    return (sd) 
    }

ss.rw2.sdref= function(values,tau) { 
    n=length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3)
    y = NA #idx =1:n
    
    A1 = matrix(1, n, 1) 
    A2 = matrix(0, n, 1) 
    for(i in 1:n) { 
        A2[i, ]= i 
    }
    
    A = rbind(c(A1), c(A2)) 
    e = rep(0, 2) 
    extraconstr = list(A = A, e= e)
    
    formula = y ~ -1 + f( values, model="rw2", constr = FALSE,
            values=values, diagonal = 1e-10, extraconstr = extraconstr, hyper
            = list(prec = list(initial = log(tau), fixed=TRUE)))
    
    r = inla(formula, data = data.frame(y, values), family =
            "gaussian", control.family = list(fixed = TRUE),
            control.compute=list(return.marginals=F),verbose=F)
    
    sd=r$summary.random$values$sd 
    return (sd) 
}

n=100 
sd.rw1=ss.rw1.sdref(1:n,1) 
sd.rw2=ss.rw2.sdref(1:n,1)
##sd.rw1.scaled=ss.rw1.sdref(1:n,1) 
##sd.rw2.scaled=ss.rw2.sdref(1:n,1)

## ----fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'----
plot(sd.rw1,xlab="Node i", ylab="Marginal standard deviation") 

## ----fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'----
plot(sd.rw2,xlab="Node i",ylab="Marginal standard deviation") 

## ----echo=TRUE,eval=F--------------------
#  U = sqrt(b/qgamma(alpha,a,1))

## ----echo=FALSE,eval=TRUE----------------
func.u = function(a,b,alpha,sigma.ref) { 
    upper.limit = sqrt(b)*sigma.ref/sqrt(qgamma(alpha,a,1)) 
    return(upper.limit) 
}
a=1 
b=5*10^{-5} 
alpha=0.001 
upper.limit=func.u(a,b,alpha,1) 

## ----echo=F,eval=TRUE--------------------
func<-function(x,t)
{
    f.x=x/t+sin(2*pi* x/t)-0.5
    return(f.x)
}

f.est <- function(y,x,a,b,option)
{
    formula = y~1+f(x,model="rw2",hyper=list(prec=list(prior="loggamma",param=c(a,b))),scale.model=option)
    result = inla(formula,family="gaussian",data=data.frame(y,x))
    f.est=result$summary.random$x$mean
    return(f.est)
}

## ----echo=F,eval=TRUE--------------------
### Same values of the function, same observations, but  defined on different intervals ###
set.seed(89236)
x2=0:n
x1=x2/100
x3=x2*10

sigma=0.5
f.x=func(x2,length(x2))
y=f.x+rnorm(length(f.x),0,sigma)

a=1
b=0.00005

mat1=cbind(f.x,f.est(y,x1,a,b,T),f.est(y,x1,a,b,F))
mat2=cbind(f.x,f.est(y,x2,a,b,T),f.est(y,x2,a,b,F))
mat3=cbind(f.x,f.est(y,x3,a,b,T),f.est(y,x3,a,b,F))

# Generalized marginal variances  
v1=exp(mean(log(ss.rw2.sdref(x1,1)^2)))
v2=exp(mean(log(ss.rw2.sdref(x2,1)^2)))
v3=exp(mean(log(ss.rw2.sdref(x3,1)^2)))

u1=func.u(a,b,alpha,sqrt(v1))
u2=func.u(a,b,alpha,sqrt(v2))
u3=func.u(a,b,alpha,sqrt(v3))

## ----echo=TRUE,eval=F--------------------
#  formula = y ~  f(x, model = "rw2", scale.model = TRUE, hyper=...)

## ----echo=TRUE,eval=F--------------------
#  result = inla(formula, family = "gaussian", data = data.frame(y,x))

## ----echo=TRUE,eval=F--------------------
#  result$summary.random$x$mean

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'----
                                        # Estimate of f using unscaled and scaled model
r=range(c(mat1,mat2,mat3))
matplot(x1,mat1,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3) 
                                        #legend("topright",c("true f","scale.model=F","scale.model=T"),lty=c(1,2,4),col=c(1,2,4))
                                        #points(x,y)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'----
matplot(x2,mat2,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'----
matplot(x3,mat3,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)

## ----echo=TRUE,eval=F--------------------
#  hyper.prec = list(prec = list(prior = "pc.prec", param = c(0.5, 0.01)))

## ----echo=TRUE,eval=F--------------------
#  data(Munich)
#  g = system.file("demodata/munich.graph", package="INLA")
#  
#  ## Note that here we what to have an estimator of the effect of year
#  ## also the for years where we have no observation, therefore we give a
#  ## vector with all possible values assumed by the covariate year, that
#  ## is seq(1918,2001)
#  
#  formula = rent ~
#     f(location, model = "besag", graph = g, initial = 1, hyper = hyper.prec) +
#     f(year, model = "rw2", values = seq(1918,2001), hyper = hyper.prec) +
#     f(floor.size, model = "rw2", hyper = hyper.prec) +
#     Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
#     Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
#     zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1

## ----echo=TRUE,eval=F--------------------
#  mod  =  inla(formula, data = Munich, control.fixed=list(prec=1))

## ----echo=FALSE,eval=TRUE----------------
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

func.munich <- function(a,b,option)
{
    hyper.prec=list(prec=list(param=c(a,b)))
    formula= rent ~ f(location,model="besag",graph=g,initial=1, hyper=hyper.prec, scale.model=option) +
        f(year,model="rw2",values=seq(1918,2001), hyper=hyper.prec, scale.model=option) +
            f(floor.size,model="rw2", hyper=hyper.prec, scale.model=option) +
                Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
                    Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
                        zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
    
    result = inla(formula, data=Munich, control.fixed=list(prec=1))
}

## ----echo=FALSE,eval=TRUE----------------
a=1
b=0.01
u.munich=func.u(a,b,alpha,1)
mod=func.munich(a,b,FALSE)
mod.scaled=func.munich(a,b,TRUE)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'----
mat=cbind(mod$summary.random$year$mean,mod.scaled$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Year of construction",ylab="",lwd=2.5)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'----
mat=cbind(mod$summary.random$floor.size$mean,mod.scaled$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Floor size",ylab="",lwd=2.5)

## ----echo=FALSE,eval=TRUE----------------
alpha=0.001
#u.vec=c(0.01,1,10,100)
u.vec=c(0.001,0.1,10,30)
a.vec=c(1,1,1,1)
b.vec=u.vec^2*qgamma(alpha,a.vec,1)

option=TRUE
c1=func.munich(a.vec[1],b.vec[1],option)
c2=func.munich(a.vec[2],b.vec[2],option)
c3=func.munich(a.vec[3],b.vec[3],option)
c4=func.munich(a.vec[4],b.vec[4],option)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'----
mat=cbind(c1$summary.random$year$mean,c2$summary.random$year$mean,c3$summary.random$year$mean,c4$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Year of construction",ylab="",lwd=2.5)
legend("topleft",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)

## ----fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'----
mat=cbind(c1$summary.random$floor.size$mean,c2$summary.random$floor.size$mean,c3$summary.random$floor.size$mean,c4$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Floor size",ylab="",lwd=2.5)
legend("topright",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)

## ----echo=TRUE,eval=FALSE----------------
#  inla.setOption(scale.model.default = TRUE)

## Export: inla.cut

## ! \name{cut}
## ! \alias{inla.cut}
## ! \alias{cut}
## !
## ! \title{Group-wise model criticism using node-splitting}
## !
## ! \description{This function performs group-wise, cross-validatory
## ! model assessment for an INLA model using so-called node-splitting
## ! (Marshall and Spiegelhalter, 2007; Presanis et al, 2013).
## ! The user inputs an object of class \code{inla} (i.e. a result
## ! of a call to \code{inla()}) as well as  a variable name (\code{split.by}) specifying a grouping:
## ! Data points that share the same value of \code{split.by} are in the same group.
## ! The function then checks whether each group is an "outlier", or in conflict with
## ! the remaining groups, using the methodology described in Ferkingstad et al (2017).
## ! The result is a vector containing a p-value for each group, corresponding to a test
## ! for each group \code{i}, where the null hypothesis is that group \code{i} is
## ! consistent with the other groups except \code{i} (so a small p-value is evidence
## ! that the group is an "outlier"). See Ferkingstad et al (2017) for further details.
## ! }
## !
## ! \usage{
## !     inla.cut(result, split.by, mc.cores = NULL, debug=FALSE)
## ! }
## !
## ! \arguments{
## !   \item{result}{An object of class \code{inla}, i.e. a result of a call to \code{inla()}}
## !   \item{split.by}{The name of the variable to group by. Data points that have
## !                   the same value of \code{split.by} are in the same group.}
## !   \item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
## !                   \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
## !                   and \code{inla.getOption("num.threads")} in that order.}
## !   \item{debug}{Print debugging information if \code{TRUE}, default is \code{FALSE}}
## !  }
## !
## ! \value{
## !  A numeric vector of p-values, corresponding to a test
## ! for each group \code{i} where the null hypothesis is that group \code{i} is
## ! consistent with the other groups except \code{i}. A small p-value for a group
## ! indicates that the group is an "outlier" (in conflict with remaining groups).
## !
## !  This function is EXPERIMENTAL!!!
## ! }
## !
## ! \author{Egil Ferkingstad \email{egil.ferkingstad@gmail.com} and Havard Rue \email{hrue@r-inla.org}}
## !
## ! \examples{
## ! ## See http://www.r-inla.org/examples/case-studies/ferkingstad-2017 and Ferkingstad et al (2017).
## ! }
## !
## ! \references{
## ! Ferkingstad, E., Held, L. and Rue, H. (2017). Fast and accurate Bayesian
## ! model criticism and conflict diagnostics using R-INLA. arXiv preprint
## ! arXiv:1708.03272, available at http://arxiv.org/abs/1708.03272.
## ! Published in Stat, 6:331-344 (2017).
## !
## ! Marshall, E. C. and Spiegelhalter, D. J. (2007). Identifying outliers
## ! in Bayesian hierarchical models: a simulation-based approach.
## ! Bayesian Analysis, 2(2):409-444.
## !
## ! Presanis, A. M., Ohlssen, D., Spiegelhalter,
## ! D. J., De Angelis, D., et al. (2013). Conflict diagnostics
## ! in directed acyclic graphs, with applications in Bayesian evidence synthesis.
## ! Statistical Science, 28(3):376-397.
## ! }

inla.cut <- function(result, split.by, mc.cores = NULL, debug = FALSE) {
    my.debug <- function(...) if (debug) cat("*** inla.cut: ", ..., "\n")

    stopifnot(!missing(result))
    stopifnot(class(result) == "inla")
    stopifnot(!missing(split.by))
    formula <- result$.args$formula
    data <- result$.args$data
    if (!is.data.frame(data)) {
        stop(paste(match.call()[[1]], "is not yet implemented when 'data' is not a data.frame"))
    }

    ## find the name of the response, store the response and remove it from the data.frame.
    ## merge the two dataframes and add the reponse back in
    resp <- formula[[2]]
    stopifnot(is.name(resp))
    resp <- as.character(resp)

    ## extract the variable which define how to split
    n <- dim(data)[1]
    split.val <- data[, split.by == names(data)]
    split.uval <- sort(unique(split.val))
    split.len <- length(split.uval)
    stopifnot(split.len > 0)

    ## store inla-results
    res <- list()
    r <- NULL
    p.linpred <- c(unlist(
        inla.mclapply(
            seq_len(split.len),
            FUN = (function(split.idx) {
                idx <- (split.val == split.uval[split.idx])
                idx.num <- which(idx)

                ## First run REP version, everything except part i, "between" group:
                data.rep <- data
                data.rep[idx.num, resp] <- NA

                ## prepare the arguments for inla()
                args <- result$.args
                args$data <- data.rep
                args$formula <- formula
                cont.compute <- args$control.compute
                if (is.null(cont.compute)) cont.compute <- list()
                cont.compute$config <- TRUE
                cont.compute$return.marginals <- TRUE
                args$control.compute <- cont.compute

                ## make linear combinations for calculating linear predictor-based p-values:
                cont.inla <- args$control.inla
                cont.inla$lincomb.derived.correlation.matrix <- TRUE
                args$control.inla <- cont.inla
                n.pred <- length(idx.num)
                lc.rep <- c()
                for (i in 1:n.pred) {
                    pred.idx <- rep(NA, n)
                    pred.idx[idx.num[i]] <- 1
                    lci <- inla.make.lincomb(Predictor = pred.idx)
                    names(lci) <- paste0("lc.pred", i)
                    lc.rep <- c(lc.rep, lci)
                }
                args$lincomb <- lc.rep
                cont.pred <- args$control.predictor
                cont.pred$link <- 1
                args$control.predictor <- cont.pred
                r.rep <- do.call("inla", args = args)
                ## this case we do not do
                if (!is.null(r.rep$.args$control.predictor$A)) {
                    stop("A-matrix in the linear predictor is not supported")
                }
                mu.rep <- r.rep$summary.lincomb.derived$mean
                sigma.rep <- r.rep$misc$lincomb.derived.covariance.matrix
                ## Then run LIK version, group i, within-group:
                data.lik <- data[idx.num, ]
                if (!is.null(args$E)) args$E <- args$E[idx.num]
                if (!is.null(args$offset)) args$offset <- args$offset[idx.num]
                if (!is.null(args$scale)) args$scale <- args$scale[idx.num]
                if (!is.null(args$weights)) args$weights <- args$weights[idx.num]
                if (!is.null(args$Ntrials)) args$Ntrials <- args$Ntrials[idx.num]
                if (!is.null(args$strata)) args$strata <- args$strata[idx.num]
                if (!is.null(args$link.covariates)) args$link.covariates <- args$link.covariates[idx.num, ]

                args$data <- data.lik
                lc.lik <- c()
                ## make new linear combinations:
                lc.names <- paste0("lc.pred:", 1:n.pred)
                for (i in 1:n.pred) {
                    pred.idx <- rep(NA, nrow(data.lik))
                    pred.idx[i] <- 1
                    lci <- inla.make.lincomb(Predictor = pred.idx)
                    names(lci) <- lc.names[i]
                    lc.lik <- c(lc.lik, lci)
                }

                args$lincomb <- lc.lik
                ## set prior of hyperpar to posterior from REP version:
                args$control.update <- list(result = r.rep)
                r.lik <- do.call("inla", args = args)
                mu.lik <- r.lik$summary.lincomb.derived$mean
                sigma.lik <- r.lik$misc$lincomb.derived.covariance.matrix
                mu.diff <- mu.rep - mu.lik
                sigma.diff <- sigma.rep + sigma.lik
                respIdx <- which(names(data.lik) == resp)
                curr.data <- data.lik[, -respIdx]
                curr.data[is.na(curr.data)] <- 0
                ## curr.data: change any factors to numeric:
                for (kk in 1:ncol(curr.data)) {
                    if (is.factor(curr.data[, kk])) {
                        curr.data[, kk] <- as.numeric(as.character(curr.data[, kk]))
                    }
                }

                ## Calculate p-values from linear predictor:
                min.eigen.value <- 1E-3
                lin.pred.mu <- mu.diff
                lin.pred.sigma <- sigma.diff
                lin.pred.df <- sum(eigen(lin.pred.sigma)$values > min.eigen.value)
                lin.pred.Delta <- t(lin.pred.mu) %*% inla.ginv(lin.pred.sigma, tol = min.eigen.value) %*% lin.pred.mu
                pval <- 1 - pchisq(as.numeric(lin.pred.Delta), df = lin.pred.df)
                my.debug(split.idx, "of", split.len, ": p-value:", round(pval, 4), "df:", lin.pred.df)
                return(pval)
            }), mc.cores = mc.cores
        )
    ))

    return(p.linpred)
}

## Nothing to Export

`inla.create.data.file` <- function(
                                    y.orig = NULL,
                                    MPredictor = NULL,
                                    mf = NULL,
                                    scale = NULL,
                                    weights = NULL,
                                    E = NULL,
                                    Ntrials = NULL,
                                    strata = NULL,
                                    event = NULL,
                                    family = NULL,
                                    data.dir = NULL,
                                    file = NULL,
                                    debug = FALSE) {
    y.attr <- attr(y.orig, "inla.ncols", exact = TRUE)
    if (is.null(y.attr)) {
        y.attr <- 0
    }

    if (is.null(y.orig)) {
        y.orig <- c(mf[, 1L])
    } else if (is.inla.surv(y.orig)) {
        ## this is not passed into the inla-program
        y.orig$.special <- NULL
        y.orig <- as.data.frame(unclass(y.orig))
    } else if (is.inla.mdata(y.orig)) {
        y.orig <- as.data.frame(unclass(y.orig))
    } else {
        y.orig <- as.data.frame(y.orig)
    }
    n.data <- dim(y.orig)[1L]
    ind <- seq(0L, n.data - 1L)

    if (debug) {
        cat("inla.create.data.file: n.data = ", n.data, "\n")
    }

    if (!is.null(weights) && !is.function(weights)) {
        if (length(weights) == 1L) {
            weights <- rep(weights, n.data)
        }
        if (length(weights) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste("Length of 'weights' has to be the same as the length of the response:", length(weights), n.data))
        }
    }

    if (inla.one.of(family, c(
        "gaussian",
        "normal",
        "lognormal",
        "t",
        "sn",
        "gev",
        "logistic",
        "circularnormal",
        "wrappedcauchy",
        "iidgamma",
        "simplex",
        "gamma",
        "beta",
        "tweedie",
        "fmri"))) {
        if (is.null(scale)) {
            scale <- rep(1.0, n.data)
        }
        if (length(scale) == 1L) {
            scale <- rep(scale, n.data)
        }

        if (length(scale) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste("Length of scale has to be the same as the length of the response:", length(scale), n.data))
        }

        response <- cbind(ind, scale, y.orig)
        null.dat <- is.na(response[, 3L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("tstrata"))) {
        if (is.null(scale)) {
            scale <- rep(1.0, n.data)
        }
        if (length(scale) == 1L) {
            scale <- rep(scale, n.data)
        }

        if (is.null(strata)) {
            strata <- rep(1L, n.data)
        }
        if (length(strata) == 1L) {
            strata <- rep(strata, n.data)
        }

        stopifnot(all(!is.na(strata)))
        stopifnot(all(as.integer(strata) == strata))
        stopifnot(all(strata > 0L))
        if (length(scale) != n.data || length(strata) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste(
                "Length of scale and strata has to be the same as the length of the response:",
                length(scale), length(strata), n.data
            ))
        }

        ## strata goes from 0L to nstrata-1L,  therefore subtract 1L.
        response <- cbind(ind, scale, as.integer(strata) - 1L, y.orig)
        null.dat <- is.na(response[, 4L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c(
        "poisson",
        "cenpoisson",
        "gammacount",
        "gpoisson",
        "nbinomial",
        "xpoisson",
        "zeroinflatedcenpoisson0",
        "zeroinflatedcenpoisson1",
        "zeroinflatednbinomial0",
        "zeroinflatednbinomial1",
        "zeroinflatednbinomial2",
        "zeroinflatedpoisson0",
        "zeroinflatedpoisson1",
        "zeroinflatedpoisson2",
        "poisson.special1"))) {
        if (is.null(E)) {
            E <- rep(1.0, n.data)
        }
        if (length(E) == 1L) {
            E <- rep(E, n.data)
        }

        response <- cbind(ind, E, y.orig)

        if (length(E) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste("Length of E has to be the same as the length of the response:", length(E), n.data))
        }

        null.dat <- is.na(response[, 3L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("exponential", "weibull", "loglogistic", "gammajw", "gompertz"))) {
        response <- cbind(ind, y.orig)
        null.dat <- is.na(response[, 2L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("zeroinflatednbinomial1strata2", "zeroinflatednbinomial1strata3"))) {
        if (is.null(E)) {
            E <- rep(1.0, n.data)
        }
        if (length(E) == 1L) {
            E <- rep(E, n.data)
        }

        if (is.null(strata)) {
            strata <- rep(1L, n.data)
        }
        if (length(strata) == 1L) {
            strata <- rep(strata, n.data)
        }

        stopifnot(all(!is.na(strata)))
        stopifnot(all(strata %in% 1:10))

        response <- cbind(ind, E, strata - 1L, y.orig)

        if (length(E) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste(
                "Length of E has to be the same as the length of the response:",
                length(E), n.data
            ))
        }

        if (length(strata) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste(
                "Length of strata has to be the same as the length of the response:",
                length(strata), n.data
            ))
        }

        null.dat <- is.na(response[, 4L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("xbinomial"))) {
        if (is.null(scale)) {
            scale <- rep(1.0, n.data)
        }
        if (length(scale) == 1L) {
            scale <- rep(scale, n.data)
        }

        if (is.null(Ntrials)) {
            Ntrials <- rep(1L, n.data)
        }
        if (length(Ntrials) == 1L) {
            Ntrials <- rep(Ntrials, n.data)
        }

        response <- cbind(ind, Ntrials, scale, y.orig)
        null.dat <- is.na(response[, 4L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(
        family,
        c(
            "binomial",
            "binomialtest",
            "betabinomial",
            "nbinomial2",
            "zeroinflatedbinomial0",
            "zeroinflatedbinomial1",
            "zeroinflatedbinomial2",
            "zeroninflatedbinomial2",
            "zeroninflatedbinomial3",
            "zeroinflatedbetabinomial0",
            "zeroinflatedbetabinomial1",
            "zeroinflatedbetabinomial2"
        )
    )) {
        if (is.null(Ntrials)) {
            Ntrials <- rep(1L, n.data)
        }
        if (length(Ntrials) == 1L) {
            Ntrials <- rep(Ntrials, n.data)
        }

        response <- cbind(ind, Ntrials, y.orig)
        null.dat <- is.na(response[, 3L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("betabinomialna"))) {
        if (is.null(Ntrials)) {
            Ntrials <- rep(1L, n.data)
        }
        if (length(Ntrials) == 1L) {
            Ntrials <- rep(Ntrials, n.data)
        }

        if (is.null(scale)) {
            scale <- rep(1.0, n.data)
        }
        if (length(scale) == 1L) {
            scale <- rep(scale, n.data)
        }

        if (length(scale) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste("Length of scale has to be the same as the length of the response:", length(scale), n.data))
        }

        response <- cbind(ind, Ntrials, scale, y.orig)
        null.dat <- is.na(response[, 4L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("cbinomial"))) {
        if (!(is.matrix(Ntrials) && all(dim(Ntrials) == c(n.data, 2)))) {
            stop(paste(
                "Argument 'Ntrials' for family='cbinomial' must be a", n.data,
                "x", 2, "-matrix; see the documentation."
            ))
        }
        response <- cbind(ind, Ntrials, y.orig)
        null.dat <- is.na(response[, 4L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c(
        "exponentialsurv", "weibullsurv", "weibullcure",
        "loglogisticsurv", "qloglogisticsurv", "lognormalsurv",
        "gammasurv", "gammajwsurv", "fmrisurv", "gompertzsurv"))) {
        if (!inla.model.properties(family, "likelihood")$survival) {
            file.remove(file)
            file.remove(data.dir)
            stop("This should not happen.")
        }
        if (is.null(y.orig$time)) {
            file.remove(file)
            file.remove(data.dir)
            stop("Responce does not contain variable `time'.")
        }
        len <- length(y.orig$time)

        if (is.null(y.orig$truncation)) {
            y.orig$truncation <- rep(0, len)
        }
        if (is.null(y.orig$lower)) {
            y.orig$lower <- rep(0, len)
        }
        if (is.null(y.orig$upper)) {
            y.orig$upper <- rep(Inf, len)
        }
        if (is.null(y.orig$event)) {
            y.orig$event <- rep(1, len)
        }

        idx <- !is.na(y.orig$time)
        response <- cbind(
            ind[idx],
            y.orig$event[idx],
            y.orig$truncation[idx],
            y.orig$lower[idx],
            y.orig$upper[idx],
            y.orig$time[idx]
        )
        if (any(is.na(response))) {
            file.remove(file)
            file.remove(data.dir)
            stop("NA in truncation/event/lower/upper/time is not allowed")
        }
    } else if (inla.one.of(family, c(
        "stochvol", "stochvolt", "stochvolnig", "stochvolsn", "loggammafrailty",
        "iidlogitbeta", "qkumar", "qloglogistic", "gp", "dgp", "pom",
        "logperiodogram"))) {
        response <- cbind(ind, y.orig)
        null.dat <- is.na(response[, 2L])
        response <- response[!null.dat, ]
    } else if (inla.one.of(family, c("nmix", "nmixnb"))) {
        if (inla.one.of(family, "nmix")) {
            mmax <- length(inla.model.properties(model = family, section = "likelihood")$hyper)
        } else if (inla.one.of(family, "nmixnb")) {
            ## remove the overdispersion parameter
            mmax <- length(inla.model.properties(model = family, section = "likelihood")$hyper) - 1
        } else {
            stop("This should not happen.")
        }

        response <- cbind(IDX = ind, y.orig)
        col.idx <- grep("^IDX$", names(response))
        col.x <- grep("^X[0-9]+", names(response))
        col.y <- grep("^Y[0-9]+", names(response))
        m.x <- length(col.x)
        m.y <- length(col.y)
        stopifnot(m.x >= 1 && m.x <= mmax)

        ## remove entries with NA's in all responses
        na.y <- apply(response[, col.y, drop = FALSE], 1, function(x) all(is.na(x)))
        response <- response[!na.y, , drop = FALSE]

        X <- response[, col.x, drop = FALSE]
        Y <- response[, col.y, drop = FALSE]
        idx <- response[, col.idx, drop = FALSE]
        yfake <- rep(-1, nrow(Y))

        ## replace NA's in the covariates with 0's
        X[is.na(X)] <- 0
        ## augment X til the maximum allowed,  padding with NA's
        X <- cbind(X, matrix(NA, nrow = nrow(response), ncol = mmax - m.x))

        ## sort each row so that the NA's are at the end. Sort numerically the non-NA's as well
        ## although it is not required
        Y <- matrix(c(apply(Y, 1, function(x) c(sort(x[!is.na(x)]), x[is.na(x)]))), ncol = ncol(Y), byrow = TRUE)
        response <- cbind(idx, X, Y, yfake)
    } else if (inla.one.of(family, c("agaussian"))) {
        response <- cbind(IDX = ind, y.orig)
        col.idx <- grep("^IDX$", names(response))
        col.y <- grep("^Y[0-9]+", names(response))
        m.y <- length(col.y)
        stopifnot(m.y == 5L)
        ## remove entries with NA's in all responses
        na.y <- apply(response[, col.y, drop = FALSE], 1, function(x) all(is.na(x)))
        response <- response[!na.y, , drop = FALSE]
        Y <- response[, col.y, drop = FALSE]
        idx <- response[, col.idx, drop = FALSE]
        response <- cbind(idx, Y)
    } else if (inla.one.of(family, c("cenpoisson2"))) {

        if (is.null(E)) {
            E <- rep(1, n.data)
        }
        if (length(E) == 1L) {
            E <- rep(E, n.data)
        }

        response <- cbind(ind, E, y.orig)
        stopifnot(ncol(response) == 5)
        null.dat <- is.na(response[, 3L])
        response <- response[!null.dat, ]
        colnames(response) <- c("IDX", "E", "Y1", "Y2", "Y3")
        idx.inf <- (is.infinite(response$Y3) | (response$Y3 < 0))
        response[idx.inf, "Y3"] <- -1 ## code for infinite
        idx.inf <- (is.infinite(response$Y2) | (response$Y2 < 0))
        response[idx.inf, "Y2"] <- -1 ## code for infinite
        col.idx <- grep("^IDX$", names(response))
        col.y <- grep("^Y[0-9]+", names(response))
        m.y <- length(col.y)
        stopifnot(m.y == 3L)
        ## remove entries with NA's in all responses
        na.y <- apply(response[, col.y, drop = FALSE], 1, function(x) all(is.na(x)))
        response <- response[!na.y, , drop = FALSE]
        ## format: IDX, E, LOW, HIGH, Y
        response <- cbind(IDX = response$IDX, E = response$E, LOW = response$Y2, HIGH = response$Y3, Y = response$Y1)
    } else if (inla.one.of(family, c("bgev"))) {
        if (is.null(scale)) {
            scale <- rep(1.0, n.data)
        }
        if (length(scale) == 1L) {
            scale <- rep(scale, n.data)
        }

        if (length(scale) != n.data) {
            file.remove(file)
            file.remove(data.dir)
            stop(paste("Length of scale has to be the same as the length of the response:", length(scale), n.data))
        }

        mmax <- length(inla.model.properties(model = family, section = "likelihood")$hyper) - 2L
        response <- cbind(IDX = ind, y.orig)
        col.idx <- grep("^IDX$", names(response))
        col.x <- grep("^X[0-9]+", names(response))
        col.y <- grep("^Y[0-9]+", names(response))
        m.x <- length(col.x)
        m.y <- length(col.y)

        ## remove entries with NA's in all responses
        na.y <- apply(response[, col.y, drop = FALSE], 1, function(x) all(is.na(x)))
        response <- response[!na.y, , drop = FALSE]
        scale <- scale[!na.y]

        X <- response[, col.x, drop = FALSE]
        Y <- response[, col.y, drop = FALSE]
        stopifnot(ncol(Y) == 1)
        idx <- response[, col.idx, drop = FALSE]
        ## replace NA's in the covariates with 0's
        X[is.na(X)] <- 0

        response <- cbind(idx, scale, X, Y)
        ## fix attr, so the order corresponds to (X, Y) and not (Y, X) as in the inla.mdata() input.
        ## y.attr[1] is number of attributes
        stopifnot(y.attr[1] > 0)
        if (y.attr[1] == 1) {
            y.attr <- c(3, y.attr[2], 0, 0)
        } else if (y.attr[1] == 2) {
            y.attr <- c(3, y.attr[2], y.attr[3], 0)
        } else if (y.attr[1] == 3) {
            y.attr <- c(3, y.attr[2], y.attr[3], y.attr[4])
        }
        y.attr <- c(y.attr[1], y.attr[-c(1, 2)], y.attr[2])
    } else {
        file.remove(file)
        file.remove(data.dir)
        stop(paste("Family", family, ", not recognised in 'create.data.file.R'"))
    }

    file.data <- inla.tempfile(tmpdir = data.dir)
    inla.write.fmesher.file(as.matrix(response), filename = file.data, debug = debug)
    file.data <- gsub(data.dir, "$inladatadir", file.data, fixed = TRUE)

    file.weights <- inla.tempfile(tmpdir = data.dir)
    if (!is.null(weights) && !is.function(weights)) {
        inla.write.fmesher.file(as.matrix(weights), filename = file.weights, debug = debug)
    } else {
        file.create(file.weights)
    }
    file.weights <- gsub(data.dir, "$inladatadir", file.weights, fixed = TRUE)

    file.attr <- inla.tempfile(tmpdir = data.dir)
    inla.write.fmesher.file(as.matrix(y.attr, ncol = 1), filename = file.attr, debug = debug)
    file.attr <- gsub(data.dir, "$inladatadir", file.attr, fixed = TRUE)

    return(list(file.data = file.data, file.weights = file.weights, file.attr = file.attr))
}

## Internal: internal.update.crs
## Internal: inla.internal.sp2segment.join
## Internal: inla.crs.transform.oblique
## Internal: inla.rotmat3213
## Internal: inla.rotmat3123
## Internal: inla.crs.graticule
## Internal: inla.crs.tissot
## Internal: internal.clip
## Internal: inla.crs.bounds
## Internal: inla.crs.bounds.check




#' PROJ6 detection
#'
#' Detect whether PROJ6 is available for INLA
#'
#' @aliases inla.has_PROJ6 inla.not_for_PROJ6 inla.not_for_PROJ4
#' inla.fallback_PROJ6 inla.requires_PROJ6
#' @details `inla.has_PROJ6` is called to check if PROJ6&GDAL3 are available.
#' @return For `inla.has_PROJ6`, logical; `TRUE` if PROJ6 is available,
#' `FALSE` otherwise
#' @examples
#'
#' inla.has_PROJ6()
#' @export inla.has_PROJ6
#' @rdname inla.has_PROJ6
inla.has_PROJ6 <- function() {
    inla.require("rgdal", stop.on.error = TRUE)
    result <- tryCatch(rgdal::new_proj_and_gdal(),
        error = function(e) FALSE
    )
    result
}

#' @describeIn inla.has_PROJ6 Called to warn about using old PROJ4
#' features even though PROJ6 is available
#' @param fun The name of the calling function

inla.not_for_PROJ6 <- function(fun) {
    if (inla.has_PROJ6()) {
        stack <- sys.calls()
        stack <- lapply(as.list(stack), function(x) as.character(deparse(x)))
        warning(paste0(
            "'",
            fun,
            "' should not be used with PROJ6 and rgdal v3\n",
            "Call stack for developer debugging:\n",
            paste0(lapply(stack, function(x) substr(x, 1, 70)),
                collapse = "\n"
            )
        ))
    }
}

#' @describeIn inla.has_PROJ6 Called to give an error when
#' calling methods that are only available for PROJ6

inla.not_for_PROJ4 <- function(fun) {
    if (!inla.has_PROJ6()) {
        stop(paste0(
            "'",
            fun,
            "' is not supported for PROJ4"
        ))
    }
}

#' @describeIn inla.has_PROJ6 Called to warn about falling back
#' to using old PROJ4 methods when a RPOJ6 method hasn't been implemented

inla.fallback_PROJ6 <- function(fun) {
    if (inla.has_PROJ6()) {
        warning(paste0(
            "'",
            fun,
            "' method for PROJ6 not implemented. Falling back to PROJ4."
        ))
    }
}

#' @describeIn inla.has_PROJ6 Called to give an error when PROJ6
#' is required but not available

inla.requires_PROJ6 <- function(fun) {
    if (!inla.has_PROJ6()) {
        stop(paste0(
            "'",
            fun,
            "' requires PROJ6/RGDAL3"
        ))
    }
}






#' @title Extract CRS information
#'
#' @description
#' Wrapper for CRS(projargs) (PROJ4) and CRS(wkt) for `sp::Spatial`
#' objects.
#'
#' This function is a convenience method to workaround PROJ4/PROJ6 differences,
#' and the lack of a crs extraction method for Spatial objects.
#'
#' @param x A `sp::Spatial` object
#' @return A `CRS` object, or NULL if no valid CRS identified
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @examples
#'
#' \dontrun{
#' if (interactive()) {
#'     s <- sp::SpatialPoints(matrix(1:6, 3, 2), proj4string = inla.CRS("sphere"))
#'     inla.sp_get_crs(s)
#' }
#' }
#'
#' @export inla.sp_get_crs
inla.sp_get_crs <- function(x) {
    if (is.null(x)) {
        return(NULL)
    }
    if (inla.has_PROJ6()) {
        suppressWarnings(crs <- CRS(SRS_string = sp::wkt(x)))
    } else {
        crs <- CRS(proj4string(x))
    }
    crs
}





internal.clip <- function(bounds, coords, eps = 0.05) {
    ## Clip 2D coordinate matrix of polylines and generate a list of Line objects
    ## bounds is from inla.crs.bounds
    ## This implementation only removes "long" line segments.
    thelines <- list()
    ## Rudimentary cutting:
    toolong <-
        which(c(
            TRUE,
            (diff(coords[, 1]) / diff(bounds$xlim))^2
                + (diff(coords[, 2]) / diff(bounds$ylim))^2
            > eps^2,
            TRUE
        ))
    start <- toolong[-length(toolong)]
    ending <- toolong[-1] - 1
    for (i in seq_along(start)) {
        if (start[i] < ending[i]) {
            thelines <-
                c(
                    thelines,
                    list(sp::Line(coords[start[i]:ending[i], 1:2]))
                )
        }
    }
    thelines
}




#' Handling CRS/WKT
#'
#' Get and set CRS object or WKT string properties.
#'
#'
#' @aliases inla.wkt_is_geocent inla.crs_is_geocent
#' inla.wkt_get_ellipsoid_radius inla.crs_get_ellipsoid_radius
#' inla.wkt_set_ellipsoid_radius inla.crs_set_ellipsoid_radius
#' inla.wkt_unit_params crs_wkt inla.wkt_get_lengthunit inla.wkt_set_lengthunit
#' inla.crs_get_wkt inla.crs_get_lengthunit inla.crs_set_lengthunit
#' @param wkt A WKT2 character string
#' @param crs A `sp::CRS` or `inla.CRS` object
#' @param unit character, name of a unit. Supported names are "metre",
#' "kilometre", and the aliases "meter", "m", International metre",
#' "kilometer", and "km", as defined by `inla.wkt_unit_params` or the
#' `params` argument. (For legacy PROJ4 use, only "m" and "km" are
#' supported)
#' @param params Length unit definitions, in the list format produced by
#' `inla.wkt_unit_params()`, Default: NULL, which invokes
#' `inla.wkt_unit_params()`
#' @return For `inla.wkt_unit_params`, a list of named unit definitions
#'
#' For `inla.wkt_get_lengthunit`, a list of length units used in the wkt
#' string, excluding the ellipsoid radius unit.
#'
#' For `inla.wkt_set_lengthunit`, a WKT2 string with altered length units.
#' Note that the length unit for the ellipsoid radius is unchanged.
#'
#' For `inla.crs_get_wkt`, WKT2 string.
#'
#' For `inla.crs_get_lengthunit`, a list of length units used in the wkt
#' string, excluding the ellipsoid radius unit. (For legacy PROJ4 code, the raw
#' units from the proj4string are returned, if present.)
#'
#' For `inla.crs_set_lengthunit`, a `sp::CRS` object with altered
#' length units. Note that the length unit for the ellipsoid radius is
#' unchanged.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.sp_get_crs()]
#' @examples
#'
#' \dontrun{
#' if (inla.has_PROJ6()) {
#'     c1 <- inla.CRS("globe")
#'     inla.crs_get_lengthunit(c1)
#'     c2 <- inla.crs_set_lengthunit(c1, "km")
#'     inla.crs_get_lengthunit(c2)
#' }
#' }
#'
#' @rdname crs_wkt
#' @export inla.wkt_is_geocent
inla.wkt_is_geocent <- function(wkt) {
    if (is.null(wkt) || identical(wkt, "")) {
        return(FALSE)
    }
    # See https://proceedings.esri.com/library/userconf/proc17/tech-workshops/tw_2588-212.pdf
    geo_crs_items <- c(
        "GEODCRS", "GEOGCRS",
        "BASEGEODCRS", "BASEGEOGCRS"
    )
    wt <- inla.as.wkt_tree.wkt(wkt)
    if (identical(wt[["label"]], "BOUNDCRS")) {
        wt <- inla.wkt_tree_get_item(wt, "SOURCECRS")
        wt <- inla.wkt_tree_get_item(wt, c("PROJCRS", geo_crs_items))
    }
    if (identical(wt[["label"]], "PROJCRS")) {
        wt <- inla.wkt_tree_get_item(wt, geo_crs_items)
    }
    if (!(wt[["label"]] %in% geo_crs_items)) {
        return(FALSE)
    }
    cs <- inla.wkt_tree_get_item(wt, "CS")
    if (is.null(cs)) {
        return(FALSE)
    }
    cart <- ((cs[["params"]][[1]] == "Cartesian") &&
        (cs[["params"]][[2]] == "3"))
    if (!cart) {
        return(FALSE)
    }
    axis_names <- c('"(X)"', '"(Y)"', '"(Z)"')
    axis_types <- c("geocentricX", "geocentricY", "geocentricZ")
    for (k in seq_len(3)) {
        axis <- inla.wkt_tree_get_item(wt, "AXIS", k)
        if (!((axis[["params"]][[1]] == axis_names[k]) &&
            (axis[["params"]][[2]] == axis_types[k]))) {
            return(FALSE)
        }
    }
    TRUE
}

#' @export
#' @rdname crs_wkt

inla.crs_is_geocent <- function(crs) {
    if (inla.has_PROJ6()) {
        wkt <- inla.crs_get_wkt(crs)
        result <- inla.wkt_is_geocent(wkt)
    } else {
        args <- inla.as.list.CRS(crs)
        result <- identical(args[["proj"]], "geocent")
    }
    result
}


#' @rdname crs_wkt
#' @export

inla.wkt_get_ellipsoid_radius <- function(wkt) {
    geo_crs_items <- c(
        "GEODCRS", "GEOGCRS",
        "BASEGEODCRS", "BASEGEOGCRS"
    )
    wt <- inla.as.wkt_tree.wkt(wkt)

    if (identical(wt[["label"]], "BOUNDCRS")) {
        wt <- inla.wkt_tree_get_item(wt, "SOURCECRS")
        wt <- inla.wkt_tree_get_item(wt, c("PROJCRS", geo_crs_items))
    }
    if (identical(wt[["label"]], "PROJCRS")) {
        wt <- inla.wkt_tree_get_item(wt, geo_crs_items)
    }
    if (is.null(wt) || !(wt[["label"]] %in% geo_crs_items)) {
        stop("Ellipsoid settings not found")
    }

    datum <- inla.wkt_tree_get_item(wt, "DATUM")
    if (is.null(datum)) {
        stop("Ellipsoid settings not found")
    }
    ellipsoid <- inla.wkt_tree_get_item(datum, "ELLIPSOID")
    if (is.null(ellipsoid)) {
        stop("Ellipsoid settings not found")
    }
    as.numeric(ellipsoid[["params"]][[2]])
}

#' @rdname crs_wkt
#' @export

inla.crs_get_ellipsoid_radius <- function(crs) {
    inla.requires_PROJ6("inla.crs_get_ellipsoid_radius")

    inla.wkt_get_ellipsoid_radius(inla.crs_get_wkt(crs))
}


#' @rdname crs_wkt
#' @export

inla.wkt_set_ellipsoid_radius <- function(wkt, radius) {
    geo_crs_items <- c(
        "GEODCRS", "GEOGCRS",
        "BASEGEODCRS", "BASEGEOGCRS"
    )

    set_radius <- function(wt) {
        if (is.null(wt)) {
            stop("Ellipsoid settings not found")
        } else if (wt[["label"]] %in% geo_crs_items) {
            datum <- inla.wkt_tree_get_item(wt, "DATUM")
            null_datum <- is.null(datum)
            if (is.null(datum)) {
                stop("Ellipsoid settings not found")
            }
            ellipsoid <- inla.wkt_tree_get_item(datum, "ELLIPSOID")
            if (is.null(ellipsoid)) {
                stop("Ellipsoid settings not found")
            }
            ellipsoid[["params"]][[2]] <- as.character(radius)
            datum <- inla.wkt_tree_set_item(datum, ellipsoid)
            wt <- inla.wkt_tree_set_item(wt, datum)
        } else if (wt[["label"]] %in% c("BOUNDCRS", "SOURCECRS", "PROJCRS")) {
            wt_sub <- inla.wkt_tree_get_item(
                wt,
                c(
                    "BOUNDCRS", "SOURCECRS", "PROJCRS",
                    geo_crs_items
                )
            )
            if (is.null(wt_sub)) {
                stop("Ellipsoid settings not found")
            }
            wt_sub_new <- set_radius(wt_sub)
            wt <- inla.wkt_tree_set_item(wt, wt_sub_new)
        } else {
            stop("Ellipsoid settings not found")
        }
        wt
    }

    wt <- inla.as.wkt_tree.wkt(wkt)
    wt <- set_radius(wt)
    inla.as.wkt.wkt_tree(wt)
}

#' @rdname crs_wkt
#' @export

inla.crs_set_ellipsoid_radius <- function(crs, radius) {
    inla.requires_PROJ6("inla.crs_set_ellipsoid_radius")

    wkt <- inla.crs_get_wkt(crs)
    wkt <- inla.wkt_set_ellipsoid_radius(wkt, radius)
    new_crs <- inla.CRS(SRS_string = wkt)
    if (inherits(crs, "inla.CRS")) {
        crs$crs <- new_crs
        crs
    } else {
        new_crs
    }
}





inla.crs.graticule <- function(x, by = c(15, 15, 45), add = FALSE, do.plot = TRUE,
                               eps = 0.05, ...) {
    inla.fallback_PROJ6("inla.crs.graticule")

    ## Graticule
    if (is.null(by)) {
        return(invisible(list()))
    }
    if (length(by) < 2) {
        by <- by[c(1, 1, 1)]
    } else if (length(by) < 3) {
        by <- by[c(1, 2, 1)]
    }
    n <- c(floor(180 / by[1]), ceiling(90 / by[2]) - 1, floor(180 / by[3]))
    bounds <- inla.crs.bounds(x)
    if (by[1] > 0) {
        special.poles <- (by[1] != by[3]) && (by[2] > 0)
        lon <- ((1 - n[1]):n[1]) * by[1]
        if (special.poles) {
            lat <- seq(-n[2] * by[2], n[2] * by[2], length = 91)
        } else {
            lat <- seq(-90 + 1e-6, 90 - 1e-6, length = 91)
        }
        meridians <- as.matrix(expand.grid(lat, lon)[, 2:1])
        proj.mer.coords <- inla.spTransform(meridians, inla.CRS("longlat"), x)
        proj.mer.coords1 <- matrix(
            proj.mer.coords[, 1], length(lat),
            length(lon)
        )
        proj.mer.coords2 <- matrix(
            proj.mer.coords[, 2], length(lat),
            length(lon)
        )

        mer.coords <-
            unlist(lapply(
                seq_along(lon),
                function(k) {
                    internal.clip(bounds, cbind(
                        proj.mer.coords1[, k, drop = FALSE],
                        proj.mer.coords2[, k, drop = FALSE]
                    ),
                    eps = eps
                    )
                }
            ),
            recursive = FALSE
            )

        if (special.poles) {
            if (by[3] > 0) {
                lon <- ((1 - n[3]):n[3]) * by[3]
                lat <- seq(-90 + 1e-6, -n[2] * by[2], length = ceiling((90 - n[2] * by[2]) / 2) + 1)
                meridians <- as.matrix(expand.grid(lat, lon)[, 2:1])
                proj.mer.coords <- inla.spTransform(meridians, inla.CRS("longlat"), x)
                proj.mer.coords1 <- matrix(
                    proj.mer.coords[, 1], length(lat),
                    length(lon)
                )
                proj.mer.coords2 <- matrix(
                    proj.mer.coords[, 2], length(lat),
                    length(lon)
                )
                mer.coords <-
                    c(
                        mer.coords,
                        unlist(lapply(
                            seq_along(lon),
                            function(k) {
                                internal.clip(bounds, cbind(
                                    proj.mer.coords1[, k, drop = FALSE],
                                    proj.mer.coords2[, k, drop = FALSE]
                                ),
                                eps = eps
                                )
                            }
                        ),
                        recursive = FALSE
                        )
                    )

                lat <- seq(n[2] * by[2], 90 - 1e-6, length = ceiling((90 - n[2] * by[2]) / 2) + 1)
                meridians <- as.matrix(expand.grid(lat, lon)[, 2:1])
                proj.mer.coords <- inla.spTransform(meridians, inla.CRS("longlat"), x)
                proj.mer.coords1 <- matrix(
                    proj.mer.coords[, 1], length(lat),
                    length(lon)
                )
                proj.mer.coords2 <- matrix(
                    proj.mer.coords[, 2], length(lat),
                    length(lon)
                )
                mer.coords <-
                    c(
                        mer.coords,
                        unlist(lapply(
                            seq_along(lon),
                            function(k) {
                                internal.clip(bounds, cbind(
                                    proj.mer.coords1[, k, drop = FALSE],
                                    proj.mer.coords2[, k, drop = FALSE]
                                ),
                                eps = eps
                                )
                            }
                        ),
                        recursive = FALSE
                        )
                    )
            }
        }

        proj.mer <-
            sp::SpatialLines(list(sp::Lines(mer.coords, ID = "meridians")),
                proj4string = CRS(inla.CRSargs(x))
            )
        if (do.plot) {
            args <- list(x = proj.mer, ...)
            args <- args[intersect(
                names(args),
                union(
                    names(formals(plot.default)),
                    union(
                        names(formals(sp:::plot.Spatial)),
                        names(formals(sp:::plotSpatialLines))
                    )
                )
            )]
            if (add) {
                do.call(plot, c(list(add = TRUE), args))
            } else {
                do.call(plot, args)
                add <- TRUE
            }
        }
    } else {
        proj.mer <- NULL
    }
    if (by[2] > 0) {
        lon <- seq(-180 + 1e-6, 180 - 1e-6, length = 181)
        lat <- ((-n[2]):n[2]) * by[2]
        parallels <- as.matrix(expand.grid(lon, lat))
        proj.par.coords <- inla.spTransform(parallels, inla.CRS("longlat"), x)
        proj.par.coords1 <- matrix(
            proj.par.coords[, 1], length(lon),
            length(lat)
        )
        proj.par.coords2 <- matrix(
            proj.par.coords[, 2], length(lon),
            length(lat)
        )
        proj.par <-
            sp::SpatialLines(list(sp::Lines(
                unlist(lapply(
                    seq_along(lat),
                    function(k) {
                        internal.clip(bounds, cbind(
                            proj.par.coords1[, k],
                            proj.par.coords2[, k]
                        ), eps = eps)
                    }
                ),
                recursive = FALSE
                ),
                ID = "parallels"
            )),
            proj4string = CRS(inla.CRSargs(x))
            )
        if (do.plot) {
            args <- list(x = proj.par, ...)
            args <- args[intersect(
                names(args),
                union(
                    names(formals(plot.default)),
                    union(
                        names(formals(sp:::plot.Spatial)),
                        names(formals(sp:::plotSpatialLines))
                    )
                )
            )]
            if (add) {
                do.call(plot, c(list(add = TRUE), args))
            } else {
                do.call(plot, args)
            }
        }
    } else {
        proj.par <- NULL
    }
    invisible(list(meridians = proj.mer, parallels = proj.par))
}

inla.crs.tissot <- function(x, by = c(30, 30, 30), add = FALSE, do.plot = TRUE,
                            eps = 0.05, diff.eps = 1e-2, ...) {
    inla.fallback_PROJ6("inla.crs.graticule")

    if (is.null(by)) {
        return(invisible(list()))
    }
    if (length(by) < 2) {
        by <- c(by[1], by[1], 30)
    } else if (length(by) < 3) {
        by <- c(by[1:2], 30)
    }
    bounds <- inla.crs.bounds(x)
    n <- c(floor(180 / by[1]), ceiling(90 / by[2]) - 1)
    lon <- ((1 - n[1]):n[1]) * by[1]
    lat <- ((-n[2]):n[2]) * by[2]
    loc0.lon <- loc0.lat <- loc0 <-
        cbind(as.matrix(expand.grid(lat, lon)[, 2:1]), 0)
    loc0.lon[, 1] <- loc0.lon[, 1] + diff.eps / cos(loc0.lat[, 2] * pi / 180)
    loc0.lat[, 2] <- loc0.lat[, 2] + diff.eps
    crs.longlat <- inla.CRS("longlat")

    loc1 <- inla.spTransform(loc0, crs.longlat, x)
    loc1.lon <- inla.spTransform(loc0.lon, crs.longlat, x)
    loc1.lat <- inla.spTransform(loc0.lat, crs.longlat, x)
    ok <- (rowSums(is.na(loc1)) +
        rowSums(is.na(loc1.lon)) +
        rowSums(is.na(loc1.lat)) == 0)
    loc1 <- loc1[ok, , drop = FALSE]
    loc1.lon <- loc1.lon[ok, , drop = FALSE]
    loc1.lat <- loc1.lat[ok, , drop = FALSE]

    diff.lon <- (loc1.lon - loc1) / eps
    diff.lat <- (loc1.lat - loc1) / eps

    scale <- by[3]
    theta <- seq(0, 2 * pi, length = 181)
    ct <- cos(theta) * scale
    st <- sin(theta) * scale

    collection <-
        sp::SpatialLines(list(sp::Lines(
            unlist(lapply(
                seq_len(nrow(loc1)),
                function(k) {
                    loc1.ellipse <-
                        cbind(
                            loc1[k, 1] + diff.lon[k, 1] * ct + diff.lat[k, 1] * st,
                            loc1[k, 2] + diff.lon[k, 2] * ct + diff.lat[k, 2] * st
                        )
                    internal.clip(bounds, loc1.ellipse, eps = eps)
                }
            ),
            recursive = FALSE
            ),
            ID = "parallels"
        )),
        proj4string = CRS(inla.CRSargs(x))
        )
    if (do.plot) {
        args <- list(x = collection, ...)
        args <- args[intersect(
            names(args),
            union(
                names(formals(plot.default)),
                union(
                    names(formals(sp:::plot.Spatial)),
                    names(formals(sp:::plotSpatialLines))
                )
            )
        )]
        if (add) {
            do.call(plot, c(list(add = TRUE), args))
        } else {
            do.call(plot, args)
        }
    }

    invisible(list(tissot = collection))
}



#' Plot CRS and inla.CRS objects
#'
#' Plot the outline of a CRS or inla.CRS projection, with optional graticules
#' (transformed parallels and meridians) and Tissot indicatrices.
#'
#'
#' @aliases plot.inla.CRS plot.CRS
#' @param x A `CRS` or [inla.CRS()] object.
#' @param xlim Optional x-axis limits.
#' @param ylim Optional y-axis limits.
#' @param outline Logical, if `TRUE`, draw the outline of the projection.
#' @param graticule Vector of length at most 3, to plot meridians with spacing
#' `graticule[1]` degrees and parallels with spacing `graticule[2]`
#' degrees. `graticule[3]` optionally specifies the spacing above and
#' below the first and last parallel.  When `graticule[1]==0` no meridians
#' are drawn, and when `graticule[2]==0` no parallels are drawn. Use
#' `graticule=NULL` to skip drawing a graticule.
#' @param tissot Vector of length at most 3, to plot Tissot's indicatrices with
#' spacing `tissot[1]` degrees and parallels with spacing `tissot[2]`
#' degrees. `tissot[3]` specifices a scaling factor.  Use
#' `tissot=NULL` to skip drawing a Tissot's indicatrices.
#' @param asp The aspect ratio for the plot, default 1.
#' @param add If `TRUE`, add the projecton plot to an existing plot.
#' @param eps Clipping tolerance for rudimentary boundary clipping
#' @param \dots Additional arguments passed on to the internal calls to
#' `plot` and `lines`.
#' @author Finn Lindgren <finn.lindgren@@gmail.com>
#' @seealso [inla.CRS()]
#' @examples
#'
#' if (require(rgdal)) {
#'     oblique <- c(0, 45, 45, 0)
#'     for (projtype in c("longlat", "lambert", "mollweide", "hammer")) {
#'         plot(inla.CRS(projtype), main = projtype)
#'         plot(inla.CRS(projtype, oblique = oblique), main = paste("oblique", projtype))
#'     }
#' }
#' @method plot inla.CRS
#' @export
plot.inla.CRS <- function(x, xlim = NULL, ylim = NULL,
                          outline = TRUE,
                          graticule = c(15, 15, 45),
                          tissot = c(30, 30, 30),
                          asp = 1,
                          add = FALSE,
                          eps = 0.05,
                          ...) {
    inla.fallback_PROJ6("plot.inla.CRS")

    bounds <- inla.crs.bounds(x)
    if (is.null(xlim)) xlim <- bounds$xlim
    if (is.null(ylim)) ylim <- bounds$ylim
    if (!add) {
        args <- list(x = NA, type = "n", xlim = xlim, ylim = ylim, asp = asp, ...)
        args <- args[intersect(names(args), names(formals(plot.default)))]
        do.call(plot, args)
    }
    ## Outline
    if (outline) {
        args <- list(x = bounds$polygon, ...)
        args <- args[intersect(names(args), union(
            names(formals(lines.default)),
            names(formals(plot.xy))
        ))]
        do.call(lines, args)
    }
    ## Graticule
    inla.crs.graticule(x, by = graticule, add = TRUE, do.plot = TRUE, eps = eps, ...)
    ## Tissot
    inla.crs.tissot(x, by = tissot, add = TRUE, do.plot = TRUE, eps = eps, ...)
    invisible(NULL)
}

#' @export
#' @method plot CRS
#' @rdname plot.inla.CRS
plot.CRS <- function(x, xlim = NULL, ylim = NULL,
                     outline = TRUE,
                     graticule = c(15, 15, 45),
                     tissot = c(30, 30, 30),
                     asp = 1,
                     add = FALSE,
                     eps = 0.05,
                     ...) {
    invisible(plot.inla.CRS(x,
        xlim = xlim, ylim = ylim,
        outline = outline, graticule = graticule, tissot = tissot,
        asp = asp, add = add, ...
    ))
}




#' Create a coordinate reference system object
#'
#' Creates either a CRS object or an inla.CRS object, describing a coordinate
#' reference system
#'
#' The first two elements of the `oblique` vector are the (longitude,
#' latitude) coordinates for the oblique centre point. The third value
#' (orientation) is a counterclockwise rotation angle for an observer looking
#' at the centre point from outside the sphere. The fourth value is the
#' quasi-longitude (orbit angle) for a rotation along the oblique observers
#' equator.
#'
#' Simple oblique: `oblique=c(0, 45)`
#'
#' Polar: `oblique=c(0, 90)`
#'
#' Quasi-transversal: `oblique=c(0, 0, 90)`
#'
#' Satellite orbit viewpoint: `oblique=c(lon0-time*v1, 0, orbitangle,
#' orbit0+time*v2)`, where `lon0` is the longitude at which a satellite
#' orbit crosses the equator at `time=0`, when the satellite is at an
#' angle `orbit0` further along in its orbit.  The orbital angle relative
#' to the equatorial plane is `orbitangle`, and `v1` and `v2`
#' are the angular velocities of the planet and the satellite, respectively.
#' Note that "forward" from the satellite's point of view is "to the right" in
#' the projection.
#'
#' When `oblique[2]` or `oblique[3]` are non-zero, the resulting
#' projection is only correct for perfect spheres.
#'
#' @aliases inla.CRS inla.wkt_predef
#' @param projargs Either 1) a projection argument string suitable as input to
#' `sp::CRS`, or 2) an existing `CRS` object, or 3) a shortcut
#' reference string to a predefined projection; run
#' `names(inla.wkt_predef())` for valid predefined projections.
#' @param doCheckCRSArgs default TRUE, must be set to FALSE by package
#' developers including `CRS` in an S4 class definition to avoid
#' uncontrollable loading of the `rgdal` namespace.
#' @param args An optional list of name/value pairs to add to and/or override
#' the PROJ4 arguments in `projargs`.  `name=value` is converted to
#' `"+name=value"`, and `name=NA` is converted to `"+name"`.
#' @param oblique Vector of length at most 4 of rotation angles (in degrees)
#' for an oblique projection, all values defaulting to zero. The values
#' indicate (longitude, latitude, orientation, orbit), as explained in the
#' Details section below.
#' @param SRS_string a WKT2 string defining the coordinate system; see
#' `sp::CRS`. This takes precedence over `projargs`.
#' @param \dots Additional parameters. Not currently in use.
#' @return Either an `sp::CRS` object or an `inla.CRS` object,
#' depending on if the coordinate reference system described by the parameters
#' can be expressed with a pure `sp::CRS` object or not.
#'
#' An S3 `inla.CRS` object is a list, usually (but not necessarily)
#' containing at least one element: \item{crs }{The basic `sp::CRS`
#' object}
#'
#' `inla.wkt_predef` returns a WKT2 string defining a projection
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [sp::CRS()], [crs_wkt()],
#' [inla.sp_get_crs()] [plot.CRS()],
#' [inla.identical.CRS()]
#' @examples
#'
#'
#' if (require(rgdal)) {
#'     crs1 <- inla.CRS("longlat_globe")
#'     crs2 <- inla.CRS("lambert_globe")
#'     crs3 <- inla.CRS("mollweide_norm")
#'     crs4 <- inla.CRS("hammer_globe")
#'     crs5 <- inla.CRS("sphere")
#'     crs6 <- inla.CRS("globe")
#' }
#' \dontrun{
#' names(inla.wkt_predef())
#' }
#'
#' @export inla.CRS
inla.CRS <- function(projargs = NULL, doCheckCRSArgs = TRUE,
                     args = NULL, oblique = NULL,
                     SRS_string = NULL,
                     ...) {
    if (inla.has_PROJ6()) {
        # PROJ6
        if (identical(projargs, "")) {
            projargs <- NULL
        }
        if (is.null(SRS_string) &&
            !is.null(projargs) &&
            !is.na(projargs) &&
            is.character(projargs)) {
            if (projargs %in% c("hammer", "lambert", "longlat", "mollweide")) {
                warning(paste0(
                    "Use of old predefined projection '",
                    projargs,
                    "' is deprecated. Converting to '",
                    projargs,
                    "_norm'"
                ))
                projargs <- paste0(projargs, "_norm")
            }
            predef <- inla.wkt_predef()
            if (projargs %in% names(predef)) {
                SRS_string <- predef[[projargs]]
                projargs <- NULL
            } else {
                warning("'inla.CRS' should be given a SRS_string for PROJ6 or a known keyword for a predefined string given in projargs. Using fallback PROJ4 method.")
                if (!is.null(args)) {
                    x <- CRS(projargs, doCheckCRSArgs = doCheckCRSArgs)
                    if (typeof(args) != "list") {
                        stop("'args' must be NULL or a list of name=value pairs.")
                    }
                    xargs <- inla.as.list.CRS(x)
                    for (name in names(args)) {
                        xargs[[name]] <- args[[name]]
                    }
                    projargs <- inla.as.CRSargs.list(xargs)
                }
                SRS_string <- rgdal::showSRID(projargs, multiline = "NO")
                projargs <- NULL
            }
        }

        if (!is.null(SRS_string)) {
            if (!is.null(projargs)) {
                warning("SRS_string specified. Ignoring non-null projargs.")
            }
            x <- CRS(SRS_string = SRS_string, doCheckCRSArgs = doCheckCRSArgs)
        } else if (inherits(projargs, "CRS")) {
            x <- projargs
        } else {
            x <- CRS(NA_character_, doCheckCRSArgs = doCheckCRSArgs)
        }
    } else {
        # PROJ4
        if (is.null(projargs)) {
            projargs <- NA_character_
        }
        halfroot <- "+a=0.7071067811865476 +b=0.7071067811865476"
        predef <- list(
            hammer = paste("+proj=hammer +ellps=sphere +units=m", halfroot),
            lambert = "+proj=cea +ellps=sphere +lat_ts=0 +units=m +a=1 +b=1",
            longlat = "+proj=longlat +ellps=sphere +a=1 +b=1",
            mollweide = paste("+proj=moll +ellps=sphere +units=m", halfroot),
            sphere = "+proj=geocent +ellps=sphere +a=1 +b=1 +units=m",
            globe = "+proj=geocent +ellps=sphere +units=m"
        )
        if (is.character(projargs)) {
            if (projargs %in% names(predef)) {
                projargs <- predef[[projargs]]
            }
            x <- CRS(projargs, doCheckCRSArgs = doCheckCRSArgs)
        } else if (inherits(projargs, "CRS")) {
            x <- projargs
        } else {
            stop(paste(
                "Unsupported projargs input class",
                paste(class(projargs), collapse = ",")
            ))
        }
        if (!is.null(args)) {
            if (typeof(args) != "list") {
                stop("'args' must be NULL or a list of name=value pairs.")
            }
            xargs <- inla.as.list.CRS(x)
            for (name in names(args)) {
                xargs[[name]] <- args[[name]]
            }
            x <- CRS(inla.as.CRSargs.list(xargs), doCheckCRSArgs = doCheckCRSArgs)
        }
    }

    if (!is.null(oblique)) {
        stopifnot(is.vector(oblique))
        if (length(oblique) < 4) {
            oblique <- c(oblique, rep(0, 4 - length(oblique)))
        }
        x <- list(crs = x, oblique = oblique)
        class(x) <- "inla.CRS"
    }
    x
}

#' @return `inla.wkt_predef` returns a WKT2 string defining a projection
#' @examples
#' \dontrun{
#' names(inla.wkt_predef())
#' }
#' @export
#' @rdname inla.CRS

inla.wkt_predef <- function() {
    list(
        hammer_norm = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["unknown",ELLIPSOID["unknown",0.707106781186548,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Reference meridian",0,ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]]],CONVERSION["unknown",METHOD["PROJ hammer"]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["metre",1,ID["EPSG",9001]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["metre",1,ID["EPSG",9001]]]]',
        lambert_norm = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["unknown",ELLIPSOID["unknown",1,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Reference meridian",0,ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]]],CONVERSION["unknown",METHOD["Lambert Cylindrical Equal Area (Spherical)",ID["EPSG",9834]],PARAMETER["Latitude of 1st standard parallel",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8823]],PARAMETER["Longitude of natural origin",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8802]],PARAMETER["False easting",0,LENGTHUNIT["metre",1],ID["EPSG",8806]],PARAMETER["False northing",0,LENGTHUNIT["metre",1],ID["EPSG",8807]]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["metre",1,ID["EPSG",9001]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["metre",1,ID["EPSG",9001]]]]',
        longlat_norm = 'GEOGCRS["unknown",DATUM["unknown",ELLIPSOID["unknown",1,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Reference meridian",0,ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]],CS[ellipsoidal,2],AXIS["longitude",east,ORDER[1],ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]],AXIS["latitude",north,ORDER[2],ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]]]',
        mollweide_norm = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["unknown",ELLIPSOID["unknown",0.707106781186548,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Reference meridian",0,ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]]],CONVERSION["unknown",METHOD["Mollweide"],PARAMETER["Longitude of natural origin",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8802]],PARAMETER["False easting",0,LENGTHUNIT["metre",1],ID["EPSG",8806]],PARAMETER["False northing",0,LENGTHUNIT["metre",1],ID["EPSG",8807]]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["metre",1,ID["EPSG",9001]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["metre",1,ID["EPSG",9001]]]]',
        hammer_globe = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["Unknown based on Normal Sphere (r=6370997) ellipsoid",ELLIPSOID["Normal Sphere (r=6370997)",6370997,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Greenwich",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8901]]],CONVERSION["unknown",METHOD["PROJ hammer"]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]]]',
        lambert_globe = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["Unknown based on Normal Sphere (r=6370997) ellipsoid",ELLIPSOID["Normal Sphere (r=6370997)",6370997,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Greenwich",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8901]]],CONVERSION["unknown",METHOD["Lambert Cylindrical Equal Area (Spherical)",ID["EPSG",9834]],PARAMETER["Latitude of 1st standard parallel",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8823]],PARAMETER["Longitude of natural origin",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8802]],PARAMETER["False easting",0,LENGTHUNIT["kilometre",1000],ID["EPSG",8806]],PARAMETER["False northing",0,LENGTHUNIT["kilometre",1000],ID["EPSG",8807]]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]]]',
        longlat_globe = 'GEOGCRS["unknown",DATUM["Unknown based on Normal Sphere (r=6370997) ellipsoid",ELLIPSOID["Normal Sphere (r=6370997)",6370997,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Greenwich",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8901]],CS[ellipsoidal,2],AXIS["longitude",east,ORDER[1],ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]],AXIS["latitude",north,ORDER[2],ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]]]',
        mollweide_globe = 'PROJCRS["unknown",BASEGEOGCRS["unknown",DATUM["Unknown based on Normal Sphere (r=6370997) ellipsoid",ELLIPSOID["Normal Sphere (r=6370997)",6370997,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Greenwich",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8901]]],CONVERSION["unknown",METHOD["Mollweide"],PARAMETER["Longitude of natural origin",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8802]],PARAMETER["False easting",0,LENGTHUNIT["kilometre",1000],ID["EPSG",8806]],PARAMETER["False northing",0,LENGTHUNIT["kilometre",1000],ID["EPSG",8807]]],CS[Cartesian,2],AXIS["(E)",east,ORDER[1],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]],AXIS["(N)",north,ORDER[2],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]]]',
        sphere = 'GEODCRS["unknown",DATUM["unknown",ELLIPSOID["unknown",1,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Reference meridian",0,ANGLEUNIT["degree",0.0174532925199433,ID["EPSG",9122]]],CS[Cartesian,3],AXIS["(X)",geocentricX,ORDER[1],LENGTHUNIT["metre",1,ID["EPSG",9001]]],AXIS["(Y)",geocentricY,ORDER[2],LENGTHUNIT["metre",1,ID["EPSG",9001]]],AXIS["(Z)",geocentricZ,ORDER[3],LENGTHUNIT["metre",1,ID["EPSG",9001]]]]',
        globe = 'GEODCRS["unknown",DATUM["Unknown based on Normal Sphere (r=6370997) ellipsoid",ELLIPSOID["Normal Sphere (r=6370997)",6370997,0,LENGTHUNIT["metre",1,ID["EPSG",9001]]]],PRIMEM["Greenwich",0,ANGLEUNIT["degree",0.0174532925199433],ID["EPSG",8901]],CS[Cartesian,3],AXIS["(X)",geocentricX,ORDER[1],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]],AXIS["(Y)",geocentricY,ORDER[2],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]],AXIS["(Z)",geocentricZ,ORDER[3],LENGTHUNIT["kilometre",1000,ID["EPSG",9036]]]]'
    )
}





#' Internal WKT handling
#'
#' Conversion between WKT and a tree representation
#'
#'
#' @aliases inla.as.wkt_tree.wkt inla.as.wkt.wkt_tree inla.wkt_tree_get_item
#' inla.wkt_tree_set_item
#' @param x A WKT2 string, or a `wkt_tree` list structure
#' @param \dots Unused
#' @param item character vector with item labels identifying a parameter item
#' entry.
#' @param duplicate For items that have more than one match, `duplicate`
#' indicates the index number of the desired version. Default: 1
#' @param item_tree An item tree identifying a parameter item entry
#' @export inla.as.wkt_tree.wkt
#' @rdname wkt_tree
inla.as.wkt_tree.wkt <- function(x, ...) {
    inla.requires_PROJ6("inla.as.wkt_tree.wkt")

    # Basic parsing of WKT string
    # ITEM[Param1, Param2, ...]
    # Param can be a constant or an ITEM[...]
    parse_item <- function(x) {
        # Parse item label
        n <- regexpr("\\[", x)
        item_label <- substr(x, 1, n - 1)
        x <- substr(x, n + 1, nchar(x))
        params <- list()
        # Parse parameters
        done <- FALSE
        while (!done) {
            # If [ comes before , or ], it's an Item, otherwise a constant
            n <- regexpr("[],[]", x)
            if (n < 1) {
                # Nothing found
                done <- TRUE
                break
            }
            if (substr(x, n, n) == "[") {
                n <- regexpr("[^ ]", x)
                x <- substr(x, n, nchar(x))
                item_info <- parse_item(x)
                params[[length(params) + 1]] <- item_info$item
                x <- item_info$x
            } else {
                const <- substr(x, 1, n - 1)
                params[[length(params) + 1]] <- const
                x <- substr(x, n, nchar(x))
            }
            n <- regexpr("[],]", x)
            done <- (substr(x, n, n) == "]")
            x <- substr(x, n + 1, nchar(x))
        }
        list(item = list(label = item_label, params = params), x = x)
    }

    x <- gsub("\n", "", x)
    item_info <- parse_item(x)
    item <- item_info$item
    item
}

#' @rdname wkt_tree
#' @export

inla.as.wkt.wkt_tree <- function(x, pretty = FALSE, ...) {
    inla.requires_PROJ6("inla.as.wkt.wkt_tree")

    construct_item <- function(x, level) {
        paste0(
            if (pretty) {
                paste0(rep("    ", level), collapse = "")
            } else {
                ""
            },
            x[["label"]],
            "[",
            paste0(vapply(
                x[["params"]],
                function(param) {
                    if (!is.list(param)) {
                        paste0(param)
                    } else {
                        paste0(
                            if (pretty) {
                                "\n"
                            } else {
                                ""
                            },
                            construct_item(param,
                                level = level + 1
                            )
                        )
                    }
                },
                ""
            ),
            collapse = ","
            ),
            "]"
        )
    }
    construct_item(x, 0)
}

#' @param item character vector with item labels identifying a parameter item
#' entry.
#' @param duplicate For items that have more than one match, `duplicate`
#' indicates the index number of the desired version. Default: 1
#' @rdname wkt_tree
#' @export

inla.wkt_tree_get_item <- function(x, item, duplicate = 1) {
    for (k in seq_along(x[["params"]])) {
        if (is.list(x[["params"]][[k]]) &&
            (!is.null(x[["params"]][[k]][["label"]])) &&
            (x[["params"]][[k]][["label"]] %in% item)) {
            if (duplicate == 1) {
                return(x[["params"]][[k]])
            }
            duplicate <- duplicate - 1
        }
    }
    NULL
}

#' @param item_tree An item tree identifying a parameter item entry
#' @rdname wkt_tree
#' @export

inla.wkt_tree_set_item <- function(x, item_tree, duplicate = 1) {
    success <- FALSE
    for (k in seq_along(x[["params"]])) {
        if (is.list(x[["params"]][[k]]) && (x[["params"]][[k]][["label"]] == item_tree[["label"]])) {
            if (duplicate == 1) {
                x[["params"]][[k]] <- item_tree
            }
            duplicate <- duplicate - 1
            success <- TRUE
            break
        }
    }
    if (!success) {
        x[["params"]] <- c(x[["params"]], list(item_tree))
    }
    x
}



#' Show expanded CRS arguments
#'
#' Wrapper for `sp::CRS` and `inla.CRS` objects to extract the
#' coordinate reference system argument string.
#' Should no longer be used with PROJ6/rgdal3;
#' see [inla.crs_get_wkt()]
#'
#' @aliases inla.CRSargs inla.as.list.CRS inla.as.list.CRSargs inla.as.CRS.list
#' inla.as.CRSargs.list
#' @param x An `sp::CRS` or `inla.CRS` object (for
#' `inla.CRSargs` and `inla.as.list.CRS`), a character string (for
#' `inla.as.list.CRSargs`), or a list (for `inla.as.CRS.list` and
#' `inla.as.CRSargs.list`).
#' @param \dots Additional arguments passed on to other methods.
#' @return For `inla.CRSargs` and `inla.as.CRSargs.list`, a character
#' string with PROJ.4 arguments.
#'
#' For `inla.as.list.CRS` and `inla.as.list.CRSargs`, a list of
#' name/value pairs.
#'
#' For `inla.as.CRS.list`, a `CRS` or `inla.CRS` object.
#' @author Finn Lindgren <finn.lindgren@@gmail.com>
#' @seealso [rgdal::CRSargs()], [inla.CRS()]
#' @examples
#'
#' if (require(rgdal)) {
#'     crs0 <- inla.CRS("longlat")
#'     p4s <- inla.CRSargs(crs0)
#'     lst <- inla.as.list.CRSargs(p4s)
#'     crs1 <- inla.as.CRS.list(lst)
#'     lst$a <- 2
#'     crs2 <- inla.CRS(p4s, args = lst)
#'     print(inla.CRSargs(crs0))
#'     print(inla.CRSargs(crs1))
#'     print(inla.CRSargs(crs2))
#' }
#' @export
#' @rdname CRSargs
inla.CRSargs <- function(x, ...) {
    inla.not_for_PROJ6("inla.CRSargs")

    if (inherits(x, "inla.CRS")) {
        x <- x[["crs"]]
    }
    if (is.null(x)) {
        as.character(NA)
    } else {
        inla.require("rgdal", stop.on.error = TRUE)
        rgdal::CRSargs(x)
    }
}


#' @details * `inla.as.CRSargs.list`: CRS proj4 string for name=value pair list
#'
#' @export
#' @rdname CRSargs
inla.as.CRSargs.list <- function(x, ...) {
    paste(lapply(
        names(x),
        function(xx) {
            if (is.na(x[[xx]])) {
                paste("+", xx, sep = "")
            } else {
                paste("+", xx, "=", x[[xx]], sep = "")
            }
        }
    ),
    collapse = " "
    )
}

#' @details * `inla.as.list.CRSargs`: List of name=value pairs from CRS proj4 string
#'
#' @export
#' @rdname CRSargs
inla.as.list.CRSargs <- function(x, ...) {
    inla.not_for_PROJ6("inla.as.list.CRSargs")

    if (is.na(x)) {
        return(list())
    }
    if (!is.character(x)) {
        stop("proj4string must be of class character")
    }
    do.call(c, lapply(
        strsplit(
            x = strsplit(
                x = paste(" ", x, sep = ""),
                split = " \\+"
            )[[1]][-1],
            split = "="
        ),
        function(x) {
            xx <- list(x[2])
            names(xx) <- x[1]
            xx
        }
    ))
}


#' @export
#' @rdname CRSargs
inla.as.list.CRS <- function(x, ...) {
    inla.not_for_PROJ6("inla.as.list.CRS")
    inla.as.list.CRSargs(inla.CRSargs(x))
}

#' @export
#' @rdname CRSargs
inla.as.CRS.list <- function(x, ...) {
    inla.not_for_PROJ6("inla.as.CRS.list")
    inla.CRS(args = x)
}





#' @param crs A `sp::CRS` or `inla.CRS` object
#' @param wkt A WKT2 character string
#' @param unit character, name of a unit. Supported names are
#' "metre", "kilometre", and the aliases "meter", "m", International metre",
#' "kilometer", and "km", as defined by `inla.wkt_unit_params` or the
#' `params` argument. (For legacy PROJ4 use, only "m" and "km" are
#' supported)
#' @param params Length unit definitions, in the list format produced by
#' `inla.wkt_unit_params()`, Default: NULL, which invokes
#' `inla.wkt_unit_params()`
#' @return For `inla.wkt_unit_params`, a
#' list of named unit definitions
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @examples
#' \dontrun{
#' if (inla.has_PROJ6()) {
#'     c1 <- inla.CRS("globe")
#'     inla.crs_get_lengthunit(c1)
#'     c2 <- inla.crs_set_lengthunit(c1, "km")
#'     inla.crs_get_lengthunit(c2)
#' }
#' }
#' @export
#' @seealso [inla.sp_get_crs()]
#' @aliases crs_wkt
#' @rdname crs_wkt

inla.wkt_unit_params <- function() {
    params <- list(
        "metre" =
            list(
                '"metre"',
                "1",
                list(
                    label = "ID",
                    params = list('"EPSG"', "9001")
                )
            ),
        "kilometre" =
            list(
                '"kilometre"',
                "1000",
                list(
                    label = "ID",
                    params = list('"EPSG"', "9036")
                )
            )
    )
    params[["meter"]] <- params[["metre"]]
    params[["m"]] <- params[["metre"]]
    params[["International metre"]] <- params[["metre"]]
    params[["kilometer"]] <- params[["kilometre"]]
    params[["km"]] <- params[["kilometre"]]
    params
}

#' @export
#' @rdname crs_wkt
#' @return For `inla.wkt_get_lengthunit`, a
#' list of length units used in the wkt string, excluding the ellipsoid radius
#' unit.

inla.wkt_get_lengthunit <- function(wkt) {
    extract <- function(wt) {
        # 1. Recursively find LENGTHUNIT, except within ELLIPSOID
        # 2. Return unit

        if (wt[["label"]] == "LENGTHUNIT") {
            result <- list(wt[["params"]])
        } else if (wt[["label"]] != "ELLIPSOID") {
            result <- list()
            for (k in seq_along(wt$param)) {
                if (is.list(wt[["params"]][[k]])) {
                    result <- c(result, extract(wt[["params"]][[k]]))
                }
            }
        } else {
            result <- list()
        }
        result
    }

    wt <- inla.as.wkt_tree.wkt(wkt)
    params <- unique(extract(wt))
    names(params) <-
        vapply(
            params,
            function(x) {
                gsub('"', "", x[[1]])
            },
            ""
        )
    params
}

#' @export
#' @rdname crs_wkt
#' @return For `inla.wkt_set_lengthunit`, a
#' WKT2 string with altered length units.
#' Note that the length unit for the ellipsoid radius is unchanged.

inla.wkt_set_lengthunit <- function(wkt, unit, params = NULL) {
    convert <- function(wt, unit) {
        # 1. Recursively find LENGTHUNIT, except within ELLIPSOID
        # 2. Change unit

        if (wt[["label"]] == "LENGTHUNIT") {
            wt[["params"]] <- unit
        } else if (wt[["label"]] != "ELLIPSOID") {
            for (k in seq_along(wt$param)) {
                if (is.list(wt[["params"]][[k]])) {
                    wt[["params"]][[k]] <- convert(wt[["params"]][[k]], unit)
                }
            }
        }
        wt
    }

    if (is.null(params)) {
        params <- inla.wkt_unit_params()
    }
    if (!(unit %in% names(params))) {
        warning(paste0(
            "'inla.wkt_set_lengthunit' unit conversion to '",
            unit,
            "' not supported. Unit left unchanged."
        ))
        return(wkt)
    }

    wt <- inla.as.wkt_tree.wkt(wkt)
    wt <- convert(wt, params[[unit]])
    inla.as.wkt.wkt_tree(wt)
}

#' @return For `inla.crs_get_wkt`, WKT2 string.
#' @export
#' @rdname crs_wkt

inla.crs_get_wkt <- function(crs) {
    inla.requires_PROJ6("inla.crs_get_wkt")

    if (inherits(crs, "inla.CRS")) {
        crs <- crs[["crs"]]
    }

    if (is.null(crs)) {
        return(NULL)
    }

    comment(crs)
}

#' @return For `inla.crs_get_lengthunit`, a
#' list of length units used in the wkt string, excluding the ellipsoid radius
#' unit. (For legacy PROJ4 code, the raw units from the proj4string are
#' returned, if present.)
#' @export
#' @rdname crs_wkt

inla.crs_get_lengthunit <- function(crs) {
    if (inla.has_PROJ6()) {
        x <- inla.wkt_get_lengthunit(inla.crs_get_wkt(crs))
    } else {
        if (inherits(crs, "inla.CRS")) {
            crs_ <- crs
            crs <- crs[["crs"]]
        } else {
            crs_ <- NULL
        }

        crs_args <- inla.as.list.CRS(crs)
        x <- crs_args[["units"]]
    }
    x
}

#' @return For `inla.crs_set_lengthunit`, a `sp::CRS` object with
#' altered length units.
#' Note that the length unit for the ellipsoid radius is unchanged.
#' @export
#' @rdname crs_wkt

inla.crs_set_lengthunit <- function(crs, unit, params = NULL) {
    if (inherits(crs, "inla.CRS")) {
        crs_ <- crs
        crs <- crs[["crs"]]
    } else {
        crs_ <- NULL
    }
    if (inla.has_PROJ6()) {
        x <- sp::CRS(SRS_string = inla.wkt_set_lengthunit(inla.crs_get_wkt(crs),
            unit,
            params = params
        ))
    } else {
        crs_args <- inla.as.list.CRS(crs)
        crs_args[["units"]] <- unit
        x <- inla.as.CRS.list(crs_args)
    }
    if (!is.null(crs_)) {
        crs_[["crs"]] <- x
        x <- crs_
    }
    x
}


inla.rotmat3213 <- function(rot) {
    cs <- cos(rot[1])
    sn <- sin(rot[1])
    R <- matrix(c(
        cs, -sn, 0,
        sn, cs, 0,
        0, 0, 1
    ), 3, 3)
    cs <- cos(rot[2])
    sn <- sin(rot[2])
    R <- R %*% matrix(c(
        cs, 0, sn,
        0, 1, 0,
        -sn, 0, cs
    ), 3, 3)
    cs <- cos(rot[3])
    sn <- sin(rot[3])
    R <- R %*% matrix(c(
        1, 0, 0,
        0, cs, -sn,
        0, sn, cs
    ), 3, 3)
    cs <- cos(rot[4])
    sn <- sin(rot[4])
    R <- R %*% matrix(c(
        cs, -sn, 0,
        sn, cs, 0,
        0, 0, 1
    ), 3, 3)
    R
}

inla.rotmat3123 <- function(rot) {
    cs <- cos(rot[4])
    sn <- sin(rot[4])
    R <- matrix(c(
        cs, -sn, 0,
        sn, cs, 0,
        0, 0, 1
    ), 3, 3)
    cs <- cos(rot[3])
    sn <- sin(rot[3])
    R <- R %*% matrix(c(
        1, 0, 0,
        0, cs, -sn,
        0, sn, cs
    ), 3, 3)
    cs <- cos(rot[2])
    sn <- sin(rot[2])
    R <- R %*% matrix(c(
        cs, 0, sn,
        0, 1, 0,
        -sn, 0, cs
    ), 3, 3)
    cs <- cos(rot[1])
    sn <- sin(rot[1])
    R <- R %*% matrix(c(
        cs, -sn, 0,
        sn, cs, 0,
        0, 0, 1
    ), 3, 3)
    R
}


inla.crs.transform.oblique <- function(x, oblique, to.oblique = TRUE) {
    if (to.oblique) {
        ## Transform to oblique orientation
        ## 1) Rotate -oblique[1] around (0,0,1)
        ## 2) Rotate +oblique[2] around (0,1,0)
        ## 3) Rotate -oblique[3] around (1,0,0)
        ## 3) Rotate -oblique[4] around (0,0,1)
        x %*% inla.rotmat3213(c(-1, 1, -1, -1) * oblique * pi / 180)
    } else {
        ## Transform back from oblique orientation
        ## 1) Rotate +oblique[4] around (0,0,1)
        ## 2) Rotate +oblique[3] around (1,0,0)
        ## 3) Rotate -oblique[2] around (0,1,0)
        ## 4) Rotate +oblique[1] around (0,0,1)
        x %*% inla.rotmat3123(c(1, -1, 1, 1) * oblique * pi / 180)
    }
}



inla.wkt_tree_projection_type <- function(wt) {
    axis1 <- inla.wkt_tree_get_item(wt, "AXIS", 1)
    axis2 <- inla.wkt_tree_get_item(wt, "AXIS", 2)
    if (identical(axis1[["params"]][[1]], '"longitude"') &&
        identical(axis2[["params"]][[1]], '"latitude"')) {
        return("longlat")
    }
    conversion <- inla.wkt_tree_get_item(wt, "CONVERSION")
    if (!is.null(conversion)) {
        method <- inla.wkt_tree_get_item(conversion, "METHOD")
        if (identical(method[["params"]][[1]], '"Lambert Cylindrical Equal Area (Sherical)"')) {
            return("lambert")
        }
        if (identical(method[["params"]][[1]], '"Mollweide"')) {
            return("mollweide")
        }
        if (identical(method[["params"]][[1]], '"PROJ hammer"')) {
            return("hammer")
        }
        if (identical(method[["params"]][[1]], '"tmerc"')) {
            return("tmerc")
        }
    }
    NULL
}

inla.wkt_projection_type <- function(wkt) {
    inla.requires_PROJ6()
    wt <- inla.as.wkt_tree.wkt(wkt)
    inla.wkt_tree_projection_type(wt)
}

inla.crs_projection_type <- function(crs) {
    inla.requires_PROJ6()
    wkt <- inla.crs_get_wkt(crs)
    inla.wkt_projection_type(wkt)
}

## +proj=longlat in (-180,180)x(-90,90)
## +proj=moll in (-2,2)x(-1,1) scaled by +a and +b, and +units
## +proj=lambert in (-pi,pi)x(-1,1) scaled by +a and +b, and +units
inla.crs.bounds <- function(crs, warn.unknown = FALSE) {
    if (inla.has_PROJ6()) {
        # PROJ6

        wkt <- inla.crs_get_wkt(crs)
        wt <- inla.as.wkt_tree.wkt(wkt)
        type <- inla.wkt_tree_projection_type(wt)

        if (is.null(type)) {
            if (inla.wkt_is_geocent(wkt)) {
                bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
            } else {
                if (warn.unknown) {
                    warning("Could not determine shape of transformation bounds. Using infinite rectangle.")
                }
                bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
            }
        } else if (type == "longlat") {
            bounds <- list(type = "rectangle", xlim = c(-180, 180), ylim = c(-90, 90))
        } else if (type == "lambert") {
            axis <- c(pi, 1)
            radius <- inla.wkt_get_ellipsoid_radius(wkt)
            axis[1] <- axis[1] * radius
            # TODO: handle eccentricity
            axis[2] <- axis[2] * sqrt(radius) * sqrt(radius)
            # TODO: Handle units"
            bounds <- list(
                type = "rectangle",
                xlim = c(-1, 1) * axis[1],
                ylim = c(-1, 1) * axis[2]
            )
        } else if (type %in% c("mollweide", "hammer")) {
            axis <- c(2, 1)
            center <- c(0, 0)
            radius <- inla.wkt_get_ellipsoid_radius(wkt)
            axis[1] <- axis[1] * radius / sqrt(1 / 2)
            axis[2] <- axis[2] * radius / sqrt(1 / 2)
            # TODO: Handle "units"
            bounds <- list(
                type = "ellipse", axis = axis, center = center,
                xlim = center[1] + c(-1, 1) * axis[1],
                ylim = center[2] + c(-1, 1) * axis[2]
            )
        } else if (type == "tmerc") {
            bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
        } else {
            stop("'inla.crs.bounds' internal error: transformation detected but not handled.")
        }
    } else {
        # PROJ4
        args <- inla.as.list.CRS(crs)
        if (args[["proj"]] == "longlat") {
            bounds <- list(type = "rectangle", xlim = c(-180, 180), ylim = c(-90, 90))
        } else if (args[["proj"]] == "cea") {
            axis <- c(pi, 1)
            if (!is.null(args[["a"]])) {
                axis[1] <- axis[1] * as.numeric(args$a)
            }
            if (!is.null(args[["b"]])) {
                axis[2] <- axis[2] * as.numeric(args$a)^0.5 * as.numeric(args$b)^0.5
            }
            ## TODO: Handle "lat_ts" and "units"
            bounds <- list(
                type = "rectangle",
                xlim = c(-1, 1) * axis[1], ylim = c(-1, 1) * axis[2]
            )
        } else if (args[["proj"]] %in% c("moll", "hammer")) {
            axis <- c(2, 1)
            center <- c(0, 0)
            if (!is.null(args[["a"]])) {
                axis[1] <- axis[1] * as.numeric(args$a) / sqrt(1 / 2)
                axis[2] <- axis[2] * as.numeric(args$a) / sqrt(1 / 2)
            }
            ## TODO: Handle "units"
            bounds <- list(
                type = "ellipse", axis = axis, center = center,
                xlim = center[1] + c(-1, 1) * axis[1],
                ylim = center[2] + c(-1, 1) * axis[2]
            )
        } else if (args[["proj"]] == "tmerc") {
            bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
        } else if (inla.crs_is_geocent(crs)) {
            bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
        } else {
            if (warn.unknown) {
                warning("Could not determine shape of transformation bounds. Using infinite rectangle.")
            }
            bounds <- list(type = "rectangle", xlim = c(-Inf, Inf), ylim = c(-Inf, Inf))
        }
    }

    if (bounds$type == "rectangle") {
        bounds$polygon <- cbind(
            bounds$xlim[c(1, 2, 2, 1, 1)],
            bounds$ylim[c(1, 1, 2, 2, 1)]
        )
    } else if (bounds$type == "ellipse") {
        theta <- seq(0, 2 * pi, length = 1000)
        bounds$polygon <- cbind(
            bounds$center[1] + bounds$axis[1] * cos(theta),
            bounds$center[2] + bounds$axis[2] * sin(theta)
        )
    } else {
        stop("Unknown transformation type. This should not happen.")
    }
    bounds
}

## TRUE/FALSE for points inside/outside projection domain.
inla.crs.bounds.check <- function(x, bounds) {
    inla.require.inherits(x, "matrix")
    if (all(is.finite(bounds$xlim)) && all(is.finite(bounds$ylim))) {
        (sp::point.in.polygon(
            x[, 1], x[, 2],
            bounds$polygon[, 1], bounds$polygon[, 2]
        )
        > 0)
    } else {
        ok <- rep(TRUE, nrow(x))
    }
}



internal.update.crs <- function(crs, newcrs, mismatch.allowed) {
    if (is.null(crs)) {
        newcrs
    } else {
        if (!mismatch.allowed && !identical(crs, newcrs)) {
            show(crs)
            show(newcrs)
            stop("CRS information mismatch.")
        }
        crs
    }
}




#' Test CRS and inla.CRS for equality
#'
#' Wrapper for identical, optionally testing only the CRS part of two objects
#'
#'
#' @param crs0 A `CRS` or `inla.CRS` object.
#' @param crs1 A `CRS` or `inla.CRS` object.
#' @param crsonly Logical. If `TRUE`, only the `CRS` part of a
#' `inla.CRS` object is compared.
#' @author Finn Lindgren <finn.lindgren@@gmail.com>
#' @seealso [inla.CRS()]
#' @examples
#'
#' crs0 <- inla.CRS("longlat")
#' crs1 <- inla.CRS("longlat", oblique = c(0, 90))
#' print(c(
#'     inla.identical.CRS(crs0, crs0),
#'     inla.identical.CRS(crs0, crs1),
#'     inla.identical.CRS(crs0, crs1, crsonly = TRUE)
#' ))
#' @export inla.identical.CRS
inla.identical.CRS <- function(crs0, crs1, crsonly = FALSE) {
    if (!crsonly) {
        identical(crs0, crs1)
    } else {
        if (inherits(crs0, "inla.CRS")) {
            crs0 <- crs0$crs
        }
        if (inherits(crs1, "inla.CRS")) {
            crs1 <- crs1$crs
        }
        identical(crs0, crs1)
    }
}






#' Wrapper method for `sp::spTransform`
#'
#' Handles transformation of various inla objects accorting to coordinate
#' reference systems of `sp::CRS` or `inla.CRS` class.
#'
#'
#' @aliases inla.spTransform inla.spTransform.default
#' inla.spTransform.SpatialPoints inla.spTransform.inla.mesh.lattice
#' inla.spTransform.inla.mesh.segment inla.spTransform.inla.mesh
#' @param x The object that should be transformed from it's current CRS to a
#' new CRS
#' @param crs0 The source `sp::CRS` or `inla.CRS` object
#' @param crs1 The target `sp::CRS` or `inla.CRS` object
#' @param CRSobj The target `sp::CRS` or `inla.CRS` object
#' @param passthrough default FALSE. Setting to TRUE allows objects with no CRS
#' information to be passed through without transformation.
#' @param \dots Potential additional arguments
#' @return The object is returned with its coordinates transformed
#' @author Finn Lindgren <finn.lindgren@@gmail.com>
#' @seealso [inla.CRS()]
#' @examples
#'
#' if (require(rgdal)) {
#'     latt <- inla.mesh.lattice(-10:10, 40:60)
#'     mesh1 <- inla.mesh.create(
#'         lattice = latt, extend = FALSE, refine = FALSE,
#'         crs = inla.CRS("longlat")
#'     )
#'     mesh2 <- inla.spTransform(mesh1, inla.CRS("lambert"))
#'     summary(mesh1)
#'     summary(mesh2)
#' }
#' @export inla.spTransform
inla.spTransform <- function(x, ...) {
    UseMethod("inla.spTransform")
}



#' @details `inla.spTransform.default` Low level transformation of raw coordinates.
#' @export
#' @rdname inla.spTransform
inla.spTransform.default <- function(x, crs0, crs1, passthrough = FALSE, ...) {
    if (inla.has_PROJ6()) {
        # PROJ6
        ok0 <- (!is.null(crs0) &&
            ((inherits(crs0, "CRS") && !is.null(inla.crs_get_wkt(crs0))) ||
                (inherits(crs0, "inla.CRS"))))
        ok1 <- (!is.null(crs1) &&
            ((inherits(crs1, "CRS") && !is.null(inla.crs_get_wkt(crs1))) ||
                (inherits(crs1, "inla.CRS"))))
        if (ok0 && ok1) {
            if (ncol(x) == 2) {
                x <- cbind(x, 0)
            }
            sphere_radius_0 <- inla.crs_get_ellipsoid_radius(crs0)
            sphere_radius_1 <- inla.crs_get_ellipsoid_radius(crs1)
            different_radii <- (sphere_radius_0 != sphere_radius_1)
            longlat_norm <- inla.CRS("longlat_norm")
            longlat_0 <- inla.crs_set_ellipsoid_radius(longlat_norm, sphere_radius_0)
            longlat_1 <- inla.crs_set_ellipsoid_radius(longlat_norm, sphere_radius_1)

            crs_sphere <- inla.CRS("sphere")
            onsphere_0 <- inla.identical.CRS(crs0, crs_sphere, crsonly = TRUE)
            onsphere_1 <- inla.identical.CRS(crs1, crs_sphere, crsonly = TRUE)
            is_geocentric_0 <- inla.crs_is_geocent(crs0)
            is_geocentric_1 <- inla.crs_is_geocent(crs1)
            if (is_geocentric_0) {
                ok <- TRUE
            } else {
                bounds <- inla.crs.bounds(crs0)
                if (identical(inla.crs_projection_type(crs0), "longlat")) {
                    ## Wrap longitudes to [-180,180]
                    needswrap <- (x[, 1] < -180) | (x[, 1] > 180)
                    if (any(needswrap)) {
                        x[needswrap, 1] <- ((x[needswrap, 1] + 180) %% 360) - 180
                    }
                }
                ok <- inla.crs.bounds.check(x, bounds)
                if (!all(ok)) {
                    xx <- x
                }
            }
            do_work_on_sphere <-
                inherits(crs0, "inla.CRS") ||
                    inherits(crs1, "inla.CRS") ||
                    different_radii
            if (inherits(crs0, "inla.CRS")) {
                crs0crs <- crs0$crs
                crs0oblique <- crs0$oblique
            } else {
                crs0crs <- crs0
                crs0oblique <- NULL
            }
            if (inherits(crs1, "inla.CRS")) {
                crs1crs <- crs1$crs
                crs1oblique <- crs1$oblique
            } else {
                crs1crs <- crs1
                crs1oblique <- NULL
            }
            x <- SpatialPoints(x[ok, , drop = FALSE], proj4string = crs0crs)
            if (do_work_on_sphere) {
                if (!onsphere_0) {
                    if (sphere_radius_0 != 1) {
                        x <- spTransform(x, longlat_0)
                        proj4string(x) <- CRS(NA_character_) # Reset CRS to avoid warning
                        proj4string(x) <- longlat_norm
                    }
                    x <- spTransform(x, crs_sphere)
                }
                if (!is.null(crs0oblique)) {
                    x <- SpatialPoints(inla.crs.transform.oblique(coordinates(x),
                        crs0oblique,
                        to.oblique = FALSE
                    ),
                    proj4string = crs_sphere
                    )
                }

                if (!is.null(crs1oblique)) {
                    x <- SpatialPoints(inla.crs.transform.oblique(coordinates(x),
                        crs1oblique,
                        to.oblique = TRUE
                    ),
                    proj4string = crs_sphere
                    )
                }
                if (sphere_radius_1 != 1) {
                    x <- spTransform(x, longlat_norm)
                    proj4string(x) <- CRS(NA_character_) # Reset CRS to avoid warning
                    proj4string(x) <- longlat_1
                }
            }

            x <- spTransform(x, crs1crs)

            if (!all(ok)) {
                xx[ok, ] <- coordinates(x)
                xx[!ok, ] <- NA
                x <- xx
            }
        } else if (!passthrough) {
            if (!ok0) {
                stop("'crs0' is an invalid coordinate reference object.")
            }
            if (!ok1) {
                stop("'crs1' is an invalid coordinate reference object.")
            }
        }
        if (is.matrix(x)) {
            invisible(x)
        } else {
            invisible(coordinates(x))
        }
    } else {
        # PROJ4
        ok0 <- (!is.null(crs0) &&
            ((inherits(crs0, "CRS") && !is.na(inla.CRSargs(crs0))) ||
                (inherits(crs0, "inla.CRS"))))
        ok1 <- (!is.null(crs1) &&
            ((inherits(crs1, "CRS") && !is.na(inla.CRSargs(crs1))) ||
                (inherits(crs1, "inla.CRS"))))
        if (ok0 && ok1) {
            if (ncol(x) == 2) {
                x <- cbind(x, 0)
            }
            onsphere <- inla.identical.CRS(crs0, inla.CRS("sphere"), crsonly = TRUE)
            isgeocentric <- identical(inla.as.list.CRS(crs0)[["proj"]], "geocent")
            if (isgeocentric) {
                ok <- TRUE
            } else {
                bounds <- inla.crs.bounds(crs0)
                if (identical(inla.as.list.CRS(crs0)[["proj"]], "longlat")) {
                    ## Wrap longitudes to [-180,180]
                    needswrap <- (x[, 1] < -180) | (x[, 1] > 180)
                    if (any(needswrap)) {
                        x[needswrap, 1] <- ((x[needswrap, 1] + 180) %% 360) - 180
                    }
                }
                ok <- inla.crs.bounds.check(x, bounds)
                if (!all(ok)) {
                    xx <- x
                }
            }
            if (inherits(crs0, "inla.CRS")) {
                if (!onsphere) {
                    x <- spTransform(
                        SpatialPoints(x[ok, , drop = FALSE], proj4string = crs0$crs),
                        inla.CRS("sphere")
                    )
                }
                if (!is.null(crs0$oblique)) {
                    x <- SpatialPoints(inla.crs.transform.oblique(coordinates(x),
                        crs0$oblique,
                        to.oblique = FALSE
                    ),
                    proj4string = inla.CRS("sphere")
                    )
                }
                onshpere <- TRUE
            } else {
                x <- SpatialPoints(x[ok, , drop = FALSE], proj4string = crs0)
            }
            if (inherits(crs1, "inla.CRS")) {
                if (!onsphere) {
                    x <- spTransform(x, inla.CRS("sphere"))
                }
                if (!is.null(crs1$oblique)) {
                    x <- SpatialPoints(inla.crs.transform.oblique(coordinates(x),
                        crs1$oblique,
                        to.oblique = TRUE
                    ),
                    proj4string = inla.CRS("sphere")
                    )
                }
                x <- spTransform(x, crs1$crs)
            } else {
                x <- spTransform(x, crs1)
            }
            if (!all(ok)) {
                xx[ok, ] <- coordinates(x)
                xx[!ok, ] <- NA
                x <- xx
            }
        } else if (!passthrough) {
            if (!ok0) {
                stop("'crs0' is an invalid coordinate reference object.")
            }
            if (!ok1) {
                stop("'crs1' is an invalid coordinate reference object.")
            }
        }
        if (is.matrix(x)) {
            invisible(x)
        } else {
            invisible(coordinates(x))
        }
    }
}


#' @export
#' @rdname inla.spTransform
inla.spTransform.SpatialPoints <- function(x, CRSobj, passthrough = FALSE, ...) {
    if (inla.has_PROJ6()) {
        crs_x <- inla.sp_get_crs(x)
        ok0 <- !is.null(inla.crs_get_wkt(crs_x))
        ok1 <- (!missing(CRSobj) && !is.null(CRSobj) &&
            (inherits(CRSobj, "CRS") && !is.null(inla.crs_get_wkt(CRSobj))))
        if (ok0 && ok1) {
            invisible(SpatialPoints(inla.spTransform(
                coordinates(x),
                crs_x,
                CRSobj
            ),
            proj4string = CRSobj
            ))
        } else if (ok1) { ## Know: !ok0 && ok1
            if (!passthrough) {
                stop("Invalid origin CRS for SpatialPoints")
            }
            invisible(SpatialPoints(coordinates(x), proj4string = CRSobj))
        } else { ## Know: (ok0 || !ok0) && !ok1
            if (!passthrough) {
                stop("Invalid target CRS for SpatialPoints")
            }
            invisible(SpatialPoints(coordinates(x), proj4string = inla.CRS()))
        }
    } else {
        # PROJ4
        ok0 <- !is.na(proj4string(x))
        ok1 <- (!missing(CRSobj) && !is.null(CRSobj) &&
            (inherits(CRSobj, "CRS") && !is.na(inla.CRSargs(CRSobj))))
        if (ok0 && ok1) {
            invisible(SpatialPoints(inla.spTransform(
                coordinates(x),
                CRS(proj4string(x)),
                CRSobj
            ),
            proj4string = CRSobj
            ))
        } else if (ok1) { ## Know: !ok0 && ok1
            if (!passthrough) {
                stop("Invalid origin CRS for SpatialPoints")
            }
            invisible(SpatialPoints(coordinates(x), proj4string = CRSobj))
        } else { ## Know: (ok0 || !ok0) && !ok1
            if (!passthrough) {
                stop("Invalid target CRS for SpatialPoints")
            }
            invisible(SpatialPoints(coordinates(x), proj4string = inla.CRS()))
        }
    }
}

#' @export
#' @rdname inla.spTransform
inla.spTransform.SpatialPointsDataFrame <- function(x,
                                                    CRSobj,
                                                    passthrough = FALSE,
                                                    ...) {
    inla.requires_PROJ6()

    ok1 <- (!missing(CRSobj) && !is.null(CRSobj) &&
        (inherits(CRSobj, "CRS") && !is.null(inla.crs_get_wkt(CRSobj))))
    if (!ok1 && !passthrough) {
        stop("Invalid target CRS for SpatialPointsDataFrame")
    }

    x_no_df <- SpatialPoints(coordinates(x),
        proj4string = inla.sp_get_crs(x)
    )
    x_no_df <- inla.spTransform(x_no_df,
        CRSobj = CRSobj,
        passthrough = passthrough
    )
    if (ok1) {
        invisible(SpatialPointsDataFrame(coordinates(x_no_df),
            proj4string = CRSobj,
            data = x@data
        ))
    } else {
        invisible(SpatialPointsDataFrame(coordinates(x_no_df),
            proj4string = inla.CRS(),
            data = x@data
        ))
    }
}

#' @export
#' @rdname inla.spTransform
inla.spTransform.inla.mesh.lattice <- function(x, CRSobj, passthrough = FALSE, ...) {
    x$segm <- inla.spTransform(x$segm, CRSobj, passthrough = passthrough)
    x$loc <- inla.spTransform(x$loc, x$crs, CRSobj, passthrough = passthrough)
    x$crs <- CRSobj
    invisible(x)
}

#' @export
#' @rdname inla.spTransform
inla.spTransform.inla.mesh.segment <- function(x, CRSobj, passthrough = FALSE, ...) {
    x$loc <- inla.spTransform(x$loc, x$crs, CRSobj, passthrough = passthrough)
    x$crs <- CRSobj
    invisible(x)
}




inla.crs_detect_manifold <- function(crs) {
    if (inla.crs_is_geocent(crs)) {
        manifold <- "S2"
    } else {
        manifold <- "R2"
    }
    manifold
}

#' @export
#' @rdname inla.spTransform
inla.spTransform.inla.mesh <- function(x, CRSobj, passthrough = FALSE, ...) {
    x$loc <- inla.spTransform(x$loc, x$crs, CRSobj, passthrough = passthrough)
    x$manifold <- inla.crs_detect_manifold(CRSobj)
    x$crs <- CRSobj
    invisible(x)
}




## Input: list of segments, all closed polygons.
inla.internal.sp2segment.join <- function(inp, grp = NULL, closed = TRUE) {
    crs <- NULL
    if (length(inp) > 0) {
        out.loc <- matrix(0, 0, ncol(inp[[1]]$loc))
        for (k in seq_along(inp)) {
            crs <- internal.update.crs(crs, inp[[k]]$crs, mismatch.allowed = FALSE)
        }
    } else {
        out.loc <- matrix(0, 0, 2)
    }
    out.idx <- matrix(0L, 0, 2)
    if (is.null(grp)) {
        out.grp <- NULL
    } else {
        out.grp <- integer(0)
    }
    for (k in seq_along(inp)) {
        inp.loc <- inp[[k]]$loc
        inp.idx <- inp[[k]]$idx
        inp.grp <- inp[[k]]$grp
        offset <- nrow(out.loc)
        n <- nrow(as.matrix(inp.idx))
        if (closed) {
            if (!is.null(grp) && is.null(inp.grp)) {
                inp.grp <- rep(grp[k], n)
            }
            if (ncol(as.matrix(inp.idx)) == 1) {
                inp.idx <- cbind(inp.idx, inp.idx[c(2:n, 1)])
            }
        } else {
            if (!is.null(grp) && is.null(inp.grp)) {
                inp.grp <- rep(grp[k], n - 1)
            }
            if (ncol(as.matrix(inp.idx)) == 1) {
                inp.idx <- cbind(inp.idx[-n], inp.idx[-1])
            }
        }
        out.loc <- rbind(out.loc, inp.loc)
        out.idx <- rbind(out.idx, inp.idx + offset)
        if (!is.null(grp)) {
            out.grp <- c(out.grp, inp.grp)
        }
    }
    inla.mesh.segment(
        loc = out.loc, idx = out.idx, grp = out.grp, is.bnd = FALSE,
        crs = crs
    )
}




#' Convert `sp` curve objects to `inla.mesh.segment` objects.
#'
#' Convert `sp` curve objects to `inla.mesh.segment` objects.
#'
#'
#' @aliases as.inla.mesh.segment as.inla.mesh.segment.Line
#' as.inla.mesh.segment.Lines as.inla.mesh.segment.SpatialLines
#' as.inla.mesh.segment.SpatialLinesDataFrame as.inla.mesh.segment.Polygon
#' as.inla.mesh.segment.Polygons as.inla.mesh.segment.SpatialPolygons
#' as.inla.mesh.segment.SpatialPolygonsDataFrame inla.sp2segment
#' @param sp An `sp` polygon object of class `Polygon`,
#' `Polygons`, `SpatialPolygons`, or `SpatialPolygonsDataFrame`.
#' @param join If `TRUE`, join multiple polygons into a single segment
#' (possibly non-simply connected).
#' @param grp Group ID specification for each polygon, as used by
#' [inla.mesh.segment()], one ID per polygon.
#' @param reverse Logical, indicating if the line sequence should be traversed
#' backwards.
#' @param crs An optional `CRS` or `inla.CRS` object
#' @param \dots Additional arguments passed on to other methods.
#' @return A [inla.mesh.segment()] object, or a list of
#' [inla.mesh.segment()] objects.
#' @author Finn Lindgren \email{finn.lindgren@@gmail.com}
#' @seealso [inla.mesh.segment()]
#' @export as.inla.mesh.segment
as.inla.mesh.segment <-
    function(sp, ...) {
        UseMethod("as.inla.mesh.segment")
    }

#' @export
#' @rdname as.inla.mesh.segment
inla.sp2segment <-
    function(sp, ...) {
        UseMethod("as.inla.mesh.segment")
    }



#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialPoints <-
    function(sp, reverse = FALSE, grp = NULL, is.bnd = TRUE, ...) {
        crs <- inla.sp_get_crs(sp)
        loc <- coordinates(sp)

        n <- dim(loc)[1L]
        if (reverse) {
            idx <- seq(n, 1L, length = n)
        } else {
            idx <- seq_len(n)
        }
        inla.mesh.segment(
            loc = loc, idx = idx, grp = grp, is.bnd = is.bnd,
            crs = crs
        )
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialPointsDataFrame <-
    function(sp, ...) {
        as.inla.mesh.segment.SpatialLines(sp, ...)
    }



#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.Line <-
    function(sp, reverse = FALSE, crs = NULL, ...) {
        loc <- sp@coords
        n <- dim(loc)[1L]
        if (reverse) {
            idx <- seq(n, 1L, length = n)
        } else {
            idx <- seq_len(n)
        }
        inla.mesh.segment(loc = loc, idx = idx, is.bnd = FALSE, crs = crs)
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.Lines <-
    function(sp, join = TRUE, crs = NULL, ...) {
        segm <- as.list(lapply(
            sp@Lines,
            function(x) as.inla.mesh.segment(x, crs = crs, ...)
        ))
        if (join) {
              segm <- inla.internal.sp2segment.join(segm, grp = NULL, closed = FALSE)
          }
        segm
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialLines <-
    function(sp, join = TRUE, grp = NULL, ...) {
        crs <- inla.sp_get_crs(sp)
        segm <- list()
        for (k in 1:length(sp@lines)) {
              segm[[k]] <- as.inla.mesh.segment(sp@lines[[k]],
                  join = TRUE,
                  crs = crs, ...
              )
          }
        if (join) {
            if (missing(grp)) {
                grp <- 1:length(segm)
            }
            segm <- inla.internal.sp2segment.join(segm, grp = grp, closed = FALSE)
        }
        segm
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialLinesDataFrame <-
    function(sp, ...) {
        as.inla.mesh.segment.SpatialLines(sp, ...)
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialPolygons <-
    function(sp, join = TRUE, grp = NULL, ...) {
        crs <- inla.sp_get_crs(sp)
        segm <- list()
        for (k in 1:length(sp@polygons)) {
              segm[[k]] <- as.inla.mesh.segment(sp@polygons[[k]], join = TRUE, crs = crs)
          }
        if (join) {
            if (missing(grp)) {
                grp <- 1:length(segm)
            }
            segm <- inla.internal.sp2segment.join(segm, grp = grp)
        }
        segm
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.SpatialPolygonsDataFrame <-
    function(sp, ...) {
        as.inla.mesh.segment.SpatialPolygons(sp, ...)
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.Polygons <-
    function(sp, join = TRUE, crs = NULL, ...) {
        segm <- as.list(lapply(
            sp@Polygons,
            function(x) as.inla.mesh.segment(x, crs = crs)
        ))
        if (join) {
              segm <- inla.internal.sp2segment.join(segm, grp = NULL)
          }
        segm
    }

#' @export
#' @rdname as.inla.mesh.segment
as.inla.mesh.segment.Polygon <-
    function(sp, crs = NULL, ...) {
        loc <- sp@coords[-dim(sp@coords)[1L], , drop = FALSE]
        n <- dim(loc)[1L]
        if (sp@hole) {
              if (sp@ringDir == 1) {
                    idx <- c(1L:n, 1L)
                } else {
                    idx <- c(1L, seq(n, 1L, length.out = n))
                }
          } else
        if (sp@ringDir == 1) {
              idx <- c(1L, seq(n, 1L, length.out = n))
          } else {
              idx <- c(1L:n, 1L)
          }
        inla.mesh.segment(loc = loc, idx = idx, is.bnd = TRUE, crs = crs)
    }

## Export: inla.changelog

## !\name{inla.changelog}
## !\alias{changelog}
## !\alias{inla.changelog}
## !\alias{inla.changes}
## !
## !\title{inla.changelog}
## !
## !\description{List the recent changes in the inla-program and its R-interface}
## !
## !\usage{
## !inla.changelog()
## !}
## !\author{Havard Rue \email{hrue@r-inla.org} }
## !\seealso{\code{\link{inla}}}

`inla.changelog` <- function() {
    browseURL("https://github.com/hrue/r-inla/commits/devel")
    return(invisible())
}

#' inla estimation object class
#'
#' The inla class is defined in the INLA package
#' @name inla-class
#' @seealso [inla][INLA::inla]
#' @importFrom methods setOldClass
#' @exportClass inla
setOldClass("inla")

---
title: "Fitting CoDa using the Logistic Gaussian distribution with Dirichlet covariance structure"
subtitle: "Version 1.0.0"
author: \normalfont{Joaquín Martínez-Minaya {\small{$\bullet$}} jmarmin@eio.upv.es}
date: '`r Sys.time()`'
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Fitting CoDa using the Logistic Gaussian distribution with Dirichlet covariance structure}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \usepackage{xcolor}
bibliography: coda_bib.bib  
biblio-style: apa 
---

```{r setup, include=FALSE}
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE,warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/conditional-logit/")
```

```{r, warning=FALSE}
  ### --- 0. Loading libraries --- ####
library(INLA)
library(dplyr)
library(ggplot2)
```


# An introduction to the Logistic Normal Dirichlet Regression
As defined in @martinezminaya2023, $\boldsymbol{y} \in \mathbb{S}^D$
follows a logistic-normal distribution with Dirichlet covariance
$\mathcal{LND}(\boldsymbol{\mu}, \boldsymbol{\Sigma})$ if and only if
$alr(\boldsymbol{y}) \sim \mathcal{N}(\boldsymbol{\mu},
\boldsymbol{\Sigma})$, and: \begin{equation} \begin{array}{rcl}
\Sigma_{dd} & = & \sigma_d^2 + \gamma \,, \ d = 1, \ldots, D-1 \\
\Sigma_{dk} & = & \gamma\,, d \neq k \, \nonumber \end{array}
\end{equation} where $\sigma_d^2 + \gamma$ represents the variance of
each log-ratio and $\gamma$ is the covariance between log-ratios. From
now on we will refer to
$\mathcal{ND}(\boldsymbol{\mu},\boldsymbol{\Sigma})$ as the
multivariate normal with Dirichlet covariance structure.

Let $\boldsymbol{y}$ be a multivariate random variable such as
$\boldsymbol{y} \sim \mathcal{LND}(\boldsymbol{\mu},
\boldsymbol{\Sigma})$, which by definition is equivalent to
$alr(\boldsymbol{y} ) \sim \mathcal{ND}(\boldsymbol{\mu},
\boldsymbol{\Sigma})$. Because of its easy interpretability in terms
of log-ratios with the reference category, we focus on modelling
$alr(\boldsymbol{y})$ as a $\mathcal{ND}(\boldsymbol{\mu},
\boldsymbol{\Sigma})$.



# Simulated example I (Type II)

The model with which we are going to operate in this example presents
the following structure:
\begin{eqnarray}
    alr(\boldsymbol{Y} ) & \sim & \mathcal{ND}((\boldsymbol{\mu}^{(1)}, \ldots, \boldsymbol{\mu}^{(D)}), \boldsymbol{\Sigma}) \\
    \boldsymbol{\mu}^{(d)} & = & \boldsymbol{X} \boldsymbol{\beta}^{(d)} 
\label{eq::model_arabidopsis}\,, 
\end{eqnarray}

Note that this is the second structure presented in
@martinezminaya2023, where we are working under the assumption that
covariates have different effect in each linear predictor. In
particular, we consider $D = 3$, and the reference category is the
third one. So, we are dealing with two $alr$-coordinates. Also, we
just generate a covariate $x$ scaled to have mean 0 and standard
deviation 1.
\begin{eqnarray}
    alr(\boldsymbol{Y} ) & \sim & \mathcal{ND}((\boldsymbol{\mu}^{(1)},  \boldsymbol{\mu}^{(2)}), \boldsymbol{\Sigma}) \,, \\
    \boldsymbol{\mu}^{(1)} & = & \beta_0^{(1)} + \beta_1^{(1)} \boldsymbol{x} \nonumber \,,\\
    \boldsymbol{\mu}^{(2)} & = & \beta_1^{(2)} + \beta_1^{(2)} \boldsymbol{x} \,.
\label{eq::model_sim}
\end{eqnarray}

## Data simulation

```{r}
set.seed(201803)
inla.seed = sample.int(n=1E6, size=1)
options(width=70, digits=3)
```

### Defining hyperparameters and dimensionality of the response

We start defining the hyperparameters of the likelihood: $\sigma_1^2 =
0.5$, $\sigma_2^2 = 0.4$ and $\gamma = 0.1$, and computing the
correlation matrix of the $alr$-coordinates.

```{r}
### --- 1. Simulation --- ####
# Parameters for the simulation
D <- 3
N <- 1000
sigma2 <- c(0.5, 0.4)
cov_param <- 0.1
sigma_diag <- sqrt(sigma2 + cov_param)
hypers_lik <- data.frame(hypers = c(sigma2, cov_param),
                         name1 = c("sigma2.1", "sigma2.2", "gamma"))
# We create the correlation parameters based on the previous idea
# We are going to have ((D-1)^2 - (D-1))/2 rhos
rho <- diag(1/sigma_diag) %*% matrix(cov_param, D-1, D-1) %*% diag(1/sigma_diag)
diag(rho) <- 1
rho
```

### Simulating a covariate

We define the covariate $\boldsymbol{x}$ and also, the corresponding
betas, constructing the corresponding linear predictor.

```{r}
x = runif(N)-0.5
# - mean 0 to not affect intercept
betas = matrix(c(-1, 3, -1, 5), nrow = D-1, byrow = TRUE)
X <- data.frame(1, x) %>% as.matrix(.)
lin.pred <- X %*% t(betas) 
```

### alr-coordinates

We construct the $alr$-coordinates

```{r}
### ------- 1.2.3. Constructing the likelihood --- ####
Sigma <- matrix(sigma_diag, ncol = 1) %*% matrix(sigma_diag, nrow = 1)
Sigma <- Sigma*rho

lin.pred %>%
  apply(., 1, function(z)
    MASS::mvrnorm( n  = 1,
             mu = z,
             Sigma = Sigma)) %>%
  t(.)-> alry
```   

### Data in the simplex

We move back to the Simplex using the $alr$-inverse, in particular, we
use the function `alrInv` form the R-package `compositions`.

```{r}
  y.simplex <- compositions::alrInv(alry)
  y.simplex <- as.numeric(t(y.simplex)) %>% matrix(., ncol = D, byrow = TRUE)
  colnames(y.simplex) <- paste0("y", 1:D)  
  data <- data.frame(alry, y.simplex, x)
colnames(data)[1:(D-1)] <- c(paste0("alry.", 1:(D-1)))
data %>% head(.)
```

### Plotting the simulated data

```{r, fig.cap = "Simulated data using alr-coordinates in terms of x"}
### Alr coordinates
data %>% 
  tidyr::pivot_longer(., cols = ,starts_with("alr"), 
                      names_to = "y.names", values_to = "y.resp") %>%
  ggplot(data = .) +
  geom_point(aes(x = x, y = y.resp, fill = x), shape = 21, size = 2) +
  ylab("alr") +
  facet_wrap(~y.names) +
  theme_bw() +
  theme(legend.position = "bottom") -> p_alr

#pdf("simulated_data.pdf", width = 8, height = 6)
p_alr
#dev.off()
```


## Data preparation for fitting

### Index for individual
```{r}
  data$id.z <- 1:dim(data)[1]
```

### Extending the dataset
We extent the data with $alr$-coordinates for introducing in `inla.stack`
```{r}
data_ext <- data %>%
  tidyr::pivot_longer(., cols = all_of(paste0("alry.", 1:(D-1))),
                      names_to  = "y.names",
                      values_to = "y.resp") %>%
  .[order(ordered(.$y.names)),]
data_ext$y.names <- ordered(data_ext$y.names)
head(data_ext)
```

### Response in `R-INLA`

We create a matrix with dimension $(N \times (D-1)) \times (D-1)$ for
including the multivariate response in `R-INLA`

```{r}
names_y <- paste0("alry.", 1:(D-1))
1:length(names_y) %>%
  lapply(., function(i){
    data_ext %>%
      dplyr::filter(y.names == names_y[i]) -> data_comp_i
    #Response
    y_alr <- matrix(ncol = names_y %>% length(.), nrow = dim(data_comp_i)[1])
    y_alr[, i] <- data_comp_i$y.resp
  }) -> y.resp

1:length(names_y) %>%
  lapply(., function(i){
    y_aux <- data_ext %>%
      dplyr::select(y.resp, y.names) %>%
      dplyr::filter(y.names == names_y[i]) %>%
      dplyr::select(y.resp) %>%
      as.matrix(.)
    aux_vec <- rep(NA, (D-1))
    aux_vec[i] <- 1
    kronecker(aux_vec, y_aux)
  }) -> y_list

y_tot <- do.call(cbind, y_list)
y_tot %>% head(.)
```

### Covariates in `R-INLA`

Covariates are going to be included in the model as random effects
with big variance. So, we need the values of the covariates, and also,
an index indicating to which alr-coordinate it belongs.

```{r}
variables <- c("intercept", data %>%
                 dplyr::select(starts_with("x")) %>%
                 colnames(.))
id.names <- paste0("id.", variables)
id.variables <- rep(data_ext$y.names %>% as.factor(.) %>% as.numeric(.), 
                    length(variables)) %>%
  matrix(., ncol = length(variables), byrow = FALSE)
colnames(id.variables) <- id.names

variables
id.variables %>% head(.)
```

### `inla.stack`
We create an `inla.stack` for estimation

```{r}
stk.est <- inla.stack(data    = list(resp = y_tot),
                      A       = list(1),
                      effects = list(cbind(data_ext %>%
                                             dplyr::select(starts_with("x")),
                                           data_ext %>%
                                             dplyr::select(starts_with("id.z")),
                                           id.variables,
                                           intercept = 1)),
                      tag     = 'est')
```

## Fitting the model

In this section, we fit a model (Type II in the manuscript), and we
obtain the marginal posterior distribution of the parameters and
hyperparameters

### Fit in `R-INLA`

```{r}
  # Have different parameters for fixed effects, and do not include spatial random effects.
list_prior <- rep(list(list(prior = "pc.prec", param = c(1, 0.01))), D-1)

### Fitting the model
formula.typeII <- resp ~ -1 +
  f(id.intercept, intercept,
    model   = "iid",
    initial = log(1/1000),
    fixed   = TRUE) +
  f(id.x, x,
    model   = "iid",
    initial = log(1/1000),
    fixed   = TRUE) +
  f(id.z,
    model = "iid",
    hyper = list(prec = list(prior = "pc.prec",
                             param = c(1, 0.01))), constr = TRUE)
model.typeII <- inla(formula.typeII,
                     family         = rep("gaussian", D-1),
                     data           = inla.stack.data(stk.est),
                     control.compute = list(config = TRUE),
                     control.predictor = list(A = inla.stack.A(stk.est),
                                              compute = TRUE),
                     control.family = list_prior,
                     inla.mode = "experimental" ,
                     verbose = FALSE)
```


### Marginal posterior distribution of the fixed effects


```{r}
### Posterior distribution of the fixed effects
data_fixed <- rbind(data.frame(inla.smarginal(model.typeII$marginals.random$id.x$index.1),
                               alr = "alr(y1/y3)",
                               var = "x",
                               param = "beta1",
                               real  = betas[1,2]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.x$index.2),
                               alr = "alr(y2/y3)",
                               var = "x",
                               param = "beta1",
                               real  = betas[2,2]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.intercept$index.1),
                               alr = "alr(y1/y3)",
                               var = "intercept",
                               param = "beta0",
                               real = betas[1,1]),
                    data.frame(inla.smarginal(model.typeII$marginals.random$id.intercept$index.2),
                               alr = "alr(y2/y3)",
                               var = "intercept",
                               param = "beta0",
                               real = betas[2,1]))

p_fixed <- ggplot() +
  geom_line(data = data_fixed, aes(x = x, y = y), size = 0.9) +
  #ggtitle("Effect of the covariate bio12") +
  theme_bw() +
  geom_vline(data = data_fixed, aes(xintercept = real), col = "red4") +
 # scale_color_manual(values=c("#E75F00", "#56B4E9"))+
  theme(legend.position = "bottom") +
  facet_wrap(~param + alr, ncol = D-1, scales = "free") +
  xlab(expression(beta^(d))) +
  ylab(expression(p(beta^(d) *'|'* theta))) +
  theme(legend.title = element_blank())

#pdf("posterior_fixed.pdf", width = 6, height = 5)
p_fixed
#dev.off()
```


### Marginal Posterior distribution of the hyperparameters

```{r, fig.asp = 0.5}
### Posterior distribution of the hyperparameters
prec <- list(sigma2.1 = model.typeII$marginals.hyperpar$`Precision for the Gaussian observations`,
             sigma2.2 = model.typeII$marginals.hyperpar$`Precision for the Gaussian observations[2]`,
             gamma = model.typeII$marginals.hyper$`Precision for id.z`)

hyper <- lapply(1:length(prec),
                function(x){
                  inla.smarginal(inla.tmarginal(prec[[x]], fun = function(y)(1/y))) %>%
                    data.frame(.)
                })
names(hyper) <- names(prec)

hyper.df <- lapply(1:length(hyper),
                   function(x){
                     cbind(data.frame(hyper[[x]]), name1 = names(hyper)[x])
                   })  %>%
  do.call(rbind.data.frame, .)

hyper.df$name1 <- ordered(hyper.df$name1,
                          levels = c("sigma2.1", "sigma2.2",
                                     "gamma"))
p.hyper <- ggplot(hyper.df) +
  geom_line(aes(x = x, y = y)) +
  geom_vline(data = hypers_lik, aes(xintercept = hypers), col = "red4") +
  facet_wrap(~ name1, scales = "free") +
  theme_bw() +
  xlab(expression(theta)) +
  ylab(expression(p(theta*'|'*y)))

#pdf("marginals_hyperpar.pdf", width = 6, height = 3)
print(p.hyper)
#dev.off()
```



## Predicting for a new observation

This section, is devoted to explain how to make predictions. We want
to predict, for the values of the covariate $x = -0.5, -0.2, 0.1,
0.4$. In particular, we show how to compute the posterior preditive
distribution for the mean of the $alr$-coordinates. Posteriorly, we
move back to the Simplex.

### Preparing dataset for predictions 
```{r}
sim <- 1000
x.pred <- seq(-0.5, 0.5, 0.3)
n.pred <- length(x.pred)
cat("\n ----------------------------------------------- \n")
cat("Creating the data.frame for predictions \n")

data_pred <- data.frame(intercept = 1,
                        x = rep(x.pred, D-1))
id.z.pred  <- rep((N + 1):(N + n.pred), D - 1) #random effect z to model the correlation

# Category
id.cat_pred <- rep(1:(D - 1), rep(n.pred, D - 1))
#Index for covariates
variables_pred <- c("intercept", data_pred %>% 
                      dplyr::select(starts_with("x")) %>% 
                      colnames(.))
id.names_pred <- paste0("id.", variables_pred)
id.variables_pred <- rep(id.cat_pred, length(variables_pred)) %>% 
  matrix(., ncol = length(variables_pred), byrow = FALSE)
colnames(id.variables_pred) <- id.names_pred
```

### Preparing `inla.stack` for predictions
```{r}
stk.pred <- inla.stack(data    = list(resp = matrix(NA, ncol = D - 1, 
                                                    nrow = n.pred*(D - 1))),
                       A       = list(1),
                       effects = list(cbind(data_pred,
                                            id.z = id.z.pred,
                                            id.variables_pred)),
                       tag     = 'pred')
### --- Total stack
stk <- inla.stack(stk.est, stk.pred)
```

### Prediction
```{r}
mod.pred <- inla(formula.typeII, 
                 family         = rep("gaussian", D - 1),
                 data              = inla.stack.data(stk), 
                 control.compute   = list(config = TRUE),
                 control.predictor = list(A = inla.stack.A(stk), compute = TRUE, link = 1),
                 control.mode      = list(theta = model.typeII$mode$theta, restart = TRUE), 
                 control.family = list_prior,   
                 num.threads       = 2,
                 inla.mode = "experimental" , 
                 verbose           = FALSE)
```


### Extracting predictions using `inla.posterior.sample`
```{r}
pred.values.mean <- mod.pred$summary.fitted.values$mean[inla.stack.index(stk, 'pred')$data] %>% 
  matrix(., ncol = D - 1, byrow = FALSE)

post_sim_pred <- inla.posterior.sample(n = sim, result = mod.pred)
post_sim_predictor <- inla.posterior.sample.eval(fun = function(...){
  APredictor}, post_sim_pred, return.matrix = TRUE)
post_sim_idz <- inla.posterior.sample.eval(fun = function(...){
  id.z}, post_sim_pred, return.matrix = TRUE)

ind.pred <- inla.stack.index(stk, 'pred')$data
ind.idz <- inla.stack.index(stk, 'est')$data #This is the shared random effect
ind.idz <- ind.idz[1:(length(ind.idz)/(D - 1))]

post_sim_predictor[ind.pred, ] <- post_sim_predictor[ind.pred, ]- 
  kronecker(rep(1, D-1), post_sim_idz[-ind.idz,])

post_sim_pred_alr <- post_sim_predictor[ind.pred,]

#Computing mean and sd
pred_alr_summary <- t(apply(post_sim_pred_alr, 1, function(x){c(mean(x), sd(x))}))
pred_alr_summary <- data.frame(pred_alr_summary, 
                               y.names = rep(names_y, rep(n.pred, D-1)),
                               x.pred = rep(x.pred, D-1))
colnames(pred_alr_summary)[1:2] <- c("mean", "sd")

pred_alr_summary
```

<!-- ### Comparing values -->

<!-- ```{r, fig.asp = 0.5} -->
<!-- ggplot(data = pred_alr_summary) + -->
<!--     geom_point(data = data_ext, aes(x = x, y = y.resp), col = "gray80") + -->
<!--   geom_line(aes(x = x.pred, y = X50.), col = "red4") + -->
<!--   geom_ribbon(aes(x = x.pred, ymin = X2.5., ymax = X97.5.), alpha = 0.4) + -->
<!--   facet_wrap(~y.names) + -->
<!--   theme_bw() -->


<!-- ``` -->

### Predictions in the simplex
```{r}
###  Prediction in the simplex --- #####
apply(post_sim_predictor[ind.pred,], 2, function(x){
  alr_pred <- matrix(x, ncol = D - 1)
  pred_simplex <- compositions::alrInv(alr_pred)
  as.numeric(t(pred_simplex)) #Byrows
}) -> post_sim_pred_simplex

#Computing credible intervals
pred_simplex_summary <- t(apply(post_sim_pred_simplex, 1, function(x){c(mean(x), sd(x))}))
pred_simplex_summary <- data.frame(pred_simplex_summary, 
                                   y.names = rep(c("y1", "y2", "y3"), n.pred),
                                   x.pred  = rep(x.pred, rep(D, n.pred)))
colnames(pred_simplex_summary)[1:2] <- c("mean", "sd")

pred_simplex_summary
```


# References

\documentclass[authoryear]{article}
%%\usepackage{graphics}
\usepackage{pgf}
\usepackage{epsfig}
\usepackage{amssymb}
\usepackage{amsmath,latexsym}
\usepackage{array}
\usepackage{hyperref}
\usepackage[round]{natbib}
   
\usepackage{setspace} 
\setlength{\parindent}{0cm} 

\headsep=50pt
%\setlength{\parskip}{1.2ex}
\setlength{\parindent}{0.0mm}
\setlength{\textwidth}{15.4cm}
\setlength{\textheight}{21.2cm}
\setlength{\hoffset}{-0.5in}
\setlength{\voffset}{-1.2in}
\linespread{1.05}

\parindent=1.1pc
%\parskip=0pt
\setcounter{section}{0}
%\setcounter{tocdepth}{0}
% \setcounter{secnumdepth}{3}
%\textwidth=33pc
%\textheight=50pc
%\renewcommand{\figurename}{Fig.}

\def\Prob{\text{Prob}}
\def\E{\text{E}}
\def\Cov{\text{Cov}}
\def\Corr{\text{CoRrr}}
\def\Var{\text{Var}}
\def\Prec{\text{Prec}}
\def\mm#1{\ensuremath{\boldsymbol{#1}}} % version: amsmath
\def\MyFigureWidth{0.4\textwidth}
\def\MyFigureWidthTwo{9.5cm} %% 2*width + spacing
\def\MyFigureSpacing{6mm}

%%  To change the page layout from the default
%\addtolength{\topmargin}{-2cm}
%\addtolength{\textheight}{3cm}
%\addtolength{\oddsidemargin}{-0.5cm}
%\addtolength{\textwidth}{1cm}

% \SweaveOpts{width = 40}

\setkeys{Gin}{width=0.45\textwidth}

%\definecolor{links}{HTML}{2A1B81}
%\hypersetup{colorlinks,linkcolor=,urlcolor=links}

%\VignetteEngine{knitr::knitr} 
%\VignetteIndexEntry{ScaleModel}

\begin{document}
\begin{titlepage}
    \title{Tutorial: Scaling IGMRF-models in \texttt{R-INLA}}
    \vspace{0.5cm} \author{Sigrunn Holbek S\o rbye \\ Department of
        Mathematics and Statistics, University of Troms{\o}}
\end{titlepage}
\maketitle


<<echo=FALSE,eval=TRUE,message=FALSE>>= %chunk 1
library(spatstat)
library(mvtnorm)
library(lattice)
library(mgcv)
library(pixmap)
library(numDeriv)
library(fields)
library(INLA)
set.seed(123)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
               fig.path='figures/scale-model/')
@

\section{Introduction}

Intrinsic Gaussian Markov random fields (IGMRFs) are widely used as
priors to model latent dependency structures in \texttt{R-INLA}.
Examples of this type of models include the \texttt{rw1},
\texttt{rw2}, \texttt{besag}, \texttt{besag2}, \texttt{bym} and the
\texttt{rw2d} models, see \texttt{www.r-inla.org} for further
descriptions. All of these models can be described in terms of having
an improper Gaussian density with a scaled precision matrix that
reflects the neighbourhood structure of the model. The scaling is
represented as a random precision parameter, and applying these models
the user has to define a hyperprior for this parameter. This is a
rather delicate issue and such hyperpriors are often selected in a
rather ad-hoc and subjective manner. One problem is that a specific
fixed hyperprior for the precision parameter does not have the same
interpretation for different IGMRFs.

The aim of this tutorial is to introduce and demonstrate a recently
implemented option in \texttt{R-INLA} named \texttt{scale.model},
which can be added when specifying the inla formula-call for the
different IGMRFs. For example, we might specify the formula as
<<echo=TRUE,eval=FALSE>>= %chunk 2
formula = y ~  f(idx, model = "rw2", scale.model = TRUE, hyper=..)
@
By defining \texttt{scale.model = TRUE}, the \texttt{rw2}-model is
scaled to have a generalized variance equal to 1, see
Section~\ref{sec:igmrf} for further explanations. The new option is
also available using \texttt{control.group},
<<echo=TRUE,eval=FALSE>>= %chunk 3
f(idx, model="..", group=g, control.group = list(model="rw2", scale.model=TRUE))
@ 
which is relevant for the \texttt{rw1}, \texttt{rw2} and the
\texttt{besag}-models. Note that \texttt{scale.model = FALSE} by
default for all models to make the option backwards-compatible.

The purpose of scaling the models is to ensure that a fixed hyperprior
for the precision parameter has a similar interpretation for different
types of IGMRFs. In general, IGMRFs can be seen as models that reflect
local deviation from an unspecified overall level. More specifically,
the models considered here penalize local deviation from a constant
level (\texttt{rw1}, \texttt{besag}, \texttt{bym}), a line
(\texttt{rw2}) or a plane (\texttt{rw2d}). A priori, the hyperprior
chosen for the precision parameter will influence how large we allow
this local deviation to be. It seems reasonable to select hyperpriors
such that different random effects in a regression model are assumed
to have a similar range for this deviation. By scaling the models,
this is achieved by assigning the same fixed hyperprior to the
precisions of all IGMRFs in the regression model. If we do this using
unscaled IGMRFs (as often done in the literature), the influence using
different hyperprior choices is less controllable as the variances of
the IGMRFs differ.

The hyperpriors for scaled models will also be invariant to the shape
and size of the graph for a specific IGMRF and to different scalings
of covariates. By mapping the random precision to the marginal
variance of an IGMRF, we also get an alternative interpretation of
different parameter choices for a hyperprior, as these can be viewed
in terms of the local deviation, see Section~\ref{sec:igmrf} for
further details.

The new option is based on ideas introduced in \cite{sorbye:13} and
these ideas have also been applied in \cite{papoila:13}. Note that the
implementation in \cite{sorbye:13} is slightly different as there the
hyperprior is assigned to scaled precision parameters, while the
current option scales the IGMRFs directly. The results are the same,
but the new option makes the ideas in \cite{sorbye:13} very easy to
apply. We demonstrate this in Section~\ref{sec:examples}, analysing
two semiparametric regression models. Concluding remarks are given in
Section~\ref{sec:conclusion}.

\section{Scaling the IGMRFs}\label{sec:igmrf}

An IGMRF can be defined as a vector $\mm{x}=(x_1,\ldots , x_n)$,
having an improper Gaussian density with a sparse precision matrix
which is not of full rank. In \cite{rueheld:05}, the order of an IGMRF
is defined as the rank deficiency of its precision matrix. This
implies that a zero-mean IGMRF of $k$th order is specified by
\begin{equation*}
    \pi(\mm{x}) = (2\pi)^{-(n-k)/2}(|\mm{Q}|^*)^{1/2} 
    \exp\{-\frac{1}{2}\mm{x}^T \mm{Q} \mm{x}\}, %\label{eq:igmrf}
\end{equation*}
where the precision matrix $\mm{Q}$ is assumed to be semi-positive
definite. $|\mm{Q}|^*$ denotes the generalized determinant equal to
the product of the $n-k$ non-zero eigenvalues of $\mm{Q}$. The
marginal variances of $\mm{x}$ are given by the diagonal elements of
the generalized inverse matrix $\mm{\Sigma^*} = \mm{Q}^{-1}$, that
is $$\sigma^2(x_i) = \Sigma^*_{ii}, \quad i=1,\ldots , n.$$ Different
IGMRFs can be described by expressing the precision matrix as
$\mm{Q}=\tau \mm{R}$. Here, $\tau$ denotes the random precision
parameter while the matrix $\mm{R}$ reflects the specific
neighbourhood structure of the model which can be represented as a
labelled graph with nodes and edges. This implies that the marginal
variances/standard deviations of the IGMRF will depend on both the
structure and size of $\mm{R}$. For example, we can consider the
marginal standard deviations of the \texttt{rw1} and
\texttt{rw2}-models when the precision is fixed as $\tau= 1$. Due to
the different structure matrices of these two models,
% , see \cite[p. 95 and p. 110]{rueheld:05},
the levels for the marginal standard deviations are quite different,
see Figure~\ref{fig:marg.std}. This illustrates that it seems 
unreasonable to assign the exact same hyperprior for the precision
parameters of these two models when these parameters are random.

<<echo=F,eval=TRUE>>= %chunk 4
ss.rw1.sdref = function(values,tau) { 
    n = length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3) 
    y = NA
    
    formula = y ~ -1 + f(values, 
            model="rw1", 
            constr =TRUE, 
            values = values, 
            diagonal = 1e-10, 
            hyper = list(prec = list(
                                 initial = log(tau), 
                                 fixed=TRUE))) 
    r =inla(formula,data=data.frame(y,values),family = "gaussian",
            control.family = list(fixed =
                    TRUE), 
            control.compute=list(return.marginals=F),verbose=F)
    sd=r$summary.random$values$sd 
    return (sd) 
    }

ss.rw2.sdref= function(values,tau) { 
    n=length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3)
    y = NA #idx =1:n
    
    A1 = matrix(1, n, 1) 
    A2 = matrix(0, n, 1) 
    for(i in 1:n) { 
        A2[i, ]= i 
    }
    
    A = rbind(c(A1), c(A2)) 
    e = rep(0, 2) 
    extraconstr = list(A = A, e= e)
    
    formula = y ~ -1 + f( values, model="rw2", constr = FALSE,
            values=values, diagonal = 1e-10, extraconstr = extraconstr, hyper
            = list(prec = list(initial = log(tau), fixed=TRUE)))
    
    r = inla(formula, data = data.frame(y, values), family =
            "gaussian", control.family = list(fixed = TRUE),
            control.compute=list(return.marginals=F),verbose=F)
    
    sd=r$summary.random$values$sd 
    return (sd) 
}

n=100 
sd.rw1=ss.rw1.sdref(1:n,1) 
sd.rw2=ss.rw2.sdref(1:n,1)
##sd.rw1.scaled=ss.rw1.sdref(1:n,1) 
##sd.rw2.scaled=ss.rw2.sdref(1:n,1)
@

\begin{figure}[ht]
    \centering 
    \mbox{%%
        \makebox[0.45\textwidth][c]{ 
<<fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'>>= %chunk 5
plot(sd.rw1,xlab="Node i", ylab="Marginal standard deviation") 
@ 
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{ 
<<fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'>>= %chunk 6
plot(sd.rw2,xlab="Node i",ylab="Marginal standard deviation") 
@ 
}}
\caption{The marginal standard deviations $\sigma(x_i)$ for the
    \texttt{rw1}-model (left) and the \texttt{rw2}-model (right),
    calculated using a fixed precision $\tau=1$ and $n=100$ nodes.}
    \label{fig:marg.std}
\end{figure}

Notice that for a fixed precision $\tau$, the marginal variances of
the components of $\mm{x}$ can be expressed, as a function of $\tau$,
by
\begin{equation}
    \sigma^2_{\tau}(x_i)=\frac{\sigma^2_{\{\tau=1\}}(x_i)}{\tau},\quad i=1,\ldots , n.\label{eq:sigma.xi}
\end{equation}
To characterize the level of the marginal variances when $\tau=1$, we
define the generalized variance of $\mm{x}$ as the geometric mean
\begin{equation*}
    \sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x}) = \exp\left(\frac{1}{n}\sum_{i=1}^n \log(\sigma^2_{\{\tau=1\}}(x_i))\right). \label{eq:sigma.gv}
\end{equation*} 
When \texttt{scale.model = TRUE}, IGMRFs are scaled such that
$\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x}) =1$. This implies that the
precision parameters of different models have similar interpretation
which makes it reasonable to assign the same fixed hyperprior to
precision parameters of different IGMRFs in a regression model. Note
that in \cite{sorbye:13}, the generalized variance was referred to as
the reference variance $\sigma^2_{\mbox{\scriptsize{ref}}}(\mm x)$.
The hyperprior was then assigned to the scaled precision
$\tau/\sigma^2_{\mbox{\scriptsize{ref}}}(\mm x)$ to achieve the same
interpretation for different models.

As already mentioned, IGMRFs penalize local deviation from an
unspecified overall level. The marginal standard deviation of the
models, integrating out the random precision, give information on how
large we allow this local deviation to be. Following \cite{sorbye:13},
we apply the identity in (\ref{eq:sigma.xi}) also when $\tau$ is
random and define an upper limit $U$ for the marginal standard
deviation of a model by
\begin{equation}
    P(\sigma(x_i)>U)\approx P\left(\tau<\frac{\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})}{U^2}\right)=\alpha, \label{eq:tau}
\end{equation}
where $\alpha$ is a fixed small probability. For scaled models, this
implies that the parameter choices for the hyperprior of $\tau$ has
the same interpretation in terms of $U$, for all models $\mm{x}$.

In \texttt{R-INLA}, precision parameters are assigned Gamma
distributions by default (but any other distribution can be specified
by the user). Assume that $\tau\sim \mbox{Gamma}(a,b)$, where $a$
denotes the shape parameter and $b$ the inverse-scale parameter. Then
$b\tau\sim\mbox{Gamma}(a,1)$ and (\ref{eq:tau}) implies that
\begin{equation}
    \frac{b\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})}{U^2} = F^{-1}(\alpha,a,1), \label{eq:upper.limit}
\end{equation}
where $F^{-1}(\cdot)$ denotes the inverse cumulative distribution
function of the Gamma distribution. For scaled
models, we again notice that the
interpretation of $a$ and $b$ stays the same for all $\mm{x}$ and the
upper limit $U$ is easily calculated as 
%%
<<echo=TRUE,eval=F>>= %chunk 7
U = sqrt(b/qgamma(alpha,a,1)) 
@ 
<<echo=FALSE,eval=TRUE>>= %chunk 8
func.u = function(a,b,alpha,sigma.ref) { 
    upper.limit = sqrt(b)*sigma.ref/sqrt(qgamma(alpha,a,1)) 
    return(upper.limit) 
}
a=1 
b=5*10^{-5} 
alpha=0.001 
upper.limit=func.u(a,b,alpha,1) 
@ 
%%
For example, if we choose the default parameter values in
\texttt{R-INLA}, $a=\Sexpr{a}$ and $b=5\cdot 10^{-5}$, the upper limit
equals $U\approx \Sexpr{round(upper.limit,2)}$ when
$\alpha =\Sexpr{alpha}$. Notice that the explicit value of $U$ has to
be interpreted with care as this depends on the selected value of
$\alpha$. Also, $U$ is not uniquely defined and different combinations
of $a$ and $b$ giving the same values of $U$, do not in general give
the same estimated results.

\section{Examples}\label{sec:examples}

The new option makes it very easy to compare results using unscaled
and scaled models. Here we consider two examples, analysing
semiparametric regression models. Other examples are given in
\cite{sorbye:13} and \cite{papoila:13}.

<<echo=F,eval=TRUE>>= %chunk 9
func<-function(x,t)
{
    f.x=x/t+sin(2*pi* x/t)-0.5
    return(f.x)
}

f.est <- function(y,x,a,b,option)
{
    formula = y~1+f(x,model="rw2",hyper=list(prec=list(prior="loggamma",param=c(a,b))),scale.model=option)
    result = inla(formula,family="gaussian",data=data.frame(y,x))
    f.est=result$summary.random$x$mean
    return(f.est)
}
@

<<echo=F,eval=TRUE>>= %chunk 10
### Same values of the function, same observations, but  defined on different intervals ###
set.seed(89236)
x2=0:n
x1=x2/100
x3=x2*10

sigma=0.5
f.x=func(x2,length(x2))
y=f.x+rnorm(length(f.x),0,sigma)

a=1
b=0.00005

mat1=cbind(f.x,f.est(y,x1,a,b,T),f.est(y,x1,a,b,F))
mat2=cbind(f.x,f.est(y,x2,a,b,T),f.est(y,x2,a,b,F))
mat3=cbind(f.x,f.est(y,x3,a,b,T),f.est(y,x3,a,b,F))

# Generalized marginal variances  
v1=exp(mean(log(ss.rw2.sdref(x1,1)^2)))
v2=exp(mean(log(ss.rw2.sdref(x2,1)^2)))
v3=exp(mean(log(ss.rw2.sdref(x3,1)^2)))

u1=func.u(a,b,alpha,sqrt(v1))
u2=func.u(a,b,alpha,sqrt(v2))
u3=func.u(a,b,alpha,sqrt(v3))
@

\subsection{Semiparametric regression with different scalings of the covariate}

We first consider a simple semiparametric regression model in which
$n$ observations are generated by
\begin{equation}
    y_i = f(x_i) + \epsilon_i,\quad i=1,\ldots, n. \label{eq:regr}
\end{equation}
The noise terms, $\epsilon_i$, are assumed independent and normally
distributed with mean 0 and a fixed standard deviation $\sigma$. The
aim is to estimate the underlying smooth function $f(.)$. Applying the
\texttt{rw2}-model as a prior, the formula-call using the scaled model
is defined as
<<echo=TRUE,eval=F>>= %chunk 11
formula = y ~  f(x, model = "rw2", scale.model = TRUE, hyper=...)
@ 
where the hyperprior assigned to the precision parameter is
specified in \texttt{hyper}. Here, we just select the default
hyperprior, that is $\tau \sim \mbox{Gamma}(1,5\cdot
10^{-5})$. 

Let the unknown true function $f(.)$ be defined as
 $$f(x)=\frac{x}{t}+\sin(\frac{2\pi x}{t})-0.5,\quad x\in(0,t).$$
 We generate $n=101$ observations based on (\ref{eq:regr}) in which
 $\sigma=\Sexpr{sigma}$ and call \texttt{inla}:
<<echo=TRUE,eval=F>>= %chunk 12
result = inla(formula, family = "gaussian", data = data.frame(y,x))
@ 
We then plot the estimated function 
<<echo=TRUE,eval=F>>= %chunk 13
result$summary.random$x$mean
@ 
both using the unscaled and scaled models for three different
intervals of $x$, see Figure \ref{fig:fx2}.

\setkeys{Gin}{width=0.33\textwidth}
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 14
                                        # Estimate of f using unscaled and scaled model
r=range(c(mat1,mat2,mat3))
matplot(x1,mat1,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3) 
                                        #legend("topright",c("true f","scale.model=F","scale.model=T"),lty=c(1,2,4),col=c(1,2,4))
                                        #points(x,y)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 15
matplot(x2,mat2,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 16
matplot(x3,mat3,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)
@
}}
\caption{The true function (black solid line) and the estimated
    functions using the scaled (blue dashed line) and unscaled (red
    dashed/dotted line) \texttt{rw2}-models for different scalings of
    the covariate.}
\label{fig:fx2}
\end{figure}


Using the scaled model, the results using a fixed hyperprior is
invariant to different scalings of the covariate and we get exactly
the same estimated curve in all three cases. For the unscaled model,
the marginal variance will decrease/increase as the distance between
nodes decreases/increases. By using the same fixed hyperprior for the
three cases, we implicitly assume very different values for the upper
limit $U$ in (\ref{eq:upper.limit}), see Table~\ref{tab:upper.limit}.
In the extreme case of assuming a very low value of $U$, the resulting
estimate is just a straight line.

\begin{table}[ht]
\begin{center}
\begin{tabular}{l|ccc}%\hline
  Interval length&  $t=1$& $t=100$ & $t=1000$ \\ \hline%\hline
  % $\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})$ & $\Sexpr{round(v1,4)}$&$\Sexpr{round(v2,1)}$ & $\Sexpr{round(v3,1)}$\\
  $U$ (unscaled model) &  $\Sexpr{round(u1,3)}$ & $\Sexpr{round(u2,1)}$ & $\Sexpr{round(u3,1)}$%\hline
\end{tabular}
\end{center}
\caption{The upper limit $U$  as defined in (\ref{eq:upper.limit}) 
    for the unscaled \texttt{rw2}-model, 
    calculated when $a=1$, $b=5\cdot 10^{-5}$ and $\alpha=0.001$.}
\label{tab:upper.limit}
\end{table}

As noted in \cite{akerkar:10} and \cite{sorbye:13}, a given hyperprior
for the precision of unscaled models can be adjusted to account for
different scalings of the covariate, adjusting the inverse-scale
parameter of the Gamma distribution. However, by scaling the models,
we don't have to think about this at all as a given fixed prior will
give the same estimated curve.


\subsection{Semiparametric regression with several random effects}

The linear predictor of a latent Gaussian model often consists of a
combination of random effects modelled by different IGMRFs. As an
example we will take a look at the Munich rental data analysed in
Section 4.2.2 in \cite{rueheld:05}, also given as an example in Volume
I at \texttt{www.r-inla.org}. The response variable in this dataset is
rent per square meter (in Euros) for a flat and the covariates include
spatial location, floor space, year of construction and various
indicator variables. Smooth effects of the covariates are modelled
using the \texttt{rw2}-model, while the spatially structured effect
for location is modelled using the \texttt{besag}-model. Previously,
this data has been analysed using a fixed Gamma$(1,0.01)$ for all the
precision parameters in the model. The inverse-scale parameter of the
Gamma distribution can be adjusted to account for different types of
IGMRFs by scaling this parameter with the generalized variance, see
\cite{sorbye:13}, but it is easier to just use the new option.

To simplify notation let
<<echo=TRUE,eval=F>>= %chunk 17
hyper.prec = list(prec = list(prior = "pc.prec", param = c(0.5, 0.01)))
@
 The formula-call at  \texttt{www.r-inla.org} is then specified by 
<<echo=TRUE,eval=F>>= %chunk 18
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

## Note that here we what to have an estimator of the effect of year
## also the for years where we have no observation, therefore we give a
## vector with all possible values assumed by the covariate year, that
## is seq(1918,2001)

formula = rent ~ 
   f(location, model = "besag", graph = g, initial = 1, hyper = hyper.prec) +
   f(year, model = "rw2", values = seq(1918,2001), hyper = hyper.prec) +
   f(floor.size, model = "rw2", hyper = hyper.prec) +
   Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
   Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
   zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
@  
Further, the call to inla is given as 
<<echo=TRUE,eval=F>>= %chunk 19
mod  =  inla(formula, data = Munich, control.fixed=list(prec=1))
@                   

<<echo=FALSE,eval=TRUE>>= %chunk 20
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

func.munich <- function(a,b,option)
{
    hyper.prec=list(prec=list(param=c(a,b)))
    formula= rent ~ f(location,model="besag",graph=g,initial=1, hyper=hyper.prec, scale.model=option) +
        f(year,model="rw2",values=seq(1918,2001), hyper=hyper.prec, scale.model=option) +
            f(floor.size,model="rw2", hyper=hyper.prec, scale.model=option) +
                Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
                    Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
                        zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
    
    result = inla(formula, data=Munich, control.fixed=list(prec=1, prec.intercept = 1))
}
@

<<echo=FALSE,eval=TRUE>>=%chunk 21
a=1
b=0.01
u.munich=func.u(a,b,alpha,1)
mod=func.munich(a,b,FALSE)
mod.scaled=func.munich(a,b,TRUE)
@
Figure~\ref{fig:munich} illustrates the resulting estimated effects
of the covariates 'Year of construction' and 'Floor size' using the
scaled and unscaled models. Applying the given hyperprior for the
scaled models, we have assumed that the upper limit in
(\ref{eq:upper.limit}) is $U\approx\Sexpr{round(u.munich,1)}$. This
seems to give quite reasonable results, while the estimated curves are
more wiggly using the unscaled models. The results for the spatially
structured effects are very similar using the scaled and unscaled
models (results not shown).

\setkeys{Gin}{width=0.45\textwidth}
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 22
mat=cbind(mod$summary.random$year$mean,mod.scaled$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Year of construction",ylab="",lwd=2.5)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 23
mat=cbind(mod$summary.random$floor.size$mean,mod.scaled$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Floor size",ylab="",lwd=2.5)
@
}}
\caption{The estimated effects of the covariates 'Year of
    construction' and 'Floor size' on the rental prizes in Munich,
    using the scaled (black solid line) and the unscaled (red dashed
    line) \texttt{rw2}-models.}
\label{fig:munich}
\end{figure}

<<echo=FALSE,eval=TRUE>>= %chunk 24
alpha=0.001
#u.vec=c(0.01,1,10,100)
u.vec=c(0.001,0.1,10,30)
a.vec=c(1,1,1,1)
b.vec=u.vec^2*qgamma(alpha,a.vec,1)

option=TRUE
c1=func.munich(a.vec[1],b.vec[1],option)
c2=func.munich(a.vec[2],b.vec[2],option)
c3=func.munich(a.vec[3],b.vec[3],option)
c4=func.munich(a.vec[4],b.vec[4],option)
@


Using the scaled models, we can now study sensitivity to the
parameters of the Gamma-prior simultaneously for the different random
effects modelled by IGMRFs, instead of doing this separately for each
of the effects. For example, we can choose to keep the shape parameter
fixed at $a=1$, while the inverse-scale parameter $b$ is calculated to
give specific values of $U$. The estimated functions for the effects
of the two covariates 'Year of construction' and 'Floor size' are
given in Figure~\ref{fig:sens.u}. Here, the value of $U$ ranges from
$\Sexpr{u.vec[1]}$ to $\Sexpr{u.vec[4]}$ which corresponds to values
of $b$ in the range $(10^{-9},0.9)$. We conclude that the results are
not very sensitive to changes in the inverse-scale parameter when
$a=1$. This also applies to the spatially structured effect which does
not change significantly for these different parameter choices
(results not shown). Naturally, we could also consider other values of $a$. 
 
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 25
mat=cbind(c1$summary.random$year$mean,c2$summary.random$year$mean,c3$summary.random$year$mean,c4$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Year of construction",ylab="",lwd=2.5)
legend("topleft",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 26
mat=cbind(c1$summary.random$floor.size$mean,c2$summary.random$floor.size$mean,c3$summary.random$floor.size$mean,c4$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Floor size",ylab="",lwd=2.5)
legend("topright",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)
@
}}
\caption{The estimated effects of the covariates 'Year of
    construction' and 'Floor size' on the rental prizes in Munich,
    varying the upper limit for the scaled models.}
\label{fig:sens.u}
\end{figure}

  
\section{Concluding remarks}\label{sec:conclusion}

By scaling all IGMRFs to a unit generalized variance, we avoid that
the precision parameters of these models have different
interpretation. This makes it reasonable to assign the same hyperprior
to precision parameters of different IGMRF-models, which greatly
simplifies the issue of selecting hyperpriors and performing
sensitivity analysis. These ideas were introduced in \cite{sorbye:13},
in which the precision parameter was scaled by the generalized
variance. The new option in \texttt{R-INLA} makes these ideas easy to
apply as we don't have to calculate the generalized variance for each
model. We simply set the \texttt{scale.model}-option equal to
\texttt{TRUE} in the inla formula-call for the relevant terms.

If you are now convinced that all IGMRF-models should in fact be
scaled, you can do this in an even simpler way than already described.
By specifying the following global option
<<echo=TRUE,eval=FALSE>>= %chunk 27
inla.setOption(scale.model.default = TRUE)
@ 
all the IGMRF-models are scaled automatically. This also includes
the situation using the \texttt{control.group} argument mentioned in
the introduction. Hyperpriors still have to be selected with care
using scaled models, but the issue of selecting hyperpriors is
simplified and we believe that hyperpriors can be chosen in a more
meaningful and controlled way.

%% Currently, the option of scaling models only applies to IGMRFs but we
%% believe that similar ideas can be useful also for other models, for
%% example auto-regressive processes. Details on this will follow later.

\bibliographystyle{plainnat}
%\bibliographystyle{chicago} 
\bibliography{shs}    

\end{document}

---
title: "Conditional sampling from a fitted model"
author: "Haavard Rue `(hrue@r-inla.org)`"
date: "Aug 19th 2017"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Conditional sampling from a fitted model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
library(INLA)
knitr::opts_chunk$set(echo = TRUE, cache = FALSE)
```
This short note describe how to do conditional sampling from a fitted
model, where we want to condition on new events. An easy example which
motivated this note, is the following.
```{r}
n = 50
grp.len = 5
x = rnorm(n)
g = rnorm(grp.len)
grp = rep(1:grp.len, each = n %/% grp.len)
y = 1 + x + g[grp] + rnorm(n, sd = 0.01)
```
and the task is to provide the posterior a new observation, condition
on that the random effect 'g' is zero. Although it is possible to get
this directly (an exercise for the reader), in general, we will show that
it is easier to compute this using Monte Carlo sampling and the function
\verb|inla.posterior.sample()|. With a little hack. 

To fit the model, we do
```{r}
r = inla(y ~ x + f(grp, model = "iid"),
         data = data.frame(y, x, grp), 
         control.compute = list(config=TRUE)) 
```
adding \verb|config=TRUE| to prepare for the use of \verb|inla.posterior.sample()|.

The trick, is to add the conditioning as constraints, like
$$
A x = e
$$
where $x$ is the latent field. In our case, it is simply `t grp.len`
constraints, $\{g_i=0\}$ for
$i$ from 1 to `r grp.len`.
To do this, we need to create the matrix $A$, and vector $e$, and add
it to the list of contraints (if any). To do this, we need to know the
index of the \verb|g|-term, which we find here
```{r}
r$misc$configs$contents
```
so we have both the start index and the length of this vector. So we
create a constraint $Ax=0$ to represent 
constraints, $\{g_i=0\}$ for
$i$ from 1 to `r grp.len`.
```{r}
m = sum(r$misc$configs$contents$length)
grp.idx = which(r$misc$configs$contents$tag == "grp")
grp.len = r$misc$configs$contents$length[grp.idx]
A = matrix(0, grp.len, m)
e = matrix(0, grp.len, 1)
for(i in 1:grp.len) {
    A[i, r$misc$configs$contents$start[grp.idx] + i - 1] = 1
    e[i] = 0
}
```
Then we need to append it to the existing one if any, or to add a new one.
```{r}
constr = r$misc$configs$constr
if (is.null(constr)) {
    ## nothing there from before
    r$misc$configs$constr = list(
        nc = dim(A)[2],
        A = A,
        e = e)
} else {
    ## create a new one
    r$misc$configs$constr = list(
        nc = constr$nc + dim(A)[2],
        A = rbind(constr$A, A),
        e = rbind(constr$e, e))
}
```
When we now do
```{r}
xx = inla.posterior.sample(1000, r)
```
we additional condition on $Ax=e$, but \verb|g| would still appear in
the list of samples as zero or something very close to it.  The rest
should be straight forward.

PS: Note that you can add any linear combination of $x$ as a
constraint, you just do not want to many, as the cost is quadratic the
number of constraints. If the constraints simply set variables to
zero, you can also add a large number to $Q_{i,i}$, which you will
find in \verb|r$misc$configs|, one for each configuration.

---
title: "The A-local feature"
author: "Håvard Rue (hrue@r-inla.org)"
date: "Dec, 2025"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{The A-local feature}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

\def\diag#1{\text{diag}(#1)}
\def\Cov#1#2{\text{Cov}(x_{#1}, x_{#2})}
\def\Var#1{\text{Var}(x_{#1})}

```{r setup, include=FALSE, cache=FALSE}
library(knitr)
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, error = FALSE, message = FALSE, cache =FALSE)
knitr::opts_chunk$set(fig.path="figures/A-local/")
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
```

## Introduction

This is a short note to discuss the use of the new ``A-local''
feature. \texttt{Warning: Some of this content is rather advanced, and
we may come with a more friendlier vignette at a later stage.}

The basic issue this feature address, is that the linear predictor
sometimes contains more than one element from one or more model
component. For example

$$
	\eta_i = \sum_j A_{ij} x_j + \ldots
$$

where $x$ is defined in $f(\cdot)$ and $A$ is a fixed matrix. Here,
"$\ldots$" represents other model components in the model that are noe
relevant for this discussion. Earlier, such cases have been adressed
by removing the A-matrix locally and redefine it globally, by
introducing $\eta^* = A \eta$ for another A-matrix. The overall model,
in terms of $\eta^*$ is the same.

The step with introducing $\eta^*$ is often unnecessary complicated
and leads to a construction which also depends on the irrelevant parts
of the model given above in "$\ldots$". This can now be avoided in
many cases.

## First example 

The general format is
$$
	y \;\sim\; f(k, w, \text{model="..."}, \ldots, \text{A.local}=A) + \ldots
$$

Let $x$ represent the model component with elements $x_1, \ldots,
x_n$, then the above says that

$$
	\eta_i = w_k x_{k_i} + \sum_j A_{ij} x_{j} + \ldots
$$	

The redundancy is obvious, $A$ represents here an alternative approach
to give $(k, w)$, and we could represent the $k$-part as $A$ as well.
The \emph{values} for which $x$ are defined, here $1, \ldots, n$, is
extracted from $k$ and not $A$. If we define \emph{values} manually,
then $(k,w)$ can be just dummy and we can move to using $A$ only. An
in-between approach is also possible: Use $k$ as normal but to set
$w=0$(-vector), so that all information $x_j$'s influence on $\eta_i$
is from the $i$-th row $A_{i1}, \ldots, A_{in}$, but the information
about the \emph{values} are taken from $k$.

Here is a simple example with the model
$$
	y_i = 1 + x_i + x_{i-1} + \epsilon_i
$$
where $x_i$ is \emph{iid}, which is simulated using

```{r}
n <- 100
x <- rnorm(n)
i <- 1:n
im <- c(NA,1:(n-1))
y <- 1 + x + c(0, x[-n]) + rnorm(n)
```

The first (traditional) approach is to use \emph{copy}

```{r}
r1 <- inla(y ~ 1 + f(i,model="iid") + f(im, copy="i"),
	family="stdnormal",	
	data = data.frame(y,i,im))
```

The second approach is to use \emph{Alocal} to define the additional
contribution

```{r}
A <- matrix(0,n,n)
A[row(A)-col(A) == 1] <- 1
A <- inla.as.sparse(A)
r2 <- inla(y ~ 1 + f(i, model="iid", A.local=A),
	family="stdnormal",	
	data = list(y=y,i=i,A=A))
```

The third approach is to use \emph{Alocal} with all info in the $A$-matrix

```{r}
diag(A) <- 1 ## we add also the diagonal
r3 <- inla(y ~ 1 + f(i.NA, model="iid", A.local=A, values=1:n),
	family="stdnormal",	
	data = list(y=y, i.NA=rep(NA,n), A=A, n=n))
```

The forth approach use \emph{Alocal} and also the $i$-index to define
\emph{values}. Note that \emph{w.zero} zero-out the contribution, so
in practice we only use the $A$-matrix.

```{r}
r4 <- inla(y ~ 1 + f(i, w.zero, model="iid", A.local=A),
	family="stdnormal",	
	data = list(y=y, i=i, w.zero=rep(0,n), A=A, n=n))
```

The four approaches gives the same results
```{r}
as.vector(c(r1$mlik[1,], r2$mlik[1,], r3$mlik[1,], r4$mlik[1,]))
```

The copy-approach (first element) is slightly different as it use an
\emph{almost}-copy and not an identical copy. 


## Example with replicate/group

A.local is \emph{not} replicated when \emph{replicate} and/or
\emph{group} is used. This allows extra flexibilty.

If the dimensions of A.local do not match the dimension of $\eta$, it
still works. Like $\eta$ has length $n$, then $A$ is normally a
(sparse) matrix of size $n'\times m$ for some $m$ and $n'=n$. 
If $n' < n$ then row $n'+1, \ldots, n$ are treated as empty or 
no A-matrix. If $n' > n$ then only the first $n$ rows are used, 
with no warnings given. Here is an example where $n'=2n$ as we 
stack the two $A$'s. The result is the same as before
```{r}
AA <- rbind(A,A)
r5 <- inla(y ~ 1 + f(i, w.zero, model="iid", A.local=AA),
	family="stdnormal",	
	data = list(y=y, i=i, w.zero=rep(0,n), AA=AA, n=n))
r5$mlik[1,]
```

Here is a somewhat more advanced example with two replicated AR1 processes.
```{r}
n <- 1000
phi <- 0.9
x <- scale(arima.sim(n=n,  model = list(ar = phi)))
xx <- scale(arima.sim(n=n,  model = list(ar = phi)))
```
where each process is convolved with its kernel
```{r}
h <- 10L
nh <- h + 1L
kern <- dnorm(-h:0, sd=h)
kern <- kern / sum(kern)
A <- inla.as.sparse(INLA:::inla.toeplitz(c(kern[nh], rep(0, n-nh), kern[1:h])))

h <- 30L
nh <- h + 1L
kern <- dnorm(-h:0, sd= h)
kern <- kern / sum(kern)
B <- inla.as.sparse(INLA:::inla.toeplitz(c(kern[nh], rep(0, n-nh), kern[1:h])))
```
to form the lin.predictor
```{r}
eta <- 5 + as.vector(A %*% x) + as.vector(B %*% xx)
y <- rpois(n, lambda = exp(eta))
```

Using that the second replicate is stored at indices $n+1, \ldots,
2n$, we can just bind $A$ and $B$ together
```{r}
AB <- cbind(A, B)
r <- inla(y ~ 1 + f(idx, w, model="ar1", constr = T, A.local = AB, nrep = 2), 
          data = list(y = y, idx = 1:n, AB = AB, w = rep(0, n)),
          family = "poisson")
```
```{r plot=T}
idx <- 1:n
plot(idx, y, pch=19, ylim = range(pretty(y)), main="Data")
```

```{r, plot=T}
plot(idx, eta, col="blue", lwd=2, type="l", ylim = range(pretty(eta)))
lines(idx, r$summary.linear.predictor$mean, col="red", lwd=2)
title("eta.true and E(eta|...)")
```

```{r, plot=T}
plot(idx, x, col="blue", lwd=2, type="l", ylim = range(pretty(x)))
lines(idx, r$summary.random$idx$mean[1:n], col="red", lwd=2)
title("AR1 and E(AR1|...)")
```

```{r, plot=T}
plot(idx, xx, col="blue", lwd=2, type="l", ylim = range(pretty(x)))
lines(idx, r$summary.random$idx$mean[n + 1:n], col="red", lwd=2)
title("AR1 and E(AR1|...)")
```

and the summary is
```{r}
summary(r)
```


## The `z` model

The new feature gives an easy way to implement the z-model, which is
the classic mixed-effect model formulation

$$
	\eta = \ldots + Z z
$$

where $Z$ is a fixed matrix and $z$ is zero mean Normal with precision
$\tau Q$. This is implemented in model "z" as follows.
```{r}
n <- 300
m <- 30
Z <- matrix(rnorm(n*m), n, m)
rho <- 0.8
Qz <- toeplitz(rho^(0:(m-1)))
z <- inla.qsample(1, Q=Qz)
eta <- Z %*% z
s <- 0.1
y <- eta + rnorm(n, sd = s)
```
Using the z-model we get
```{r}
r <- inla(y ~ -1 + f(idx, model="z", Z=Z, Cmatrix=Qz, precision=exp(20),
					hyper = list(prec = list(initial = 0, fixed = TRUE))),
        data = list(y=y, idx=1:n, n=n),
        control.family = list(hyper = list(prec = list(initial = log(1/s^2), fixed= TRUE))))
```
Using A.local we get
```{r}
rr <- inla(y ~ -1 + f(idx, model="generic", A.local=Z, Cmatrix=Qz, values=1:m,
					hyper = list(prec = list(initial = 0, fixed = TRUE))),
        data = list(y=y, idx=rep(NA,n), n=n, m=m),
        control.family = list(hyper = list(prec = list(initial = log(1/s^2), fixed= TRUE))))
```
and we can compare (noting that "generic" model does not include the
normalization constant wrt to the precision matrix; see the documentation)
```{r}
r$mlik - (rr$mlik + 0.5 * determinant(Qz, log=TRUE)$modulus)
```
```{r}
mean(abs(r$summary.random$idx$mean[n + 1:m] - rr$summary.random$idx$mean))
mean(abs(r$summary.random$idx$sd[n + 1:m] / rr$summary.random$idx$sd - 1))
```


## Using replicate, again

This is motivated from an example from the discussion-list, where
$$
	y = \ldots  + \mbox{f(idx, x1, model="besag", ..., replicate=r1) +
	f(idx2, x2, copy="idx", replicate=r2)}
$$

where the second term using a replication hence sharing the same
hyperparameter. Here its written out for $m=2$ terms, but the aim was
several terms like $m\approx 15$.

This can now be done very efficient
```{r}
g <- inla.read.graph(system.file("demodata/germany.graph", package="INLA"))
source(system.file("demodata/Bym-map.R", package="INLA"))
n <- g$n
m <- 4
s <- 0.1
X <- matrix(rnorm(n*m), n, m)
y <- rowSums(X) + rnorm(n, sd = s)
A <- inla.as.sparse(diag(X[, 1]))
for(i in 2:m) {
    A <- cbind(A, inla.as.sparse(diag(X[, i])))
}

r <- inla(y ~ -1 +
              f(idx.na,
                model = "besag",
                scale.model = TRUE,
                constr = FALSE,
                nrep = m,
                graph = g, 
                A.local = A,
                values = 1:n,
                hyper = list(prec = list(prior = "pc.prec",
                                         param = c(0.5, 0.01)))), 
          family = "normal",
          control.family = list(hyper = list(prec = list(initial = log(1/s^2),
                                                         fixed = TRUE))), 
          data = list(n = n, m = m, y = y, X = X, 
			  idx.na = rep(NA, n)))
```
```{r}
beta <- matrix(r$summary.random$idx.na$mean, n,m)
summary(beta)
```

```{r plot=T}
i=1; Bym.map(r$summary.random$idx.na$mean[ (i-1) * n + 1:n])
title(paste("coeff ",  i))
```
```{r plot=T}
i=2; Bym.map(r$summary.random$idx.na$mean[ (i-1) * n + 1:n])
title(paste("coeff ",  i))
```
```{r plot=T}
i=3; Bym.map(r$summary.random$idx.na$mean[ (i-1) * n + 1:n])
title(paste("coeff ",  i))
```
```{r plot=T}
i=4; Bym.map(r$summary.random$idx.na$mean[ (i-1) * n + 1:n])
title(paste("coeff ",  i))
```

## Using 'Alocal' in copy

This is to verify that Alocal-feature works as expected with copy.
Consider this simple example
```{r}
n <- 10
y <- 1:n
A <- matrix(0, n, n)
A[lower.tri(A, diag = TRUE)] <- 1
```
where $\eta_i=1, i=1,\ldots,n$ match data $y$ as 
```{r}
cbind(y, A %*% rep(1,n))
```
We can verify that this works as expected with
```{r}
idx <- 1:n
iidx <- rep(NA, n)
w0 <- rep(0, n)
r <- inla(y ~ -1 +
              f(idx,
                w0, 
                model = "iid",
                values = 1:n, 
                hyper = list(prec = list(initial = -20,
                                         fixed = TRUE))) + 
              f(iidx,
                copy = "idx",
                values = 1:n, 
                A.local = A),
          data = list(idx = idx,
                      iidx = iidx,
                      w0 = w0,
                      A = A,
                      n = n),
          family = "stdnormal")
```
The `idx` part only defines the iid model, but is not used, as the
weights `w0` are all zero.  The distribution is essentially uniform
due to the small fixed precision. 

The `iidx` part copy `idx`, and then apply matrix `A` in `A.local`.
This mean that both `idx` and `iidx` should have mean 1, and the
$A$-matrix will make this fit the data. 
```{r}
round(dig=3, cbind(r$summary.random$idx$mean,
	               r$summary.random$iidx$mean))
```
```{r}
round(dig=3, cbind(y, A %*% r$summary.random$iidx$mean))
```

If we add scaling this still works
```{r}
r <- inla(y ~ -1 +
              f(idx,
                w0, 
                model = "iid",
                values = 1:n, 
                hyper = list(prec = list(initial = -20,
                                         fixed = TRUE))) + 
              f(iidx,
                copy = "idx",
                values = 1:n, 
                A.local = A,
                hyper = list(beta = list(initial = 0.5))),
          data = list(idx = idx,
                      iidx = iidx,
                      w0 = w0,
                      A = A,
                      n = n),
          family = "stdnormal")
```
Due to the scaling, then `idx` has to bump to $2$, but `iidx` is still
$1$ to fit data
```{r}
round(dig=3, cbind(r$summary.random$idx$mean,
	               r$summary.random$iidx$mean))
```

---
title: "Multinomial logit models with INLA"
author: "Francesco Serafini (serafini314@gmail.com)"
date: "April 2018, revised Nov 2019"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Multinomial logit models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
bibliography: multinomial.bib
---


```{r setup, include=FALSE}
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.path="figures/multinomial/")
```

## Abstract

The aim of this tutorial is to show how to perform Bayesian analysis in the case of multinomial data. Every time we have to model a situation with mutually exclusive alternatives then a multinomial likelihood arises. Think of a customer that have to choose among different brands for the same product or to the result of a football match (win, draw, loss) or to count data with a fixed sum. All these situations may be modeled with a multinomial model. The multinomial probabilities are a ratio of a function of parameters to a sum of those functions. From the complex functional form of this likelihood derives the difficulties on maximizing it. These difficulties become bigger if we introduce random effects in the model.

In this tutorial, we will use the Multinomial-Poisson transform (@baker) to transform the multinomial likelihood into a Poisson likelihood with additional parameters. The use of this transformation ensures us the same estimates and asymptotic variances. This technique is also called the Poisson Trick. Given that we can use the INLA (@inla) package to deal with Poisson variables. The key advantage of using INLA is that we can introduce complex random effects in the model paying a "discounted price" for them. @chen and @lee applied the Poisson Trick to Multinomial data studying the Yogurt Data (first introduced by @jain) but both of them use SAS and no random effects are included in the models. 
There exist packages to work directly with multinomial data, like mlogit (@mlogit) and nnet (@nnet) but even in this case is difficult to introduce a random effect in the model.    

The tutorial is organized in two main part: the first one is theoretical and talks about the multinomial logit model and the Poisson trick. The second part is more practical and talks about the parameters specification and the different models that we can implement in INLA. This part has the purpose of laying the foundations of a long-lasting friendship between the reader and INLA. All the steps needed to be friends are retraced: from the first sight (simplest model) to help each other in struggling situations (continuous space models, here INLA really helps us). The section on multinomial logit model and the multinomial data is taken by (@croissant) given the clarity and elegance of his illustration.
To make the explanation as clear as possible and to convince the reader of the effectiveness of this approach we will use simulated data. In this way, the true value of the parameters is known and we can check how well we are able to retrieve it. The code is provided and explained as well. In the end, we will report the results of our technique on the famous Yogurt Data. 

## Theoretical part

### On the Multinomial logit model

The multinomial logit model arises naturally when we have to model discrete choices. The sample unit has to make a choice between mutually exclusive alternatives. Let see how to build this kind of models. Suppose a customer has to choose between three brands of a certain good. We can define an index of satisfaction $V_{ij}$ for customer $i$ with respect to brand $j$. Assume that $V_{ij}$ is a linear function of some covariates. The situation for $i$-th customer is:
\newline

$$
\begin{cases}
V_{i1} = \beta_0 + \beta_1x_{i1} + \beta_2z_{i1} \\
\\
V_{i2} = \beta_0 + \beta_1x_{i2} + \beta_2z_{i2} \\
\\
V_{i3} = \beta_0 + \beta_1x_{i3} + \beta_2z_{i3}
\end{cases}
$$

Now we have to transform these satisfaction indexes into probabilities. Considering that the probability of choosing alternative $j$ should increase with $V_{ij}$ then the multinomial logit model is obtained by considering 
\newline
$$
P_{ij} = \frac{g_{ij}(\boldsymbol\beta)}{G_i(\boldsymbol\beta)} 
$$
Where $\boldsymbol\beta = \{\beta_0, \beta_1, \beta_2\}$ and 

$$
g_{ij}(\boldsymbol\beta) = \exp V_{ij} \quad \quad G_i(\boldsymbol\beta) = \sum_{j =1}^3 g_{ij}(\boldsymbol\beta)
$$
This specification of $P_{ij}$ ensures that the obtained probabilities are proper.
Once fitted, this model can be seen at two levels: *individual* and *aggregate*.
On the *individual* level we can study how the covariates influence the choice of the specific customer and given new data make prediction relative to the single unit.    
On the other hand, at the *aggregate* level, the estimated probabilities can be seen as market shares and study how these shares change varying the covariates could be a useful tool for brand market policies.

### On the Poisson Trick

For the sake of coherence, we adopt the same notation used in the original paper on the Multinomial-Poisson transformation (@baker). 
Let $\textbf{Y}_i = \{Y_{i1},...,Y_{ij},...\}$, for $i = 1,...,I$ and $j \in J_i$, be a vector of random variables with realization $\textbf{y}_i = \{y_{i1},...,y_{ij},...\}$. The subscript $i$ represents the sample unit and $j$ represents the alternatives. We assume that $\textbf{Y}_i$ follows a multinomial distribution with parameters proportional to $\{ g_{ij} ( \boldsymbol{\beta} ), j \in J_i \}$. Where $g_{ij}(\boldsymbol{\beta})$ is a regression function like the one seen in the previous section. To lighten the notation we omit the dependence from the covariates. The goal is to estimate the regression parameter $\boldsymbol{\beta}$. The likelihood is:


\begin{align*}
L_M(\boldsymbol{\beta}) = \prod_{i = 1}^I N_i ! \prod_{j \in J_i} \frac{1}{y_{ij}!}\left( \frac{g_{ij}(\boldsymbol\beta)}{G_i(\boldsymbol\beta)}  \right)^{y_{ij}}
\end{align*}


Where

$$
G_i(\boldsymbol\beta) = \sum_{j = 1}^{J_i} g_{ij}(\boldsymbol\beta) \, , \quad N_i = \sum_{j = 1}^{J_i} y_{ij}
$$

The problems in dealing with this likelihood come from the presence of $G_i(\boldsymbol\beta)$ which makes the use of logarithm pointless.
The idea behind the Multinomial-Poisson (MP) transformation is to treat this set of functions as additional parameters to be estimated.
The MP transformation of equation (1) is:

\begin{align*}
L_P(\boldsymbol{\beta,\phi}) = \prod_{i = 1}^{I}\prod_{j \in J_i} (g_{ij}(\boldsymbol\beta)\exp\phi_i )^{y_{ij}}\exp\{-g_{ij}(\boldsymbol\beta)\exp\phi_i \}
\end{align*}


Where $\boldsymbol\phi = \{\phi_1, ...,\phi_I\}$ is the vector of additional parameters to make the transformation work. In fact, considering the logarithm of equation (2), taking the derivative with respect to $\phi_i$ and setting it to zero we obtain the following result:

$$
\exp(\phi_i) = \frac{N_i}{G_i(\boldsymbol\beta)}
$$
\ \newline
Where $J_i$ is the size of observation $i$. this ensures that $L_P(\phi(\boldsymbol\beta), \boldsymbol\beta) \propto L_M(\boldsymbol\beta)$. On the basis of results in @richards it follows that the maximum likelihood estimates of $\boldsymbol\beta$ and their asymptotic variances are identical for $L_P(\boldsymbol{\phi, \beta})$ and $L_M(\boldsymbol\beta)$.


The name of the transformation derives from the fact that $L_P(\boldsymbol{\phi, \beta})$ is the likelihood kernel of a vector of Poisson variables 

\ \newline
$$
Y_{ij} \sim \text{Poisson}\{g_{ij}(\boldsymbol\beta)\exp\phi_i\}, \quad j \in J_i
$$

Notice that this notation is improper because in this case the $Y_{ij}$ are finite and bounded by the size of the multinomial. In a true Poisson model, the sample space is infinite. However, this expression is very useful to memorize the transformation and gives us the idea of what is going on.

Given that INLA is used to perform Bayesian analysis, we usually would have to set a prior on $\phi$. Let's assume to not have any information and to choose an improper prior for $\phi$, $\pi(\phi) \propto c$. 
Then, the marginal likelihood for the Poisson surrogate model for a single multinomial observation $Y_{ij}$ is (we will omit subscript i:

$$
L_P(\boldsymbol \beta,\phi) = \prod_{j=1}^{J}  (g_{j}(\boldsymbol\beta)\exp\phi )^{y_{j}}\exp\{-g_{j}(\boldsymbol\beta)\exp\phi \} /y_j !
$$

Integrating out $\phi$ we obtain


\begin{align*}
L_P(\boldsymbol \beta) & = \int_{-\infty}^{+\infty} \left( \prod_{j=1}^{J}  \{g_{j}(\boldsymbol\beta)\exp\phi \}^{y_{j}}\exp\{-g_{j}(\boldsymbol\beta)\exp\phi \} /y_{j} !  \right) \pi(\phi) d\phi \\
\\
& = \prod_{j=1}^{J}  \left( \frac{\{g_{j}(\boldsymbol\beta)\}^{y_{j}}}{y_j!}\right) \int_{-\infty}^{+\infty} \phi ^{J}\exp\{-G(\boldsymbol\beta)\exp\phi \} d\phi \\
\\
& = \prod_{j=1}^{J}  \left( \frac{g_{j}(\boldsymbol\beta)^{y_{j}}}{y_j!}\right) \frac{\Gamma(N)}{G(\boldsymbol \beta)^J} \\
\\
& = (N-1)!\prod_{j=1}^{J}  \frac{1}{y_{ij!}} \left( \frac{g_{j}(\boldsymbol\beta)}{G(\boldsymbol \beta)}\right)^{y_{ij}}  
\end{align*}


Where, from the second passage we omit the constant $c$ from the calculations. The obtained expression is very close to the likelihood of a Multinomial distribution. 

$$
\frac{L_P(\boldsymbol\beta)N}{c} \approx L_M(\boldsymbol \beta) 
$$
Here, we provide a small example just to prove to the reader the value of this approximation, we are going to use a binomial model that are the most simple version of multinomial. The transformation will seem obscure at the beginning, however, the goal here is to show that the likelihood actually matches and to check the correctness of the correction factor.

First of all generate a vector of proababilities and a vector of sizes.

```{r, message = FALSE, warning = FALSE}
# number of observations 
n = 10

x = rnorm(n, sd = 1)
eta = 1 + x
prob = exp(eta)/(1+exp(eta))

# Sizes
Ntrials = sample(100:1000, n, replace=TRUE)

# Observations
y = rbinom(n, size=Ntrials, prob = prob)
```

Then we set the variance of the prior of $\phi$, in particular we specify a value for the log precision.

```{r}
lprec = 25
```

This imply a standard deviation of

```{r}
sqrt(1/exp(-lprec))
```

Then, we run the INLA model

```{r}
mult.model = inla(y ~ -1 + intercept + x,
         data = data.frame(y, x, Ntrials, intercept = rep(1, n)),
         family = "binomial", Ntrials = Ntrials,
         control.predictor = list(hyper = list(prec = list(initial = lprec))),
         control.inla = list(tolerance = 1E-12))

```


Now, we have to replicate it with a surrogate Poisson model. The $n$ binomial observations become $2*n$ Poisson observation, consisting of $y_i$ and $N_i - y_i$. 

```{r}
# initialize
Y = intercept = xx = c()
for(i in 1:n) {
    # corresponding vector of observations
    Y = c(Y, c(y[i], Ntrials[i] - y[i]))
    # corresponding covariates
    xx = c(xx, c(x[i], 0))
    intercept = c(intercept, c(1, 0))
}

```

Then, we have to create a vector representing $\boldsymbol \phi$ which assume a different value for each binomial observations

```{r}
phi = rep(1:n, each=2)
```

Now, we can fit the Poisson model

```{r}
pois.model = inla(Y ~ -1
          + intercept
          + f(phi, model="iid",
              hyper = list(prec = list(initial = -lprec, fixed=TRUE)))
          + xx,
          family="poisson",
          data = data.frame(Y, xx, phi , intercept), 
          control.predictor = list(hyper = list(prec = list(initial = lprec))), 
          control.inla = list(tolerance = 1E-12))

```

The correction factor for the log-likelihood for each binomial observation will be the sum of $\log(N_i)$ and the logarithm of the flat prior.

```{r}
corr.factor = - sum(dnorm(pois.model$summary.random$phi$mode,
                          sd = sqrt(1/exp(-lprec)), log=TRUE)) + sum(log(Ntrials))
print(mult.model$mlik)
print(pois.model$mlik + corr.factor)
```


## Practical Part

Before going through this tutorial be sure to have the following packages installed
\ \newline
```{r, message=F}
library(Ecdat)
library(deldir)
library(sp)
library(rgeos)
library(mvtnorm)
library(gridExtra)
library(mlogit)
```


### On the Multinomial Data and Model Parameters

In this section, we illustrate the shape that our data has to have in order to work with INLA properly. This shape is different from the ones used in the literature, therefore, we strongly recommend the reader to read this part carefully. The two dataset shapes known in literature are the *wide* format and the *long* format. More detail on @croissant. We will use a mix of these two, the shape of a variable depends on its nature. The types of variable that we could encounter will be listed and explained in a minute. Before of that we want to be sure that the structure of the data is clear. 

As we said before the Multinomial model explains choice situations; in these situations there is an individual which choose between mutually exclusive alternatives. This structure is reflected in the data that is characterized by three indexes:

* the alternative
* the choice situation
* the individual

The last one is relevant only if we have repeated observation for the same unit. We will not study this kind of situations in the tutorial. So we have two indexes to take into account, one for the alternative and one for the choice situation.  
While working with Multinomial logit models, there exist three kinds of variables:

* **alternative specific** variables $x_{ij}$ with a **generic** coefficient $\beta$. These kind of variables are relative to the alternative and could change for each choice situation. The coefficient is generic: it is not supposed to depend on the alternative. Think to a transport mode situation; spending one euro in car or train is the same, then, we can consider the price as an alternative specific variable with a generic coefficient

* **alternative specific** variables $w_{ij}$ with an **alternative specific** coefficient $\delta_j$. The difference between this category and the previous lies in the coefficient. For these variables, the coefficient depends on the alternative. Continue with the transport mode example; one hour more of travel time has a different impact if spent in a car or in a train. Then, it is reasonable to consider the travel time as alternative specific with an alternative specific coefficient

* **individual specific** variables $z_{i}$ with an **alternative specific** coefficient $\gamma_j$. These variables are relative only to the individual and they have the same value for all the alternatives. The coefficient has to be alternative specific for reason that will see soon. Finishing with the transport mode example we can think of the income of the individual; this variable is clearly related only to the individual. 

In the example that we will use from now on there are all these kind of variables. The model that we are considering is the following:

$$
V_{ij} = \beta x_{ij} + \delta_j w_{ij} + \gamma_j z_i
$$
\ \newline
The model may have an intercept, we choose to omit it just to lighten the formulas. Notice that to model the choice of one alternative over the others we will look only on the differences between satisfaction values. Satisfaction is ordinal. Moreover, given the construction of the multinomial logit model, differences in satisfaction stand for ratio on probabilities. Let us write down these differences:

$$
V_{ij} - V_{ik} = \beta(x_{ij} - x_{ik}) + (\delta_jw_{ij} - \delta_kw_{ik}) + z_i(\gamma_j - \gamma_k)
$$
\ \newline
Focusing on $z_i$: if the $\gamma$ is not alternative specific the contribution of this covariate would disappear in the differentiation. Furthermore, only difference $\gamma$'s may be identified. We will back to this point later.

### Data structure

Now we are ready to start with our example. We will work with a multinomial model with three alternatives: A, B, and C. The covariates are also three; one per category. We consider a small data set of $500$ data points. The size of the multinomial will be variable and we will study the effect of this parameter on the posterior estimates. Declare the parameters and the covariates:
\ \newline
```{r, message=FALSE}
beta = -0.3
deltas = c(1, 4, 3)
gammas = c(0.3, 0.2, 0.4)
param = c(beta, deltas, gammas)
n = 500

set.seed(123)
# alternative specific with generic coefficient beta
X.A = rnorm(n, mean = 30, sd = 2.5)
X.B = rnorm(n, mean = 35, sd = 3.5)
X.C = rnorm(n, mean = 40, sd = 1)

# alternative specific with alternative specific coefficient delta 
W.A = abs(rnorm(n, mean = 1))
W.B = abs(rnorm(n, mean = 1))
W.C = abs(rnorm(n, mean = 1))

# individual specific with alternative specific coefficient gamma 
Z = rnorm(n, 20, 3)

```
\ \newline
As first example we don't consider any random effect. 
We write a function of the size $N$ of the multinomial to generate the sample.
\ \newline
```{r}
Multinom.sample = function(N){
  Y = matrix(NA, ncol = 3, nrow = n)
  for(i in 1:n){
    V.A = beta*X.A[i] + deltas[1]*W.A[i] + gammas[1]*Z[i]  
    V.B = beta*X.B[i] + deltas[2]*W.B[i] + gammas[2]*Z[i] 
    V.C = beta*X.C[i] + deltas[3]*W.C[i] + gammas[3]*Z[i] 
    
    probs = c(V.A, V.B, V.C)
    probs = exp(probs)/sum(exp(probs))
    samp = rmultinom(1, N, prob = probs)
    
    Y[i,] = as.vector(samp)
  }
  colnames(Y ) = c("Y.A", "Y.B", "Y.C")
  return(Y)
}
```
\ \newline
Take a look of what it produces.
\ \newline
```{r}
head(Multinom.sample(1), 5)
head(Multinom.sample(100), 5)
```

Once we have data we have to build the data set to give to INLA. 
The dataset will have $n*3$ rows, one for each choice alternative in each choice situation. Now we have to add the covariates and this is the tricky part.
The structure of each covariate depends on its type, fortunately, there are only three types and two of them have the same structure. For an **alternative specific** variable with a generic coefficient and for an **individual specific** variable we just use the long format. For alternative specific, each row contain just the value of the variable referred to the corresponding alternative; for the individual specific the value is repeated along the alternatives. For an **alternative specific** with an alternative coefficient, we need three columns, one for each alternative (like the wide format) and $n*3$ rows (like the long format). This is the mixed structure. Each column will have the value of the covariate for the corresponding alternative and zeros for the others. The complicated part is done but the data construction is not finished yet. We need two more columns. An index column representing the **choice situation**, this column will be our additional parameter $\phi$. The last column is another index column representing the **choice alternatives**. We hope the code will clarify what it is hazy to the reader. The following function takes as input a data frame in wide format and transforms it into the desired structure.
\ \newline
```{r}
Y = Multinom.sample(1)
df = data.frame(cbind(Y, X.A, X.B, X.C, W.A, W.B, W.C, Z))

Data.structure = function(df){
  Data = matrix(NA, ncol = 8, nrow = n*3)
  for(i in 1:n){
    # simulated variable
    Data[((i-1)*3+1):(i*3), 1] = c(df$Y.A[i], df$Y.B[i], df$Y.C[i])
    # alternative specific with generic coeff
    Data[((i-1)*3+1):(i*3), 2] = c(df$X.A[i], df$X.B[i], df$X.C[i])
    # alternative specific with alternative coeff
    Data[((i-1)*3+1):(i*3), 3:5] = diag(c(df$W.A[i], df$W.B[i], df$W.C[i]))
    # individual specific with alternative coeff
    Data[((i-1)*3+1):(i*3), 6] = rep(df$Z[i],3)
    # choice situation index
    Data[((i-1)*3+1):(i*3), 7] = rep(i,3)
    # choice alternative index
    Data[((i-1)*3+1):(i*3), 8] = c(1, 2, 3)
  }
  Data = data.frame(Data)
  names(Data) = c('Y', "X","W.A","W.B","W.C","Z",'phi','alt.idx')
  return(Data)
}

round(head(Data.structure(df)),3)
```

### A Simple Model

Once we have the data, we can fit our first model, without random effect and intercept. First of all, we need to build a formula representing our model. We remember to the reader that the model at hand is the following:
$$
Y_{ij} \sim \text{Poisson}(\lambda_{ij})
$$
\ \newline
Where
\ \newline
$$
\log(\lambda_{ij}) = \phi_i + \beta x_{ij} + \delta_{ij}w_{ij} + \gamma_j z_i
$$

The corresponding formula is:

```{r}
formula = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T)
```

Let us explain it. The $-1$, in the beginning, means that we are not considering an intercept. Again, the way in which a variable is declared depends on his type. For the alternative specific variable, we just declare them as they are, without any particular adjustments. The choice situation index variable is declared as a random effect to speed up the code. The individual specific variable is also defined as a random effect. In this way, we can add to the model a sum up to zero constraint on the $\gamma$'s imposing $\texttt{constr = T}$.     
As we said before we may identify only the differences between $\gamma$'s, that gives rise to a system of two equation (one for each difference) in three unknowns. In order to have a unique solution, we need to consider an additional constraint. The only thing missing is to fit the model, if not specified INLA considers a diffuse prior for the parameters. We consider $N = 1$
\ \newline
```{r}
Data = Data.structure(df)
model = inla(formula, data = Data, family = 'Poisson')
```
\ \newline
Take a look of what we get
\ \newline
```{r}
result = rbind(model$summary.fixed[1:5], model$summary.random$alt.idx[2:6])
result = cbind(result, true = param)
row.names(result) = c("beta","delta.A","delta.B","delta.C","gamma.A","gamma.B","gamma.C" )
round(result,3)
```
\ \newline
As we can see the estimates for the $\beta$ and $\delta$'s are fine. Instead, the estimates for $\gamma$'s are pretty bad. But, let us look at the difference between them
\ \newline
```{r}
diff.result = 
  cbind("0.025quant"= diff(model$summary.random$alt.idx$`0.025quant`),
      "0.5quant" = diff(model$summary.random$alt.idx$`0.5quant`),
      "0.975quant" = diff(model$summary.random$alt.idx$`0.975quant`),
      "true" = diff(gammas))
row.names(diff.result) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
round(diff.result,3)
```
\ \newline
As we said before even if we are not able to estimates the true value of the $\gamma$ parameters we can retrieve their differences.

#### Changing the size

What happens if we change the size of the multinomial? Increasing the size of the multinomial gives us more information about the underlying model. Think about it: having a multinomial distribution of size one implies that $y_{ij} \in \{0,1\}$. Remember that we are treating the observations as they come from a collection of Poisson variables. The Poisson is a model for counts data and having only zeros and ones is not the best situation in which use it. Increasing the size we have a more likely counts data and our estimates are more precise.  

To prove that, we run one hundred simulation for each $N \in \{1,100,500,1000\}$. Figure 1 shows the confidence intervals for $\beta$, the results are pretty similar for $\delta$'s. The estimates remain centered on the true value of the parameters and the variance reduces as $N$ increases. The situation is a bit different for the difference between $\gamma$ parameters. In this case, the variance of the posterior is small even for $N = 1$ but the intervals get closer and closer to the true value as $N$ increase as it is shown in Figure 2. The take-home message here is to find a way to have $N$ as large as possible. Think to the customer example, instead of considering each customer as a single unit we could aggregate to some covariate level. For example, knowing where the customer made his choice and aggregate to the shop level could be interesting.

Be careful, *All that glitters is not gold*. In this situation, we know the truth and we are using the true model. Using the wrong model could give us very concentrated results in an unrealistic part of the parameters space. If we are not sure of the model, then, take a small $N$ is better because it ensures more flexible posterior results.    


![Confidence Interval for beta varying the size parameter N](multinomial-img/beta.confidence.jpeg){width=500px}

![Confidence Interval for the difference between gamma.B and gamma.A varying the size parameter N](multinomial-img/diff.confidence.jpeg){height=350px}



\ \newline 
\ \newline

### A First Random Effect

Now that we have gain confidence with the multinomial logit model it's time to add a first, very simple, random effect to the model. As a first example, we consider an independent and identical distributed random effect. The random effect influences only the satisfaction index related to choice $A$, otherwise, considering three random effects would make the model non-identifiable. The model for the other probabilities is exactly the same of the previous section. The modified model is the following:
$$
V_{iA} = \beta x_{iA} + \delta_A w_{iA} + \gamma_A z_i + v_i 
$$
$$
v_i \sim N(0, 1) \quad\quad  i = 1,...,n
$$

In the above formula, $v_i$ represents the random effect that acts in each choice situation, the way in which it influences the choice is independent between different situations. Using the Poisson transformation we are saying that the random effect plays a role only on the observations relative to alternative $A$.
In other words the observations for which $\texttt{alt.idx = 1}$. 
Let us generate the random effect and write a function to generate data following this structure. To have a better fit for this example we consider a multinomial of size $N = 100$
```{r}
random.effect = rnorm(n)
Multinom.sample.rand = function(N, random.effect){
  Y = matrix(NA, ncol = 3, nrow = n)
  for(i in 1:n){
    V.A = beta*X.A[i] + deltas[1]*W.A[i] + gammas[1]*Z[i] + 
      random.effect[i]
    V.B = beta*X.B[i] + deltas[2]*W.B[i] + gammas[2]*Z[i] 
    V.C = beta*X.C[i] + deltas[3]*W.C[i] + gammas[3]*Z[i] 
    
    probs = c(V.A, V.B, V.C)
    probs = exp(probs)/sum(exp(probs))
    samp = rmultinom(1, N, prob = probs)
    
    Y[i,] = as.vector(samp)
  }
  colnames(Y) = c("Y.A", "Y.B", "Y.C")
  return(Y)
}

Y.rand1 = Multinom.sample.rand(100, random.effect)
df.rand1 = data.frame(cbind(Y.rand1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.rand1 = Data.structure(df.rand1)
```
\ \newline
In INLA adding a random effect is simple. We just need an index variable indicating on which observations it acts, the others will be set to $\texttt{NA}$. Then, first step: add an index variable to the data.
\ \newline
```{r}
rand.idx = rep(NA, n*3)
rand.idx[seq(1,n*3, by = 3)] = seq(1,n)
Data.rand1$rand.idx = rand.idx
round(head(Data.rand1),3)
```
\ \newline
The other thing that we need in INLA to consider a random effect is to specify it in the formula. To do that we use the function $\texttt{f()}$; it takes as inputs the index column that we have created before and the model, equal to $\texttt{iid}$ in this case. In the next sections, we will see that to consider a structured random effect it is sufficient to change the model option.
\ \newline
```{r}
formula.rand1 = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rand.idx, model = "iid")  #random effect
```
\ \newline
Then, just run the $\texttt{inla()}$ function and check the result.
\ \newline
```{r}
model.rand1 = inla(formula.rand1, data = Data.rand1, family = 'Poisson')
result.rand1 = rbind(model.rand1$summary.fixed[1:5])
result.rand1 = cbind(result.rand1, true = param[1:4])
row.names(result.rand1) = c("beta","delta.A","delta.B","delta.C")
round(result.rand1,3)
```
\ \newline
```{r}
diff.result.rand1 = 
  cbind("0.025quant"= diff(model.rand1$summary.random$alt.idx$`0.025quant`),
      "0.5quant" = diff(model.rand1$summary.random$alt.idx$`0.5quant`),
      "0.975quant" = diff(model.rand1$summary.random$alt.idx$`0.975quant`),
      "true" = diff(gammas))
row.names(diff.result.rand1) = c("gamma.B - gamma.A", "gamma.C - gamma.B")
round(diff.result.rand1,3)
```
\ \newline
Regarding the random effect, there are many indexes that we can use to evaluate the estimates. We choose the simplest one and check the accuracy of our estimates calculating the mean square error which is satisfyingly low.
\ \newline
```{r}
mean((random.effect - model.rand1$summary.random$rand.idx$`0.5quant`)^2)
```
\ \newline
Exactly as before, increasing the size of the multinomial we have better estimates. We run one hundred simulations for each value of size $N \in \{ 1,100,500,1000\}$. The statements that we have done in the previous section are valid also for the random effect estimates. Greater the size better the accuracy. In Figure 3 are reported the box plots of the mean square errors for each value of $N$.

![Boxplot of the estimated random effect mean square errors varying the size parameter N](multinomial-img/MSE.iid.jpeg){width=350px}


The estimates with $N = 1$ are particularly bad, that's why there is not enough information to evaluate correctly the random effect and the resulting estimates are almost zero. To make this point clearer in Figure 4 are reported the confidence intervals relative to $20$ samples from the random effect.

![Confidence intervals for a sample of 20 points from the random effect varying the size parameter N](multinomial-img/conf.iid.jpeg){width=485px}

\ \newline
\ \newline

### Time Random Effects

Now that we are a little bit more familiar with INLA (not friends yet), we can go further and add a simple structure to our random effect. Consider one of the easiest and most used structures for random effects: a random walk. This kind of random effects is particularly useful when we have observations repeated in time. For example, we have the choices story of a customer. In this situation we can imagine that these choices are linked together, that exists a connection between adjacent observations. In this situation, we may use a random walk of some order to link the observations. Generate a random walk of order one and the corresponding data.
\ \newline
```{r, fig.height=2.5}
#  Generate the random effect
random.walk1 = rep(NA, n)
random.walk1[1] = 0
for(i in 2:n){
  random.walk1[i] = rnorm(1, mean = random.walk1[i-1], sd = 0.1)
}

# Generate data
Y.rw1 = Multinom.sample.rand(100, random.walk1)
df.rw1 = data.frame(cbind(Y.rw1, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.rw1 = Data.structure(df.rw1)

```
\ \newline
The INLA package needs exactly the same thing of the previous section: an index variable and a $\texttt{f()}$ function. The difference is in that now we have to write $\texttt{model = "rw1"}$ when we specify the formula. This syntax is pretty general in INLA for the random effects. For example, if we want to consider a random walk of order two instead of one, we have to specify $\texttt{model = "rw2"}$. For a complete list of the models just run this command $\texttt{names(inla.models()\$latent)}$. For more details on the use of INLA check (@rinla).
Back to the problem, the index column is the same of the previous section since only the "first" probability is affected by the random effect. The model is the following:    

$$
V_{itA} = \beta x_{itA} + \delta_A w_{itA} + \gamma_A z_{it} + u_t 
$$
\ \newline
Where $i$ is an index for the individual, in our example we consider all the observations as relative to the same customer. Subscript $t$ represents the time in which the choice is made and $u_t$ is a random walk of order one such that:
\ \newline
$$
u_t|u_{t-1} \sim N(u_{t-1}, \sigma^2_u)
$$
\ \newline
Also the syntax to fit the model is exactly the same as before.
\ \newline
```{r}
rw1.idx = rep(NA, n*3)
rw1.idx[seq(1,n*3, by = 3)] = seq(1,n)
Data.rw1$rw1.idx = rw1.idx

formula.rw1 = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rw1.idx, model = "rw1")  #random walk of order 1

model.rw1 = inla(formula.rw1, family = 'poisson', data = Data.rw1)
```
\ \newline
To check the goodness of our estimate we show the trace plots of the true random walk and the estimated one. In particular to make the curves match we have to subtract the mean from the true random walk. That's why the model assumes a zero mean random walk and the simulated one has not mean equal to zero. This technicality is relevant only if we know the truth and we have to plot it. As it is shown in Figure 5 the curves match very well.
\ \newline
```{r, fig.cap=c("Estimate of a random walk, the black line represents the true one and the red line the estimate"), echo=FALSE, fig.height=4}
plot(random.walk1 - mean(random.walk1), type = 'l', 
     main = 'Random walk estimate', ylab = 'values', xlab = 'time')
points(model.rw1$summary.random$rw1.idx$`0.5quant`, type = 'l', col = 'red')
```


### Discrete Spatial Random effect

Now that the seeds of our INLA-friendship are sowed we can move forward and start talking about spatial random effect. Spatially structured random effects are useful in many situation; take again the customer example and assume that we know the location of each choice situation, where the individual made its choice. This choice could be influenced by some advertising campaign for which we don't have data. Think of a billboard, only the customers near it may be influenced.

In general, whenever the covariates don't explain enough the variability of the observations, the observations given the covariates are not independent, then considering a spatial structure may mitigate the problem. Let us start with the discrete case. We have the space that is divided in a certain number of regions (e.g. municipality, cities, countries,...) and each region is influenced and influence only its neighbors. Each data point in a region is subject to the same random effect. To simulate data with this structure we the points locations and a partition of the space. For the locations we just pick $n$ points at random in the set $[0,1]\times[0,1]$.

```{r}
x.loc = runif(n)
y.loc = runif(n)
loc = cbind("x.loc" = x.loc, "y.loc" = y.loc)
```
\ \newline
For the partition is a little bit more complicated. One way to do that is to select some point at random in $[0,1]\times[0,1]$ and construct the Voronoi Tessellation using these points as centers. We are not going through the details of Voronoi tessellation, for whom is interested more details on @compgeom.
This is a function to generate a Voronoi polygons given points, it uses functions from the $\texttt{sp}$ (@sp) and $\texttt{deldir}$ (@deldir) libraries.
\ \newline
```{r, message=FALSE}

voronoi.polygons <- function(x) {
  require(deldir)
  if (.hasSlot(x, 'coords')) {
    crds <- x@coords  
  } else crds <- x
  z <- deldir(crds[,1], crds[,2])
  w <- tile.list(z)
  polys <- vector(mode='list', length=length(w))
  require(sp)
  for (i in seq(along=polys)) {
    pcrds <- cbind(w[[i]]$x, w[[i]]$y)
    pcrds <- rbind(pcrds, pcrds[1,])
    polys[[i]] <- Polygons(list(Polygon(pcrds)), ID=as.character(i))
  }
  SP <- SpatialPolygons(polys)
  voronoi <- SpatialPolygonsDataFrame(SP, 
                                      data=data.frame(x=crds[,1],
                                                      y=crds[,2], 
                                                      row.names=sapply(slot(SP, 'polygons'), 
                                                                       function(x) slot(x, 'ID'))))
}
```
\ \newline
To generate a partition of the space just pick some random points, be sure that the corners of your study area are included and the above function will generate one region (polygon) for each point. Before move on we would like to give some information on the output of $\texttt{voronoipolygons()}$. This function gives us a SpatialPolygonsDataFrame. This kind of object is very useful when we are dealing with spatial data. Each row of the data frame is related to a region and we can put there whatever we want as a column. How to use it and how to make nice plots will be shown in a while, first of all, let us generate the partition.
\ \newline
```{r, message=F}
# number of regions
n.reg = 50
# corners
boundaries = rbind(c(0,0), c(1,1), c(0,1), c(1,0))
# points
points = rbind(boundaries, cbind(runif(n.reg - 4), runif(n.reg - 4)))
# generate Voronoi polygons
vor = voronoi.polygons(points)
```
\ \newline
Now let us check how many non-empty regions do we have and make the first plot with the Voronoi polygons and the locations.
\ \newline
```{r}
# Create an ID column for the regions, it will be our index
vor@data$id = seq(1,n.reg)
# Transform the locations in SpatialPoint
pp = SpatialPoints(loc)
# This vector contains the region in which each point is located
id.samples = over(pp,vor)$id
# Check the number of non epty regions 
check = as.integer(length(unique(id.samples)))
check
```

```{r, fig.cap=c("Partiotion of the region [0,1]x[0,1], each red point correspond to a different location"), fig.height=5}
# plot the partition and the locations
plot(vor)
points(loc, pch = 21, bg = 2, cex = 0.7)
```

Once we have a partition of the space we need to generate a random effect. To take into account the neighbors structures of the partition we choose to use a **Gaussian Markov Random Field**. More details on (@gmrf) and (@rinla). Skipping all the theoretical stuff, we need to synthesize the neighbors' structure; we need the adjacency matrix $A$. With $A$ in the hands, we will generate the random effect from a multivariate Gaussian distribution with a covariance matrix that depends on the adjacency matrix. This model is also called CAR model or iCAR model depending on the specification of the covariance matrix. To calculate the adjacency matrix we use the function $\texttt{gTouches()}$ of the package $\texttt{rgeos}$, @rgeos. To sample from the multivariate normal we use $\texttt{rmvnorm()}$ function of the package $\texttt{mvtnorm}$, @mvtnorm.
More details on polygons here [Polygons web Page]
\ \newline
```{r, message = F, warnings = F}
# adjacency matrix
ADJ = gTouches(vor, byid = T)
# marginal variance of each random effect component
S = diag(n.reg)
# covariance matrix
cov.matrix = solve(diag(n.reg) - ADJ)%*%S
# generate the random effect and add it to the SpatialPolygonDataFrame
vor@data$rand.eff = as.numeric(rmvnorm(1,sigma = cov.matrix))

```

```{r, fig.cap=c("Random effect on the partition"), fig.height=4}
spplot(vor, 'rand.eff', col.regions = terrain.colors(32))
```

\ \newline
We can breathe a sigh of relief, this tricky part about polygons is over. What remains to do is to generate data accordingly to this model, remember, each location in the same region is affected by the same value of the random effect.
Then we can use the same function used before to sample from the data, we just need to build the random effect vector in the proper way.
\ \newline
```{r}
# generate random effect vector
random.effect = vor@data$rand.eff[id.samples]
# generate sample
Y.spatial = Multinom.sample.rand(N = 100, random.effect)
# construct data set
df.spatial = data.frame(cbind(Y.spatial, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.spatial = Data.structure(df.spatial)
```
\ \newline
Using the same function to generate data as before means that the spatial random effect influences only the first probability, then we have to build an index column with non zero elements only on the row relative to the first alternative. These elements are the id of the region in which the observation is located. The index column plays the same role of the previous examples: using it as input on the $\texttt{f()}$ function in the formula we specify our spatial random effect.
In this situation we set $\texttt{model = "besag"}$ because was him to introduce this model in @besag. The only additional requirements is that we need to specify a neighbors structure in the $\texttt{f()}$ function, it is done using the adjacency matrix setting $\texttt{graph = ADJ}$.

```{r}
# create and add index column
rand.eff.idx = rep(NA, n*3)
rand.eff.idx[seq(1,n*3, by = 3)] = id.samples
Data.spatial$rand.eff.idx = rand.eff.idx

# Formula
formula.spatial = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) + 
  f(alt.idx, Z, fixed = T, constr = T) + 
  f(rand.eff.idx, model = 'besag', graph = ADJ) #spatial random effect

# Fit the model
model.spatial = inla(formula.spatial, family = "poisson", data = Data.spatial)

# Check the results
result.spatial = cbind(model.spatial$summary.fixed[3:5], true = param[1:4])
row.names(result.spatial) = c("beta","delta.A","delta.B","delta.C")

diff.result.spatial = 
  cbind("0.025quant"= diff(model.spatial$summary.random$alt.idx$`0.025quant`),
        "0.5quant" = diff(model.spatial$summary.random$alt.idx$`0.5quant`),
        "0.975quant" = diff(model.spatial$summary.random$alt.idx$`0.975quant`),
        "true" = diff(gammas))
row.names(diff.result.spatial) = c("gamma.B - gamma.A", "gamma.C - gamma.B")

round(rbind(result.spatial, diff.result.spatial),3)
```
\ \newline
We have just seen that, regarding the parameters, the model works. To check if it works for the random effect as well we calculate the mean square error and make a plot with the true random effect, the estimated one, and the residuals. 
Using the SpatialPolygonsDataFrame we just need to add the values that we want to plot to the data frame and then plot them. To plot multiple figure at the same time we have to use the function $\texttt{grid.arange()}$ of the $\texttt{gridExtra}$ package (@gridextra).

```{r, fig.cap = c("Comparison between the true random effect and the estimated one"), fig.height=5}
# compute mean square error
mean((vor@data$rand.eff -
        model.spatial$summary.random$rand.eff.idx$`0.5quant`)^2)

# add columns to the SpatialPolygonDataFralme
vor@data$rand.eff.est = model.spatial$summary.random$rand.eff.idx$`0.5quant`
vor@data$residuals = (vor@data$rand.eff-
                        model.spatial$summary.random$rand.eff.idx$`0.5quant`)^2
# plot them
grid.arrange(spplot(vor, 'rand.eff', at = seq(-2.5, 2.5, length.out = 32),
                    col.regions = terrain.colors(32), main = "True"),
             spplot(vor, 'rand.eff.est', at = seq(-2.5,2.5,length.out = 32),
                    col.regions = terrain.colors(32), main = "Estimates"),
             spplot(vor, 'residuals', at = seq(-2.5,2.5,length.out = 32),
                    col.regions = terrain.colors(32), main = "Residuals"),
             nrow = 1)
```

\ \newline
\ \newline
\ \newline

### Continuos spatial random effect

This is the last step to definitively consolidate our friendship with INLA, it's here that this package shows its full potential and becomes a truly desirable friend. In the previous section we assumed a discrete spatial effect, it means that the effect in a specific region is constant. This approach works well in many situations in which we have an effect related to the county or to the municipality or to the district. What happens if we are working with natural phenomena? In this case, we may suppose that the effect is not constant inside a region or inside a municipality. Think about temperature or air pollution level, in this case, considering the effect derived from these variables as constant inside a region doesn't seem reasonable. The solution is to consider a continuous random effect (or at least very small regions). We can do that using the so-called SPDE approach. You can find an entire tutorial on this argument on @spde.

Few words about this: we are considering a **Gaussian random field** with zero mean and Matérn correlation function. Zero mean because the mean is supposed to be modeled by the fixed effects. About the Matérn correlation function, the important thing to have in mind is that it depends on the Euclidean distance between points. This means high correlation for points close to each other and low correlation for far away points. Of course, we cannot deal with something really continuous, otherwise, it will be computationally infisible, but we can deal with something discrete defined on a set of very small regions. Hence, the purpose is to approximate the continuous spatial random effect with a discrete spatial effect (@explicit). Let us start from here: discretization of the study area. We divide the area into small triangles, in each triangle the random effect is supposed to be linear. This operation is called Delaunay Triangulation and the result is a mesh. Even if it seems a complex operation we can build a mesh using INLA in one line of code. The function $\texttt{inla.mesh.2d()}$ requires a set of location and some parameters to guarantee a "nice" triangulation. More details on the parameters and what "nice" means on [Mesh website].
```{r}
mesh = inla.mesh.2d(loc, max.edge = c(0.02, 0.2), cutoff = 0.02)
```
\ \newline
Let us plot it and check how many triangles we have created for the area of interest. The mesh, being the support for our approximation, more triangles it has, more precise the approximation will be.
\ \newline
```{r, fig.cap = c("Triangulation of the study area [0,1]x[0,1], the red dots represent the data points locations"), fig.height=5}
mesh$n
par(mfrow = c(1,1))
plot(mesh)
points(loc, pch = 21, bg = 2)
```
\ \newline
\ \newline
Now we have to simulate our continuous spatial random effect, it depends on two parameters: the range $r$ and the marginal variance $\sigma_0^2$. The range is the distance at which the correlation of two points is around 0.13. The marginal variance is the marginal variance. We will set the range $r$ equal to a fifth of the size of the study area and $\sigma^2_0 = 1$. We pass the parameters to INLA using a different parametrization.
\ \newline
```{r}
# starting parametrization
size <- min(c(diff(range(mesh$loc[, 1])), diff(range(mesh$loc[, 2]))))
range <- size/5
sigma0 = 1

# new parametrization
kappa0 <- sqrt(8)/range
tau0 <- 1/(sqrt(4 * pi) * kappa0 * sigma0)

# create an spde object
spde <- inla.spde2.matern(mesh, B.tau = cbind(log(tau0), -1, +1),
                          B.kappa = cbind(log(kappa0), 0, -1))
```
\ \newline
It remains only to simulate the random effect. To do that we simulate from a **Gaussian Markow random field**, each component is related to a triangle and the precision Matrix $Q$ is defined through a Matérn covariance function. The INLA package helps us to retrieve the precision matrix $Q$
\ \newline
```{r}
Q = inla.spde.precision(spde, theta = c(0,0))
# the number of elements of Q is the square of the number of triangles 
dim(Q)
```
\ \newline
Let us simulate
\ \newline
```{r}
sample = inla.qsample(n = 2, Q, seed = 123)
```
\ \newline
Now we have a sample in which each element is related to a triangle and we need the values of the effect on the data points locations. To pass from the mesh locations to the data locations we need a matrix $A$ such that:
$$
u(\boldsymbol{s}) = Au(\boldsymbol s')  
$$
Where $\boldsymbol s = \{s_1,...s_N\}$ are the data points locations and $\boldsymbol s' = \{s'_1,...,s'_{N'}\}$ are the mesh locations. The INLA package comes to help us again with the $\texttt{inla.spde.make.A()}$ function. This function takes as input a mesh and a set of location and computes the projection matrix $A$. Please pay attention: remember that working with Multinomial-Poisson transformation implies that for each location (multinomial observation) we have a number of Poisson observation equal to the number of alternatives. All of them are related to the same location, then, we have to replicate each location $3$ (number of alternatives) times. As before, we are considering a random effect acting only on the probability of the first alternative, hence, we have to select only the rows relatives to that one.
\ \newline
```{r}
# create A
A = inla.spde.make.A(mesh = mesh,
                      loc = cbind(rep(loc[,1], each = 3), 
                                  rep(loc[,2], each = 3)))

# select rows relative to the first alternatives
A = A*rep(c(1,2,3) == 1, n)
```
\ \newline
The random effect will be the first column of the product between the projection matrix $A$ and the simulated **Gaussian Markov random field** on the mesh locations.
\ \newline
```{r}
random.effect = matrix(drop(A%*%sample[,2]), ncol = 3, byrow = T)[,1]
```
\ \newline
To convince the reader about what we have done since now we show the plots of the continuous random effect and the obtained projection on the data points locations. We are not going through the details of how to obtain these plots, the interested reader can find more detail on (page Mesh). Figure 8 shows the comparison between the continuous random effect and the projection on the sample locations.
\ \newline
```{r, fig.cap=c("The right figure represents the continuos spatial effect, the left figure represents the value of the random effect on the sample locations" ), eval.after='fig.cap'}
par(mfrow = c(1,2))

# plot the continuos random effect
proj <- inla.mesh.projector(mesh, dims = c(100, 100))
sample_proj = inla.mesh.project(proj, field = sample[,2])
image(proj$x, proj$y, sample_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '')
contour(proj$x, proj$y, sample_proj, add = T)

# plot the projection on the sample locations
rbPal <- colorRampPalette(heat.colors(100))
Col <- rbPal(100)[as.numeric(cut(random.effect, breaks = 100))]
plot(loc, pch = 20, col = Col, xlab = '', ylab = '')
```

\ \newline
\ \newline
Now that we have a random effect defined on the locations we can go through the simulation of the multinomial data.
\ \newline
```{r}
# generate sample
Y.spde = Multinom.sample.rand(N = 100, random.effect)
# construct data set
df.spde = data.frame(cbind(Y.spde, X.A, X.B, X.C, W.A, W.B, W.C, Z))
Data.spde = Data.structure(df.spde)
round(head(Data.spde),3)
```
\ \newline
To perform a Bayesian analysis with this kind of random effect with INLA we need to specify two more objects. This part is crucial to the success of our analysis, we shall never tire of repeating, the data is defined on a set of locations and the random effect is defined on another set of locations. From this incongruity derives the needing of an object that manages it for us. Again, INLA proves to be a good friend and provides us this object: the stack.
The $\texttt{inla.stack()}$ function does the job, it mainly needs three thing:

* **data**: a *list* containing the variable subject of study and the offset if it is present.

* **effect**: a *list* containing the effects included in the model. The spatial random effect is represented by an index created with the function $\texttt{inla.spde.make.index()}$; the fixed effects are represented by the columns of the data frame containing the covariates.

* **A**: a *list* containing the projection matrices: one for the random effect and one for the fixed effects. The projection matrix for the random effect is the matrix $A$ created above. The projection matrix for the fixed effects is just $1$ because the covariates are recorded in the same locations of the data. Notice that if the covariates are recorded in different locations with respect the data, then we have to specify a different matrix.
\ \newline
```{r}
# index for the random effect
s_index <- inla.spde.make.index(name="spatial.field",
                                n.spde= spde$n.spde)

# stack object
stack = inla.stack(data = list(Y = Data.spde$Y),
                   effects = list(s_index, list(Data.spde[,2:8])),
                   A = list(A, 1))

```
\ \newline
We are at the end of our SPDE-journey and, finally, we are ready to fit the model. As always we need to specify a formula and an $\texttt{f()}$ function for the spatial random effect, in this case the index variable is the $\texttt{spatial.field}$ (element of the SPDE index created above). The model option is $\texttt{spde}$ and we need also to specify an additional option $\texttt{group = spatial.field.group}$ (another element of the SPDE index). After that, we can run the $\texttt{inla()}$ function and wait for the results. It will take some time especially if we work with a lot of data. Notice that the data is specified using the stack object. In particular we need to use the function $\texttt{inla.stack.data()}$. Furthermore, we need to specify also the projection matrix $A$. It is done in the $\texttt{control.predictor}$ option. This option requires a list containing an element $A$ and $\texttt{compute = T}$.

In the following code, we use a little trick to speed up the computations. This trick can be done only after a first fit of the model. It consists in specify values for the hyper-parameters and then use the option $\texttt{control.inla= list(int.strategy = "eb")}$. The last line of the chunk shows how to retrieve the hyper-parameters values. Without this adjustment, INLA takes around $2$ minutes to fit the model. Adding the starting value and using $\texttt{eb}$ strategy it takes around $7$ seconds. We strongly recommend doing this every time you have to fit a model multiple times trying little changes.
\ \newline
```{r}
formula.spde = Y ~ -1 + X + W.A + W.B + W.C + 
  f(phi, initial = -10, fixed = T) +
  f(alt.idx, Z, fixed = T, constr = T) +
  f(spatial.field, model = spde, group = spatial.field.group)


init = c(-0.008, -0.093)
model.spde <- inla(formula.spde,
                   data=inla.stack.data(stack, spde = spde),
                   family="poisson",
                   control.predictor=list(A = inla.stack.A(stack), compute=TRUE),
                   control.inla= list(int.strategy = "eb"),
                   control.mode=list(restart=T, theta=init))

round(model.spde$internal.summary.hyperpar$mode, 3)
```
\ \newline
We don't show the results about the parameters because they are pretty similar to the ones showed in the other sections. The only thing that we like to show is the prediction of the spatial random field, we compute the accuracy and reconstruct the continuous spatial random field. Figure 9 shows the comparison between the estimated spatial field and the true one.
\ \newline
```{r}
# estimates on the mesh locations
spde.mesh.est = model.spde$summary.random$spatial.field$`0.5quant`
# estimates on the data locations
spde.loc.est = matrix(drop(A%*%spde.mesh.est),
                 ncol=3, byrow=TRUE)[,1]
# Mean Square Error
mean((spde.loc.est - random.effect)^2)
```

```{r, fig.cap=c("Recostruction of the spatial random field")}
par(mfrow = c(1,2))

# plot the estimates
output_proj = inla.mesh.project(proj, field = spde.mesh.est)
image(proj$x, proj$y, output_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '', main = "Estimate")

# plot the true one
sample_proj = inla.mesh.project(proj, field = sample[,2])
image(proj$x, proj$y, sample_proj , xlim = c(0,1), ylim = c(0,1),
      xlab = '',ylab = '', main = "True")

```


### Yogurt Data

This is the last section of the tutorial; here we analyze the Yogurt Data studied in the literature on Multinomial models. The first to introduce it was Jain in 1994 (@jain), the same results are obtained also by @lee and @chen. 
In the first part of this section, we implement the model used in literature and in the second we propose some modification to the model.

#### Model

The data is composed of 2412 observations, each of them represents a choice situation in which a customer choose between four yogurt brands: yoplait, dannon, hiland, weight. The covariates are two alternative specific variables: price (continuous) and feat (Boolean). The latter indicates the presence of a feature advertisement on the newspapers. The data includes repeated choices for each individual. The data is included in the $\texttt{mlogit}$ package. The starting model is the following:
$$
V_{ij} = \alpha_j + \beta_1 P_{ij} + \beta_2 F_{ij} 
$$
Where $i$ stands for the choice situation and not the customer. $P_{ij}$ stands for the price of brand $j$ in choice situation $i$ and $F_{ij}$ indicates the presence of a feature advertisement. Translating it into Poisson's terms the model becomes:
$$
Y_{ij} \sim \text{Poisson}(\lambda_{ij})
$$
$$
\log(\lambda_{ij}) = \phi_i + \alpha_j + \beta_1 P_{ij} + \beta_2 F_{ij}
$$

Where $\alpha_j$ represents an intercept that depends on the yogurt brand. In this model, there are only alternative specific variables with generic coefficients. As you can see $\boldsymbol\beta = \{ \beta_1, \beta_2\}$ is the same for each choice situation and for each alternative. Hence, the data will be in *long* format. The original data is in *wide* format and looks like this
\ \newline
```{r, message=F}
data("Yogurt")
head(Yogurt)
```
\ \newline
Instead, the data that we are going to use is in this format:
\ \newline
```{r, echo = F}
n = nrow(Yogurt)
Data = matrix(NA, ncol = 5, nrow = n*4)
for(i in 1:n){
  choice = rep(0, 4)
  choice[Yogurt$choice[i]] = 1
  Data[((i-1)*4+1):(i*4), 1] = choice
  Data[((i-1)*4+1):(i*4), 2] = c(Yogurt$price.yoplait[i],
                                 Yogurt$price.dannon[i],
                                 Yogurt$price.hiland[i],
                                 Yogurt$price.weight[i])
  Data[((i-1)*4+1):(i*4), 3] = c(Yogurt$feat.yoplait[i],
                                 Yogurt$feat.dannon[i],
                                 Yogurt$feat.hiland[i],
                                 Yogurt$feat.weight[i])
  Data[((i-1)*4+1):(i*4), 4] = rep(Yogurt$id[i], 4)
  Data[((i-1)*4+1):(i*4), 5] = rep(i, 4)
}

Data = data.frame(Data)
names(Data) = c('Y', 'price', 'feat', 'cust.id', 'phi')
Data$price = Data$price/100 
Data$phi = as.factor(Data$phi)
Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)
```

```{r}
head(Data)
```
\ \newline
The formula in this situation is straightforward
\ \newline
```{r}
formula = Y ~ -1 + price + feat + alpha.idx +
  f(phi, initial = -10, fixed = T) 
  

model = inla(formula, data = Data, family = 'Poisson') 
```
\ \newline
These are the results and you can check the literature and see that they are almost the same.
\ \newline
```{r}
results = model$summary.fixed[,1:5]
rownames(results) = c("price", "feat", "dannon", "hiland", "weight", "yoplait")
round(results,3)
```
\ \newline
Now that we have verified the correctness of our approach retrieving the literature results, we can do something more interesting. 
Think about the price, in the previous model the coefficient is unique no matter what the price is. We can suppose that lower prices encourage the purchase and, instead, that high prices discourage it. To check this hypothesis we divide the price classes and then we look for a different coefficient for each of them. Just to make the story more interesting we also suppose that the relation between the outcome and the price is non-linear. We can express this behavior fitting a random walk of order 1 on the price. In this way the coefficient $\beta_1$ depends on the price level, it becomes $\beta_1(P_{ij})$. The model is the following:
\ \newline
$$
\log(\lambda_{ij}) = \phi_i + \alpha_j + P_{ij}\beta_1(P_{ij}) + \beta_2 F_{ij}
$$
\ \newline
```{r}
Data$price.class = inla.group(Data$price, n = 12, method = "cut")

formula.price = Y ~ -1 + feat + alpha.idx +
  f(price.class, model = "rw1") + 
  f(phi, initial = -10, fixed = T) 

model.price = inla(formula.price, data = Data, family = 'Poisson') 

res = model.price$summary.fixed[,1:5]
rownames(res) = c('feat', 'dannon','hiland','weight','yoplait')
round(model.price$summary.random$price.class[,1:5],3)
```
\ \newline
As we can see from the results, considering a unique coefficient for the price ignores part of the information that we have. From the above table, we can see that lower prices have a positive effect on the probability of buying an item. On the other hand, higher prices have a negative effect. The relation between the coefficient $\beta_1$ and the price level is almost linear. Studying this kind of effects may be an interesting tool in the design of brand market policies. 
\ \newline
```{r, fig.cap=c("Relation between the coefficient and the price level")}
plot(model.price$summary.random$price.class$ID,
     model.price$summary.random$price.class$mean, type = 'l', 
     ylab = expression(beta[1]),
     xlab = 'price')
```
\ \newline
\ \newline
Another modification to the standard model that we can consider is about the covariates representing the presence of a feature advertisement. Here, we can consider an alternative specific coefficient. This choice materializes the following intuition: different brands have different advertisers and their advertising campaigns may have a different impact on the market. To consider the $\texttt{feat}$ covariate as related to an alternative specific coefficient we have to change the format. The data should be like this
\ \newline
```{r, echo = F}
Data = matrix(NA, ncol = 8, nrow = n*4)
for(i in 1:n){
  choice = rep(0, 4)
  choice[Yogurt$choice[i]] = 1
  Data[((i-1)*4+1):(i*4), 1] = choice
  Data[((i-1)*4+1):(i*4), 2] = c(Yogurt$price.yoplait[i],
                                 Yogurt$price.dannon[i],
                                 Yogurt$price.hiland[i],
                                 Yogurt$price.weight[i])
  Data[((i-1)*4+1):(i*4), 3:6] = rbind(c(Yogurt$feat.yoplait[i], NA, NA, NA),
                                       c(NA, Yogurt$feat.dannon[i], NA, NA),
                                       c(NA, NA, Yogurt$feat.hiland[i], NA),
                                       c(NA, NA, NA, Yogurt$feat.weight[i]))
                                
  Data[((i-1)*4+1):(i*4), 7] = rep(Yogurt$id[i], 4)
  Data[((i-1)*4+1):(i*4), 8] = rep(i, 4)
}


Data = data.frame(Data)
names(Data) = c('Y', 'price', 'feat.yoplait','feat.dannon', 'feat.hiland', 'feat.weight', 'cust.id', 'phi')
Data$price = Data$price/100 #(Data$price - mean(Data$price))/sd(Data$price)
Data$phi = as.factor(Data$phi)
Data$alpha.idx = rep(c("yoplait","dannon", "hiland","weight"), n)
head(Data)
```
\ \newline
The corresponding model is:
\ \newline
$$
\log(\lambda_{ij}) = \phi_i + \alpha_j + \beta_1P_{ij} + \beta_{2j} F_{ij}
$$
\ \newline
```{r}
formula.feat = Y ~ -1 + price + feat.yoplait + feat.dannon + 
  feat.hiland + feat.weight + alpha.idx + 
  f(phi, initial = -10, fixed = T)  
  

model.feat = inla(formula.feat, data = Data, family = 'Poisson') 
```
\ \newline
The results about the price and $\alpha_j$'s are almost the same that we found in the standard model. Then we show just the results relative to the feature advertisement.
\ \newline
```{r}
model.feat$summary.fixed[2:5, 1:5]
```
\ \newline
From these results, it seems that the advertising campaign of hiland has a great impact on the choice compared with the other. On the other hand, maybe dannon has to review its advertising group. Just one problem, in our dataset hiland has been chosen only the $2\%$ of the times. This means that the above results could be due to over-fitting but going deeper is not in our purposes. 


### Conclusion

Just few words to conclude this tutorial. As you have seen the Multinomial-Poisson transformation combined with INLA is an amazing combination and it allows us to implement complex models in a reasonable computational time. We hope that the reader appreciates this tutorial and that it is the beginning of a lasting friendship with INLA. For any question feel free to send an e-mail to [someone] or to write a post on the https://groups.google.com/forum/#!forum/r-inla-discussion-group. 


## References






 

---
title: "Defining a latent model in R or C"
author: "Haavard Rue `(hrue@r-inla.org)`"
date: \today
output: rmarkdown::pdf_document
header-includes:
  - \usepackage{pdfpages}
vignette: >
  %\VignetteIndexEntry{Defining a latent model in R or C}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---
\includepdf[pages={-}]{rgeneric-core.pdf}

---
title: "Global oceanic barrier model on the sphere"
author: Alba Fuster-Alonso^[afuster@icm.csic.es, Institut de Ciències del Mar (ICM) - CSIC, Barcelona, Spain.] $\;$  Elias T Krainski^[King Abdullah University of Science and Technology (KAUST), Thuwal, Saudi Arabia.] $\;$  Haavard Rue^[King Abdullah University of Science and Technology (KAUST), Thuwal, Saudi Arabia.] $\;$ Finn Lindgren^[University of Edinburgh (UoE), Edinburgh, Scotland.]
format:
  html:
    page-layout: full
    fig-width: 11.5
    fig-height: 8.5
    embed-resources: true
    self-contained-math: true
    toc: true
    toc-location: left
    toc-title: Outline
    toc-expand: 1
    toc_float: 
      collapsed: true
    code-fold: true
    code-tools: true
    number-sections: true
editor_options: 
  chunk_output_type: console
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Global oceanic barrier model on the sphere}
  %\VignetteEncoding{UTF-8} 
  %\VignetteEngine{knitr::rmarkdown}
---

```{=html}
<style> body {text-align: justify} </style>
```
<!-- Justify text. -->

```{r setup, include=FALSE}
set.seed(123)
library(INLA)
##inla.setOption(num.threads="4:1")
##inla.setOption(smtp="taucs")
##if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/barrier-global/")
```

# Introduction

In this tutorial, we illustrate the implementation of the barrier
model proposed in Bakka et al. (2019) in the sphere. It considers the
problem of modeling over a spatial domain, taking into account
physical barriers and the spherical shape of the Earth. Since we are
working on the global oceans, continents can be considered as physical
barriers, and this aspect has to be taken into account in the
modeling. When using a barrier model, the range is not only determined
by the distance to points. If there is a barrier between two points,
the range should decrease quickly, and the correlation over the
barrier should approach zero. Additionally, the Earth's spherical
shape is a crucial factor in this particular case, as considering a
planar approximation can bias the results when working on a global
scale. Therefore, this tutorial is developed using a projection onto a
sphere. We are using the new implementation of the INLA package.
Please find the compiled vignette 
[here](https://eliaskrainski.github.io/INLAspacetime/articles/web/barrier_global.html).

**Since the compiled vignette is large is size, we do not include it the
R-INLA package, but you can download the vignette source and
downloaded data, at
``[https://github.com/hrue/r-inla/tree/devel/rinla/vignettes/barrier-global](https://github.com/hrue/r-inla/tree/devel/rinla/vignettes/barrier-global)''
and compile it yourself.**

\documentclass[authoryear]{article}
%%\usepackage{graphics}
\usepackage{pgf}
\usepackage{epsfig}
\usepackage{amssymb}
\usepackage{amsmath,latexsym}
\usepackage{array}
\usepackage{hyperref}
\usepackage[round]{natbib}
   
\usepackage{setspace} 
\setlength{\parindent}{0cm} 

\headsep=50pt
%\setlength{\parskip}{1.2ex}
\setlength{\parindent}{0.0mm}
\setlength{\textwidth}{15.4cm}
\setlength{\textheight}{21.2cm}
\setlength{\hoffset}{-0.5in}
\setlength{\voffset}{-1.2in}
\linespread{1.05}

\parindent=1.1pc
%\parskip=0pt
\setcounter{section}{0}
%\setcounter{tocdepth}{0}
% \setcounter{secnumdepth}{3}
%\textwidth=33pc
%\textheight=50pc
%\renewcommand{\figurename}{Fig.}

\def\Prob{\text{Prob}}
\def\E{\text{E}}
\def\Cov{\text{Cov}}
\def\Corr{\text{CoRrr}}
\def\Var{\text{Var}}
\def\Prec{\text{Prec}}
\def\mm#1{\ensuremath{\boldsymbol{#1}}} % version: amsmath
\def\MyFigureWidth{0.4\textwidth}
\def\MyFigureWidthTwo{9.5cm} %% 2*width + spacing
\def\MyFigureSpacing{6mm}

%%  To change the page layout from the default
%\addtolength{\topmargin}{-2cm}
%\addtolength{\textheight}{3cm}
%\addtolength{\oddsidemargin}{-0.5cm}
%\addtolength{\textwidth}{1cm}

% \SweaveOpts{width = 40}

\setkeys{Gin}{width=0.45\textwidth}

%\definecolor{links}{HTML}{2A1B81}
%\hypersetup{colorlinks,linkcolor=,urlcolor=links}

%\VignetteEngine{knitr::knitr} 
%\VignetteIndexEntry{ScaleModel}

\begin{document}
\begin{titlepage}
    \title{Tutorial: Scaling IGMRF-models in \texttt{R-INLA}}
    \vspace{0.5cm} \author{Sigrunn Holbek S\o rbye \\ Department of
        Mathematics and Statistics, University of Troms{\o}}
\end{titlepage}
\maketitle


<<echo=FALSE,eval=TRUE,message=FALSE>>= %chunk 1
library(spatstat)
library(mvtnorm)
library(lattice)
library(mgcv)
library(pixmap)
library(numDeriv)
library(fields)
library(INLA)
set.seed(123)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
               fig.path='figures/scale-model/')
@

\section{Introduction}

Intrinsic Gaussian Markov random fields (IGMRFs) are widely used as
priors to model latent dependency structures in \texttt{R-INLA}.
Examples of this type of models include the \texttt{rw1},
\texttt{rw2}, \texttt{besag}, \texttt{besag2}, \texttt{bym} and the
\texttt{rw2d} models, see \texttt{www.r-inla.org} for further
descriptions. All of these models can be described in terms of having
an improper Gaussian density with a scaled precision matrix that
reflects the neighbourhood structure of the model. The scaling is
represented as a random precision parameter, and applying these models
the user has to define a hyperprior for this parameter. This is a
rather delicate issue and such hyperpriors are often selected in a
rather ad-hoc and subjective manner. One problem is that a specific
fixed hyperprior for the precision parameter does not have the same
interpretation for different IGMRFs.

The aim of this tutorial is to introduce and demonstrate a recently
implemented option in \texttt{R-INLA} named \texttt{scale.model},
which can be added when specifying the inla formula-call for the
different IGMRFs. For example, we might specify the formula as
<<echo=TRUE,eval=FALSE>>= %chunk 2
formula = y ~  f(idx, model = "rw2", scale.model = TRUE, hyper=..)
@
By defining \texttt{scale.model = TRUE}, the \texttt{rw2}-model is
scaled to have a generalized variance equal to 1, see
Section~\ref{sec:igmrf} for further explanations. The new option is
also available using \texttt{control.group},
<<echo=TRUE,eval=FALSE>>= %chunk 3
f(idx, model="..", group=g, control.group = list(model="rw2", scale.model=TRUE))
@ 
which is relevant for the \texttt{rw1}, \texttt{rw2} and the
\texttt{besag}-models. Note that \texttt{scale.model = FALSE} by
default for all models to make the option backwards-compatible.

The purpose of scaling the models is to ensure that a fixed hyperprior
for the precision parameter has a similar interpretation for different
types of IGMRFs. In general, IGMRFs can be seen as models that reflect
local deviation from an unspecified overall level. More specifically,
the models considered here penalize local deviation from a constant
level (\texttt{rw1}, \texttt{besag}, \texttt{bym}), a line
(\texttt{rw2}) or a plane (\texttt{rw2d}). A priori, the hyperprior
chosen for the precision parameter will influence how large we allow
this local deviation to be. It seems reasonable to select hyperpriors
such that different random effects in a regression model are assumed
to have a similar range for this deviation. By scaling the models,
this is achieved by assigning the same fixed hyperprior to the
precisions of all IGMRFs in the regression model. If we do this using
unscaled IGMRFs (as often done in the literature), the influence using
different hyperprior choices is less controllable as the variances of
the IGMRFs differ.

The hyperpriors for scaled models will also be invariant to the shape
and size of the graph for a specific IGMRF and to different scalings
of covariates. By mapping the random precision to the marginal
variance of an IGMRF, we also get an alternative interpretation of
different parameter choices for a hyperprior, as these can be viewed
in terms of the local deviation, see Section~\ref{sec:igmrf} for
further details.

The new option is based on ideas introduced in \cite{sorbye:13} and
these ideas have also been applied in \cite{papoila:13}. Note that the
implementation in \cite{sorbye:13} is slightly different as there the
hyperprior is assigned to scaled precision parameters, while the
current option scales the IGMRFs directly. The results are the same,
but the new option makes the ideas in \cite{sorbye:13} very easy to
apply. We demonstrate this in Section~\ref{sec:examples}, analysing
two semiparametric regression models. Concluding remarks are given in
Section~\ref{sec:conclusion}.

\section{Scaling the IGMRFs}\label{sec:igmrf}

An IGMRF can be defined as a vector $\mm{x}=(x_1,\ldots , x_n)$,
having an improper Gaussian density with a sparse precision matrix
which is not of full rank. In \cite{rueheld:05}, the order of an IGMRF
is defined as the rank deficiency of its precision matrix. This
implies that a zero-mean IGMRF of $k$th order is specified by
\begin{equation*}
    \pi(\mm{x}) = (2\pi)^{-(n-k)/2}(|\mm{Q}|^*)^{1/2} 
    \exp\{-\frac{1}{2}\mm{x}^T \mm{Q} \mm{x}\}, %\label{eq:igmrf}
\end{equation*}
where the precision matrix $\mm{Q}$ is assumed to be semi-positive
definite. $|\mm{Q}|^*$ denotes the generalized determinant equal to
the product of the $n-k$ non-zero eigenvalues of $\mm{Q}$. The
marginal variances of $\mm{x}$ are given by the diagonal elements of
the generalized inverse matrix $\mm{\Sigma^*} = \mm{Q}^{-1}$, that
is $$\sigma^2(x_i) = \Sigma^*_{ii}, \quad i=1,\ldots , n.$$ Different
IGMRFs can be described by expressing the precision matrix as
$\mm{Q}=\tau \mm{R}$. Here, $\tau$ denotes the random precision
parameter while the matrix $\mm{R}$ reflects the specific
neighbourhood structure of the model which can be represented as a
labelled graph with nodes and edges. This implies that the marginal
variances/standard deviations of the IGMRF will depend on both the
structure and size of $\mm{R}$. For example, we can consider the
marginal standard deviations of the \texttt{rw1} and
\texttt{rw2}-models when the precision is fixed as $\tau= 1$. Due to
the different structure matrices of these two models,
% , see \cite[p. 95 and p. 110]{rueheld:05},
the levels for the marginal standard deviations are quite different,
see Figure~\ref{fig:marg.std}. This illustrates that it seems 
unreasonable to assign the exact same hyperprior for the precision
parameters of these two models when these parameters are random.

<<echo=F,eval=TRUE>>= %chunk 4
ss.rw1.sdref = function(values,tau) { 
    n = length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3) 
    y = NA
    
    formula = y ~ -1 + f(values, 
            model="rw1", 
            constr =TRUE, 
            values = values, 
            diagonal = 1e-10, 
            hyper = list(prec = list(
                                 initial = log(tau), 
                                 fixed=TRUE))) 
    r =inla(formula,data=data.frame(y,values),family = "gaussian",
            control.family = list(fixed =
                    TRUE), 
            control.compute=list(return.marginals=F),verbose=F)
    sd=r$summary.random$values$sd 
    return (sd) 
    }

ss.rw2.sdref= function(values,tau) { 
    n=length(values)
    stopifnot(!missing(n)) 
    stopifnot(n > 3)
    y = NA #idx =1:n
    
    A1 = matrix(1, n, 1) 
    A2 = matrix(0, n, 1) 
    for(i in 1:n) { 
        A2[i, ]= i 
    }
    
    A = rbind(c(A1), c(A2)) 
    e = rep(0, 2) 
    extraconstr = list(A = A, e= e)
    
    formula = y ~ -1 + f( values, model="rw2", constr = FALSE,
            values=values, diagonal = 1e-10, extraconstr = extraconstr, hyper
            = list(prec = list(initial = log(tau), fixed=TRUE)))
    
    r = inla(formula, data = data.frame(y, values), family =
            "gaussian", control.family = list(fixed = TRUE),
            control.compute=list(return.marginals=F),verbose=F)
    
    sd=r$summary.random$values$sd 
    return (sd) 
}

n=100 
sd.rw1=ss.rw1.sdref(1:n,1) 
sd.rw2=ss.rw2.sdref(1:n,1)
##sd.rw1.scaled=ss.rw1.sdref(1:n,1) 
##sd.rw2.scaled=ss.rw2.sdref(1:n,1)
@

\begin{figure}[ht]
    \centering 
    \mbox{%%
        \makebox[0.45\textwidth][c]{ 
<<fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'>>= %chunk 5
plot(sd.rw1,xlab="Node i", ylab="Marginal standard deviation") 
@ 
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{ 
<<fig.keep='high',echo=FALSE,out.width='0.45\\linewidth'>>= %chunk 6
plot(sd.rw2,xlab="Node i",ylab="Marginal standard deviation") 
@ 
}}
\caption{The marginal standard deviations $\sigma(x_i)$ for the
    \texttt{rw1}-model (left) and the \texttt{rw2}-model (right),
    calculated using a fixed precision $\tau=1$ and $n=100$ nodes.}
    \label{fig:marg.std}
\end{figure}

Notice that for a fixed precision $\tau$, the marginal variances of
the components of $\mm{x}$ can be expressed, as a function of $\tau$,
by
\begin{equation}
    \sigma^2_{\tau}(x_i)=\frac{\sigma^2_{\{\tau=1\}}(x_i)}{\tau},\quad i=1,\ldots , n.\label{eq:sigma.xi}
\end{equation}
To characterize the level of the marginal variances when $\tau=1$, we
define the generalized variance of $\mm{x}$ as the geometric mean
\begin{equation*}
    \sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x}) = \exp\left(\frac{1}{n}\sum_{i=1}^n \log(\sigma^2_{\{\tau=1\}}(x_i))\right). \label{eq:sigma.gv}
\end{equation*} 
When \texttt{scale.model = TRUE}, IGMRFs are scaled such that
$\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x}) =1$. This implies that the
precision parameters of different models have similar interpretation
which makes it reasonable to assign the same fixed hyperprior to
precision parameters of different IGMRFs in a regression model. Note
that in \cite{sorbye:13}, the generalized variance was referred to as
the reference variance $\sigma^2_{\mbox{\scriptsize{ref}}}(\mm x)$.
The hyperprior was then assigned to the scaled precision
$\tau/\sigma^2_{\mbox{\scriptsize{ref}}}(\mm x)$ to achieve the same
interpretation for different models.

As already mentioned, IGMRFs penalize local deviation from an
unspecified overall level. The marginal standard deviation of the
models, integrating out the random precision, give information on how
large we allow this local deviation to be. Following \cite{sorbye:13},
we apply the identity in (\ref{eq:sigma.xi}) also when $\tau$ is
random and define an upper limit $U$ for the marginal standard
deviation of a model by
\begin{equation}
    P(\sigma(x_i)>U)\approx P\left(\tau<\frac{\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})}{U^2}\right)=\alpha, \label{eq:tau}
\end{equation}
where $\alpha$ is a fixed small probability. For scaled models, this
implies that the parameter choices for the hyperprior of $\tau$ has
the same interpretation in terms of $U$, for all models $\mm{x}$.

In \texttt{R-INLA}, precision parameters are assigned Gamma
distributions by default (but any other distribution can be specified
by the user). Assume that $\tau\sim \mbox{Gamma}(a,b)$, where $a$
denotes the shape parameter and $b$ the inverse-scale parameter. Then
$b\tau\sim\mbox{Gamma}(a,1)$ and (\ref{eq:tau}) implies that
\begin{equation}
    \frac{b\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})}{U^2} = F^{-1}(\alpha,a,1), \label{eq:upper.limit}
\end{equation}
where $F^{-1}(\cdot)$ denotes the inverse cumulative distribution
function of the Gamma distribution. For scaled
models, we again notice that the
interpretation of $a$ and $b$ stays the same for all $\mm{x}$ and the
upper limit $U$ is easily calculated as 
%%
<<echo=TRUE,eval=F>>= %chunk 7
U = sqrt(b/qgamma(alpha,a,1)) 
@ 
<<echo=FALSE,eval=TRUE>>= %chunk 8
func.u = function(a,b,alpha,sigma.ref) { 
    upper.limit = sqrt(b)*sigma.ref/sqrt(qgamma(alpha,a,1)) 
    return(upper.limit) 
}
a=1 
b=5*10^{-5} 
alpha=0.001 
upper.limit=func.u(a,b,alpha,1) 
@ 
%%
For example, if we choose the default parameter values in
\texttt{R-INLA}, $a=\Sexpr{a}$ and $b=5\cdot 10^{-5}$, the upper limit
equals $U\approx \Sexpr{round(upper.limit,2)}$ when
$\alpha =\Sexpr{alpha}$. Notice that the explicit value of $U$ has to
be interpreted with care as this depends on the selected value of
$\alpha$. Also, $U$ is not uniquely defined and different combinations
of $a$ and $b$ giving the same values of $U$, do not in general give
the same estimated results.

\section{Examples}\label{sec:examples}

The new option makes it very easy to compare results using unscaled
and scaled models. Here we consider two examples, analysing
semiparametric regression models. Other examples are given in
\cite{sorbye:13} and \cite{papoila:13}.

<<echo=F,eval=TRUE>>= %chunk 9
func<-function(x,t)
{
    f.x=x/t+sin(2*pi* x/t)-0.5
    return(f.x)
}

f.est <- function(y,x,a,b,option)
{
    formula = y~1+f(x,model="rw2",hyper=list(prec=list(prior="loggamma",param=c(a,b))),scale.model=option)
    result = inla(formula,family="gaussian",data=data.frame(y,x))
    f.est=result$summary.random$x$mean
    return(f.est)
}
@

<<echo=F,eval=TRUE>>= %chunk 10
### Same values of the function, same observations, but  defined on different intervals ###
set.seed(89236)
x2=0:n
x1=x2/100
x3=x2*10

sigma=0.5
f.x=func(x2,length(x2))
y=f.x+rnorm(length(f.x),0,sigma)

a=1
b=0.00005

mat1=cbind(f.x,f.est(y,x1,a,b,T),f.est(y,x1,a,b,F))
mat2=cbind(f.x,f.est(y,x2,a,b,T),f.est(y,x2,a,b,F))
mat3=cbind(f.x,f.est(y,x3,a,b,T),f.est(y,x3,a,b,F))

# Generalized marginal variances  
v1=exp(mean(log(ss.rw2.sdref(x1,1)^2)))
v2=exp(mean(log(ss.rw2.sdref(x2,1)^2)))
v3=exp(mean(log(ss.rw2.sdref(x3,1)^2)))

u1=func.u(a,b,alpha,sqrt(v1))
u2=func.u(a,b,alpha,sqrt(v2))
u3=func.u(a,b,alpha,sqrt(v3))
@

\subsection{Semiparametric regression with different scalings of the covariate}

We first consider a simple semiparametric regression model in which
$n$ observations are generated by
\begin{equation}
    y_i = f(x_i) + \epsilon_i,\quad i=1,\ldots, n. \label{eq:regr}
\end{equation}
The noise terms, $\epsilon_i$, are assumed independent and normally
distributed with mean 0 and a fixed standard deviation $\sigma$. The
aim is to estimate the underlying smooth function $f(.)$. Applying the
\texttt{rw2}-model as a prior, the formula-call using the scaled model
is defined as
<<echo=TRUE,eval=F>>= %chunk 11
formula = y ~  f(x, model = "rw2", scale.model = TRUE, hyper=...)
@ 
where the hyperprior assigned to the precision parameter is
specified in \texttt{hyper}. Here, we just select the default
hyperprior, that is $\tau \sim \mbox{Gamma}(1,5\cdot
10^{-5})$. 

Let the unknown true function $f(.)$ be defined as
 $$f(x)=\frac{x}{t}+\sin(\frac{2\pi x}{t})-0.5,\quad x\in(0,t).$$
 We generate $n=101$ observations based on (\ref{eq:regr}) in which
 $\sigma=\Sexpr{sigma}$ and call \texttt{inla}:
<<echo=TRUE,eval=F>>= %chunk 12
result = inla(formula, family = "gaussian", data = data.frame(y,x))
@ 
We then plot the estimated function 
<<echo=TRUE,eval=F>>= %chunk 13
result$summary.random$x$mean
@ 
both using the unscaled and scaled models for three different
intervals of $x$, see Figure \ref{fig:fx2}.

\setkeys{Gin}{width=0.33\textwidth}
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 14
                                        # Estimate of f using unscaled and scaled model
r=range(c(mat1,mat2,mat3))
matplot(x1,mat1,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3) 
                                        #legend("topright",c("true f","scale.model=F","scale.model=T"),lty=c(1,2,4),col=c(1,2,4))
                                        #points(x,y)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 15
matplot(x2,mat2,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.28\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.28\\linewidth'>>= %chunk 16
matplot(x3,mat3,type="lll",lty=c(1,2,4),col=c(1,4,2),xlab="x",ylab="f(x)",ylim=r,lwd=3)
@
}}
\caption{The true function (black solid line) and the estimated
    functions using the scaled (blue dashed line) and unscaled (red
    dashed/dotted line) \texttt{rw2}-models for different scalings of
    the covariate.}
\label{fig:fx2}
\end{figure}


Using the scaled model, the results using a fixed hyperprior is
invariant to different scalings of the covariate and we get exactly
the same estimated curve in all three cases. For the unscaled model,
the marginal variance will decrease/increase as the distance between
nodes decreases/increases. By using the same fixed hyperprior for the
three cases, we implicitly assume very different values for the upper
limit $U$ in (\ref{eq:upper.limit}), see Table~\ref{tab:upper.limit}.
In the extreme case of assuming a very low value of $U$, the resulting
estimate is just a straight line.

\begin{table}[ht]
\begin{center}
\begin{tabular}{l|ccc}%\hline
  Interval length&  $t=1$& $t=100$ & $t=1000$ \\ \hline%\hline
  % $\sigma^2_{\mbox{\scriptsize{GV}}}(\mm{x})$ & $\Sexpr{round(v1,4)}$&$\Sexpr{round(v2,1)}$ & $\Sexpr{round(v3,1)}$\\
  $U$ (unscaled model) &  $\Sexpr{round(u1,3)}$ & $\Sexpr{round(u2,1)}$ & $\Sexpr{round(u3,1)}$%\hline
\end{tabular}
\end{center}
\caption{The upper limit $U$  as defined in (\ref{eq:upper.limit}) 
    for the unscaled \texttt{rw2}-model, 
    calculated when $a=1$, $b=5\cdot 10^{-5}$ and $\alpha=0.001$.}
\label{tab:upper.limit}
\end{table}

As noted in \cite{akerkar:10} and \cite{sorbye:13}, a given hyperprior
for the precision of unscaled models can be adjusted to account for
different scalings of the covariate, adjusting the inverse-scale
parameter of the Gamma distribution. However, by scaling the models,
we don't have to think about this at all as a given fixed prior will
give the same estimated curve.


\subsection{Semiparametric regression with several random effects}

The linear predictor of a latent Gaussian model often consists of a
combination of random effects modelled by different IGMRFs. As an
example we will take a look at the Munich rental data analysed in
Section 4.2.2 in \cite{rueheld:05}, also given as an example in Volume
I at \texttt{www.r-inla.org}. The response variable in this dataset is
rent per square meter (in Euros) for a flat and the covariates include
spatial location, floor space, year of construction and various
indicator variables. Smooth effects of the covariates are modelled
using the \texttt{rw2}-model, while the spatially structured effect
for location is modelled using the \texttt{besag}-model. Previously,
this data has been analysed using a fixed Gamma$(1,0.01)$ for all the
precision parameters in the model. The inverse-scale parameter of the
Gamma distribution can be adjusted to account for different types of
IGMRFs by scaling this parameter with the generalized variance, see
\cite{sorbye:13}, but it is easier to just use the new option.

To simplify notation let
<<echo=TRUE,eval=F>>= %chunk 17
hyper.prec = list(prec = list(prior = "pc.prec", param = c(0.5, 0.01)))
@
 The formula-call at  \texttt{www.r-inla.org} is then specified by 
<<echo=TRUE,eval=F>>= %chunk 18
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

## Note that here we what to have an estimator of the effect of year
## also the for years where we have no observation, therefore we give a
## vector with all possible values assumed by the covariate year, that
## is seq(1918,2001)

formula = rent ~ 
   f(location, model = "besag", graph = g, initial = 1, hyper = hyper.prec) +
   f(year, model = "rw2", values = seq(1918,2001), hyper = hyper.prec) +
   f(floor.size, model = "rw2", hyper = hyper.prec) +
   Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
   Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
   zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
@  
Further, the call to inla is given as 
<<echo=TRUE,eval=F>>= %chunk 19
mod  =  inla(formula, data = Munich, control.fixed=list(prec=1))
@                   

<<echo=FALSE,eval=TRUE>>= %chunk 20
data(Munich)
g = system.file("demodata/munich.graph", package="INLA")

func.munich <- function(a,b,option)
{
    hyper.prec=list(prec=list(param=c(a,b)))
    formula= rent ~ f(location,model="besag",graph=g,initial=1, hyper=hyper.prec, scale.model=option) +
        f(year,model="rw2",values=seq(1918,2001), hyper=hyper.prec, scale.model=option) +
            f(floor.size,model="rw2", hyper=hyper.prec, scale.model=option) +
                Gute.Wohnlage + Beste.Wohnlage + Keine.Wwv + Keine.Zh +
                    Kein.Badkach  + Besond.Bad + Gehobene.Kueche +
                        zim1 + zim2 + zim3 + zim4 + zim5 + zim6 -1
    
    result = inla(formula, data=Munich, control.fixed=list(prec=1))
}
@

<<echo=FALSE,eval=TRUE>>=%chunk 21
a=1
b=0.01
u.munich=func.u(a,b,alpha,1)
mod=func.munich(a,b,FALSE)
mod.scaled=func.munich(a,b,TRUE)
@
Figure~\ref{fig:munich} illustrates the resulting estimated effects
of the covariates 'Year of construction' and 'Floor size' using the
scaled and unscaled models. Applying the given hyperprior for the
scaled models, we have assumed that the upper limit in
(\ref{eq:upper.limit}) is $U\approx\Sexpr{round(u.munich,1)}$. This
seems to give quite reasonable results, while the estimated curves are
more wiggly using the unscaled models. The results for the spatially
structured effects are very similar using the scaled and unscaled
models (results not shown).

\setkeys{Gin}{width=0.45\textwidth}
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 22
mat=cbind(mod$summary.random$year$mean,mod.scaled$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Year of construction",ylab="",lwd=2.5)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 23
mat=cbind(mod$summary.random$floor.size$mean,mod.scaled$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="ll",lty=c(2,1),col=c(2,1),xlab="Floor size",ylab="",lwd=2.5)
@
}}
\caption{The estimated effects of the covariates 'Year of
    construction' and 'Floor size' on the rental prizes in Munich,
    using the scaled (black solid line) and the unscaled (red dashed
    line) \texttt{rw2}-models.}
\label{fig:munich}
\end{figure}

<<echo=FALSE,eval=TRUE>>= %chunk 24
alpha=0.001
#u.vec=c(0.01,1,10,100)
u.vec=c(0.001,0.1,10,30)
a.vec=c(1,1,1,1)
b.vec=u.vec^2*qgamma(alpha,a.vec,1)

option=TRUE
c1=func.munich(a.vec[1],b.vec[1],option)
c2=func.munich(a.vec[2],b.vec[2],option)
c3=func.munich(a.vec[3],b.vec[3],option)
c4=func.munich(a.vec[4],b.vec[4],option)
@


Using the scaled models, we can now study sensitivity to the
parameters of the Gamma-prior simultaneously for the different random
effects modelled by IGMRFs, instead of doing this separately for each
of the effects. For example, we can choose to keep the shape parameter
fixed at $a=1$, while the inverse-scale parameter $b$ is calculated to
give specific values of $U$. The estimated functions for the effects
of the two covariates 'Year of construction' and 'Floor size' are
given in Figure~\ref{fig:sens.u}. Here, the value of $U$ ranges from
$\Sexpr{u.vec[1]}$ to $\Sexpr{u.vec[4]}$ which corresponds to values
of $b$ in the range $(10^{-9},0.9)$. We conclude that the results are
not very sensitive to changes in the inverse-scale parameter when
$a=1$. This also applies to the spatially structured effect which does
not change significantly for these different parameter choices
(results not shown). Naturally, we could also consider other values of $a$. 
 
\begin{figure}[ht]
    \centering
    \mbox{
        \makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 25
mat=cbind(c1$summary.random$year$mean,c2$summary.random$year$mean,c3$summary.random$year$mean,c4$summary.random$year$mean)
matplot(mod$summary.random$year$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Year of construction",ylab="",lwd=2.5)
legend("topleft",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)
@
}
\makebox[\MyFigureSpacing][c]{}
\makebox[0.45\textwidth][c]{
<<fig.keep='high',echo=FALSE,eval=TRUE,out.width='0.45\\linewidth'>>= %chunk 26
mat=cbind(c1$summary.random$floor.size$mean,c2$summary.random$floor.size$mean,c3$summary.random$floor.size$mean,c4$summary.random$floor.size$mean)
matplot(mod$summary.random$floor.size$ID,mat,type="llll",lty=c(1,2,3,4),col=c(1,2,3,4),xlab="Floor size",ylab="",lwd=2.5)
legend("topright",paste("U= ",u.vec),lty=1:4,col=1:4,lwd=2.5)
@
}}
\caption{The estimated effects of the covariates 'Year of
    construction' and 'Floor size' on the rental prizes in Munich,
    varying the upper limit for the scaled models.}
\label{fig:sens.u}
\end{figure}

  
\section{Concluding remarks}\label{sec:conclusion}

By scaling all IGMRFs to a unit generalized variance, we avoid that
the precision parameters of these models have different
interpretation. This makes it reasonable to assign the same hyperprior
to precision parameters of different IGMRF-models, which greatly
simplifies the issue of selecting hyperpriors and performing
sensitivity analysis. These ideas were introduced in \cite{sorbye:13},
in which the precision parameter was scaled by the generalized
variance. The new option in \texttt{R-INLA} makes these ideas easy to
apply as we don't have to calculate the generalized variance for each
model. We simply set the \texttt{scale.model}-option equal to
\texttt{TRUE} in the inla formula-call for the relevant terms.

If you are now convinced that all IGMRF-models should in fact be
scaled, you can do this in an even simpler way than already described.
By specifying the following global option
<<echo=TRUE,eval=FALSE>>= %chunk 27
inla.setOption(scale.model.default = TRUE)
@ 
all the IGMRF-models are scaled automatically. This also includes
the situation using the \texttt{control.group} argument mentioned in
the introduction. Hyperpriors still have to be selected with care
using scaled models, but the issue of selecting hyperpriors is
simplified and we believe that hyperpriors can be chosen in a more
meaningful and controlled way.

%% Currently, the option of scaling models only applies to IGMRFs but we
%% believe that similar ideas can be useful also for other models, for
%% example auto-regressive processes. Details on this will follow later.

\bibliographystyle{plainnat}
%\bibliographystyle{chicago} 
\bibliography{shs}    

\end{document}

---
title: "Conditional Logistic Regression Model"
author: "Stefanie Muff (stefanie.muff@uzh.ch), Johannes Signer, John Fieberg"
date: "July 16th 2018"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Conditional Logistic Regression Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
set.seed(123)
library(INLA)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
library(knitr)
library(rmarkdown)
knitr::opts_chunk$set(echo=TRUE, cache=FALSE, message=FALSE,warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/conditional-logit/")
```

## Parametrization

Some binomial sampling schemes in Biostatistics or Biology may result
in what is called *matched case-control* data, which require a
*conditional logistic regression* model. For the $j^\text{th}$
observed binary response $y_{nj}$ in stratum $n$, the model is given
as
$$\mathsf{Prob}(y_{nj}=1 \,|\, \eta_{n\cdot}) = p_{nj} =
\frac{\exp(\eta_{nj})}{\sum_i \exp( \eta_{ni})} \ ,
\quad  y_{nj} \sim \mathsf{Bern}(p_{nj}) \ ,$$
with linear predictor $\eta_{nj}$ and success probability $p_{nj}$.
The sum in the denominator is over all observations in the respective
stratum. This model is a special case of a multinomial model, and as
such it can be fitted by using a likelihood-equivalent reformulation
as a Poisson model
$$\mathsf{E}(y_{nj}) = \mu_{nj} =
\exp(\alpha_{n} + \eta_{nj}) \ ,
\quad \quad  y_{nj} \sim \mathsf{Po}(\mu_{nj}) \ ,$$
with stratum-specific intercepts $\alpha_{n}$. If the number of strata
is large, the explicit estimation of these intercepts can be
circumvented by $\alpha_{n} \sim \mathsf{N}(0,\tau_\alpha)$ and fixing
the precision $\tau_\alpha$ at a very small value, e.g. $10^{-6}$ or
$10^{-12}$, which corresponds to a large variance. This mimicks a
uniform distribution and ensures that the $\alpha_{n}$ can be
estimated freely instead of being shrunken towards 0.

## Hyperparameters
None.

## Specification

* family `=Poisson`
* To fix the variance at a large value the
  stratum-specific intercept $\alpha_{n}$ use 
    + `model="iid"`
    + `hyper=list(theta = list(initial=log(1e-6),fixed=T))`


## Example

The following example stems from a habitat selection study of 6 radio
collared fishers (*Pekania pennanti*) (LaPoint et al. 2013), and was
adapted from Signer et al. (2018). Outcomes with $y=1$ represent
locations that were visited by fishers, and $y=0$ represents nearby
locations that were not visited. Each visited location was matched to
2 nearby available locations, and together these 3 observations form a
stratum (indicated by `stratum`). By design, only exactly one location
can be visited in each stratum, thus these data need to be analyzed by
a conditional logistic regression model. Covariates include sex
(`sex`), land use (`landuse`, categorical covariate) and distance to
the center of the habitat (`dist_center`), with individual-dependent
random slopes for `dist_cent`. The 6 individuals are represented using
`id` and `id1`. Shown is a reduced dataset with only 100 steps per
individual and a sampling ratio of 1:2.


```{r formula, include=TRUE,eval=F}
fisher.dat <- readRDS(system.file("demodata/data_fisher2.rds", package
= "INLA"))
fisher.dat$id1 <- fisher.dat$id
fisher.dat$dist_cent <- scale(fisher.dat$dist_cent)

formula.inla <- y ~ sex + landuse + dist_cent + 
   f(stratum,model="iid",hyper=list(theta = list(initial=log(1e-6),fixed=T))) +
   f(id1,dist_cent, model="iid")
```
 
```{r inla_call, include=TRUE,eval=F}
r.inla <- inla(formula.inla, family ="Poisson", data=fisher.dat)
```


## References

Muff, S., Signer, J. and Fieberg, J. (preprint) *Accounting for
individual-specific variation in habitat selection studies: Efficient
estimation using integrated nested Laplace approximations*

Signer, J., Fieberg, J. and Avgar, T. In press. *Animal Movement
Tools* `(amt)`*: R-Package for Managing Tracking Data and Conducting
Habitat Selection Analyses*. Ecology and Evolution.

LaPoint, S., Gallery, P., Wikelski, M. and Kays, R. (2013) *Animal
behavior, cost-based corridor models, and real corridors*. Landscape
Ecology, 28, 1615–1630.

---
title: "SPDE one dimensional example"
author: "Elias T. Krainski"
date: 'created: April 2, 2016, last update `r Sys.Date()`'
output: pdf_document
vignette: |
  %\\VignetteIndexEntry{SPDE1d}   %\\VignetteEngine{knitr::knitr}   \usepackage[utf8]{inputenc}
---

# A **SPDE** model for one dimensional data 

## Introduction 

In this example we show row to analyse a time series 
of daily temperature using a one dimension SPDE model.
More details about it are on the paber at 
<https://www.jstatsoft.org/article/view/v063i19>

```{r settings, include=FALSE}
library(knitr);  library(INLA);  library(fields) 
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
fig.path='figures/SPDE1d')
knit_hooks$set(small.mar = function(before, options, envir) {
    if (before) par(mar = c(0.1, 0.1, .1, .1))  # smaller margin on top and right
})
knit_hooks$set(mar3311 = function(before, options, envir) {
    if (before) par(mar = c(3, 3, 1, 1), mgp=c(2, 0.7, 0))  # smaller margin on top and right
})
```

## The data 

We consider the daily weather data available at 
<http://www.yr.no/>. 
We have the following set the URL for the daily data for 
Trondheim considerint in the last 13 months 
```{r url}
u0 <- paste0('http://www.yr.no/place/Norway/S%C3%B8r-Tr%C3%B8ndelag/',
             'Trondheim/Trondheim/detailed_statistics.html')
### browseURL(u0) ### to visit the web page 
```

One can read and extract the desired data table 
(the second one at the URL) using the **XML** package with
```{r xml, eval=FALSE}
require(XML)
d <- readHTMLTable(u0)[[2]]
```
However, it still need some pre-processing.

Without the **XML** package one can use 
```{r read}
d0 <- readLines(u0) ### read it as text
i <- grep('<tr>', d0) ### index for each table line
i <- i[i>grep('<tbody>', d0)[2]] ### select those for the second table
```

The desired data we would like to analyse is the 
minimum and maximum temperature. 
Commands to extract and pre-process these data
```{r data}
dates <- as.Date(d0[i+1], format='      <th>%b %d, %Y</th>')
tmed <- as.numeric(gsub('<td>', '', gsub('°</td>', '', d0[i+4])))
(n <- length(dates)) ### it is daily over last 13 months
```

Visualize it
```{r visualize, fig.width=10, fig.height=3}
pd <- pretty(c(dates, max(dates+30)), n=13)
par(mfrow=c(1,1), mar=c(3,3,0.5,2), mgp=c(2,.7,0), las=2, xaxs='i')
plot(dates, tmed, type='l', lwd=2,
     axes=FALSE, xlab='day', ylab='Temperature')
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))
box()
axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, TRUE))
```

## Model fitting

*  __Mesh__:
in 1d it is a matter of chosing a set of knots, 
the order of the basis functions and the boundary. 
Choosing first order basis function and Neumann boundary.

```{r mesh}
coo <- as.numeric(dates-min(dates)) ## have numeric temporal coordinates
mesh <- inla.mesh.1d(loc=seq(min(coo), max(coo), by=7), ## knots (7 days)
                 boundary='neumann', degree=2)  ### boundary and basis function degree
```

* __Define the__ _n x m_ __projector matrix__ to project 
 the process at the mesh nodes to locations 

```{r, projector, fig.width=8, fig.height=2, small.mar=TRUE}
A <- inla.spde.make.A( ## projector creator
    mesh=mesh, ## provide the mesh
    loc=coo) ### locations where to project the field
dim(A) ## an 'n' by 'm' projector matrix
summary(rowSums(A)) ### each line sums up to one
summary(colSums(A)) ### 'how many' observations per knot
```

* __Build the SPDE model__ on the mesh.
Set $\alpha=2$ to build the precision 

```{r SPDE model}
spde <- inla.spde2.matern( ## precision components creator
    mesh=mesh, ## mesh supplied
    alpha=2) ## smoothness parameter
```

* __Create a data stack__ to organize the data. 
This is a way to allow models with complex 
linear predictors. 
In our case, we have a SPDE 
model defined on _m_ nodes. 
It must be combined 
with the covariate (and the intercept) 
effect at _n_ locations. 
We do it using different projector matrices. 

```{r data stack}
stk.e <- inla.stack( ## stack creator
  data=list(y=tmed),  ## response
  effects=list(## two elements:
    data.frame(b0=rep(1, n)), ## regressor part
    i=1:spde$n.spde),  ## RF index
  A=list(## projector list of each effect
    1, ## for the covariates
    A), ## for the RF
  tag='est') ## tag
``` 

* __Fit__ the posterior marginal distributions
  for all model parameters 

```{r fitting}
formula <- y ~ 0 + b0 + ## fixed part
  f(i, model=spde) ## RF term
res <- inla( ## main function in INLA package
  formula, ## model formula
  data=inla.stack.data(stk.e), ## dataset
  control.predictor=list( ## inform projector needed in SPDE models
    A = inla.stack.A(stk.e), compute=TRUE)) ## projector from the stack data
```

## Posterior marginal distributions - PMDs

Summary of the regression coefficients PMDs

```{r fixed}
round(res$summary.fixed, 4) 
```
We have to transform the precision PMD to have 
the variance PMD. It can be done and visialized by

```{r nugget, fig.width=4, fig.height=3, mar3311=TRUE}
m.prec <- res$marginals.hyperpar$'Precision for the Gaussian observations' ## the marginal
post.s2e <- inla.tmarginal(## function to compute a tranformation 
  function(x) 1/x, ## inverse transformation
  m.prec) ## marginal to be applied
plot(post.s2e, type='l', ylab='Density', 
     xlab=expression(sigma[e]^2))
```

The SPDE approach uses a local variance, $\tau^2$, 
such that $\sigma_{s}^2=1/(2\pi\kappa^2\tau^2)$. 
On \textbf{\textsf{INLA}} we work log($\tau^2$)
and log($\kappa$). So, especially for 
$\sigma_{s}^2$, we have to do 
an additional computation. 
The PMDs for all RF parameters on 
user scale are computed by

```{r rf} 
rf <- inla.spde.result( ## function to compute the 'interpretable' parameters
    inla=res, ## the inla() output
    name='i', ## name of RF index set
    spde=spde, ## SPDE model object
    do.transf=TRUE) ## to user scale
``` 

It can be visualized by 

```{r parameters, fig.width=10, fig.height=3}
par(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0))
plot(rf$marginals.var[[1]], ty='l', 
     xlab=expression(sigma[s]^2), yla='Density')
plot(rf$marginals.kap[[1]], type='l',
     xlab=expression(kappa), ylab='Density')
plot(rf$marginals.range[[1]], type='l', 
     xlab='range nominal', ylab='Density')
```

## Predicted 

Visualize it with the commands bellow

```{r predicted, fig.width=10, fig.height=3}
par(mfrow=c(1,1), mar=c(3,3,0.3,2), mgp=c(2,0.5,0), las=2, xaxs='i')
id <- inla.stack.index(stk.e, tag='est')$data
plot(dates, tmed, type='l', axes=FALSE, ylab='Temperature')
for (j in 3:5)
  lines(dates, res$summary.fitted.values[id, j], lty=3)
box(); axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, T))
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))
``` 

## Just a look to the rest of the data 

Pre-processing the maximum, minimum and normal temperature, 
the precipitation, and the average and maximum wind:
```{r other}
tmax <- as.numeric(gsub('<td>', '', gsub('°</td>', '', d0[i+2])))
tmin <- as.numeric(gsub('<td>', '', gsub('°</td>', '', d0[i+3])))
tnormal <- as.numeric(gsub('<td>', '', gsub('°</td>', '', d0[i+5])))
prec <- as.numeric(gsub('<td>', '', gsub('mm</td>', '', d0[i+6])))
wind <- as.numeric(gsub('<td>', '', gsub('m/s</td>', '', d0[i+10])))
wmax <- as.numeric(gsub('<td>', '', gsub('m/s</td>', '', d0[i+9])))
```

Visualize it
```{r visualize3, fig.width=10, fig.height=9}
par(mfrow=c(3,1), mar=c(0.1,3,0.1,2), mgp=c(2,.7,0), las=2, xaxs='i')
plot(dates, tmed, type='l', ylim=range(tmin, tmax, na.rm=TRUE), 
     axes=FALSE, xlab='', ylab='Temperature', col='green')
lines(dates, tmin, col='blue')
lines(dates, tmax, col='red')
lines(dates, tnormal)
legend(dates[which.min(tmin)], par()$usr[4], c('normal', 'max.', 'aver.', 'min.'), 
      col=1:4, lty=1, ncol=2, xjust=0.5, bty='n')
abline(h=5*(-5:6), v=pd, lty=3, col=gray(.5))
box(); axis(2, 5*(-5:6)); axis(4, 5*(-5:6))

plot(dates, prec, type='l', axes=FALSE, xlab='')
box(); axis(2); axis(4)
abline(v=pd, h=10*(1:4), lty=3, col=gray(0.5))

par(mar=c(3, 3, 0.1, 2), new=FALSE)
plot(dates, wind, type='l', axes=FALSE, xlab='',
     ylim=range(wind, wmax, na.rm=TRUE))
lines(dates, wmax, col=2)
box(); axis(2); axis(4)
abline(v=pd, h=5*(1:3), lty=3, col=gray(0.5))
axis(1, pd, months(pd, TRUE))
```

We can have a look at the difference between the daily mean temperature and 
the normal temperature. 

```{r anomalia, fig.width=10, fig.height=3.5}
par(mar=c(3, 3, 0.1, 2), mgp=c(2,0.7,0), las=2, xaxs='i')
plot(dates, tmed-tnormal, type='l', axes=FALSE, 
     xlab='', ylab='Deviation from the normal temperature')
box(); axis(2); axis(4)
abline(h=5*(-2:2), v=pd, lty=2, col=gray(0.5))
axis(1, pd, months(pd, TRUE))
```

---
title: "Approximating joint marginals in R-INLA"
author: "Cristian Chiuchiolo and Haavard Rue"
date: "KAUST, Aug 2020"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Joint marginals}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
bibliography: jmarginal.bib
---

```{r setup, include=FALSE}
set.seed(1234)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="classic")
if (file.exists("myinit.R")) source("myinit.R")
library(ggplot2)
library(sn)
library(knitr)
set.seed(123)
knitr::opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/jmarginal/'
               , fig.pos='H'
               , background='#ffffff'
               , results='hold'
               , eval=TRUE)
```

## Introduction

The primary use of `R-INLA` is to approximate univariate marginals of
the latent field, so we can compute their marginal summaries and
densities. In applications, we sometimes need more than this, as we
are interested also in statistics which involve more several of the
components of the latent field, and/or, a joint posterior
approximation to a subset of the latent field.

The way around this issue, have earlier resolved to stochastic
simulation, using the function `inla.posterior.sample`. This function
samples from joint approximation to the full posterior which `INLA`
construct, but do so for the whole latent field. Using these samples,
we can compute the density of the relevant statistics and/or use
standard methods to represent a joint marginal.

This vignette introduces a new tool which computes a deterministic
approximation to the joint marginal for a subset of the latent field
using `R-INLA`. This approximation is explicitly available, and
constructed using skew-normal marginals and a Gaussian copula, hence
restricted to a joint approximation of a modest dimension.

The key specification is using an argument `selection` in the
`inla()`-call, which defines the subset, and then the joint marginal
approximation is made available in `result$selection`.

## Theory reference

For any \emph{Latent Gaussian Model} with data $\boldsymbol{y}$ and
set of parameters $(\boldsymbol{x}, \boldsymbol{\theta})$ with
$\boldsymbol{x}$ being the latent field and $\boldsymbol{\theta}$ the
hyperparameters, the resulting joint posterior approximation is stated
as

\begin{equation}
\tilde{\pi}(\boldsymbol{x},\boldsymbol{\theta}|\boldsymbol{y}) \propto \sum_k \tilde{\pi}_G (\boldsymbol{x}|\boldsymbol{\theta},\boldsymbol{y}) \tilde{\pi}(\boldsymbol{\theta}_k|\boldsymbol{y})  \Delta_k
\label{approx_joint}
\end{equation}

where $\tilde{\pi}_G
(\boldsymbol{x}|\boldsymbol{\theta},\boldsymbol{y})$ is the Gaussian
approximation. This expression recalls a Gaussian mixture distribution
with weights $\tilde{\pi}(\boldsymbol{\theta}_k|\boldsymbol{y})
\Delta_k$ obtained in the grid exploration of the hyperparameter
posterior marginals. For more insights, we suggest checking sources
like @inla, @thiago, @marta, or the recent review by @sara2019. The
Gaussian approximation used in \eqref{approx_joint} is both mean and
skewness corrected since it exploits Skew-Normal marginals of the
latent field into a Gaussian copula structure (see @egil for details).
These corrections are available in `inla.posterior.sample` as

\begin{verbatim}
inla.posterior.sample(..., skew.corr = TRUE)
\end{verbatim}

## First example

We will illustrate this new feature using a simple example.
```{r}
n = 100
x = rnorm(n, mean = 1, sd = 0.3)
xx = rnorm(n, mean = -2, sd = 0.3)
y = rpois(n, lambda = exp(x + xx))
r = inla(y ~ 1 + x + xx,
         data = data.frame(y, x, xx), 
	     family = "poisson")
```
Let us compute the joint marginal for the effect of `x`, `xx` and the
intercept. This is specified using the argument `selection`, which is
a named list of indices to select. Names are those given the formula,
plus standard names like `(Intercept)`, `Predictor` and `APredictor`.
So that
```{r}
selection = list(Predictor = 3:4, x = 1, xx = 1)
```
say that we want the joint marginal for the $3^{rd}$ and $4^{th}$ element of
`Predictor` and the first element of `x` and `xx`. (Well, `x` and `xx`
only have one element, so then there is not else we can do in this
case.)

If we pass `selection`, then we have to rerun `inla()` as
```{r}
rs = inla(y ~ 1 + x + xx,
          data = data.frame(y, x, xx), 
	      family = "poisson",
          control.compute = list(return.marginals.predictor = TRUE), 
	      control.predictor = list(compute = TRUE),
		  selection = selection)
```		 
and we obtain
```{r}
#summary(rs$selection)
print(rs$selection)
```
The Gaussian copula is given by the `mean` and the `cov.matrix`
objects, while the Skew-Normal marginals are given implicitly using
the marginal mean and variance in the Gaussian copula and the listed
skewness. Moreover the respective Skew-Normal mapping parameters for
the marginals $(\xi, \omega, \alpha)$ are provided in the object
'marginal.sn.par'. The `names` are given as a separate entry instead
of naming each individual result, to save some storage.

There are utility functions to sample and evaluate samples from this
joint marginal, similar to `inla.posterior.sample` and
`inla.posterior.sample.eval`. 
```{r}
ns = 10000
xs = inla.rjmarginal(ns, rs) ## or rs$selection
str(xs)
```
whose output is a matrix where each row contains the samples for the
variable in each column
```{r}
pairs(t(xs$samples[, 1:3000]), cex = 0.1)
```
We can compare the approximation of `Predictor:3` to the one computed
by the `inla()` call, 
```{r}
hist(xs$samples["Predictor:3",], n = 300, prob = TRUE, 
     main = 'Histogram of Predictor:3', xlab = 'Samples 
     from the joint and linear predictor marginal (black straight line)')
lines(inla.smarginal(rs$marginals.linear.predictor[[3]]), lwd = 3)
```
These marginals are not *exactly* the same (as they are different
approximations), but should be very similar. 

## Deterministic Joint approximation

As a conclusion to this vignette we show an additional joint posterior
related tool. The following INLA function computes a deterministic
approximation for the joint posterior sampler and must be considered
experimental. The function is strictly related to the selection type
INLA setting. Deterministic posterior marginals for the previous
example can be obtained as follows

```{r}
dxs = inla.1djmarginal(jmarginal = rs$selection)
str(dxs)
```

The output is a list with all computed marginals and a matrix summary
output in INLA style. Marginal can be accessed and plotted by using
the respective names in the selection

```{r}
ggplot(data = data.frame(y = xs$samples["Predictor:3",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`Predictor:3`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Predictor:3"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["Predictor:4",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`Predictor:4`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Predictor:4"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["x:1",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`x:1`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for x:1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs$samples["xx:1",]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs$`xx:1`), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for xx:1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red"))  
```

Here we compare the deterministic marginals with their sampling
version. They are quite accurate and can provide more informations.
Indeed, a complete summary based on these deterministic results is
achievable with a personalized `summary` function

```{r}
summary(rs$selection)
```

where posterior estimates and quantiles are computed for all the
selected marginals. Along the same line, we can easily compute
multiple deterministic linear combinations through the function
`inla.tjmarginal`and a matrix object A with the respective indexes

```{r}
A = matrix(c(1,1,0,0,0,0,1,1), nrow = 2, ncol = 4, byrow = T)
rownames(A) <- c("Test1", "Test2")
A
```

We define two linear combinations: `Predictor:3+Predictor:4` and
`x:1+xx:1` respectively. Then we can use the cited function which has
the same class of `selection` object

```{r}
m = inla.tjmarginal(jmarginal = rs, A = A)
m
class(m)

```

```{r}
dxs.lin = inla.1djmarginal(jmarginal = m)
str(dxs.lin)

fun1 <- function(...) {Predictor[1]+Predictor[2]}
fun2 <- function(...) {x+xx}

xs.lin1 = inla.rjmarginal.eval(fun1, xs)
xs.lin2 = inla.rjmarginal.eval(fun2, xs)

ggplot(data = data.frame(y = xs.lin1[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs.lin$Test1), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Lin1"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = xs.lin2[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(dxs.lin$Test2), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for Lin2"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 
```

and accomplish the job with summaries

```{r}
summary(m)
```

Transformations of the marginal terms or linear combinations are
possible as well. We just need to use `inla.tmarginal` as follows

```{r}
fun.exp <- function(x) exp(x)

fun5 <- function(...) {exp(x)}
fun6 <- function(...) {exp(xx)}
fun7 <- function(...) {exp(x+xx)}

tdx <- inla.tmarginal(fun = fun.exp, marginal = dxs$`x:1`)
tdxx <- inla.tmarginal(fun = fun.exp, marginal = dxs$`xx:1`)
tdx.lin <- inla.tmarginal(fun = fun.exp, marginal = dxs.lin$Test2)

tx = inla.rjmarginal.eval(fun5, xs)
txx = inla.rjmarginal.eval(fun6, xs)
tx.lin = inla.rjmarginal.eval(fun7, xs)

ggplot(data = data.frame(y = tx[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdx), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(x:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = txx[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdxx), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(xx:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 

ggplot(data = data.frame(y = tx.lin[1, ]), aes(y, after_stat(density), colour = "True")) +
  stat_density(alpha = .1) +
  geom_line(data = as.data.frame(tdx.lin), aes(x = x, y = y, colour = "Deterministic"))+
  labs(title= '"Marginal Results for exp(x:1+xx:1)"', x='x', y='Density') +
  scale_colour_manual("", 
                      breaks = c("True", "Deterministic"),
                      values = c("black", "red")) 
```

Summaries for all marginal transformations can be obtained through
`inla.zmarginal`

```{r}
expx = inla.zmarginal(marginal = tdx, silent = TRUE)
expxx = inla.zmarginal(marginal = tdxx, silent = TRUE)
expx.lin = inla.zmarginal(marginal = tdx.lin, silent = TRUE)

exp.summaries = rbind(expx, expxx, expx.lin)
exp.summaries
```

## References

---
title: "Spatially Varying Coefficient Models with R-INLA"
author: "Timothy D. Meehan (tmeeha@gmail.com), Elias T. Krainski, Finn Lindgren, and Håvard Rue"
date: "Generated on `r Sys.Date()`"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Spatially Varying Coefficient Models with R-INLA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE, cache=TRUE}
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, error = FALSE, message = FALSE)
```

## Introduction

Spatially varying coefficient models (SVCs, Gelfand et al. 2003) are
often used to model data when relationships between dependent and
independent variables are not uniform across space, a common situation
when exploring phenomena across large spatial extents (Finley 2011).
Meehan et al. (2019) described an SVC model to evaluate
continent-scaled variation in bird abundance trends. The SVC model
used in that analysis employed discrete aerial units (100 km grid
cells), with spatial structure described by neighborhood matrices and
spatial relationships described by an intrinsic conditional
autoregressive model (Besag 1974). The online supplement for the paper
included code for building the model using the `R-INLA` package (Rue
et al. 2009) for the `R` statistical programming language (R Core Team
2021). Both the manuscript and code can be accessed at
<https://github.com/tmeeha/inlaSVCBC>.

In this vignette, we will describe how to build an SVC model similar
to that described in Meehan et al. (2019), but within a
continuous-space framework. This model will be computed using the
stochastic partial differential equation (SPDE) approach of Lindgren
et al. (2011, 2022), implemented in the `R-INLA` package for `R`. The
SPDE approach employs a computationally efficient approximation of a
Gaussian random field with parameters directly comparable to those of
a Matérn covariance function. The benefits of a continuous-space
versus a discrete-space SVC include the potential for finer resolution
estimation and prediction, a better understanding of the range of
spatial correlation, and a reduction in boundary effects associated
with discrete-space analyses.

We build the model using a subset of the data described in Meehan et
al. (2019). 
Specifically, we use counts of American Robin (*Turdus migratorius*)
from 
south central North America collected between 1987 and 2016 during
the Audubon Christmas Bird Count (CBC). The overall goal of the
analysis is to produce spatially explicit estimates of 
annual relative abundance as well as long-term relative abundance 
trends for robins that account for spatial and temporal variation in count effort.  

## Model

The model used to analyze these data assumes that counts come from a
negative binomial distribution with an expected count and dispersion
parameter. The expected count has a log-linear predictor:

$$\log(\lambda_{st}) = \kappa_{s} + \alpha_s + \epsilon_{s} \log[\text{Effort}_{st}] + \tau_{s} \text{Year}_{st}$$

where the natural log of expected count, $\log(\lambda_{st})$, at site
$s$ during year $t$, is modeled with a zero-centered, normally
distributed intercept per site, $\kappa_s$, a spatially varying
intercept, $\alpha_s$, a spatially varying effect of the log of count
effort in hours, $\epsilon_s$, and a spatially varying linear effect
of year, $\tau_s$. The spatially structured effects are modeled as
Gaussian random fields with Matérn covariance functions with range and
variance parameters.

Model parameters $\kappa_s$, $\alpha_s$, $\epsilon_s$ and $\tau_s$ are
analogous to those in Meehan et al. (2019). For example, $\kappa_s$ is
included to account for site-level differences in counts, possibly due
to habitat availability or observer experience. $\alpha_s$ can be
interpreted as an effort-corrected abundance index at year zero.
$\epsilon_s$ is the exponent for a power-law effort-correction
function. And $\tau_s$ is the long-term temporal trend at a given
site.

## Environment

To get started with data analysis, we set up the environment,
by
loading some packages, and
setting some options.
```{r libs, warning=FALSE, message=FALSE, errors=FALSE}
# libraries
library(maps)
library(ggplot2)
library(sf)
library(terra)
library(tidyterra) # raster plotting
library(tidyr)
library(scales)
library(inlabru)
library(INLA)
library(dplyr)
# Note: the 'splancs' package also needs to be installed,
# but doesn't need to be loaded

# set option
select <- dplyr::select
options(scipen = 99999)
options(max.print = 99999)
options(stringsAsFactors = FALSE)
```

```{r echo=F}
inla.setOption(num.threads = "1:1")
inla.setOption(smtp = "taucs")
inla.setOption(inla.mode = "experimental")
if (file.exists("myinit.R")) source("myinit.R")
```

Next we define a coordinate reference system (CRS) for spatial
analysis and create a base map for later use. The CRS uses the USA
Contiguous Albers Equal-Area Conic projection, and is identified by
the EPSG code 6703. We modify the CRS slightly to have units of
kilometers, so that distances between widespread count sites are not
especially large numbers (Krainski et al. 2018).

```{r proj, warning=F, message=FALSE}
# define a crs
epsg6703km <- paste(
  "+proj=aea +lat_0=23 +lon_0=-96 +lat_1=29.5",
  "+lat_2=45.5 +x_0=0 +y_0=0 +datum=NAD83",
  "+units=km +no_defs"
)

# make a base map
states <- st_as_sf(maps::map("state", plot = FALSE, fill = TRUE)) %>%
  filter(ID %in% c(
    "texas", "oklahoma", "kansas", "missouri",
    "arkansas", "louisiana"
  )) %>%
  st_transform(epsg6703km)
```

## Import data

Next we import some bird count data from the GitHub repository
associated with Meehan et al. (2019), and turn the data set into
spatially referenced points. We use a subset of the data (30 years, 6
US states) for this analysis to reduce computing time (~ 1 min). Note 
that site selection and zero filling, important components of trend 
analyses, have already been conducted and this is the resulting data set.

```{r data, eval=T, cache=T}
# get data
download_dat <- read.csv(paste0(
  "https://raw.github.com/tmeeha/inlaSVCBC",
  "/master/code/modeling_data.csv"
))

# select subset
count_dat <- download_dat %>%
  select(
    circle, bcr, state, year, std_yr, count, log_hrs,
    lon, lat, obs
  ) %>%
  mutate(year = year + 1899) %>%
  filter(
    state %in% c(
      "TEXAS", "OKLAHOMA", "KANSAS", "MISSOURI",
      "ARKANSAS", "LOUISIANA"
    ),
    year >= 1987
  ) %>%
  mutate(site_idx = as.numeric(factor(paste(circle, lon, lat)))) %>%
  group_by(site_idx) %>%
  mutate(n_years = n()) %>%
  filter(n_years >= 20) %>%
  ungroup() %>%
  mutate(
    std_yr = year - max(year),
    obs = seq_len(nrow(.)),
    site_idx = as.numeric(factor(paste(circle, lon, lat))),
    year_idx = as.numeric(factor(year)),
    site_year_idx = as.numeric(factor(paste(circle, lon, lat, year)))
  ) %>%
  st_as_sf(coords = c("lon", "lat"), crs = 4326, remove = FALSE) %>%
  st_transform(epsg6703km) %>%
  mutate(
    easting = st_coordinates(.)[, 1],
    northing = st_coordinates(.)[, 2]
  ) %>%
  arrange(circle, year)
```

A rough view of changes in robin relative abundance, which does not
account for variation in count effort, can be seen by plotting raw
counts per site and year.

```{r data_map, fig.pos='b', fig.align='center'}
# map it
ggplot() +
  geom_sf(
    data = count_dat %>% filter(year_idx %in% seq(1, 30, 3)),
    aes(col = log(count + 1))
  ) +
  geom_sf(data = states, fill = NA) +
  coord_sf(datum = NA) +
  facet_wrap(~year) +
  scale_color_distiller(palette = "Spectral") +
  theme_bw()
```

## Make spatial data

Next we use the count data to make a map of distinct count sites and
save the coordinates of the sites, unique and across all years, for
later spatial modeling and plotting.

```{r spat_dat}
# make a set of distinct study sites for mapping
site_map <- count_dat %>%
  select(circle, easting, northing) %>%
  distinct() %>%
  select(circle, easting, northing)

# save coordinates as matrices for later
unique_coords <- count_dat %>%
  select(circle, easting, northing) %>%
  distinct() %>%
  select(easting, northing) %>%
  st_drop_geometry() %>%
  as.matrix()
all_coords <- as.matrix(st_coordinates(count_dat))
```

## SPDE components

Computing a continuous-space model with `R-INLA` using the SPDE
approach requires construction of four distinct sets of data and model
objects (Blangiardo and Cameletti 2015, Krainski et al. 2018). First,
we create a **modeling mesh**, which is used to provide a piecewise
linear representation of the continuous spatial surface, based on a
triangulation of the modeled region. Here, the same mesh will get
reused for each of the spatial and spatio-temporal terms in the model.
Second, we construct an **SPDE model object** that specifies
properties of the spatial model. Again, we will use the same SPDE
object for each of the spatial terms in the model. Third, we create
**named index sets** that identify data locations and mesh nodes for
use in model construction, computation, and output extraction. Note
that a separate set of indices needs to be constructed for each
spatial term in the model. Fourth, we construct **projector matrices**
or **A matrices** that create formulaic mappings between the observed
data locations and the mesh nodes. Again, note that a separate set of
projector matrices needs to be constructed for each spatial term in
the model. The four corresponding INLA functions for these steps are:
`inla.mesh.2D()`, `inla.spde2.pcmatern()`, `inla.spde.make.index()`,
and `inla.spde.make.A()`.

## Modeling mesh

There are various things to consider when constructing a mesh 
(Lindgren and Rue 2015, Blangiardo and Cameletti 2015, Krainski 
et al. 2018, Bakka et al. 2018). In constructing one, we balance a 
trade-off between capturing fine-scaled features of the Gaussian 
random field and computing times. Here, we create a non-convex 
hull around the count sites, and then build a triangular mesh by 
specifying minimum and maximum edge lengths within and slightly 
outside the hull.

```{r mesh, warning=F, message=F, fig.pos='t', fig.align='center'}
# make a hull and mesh for spatial model
hull <- inla.nonconvex.hull(
  points = as.matrix(st_coordinates(count_dat)),
  convex = 200, concave = 350
)
mesh <- inla.mesh.2d(
  boundary = hull, max.edge = c(100, 600), # km inside and outside
  cutoff = 50, offset = c(100, 300)
) # cutoff is min edge

# plot it
ggplot() +
  gg(data = mesh) +
  geom_sf(data = site_map, col = "darkgreen", size = 1) +
  geom_sf(data = states, fill = NA) +
  theme_bw() +
  labs(x = "", y = "")
```

## SPDE model object

Next we create an SPDE object to define the model smoothness, with 
prior distributions for the variance and range parameters, and the 
mesh. Here we assume a Gaussian random field characterized with a 
Matérn covariance function with penalized complexity priors 
(Simpson et al. 2017) for the practical range (distance where spatial
correlation approaches 0.1) and variation explained by the function
(Fuglstad et al. 2019). The prior for the spatial range is set such
that the probability of a range exceeding 500 km is 0.5. The prior for
the variance explained by the spatial effect is set such that the
probability of a standard deviation exceeding 1 is 0.5 (Krainski et
al. 2018). If one wants to constrain this kind of spatial effect to integrate
to zero, `constr=TRUE` should be added at this stage.

```{r spde}
# make spde
spde <- inla.spde2.pcmatern(
  mesh = mesh,
  prior.range = c(500, 0.5),
  prior.sigma = c(1, 0.5)
)
```

## Index sets

Next we create index sets for the spatial terms in the model. Note
that the `tag` names applied to the index sets in this step will be
used in the model formula statement below. The index vectors for the
spatial terms are essentially vectors of integers going from 1 to the
number of nodes on the mesh.

```{r indices}
# make index sets
alpha_idx <- inla.spde.make.index(name = "alpha", n.spde = mesh$n)
eps_idx <- inla.spde.make.index(name = "eps", n.spde = mesh$n)
tau_idx <- inla.spde.make.index(name = "tau", n.spde = mesh$n)
```

## Projector matrices

Next we create projector matrices for the spatial terms in the model.
Note that this is the step where SVCs are specified. In our model,
$\epsilon_s$ is a spatially varying effect of the log of count effort.
The SVC effort effect is specified using the `weights` argument to
`inla.spde.make.A()` function. Similarly, $\tau_s$ is a spatially
varying effect of year, so standardized year (with 1987 = 0) is also
specified in the `weights` argument. $\alpha_s$ is also an SVC, but it
is a spatially varying intercept. For intercepts it is not necessary
to specify a constant weight of 1 when making a projector matrix. The
only model component ignored here is $\kappa_{s}$, because it is not
spatially structured and not modeled on the mesh. Note that the
current use of the term 'weights' is different from that often
encountered when defining mixed effect models in `R`. Here it is used
to define covariate values, as opposed to importance values in other
contexts.

```{r projector}
# make projector matrices
A_alpha <- inla.spde.make.A(mesh = mesh, loc = all_coords)
A_eps <- inla.spde.make.A(
  mesh = mesh, loc = all_coords,
  weights = count_dat$log_hrs
) # note weights argument
A_tau <- inla.spde.make.A(
  mesh = mesh, loc = all_coords,
  weights = count_dat$std_yr
) # note weights argument
```

## Data stack for model fitting

With the SPDE method, there are several data structures that need to
be bundled and used for analysis of the spatial model. There is the
observed response at the count sites, the covariate values at the
count sites, and the indices and mapping of those data to the modeling
mesh. All of this information gets bundled into a data stack for model
analysis using the `inla.stack()` function. This function is a very
flexible and can be difficult to understand at first. See Blandgiardo
and Cameletti (2015) and Gomez-Rubio (2020) for practical information
on how to use it.

The first argument to the function, shown below, is the `tag` name for
the stack. In this case we are including observed data for model
estimation, so we name the stack 'obs'. The next argument, `data`,
specifies the response variable. Here we have a single response
variable called 'count'. We tell the stacking function that observed
counts reside in the 'count_dat' data set. The next argument,
`effects`, is used to specify the model predictors. These are defined
using the observed data for non-spatial predictors, and using the
named index sets for the spatial predictors. The last argument, `A`,
asks for a list of projector matrices or A matrices for the predictor
variables. We did not need to make a projector for $\kappa_s$, because
it is not defined as a spatial parameter. Nevertheless, we add a value
of 1 because it is the value that will be multiplied by the effect
during model computations and serves as a way for the stacking
function to match the number of A matrices with the number of effects
-- if there are 4 effects, then there must be four entries for `A`.
Note that if there were more than one non-spatial predictor in the
model, whether a grouped random intercept or a global fixed effect,
then each one would be given its own value of 1 during this step. And
note that the order of the A matrices must match the order of the
effects.

```{r stack1}
# stack observed data
stack_fit <- inla.stack(
  tag = "obs",
  data = list(count = as.vector(count_dat$count)), # response from data frame
  effects = list(data.frame(
    intercept = 1,
    kappa = count_dat$site_idx
  ), # predictors from data frame
  alpha = alpha_idx, # or index sets if spatial
  eps = eps_idx,
  tau = tau_idx
  ),
  A = list(
    1, # a value of 1 is given for non-spatial terms
    A_alpha,
    A_eps,
    A_tau
  )
)
```

## Model formula

The last required input for the analysis is the model formula, which
includes information on the prior for explained variation for the
unstructured random intercept. We define the prior for $\kappa_s$ as a
penalized complexity prior (Simpson et al. 2017), set such that
the probability of the standard deviation associated with the random
effect exceeding 1 is 0.01.

```{r prior}
# iid prior
pc_prec <- list(prior = "pcprec", param = c(1, 0.1))
```

Notice that if one wants to constrain a spatial spde effect to
integrate to zero, it should be added `constr=TRUE` in the SPDE model
definition rather than in the `f()` terms. As we want to constrain
$\kappa_s$ and it is a non-spatial term we can use `constr=TRUE` in
its corresponding `f()` below, which imposes a sum-to-zero constraint.

The model described above is translated to `R-INLA` modeling syntax
as:

```{r form}
# formula
svc_form <- count ~ -1 +
  f(kappa, model = "iid", constr = TRUE, hyper = list(prec = pc_prec)) +
  f(alpha, model = spde) +
  f(eps, model = spde) +
  f(tau, model = spde)
```

Here, we define the response as `count`, remove the automatic global
intercept with a `-1`, and then specify the other terms in the model
with `f()` statements. The first `f()` statement defines $\kappa_s$,
the site effect, as a normally distributed (`model="iid"`),
zero-centered (`constr=TRUE`), deviation from $\alpha_s$. The name
'kappa' given in the `f()` argument is associated with the effect
'kappa' defined in the estimation stack. The second `f()` statement
defines $\alpha_s$ as a spatially varying intercept with spatial
structure described by the SPDE object called 'spde'. The third `f()`
statement defines $\epsilon_s$ as an SVC for the effect of count
effort, with spatial structure also described by the SPDE object. The
weights for this spatially structured random slope were specified
during construction of the projector A matrix, `A_eps`. The fourth
`f()` statement defines $\tau_s$ as an SVC for the year effect, with
spatial structure described in the SPDE object. Again, the weights for
this spatially structured random slope were specified during
construction of the projector A matrix, `A_tau`. Again, note that the
current use of the term 'weights' is different from that often
encountered when defining mixed effect models in `R`. Here it is used
to define covariate values, as opposed to importance values in other
contexts.

## Run model

We estimate the model with a call to `inla()`. First we set the option
to use the (new) experimental way to do internal computations, see Van
Niekerk et. al. (2022), for the sake of computing speed and better
numerics. In the call to `inla()`, we give the model formula, specify
the negative binomial distribution for the counts, define the
estimation data, and describe the location of the projector A matrix.
Note that we are not generating predictions (`compute=F`).
Nevertheless, the `control.preditor` argument still needs to be
included so the software knows how to find the A matrices. Then we ask
`inla()` to compute WAIC and CPO to evaluate model fit, and to save
the information necessary for posterior sampling (`config=T`). For
computing speed, we choose to use the adaptive integration strategy
and Empirical Bayes estimation. The `inla()` run for this model takes
about 1 minutes on a standard laptop computer. Another option is to
use the Variational Bayes approximation as detailed in Van Niekerk and
Rue (2021) and Van Niekerk et. al. (2022).

```{r run, eval=T, cache=TRUE}
res <- inla(svc_form,
  family = "nbinomial",
  data = inla.stack.data(stack_fit), # stack the stack
  control.predictor = list(
    A = inla.stack.A(stack_fit),
    compute = FALSE
  ), # must define A
  control.compute = list(waic = TRUE, cpo = TRUE, config = TRUE),
  control.inla = list(strategy = "adaptive", int.strategy = "eb"),
  verbose = FALSE
)
```

```{r load_res, echo=F, eval=F}
load("spatial_cbc/code/spde_vignette_v3.RData")
```

## Model summaries

Once computation is complete, we look at the initial results to see
how things went. First we check the posterior means for the
hyperparameters of the model, mainly the variance components and the
spatial ranges of the spatially structured parameters.

```{r vc}
# view results
res$summary.hyperpar[-1, c(1, 2)]
```

Next we examine some summaries of the random effect estimates,
starting with $\exp(\alpha_s)$, which is effort-corrected relative abundance
at year = 0 (1987), given 1 hour of count effort (i.e., log[1]=0). 

```{r alpha_res}
summary(exp(res$summary.random$alp$"0.5quant")) # exp(alpha) posterior median
```
Note that to avoid issues due to $E[\exp(x)|y]\neq\exp[E(x|y)]$, we use the posterior median instead of the posterior mean.

The summary for $\epsilon_s$ shows variation in the exponent for the
effort correction function across space.

```{r eps_res}
summary(res$summary.random$eps$mean) # epsilon
```

The $\tau_s$ summary shows how long-term, log-linear trends of robin
relative abundance have varied across space, from annual decreases of
around 10% to annual increases of around 10%.

```{r tau_res}
summary((exp(res$summary.random$tau$"0.5quant") - 1) * 100) # (exp(tau)-1)*100
```

## SVC maps

Next we create maps of $\alpha_s$, $\epsilon_s$, and $\tau_s$ to
inspect the spatial structure of these parameter estimates. We start
by creating a 25-km mapping grid, and then projecting this mapping
grid to the modeling mesh.

```{r map_grid}
# get easting and northing limits
xlim <- range(hull$loc[, 1])
ylim <- range(hull$loc[, 2])
grd_dims <- round(c(x = diff(range(xlim)), y = diff(range(ylim))) / 25)

# make mesh projector to get model summaries from the mesh to the mapping grid
mesh_proj <- inla.mesh.projector(
  mesh,
  xlim = xlim, ylim = ylim, dims = grd_dims
)
```

Then we populate the mapping grids with parameter estimates (posterior
median and range95), turn them into a raster stack, and mask the raster stack
to the study area.

```{r map_vals}
# pull data
kappa <- data.frame(
  median = exp(res$summary.random$kappa$"0.5quant"),
  range95 = exp(res$summary.random$kappa$"0.975quant") -
    exp(res$summary.random$kappa$"0.025quant")
)
alph <- data.frame(
  median = exp(res$summary.random$alpha$"0.5quant"),
  range95 = exp(res$summary.random$alpha$"0.975quant") -
    exp(res$summary.random$alpha$"0.025quant")
)
epsi <- data.frame(
  median = res$summary.random$eps$"0.5quant",
  range95 = (res$summary.random$eps$"0.975quant" -
    res$summary.random$eps$"0.025quant")
)
taus <- data.frame(
  median = (exp(res$summary.random$tau$"0.5quant") - 1) * 100,
  range95 = (exp(res$summary.random$tau$"0.975quant") -
    exp(res$summary.random$tau$"0.025quant")) * 100
)

# loop to get estimates on a mapping grid
pred_grids <- lapply(
  list(alpha = alph, epsilon = epsi, tau = taus),
  function(x) as.matrix(inla.mesh.project(mesh_proj, x))
)
```


```{r map_rast}
# make a terra raster stack with the posterior median and range95
out_stk <- rast()
for (j in 1:3) {
  mean_j <- cbind(expand.grid(x = mesh_proj$x, y = mesh_proj$y),
    Z = c(matrix(pred_grids[[j]][, 1], grd_dims[1]))
  )
  mean_j <- rast(mean_j, crs = epsg6703km)
  range95_j <- cbind(expand.grid(X = mesh_proj$x, Y = mesh_proj$y),
    Z = c(matrix(pred_grids[[j]][, 2], grd_dims[1]))
  )
  range95_j <- rast(range95_j, crs = epsg6703km)
  out_j <- c(mean_j, range95_j)
  terra::add(out_stk) <- out_j
}
names(out_stk) <- c(
  "alpha_median", "alpha_range95", "epsilon_median",
  "epsilon_range95", "tau_median", "tau_range95"
)
out_stk <- terra::mask(out_stk, states, touches = FALSE)
```

Finally, we plot the SVCs with the following code. We plot the posterior median and 95% uncertainty width ("range95")
for $\exp(\kappa_s)$, $\exp(\alpha_s)$, $\epsilon_s$, and $100(\exp(\tau_s)-1)$.

```{r svc_plot_construction, warning=F, message=F}
make_plot_field <- function(data_stk, scale_label) {
  ggplot(states) +
    geom_sf(fill = NA) +
    coord_sf(datum = NA) +
    geom_spatraster(data = data_stk) +
    labs(x = "", y = "") +
    scale_fill_distiller(scale_label,
                         palette = "Spectral",
                         na.value = "transparent") +
    theme_bw() +
    geom_sf(fill = NA)
}
make_plot_site <- function(data, scale_label) {
  ggplot(states) +
    geom_sf() +
    coord_sf(datum = NA) +
    geom_sf(data = data, size = 1, mapping = aes(colour = value)) +
    scale_colour_distiller(scale_label, palette = "Spectral") +
    labs(x = "", y = "") +
    theme_bw() +
    geom_sf(fill = NA)
}

# medians
# fields alpha_s, epsilon_s, tau_s
pa <- make_plot_field(
  data_stk = out_stk[["alpha_median"]],
  scale_label = "posterior\nmedian\nexp(alpha_s)"
)
pe <- make_plot_field(
  data_stk = out_stk[["epsilon_median"]],
  scale_label = "posterior\nmedian\nepsilon_s"
)
pt <- make_plot_field(
  data_stk = out_stk[["tau_median"]],
  scale_label = "posterior\nmedian\n100(exp(tau_s)-1)"
)
# sites kappa_s
ps <- make_plot_site(
  data = cbind(site_map, data.frame(value = kappa$median)),
  scale_label = "posterior\nmedian\nexp(kappa_s)"
)
# range95
# fields alpha_s, epsilon_s, tau_s
pa_range95 <- make_plot_field(
  data_stk = out_stk[["alpha_range95"]],
  scale_label = "posterior\nrange95\nexp(alpha_s)"
)
pe_range95 <- make_plot_field(
  data_stk = out_stk[["epsilon_range95"]],
  scale_label = "posterior\nrange95\nepsilon_s"
)
pt_range95 <- make_plot_field(
  data_stk = out_stk[["tau_range95"]],
  scale_label = "posterior\nrange95\n100(exp(tau_s)-1)"
)
# sites kappa_s
ps_range95 <- make_plot_site(
  data = cbind(site_map, data.frame(value = kappa$range95)),
  scale_label = "posterior\nrange95\nexp(kappa_s)"
)
```
```{r svc_plots, warning=F, message=F, fig.width=8, fig.align='center'}
# plot together
multiplot(ps, pa, pe, pt, cols = 2)
```
The map for the posterior mean of $\tau_s$ shows that robins have 
decreased in the southern part of the study area and increased in 
the northern part. This demonstrates how the wintering range of robins 
is shifting northward as winters become warmer due to climate change.

```{r svc_plots_range95, warning=F, message=F, fig.width=8, fig.align='center'}
# plot together
multiplot(ps_range95, pa_range95, pe_range95, pt_range95, cols = 2)
```

## More information

More information on building spatial models using the SPDE approach in
R-INLA can be found in Lindgren and Rue (2015), Blangiardo and
Camaletti (2015), Bakka et al. (2018), Krainski et al. (2018), and
Moraga (2019).

## Citations

Bakka, H., Rue, H., Fuglstad, G.A., Riebler, A., Bolin, D., Illian,
J., Krainski, E., Simpson, D. and Lindgren, F., 2018. Spatial modeling
with R‐INLA: A review. Wiley Interdisciplinary Reviews: Computational
Statistics, 10(6), p.e1443.

Besag, J., 1974. Spatial interaction and the statistical analysis of
lattice systems. Journal of the Royal Statistical Society: Series B
(Methodological), 36(2), pp.192-225.

Blangiardo, M., Cameletti, M., Baio, G. and Rue, H., 2013. Spatial and
spatio-temporal models with R-INLA. Spatial and spatio-temporal
epidemiology, 4, pp.33-49.

Finley, A.O., 2011. Comparing spatially‐varying coefficients models
for analysis of ecological data with non‐stationary and anisotropic
residual dependence. Methods in Ecology and Evolution, 2(2),
pp.143-154.

Fuglstad, G.A., Simpson, D., Lindgren, F. and Rue, H., 2019.
Constructing priors that penalize the complexity of Gaussian random
fields. Journal of the American Statistical Association, 114(525),
pp.445-452.

Gelfand, A.E., Kim, H.J., Sirmans, C.F. and Banerjee, S., 2003.
Spatial modeling with spatially varying coefficient processes. Journal
of the American Statistical Association, 98(462), pp.387-396.

Gómez-Rubio, V., 2020. Bayesian inference with INLA. CRC Press.

Krainski, E., Gómez-Rubio, V., Bakka, H., Lenzi, A., Castro-Camilo,
D., Simpson, D., Lindgren, F. and Rue, H., 2018. Advanced spatial
modeling with stochastic partial differential equations using R and
INLA. Chapman and Hall/CRC.

Lindgren, F., Rue, H. and Lindström, J., 2011. An explicit link
between Gaussian fields and Gaussian Markov random fields: the
stochastic partial differential equation approach. Journal of the
Royal Statistical Society: Series B (Statistical Methodology), 73(4),
pp.423-498.

Lindgren, F. and Rue, H., 2015. Bayesian spatial modelling with
R-INLA. Journal of statistical software, 63, pp.1-25.

Lindgren, F., Bolin, D. and Rue, H., 2022. The SPDE approach for
Gaussian and non-Gaussian fields: 10 years and still running. Spatial
Statistics, p.100599.

Link, W.A., Sauer, J.R. and Niven, D.K., 2006. A hierarchical model
for regional analysis of population change using Christmas Bird Count
data, with application to the American Black Duck. The Condor, 108(1),
pp.13-24.

Meehan, T.D., Michel, N.L. and Rue, H., 2019. Spatial modeling of
Audubon Christmas Bird Counts reveals fine‐scale patterns and drivers
of relative abundance trends. Ecosphere, 10(4), p.e02707.

Moraga, P., 2019. Geospatial health data: Modeling and visualization
with R-INLA and shiny. CRC Press.

R Core Team (2021). R: A language and environment for statistical
computing. R Foundation for Statistical Computing, Vienna, Austria.

Rue, H., Martino, S. and Chopin, N., 2009. Approximate Bayesian
inference for latent Gaussian models by using integrated nested
Laplace approximations. Journal of the royal statistical society:
Series b (statistical methodology), 71(2), pp.319-392.

Simpson, D., Rue, H., Riebler, A., Martins, T.G. and Sørbye,
S.H., 2017. Penalising model component complexity: A principled,
practical approach to constructing priors. Statistical science, 32(1),
pp.1-28.

Soykan, C.U., Sauer, J., Schuetz, J.G., LeBaron, G.S., Dale, K. and
Langham, G.M., 2016. Population trends for North American winter birds
based on hierarchical models. Ecosphere, 7(5), p.e01351.

Van Niekerk, J. and Rue, H., 2021. Correcting the Laplace Method with
Variational Bayes. Journal of Machine Learning Research, Under review.

Van Niekerk, J. and Krainski, E. T. and Rustand, D. and Rue, H., 2022.
A new avenue for Bayesian inference with INLA. Submitted.

---
title: "Short tutorials from old www-page"
author: "Haavard Rue (hrue@r-inla.org)"
date: "KAUST, Aug 2020"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Short tutorials}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
library(INLA)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(smtp='taucs')
inla.setOption(num.threads="1:1")
knitr::opts_chunk$set(fig.path="figures/old-faq/")
set.seed(123)
source("render_toc.R")
```

## Contents
```{r toc, echo=FALSE} 
render_toc("old-faq.Rmd")
```


## Introduction

This vignette is a copy and slightly edited FAQ-entries and other
short tutorials from the old \verb|www.r-inla.org| page. The content
comes is slightly random order, sorry about that.

## User-defined priors for the hyperparameters

If you want to use a prior for the hyperparameter that is not yet
implemented there are two choices. If you think that your prior should
be on the list and that other might use it to, please let us know.
Alternatively, you can define your own prior using \verb|prior =
"expression: ...."|, or by specifiying a table of
$x$ and $y$ values which define the prior distribution.

There are three ways to specify prior distributions for
hyperparameters in INLA:

* Use an available prior distribution
* Define your own prior distribution function using 
  \verb|R|-like (not equal) syntax as expression.
* Create a table of $(x,y)$ values which represent your prior distribution.

In the following we will provide more details regarding the two last
options. Finally, we will present an example illustrating (and
comparing) the three different possibilities by means of the log-gamma
distribution for the precision parameter.

A user can specify any (univariate) prior distribution for the
hyperparameter $\theta$ by defining an expression for the log-density
log $\pi(\theta)$, as a function of the corresponding $\theta$. It is
important to be aware that $\theta$ is on the internal scale.

The format is
```{r, eval=FALSE}
    expression: statement; statement; ...; return(value)
```

where "statement" is any regular statement (more below) and
"value" is the value for the log-density of the prior, evaluated at
the current value for $\theta$. 

Here, is an example defining the log-gamma distribution:
```{r,eval=FALSE}
    prior.expression = "expression:
        a = 1;
        b = 0.1;
        precision = exp(log_precision);
        logdens = log(b^a) - lgamma(a)
           + (a-1)*log_precision - b*precision;
        log_jacobian = log_precision;
        return(logdens + log_jacobian);"
```
Some syntax specific notes:
* No white-space before "(.)" in the return statement.
* A ";" is needed to terminate each expression.
* A "_" is allowed in variable names.

Known functions that can be used within the expression statement are

* common math functions, such as exp, sin, ...
* "gamma" denotes the gamma-function and "lgamma" is its log
* \verb|x^y| is expressed as either \verb|x^y| or \verb|pow(x;y)|

Instead of defining a prior distribution function, it is possible to
provide a table of suitable values $x$ (internal scale) and the
corresponding log-density values $y$. INLA fits a spline through the
provided points and continues with this in the succeeding
computations. Note, there is no transformation into a functional form
performed or required. The input-format for the table is a string,
which starts with \verb|table:| and is then followed by a block of
$x$-values and a block of the corresponding $y$-values, which
represent the values of the log-density evaluated on x. Thus
```{r,eval=FALSE}
    table: x_1 ... x_n y_1 ... y_n
```

We illustrate all three different ways of defining a prior
distribution for the precision of a normal likelihood. To show that
the three definitions lead to the same result we inspect the
logmarginal likelihood.
```{r}
## the loggamma-prior
prior.function = function(log_precision) {
    a = 1;
    b = 0.1;
    precision = exp(log_precision);
    logdens = log(b^a) - lgamma(a) + (a-1)*log_precision - b*precision;
    log_jacobian = log_precision;
    return(logdens + log_jacobian)
}

## implementing the loggamma-prior using "expression:"
prior.expression = "expression:
a = 1;
b = 0.1;
precision = exp(log_precision);
logdens = log(b^a) - lgamma(a)
    + (a-1)*log_precision - b*precision;
log_jacobian = log_precision;
return(logdens + log_jacobian);"

## use suitable support points x
lprec = seq(-10, 10, len=100)
## link the x and corresponding y values into a 
## string which begins with "table:""
prior.table = paste(c("table:", cbind(lprec,
                    prior.function(lprec))), collapse=" ", sep="")

# simulate some data
n = 50
y = rnorm(n)

## use the built-in loggamma prior
r1 = inla(y~1,data = data.frame(y),
control.family = list(hyper = list(prec = list(
                      prior = "loggamma", param = c(1, 0.1)))))

## use the definition using expression
r2 = inla(y~1, data = data.frame(y), 
          control.family = list(hyper = list(
		                        prec = list(prior = prior.expression))))

## use a table of x and y values representing the loggamma prior
r3 = inla(y~1, data = data.frame(y), 
          control.family = list(hyper = list(
		                        prec = list(prior = prior.table))))

print(round(c(r1$mlik[1], r2$mlik[1], r3$mlik[1]), dig=3))
```


## Does INLA support the use of different link-functions?

Yes, the type of link function is given in the \verb|control.family|
statement using \verb|control.link=...|, and the type of link-functions
implemented are listed on the documentation for each likelihood. The
default link is \verb|default| which corresponds to the second link
function in the list. Here is an example

```{r}
n = 100
z = rnorm(n)
eta = 1 + 0.1*z
N = 2

p = inla.link.invlogit(eta)
y = rbinom(n,  size = N, prob = p)
r = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="logit")), 
		control.predictor = list(compute=TRUE))

p = inla.link.invprobit(eta)
y = rbinom(n,  size = N, prob = p)
rr = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="probit")), 
		control.predictor = list(compute=TRUE))

p = inla.link.invcloglog(eta)
y = rbinom(n,  size = N, prob = p)
rrr = inla(y ~ 1 + z,  data = data.frame(y, z), family = "binomial", Ntrials = rep(N, n),
        control.family = list(control.link = list(model="cloglog")), 
		control.predictor = list(compute=TRUE))        
```
Other linkfunctions/models are also avilable from within \verb|R|, see \verb|?inla.link|



## How can I do predictions using INLA?

In \verb|INLA| there is no function \verb|predict| as for \verb|glm/lm|
in \verb|R|. Predictions must to done as a part of the model fitting
itself. As prediction is the same as fitting a model with some missing
data, we can simply set \verb|y[i] = NA| for those "locations" we
want to predict. Here is a simple example
```{r}
n = 100
n.pred = 10
y = arima.sim(n=n, model=list(ar=0.9))
N = n + n.pred
yy = c(y, rep(NA, n.pred))
i = 1:N
formula = yy ~ f(i, model="ar1")
r = inla(formula, data = data.frame(i,yy),
        control.family = list(initial = 10, fixed=TRUE)) ## no observational noise
```
which gives predictions
```{r}
r$summary.random$i[(n+1):N, c("mean", "sd") ]
```

Quantiles such like \verb|r$summary.fitted.values| and
\verb|r$marginals.fitted.values|, if computed, use the identity link
if \verb|y[i] = NA| by default. If you want the \verb|fitted.values|
computed with a different link function, then there are two ways to
doit.

In the case you want to use the link-function from the likelihood
already used (most often the case), there is the argument \verb|link|
in \verb|control.predictor|. If the response \verb|y[idx] = NA|, then
set \verb|link[idx] = 1|, to indicate that you want to compute that
fitted value using the link function from \verb|family[1]|. With
several likelihoods, set \verb|link[idx]| to family-index which is
correct, ie the column number in the response. The following example
shows the usage:
```{r,plot=TRUE}
## simple poisson regression
n = 100
x = sort(runif(n))
eta = 1 + x
lambda = exp(eta)
y = rpois(n, lambda = lambda)

## missing values:
y[1:3] = NA
y[(n-2):n] = NA

## link = 1 is a shortcut for rep(1, n) where n is the appropriate
## length. here '1' is a reference to the first 'family', ie
## 'family[1]'
r = inla(y ~ 1 + x,  family = "poisson",
        data = data.frame(y, x),
        control.predictor = list(link = 1))
plot(exp(eta),type ="l")
points(r$summary.fitted.values$mean, pch=19)
```

We only need to define \verb|link| where there
are missing values. Entries for which the observation is
not \verb|NA|, is ignored.

For more than one likelihood, use '2' to refer to the second
likelihood. Here is an example where we split the data in two, and
assign the second half the \verb|nbinomial| distribution.
```{r,plot=T}
n2 = n %/% 2L
Y = matrix(NA, n, 2)
Y[1:n2, 1] = y[1:n2]
Y[1:n2 + n2, 2] = y[1:n2 + n2]
link = rep(NA, n)
link[which(is.na(y[1:n2]))] = 1
link[n2  + which(is.na(y[1:n2 + n2]))] = 2

r = inla(Y ~ 1 + x,  family = c("poisson", "nbinomial"),
        data = list(Y=Y, x=x),
        control.predictor = list(link = link))
plot(exp(eta),type ="l")
points(r$summary.fitted.values$mean, pch=19)
```

We can transform marginals manually using the function
\verb|inla.marginal.transform| or compute expectations using 
\verb|inla.emarginal|, like in this example (taken
from \verb|demo(Tokyo)|).

```{r}
## Load the data
data(Tokyo)
summary(Tokyo)

Tokyo$y[300:366] <- NA

## Define the model
formula = y ~ f(time, model="rw2", scale.model=TRUE, 
	            constr=FALSE, cyclic=TRUE,
				hyper = list(prec=list(prior="pc.prec",
	                                   param=c(2,0.01)))) -1 

## We'll get a warning since we have not defined the link argument
result = inla(formula, family="binomial", Ntrials=n, data=Tokyo,
	control.compute = list(return.marginals.predictor = TRUE), 
	control.predictor=list(compute=T))

## need to recompute the fitted values for those with data[i] = NA,
## as the identity link is used.
n = 366
fitted.values.mean = numeric(n)
for(i in 1:366) {
    if (is.na(Tokyo$y[i])) {
        if (FALSE) {
            ## either like this, which is slower
            marg = inla.marginal.transform(
                            function(x) exp(x)/(1+exp(x)),
                            result$marginals.fitted.values[[i]] )
            fitted.values.mean[i] = inla.emarginal(function(x) x, marg)
        } else {
            ## or like this,  which is faster
            fitted.values.mean[i] = inla.emarginal(
                            function(x) exp(x)/(1 +exp(x)),
                            result$marginals.fitted.values[[i]])
        }
    } else {
        fitted.values.mean[i] = result$summary.fitted.values[i,"mean"]
    }
}
plot(fitted.values.mean)
```



## Some of the models needs a graph, how do I specify it?

Some of the models in \verb|INLA| needs the user to specify a graph,
saying which nodes are neighbours to each other. A 'graph' can be
specified in three different ways.

* As an ascii or binary file with a graph specification, or the same
  contents given as (possible list of) mix of character and numerics
  arguments.
* As a symmetric (dense or sparse) matrix, where the non-zero pattern
  of the matrix defines the graph.
* As an inla.graph-object


A graph defined in an ascii-file, must have the following format.
The first entry is the number of nodes in the graph, $n$. The nodes in
the graph are labelled $1, 2, \ldots, n$. The next entries, specify the
number of neighbours and the neighbours for each node.
A simple example is the following
```{r, eval=FALSE}
4    
1 2 3 4
2 0
3 1 1
4 1 1
```

This defines a graph with four nodes, where node 1 has 2 neighbours 3
and 4, node 2 as 0 neighbours, node 3 has 1 neighbour 1, and node 4
has 1 neighbour 1, and the graph looks like this
```{r,plot=TRUE,fig.width=2,fig.align='center'}
g = inla.read.graph("4 1 2 3 4 2 0 3 1 1 4 1 1")
plot(g)
```

Note that we need to specify the implied symmetry as well. In this
example 4 is a neighbour of 1, then we also need to specify that 1 is
a neighbour of 4.

Instead of storing the graph specification on a file, it can also be
specified as a character string with the same contents as a file, like
```{r,eval=FALSE}
"4 1 2 3 4  2 0 3 1 1 4 1 1"
```
as used in \verb|inla.read.graph| above.

Due to imitations of the length of a string/line, so in practice, this
way specifying the graph, seems more useful for teaching or
demonstration purposes than for practical analysis.

Within \verb|INLA|, this would look like
```{r,eval=FALSE}
formula = y ~ f(idx, model = "besag", graph = "graph.dat")
```
or
```{r,eval=FALSE}
formula = y ~ f(idx, model = "besag", graph = "4 1 2 3 4  2 0 3 1 1 4 1 1")
```

A graph can also be defined as a symmetric (dense or sparse) matrix,
where the non-zero pattern of the matrix defines the graph A neighbour
matrix is often used for defining which nodes that are neighbours,
with the convention that if \verb|Q[i,j] != 0| then $i$ and $j$ are neighbours
if $i\not=j$.

For example, the (dense) matrix C
```{r}
C = matrix(c(1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1),4,4)
C
```
defines the same graph show above. 

Since graphs tends to be large, we often
define them as a sparse matrix
```{r}
C.sparse= inla.as.sparse(C)
C.sparse
```
We can then use \verb|graph=C| or \verb|graph=C.sparse| in the formula.

We can also define the graph as an \verb|inla.graph|-object, which 
is used internally, represent a graph. For example
```{r}
str(g)
```
and use \verb|graph = g| as the argument.

The internal format, are as follows. \verb|n| is the size of the
graph. \verb|nnbs| are the number of neighbours to each node, \verb|nbs|
list all the neighbours to each node, and the class is \verb|inla.graph|.
The \verb|cc|-list is for internal use only and specify the connected
components in the graph. 

\verb|INLA| has some functions to work with graphs, and here is a
short summary.

* \verb|inla.read.graph()| and \verb|inla.write.graph()|, read and
  write graphs using any of the graph specifications above.
  
* You can plot a \verb|inla.graph|-object using \verb|plot()| and 
  get a summary using \verb|summary()|. The plotting requires the
  \verb|Rgraphviz| package.

* From a graph specification, you can generate the neighbour matrix, 
  using \verb|inla.graph2matrix()|. You can plot a graph 
  specification as a neighbour matrix, using \verb|inla.spy()|

* If you have 'errors' in your graph, you may read it using
  \verb|inla.debug.graph()|. This is only available for a graph
  specification in an ascii-file.



## How \texttt{INLA} deal with \texttt{NA}

For a formula like
```{r eval=FALSE}
formula = y ~ x + f(k, model= <some model>)
```
then \verb|NA|'s in either $y$, $x$ or $k$ are treated differently.

\verb|NA|'s in the response $y$. If \verb|y[i] = NA|, this means that
\verb|y[i]| is not observed, hence gives no contribution to the likelihood.

\verb|NA|'s in fixed effect $x$. If \verb|x[i] = NA| this means that \verb|x[i]| is not part
of the linear predictor for \verb|y[i]|. For fixed effects, this is
equivalent to \verb|x[i]=0|, hence internally we make this change:
\verb|x[is.na(x)] = 0|.

\verb|NA|'s in random effect $k$. If \verb|k[i] = NA|, this means that
the random effect does not contribute to the linear predictor for
\verb|y[i]|. 

\verb|NA|'s in a factor $x$. \verb|NA|'s in a factor $x$ is not allowed
unless \verb|NA| is a level in itself, or 
```{r eval=F}
control.fixed = list(expand.factor.strategy = "inla")
```
is set. With this option, then \verb|NA| is interpreted similarly as a
fixed effect, where \verb|NA| means no contribution from x. The effect of
\verb|expand.factor.strategy="inla"|, is best explained with an example.

```{r}
r = inla(y ~ 1 + x, data = data.frame(y=1:3, x=factor(c("a","b","c"))))
as.matrix(r$model.matrix)
```
for default value of the argument \verb|contrasts|. The effect of \verb|xa| is
removed to make the corresponding matrix non-singular. If we want to
expand \verb|x| into each of each three effects, then we can do
```{r}
r = inla(y ~ 1 + x, data = data.frame(y=1:3,x=factor(c("a","b","c"))), 
         control.fixed = list(expand.factor.strategy="inla"))
as.matrix(r$model.matrix)
```

As we see, each level of the factor is now treated symetrically.
Although the corresponding frequentist matrix is singular as we have
confounding with the intercept, the Bayesian posterior is still proper
with proper priors. 

With a \verb|NA| in \verb|x|, we get

```{r}
r = inla(y ~ 1 + x, data = data.frame(y=1:3,x=factor(c("a","b",NA))), 
         control.fixed = list(expand.factor.strategy="inla"))
as.matrix(r$model.matrix)
```
so that the 3rd element of the linear predictor has no contribution
from \verb|x|, as it should.


## Can INLA deal with missing covariates?

No, INLA has no generic way to "impute" or integrate-out missing
covariates. You have to adjust your model to account for missing
covariates, like using one of the measurement error models ("meb",
"mec"), or construct a joint model for the data and the covariates,
but this is case-specific.


##  Compute cross-validation or predictive measures of fit

\verb|INLA| provides two types of leave-one-out predictive
measures of fit. It is the CPO value, which is 
$$
	\text{Prob}(y_i | y_{-i}),
$$	
the PIT value 
$$
	\text{Prob}(y_i^\text{new} \le y_i | y_{-i})
$$
To enable the computation of these quantities, you will need to add the argument
```{r eval=FALSE}
control.compute=list(cpo=TRUE)
```
We can also compute PO values
$$
	\text{Prob}(y_i | y),
$$	
when argument \verb|po=TRUE| is added.


If the resulting object is \verb|result|, then you will find the predictive
quantities as \verb|result$cpo$cpo| and \verb|result$cpo$pit|.

Implicit assumptions made in for computations, and there are internal
checks that these are satisfied. The results of these checks will
appear as \verb|result$cpo$failure|. In short, if
\verb|result$cpo$failure[i] > 0| then some assumption is violated, the
higher the value (maximum 1) the more seriously. If
\verb|result$cpo$failure[i] == 0| then the assumptions should be ok.

You \emph{may} want to recompute those with non-zero failure. However,
this must be done \emph{manually} by removing \verb|y[i]| from the dataset, fit
the model and then predict \verb|y[i]|. To provide a more efficient
implementation of this, we have provided
```{r eval=FALSE}
improved.result = inla.cpo(result)
```
which take an inla-object which is the output from \verb|inla()|, and
recompute (in an efficient way) the cpo/pit for which
\verb|result$cpo$failure > 0|, and return 'result' with the improved
estimates of cpo/pit. See \verb|?inla.cpo| for details.



## I have access to a remote Linux/MacOS server, is it possible to run the computations remotely and running R locally?

Yes! This option allow \verb|INLA| to use a remote server to do the
computations. In order to use this feature, you need to do some setup
which is different from (various) Linux distributions, Mac and
Windows. In short:

* install \verb|R| and \verb|R-INLA| a remote server, for example
  \verb|foo.bar.org|.
* Install your public ssh-key on \verb|foo.bar.org| to setup password
  free access to the remove server using \verb|ssh|. And please check
  that this is indeed working before moving forward!
* On your local host, run  \verb|inla.remote()| to initialise the 
  init-file \verb|~/.inlarc|  and then edit this file to fit your needs.
* You may now have to log out and log in again, to make sure your 
  ssh key is signed out.
* You can now use option \verb|inla.call="remote"| to do the computations
  on your remote server, or set this globally with 
  \verb|inla.setOption("inla.call", "remote")|

You can also submit a job on the remote server, so you do not need to
sit and wait for it to finish, but you can collect the results later.
Basically, you do
```{r eval=FALSE}
r = inla(..., inla.call = "submit")
```
which will start the job on the server. You can start many jobs, and
list them using
```{r eval=FALSE}
inla.qstat()
```
and you can fetch the results (for the job above) using
```{r eval=FALSE}
r = inla.qget(r)
```

You can also delete jobs and fetch the jobs from another machine; see
\verb|?inla.q| for further details.

HOWTO setup ssh-keys: For the unexperienced user, this is somewhat
tricky; sorry about that. The easiest is to find a friend that knows
this and can help you. Newer system do a lot of these things very
nicely these days.

It is also possible to setup this from Windows using CYGWIN, and INLA
can work with this interface as well. Please see the old web-page for
details, which are long and technical. HOWEVER, I am no longer
convinced that this work anymore, as I haven't seen this is use for
years. It is much much easier to use a virtual machine with Linux on
Windows.



## Posteriors for linear combinations

I have some linear combinations of the nodes in the latent field that
I want to compute the posterior marginal of, is that possible? Yes!
These are called 'linear combinations'. There are handy functions,
'inla.make.lincomb()' and 'inla.make.lincombs()', to define one or
many such linear combinations. Single linear combinations made by
using 'inla.make.lincomb()' can easily be joined into many. Its use is
easiest explained using a rather long example...

Here is the example, that explains these features.

```{r}
## A simple model
n = 100
a = rnorm(n)
b = rnorm(n)
idx = 1:n

y = rnorm(n) + a + b
formula = y ~ 1 + a + b + f(idx, model="iid")

## assume we want to compute the posterior for
##
##  2 * beta_a + 3 * beta_b + idx[1] - idx[2]
##
## which we specify as follows (and giving it a unique name)

lc1 = inla.make.lincomb(a=2, b=3, idx = c(1,-1,rep(NA,n-2)))
names(lc1) = "lc1"

## strictly speaking, it is sufficient to use `idx = c(1,-1)', as the
## remaining idx's are not used in any case.

r = inla(formula, data = data.frame(a,b,y),
        ## add the linear combinations here
        lincomb = lc1,
        ## force noise variance to be essiatially zero
        control.family = list(initial=10, fixed=TRUE))
		
## to verify the result, we can compare the mean but the variance and
## marginal cannot be computed from the simpler marginals alone.
lc1.1 = 2 * r$summary.fixed["a", "mean"] + 3 * r$summary.fixed["b",
    "mean"] + r$summary.random$idx$mean[1] -
    r$summary.random$idx$mean[2]
lc1.2= r$summary.lincomb.derived$mean
print(round(c(lc1.1 = lc1.1, lc1.2 = lc1.2), dig=3))
```
The marginals are available as \verb|r$marginals.lincomb$...|

There is an another function which is handy for specifying many linear
combinations at once, that is \verb|inla.make.lincombs()| (note the
plural s). Here each 'row' define one linear combination
```{r}
## let wa and wb be vectors, and we want to compute the marginals for
## beta_a * wa[i] + beta_b * wb[i], for i=1..m. this is done
## conveniently as follows

m = 10
wa = runif(m)
wb = runif(m)
lc.many = inla.make.lincombs(a = wa, b=wb)

## we can give them names as well, but there are also default names, like
print(names(lc.many))
r = inla(formula, data = data.frame(a,b,y),
        lincomb = lc.many,
        control.family = list(initial=10, fixed=TRUE))
print(round(r$summary.lincomb.derived, dig=3))
```

Terms like 'idx' above, can be added as \verb|idx = IDX| into
\verb|inla.make.lincombs()|, where IDX is a matrix. Again, each column
of the arguments define one linear combination.

There is a further option available for the derived linear
combinations, that is the option to compute also the posterior
correlation matrix between all the linear combinations. To activate
this option, use
```{r eval=FALSE}
control.inla = list(lincomb.derived.correlation.matrix = TRUE)
```
and you will find the resulting posterior correlation matrix as
```{r eval=FALSE}
result$misc$lincomb.derived.correlation.matrix
```

Here is a small example where we compute the correlation matrix for
the predicted values of a hidden AR(1) model with an intercept.
```{r}
n = 100
nPred = 10
phi = 0.9
x = arima.sim(n,  model = list(ar=phi)) * sqrt(1-phi^2)
y = 1 + x + rnorm(n,  sd=0.1)

time = 1:(n + nPred)
Y = c(y, rep(NA, nPred))
formula = Y ~ 1 + f(time, model="ar1")

## make linear combinations which are the nPred linear predictors
B = matrix(NA, nPred, n+nPred)
for(i in 1:nPred) {
    B[i, n+i] = 1
}
lcs = inla.make.lincombs(Predictor = B)

r = inla(formula,  data = data.frame(Y, time),
        control.predictor = list(compute=TRUE),
        lincomb = lcs,
        control.inla = list(lincomb.derived.correlation.matrix=TRUE))

print(round(r$misc$lincomb.derived.correlation.matrix,dig=3))
```


## INLA seems to work great for near all cases, but are there cases where INLA is known to have problems?

The methodology needs the full conditional density for the latent
field to be "near" Gaussian. This is usually achived by either
replications or smoothing/"borrowing strength". A simple example which
do not have this, is the following:
```{r}
n = 100
u = rnorm(n)
eta = 1 + u
p = exp(eta)/(1+exp(eta))
y = rbinom(n, size=1,  prob = p)

idx = 1:n
result = inla(y ~ 1 + f(idx, model="iid",
                        hyper = list(prec = list(prior="pc.prec",
						                         prior = c(1,0.01)))), 
              data =data.frame(y,idx), family = "binomial", 
	          Ntrials = 1)
summary(result)
```
For each binary observation there is an iid "random effect" $u$, and
there is no smoothing/``borrowing strength'' (apart from the weak
intercept). If you plot the loglikelihood for eta for $y=1$, say, then
its an increasing function for increasing eta, so the likelihood
itself would like $\eta = \infty$. With an unknown precision for $u$
we run into problems; INLA has a tendency to estimate a to high
precision for $u$. However, it must be noted that the model is almost
singular and you'll have a strong prior sensitivity in the (exact)
results as well. There is a similar discussion in here as well for the
Salamander data example.

## Can I have the linear predictor from one model as a covariate in a different model?

Yes, this is possible. Essentially, you have to set the linear
predictor for the first model equal to 'u', and then you can copy 'u'
and use the scaling to get the regression coefficient. A simple
example will illustrate the idea:

```{r}
## simple example
n = 100
x1 = rnorm(n)
eta1 = 1 + x1
x2 = rnorm(n)
eta2 = 2 + 2*eta1 + 2*x2
y1 = rnorm(n, mean=eta1, sd = 0.01)
y2 = rnorm(n, mean=eta2, sd = 0.01)

## the trick is to create a vector 'u' (iid) which is 
## equal to eta1, and then we can copy 'u' to
## create beta*u or beta*eta1. we do this by 
## using 0 = eta1 -u + tiny.noise

formula = Y ~ -1 + intercept1 + X1 + intercept2 + f(u, w, model="iid",
    hyper = list(prec = list(initial = -6, fixed=TRUE))) + f(b.eta2,
    copy="u", hyper = list(beta = list(fixed = FALSE))) + X2
    
Y = matrix(NA, 3*n, 3)

## part 1: y1
intercept1 = rep(1, n)
X1 = x1
intercept2 = rep(NA, n)
u = rep(NA, n)
w = rep(NA, n)
b.eta2 = rep(NA, n)
X2 = rep(NA, n)
Y[1:n, 1] = y1

## part 2: 0 = eta1 - u + tiny.noise
intercept1 = c(intercept1,  intercept1)
X1 = c(X1,  x1)
intercept2 = c(intercept2,  rep(NA, n))
u = c(u, 1:n)
w = c(w, rep(-1, n))
b.eta2 = c(b.eta2,  rep(NA, n))
X2 = c(X2,  rep(NA, n))
Y[n + 1:n, 2] = 0

## part 3: y2
intercept1 = c(intercept1,  rep(NA, n))
X1 = c(X1,  rep(NA, n))
intercept2 = c(intercept2, rep(1, n))
u = c(u, rep(NA, n))
w = c(w, rep(NA, n))
b.eta2 = c(b.eta2, 1:n)
X2 = c(X2, x2)
Y[2*n + 1:n, 3] = y2

r = inla(formula,
    data =  list(Y=Y, intercept1=intercept1, X1=X1,
        intercept2=intercept2, u=u, w=w, b.eta2=b.eta2, X2=X2),
    family = rep("gaussian", 3),
	control.inla = list(h = 1e-3),
    control.family = list(
        list(),
        list(hyper = list(prec = list(initial = 10, fixed=TRUE))),
        list()))

summary(r)
```

## Latent models, likelihoods and priors.

The list of latent models, likelihood and priors implemented, can be
found by doing (or give a spesific section, see
\verb|?inla.list.models|)
```{r}
inla.list.models()
```

## Copying a model

We often encounter the situation where an element of a model is needed
more than once for each observation. One example is where
```{r eval=FALSE}
y = a + b*w + ... 
```
for fixed weights $w$ and where $(a_i, b_i)$ is bivariate Normal and
all 2-vectors are independent.

Using the model
```{r eval=FALSE}
f(idx, model="iid2d", n=2*m, ...)
```
provide a random vector $v$, say, with length $2m$ stored as
$$
v = (a_1, a_2, ..., a_m, b_1, b_2, ...., b_m).
$$
To implement this, we simply create an indentical copy of $v$, $v^*$,
where $v == v^*$ (nearly). Using the copy-feature, we can do
```{r eval=FALSE}
idx = 1:m
idx.copy = m + 1:m
formula = y ~  f(idx, model="iid2d", n=2*m) + f(idx.copy, w, copy="idx") + ....
```
recalling that the first $m$ elements is $a$ and 
the last $m$ elements are $b$, and where $w$ are the weights.

The second \verb|f()| terms define itself as a copy of \verb|f(idx, ...)|, and
it inherit some of its features, like \verb|n| and \verb|values|.

A copied model may also have an unknown scaling (hyperparameter),
which is default fixed to be 1. In the following example, we will use
this feature to estimate the unknown scaling (in this case, scaling is
2), assuming we know the precision for $z$.
```{r}
n=1000
i=1:n
j = i
z = rnorm(n)
w = runif(n)
y = z  + 2*z*w + rnorm(n)
formula = y ~ f(i, model="iid",initial=0, fixed=T) +
              f(j, w, copy="i", fixed=FALSE)
r = inla(formula, data = data.frame(i,j,w,y))
summary(r)
```

If the scaling parameter is within given range, then option \verb|range =
c(low, high)|, can be given. In this case 
```{r eval=FALSE}
beta = low + (high-low)*exp(beta.local)/(1+exp(beta.local))
```
and the prior is defined on \verb|beta.local|. 

If \verb|low=high| or \verb|range = NULL|, then the identity mapping
is used. If \verb|high=Inf| and |verb|low!=Inf|, then the mapping
\verb|low + exp(beta.local)| is used. The case low=Inf and high!=Inf
is not yet implemented.

A model or a copied model can be copied several times. The degree of
closeness of $v$ and $v^*$ is specified by the argument
\verb|precision|, as the precision of the noise added to $v$ to get
$v^*$.


## Replicate a model

Independent replications of a model with the same hyperparmeters can
be defined using the argument \verb|replicate|,
```{r eval=FALSE}
f(idx, model = ..,  replicate = r)
```

Here, \verb|r| is a vector of the same length as \verb|idx|. In this
case, we use a two-dimensional index to index this (sub-)model:
\verb|(idx, r)|, so \verb|(1,2)| identify the first value of the
second replication of this model (component). Number of replications
are defined as \verb|max(replicate)|, unless it is defined by the argument
\verb|nrep|.

One example is the model `iid':
```{r eval=FALSE}
i = 1:n
formula = y ~  f(i, model = "iid") + ...
```

which has an alternative equivalent formulation as `n' replications of
an iid-model with length 1
```{r eval=FALSE}
i = rep(1,n)
r = 1:n
formula = y ~  f(i, model="iid", replicate = r) + ...
```

In the following example, we estimate the parameters in three AR(1)
time-series with the same hyperparameters (ie its replicated) but with
separate means:
```{r}
n = 100
y1 = arima.sim(n=n, model=list(ar=c(0.9)))+10
y2 = arima.sim(n=n, model=list(ar=c(0.9)))+20
y3 = arima.sim(n=n, model=list(ar=c(0.9)))+30

formula = y ~ mean -1 + f(i, model="ar1", replicate=r)
y = c(y1,y2,y3)
i = rep(1:n, 3)
r = rep(1:3, each=n)
mean = as.factor(r)
result = inla(formula, family = "gaussian",
              data = data.frame(y, i, mean),
              control.family = list(initial = 12, fixed=TRUE))
summary(result)
```
All other arguments is interpreted for the basic model and also
replicated. Like argument \verb|constr=TRUE|, is interpreted as each
replication sums to zero, and additional constraints are also
replicated.


## Models with more than one type of likelihood

There is no constraint in INLA that the type of likelihood must be the
same for all observations. In fact, every observation could have its
own likelihood. Extentions include more than one familily, like the
Normal, Poisson, etc, but also having in the model groups of
observations with separate hyperparameters within each group, where
the family, for example, can be the same.

In the formula
```{r eval=FALSE}
y ~ a + 1
```
we allow \verb|y| to be a matrix. In this case each column of \verb|y|
define one likelihood where the family is the same the hyperparameters
are the same. For each row, only one of the columns could (but don't
have to) have an observation (non-\verb|NA| value), the other colums
must have value \verb|NA|. All other parameters to the likelihood,
like \verb|E| \verb|Ntrials|, \verb|offset| and \verb|scale| are used
as appropriate. Example: If row $i$ column $j$ is a Poission
observation, then \verb|E[i]| is used as the scaling. Similar with the
others. This works as only one column for each row is non-\verb|NA|.

The argument \verb|family| is in the case where \verb|y| is a matrix, a list of
families. The argument \verb|control.family| is then a list of lists; one
for each family. 

The first example, is a simple linear regression, where the first half
of the data is observed with unknown precision \verb|tau.1| (with a
'default' noninformative prior) and the second half of the data is
observed with unknown precision \verb|tau.2|. Otherwise, the two
models have the same form for the linear predictor.
```{r}
##  Simple linear regression with observations with two different
##  variances.
n = 100
N = 2*n
y = numeric(N)
x = rnorm(N)

y[1:n] = 1 + x[1:n] + rnorm(n, sd = 1/sqrt(1))
y[1:n + n] = 1 + x[1:n + n] + rnorm(n, sd = 1/sqrt(2))

Y = matrix(NA, N, 2)
Y[1:n, 1] = y[1:n]
Y[1:n + n, 2] = y[1:n + n]

formula = Y ~ x + 1
result = inla(
        formula,
        data = list(Y=Y, x=x),
        family = c("gaussian", "gaussian"),
        control.family = list(list(prior = "flat", param = numeric()),
                            list()))
summary(result)
```

The second example shows how to use information from two
sources to estimate the effect of the covariate \verb|x|.
```{r}
## Simple example with two types of likelihoods
n = 10
N = 2*n

## common covariates
x = rnorm(n)

## Poisson, depends on x
E1 = runif(n)
y1 = rpois(n, lambda = E1*exp(x))

## Binomial, depends on x
size = sample(1:10, size=n, replace=TRUE)
prob = exp(x)/(1+exp(x))
y2 = rbinom(n, size= size, prob = prob)

## Join them together
Y = matrix(NA, N, 2)
Y[1:n, 1] = y1
Y[1:n + n, 2] = y2

## The E for the Poisson
E = numeric(N)
E[1:n] = E1
E[1:n + n] = NA

## Ntrials for the Binomial
Ntrials = numeric(N)
Ntrials[1:n] = NA
Ntrials[1:n + n] = size

## Duplicate the covariate which is shared
X = numeric(N)
X[1:n] = x
X[1:n + n] = x

## Formula involving Y as a matrix
formula = Y ~ X - 1

## `family' is now
result = inla(formula,
        family = c("poisson", "binomial"),
        data = list(Y=Y, X=X),
        E = E, Ntrials = Ntrials)
summary(result)
```

If the covariate 'x' is different for the two families, \verb|x| and
\verb|xx|, say, then we only need to make the following changes
```{r eval=FALSE}
X = numeric(N)
X[1:n] = x
X[1:n + n] = NA

XX = numeric(N)
XX[1:n] = NA
XX[1:n + n] = xx

formula = Y  ~ X + XX -1
```
and add \verb|XX| into the data.frame. Note how we can express the
joint model as a 'union' of models with the use of \verb|NA|'s to
remove terms.


In the next example, we use also the \verb|replicate| feature to
estimate three replicated AR(1) models with the same hyperparamters,
each observed differently.
```{r}
## An example with three independent AR(1)'s with separate means, but
## with the same hyperparameters. These are observed with three
## different likelihoods.

n = 100
x1 = arima.sim(n=n, model=list(ar=c(0.9))) + 0
x2 = arima.sim(n=n, model=list(ar=c(0.9))) + 1
x3 = arima.sim(n=n, model=list(ar=c(0.9))) + 2

## Binomial observations
Nt = 10 + rpois(n,lambda=1)
y1 = rbinom(n, size=Nt, prob = exp(x1)/(1+exp(x1)))

## Poisson observations
Ep = runif(n, min=1, max=10)
y2 = rpois(n, lambda = Ep*exp(x2))

## Gaussian observations
y3 = rnorm(n, mean=x3, sd=0.1)

## stack these in a 3-column matrix with NA's where not observed
y = matrix(NA, 3*n, 3)
y[1:n, 1] = y1
y[n + 1:n, 2] = y2
y[2*n + 1:n, 3] = y3

## define the model
r = c(rep(1,n), rep(2,n), rep(3,n))
rf = as.factor(r)
i = rep(1:n, 3)
formula = y ~ f(i, model="ar1", replicate=r, constr=TRUE) + rf -1
data = list(y=y, i=i, r=r, rf=rf)

## parameters for the binomial and the poisson
Ntrial = rep(NA, 3*n)
Ntrial[1:n] = Nt
E = rep(NA, 3*n)
E[1:n + n] = Ep

result = inla(formula, family = c("binomial", "poisson", "normal"),
              data = data, Ntrial = Ntrial, E = E,
              control.family = list(
                      list(),
                      list(),
                      list()))
summary(result)
```

## Models where the response/data depends on linear combinations of the "linear predictor" (or the sum of "fixed" and "random" effects)

In some cases, the data/response might depend on a linear combination
of the "linear predictor" defined in the formula, like
```{r eval=FALSE}
y ~ 1 + z
```
then this implies that \verb|y[1]| depends on \verb|intercept + beta*z[1]|.
Suppose if \verb|y[1]| depends on \verb|2*intercept + beta*z[1] + beta*z[2]|?
Although it is possible to express this, using the tools we already
have, it is more convenient to add another layer into the model. Let
\verb|A| be a \verb|m x n| matrix, which defines new linear predictors, \verb|eta~|
from \verb|eta|, like
```{r eval=FALSE}
eta~ = A %*% eta
```
Here, \verb|eta| is the ordinary linear predictor defined using the
formula, and the data depends on the linear predictor \verb|eta~|. So
we might express this as
```{r eval=FALSE}
y ~ 1 + z, with addition matrix A
```
meaning in short, that
```{r eval=FALSE}
y    ~ eta~   ## no intercept...
eta~ = A %*% eta
eta  = intercept + beta*z
```
This is specified by adding the $A$-matrix, using 
```{r eval=FALSE}
control.predictor=list(A=A)
```

The argument \verb|offset|, which might be defined in the formula as
\verb|offset(value)| or as an argument \verb|inla(..., offset = value)|, does
have different interpretation in the case where the $A$-matrix is used.
The rule is that \verb|offset| in the formula, goes into \verb|eta|, whereas an
argument \verb|offset| goes into \verb|eta~|. If we write out the expressions
above adding offsets, \verb|offset.formula| and \verb|offset.arg|, we get
```{r eval=FALSE}
eta~ = A %*% eta + offset.arg
eta = intercept + beta*z + offset.formula
```
In the case where there is no $A$-matrix, then \verb|offset.total =
offset.arg + offset.formula|.

The following example should provide more insight. You may change $n$
and $m$, such that $m < n$, $m=n$ or $m > n$. Note that since the
response has dimension $m$ and the covariates dimension $n$, we need
to use \verb|list(y=y, z=z)| and not a \verb|data.frame()|. This
example also illustrates the use of offset's.
```{r}
## 'm' is the number of observations of eta*, where eta* = A eta +
## offset.arg, and A is a fixed m x n matrix, and eta has length n. An
## offset in the formula goes into 'eta' whereas an offset in the
## argument of the inla-call, goes into eta*
n = 10
m = 100
offset.formula = 10+ 1:n
offset.arg = 1 + 1:m

## a covariate
z = runif(n)

## the linear predictor eta
eta = 1 + z + offset.formula

## the linear predictor eta* = A eta + offset.arg.
A = matrix(runif(n*m), m, n);
##A = inla.as.sparse(A)  ## sparse is ok
## need 'as.vector', as 'Eta' will be a sparseMatrix if 'A' is sparse
## even if ncol(Eta) = 1
Eta = as.vector(A %*% eta) + offset.arg

s = 1e-6
Y = Eta + rnorm(m, sd=s)

## for a check, we can use several offsets. here, m1=-1 and p1=1, so
## they m1+p1 = 0.
r = inla(Y ~ 1+z + offset(offset.formula) + offset(m1) + offset(p1),
        ## The A-matrix defined here
        control.predictor = list(A = A, compute=TRUE, precision = 1e6),
        ## we need to use a list() as the different lengths of Y
        ## and z
        data = list(Y=Y, z=z,
                m1 = rep(-1, n),
                p1 = rep(1, n),
                offset.formula = offset.formula,
                offset.arg = offset.arg),
        ## this is the offset defined in the argument of inla
        offset = offset.arg,
        ##
        control.family = list(initial = log(1/s^2), fixed=TRUE))
summary(r)

## this should be a small number
print(max(abs(r$summary.linear.predictor$mean - c(Eta, eta))))
```


Here is a another example where the informal formula is 
```{r eval=FALSE}
y = intercept + s[j] + 0.5*s[k] + noise
```
Instead of using the copy feature, we can implement this model using
the A-matrix feature. What we do, is to first define a linear
predictor being the intercept and $s$, then we use the $A$-matrix to
'construct the model'. 
```{r plot=TRUE}
n = 100
s = c(-1, 0, 1)
nS = length(s)
j = sample(1L:nS, n, replace=TRUE)
k = j
k[j == 1L] = 2
k[j == 2L] = 3
k[k == 3L] = 1

noise = rnorm(n, sd=0.0001)
y = 1 + s[j] + 0.5*s[k] + noise

## build the formula such that the linear predictor is the intercept
## (index 1) and the 's' term (index 2:(n+1)). then kind of
## 'construct' the model using the A-matrix.
formula = y ~ -1 + intercept + f(idx)
A = matrix(0, n, nS+1L)
for(i in 1L:n) {
  A[i, 1L]        = 1
  A[i, 1L + j[i]] = 1
  A[i, 1L + k[i]] = 0.5
}

data = list(intercept = c(1, rep(NA, nS)), idx = c(NA, 1L:nS))
result = inla(formula, data=data, control.predictor=list(A=A))
## should be a straight line
plot(result$summary.random$idx$mean, s, pch=19)
abline(a=0,b=1)
```

---
title: "User defined integration points"
author: "Haavard Rue `(hrue@r-inla.org)`"
date: "May 29th 2017 (updated April 9th 2019)"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{User defined integration points}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, cache = FALSE)
Design = matrix()
library(INLA)
set.seed(123)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
```

## Introduction

This short note describe a new option that allow the user to use
user-defined integration points (or "design" points), instead of the
default ones. The relevant integration in INLA does
$$
	\int f(x | \theta, y) \; \pi(\theta | y)
		\;
			d\theta = f(x | y)
$$
where $\pi(\theta | y)$ is the approximated posterior marginal for the
hyperparameters, and where $f(x | \theta, y)$ is the approximated
marginal for $x$ for that configuration.
The output of this integral is the posterior marginal $f(x|y)$.
In practice, we use a discrete set of integration points for $\theta$,
and corresponding weights $w$, to  get
$$
	f(x|y) \approx \sum_i f(x|\theta_i, y) w_i \pi(\theta_i | y)
$$
for which we require $w_i \ge 0$ and $\sum_i w_i = 1$.
Usually, the integration is done in a *standardised scale*,
$$
z = A(\theta - \gamma)
$$
i.e.\ with respect to $\pi(z|y)$. Here, 
$\gamma$ is the mode of $\pi(\theta|y)$ and the matrix $A$ is
the negative square root of the approximated covariance matrix for $\theta|y$
at the mode. 

The relevant options
are
```{r}
	opts = control.inla(int.strategy = "user", int.design = Design)
```
where `Design` is a matrix with the integration points and the
integration weights. The $j$th row of `Design` consists of the values
$\theta_j = (\theta_{1j}, \ldots, \theta_{mj})$, and the integration
weight for this configuration, $w_j$. The values are in the
$\theta$-scale, meaning that you have to know exactly what you are
doing, including knowing the ordering of the hyperparameters.

Another version, is to define the points in the standardised scale
$z$. To do this, use
```{r, eval=FALSE}
	opts = control.inla(int.strategy = "user.std", int.design = Design)
```
instead. The meaning of `Design` is unchanged, except that these can
be given in standardised coordinates.
This version is more relevant if you want to
implement a generic new integration design instead of the ones
already provided.

## First example 

In this artificial example, we want to compute the change of the
marginal variance of one component, $x_1$, of a hidden AR$(1)$
process, with respect to lag one correlation $\rho$. So we want to
compute
$$
\frac{\partial \mbox{SD}(x_1|y)}{\partial \rho}
$$
for a fixed value of $\rho = \rho_0$.
We have to compute a numerical approximation, using finite difference.
While doing this, it is a good idea to keep the design fixed, to avoid
introducing an error for changing that part as well.

Let us first setup the experiment
```{r, eval=TRUE, echo=TRUE}
n = 100
rho = 0.9
x = scale(arima.sim(n, model = list(ar = rho)))
y = x + rnorm(n, sd = 0.1)
```
this gives the following
```{r}
plot(y, xlab = "time", ylab = "value")
lines(x, lwd=2)
```
To compute the derivative, we do
```{r}
rho.0 = rho
to.theta = inla.models()$latent$ar1$hyper$theta2$to.theta
rho.0.internal = to.theta(rho.0)

r = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.0.internal,
		              fixed=TRUE))),
    control.inla = list(int.strategy = "grid"),
	data = data.frame(y, time = 1:n))

sd.0 = r$summary.random$time[1,"sd"]
print(sd.0)
```
The ordering of the hyperparamters are as follows,
```{r}
nm = names(r$joint.hyper)
nm = nm[-length(nm)]
print(nm)
```
which may sometimes be useful to know about.

Anyway, we will now change $\rho$ a little, while we keep the same
integration
points,
```{r}
Design = as.matrix(cbind(r$joint.hyper[, seq_along(nm)], 1))
head(Design)
```
where the last column is the (un-normalised) integration weights.
`Design` has dimension `r dim(Design)`.
We  call `inla()` again reusing the previous found mode
```{r}
h.rho = 0.01
rho.1.internal = to.theta(rho.0 + h.rho)
rr = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.1.internal,
		              fixed=TRUE))),
	control.mode = list(result = r, restart=FALSE),	
    data = data.frame(y, time = 1:n),
	control.inla = list(
		int.strategy = "user",
		int.design = Design))
sd.1 = rr$summary.random$time[1,"sd"]
print(sd.1)
```
and then our estimate of the derivative is
```{r}
deriv.1 = (sd.1 - sd.0) / h.rho
print(deriv.1)
```

PS: In the logfile of the `inla()`-call, the configurations are shown
in the $z$ scale even for `int.strategy="user"`.


## Second example

There is also another (experimental) option, that is
```{r, eval=FALSE}
control.inla = list(int.stategy = "user.expert")
```
for which the weights \textbf{includes}  $\pi(\theta_i | y)$. The
following example show how to use it, fitting the same model in three
different ways. 
```{r}
n = 50
x = rnorm(n)
y = 1 + x + rnorm(n, sd = 0.2)
param = c(1, 0.04)
dz = 0.1
r.std = inla(y ~ 1 + x, data = data.frame(y, x),
             control.inla = list(int.strategy = "grid",
                                 dz = dz,
                                 diff.logdens = 8), 
             control.family = list(
                 hyper = list(
                     prec = list(
                         prior = "loggamma",
                         param = param))))

s = r.std$internal.summary.hyperpar[1,"sd"]
m = r.std$internal.summary.hyperpar[1,"mean"]
theta = m + s*seq(-4, 4, by = dz)
weight = dnorm(theta,  mean = m, sd = s)

r = rep(list(list()), length(theta))
for(k in seq_along(r)) {
    r[[k]] = inla(y ~ 1 + x,
                  control.family = list(
                      hyper = list(
                          prec = list(
                              initial = theta[k],
                              fixed=TRUE))),
                  data = data.frame(y, x))
}
r.merge = inla.merge(r, prob = weight)

r.design = inla(y ~ 1 + x,
                data = data.frame(y, x),
                control.family = list(
                    hyper = list(
                        prec = list(
                            ## the prior here does not really matter, as we will override
                            ## it with the user.expert in any case.
                            prior = "pc.prec",
                            param = c(1, 0.01)))), 
                control.inla = list(int.strategy = "user.expert",
                                    int.design = cbind(theta, weight)))
for(k in 1:2) {
    plot(inla.smarginal(r.std$marginals.fixed[[k]]),
         lwd = 2, lty = 1, type = "l", 
         xlim = inla.qmarginal(c(0.0001, 0.9999), r.std$marginals.fixed[[k]]))
    lines(inla.smarginal(r.design$marginals.fixed[[k]]),
          lwd = 2, col = "blue", lty = 1)
    lines(inla.smarginal(r.merge$marginals.fixed[[k]]),
          lwd = 2, col = "yellow", lty = 1)
}
```

\documentclass[a4paper,11pt]{article}

\usepackage[margin={2cm,2cm}]{geometry}

\usepackage[utf8]{inputenc}
\usepackage[T1]{fontenc}
\usepackage{textcomp}

\title{\vspace{-2cm}
  SPDE one dimensional example}
\author{Elias T. Krainski and H{\aa}vard Rue}
\date{Created: September 26th 2016 - Last update: August 9th 2017}

%\VignetteEngine{knitr::knitr} 
%\VignetteIndexEntry{SPDE1d}
\begin{document}

\maketitle

\vspace{-1cm}
\begin{center}\begin{minipage}[c]{0.9\textwidth}\centering

In this example we show row to analyse a time series 
of daily temperature using a one dimension SPDE model.
More details about it are on the paber at 
<https://www.jstatsoft.org/article/view/v063i19>
\end{minipage}\end{center}

<<setting,include=FALSE>>=
library(knitr);  library(INLA);  library(fields) 
set.seed(123)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
fig.path="figures/SPDE1d/")
knit_hooks$set(small.mar = function(before, options, envir) {
    if (before) par(mar = c(0.1, 0.1, .1, .1))  # smaller margin on top and right
})
knit_hooks$set(mar3311 = function(before, options, envir) {
    if (before) par(mar = c(3, 3, 1, 1), mgp=c(2, 0.7, 0))  # smaller margin on top and right
})
@

\section{The data}

We consider the daily weather data available at 
<http://www.yr.no/>. 
We have the following set the URL for the daily data for 
Trondheim considerint in the last 13 months 
<<url>>=
u0 <- paste0("http://www.yr.no/place/Norway/S%C3%B8r-Tr%C3%B8ndelag/",
             "Trondheim/Trondheim/detailed_statistics.html")
### browseURL(u0) ### to visit the web page 
@

One can read and extract the desired data table 
(the second one at the URL) using the **XML** package with the 
\texttt{readHTMLTable()} function. 
However, it still need some pre-processing.

We do not consider it and just read the web page as a text
and play with the text and its structure directly. 
<<read,eval=FALSE>>=
d0 <- readLines(u0) ### read it as text (Done at 26 September 2016)
@ 
<<echo=FALSE,results="hide">>=
load("TRD20160926weather.RData")
@ 

First, we have to find the index for each table line 
and consider only those for the main table:
<<index>>=
i <- grep("<tr>", d0) ### index for each table line
i <- i[i>grep("<tbody>", d0)[2]] ### select those for the second table
@

The desired data we would like to analyse is the 
minimum and maximum temperature. 
Commands to extract and pre-process these data
<<data>>=
if (Sys.getlocale("LC_TIME")!="C")
    Sys.setlocale("LC_TIME", "C")
dates <- as.Date(d0[i+1], format="      <th>%b %d, %Y</th>")
tmed <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+4])))
(n <- length(dates)) ### it is daily over last 13 months
@

Visualize it with the following commands
<<visualize, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
pd <- pretty(c(dates, max(dates+30)), n=13)
par(mfrow=c(1,1), mar=c(3,3,0.5,2), mgp=c(2,.7,0), las=2, xaxs="i")
plot(dates, tmed, type="l", lwd=2,
     axes=FALSE, xlab="day", ylab="Temperature")
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))
box()
axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, TRUE))
@

\section{Model fitting}

\paragraph{Mesh}
in 1d it is a matter of chosing a set of knots, 
the order of the basis functions and the boundary. 
Choosing first order basis function and Neumann boundary.

<<mesh>>=
coo <- as.numeric(dates-min(dates)) ## have numeric temporal coordinates
mesh <- inla.mesh.1d(loc=seq(min(coo), max(coo), by=7), ## knots (7 days)
                 boundary=c("neumann", "neumann"), ## boundaries 
                 degree=2)  ### basis function degree
@

\paragraph{Projector matrix} is the matrix built to project 
 the process at the mesh nodes to the locations 

<<projector, pars=list(mar=c(0,0,1,0)), fig.width=8, fig.height=2, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
A <- inla.spde.make.A( ## projector creator
    mesh=mesh, ## provide the mesh
    loc=coo) ### locations where to project the field
dim(A) ## an "n" by "m" projector matrix
summary(rowSums(A)) ### each line sums up to one
summary(colSums(A)) ### "how many" observations per knot
@

\paragraph{Build the SPDE model} on the mesh, choosing the 
smoothness ($\alpha$), to define the precision structure, 
and priors for the hyperparameters. We just set $\alpha=2$ 
and keep the default priors.

<<SPDE model>>=
spde <- inla.spde2.pcmatern( ## model for the precision 
    mesh=mesh, ## mesh supplied
    alpha=2, ## smoothness parameter
    prior.range = c(1, 0.01), ## P(range < 1) = 0.01
    prior.sigma = c(1, 0.5)) ## P(sigma > 1) = 0.5
@

\paragraph{Create a data stack} in order to organize the data. 
This is a way to allow models with complex 
linear predictors. 
In our case, we have a SPDE 
model defined on $m$ nodes. 
It must be combined 
with the covariate (and the intercept) 
effect at $n$ locations. 
We do it using different projector matrices. 

<<datastack>>=
stk.e <- inla.stack( ## stack creator
  data=list(y=tmed),  ## response
  effects=list(## two elements:
    data.frame(b0=rep(1, n)), ## regressor part
    i=1:spde$n.spde),  ## RF index
  A=list(## projector list of each effect
    1, ## for the covariates
    A), ## for the RF
  tag="est") ## tag
@

\paragraph{Fit} the posterior marginal distributions
  for all model parameters, 
  syplying the model formula, data and some additional controls 
  to the main function in the \textbf{\textsf{INLA}} package

<<fitting>>=
formula <- y ~ 0 + b0 + ## fixed part
  f(i, model=spde) ## RF term
res <- inla( ## main function in INLA package
  formula, ## model formula
  data=inla.stack.data(stk.e), ## dataset
  control.predictor=list( ## inform projector needed in SPDE models
    A = inla.stack.A(stk.e), compute=TRUE)) ## projector from the stack data
@

\section{Posterior marginal distributions - PMDs}

Summary of the regression coefficients PMDs

<<fixed>>=
round(res$summary.fixed, 4) 
@

The PMDs summary for the Gaussian likelihood precision and the 
two RF parameters 
<<hypeparsummary>>=
round(res$summary.hyperpar, 4)
@

We have to transform the likelihood precision PMD to have 
the variance PMD. It can be done by 

<<nugget>>=
m.prec <- res$marginals.hyperpar$"Precision for the Gaussian observations" ## the marginal
post.s2e <- inla.tmarginal(## function to compute a tranformation 
  function(x) sqrt(1/x), ## inverse transformation and square root
  m.prec) ## marginal to be applied
@

The PMDs for likelihood standard deviation and 
the RF parameters can be visualized by 

<<parameters, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
par(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0))
plot(post.s2e, type="l", ylab="Density", 
     xlab=expression(sigma[e]^2))
plot(res$marginals.hyperpar[[2]], type="l",
     xlab="range", ylab="Density")
plot(res$marginals.hyperpar[[3]], ty="l", 
     xlab=expression(sigma[s]), yla="Density")
@

\section{Predicted}

Visualize it with the commands bellow

<<predicted, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
par(mfrow=c(1,1), mar=c(3,3,0.3,2), mgp=c(2,0.5,0), las=2, xaxs="i")
id <- inla.stack.index(stk.e, tag="est")$data
plot(dates, tmed, type="l", axes=FALSE, ylab="Temperature", lwd=2)
for (j in 3:5)
  lines(dates, res$summary.fitted.values[id, j], lty=2, lwd=2)
box(); axis(2, 3*(-8:9)); axis(4, 3*(-8:9))
axis(1, pd, months(pd, T))
abline(h=0)
abline(h=3*(-8:9), v=pd, lty=3, col=gray(.5))
@

\section{Just a look to the rest of the data}

Pre-processing the maximum, minimum and normal temperature, 
the precipitation, and the average and maximum wind:
<<other>>=
tmax <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+2])))
tmin <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+3])))
tnormal <- as.numeric(gsub("<td>", "", gsub("..</td>", "", d0[i+5])))
prec <- as.numeric(gsub("<td>", "", gsub("mm</td>", "", d0[i+6])))
wind <- as.numeric(gsub("<td>", "", gsub("m/s</td>", "", d0[i+10])))
wmax <- as.numeric(gsub("<td>", "", gsub("m/s</td>", "", d0[i+9])))
@

Visualize it
<<visualize3, fig.width=9, fig.height=6, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
par(mfrow=c(3,1), mar=c(0.1,3,0.1,2), mgp=c(2,.7,0), las=2, xaxs="i")
plot(dates, tmed, type="l", ylim=range(tmin, tmax, na.rm=TRUE), 
     axes=FALSE, xlab="", ylab="Temperature", col="green")
lines(dates, tmin, col="blue")
lines(dates, tmax, col="red")
lines(dates, tnormal)
legend(dates[which.min(tmin)], par()$usr[4], c("normal", "max.", "aver.", "min."), 
      col=1:4, lty=1, ncol=2, xjust=0.5, bty="n")
abline(h=5*(-5:6), v=pd, lty=3, col=gray(.5))
box(); axis(2, 5*(-5:6)); axis(4, 5*(-5:6))

plot(dates, prec, type="l", axes=FALSE, xlab="")
box(); axis(2); axis(4)
abline(v=pd, h=10*(1:4), lty=3, col=gray(0.5))

par(mar=c(3, 3, 0.1, 2), new=FALSE)
plot(dates, wind, type="l", axes=FALSE, xlab="",
     ylim=range(wind, wmax, na.rm=TRUE))
lines(dates, wmax, col=2)
box(); axis(2); axis(4)
abline(v=pd, h=5*(1:3), lty=3, col=gray(0.5))
axis(1, pd, months(pd, TRUE))
@

We can have a look at the difference between the daily mean temperature and 
the normal temperature. 
The current normal is the average over the period from 1961 to 1990.

<<anomalia, fig.width=9, fig.height=3, out.width="0.99\\textwidth", fig.align="center", fig.pos="h">>=
par(mar=c(3, 3, 0.1, 2), mgp=c(2,0.7,0), las=2, xaxs="i")
plot(dates, tmed-tnormal, type="l", axes=FALSE, 
     xlab="", ylab="Deviation from normal temperature")
box(); axis(2); axis(4)
abline(h=5*(-2:2), v=pd, lty=2, col=gray(0.5))
axis(1, pd, months(pd, TRUE))
@

\end{document}

---
title: "The inla.posterior.sample.eval-function"
author: "Haavard Rue"
date: "KAUST, Aug 2022"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{The inla.posterior.sample.eval-function}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
library(knitr)
set.seed(123)
knitr::opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/eval/'
               , fig.pos='H'
               , background='#ffffff'
               , results='hold'
               , eval=TRUE)
```

## Introduction

This short note add some more explanation to the
`inla.posterior.sample.eval()`-function, as its a constant source of
confusing (which is understandable). The purpose of this function is
to ease function-evaluations of samples from the fitted model. 

## Simple example
As often, its easier to work with an example.
```{r}
n <- 100
x <- rnorm(n)
eta <- 1 + x
y <- rnorm(n, mean = eta, sd = 0.1)
r <- inla(y ~ 1 + x,
	data = data.frame(y,x),
    control.compute = list(config=TRUE))
```
where the *config* argument is required. We can now generate samples
from the fitted model
```{r}
samples <- inla.posterior.sample(10000, r)
```
Here, *samples* contains the samples, but in long vectors with
additional information about where to find what which makes it complicated. 

The information available is what is in the output
```{r}
summary(r)
```
so its **x**, **(Intercept)** and the **Precision for the Gaussian
observations**, plus the linear predictor(s).
 
The `...eval()` function simplifies the evaluation of a function over
(joint) samples, by assigning sample-values to each variable. 
To extract samples of **x**, which is here the regression coefficient
for the covariates $x$ (this is confusing, I know), then we can do
```{r}
fun1 <- function() return (x)
```

We can now evaluate `fun1` for each sample, using the
`...eval()`-function, like
```{r}
eval.fun1 <- inla.posterior.sample.eval(fun1, samples)
str(eval.fun1)
```
since **x** is automatically assigned the sample value before `fun1`
is called. This happens for each sample. 

We can compare with the the INLA-output
```{r}
hist(eval.fun1[1,], prob=TRUE, n=300)
lines(inla.smarginal(r$marginals.fixed$x), lwd=3)
```
and the results seems to agree. 

Also the variable **(Intercept)** is automatically created, but since
this form is awkward to use in **R**, it is equivalent to
**Intercept**. We can for example sample the linear predictor with
```{r}
fun2 <- function(x.cov) return (Intercept + x * x.cov)
```
Here, we need to pass the covariates $x$ (which is *not* the same as
**x**) separately as a named argument,
```{r}
eval.fun2 <- inla.posterior.sample.eval(fun2, samples, x.cov = x)
```
and we plot the regression-line
```{r}
plot(x, apply(eval.fun2, 1, mean))
abline(a=1, b=1) # this is the true curve
```
The predictor is also available automatically as **Predictor**, so
```{r}
fun3 <- function(x.cov) return (Predictor - (Intercept + x * x.cov))
eval.fun3 <- inla.posterior.sample.eval(fun3, samples, x.cov = x)
summary(eval.fun3[1,])
```
as it should. 

## Samples of hyper-parameters

It gets a little more involved with the hyper-parameters. In the
example above, there is only one, the precision for the observational
noise. We can use this to sample new data from the fitted model.
Hyper-parameters are automatically assigned values in the vector
**theta**.
```{r}
fun4 <- function() return (theta)
eval.fun4 <- inla.posterior.sample.eval(fun4, samples)
table(eval.fun4[1, ])
```

A feature here, is that only the integration points for **theta** are
used, hence samples of **theta** are discrete with finite number of
values. (To only sample the hyper-parameters, please use function
`inla.hyperpar.sample()`.) Note that **theta**, by default, is the
hyper-parameters in the user-scale (like precision, correlation, etc).
If argument `intern=TRUE` is used in the
`inla.posterior.sample()`-function, then they will appear in the
internal-scale (like `log(precision)`, etc).

We can generate a new dataset from the fitted model, with
```{r}
samples <- inla.posterior.sample(1, r)
fun4 <- function() {
	n <- length(Predictor)
	return (Predictor + rnorm(n, sd = sqrt(1/theta)))
}
eval.fun4 <- inla.posterior.sample.eval(fun4, samples)
plot(x, eval.fun4[,1])
```

With more than one hyper-parameter, then `theta` is vector, and the
order of hyper-parameters is the same as is stored in the
result-object. The user has to organise this manually.
With
```{r}
r <- inla(y ~ 1 + x,
	family = "sn",
	data = data.frame(y,x),
    control.compute = list(config=TRUE))
```
then 
```{r}
rownames(r$summary.hyperpar)
```
so that `theta[1]` is the precision while `theta[2]` is the skewness. 

## Example: Predictor with and without random effects

Here is an example that pops up from time to time, using the tools
above. We are interested in comparing the linear predictor with and
without some random effects. The below example is artificial but shows
how this works. 

First we simulate some data
```{r}
m <- 100
n <- m^2
## fixed effects
x <- rnorm(n)
xx <- rnorm(n)
## random effects
v <- rnorm(m, sd=0.2)
v.idx <- rep(1:m, each = m)
eta <- 1 + 0.2 * (x + xx) + v
y <- rpois(n, exp(eta))
```
and then fit the model
```{r}
r <- inla(y ~ 1 + x + xx + f(v.idx, model = "iid"),
          data = data.frame(y, x, xx, v.idx),
		  family = "poisson",
          control.compute = list(config = TRUE))
samples <- inla.posterior.sample(10000, r)
```
Now we want to compare the linear predictor with and without the
`f(v.idx, model = "iid")` term. The easy way out, is to use the
predictor and then subtract the iid-term, instead of building it
up manually.
```{r}	
fun5 <- function(v.index) {
    return (c(Predictor, Predictor - v.idx[v.index]))
}
eval.fun5 <- inla.posterior.sample.eval(fun5, samples, v.index=v.idx)
```
And we can compare with and without for some components
```{r}
i <- 2
hist(eval.fun5[i,], prob=TRUE, n=300)
lines(density(eval.fun5[n + i,]), col="blue", lwd=3)
```


## Predictor $A$-matrix (experimental-mode only)

For some models, especially models using the SPDE, then a projector
matrix is used, so we need the A-matrix for the predictor. Often this
looks like
```{r eval=FALSE}
r <- inla(...., control.predictor = list(A=inla.stack.A(...)))
```

For these models, then the observations depend on $\eta^*$, where
$\eta^* = A \eta$, and $\eta$ is defined with the formula. In these
cases, then `Predictor` is $\eta$ and `APredictor` is $\eta^*$.
Moreover, the $A$ matrix is available as `pA` in the
`...eval()`-function.

Here is the same example above, with a random $A$-matrix showing how
to use this.
```{r}
n <- 100
m <- 25
## fixed effects
x <- rnorm(n)
xx <- rnorm(n)
## random effects
v <- rnorm(m, sd=0.2)
v.idx <- rep(1:m, each = n %/% m)
eta <- 1 + 0.2 * (x + xx) + v
A <- matrix(rnorm(n^2, sd=sqrt(1/n)), n, n)
eta.star <- A %*% eta
y <- rpois(n, exp(eta.star))

r <- inla(y ~ 1 + x + xx + f(v.idx, model = "iid"),
          data = data.frame(y, x, xx, v.idx),
		  family = "poisson",
	      inla.mode = "experimental",
		  control.predictor = list(A=A),
          control.compute = list(config = TRUE))
samples <- inla.posterior.sample(10000, r)
```
We will compare the same change, with and without the iid-term.
```{r}
fun6 <- function(v.index) {
    return (c(APredictor, as.numeric(pA %*% (Predictor - v.idx[v.index]))))
}
eval.fun6 <- inla.posterior.sample.eval(fun6, samples, v.index=v.idx)
i <- 2
hist(eval.fun6[i,], prob=TRUE, n=300)
lines(density(eval.fun6[n + i,]), col="blue", lwd=3)
```

---
title: "Avoiding trouble with factors in inla.stack"
author: "Elias T. Krainski `(elias@r-inla.org)`"
date: "Aug 7th 2017"
output: rmarkdown::pdf_document
bibliography: spde-tutorial.bib
vignette: >
  %\VignetteIndexEntry{Avoiding trouble with factors in inla.stack}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo=TRUE, message=FALSE, warning=FALSE)
knitr::opts_chunk$set(fig.path="figures/stack-with-factors/")
set.seed(123)
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
```

## Introduction

This short note describe a way to work with factor covariates
when working with _inla.stack()_. 
You can skip the next section if you already know how to work with 
factors in functions such as _lm()_ or _glm()_.

### Working with factors in **R**

Let the simple linear model 
\[ y_{ijk} = \mu_0 + a_i + b_j + f_k + \beta x_{ijk} + e_{ijk} \]
and the following data from it 
```{r dat}
  dat <- expand.grid(a=0:1, b=0:1, d=0:2) ### three discrete
  dat$x <- runif(12) ### one continuous covariate
  dat$y <- 3*dat$a - dat$b + (dat$d-1)*2 + dat$x + rnorm(12,0,0.1)
```

A factor with only two levels can be encoded 
as numeric $0$ or $1$ or as _factor_. 
A factor with more than two levels always 
have to be encoded as factors. 
So we must encode $d$ as factor 
```{r factor}
  dat$d <- factor(dat$d) 
```

The model can be fitted as
```{r, fit}
	coef(lm(y ~ 1 + a + b + d + x, dat))
```
where we explicited each model term. 

Let us pay attention for the fact that the 
default option for contrasts is the **treatment**. 
In this case a reference level from each covariate 
coded as factor is droped, the first level by default. 
In this case the '(Intercept)' is the scenario when 
each numeric covariate is zero and the factor covariates 
are at its reference level. 
The coefficient associated to a numeric covariate 
measures the effect of changing one unity in this covariate. 
For the covariates encoded as factors, there is one 
coefficient for each not droped level and it measures the 
effect of changing from the reference level to this level. 

The intercept is considered by default 
no needing to be explicited. 
```{r, fit2}
	coef(lm(y ~ a + b + d + x, dat))
```
It can be removed by adding '-1' in any part of the 
right side of the formula 
```{r, fit3}
	coef(lm(y ~ -1 + a + b + d + x, dat))
```
or writing a literal formula explicty saying 'no intercept' 
inserting '0' in any part of the right side of it 
```{r, fit4}
	coef(lm(y ~ 0 + a + b + d + x, dat))
```
Notice that the '(Intercept)' from before is now 'd0' 
because it is the scenario for the first level in 'd' 
and when all the other terms (all numeric) as zero. 
When there is no intercept this happens to be the case 
for the first factor in the formula. 

When the '(Intercept)' is in the model, changing the order 
of the terms in the formula does effect what is being 
fitted, only the order 
```{r, orderi}
	coef(lm(y ~ b + a + d + x, dat))
	coef(lm(y ~ d + a + b + x, dat))
```

The covariates with only two levels encoded as "0" or "1" 
can be encoded as factor assuming any two labels 
```{r, factor2}
dat$a <- factor(dat$a, levels=0:1, labels=c('1st', '2nd'))
dat$b <- factor(dat$b, levels=1:0, labels=c('2nd', '1st')) ## OBS: reference level changed
```
and we will have the same coefficients as when not having 
it as factors only when '0' is the reference level
```{r reg3f}
coef(lm(y ~ a + b + d + x, dat))
```

When the intercept is not in the formula it will be 
the reference level of the first factor and 
the order matters
```{r orderf}
	coef(lm(y ~ 0 + a + b + d + x, dat))
	coef(lm(y ~ 0 + b + a + d + x, dat))
	coef(lm(y ~ 0 + d + a + b + x, dat))
```

It is important to notice that 
the way one codes de model matters. 
We showed the case under the **contrast parametrization**. 
Other contrast options are available in **R** 
and one can see these in _help(contrast)_. 

Another important point is to notice that it is not a feature 
of the _lm()_ or _glm()_ function. 
The acual model matrix being prepared using the _model.matrix()_ 
function. 
This fuction is used internally to create tge dummie variables. 
Everyone can use it directly just suplying the formula and data, 
```{r mm, results='hide'}
model.matrix(~a+b+d+x, dat) 
```
and you can see what happens with each above cases.

Now we prepare ourselves to understand how to work with 
factors in _inla.stack()_. 

### Dealing with factors in _inla.stack()_ 

The _inla.stack()_ function helps to organize 
data when the model has components with 
different projection matrices. 
Let us consider the _Tokyo_ data which is 
data about rain each day over two years 
grouped by day of the year. 
```{r tokyo}
data(Tokyo)
str(Tokyo)
``` 
$y$ is $0$, $1$ or $2$, 
$n$ is $1$ (February 29) or $2$ 
and $time$ is $1$, $2$, ..., $366$. 

Let us have a set of knots over time in order to build 
a model, see @lindgrenR:2015. 

```{r tokyomesh}
knots <- seq(1, 367, length = 25)
mesh <- inla.mesh.1d(knots, interval = c(1, 367), degree = 2, boundary = "cyclic")
spde <- inla.spde2.pcmatern(mesh, 
    prior.sigma=c(1, 0.01), ## P(sigma > 1) = 0.01
    prior.range=c(1, 0.01)) ## P(range < 1) = 0.01
A <- inla.spde.make.A(mesh, loc = Tokyo$time)
time.index <- inla.spde.make.index("time", n.spde = spde$n.spde)
```
Let us add two factor covariates to Tokyo data and a numeric one. 
```{r addf}
Tokyo$a <- factor(rbinom(366, 1, 0.5))
Tokyo$b <- factor(rbinom(366, 2, 0.5))
Tokyo$x <- runif(366)
```

When working with factor covariates it is better to 
to build the design matrix and supply it to 
_inla.stack()_. 
We can include the other covariates as well.
```{r ab}
abx <- model.matrix(~a+b+x, Tokyo)[, -1]
```
The automatic intercept at the first column was droped. 
When supplying it in _inla.stack()_ we will join an explict 
intercept as we usually do when working with SPDE models. 

```{r tokyostack}
stack <- inla.stack(
  data = list(y = Tokyo$y, link = 1, Ntrials = Tokyo$n),
  A = list(A, 1),
  effects = list(time.index, data.frame(mu0=1, abx)),
  tag = "est")
formula <- y ~ 0 + mu0 + a1 + b1 + b2 + x + f(time, model = spde)
data <- inla.stack.data(stack)
result <- inla(formula, family = "binomial", 
              data = data, 
              Ntrials = data$Ntrials,
              control.predictor = list(
                A = inla.stack.A(stack), 
                link = data$link, 
                compute = TRUE))
result$summary.fixed[, 1:5]
```

### When there is a prediction scenario

Let us build a prediction scenario 
```{r preds}
pred.sc <- expand.grid(a1=0:1, b1=0:1, b2=0, x=c(0.5))
pred.sc
```

For the random effect, over time, we do need 
to build a projection as well. 
```{r predt}
A.pred <- inla.spde.make.A(mesh, loc=rep(180, nrow(pred.sc)))
```

This scenario can be supplied in a new data stack as 
```{r stackpred}
stack.pred <- inla.stack(
  data = list(y = NA, link = 1, Ntrials = 2),
  A = list(A.pred, 1),
  effects = list(time.index, data.frame(mu0=1, pred.sc)),
  tag = "pred")
stack.full <- inla.stack(stack, stack.pred)
data <- inla.stack.data(stack.full)
result <- inla(formula, family = "binomial", 
               data = data, 
               Ntrials = data$Ntrials,
               control.predictor = list(
                 A = inla.stack.A(stack.full), 
                 link = data$link, 
                 compute = TRUE),
               control.mode=list(theta=result$mode$theta, 
                                 restart=FALSE))
```

Getting the predictions
```{r predicts}
idx.pred <- inla.stack.index(stack.full, tag='pred')$data
result$summary.fitted.val[idx.pred, 1:5]
```

## References

---
title: "Blended GEV: a tutorial using R-INLA"
subtitle: "Version 1.2"
author: \normalfont{Daniela Castro-Camilo {\small{$\bullet$}} daniela.castrocamilo@glasgow.ac.uk}
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Blended GEV: a tutorial using R-INLA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \usepackage{xcolor}
---
\def\F{\text{F}}
\def\G{\text{G}}
\def\H{\text{H}}
```{r setup, include=FALSE}
set.seed(123)
library(knitr)
knitr::opts_chunk$set(echo = TRUE, results = 'show', 
	fig.keep = 'high', fig.align = 'center', fig.pos = 'h')
knitr::opts_chunk$set(fig.path="figures/bgev/")

library(INLA)
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")
inla.setOption(inla.mode="experimental")
if (file.exists("myinit.R")) source("myinit.R")
```


```{r, echo=F, message=F, warning=FALSE}
colFmt = function(x,color){
  outputFormat = opts_knit$get("rmarkdown.pandoc.to")
  if(outputFormat == 'latex')
    paste("\\textcolor[HTML]{",color,"}{",x,"}",sep="")
  else if(outputFormat == 'html')
    paste("<font color='",color,"'>",x,"</font>",sep="")
  else
    x
}

library(INLA)
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}
```

# A reparametrisation of the GEV

The blended Generalised extreme value (bGEV) model is an alternative
to the usual GEV distribution when the tail parameter $\xi$ is
positive, and it is designed to tackle the \emph{artificial boundary
restrictions} inherited by the GEV. By artificial boundary
restrictions, we mean the following: in practical applications, we
assume that the GEV is a reasonable approximation for the distribution
of maxima over blocks, and we fit it accordingly. This implies that
GEV properties, such as finite lower endpoint in the case $\xi>0$, are
inherited by the original maxima distribution, which might not be
bounded-supported. This is particularly problematic in the presence of
covariates.


Before defining the bGEV distribution, we need to introduce an
alternative parametrisation for the GEV. The GEV distribution
parametrised in terms of the \textbf{location}
$q_\alpha\in\mathbb{R}$, the \textbf{spread} $s_\beta\in(0,\infty)$
and the \textbf{tail} parameter $\xi\in\mathbb{R}$ has the form

$$ F(y\mid q_\alpha,s_\beta,\xi) =
	\exp\left\{-\left[\left(\frac{y-q_\alpha}{s_\beta(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})^{-1}} +
	\ell_{a,\xi}\right)\right]_+^{-1/\xi}\right\},$$ where
	$a_+=\max(a,0)$ and for any $a>0$, $\ell_{a,\xi} = \{-\log
	a\}^{-\xi}$. Note that the case $\xi=0$ simplifies to $$F(y\mid
	q_\alpha,s_\beta) = \exp\left\{-\exp\left[-\left(\frac{y -
	q_\alpha}{s_\beta(\ell_{1-\beta/2} - \ell_{\beta/2})^{-1}} -
	\ell_\alpha\right)\right]\right\}, 
$$

with $\ell_a = \log\{-\log a\}.$ There is a one-to-one mapping between
$(q_\alpha, s_\beta, \xi)$ and the usual location-scale-shape GEV
parameters, $(\mu,\sigma,\xi)$ (see, e.g., `r colFmt("Coles, 2001,
Chapter 3",'0352fc')`). For the case $\xi\neq 0$, the mapping is given
by

$$
	\mu=q_\alpha-\frac{s_\beta(\ell_{\alpha,\xi}-1)}{\xi(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})}, \qquad \sigma =
	\frac{s_\beta}{(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})},\qquad\xi = \xi. 
$$

The case $\xi=0$ is interpreted as the limit when $\xi\to0$, i.e.,
$$\mu = q_\alpha + \frac{s_\beta\ell_\alpha}{\ell_{\beta/2} -
\ell_{1-/\beta/2}},\qquad \sigma = \frac{s_\beta}{\ell_{\beta/2} -
\ell_{1-/\beta/2}}.$$

This reparametrisation is proposed to provide a more meaningful
interpretation of the parameters. In statistics, the location-scale
parametrisation is quite popular as it relates to the mean and the
standard deviation of the distribution. In skewed distributions such
as the GEV, the mean is no longer a reasonable proxy for the location
of the distribution. Moreover, the mean and variance of the GEV are
only defined when $\xi<1$ and $\xi<0.5$, respectively. This effect of
the tail parameter over the mean and variance does not allow us to
interpret the location-scale (and tail) GEV parametrisation as we do
in other models. This problem is particularly troublesome for the case
where the parameters vary according to a set of covariates. Assigning
sensible priors to the GEV distribution with the usual parametrisation
is also tricky when the mean and variance are not defined.


# The blended GEV model

The bGEV distribution is defined as

\begin{equation}\label{eq:bGEV}
	\H(x \mid \theta) = \F(x \mid q_{\alpha_{\F}}, s_{\beta_{\F}},
	\xi)^{p(x \mid s_{p_1}, s_{p_2}, a, b)} 
	\G(x \mid q_{\alpha_{\G}}, s_{\beta_{\G}})^{1-p(x \mid s_{p_1}, s_{p_2}, a, b)},
\end{equation}

where $\theta = (q_{\alpha_{\F}}, s_{\beta_{\F}}, \xi,
q_{\alpha_{\G}}, s_{\beta_{\G}}, s_{p_1}, s_{p_2}, a, b)$, $\F$ is the
Frechét (or type II GEV) distribution with location parameter
$q_{\alpha_{\F}}$, spread parameter $s_{\beta_{\F}}$, and
\textbf{tail} parameter $\xi$, and $\G$ is the Gumbel (or type I GEV)
distribution with location parameter $q_{\alpha_{\G}}$ and spread
parameter $s_{\beta_{\G}}$. The function $p$ is a weight function
defined as the cumulative distribution function of a Beta distribution
with shape parameters $s_{p_1}>1$ and $s_{p_2}>1$, evaluated in the
point $(x - a)/(b - a)$, i.e.,

\begin{equation}\label{eq:weightfunction}
	p(x \mid s_{p_1},s_{p_2}, a, b) = \text{Pr}\left(Y\leq \frac{x-a}{b-a} \mid s_{p_1}, s_{p_2}\right),
\end{equation}

where $Y$ follows a Beta distribution with shape parameters
$s_{p_1}>1$ and $s_{p_2}>1$. The Beta weight controls the way the
distributions $\F$ (Frechét) and $\G$ (Gumbel) influence the model
$\H$. The lower and upper bounds of the weight function ($a$ and $b$,
respectively) define the \emph{mixing area}, i.e., where $F$ and $G$
are merged (see Figure \ref{fig:distributionH.pdf}). Here, we choose
them as quantiles of the Frechét distribution, i.e., $a=F^{-1}(p_a)$
and $b=F^{-1}(p_b)$, with $0<p_a,p_b<1$. Below, $p_a$ and $p_b$ will
be refered as the \emph{mixing area quantiles}. The current INLA
implementation assumes that $s_{p_1}=s_{p_2}=5$.


```{r pressure, echo=FALSE, fig.cap="\\label{fig:distributionH.pdf} bGEV distribution (H, black) constructed from distributions F (Frechét, red), G (Gumbel, green) and Beta weight function $p$ (purple). The shaded area is the mixing area, where F and G are merged.", out.width = '50%'}
knitr::include_graphics("fig_distribution1.png")
```

In the following sections, we will learn how to fit the bGEV using
R-INLA using three simulated examples with increasing level of
complexity. <!-- Additionally, we also provide the data application in
the bGEV article (`r colFmt("Castro-Camilo et al., 2020",'0352fc')`).
-->

# Simulated example 1

To get familiar with the bGEV R-INLA implementation, we consider a
simple model where the linear predictor is linked to the
$\alpha-$quantile, $q_\alpha$. The model we want to fit is

\begin{equation}\label{eq:sim.model1}
	q_\alpha = \eta(\texttt{x}) =  1 + 0.4\texttt{x}
\end{equation}

## Data simulation{#sec:datasim1}

We start by generating $n=1000$ samples from \eqref{eq:sim.model1}

```{r}
n = 1000
x = rnorm(n, sd=0.5) # we generate values for x from a N(0,0.5^2) dist.
eta.x = 1 + 0.4*x
```

The spread and tail parameters are assumed to be covariate-free and
unknown and are treated as hyperparameters within the INLA framework.
We assume that the true spread and tail parameters are 0.3 and 0.1,
respectively.

```{r}
spread = 0.3
tail = 0.1
```

To generate the GEV samples, we need to define the probabilities
$\alpha$ and $\beta$ that define the location ($q_\alpha$) and spread
($s_\beta$) parameters, respectively. In our case, they will be fixed
to $\alpha=0.5$ (the median) and $\beta=0.25$: <!-- In our case, they
will be fixed to $\alpha=0.5$ (the median) and $\beta=0.5$ (so then
$s_\beta$ is the interquartile range): -->

```{r}
p.alpha = 0.5
p.beta = 0.25
```
 
Now we are ready to generate the samples. We use the function
[$\texttt{giveme.gev.par}$](#sec:tools) (See [Section 5](#sec:tools))
to obtain the usual GEV parameters ($\mu,\sigma,\xi$) and plug them
into the $\texttt{evd::rgev}$ function to generate the samples

```{r}
par = giveme.gev.par(q = eta.x, sbeta = spread, alpha = p.alpha, beta = p.beta, 
                     xi = tail)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma, shape = par$xi)
```

## Prior specification{#sec:prior1}

The default prior distribution for the spread $s_\beta$ is a Gamma
with shape and rate parameters equal to 3 (note that a log-scale is
used below). For the tail parameter we consider a PC prior approach
with parameters $\lambda = 7$, $\texttt{low} = 0$ and
$\texttt{high}=0.5$. For more details on the PC prior for the tail GEV
parameter see [Section 4.2](#sec:pcpriortail).

For the sake of illustration, we here define the priors for all the
parameters involved. The priors for the spread is

```{r}
hyper.spread = list(initial = 1,
                    fixed=FALSE,
                    prior = "loggamma",
                    param = c(3, 3))
```

The prior for the tail parameter requires a bit more explanation. For
computational reasons, R-INLA uses an internal parametrisation of the
tail parameter, which is unbounded. The hyperparameter specification
is defined for this internal parameter instead of the usual one, so in
order to know how to specify a prior for the tail parameter, we need
to understand how both parametrisations are connected.

The map $\phi:[0,\infty)\to\mathbb{R}$ specifies the link between the
usual and the internal parametrisations and it is defined as
$$\phi(\xi_{\text{int}}) = \texttt{low} +
(\texttt{high}-\texttt{low})\frac{\exp(\xi_{\text{int}})}{1+\exp(\xi_{\text{int}})}:=
\xi,$$ where $\xi\in[0,\infty)$ is the usual tail parameter and
$\xi_{\text{int}}$ refers to the unbounded internal tail parameter.
The interval $(\texttt{low},\texttt{high})$ constraints the possible
values for $\xi$. The map and its inverse are defined in the function
[$\texttt{map.tail}$](#sec:tools) (See [Section 5](#sec:tools)).

In the internal parametrisation, the default initial value is -4 with
$(\texttt{low},\texttt{high}) = (0,0.5)$, which correspond to
$\xi\approx0.04$. If we want to provide an initial value of $\xi=0.1$,
then we can do

```{r}
tail.interval = c(0, 0.5)
tail.intern = map.tail(tail, tail.interval, inverse=TRUE)
```

We kept $(\texttt{low},\texttt{high}) = (0,0.5)$ to ensure the existence of second moments\footnote{Recall that the variance for the GEV is defined for $\xi<0.5$.}.

The prior for the tail parameter is defined as

```{r}
hyper.tail = list(initial = tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= FALSE)

```

\textbf{Note:} if we have reasons to believe that $\xi=0$ (Gumbel
case), then a good initial value is $-\infty$ (in the internal
parametrisation). We can generalise the prior specification for the
tail to allow $\xi>0$ or $\xi=0$ as follows

```{r}
hyper.tail = list(initial = if (tail == 0.0) -Inf else tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= if (tail == 0.0) TRUE else FALSE)

```

Therefore, the (default) hyperparameter specification for the bGEV model is

```{r}

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail)
```

## Control variables {#sec:control1}

As part of the $\texttt{control.family}$ argument in R-INLA, the
argument $\texttt{control.bgev}$ allows us to include additional bGEV
parameters. Specifically, the probabilities $\alpha$ and $\beta$, the
mixing area quantiles $p_a$ and $p_b$, and the Beta weight function
parameters, which we know are equal and fixed to 5 (for now, this
cannot be changed).

```{r}
control.bgev = list(q.location = p.alpha,
                    q.spread = p.beta,
                    # quantile levels for the mixing part
                    q.mix= c(0.05, 0.20), 
                    # the Beta(s1, s2) mixing distribution parameters. 
                    # Hard-coded for the moment: s1=s2=5
                    beta.ab = 5)
```

## INLA fit{#sec:fit1}

The INLA formula for the bGEV model uses the function
[$\texttt{inla.mdata}$](#sec:inlamdata) to allow the inclusion of
simple linear models in the spread and the tail parameters (for
details on the $\texttt{inla.mdata}$ function see [Section
4.3](#sec:inlamdata)). A null matrix can be used to indicate that
these parameters are covariate-free (as it is the case in this
example).

```{r}
null.matrix = matrix(nrow = n, ncol= 0)
spread.x = null.matrix
tail.x = null.matrix
```

\textbf{Note:} matrices for the spread and tail covariates (here
spread.x and tail.x) should always be defined and passed to the R-INLA
formula, even if they are empty.

Then, the INLA data and formula can be defined as
```{r}
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x
```

\textbf{Note:} $\texttt{data.bgev}$ only contains three columns, as
null matrices cannot be passed to data frames in R. As
$\texttt{spread.x}$ and $\texttt{tail.x}$ are not defined in
$\texttt{data.bgev}$, INLA will search for these variables in the
global R environment. Therefore, $\texttt{spread.x}$ and
$\texttt{tail.x}$ should be defined as null matrices in the global
environment (as we do here). We write $\texttt{data.bgev}$ this way
for consistency with the following examples.

Finally, we fit the model

```{r, message=F, eval = T, warning=FALSE}
r1 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE, safe=TRUE)
```

A summary of the fitted fixed effects and hyperparameters can be
obtained as follows

```{r}
round(r1$summary.fixed,4)
round(r1$summary.hyperpar,4)
```



<!-- ```{r, fig.width=4.5, fig.height=3.5, fig.cap="\\label{fig:pit_r1.pdf} The probability integral transofrm (PIT) is a \\textit{leave-one-out} predictive measure of fit. It is computed as the probability of a new response being less than the observed response using a model based on the rest of the data. If the model assumptions are correct, we would expect the PIT values to be uniformly distributed."} -->
<!-- round(r1$summary.fixed,4) -->
<!-- round(r1$summary.hyperpar,4) -->
<!-- hist(r1$cpo$pit, freq = F, col = 'lightblue', border = 'white', -->
<!--      main = 'PITs for model r1', xlab = '') -->
<!-- abline(h = 1, lty = 2, col = 2) -->
<!-- ``` -->

# Simulated example 2

Although R-INLA does not allow more than one linear predictor, the
bGEV implementation does allow for simpler regression models on the
spread and tail parameters. We then extend model \eqref{eq:sim.model1}
as follows

\begin{align}\label{eq:sim.model2}
q_\alpha &= \eta(\texttt{x}_{1}) =  1 + 0.4\texttt{x}_{1}\nonumber\\
s_\beta &= \exp(0.1 + 0.3\texttt{x}_{2})\nonumber\\
\xi &= 0.1 + 0.2\texttt{x}_{3}
\end{align}

<!-- \xi_{\text{int}} &= \phi^{-1}(0.1) + 0.1\texttt{x}_{3}\nonumber\\ -->
<!-- \xi &= \phi(\xi_{\text{int}}) -->
<!-- Note that we provide the model for the usual tail parameter, $\xi$, in terms of the internal tail parameter $\xi_{\text{int}}$. We want the true $\xi$ to be close to 0.1 and define the covariate-dependent model in terms of $\xi_{\text{int}}$ to avoid getting unreasonable values for $\xi$. -->

## Data simulation{#sec:datasim2}

As before, we start by generating $n=1000$ samples from \eqref{eq:sim.model2}

```{r}
n = 1000
x1 = rnorm(n)
eta.x = 1 + 0.4*x1
```

The spread and tail parameter can be simulated as follows

```{r}
x2 = rnorm(n, sd= 0.2)
s.x = exp(0.1 + 0.3*x2)
x3 = runif(n,-0.25,1)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi
```

Note that since $-0.5<\texttt{x}_{3}<2$, we have that $0<\xi<0.5$. As
before, we use the function [$\texttt{giveme.gev.par}$](#sec:tools) to
obtain the usual GEV parameters and plug them into the
$\texttt{evd::rgev}$ function to generate the samples (note that
$\alpha$ and $\beta$ are the same as before)

```{r, eval = T}
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])
```
 
## Prior specification{#sec:prior2}

Additional to what was discussed in [Section 3.2]({#sec:prior1}), we
can also adjust the priors for the regression coefficient of the
covariates for the spread and tail parameters, which we will call
$\beta_1$ and $\beta_2$. Within INLA, $\beta_1$ and $\beta_2$ are
treated as hyperparameters, with default prior given by a zero-mean
Gaussian distribution with precision equal to 300, as specified below.

```{r}
hyper.beta1 = hyper.beta2 = list(prior = "normal",
                                 param = c(0, 300),
                                 initial = 0)
```

In this case, the (default) hyperparameter specification for the bGEV model is

```{r}

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2)
```


## INLA fit{#sec:fit2}

As mentioned in [Section 3.4]({#sec:fit1}), we can use the
[$\texttt{inla.mdata}$](#sec:inlamdata) function to define linear
models for the spread and the tail parameters.

```{r}
spread.x = x2
tail.x = x3
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x
```

Then, the INLA data can be defined as
```{r}
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
```

We fit the model using the same priors and mixing area quantiles
($p_a$, $p_b$) as before.

```{r, message=F, eval = T, warning=FALSE}
r2 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE, safe=TRUE)
```

A summary of the fitted fixed effects and hyperparameters can be
obtained as follows

```{r, eval = T}
round(r2$summary.fixed,4)
round(r2$summary.hyperpar,4)
```


<!-- ```{r, eval = T, fig.width=4.5, fig.height=3.5, fig.cap="\\label{fig:pit_r2.pdf} The probability integral transform for model $\texttt{r2}$."} -->
<!-- round(r2$summary.fixed,4) -->
<!-- round(r2$summary.hyperpar,4) -->
<!-- hist(r2$cpo$pit, freq = F, col = 'lightblue', border = 'white', -->
<!--      main = 'PITs for model r2', xlab = '') -->
<!-- abline(h = 1, lty = 2, col = 2) -->
<!-- ``` -->

# Simulated example 3

The linear predictor can include more complicate structures defined as
functions that depend on a set of covariates. By varying the form of
these functions, we can accommodate a wide range of models, from
standard and hierarchical regression to spatial and spatio-temporal
models (`r colFmt("Rue et al., 2009",'0352fc')`). Here we extend the
model in \eqref{eq:sim.model2} by assuming the following structure

\begin{align}\label{eq:sim.model3}
	q_\alpha &= \eta(\texttt{x}_{1},\texttt{z}_{1},\texttt{z}_{2}) =  1 + 0.4\texttt{x}_{1} + f_1(\texttt{z}_{1}) + f_2(\texttt{z}_{2})\nonumber\\
	s_\beta &= \exp(0.1 + 0.3\texttt{x}_{2} + \texttt{x}_{4})\nonumber\\
	\xi &= 0.1 + 0.2\texttt{x}_{3}
\end{align}

where $f_1$ is a random walk of order 1 and $f_2$ is an autoregressive
process of order 2. Note that we also extended the linear model for
the spread parameter.

## Data simulation{#sec:datasim3}

There are many ways we can simulate dara to fit \eqref{eq:sim.model3}.
One alternative is

```{r, eval = T}
n = 1000
x = rnorm(n)
z1 = seq(0, 6, length.out = n)
z2 = 1:n
p = 2 # AR order
pacf = runif(p)
phi = inla.ar.pacf2phi(pacf)
eta.x = 1 + 0.4*x + sin(z1) + c(scale(arima.sim(n, model = list(ar = phi))))
```

The spread and tail parameter are simulated as before:

```{r}
x2 = rnorm(n, sd = 0.2)
x4 = rnorm(n, sd = 0.2)
s.x = exp(0.1 + 0.3*x2 + x4)
x3 = runif(n,-0.2, 0.2)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi
```

The data are simulated as

```{r, eval = T}
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])
```

## INLA fit{#sec:fit3}

We have two covariates for the spread parameter and one for the tail,
and we can have priors for each of their coefficients. Below,
$\texttt{hyper.beta1}$ and $\texttt{hyper.beta2}$ are the priors for
the coefficients of the covariates in the spread parameters, while
$\texttt{hyper.beta3}$ is the prior for the coefficient of the
covariate in the tail parameter.

```{r}
hyper.beta1 = hyper.beta2 = hyper.beta3 = list(prior = "normal",
                                               param = c(0, 300),
                                               initial = 0)
hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2,
                  beta3 = hyper.beta3)
```

Using the same mixing area quantiles ($q_a$, $q_b$) as before, we only
need to specify the INLA formula, the new data, and run the model.

```{r, message=F, eval = T, warning=FALSE}
spread.x = x2
spread.xx = x4
tail.x = x3
# With this change of variable it is easier to keep track of the effect 
# of the covariates in each parameter, but it is not needed.
formula = inla.mdata(y, cbind(spread.x, spread.xx), tail.x) ~ -1 + intercept + x + 
	f(z1, model = "rw1", scale.model=TRUE, constr=TRUE,
		hyper = list(prec = list(prior = "pc.prec", 
		param = c(0.1, 0.01)))) +
    f(z2, model = 'ar', order = 1, 
		   hyper=list(prec=list(prior="pc.prec", constr=TRUE,
		                        param=c(0.1,0.01)),
                      pacf1=list(param=c(0.5,0.8),
                      pacf2=list(param=c(0.5,0.8)))))
data.bgev = data.frame(y = y, intercept = 1, x = x, z1 = z1, z2 = z2,
                       spread.x = spread.x, spread.xx = spread.xx, tail.x = tail.x)
r3 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=1000,prec.intercept=1000),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb",
	                         cmin=0,
	                         b.strategy="keep"),
         verbose=FALSE, safe=TRUE)
```

A summary of the fitted effects and hyperparameters can be obtained as
follows

```{r, eval = T}
round(r3$summary.fixed,4)
round(r3$summary.hyperpar,4)
```


<!-- ### Model fitting -->



<!-- ## Example using pollution data -->
<!-- Here we replicate the application in Castro-Camilo et al. (2020). Specifically, we want to fit the model -->


<!-- The data is available at <https://github.com/dcastrocamilo/bgev>. -->

# Notes{#sec:notes}

## Linear predictor for the bGEV model{#sec:lpextensions}

The linear predictor can take any combinations of the latent
structures currently implemented. To see a list of the available
latent models, we can do

```{r, eval=F}
library(INLA)
inla.list.models('latent')
```


## PC prior for the tail parameter{#sec:pcpriortail}

Although non-informative priors are a common choice when little expert
knowledge is available, the PC prior approach allows us to select
moderately informative prior distributions in a $\textit{reasonable}$
way. This procedure penalises excessively complex models at a constant
rate by putting an exponential prior on a distance (specifically, the
Kullback-Leibler distance, or KLD) to a simpler baseline model.

The PC prior for the tail GEV parameter is defined in terms of the KLD
$\xi^2/(1-\xi)$ for $0\leq \xi<1$. In R-INLA, the prior is specified
as $$\texttt{hyper = list(<theta> = list(prior="pc.gevtail",
param=c(<lambda>, <low>, <high>)))}$$ where

* $\texttt{<lambda>}$: the constant penalisation rate
* $\texttt{<low>,<high>}$: interval to restrict possible values for the tail parameter. The default is [0,0.5].

For more details on the PC prior for the tail GEV parameter see
$\texttt{inla.doc("pc.gevtail")}$.

## About $\texttt{inla.mdata}${#sec:inlamdata}

$\texttt{inla.mdata(y, x1, x2,...)}$ is a matrix where each row are
replicates, and responses that are $\texttt{NA}$s are ignored. The
function $\texttt{inla.mdata}$ accept covariates that are one or many
vectors, matrices or data frames. If we pass $m$ covariates
$\texttt{x}_1,\ldots\texttt{x}_m$, then each row $(\texttt{x}_{i1},
\texttt{x}_{i2}, \ldots, \texttt{x}_{im})$ defines the covariates used
for the $i$-th row of $\texttt{y}$.

# Tools{#sec:tools}

The function $\texttt{giveme.gev.par}$ computes the usual GEV
parameters $(\mu,\sigma,\xi)$ given the bGEV parameters
$(q,s_\beta,\xi)$. Note that in both parametrisations, the tail
parameter is the same.

```{r}
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

```

The function $\texttt{map.tail}$ specifies the link between the
internal and usual parametrisations. In the code below,
$\texttt{interval}$ constraints the possible values for the tail
parameter, while $\texttt{inverse}$ is a boolean variable indicating
whether $\phi$ or $\phi^{-1}$ should be computed.


```{r}
map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}
```

\textbf{\Large{References}}

1. Coles, S. (2001). \emph{An introduction to statistical modeling of
   extreme values} (Vol. 208, p. 208). London: Springer.

2. Rue, H., Martino, S., & Chopin, N. (2009). Approximate Bayesian
   inference for latent Gaussian models by using integrated nested
   Laplace approximations. \emph{Journal of the royal statistical
   society: Series B (statistical methodology)}, 71(2), 319-392.

---
title: "Installation of INLA and inlabru with Apptainer on HPC"
author: Man Ho Suen (m.h.suen@sms.ed.ac.uk)
output:
  rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Installation of INLA and inlabru with Apptainer on HPC}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

(This vignette is a copy of the vignette in the *inlabru*-package.)

High Performance Computing (HPC) is surging in popularity. Building
INLA package on the HPC systems with the Anaconda environment
sometimes can be tricky due to the Linux distributions and glibc
versions. This article is to suggest a workaround with container
images using [Apptainer](https://apptainer.org/) (formerly known as
Singularity) because [Docker](https://www.docker.com/) requires root
access and is usually not available on HPC. Apptainer can pull Docker
images. Despite its convenience, the image size can be large.

## Installation Procedures 

1. Check if [Apptainer](https://apptainer.org/docs/user/main/cli/apptainer_version.html)
   is available on your HPC. If not, ask the administrator to install it. Type 
   on your HPC terminal to check which version is installed. 
```{r apptainer check, eval=FALSE}
$ apptainer version
```

2. Type the code below to pull the
[rocker/geospatial](https://rocker-project.org/images/versioned/rstudio.html)
Singularity Image Format (SIF) file with the latest R version. For
illustration, we'll assume the file is stored at
`./your_container.sif`.

```{r apptainer, eval=FALSE}
# To pull the docker image 
$ apptainer pull your_container.sif docker://rocker/geospatial:latest
# once the sif is downloaded, get into an interactive shell 
$ apptainer shell your_container.sif
# Inside the interactive shell, one can install INLA on a personal library path
$ R --verbose 
# Now, in a R environment
> options(repos = c(
    INLA = 'https://inla.r-inla-download.org/R/testing',
    CRAN = 'https://cloud.r-project.org'))
> install.packages("INLA") 
# One may be asked to create a personal library path.
> install.packages("inlabru") 
# quit R
> q()
```

3. Put the following in your job script to execute an R file non-interactively.

```{r exec, eval=FALSE}
# To execute an R file with apptainer
$ apptainer exec ./your_container.sif Rscript --no-restore --no-save --verbose file_to_run.R
```

Note: Use `inla.setOption(num.threads = ncpu)` to limit the number of
threads (ncpu) since INLA does not set it automatically. The `ncpu`
also directly links to RAM requirements.

---
title: "Cross-validation and group-cross-validation in INLA"
author: "Zhedong Liu (zhedong.liu@kaust.edu.sa)"
date: "Dec 2023"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Cross-validation and group-cross-validation in INLA}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
bibliography: groupcv.bib
---
```{r setup, include=FALSE}
library(INLA)
library(knitr)
library(formatR)
library(fastGHQuad)
library(pracma)
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")
if (file.exists("myinit.R")) source("myinit.R")
set.seed(20202)
knitr::opts_chunk$set(echo = TRUE)
opts_chunk$set(size='small'
               , cache=FALSE
               , cache.path='cache/group-cv'
               , comment=NA
               , warning=FALSE
               , message=FALSE
               , fig.align='center'
               , fig.path='figures/group-cv/'
               , fig.pos='H'
               , background='#ffffff'
               , results='hold'
               , eval=TRUE)

# gh.x = c(-6.995680e+00,-6.275079e+00,-5.673961e+00,-5.133596e+00,-4.631560e+00,-4.156272e+00,-3.700743e+00,-3.260321e+00,-2.831680e+00,-2.412318e+00,-2.000259e+00,-1.593886e+00,-1.191827e+00,-7.928770e-01,-3.959427e-01,-2.722499e-16,3.959427e-01,7.928770e-01,1.191827e+00,1.593886e+00,2.000259e+00,2.412318e+00,2.831680e+00,3.260321e+00,3.700743e+00,4.156272e+00,4.631560e+00,5.133596e+00,5.673961e+00,6.275079e+00,6.995680e+00)
# gh.w = c(4.618968e-22,5.110609e-18,5.899556e-15,1.860374e-12,2.352492e-10,1.461199e-08,5.043713e-07,1.049860e-05,1.395209e-04,1.233683e-03,7.482800e-03,3.184723e-02,9.671795e-02,2.121328e-01,3.387727e-01,3.957786e-01,3.387727e-01,2.121328e-01,9.671795e-02,3.184723e-02,7.482800e-03,1.233683e-03,1.395209e-04,1.049860e-05,5.043713e-07,1.461199e-08,2.352492e-10,1.860374e-12,5.899556e-15,5.110609e-18,4.618968e-22)

gh.x = fastGHQuad::gaussHermiteData(51)$x
gh.w = fastGHQuad::gaussHermiteData(51)$w

inla.lik.pred = function(y,args = NULL,args.string,eta.mean,eta.sd,eta.skewness){
    res = dnorm(y,mean = eta.mean,sd = sqrt(eta.sd^2 + 1/exp(args$theta)))
    return(res)
}
```
\section{Introduction}

We present examples to illustrate the use of leave-group-out
cross-validation (LGOCV) in INLA. The first example demonstrates the
primary function used to compute the LGOCV, while the second example
introduces additional features. For more advanced examples, please
refer to [@adin2023automatic].


\section{Basic Functionality: Multilevel Model}

Let us assume that we have collected repetitive measurements of the
weights of some rats with measurement errors. We denote each
measurement as $y_i$, and each $y_i$ is associated with the ID of the
corresponding rat, denoted as $id_i$. For the purposes of our
analysis, we have simulated the data according to a true model.
Specifically, we have simulated data for 10 rats, each with 10
measurements. We then plot the vector $\boldsymbol{y}$ against the
group ID for each measurement.

```{R message=FALSE, warning=FALSE,echo = FALSE}
n_rat = 10
n_each = 10
n = n_rat*n_each
id = rep(1:n_rat, each = n_each)
mu_rat = rep(x = rnorm(n_rat,mean = 30,sd = 1),each = n_each)
sd_measure = .1
y = rnorm(n = n, mean = mu_rat,sd = sd_measure)
plot(id,y,main = "Data against its group id")
```

We can fit a multilevel model to the data and use it to make two types
of predictions. The first prediction is to estimate the weight of
another measurement from rat one, while the second prediction is to
estimate the weight of a measurement from another rat. In INLA, there
is no separate predict function for making predictions. Instead,
predictions are made as a part of the model fitting process itself.
Since making predictions is essentially the same as fitting a model
with some missing data, we can simply set y[i] = NA for those
locations for which we wish to make predictions.

```{R message = FALSE, warning=FALSE}
data = data.frame(y = c(y,rep(NA,n_rat+1)),id = c(id,1:n_rat,n_rat+1))
formula = y ~ 1 + f(id,model = "iid")
res = inla(formula = formula,
           data = data,
           family = "normal"
           )
```

 We present the predictive densities for the two prediction tasks
below, which are given by $$ \pi(\tilde{Y}|\boldsymbol{y}) = \int
\pi(\boldsymbol{\theta}|\boldsymbol{y}) \int
\pi(\tilde{\eta}|\boldsymbol{\theta},\boldsymbol{y})\pi(\tilde{Y}|\tilde{\eta},\boldsymbol{\theta})
d\tilde{\eta} d\boldsymbol{\theta}, $$where each prediction task is
associated with a specific
$\pi(\tilde{Y}|\tilde{\eta},\boldsymbol{\theta})$.

```{R message = FALSE, warning=FALSE, echo = TRUE, eval=FALSE}
##
## this example needs to be fixed so it does not depend on the removed
## inla.likelihood* feature
##
res = inla(formula = formula,
           data = data.frame(y = c(y,rep(NA,n_rat+1)), id = c(id,1:n_rat,n_rat+1)),
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )
nconfig = res$misc$configs$nconfig
weights = unlist(lapply(1:nconfig,FUN = function(i){res$misc$configs$config[[i]]$log.posterior}))
weights = exp(weights)/sum(exp(weights))
eps = 0.001
q = seq(eps,1-eps,eps)
preddensity = vector(mode = "list",length = n_rat+1)
for(rat_idx in c(1,11)){
    predictor_idx = n + rat_idx
    mode_idx = which.max(weights)
    args = INLA:::inla.likelihood.parser(res$misc$configs$config[[1]]$arg.str[1])
    for(mode_idx in mode_idx){
        args$theta = res$misc$configs$config[[mode_idx]]$theta[1]
        mode_eta_mu = res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"mean"]
        mode_eta_sd = sqrt(res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"variance"])
        eta_eval = mode_eta_mu + seq(-5,5)*mode_eta_sd
        args$linear.predictor = eta_eval
        qfunc = inla.likelihood(type = "q",args)
        y_eval = c()
        for(i in 1:length(q)){
            y_eval = c(y_eval,qfunc(q[i]))
        }
    }
    y_eval = sort(y_eval)
    
    preddensity_conditional = list(mode = "list",length = nconfig)
    preddensity[[rat_idx]]$y_eval = y_eval
    preddensity[[rat_idx]]$density = numeric(length(y_eval))
    for(config_idx in 1:nconfig){
        args$theta = res$misc$configs$config[[config_idx]]$theta[1]
        preddensity_conditional[[config_idx]] = inla.lik.pred(y_eval,
                                                  args = args,
                                                  eta.mean = res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"mean"],
                                                  eta.sd = sqrt(res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"variance"])
                                                  )
        preddensity[[rat_idx]]$density  = preddensity[[rat_idx]]$density +  weights[config_idx]*preddensity_conditional[[config_idx]]
    }
}
plot(preddensity[[1]]$y_eval,preddensity[[1]]$density,
     main = "Predictive Density of A Measure from Rat 1",
     xlab = "weight",
     ylab = "density",
     type="l")
plot(preddensity[[11]]$y_eval,preddensity[[11]]$density,
     main = "Predictive Density of A Measure from An Unobserved Rat",
     xlab = "weight",
     ylab = "density",
     type="l")
```


To evaluate the accuracy of the first prediction, we can use LOOCV
with a logarithmic utility. The LOOCV logarithmic utility can be
expressed as: $$ U_{\text{LOOCV}} = \frac{1}{n}\sum_{i = 1}^n \log \pi(Y_i
= y_i| \boldsymbol{y}_{-i}). $$

To assess the quality of the second prediction, we can use
leave-group-out cross-validation (LGOCV), where the group includes all
measurements from a particular rat. The LGOCV logarithmic utility can
be expressed as: $$ U_{LGOCV} = \frac{1}{n}\sum_{i = 1}^n \log \pi(Y_i =
y_i|\boldsymbol{y}_{-I_i}). $$ Here, $I_i$ represents the indices of
data points from the same rat. We can compute $U_{\text{LOOCV}}$ and
$U_{\text{LGOCV}}$ using the inla.group.cv function.

```{R message = FALSE, warning=FALSE}
loocv_res = inla.group.cv(result = res)
ULOOCV = mean(log(loocv_res$cv[1:n]))

groups = lapply(1:n,FUN = function(i){which(id == id[i])})
lgocv_res = inla.group.cv(result = res,groups = groups)
ULGOCV = mean(log(lgocv_res$cv[1:n]))

print(paste0("ULOOCV is ", round(ULOOCV,5),"."))
print(paste0("ULGOCV is ", round(ULGOCV,5),"."))
```
In the previous code, we manually provided the groups. However,
`inla.group.cv` also has the ability to automatically construct groups
for LGOCV. This method was described in a preprint by
[@liu2022leave]. Using this automatic method, the resulting groups are
the same as the manual ones we provided earlier. The extra data points
$101$ does not have 
effect on the result since $y_{101}$ is `NA`. 

```{R message = FALSE, warning=FALSE}
lgocv_auto_res = inla.group.cv(result = res,num.level.sets = 1)
ULGOCV_auto = mean(log(lgocv_auto_res$cv[1:n]))
print(paste0("ULGOCV_auto is ", round(ULGOCV_auto,5),"."))
print(paste0("The group for data point 1 is:"))
print(paste(lgocv_auto_res$groups[[1]]$idx,collapse=" "))
```

The automatic construction of the groups is based on the posterior correlation coefficients of linear predictors. The parameter ``num.level.sets`` controls the degree of independence between $y_i$ and $\boldsymbol{y}_{-I_i}$. Higher ``num.level.sets`` means higher independence between $y_i$ and $\boldsymbol{y}_{-I_i}$. 

\subsection{How do `num.level.sets` and correlation define a group?}

We use a small example to illustrate the meaning of `num.level.sets =
3`. If $C_i$ contains the correlation of all $j$ to $i$, we construct
the group for $i$ using the following procedure.

```{R}
num.level.sets = 3
# We intentionally create some replicated correlations since it is not rare to have 
#|corr(eta_i,eta_j)| == |corr(eta_i,eta_k)|, where j != k.
C_i = c(1,1,0.9,0.9,0.8,0.8,-0.1,-0.1,0,0)
# L_i contains the correlation levels.
L_i = sort(unique(abs(C_i)),decreasing = T)
# Now we construct I_i
I_i = c()
for (l in 1:num.level.sets){
  I_i = c(I_i,which(abs(C_i) == L_i[l]))
}
print(paste(I_i,collapse=" "))
```



\section{Additional Features: Multilevel Model with Covariates}

We have multiple weight measurements for several rats denoted by
$y_i$, each with an associated rat id $id_i$, and a covariate $x_i$
associated with each measurement but not with the rat.

```{R message=FALSE, warning=FALSE,echo = FALSE}
n_rat = 10
n_each = 10
n = n_rat*n_each
id = rep(1:n_rat, each = n_each)
x = rnorm(n)
mu_rat = rep(x = rnorm(n_rat,mean = 30,sd = 2),each = n_each) + x
sd_measure = .1
y = rnorm(n = n, mean = mu_rat,sd = sd_measure)
plot(id,y,main = "Data against its group id")
plot(x,y,main = "Data against x")
```

We fit a multilevel model with random intercepts and random slopes.
```{R message = FALSE, warning=FALSE}
data = data.frame(y = y,id = id,id2 = id,x = x)
formula = y ~ 1 + x + f(id, model = "iid") + f(id2, W = x,model = "iid")
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )
```

\subsection{Model Comparasion: Unify Groups}

We want to compare two models: one with random slopes and one without
random slopes. To ensure that we have the same groups for the
different models, we can pass an `inla.group.cv` class object to
`inla.group.cv` function. Here is an example:

We first fit the model without random slopes.
```{R message = FALSE, warning=FALSE}
formula_alt =  y ~ 1 + x + f(id, model = "iid")
res_alt = inla(formula = formula_alt,
           data = data,
           family = "normal",
           control.compute = list(config = TRUE,likelihood.info = T)
           )
```


If we want to use the groups constructed by the full model to compute
LGOCV, we have the following code.

```{R message = FALSE, warning=FALSE}
lgocv_res = inla.group.cv(result = res,num.level.sets = 1)
lgocv_res_alt = inla.group.cv(result = res_alt,group.cv = lgocv_res)
```

\subsection{Using Prior Correlation to Construct Groups Automatically}

We check the result using automatic groups construction. We first
check the group for data point 1, 2 and 3.

```{R, warning = FALSE}
lgocv_res = inla.group.cv(result = res,num.level.sets = 1)
print(paste(lgocv_res$groups[[1]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[2]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[3]]$idx,collapse=" "))
```

The groups are different from the manual group. The reason is that the
posterior correlation of the linear predictors considering all the
effects including the linear effects and the ``iid`` effects. The
manual group is formed based on the prior information of the ``iid``
effect. We can pass the message to the automatic construction through
``keep`` option, which should work with ``strategy = "prior"``. That
means we construct groups using prior correlation with only effects
specified by the arguments ``keep``.

```{R, warning = FALSE}
lgocv_res = inla.group.cv(result = res,num.level.sets = 1,strategy = "prior",keep = "id")
print(paste(lgocv_res$groups[[1]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[2]]$idx,collapse=" "))
print(paste(lgocv_res$groups[[3]]$idx,collapse=" "))
```

\subsection{Select Testing Points}

In some applications, we may not want to iterate over the whole data
set. A typical scenario is that we are working on a very large data
set. To select a subset as our testing points, we can use `select`
arguments in `inla.group.cv`. Here is an example. We only want to have
$12$th data point and $53$th data point as our testing points.

```{R, warning = FALSE}
lgocv_res = inla.group.cv(result = res,num.level.sets = 1,select = c(12,53))
```
In this way, we only compute quantities related to 
$\pi(y_{12}|\boldsymbol{y}_{-I_{12}})$ and $\pi(y_{53}|\boldsymbol{y}_{-I_{53}})$.

If we are using manual groups construct we only need to specify the
groups for $12$th and $53$th data point.

```{R, warning = FALSE}
groups = vector(mode = "list",length = n)
groups[[12]] = which(id==id[12])
groups[[53]] = which(id==id[53])
lgocv_res = inla.group.cv(result = res,groups = groups)
```

\subsection{Using All the Hyperarameter Configrations}

To speed up the computation speed, `inla.group.cv` computes
$\pi(y_i|\boldsymbol{\theta}^*,\boldsymbol{y}_{-I_i})$, where
$\boldsymbol{\theta}^*$ is the mode of
$\pi(\boldsymbol{\theta}|\boldsymbol{y})$. In order to compute
$\pi(y_i|\boldsymbol{y}_{-I_i}) = \int
\pi(y_i|\boldsymbol{\theta},\boldsymbol{y}_{-I_i})
\pi(\boldsymbol{\theta}|\boldsymbol{y}_{-I_i})d \boldsymbol{\theta}$,
we need to enable the `control.gcpo` option while fitting the model,
which is in `control.compute`. The result are stored in `res$gcpo` in
the same format as the output of the `inla.group.cv`. Here is the
example code.

```{R, warning = FALSE}
control.gcpo = list(enable = TRUE,num.level.sets = 1)
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(control.gcpo = control.gcpo,config = TRUE)
           )
```


\subsection{Compute Other Utility Functions}

In the outputs of `inla.group.cv`, we have access to mean
(`lgocv.res$mean`) and standard deviation(`lgocv.res$sd`) of a
Gaussian approximation of
$\pi(\eta_i|\boldsymbol{\theta}^*,\boldsymbol{y}_{-I_i})$. With this
we can compute any scoring rules, which is a function of the observed
data $y_i$ and the predictive density
$\pi(Y_i|\boldsymbol{\theta}^*,\boldsymbol{y}_{-I_i})$. The predictive
density can be computed by the integral:
$$\pi(Y_i|\boldsymbol{\theta}^*,\boldsymbol{y}_{-I_i}) = \int
\pi(Y_i|\eta_i,\boldsymbol{\theta}^*)
\pi(\eta_i|\boldsymbol{\theta}^*,\boldsymbol{y}_{-I_i})d\eta_i,$$
where the likelihood $\pi(Y_i|\eta_i,\boldsymbol{\theta}^*)$ is given
by the model and the posterior is contained in the outputs.

If we want to also integrate out the uncertainty of the hyperparameters, we need to compute
$$\pi(Y_i|\boldsymbol{\theta},\boldsymbol{y}_{-I_i}) = \int
\pi(\boldsymbol{\theta}|\boldsymbol{y}_{-I_i})\int
\pi(Y_i|\eta_i,\boldsymbol{\theta})
\pi(\eta_i|\boldsymbol{\theta},\boldsymbol{y}_{-I_i})d\eta_i
d\boldsymbol{\theta},$$ where
$\pi(\boldsymbol{\theta}|\boldsymbol{y}_{-I_i})$ is computed using
$\pi(\boldsymbol{\theta}|\boldsymbol{y}_{-I_i}) \propto
\frac{\pi(\boldsymbol{\theta}|\boldsymbol{y})}{f(\boldsymbol{\theta}|\boldsymbol{y}_{I_i})}$.
[@liu2022leave] The location of each component is organized in the
following list:

$\log \pi(\boldsymbol{\theta}|\boldsymbol{y})$ is contained in 

`res$misc$configs$config[k]$log.posterior`.

The correction term $-\log f(\boldsymbol{\theta}|\boldsymbol{y}_{I_i})$ is contained in 

`res$misc$configs$config[[1]]$gcpodens.moments[,"log.theta.correction"]`. 

The mean of $\pi(\eta_i|\boldsymbol{\theta},\boldsymbol{y}_{-I_i})$ is contained in 

`res$misc$configs$config[[1]]$gcpodens.moments[,"mean"]`.

The variance of $\pi(\eta_i|\boldsymbol{\theta},\boldsymbol{y}_{-I_i})$ is contained in

`res$misc$configs$config[[1]]$gcpodens.moments[,"variance"]`.

As an example, we compute the Continuous Ranked Probability Score
(CRPS) and present its definition. Suppose $Y$ is a random variable
with cumulative distribution function $F(y)$, and let $y_{observed}$
be the observed value. The CRPS is defined as: $$CRPS(F,y_{observed})
= \int_{-\infty}^{\infty}(F(y)-(\boldsymbol{1}_{y \geq
y_{observed}}))^2dy,$$ where $\boldsymbol{1}{y \geq y{observed}}$ is
the indicator function that takes the value 1 when $y \geq
y_{observed}$ and 0 otherwise.

To evaluate the non-parametric predictive density, we obtain samples
by first sampling $\boldsymbol{\theta}$ from
$\pi(\boldsymbol{\theta}|\boldsymbol{y}{-I_i})$, then sampling
$\pi(\eta_i|\boldsymbol{\theta},\boldsymbol{y}{-I_i})$, and finally
obtaining samples of the predictive density by sampling from
$\pi(Y_i|\eta_i,\boldsymbol{\theta})$. With the samples of the
predictive density, we can compute its empirical cumulative function
to evaluate $CRPS(F,y_{observed})$. The following code demonstrates
this process:

```{R, warning = FALSE}
control.gcpo = list(enable = TRUE,num.level.sets = 1,strategy = "prior",keep = "id")
res = inla(formula = formula,
           data = data,
           family = "normal",
           control.compute = list(control.gcpo = control.gcpo,config = TRUE)
           )
# 1. choose the testing point
testing.point = 1
y_observed = y[testing.point]
# 2. get the hyperparameter density
nconfigs = res$misc$configs$nconfig
weights = unlist(lapply(X = 1:nconfigs,
	FUN = function(config.idx){
    res$misc$configs$config[[config.idx]]$log.posterior 
    + res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,3]
    }))
weights = exp(weights)/sum(exp(weights))
#3. get samples of y_i|y_{-I_i},theta
n = 1e6
n_theta = as.numeric(rmultinom(n = 1,size = n,prob = weights))
y_sample = c()
for(config.idx in 1:nconfigs){
    if(n_theta[config.idx]>0){
		eta = rnorm(n_theta[config.idx],
			mean = res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,1],
			sd = sqrt(res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,2]))
         sd = sqrt(1/exp(res$misc$configs$config[[config.idx]]$theta[1]))
         y_sample_config = rnorm(n = n_theta[config.idx],mean = eta,sd = sd)
         y_sample = c(y_sample,y_sample_config)
    }
}
#4 We can use y_sample to compute the empirical cumulative function then evaluate CRPS
F_y = function(y){sum(y_sample<=y)/n} #inefficient but straightforward...
yy = seq(min(y_sample)-10,max(y_sample)+10,0.01)
FF = unlist(lapply(yy,F_y))
CRPS = pracma::trapz(yy,(FF-(yy>=y_observed))^2)
```

# References

---
title: "SPDE example with multiple kind of outcomes"
subtitle: "Joint modeling with `mdata`, `inla.surv` and counts with exposure"
author: "Elias T Krainski"
date: "2023, March"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{SPDE example with multiple kind of outcomes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: spdeshort.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dev = "png",
  dev.args = list(type = "cairo-png"),
  fig.width = 10,
  fig.height = 7, 
  out.width = "69%",
  fig.align = "center"
)
knitr::opts_chunk$set(echo = TRUE)
set.seed(1)
```

# Abstract 

The **INLA** package supports different kinds of data likelihoods,
including the `inla.surv`, which is a generalization of the
`Surv` from the **survival** package, and `mdata`.
It also support joint modeling different kind of outcomes.
In this vignette we illustrate the case when we have
three outcomes, of different type, and model them jointly.

# Introduction 

Usually correlations between outcomes are modeled by
sharing common model terms in the linear predictor.
These terms can be covariates, random effects or sums of these.
In some cases, the likelihood requires more information 
than just the vector of observations. 
As an example, the likelihood for survival models requires
both observed times and information about censoring.

In this vignette we consider a joint model for three outcomes,
simulate data from this model and illustrate how to fit the 
joint model accounting for shared terms.
The linear predictor for each outcome includes covariates and 
a spatial random effect modeled using the SPDE approach.

The first outcome is known at an aggregated level,
in a setting were we model it considering a kind of likelihood designed in
**INLA** to account for all the information available with respect to the aggregation.
The second outcome is assumed to be censored,
so that we have to consider the observed value and the censoring information.
The third outcome is just the usual kind of data: one scalar per data unit.

## The spatial domain 

The SPDE models available in **INLA**
could be applied for processes in $\mathbb{R}^d$
or $\mathbb{S}^d$, for $d=1,2$, and the theory in @lindgrenRL2011 
include higher dimentional domains.
We consider a spatial domain, $D \in \mathbb{R}^2$
to keep the example simple.
Without loss off generality we consider $D$ as a rectangle defined as
```{r domain}
bb <- rbind(c(0, 10), c(0, 7))
```

Next, we store the length of each side and its average for later use
```{r rxy}
rxy <- apply(bb, 1, diff)
rr <- mean(rxy)
```

We sample a set of $n$ locations,
$l_1, ..., l_n$, in this domain, $l_i \in D$, 
completely at random as follows:
```{r loc}
n <- 1000
loc <- cbind(
    runif(n, bb[1, 1], bb[1, 2]),
    runif(n, bb[2, 1], bb[2, 2]))
```

## A spatial field

Instead of simulating from a model to play as a spatial smooth term, 
we define a smooth function on this domain as follows
```{r sfn}
sfn <- function(a, b)
    sin(a - b) + cos(a + b)
```

This function is evaluated at the $n$ locations and visualized with
```{r sloc}
u <- sfn(2 * pi * loc[, 1] / rr,
         2 * pi * loc[, 2] / rr) 
summary(u)
par(mar = c(2, 2, 0, 0), mgp = c(1.5, 0.5, 0), las = 1)
plot(loc, asp = 1, pch = 19, cex = 0.5 + (2 + u)/4, xlab = "", ylab = "")
```

## Spatial sub-domains 

In order to define the first outcome,
let us partition our domain into a set of sub-regions, 
$r_1, r_2, ..., r_g$, 
so that $r_i \cap r_i = \oslash$ and $\cup_{i=1}^g r_j = D$.

For simplicity of coding, but without loss of generality,
we assume these small sub-regions to be pixels based in
the following set of centers:
```{r centers}
h <- 1 ## size of the side of each sub-region
group.ce <- as.matrix(expand.grid(
    seq(bb[1, 1] + h/2, bb[1, 2], h),
    seq(bb[2, 1] + h/2, bb[2, 2], h)))
(ng <- nrow(group.ce))
```

We can avoid the need of dealing with the general
spatial objects and operations available in **R**
and identify each location $l_i$ to each sub-region $r_j$
with an integer vector as
```{r groupid}
group.id <- ceiling(loc[,1]/h) + 
  ceiling(rxy[1]/h) * (ceiling(loc[,2]/h)-1)
```

This will be used as a grouping index, so that each group is made of the
observations falling in each of the pixels.
We will visualize the locations with color depending on this grouping later.

## Covariates

Let us consider a set of three covariates for each location as 
```{r xxx}
xxx <- cbind(x1 = runif(n), x2 = runif(n), x3 = runif(n))
```

We average the covariates by group as we will consider that
we have data at the group level to model the first outcome.
```{r x.g}
xxx.g <- aggregate(xxx, list(g = group.id), mean)
str(xxx.g)
```


# Outcomes

We will consider three outcomes.

### First outcome: aggregated Gaussian

The first outcome is Gaussian distributes, 
with unknown mean and variance,
and a known scaling factor, $s$, such that 
\[ 
y_i \sim \textrm{N}(\mu_i, s_i\sigma^2).
\]

We assume $\boldsymbol{\eta}_y = \{\mu_1, ..., \mu_n\}$ and
\[ 
\boldsymbol{\eta}_y = \mathbf{X}\boldsymbol{\beta}_y + \mathbf{u}.
\]
This is being evaluated considering the covariate and random effect averaged
per group:
```{r y}
beta.y <- c(5, 0, -3, 3) 
u.g <- tapply(u, group.id, mean)
eta.y.g <- drop(cbind(1, as.matrix(xxx.g[, 2:ncol(xxx.g)])) %*% beta.y) + u.g
```

We will simulate the scaling factor for each observation with
```{r s}
s <- rgamma(n, 7, 3)
summary(s)
```

```{r ysim}
sigma.y <- 1
y <- rnorm(n, eta.y.g[group.id], sqrt(sigma.y/s))
summary(y)
```

We simulated $y_i$ for each location $l_i$.
However, we consider that observations of this variable are
available only at the aggregated level, at each of the sub-regions $r_j$.
Furthermore, for each sub-region $r_j$, we know 

 - how many observations were taken in each sub-region, $n_j = \sum_{i=1}^n I(l_i \in r_j)$
 - the weighted average $\overline{y}_j$ = $(n_j)^{-1}\sum_{i,l_i\in r_j} s_iy_i$,
 - the sum o the scaling factors, $m_j$ = $\sum_{i,l_i\in r_j} s_i$, 
 - the half of the sum of the log of the scaling factors, $\frac{1}{2}\sum_{i,l_i\in r_j} \log(s_i)$,
 - and the sample variance within each unit, defined as
 $v_j$ = $\frac{1}{n_j}\sum_{i,l_i\in r_j}^n(s_i y_i^2-\overline{y}_j^2)$

The function `inla.agaussian` computes these five statistics, 
given $y_i$ and $s_i$ for each group.
We use this function to compute these statistics, as follows 
```{r ag, warning}
library(INLA)
agg <- lapply(1:ng, function(g) {
    ig <- which(group.id==g)
    if(length(ig)>0) 
        return(inla.agaussian(y[ig], s[ig]))
    return(inla.mdata(list(NA, 0, 1, 1, NA))) ### a deal with missing
})
str(YY <- Reduce('rbind', lapply(agg, unlist))) ## five columns matrix
```

## Second outcome: Survival

For the second outcome we choose a Weibull distribution with
the following parametrization
\[
f_W(w_i) = \alpha w_i^{\alpha - 1} \lambda_i^\alpha \textrm{e}^{-(\lambda_i w_i)^\alpha}.
\]

We define the linear predictor as 
$\boldsymbol{\eta}_w = \{\log(\lambda_1), ..., \log(\lambda_n)\}$ and
\[ 
\boldsymbol{\eta}_w = \mathbf{X}\beta_w + \gamma_w\mathbf{u}
\]
and we consider the following code to sample from a Weibull distribution
```{r w0}
beta.w <- c(1, -1, 0, 1)
alpha <- 2
gamma.w <- 0.5
lambdaw <- exp(cbind(1, xxx) %*% beta.w + gamma.w * u)
we0 <- rweibull(n, shape = alpha, scale = 1/lambdaw)
summary(we0)
```

However, we suppose that some observations are censored
```{r event}
summary(u.ev <- runif(n, 0.3, 1)) ## censoring factor
table(event <- rbinom(n, 1, 0.5)) ## censored (=0) or event (=1)
summary(we <- ifelse(event == 1, we0, we0 * u.ev)) ## censored outcome
```

## Third outcome: Count under exposure

We consider that we have different exposure at each location
```{r ee}
summary(ee <- rgamma(n, 10, 2))
``` 

We assume an outcome following a Poisson distribution with 
\[ 
O_i \sim \textrm{Poisson}(\delta_i E_i). 
\]

We assume the linear predictor as 
$\boldsymbol{\eta}_p = \{\log(\delta_1),\;...,\; \log(\delta_n)\}$ and
\[
\boldsymbol{\eta}_p = \mathbf{X}\beta_p + \gamma_p\mathbf{u}
\]

This outcome is being simulated with
```{r po}
beta.p <- c(2, 1, -1, 0)
gamma.p <- -0.3
delta.p <- exp(cbind(1, xxx) %*% beta.p + gamma.p * u)
po <- rpois(n, delta.p * ee)
summary(po)
```

# Data model setup 

We now have to setup the objects in order to fit the model.

## SPDE model setup

We have to define a mesh over the domain. 
We do it considering the following code
```{r mesh}
mesh <- inla.mesh.2d(
    loc.domain = matrix(bb[c(1,3,3,1,1, 2,2,4,4,2)], ncol = 2), 
    offset = c(.0001, .5) * rr, ## extensions size
    max.edge = c(.05, .3) * rr, 
    cutoff = 0.025 * rr)
mesh$n
```

This mesh and the locations colored by the group defined for the first
outcome can be visualized with
```{r vmesh}
par(mar = c(0, 0, 1, 0), mgp = c(1.5, 0.5, 0))
plot(mesh, asp = 1, edge.color = gray(0.5))
points(loc, pch = 19, col = group.id, cex = 0.3 + (2 + u)/4)
```

The SPDE model is defined with
```{r spde}
spde <- inla.spde2.pcmatern(
    mesh,
    prior.range=c(1, 0.1),
    prior.sigma=c(1, 0.1))
```

## Linear predictor components

We will fit a joint model considering the linear predictor
with terms as how it was simulated.
The `inla.stack()` function in meant to me used
when working with SPDE models.
However, it also helps when considering multiple likelihood data
as detailed below.

One way to deal with multiple likelihoods in **INLA**
is defining `matrix` data for the outcome where each column
is used for each likelihood.
However, in this vignette we are dealing with more complex cases.
Therefore, he need to use `list` in the left hand side of the `formula`.
This is the key point to implement this example.

The right hand side of the formula includes each term in the linear predictor.
When working with multiple likelihood, each term in each likelihood
should be named uniquely.
For example, if two likelihoods include the same covariate
but with a different coefficient,
each one has to have its own name.
Therefore, we will pass the covariate `x1` as `x1y`, 
for the first outcome, `x1w` for the second outcome, 
and `x1p` for the third outcome.
Similarly for the other ones, including the intercept.

In our example, the SPDE model is in the linear predictor for the first outcome,
a copy from this random effect in in the linear predictor for the second outcome,
and another copy of this term is in the linear predictor for the third outcome.

The model formula considering all of these details is defined as
```{r formula}
fff <- list(
  inla.mdata(Y),
  inla.surv(w, ev),
  o) ~ 0 +
  a1 + x1y + x2y + x3y + f(s, model = spde) +
  a2 + x1w + x2w + x3w + f(sc1, copy = "s", fixed = FALSE) +
  a3 + x1p + x2p + x3p + f(sc2, copy = "s", fixed = FALSE)
```

## Building data stacks

The data has to be organized so it contains all the information needed,
that is all the outcomes, covariates and random effect indexes.
For the SPDE models it also has to include the projector matrices.

The main work of the `inla.stack()` function is to check and organize
outcomes, effects and projector matrices
in order to guarantee that the dimentions match.
Additionally, `inla.stack()` will deal with the problem of
arranging the outcomes to work properly with multiple likelihoods.

For the first outcome, we need to supply a five column matrix,
each with one of the five statistics previously computed for the `ng` groups.
For the effects we define an intercept, 
covariates with names according to what the
`formula` specify and the index for the SPDE model.
The projector matrices is one for the fixed effect, which is just `1`,
and the one for the SPDE model.

We will include a different `tag` for each stack to keep track.
This is useful to collect results associated to each outcome.
For each outcome we can also define an indicator vector to identify it.
This is useful for the case of having some missing data. 
In this case the indicator can be used to identify which link to be used
for making the predictions.

The stack for the first outcome can be defined as
```{r stack1}
stackY <- inla.stack(
    tag = 'y',
    data = list(Y = YY),
    effects= list(
        data.frame(a1 = 1,
                   x1y = xxx.g$x1, 
                   x2y = xxx.g$x2, 
                   x3y = xxx.g$x3),
        s = 1:mesh$n),
    A=list(1,
           inla.spde.make.A(mesh, group.ce))
)
```

For the second outcome, we have to supply
two vectors (observed time and censoring) to build the `inla.surv` data,
the covariates, with names according to the terms in the formula, 
and the projector matrices, the single `1` for the covariates and
the projector for the shared SPDE term.

The stack for the second outcome can be defined as
```{r stack2}
stackW <- inla.stack(
    tag = 'w', 
    data = list(w = we,
                ev = event), 
    effects = list(
        data.frame(a2 = 1,
                   x1w = xxx[, 1], 
                   x2w = xxx[, 2], 
                   x3w = xxx[, 3]),
        sc1 = 1:mesh$n),
    A = list(1,
             inla.spde.make.A(mesh, loc))
)
```

For the third outcome we have to supply the vector of counts
and the vector with the exposure. 
The effects and projector are similar with the previous others,
just that we have to account for its names.

The stack for the third outcome can be defined as
```{r stack3}
stackP <- inla.stack(
    tag = 'p', 
    data = list(o = po,
                E = ee),
    effects = list(
        data.frame(a3 = 1,
                   x1p = xxx[, 1], 
                   x2p = xxx[, 2], 
                   x3p = xxx[, 3]),
        sc2 = 1:mesh$n),
    A = list(1,
             inla.spde.make.A(mesh, loc))
)
```

# Fitting the model and some results

We now fit the model and look at some results.

## Model fitting

To fit the model we start by
joining all the data into one data stack
```{r jstack}
stacked <- inla.stack(stackY, stackW, stackP)
```

We now supply this to the `inla()` function in order to fit the model
```{r fit}
result <- inla(
   formula = fff,
    family = c("agaussian", "weibullsurv", "poisson"),
    data = inla.stack.data(stacked),
    E = E, 
    control.predictor = list(
        A=inla.stack.A(stacked)),
    control.family = list(
        list(),
        list(variant = 1),
        list()), 
   control.compute = list(dic = TRUE, waic = TRUE, cpo = TRUE)
)
result$cpu
```

## Summaries

We now compare the true value for each 
fixed effect with its posterior summary.

```{r fixef}
round(cbind(
    true = c(beta.y, beta.w, beta.p),
    result$summary.fix), 2)
```

For the hyper-parameters, we have one for the
`agaussian` likelihood, one for the `weibulsuv`, 
two for the SPDE model and two that define the scaling 
for the shared SPDE terms in the second and third outome.

```{r hpars}
round(cbind(true = c(1/sigma.y^2, alpha, NA, NA, gamma.w, gamma.p), 
            result$summary.hy), 2)

```

## Results considering each outcome

To collect outputs associated with each data, 
we need to consider the `tag` from each data stack to
get the data index from the joint stack. 
For example, the data index for the third outcome is
```{r dindex}
idx1 <- inla.stack.index(stacked, tag = "p")$data
```

We can see the summary for the fitted values
for the first few observations of this outcome with
```{r fitted}
head(cbind(true = delta.p, 
           result$summary.fitted.values[idx1, ]), 3)
```

For access the DIC, WAIC and log score 
can consider the family indicator stored 
in the DIC output as follows
```{r dic}
str(result$dic)
str(result$waic)
str(result$cpo)
result$dic$family.dic ### DIC for each outcome
tapply(result$waic$local.waic, 
       result$dic$family, ## taken from dic family id
       sum) ## WAIC for each outcome
tapply(result$cpo$cpo, 
       result$dic$family, ## taken fron dic family id
        function(x) 
         c(nlCPO = -sum(log(x)))) ## -sum log CPO for each outcome
```


# References

---
title: "SPDE how to"
author: "Elias T. Krainski"
date: 'created: March 31, 2016, last update `r Sys.Date()`'
output: pdf_document
vignette: >
 %\\VignetteIndexEntry{SPDEhowto} 
 %\\VignetteEngine{knitr::knitr} 
 \usepackage[utf8]{inputenc}
---

# How to fit a simple **SPDE** model in **INLA**

This document ilustrates how to do a 
geostatistical fully Bayesian analysis
through the **S**tochastic **P**artial 
**D**iferential **E**quation approach 
<http://onlinelibrary.wiley.com/doi/10.1111/j.1467-9868.2011.00777.x/full>
using the **I**ntegrated **N**ested 
**L**aplace **A**proximation, 
<http://onlinelibrary.wiley.com/doi/10.1111/j.1467-9868.2008.00700.x/full>
implementation in the package available at 
<http://www.r-inla.org>.


```{r settings, include=FALSE}
library(knitr);  library(INLA);  library(fields) 
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE, 
fig.path='figures/SPDEhowto')
knit_hooks$set(small.mar = function(before, options, envir) {
    if (before) par(mar = c(0.1, 0.1, .1, .1))  # smaller margin on top and right
})
knit_hooks$set(mar3311 = function(before, options, envir) {
    if (before) par(mar = c(3, 3, 1, 1), mgp=c(2, 1, 0))  # smaller margin on top and right
})
set.seed(1) 
```

## Simulating some data


Define some random **Locations** 
and the Random Field (RF) 
**covariance** matrix, considering
exponential correlation function: 

```{r locations}
n = 200  ## number of location points
coo = matrix(runif(2*n), n) ## location points
k <- 10;   s2s <- 0.7 ## RF parameters
R <- s2s*exp(-k*as.matrix(dist(coo))) ## covariance matrix
```

Draw a **RF** sample: a multivariate Normal realization 
  
```{r field simulate}
s <- drop(rnorm(n)%*%chol(R)) ## one RF realization
```

Adding a **covariate** effect and a noise

```{r covariate and outcome}
x <- runif(n)  ## covariate
beta <- 1:2  ## regression coefficients
lin.pred <- beta[1] + beta[2]*x + s ## linear predictor
s2e <- 0.3 ## error variance (nugget)
y <- lin.pred + rnorm(n, 0, sqrt(s2e))  ## the outcome
```

## Model fitting steps

*  __Mesh__:
a triangulation to discretize the 
random field (RF) at 'm' nodes. 

```{r mesh, fig.width=4.2, fig.height=4.2}
r0.1 <- sqrt(0.5 * 8)/k ## distance with correlation around 0.139
mesh <- inla.mesh.2d( ## 2D mesh creator
  loc=coo, ## provided locations 
  max.edge=c(r0.1/3, r0.1), ## maximum edge length (inner, outer): mandatory 
  offset=c(r0.1/3, r0.1*2), ## outer extension
  cutoff=r0.1/10) ## good to have >0
par(mar=c(0,0,1,0))
plot(mesh, asp=1) ## plot the mesh
points(coo, col='red') ## add the points 
```

A little *warning* about the mesh. 
The additional triangles outer domain 
is to avoid boundary effects. 
Is good to have aproximately 
isosceles triangles. 
And, to avoid tiny triangles. 
We need to have edges lengths 
of the inner mesh triangles 
less than the range of the process. 
Of course, if it is too small, there 
might not be any spatial effect. 

* __Define the__ _n x m_ __projector matrix__ to project 
 the process at the mesh nodes to locations 

```{r, projector, fig.width=9, fig.height=3, small.mar=TRUE}
image(A <- inla.spde.make.A( ## projector creator
    mesh=mesh, ## provide the mesh
    loc=coo) ### locations where to project the field
    ) ## an 'n' by 'm' projector matrix
```

* __Build the SPDE model__ on the mesh.
Set $\alpha=3/2$ to build the precision structure for 
an Exponential correlation function

```{r SPDE model}
spde <- inla.spde2.matern( ## precision components creator
    mesh=mesh, ## mesh supplied
    alpha=1.5) ## smoothness parameter
```

* __Create a data stack__ to organize the data. 
This is a way to allow models with complex 
linear predictors. In our case, we have a SPDE 
model defined on _m_ nodes. It must be combined 
with the covariate (and the intercept) 
effect at _n_ locations. 
We do it using different projector matrices. 

```{r data stack}
stk.e <- inla.stack( ## stack creator
  data=list(y=y),  ## response
  effects=list(## two elements:
    data.frame(b0=1, x=x), ## regressor part
    s=1:spde$n.spde),  ## RF index
  A=list(## projector list of each effect
    1, ## for the covariates
    A), ## for the RF
  tag='est') ## tag
``` 

* __Fit__ the posterior marginal distributions
  for all model parameters 

```{r fitting}
formula <- y ~ 0 + b0 + x + ## fixed part
  f(s, model=spde) ## RF term
res <- inla( ## main function in INLA package
  formula, ## model formula
  data=inla.stack.data(stk.e), ## dataset
  control.predictor=list( ## inform projector needed in SPDE models
    A = inla.stack.A(stk.e))) ## projector from the stack data
```


## Posterior marginal distributions - PMDs

Summary of the regression coefficients PMDs

```{r fixed}
round(res$summary.fixed, 4) 
```
We have to transform the precision PMD to have 
the variance PMD. It can be done and visialized by

```{r nugget, fig.width=3.5, fig.height=3, mar3311=TRUE}
m.prec <- res$marginals.hyperpar$'Precision for the Gaussian observations' ## the marginal
post.s2e <- inla.tmarginal(## function to compute a tranformation 
  function(x) 1/x, ## inverse transformation
  m.prec) ## marginal to be applied
### visualize it
plot(post.s2e, type='l', ylab='Density', 
     xlab=expression(sigma[e]^2))
abline(v=s2e, col=2) ## add value used to generate the data
```

The SPDE approach uses a local variance, $\tau^2$, 
such that $\sigma_{s}^2=1/(2\pi\kappa^2\tau^2)$. 
On \textbf{\textsf{INLA}} we work log($\tau^2$)
and log($\kappa$). So, especially for 
$\sigma_{s}^2$, we have to do 
an additional computation. 
The PMDs for all RF parameters on 
user scale are computed by

```{r rf} 
rf <- inla.spde.result( ## function to compute the 'interpretable' parameters
    inla=res, ## the inla() output
    name='s', ## name of RF index set
    spde=spde, ## SPDE model object
    do.transf=TRUE) ## to user scale
``` 

It can be visualized by 

```{r parameters, fig.width=10, fig.height=3}
par(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0))
plot(rf$marginals.var[[1]], ty='l', 
     xlab=expression(sigma[s]^2), yla='Density')
abline(v=s2s, col=2) ## add the true value
plot(rf$marginals.kap[[1]], type='l', 
     xlab=expression(kappa), ylab='Density')
abline(v=k, col=2) ## add the true value
plot(rf$marginals.range[[1]], type='l', 
     xlab='range nominal', ylab='Density')
abline(v=sqrt(0.5 * 8)/k, col=2) ## add the 'true' value
```

## Projection on a grid / visualization

An interesting result is the map of the RF on a grid. 
The simplest way to have it is by projection. 
We just have to define the projector matrix 
and project, for example, the posterior 
mean and posterior standard deviation on the grid. 

```{r project}
gproj <- inla.mesh.projector( ## projector builder
  mesh, ## mesh used to define the model
  xlim=0:1, ylim=0:1, ## limits where to create the grid
  dims=c(300,300)) ## grid dimension
## project the mean and the SD
g.mean <- inla.mesh.project(gproj, res$summary.random$s$mean)
g.sd <- inla.mesh.project(gproj, res$summary.random$s$sd)
```

We can visualize it by 
```{r visualize, fig.width=9, fig.height=5}
par(mfrow=c(1,2), mar=c(0,0,1,0))
require(fields)
image.plot(g.mean, asp=1, main='RF posterior mean', axes=FALSE, horizontal=TRUE)
image.plot(g.sd, asp=1, main='RF posterior SD', axes=FALSE, horizontal=TRUE)
```

## Prediction 

Define the set of target locations, 
the corresponding projector 
matrix and covariate values at target locations 

```{r prediction scenario}
tcoo <- rbind(c(0.3,0.3), c(0.5,0.5), c(0.7,0.7))
dim(Ap <- inla.spde.make.A(mesh=mesh, loc=tcoo)) 
x0 <- c(0.5, 0.5, 0.5)
```

To do a fully Bayesian analysis, 
we have to include the target locations on the 
estimation process by assigning NA 
for the response at these locations. 
Defining the prediction stack 

```{r prediction stack} 
stk.pred <- inla.stack(
  tag='pred', ## will be used to collect the posterior marginals
  data=list(y=NA), ## response set as NA
  effects=list(
    data.frame(x=x0, b0=1), ## covariate scenario
    s=1:spde$n.spde), ## same as before
  A=list(1, Ap)) ## covariate and target locations field projectors
``` 

Fit the model again with the full stack 

```{r refitting}
stk.full <- inla.stack(stk.e, stk.pred) ## join both data
p.res <- inla(
  formula, data=inla.stack.data(stk.full), ## using the full data 
  control.predictor=list(compute=TRUE, ## compute the predictor
                         A=inla.stack.A(stk.full)), ## from full data
  control.mode=list(theta=res$mode$theta)) ## use the mode previously found
``` 

Get the prediction data index and collect the 
linear predictor PMDs  to work with 

```{r prediction index}
pred.ind <- inla.stack.index( ## stack index extractor function
  stk.full, ## the data stack to be considered
  tag='pred' ## which part of the data to look at
  )$data ## which elements to collect
ypost <- p.res$marginals.fitted.values[pred.ind]
```

Visualize with commands bellow

```{r predicted, fig.width=10, fig.height=3}
xyl <- apply(Reduce('rbind', ypost), 2, range)
par(mfrow=c(1,3), mar=c(3,3,2,1), mgp=c(2,1,0))
for (j in 1:3) 
  plot(ypost[[j]], type='l', xlim=xyl[,1], ylim=xyl[,2], 
       xlab='y', ylab='density', main=paste0('y', j))
``` 

### Manipulating marginals

We have already used the *inla.tmarginal()* function. 
There are some other functions to work with 
marginal distributions which may be usefull as well: 

```{r marginals}
apropos('marginal')
```

Playing with the posterior marginal for the first target location
```{r marginals playing}
inla.qmarginal(c(0.15, 0.7), ypost[[1]]) ## quantiles
inla.emarginal(function(x) x^2, ypost[[1]]) - ## E(y^2) -
  inla.emarginal(function(x) x, ypost[[1]])^2 ## E(y)^2 to compute the variance
inla.pmarginal(inla.qmarginal(0.3, ypost[[1]]), ypost[[1]]) 
inla.zmarginal(ypost[[1]]) ## posterior summary
```

---
title: "A note about values"
author: "Håvard Rue (hrue@r-inla.org) & Sigrunn Holbek Sørbye (sigrunn.sorbye@uit.no)"
date: "Dec, 2025"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{A note about values}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE, cache=FALSE}
library(knitr)
knitr::opts_chunk$set(echo = TRUE, warning = FALSE, error = FALSE, message = FALSE, cache =FALSE)
knitr::opts_chunk$set(fig.path="figures/a-note-on-values/")
library(INLA)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
if (file.exists("myinit.R")) source("myinit.R")
```

## Introduction

This a short note discussing the argument \texttt{values} in
\texttt{f()}, to avoid unfortunate confusion. This is
especially an issue for models like \texttt{ar1}, \texttt{ar} and 
\texttt{seasonal}, which are only defined for integer-valued arguments. 


Consider the AR1-model
$$
	x_t = \phi x_{t-1} + \epsilon_t
$$
for $t = 1, 2, \ldots, m$. 

In practice, we might use the AR1-model for various sequential indexes, like
years, weeks, etc. In these cases, it is naturally to write
```{r eval=FALSE}
y ~ ... + f(year, model = "ar1", ...) 
```
where for example \texttt{year} is given by 
```{r}
year <- 2000:2025
```
What happens internally, is that the \texttt{year}-variable specifies
the \texttt{values} (time points) at which the AR1-model is defined. So, the variable 
\texttt{year} would be mapped to the $t$-index, which runs from $1$ to $m$.

To be more specific, a mapping between \texttt{year} and the $t$-index is
defined as
```{r}
values <- sort(unique(year[!is.na(year)]))
values

map.year.to.t <- function(year, values) {
	a <- numeric(length(year))
	for(i in seq_along(year)) 
		a[i] = which(year[i] == values)
	return (a) 
}
yy <- c(2000, 2020, 2025)
print(cbind(year=yy, t=map.year.to.t(yy, values)))
```

An unfortunate consequence is that if some years
are missing, then "confusing" things happen. Suppose, we do not have
obsevations for some years,

```{r}
year <- c(2000:2020, 2023:2025)
```

Then the above mapping still applies but with

```{r} 
values <- sort(unique(year[!is.na(year)]))
values
```
in which the years 2021 and 2022 are missing. The internal mapping now gives

```{r}
yy <- c(2000, 2020, 2025)
print(cbind(year = yy, t = map.year.to.t(yy, values)))
```
Note that for \texttt{year} $=2025$ the index $t$ is now 24 and not 26 as earlier. This
implies that the correlation between year 2020 and 2023 is $\phi$ while it was
$\phi^3$ as before. Often, this is not what we expect to happen.

Another case is when the argument denote non-integers, like
```{r eval=FALSE}
time <- c(1.1, 2.2, 3.3, 4.4, 10.0)
```
A call using \texttt{f(time, model = "ar1")} would run, but \texttt{time} will be mapped to the discrete indexes $1, \ldots, 5$. This is because the models
\texttt{ar1/ar/seasonal} are defined on integers and cannot handle
irregular locations. 

We can ensure consistent behaviour by adding the argument
\texttt{values} to \texttt{f()}, specifying for which indexes the model
is defined, like
```{r eval=FALSE}
y ~ ... + f(year, model = "ar1", values = 2000:2025)
``` 
or, if we want predictions for additional years,
```{r eval=FALSE}
y ~ ... + f(year, model = "ar1", values = 1990:2035)
```
In this case, the \texttt{values} are defined by the corresponding
named argument and not by the variable \texttt{year} as before
```{r}
values <- sort(unique(values[!is.na(values)]))
```
The requirement for \texttt{year} is now that these are either \texttt{NA}-values or
a subset of the defined \texttt{values}. 

## An example

This is a simple simulated example demonstrating the issue about
\texttt{values}, having missing observations. 
```{r, plot=TRUE, out.width = "70%", fig.align = "center"}
year <- 2000:2025
n <- length(year)
phi <- 0.9
phi.intern <- inla.models()$latent$ar1$hyper$theta2$to.theta(phi)
x <- scale(arima.sim(n, model=list(ar=phi)))
s <- 0.3
y <- x + rnorm(n, sd=s)
plot(year, y, pch=19)
```

Let us fit an AR1 model
```{r, plot=TRUE, out.width = "70%", fig.align = "center"}
r <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year))
plot(year, y, pch=19)
lines(year, r$summary.random$year$mean)
```		

Assume \texttt{y[10]} is missing, then we will get the wrong results
by removing \texttt{y[10]} from the data, because \texttt{values} are
not given (hence \texttt{year[10]} is removed as well).

```{r}
## this is correct
y[10] <- NA
r <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year))  ## missing data included

## this is wrong
rr <- inla(y ~ -1 + f(year, model="ar1",
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year)[-10,]) ## missing data removed
## we compare the results
r$mlik - rr$mlik
```

If we add \texttt{values}, then we get the correct result when we remove
missing data.

```{r}
rrr <- inla(y ~ -1 + f(year, model="ar1", 
	                  values = 2000:2025,  ## values added
	                  hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = data.frame(y, year)[-10,]) ## missing data removed
## we compare the results
r$mlik - rrr$mlik
```

## Another example

We can extend the range using \texttt{values}, to get predictions.
```{r, plot=TRUE, out.width = "70%", fig.align = "center"}
values <- 1990:2035
r <- inla(y ~ -1 + f(year, model="ar1",
	                 values = values,  ## values added
	                 hyper = list(prec = list(initial=0, 
						                      fixed=TRUE),
								  rho = list(initial=phi.intern,
									         fixed=TRUE))),
	    family="gaussian",
		control.family = list(hyper = list(
                                  prec = list(initial=log(1/s^2),
								  fixed = TRUE))),
		data = list(y=y, year=year, values=values))
plot(year, y, pch=19, xlim=range(values))
lines(values, r$summary.random$year$mean)
```		

\documentclass[a4paper,11pt]{article} 

\usepackage[margin={2cm,2cm}]{geometry} 
\usepackage{multicol,hyperref}

\title{\vspace{-2cm}
  A short introduction on how to fit a \textbf{SPDE} model 
  with \textbf{\textsf{INLA}}}
\author{Elias T. Krainski and H{\aa}vard Rue} 
\date{Created: November, 20 2013 - Last update: August, 9 2017}

%\VignetteEngine{knitr::knitr} 
%\VignetteIndexEntry{SPDEhowto}
\begin{document} 

\maketitle 

\vspace{-1cm}
\begin{center}\begin{minipage}[c]{0.9\textwidth}\centering
This document ilustrates how to do a 
geostatistical fully Bayesian analysis
through the \textbf{S}tochastic \textbf{P}artial 
\textbf{D}iferential \textbf{E}quation 
approach, \cite{lindgrenRL:2011}, 
with \textbf{I}ntegrated \textbf{N}ested 
\textbf{L}aplace \textbf{A}proximation, 
\cite{rueMC:2009}, 
using the \textbf{\textsf{INLA}} package, 
\texttt{http://www.r-inla.org}.
\end{minipage}\end{center}

<<setting,include=FALSE>>= 
set.seed(1) 
library(knitr)
knit_hooks$set(pars = function(before, options, envir) {
    if (before) graphics::par(options$pars)
})
opts_chunk$set(message=FALSE, warning=FALSE, tidy=FALSE,
               fig.path='SPDEhowto-figs/SPDEhowto-')
library(lattice);   library(gridExtra);  library(INLA);  library(plyr) 
set.seed(123)
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
lcall <- inla.getOption('inla.call')
@ 

\begin{multicols}{2} 

\section{Data simulation} 

Set $n$ \textbf{locations} and a 
exponential \textbf{covariance} matrix 
to define the \textbf{R}andon \textbf{F}ield (RF)
<<locations,tidy=FALSE>>=
n <- 200; coo <- matrix(runif(2*n), n) 
s2u <- .5; k <- 10; r <- 2/k ## RF params.
R <- s2u*exp(-k*as.matrix(dist(coo))) 
@ 

\textbf{Sample}: one multivariate Normal realization 
  
<<rmvnorm,tidy=FALSE>>=
u <- drop(rnorm(n)%*%chol(R)) 
@ 

Add a \textbf{covariate} effect and a noise (nugget) 

<<noise,tidy=FALSE>>=
x <- runif(n);  beta <- 1:2;  s2e <- 0.2
lin.pred <- beta[1] + beta[2]*x + u 
y <- lin.pred + rnorm(n, 0, sqrt(s2e)) 
@ 

\section{Model fitting} 

1. \textbf{Mesh:} 
Triangulation around the locations

<<fmesh, tidy=FALSE, pars=list(mar=c(0,0,0.7,0)), out.width='0.45\\textwidth', fig.align='center'>>= 
mesh <- inla.mesh.2d(coo, cutoff=r/10, 
  max.edge=c(r/4, r/2), offset=c(r/2, r)) 
plot(mesh, asp=1);  points(coo, col='red') 
@ 

The outer domain additional triangles 
is to avoid boundary effects. 
Is good to have aproximately 
isosceles triangles and 
to avoid tiny triangles, see \texttt{cuttof}. 
We need to have edge length 
for the inner mesh triangles 
less than the range of the process, 
if there is a spatial effect. 

2. The linear predictor at location $s$: 
\[ \eta(s) = \beta_0 + \beta_1 x(s) + A(s,s_0) u(s_0), \]
where $u(s_0)$ is the RF at the mesh nodes, 
\cite{lindgrenR:2015}, and 
$A(s,s_0)$ is a $n\times m$ 
\textbf{projection matrix} 
to project the process from the mesh 
nodes to the observation locations: 
<<projector, tidy=FALSE>>=
A <- inla.spde.make.A(mesh=mesh, loc=coo)
@ 

3. \textbf{Build the SPDE model} on the mesh, 
where $\alpha=3/2$ for the exponential correlation 
and set the \textbf{RF parameters prior} 
distributions, \cite{fuglstadetal:2017}:

<<spde,tidy=FALSE>>=
spde <- inla.spde2.pcmatern(
 mesh=mesh, alpha=1.5, 
 prior.range=c(0.2, 0.5),#P(range<0.2)=0.5
 prior.sigma=c(1, 0.5)) ## P(sigma>1)=0.5 
@ 

4. Use \texttt{inla.stack()} to deal with 
effects having different projection matrices: 

<<stack-estimation,tidy=FALSE>>=
stk.e <- inla.stack(tag='est', ## tag id
  data=list(y=y),  ## response
  A=list(1, A), ## two projection matrices
  effects=list(## two elements: 
    data.frame(b0=1, x=x), ## covariate
    idx.u=1:spde$n.spde)) ## RF index 
@ 

5. Set the \textbf{nugget prior} distribution,
the linear predictor \textbf{formula} 
and \textbf{fit} the posterior marginal distributions 
(PMD)s for all the model parameters 
using \texttt{inla()} 
<<fitting>>= 
pcprec <- list(hyper=list(theta=list(
  prior='pc.prec',param=c(1,.1))))
mf <- y ~ 0 + b0 + x + f(idx.u, model=spde) 
res <- inla(mf, control.family=pcprec,
 data=inla.stack.data(stk.e), ## data 
 control.predictor=list(compute=TRUE, 
  A=inla.stack.A(stk.e)))# full projector
@ 

\end{multicols} 

\section{Posterior marginal distributions - PMDs} 

The \texttt{summary.hyperpar} element of \texttt{res} 
has a summary from each regression coefficient PMD:
<<fixed-summary>>=
round(res$summary.fixed, 4) 
@ 

Similarly, the \texttt{summary.hyperpar} for the hyperparameters. 
The likelihood hyperparameter is parametrized as precision. 
We can work with its PMDs transforming it to standard deviation: 
<<sdlik>>=
pmd.s2e <- inla.tmarginal(function(x) sqrt(1/x), ## inverse and square root
  res$marginals.hyperpar$'Precision for the Gaussian observations')
@ 

The PMDs for the likelihood SD and RF parameters can be visualized by 
<<rfpars, pars=list(mfrow=c(1,3), mar=c(3,3,0.3,0.3), mgp=c(2,0.5,0)), fig.width=7.5, fig.height=2.5, out.width='0.99\\textwidth', out.height='0.33\\textwidth', fig.align='center', fig.pos='h', fig.keep='last'>>=
plot(pmd.s2e, type='l', ylab='Density',  xlab=expression(sigma[e]))
abline(v=sqrt(s2e), col=2) ## add the 'true' value
plot(res$marginals.hy[[3]], type='l', xlab=expression(sigma[u]), yla='Density')
abline(v=sqrt(s2u), col=2) ## add the 'true' value
plot(res$marginals.hy[[2]], type='l', xlab='range nominal', ylab='Density')
abline(v=r, col=2) ## add the 'true' value
@

To look at the PMD for the linear predictor, 
$E(Y|X,U)$, at each data location will be too much. 
For now, we can show its the 2.5\% and 97.5\% quantiles 
ordered by its mean on top of $y$ values:
<<predicts, pars=list(mfrow=c(1,1), mar=c(1.5,3,0.1,0.1), mgp=c(2,0.5,0)), fig.width=10, fig.height=2.5, out.width='0.99\\textwidth', fig.align='center', fig.pos='h', fig.keep='last'>>=
idx.obs <- inla.stack.index(stk.e, tag='est')$data ## index in the stack 
order.eta <- order(res$summary.fitted.values$mean[idx.obs]) 
plot(y[order.eta], pch=19, ylab='y')
segments(1:n, res$summary.fitted.val$'0.025quant'[idx.obs][order.eta], 
         1:n, res$summary.fitted.val$'0.975quant'[idx.obs][order.eta]) 
@ 

\section{RF projection on a grid}

An interesting result is the map of the RF on a grid. 
The simplest way to have it is by projection. 
We just have to define the projector matrix 
and project, for example, the posterior 
mean and posterior standard deviation on the grid. 

<<project-grid>>=
nxy <- c(200, 200)
gproj <- inla.mesh.projector(mesh,  xlim=0:1, ylim=0:1, dims=nxy)
g.mean <- inla.mesh.project(gproj, res$summary.random$idx.u$mean)
g.sd <- inla.mesh.project(gproj, res$summary.random$idx.u$sd)
@ 

We can visualize it by 

<<fgrid, tidy=FALSE, fig.width=9.7, fig.height=4.5, out.width='0.97\\textwidth', out.height='0.45\\textwidth', fig.pos='h'>>=
library(lattice);     library(gridExtra) 
trellis.par.set(regions=list(col=terrain.colors(16))) 
grid.arrange(levelplot(g.mean, scales=list(draw=F), xlab='', ylab='', main='mean'), 
             levelplot(g.sd, scal=list(draw=F), xla='', yla='', main='sd'), nrow=1)
@ 

\section{Prediction} 

The prediction is usually needed when one wants to know the distribution the 
expected value for the outcome given the data. 
It consider each model component, not only the random field, 
which is only one component in the model, showed in the previous section. 

First, one has to define the scenario for the prediction, 
that is the locationas and value for the covariates. 
We show an example with only three locations, 
predictions over a fine grid can also be considered, 
and covariate values set in its the mean value
<<target-loc>>=
tcoo <- rbind(c(0.3,0.9), c(0.5,0.5), c(0.7,0.3))
dim(Ap <- inla.spde.make.A(mesh=mesh, loc=tcoo)) 
x0 <- c(0.5, 0.5, 0.5)
@ 

There is more than one ways to compute the posterior marginals for 
the linear predictor. 
When predictions over a fine grid is needed, 
it will be preferable a computationally cheaper way.

\subsection{Expensive way: NA's in the response vector}

An usual way is to build a scenario, 
for fixed and random effects, 
and assign NA for the outcome. 
In this case, the linear predictor for 
such missing observations is also 
part of the model graph, \cite{Rueetal:2017},
and is treated in the entire model fitting process. 

Defining a prediction stack, join and use the full stack in \texttt{inla()}
<<prediction-stack>>=
stk.pred <- inla.stack(tag='pred', A=list(Ap, 1), data=list(y=NA), ## response as NA
  effects=list(idx.u=1:spde$n.spde, data.frame(x=x0, b0=1))) ## all idx.u
stk.full <- inla.stack(stk.e, stk.pred) ## join the data and prediction scenario
p.res <- inla(mf, data=inla.stack.data(stk.full), ## supply the full data 
  control.predictor=list(compute=TRUE, A=inla.stack.A(stk.full)), ## full 
  control.mode=list(theta=res$mode$theta, restart=FALSE))## use mode already found
@ 

Get the prediction data index and have a look into the summary
<<prdind>>=
pred.ind <- inla.stack.index(stk.full, tag='pred')$data
round(p.res$summary.fitted.val[pred.ind,], 4)
@ 

Collect the linear predictor PMDs to work with, 
and isualize with commands bellow
<<ppred, tidy=FALSE, fig.width=9.9, fig.height=3.5, out.width='0.99\\textwidth', out.height='0.35\\textwidth', fig.pos='h', fig.keep='last'>>=
ypost <- p.res$marginals.fitted.values[pred.ind]
names(ypost) <- paste('y', seq_along(ypost), sep='_');     library(plyr) 
xyplot(y~x | .id, ldply(ypost), panel='llines', xlab='y', ylab='Density')
@ 

\begin{multicols}{2} 
%\addtolength{\linewidth}{2in}
In \textbf{\textsf{INLA}} we have some functions to work 
with marginals distributions 

<<echo=FALSE>>=
options(width=43)
@ 

<<marginals-funcs, comment=NA>>=
apropos('marginal')
@ 

<<marginals-examples>>=
## inla.mmarginal(ypost[[1]]) ## mode
inla.qmarginal(c(0.15, 0.7), 
               ypost[[1]]) ## quantiles
inla.pmarginal(inla.qmarginal(
    0.3, ypost[[1]]), ypost[[1]]) 
@ 
\end{multicols} 

\subsection{Cheaper way: Monte Carlo samples}

The way we show here is cheaper if the number of locations 
to be predicted is not small. 
The idea is to drawn samples from the joint posterior distribution. 
As any functional of interest can be computed we can compute 
the linear predictor at a set of target locations. 

We can ask \texttt{inla()} to store the precision matrix for each 
hyperparameter configuration and use it to drawn Monte Carlo samples, 
\cite{Rueetal:2017}. 
The \texttt{inla.posterior.sample()} function drawn samples 
from each hyperparameter configuration using its relative posterior 
as weights and them sample from the latent field for each 
hyperparameter sampled configuration. 

We can set \texttt{control.compute=list(cofig=TRUE)} 
before the \texttt{inla()} to have such configurations. 
Or, we can rerun the model after asking it 
<<configs, warning=FALSE, message=FALSE>>=
res$.args$control.compute$config <- TRUE
res <- inla.rerun(res)
@ 
<<echo=FALSE,results=FALSE>>=
inla.setOption(inla.call=lcall) 
@ 

Drawn Monte Carlo samples 
<<samples>>=
samples <- inla.posterior.sample(n=2e3, result=res, add.names=FALSE)
@ 

We have to find the index set for the elements we need samples, 
the fixed effecs ('b0' and 'x') and the random effect ('s'). 
The names were stored for the first sample as rownames of the latent field
<<xnames>>=
xnames <- rownames(samples[[1]]$latent) ### collect the names
idx <- lapply(c('b0', 'x', 'idx.u'), function(nam) ## for each effect
    which(substr(xnames, 1, nchar(nam))==nam)) ## find the index
@ 

These indexes are used to collect the desired part of the latent field 
and organize it into a matrix
<<samplesmat>>=
mat.samples <- sapply(samples, function(spl) 
    c(bo=spl$latent[idx[[1]]], x=spl$latent[idx[[2]]], u=spl$latent[idx[[3]]]))
@ 

The next step is to compute the linear predictor 
for the scenario needed for each sample.  
<<y3sample>>=
eta3.sample <- as.matrix(cbind(b0=1, x=0.5, u=Ap)%*%mat.samples)
@ 
We can visualize it comparing wit the previous result with
<<comparey3, fig.width=9.9, fig.height=3.5, out.width='0.99\\textwidth', fig.pos='h', fig.keep='last'>>=
par(mfrow=c(1,3), mar=c(3,3,1,1), mgp=c(2,1,0))
for (j in 1:3) {
    hist(eta3.sample[j,], freq=FALSE, xlab=paste0('y',j), main='', col=gray(0.7))
    lines(ypost[[j]], lwd=2, col='blue')
}
@ 

\begin{multicols}{2}
\subsection{Prediction on a grid, cheaper way}

We can consider the previously defined grid projection matrix and samples 
<<linpredsample>>=
eta.g.samp <- as.matrix(cbind(b0=1, x=0.5, 
        s=gproj$proj$A)%*%mat.samples)
@ 

Any summary statistics can be computed from that. 
Computing the mean, standard deviation, 
$P(E(Y)>3)$ and the 95\% quantile for $E(Y)$: 
<<meansdgrid,warning=FALSE,message=FALSE>>=
library(matrixStats)
ss <- list(mean=rowMeans(eta.g.samp), 
 sd=rowSds(eta.g.samp), 
 p3=rowMeans(eta.g.samp>3), 
 q95=rowQuantiles(eta.g.samp, probs=0.95))
@ 

It is visualized with the commands bellow 
(locations added on top of the sd map): 

<<predmaps, eval=FALSE>>=
library(fields); for (s in c(1,3,4,2)) 
  image.plot(list(x=gproj$x, y=gproj$y, 
      z=matrix(ss[[s]], nxy[1])), asp=1,
    legend.mar=3, main=names(ss)[s]) 
points(coo, pch=4, cex=0.5)
@ 

\bibliographystyle{apalike}
\bibliography{spde-tutorial}

<<predmapsview,echo=FALSE, pars=list(mfrow=c(4,1), mar=c(1.5,2,1,0.5), mgp=c(1,0.5,0)), fig.width=3.75, fig.height=11.7, out.width='0.49\\textwidth', fig.pos='h', fig.align='center'>>=
<<predmaps>>
@ 
\end{multicols}

\end{document} 

---
title: "Blended GEV: a tutorial using R-INLA"
subtitle: "Version 1.2"
author: \normalfont{Daniela Castro-Camilo {\small{$\bullet$}} daniela.castrocamilo@glasgow.ac.uk}
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{Blended GEV}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
header-includes:
  - \usepackage{xcolor}
---
\def\F{\text{F}}
\def\G{\text{G}}
\def\H{\text{H}}
```{r setup, include=FALSE}
set.seed(123)
library(knitr)
knitr::opts_chunk$set(echo = TRUE, results = 'show', 
	fig.keep = 'high', fig.align = 'center', fig.pos = 'h')
knitr::opts_chunk$set(fig.path="figures/bgev/")

library(INLA)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(smtp="taucs")
inla.setOption(num.threads="1:1")
```


```{r, echo=F, message=F, warning=FALSE}
colFmt = function(x,color){
  outputFormat = opts_knit$get("rmarkdown.pandoc.to")
  if(outputFormat == 'latex')
    paste("\\textcolor[HTML]{",color,"}{",x,"}",sep="")
  else if(outputFormat == 'html')
    paste("<font color='",color,"'>",x,"</font>",sep="")
  else
    x
}

library(INLA)
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}
```

# A reparametrisation of the GEV

The blended Generalised extreme value (bGEV) model is an alternative
to the usual GEV distribution when the tail parameter $\xi$ is
positive, and it is designed to tackle the \emph{artificial boundary
restrictions} inherited by the GEV. By artificial boundary
restrictions, we mean the following: in practical applications, we
assume that the GEV is a reasonable approximation for the distribution
of maxima over blocks, and we fit it accordingly. This implies that
GEV properties, such as finite lower endpoint in the case $\xi>0$, are
inherited by the original maxima distribution, which might not be
bounded-supported. This is particularly problematic in the presence of
covariates.


Before defining the bGEV distribution, we need to introduce an
alternative parametrisation for the GEV. The GEV distribution
parametrised in terms of the \textbf{location}
$q_\alpha\in\mathbb{R}$, the \textbf{spread} $s_\beta\in(0,\infty)$
and the \textbf{tail} parameter $\xi\in\mathbb{R}$ has the form

$$ F(y\mid q_\alpha,s_\beta,\xi) =
	\exp\left\{-\left[\left(\frac{y-q_\alpha}{s_\beta(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})^{-1}} +
	\ell_{a,\xi}\right)\right]_+^{-1/\xi}\right\},$$ where
	$a_+=\max(a,0)$ and for any $a>0$, $\ell_{a,\xi} = \{-\log
	a\}^{-\xi}$. Note that the case $\xi=0$ simplifies to $$F(y\mid
	q_\alpha,s_\beta) = \exp\left\{-\exp\left[-\left(\frac{y -
	q_\alpha}{s_\beta(\ell_{1-\beta/2} - \ell_{\beta/2})^{-1}} -
	\ell_\alpha\right)\right]\right\}, 
$$

with $\ell_a = \log\{-\log a\}.$ There is a one-to-one mapping between
$(q_\alpha, s_\beta, \xi)$ and the usual location-scale-shape GEV
parameters, $(\mu,\sigma,\xi)$ (see, e.g., `r colFmt("Coles, 2001,
Chapter 3",'0352fc')`). For the case $\xi\neq 0$, the mapping is given
by

$$
	\mu=q_\alpha-\frac{s_\beta(\ell_{\alpha,\xi}-1)}{\xi(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})}, \qquad \sigma =
	\frac{s_\beta}{(\ell_{1-\beta/2,\xi} -
	\ell_{\beta/2,\xi})},\qquad\xi = \xi. 
$$

The case $\xi=0$ is interpreted as the limit when $\xi\to0$, i.e.,
$$\mu = q_\alpha + \frac{s_\beta\ell_\alpha}{\ell_{\beta/2} -
\ell_{1-/\beta/2}},\qquad \sigma = \frac{s_\beta}{\ell_{\beta/2} -
\ell_{1-/\beta/2}}.$$

This reparametrisation is proposed to provide a more meaningful
interpretation of the parameters. In statistics, the location-scale
parametrisation is quite popular as it relates to the mean and the
standard deviation of the distribution. In skewed distributions such
as the GEV, the mean is no longer a reasonable proxy for the location
of the distribution. Moreover, the mean and variance of the GEV are
only defined when $\xi<1$ and $\xi<0.5$, respectively. This effect of
the tail parameter over the mean and variance does not allow us to
interpret the location-scale (and tail) GEV parametrisation as we do
in other models. This problem is particularly troublesome for the case
where the parameters vary according to a set of covariates. Assigning
sensible priors to the GEV distribution with the usual parametrisation
is also tricky when the mean and variance are not defined.


# The blended GEV model

The bGEV distribution is defined as

\begin{equation}\label{eq:bGEV}
	\H(x \mid \theta) = \F(x \mid q_{\alpha_{\F}}, s_{\beta_{\F}},
	\xi)^{p(x \mid s_{p_1}, s_{p_2}, a, b)} 
	\G(x \mid q_{\alpha_{\G}}, s_{\beta_{\G}})^{1-p(x \mid s_{p_1}, s_{p_2}, a, b)},
\end{equation}

where $\theta = (q_{\alpha_{\F}}, s_{\beta_{\F}}, \xi,
q_{\alpha_{\G}}, s_{\beta_{\G}}, s_{p_1}, s_{p_2}, a, b)$, $\F$ is the
Frechét (or type II GEV) distribution with location parameter
$q_{\alpha_{\F}}$, spread parameter $s_{\beta_{\F}}$, and
\textbf{tail} parameter $\xi$, and $\G$ is the Gumbel (or type I GEV)
distribution with location parameter $q_{\alpha_{\G}}$ and spread
parameter $s_{\beta_{\G}}$. The function $p$ is a weight function
defined as the cumulative distribution function of a Beta distribution
with shape parameters $s_{p_1}>1$ and $s_{p_2}>1$, evaluated in the
point $(x - a)/(b - a)$, i.e.,

\begin{equation}\label{eq:weightfunction}
	p(x \mid s_{p_1},s_{p_2}, a, b) = \text{Pr}\left(Y\leq \frac{x-a}{b-a} \mid s_{p_1}, s_{p_2}\right),
\end{equation}

where $Y$ follows a Beta distribution with shape parameters
$s_{p_1}>1$ and $s_{p_2}>1$. The Beta weight controls the way the
distributions $\F$ (Frechét) and $\G$ (Gumbel) influence the model
$\H$. The lower and upper bounds of the weight function ($a$ and $b$,
respectively) define the \emph{mixing area}, i.e., where $F$ and $G$
are merged (see Figure \ref{fig:distributionH.pdf}). Here, we choose
them as quantiles of the Frechét distribution, i.e., $a=F^{-1}(p_a)$
and $b=F^{-1}(p_b)$, with $0<p_a,p_b<1$. Below, $p_a$ and $p_b$ will
be refered as the \emph{mixing area quantiles}. The current INLA
implementation assumes that $s_{p_1}=s_{p_2}=5$.


```{r pressure, echo=FALSE, fig.cap="\\label{fig:distributionH.pdf} bGEV distribution (H, black) constructed from distributions F (Frechét, red), G (Gumbel, green) and Beta weight function $p$ (purple). The shaded area is the mixing area, where F and G are merged.", out.width = '50%'}
knitr::include_graphics("fig_distribution1.png")
```

In the following sections, we will learn how to fit the bGEV using
R-INLA using three simulated examples with increasing level of
complexity. <!-- Additionally, we also provide the data application in
the bGEV article (`r colFmt("Castro-Camilo et al., 2020",'0352fc')`).
-->

# Simulated example 1

To get familiar with the bGEV R-INLA implementation, we consider a
simple model where the linear predictor is linked to the
$\alpha-$quantile, $q_\alpha$. The model we want to fit is

\begin{equation}\label{eq:sim.model1}
	q_\alpha = \eta(\texttt{x}) =  1 + 0.4\texttt{x}
\end{equation}

## Data simulation{#sec:datasim1}

We start by generating $n=1000$ samples from \eqref{eq:sim.model1}

```{r}
n = 1000
x = rnorm(n) # we generate values for x from a N(0,1) dist.
eta.x = 1 + 0.4*x
```

The spread and tail parameters are assumed to be covariate-free and
unknown and are treated as hyperparameters within the INLA framework.
We assume that the true spread and tail parameters are 0.3 and 0.1,
respectively.

```{r}
spread = 0.3
tail = 0.1
```

To generate the GEV samples, we need to define the probabilities
$\alpha$ and $\beta$ that define the location ($q_\alpha$) and spread
($s_\beta$) parameters, respectively. In our case, they will be fixed
to $\alpha=0.5$ (the median) and $\beta=0.25$: <!-- In our case, they
will be fixed to $\alpha=0.5$ (the median) and $\beta=0.5$ (so then
$s_\beta$ is the interquartile range): -->

```{r}
p.alpha = 0.5
p.beta = 0.25
```
 
Now we are ready to generate the samples. We use the function
[$\texttt{giveme.gev.par}$](#sec:tools) (See [Section 5](#sec:tools))
to obtain the usual GEV parameters ($\mu,\sigma,\xi$) and plug them
into the $\texttt{evd::rgev}$ function to generate the samples

```{r}
par = giveme.gev.par(q = eta.x, sbeta = spread, alpha = p.alpha, beta = p.beta, 
                     xi = tail)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma, shape = par$xi)
```

## Prior specification{#sec:prior1}

The default prior distribution for the spread $s_\beta$ is a Gamma
with shape and rate parameters equal to 3 (note that a log-scale is
used below). For the tail parameter we consider a PC prior approach
with parameters $\lambda = 7$, $\texttt{low} = 0$ and
$\texttt{high}=0.5$. For more details on the PC prior for the tail GEV
parameter see [Section 4.2](#sec:pcpriortail).

For the sake of illustration, we here define the priors for all the
parameters involved. The priors for the spread is

```{r}
hyper.spread = list(initial = 1,
                    fixed=FALSE,
                    prior = "loggamma",
                    param = c(3, 3))
```

The prior for the tail parameter requires a bit more explanation. For
computational reasons, R-INLA uses an internal parametrisation of the
tail parameter, which is unbounded. The hyperparameter specification
is defined for this internal parameter instead of the usual one, so in
order to know how to specify a prior for the tail parameter, we need
to understand how both parametrisations are connected.

The map $\phi:[0,\infty)\to\mathbb{R}$ specifies the link between the
usual and the internal parametrisations and it is defined as
$$\phi(\xi_{\text{int}}) = \texttt{low} +
(\texttt{high}-\texttt{low})\frac{\exp(\xi_{\text{int}})}{1+\exp(\xi_{\text{int}})}:=
\xi,$$ where $\xi\in[0,\infty)$ is the usual tail parameter and
$\xi_{\text{int}}$ refers to the unbounded internal tail parameter.
The interval $(\texttt{low},\texttt{high})$ constraints the possible
values for $\xi$. The map and its inverse are defined in the function
[$\texttt{map.tail}$](#sec:tools) (See [Section 5](#sec:tools)).

In the internal parametrisation, the default initial value is -4 with
$(\texttt{low},\texttt{high}) = (0,0.5)$, which correspond to
$\xi\approx0.04$. If we want to provide an initial value of $\xi=0.1$,
then we can do

```{r}
tail.interval = c(0, 0.5)
tail.intern = map.tail(tail, tail.interval, inverse=TRUE)
```

We kept $(\texttt{low},\texttt{high}) = (0,0.5)$ to ensure the existence of second moments\footnote{Recall that the variance for the GEV is defined for $\xi<0.5$.}.

The prior for the tail parameter is defined as

```{r}
hyper.tail = list(initial = tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= FALSE)

```

\textbf{Note:} if we have reasons to believe that $\xi=0$ (Gumbel
case), then a good initial value is $\infty$ (in the internal
parametrisation). We can generalise the prior specification for the
tail to allow $\xi>0$ or $\xi=0$ as follows

```{r}
hyper.tail = list(initial = if (tail == 0.0) -Inf else tail.intern, 
                  prior = "pc.gevtail",
                  param = c(7, tail.interval), 
                  fixed= if (tail == 0.0) TRUE else FALSE)

```

Therefore, the (default) hyperparameter specification for the bGEV model is

```{r}

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail)
```

## Control variables {#sec:control1}

As part of the $\texttt{control.family}$ argument in R-INLA, the
argument $\texttt{control.bgev}$ allows us to include additional bGEV
parameters. Specifically, the probabilities $\alpha$ and $\beta$, the
mixing area quantiles $p_a$ and $p_b$, and the Beta weight function
parameters, which we know are equal and fixed to 5 (for now, this
cannot be changed).

```{r}
control.bgev = list(q.location = p.alpha,
                    q.spread = p.beta,
                    # quantile levels for the mixing part
                    q.mix= c(0.05, 0.20), 
                    # the Beta(s1, s2) mixing distribution parameters. 
                    # Hard-coded for the moment: s1=s2=5
                    beta.ab = 5)
```

## INLA fit{#sec:fit1}

The INLA formula for the bGEV model uses the function
[$\texttt{inla.mdata}$](#sec:inlamdata) to allow the inclusion of
simple linear models in the spread and the tail parameters (for
details on the $\texttt{inla.mdata}$ function see [Section
4.3](#sec:inlamdata)). A null matrix can be used to indicate that
these parameters are covariate-free (as it is the case in this
example).

```{r}
null.matrix = matrix(nrow = n, ncol= 0)
spread.x = null.matrix
tail.x = null.matrix
```

\textbf{Note:} matrices for the spread and tail covariates (here
spread.x and tail.x) should always be defined and passed to the R-INLA
formula, even if they are empty.

Then, the INLA data and formula can be defined as
```{r}
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x
```

\textbf{Note:} $\texttt{data.bgev}$ only contains three columns, as
null matrices cannot be passed to data frames in R. As
$\texttt{spread.x}$ and $\texttt{tail.x}$ are not defined in
$\texttt{data.bgev}$, INLA will search for these variables in the
global R environment. Therefore, $\texttt{spread.x}$ and
$\texttt{tail.x}$ should be defined as null matrices in the global
environment (as we do here). We write $\texttt{data.bgev}$ this way
for consistency with the following examples.

Finally, we fit the model

```{r, message=F, eval = T, warning=FALSE}
r1 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE)
```

A summary of the fitted fixed effects and hyperparameters can be
obtained as follows

```{r}
round(r1$summary.fixed,4)
round(r1$summary.hyperpar,4)
```



<!-- ```{r, fig.width=4.5, fig.height=3.5, fig.cap="\\label{fig:pit_r1.pdf} The probability integral transofrm (PIT) is a \\textit{leave-one-out} predictive measure of fit. It is computed as the probability of a new response being less than the observed response using a model based on the rest of the data. If the model assumptions are correct, we would expect the PIT values to be uniformly distributed."} -->
<!-- round(r1$summary.fixed,4) -->
<!-- round(r1$summary.hyperpar,4) -->
<!-- hist(r1$cpo$pit, freq = F, col = 'lightblue', border = 'white', -->
<!--      main = 'PITs for model r1', xlab = '') -->
<!-- abline(h = 1, lty = 2, col = 2) -->
<!-- ``` -->

# Simulated example 2

Although R-INLA does not allow more than one linear predictor, the
bGEV implementation does allow for simpler regression models on the
spread and tail parameters. We then extend model \eqref{eq:sim.model1}
as follows

\begin{align}\label{eq:sim.model2}
q_\alpha &= \eta(\texttt{x}_{1}) =  1 + 0.4\texttt{x}_{1}\nonumber\\
s_\beta &= \exp(0.1 + 0.3\texttt{x}_{2})\nonumber\\
\xi &= 0.1 + 0.2\texttt{x}_{3}
\end{align}

<!-- \xi_{\text{int}} &= \phi^{-1}(0.1) + 0.1\texttt{x}_{3}\nonumber\\ -->
<!-- \xi &= \phi(\xi_{\text{int}}) -->
<!-- Note that we provide the model for the usual tail parameter, $\xi$, in terms of the internal tail parameter $\xi_{\text{int}}$. We want the true $\xi$ to be close to 0.1 and define the covariate-dependent model in terms of $\xi_{\text{int}}$ to avoid getting unreasonable values for $\xi$. -->

## Data simulation{#sec:datasim2}

As before, we start by generating $n=1000$ samples from \eqref{eq:sim.model2}

```{r}
n = 1000
x1 = rnorm(n)
eta.x = 1 + 0.4*x1
```

The spread and tail parameter can be simulated as follows

```{r}
x2 = rnorm(n)
s.x = exp(0.1 + 0.3*x2)
x3 = runif(n,-0.25,1)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi
```

Note that since $-0.5<\texttt{x}_{3}<2$, we have that $0<\xi<0.5$. As
before, we use the function [$\texttt{giveme.gev.par}$](#sec:tools) to
obtain the usual GEV parameters and plug them into the
$\texttt{evd::rgev}$ function to generate the samples (note that
$\alpha$ and $\beta$ are the same as before)

```{r, eval = T}
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])
```
 
## Prior specification{#sec:prior2}

Additional to what was discussed in [Section 3.2]({#sec:prior1}), we
can also adjust the priors for the regression coefficient of the
covariates for the spread and tail parameters, which we will call
$\beta_1$ and $\beta_2$. Within INLA, $\beta_1$ and $\beta_2$ are
treated as hyperparameters, with default prior given by a zero-mean
Gaussian distribution with precision equal to 300, as specified below.

```{r}
hyper.beta1 = hyper.beta2 = list(prior = "normal",
                                 param = c(0, 300),
                                 initial = 0)
```

In this case, the (default) hyperparameter specification for the bGEV model is

```{r}

hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2)
```


## INLA fit{#sec:fit2}

As mentioned in [Section 3.4]({#sec:fit1}), we can use the
[$\texttt{inla.mdata}$](#sec:inlamdata) function to define linear
models for the spread and the tail parameters.

```{r}
spread.x = x2
tail.x = x3
formula = inla.mdata(y, spread.x, tail.x) ~ -1 + intercept + x
```

Then, the INLA data can be defined as
```{r}
data.bgev = data.frame(y = y, intercept = 1, x = x, spread.x = spread.x, tail.x = tail.x)
```

We fit the model using the same priors and mixing area quantiles
($p_a$, $p_b$) as before.

```{r, message=F, eval = T, warning=FALSE}
r2 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb"),
         verbose=FALSE)

```

A summary of the fitted fixed effects and hyperparameters can be
obtained as follows

```{r, eval = T}
round(r2$summary.fixed,4)
round(r2$summary.hyperpar,4)
```


<!-- ```{r, eval = T, fig.width=4.5, fig.height=3.5, fig.cap="\\label{fig:pit_r2.pdf} The probability integral transform for model $\texttt{r2}$."} -->
<!-- round(r2$summary.fixed,4) -->
<!-- round(r2$summary.hyperpar,4) -->
<!-- hist(r2$cpo$pit, freq = F, col = 'lightblue', border = 'white', -->
<!--      main = 'PITs for model r2', xlab = '') -->
<!-- abline(h = 1, lty = 2, col = 2) -->
<!-- ``` -->

# Simulated example 3

The linear predictor can include more complicate structures defined as
functions that depend on a set of covariates. By varying the form of
these functions, we can accommodate a wide range of models, from
standard and hierarchical regression to spatial and spatio-temporal
models (`r colFmt("Rue et al., 2009",'0352fc')`). Here we extend the
model in \eqref{eq:sim.model2} by assuming the following structure

\begin{align}\label{eq:sim.model3}
	q_\alpha &= \eta(\texttt{x}_{1},\texttt{z}_{1},\texttt{z}_{2}) =  1 + 0.4\texttt{x}_{1} + f_1(\texttt{z}_{1}) + f_2(\texttt{z}_{2})\nonumber\\
	s_\beta &= \exp(0.1 + 0.3\texttt{x}_{2} + \texttt{x}_{4})\nonumber\\
	\xi &= 0.1 + 0.2\texttt{x}_{3}
\end{align}

where $f_1$ is a random walk of order 1 and $f_2$ is an autoregressive
process of order 2. Note that we also extended the linear model for
the spread parameter.

## Data simulation{#sec:datasim3}

There are many ways we can simulate dara to fit \eqref{eq:sim.model3}.
One alternative is

```{r, eval = T}
n = 1000
x1 = rnorm(n)
z1 = seq(0, 6, length.out = n)
z2 = 1:n
p = 2 # AR order
pacf = runif(p)
phi = inla.ar.pacf2phi(pacf)
eta.x = 1 + 0.4*x + sin(z1) + c(scale(arima.sim(n, model = list(ar = phi))))
```

The spread and tail parameter are simulated as before:

```{r}
x2 = rnorm(n)
x4 = rnorm(n)
s.x = exp(0.1 + 0.3*x2 + x4)
x3 = runif(n,-0.5,2)
t.x = 0.1 + 0.2*x3
tail.intern = map.tail(t.x, tail.interval, inverse=TRUE) # internal xi
```

The data are simulated as

```{r, eval = T}
par = giveme.gev.par(q = eta.x, sbeta = s.x, alpha = p.alpha, beta = p.beta, 
                     xi = t.x)
y = numeric(n)
for(i in 1:n) 
  y[i] = rgev(1, loc = par$mu[i], scale = par$sigma[i], shape = par$xi[i])
```

## INLA fit{#sec:fit3}

We have two covariates for the spread parameter and one for the tail,
and we can have priors for each of their coefficients. Below,
$\texttt{hyper.beta1}$ and $\texttt{hyper.beta2}$ are the priors for
the coefficients of the covariates in the spread parameters, while
$\texttt{hyper.beta3}$ is the prior for the coefficient of the
covariate in the tail parameter.

```{r}
hyper.beta1 = hyper.beta2 = hyper.beta3 = list(prior = "normal",
                                               param = c(0, 300),
                                               initial = 0)
hyper.bgev = list(spread = hyper.spread, 
                  tail = hyper.tail,
                  beta1 = hyper.beta1,
                  beta2 = hyper.beta2,
                  beta3 = hyper.beta3)
```

Using the same mixing area quantiles ($q_a$, $q_b$) as before, we only
need to specify the INLA formula, the new data, and run the model.

```{r, message=F, eval = T, warning=FALSE}
spread.x = x2
spread.xx = x4
tail.x = x3
# With this change of variable it is easier to keep track of the effect 
# of the covariates in each parameter, but it is not needed.
formula = inla.mdata(y, cbind(spread.x, spread.xx), tail.x) ~ -1 + intercept + x + 
	f(z1, model = "rw1", scale.model=TRUE, constr=TRUE,
		hyper = list(prec = list(prior = "pc.prec", 
		param = c(0.1, 0.01)))) +
		f(z2, model = 'ar', order = 2, 
		   hyper=list(prec=list(prior="pc.prec", constr=TRUE,
		                        param=c(0.1,0.01)),
                      pacf1=list(param=c(0.5,0.8),
                      pacf2=list(param=c(0.5,0.8)))))
data.bgev = data.frame(y = y, intercept = 1, x = x, z1 = z1, z2 = z2,
                       spread.x = spread.x, spread.xx = spread.xx, tail.x = tail.x)
r3 = inla(formula,
         family = "bgev",
         data = data.bgev,
         control.family = list(hyper = hyper.bgev,
                               control.bgev = control.bgev),
         control.predictor = list(compute = TRUE),
	     control.fixed = list(prec=100,prec.intercept=100),
         control.compute = list(cpo = TRUE),
         control.inla = list(int.strategy = "eb",cmin=0),
         verbose=FALSE)
```

A summary of the fitted effects and hyperparameters can be obtained as
follows

```{r, eval = T}
round(r3$summary.fixed,4)
round(r3$summary.hyperpar,4)
```


<!-- ### Model fitting -->



<!-- ## Example using pollution data -->
<!-- Here we replicate the application in Castro-Camilo et al. (2020). Specifically, we want to fit the model -->


<!-- The data is available at <https://github.com/dcastrocamilo/bgev>. -->

# Notes{#sec:notes}

## Linear predictor for the bGEV model{#sec:lpextensions}

The linear predictor can take any combinations of the latent
structures currently implemented. To see a list of the available
latent models, we can do

```{r, eval=F}
library(INLA)
inla.list.models('latent')
```


## PC prior for the tail parameter{#sec:pcpriortail}

Although non-informative priors are a common choice when little expert
knowledge is available, the PC prior approach allows us to select
moderately informative prior distributions in a $\textit{reasonable}$
way. This procedure penalises excessively complex models at a constant
rate by putting an exponential prior on a distance (specifically, the
Kullback-Leibler distance, or KLD) to a simpler baseline model.

The PC prior for the tail GEV parameter is defined in terms of the KLD
$\xi^2/(1-\xi)$ for $0\leq \xi<1$. In R-INLA, the prior is specified
as $$\texttt{hyper = list(<theta> = list(prior="pc.gevtail",
param=c(<lambda>, <low>, <high>)))}$$ where

* $\texttt{<lambda>}$: the constant penalisation rate
* $\texttt{<low>,<high>}$: interval to restrict possible values for the tail parameter. The default is [0,0.5].

For more details on the PC prior for the tail GEV parameter see
$\texttt{inla.doc("pc.gevtail")}$.

## About $\texttt{inla.mdata}${#sec:inlamdata}

$\texttt{inla.mdata(y, x1, x2,...)}$ is a matrix where each row are
replicates, and responses that are $\texttt{NA}$s are ignored. The
function $\texttt{inla.mdata}$ accept covariates that are one or many
vectors, matrices or data frames. If we pass $m$ covariates
$\texttt{x}_1,\ldots\texttt{x}_m$, then each row $(\texttt{x}_{i1},
\texttt{x}_{i2}, \ldots, \texttt{x}_{im})$ defines the covariates used
for the $i$-th row of $\texttt{y}$.

# Tools{#sec:tools}

The function $\texttt{giveme.gev.par}$ computes the usual GEV
parameters $(\mu,\sigma,\xi)$ given the bGEV parameters
$(q,s_\beta,\xi)$. Note that in both parametrisations, the tail
parameter is the same.

```{r}
library(evd)
giveme.gev.par = function(q, sbeta, alpha, beta, xi) 
{
  .mu = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    c = -log(alpha)
    if (all(xi > 0.0)) {
      tmp0 = (c^(-xi) - 1)/xi
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(q - (sbeta/dbeta) * tmp0)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      tmp0 = log(c)
      return(q + (sbeta/dbeta) * tmp0)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  .sigma = function(q, sbeta, alpha, beta, xi) {
    a = -log(1-beta/2)
    b = -log(beta/2)
    if (all(xi > 0.0)) {
      tmp1 = a^(-xi)
      tmp2 = b^(-xi)
      dbeta = (tmp1 - tmp2)/xi
      return(sbeta/dbeta)
    } else if (all(xi == 0.0)) {
      dbeta = log(b) - log(a)
      return(sbeta/dbeta)
    } else {
      stop("mixed case not implemented")
    }
  }
  
  return(list(mu = .mu(q, sbeta, alpha, beta, xi),
              sigma = .sigma(q, sbeta, alpha, beta, xi),
              xi = xi))
}

```

The function $\texttt{map.tail}$ specifies the link between the
internal and usual parametrisations. In the code below,
$\texttt{interval}$ constraints the possible values for the tail
parameter, while $\texttt{inverse}$ is a boolean variable indicating
whether $\phi$ or $\phi^{-1}$ should be computed.


```{r}
map.tail = function(x, interval, inverse = FALSE) {
  if (!inverse) {
    return (interval[1] + (interval[2] - interval[1]) * exp(x)/(1.0 + exp(x)))
  } else {
    return (log((x-interval[1])/(interval[2]-x)))
  }
}
```

\textbf{\Large{References}}

1. Coles, S. (2001). \emph{An introduction to statistical modeling of
   extreme values} (Vol. 208, p. 208). London: Springer.

2. Rue, H., Martino, S., & Chopin, N. (2009). Approximate Bayesian
   inference for latent Gaussian models by using integrated nested
   Laplace approximations. \emph{Journal of the royal statistical
   society: Series B (statistical methodology)}, 71(2), 319-392.

---
title: "User defined integration points"
author: "Haavard Rue `(hrue@r-inla.org)`"
date: "May 29th 2017 (updated April 9th 2019)"
output: rmarkdown::pdf_document
vignette: >
  %\VignetteIndexEntry{User defined integration points}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8} 
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE, cache = FALSE)
knitr::opts_chunk$set(fig.path="figures/int-design/")
Design = matrix()
set.seed(123)
library(INLA)
if (file.exists("myinit.R")) source("myinit.R")
inla.setOption(num.threads="1:1")
inla.setOption(smtp="taucs")
```

## Introduction

This short note describe a new option that allow the user to use
user-defined integration points (or "design" points), instead of the
default ones. The relevant integration in INLA does
$$
	\int f(x | \theta, y) \; \pi(\theta | y)
		\;
			d\theta = f(x | y)
$$
where $\pi(\theta | y)$ is the approximated posterior marginal for the
hyperparameters, and where $f(x | \theta, y)$ is the approximated
marginal for $x$ for that configuration.
The output of this integral is the posterior marginal $f(x|y)$.
In practice, we use a discrete set of integration points for $\theta$,
and corresponding weights $w$, to  get
$$
	f(x|y) \approx \sum_i f(x|\theta_i, y) w_i \pi(\theta_i | y)
$$
for which we require $w_i \ge 0$ and $\sum_i w_i = 1$.
Usually, the integration is done in a *standardised scale*,
$$
z = A(\theta - \gamma)
$$
i.e.\ with respect to $\pi(z|y)$. Here, 
$\gamma$ is the mode of $\pi(\theta|y)$ and the matrix $A$ is
the negative square root of the approximated covariance matrix for $\theta|y$
at the mode. 

The relevant options
are
```{r}
	opts = control.inla(int.strategy = "user", int.design = Design)
```
where `Design` is a matrix with the integration points and the
integration weights. The $j$th row of `Design` consists of the values
$\theta_j = (\theta_{1j}, \ldots, \theta_{mj})$, and the integration
weight for this configuration, $w_j$. The values are in the
$\theta$-scale, meaning that you have to know exactly what you are
doing, including knowing the ordering of the hyperparameters.

Another version, is to define the points in the standardised scale
$z$. To do this, use
```{r, eval=FALSE}
	opts = control.inla(int.strategy = "user.std", int.design = Design)
```
instead. The meaning of `Design` is unchanged, except that these can
be given in standardised coordinates.
This version is more relevant if you want to
implement a generic new integration design instead of the ones
already provided.

## First example 

In this artificial example, we want to compute the change of the
marginal variance of one component, $x_1$, of a hidden AR$(1)$
process, with respect to lag one correlation $\rho$. So we want to
compute
$$
\frac{\partial \mbox{SD}(x_1|y)}{\partial \rho}
$$
for a fixed value of $\rho = \rho_0$.
We have to compute a numerical approximation, using finite difference.
While doing this, it is a good idea to keep the design fixed, to avoid
introducing an error for changing that part as well.

Let us first setup the experiment
```{r, eval=TRUE, echo=TRUE}
n = 100
rho = 0.9
x = scale(arima.sim(n, model = list(ar = rho)))
y = x + rnorm(n, sd = 0.1)
```
this gives the following
```{r}
plot(y, xlab = "time", ylab = "value")
lines(x, lwd=2)
```
To compute the derivative, we do
```{r}
rho.0 = rho
to.theta = inla.models()$latent$ar1$hyper$theta2$to.theta
rho.0.internal = to.theta(rho.0)

r = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.0.internal,
		              fixed=TRUE))),
    control.inla = list(int.strategy = "grid"),
	data = data.frame(y, time = 1:n))

sd.0 = r$summary.random$time[1,"sd"]
print(sd.0)
```
The ordering of the hyperparamters are as follows,
```{r}
nm = names(r$joint.hyper)
nm = nm[-length(nm)]
print(nm)
```
which may sometimes be useful to know about.

Anyway, we will now change $\rho$ a little, while we keep the same
integration
points,
```{r}
Design = as.matrix(cbind(r$joint.hyper[, seq_along(nm)], 1))
head(Design)
```
where the last column is the (un-normalised) integration weights.
`Design` has dimension `r dim(Design)`.
We  call `inla()` again reusing the previous found mode
```{r}
h.rho = 0.01
rho.1.internal = to.theta(rho.0 + h.rho)
rr = inla(y ~ -1 + f(time, model="ar1",
	hyper = list(
		theta1 = list(prior = "loggamma",
		              param = c(1,1)),
	    theta2 = list(initial = rho.1.internal,
		              fixed=TRUE))),
	control.mode = list(result = r, restart=FALSE),	
    data = data.frame(y, time = 1:n),
	control.inla = list(
		int.strategy = "user",
		int.design = Design))
sd.1 = rr$summary.random$time[1,"sd"]
print(sd.1)
```
and then our estimate of the derivative is
```{r}
deriv.1 = (sd.1 - sd.0) / h.rho
print(deriv.1)
```

PS: In the logfile of the `inla()`-call, the configurations are shown
in the $z$ scale even for `int.strategy="user"`.


## Second example

There is also another (experimental) option, that is
```{r, eval=FALSE}
control.inla = list(int.stategy = "user.expert")
```
for which the weights \textbf{includes}  $\pi(\theta_i | y)$. The
following example show how to use it, fitting the same model in three
different ways. 
```{r}
n = 50
x = rnorm(n)
y = 1 + x + rnorm(n, sd = 0.2)
param = c(1, 0.04)
dz = 0.1
r.std = inla(y ~ 1 + x, data = data.frame(y, x),
             control.inla = list(int.strategy = "grid",
                                 dz = dz,
                                 diff.logdens = 8), 
             control.family = list(
                 hyper = list(
                     prec = list(
                         prior = "loggamma",
                         param = param))))

s = r.std$internal.summary.hyperpar[1,"sd"]
m = r.std$internal.summary.hyperpar[1,"mean"]
theta = m + s*seq(-4, 4, by = dz)
weight = dnorm(theta,  mean = m, sd = s)

r = rep(list(list()), length(theta))
for(k in seq_along(r)) {
    r[[k]] = inla(y ~ 1 + x,
                  control.family = list(
                      hyper = list(
                          prec = list(
                              initial = theta[k],
                              fixed=TRUE))),
                  data = data.frame(y, x))
}
r.merge = inla.merge(r, prob = weight)

r.design = inla(y ~ 1 + x,
                data = data.frame(y, x),
                control.family = list(
                    hyper = list(
                        prec = list(
                            ## the prior here does not really matter, as we will override
                            ## it with the user.expert in any case.
                            prior = "pc.prec",
                            param = c(1, 0.01)))), 
                control.inla = list(int.strategy = "user.expert",
                                    int.design = cbind(theta, weight)))
for(k in 1:2) {
    plot(inla.smarginal(r.std$marginals.fixed[[k]]),
         lwd = 2, lty = 1, type = "l", 
         xlim = inla.qmarginal(c(0.0001, 0.9999), r.std$marginals.fixed[[k]]))
    lines(inla.smarginal(r.design$marginals.fixed[[k]]),
          lwd = 2, col = "blue", lty = 1)
    lines(inla.smarginal(r.merge$marginals.fixed[[k]]),
          lwd = 2, col = "yellow", lty = 1)
}
```

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-cormat.R
%%
 \name{pc.cormat}
 \alias{inla.pc.cormat}
 \alias{inla.pc.cormat.dim2p}
 \alias{cormat.dim2p}
 \alias{inla.pc.cormat.p2dim}
 \alias{cormat.p2dim}
 \alias{inla.pc.cormat.theta2R}
 \alias{cormat.theta2R}
 \alias{inla.pc.cormat.R2theta}
 \alias{cormat.R2theta}
 \alias{inla.pc.cormat.r2R}
 \alias{cormat.r2R}
 \alias{inla.pc.cormat.R2r}
 \alias{cormat.R2r}
 \alias{inla.pc.cormat.r2theta}
 \alias{cormat.r2theta}
 \alias{inla.pc.cormat.theta2r}
 \alias{cormat.theta2r}
 \alias{inla.pc.cormat.permute}
 \alias{cormat.permute}
 \alias{inla.pc.cormat.rtheta}
 \alias{cormat.rtheta}
 \alias{inla.pc.cormat.dtheta}
 \alias{cormat.dtheta}

 \title{Utility functions for the PC prior for a correlation matrix}

 \description{Functions to evaluate and sample from the
              PC prior for a correlation matrix.}
 \usage{
    inla.pc.cormat.dim2p(dim)
    inla.pc.cormat.p2dim(p)
    inla.pc.cormat.theta2R(theta)
    inla.pc.cormat.R2theta(R)
    inla.pc.cormat.r2R(r)
    inla.pc.cormat.R2r(R)
    inla.pc.cormat.r2theta(r)
    inla.pc.cormat.theta2r(theta)
    inla.pc.cormat.permute(R)
    inla.pc.cormat.rtheta(n=1, p, lambda = 1)
    inla.pc.cormat.dtheta(theta, lambda = 1, log = FALSE)
 }
 \arguments{
   \item{dim}{The dimension of \code{theta}, the parameterisatin of the correlation matrix}
   \item{p}{The dimension the correlation matrix}
   \item{theta}{A vector of parameters for the correlation matrix}
   \item{r}{The off diagonal elements of a correlation matrix}
   \item{R}{A correlation matrix}
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter in the prior}
   \item{log}{Logical. Return the density in natural or log-scale.}
 }
 \details{
    The parameterisation of a correlation matrix of dimension \code{p} has \code{dim}
    parameters: \code{theta} which are in the interval -pi to pi.
    The alternative parameterisation is through the off-diagonal elements \code{r} of the
    correlation matrix \code{R}. The functions \code{inla.pc.cormat.<A>2<B>} convert between
    parameterisations \code{<A>} to parameterisations \code{<B>},  where both
    \code{<A>} and \code{<B>} are one of \code{theta},  \code{r} and \code{R},
    and \code{p} and \code{dim}.
 }
 \value{%%
     \code{inla.pc.cormat.rtheta} generate samples from the prior,  returning a matrix
     where each row is a sample of \code{theta}.
     \code{inla.pc.cormat.dtheta} evaluates the density of \code{theta}.
     \code{inla.pc.cormat.permute} randomly permutes a correlation matrix,
     which is useful if an exchangable sample of a correlation matrix is required.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
  p = 4
  print(paste("theta has length", inla.pc.cormat.p2dim(p)))
  theta = inla.pc.cormat.rtheta(n=1, p=4, lambda = 1)
  print("sample theta:")
  print(theta)
  print(paste("log.dens", inla.pc.cormat.dtheta(theta, log=TRUE)))
  print("r:")
  r = inla.pc.cormat.theta2r(theta)
  print(r)
  print("A sample from the non-exchangable prior, R:")
  R = inla.pc.cormat.r2R(r)
  print(R)
  print("A sample from the exchangable prior, R:")
  R = inla.pc.cormat.permute(R)
  print(R)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.deriv}
\alias{inla.mesh.deriv}
\title{Directional derivative matrices for functions on meshes.}
\usage{
inla.mesh.deriv(mesh, loc)
}
\arguments{
\item{mesh}{An \code{\link[=inla.mesh]{inla.mesh()}} object.}

\item{loc}{Coordinates where the derivatives should be evaluated.}
}
\value{
\item{A }{The projection matrix, \verb{u(loc_i)=sum_j A_ij w_i}}
\item{dx, dy, dz }{Derivative weight matrices, \verb{du/dx(loc_i)=sum_j dx_ij w_i}, etc.}
}
\description{
Calculates directional derivative matrices for functions on
\code{\link[=inla.mesh]{inla.mesh()}} objects.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from lattice2node.R
%%
\name{lattice2node}
\alias{lattice2node}
\alias{inla.lattice2node}
\alias{node2lattice}
\alias{inla.node2lattice}
\alias{node2lattice.mapping}
\alias{inla.node2lattice.mapping}
\alias{lattice2node.mapping}
\alias{inla.lattice2node.mapping}
\alias{matrix2vector}
\alias{vector2matrix}
\alias{inla.matrix2vector}
\alias{inla.vector2matrix}

\title{Functions to define mapping between a lattice and nodes}

\description{These functions define mapping in between two-dimensional
  indices on a lattice and the one-dimensional node representation used
  in \code{inla}.

  The mapping from node to lattice follows the default \code{R}
  behaviour (which is column based storage), and \code{as.vector(A)} and
  \code{matrix(a, nrow, ncol)} can be used instead of
  \code{inla.matrix2vector} and \code{inla.vector2matrix}.
}

\usage{
inla.lattice2node.mapping(nrow, ncol)
inla.node2lattice.mapping(nrow, ncol)
inla.lattice2node(irow, icol, nrow, ncol)
inla.node2lattice(node, nrow, ncol)
inla.matrix2vector(a.matrix)
inla.vector2matrix(a.vector, nrow, ncol)
}

\arguments{

  \item{nrow}{Number of rows in the lattice.}

  \item{ncol}{Number of columns in the lattice.}

  \item{irow}{Lattice row index, between \code{1} and \code{nrow}}

  \item{icol}{Lattice column index, between \code{1} and \code{ncol}}

  \item{node}{The node index, between \code{1} and \code{ncol*nrow}}

  \item{a.matrix}{is a matrix to be mapped to a vector using internal
  representation defined by \code{inla.lattice2node}}

  \item{a.vector}{is a vector to be mapped into a matrix using the
    internal representation defined by \code{inla.node2lattice}}
}

\value{\code{inla.lattice2node.mapping} returns the hole mapping as a
  matrix, and \code{inla.node2lattice.mapping} returns the hole mapping
  as \code{list(irow=..., icol=...)}. \code{inla.lattice2node} and
  \code{inla.node2lattice} provide the mapping for a given set of
  lattice indices and nodes. \code{inla.matrix2vector} provide the
  mapped vector from a matrix, and \code{inla.vector2matrix} provide the
  inverse mapped matrix from vector.}

%%

\author{Havard Rue \email{hrue@r-inla.org}}

\seealso{\link{inla}}

\examples{
## write out the mapping using the two alternatives
nrow = 2
ncol = 3
mapping = inla.lattice2node.mapping(nrow,ncol)

for (i in 1:nrow){
    for(j in 1:ncol){
        print(paste("Alt.1: lattice index [", i,",", j,"] corresponds",
                    "to node [", mapping[i,j],"]", sep=""))
    }
}

for (i in 1:nrow){
    for(j in 1:ncol){
        print(paste("Alt.2: lattice index [", i,",", j,"] corresponds to node [",
                    inla.lattice2node(i,j,nrow,ncol), "]", sep=""))
    }
}

inv.mapping = inla.node2lattice.mapping(nrow,ncol)
for(node in 1:(nrow*ncol))
   print(paste("Alt.1: node [", node, "] corresponds to lattice index [",
               inv.mapping$irow[node], ",",
               inv.mapping$icol[node],"]", sep=""))

for(node in 1:(nrow*ncol))
   print(paste("Alt.2: node [", node, "] corresponds to lattice index [",
               inla.node2lattice(node,nrow,ncol)$irow[1], ",",
               inla.node2lattice(node,nrow,ncol)$icol[1],"]", sep=""))

## apply the mapping from matrix to vector and back
n = nrow*ncol
z = matrix(1:n,nrow,ncol)
z.vector = inla.matrix2vector(z)  # as.vector(z) could also be used
print(mapping)
print(z)
print(z.vector)

## the vector2matrix is the inverse, and should give us the z-matrix
## back. matrix(z.vector, nrow, ncol) could also be used here.
z.matrix = inla.vector2matrix(z.vector, nrow, ncol)
print(z.matrix)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from list-models.R
%%
\name{inla.list.models}
\alias{inla.list.models}
\alias{list.models}
\title{List available model components, likelihoods,  priors,  etc}
\description{
    List available model components, likelihoods,  priors,  etc.
    To read specific documentation for the individual elements, use \code{\link{inla.doc}}.
}
\usage{
    inla.list.models(section = names(inla.models()), ...)
}
\arguments{
   \item{section}{The section(s) to list, missing \code{section} will list all sections.
                  \code{names(inla.models())} lists available sections.}
   \item{...}{Additional argument to \code{cat}}
}
\details{
   The list is \code{cat}'ed with \code{...} arguments.

   This function is EXPERIMENTAL.
}
\value{
   Nothing is returned
}
\author{Havard Rue}
\examples{
\dontrun{
inla.list.models("likelihood")
inla.list.models(c("prior", "group"))
inla.list.models(file=file("everything.txt"))

#Show detailed doc for a specific prior/likelihood/latent model
inla.doc("binomial")
}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from inla.R
%%
 \name{inla}
 \alias{inla}

 \title{Bayesian analysis of
 structured additive models} \description{\code{inla} performs a
 full Bayesian analysis of additive models using Integrated Nested
 Laplace approximation
 }

 \usage{
 inla(
    formula,
    family = "gaussian",
    contrasts = NULL,
    data,
    quantiles=c(0.025, 0.5, 0.975),
    E = NULL,
    offset=NULL,
    scale = NULL,
    weights = NULL,
    Ntrials = NULL,
    strata = NULL,
    link.covariates = NULL,
    verbose = FALSE,
    lincomb = NULL,
    selection = NULL,
    control.compute = list(),
    control.predictor = list(),
    control.family = list(),
    control.inla = list(),
    control.fixed = list(),
    control.mode = list(),
    control.expert = list(),
    control.hazard = list(),
    control.lincomb = list(),
    control.update = list(),
    control.pardiso = list(),
    only.hyperparam = FALSE,
    inla.call = inla.getOption("inla.call"),
    inla.arg = inla.getOption("inla.arg"),
    num.threads = inla.getOption("num.threads"),
    blas.num.threads = inla.getOption("blas.num.threads"),
    keep = inla.getOption("keep"),
    working.directory = inla.getOption("working.directory"),
    silent = inla.getOption("silent"),
    inla.mode = c("classic", "twostage", "experimental"), 
    debug = inla.getOption("debug"),
    .parent.frame = parent.frame()
    )
 }
 \arguments{
 \item{formula}{ A \code{inla} formula like \code{y
 ~1 + z + f(ind, model="iid")} + f(ind2,
 weights, model="ar1") This is much like the formula
 for a \code{glm} except that smooth or spatial terms
 can be added to the right hand side of the formula.
 See \code{\link{f}} for full details and the web site
 \url{www.r-inla.org} for several worked out
 examples. Each smooth or spatial term specified
 through \code{f} should correspond to separate column
 of the data frame \code{data}.
 The response
 variable, \code{y} can be a univariate response
 variable, a list or the output of the function
 \code{inla.surf} for survival analysis models.}
 \item{family}{ A string indicating the likelihood
 family. The default is \code{gaussian} with identity
 link. See \code{names(inla.models()$likelihood)} for a
 list of possible alternatives and use \code{\link{inla.doc}}
 for detailed docs for individual families.}
 \item{contrasts}{Optional contrasts for the fixed
 effects; see \code{?lm} or \code{?glm} for details.}
 \item{data}{ A data frame or list containing the
 variables in the model.  The data frame MUST be
 provided}
 \item{quantiles}{ A vector of quantiles,
 \eqn{p(0), p(1),\dots}{p(0), p(1),\ldots} to compute
 for each posterior marginal. The function returns,
 for each posterior marginal, the values
 \eqn{x(0), x(1),\dots}{x(0), x(1),\ldots} such that
 \deqn{\mbox{Prob}(X<x(p))=p}{Prob(X<x)=p} }
 \item{E}{ Known component in the mean for the Poisson
 likelihoods defined as \deqn{E_i\exp(\eta_i)}{E
 exp(eta)} where \deqn{\eta_i}{eta} is the linear
 predictor. If not provided it is set to \code{rep(1, n.data)}.}
 \item{offset}{This argument is used to specify an
 a-priori known and fixed component to be included in
 the linear predictor during fitting.  This should be
 \code{NULL} or a numeric vector of length either one
 or equal to the number of cases. One or more
 \code{offset()} terms can be included in the formula
 instead or as well, and if both are used, they are
 combined into a common offset.  If the
 \code{A}-matrix is used in the linear predictor
 statement \code{control.predictor}, then the
 \code{offset} given in this argument is added to
 \code{eta*}, the linear predictor related to the
 observations, as \code{eta* = A eta + offset},
 whereas an offset in the formula is added to
 \code{eta}, the linear predictor related to the
 formula, as \code{eta = ... + offset.formula}. So in
 this case, the offset defined here and in the formula
 has a different meaning and usage.}
 \item{scale}{ Fixed (optional) scale parameters of
 the precision for Gaussian and Student-T response
 models. Default value is rep(1, n.data).}
 \item{weights}{ Fixed (optional) weights parameters of
 the likelihood, so the log-likelihood[i] is changed into
 weights[i]*log-likelihood[i]. Default value is rep(1,
 n.data). 
 WARNING: The normalizing constant for the likelihood is NOT recomputed, so
 ALL marginals (and the marginal likelihood) must be interpreted with great care.}
 \item{Ntrials}{A vector containing the number of trials for the \code{binomial}
 likelihood and variantes, or the number of required successes for the
 \code{nbinomial2} likelihood. Default value is \code{rep(1, n.data)}.}
 \item{strata}{Fixed (optional) strata indicators
 for tstrata likelihood model.}
 \item{link.covariates}{A vector or matrix with covariates for link functions}
 \item{verbose}{
 Boolean indicating if the \code{inla}-program should
 run in a verbose mode (default \code{FALSE}).}
 \item{lincomb}{Used to define linear combination of
 nodes in the latent field. The posterior distribution
 of such linear combination is computed by the
 \code{inla} function. See
 \url{www.r-inla.org/faq} for examples of
 how to define such linear combinations.}
 \item{selection}{ This is a similar argument to the one in
 \code{inla.posterior.sample}  and follow the same format.
 This argument allows to define a subset of the latent field
 for which to compute an approximated joint
 distribution. It will appear in \code{result$selection}.
 See also \code{?inla.rjmarginal}. }
 \item{control.compute}{ See \code{?control.compute}}
 \item{control.predictor}{ See
 \code{?control.predictor}}
 \item{control.family}{ See \code{?control.family}}
 \item{control.inla}{ See \code{?control.inla}}
 \item{control.fixed}{ See \code{?control.fixed}}
 \item{control.mode}{ See \code{?control.mode}}
 \item{control.expert}{ See \code{?control.expert}}
 \item{control.hazard}{ See \code{?control.hazard}}
 \item{control.lincomb}{ See \code{?control.lincomb}}
 \item{control.update}{ See \code{?control.update}}
 \item{control.pardiso}{ See \code{?control.pardiso}}
 \item{only.hyperparam}{ A boolean variable saying if
 only the hyperparameters should be computed. This option is mainly used
 internally. (TODO: This option should not be located here,  change it!)}
 \item{inla.call}{ The path to, or the name of, the
 \code{inla}-program. This is program is installed
 together with the \code{R}-package, but, for example,
 a native compiled version can be used instead to
 improve the performance.}
 \item{inla.arg}{ A string indicating ALL arguments to
 the 'inla' program and do not include default
 arguments. (OOPS: This is an expert option!)}
 \item{num.threads}{ Maximum number of threads the
 \code{inla}-program will use, or as 'A:B' defining the number threads in the
 outer (A) and inner (B) layer for nested parallelism. }
 \item{blas.num.threads}{The absolute value of \code{blas.num.threads} is the maximum
 number of threads the the \code{openblas}/\code{mklblas} will use (if
 available). Value is ignored if \code{<=0} (then environment variables are used).}
 \item{keep}{ A boolean variable indicating that the
 working files (ini file, data files and results
 files) should be kept. If TRUE and no
 \code{working.directory} is specified the working
 files are stored in a directory called "inla".  }
 \item{working.directory}{ A string giving the name
 of an non-existing directory where to store the
 working files.}
 \item{silent}{If equal to 1L or TRUE, then the
 \code{inla}-program would be ``silent''. If equal to
 2L, then supress also error messages from the
 \code{inla}-program.}
 \item{inla.mode}{Run \code{inla} in \code{classic}-mode,
             \code{twostage}-mode or the new \code{experimental}-mode?
             Not all features are available int the \code{experimental}-mode and this mode
             is really work-in-progress at the moment!!!!
             Default is to use the 
             mode set by \code{inla.getOption("inla.mode")} which is default
             \code{classic}-mode. }
 \item{debug}{ If \code{TRUE}, then enable some debug
 output.  }
 \item{.parent.frame}{Internal use only}
 }
 \value{%%
 \code{inla} returns an object of class \code{"inla"}. This is a list
 containing at least the following arguments:
 \item{summary.fixed}{Matrix containing the mean and standard
 deviation (plus, possibly quantiles and cdf) of the the fixed
 effects of the model.}
 \item{marginals.fixed}{
 A list containing the posterior marginal
 densities of the fixed effects of the model.}
 \item{summary.random}{List of matrices containing the mean and
 standard deviation (plus, possibly quantiles and cdf) of the
 the smooth or spatial effects defined through \code{f()}.}
 \item{marginals.random}{A list containing the
 posterior marginal densities of the random effects defined
 through \code{f}.}
 \item{summary.hyperpar}{
 A matrix containing the mean and sd
 (plus, possibly quantiles and cdf) of the hyperparameters of
 the model }
 \item{marginals.hyperpar}{
 A list containing the posterior marginal
 densities of the hyperparameters of the model.}
 \item{summary.linear.predictor}{
 A matrix containing the mean and sd
 (plus, possibly quantiles and cdf) of the linear predictors
 \eqn{\eta} in
 the model
 }
 \item{marginals.linear.predictor}{
 If \code{compute=TRUE} in
 \code{control.predictor}, a list containing the posterior
 marginals of the linear predictors \eqn{\eta} in the model.
 }
 \item{summary.fitted.values}{ A matrix containing the mean and
 sd (plus, possibly quantiles and cdf) of the fitted values
 \eqn{g^{-1}(\eta)} obtained by transforming the linear
 predictors by the inverse of the link function. This quantity
 is only computed if \code{marginals.fitted.values} is
 computed. Note that if an observation is \code{NA} then the
 identity link is used. You can manually transform a marginal
 using \code{inla.marginal.transform()} or set the argument
 \code{link} in the \code{control.predictor}-list;
 see \code{?control.predictor}
 }
 \item{marginals.fitted.values}{
 If \code{compute=TRUE} in
 \code{control.predictor}, a list containing the posterior
 marginals  of the fitted values
 \eqn{g^{-1}(\eta)} obtained by
 transforming the linear predictors by the inverse of the link
 function.
 Note that if an observation is \code{NA} then the
 identity link is used. You can manually transform a marginal
 using \code{inla.marginal.transform()} or set the argument
 \code{link} in the \code{control.predictor}-list;
 see \code{?control.predictor}
 }
 \item{summary.lincomb}{
 If \code{lincomb != NULL} a list of
 matrices containing the mean and sd (plus, possibly quantiles
 and cdf) of all linear combinations defined.  }
 \item{marginals.lincomb}{
 If \code{lincomb != NULL} a list of
 posterior marginals of all linear combinations defined.  }
 \item{selection}{Provide the approximated joint
 distribution for the \code{selection}}
 \item{dic}{
 If \code{dic}=\code{TRUE} in \code{control.compute}, the
 deviance information criteria and effective number of parameters,
 otherwise \code{NULL}
 }
 \item{cpo}{
 If \code{cpo}=\code{TRUE} in \code{control.compute}, a list
 of three elements: \code{cpo$cpo} are the values of the conditional
 predictive ordinate (CPO), \code{cpo$pit} are the values of the
 probability integral transform (PIT) and \code{cpo$failure}
 indicates whether some assumptions are violated. In short, if
 cpo$failure[i] > 0 then some assumption is violated, the higher the
 value (maximum 1) the more seriously.
 }
 \item{po}{
 If \code{po}=\code{TRUE} in \code{control.compute}, a list
 of one elements: \code{po$po} are the values of the
 predictive ordinate (CPO) (\code{pi(yi|y)})
 }
 \item{waic}{
 If \code{waic}=\code{TRUE} in \code{control.compute}, a list
 of two elements: \code{waic$waic} is the Watanabe-Akaike information criteria,  and
 \code{waic$p.eff} is the estimated effective number of parameters
 }
 \item{mlik}{
 If \code{mlik}=\code{TRUE} in \code{control.compute}, the
 log marginal likelihood of the model (using two different estimates), otherwise \code{NULL}
 }
 \item{neffp}{
 Expected effective number of parameters in the model. The
 standard deviation of the expected number of parameters and the
 number of replicas for parameter are also returned}
 \item{mode}{
 A list of two elements: \code{mode$theta} is the
 computed mode of the hyperparameters and \code{mode$x} is the
 mode of the latent field given the modal value of the
 hyperparamters.
 }
 \item{call}{
 The matched call.}
 \item{formula}{
 The formula supplied}
 \item{nhyper}{
 The number of hyperparameters in the model}
 \item{cpu.used}{
 The cpu time used by the \code{inla} function}
 }
 \references{
 Rue, H. and Martino, S. and Chopin, N. (2009)
 \emph{Approximate Bayesian Inference for latent Gaussian models
 using Integrated Nested Laplace Approximations, JRSS-series B
 (with discussion)}, vol 71, no 2, pp 319-392.
 Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields
 - Theory and Applications} Chapman and Hall}
 \author{Havard Rue \email{hrue@r-inla.org} and Sara Martino}
 \seealso{\code{\link{f}},
 \code{\link{inla.hyperpar}} }
 \examples{
 \dontrun{
 ##See the web page \url{www.r-inla.org} for a series of worked out examples
 }
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.create}
\alias{inla.mesh.create}
\alias{inla.mesh}
\alias{inla.delaunay}
\title{Low level function for high-quality triangulations}
\usage{
inla.mesh.create(
  loc = NULL,
  tv = NULL,
  boundary = NULL,
  interior = NULL,
  extend = (missing(tv) || is.null(tv)),
  refine = FALSE,
  lattice = NULL,
  globe = NULL,
  cutoff = 1e-12,
  plot.delay = NULL,
  data.dir,
  keep = (!missing(data.dir) && !is.null(data.dir)),
  timings = FALSE,
  quality.spec = NULL,
  crs = NULL
)

inla.delaunay(loc, ...)
}
\arguments{
\item{loc}{Matrix of point locations.  Can alternatively be a
\code{SpatialPoints} or \code{SpatialPointsDataFrame} object.}

\item{tv}{A triangle-vertex index matrix, specifying an existing
triangulation.}

\item{boundary}{A list of \code{inla.mesh.segment} objects, generated by
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}, specifying boundary constraint segments.}

\item{interior}{A list of \code{inla.mesh.segment} objects, generated by
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}, specifying interior constraint segments.}

\item{extend}{\code{logical} or \code{list} specifying whether to extend the
data region, with parameters \describe{ \item{list("n")}{the number of edges
in the extended boundary (default=8)} \item{list("offset")}{the extension
distance.  If negative, interpreted as a factor relative to the approximate
data diameter (default=-0.10)} } Setting to \code{FALSE} is only useful in
combination \code{lattice} or \code{boundary}.}

\item{refine}{\code{logical} or \code{list} specifying whether to refine the
triangulation, with parameters \describe{ \item{list("min.angle")}{the
minimum allowed interior angle in any triangle.  The algorithm is guaranteed
to converge for \code{min.angle} at most 21 (default=\code{21})}
\item{list("max.edge")}{the maximum allowed edge length in any triangle.  If
negative, interpreted as a relative factor in an ad hoc formula depending on
the data density (default=\code{Inf})} \item{list("max.n.strict")}{the
maximum number of vertices allowed, overriding \code{min.angle} and
\code{max.edge} (default=-1, meaning no limit)} \item{list("max.n")}{the
maximum number of vertices allowed, overriding \code{max.edge} only
(default=-1, meaning no limit)} }}

\item{lattice}{An \code{inla.mesh.lattice} object, generated by
\code{\link[=inla.mesh.lattice]{inla.mesh.lattice()}}, specifying points on a regular lattice.}

\item{globe}{Subdivision resolution for a semi-regular spherical
triangulation with equidistant points along equidistant latitude bands.}

\item{cutoff}{The minimum allowed distance between points.  Point at most as
far apart as this are replaced by a single vertex prior to the mesh
refinement step.}

\item{plot.delay}{On Linux (and Mac if appropriate X11 libraries are
installed), specifying a numeric value activates a rudimentary plotting
system in the underlying \code{fmesher} program, showing the triangulation
algorithm at work.}

\item{data.dir}{Where to store the \code{fmesher} data files.  Defaults to
\code{tempdir()} if \code{keep} is \code{FALSE}, otherwise
\code{"inla.mesh.data"}.}

\item{keep}{\code{TRUE} if the data files should be kept in \code{data.dir}
or deleted afterwards.  Defaults to true if \code{data.dir} is specified,
otherwise false.  Warning: If \code{keep} is false, \code{data.dir} and its
contents will be deleted (unless set to \code{tempdir()}).}

\item{timings}{If \code{TRUE}, obtain timings for the mesh construction.}

\item{quality.spec}{List of vectors of per vertex \code{max.edge} target
specification for each location in \code{loc}, \code{boundary/interior}
(\code{segm}), and \code{lattice}.  Only used if refining the mesh.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}

\item{...}{Optional parameters passed on to \code{inla.mesh.create}.}
}
\value{
An \code{inla.mesh} object.
}
\description{
Create a constrained refined Delaunay triangulation (CRDT) for a set of
spatial locations.
}
\details{
\code{inla.mesh.create} generates triangular meshes on subsets of
\eqn{R^2}{R^2} and \eqn{S^2}{S^2}.  Use the higher level wrapper function
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}} for greater control over mesh resolution and
coarser domain extensions.

\code{inla.delaunay} is a wrapper function for obtaining the convex hull of
a point set and calling \code{inla.mesh.create} to generate the classical
Delaunay tringulation.
}
\examples{

loc <- matrix(runif(10 * 2), 10, 2)

mesh <- inla.delaunay(loc)
plot(mesh)

mesh <- inla.mesh.create(loc,
    interior = inla.mesh.segment(idx = 1:2),
    extend = TRUE,
    refine = list(max.edge = 0.1)
)
plot(mesh)

loc2 <- matrix(c(0, 1, 1, 0, 0, 0, 1, 1), 4, 2)
mesh2 <- inla.mesh.create(
    loc = loc,
    boundary = inla.mesh.segment(loc2),
    interior = inla.mesh.segment(idx = 1:2),
    quality.spec = list(segm = 0.2, loc = 0.05),
    refine = list(min.angle = 26)
)
plot(mesh2)
}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}, \code{\link[=inla.mesh.1d]{inla.mesh.1d()}},
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}, \code{\link[=inla.mesh.lattice]{inla.mesh.lattice()}},
\code{\link[=inla.mesh.query]{inla.mesh.query()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.generate.colors}
\alias{inla.generate.colors}
\title{Generate text RGB color specifications.}
\usage{
inla.generate.colors(
  color,
  color.axis = NULL,
  color.n = 512,
  color.palette = cm.colors,
  color.truncate = FALSE,
  alpha = NULL
)
}
\arguments{
\item{color}{\code{character}, \code{matrix} or \code{vector}}

\item{color.axis}{The min/max limit values for the color mapping.}

\item{color.n}{The number of colors to use in the color palette.}

\item{color.palette}{A color palette function.}

\item{color.truncate}{If \code{TRUE}, truncate the colors at the color axis
limits.}

\item{alpha}{Transparency/opaqueness values.}
}
\description{
Generates a tex RGB color specification matrix based on a color palette.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.create.helper}
\alias{inla.mesh.create.helper}
\title{High-quality triangulations}
\usage{
inla.mesh.create.helper(points = NULL, points.domain = NULL, ...)
}
\arguments{
\item{points}{Matrix of point locations to be used as initial triangulation
nodes.}

\item{points.domain}{Matrix of point locations used to determine the domain
extent.}

\item{\dots}{Additional arguments passed on to \code{\link[=inla.mesh.2d]{inla.mesh.2d()}}.}
}
\description{
Create a triangle mesh based on initial point locations, specified or
automatic boundaries, and mesh quality parameters.
}
\note{
Obsolete function kept for backwards compatibility.  Use
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}} instead.
}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.bgev.default}
\alias{control.bgev.default}
\alias{inla.control.bgev.default}
\alias{inla.set.control.bgev.default.default}
\alias{set.control.bgev.default.default}
\alias{control.bgev.default.default}
\title{Control variables in control.bgev.default}
\description{Control variables in \code{control.bgev.default} for use in \code{inla}}
\usage{
inla.set.control.bgev.default.default(...)
control.bgev.default(beta.ab, q.location, q.mix, q.spread)
}
\arguments{
\item{...}{Possible arguments}
\item{q.location}{   The quantile level for the location parameter}
\item{q.spread}{   The quantile level for the spread parameter (must be < 0.5)}
\item{q.mix}{   The lower and upper quantile level for the mixing function}
\item{beta.ab}{   The parameters a and b in the Beta mixing function}
}
\value{
   The \code{control.bgev}-list is set within the corresponding \code{control.family}-list as control parameters to the \code{family="bgev"}
The function \code{control.bgev.default} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.bgev.default.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from marginal.R
%%
 \name{marginal}
 \alias{inla.marginal}
 \alias{marginal}
 \alias{pmarginal}
 \alias{inla.pmarginal}
 \alias{qmarginal}
 \alias{inla.qmarginal}
 \alias{dmarginal}
 \alias{inla.dmarginal}
 \alias{rmarginal}
 \alias{inla.rmarginal}
 \alias{inla.hpdmarginal}
 \alias{hpdmarginal}
 \alias{inla.emarginal}
 \alias{emarginal}
 \alias{inla.smarginal}
 \alias{smarginal}
 \alias{inla.tmarginal}
 \alias{inla.mmarginal}
 \alias{mmarginal}
 \alias{inla.zmarginal}
 \alias{zmarginal}

 \title{Functions which operates on marginals}

 \description{Density, distribution function, quantile function, random
      generation, hpd-interval, interpolation, expectations, mode and transformations of
      marginals obtained by \code{inla} or \code{inla.hyperpar()}.
 These functions computes the density (inla.dmarginal),
 the distribution function (inla.pmarginal),
 the quantile function (inla.qmarginal),
 random generation (inla.rmarginal),
 spline smoothing (inla.smarginal),
 computes expected values (inla.emarginal),
 computes the mode (inla.mmarginal),
 transforms the marginal (inla.tmarginal), and provide summary statistics (inla.zmarginal).
 }

 \usage{
 inla.dmarginal(x, marginal, log = FALSE)
 inla.pmarginal(q, marginal, normalize = TRUE, len = 2048L)
 inla.qmarginal(p, marginal, len = 2048L)
 inla.rmarginal(n, marginal)
 inla.hpdmarginal(p, marginal, len = 2048L)
 inla.smarginal(marginal, log = FALSE, extrapolate = 0.0, keep.type = FALSE, factor=15L)
 inla.emarginal(fun, marginal, ...)
 inla.mmarginal(marginal)
 inla.tmarginal(fun, marginal, n=2048L, h.diff = .Machine[["double.eps"]]^(1.0/3.0),
                method = c("quantile", "linear"))
 inla.zmarginal(marginal, silent = FALSE)
 }
 \arguments{

   \item{marginal}{A marginal object from either \code{inla} or
     \code{inla.hyperpar()}, which is either \code{list(x=c(), y=c())}
     with density values \code{y} at locations \code{x}, or a
     \code{matrix(,n,2)} for which the density values are the second
     column and the locations in the first column.
     The\code{inla.hpdmarginal()}-function
     assumes a unimodal density.}

   \item{fun}{A (vectorised) function like \code{function(x) exp(x)} to
     compute the expectation against, or which define the transformation
     new = fun(old)}

   \item{x}{Evaluation points}

   \item{q}{Quantiles}

   \item{p}{Probabilities}

   \item{n}{The number of observations. If \code{length(n) > 1}, the
     length is taken to be the number required.}

   \item{h.diff}{The step-length for the numerical differeniation inside \code{inla.tmarginal}}

   \item{...}{Further arguments to be passed to function which
     expectation is to be computed.}

   \item{log}{Return density or interpolated density in log-scale?}

   \item{normalize}{Renormalise the density after interpolation?}
   \item{len}{Number of locations used to interpolate the distribution
   function.}

   \item{keep.type}{If \code{FALSE} then return a \code{list(x=, y=)},  otherwise if \code{TRUE},
                   then return a matrix if the input is a matrix}

   \item{extrapolate}{How much to extrapolate on each side when computing the
   interpolation. In fraction of the range.}

   \item{factor}{The number of points after interpolation is \code{factor} times the original number of points;
                 which is argument \code{n} in \code{spline}}

    \item{method}{Which method should be used to layout points for where the transformation is computed.}

    \item{silent}{Output the result visually (TRUE) or just through the call.}
 }

 \value{%%
   \code{inla.smarginal} returns \code{list=c(x=c(), y=c())} of
   interpolated values do extrapolation using the factor given,
   and the remaining function returns what they say they should do.  }
 %%

 \author{Havard Rue \email{hrue@r-inla.org}}

 \seealso{\code{\link{inla}}, \code{\link{inla.hyperpar}}}

 \examples{
 ## a simple linear regression example
 n = 10
 x = rnorm(n)
 sd = 0.1
 y = 1+x + rnorm(n,sd=sd)
 res = inla(y ~ 1 + x, data = data.frame(x,y),
            control.family=list(initial = log(1/sd^2L),fixed=TRUE))

 ## chose a marginal and compare the with the results computed by the
 ## inla-program
 r = res$summary.fixed["x",]
 m = res$marginals.fixed$x

 ## compute the 95% HPD interval
 inla.hpdmarginal(0.95, m)

 x = seq(-6, 6, len = 1000)
 y = dnorm(x)
 inla.hpdmarginal(0.95, list(x=x, y=y))

 ## compute the the density for exp(r), version 1
 r.exp = inla.tmarginal(exp, m)
 ## or version 2
 r.exp = inla.tmarginal(function(x) exp(x), m)

 ## to plot the marginal, we use the inla.smarginal, which interpolates (in
 ## log-scale). Compare with some samples.
 plot(inla.smarginal(m), type="l")
 s = inla.rmarginal(1000, m)
 hist(inla.rmarginal(1000, m), add=TRUE, prob=TRUE)
 lines(density(s), lty=2)

 m1 = inla.emarginal(function(x) x, m)
 m2 = inla.emarginal(function(x) x^2L, m)
 stdev = sqrt(m2 - m1^2L)
 q = inla.qmarginal(c(0.025,0.975), m)

 ## inla-program results
 print(r)

 ## inla.marginal-results (they shouldn't be perfect!)
 print(c(mean=m1, sd=stdev, "0.025quant" = q[1], "0.975quant" = q[2L]))
 ## using the buildt-in function
 inla.zmarginal(m)
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from scale.model.R
%%
 \name{scale.model}
 \alias{inla.scale.model}
 \alias{scale.model}

 \title{Scale an intrinsic GMRF model}

 \description{This function scales an intrinsic GMRF model so the geometric mean of the
              marginal variances is one}

 \usage{
     inla.scale.model(Q, constr = NULL, eps = sqrt(.Machine$double.eps))
 }

 \arguments{
   \item{Q}{A SPD matrix,  either as a (dense) matrix or \code{sparseMatrix}}
   \item{constr}{Linear constraints spanning the null-space of \code{Q};
                 see \code{?INLA::f} and argument \code{extraconstr}}
   \item{eps}{A small constant added to the diagonal of \code{Q} if \code{constr}}
  }
 \value{
   \code{inla.scale.model} returns a \code{sparseMatrix} of type \code{dgTMatrix}
   scaled so the geometric mean of the marginal variances (of the possible
   non-singular part of \code{Q}) is one,  for each connected component of the matrix.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}

 \examples{
 ## Q is singular
 data(Germany)
 g = system.file("demodata/germany.graph", package="INLA")
 Q = -inla.graph2matrix(g)
 diag(Q) = 0
 diag(Q) = -rowSums(Q)
 n = dim(Q)[1]
 Q.scaled = inla.scale.model(Q, constr = list(A = matrix(1, 1, n), e=0))
 print(diag(MASS::ginv(Q.scaled)))

 ## Q is singular with 3 connected components
 g = inla.read.graph("6 1 2 2 3 2 2 1 3 3 2 1 2 4 1 5 5 1 4 6 0")
 print(paste("Number of connected components", g$cc$n))
 Q = -inla.graph2matrix(g)
 diag(Q) = 0
 diag(Q) = -rowSums(Q)
 n = dim(Q)[1]
 Q.scaled = inla.scale.model(Q, constr = list(A = matrix(1, 1, n), e=0))
 print(diag(MASS::ginv(Q.scaled)))

 ## Q is non-singular with 3 connected components. no constraints needed
 diag(Q) = diag(Q) + 1
 Q.scaled = inla.scale.model(Q)
 print(diag(MASS::ginv(Q.scaled)))
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.mode}
\alias{control.mode}
\alias{inla.control.mode}
\alias{inla.set.control.mode.default}
\alias{set.control.mode.default}
\alias{control.mode.default}
\title{Control variables in control.mode}
\description{Control variables in \code{control.mode} for use in \code{inla}}
\usage{
inla.set.control.mode.default(...)
control.mode(fixed, restart, result, theta, x)
}
\arguments{
\item{...}{Possible arguments}
\item{result}{   Prevous result from inla(). Use the theta- and x-mode from this run.}
\item{theta}{   The theta-mode/initial values for theta. This option has preference over result$mode$theta.}
\item{x}{   The x-mode/intitial values for x. This option has preference over result$mode$x.}
\item{restart}{   A boolean variable; should we restart the optimisation from this configuration or fix the mode at this configuration? (Default FALSE.)}
\item{fixed}{   A boolean variable. If TRUE then treat all thetas as known and fixed, and if FALSE then treat all thetas as unknown and random (default).}
}
\value{
  
The function \code{control.mode} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.mode.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{lines.inla.mesh.segment}
\alias{lines.inla.mesh.segment}
\title{Draw \code{inla.mesh.segment} objects.}
\usage{
\method{lines}{inla.mesh.segment}(
  x,
  loc = NULL,
  col = NULL,
  colors = c("black", "blue", "red", "green"),
  add = TRUE,
  xlim = NULL,
  ylim = NULL,
  rgl = FALSE,
  ...
)
}
\arguments{
\item{x}{An \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object.}

\item{loc}{Point locations to be used if \code{x$loc} is \code{NULL}.}

\item{col}{Segment color specification.}

\item{colors}{Colors to cycle through if \code{col} is \code{NULL}.}

\item{add}{If \code{TRUE}, add to the current plot, otherwise start a new
plot.}

\item{xlim}{X axis limits for a new plot.}

\item{ylim}{Y axis limits for a new plot.}

\item{rgl}{If \code{TRUE}, use \code{rgl} for plotting.}

\item{\dots}{Additional parameters, passed on to graphics methods.}
}
\description{
Draws a \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object with generic or \code{rgl}
graphics.
}
\seealso{
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde2.R
\name{inla.spde2.generic}
\alias{inla.spde2.generic}
\alias{inla.spde2}
\alias{inla.spde2.theta2phi0}
\alias{inla.spde2.theta2phi1}
\alias{inla.spde2.theta2phi2}
\title{Generic spde2 model creation.}
\usage{
inla.spde2.generic(
  M0,
  M1,
  M2,
  B0,
  B1,
  B2,
  theta.mu,
  theta.Q,
  transform = c("logit", "log", "identity"),
  theta.initial = theta.mu,
  fixed = rep(FALSE, length(theta.mu)),
  theta.fixed = theta.initial[fixed],
  BLC = cbind(0, diag(nrow = length(theta.mu))),
  ...
)
}
\arguments{
\item{M0}{The symmetric \code{M0} matrix.}

\item{M1}{The square \code{M1} matrix.}

\item{M2}{The symmetric \code{M2} matrix.}

\item{B0}{Basis definition matrix for \eqn{\phi_0}{phi0}.}

\item{B1}{Basis definition matrix for \eqn{\phi_2}{phi1}.}

\item{B2}{Basis definition matrix for \eqn{\phi_2}{phi2}.}

\item{theta.mu}{Prior expectation for the \eqn{\theta}{theta} vector}

\item{theta.Q}{Prior precision for the \eqn{\theta}{theta} vector}

\item{transform}{Transformation link for \eqn{\phi_2}{phi2}.  Valid settings
are \code{"logit"}, \code{"log"}, and \code{"identity"}}

\item{theta.initial}{Initial value for the \eqn{\theta}{theta} vector.
Default \code{theta.mu}}

\item{fixed}{Logical vector.  For every \code{TRUE} value, treat the
corresponding \code{theta} value as known.}

\item{theta.fixed}{Vector holding the values of fixed \code{theta} values.
Default \verb{=theta.initial[fixed]}}

\item{BLC}{Basis definition matrix for linear combinations of \code{theta}.}

\item{\dots}{Additional parameters, currently unused.}

\item{spde}{An \code{inla.sdpe2} object.}

\item{theta}{parameter values to be mapped.}
}
\value{
For \code{inla.spde2.generic}, an \code{\link[=inla.spde2]{inla.spde2()}} object.

For \code{inla.spde2.theta2phi0/1/2}, a vector of \eqn{\phi}{phi} values.
}
\description{
Creates and inla.spde2 object describing the internal structure of an
\code{'spde2'} model.
}
\seealso{
\code{\link[=inla.spde2.models]{inla.spde2.models()}}, \code{\link[=inla.spde2.matern]{inla.spde2.matern()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-prec.R
%%
 \name{pc.prec}
 \alias{inla.pc.prec}
 \alias{pc.prec}
 \alias{pc.rprec}
 \alias{inla.pc.rprec}
 \alias{pc.dprec}
 \alias{inla.pc.dprec}
 \alias{pc.pprec}
 \alias{inla.pc.pprec}
 \alias{pc.qprec}
 \alias{inla.pc.qprec}

 \title{Utility functions for the PC prior for the precision}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the precision
              in the Gaussian distribution.}
 \usage{
 inla.pc.rprec(n, u, alpha, lambda)
 inla.pc.dprec(prec, u, alpha, lambda, log = FALSE)
 inla.pc.qprec(p, u, alpha, lambda)
 inla.pc.pprec(q, u, alpha, lambda)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{u}{The upper limit (see Details)}
   \item{alpha}{The probability going above the upper limit (see Details)}
   \item{lambda}{The rate parameter (see Details)}
   \item{prec}{Vector of precisions}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
     The statement \code{Prob(1/sqrt(prec) > u) = alpha} is used to
     determine \code{lambda} unless \code{lambda} is given.
     Either \code{lambda}  must be given,  or
     \code{u} AND \code{alpha}.
 }
\value{%%
  \code{inla.pc.dprec} gives the density,
  \code{inla.pc.pprec} gives the distribution function,
  \code{inla.pc.qprec} gives the quantile function, and
  \code{inla.pc.rprec} generates random deviates.
 }
 \seealso{inla.doc("pc.prec")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 prec = inla.pc.rprec(100,  lambda = 1)
 d = inla.pc.dprec(prec, lambda = 1)
 prec = inla.pc.qprec(0.5, u = 1, alpha=0.01)
 inla.pc.pprec(prec, u = 1, alpha=0.01)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.1d}
\alias{inla.mesh.1d}
\alias{inla.mesh.1d.fem}
\title{Function space definition objects for 1D SPDE models.}
\usage{
inla.mesh.1d(
  loc,
  interval = range(loc),
  boundary = NULL,
  degree = 1,
  free.clamped = FALSE,
  ...
)

inla.mesh.1d.fem(mesh)
}
\arguments{
\item{loc}{B-spline knot locations.}

\item{interval}{Interval domain endpoints.}

\item{boundary}{Boundary condition specification.  Valid conditions are
\code{c('neumann', 'dirichlet', 'free', 'cyclic')}.  Two separate values can
be specified, one applied to each endpoint.}

\item{degree}{The B-spline basis degree.  Supported values are 0, 1, and 2.}

\item{free.clamped}{If \code{TRUE}, for \code{'free'} boundaries, clamp the
basis functions to the interval endpoints.}

\item{\dots}{Additional option, currently unused.}

\item{mesh}{An inla.mesh.1d object}
}
\description{
Create a 1D mesh specification \code{inla.mesh.1d} object, that defines a
function space for 1D SPDE models.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.spde.make.index}
\alias{inla.spde.make.index}
\title{SPDE model index vector generation}
\usage{
inla.spde.make.index(name, n.spde, n.group = 1, n.repl = 1, ...)
}
\arguments{
\item{name}{A character string with the base name of the effect.}

\item{n.spde}{The size of the model, typically from \code{spde$n.spde}.}

\item{n.group}{The size of the \code{group} model.}

\item{n.repl}{The number of model replicates.}

\item{...}{Additional parameters.  Currently unused.}
}
\value{
A list of named index vectors. \item{name }{Indices into the vector
of latent variables} \item{name.group }{'group' indices} \item{name.repl
}{Indices for replicates}
}
\description{
Generates a list of named index vectors for an SPDE model.
}
\examples{

loc <- matrix(runif(100 * 2), 100, 2)
mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(0.1, 0.5))
spde <- inla.spde2.matern(mesh)
index <- inla.spde.make.index("spatial", spde$n.spde, n.repl = 2)
spatial.A <- inla.spde.make.A(mesh, loc,
    index = rep(1:nrow(loc), 2),
    repl = rep(1:2, each = nrow(loc))
)
y <- 10 + rnorm(100 * 2)
stack <- inla.stack(
    data = list(y = y),
    A = list(spatial.A),
    effects = list(c(index, list(intercept = 1))),
    tag = "tag"
)
data <- inla.stack.data(stack, spde = spde)
formula <- y ~ -1 + intercept + f(spatial,
    model = spde,
    replicate = spatial.repl
)
result <- inla(formula,
    family = "gaussian", data = data,
    control.predictor = list(A = inla.stack.A(stack))
)
spde.result <- inla.spde2.result(result, "spatial", spde)
}
\seealso{
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}, \code{\link[=inla.spde2.result]{inla.spde2.result()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pardiso.R
%%
\name{inla.pardiso}
\alias{inla.pardiso}
\alias{inla.pardiso.check}
\title{PARDISO support in R-INLA}
\description{
Describe and check the PARDISO support in R-INLA
}
\usage{
inla.pardiso()
inla.pardiso.check()
}
\arguments{
}
\details{
\code{inla.pardiso()} describes the \code{PARDISO} support in R-INLA, how to get the license
key and enable it in the \code{R-INLA} package. \code{inla.pardiso.check()} check if the
\code{PARDISO} support is working.
}
\value{
}
\author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-alphaw.R
%%
 \name{pc.alphaw}
 \alias{inla.pc.alphaw}
 \alias{pc.alphaw}
 \alias{pc.ralphaw}
 \alias{inla.pc.ralphaw}
 \alias{pc.dalphaw}
 \alias{inla.pc.dalphaw}
 \alias{pc.palphaw}
 \alias{inla.pc.palphaw}
 \alias{pc.qalphaw}
 \alias{inla.pc.qalphaw}

 \title{Utility functions for the PC prior for the \code{alpha} parameter in the Weibull likelihood}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the \code{alpha} parameter
              in the Weibull likelihood}
 \usage{
 inla.pc.ralphaw(n, lambda = 5)
 inla.pc.dalphaw(alpha, lambda = 5, log = FALSE)
 inla.pc.qalphaw(p, lambda = 5)
 inla.pc.palphaw(q, lambda = 5)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter in the PC-prior}
   \item{alpha}{Vector of evaluation points, where \code{alpha>0}.}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
 This gives the PC prior for the \code{alpha} parameter for the Weibull likelihood,
 where \code{alpha=1} is the base model.
 }
\value{%%
  \code{inla.pc.dalphaw} gives the density,
  \code{inla.pc.palphaw} gives the distribution function,
  \code{inla.pc.qalphaw} gives the quantile function, and
  \code{inla.pc.ralphaw} generates random deviates.
 }
 \seealso{inla.doc("pc.alphaw")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 x = inla.pc.ralphaw(100,  lambda = 5)
 d = inla.pc.dalphaw(x, lambda = 5)
 x = inla.pc.qalphaw(0.5, lambda = 5)
 inla.pc.palphaw(x, lambda = 5)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.row.kron}
\alias{inla.row.kron}
\title{Row-wise Kronecker products}
\usage{
inla.row.kron(M1, M2, repl = NULL, n.repl = NULL, weights = NULL)
}
\arguments{
\item{M1}{A matrix that can be transformed into a sparse Matrix.}

\item{M2}{A matrix that can be transformed into a sparse Matrix.}

\item{repl}{An optional index vector.  For each entry, specifies which
replicate the row belongs to, in the sense used in
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}.}

\item{n.repl}{The maximum replicate index, in the sense used in
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}.}

\item{weights}{Optional scaling weights to be applied row-wise to the
resulting matrix.}
}
\value{
A \code{sparseMatrix} object.
}
\description{
Takes two Matrices and computes the row-wise Kronecker product.  Optionally
applies row-wise weights and/or applies an additional 0/1 row-wise Kronecker
matrix product, as needed by \code{\link[=inla.spde.make.A]{inla.spde.make.A()}}.
}
\seealso{
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from sm.R
%%
\name{inla.as.sparse}
\alias{inla.as.sparse}
\alias{inla.as.dgTMatrix}

\title{Convert a matrix or sparse matrix into the sparse formate used by INLA}

\description{Convert a matrix or sparse matrix into the sparse format used by INLA (dgTMatrix)}

\usage{
 inla.as.sparse(...)
 inla.as.dgTMatrix(A, unique = TRUE, na.rm = FALSE, zeros.rm = FALSE)
}

\arguments{
  \item{...}{The arguments. The matrix or sparse matrix,  and the additonal arguments}
  \item{A}{The matrix}
  \item{unique}{Logical. If \code{TRUE}, then ensure that the internal representation is unique and
                there are no duplicated entries.
                (Do not change this unless you know what you are doing.)}
  \item{na.rm}{Replace \code{NA}'s in the matrix with zeros.}
  \item{zeros.rm}{Remove zeros in the matrix.}
}

\value{%%
  \code{inla.as.sparse} and \code{inla.as.dgTMatrix} is the same function.
  The returned value is a sparse matrix in the \code{dgTMatrix}-format.
}
%%

\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
 A = matrix(1:9, 3, 3)
 inla.as.sparse(A)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.sp_get_crs}
\alias{inla.sp_get_crs}
\title{Extract CRS information}
\usage{
inla.sp_get_crs(x)
}
\arguments{
\item{x}{A \code{sp::Spatial} object}
}
\value{
A \code{CRS} object, or NULL if no valid CRS identified
}
\description{
Wrapper for CRS(projargs) (PROJ4) and CRS(wkt) for \code{sp::Spatial}
objects.

This function is a convenience method to workaround PROJ4/PROJ6 differences,
and the lack of a crs extraction method for Spatial objects.
}
\examples{

\dontrun{
if (interactive()) {
    s <- sp::SpatialPoints(matrix(1:6, 3, 2), proj4string = inla.CRS("sphere"))
    inla.sp_get_crs(s)
}
}

}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from version.R
%%
\name{inla.version}
\alias{version}
\alias{inla.version}

\title{Show the version of the INLA-package}

\description{Show the version of the INLA-package}

\usage{
inla.version(what = c("default", "version", "date"))
}

\arguments{
  \item{what}{What to show version of}
}

\value{%%
  \code{inla.version} display the current version information using \code{cat}
  with
  \code{default} or \code{info},  or return
  other spesific requests through the call.
}
%%

\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
## Summary of all
inla.version()
## The building date
inla.version("date")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.wkt_is_geocent}
\alias{inla.wkt_is_geocent}
\alias{inla.crs_is_geocent}
\alias{inla.wkt_get_ellipsoid_radius}
\alias{inla.crs_get_ellipsoid_radius}
\alias{inla.wkt_set_ellipsoid_radius}
\alias{inla.crs_set_ellipsoid_radius}
\alias{inla.wkt_unit_params}
\alias{crs_wkt}
\alias{inla.wkt_get_lengthunit}
\alias{inla.wkt_set_lengthunit}
\alias{inla.crs_get_wkt}
\alias{inla.crs_get_lengthunit}
\alias{inla.crs_set_lengthunit}
\title{Handling CRS/WKT}
\usage{
inla.wkt_is_geocent(wkt)

inla.crs_is_geocent(crs)

inla.wkt_get_ellipsoid_radius(wkt)

inla.crs_get_ellipsoid_radius(crs)

inla.wkt_set_ellipsoid_radius(wkt, radius)

inla.crs_set_ellipsoid_radius(crs, radius)

inla.wkt_unit_params()

inla.wkt_get_lengthunit(wkt)

inla.wkt_set_lengthunit(wkt, unit, params = NULL)

inla.crs_get_wkt(crs)

inla.crs_get_lengthunit(crs)

inla.crs_set_lengthunit(crs, unit, params = NULL)
}
\arguments{
\item{wkt}{A WKT2 character string}

\item{crs}{A \code{sp::CRS} or \code{inla.CRS} object}

\item{unit}{character, name of a unit. Supported names are
"metre", "kilometre", and the aliases "meter", "m", International metre",
"kilometer", and "km", as defined by \code{inla.wkt_unit_params} or the
\code{params} argument. (For legacy PROJ4 use, only "m" and "km" are
supported)}

\item{params}{Length unit definitions, in the list format produced by
\code{inla.wkt_unit_params()}, Default: NULL, which invokes
\code{inla.wkt_unit_params()}}
}
\value{
For \code{inla.wkt_unit_params}, a list of named unit definitions

For \code{inla.wkt_get_lengthunit}, a list of length units used in the wkt
string, excluding the ellipsoid radius unit.

For \code{inla.wkt_set_lengthunit}, a WKT2 string with altered length units.
Note that the length unit for the ellipsoid radius is unchanged.

For \code{inla.crs_get_wkt}, WKT2 string.

For \code{inla.crs_get_lengthunit}, a list of length units used in the wkt
string, excluding the ellipsoid radius unit. (For legacy PROJ4 code, the raw
units from the proj4string are returned, if present.)

For \code{inla.crs_set_lengthunit}, a \code{sp::CRS} object with altered
length units. Note that the length unit for the ellipsoid radius is
unchanged.

For \code{inla.wkt_unit_params}, a
list of named unit definitions

For \code{inla.wkt_get_lengthunit}, a
list of length units used in the wkt string, excluding the ellipsoid radius
unit.

For \code{inla.wkt_set_lengthunit}, a
WKT2 string with altered length units.
Note that the length unit for the ellipsoid radius is unchanged.

For \code{inla.crs_get_wkt}, WKT2 string.

For \code{inla.crs_get_lengthunit}, a
list of length units used in the wkt string, excluding the ellipsoid radius
unit. (For legacy PROJ4 code, the raw units from the proj4string are
returned, if present.)

For \code{inla.crs_set_lengthunit}, a \code{sp::CRS} object with
altered length units.
Note that the length unit for the ellipsoid radius is unchanged.
}
\description{
Get and set CRS object or WKT string properties.
}
\examples{

\dontrun{
if (inla.has_PROJ6()) {
    c1 <- inla.CRS("globe")
    inla.crs_get_lengthunit(c1)
    c2 <- inla.crs_set_lengthunit(c1, "km")
    inla.crs_get_lengthunit(c2)
}
}

\dontrun{
if (inla.has_PROJ6()) {
    c1 <- inla.CRS("globe")
    inla.crs_get_lengthunit(c1)
    c2 <- inla.crs_set_lengthunit(c1, "km")
    inla.crs_get_lengthunit(c2)
}
}
}
\seealso{
\code{\link[=inla.sp_get_crs]{inla.sp_get_crs()}}

\code{\link[=inla.sp_get_crs]{inla.sp_get_crs()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}

Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Surg}
\alias{Surg}
\title{Surgical: Institutional ranking}
\format{
A data frame with 12 observations on the following 3 variables.
\describe{ \item{n}{Number of deaths} \item{r}{Total number
of cases} \item{hospital}{a factor with levels \code{A} \code{B}
\code{C} \code{D} \code{E} \code{F} \code{G} \code{H} \code{I} \code{J}
\code{K} \code{L}} }
}
\source{
WinBUGS/OpenBUGS Manual Examples Vol. I
}
\description{
This example considers mortality rates in 12 hospitals performing cardiac
surgery in babies
}
\examples{

data(Surg)
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from collect.results.R
%%
 \name{inla.collect.results}
 \alias{inla.collect.results}
 \alias{collect.results}
 \title{Collect results from a inla-call}
 \description{\code{inla.collect.results} collect results  from a inla-call}
 \usage{
 inla.collect.results(
              results.dir,
              debug=FALSE,
              only.hyperparam=FALSE,
              file.log = NULL,
              file.log2 = NULL)
}
 \arguments{
 \item{results.dir}{The directory where the results of the inla run are stored}
 \item{debug}{Logical. If \code{TRUE} some debugging information are printed}
 \item{only.hyperparam}{Binary variable indicating wheather only the
 results for the hyperparameters should be collected}
 \item{file.log}{Character. The filename, if any, of the logfile for
 the internal calculations}
 \item{file.log2}{Character. The filename, if any, of the logfile2 for
 the internal calculations}
 }
 \value{ The function returns an object of class \code{"inla"}, see the
 help file for \code{inla} for details.}

 \details{This function is mainly used inside \code{inla}
  to collect results after running the inla
  function. It can also be used to collect results into R after having
  runned a inla section outside R.  }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from graph.matrix.R
%%
\name{graph2matrix}
\alias{graph2matrix}
\alias{inla.graph2matrix}
\alias{spy}
\alias{inla.spy}
\title{Construct a neighbour-matrix from a \code{graph}}
\description{Construct a neighbour-matrix from a \code{graph} and disaply it}
\usage{
    inla.graph2matrix(graph, ...)
    inla.spy(graph, ..., reordering = NULL, factor = 1.0, max.dim = NULL)
}
\arguments{
    \item{graph}{An \code{inla.graph}-object, a (sparse) symmetric matrix, a filename containing the graph,
                 or a list or collection of characters and/or numbers defining the graph.}
    \item{reordering}{A possible reordering. Typical the one obtained from a \code{inla}-call,  \code{result$misc$reordering},
                      or the result of \code{inla.qreordering}.}
    \item{factor}{A scaling of the \code{inla.graph}-object to reduce the size.}
    \item{max.dim}{Maximum dimension of the \code{inla.graph}-object plotted;
                   if \code{missing(factor)} and \code{max.dim} is set,  then \code{factor}
                   is computed automatically to give the given \code{max.dim}.}
    \item{...}{Additional arguments to \code{inla.read.graph()}}
}
\value{
     \code{inla.graph2matrix} returns a sparse symmetric matrix where the non-zero pattern is defined by the \code{graph}.
     The \code{inla.spy} function, plots a binary image of a \code{graph}. The \code{reordering} argument
     is typically the reordering used by \code{inla}, found in \code{result$misc$reordering}.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{
    \code{\link{inla.read.graph}}, \code{inla.qreordering}
}
\examples{
n = 50
Q = matrix(0, n, n)
idx = sample(1:n, 2*n, replace=TRUE)
Q[idx, idx] = 1
diag(Q) = 1
g = inla.read.graph(Q)
QQ = inla.graph2matrix(g)
inla.spy(QQ)
print(all.equal(as.matrix(Q), as.matrix(QQ)))

g.file = inla.write.graph(g)
inla.dev.new()
inla.spy(g.file)
inla.spy(g.file,  reordering = inla.qreordering(g))

g = inla.read.graph(g.file)
inla.dev.new()
inla.spy(g)

inla.dev.new()
inla.spy(3, 1, "1 2 2 1 1 3 0")
inla.dev.new()
inla.spy(3, 1, "1 2 2 1 1 3 0", reordering = 3:1)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from fgn.R
%%
 \name{fgn}
 \alias{inla.fgn}
 \alias{fgn}

 \title{Return the coefficients in the 3-component AR(1) mixture representing FGN(H)}

 \description{This function will return the coefficients in the 3-component AR(1)
              mixture representing FGN(H)}

 \usage{
     inla.fgn(H, K=4L, lag.max = NULL, approx = TRUE)
 }

 \arguments{
   \item{H}{The Hurst coeffcient (0<H<1),  or a vector of those}
   \item{K}{The number of components in representation,  must be 3L or 4L}
   \item{lag.max}{Integer. If positive integer, return the coeffcients implicitely as the ACF
                  from 0 to \code{lag.max}}
   \item{approx}{Logical. If \code{lag.max} is an positive integer and \code{approx} is \code{FALSE},
                   then return the true ACF instead of the approximated one.}
  }
 \value{
  \code{inla.fgn} returns a named matrix.
  If \code{is.null(lag.max)},  then
  first column is \code{H},
  columns \code{1+1:K} are lag one correlations (or phi's),
  and columns \code{1+K+1:K} are the weights.
  If \code{lag.max > 0},  then return the ACFs in columns \code{2+(0:lag.max)},
  for the H in column 1,  either the approximated ones or the the true ones.

  This function is EXPERIMENTAL!!!
 }
 \author{Havard Rue \email{hrue@r-inla.org}}

 \examples{
     r = c(inla.fgn(0.7))
     r_m = inla.fgn(seq(0.6, 0.8, by=0.01))
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.diameter}
\alias{inla.diameter}
\alias{inla.diameter.default}
\alias{inla.diameter.inla.mesh}
\alias{inla.diameter.inla.mesh.segment}
\alias{inla.diameter.inla.mesh.lattice}
\alias{inla.diameter.inla.mesh.1d}
\title{Diameter of a point set}
\usage{
inla.diameter(x, ...)

\method{inla.diameter}{default}(x, manifold = "", ...)

\method{inla.diameter}{inla.mesh.1d}(x, ...)

\method{inla.diameter}{inla.mesh}(x, ...)

\method{inla.diameter}{inla.mesh.segment}(x, ...)

\method{inla.diameter}{inla.mesh.lattice}(x, ...)
}
\arguments{
\item{x}{A point set as an \eqn{n\times d}{n x d} matrix, or an
\code{inla.mesh} related object.}

\item{\dots}{Additional parameters passed on to other methods.}

\item{manifold}{Character string specifying the manifold type. Default is to
treat the point set with Euclidean \eqn{R^d} metrics. Use
\code{manifold="S2"} for great circle distances on the unit sphere (this is
set automatically for \code{inla.mesh} objects).}
}
\value{
A scalar, upper bound for the diameter of the convex hull of the
point set.
}
\description{
Find an upper bound to the convex hull of a point set
}
\details{
\itemize{
\item \code{inla.diameter.default}
Calculate upper bound for the diameter of a point set,
by encapsulating in a circular domain.
}
}
\examples{

inla.diameter(matrix(c(0, 1, 1, 0, 0, 0, 1, 1), 4, 2))
}
\author{
Finn Lindgren \href{mailto:finn.lindgren@gmail.com}{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-cor1.R
%%
 \name{pc.cor1}
 \alias{inla.pc.cor1}
 \alias{pc.cor1}
 \alias{pc.rcor1}
 \alias{inla.pc.rcor1}
 \alias{pc.dcor1}
 \alias{inla.pc.dcor1}
 \alias{pc.pcor1}
 \alias{inla.pc.pcor1}
 \alias{pc.qcor1}
 \alias{inla.pc.qcor1}

 \title{Utility functions for the PC prior for correlation in AR(1)}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the correlation
              in the Gaussian AR(1) model where the base-model
              is correlation one.}
 \usage{
 inla.pc.rcor1(n, u, alpha, lambda)
 inla.pc.dcor1(cor, u, alpha, lambda, log = FALSE)
 inla.pc.qcor1(p, u, alpha, lambda)
 inla.pc.pcor1(q, u, alpha, lambda)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{u}{The upper limit (see Details)}
   \item{alpha}{The probability going above the upper limit (see Details)}
   \item{lambda}{The rate parameter (see Details)}
   \item{cor}{Vector of correlations}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
     The statement \code{Prob(cor > u) = alpha} is used to
     determine \code{lambda} unless \code{lambda} is given.
     Either \code{lambda}  must be given,  or
     \code{u} AND \code{alpha}.
 }
\value{%%
  \code{inla.pc.dcor1} gives the density,
  \code{inla.pc.pcor1} gives the distribution function,
  \code{inla.pc.qcor1} gives the quantile function, and
  \code{inla.pc.rcor1} generates random deviates.
 }
 \seealso{inla.doc("pc.rho1")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 cor = inla.pc.rcor1(100,  lambda = 1)
 d = inla.pc.dcor1(cor, lambda = 1)
 cor = inla.pc.qcor1(c(0.3, 0.7), u = 0.5, alpha=0.75)
 inla.pc.pcor1(cor, u = 0.5, alpha=0.75)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.fem}
\alias{inla.mesh.fem}
\alias{inla.mesh.1d.fem}
\title{Finite element matrices}
\usage{
inla.mesh.fem(mesh, order = 2)
}
\arguments{
\item{mesh}{An \code{\link[=inla.mesh]{inla.mesh()}} or \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object.}

\item{order}{The model order.}
}
\value{
A list of sparse matrices based on basis functions \code{psi_i}:
\item{c0 }{\verb{c0[i,j] = < psi_i, 1 >}} \item{c1 }{\verb{c1[i,j] = < psi_i, psi_j >}} \item{g1 }{\verb{g1[i,j] = < grad psi_i, grad psi_j >}}
\item{g2 }{\code{g2 = g1 * c0^-1 * g1}} \item{gk }{\code{gk = g1 * (c0^-1 * g1)^(k-1)}, up to and including \code{k=order}}
}
\description{
Constructs finite element matrices for \code{\link[=inla.mesh]{inla.mesh()}} and
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} objects.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from knmodels.R
%%
 \name{inla.knmodels}
 \alias{inla.knmodels}
 \alias{knmodels}
 \title{Spacetime interaction models}
 \description{
    It implements the models in Knorr-Held, L. (2000)
    with three different constraint approaches:
    sum-to-zero, contrast or diagonal add.
 }
 \usage{
inla.knmodels(
 formula,
 data,
 progress=FALSE,
 control.st=list(
   t=NULL,
   s=NULL,
   st=NULL,
   graph=NULL,
   type=c(paste(1:4), paste0(2:4, 'c'), paste0(2:4, 'd')),
   diagonal=1e-5,
   ...)
 )
}
 \arguments{
 \item{formula}{The formula specifying the other
   model components, without the spacetime
   interaction term. The spacetime interaction term
   will be added accordly to the specification in
   the \code{control.st} argument. See \code{inla}}
 \item{progress}{If it is to be shown the model
   fitting progress. Useful if more than one
   interaction type is being fitted.}
 \item{control.st}{Named list of arguments to control
   the spacetime interaction. It should contain:
 \describe{
  \item{time}{to be used as the index set for the
   main temporal effect which will be considered
   for the constraints when it is the case.}
  \item{space}{to be used as the index set for the
   main spatial effect which will be considered
   for the constraints when it is the case.}
  \item{spacetime}{to be the index set for the
   spacetime interaction effect.}
 \item{graph}{to be the graph for the spatial neighbor
   structure to be used in a \code{\link{f}} term
   for the main spatial random effect term or for
   building the spacetime interaction model.}
 \item{type}{to specify the spacetime interaction type.
   \code{1} to \code{4} corresponds to the four
   interaction types in Knorr-Held, L. (2000) with
   all the needed sum-to-zero constraints.
   \code{2c}, \code{3c} and \code{4c} are
   the contrast version considering the first time
   or space constrained to be equal to zero.
   \code{2d}, \code{3d} and \code{4d} are the
   corresponding versions when considering the
   diagonal add approach.}
 \item{diagonal}{to be the value to be added to the
   diagonal when using the diagonal add approach.}
 \item{timeref}{to specify the time point to be the
   reference time in the contrast parametrization.}
 \item{spaceref}{to specify the area to be the
   reference for the contrast parametrization.}
  \item{...}{where additional arguments can be
   passed to \code{\link{f}} function.
   Specification of the hyperparameter,
   fixed or random, initial value, prior and its
   parameters for the spacetime interaction. See
   \code{?inla.models} and look for \code{generic0}.
   By default we scale it and use the PC-prior to set
   the prior using the \code{pc.prec} prior with
   \code{param = c(0.5, 0.5)}. See documentation with
   \code{?inla.doc("pc.prec")}.}
 }
 }
 \item{...}{Arguments to be passed to the
   \code{\link{inla}} function.}
}
 \value{
  \code{inla.knmodels} returns an object of class \code{"inla"}.
    or a list of objects of this class if it is asked to compute
    more than one interaction type at once.
 Note: when the model type is 2c, 3c, 4c, 2d, 3d or 4d, it also
   includes linear combinations summary.
 }
 \author{Elias T. Krainski}
 \seealso{
     \code{\link{inla.knmodels.sample}} to sample from
 }
 \examples{
### define space domain as a grid
grid <- SpatialGrid(GridTopology(c(0,0), c(1, 1), c(4, 5)))
(n <- nrow(xy <- coordinates(grid)))

### build a spatial neighborhood list
jj <- lapply(1:n, function(i)
    which(sqrt((xy[i,1]-xy[,1])^2 + (xy[i,2]-xy[,2])^2)==1))

### build the spatial adjacency matrix
graph <- sparseMatrix(rep(1:n, sapply(jj, length)),
                      unlist(jj), x=1, dims=c(n, n))

### some random data at 10 time point
dat <- inla.knmodels.sample(graph, m=10, tau.t=2, tau.s=2, tau.st=3)
str(dat)
sapply(dat$x, summary)

nd <- length(dat$x$eta)
dat$e <- runif(nd, 0.9, 1.1)*rgamma(n, 40, 2)
dat$y <- rpois(nd, dat$e*exp(dat$x$eta-3))
summary(dat$y)

### fit the type 4 considering three different approaches
tgraph <- sparseMatrix(i=c(2:10, 1:9), j=c(1:9, 2:10), x=1)
res <- inla.knmodels(y ~ f(time, model='bym2', graph=tgraph) +
     f(space, model='bym2', graph=graph),
     data=dat, family='poisson', E=dat$E, progress=TRUE,
     control.st=list(time=time, space=space,
        spacetime=spacetime, graph=graph, type=c(4, '4c', '4d')),
     control.compute=list(dic=TRUE, waic=TRUE, cpo=TRUE))
sapply(res, function(x)
       c(dic=x$dic$dic, waic=x$waic$waic, cpo=-sum(log(x$cpo$cpo))))
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from prune.R
%%
\name{inla.prune}
\alias{inla.prune}
\alias{prune}

\title{Prune the INLA-package}
\description{Prune the INLA-package by deleting binary files not supported by the running OS}
\usage{
inla.prune(ask = TRUE)
}
\arguments{
\item{ask}{Logical. If TRUE, then ask for user confirmation before deleting.
           If FALSE, then delete without user confirmation.}
}
\value{No value is returned.}
\author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from q.R
%%
\name{inla.qstat}
\alias{inla.qstat}
\alias{inla.q}
\alias{inla.qget}
\alias{inla.qdel}
\alias{inla.qnuke}
\alias{inla.qlog}
\alias{summary.inla.q}
\alias{print.inla.q}
\title{Control and view a remote inla-queue}
\description{
Control and view a remote inla-queue of submitted jobs
}
\usage{
inla.qget(id, remove = TRUE)
inla.qdel(id)
inla.qstat(id)
inla.qlog(id)
inla.qnuke()
\method{summary}{inla.q}(object,...)
\method{print}{inla.q}(x,...)
}
\arguments{
  \item{id}{The job-id which is the output from \code{inla} when the job is submitted,  the
            job-number or job-name. For \code{inla.qstat}, \code{id} is optional and if omitted
            all the jobs will be listed.}
  \item{remove}{Logical If FALSE, leave the job on the server after retrival,
                otherwise remove it (default).}
  \item{x}{An \code{inla.q}-object which is the output from \code{inla.qstat}}
  \item{object}{An \code{inla.q}-object  which is the output from \code{inla.qstat}}
  \item{...}{ other arguments.}
}
\details{
\code{inla.qstat} show job(s) on the server,
\code{inla.qget} fetch the results (and by default remove
the files on the server),  \code{inla.qdel} removes
a job on the server and \code{inla.qnuke} remove all jobs on the server.
\code{inla.qlog} fetches the logfile only.

The recommended procedure is to use \code{r=inla(...,
inla.call="submit")} and then do \code{r=inla.qget(r)} at a later
stage.  If the job is not finished, then \code{r} will not be
overwritten and this step can be repeated.  The reason for this
procedure, is that some information usually stored in the result
object does not go through the remote server, hence have to be
appended to the results that are retrieved from the server. Hence
doing \code{r=inla(..., inla.call="submit")} and then later retrive
it using \code{r=inla.qget(1)}, say, then \code{r} does not contain
all the usual information.  All the main results are there, but
administrative information which is required to call
\code{inla.hyperpar} or \code{inla.rerun} are not there.
}
\value{
  \code{inla.qstat} returns an \code{inla.q}-object with information about current jobs.
}
\author{Havard Rue}
\seealso{ \code{\link{inla}} }
\examples{
\dontrun{
r = inla(y~1, data = data.frame(y=rnorm(10)), inla.call="submit")
inla.qstat()
r = inla.qget(r, remove=FALSE)
inla.qdel(1)
inla.qnuke()
}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.link}
\alias{control.link}
\alias{inla.control.link}
\alias{inla.set.control.link.default}
\alias{set.control.link.default}
\alias{control.link.default}
\title{Control variables in control.link}
\description{Control variables in \code{control.link} for use in \code{inla}}
\usage{
inla.set.control.link.default(...)
control.link(a, fixed, hyper, initial, model, order, param, prior, quantile, variant)
}
\arguments{
\item{...}{Possible arguments}
\item{model}{   The name of the link function/model}
\item{order}{   The \code{order} of the link function, where the interpretation of \code{order} is model-dependent.}
\item{variant}{   The \code{variant} of the link function, where the interpretation of \code{variant} is model-dependent.}
\item{hyper}{   Definition of the hyperparameter(s) for the link model chosen}
\item{quantile}{   The quantile for quantile link function}
\item{a}{   The parameter \code{a} in the LOGa link}
\item{initial}{   (OBSOLETE!) The initial value(s) for the hyperparameter(s)}
\item{fixed}{   (OBSOLETE!) A boolean variable if hyperparmater(s) is/are fixed or random}
\item{prior}{   (OBSOLETE!) The name of the prior distribution(s) for the hyperparmater(s)}
\item{param}{   (OBSOLETE!) The parameters for the prior distribution(s) for the hyperparmater(s)}
}
\value{
   The \code{control.link}-list is set within the corresponding \code{control.family}-list as the link is likelihood-familiy spesific.
The function \code{control.link} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.link.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from plot.R
%%
 \name{plot.inla}
 \alias{plot.inla}
 \alias{inla.plot}
 \title{Default INLA plotting}
 \description{
   Takes am \code{inla} object produced by \code{inla} and plot the results
 }
 \usage{
 \method{plot}{inla}(x,
              plot.fixed.effects = TRUE,
              plot.lincomb = TRUE,
              plot.random.effects = TRUE,
              plot.hyperparameters = TRUE,
              plot.predictor = TRUE,
              plot.q = TRUE,
              plot.cpo = TRUE,
              plot.prior = FALSE,
              single = FALSE,
              postscript = FALSE,
              pdf = FALSE,
              prefix = "inla.plots/figure-",
              intern = FALSE,
              debug = FALSE,
              cex = 1.75,
              ...)
 }
 \arguments{
   \item{x}{A fitted  \code{inla} object produced by \code{inla} }
   \item{plot.fixed.effects}{Boolean indicating if posterior marginals
     for the fixed effects in the model should be plotted }
   \item{plot.lincomb}{Boolean indicating if posterior marginals
     for the linear combinations should be plotted }
   \item{plot.random.effects}{Boolean indicating if posterior mean and quantiles
     for the random effects in the model should be plotted  }
   \item{plot.hyperparameters}{Boolean indicating if posterior marginals
     for the hyperparameters in the model should be plotted }
   \item{plot.predictor}{Boolean indicating if posterior mean and quantiles
     for the linear predictor in the model should be plotted }
   \item{plot.q}{Boolean indicating if precision matrix should be displayed}
   \item{plot.cpo}{Boolean indicating if CPO/PIT valuesshould be plotted}
   \item{plot.prior}{Plot also the prior density for the hyperparameters}
   \item{single}{Boolean indicating if there should be more than one plot per page
                 (FALSE) or just one (TRUE)}
   \item{postscript}{Boolean indicating if postscript files should be produced instead}
   \item{pdf}{Boolean indicating if PDF files should be produced instead}
   \item{prefix}{The prefix for the created files. Additional numbering and suffix is added.}
   \item{intern}{Plot also the hyperparameters in its internal scale.}
   \item{debug}{Write some debug information}
   \item{cex}{The \code{cex} parameter in \code{par()}. If set to \code{NULL} or \code{0},
              then default values will be used for graphics parameters}
   \item{...}{Additional arguments to \code{postscript()}, \code{pdf()} or \code{dev.new()}.}
 }
 \value{The return value is a list of the files created (if any).}
 \author{Havard Rue \email{hrue@r-inla.org} }
 \seealso{\code{\link{inla}}}
 \examples{
\dontrun{
result = inla(...)
plot(result)
plot(result, single = TRUE, plot.prior = TRUE)
plot(result, single = TRUE, pdf = TRUE, paper = "a4")
   }
 }
 \keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{BivMetaAnalysis}
\alias{BivMetaAnalysis}
\title{Bivariate Meta Analysis}
\format{
A data frame with 92 observations on the following 9 variables.
\describe{
\item{N}{a numeric vector}
\item{Y}{a numeric vector}
\item{diid}{a numeric vector}
\item{lag.tp}{a numeric vector}
\item{lag.tn}{a numeric vector}
\item{ct.tp}{a numeric vector}
\item{ct.tn}{a numeric vector}
\item{mr.tp}{a numeric vector}
\item{mr.tn}{a numeric vector}
}
}
\usage{
BivMetaAnalysis
}
\description{
Data are taken from a meta-analysis to compare the utility of three types of
diagnostic imaging - lymphangiography (LAG), computed tomography (CT) and
magnetic resonance (MR) - to detect lymph node metastases in patients with
cervical cancer. The dataset consists of a total of 46 studies: the first 17
for LAG, the following 19 for CT and the last 10 for MR.
}
\examples{

data(BivMetaAnalysis)
}
\references{
J. Scheidler and H. Hricak and K. K. Yu and L. Subak and M. R.
Segal,"Radiological evaluation of lymph node metastases in patients with
cervical cancer: a meta-analysis",JAMA 1997
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-gamma.R
%%
 \name{pc.gamma}
 \alias{inla.pc.gamma}
 \alias{pc.gamma}
 \alias{pc.rgamma}
 \alias{inla.pc.rgamma}
 \alias{pc.dgamma}
 \alias{inla.pc.dgamma}
 \alias{pc.pgamma}
 \alias{inla.pc.pgamma}
 \alias{pc.qgamma}
 \alias{inla.pc.qgamma}

 \title{Utility functions for the PC prior for \code{Gamma(1/a, 1/a)}}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for \code{Gamma(1/a, 1/a)}}
 \usage{
 inla.pc.rgamma(n, lambda = 1)
 inla.pc.dgamma(x, lambda = 1, log = FALSE)
 inla.pc.qgamma(p, lambda = 1)
 inla.pc.pgamma(q, lambda = 1)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter (see Details)}
   \item{x}{Evaluation points}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
 This gives the PC prior for the \code{Gamma(1/a, 1/a)} case,  where \code{a=0} is the
 base model.
 }
\value{%%
  \code{inla.pc.dgamma} gives the density,
  \code{inla.pc.pgamma} gives the distribution function,
  \code{inla.pc.qgamma} gives the quantile function, and
  \code{inla.pc.rgamma} generates random deviates.
 }
 \seealso{inla.doc("pc.gamma")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 x = inla.pc.rgamma(100,  lambda = 1)
 d = inla.pc.dgamma(x, lambda = 1)
 x = inla.pc.qgamma(0.5, lambda = 1)
 inla.pc.pgamma(x, lambda = 1)
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from merge.R
%%
 \name{inla.merge}
 \alias{inla.merge}
 \alias{merge.inla}
 \title{Merge a mixture of \code{inla}-objects}
 \description{Merge a mixture of \code{inla}-objects}
 \usage{
     \method{merge}{inla}(x, y, ..., prob = rep(1,  length(loo)),
                          mc.cores = NULL, verbose = FALSE)
     inla.merge(loo, prob = rep(1,  length(loo)), verbose = FALSE)
 }
 \arguments{
   \item{x}{An \code{inla}-object to be merged}
   \item{y}{An \code{inla}-object to be merged}
   \item{...}{Additional \code{inla}-objects to be merged}
   \item{loo}{List of \code{inla}-objects to be merged}
   \item{prob}{The mixture of (possibly unnormalized) probabilities}
   \item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
                   \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
                   and \code{inla.getOption("num.threads")} in that order.}
   \item{verbose}{Turn on verbose-output or not}
  }
 \value{
   A merged \code{inla}-object.
 }
 \details{
    The function \code{merge.inla} implements method \code{merge} for
    \code{inla}-objects. \code{merge.inla} is a wrapper for the function
    \code{inla.merge}. The interface is slightly different, \code{merge.inla}
    is more tailored for interactive use, whereas \code{inla.merge} is better
    in general code.

    \code{inla.merge} is intented for merging a mixture of \code{inla}-objects,
    each run with the same formula and settings, except for a set of
    hyperparameters that are fixed to different values. Using this function,
    we can then integrate over these hyperparameters using (unnormalized)
    integration weights
    \code{prob}. The main objects to be merged,  are the summary statistics
    and marginal densities (like for hyperparameters, fixed,  random,  etc).
    Not all entries in the object can be merged, and by default these
    are inheritated from the first object in the list,
    while some are just set to  \code{NULL}.
    Those objectes that are merged,
    will be listed if run with option \code{verbose=TRUE}.

    Note that merging hyperparameter in the user-scale is prone to
    discretization error in general, so it is more stable to convert
    the marginal of the hyperparameter from the merged internal scale
    to the user-scale. (This is not done by this function.)
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 set.seed(123)
 n = 100
 y = rnorm(n)
 y[1:10] = NA
 x = rnorm(n)
 z1 = runif(n)
 z2 = runif(n)*n
 idx = 1:n
 idx2 = 1:n
 lc1 = inla.make.lincomb(idx = c(1, 2, 3))
 names(lc1) = "lc1"
 lc2 = inla.make.lincomb(idx = c(0, 1, 2, 3))
 names(lc2) = "lc2"
 lc3 = inla.make.lincomb(idx = c(0, 0, 1, 2, 3))
 names(lc3) = "lc3"
 lc = c(lc1, lc2, lc3)
 rr = list()
 for (logprec in c(0, 1, 2))
     rr[[length(rr)+1]] = inla(y ~ 1 + x + f(idx, z1) + f(idx2, z2),
              lincomb = lc,
              control.family = list(hyper = list(prec = list(initial = logprec))),
              control.predictor = list(compute = TRUE, link = 1),
              data = data.frame(y, x, idx, idx2, z1, z2))
 r = inla.merge(rr, prob = seq_along(rr), verbose=TRUE)
 summary(r)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from sampler.R
%%
\name{inla.hyperpar.sample}
\alias{inla.hyperpar.sample}
\alias{inla.hyperpar.sampler}
\alias{hyperpar.sample}
\alias{hyperpar.sampler}

\title{Produce samples from the approximated joint posterior for the hyperparameters}

\description{Produce samples from the approximated joint posterior for the hyperparameters}

\usage{
inla.hyperpar.sample(n, result, intern=FALSE, improve.marginals = FALSE)
}

\arguments{
  \item{n}{Integer. Number of samples required.}
  \item{result}{An \code{inla}-object,  f.ex the output from an \code{inla}-call.}
  \item{intern}{Logical. If \code{TRUE} then produce samples in the
        internal scale for the hyperparmater, if \code{FALSE} then produce
        samples in the user-scale. (For example log-precision (intern)
        and precision (user-scale))}
  \item{improve.marginals}{Logical. If \code{TRUE}, then improve the samples taking into account
        possible better marginal estimates for the hyperparameters in \code{result}.}
}

\value{%%
 A matrix where each sample is a row. The contents of the column
 is described in the rownames.
}
%%

\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
n = 100
r = inla(y ~ 1 + f(idx), data = data.frame(y=rnorm(n), idx = 1:n))
ns = 500
x = inla.hyperpar.sample(ns, r)

rr = inla.hyperpar(r)
xx = inla.hyperpar.sample(ns, rr, improve.marginals=TRUE)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from reorderings.R
%%
\name{inla.reorderings}
\alias{inla.reorderings}
\alias{reorderings}

\title{Reorderings methods for sparse matrices}

\description{Provide the names of all implemented reordering schemes}

\usage{
 inla.reorderings()
}

\arguments{
 None
}

\value{The names of all available reorderings}
%%

\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
inla.reorderings()
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from hyperpar.R
%%
\name{inla.hyperpar}
\alias{inla.hyperpar}
\alias{hyperpar.inla}

\title{Improved estimates for the hyperparameters}
\description{
 Improve the estimates of the  posterior marginals
 for the hyperparameters of
 the model using the grid integration strategy.
}
\usage{
 inla.hyperpar(
         result,
         skip.configurations = TRUE,
         verbose = FALSE,
         dz = 0.75,
         diff.logdens = 15,
         h = NULL,
         restart = FALSE,
         quantiles = NULL,
         keep = FALSE)
}

\arguments{
\item{result}{An object of class
\code{inla},  ie a result of a call to
\code{inla()}}
\item{skip.configurations}{ A boolean variable; skip
configurations if the values at the main axis are to
small. (Default TRUE)}
\item{verbose}{Boolean indicating wheather the inla
program should run in a verbose mode.}
\item{dz}{Step length in the standardized scale used
in the construction of the grid, default 0.75.}
\item{diff.logdens}{The difference of the
log.density for the hyperpameters to stop numerical
integration using int.strategy='grid'.  Default 15}
\item{h}{The step-length for the gradient
calculations for the hyperparameters. Default 0.01.}
\item{restart}{A boolean defining wheather the
optimizer should start again to ind the mode or if it
should use the mode contained in the \code{object}}
\item{quantiles}{A vector of quantiles,
to compute for each posterior marginal.}
\item{keep}{A boolean variable indicating the
working files (ini file, data files and results files)
should be kept}
}
\value{
The object returned is the same as  \code{object} but the
estimates of the hyperparameters are replaced by improved estimates.}
\references{
See the references in \code{inla}}
\author{Havard Rue \email{hrue@r-inla.org}}
\note{This function might take a long time or if the number of
hyperparameters in the model is large. If it complains and says \code{I cannot get enough memory},
try to increase the value of the argument \code{dz} or decrease \code{diff.logdens}.}
\seealso{\code{\link{inla}}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ar.R
\name{inla.ar.pacf2phi}
\alias{inla.ar.pacf2phi}
\alias{ar.pacf2phi}
\alias{pacf2phi}
\alias{inla.ar.phi2pacf}
\alias{ar.phi2pacf}
\alias{phi2pacf}
\alias{inla.ar.phi2acf}
\alias{ar.phi2acf}
\alias{phi2acf}
\alias{inla.ar.pacf2acf}
\alias{ar.pacf2acf}
\alias{pacf2acf}
\title{Convert between parameterizations for the AR(p) model}
\usage{
inla.ar.pacf2phi(pac)

inla.ar.phi2pacf(phi)

inla.ar.phi2acf(phi, lag.max = length(phi))

inla.ar.pacf2acf(pac, lag.max = length(pac))
}
\arguments{
\item{pac}{The partial autorcorrelation coefficients}

\item{phi}{The AR(p) parameters \code{phi}}

\item{lag.max}{The maximum lag to compute the ACF for}
}
\value{
\itemize{
\item \code{inla.ar.pacf2phi} returns \code{phi} for given \code{pacf}.
\item \code{inla.ar.phi2pacf} returns \code{pac} for given \code{phi}.
\item \code{inla.ar.phi2acf} returns \code{acf} for given \code{phi}.
\item \code{inla.ar.pacf2acf} returns \code{acf} for given \code{pacf}.
}
}
\description{
These functions convert between the AR(p) coefficients \code{phi}, the
partial autorcorrelation coefficients \code{pacf} and the autocorrelation
function \code{acf}.  The \code{phi}-parameterization is the same as used
for \code{arima}-models in \code{R}; see \code{?arima} and the
parameter-vector \code{a} in \code{Details}.
}
\examples{

pac <- runif(5)
phi <- inla.ar.pacf2phi(pac)
pac2 <- inla.ar.phi2pacf(phi)
print(paste("Error:", max(abs(pac2 - pac))))
print("Correlation matrix (from pac)")
print(toeplitz(inla.ar.pacf2acf(pac)))
print("Correlation matrix (from phi)")
print(toeplitz(inla.ar.phi2acf(phi)))
}
\author{
Havard Rue \email{hrue@r-inla.org}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from barrier.R
%%
 \name{inla.barrier}
 \alias{inla.barrier}
 \alias{barrier}
 \alias{inla.barrier.pcmatern}
 \alias{barrier.pcmatern}
 \alias{inla.barrier.polygon}
 \alias{barrier.polygon}
 \alias{inla.barrier.q}
 \alias{barrier.q}
 \alias{inla.barrier.fem}
 \alias{barrier.fem}
 
 \title{Functions for defining the Barrier models}
 
 \description{Functions for defining Barrier models as an \code{inla rgeneric} model}

 \usage{
 inla.barrier.pcmatern(mesh, barrier.triangles, prior.range,
                       prior.sigma, range.fraction=0.2)
 inla.barrier.polygon(mesh, barrier.triangles, Omega=NULL)
 inla.barrier.q(fem, ranges, sigma=1)
 inla.barrier.fem(mesh, barrier.triangles, Omega=NULL)
 }
 \arguments{
   \item{mesh}{The mesh to build the model on, from inla.mesh.2d}
   \item{barrier.triangles}{The numerical ids of the triangles that make up the barrier area}
   \item{prior.range}{2 parameters \code{(range0,Prange)} for the prior spatial range. 
         If \code{Prange} is \code{NA}, then \code{range0} is used as a fixed range value (not tested).}
   \item{prior.sigma}{2 parameters \code{(sig0,Psig)} for the prior marginal standard deviation sigma. 
         If \code{Psig} is \code{NA}, then \code{sig0} is used as a fixed sigma value (not tested).}
   \item{range.fraction}{The length of the spatial range inside the barrier area,
                         as a fraction of the range parameter.}
   \item{Omega}{Advanced option for creating a set of permeable barriers (not documented)}
 }
 \details{
     This model is described in the ArXiv preprint arXiv:1608.03787.
     For examples, see \url{https://haakonbakka.bitbucket.io/btopic107.html}.
 }
\value{%%
  \code{inla.barrier.pcmatern} gives the (rgeneric) model object for fitting the model in INLA,
  \code{inla.barrier.polygon} gives the polygon around the barrier (mainly for plotting),
  \code{inla.barrier.q} is an internal method producing the Q matrix from a result of inla.barrier.fem,
  \code{inla.barrier.fem} is an internal method producing the Finite Element matrices.
 }
 \seealso{inla.spde2.pcmatern}
 \author{Haakon Bakka \email{bakka@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from priors.used.R
%%
\name{inla.priors.used}
\alias{inla.priors.used}
\alias{priors.used}
\title{Print priors used}
\description{Print the priors used for the hyperparameters}
\usage{
    inla.priors.used(result, digits=6L)
}
\arguments{
\item{result}{An \code{inla}-object, typically the output from an \code{inla()}-call}
\item{digits}{The \code{digits} argument to the function \code{format()}}
}
\details{
This function provides a more human-friendly output of \code{result$all.hyper}
of all the priors used for the hyperparameters.
Since not all information about the model is encoded in this object, more hyperparameters
than actually used, may be printed. In particular, \code{group.theta1} is printed
even though the argument \code{group} in \code{f()} is not used. Similarly for spde-models,
but the user should know that, for example, only the two first ones
are actually used.
Hopefully, this issue will be fixed in the future.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\examples{
r = inla(y ~ 1 + x, data = data.frame(y = 1:10, x = rep(1:5, 2)))
inla.priors.used(r)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.boundary}
\alias{inla.mesh.boundary}
\alias{inla.mesh.interior}
\title{Constraint segment extraction for inla.mesh}
\usage{
inla.mesh.boundary(mesh, grp = NULL)

inla.mesh.interior(mesh, grp = NULL)
}
\arguments{
\item{mesh}{An \code{inla.mesh} object.}

\item{grp}{Group indices to extract.  If \code{NULL}, all boundary/interior
constrain groups are extracted.}
}
\value{
A list of \code{inla.mesh.segment} objects.
}
\description{
Constructs an list of \code{inla.mesh.segment} object from boundary or
interior constraint information in an \code{\link[=inla.mesh]{inla.mesh()}} object.
}
\examples{

loc <- matrix(runif(100 * 2) * 1000, 100, 2)
mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(50, 500))
boundary <- inla.mesh.boundary(mesh)
interior <- inla.mesh.interior(mesh)
}
\seealso{
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}, \code{\link[=inla.mesh.create]{inla.mesh.create()}},
\code{\link[=inla.mesh.create.helper]{inla.mesh.create.helper()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Scotland}
\alias{Scotland}
\title{Conditional Autoregressive (CAR) model for disease mapping}
\format{
A data frame with 56 observations on the following 4 variables.
\describe{ \item{Counts}{The number of lip cancer registered}
\item{E}{The expected number of lip cancer } \item{X}{The
percentage of the population engaged in agricolture, fishing or forestry }
\item{Region}{The county} }
}
\source{
OpenBUGS Example manual, GeoBUGS
}
\description{
The rate of lip cancer in 56 counties in Scotland is recorder. The data set
includes the observed and expected cases (based on the population and its
age and sex distribution in the country), a covariate measuring the
percentage of the population engaged in agricolture, fishing or forestry and
the "position" of each county expressed as a list of adjacent counties
}
\examples{

data(Scotland)
}
\references{
Clayton and Kaldor (1987) and Breslow and Clayton (1993)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Tokyo}
\alias{Tokyo}
\title{Binomial time series}
\format{
A data frame with 366 observations on the following 3 variables.
\describe{
\item{y}{number of days with rain}
\item{n}{total number of days}
\item{time}{day of the year}
}
}
\source{
\url{http://www.math.ntnu.no/~hrue/GMRF-book/tokyo.rainfall.data.dat}
}
\description{
Recorded days of rain above 1 mm in Tokyo for 2 years, 1983:84
}
\examples{

data(Tokyo)
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from load.R
%%
 \name{inla.load}
 \alias{inla.load}
 \title{Load or source a file}
 \description{Load or source a file: (internal use)}
 \usage{
     inla.load(filename, debug = TRUE)
 }
 \arguments{
   \item{filename}{The name of the file to be loaded, alternatively, sourced. }
   \item{debug}{Logical. Turn on/off debug information.}
  }
 \value{
   None
 }
 \details{
   Try to \code{load} the file into the global environment,
   if that fail,  try to \code{source} the file into the
   global environment.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{summary.inla.mesh}
\alias{summary.inla.mesh}
\alias{print.summary.inla.mesh}
\title{Summarizing triangular mesh objects}
\usage{
\method{summary}{inla.mesh}(object, verbose = FALSE, ...)

\method{print}{summary.inla.mesh}(x, ...)
}
\arguments{
\item{object}{an object of class \code{"inla.mesh"}, usually a result of a
call to \code{\link[=inla.mesh.create]{inla.mesh.create()}} or \code{\link[=inla.mesh.2d]{inla.mesh.2d()}}.}

\item{verbose}{If \code{TRUE}, produce a more detailed output.}

\item{\dots}{further arguments passed to or from other methods.}

\item{x}{an object of class \code{"summary.inla.mesh"}, usually a result of
a call to \code{\link[=summary.inla.mesh]{summary.inla.mesh()}}.}
}
\description{
Construct and print \code{inla.mesh} object summaries
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Epil}
\alias{Epil}
\title{Repeated measures on Poisson counts}
\format{
A data frame with 236 observations on the following 7 variables.
\describe{
\item{y}{Number of seizures}
\item{Trt}{indicator for the presence of treatment}
\item{Base}{8-week baseline seizure counts}
\item{Age}{Age of the patient}
\item{V4}{indicator variable for the 4th visit.}
\item{rand}{a numeric vector}
\item{Ind}{indicator for the specific patient}
}
}
\source{
WinBUGS/OpenBUGS Manual Examples Vol I
}
\description{
Seizure counts in a randomised trial of anti-convulsant therpay in epilepsy
for 59 patients.
}
\examples{

data(Epil)
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from jmarginal.R
%%
 \name{joint.marginal}
 \alias{inla.joint.marginal}
 \alias{inla.joint.marginal.eval}
 \alias{joint.marginal}
 \alias{joint.marginal.eval}
 \alias{rjmarginal}
 \alias{rjmarginal.eval}
 \alias{inla.rjmarginal}
 \alias{inla.rjmarginal.eval}
 \alias{inla.tjmarginal}
 \alias{tjmarginal}
 \alias{inla.1djmarginal}
 \alias{1djmarginal}

 \title{Sample, transform and evaluate from a joint marginal approximation}

 \description{Sample, transform and evalue from from a joint marginal approximation
 as returned using argument \code{selection} in \code{inla}.
 }

 \usage{
 inla.rjmarginal(n, jmarginal, constr)
 inla.rjmarginal.eval(fun, samples, ...)
 inla.tjmarginal(jmarginal, A)
 inla.1djmarginal(jmarginal)
 }
 \arguments{
   \item{n}{The number of samples}
   \item{jmarginal}{A marginal object given  either by a \code{inla} object
     or \code{result$selection}}
   \item{constr}{Optional linear constraints;
                 see \code{?INLA::f} and argument \code{extraconstr}}
   \item{fun}{A function which is evaluated for each sample, similar to
              \code{inla.posterior.sample.eval}: please see the documentation
              for this functions for details. }
   \item{samples}{The samples, as in the form of the output from \code{inla.rjmarginal}}
   \item{A}{A matrix used for the linear combination}
 }

 \value{%%
 THESE FUNCTIONS ARE EXPERIMENTAL FOR THE MOMENT (JULY 2020)


 \code{inla.rjmarginal} returns a list with the samples in \code{samples}
 (matrix) and the corresponding log-densities
 in \code{log.density} (vector). Each column in \code{samples} contains one sample.

 \code{inla.rjmarginal.eval} returns a matrix, where each row is the (vector) function
 evaluated at each sample.

 \code{inla.tjmarginal} returns a \code{inla.jmarginal}-object of the linear combination
 defined by the matrix \code{A}.

 \code{inla.1djmarginal} return the marginal densities from a joint approximation.
 }

 \author{Cristian Chiuchiolo and Havard Rue \email{hrue@r-inla.org}}
 \seealso{\code{\link{inla}}}

 \examples{
 n = 10
 x = 1+rnorm(n)
 xx = 3 + rnorm(n)
 y = 1 + x + xx + rnorm(n)
 selection = list(xx=1, Predictor = 3:4, x=1)
 r = inla(y ~ 1 + x + xx,
          data = data.frame(y, x, xx),
          selection = selection)
 ns = 100
 xx = inla.rjmarginal(ns, r)

 print(cbind(mean = r$selection$mean, sample.mean = rowMeans(xx$samples)))
 print("cov matrix")
 print(round(r$selection$cov.matrix, dig=3))
 print("sample cov matrix")
 print(round(cov(t(xx$samples)), dig=3))

 skew = function(z) mean((z-mean(z))^3)/var(z)^1.5
 print(round(cbind(skew = r$selection$skewness,
                   sample.skew = apply(xx$sample, 1, skew)), dig=3))

 ## illustrating the eval function
 n = 10
 x = rnorm(n)
 eta = 1 + x
 y = eta + rnorm(n, sd=0.1)
 selection = list(x = 1, Predictor = c(1, 2, 4, 5),  '(Intercept)' = 1)
 r = inla(y ~ 1 + x,
          data = data.frame(y, x),
          selection = selection)
 xx = inla.rjmarginal(100,  r)
 xx.eval = inla.rjmarginal.eval(function() c(x, Predictor, Intercept),  xx)
 print(cbind(xx$samples[, 1]))
 print(cbind(xx.eval[, 1]))

 constr <- list(A = matrix(1, ncol = n, nrow = 1), e = 1)
 x <- inla.rjmarginal(10, r, constr = constr)

 A <- matrix(rnorm(n^2), n, n)
 b <- inla.tjmarginal(r, A)
 b.marg <- inla.1djmarginal(b)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.hazard}
\alias{control.hazard}
\alias{inla.control.hazard}
\alias{inla.set.control.hazard.default}
\alias{set.control.hazard.default}
\alias{control.hazard.default}
\title{Control variables in control.hazard}
\description{Control variables in \code{control.hazard} for use in \code{inla}}
\usage{
inla.set.control.hazard.default(...)
control.hazard(constr, cutpoints, diagonal, fixed, hyper, initial, model, n.intervals, param, prior, scale.model, strata.name)
}
\arguments{
\item{...}{Possible arguments}
\item{model}{   The model for the baseline hazard model. One of 'rw1', 'rw2' or 'iid'. (Default 'rw1'.)}
\item{hyper}{   The definition of the hyperparameters.}
\item{fixed}{   (OBSOLETE!) A boolean variable; is the precision for 'model' fixed? (Default FALSE.)}
\item{initial}{   (OBSOLETE!) The initial value for the precision.}
\item{prior}{   (OBSOLETE!) The prior distribution for the precision for 'model'}
\item{param}{   (OBSOLETE!) The parameters in the prior distribution}
\item{constr}{   A boolean variable; shall the 'model' be constrained to sum to zero?}
\item{diagonal}{   An extra constant added to the diagonal of the precision matrix}
\item{n.intervals}{   Number of intervals in the baseline hazard. (Default 15)}
\item{cutpoints}{   The cutpoints to use. If not specified the they are compute from 'n.intervals' and the maximum length of the interval. (Default NULL)}
\item{strata.name}{   The name of the stratefication variable for the baseline hazard in the data.frame}
\item{scale.model}{   Scale the baseline hazard model (RW1, RW2) so the generalized variance is 1. (Default \code{inla.getOption("scale.model.default")}.)}
}
\value{
  
The function \code{control.hazard} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.hazard.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.as.wkt_tree.wkt}
\alias{inla.as.wkt_tree.wkt}
\alias{inla.as.wkt.wkt_tree}
\alias{inla.wkt_tree_get_item}
\alias{inla.wkt_tree_set_item}
\title{Internal WKT handling}
\usage{
inla.as.wkt_tree.wkt(x, ...)

inla.as.wkt.wkt_tree(x, pretty = FALSE, ...)

inla.wkt_tree_get_item(x, item, duplicate = 1)

inla.wkt_tree_set_item(x, item_tree, duplicate = 1)
}
\arguments{
\item{x}{A WKT2 string, or a \code{wkt_tree} list structure}

\item{\dots}{Unused}

\item{item}{character vector with item labels identifying a parameter item
entry.}

\item{duplicate}{For items that have more than one match, \code{duplicate}
indicates the index number of the desired version. Default: 1}

\item{item_tree}{An item tree identifying a parameter item entry}
}
\description{
Conversion between WKT and a tree representation
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from summary.R
%%
\name{summary.inla}
\alias{summary.inla}
\alias{summary.surv.inla}
\alias{print.summary.inla}
\title{ Summary for a INLA fit }
\description{
Takes a fitted \code{inla} or \code{surv.inla} object produced by
\code{inla} or  \code{surv.inla} and produces
a summary from it.

}
\usage{
\method{summary}{inla}(object, digits = 3L, include.lincomb = TRUE, ...)
\method{print}{summary.inla}(x, digits = 3L, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{object}{  a fitted \code{inla} object as produced by
    \code{inla}.}
  \item{x}{a \code{summary.inla} object produced by \code{summary.inla}}
  \item{digits}{Integer Number of digits}
  \item{include.lincomb}{Logcial Include the summary for the the linear combinations or not}
  \item{...}{ other arguments.}

}
\details{
Posterior mean and standard deviation (together with quantiles or
cdf) are printed for the fixed effects in the model.

For the random effects the function \code{summary()} prints the
posterior mean and standard deviations for the hyperparameters

If the option \code{short.summary} is set to \code{TRUE} using
\code{inla.setOption},
then  a less verbose summary
variant will be used,  which might be
more suitable for Markdown documents.
}
\value{
  \code{summary.inla} returns an object of class \code{summary.inla}, a
  list of components to print.
}
\author{Sara Martino and Havard Rue}
\seealso{ \code{\link{inla}} }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.nonconvex.hull}
\alias{inla.nonconvex.hull}
\alias{inla.nonconvex.hull.basic}
\title{Nonconvex set extensions.}
\usage{
inla.nonconvex.hull(
  points,
  convex = -0.15,
  concave = convex,
  resolution = 40,
  eps = NULL,
  crs = NULL
)

inla.nonconvex.hull.basic(
  points,
  convex = -0.15,
  resolution = 40,
  eps = NULL,
  crs = NULL
)
}
\arguments{
\item{points}{2D point coordinates (2-column matrix).  Can alternatively be
a \code{SpatialPoints} or \code{SpatialPointsDataFrame} object.}

\item{convex}{The desired extension radius.  Also determines the smallest
allowed convex curvature radius.  Negative values are interpreted as
fractions of the approximate initial set diameter.}

\item{concave}{The desired minimal concave curvature radius. Default is
\code{concave=convex}.}

\item{resolution}{The internal computation resolution.  A warning will be
issued when this needs to be increased for higher accuracy, with the
required resolution stated.}

\item{eps}{The polygonal curve simplification tolerance used for simplifying
the resulting boundary curve.  See \code{\link[=inla.simplify.curve]{inla.simplify.curve()}} for
details.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}
}
\value{
An \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object.
}
\description{
Constructs a nonconvex boundary for a point set using morphological
operations.
}
\details{
Morphological dilation by \code{convex}, followed by closing by
\code{concave}, with minimum concave curvature radius \code{concave}.  If
the dilated set has no gaps of width between \deqn{2 convex (\sqrt{1+2
concave/convex} - 1)}{2*convex*(sqrt(1+2*concave/convex) - 1)} and \eqn{2
concave}{2*concave}, then the minimum convex curvature radius is
\code{convex}.  Special case \code{concave=0} delegates to
\code{inla.nonconvex.hull.basic}

The implementation is based on the identity \deqn{dilation(a) \& closing(b)
= dilation(a+b) \& erosion(b)}{dilation(a) & closing(b) = dilation(a+b) &
erosion(b)} where all operations are with respect to disks with the
specified radii.
}
\note{
Requires \code{nndistF} from the \code{splancs} package.
}
\examples{

if (require(splancs)) {
    loc <- matrix(runif(20), 10, 2)
    boundary <- inla.nonconvex.hull(loc, convex = 0.2)
    lines(boundary, add = FALSE)
    points(loc)
}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from dev.new.R
%%
 \name{inla.dev.new}
 \alias{inla.dev.new}

 \title{Opens a new device}

 \description{Open a new device using \code{\link{dev.new}} unless using RStudio}
 \usage{
     inla.dev.new(...)
 }
 \arguments{
   \item{...}{Optional arguments to \code{\link{dev.new}}}
}
 \value{
   The value of \code{\link{dev.new}} if not running RStudio, otherwise \code{NULL}
 }
 \author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.predictor}
\alias{control.predictor}
\alias{inla.control.predictor}
\alias{inla.set.control.predictor.default}
\alias{set.control.predictor.default}
\alias{control.predictor.default}
\title{Control variables in control.predictor}
\description{Control variables in \code{control.predictor} for use in \code{inla}}
\usage{
inla.set.control.predictor.default(...)
control.predictor(A, cdf, compute, cross, fixed, hyper, initial, link, param, precision, prior, quantiles, vb.correct)
}
\arguments{
\item{...}{Possible arguments}
\item{hyper}{   Definition of the hyperparameters.}
\item{fixed}{   (OBSOLETE!) If the precision for the artificial noise is fixed or not (defualt TRUE)}
\item{prior}{   (OBSOLETE!) The prior for the artificial noise}
\item{param}{   (OBSOLETE!) Prior parameters for the artificial noise}
\item{initial}{   (OBSOLETE!) The value of the log precision of the artificial noise}
\item{compute}{   A boolean variable; should the marginals for the linear predictor be computed? (Default FALSE.)}
\item{cdf}{   A list of values to compute the CDF for the linear predictor}
\item{quantiles}{   A list of quantiles to compute for the linear predictor}
\item{cross}{   Cross-sum-to-zero constraints with the linear predictor. All linear predictors with the same level of 'cross' are constrained to have sum zero. Use 'NA' for no contribution. 'Cross' has the same length as the linear predictor (including the 'A' matrix extention). (THIS IS AN EXPERIMENTAL OPTION, CHANGES MAY APPEAR.)}
\item{A}{   The observation matrix (matrix or Matrix::sparseMatrix).}
\item{precision}{   The precision for eta* - A*eta, (default \code{exp(15)})}
\item{link}{   Define the family-connection for unobserved observations (\code{NA}). \code{link} is integer values which defines the family connection; \code{family[link[idx]]} unless \code{is.na(link[idx])} for which the identity-link is used. The \code{link}-argument only influence the \code{fitted.values} in the \code{result}-object. If \code{is.null(link)} (default) then the identity-link is used for all missing observations. If the length of \code{link} is 1, then this value is replicated with the length of the responce vector. If an element of the responce vector is \code{!NA} then the corresponding entry in \code{link} is not used (but must still be a legal value). Setting this variable implies \code{compute=TRUE}.}
\item{vb.correct}{   Logical Add this model component to the list of variables to be used for the vb corrected Laplace approximation? If \code{NULL} use default choice, otherwise correct if \code{TRUE} and do not if \code{FALSE} (expert option)}
}
\value{
  
The function \code{control.predictor} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.predictor.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.stack.remove.unused}
\alias{inla.stack.remove.unused}
\alias{inla.stack.compress}
\alias{inla.stack}
\alias{inla.stack.sum}
\alias{inla.stack.join}
\alias{inla.stack.index}
\alias{inla.stack.LHS}
\alias{inla.stack.RHS}
\alias{inla.stack.data}
\alias{inla.stack.A}
\title{Data stacking for advanced INLA models}
\usage{
inla.stack.remove.unused(stack)

inla.stack.compress(stack, remove.unused = TRUE)

inla.stack(..., compress = TRUE, remove.unused = TRUE)

inla.stack.sum(
  data,
  A,
  effects,
  tag = "",
  compress = TRUE,
  remove.unused = TRUE
)

inla.stack.join(..., compress = TRUE, remove.unused = TRUE)

inla.stack.index(stack, tag)

inla.stack.LHS(stack)

inla.stack.RHS(stack)

inla.stack.data(stack, ...)

inla.stack.A(stack)
}
\arguments{
\item{stack}{A \code{inla.data.stack} object, created by a call to
\code{inla.stack}, \code{inla.stack.sum}, or \code{inla.stack.join}.}

\item{remove.unused}{If \code{TRUE}, compress the model by removing rows of
effects corresponding to all-zero columns in the \code{A} matrix (and
removing those columns).}

\item{...}{For \code{inla.stack.join}, two or more data stacks of class
\code{inla.data.stack}, created by a call to \code{inla.stack},
\code{inla.stack.sum}, or \code{inla.stack.join}. For
\code{inla.stack.data}, a list of variables to be joined with the data list.}

\item{compress}{If \code{TRUE}, compress the model by removing duplicated
rows of effects, replacing the corresponding A-matrix columns with a single
column containing the sum.}

\item{data}{A list or codedata.frame of named data vectors. Scalars are
expanded to match the number of rows in the A matrices, or any non-scalar
data vectors. An error is given if the input is inconsistent.}

\item{A}{A list of observation matrices. Scalars are expanded to diagonal
matrices matching the effect vector lengths. An error is given if the input
is inconsistent or ambiguous.}

\item{effects}{A collection of effects/predictors.  Each list element
corresponds to an observation matrix, and must either be a single vector, a
list of vectors, or a \code{data.frame}. Single-element effect vectors are
expanded to vectors matching the number of columns in the corresponding A
matrix.  An error is given if the input is inconsistent or ombiguous.}

\item{tag}{A string specifying a tag for later identification.}
}
\value{
A data stack of class \code{inla.data.stack}.
Elements: \itemize{
\item\code{data} \eqn{=(y^1, \ldots, y^p, \tilde{x}^1, \ldots,
\tilde{x}^{n_k})}{=(y^1, \dots, y^p, tilde{x}^1, \dots,
tilde{x}^{n_k})}
\item\code{A} \eqn{=\tilde{A}}{=tilde{A}}
\item\code{data.names} List
of data names, length \eqn{p}
\item\code{effect.names} List of effect names,
length \eqn{n_k}
\item\code{n.data} Data length, \eqn{n_i}
\item\code{index}
List indexed by \code{tag}s, each element indexing into \eqn{i=1, \ldots,
n_i} }
}
\description{
Functions for combining data, effects and observation matrices into
\code{inla.stack} objects, and extracting information from such objects.
}
\details{
For models with a single effects collection, the outer list container for
\code{A} and \code{effects} may be omitted.

Component size definitions:
\itemize{
\item \eqn{n_l}{n_l} effect blocks
\item \eqn{n_k}{n_k} effects
\item \eqn{n_i}{n_i} data values
\item \eqn{n_{j,l}}{n_jl} effect size for block \eqn{l}{l}
\item \eqn{n_j}{n_j} \eqn{= \sum_{l=1}^{n_l} n_{j,l}}{sum_l n_jl} total
effect size
}

Input: \describe{
\item{\code{data}}{\eqn{(y^1, \ldots, y^p)}{(y1,\dots,y2)} \eqn{p}{p}
vectors, each of length \eqn{n_i}{n_i}}
\item{\code{A}}{\eqn{(A^1, \ldots, A^{n_l})}{(A1,\dots,A2)} matrices of size
\eqn{n_i \times n_{j,l}}{n_i by n_jl}}
\item{\code{effects}}{\eqn{\left((x^{1,1},\ldots,x^{n_k,1}), \ldots,
(x^{1,n_l},\ldots,x^{n_k,n_l})\right)}{((x_[1,1],\dots,x_[n_k,1]),\dots(x_[1,n_l],\dots,x_[n_k,n_l]))}
collections of effect
vectors of length \eqn{n_{j,l}}{n_jl} }
}

\deqn{\mbox{predictor}(y^1, \ldots, y^p) \sim
\sum_{l=1}^{n_l} A^l \sum_{k=1}^{n_k} g(k, x^{k,l})
= \tilde{A} \sum_{k=1}^{n_k} g(k, \tilde{x}^k)
}{ predictor(y^1, \ldots, y^p) ~ sum_{l=1}^{n_l} A^l sum_{k=1}^{n_k}
g(k, x^{k,l}) = tilde{A} sum_{k=1}^{n_k} g(k, tilde{x}^k) }
where
\deqn{\tilde{A} = \mbox{cbind}\left( A^1, \ldots, A^{n_l} \right)
}{ tilde{A} = cbind( A^1, ..., A^{n_l} ) }
and
\deqn{\tilde{x}^k = \mbox{rbind}\left( x^{k,1}, \ldots, x^{k,n_l} \right)
}{ tilde{x}^k = rbind( x^{k,1}, ..., x^{k,n_l} ) }
and for each block \eqn{l}{l}, any missing
\eqn{x^{k,l}} is replaced by an \code{NA} vector.
}
\section{Functions}{
\itemize{
\item \code{inla.stack.remove.unused}: Remove unused entries from an existing stack

\item \code{inla.stack.compress}: Compress an existing stack by removing duplicates

\item \code{inla.stack.sum}: Create data stack as a sum of predictors

\item \code{inla.stack.join}: Join two or more data stacks

\item \code{inla.stack.index}: Extract tagged indices

\item \code{inla.stack.LHS}: Extract data associated with the "left hand side" of the model
(e.g. the data itself, \code{Ntrials}, \code{link}, \code{E})

\item \code{inla.stack.RHS}: Extract data associated with the "right hand side" of the model
(all the covariates/predictors)

\item \code{inla.stack.data}: Extract data for an inla call, and optionally join with other variables

\item \code{inla.stack.A}: Extract the "A matrix" for control.predictor
}}

\section{Functions}{
 \itemize{
\item \code{inla.stack.remove.unused}: Remove
unused entries from an existing stack

\item \code{inla.stack.compress}: Compress an existing stack by removing
duplicates

\item \code{inla.stack}: Shorthand for inla.stack.join and inla.stack.sum

\item \code{inla.stack.sum}: Create data stack as a sum of predictors

\item \code{inla.stack.join}: Join two or more data stacks

\item \code{inla.stack.index}: Extract tagged indices

\item \code{inla.stack.LHS}: Extract data associated with the "left hand
side" of the model (e.g. the data itself, \code{Ntrials}, \code{link},
\code{E})

\item \code{inla.stack.RHS}: Extract data associated with the "right hand
side" of the model (all the covariates/predictors)

\item \code{inla.stack.data}: Extract data for an inla call, and optionally
join with other variables

\item \code{inla.stack.A}: Extract the "A matrix" for control.predictor }
}

\examples{

n <- 200
loc <- matrix(runif(n * 2), n, 2)
mesh <- inla.mesh.2d(
    loc.domain = loc,
    max.edge = c(0.05, 0.2)
)
proj.obs <- inla.mesh.projector(mesh, loc = loc)
proj.pred <- inla.mesh.projector(mesh, loc = mesh$loc)
spde <- inla.spde2.pcmatern(mesh,
    prior.range = c(0.01, 0.01),
    prior.sigma = c(10, 0.01)
)

covar <- rnorm(n)
field <- inla.qsample(n = 1, Q = inla.spde.precision(spde, theta = log(c(0.5, 1))))[, 1]
y <- 2 * covar + inla.mesh.project(proj.obs, field)

A.obs <- inla.spde.make.A(mesh, loc = loc)
A.pred <- inla.spde.make.A(mesh, loc = proj.pred$loc)
stack.obs <-
    inla.stack(
        data = list(y = y),
        A = list(A.obs, 1),
        effects = list(c(
            list(Intercept = 1),
            inla.spde.make.index("spatial", spde$n.spde)
        ),
        covar = covar
        ),
        tag = "obs"
    )
stack.pred <-
    inla.stack(
        data = list(y = NA),
        A = list(A.pred),
        effects = list(c(
            list(Intercept = 1),
            inla.spde.make.index("spatial", mesh$n)
        )),
        tag = "pred"
    )
stack <- inla.stack(stack.obs, stack.pred)

formula <- y ~ -1 + Intercept + covar + f(spatial, model = spde)
result1 <- inla(formula,
    data = inla.stack.data(stack.obs, spde = spde),
    family = "gaussian",
    control.predictor = list(
        A = inla.stack.A(stack.obs),
        compute = TRUE
    )
)

plot(y, result1$summary.fitted.values[inla.stack.index(stack.obs, "obs")$data, "mean"],
    main = "Observations vs posterior predicted values at the data locations"
)

result2 <- inla(formula,
    data = inla.stack.data(stack, spde = spde),
    family = "gaussian",
    control.predictor = list(
        A = inla.stack.A(stack),
        compute = TRUE
    )
)

field.pred <- inla.mesh.project(
    proj.pred,
    result2$summary.fitted.values[inla.stack.index(stack, "pred")$data, "mean"]
)
field.pred.sd <- inla.mesh.project(
    proj.pred,
    result2$summary.fitted.values[inla.stack.index(stack, "pred")$data, "sd"]
)

plot(field, field.pred, main = "True vs predicted field")
abline(0, 1)
image(inla.mesh.project(mesh,
    field = field,
    dims = c(200, 200)
),
main = "True field"
)
image(inla.mesh.project(mesh,
    field = field.pred,
    dims = c(200, 200)
),
main = "Posterior field mean"
)
image(inla.mesh.project(mesh,
    field = field.pred.sd,
    dims = c(200, 200)
),
main = "Prediction standard deviation"
)
plot(field, (field.pred - field) / 1,
    main = "True field vs standardised prediction residuals"
)
}
\seealso{
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}, \code{\link[=inla.spde.make.index]{inla.spde.make.index()}}
}
\keyword{fmesher}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from knmodels.sample.R
%%
 \name{inla.knmodels.sample}
 \alias{inla.knmodels.sample}
 \title{Spacetime interaction models sampler function}
 \description{
    It implements the sampling method for the models
    in Knorr-Held, L. (2000) considering the algorithm
    3.1 in Rue & Held (2005) book.
 }
 \usage{
inla.knmodels.sample(
  graph,
  m,
  type=4,
  intercept=0,
  tau.t=1,
  phi.t=0.7,
  tau.s=1,
  phi.s=0.7,
  tau.st=1,
  ev.t=NULL,
  ev.s=NULL)
}
 \arguments{
 \item{graph}{}
 \item{m}{Time dimention.}
 \item{type}{Integer from 1 to 4 to identify one
    of the four interaction type.}
 \item{intercept}{A constant to be added to the linear predictor}
 \item{tau.t}{Precision parameter for the main temporal effect.}
 \item{phi.t}{Mixing parameter in the \code{bym2} model
    assumed for the main temporal effect.}
 \item{tau.s}{Precision parameter for the main spatial effect.}
 \item{phi.s}{Mixing parameter in the \code{bym2} model
    assumed for the main spatial effect.}
 \item{tau.st}{Precision parameter for the spacetime effect.}
 \item{ev.t}{Eigenvalues and eigenvectors of the temporal
    precision matrix structure.}
 \item{ev.s}{Eigenvalues and eigenvectors of the spatial
    precision matrix structure.}
 }
 \value{
  A list with the following elements
   \item{time}{The time index for each obervation,
       with length equals m*n.}
   \item{space}{The spatial index for each obervation,
       with length equals m*n.}
   \item{spacetime}{The spacetime index for each obervation,
       with length equals m*n.}
   \item{x}{A list with the following elements}
   \item{t.iid}{The unstructured main temporal effect part.}
   \item{t.str}{The structured main temporal effect part.}
   \item{t}{The main temporal effect with length equals 2m.}
   \item{s.iid}{The unstructured main spatial effect part.}
   \item{s.str}{The structured main spatial effect part.}
   \item{s}{The main spatial effect with length equals 2n.}
   \item{st}{The spacetime interaction effect with length equals m*n.}
   \item{eta}{The linear predictor with length equals n*m.}
 }
 \author{Elias T. Krainski}
 \seealso{
     \code{\link{inla.knmodels}} for model fitting
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{nwEngland}
\alias{nwEngland}
\alias{NewEngland}
\title{The New England map}
\format{
A SpatialPolygons object.
}
\source{
This map are used to analyse the Leukaemia dataset from

Henderson, R. and Shimakura, S. and Gorst, D., 2002, Modeling spatial
variation in leukemia survival data, JASA, 97, 460, 965--972.
}
\description{
This map is used in association to the Leukemia data from Henderson et al
(2003); see source.
}
\examples{

data(Leuk)
plot(nwEngland)
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from qsolve.R
%%
 \name{qsolve}
 \alias{inla.qsolve}
 \alias{qsolve}

 \title{Solves linear SPD systems}

 \description{This routine use the GMRFLib implementation
              to solve linear systems with a SPD matrix.}
 \usage{
     inla.qsolve(Q, B, reordering = inla.reorderings(), method = c("solve", "forward", "backward"))
 }

 \arguments{
   \item{Q}{A SPD matrix,  either as a (dense) matrix or sparse-matrix}.
   \item{B}{The right hand side matrix, either as a (dense) matrix or  sparse-matrix.}
   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
        either one of the names listed in \code{inla.reorderings()}
        or the output from \code{inla.qreordering(Q)}.
        The default is "auto" which try several reordering
        algorithm and use the best one for this particular matrix (using the TAUCS library).}
   \item{method}{The system to solve, one of \code{"solve"},
                 \code{"forward"} or \code{"backward"}. Let \code{Q = L L^T},
                 where \code{L} is lower triangular
                 (the Cholesky triangle),  then \code{method="solve"} solves \code{L L^T X = B} or
                  equivalently \code{Q X = B},  \code{method="forward"} solves \code{L X = B},  and
                 \code{method="backward"} solves \code{L^T X  = B}. }
}
\value{
  \code{inla.qsolve} returns a matrix \code{X},
  which is the solution of \code{Q X = B},  \code{L X = B} or \code{L^T X = B}
  depending on the value of \code{method}.
}
\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
 n = 10
 nb <- n-1
 QQ = matrix(rnorm(n^2), n, n)
 QQ <- QQ \%*\% t(QQ)

 Q = inla.as.sparse(QQ)
 B = matrix(rnorm(n*nb), n, nb)

 X = inla.qsolve(Q, B, method = "solve")
 XX = inla.qsolve(Q, B, method = "solve", reordering = inla.qreordering(Q))
 print(paste("err solve1", sum(abs( Q \%*\% X - B))))
 print(paste("err solve2", sum(abs( Q \%*\% XX - B))))

 ## the forward and backward solve is tricky, as after permutation and with Q=LL', then L is
 ## lower triangular, but L in the orginal ordering is not lower triangular. if the rhs is iid
 ## noise, this is not important. to control the reordering, then the 'taucs' library must be
 ## used.
 inla.setOption(smtp = 'taucs')

 ## case 1. use the matrix as is, no reordering
 r <- "identity"
 L = t(chol(Q))
 X = inla.qsolve(Q, B, method = "forward", reordering = r)
 XX = inla.qsolve(Q, B, method = "backward", reordering = r)
 print(paste("err forward ", sum(abs(L \%*\% X - B))))
 print(paste("err backward", sum(abs(t(L) \%*\% XX - B))))

 ## case 2. use a reordering from the library
 r <- inla.qreordering(Q)
 im <- r$ireordering
 m <- r$reordering
 print(cbind(idx = 1:n, m, im) )
 Qr <- Q[im, im]
 L = t(chol(Qr))[m, m]

 X = inla.qsolve(Q, B, method = "forward", reordering = r)
 XX = inla.qsolve(Q, B, method = "backward", reordering = r)
 print(paste("err forward ", sum(abs( L \%*\% X - B))))
 print(paste("err backward", sum(abs( t(L) \%*\% XX - B))))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{package}
\name{INLA-package}
\alias{INLA-package}
\alias{INLA}
\title{Integrated Nested Laplace Approximation}
\description{
Package to perform full Bayesian analysis on generalised additive mixed
models using Integrated Nested Laplace Approximations.
}
\details{
\describe{
\item{Package}{INLA}
\item{Version}{Currently, this package uses a YY.MM.DD versioning
system, and is in heavy development.
See \url{https://github.com/hrue/r-inla/} and
\url{https://www.r-inla.org}}
\item{License}{GPL2}
}

See the web-site \url{https://www.r-inla.org} for further details.
}
\author{
Havard Rue, Sara Martino, Finn Lindgren, Daniel Simpson and Andrea
Riebler
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{plot.inla.mesh}
\alias{plot.inla.mesh}
\title{Draw a triangulation mesh object}
\usage{
\method{plot}{inla.mesh}(
  x,
  col = "white",
  t.sub = 1:nrow(mesh$graph$tv),
  add = FALSE,
  lwd = 1,
  xlim = range(mesh$loc[, 1]),
  ylim = range(mesh$loc[, 2]),
  main = NULL,
  rgl = FALSE,
  size = 2,
  draw.vertices = FALSE,
  vertex.color = "black",
  draw.edges = TRUE,
  edge.color = rgb(0.3, 0.3, 0.3),
  draw.segments = draw.edges,
  ...
)
}
\arguments{
\item{x}{An \code{\link[=inla.mesh]{inla.mesh()}} object.}

\item{col}{Color specification.  A single named color, a vector of scalar
values, or a matrix of RGB values.  Requires \code{rgl=TRUE}.}

\item{t.sub}{Optional triangle index subset to be drawn.}

\item{add}{If \code{TRUE}, adds to the current plot instead of starting a
new one.}

\item{lwd}{Line width for triangle edges.}

\item{xlim}{X-axis limits.}

\item{ylim}{Y-axis limits.}

\item{main}{The main plot title.  If not specified, a default title is
generated based on the mesh type.}

\item{rgl}{When \code{TRUE}, generates an \code{rgl} plot instead of a
generic graphics plot.  Allows 3D plotting and color surface plotting.}

\item{size}{Size of vertex points in \code{rgl} plotting. See
\code{rgl.material}.}

\item{draw.vertices}{If \code{TRUE}, draw triengle vertices.}

\item{vertex.color}{Color specification for all vertices.}

\item{draw.edges}{If \code{TRUE}, draw triangle edges.}

\item{edge.color}{Color specification for all edges.}

\item{draw.segments}{If \code{TRUE}, draw boundary and interior constraint
edges more prominently.}

\item{\dots}{Further graphics parameters, interpreted by the respective
plotting systems.}
}
\description{
Plots an \code{\link[=inla.mesh]{inla.mesh()}} object using either standard graphics or
with \code{rgl}.
}
\examples{

mesh <- inla.mesh.create(globe = 10)
plot(mesh)
\donttest{
if (require(rgl)) {
    plot(mesh, rgl = TRUE, col = mesh$loc[, 1])
}
}

}
\seealso{
\code{\link[=plot.inla.trimesh]{plot.inla.trimesh()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Seeds}
\alias{Seeds}
\title{Factorial design}
\format{
A data frame with 21 observations on the following 5 variables.
\describe{ \item{r}{number of germinated seeds per plate}
\item{n}{number of total seeds per plate} \item{x1}{seed
type} \item{x2}{root extracted} \item{plate}{indicator for
the plate} }
}
\source{
WinBUGS/OpenBUGS Manual Example, Vol. I
}
\description{
Proportion of seeds that germinated on each of 21 plates arranged according
to a 2 by 2 factorial layout by seed and type of root extract
}
\examples{

data(Seeds)
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from idx.R
%%
\name{idx}
\alias{idx}
\alias{inla.idx}

\title{Convert indexes}

\description{Convert indexes given by to triplet `(idx, group,
  replicate)' to the (one-dimensional) index used in the grouped and
  replicated model}

\usage{
inla.idx(idx, n = max(idx),
         group = rep(1, length(idx)), ngroup = max(group),
         replicate = rep(1, length(idx)),  nrep = max(replicate))
}

\arguments{

  \item{idx}{The index within the basic model. (Legal values from `1' to `n'.)}
  \item{n}{The length `n' of the basic model.}
  \item{group}{The index within group. (Legal values from `1' to `ngroup'.)}
  \item{ngroup}{Number of groups.}
  \item{replicate}{The index within replication. (Legal values from
  `1' to `nrep'.)}
  \item{nrep}{Number of replications.}
}

\value{%%
  \code{inla.idx} returns indexes in the range `1' to `n*ngroup*nrep'
  representing where the triplet `(idx,group,replicate)' is stored
  internally in the full grouped and replicated model.
}
%%

\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
##TODO
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from inlaEnv.R
%%
\name{inla.get.inlaEnv}
\alias{inla.get.inlaEnv}
\alias{get.inlaEnv}
\title{Return the internal environment used by INLA}
\description{A function which return the internal environment used by INLA}
\usage{
inla.get.inlaEnv()
}
\arguments{
}

\value{%%
  This function returns the internal environment used by INLA to
  keep internal variables.
}
\author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from cut.R
%%
 \name{cut}
 \alias{inla.cut}
 \alias{cut}

 \title{Group-wise model criticism using node-splitting}

 \description{This function performs group-wise, cross-validatory
 model assessment for an INLA model using so-called node-splitting
 (Marshall and Spiegelhalter, 2007; Presanis et al, 2013).
 The user inputs an object of class \code{inla} (i.e. a result
 of a call to \code{inla()}) as well as  a variable name (\code{split.by}) specifying a grouping:
 Data points that share the same value of \code{split.by} are in the same group.
 The function then checks whether each group is an "outlier", or in conflict with
 the remaining groups, using the methodology described in Ferkingstad et al (2017).
 The result is a vector containing a p-value for each group, corresponding to a test
 for each group \code{i}, where the null hypothesis is that group \code{i} is
 consistent with the other groups except \code{i} (so a small p-value is evidence
 that the group is an "outlier"). See Ferkingstad et al (2017) for further details.
 }

 \usage{
     inla.cut(result, split.by, mc.cores = NULL, debug=FALSE)
 }

 \arguments{
   \item{result}{An object of class \code{inla}, i.e. a result of a call to \code{inla()}}
   \item{split.by}{The name of the variable to group by. Data points that have
                   the same value of \code{split.by} are in the same group.}
   \item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
                   \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
                   and \code{inla.getOption("num.threads")} in that order.}
   \item{debug}{Print debugging information if \code{TRUE}, default is \code{FALSE}}
  }

 \value{
  A numeric vector of p-values, corresponding to a test
 for each group \code{i} where the null hypothesis is that group \code{i} is
 consistent with the other groups except \code{i}. A small p-value for a group
 indicates that the group is an "outlier" (in conflict with remaining groups).

  This function is EXPERIMENTAL!!!
 }

 \author{Egil Ferkingstad \email{egil.ferkingstad@gmail.com} and Havard Rue \email{hrue@r-inla.org}}

 \examples{
 ## See http://www.r-inla.org/examples/case-studies/ferkingstad-2017 and Ferkingstad et al (2017).
 }

 \references{
 Ferkingstad, E., Held, L. and Rue, H. (2017). Fast and accurate Bayesian
 model criticism and conflict diagnostics using R-INLA. arXiv preprint
 arXiv:1708.03272, available at http://arxiv.org/abs/1708.03272.
 Published in Stat, 6:331-344 (2017).

 Marshall, E. C. and Spiegelhalter, D. J. (2007). Identifying outliers
 in Bayesian hierarchical models: a simulation-based approach.
 Bayesian Analysis, 2(2):409-444.

 Presanis, A. M., Ohlssen, D., Spiegelhalter,
 D. J., De Angelis, D., et al. (2013). Conflict diagnostics
 in directed acyclic graphs, with applications in Bayesian evidence synthesis.
 Statistical Science, 28(3):376-397.
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from remote.R
%%
\name{inla.ssh.copy.id}
\alias{inla.ssh.copy.id}
\alias{ssh.copy.id}
\alias{inla.remote}

\title{Setup remote computing}

\description{Initialize the definition file and print the path to the internal script to transfer ssh-keys}

\usage{
inla.remote()
inla.ssh.copy.id()
}
\arguments{
 None
}

\value{%%
 \code{inla.remote} is used once to setup the remote host information
 file (definition file) in the users home directory; see the FAQ entry on this
 issue for more information.
 \code{inla.ssh.copy.id} will return the path to the internal script to transfer ssh-keys.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\examples{
##See the FAQ entry on this issue on r-inla.org.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.project}
\alias{inla.mesh.project}
\alias{inla.mesh.projector}
\alias{inla.mesh.projector.inla.mesh}
\alias{inla.mesh.project.inla.mesh}
\alias{inla.mesh.project.inla.mesh.projector}
\alias{inla.mesh.project.inla.mesh.1d}
\alias{inla.mesh.projector.inla.mesh.1d}
\title{Methods for projecting to/from an inla.mesh}
\usage{
inla.mesh.project(...)

\method{inla.mesh.project}{inla.mesh}(mesh, loc = NULL, field = NULL, crs = NULL, ...)

\method{inla.mesh.project}{inla.mesh.1d}(mesh, loc, field = NULL, ...)

\method{inla.mesh.project}{inla.mesh.projector}(projector, field, ...)

inla.mesh.projector(...)

\method{inla.mesh.projector}{inla.mesh}(
  mesh,
  loc = NULL,
  lattice = NULL,
  xlim = NULL,
  ylim = NULL,
  dims = c(100, 100),
  projection = NULL,
  crs = NULL,
  ...
)

\method{inla.mesh.projector}{inla.mesh.1d}(mesh, loc = NULL, xlim = mesh$interval, dims = 100, ...)
}
\arguments{
\item{\dots}{Additional arguments passed on to methods.}

\item{mesh}{An \code{\link[=inla.mesh]{inla.mesh()}} or \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object.}

\item{loc}{Projection locations.  Can be a matrix or a \code{SpatialPoints}
or a \code{SpatialPointsDataFrame} object.}

\item{field}{Basis function weights, one per mesh basis function, describing
the function to be avaluated at the projection locationssFunction values for
on the mesh}

\item{crs}{An optional CRS or inla.CRS object associated with \code{loc}
and/or \code{lattice}.}

\item{projector}{An \code{inla.mesh.projector} object.}

\item{lattice}{An \code{\link[=inla.mesh.lattice]{inla.mesh.lattice()}} object.}

\item{xlim}{X-axis limits for a lattice. For R2 meshes, defaults to covering
the domain.}

\item{ylim}{Y-axis limits for a lattice. For R2 meshes, defaults to covering
the domain.}

\item{dims}{Lattice dimensions.}

\item{projection}{One of \code{c("default", "longlat", "longsinlat", "mollweide")}.}
}
\value{
For \code{inla.mesh.project(mesh, ...)}, a list with projection
information.  For \code{inla.mesh.projector(mesh, ...)}, an
\code{inla.mesh.projector} object.  For \code{inla.mesh.project(projector, field, ...)}, a field projected from the mesh onto the locations given by
the projector object.
}
\description{
Calculate a lattice projection to/from an \code{\link[=inla.mesh]{inla.mesh()}}
}
\details{
The call \code{inla.mesh.project(mesh, loc, field=..., ...)}, is a shortcut
to inla.mesh.project(inla.mesh.projector(mesh, loc), field).
}
\examples{

n <- 20
loc <- matrix(runif(n * 2), n, 2)
mesh <- inla.mesh.create(loc, refine = list(max.edge = 0.05))
proj <- inla.mesh.projector(mesh)
field <- cos(mesh$loc[, 1] * 2 * pi * 3) * sin(mesh$loc[, 2] * 2 * pi * 7)
image(proj$x, proj$y, inla.mesh.project(proj, field))
\donttest{
if (require(rgl)) {
    plot(mesh, rgl = TRUE, col = field, draw.edges = FALSE, draw.vertices = FALSE)
}
}

}
\seealso{
\code{\link[=inla.mesh]{inla.mesh()}}, \code{\link[=inla.mesh.1d]{inla.mesh.1d()}},
\code{\link[=inla.mesh.lattice]{inla.mesh.lattice()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-gammacount.R
%%
 \name{pc.gammacount}
 \alias{inla.pc.gammacount}
 \alias{pc.gammacount}
 \alias{pc.rgammacount}
 \alias{inla.pc.rgammacount}
 \alias{pc.dgammacount}
 \alias{inla.pc.dgammacount}
 \alias{pc.pgammacount}
 \alias{inla.pc.pgammacount}
 \alias{pc.qgammacount}
 \alias{inla.pc.qgammacount}

 \title{Utility functions for the PC prior for the \code{gammacount} likelihood}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the \code{gammacount} likelihood}
 \usage{
 inla.pc.rgammacount(n, lambda = 1)
 inla.pc.dgammacount(x, lambda = 1, log = FALSE)
 inla.pc.qgammacount(p, lambda = 1)
 inla.pc.pgammacount(q, lambda = 1)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter (see Details)}
   \item{x}{Evaluation points}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
 This gives the PC prior for the \code{gammacount} likelihood,  which is the PC prior for
 \code{a} in \code{Gamma(a, 1)} where \code{Gamma(1, 1)} is the base model.
 }
\value{%%
  \code{inla.pc.dgammacount} gives the density,
  \code{inla.pc.pgammacount} gives the distribution function,
  \code{inla.pc.qgammacount} gives the quantile function, and
  \code{inla.pc.rgammacount} generates random deviates.
 }
 \seealso{inla.doc("pc.gammacount")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 x = inla.pc.rgammacount(100,  lambda = 1)
 d = inla.pc.dgammacount(x, lambda = 1)
 x = inla.pc.qgammacount(0.5, lambda = 1)
 inla.pc.pgammacount(x, lambda = 1)
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from group.R
%%
\name{inla.group}
\alias{inla.group}

\title{Group or cluster covariates}

\description{\code{inla.group} group or cluster covariates so to reduce
  the number of unique values}
\usage{
inla.group(x, n = 25, method = c("cut", "quantile"), idx.only = FALSE)
}
\arguments{

  \item{x}{The vector of covariates to group.}
  \item{n}{Number of classes or bins to group into.}
  \item{method}{Group either using bins with equal length intervals
    (\code{method = "cut"}), or equal distance in the `probability'
    scale using the quantiles (\code{method = "quantile"}).}
  \item{idx.only}{Option to return the index only and not the
    \code{method}.}
}


\value{
  \code{inla.group} return the new grouped covariates where the classes
  are set to the median of all the covariates belonging to that group.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{\code{\link{f}}}
\examples{
## this gives groups 3 and 8
x = 1:10
x.group = inla.group(x, n = 2)

## this is the intended use, to reduce the number of unique values in
## the of first argument of f()
n = 100
x = rnorm(n)
y = x + rnorm(n)
result = inla(y ~ f(inla.group(x, n = 20), model = "iid"), data=data.frame(y=y,x=x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Leuk}
\alias{Leuk}
\alias{Leukemia}
\title{The Leukemia data}
\format{
A data frame with 1043 observations on the following 9 variables.
\describe{
\item{time}{TODO}
\item{cens}{TODO}
\item{xcoord}{TODO}
\item{ycoord}{TODO}
\item{age}{TODO}
\item{sex}{TODO}
\item{wbc}{TODO}
\item{tpi}{TODO}
\item{district}{TODO}
}
}
\source{
This is the dataset from

Henderson, R. and Shimakura, S. and Gorst, D., 2002, Modeling spatial
variation in leukemia survival data, JASA, 97, 460, 965--972.
}
\description{
This the Leukemia data from Henderson et al (2003); see source.
}
\examples{

data(Leuk)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Kidney}
\alias{Kidney}
\title{Kidney infection data}
\format{
A data frame with 76 observations on the following 9 variables.
\describe{
\item{time}{a numeric vector. Time to infection from the
insertion of catheter}
\item{event}{a numeric vector. 1: time of
infection 0: time of censuring }
\item{age}{a numeric vector. Age of
the patient at the time of infection}
\item{sex}{a numeric vector.
Sex of the patient 0: male 1:female}
\item{disease}{a numeric
vector. Type of disease}
\item{dis1}{a numeric vector. Dummy
variable to codify the disease type.}
\item{dis2}{a numeric vector.
Dummy variable to codify the disease type.}
\item{dis3}{a numeric
vector. Dummy variable to codify the disease type.}
\item{ID}{a
numeric vector. Patient code.}
}
}
\description{
Times of infection from the time to insertion of the catheter for 38 kindey
patients using portable dialysis equipment
}
\references{
McGilchrist and C.W. Aisbett (1991), Regression with frailty in
survival analysis, Biometrics,vol.47,pages 461--166.

D.J. Spiegelhalter and A. Thomas and N.G. Best and W.R. Gilks (1995) BUGS:
Bayesian Inference Using Gibbs sampling, Version 0.50., MRC Biostatistics
Unit, Cambridre, England.
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from f.R
%%
\name{f}
\alias{f}
\title{Define general Gaussian models in the INLA formula }
\description{

  Function used for defining of smooth and spatial terms within \code{inla} model
  formulae. The function does not evaluate anything - it
  exists purely to help set up a model. The function specifies one
  smooth function in the linear predictor (see \code{\link{inla.list.models}}) as
  \deqn{w\ f(x)}{weight*f(var)}

}
\usage{
    f(...,
         model = "iid",
         copy=NULL,
         same.as = NULL,
         n=NULL,
         nrep = NULL,
         replicate = NULL,
         ngroup = NULL,
         group = NULL,
         control.group = inla.set.control.group.default(),
         hyper = NULL,
         initial=NULL,
         prior=NULL,
         param = NULL,
         fixed = NULL,
         season.length=NULL,
         constr = NULL,
         extraconstr=list(A=NULL, e=NULL),
         values=NULL,
         cyclic = NULL,
         diagonal = NULL,
         graph=NULL,
         graph.file=NULL,
         cdf=NULL,
         quantiles=NULL,
         Cmatrix=NULL,
         rankdef=NULL,
         Z = NULL,
         nrow = NULL,
         ncol = NULL,
         nu = NULL,
         bvalue = NULL,
         spde.prefix = NULL,
         spde2.prefix = NULL,
         spde2.transform = c("logit", "log", "identity"),
         spde3.prefix = NULL,
         spde3.transform = c("logit", "log", "identity"),
         mean.linear = inla.set.control.fixed.default()$mean,
         prec.linear = inla.set.control.fixed.default()$prec,
         compute = TRUE,
         of=NULL,
         precision = exp(14),
         range = NULL,
         adjust.for.con.comp = TRUE,
         order = NULL,
         scale = NULL,
         strata = NULL,
         rgeneric = NULL,
         scale.model = NULL,
         args.slm = list(rho.min = NULL, rho.max = NULL,
                         X = NULL, W = NULL, Q.beta = NULL),
         args.ar1c = list(Z = NULL, Q.beta = NULL),
         args.intslope = list(subject = NULL, strata = NULL, covariates = NULL),
         vb.correct = NULL,
         locations = NULL,
         debug = FALSE)
}
\arguments{
\item{...}{ Name of the covariate and, possibly of the
weights vector. NB: order counts!!!! The first specified
term is the covariate and the second one is the vector of
weights (which can be negative).}
\item{model}{ A string indicating the choosen model. The
 default is \code{iid}. See
 \code{names(inla.models()$latent)} for a list of possible
 alternatives and \code{\link{inla.doc}} for detailed docs.}
\item{copy}{TODO}
\item{same.as}{TODO}
\item{n}{An optional argument which defines the dimension
of the model if this is different from
\code{length(sort(unique(covariate)))}}
\item{nrep}{TODO}
\item{replicate}{We need to write documentation here}
\item{ngroup}{TODO}
\item{group}{TODO}
\item{control.group}{TODO}
\item{hyper}{Specification of the hyperparameter, fixed or
random, initial values, priors and its parameters. See
\code{?inla.models} for the list of hyparameters for each
model and its default options or
use \code{inla.doc()} for
detailed info on the family and
supported prior distributions.}
\item{initial}{THIS OPTION IS OBSOLETE; use
\code{hyper}!!! Vector indicating the starting values for
the optimization algorithm. The length of the vector
depends on the number of hyperparamters in the choosen
\code{model}. If \code{fixed=T} the value at which the
parameters are fixed is determines through \code{initial}.
See \code{inla.models()$latent$'model name'} to have info
about the choosen model.}
\item{prior}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
Prior distribution(s) for the hyperparameters of the
!random model. The default value depends on the type of
model, see !\url{www.r-inla.org} for a detailed
description of the models. See
\code{names(inla.models()$priors)} for possible prior
choices}
\item{param}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
Vector indicating the parameters \eqn{a}{a} and \eqn{b}{b}
of the prior distribution for the hyperparameters. The
length of the vector depends on the choosen \code{model}.
See \code{inla.models()$latent$'model name'} to have info
about the choosen model.}
\item{fixed}{THIS OPTION IS OBSOLETE; use \code{hyper}!!!
Vector of boolean variables indicating wheater the
hyperparameters of the model are fixed or random. The
length of the vector depends on the choosen \code{model}
See \code{inla.models()$latent$'model name'} to have info
about the choosen model.}
\item{season.length}{Lenght of the seasonal compoment
(ONLY if \code{model="seasonal"}) }
\item{constr}{A boolean variable indicating whater to set
a sum to 0 constraint on the term. By default the sum to 0
constraint is imposed on all intrinsic models
("iid","rw1","rw1","besag", etc..).}
\item{extraconstr}{This argument defines extra linear
constraints. The argument is a list with two elements, a
matrix \code{A} and a vector \code{e}, which defines the
extra constraint \code{Ax = e}; for example
\code{extraconstr = list(A = A, e=e)}. The number of
columns of \code{A} must correspond to the length of this
\code{f}-model.  Note that this constraint comes
additional to the sum-to-zero constraint defined if
\code{constr = TRUE}.}
\item{values}{An optional vector giving all values
assumed by the covariate for which we want estimated the
effect. It must be a numeric vector, a vector of factors
or \code{NULL}.}
\item{cyclic}{A boolean specifying wheather the model is
cyclical. Only valid for "rw1" and "rw2" models, is
cyclic=T then the sum to 0 constraint is removed. For the
correct form of the grah file see \cite{Martino and Rue
(2008)}.}
\item{diagonal}{An extra constant added to the diagonal of
the precision matrix.}
\item{graph}{Defines the graph-object either as a file with
a graph-description, an \code{inla.graph}-object, or as a
(sparse) symmetric matrix.}
\item{graph.file}{THIS OPTION IS OBSOLETE AND REPLACED BY
THE MORE GENERAL ARGUMENT \code{graph}. PLEASE CHANGE YOUR
CODE.
Name of the file containing the graph
of the model; see
\url{www.r-inla.org/faq}.}
\item{cdf}{A vector of maximum 10 values between 0 and 1
\eqn{x(0), x(1),\ldots}{x(0), x(1),\ldots}. The function
returns, for each posterior marginal the probabilities
\deqn{\mbox{Prob}(X<x(p))}{Prob(X<x(p))} }
\item{quantiles}{A vector of maximum 10 quantiles,
\eqn{p(0), p(1),\dots}{p(0), p(1),\ldots} to compute for
each posterior marginal. The function returns, for each
posterior marginal, the values
\eqn{x(0), x(1),\dots}{x(0), x(1),\ldots} such that
\deqn{\mbox{Prob}(X<x(p))=p}{Prob(X<x)=p}}
\item{Cmatrix}{The specification of the precision matrix
for the generic,  generic3 or z models (up to a scaling constant).
\code{Cmatrix} is either a
(dense) matrix, a matrix created using
\code{Matrix::sparseMatrix()}, or a filename which stores the
non-zero elements of \code{Cmatrix}, in three columns:
\code{i}, \code{j} and \code{Qij}. In case of the generic3 model,
it is a list of such specifications.}
\item{rankdef}{A number \bold{defining} the rank
deficiency of the model, with sum-to-zero constraint and
possible extra-constraints taken into account. See
details.}
\item{Z}{The matrix for the z-model}
\item{nrow}{Number of rows for 2d-models}
\item{ncol}{Number of columns for 2d-models}
\item{nu}{Smoothing parameter for the Matern2d-model,
possible values are \code{c(0, 1, 2, 3)}}
\item{bvalue}{TODO}
\item{spde.prefix}{TODO}
\item{spde2.prefix}{TODO}
\item{spde2.transform}{TODO}
\item{spde3.prefix}{TODO}
\item{spde3.transform}{TODO}
\item{mean.linear}{Prior mean for the linear component,
only used if \code{model="linear"}}
\item{prec.linear}{Prior precision for the linear
component, only used if \code{model="linear"}}
\item{compute}{ A boolean variable indicating wheather the
 marginal posterior distribution for the nodes in the
 \code{f()} model should be computed or not. This is
 usefull for large models where we are only interested in
 some posterior marginals.}
\item{of}{TODO}
\item{precision}{The precision for the artifical noise added when creating a copy of a model and others.}
\item{range}{A vector of size two giving the lower and
upper range for the scaling parameter \code{beta} in the
model \code{COPY}, \code{CLINEAR}, \code{MEC} and \code{MEB}.
If \code{low = high} then the identity mapping
is used.}
\item{adjust.for.con.comp}{If TRUE (default), adjust some
of the models (currently: besag, bym, bym2 and besag2) if the
number of connected components in graph is larger than
1. If FALSE, do nothing.}
\item{order}{Defines the \code{order} of the model: for
model \code{ar} this defines the order p, in AR(p). Not
used for other models at the time being.}
\item{scale}{A scaling vector. Its meaning depends on the model.}
\item{strata}{Currently not in use}
\item{rgeneric}{A object of class \code{inla.rgeneric} which defines the model. (EXPERIMENTAL!)}
\item{scale.model}{Logical. If \code{TRUE} then scale the RW1 and RW2 and BESAG and BYM and BESAG2 and RW2D models so the their (generlized) variance is 1. Default value is \code{inla.getOption("scale.model.default")}}
\item{args.slm}{Required arguments to the model="slm"; see the documentation for further details.},
\item{args.ar1c}{Required arguments to the model="ar1c"; see the documentation for further details.},
\item{args.intslope}{A list with the \code{subject} (factor),  \code{strata} (factor) and \code{covariates} (numeric) for the \code{intslope} model; see the documentation for further details.},
\item{vb.correct}{Add this model component to the list of variables to be used for the vb corrected Laplace approximation? If \code{NULL} use default choice,  otherwise correct if \code{TRUE} and do not if \code{FALSE}. (expert option)},
\item{locations}{A matrix with locations for the model \code{dmatern}. This also defines \code{n}.}
\item{debug}{Enable local debug output}
}
\value{TODO}
\details{There is no default value for \code{rankdef}, if it
is not defined by the user then it is computed by the rank
deficiency of the prior model (for the generic model, the
default is zero), plus 1 for the sum-to-zero constraint if the
prior model is proper, plus the number of extra
constraints. \bold{Oops:} This can be wrong, and then the user
must define the \code{rankdef} explicitely.}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{\code{\link{inla}}, \code{\link{hyperpar.inla}}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from mdata.R
%%
\name{inla.mdata}
\alias{inla.mdata}
\alias{is.inla.mdata}
\alias{as.inla.mdata}
\alias{print.inla.mdata}

\title{
Create an mdata-object for INLA
}

\description{
This defines an mdata-object for matrix valued response-families
}
\usage{
inla.mdata(y, ...)
is.inla.mdata(object)
as.inla.mdata(object)
}

\arguments{
  \item{y}{The response vector/matrix}
  \item{...}{Additional vectors/matrics of same length as \code{y}}
  \item{object}{Any \code{R}-object}
  \item{x}{An mdata object}
  }

\value{
     An object of class \code{inla.mdata}.  There is method for \code{print}.

     \code{is.inla.mdata} returns \code{TRUE} if \code{object}
     inherits from class \code{inla.mdata}, otherwise \code{FALSE}.

     \code{as.inla.mdata} returns an object of class \code{inla.mdata}
}
\note{It is often required to set \code{Y=inla.mdata(...)} and then
      define the formula as \code{Y~...},  especially when used with
      \code{inla.stack}.}
\author{
 Havard Rue
}

\seealso{
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde1.R
\name{inla.spde1.create}
\alias{inla.spde1.create}
\alias{inla.spde1.matern}
\alias{inla.spde1.imatern}
\alias{inla.spde1.matern.osc}
\alias{inla.spde1}
\title{Old SPDE model objects for INLA}
\usage{
inla.spde1.create(
  mesh,
  model = c("matern", "imatern", "matern.osc"),
  param = NULL,
  ...
)

inla.spde1.matern(mesh, ...)

inla.spde1.imatern(mesh, ...)

inla.spde1.matern.osc(mesh, ...)
}
\arguments{
\item{mesh}{The mesh to build the model on, as an \code{\link[=inla.mesh]{inla.mesh()}}
object.}

\item{model}{The name of the model.}

\item{param}{Model specific parameters.}

\item{...}{Additional parameters passed on to other methods.}
}
\value{
An \code{inla.spde1} object.
}
\description{
Create an \code{inla.spde1} model object.
}
\details{
Note: This is an old spde object format retained for backwards
compatibility.  Please use \code{\link[=inla.spde2]{inla.spde2()}} models for new code.

This method constructs an object for SPDE models.  Currently implemented:

\code{model="matern"}

\deqn{(\kappa^2(u)-\Delta)^{\alpha/2}(\tau(u) }{(kappa^2(u)-Delta)^(alpha/2)
(tau(u) x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2(u)-Delta)^(alpha/2) (tau(u)
x(u)) = W(u)}

\code{param}: \itemize{ \item\code{alpha} = 1 or 2 \item\code{basis.T} =
Matrix of basis functions for \eqn{\log\tau(u)}{log tau(u)}
\item\code{basis.K} = Matrix of basis functions for
\eqn{\log\kappa^2(u)}{log kappa^2(u)} }

\code{model="imatern"}

\deqn{(-\Delta)^{\alpha/2}(\tau(u) }{(-Delta)^(alpha/2) (tau(u) x(u)) =
W(u)}\deqn{ x(u))=W(u)}{(-Delta)^(alpha/2) (tau(u) x(u)) = W(u)}

\code{param}: \itemize{ \item\code{alpha} = 1 or 2 \item\code{basis.T} =
Matrix of basis functions for \eqn{\log\tau(u)}{log tau(u)} }
}
\examples{

n <- 100
field.fcn <- function(loc) (10 * cos(2 * pi * 2 * (loc[, 1] + loc[, 2])))
loc <- matrix(runif(n * 2), n, 2)
## One field, 2 observations per location
idx.y <- rep(1:n, 2)
y <- field.fcn(loc[idx.y, ]) + rnorm(length(idx.y))

mesh <- inla.mesh.create(loc, refine = list(max.edge = 0.05))
spde <- inla.spde1.create(mesh, model = "matern")
data <- list(y = y, field = mesh$idx$loc[idx.y])
formula <- y ~ -1 + f(field, model = spde)
result <- inla(formula, data = data, family = "normal")

## Plot the mesh structure:
plot(mesh)
\donttest{
if (require(rgl)) {
    ## Plot the posterior mean:
    plot(mesh,
        rgl = TRUE,
        result$summary.random$field[, "mean"],
        color.palette = colorRampPalette(c("blue", "green", "red"))
    )
    ## Plot residual field:
    plot(mesh,
        rgl = TRUE,
        result$summary.random$field[, "mean"] - field.fcn(mesh$loc),
        color.palette = colorRampPalette(c("blue", "green", "red"))
    )
}
}

}
\seealso{
\code{\link[=inla.spde2.matern]{inla.spde2.matern()}}, \code{\link[=inla.mesh.2d]{inla.mesh.2d()}},
\code{\link[=inla.mesh.basis]{inla.mesh.basis()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.matern.cov}
\alias{inla.matern.cov}
\alias{inla.matern.cov.s2}
\title{Numerical evaluation of Matern and related covariance functions.}
\usage{
inla.matern.cov(
  nu,
  kappa,
  x,
  d = 1,
  corr = FALSE,
  norm.corr = FALSE,
  theta,
  epsilon = 1e-08
)

inla.matern.cov.s2(nu, kappa, x, norm.corr = FALSE, theta = 0)
}
\arguments{
\item{nu}{The Matern smoothness parameter.}

\item{kappa}{The spatial scale parameter.}

\item{x}{Distance values.}

\item{d}{Space dimension; the domain is \eqn{R^d}{R^d}.}

\item{corr}{If \code{TRUE}, calculate correlations, otherwise calculate
covariances.  Only used for pure Matern models (i.e. with
\eqn{\theta=0}{theta=0}).}

\item{norm.corr}{If \code{TRUE}, normalise by the estimated variance, giving
approximate correlations.}

\item{theta}{Oscillation strength parameter.}

\item{epsilon}{Tolerance for detecting points close to distance zero.}
}
\description{
Calculates covariance and correlation functions for Matern models and
related oscillating SPDE models, on \eqn{R^d}{R^d} and on the sphere,
\eqn{S^2}{S^2}.
}
\details{
On \eqn{R^d}{R^d}, the models are \emph{defined} by the spectral density
given by
\deqn{S(w) = \frac{1}{(2\pi)^d (\kappa^4 + 2 \kappa^2 \cos(\pi
\theta) |w|^2 + |w|^4)^{(\nu + d/2)/2}}
}{S(w) = 1 / ( (2\pi)^d * (kappa^4 + 2 kappa^2 * cos(pi * theta) * |w|^2 +
|w|^4)^((nu + d/2)/2) )}

On \eqn{S^2}{S^2}, the models are \emph{defined} by the spectral
coefficients
\deqn{S(k) = \frac{2k+1}{4\pi (\kappa^4 + 2 \kappa^2 \cos(\pi
\theta) k(k+1) + k^2(k+1)^2)^{(\nu +
1)/2}}
}{S(k) = (2k+1) / (4 pi (kappa^4 + 2 kappa^2 cos(pi theta) k(k+1) +
k^2(k+1)^2)^((\nu + 1)/2) )}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from qinv.R
%%
 \name{qinv}
 \alias{inla.qinv}
 \alias{qinv}

 \title{Computes (parts of) the inverse of a SPD sparse matrix}

 \description{This routine use the GMRFLib implementation
              which compute parts of the inverse of a SPD sparse matrix.
              The diagonal and values for the neighbours in the inverse, are provided.}

 \usage{
     inla.qinv(Q, constr, reordering = INLA::inla.reorderings(), num.threads = NULL)
 }

 \arguments{

   \item{Q}{A SPD matrix,  either as a (dense) matrix or sparseMatrix.}
   \item{constr}{Optional linear constraints;
                 see \code{?INLA::f} and argument \code{extraconstr}}
   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
        either one of the names listed in \code{inla.reorderings()}
        or the output from \code{inla.qreordering(Q)}.
        The default is "auto" which try several reordering algorithm and use the best one for this particular matrix.}
   \item{num.threads}{Maximum number of threads the
                      \code{inla}-program will use, or as 'A:B' defining the number threads in the
                      outer (A) and inner (B) layer for nested parallelism. }
  }
 \value{
   \code{inla.qinv} returns a \code{sparseMatrix} of type \code{dgTMatrix} with the
   diagonal and values for the neigbours in the inverse. Note that the full inverse is NOT provided!
 }
 \author{Havard Rue \email{hrue@r-inla.org}}

 \examples{

 ## dense matrix example
 n = 10
 A = matrix(runif(n^2), n, n)
 Q = A \%*\% t(A)
 print(mean(abs(inla.qinv(Q) - solve(Q))))

 ## sparse matrix example
 rho = 0.9
 Q = toeplitz(c(1+rho^2, -rho,  rep(0, n-3), -rho)) / (1-rho^2)
 Q = inla.as.dgTMatrix(Q)
 Q.inv = inla.qinv(Q)

 ## compute the marginal variances as a vector from a precision matrix
 marginal.variances = diag(inla.qinv(Q))

 ## read the sparse matrix from a file in the 'i, j, value' format
 filename = tempfile()
 write(t(cbind(Q@i+1L,  Q@j+1L,  Q@x)), ncol=3, file=filename)
 Qinv = inla.qinv(filename)
 unlink(filename)
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from rerun.R
%%
 \name{inla.rerun}
 \alias{inla.rerun}
 \alias{rerun}

 \title{Rerun an analysis}
 \description{Rerun \code{\link{inla}} on an
              inla-object (output from \code{link{inla}})}
 \usage{
 inla.rerun(object, plain=FALSE)
 }
 \arguments{

\item{object}{An \code{inla}-object, ie the output from an \code{inla}-call}
\item{plain}{Logical. If \code{FALSE} (default), then make
changes in \code{object} to improve the performance}
}
\value{This function will take the result in \code{object},
       and rerun \code{inla} again.
       If \code{plain}  is \code{FALSE},  start  the optimization
       from the mode in \code{object} so that
       we can obtain an  improvement the mode for the hyperparameters.
       Otherwise,  start from the same configuration
       as for \code{object}.
       The returned value is an \code{inla}-object.}
\seealso{\code{\link{inla}}}
\examples{
r = inla(y ~ 1,  data = data.frame(y=1:10))
r = inla.rerun(r)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/over_sp_mesh.R
\name{inla.over_sp_mesh}
\alias{inla.over_sp_mesh}
\title{Check which mesh triangles are inside a polygon}
\usage{
inla.over_sp_mesh(x, y, type = c("centroid", "vertex"), ignore.CRS = FALSE)
}
\arguments{
\item{x}{geometry (typically a \code{\link[sp:SpatialPolygons]{sp::SpatialPolygons()}} object) for the queries}

\item{y}{an \code{\link[=inla.mesh]{inla.mesh()}} object}

\item{type}{the query type; either \code{'centroid'} (default, for triangle centroids),
or \code{'vertex'} (for mesh vertices)}

\item{ignore.CRS}{logical; whether to ignore the coordinate system information in \code{x} and \code{y}
(default \code{FALSE})}
}
\value{
A vector of triangle indices (when \code{type} is \code{'centroid'}) or
vertex indices (when \code{type} is \code{'vertex'})
}
\description{
Wrapper for the \code{\link[sp:over]{sp::over()}} method to find triangle centroids
or vertices inside \code{sp} polygon objects
}
\examples{
# Create a polygon and a mesh
obj <- sp::SpatialPolygons(list(Polygons(list(Polygon(rbind(
    c(0, 0),
    c(50, 0),
    c(50, 50),
    c(0, 50)
))),
ID = 1
)),
proj4string = inla.CRS("longlat_globe")
)
mesh <- inla.mesh.create(globe = 2, crs = inla.CRS("sphere"))

## 3 vertices found in the polygon
inla.over_sp_mesh(obj, mesh, type = "vertex")

## 3 triangles found in the polygon
inla.over_sp_mesh(obj, mesh)

## Multiple transformations can lead to slightly different results due to edge cases
## 4 triangles found in the polygon
inla.over_sp_mesh(obj, inla.spTransform(mesh, CRSobj = inla.CRS("mollweide_norm")), ignore.CRS = FALSE)

## Ignoring mismatching coordinate systems is rarely useful
## 20 triangles "found in" the polygon
inla.over_sp_mesh(obj, inla.spTransform(mesh, CRSobj = inla.CRS("mollweide_norm")), ignore.CRS = TRUE)
}
\author{
Haakon Bakka, \email{bakka@r-inla.org}, and Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.spTransform}
\alias{inla.spTransform}
\alias{inla.spTransform.default}
\alias{inla.spTransform.SpatialPoints}
\alias{inla.spTransform.inla.mesh.lattice}
\alias{inla.spTransform.inla.mesh.segment}
\alias{inla.spTransform.inla.mesh}
\alias{inla.spTransform.SpatialPointsDataFrame}
\title{Wrapper method for \code{sp::spTransform}}
\usage{
inla.spTransform(x, ...)

\method{inla.spTransform}{default}(x, crs0, crs1, passthrough = FALSE, ...)

\method{inla.spTransform}{SpatialPoints}(x, CRSobj, passthrough = FALSE, ...)

\method{inla.spTransform}{SpatialPointsDataFrame}(x, CRSobj, passthrough = FALSE, ...)

\method{inla.spTransform}{inla.mesh.lattice}(x, CRSobj, passthrough = FALSE, ...)

\method{inla.spTransform}{inla.mesh.segment}(x, CRSobj, passthrough = FALSE, ...)

\method{inla.spTransform}{inla.mesh}(x, CRSobj, passthrough = FALSE, ...)
}
\arguments{
\item{x}{The object that should be transformed from it's current CRS to a
new CRS}

\item{\dots}{Potential additional arguments}

\item{crs0}{The source \code{sp::CRS} or \code{inla.CRS} object}

\item{crs1}{The target \code{sp::CRS} or \code{inla.CRS} object}

\item{passthrough}{default FALSE. Setting to TRUE allows objects with no CRS
information to be passed through without transformation.}

\item{CRSobj}{The target \code{sp::CRS} or \code{inla.CRS} object}
}
\value{
The object is returned with its coordinates transformed
}
\description{
Handles transformation of various inla objects accorting to coordinate
reference systems of \code{sp::CRS} or \code{inla.CRS} class.
}
\details{
\code{inla.spTransform.default} Low level transformation of raw coordinates.
}
\examples{

if (require(rgdal)) {
    latt <- inla.mesh.lattice(-10:10, 40:60)
    mesh1 <- inla.mesh.create(
        lattice = latt, extend = FALSE, refine = FALSE,
        crs = inla.CRS("longlat")
    )
    mesh2 <- inla.spTransform(mesh1, inla.CRS("lambert"))
    summary(mesh1)
    summary(mesh2)
}
}
\seealso{
\code{\link[=inla.CRS]{inla.CRS()}}
}
\author{
Finn Lindgren \href{mailto:finn.lindgren@gmail.com}{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from mesh.components.R
%%
\name{inla.mesh.components}
\alias{inla.mesh.components}
\title{Compute connected mesh subsets}
\description{Compute subsets of vertices and triangles in an inla.mesh object
  that are connected by edges.
}
\usage{
inla.mesh.components(mesh)
}
\value{A list with elements \code{vertex} and \code{triangle}, vectors of
  integer labels for which connected component they belong, and \code{info},
  a \code{data.frame} with columns
  \item{component}{Connected component integer label.} 
  \item{nV}{The number of vertices in the component.}
  \item{nT}{The number of triangles in the component.}
  \item{area}{The surface area associated with the component.
  Component lables are not comparable across different meshes, but
  some ordering stability is guaranteed by initiating each component
  from the lowest numbered triangle whenever a new component is initiated.}
}
\author{Finn Lindgren \email{finn.lindgren@gmail.com}}
\seealso{inla.mesh.2d, inla.mesh.create}
\examples{
# Construct two simple meshes:
loc <- matrix(c(0,1,0,1), 2, 2)
mesh1 <- inla.mesh.2d(loc = loc, max.edge=0.1)
bnd <- inla.nonconvex.hull(loc, 0.3)
mesh2 <- inla.mesh.2d(boundary = bnd, max.edge=0.1)

# Compute connectivity information:
conn1 <- inla.mesh.components(mesh1)
conn2 <- inla.mesh.components(mesh2)
# One component, simply connected mesh
conn1$info
# Two disconnected components
conn2$info

# Extract the subset mesh for the largest component:
# (Note: some information is lost, such as fixed segments,
#  and boundary edge labels.)
maxi <- conn2$info$component[which.max(conn2$info$area)]
mesh3 <- inla.mesh.create(loc = mesh2$loc,
                          tv = mesh2$graph$tv[conn2$triangle == maxi,,drop=FALSE])
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from meshassessment.R
%%
\name{inla.mesh.assessment}
\alias{inla.mesh.assessment}
\title{Interactive mesh building and diagnostics}
\description{Assess the finite element
  approximation errors in a mesh for interactive R sessions.
  More detailed assessment tools are in \code{\link{meshbuilder}}.
}
\usage{
inla.mesh.assessment(mesh)
}
\author{Finn Lindgren \email{finn.lindgren@gmail.com}}
\seealso{inla.mesh.2d, inla.mesh.create, meshbuilder}
\examples{
bnd <- inla.mesh.segment(cbind(c(0, 10, 10, 0, 0),
                               c(0, 0, 10, 10, 0)), bnd = TRUE)
mesh <- inla.mesh.2d(boundary = bnd, max.edge = 1)
out <- inla.mesh.assessment(mesh, spatial.range = 3, alpha = 2)
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.group}
\alias{control.group}
\alias{inla.control.group}
\alias{inla.set.control.group.default}
\alias{set.control.group.default}
\alias{control.group.default}
\title{Control variables in control.group}
\description{Control variables in \code{control.group} for use in \code{inla}}
\usage{
inla.set.control.group.default(...)
control.group(adjust.for.con.comp, cyclic, fixed, graph, hyper, initial, model, order, param, prior, scale.model)
}
\arguments{
\item{...}{Possible arguments}
\item{model}{   Group model (one of 'exchangable', 'exchangablepos', 'ar1', 'ar', 'rw1', 'rw2', 'besag', or 'iid')}
\item{order}{   Defines the \code{order} of the model: for model \code{ar} this defines the order p, in AR(p). Not used for other models at the time being.}
\item{cyclic}{   Make the group model cyclic? (Only applies to models 'ar1', 'rw1' and 'rw2')}
\item{graph}{   The graph spesification (Only applies to model 'besag')}
\item{scale.model}{   Scale the intrinsic model (RW1, RW2, BESAG) so the generalized variance is 1. (Default \code{TRUE})}
\item{adjust.for.con.comp}{   Adjust for connected components when \code{scale.model=TRUE}? (default \code{TRUE})}
\item{hyper}{   Definition of the hyperparameter(s)}
\item{initial}{   (OBSOLETE!) The initial value for the group correlation or precision in the internal scale.}
\item{fixed}{   (OBSOLETE!) A boolean variable if the group correction or precision is assumed to be fixed or random.}
\item{prior}{   (OBSOLETE!) The name of the prior distribution for the group correlation or precision in the internal scale}
\item{param}{   (OBSOLETE!) Prior parameters}
}
\value{
  
The function \code{control.group} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.group.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.CRSargs}
\alias{inla.CRSargs}
\alias{inla.as.list.CRS}
\alias{inla.as.list.CRSargs}
\alias{inla.as.CRS.list}
\alias{inla.as.CRSargs.list}
\title{Show expanded CRS arguments}
\usage{
inla.CRSargs(x, ...)

inla.as.CRSargs.list(x, ...)

inla.as.list.CRSargs(x, ...)

inla.as.list.CRS(x, ...)

inla.as.CRS.list(x, ...)
}
\arguments{
\item{x}{An \code{sp::CRS} or \code{inla.CRS} object (for
\code{inla.CRSargs} and \code{inla.as.list.CRS}), a character string (for
\code{inla.as.list.CRSargs}), or a list (for \code{inla.as.CRS.list} and
\code{inla.as.CRSargs.list}).}

\item{\dots}{Additional arguments passed on to other methods.}
}
\value{
For \code{inla.CRSargs} and \code{inla.as.CRSargs.list}, a character
string with PROJ.4 arguments.

For \code{inla.as.list.CRS} and \code{inla.as.list.CRSargs}, a list of
name/value pairs.

For \code{inla.as.CRS.list}, a \code{CRS} or \code{inla.CRS} object.
}
\description{
Wrapper for \code{sp::CRS} and \code{inla.CRS} objects to extract the
coordinate reference system argument string.
Should no longer be used with PROJ6/rgdal3;
see \code{\link[=inla.crs_get_wkt]{inla.crs_get_wkt()}}
}
\details{
\itemize{
\item \code{inla.as.CRSargs.list}: CRS proj4 string for name=value pair list
}

\itemize{
\item \code{inla.as.list.CRSargs}: List of name=value pairs from CRS proj4 string
}
}
\examples{

if (require(rgdal)) {
    crs0 <- inla.CRS("longlat")
    p4s <- inla.CRSargs(crs0)
    lst <- inla.as.list.CRSargs(p4s)
    crs1 <- inla.as.CRS.list(lst)
    lst$a <- 2
    crs2 <- inla.CRS(p4s, args = lst)
    print(inla.CRSargs(crs0))
    print(inla.CRSargs(crs1))
    print(inla.CRSargs(crs2))
}
}
\seealso{
\code{\link[rgdal:CRS-class]{rgdal::CRSargs()}}, \code{\link[=inla.CRS]{inla.CRS()}}
}
\author{
Finn Lindgren \href{mailto:finn.lindgren@gmail.com}{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.1d.bary}
\alias{inla.mesh.1d.bary}
\alias{inla.mesh.1d.A}
\title{Mapping matrix for 1D meshes}
\usage{
inla.mesh.1d.bary(mesh, loc, method = c("linear", "nearest"))

inla.mesh.1d.A(
  mesh,
  loc,
  weights = NULL,
  derivatives = NULL,
  method = c("linear", "nearest", "quadratic")
)
}
\arguments{
\item{mesh}{An \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object.}

\item{loc}{Coordinate values.}

\item{method}{Interpolation method.  If not specified for
\code{inla.mesh.1d.A} (recommended), it is determined by the \code{mesh}
basis function properties.}

\item{weights}{Weights to be applied to the \code{A} matrix rows.}

\item{derivatives}{If \code{TRUE}, also compute derivative weight matrices
\code{dA} and \code{d2A}.}
}
\description{
Calculates barycentric coordinates and weight matrices for
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} objects.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Munich}
\alias{Munich}
\title{The Munich rent data}
\format{
A data frame with 2035 observations on the following 17 variables.
\describe{

\item{"rent}{Net rent per square meter.}
\item{"floor.size}{Size of the flat in square meters.}
\item{"year}{Year of construction of the building in which the flat
is located.} \item{location}{Location index (in terms of
subquarters).} \item{Gute.Wohnlage}{Dummy variable for good
locations / good neighborhoods.} \item{Beste.Wohnlage}{Dummy
variable for very good locations / very good neighborhoods.}
\item{Keine.Wwv}{Dummy for absence of warm water supply.}
\item{Keine.Zh}{Dummy for absence of central heating system.}
\item{Kein.Badkach}{Dummy for absence of flagging in the bathroom.}
\item{Besond.Bad}{Dummy for special features of the bathroom.}
\item{Gehobene.Kueche}{Dummy for more refined kitchen equipment.}
\item{zim1}{Dummy for a flat with 1 room.} \item{zim2}{Dummy
for a flat with 2 rooms.} \item{zim3}{Dummy for a flat with 3
rooms.} \item{zim4}{Dummy for a flat with 4 rooms.}
\item{zim5}{Dummy for a flat with 5 rooms.}
\item{zim6}{Dummy for a flat with 6 rooms.}

}
}
\source{
See Rue and Held (2005), Chapter 4.
}
\description{
The Munich rent data
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.pardiso.default}
\alias{control.pardiso.default}
\alias{inla.control.pardiso.default}
\alias{inla.set.control.pardiso.default.default}
\alias{set.control.pardiso.default.default}
\alias{control.pardiso.default.default}
\title{Control variables in control.pardiso.default}
\description{Control variables in \code{control.pardiso.default} for use in \code{inla}}
\usage{
inla.set.control.pardiso.default.default(...)
control.pardiso.default(debug, nrhs, parallel.reordering, verbose)
}
\arguments{
\item{...}{Possible arguments}
\item{verbose}{   Show detailed output (default FALSE)}
\item{debug}{   Show internal debug output (default FALSE)}
\item{parallel.reordering}{   Do reordering in parallel (default TRUE)}
\item{nrhs}{   Number of right-hand sides to solve for in parallel (default depends on the argument \code{num.threads} and \code{-1} will not change it)}
}
\value{
   Extra options controlling the PARDISO library
The function \code{control.pardiso.default} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.pardiso.default.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{plot.inla.CRS}
\alias{plot.inla.CRS}
\alias{plot.CRS}
\title{Plot CRS and inla.CRS objects}
\usage{
\method{plot}{inla.CRS}(
  x,
  xlim = NULL,
  ylim = NULL,
  outline = TRUE,
  graticule = c(15, 15, 45),
  tissot = c(30, 30, 30),
  asp = 1,
  add = FALSE,
  eps = 0.05,
  ...
)

\method{plot}{CRS}(
  x,
  xlim = NULL,
  ylim = NULL,
  outline = TRUE,
  graticule = c(15, 15, 45),
  tissot = c(30, 30, 30),
  asp = 1,
  add = FALSE,
  eps = 0.05,
  ...
)
}
\arguments{
\item{x}{A \code{CRS} or \code{\link[=inla.CRS]{inla.CRS()}} object.}

\item{xlim}{Optional x-axis limits.}

\item{ylim}{Optional y-axis limits.}

\item{outline}{Logical, if \code{TRUE}, draw the outline of the projection.}

\item{graticule}{Vector of length at most 3, to plot meridians with spacing
\code{graticule[1]} degrees and parallels with spacing \code{graticule[2]}
degrees. \code{graticule[3]} optionally specifies the spacing above and
below the first and last parallel.  When \code{graticule[1]==0} no meridians
are drawn, and when \code{graticule[2]==0} no parallels are drawn. Use
\code{graticule=NULL} to skip drawing a graticule.}

\item{tissot}{Vector of length at most 3, to plot Tissot's indicatrices with
spacing \code{tissot[1]} degrees and parallels with spacing \code{tissot[2]}
degrees. \code{tissot[3]} specifices a scaling factor.  Use
\code{tissot=NULL} to skip drawing a Tissot's indicatrices.}

\item{asp}{The aspect ratio for the plot, default 1.}

\item{add}{If \code{TRUE}, add the projecton plot to an existing plot.}

\item{eps}{Clipping tolerance for rudimentary boundary clipping}

\item{\dots}{Additional arguments passed on to the internal calls to
\code{plot} and \code{lines}.}
}
\description{
Plot the outline of a CRS or inla.CRS projection, with optional graticules
(transformed parallels and meridians) and Tissot indicatrices.
}
\examples{

if (require(rgdal)) {
    oblique <- c(0, 45, 45, 0)
    for (projtype in c("longlat", "lambert", "mollweide", "hammer")) {
        plot(inla.CRS(projtype), main = projtype)
        plot(inla.CRS(projtype, oblique = oblique), main = paste("oblique", projtype))
    }
}
}
\seealso{
\code{\link[=inla.CRS]{inla.CRS()}}
}
\author{
Finn Lindgren \href{mailto:finn.lindgren@gmail.com}{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from graph.convert.R
%%
\name{geobugs2inla}
\alias{geobugs2inla}
\alias{inla.geobugs2inla}
\keyword{graph}

\title{INLA utility functions}

\description{Various utility functions for INLA}

\usage{
     inla.geobugs2inla(adj, num, graph.file="graph.dat")
}

\arguments{

  \item{adj}{A vector listing the ID numbers of the adjacent areas for
  each area. This is a sparse representation of the full adjacency
  matrix for the study region, and can be generated using the Adjacency
  Tool from the Map menu in GeoBUGS.}

  \item{num}{A vector of length N (the total number of areas) giving the
  number of neighbours n.i for each area.}

  \item{graph.file}{Name of the file of the new graph in the INLA
    format.}

}


\value{The return value is the name of the graph-file created.}

\note{These are all the same function,  and the two different names are due to backward-compatibility}

\author{Havard Rue \email{hrue@r-inla.org}}

\seealso{\code{\link{inla}}, \code{\link{inla.surv}}, \code{\link{hyperpar.inla}}}

%%\examples{}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from surv.R
%%
\name{inla.surv}
\alias{inla.surv}
\alias{is.inla.surv}
\alias{as.inla.surv}
\alias{plot.inla.surv}
\alias{print.inla.surv}

\title{
Create a Survival Object for INLA
}

\description{
Create a survival object, to be used as a response variable in a
model  formula for the \code{\link{inla}} function for survival models.
}
\usage{
inla.surv(time, event, time2, truncation, subject = NULL, .special = NULL)
\method{plot}{inla.surv}(x, y, ...)
\method{print}{inla.surv}(x, ...)
is.inla.surv(object)
as.inla.surv(object, ...)
}

\arguments{
  \item{time}{For right censored data, this is the follow up time.  For
    interval data, this is the starting time for the interval. For in-interval event, this is
    the observed time (in the interval) for the event. For left censored data, this the
    censoring time. }
  \item{event}{The status indicator, 1=observed event, 0=right censored
    event, 2=left censored event, 3=interval censored event,
    and 4=observed event in an interval (left, right).}
  \item{time2}{Ending time for the interval censored data or an in-interval event.}
  \item{truncation}{Left truncation. If missing it is assumed to be 0. The lower limit for event=4.}
  \item{subject}{Patient number in multiple event data, not needed otherwise. }
  \item{.special}{An internal object, not for public use}
  \item{object}{Any \code{R}-object}
  \item{x}{Object to plot or print}
  \item{y}{Object to plot (not in use)}
  \item{...}{Additional argument}
  }

\value{An object of class \code{inla.surv}.  There are methods for \code{print},
  \code{plot} for \code{inla.surv} objects.

     \code{is.inla.surv} returns \code{TRUE} if \code{object}
     inherits from class \code{inla.surv}, otherwise \code{FALSE}.

     \code{as.inla.surv} returns an object of class \code{inla.surv}
}

\author{
Sara Martino, Rupali Akerkar and Haavard Rue
}

\seealso{
\code{\link{inla}}
}

\examples{
  ## First example
  trt = c(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
          0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
          1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)
  time = c(17,42,44,48,60,72,74,95,103, 108, 122, 144, 167, 170, 183, 185,
           193, 195, 197, 208, 234, 235, 254, 307, 315, 401, 445, 464, 484,  528, 542, 567,
           577, 580, 795, 855, 1174, 1214, 1232, 1366, 1455, 1585, 1622, 1626, 1736, 1,63,
           105, 125, 182, 216, 250, 262, 301, 301, 342, 354, 356, 358, 380, 383, 383, 388,
           394, 408, 460, 489, 499, 523, 524, 535, 562, 569, 675, 676, 748, 778, 786, 797,
           955, 968, 977, 1245, 1271, 1420, 1460, 1516, 1551, 1690, 1694)
  event = c(1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
            1,1,1,1,0,1,0,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,0,1)
  y = inla.surv(time, event)

  ## Second example
  time = c(182,182,63,68,182,152,182,130,134,145,152,182,98,152,182,88,95,105,130,137,167,182,
           152,182,81,182,71,84,126,134,152,182)
  event = c(1,0,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0,1,0,1,0,1,1,1,1,1,0)
  subject = c(1,2,3,3,3,4,4,5,5,5,5,5,6,6,6,7,7,7,7,7,7,7,8,8,9,9,10,10,10,10,10,10)
  y = inla.surv(time, event, subject=subject)
}

\keyword{Survival models}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from qsample.R
%%
 \name{qsample}
 \alias{inla.qsample}
 \alias{qsample}

 \title{Generate samples from a GMRF using the GMRFLib implementation}

 \description{This function generate samples from a GMRF using the GMRFLib implementation}
 \usage{
 inla.qsample(
        n = 1L,
        Q,
        b,
        mu,
        sample,
        constr,
        reordering = INLA::inla.reorderings(),
        seed = 0L,
        logdens = ifelse(missing(sample), FALSE, TRUE),
        compute.mean = ifelse(missing(sample), FALSE, TRUE),
        num.threads = if (seed == 0L) "0:0" else NULL,
        selection = NULL, verbose = FALSE)
 }

 \arguments{
   \item{n}{Number of samples. Only used if \code{missing(sample)}}
   \item{Q}{The precision matrix or a filename containing it.}
   \item{b}{The linear term}
   \item{mu}{The mu term}
   \item{sample}{A matrix of optional samples where each column is a sample. If set, then evaluate the log-density for each sample only.}
   \item{constr}{Optional linear constraints; see \code{?INLA::f} and argument \code{extraconstr}}
   \item{reordering}{The type of reordering algorithm to be used for \code{TAUCS};
        either one of the names listed in \code{inla.reorderings()}
        or the output from \code{inla.qreordering(Q)}.
        The default is "auto" which try several reordering algorithm and use the best one for this particular matrix.}
   \item{seed}{Control the RNG. If \code{seed=0L} then GMRFLib will set the seed
               intelligently/at 'random',  and this is and should be the default behaviour.
               If \code{seed < 0L}  then the saved state of the RNG will be reused if possible, otherwise,
               GMRFLib will set the seed intelligently/at 'random'.
               If \code{seed > 0L} then this value is used as the seed for the RNG.

               PLEASE NOTE1: If \code{seed!=0} then the computations will run in serial mode,  over-riding
               whatever is set in \code{num.threads} (a warning might be issued).

               PLEASE NOTE2: If the PARDISO sparse matrix library is used, continuity of the samples
               with respect to small changes in the precision matrix, can be expected but is
               not guaranteed. If this feature is required, please use the TAUCS sparse matrix library.}
   \item{logdens}{If \code{TRUE}, compute also the log-density of each sample. Note that the output format then change.}
   \item{compute.mean}{If \code{TRUE}, compute also the (constrained) mean. Note that the output format then change.}
   \item{num.threads}{Maximum number of threads the
                      \code{inla}-program will use, or as 'A:B' defining the number threads in the
                      outer (A) and inner (B) layer for nested parallelism.
                      \code{seed!=0} requires serial comptuations.}
   \item{selection}{A vector of indices of each sample to
                    return. \code{NULL} means return the whole sample.
                    (Note that the log-density retured,  is for the whole sample.)
                    The use of \code{selection} cannot be combined with the use of \code{sample}.}
   \item{verbose}{Logical. Run in verbose mode or not.}
 }
\value{
      The log-density has form {-1/2(x-mu)^T Q (x-mu) + b^T x}

  If \code{logdens} is \code{FALSE},  then \code{inla.qsample} returns
  the samples in a matrix,  where each column is a sample.
  If \code{logdens} or \code{compute.mean} is \code{TRUE}, then a list
   with names \code{sample},
  \code{logdens} and \code{mean} is returned. The samples are stored in the matrix
  \code{sample} where each column is a sample, and the log
  densities of each sample are stored as the vector \code{logdens}.
  The mean (include corrections for the constraints,  if any) is store in
  the vector \code{mean}.
}
\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
 g = system.file("demodata/germany.graph", package="INLA")
 Q = inla.graph2matrix(g)
 diag(Q) = dim(Q)[1L]
 x = inla.qsample(10, Q)
 \dontrun{matplot(x)}
 x = inla.qsample(10, Q, logdens=TRUE)
 \dontrun{matplot(x$sample)}

 n = 3
 Q = diag(n)
 ns = 2

 ## sample and evaluate a sample
 x = inla.qsample(n, Q=Q, logdens=TRUE)
 xx = inla.qsample(Q=Q,  sample = x$sample)
 print(x$logdens - xx$logdens)

 ## the use of a constraint
 constr = list(A = matrix(rep(1, n), 1, n), e = 0)
 x = inla.qsample(n, Q=Q, constr=constr)
 print(constr$A \%*\% x)

 ## control the RNG (require serial mode)
 x = inla.qsample(n, Q=Q, seed = 123, num.threads="1:1")
 ## restart from same seed,  only sample 1
 xx = inla.qsample(n=1, Q=Q, seed = 123, num.threads="1:1")
 ## continue from the save state, sample the remaining 2
 xxx = inla.qsample(n=n-1, Q=Q, seed = -1, num.threads="1:1")
 ## should be 0
 print(x - cbind(xx, xxx))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.basis}
\alias{inla.mesh.basis}
\title{Basis functions for inla.mesh}
\usage{
inla.mesh.basis(
  mesh,
  type = "b.spline",
  n = 3,
  degree = 2,
  knot.placement = "uniform.area",
  rot.inv = TRUE,
  boundary = "free",
  free.clamped = TRUE,
  ...
)
}
\arguments{
\item{mesh}{An \code{inla.mesh.1d} or \code{inla.mesh} object.}

\item{type}{\code{b.spline} (default) for B-spline basis functions,
\code{sph.harm} for spherical harmonics (available opnly for meshes on the
sphere)}

\item{n}{For B-splines, the number of basis functions in each direction (for
1d meshes \code{n} must be a scalar, and for planar 2d meshes a 2-vector).
For spherical harmonics, \code{n} is the maximal harmonic order.}

\item{degree}{Degree of B-spline polynomials.  See
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}}.}

\item{knot.placement}{For B-splines on the sphere, controls the latitudinal
placements of knots. \code{"uniform.area"} (default) gives uniform spacing
in sin(latitude), \code{"uniform.latitude"} gives uniform spacing in
latitudes.}

\item{rot.inv}{For spherical harmonics on a sphere, \code{rot.inv=TRUE}
gives the rotationally invariant subset of basis functions.}

\item{boundary}{Boundary specification, default is free boundaries.  See
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} for more information.}

\item{free.clamped}{If \code{TRUE} and \code{boundary} is \code{"free"}, the
boundary basis functions are clamped to 0/1 at the interval boundary by
repeating the boundary knots.}

\item{...}{}
}
\description{
Calculate basis functions on a 1d or 2d \code{\link[=inla.mesh]{inla.mesh()}}
}
\examples{

n <- 100
loc <- matrix(runif(n * 2), n, 2)
mesh <- inla.mesh.2d(loc, max.edge = 0.05)
basis <- inla.mesh.basis(mesh, n = c(4, 5))

proj <- inla.mesh.projector(mesh)
image(proj$x, proj$y, inla.mesh.project(proj, basis[, 7]))
\donttest{
if (require(rgl)) {
    plot(mesh, rgl = TRUE, col = basis[, 7], draw.edges = FALSE, draw.vertices = FALSE)
}
}

}
\seealso{
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} \code{\link[=inla.mesh.2d]{inla.mesh.2d()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barrier.R
\name{inla.barrier.pcmatern}
\alias{inla.barrier.pcmatern}
\alias{inla.barrier}
\alias{barrier}
\alias{barrier.pcmatern}
\alias{inla.barrier.polygon}
\alias{barrier.polygon}
\alias{inla.barrier.q}
\alias{barrier.q}
\alias{inla.barrier.fem}
\alias{barrier.fem}
\title{Functions for defining the Barrier models}
\usage{
inla.barrier.pcmatern(
  mesh,
  barrier.triangles,
  prior.range,
  prior.sigma,
  range.fraction = 0.2
)

inla.barrier.polygon(mesh, barrier.triangles, Omega = NULL)

inla.barrier.q(fem, ranges, sigma = 1, envir = NULL)

inla.barrier.fem(mesh, barrier.triangles, Omega = NULL)
}
\arguments{
\item{mesh}{The mesh to build the model on, from inla.mesh.2d}

\item{barrier.triangles}{The numerical ids of the triangles that make up the
barrier area}

\item{prior.range}{2 parameters \verb{(range0,Prange)} for the prior spatial
range.  If \code{Prange} is \code{NA}, then \code{range0} is used as a fixed
range value (not tested).}

\item{prior.sigma}{2 parameters \verb{(sig0,Psig)} for the prior marginal
standard deviation sigma.  If \code{Psig} is \code{NA}, then \code{sig0} is
used as a fixed sigma value (not tested).}

\item{range.fraction}{The length of the spatial range inside the barrier
area, as a fraction of the range parameter.}

\item{Omega}{Advanced option for creating a set of permeable barriers (not
documented)}

\item{fem}{represents the Barrier model or the Different Terrains (DT) model,
by containing all the needed matrices to solve the SPDE}

\item{ranges, sigma}{the hyperparameters that determine Q}

\item{envir}{the environment used for caching (with optimize=TRUE), if any}
}
\value{
\itemize{
\item \code{inla.barrier.pcmatern} gives the (rgeneric) model object
for fitting the model in INLA
\item \code{inla.barrier.polygon} gives the polygon
around the barrier (mainly for plotting)
\item \code{inla.barrier.q} is an
internal method producing the Q matrix from a result of inla.barrier.fem,
\item \code{inla.barrier.fem} is an internal method producing the Finite Element
matrices.
}
}
\description{
Functions for defining Barrier models as an \verb{inla rgeneric} model
}
\details{
This model is described in the ArXiv preprint arXiv:1608.03787.  For
examples, see \url{https://haakonbakkagit.github.io/btopic128.html}
\itemize{
\item \code{inla.barrier.pcmatern}
This function creates the model component used in inla(...)
}

\itemize{
\item \code{inla.barrier.polygon} This function constructs SpatialPolygons for the different subdomains (areas)
}

\itemize{
\item \code{inla.barrier.q}: This function computes a specific precision matrix
}

\itemize{
\item \code{inla.barrier.fem} This function computes the Finite Element
matrices that are needed to compute the precision matrix Q later
}
}
\seealso{
inla.spde2.pcmatern
}
\author{
Haakon Bakka \email{bakka@r-inla.org}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from rgeneric.R
%%
\name{rgeneric.define}
\alias{rgeneric}
\alias{rgeneric.define}
\alias{inla.rgeneric.define}
\alias{rgeneric.ar1.model}
\alias{inla.rgeneric.ar1.model}
\alias{rgeneric.iid.model}
\alias{inla.rgeneric.iid.model}
\alias{rgeneric.wrapper}
\alias{inla.rgeneric.wrapper}
\alias{rgeneric.q}
\alias{inla.rgeneric.q}

\title{rgeneric models}

\description{A framework for defining latent models in R}

\usage{
inla.rgeneric.define(model = NULL, debug = FALSE, compile = TRUE, optimize = FALSE, ...)
inla.rgeneric.iid.model(
        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
        theta = NULL)
inla.rgeneric.ar1.model(
        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
        theta = NULL)
inla.rgeneric.ar1.model.opt(
        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
        theta = NULL)
inla.rgeneric.wrapper(
        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
        model, theta = NULL)
inla.rgeneric.q(
        rmodel,
        cmd = c("graph", "Q", "mu", "initial", "log.norm.const", "log.prior", "quit"),
        theta = NULL)
}

\arguments{
  \item{model}{The definition of the model; see \code{inla.rgeneric.ar1.model}}
  \item{rmodel}{The rgeneric model-object, the output of \code{inla.rgeneric.define}}
  \item{debug}{Logical. Turn on/off debugging}
  \item{compile}{Logical. Compile the definition of the model or not.}
  \item{optimze}{Logical. With this option \code{TRUE}, then \code{model}
                 pass only the values of \code{Q} and not the whole matrix.
                 Please see the vignette for details and
                 \code{inla.rgeneric.ar1.model.opt} for an example.}
  \item{cmd}{An allowed request}
  \item{theta}{Values of theta}
  \item{...}{Named list of variables that defines the environment of \code{model}}
  \item{debug}{Logical. Enable debug output}
}

\value{%%
  This allows a latent model to be
  defined in \code{R}.
  See \code{inla.rgeneric.ar1.model} and
  \code{inla.rgeneric.iid.model} and the documentation for
  worked out examples of how to define latent models in  this way.
  This will be somewhat slow and is intended for special cases and
  protyping. The function \code{inla.rgeneric.wrapper} is for
  internal use only.}
\author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-sn.R
%%
 \name{pc.sn}
 \alias{inla.pc.sn}
 \alias{pc.sn}
 \alias{pc.rsn}
 \alias{inla.pc.rsn}
 \alias{pc.dsn}
 \alias{inla.pc.dsn}
 \alias{pc.psn}
 \alias{inla.pc.psn}
 \alias{pc.qsn}
 \alias{inla.pc.qsn}

 \title{Utility functions for the PC prior for \code{skewness} in the skew-normal
 linkfunction and likelihood}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the \code{skewness}
              in the skew-normal link-function and likelihood}
 \usage{
 inla.pc.rsn(n, lambda = 40)
 inla.pc.dsn(skew, lambda = 40, log = FALSE)
 inla.pc.qsn(p, lambda = 40)
 inla.pc.psn(q, lambda = 40)
 }
 \arguments{
   \item{n}{number of observations}
   \item{lambda}{the rate parameter in the PC prior}
   \item{skew}{vector of evaluation points}
   \item{log}{logical. return the density in natural or log-scale.}
   \item{p}{vector of probabilities}
   \item{q}{vector of quantiles}
 }
 \details{
 Defines the PC prior for the \code{skewness} for the skew-normal linkfunction
 and likelihood, where \code{skew=0} is the base model. The skewness range from
 -0.99527... to 0.99527....  ca.
 }
\value{%%
  \code{inla.pc.dsn} gives the density,
  \code{inla.pc.psn} gives the distribution function,
  \code{inla.pc.qsn} gives the quantile function, and
  \code{inla.pc.rsn} generates random deviates.
 }
 \seealso{inla.doc("pc.sn")}
 \author{havard rue \email{hrue@r-inla.org}}
 \examples{
 x = inla.pc.rsn(100,  lambda = 40)
 d = inla.pc.dsn(x, lambda = 40)
 x = inla.pc.qsn(0.5, lambda = 40)
 inla.pc.psn(x, lambda = 40)
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-multvar.R
%%
 \name{pc.multvar}
 \alias{pc.multvar}
 \alias{inla.pc.multvar}
 \alias{inla.pc.multvar.simplex}
 \alias{inla.pc.multvar.simplex.d}
 \alias{inla.pc.multvar.simplex.r}
 \alias{inla.pc.multvar.sphere}
 \alias{inla.pc.multvar.sphere.d}
 \alias{inla.pc.multvar.sphere.r}
 \alias{inla.pc.multvar.h.default}
 \alias{pc.multvar}
 \alias{pc.multvar.simplex}
 \alias{pc.multvar.simplex.d}
 \alias{pc.multvar.simplex.r}
 \alias{pc.multvar.sphere}
 \alias{pc.multvar.sphere.d}
 \alias{pc.multvar.sphere.r}
 \alias{pc.multvar.h.default}

 \title{Multivariate PC priors}

 \description{Functions to evaluate and simulate from
              multivariate PC priors: The simplex and sphere case}
 \usage{
 inla.pc.multvar.h.default(x, inverse = FALSE, derivative = FALSE)
 inla.pc.multvar.simplex.r(n = NULL, lambda = 1, h = inla.pc.multvar.h.default, b = NULL)
 inla.pc.multvar.simplex.d(x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, b = NULL)
 inla.pc.multvar.sphere.r(n = NULL, lambda = 1, h = inla.pc.multvar.h.default, H = NULL)
 inla.pc.multvar.sphere.d(x = NULL, lambda = 1, log = FALSE, h = inla.pc.multvar.h.default, H = NULL)
 }
 \arguments{
   \item{x}{Samples to evaluate. If input is a matrix then each row is a sample. If input is
            a vector then this is the sample.}
   \item{inverse}{Compute the inverse of the h()-function.}
   \item{derivative}{Compute the derivative of the h()-function. (derivative of the inverse
                     function is not used).}
   \item{n}{Number of samples to generate.}
   \item{lambda}{The lambda-parameter in the PC-prior.}
   \item{log}{Evaluate the density in log-scale or ordinary scale.}
   \item{h}{The h()-function,  defaults to \code{inla.pc.multvar.h.default}. See that code
            for an example of how to write a user-spesific function.}
   \item{b}{The b-vector (gradient) in the expression for the simplex option,  \code{d(xi) = h(b^T xi)}}
   \item{H}{The H(essian)-matrix in the expression for the sphere option,  \code{d(xi) =
            h(1/2 *xi^T H xi)}. If \code{H} is a vector, then it is interpreted as the
            diagonal of a (sparse) diagonal matrix.}
}
 \details{
   These functions implements multivariate PC-priors of the simplex and sphere type.
 }
 \value{%%
     \code{inla.pc.multvar.simplex.r} generate samples from the simplex case, and
     \code{inla.pc.multvar.simplex.d} evaluate the density.
     \code{inla.pc.multvar.sphere.r} generate samples from the sphere case,  and
     \code{inla.pc.multvar.sphere.d} evaluate the density.
     \code{inla.pc.multvar.h.default} implements the default h()-function and illustrate how
     to code your own spesific one, if needed.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.inla}
\alias{control.inla}
\alias{inla.control.inla}
\alias{inla.set.control.inla.default}
\alias{set.control.inla.default}
\alias{control.inla.default}
\title{Control variables in control.inla}
\description{Control variables in \code{control.inla} for use in \code{inla}}
\usage{
inla.set.control.inla.default(...)
control.inla(adapt.hessian.max.trials, adapt.hessian.mode, adapt.hessian.scale, adaptive.max, adjust.weights, b.strategy, cmin, constr.marginal.diagonal, cpo.diff, cutoff, diagonal, diff.logdens, dz, fast, fast, force.diagonal, global.node.degree, global.node.factor, h, huge, improved.simplified.laplace, int.design, int.strategy, interpolator, lincomb.derived.correlation.matrix, linear.correction, mode.known, npoints, num.gradient, num.hessian, numint.abserr, numint.maxfeval, numint.relerr, optimise.strategy, optimiser, print.joint.hyper, reordering, restart, skip.configurations, stencil, step.factor, step.len, strategy, stupid.search, stupid.search.factor, stupid.search.max.iter, tolerance, tolerance.f, tolerance.g, tolerance.step, tolerance.x, use.directions, verbose)
}
\arguments{
\item{...}{Possible arguments}
\item{strategy}{   Character The strategy to use for the approximations; one of 'auto' (default), 'gaussian', 'simplified.laplace', 'laplace' or 'adaptive'}
\item{int.strategy}{   Character The integration strategy to use; one of 'auto' (default), 'ccd', 'grid', 'eb' (empirical bayes), 'user' or 'user.std'}
\item{int.design}{   Matrix Matrix of user-defined integration points and weights. Each row consists theta values and the integration weight. (EXPERIMENTAL!)}
\item{interpolator}{   Character The interpolator used to compute the marginals for the hyperparameters. One of 'auto', 'nearest', 'quadratic', 'weighted.distance', 'ccd', 'ccdintegrate', 'gridsum', 'gaussian'. Default is 'auto'.}
\item{fast}{   Logical If TRUE, then replace conditional modes in the Laplace approximation with conditional expectation (default TRUE)}
\item{linear.correction}{   Logical Default TRUE for the 'strategy = laplace' option.}
\item{h}{   Numerical The step-length for the gradient calculations for the hyperparameters. Default 0.005.}
\item{dz}{   Numerical The step-length in the standarised scale for the integration of the hyperparameters. Default 0.75.}
\item{diff.logdens}{   Numerical The difference of the log.density for the hyperpameters to stop numerical integration using int.strategy='grid'. Default 6.}
\item{print.joint.hyper}{   Logical If TRUE, the store also the joint distribution of the hyperparameters (without any costs). Default TRUE.}
\item{force.diagonal}{   Logical If TRUE, then force the Hessian to be diagonal. (Default \code{FALSE})}
\item{skip.configurations}{   Logical Skip configurations if the values at the main axis are to small. (Default \code{TRUE})}
\item{mode.known}{   Logical If TRUE then no optimisation is done. (Default FALSE.)}
\item{adjust.weights}{   Logical If TRUE then just more accurate integration weights. (Default TRUE.)}
\item{tolerance}{   Numerical The tolerance for the optimisation of the hyperparameters. If set, this is the default value for for 'tolerance.f/2', 'tolerance.g' and 'tolerance.x'; see below.}
\item{tolerance.f}{   Numerical The tolerance for the absolute change in the log posterior in the optimisation of the hyperparameters.}
\item{tolerance.g}{   Numerical The tolerance for the absolute change in the gradient of the log posterior in the optimisation of the hyperparameters.}
\item{tolerance.x}{   Numerical The tolerance for the change in the hyperparameters (root-mean-square) in the optimisation of the hyperparameters.}
\item{tolerance.step}{   Numerical The tolerance for the change in root-mean_squre in the inner Newton-like optimisation of the latent field.}
\item{restart}{   Numerical To improve the optimisation, the optimiser is restarted at the found optimum 'restart' number of times.}
\item{optimiser}{   Character The optimiser to use; one of 'gsl' or 'default'.}
\item{verbose}{   Logical Run in verbose mode? (Default FALSE)}
\item{reordering}{   Character Type of reordering to use. (EXPERT OPTION; one of "AUTO", "DEFAULT", "IDENTITY", "REVERSEIDENTITY", "BAND", "METIS", "GENMMD", "AMD", "MD", "MMD", "AMDBAR", "AMDC", "AMDBARC", or the output from \code{inla.qreordering}. Default is 'auto'.)}
\item{cpo.diff}{   Numerical Threshold to define when the cpo-calculations are inaccurate. (EXPERT OPTION.)}
\item{npoints}{   Numerical Number of points to use in the 'stratey=laplace' approximation (default 9)}
\item{cutoff}{   Numerical The cutoff used in the 'stratey=laplace' approximation. (Smaller value is more accurate and more slow.) (default 1e-4)}
\item{adapt.hessian.mode}{   Logical Should optimisation be continued if the Hessian estimate is void? (Default TRUE)}
\item{adapt.hessian.max.trials}{   Numerical Number of steps in the adaptive Hessian optimisation}
\item{adapt.hessian.scale}{   Numerical The scaling of the 'h' after each trial.}
\item{adaptive.max}{   Selecting \code{strategy="adaptive"} will chose the default strategy for all fixed effects and model components with length less or equal to \code{adaptive.max}, for others, the gaussian strategy will be applied.}
\item{huge}{   Logical If TRUE then try to do some of the internal parallisations differently. Hopefully this will be of benefite for 'HUGE' models. (Default FALSE.) [THIS OPTION IS OBSOLETE AND NOT USED!]}
\item{step.len}{   Numerical The step-length used to compute numerical derivaties of the log-likelihood}
\item{stencil}{   Numerical Number of points in the stencil used to compute the numerical derivaties of the log-likelihood (3, 5, 7 or 9). (default 5)}
\item{lincomb.derived.correlation.matrix}{   Logical If TRUE compute also the correlations for the derived linear combinations, if FALSE do not (Default FALSE)}
\item{diagonal}{   Numerical Expert use only! Add a this value on the diagonal of the joint precision matrix. (default 0.0)}
\item{numint.maxfeval}{   Numerical Maximum number of function evaluations in the the numerical integration for the hyperparameters. (Default 100000.)}
\item{numint.relerr}{   Numerical Relative error requirement in the the numerical integration for the hyperparameters. (Default 1e-5)}
\item{numint.abserr}{   Numerical Absolute error requirement in the the numerical integration for the hyperparameters. (Default 1e-6)}
\item{cmin}{   Numerical The minimum value for the negative Hessian from the likelihood. Increasing this value will stabalise the optimisation but can introduce bias. (Default -Inf)}
\item{b.strategy}{   Character If \code{cmin} is used, either keep the linear term (with \code{b.strategy="keep"}) or skip the contribution by setting the linear term to zero (\code{b.strategy="skip"}). The default value is \code{"skip"}}
\item{step.factor}{   Numerical The step factor in the Newton-Raphson algorithm saying how large step to take (Default 1.0)}
\item{global.node.factor}{   Numerical The factor which defines the degree     required (how many neighbors), as a fraction of n-1, that is required to be     classified as a global node and numbered last (whatever the reordering routine     says). Here, n, is the size of the graph. (Disabled if larger than 1.) (default     2.0)}
\item{global.node.degree}{   Numerical The degree required (number of neighbors) to be classified as a global node and numbered last (whatever the reordering routine says). (default \code{.Machine$integer.max})}
\item{stupid.search}{   Logical Enable or disable the stupid-search-algorithm, if the Hessian calculations reveals that the mode is not found. (Default \code{TRUE}.)}
\item{stupid.search.max.iter}{   Numerical Maximum number of iterations allowed for the stupid-search-algorithm. (default 1000)}
\item{stupid.search.factor}{   Numerical Factor (>=1) to increase the step-length with after each new interation. (default 1.05)}
\item{control.vb}{   List of arguments for various VB corrections. \code{enable}       Logical/Character Use this feature? If \code{"auto"} this will be selected   automatically. \code{strategy} Charactor What to correct, one of     "mean",   "more.to.come.later...". \code{verbose} Logical Be verbose or not.       \code{refinement} Integer Number of extra possible refinement iterations.     \code{max.correct} Numerical Bound the allowed correction     (\code{strategy="mean"}: \code{|diff.mean/stdev| < max.correct}).     \code{hyperpar.correct} Logical Correct the marginal posterior for the     hyperparameters? \code{f.enable.limit} The size limit to correct for a \code{f()}     model component.}
\item{num.gradient}{   Character Set the numerical scheme to compute the gradient, one of \code{"forward"} or \code{"central"} (default).}
\item{num.hessian}{   Character Set the numerical scheme to compute the Hessian, one of \code{"forward"} or \code{"central"} (default).}
\item{optimise.strategy}{   Character THIS OPTION IS EXPERIMENTAL. Chose the optimiser strategy, one of \code{"plain"} or \code{"smart"} (default)}
\item{use.directions}{   THIS OPTION IS EXPERIMENTAL. Unless \code{FALSE} or \code{NULL}, use directions for computing gradient and Hessian, initialised with \code{use.directions} if a matrix.}
\item{constr.marginal.diagonal}{   Add stability to \code{AQ^-1A^T} by adding a small diagonal term. (default \code{epsilon^0.5})}
\item{improved.simplified.laplace}{   If \code{TRUE} use an experimental improved variant, otherwise, use the standard one.}
\item{fast}{   If \code{stage1only} then try to do all calculations during stage1 by   enabling Gaussian strategy with VB correction enabled.}
}
\value{
  
The function \code{control.inla} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.inla.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R, R/spde1.R, R/spde2.R
\name{inla.spde.models}
\alias{inla.spde.models}
\alias{inla.spde1.models}
\alias{inla.spde2.models}
\title{List SPDE models supported by inla.spde objects}
\usage{
inla.spde.models(function.names = FALSE)

inla.spde1.models()

inla.spde2.models()
}
\arguments{
\item{function.names}{If \code{FALSE}, return list model name lists.  If
\code{TRUE}, return list of model object constructor function names.}
}
\value{
List of available SPDE model type name lists.
}
\description{
List SPDE models supported by inla.spde objects
}
\details{
Returns a list of available SPDE model type name lists, one for each
inla.spde model class (currently \code{\link[=inla.spde1]{inla.spde1()}} and
\code{\link[=inla.spde2]{inla.spde2()}}).
}
\examples{

## Display help for each supported inla.spde2 model:
for (model in inla.spde2.models()) {
      print(help(paste("inla.spde2.", model, sep = "")))
  }

## Display help for each supported inla.spde* model:
models <- inla.spde.models()
for (type in names(models)) {
      for (model in models[[type]]) {
            print(help(paste("inla.", type, ".", model, sep = "")))
        }
  }

## Display help for each supported inla.spde* model (equivalent to above):
for (model in inla.spde.models(function.names = TRUE)) {
      print(help(model))
  }
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Oral}
\alias{Oral}
\title{~~ data name/kind ... ~~}
\format{
A data frame with 544 observations on the following 3 variables.
\describe{ \item{region}{a numeric vector} \item{E}{a
numeric vector} \item{Y}{a numeric vector} }
}
\description{
~~ A concise (1-5 lines) description of the dataset. ~~
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.map.lim}
\alias{inla.mesh.map.lim}
\alias{inla.mesh.map}
\title{Coordinate mappings for \code{inla.mesh} projections.}
\usage{
inla.mesh.map.lim(
  loc = NULL,
  projection = c("default", "longlat", "longsinlat", "mollweide")
)

inla.mesh.map(
  loc,
  projection = c("default", "longlat", "longsinlat", "mollweide"),
  inverse = TRUE
)
}
\arguments{
\item{loc}{Coordinates to be mapped.}

\item{projection}{The projection type.}

\item{inverse}{If \code{TRUE}, \code{loc} are map coordinates and
coordinates in the mesh domain are calculated.  If \code{FALSE}, \code{loc}
are coordinates in the mesh domain and the forward map projection is
calculated.}
}
\value{
For \code{inla.mesh.map.lim}, a list: \item{xlim }{X axis limits in
the map domain} \item{ylim }{Y axis limits in the map domain} No attempt is
made to find minimal limits for partial spherical domains.
}
\description{
Calculates coordinate mappings for \code{inla.mesh} projections.
}
\section{Functions}{
\itemize{
\item \code{inla.mesh.map.lim}: Projection extent limit calculations
}}

\seealso{
\code{\link[=inla.mesh.project]{inla.mesh.project()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export-class.R
\name{inla-class}
\alias{inla-class}
\title{inla estimation object class}
\description{
The inla class is defined in the INLA package
}
\seealso{
\link[=inla]{inla}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from binary.install.R
%%
 \name{inla.binary.install}
 \alias{inla.binary.install}
 \alias{binary.install}
 \title{Install alternative binary builds}
 \description{
    Install alternative binary builds.
 }
 \usage{
    inla.binary.install(os = c("CentOS Linux-6", "CentOS Linux-7", "CentOS Linux-8",
                               "CentOS Stream-8", "Fedora-33", "Fedora-34", "Manjaro Linux",
                               "Ubuntu-16.04", "Ubuntu-18.04", "Ubuntu-20.04"), 
                        verbose = TRUE, md5.check = TRUE)
 }
 \arguments{
    \item{os}{If \code{os} is given, install binary build for this \code{os}.
              If \code{os} is not given, chose \code{os} interactively
              among available builds.}
    \item{verbose}{Logical. Verbose output if \code{TRUE}}
    \item{md5.check}{Logical. If \code{TRUE}, stop if md5-checksum-file is not present
                     or md5-checksum fail. If \code{FALSE}, ignore md5-checksum check.}
 }
 \details{
 Install a new binary for \code{os} unless
 \code{missing(os)}, for which the \code{os} is chosen
 interactively among the available builds.
 }
 \value{
 Return \code{TRUE} if installation was sucessful and \code{FALSE} if not.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
   \dontrun{
     inla.binary.install()
     inla.binary.install(os = "CentOS Linux-7")
   }
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.compute}
\alias{control.compute}
\alias{inla.control.compute}
\alias{inla.set.control.compute.default}
\alias{set.control.compute.default}
\alias{control.compute.default}
\title{Control variables in control.compute}
\description{Control variables in \code{control.compute} for use in \code{inla}}
\usage{
inla.set.control.compute.default(...)
control.compute(config, cpo, dic, graph, hyperpar, mlik, openmp.strategy, po, q, return.marginals, return.marginals.predictor, smtp, waic)
}
\arguments{
\item{...}{Possible arguments}
\item{openmp.strategy}{   The computational strategy to use: 'small', 'medium', 'large', 'huge', 'default' and 'pardiso'.}
\item{hyperpar}{   A boolean variable if the marginal for the hyperparameters should be computed. Default TRUE.}
\item{return.marginals}{   A boolean variable if the marginals for the latent field should be returned (although it is computed). Default TRUE}
\item{return.marginals.predictor}{   A boolean variable if the marginals for the linear predictor should be returned (although it is computed). Default FALSE}
\item{dic}{   A boolean variable if the DIC-value should be computed. Default FALSE.}
\item{mlik}{   A boolean variable if the marginal likelihood should be computed. Default \code{TRUE}.}
\item{cpo}{   A boolean variable if the cross-validated predictive measures (cpo, pit) should be computed (default \code{FALSE})}
\item{po}{   A boolean variable if the predictive ordinate should be computed (default \code{FALSE})}
\item{waic}{   A boolean variable if the Watanabe-Akaike information criteria should be computed (default \code{FALSE})}
\item{q}{   A boolean variable if binary images of the precision matrix, the reordered precision matrix and the Cholesky triangle should be generated. (Default FALSE.)}
\item{config}{   A boolean variable if the internal GMRF approximations be stored. (Default FALSE. EXPERIMENTAL)}
\item{smtp}{   The sparse-matrix solver, one of 'default', 'taucs', 'band' or 'pardiso' (default \code{inla.getoption("smtp")}). \code{smtp='pardiso'} implies \code{openmp.strategy='pardiso'}.}
\item{graph}{   A boolean variable if the graph itself should be returned. (Default FALSE.)}
}
\value{
  
The function \code{control.compute} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.compute.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-ar.R
%%
 \name{pc.ar}
 \alias{inla.pc.ar}
 \alias{inla.pc.ar.rpacf}
 \alias{ar.rpacf}
 \alias{inla.pc.ar.dpacf}
 \alias{ar.dpacf}

 \title{Utility functions for the PC prior for a an AR(p) model}

 \description{Functions to evaluate and sample from the
              PC prior for an AR(p) model}
 \usage{
    inla.pc.ar.rpacf(n=1, p, lambda = 1)
    inla.pc.ar.dpacf(pac, lambda = 1, log = TRUE)
 }
 \arguments{
   \item{p}{The order of the AR-model}
   \item{pac}{A vector of partial autocorrelation coefficients}
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter in the prior}
   \item{log}{Logical. Return the density in natural or log-scale.}
 }
 \details{
 }
 \value{%%
     \code{inla.pc.ar.rpac} generate samples from the prior, returning a matrix
     where each row is a sample of \code{theta}.
     \code{inla.pc.ar.dpac} evaluates the density of \code{pac}.
     Use \code{inla.ar.pacf2phi}, \code{inla.ar.phi2pacf},
     \code{inla.ar.pacf2acf} and \code{inla.ar.acf2pacf} to convert between various
    parameterisations.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Cancer}
\alias{Cancer}
\title{~~ data name/kind ... ~~}
\format{
A data frame with 6690 observations on the following 4 variables.
\describe{
\item{Y}{Number of cases}
\item{N}{a numeric vector}
\item{Age}{a numeric vector}
\item{region}{a numeric vector}
}
}
\description{
~~ A concise (1-5 lines) description of the dataset. ~~
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from link-functions.R
%%
 \name{link}
 \alias{link}
 \alias{inla.link}
 \alias{inla.link.log}
 \alias{inla.link.invlog}
 \alias{inla.link.neglog}
 \alias{inla.link.invneglog}
 \alias{inla.link.logit}
 \alias{inla.link.invlogit}
 \alias{inla.link.probit}
 \alias{inla.link.invprobit}
 \alias{inla.link.cloglog}
 \alias{inla.link.invcloglog}
 \alias{inla.link.loglog}
 \alias{inla.link.invloglog}
 \alias{inla.link.tan}
 \alias{inla.link.invtan}
 \alias{inla.link.identity}
 \alias{inla.link.invidentity}
 \alias{inla.link.invalid}
 \alias{inla.link.invinvalid}
 \alias{inla.link.cauchit}
 \alias{inla.link.invcauchit}
 \alias{inla.link.inverse}
 \alias{inla.link.invinverse}
 \alias{inla.link.robit}
 \alias{inla.link.invrobit}
 \alias{inla.link.sn}
 \alias{inla.link.invsn}
 \alias{inla.link.invqpoisson}

 \title{Link functions in INLA}

 \description{Define link-functions and its inverse}

 \usage{
 inla.link.log(x, inverse=FALSE)
 inla.link.invlog(x, inverse=FALSE)
 inla.link.neglog(x, inverse=FALSE)
 inla.link.invneglog(x, inverse=FALSE)
 inla.link.logit(x, inverse=FALSE)
 inla.link.invlogit(x, inverse=FALSE)
 inla.link.probit(x, inverse=FALSE)
 inla.link.invprobit(x, inverse=FALSE)
 inla.link.cloglog(x, inverse=FALSE)
 inla.link.invcloglog(x, inverse=FALSE)
 inla.link.loglog(x, inverse=FALSE)
 inla.link.invloglog(x, inverse=FALSE)
 inla.link.tan(x, inverse=FALSE)
 inla.link.invtan(x, inverse=FALSE)
 inla.link.cauchit(x, inverse=FALSE)
 inla.link.invcauchit(x, inverse=FALSE)
 inla.link.identity(x, inverse=FALSE)
 inla.link.invidentity(x, inverse=FALSE)
 inla.link.inverse(x, inverse=FALSE)
 inla.link.invinverse(x, inverse=FALSE)
 inla.link.robit(x, df=7, inverse=FALSE)
 inla.link.invrobit(x, df=7, inverse=FALSE)
 inla.link.sn(x, intercept=0.5, skew=0, a=0, inverse=FALSE)
 inla.link.invsn(x, intercept=0.5, skew=0, a=0, inverse=FALSE)
 inla.link.invalid(x, inverse=FALSE)
 inla.link.invinvalid(x, inverse=FALSE)
 inla.link.invqpoisson(x, inverse = FALSE, quantile = 0.5)
 }

 \arguments{
     \item{x}{The argument. A numeric vector.}
     \item{df}{The degrees of freedom for the Student-t}
     \item{inverse}{Logical. Use the link (\code{inverse=FALSE})
                    or its inverse (\code{inverse=TRUE})}
     \item{intercept}{The quantile level for the intercept in the Skew-Normal link}
     \item{skew}{The skewness in the Skew-Normal.
                 Not both of \code{skew} and \code{a} can be given.}
     \item{a}{The \code{a}-paraeter in the Skew-Normal.
              Not both of \code{skew} and \code{a} can be given.}
     \item{quantile}{The quantile level for quantile links}
}

 \value{Return the values of the link-function or its inverse.}
 \note{The \code{inv}-functions are redundant,  as
       \code{inla.link.invlog(x) = inla.link.log(x, inverse=TRUE)}
       and so on,  but they are simpler to use a arguments
       to other functions.}
 \author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from read.graph.R
%%
\name{read.graph}
\alias{read.graph}
\alias{write.graph}
\alias{inla.read.graph}
\alias{inla.write.graph}
\alias{inla.graph}
\alias{summary.inla.graph}
\alias{plot.inla.graph}
\alias{print.inla.graph.summary}
\title{Read and write a graph-object}
\description{Construct a graph-object from a file or a matrix; write graph-object to file}
\usage{
inla.read.graph(..., size.only = FALSE)
inla.write.graph(graph, filename = "graph.dat", mode = c("binary", "ascii"), ...)

\method{summary}{inla.graph}(object, ...)
\method{plot}{inla.graph}(x, y, ...)
\method{print}{inla.graph.summary}(x, ...)
}
\arguments{
    \item{filename}{The filename of the graph.}
    \item{graph}{An \code{inla.graph}-object, a (sparse) symmetric matrix, a filename containing the graph,
                 a list or collection of characters and/or numbers defining the graph,
                 or a neighbours list with class \code{nb} (see \code{spdep::card} and
                 \code{spdep::poly2nb} for for details of \code{nb} and an example a function
                 returning an \code{nb} object}
    \item{mode}{The mode of the file; ascii-file or a (gzip-compressed) binary.}
    \item{object}{An \code{inla.graph} -object}
    \item{x}{An \code{inla.graph} -object}
    \item{y}{Not used}
    \item{size.only}{Only read the size of the graph}
    \item{...}{Additional arguments. In \code{inla.read.graph},
               then it is the graph definition (object, matrix, character, filename),  plus extra arguments.
               In \code{inla.write.graph} it is extra arguments to \code{inla.read.graph}.}
}
\value{
    The output of \code{inla.read.graph}, is an \code{inla.graph} object, with elements
    \item{n}{is the size of the graph}
    \item{nnbs}{is a vector with the number of neigbours}
    \item{nbs}{is a list-list with the neigbours}
    \item{cc}{list with connected component information
        \itemize{
            \item{\code{id}}{is a vector with the connected component id for each node (starting from 1)}
            \item{\code{n}}{is the number of connected components}
            \item{\code{nodes}}{is a list-list of nodes belonging to each connected component}
        }
    }
    Methods implemented for \code{inla.graph} are \code{summary} and \code{plot}.
    The method \code{plot} require the libraries \code{Rgraphviz} and \code{graph} from the Bioconductor-project,
    see \url{https://www.bioconductor.org}.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{
    \code{\link{inla.spy}}
}
\examples{
## a graph from a file
cat("3 1 1 2 2 1 1 3 0\n", file="g.dat")
g = inla.read.graph("g.dat")
## writing an inla.graph-object to file
g.file = inla.write.graph(g, mode="binary")
## re-reading it from that file
gg = inla.read.graph(g.file)
summary(g)
##
Not run:
plot(g)
inla.spy(g)
## when defining the graph directly in the call,
## we can use a mix of character and numbers
g = inla.read.graph(c(3, 1, "1 2 2 1 1 3", 0))
inla.spy(c(3, 1, "1 2 2 1 1 3 0"))
inla.spy(c(3, 1, "1 2 2 1 1 3 0"),  reordering=3:1)
inla.write.graph(c(3, 1, "1 2 2 1 1 3 0"))

## building a graph from adjacency matrix
adjacent = matrix(0, nrow = 4, ncol = 4)
adjacent[1,4] = adjacent[4,1] = 1
adjacent[2,4] = adjacent[4,2] = 1
adjacent[2,3] = adjacent[3,2] = 1
adjacent[3,4] = adjacent[4,3] = 1
g = inla.read.graph(adjacent)
plot(g)
summary(g)
End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde2.R
\name{inla.spde2.pcmatern}
\alias{inla.spde2.pcmatern}
\title{Matern SPDE model object with PC prior for INLA}
\usage{
inla.spde2.pcmatern(
  mesh,
  alpha = 2,
  param = NULL,
  constr = FALSE,
  extraconstr.int = NULL,
  extraconstr = NULL,
  fractional.method = c("parsimonious", "null"),
  n.iid.group = 1,
  prior.range = NULL,
  prior.sigma = NULL
)
}
\arguments{
\item{mesh}{The mesh to build the model on, as an \code{\link[=inla.mesh]{inla.mesh()}} or
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object.}

\item{alpha}{Fractional operator order, \eqn{0<\alpha\leq 2}{0 < alpha <= 2}
supported, for \eqn{\nu=\alpha-d/2>0}.}

\item{param}{Further model parameters. Not currently used.}

\item{constr}{If \code{TRUE}, apply an integrate-to-zero constraint.
Default \code{FALSE}.}

\item{extraconstr.int}{Field integral constraints.}

\item{extraconstr}{Direct linear combination constraints on the basis
weights.}

\item{fractional.method}{Specifies the approximation method to use for
fractional (non-integer) \code{alpha} values. \code{'parsimonious'} gives an
overall approximate minimal covariance error, \code{'null'} uses
approximates low-order properties.}

\item{n.iid.group}{If greater than 1, build an explicitly iid replicated
model, to support constraints applied to the combined replicates, for
example in a time-replicated spatial model. Constraints can either be
specified for a single mesh, in which case it's applied to the average of
the replicates (\code{ncol(A)} should be \code{mesh$n} for 2D meshes,
\code{mesh$m} for 1D), or as general constraints on the collection of
replicates (\code{ncol(A)} should be \code{mesh$n * n.iid.group} for 2D
meshes, \code{mesh$m * n.iid.group} for 1D).}

\item{prior.range}{A length 2 vector, with \verb{(range0,Prange)} specifying
that \eqn{P(\rho < \rho_0)=p_\rho}, where \eqn{\rho} is the spatial range of
the random field. If \code{Prange} is \code{NA}, then \code{range0} is used
as a fixed range value.}

\item{prior.sigma}{A length 2 vector, with \verb{(sigma0,Psigma)} specifying
that \eqn{P(\sigma > \sigma_0)=p_\sigma}, where \eqn{\sigma} is the marginal
standard deviation of the field.  If \code{Psigma} is \code{NA}, then
\code{sigma0} is used as a fixed range value.}
}
\value{
An \code{inla.spde2} object.
}
\description{
Create an \code{inla.spde2} model object for a Matern model, using a PC
prior for the parameters.
}
\details{
This method constructs a Matern SPDE model, with spatial range \eqn{\rho}
and standard deviation parameter \eqn{\sigma}.  In the parameterisation

\deqn{(\kappa^2-\Delta)^{\alpha/2}(\tau }{(kappa^2-Delta)^(alpha/2) (tau
x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2-Delta)^(alpha/2) (tau x(u)) = W(u)}

the spatial scale parameter \eqn{\kappa=\sqrt{8\nu}/\rho}, where
\eqn{\nu=\alpha-d/2}, and \eqn{\tau} is proportional to \eqn{1/\sigma}.

Stationary models are supported for \eqn{0 < \alpha \leq 2}{0 < alpha <= 2},
with spectral approximation methods used for non-integer \eqn{\alpha}, with
approximation method determined by \code{fractional.method}.

Integration and other general linear constraints are supported via the
\code{constr}, \code{extraconstr.int}, and \code{extraconstr} parameters,
which also interact with \code{n.iid.group}.

The joint PC prior density for the spatial range, \eqn{\rho}, and the
marginal standard deviation, \eqn{\sigma}, and is \deqn{ }{p(rho, sigma) =
(d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S sigma) }\deqn{ \pi(\rho,
\sigma) = }{p(rho, sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S
sigma) }\deqn{ \frac{d \lambda_\rho}{2} \rho^{-1-d/2} \exp(-\lambda_\rho
\rho^{-d/2}) }{p(rho, sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S
exp(-S sigma) }\deqn{ \lambda_\sigma\exp(-\lambda_\sigma \sigma) }{p(rho,
sigma) = (d R)/2 rho^(-1-d/2) exp(-R rho^(-d/2)) S exp(-S sigma) } where
\eqn{\lambda_\rho}{R} and \eqn{\lambda_\sigma}{S} are hyperparameters that
must be determined by the analyst. The practical approach for this in INLA
is to require the user to indirectly specify these hyperparameters through
\deqn{P(\rho < \rho_0) = p_\rho} and \deqn{P(\sigma > \sigma_0) = p_\sigma}
where the user specifies the lower tail quantile and probability for the
range (\eqn{\rho_0} and \eqn{p_\rho}) and the upper tail quantile and
probability for the standard deviation (\eqn{\sigma_0} and
\eqn{\alpha_\sigma}).

This allows the user to control the priors of the parameters by supplying
knowledge of the scale of the problem. What is a reasonable upper magnitude
for the spatial effect and what is a reasonable lower scale at which the
spatial effect can operate? The shape of the prior was derived through a
construction that shrinks the spatial effect towards a base model of no
spatial effect in the sense of distance measured by Kullback-Leibler
divergence.

The prior is constructed in two steps, under the idea that having a spatial
field is an extension of not having a spatial field. First, a spatially
constant random effect (\eqn{\rho = \infty}) with finite variance is more
complex than not having a random effect (\eqn{\sigma = 0}). Second, a
spatial field with spatial variation (\eqn{\rho < \infty}) is more complex
than the random effect with no spatial variation. Each of these extensions
are shrunk towards the simpler model and, as a result, we shrink the spatial
field towards the base model of no spatial variation and zero variance
(\eqn{\rho = \infty} and \eqn{\sigma = 0}).

The details behind the construction of the prior is presented in Fuglstad,
et al. (2016) and is based on the PC prior framework (Simpson, et al.,
2015).
}
\examples{

## Spatial interpolation
  n = 100
  field.fcn = function(loc) (10*cos(2*pi*2*(loc[,1]+loc[,2])))
  loc = matrix(runif(n*2),n,2)
  ## One field, 2 observations per location
  idx.y = rep(1:n,2)
  y = field.fcn(loc[idx.y,]) + rnorm(length(idx.y))

  mesh = inla.mesh.2d(loc, max.edge=0.05, cutoff=0.01)
  spde = inla.spde2.pcmatern(mesh,
           prior.range=c(0.01,0.1), prior.sigma=c(100,0.1))
  data = list(y=y, field=mesh$idx$loc[idx.y])
  formula = y ~ -1 + f(field, model=spde)
  result = inla(formula, data=data, family="normal")

  ## Plot the mesh structure:
  plot(mesh)
  \donttest{
  if (require(rgl)) {
    col.pal = colorRampPalette(c("blue","cyan","green","yellow","red"))
    ## Plot the posterior mean:
    plot(mesh, rgl=TRUE,
         result$summary.random$field[,"mean"],
         color.palette = col.pal)
    ## Plot residual field:
    plot(mesh, rgl=TRUE,
         result$summary.random$field[,"mean"]-field.fcn(mesh$loc),
         color.palette = col.pal)
  }
  }

  result.field = inla.spde.result(result, "field", spde)
  par(mfrow=c(2,1))
  plot(result.field$marginals.range.nominal[[1]],
       type="l", main="Posterior density for range")
  plot(inla.tmarginal(sqrt, result.field$marginals.variance.nominal[[1]]),
       type="l", main="Posterior density for std.dev.")
  par(mfrow=c(1,1))

## Spatial model
  set.seed(1234234)
  
  ## Generate spatial locations
  nObs = 200
  loc = matrix(runif(nObs*2), nrow = nObs, ncol = 2)

  ## Generate observation of spatial field
  nu = 1.0
  rhoT = 0.2
  kappaT = sqrt(8*nu)/rhoT
  sigT = 1.0
  Sig = sigT^2*inla.matern.cov(nu = nu,
                               kappa = kappaT,
                               x = as.matrix(dist(loc)),
                               d = 2,
                               corr = TRUE)
  L = t(chol(Sig))
  u = L \%*\% rnorm(nObs)

  ## Construct observation with nugget
  sigN = 0.1
  y = u + sigN*rnorm(nObs)

  ## Create the mesh and spde object
  mesh = inla.mesh.2d(loc,
                      max.edge = 0.05,
                      cutoff = 0.01)
  spde = inla.spde2.pcmatern(mesh,
                             prior.range = c(0.01, 0.05),
                             prior.sigma = c(10, 0.05))

  ## Create projection matrix for observations
  A = inla.spde.make.A(mesh = mesh,
                       loc = loc)

  ## Run model without any covariates
  idx = 1:spde$n.spde
  res = inla(y ~ f(idx, model = spde) - 1,
             data = list(y = y, idx = idx, spde = spde),
             control.predictor = list(A = A))

## Re-run model with fixed range
  spde.fixed = inla.spde2.pcmatern(mesh,
                                   prior.range = c(0.2, NA),
                                   prior.sigma = c(10, 0.05))

  res.fixed = inla(y ~ f(idx, model = spde) - 1,
                   data = list(y = y, idx = idx, spde = spde.fixed),
                   control.predictor = list(A = A))


}
\references{
Fuglstad, G.-A., Simpson, D., Lindgren, F., and Rue, H. (2016)
Constructing Priors that Penalize the Complexity of Gaussian Random Fields.
arXiv:1503.00256

Simpson, D., Rue, H., Martins, T., Riebler, A., and Sørbye, S. (2015)
Penalising model component complexity: A principled, practical approach to
constructing priors. arXiv:1403.4630
}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}, \code{\link[=inla.mesh.create]{inla.mesh.create()}},
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}}, \code{\link[=inla.mesh.basis]{inla.mesh.basis()}},
\code{\link[=inla.spde2.matern]{inla.spde2.matern()}}, \code{\link[=inla.spde2.generic]{inla.spde2.generic()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{SurvSim}
\alias{SurvSim}
\title{Survival data}
\format{
A data frame with 100 observations on the following 3 variables.
\describe{ \item{y}{a numeric vector of survival times}
\item{cens}{a numeric vector of event indicator (0=censured
1=failure)} \item{x}{a numeric vector of covariate } }
}
\description{
Simulated data set for Weibull survival model
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{as.inla.mesh.segment}
\alias{as.inla.mesh.segment}
\alias{as.inla.mesh.segment.Line}
\alias{as.inla.mesh.segment.Lines}
\alias{as.inla.mesh.segment.SpatialLines}
\alias{as.inla.mesh.segment.SpatialLinesDataFrame}
\alias{as.inla.mesh.segment.Polygon}
\alias{as.inla.mesh.segment.Polygons}
\alias{as.inla.mesh.segment.SpatialPolygons}
\alias{as.inla.mesh.segment.SpatialPolygonsDataFrame}
\alias{inla.sp2segment}
\alias{as.inla.mesh.segment.SpatialPoints}
\alias{as.inla.mesh.segment.SpatialPointsDataFrame}
\title{Convert \code{sp} curve objects to \code{inla.mesh.segment} objects.}
\usage{
as.inla.mesh.segment(sp, ...)

inla.sp2segment(sp, ...)

\method{as.inla.mesh.segment}{SpatialPoints}(sp, reverse = FALSE, grp = NULL, is.bnd = TRUE, ...)

\method{as.inla.mesh.segment}{SpatialPointsDataFrame}(sp, ...)

\method{as.inla.mesh.segment}{Line}(sp, reverse = FALSE, crs = NULL, ...)

\method{as.inla.mesh.segment}{Lines}(sp, join = TRUE, crs = NULL, ...)

\method{as.inla.mesh.segment}{SpatialLines}(sp, join = TRUE, grp = NULL, ...)

\method{as.inla.mesh.segment}{SpatialLinesDataFrame}(sp, ...)

\method{as.inla.mesh.segment}{SpatialPolygons}(sp, join = TRUE, grp = NULL, ...)

\method{as.inla.mesh.segment}{SpatialPolygonsDataFrame}(sp, ...)

\method{as.inla.mesh.segment}{Polygons}(sp, join = TRUE, crs = NULL, ...)

\method{as.inla.mesh.segment}{Polygon}(sp, crs = NULL, ...)
}
\arguments{
\item{sp}{An \code{sp} polygon object of class \code{Polygon},
\code{Polygons}, \code{SpatialPolygons}, or \code{SpatialPolygonsDataFrame}.}

\item{\dots}{Additional arguments passed on to other methods.}

\item{reverse}{Logical, indicating if the line sequence should be traversed
backwards.}

\item{grp}{Group ID specification for each polygon, as used by
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}, one ID per polygon.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}

\item{join}{If \code{TRUE}, join multiple polygons into a single segment
(possibly non-simply connected).}
}
\value{
A \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object, or a list of
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}} objects.
}
\description{
Convert \code{sp} curve objects to \code{inla.mesh.segment} objects.
}
\seealso{
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.spde.sample}
\alias{inla.spde.sample}
\alias{inla.spde.sample.default}
\alias{inla.spde.sample.inla.spde}
\title{Sample from SPDE models}
\usage{
inla.spde.sample(...)

\method{inla.spde.sample}{default}(precision, seed = NULL, ...)

\method{inla.spde.sample}{inla.spde}(spde, seed = NULL, ...)
}
\arguments{
\item{\dots}{Parameters passed on to other methods.}

\item{precision}{A precision matrix.}

\item{seed}{The seed for the pseudo-random generator.}

\item{spde}{An \code{inla.spde} object.}
}
\description{
Old methods fo sampling from a SPDE model.  For new code, use
\code{\link[=inla.spde.precision]{inla.spde.precision()}} and \code{\link[=inla.qsample]{inla.qsample()}} instead.
}
\seealso{
\code{\link[=inla.spde.precision]{inla.spde.precision()}}, \code{\link[=inla.qsample]{inla.qsample()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.has_PROJ6}
\alias{inla.has_PROJ6}
\alias{inla.not_for_PROJ6}
\alias{inla.not_for_PROJ4}
\alias{inla.fallback_PROJ6}
\alias{inla.requires_PROJ6}
\title{PROJ6 detection}
\usage{
inla.has_PROJ6()

inla.not_for_PROJ6(fun)

inla.not_for_PROJ4(fun)

inla.fallback_PROJ6(fun)

inla.requires_PROJ6(fun)
}
\arguments{
\item{fun}{The name of the calling function}
}
\value{
For \code{inla.has_PROJ6}, logical; \code{TRUE} if PROJ6 is available,
\code{FALSE} otherwise
}
\description{
Detect whether PROJ6 is available for INLA
}
\details{
\code{inla.has_PROJ6} is called to check if PROJ6&GDAL3 are available.
}
\section{Functions}{
\itemize{
\item \code{inla.not_for_PROJ6}: Called to warn about using old PROJ4
features even though PROJ6 is available

\item \code{inla.not_for_PROJ4}: Called to give an error when
calling methods that are only available for PROJ6

\item \code{inla.fallback_PROJ6}: Called to warn about falling back
to using old PROJ4 methods when a RPOJ6 method hasn't been implemented

\item \code{inla.requires_PROJ6}: Called to give an error when PROJ6
is required but not available
}}

\examples{

inla.has_PROJ6()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.CRS}
\alias{inla.CRS}
\alias{inla.wkt_predef}
\title{Create a coordinate reference system object}
\usage{
inla.CRS(
  projargs = NULL,
  doCheckCRSArgs = TRUE,
  args = NULL,
  oblique = NULL,
  SRS_string = NULL,
  ...
)

inla.wkt_predef()
}
\arguments{
\item{projargs}{Either 1) a projection argument string suitable as input to
\code{sp::CRS}, or 2) an existing \code{CRS} object, or 3) a shortcut
reference string to a predefined projection; run
\code{names(inla.wkt_predef())} for valid predefined projections.}

\item{doCheckCRSArgs}{default TRUE, must be set to FALSE by package
developers including \code{CRS} in an S4 class definition to avoid
uncontrollable loading of the \code{rgdal} namespace.}

\item{args}{An optional list of name/value pairs to add to and/or override
the PROJ4 arguments in \code{projargs}.  \code{name=value} is converted to
\code{"+name=value"}, and \code{name=NA} is converted to \code{"+name"}.}

\item{oblique}{Vector of length at most 4 of rotation angles (in degrees)
for an oblique projection, all values defaulting to zero. The values
indicate (longitude, latitude, orientation, orbit), as explained in the
Details section below.}

\item{SRS_string}{a WKT2 string defining the coordinate system; see
\code{sp::CRS}. This takes precedence over \code{projargs}.}

\item{\dots}{Additional parameters. Not currently in use.}
}
\value{
Either an \code{sp::CRS} object or an \code{inla.CRS} object,
depending on if the coordinate reference system described by the parameters
can be expressed with a pure \code{sp::CRS} object or not.

An S3 \code{inla.CRS} object is a list, usually (but not necessarily)
containing at least one element: \item{crs }{The basic \code{sp::CRS}
object}

\code{inla.wkt_predef} returns a WKT2 string defining a projection

\code{inla.wkt_predef} returns a WKT2 string defining a projection
}
\description{
Creates either a CRS object or an inla.CRS object, describing a coordinate
reference system
}
\details{
The first two elements of the \code{oblique} vector are the (longitude,
latitude) coordinates for the oblique centre point. The third value
(orientation) is a counterclockwise rotation angle for an observer looking
at the centre point from outside the sphere. The fourth value is the
quasi-longitude (orbit angle) for a rotation along the oblique observers
equator.

Simple oblique: \code{oblique=c(0, 45)}

Polar: \code{oblique=c(0, 90)}

Quasi-transversal: \code{oblique=c(0, 0, 90)}

Satellite orbit viewpoint: \code{oblique=c(lon0-time*v1, 0, orbitangle, orbit0+time*v2)}, where \code{lon0} is the longitude at which a satellite
orbit crosses the equator at \code{time=0}, when the satellite is at an
angle \code{orbit0} further along in its orbit.  The orbital angle relative
to the equatorial plane is \code{orbitangle}, and \code{v1} and \code{v2}
are the angular velocities of the planet and the satellite, respectively.
Note that "forward" from the satellite's point of view is "to the right" in
the projection.

When \code{oblique[2]} or \code{oblique[3]} are non-zero, the resulting
projection is only correct for perfect spheres.
}
\examples{


if (require(rgdal)) {
    crs1 <- inla.CRS("longlat_globe")
    crs2 <- inla.CRS("lambert_globe")
    crs3 <- inla.CRS("mollweide_norm")
    crs4 <- inla.CRS("hammer_globe")
    crs5 <- inla.CRS("sphere")
    crs6 <- inla.CRS("globe")
}
\dontrun{
names(inla.wkt_predef())
}

\dontrun{
names(inla.wkt_predef())
}
}
\seealso{
\code{\link[sp:CRS-class]{sp::CRS()}}, \code{\link[=crs_wkt]{crs_wkt()}},
\code{\link[=inla.sp_get_crs]{inla.sp_get_crs()}} \code{\link[=plot.CRS]{plot.CRS()}},
\code{\link[=inla.identical.CRS]{inla.identical.CRS()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.family}
\alias{control.family}
\alias{inla.control.family}
\alias{inla.set.control.family.default}
\alias{set.control.family.default}
\alias{control.family.default}
\title{Control variables in control.family}
\description{Control variables in \code{control.family} for use in \code{inla}}
\usage{
inla.set.control.family.default(...)
control.family(beta.censor.value, cenpoisson.I, control.bgev, control.link, control.mix, dummy, fixed, gev.scale.xi, hyper, initial, link, param, prior, sn.shape.max, variant)
}
\arguments{
\item{...}{Possible arguments}
\item{dummy}{   A dummy argument that can be used as a workaround}
\item{hyper}{   Definition of the hyperparameters}
\item{initial}{   (OBSOLETE!) Initial value for the hyperparameter(s) of the likelihood in the internal scale.}
\item{prior}{   (OBSOLETE!) The name of the prior distribution(s) for othe hyperparameter(s).}
\item{param}{   (OBSOLETE!) The parameters for the prior distribution}
\item{fixed}{   (OBSOLETE!) Boolean variable(s) to say if the hyperparameter(s) is fixed or random.}
\item{link}{   (OBSOLETE! Use \code{control.link=list(model=)} instead.) The link function to use.}
\item{sn.shape.max}{   Maximum value for the shape-parameter for Skew Normal observations (default 5.0)}
\item{gev.scale.xi}{   (Expert option, do not use unless you know what you are doing.) The internal scaling of the shape-parameter for the GEV distribution. (default 0.1)}
\item{control.bgev}{   See \code{?control.bgev}}
\item{cenpoisson.I}{   The censoring interval for the censored Poisson}
\item{beta.censor.value}{   The censor value for the Beta-likelihood \code{(0 <= beta.censor.value < 1/2)}}
\item{variant}{   This variable is used to give options for various variants of the likelihood, like chosing different parameterisations for example. See the relevant likelihood documentations for options (does only apply to some likelihoods).}
\item{control.mix}{   See \code{?control.mix}}
\item{control.link}{   See \code{?control.link}}
}
\value{
  
The function \code{control.family} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.family.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from nmix.lambda.fitted.R
%%
\name{inla.nmix.lambda.fitted}
\alias{inla.nmix.lambda.fitted}
\alias{nmix.lambda.fitted}
\alias{inla.nmix.fitted}

\title{
  Estimate posterior distributions of fitted lambda values
}

\description{For use with \code{'nmix'} and \code{'nmixnb'} models. This function takes the
    information contained in an object returned by \code{inla()} and uses the contents to create
    fitted lambda values using the linear predictor for log(lambda), the input covariate values,
    and samples from the posteriors of the model hyperparameters. Fitted values from the linear
    predictor are exponentiated, by default, before being returned.}

\usage{
inla.nmix.lambda.fitted(result, sample.size = 1000,
                        return.posteriors = FALSE, scale = "exp")
}

\arguments{
    \item{result}{The output object from a call to \code{inla()}, where the family argument has
  been set to \code{'nmix'} or \code{'nmixnb'}. For the function to work, the call to
  \code{inla()} should also include the argument \code{control.compute=list(config = TRUE))}.}

  \item{sample.size}{The size of the sample from the posteriors of the model hyperparameters.
  This sample size ends up being the size of the estimated posterior for a fitted lambda value.
  Default is 1000. Larger values are recommended.}

  \item{return.posterior}{A logical value for whether or not to return the full estimated
  posteriors for each fitted value (\code{TRUE}), or just a summary of the posteriors
  (\code{FALSE}). Default is \code{FALSE}.}

  \item{scale}{A character string, where the default string, \code{"exp"}, causes values from
  the linear predictor to be exponentiated before being returned. The string, \code{"log"},
  causes values to be returned on the \code{log(lambda)} scale.}
}

\value{
    \item{fitted.summary}{A data frame with summaries of estimated posteriors of fitted lambda
  values. The number of rows equals the number of rows in the data used to create the
  \code{'nmix'} or \code{'nmixnb'} model. There are six columns of summary statistics for each
  estimated posterior. Columns include an \code{index}, \code{mean.lambda}, \code{sd.lambda},
  \code{quant025.lambda}, \code{median.lambda}, \code{quant975.lambda}, and \code{mode.lambda}.}

  \item{fitted.posteriors}{A data frame containing samples that comprise the full estimated
  posteriors of fitted values. The number of rows equals the number of rows in the data used to
  create the \code{'nmix'} or \code{'nmixnb'} model. The number of columns equals one plus the
  number of samples specified by the \code{sample.size} argument.}
}

\references{
    See documentation for families "nmix" and "nmixmb": \code{inla.doc("nmix")}
}
\author{
    Tim Meehan <tmeehan@audubon.org>
}
\note{
    This function is experimental.
}

\examples{
## an example analysis of an N-mixture model using simulated data
## set parameters
n <- 75                       # number of study sites
nrep.max <- 5                 # number of surveys per site
b0 <- 0.5                     # lambda intercept, expected abundance
b1 <- 2.0                     # effect of x1 on lambda
a0 <- 1.0                     # p intercept, detection probability
a2 <- 0.5                     # effect of x2 on p
size <- 3.0                   # size of theta
overdispersion <- 1 / size    # for negative binomial distribution

## make empty vectors and matrix
x1 <- c(); x2 <- c()
lambdas <- c(); Ns <- c()
y <- matrix(NA, n, nrep.max)

## fill vectors and matrix
for(i in 1:n) {
    x1.i <- runif(1) - 0.5
    lambda <- exp(b0 + b1 * x1.i)
    N <- rnbinom(1, mu = lambda, size = size)
    x2.i <- runif(1) - 0.5
    eta <- a0 + a2 * x2.i
    p <- exp(eta) / (exp(eta) + 1)
    nr <- sample(1:nrep.max, 1)
    y[i, 1:nr] <- rbinom(nr, size = N, prob = p)
    x1 <- c(x1, x1.i); x2 <- c(x2, x2.i)
    lambdas <- c(lambdas, lambda); Ns <- c(Ns, N)
}

## bundle counts, lambda intercept, and lambda covariates
Y <- inla.mdata(y, 1, x1)

## run inla and summarize output
result <- inla(Y ~ 1 + x2,
  data = list(Y=Y, x2=x2),
  family = "nmixnb",
  control.fixed = list(mean = 0, mean.intercept = 0, prec = 0.01,
                      prec.intercept = 0.01),
  control.family = list(hyper = list(theta1 = list(param = c(0, 0.01)),
                                    theta2 = list(param = c(0, 0.01)),
                                    theta3 = list(prior = "flat",
                                                 param = numeric()))),
  control.compute=list(config = TRUE)) # important argument
summary(result)

## get and evaluate fitted values
lam.fits <- inla.nmix.lambda.fitted(result, 5000)$fitted.summary
plot(lam.fits$median.lambda, lambdas)
round(sum(lam.fits$median.lambda), 0); sum(Ns)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.fmesher.smorg}
\alias{inla.fmesher.smorg}
\title{Compute various mesh related quantities.}
\usage{
inla.fmesher.smorg(
  loc,
  tv,
  fem = NULL,
  aniso = NULL,
  gradients = FALSE,
  sph0 = NULL,
  sph = NULL,
  bspline = NULL,
  points2mesh = NULL,
  splitlines = NULL,
  output = NULL,
  keep = FALSE
)
}
\arguments{
\item{loc}{3-column triangle vertex coordinate matrix.}

\item{tv}{3-column triangle vertex index matrix.}

\item{fem}{Maximum finite element matrix order to be computed.}

\item{aniso}{A two-element list with \eqn{\gamma}{gamma} and \eqn{v}{v} for
an anisotropic operator \eqn{\nabla\cdot H \nabla}{div H grad}, where
\eqn{H=\gamma I + v v^\top}{H = gamma I + v v'}}

\item{gradients}{When \code{TRUE}, calculate derivative operator matrices
\code{dx}, dy, and dz.}

\item{sph0}{Maximal order of rotationally invariant spherical harmonics.}

\item{sph}{Maximal order of general spherical harmonics.}

\item{bspline}{Rotationally invariant B-splines on a sphere.  3-vector with
number of basis functions \code{n}, basis degree \code{degree}, and a
logical; \code{TRUE} uniform knot angles, \code{FALSE} for uniform spacing
in \eqn{\sin(latitude)}{sin(latitude)}.}

\item{points2mesh}{3-column matrix with points to be located in the mesh.}

\item{splitlines}{A list with elements \code{loc} (3-column coordinate
matrix) and \code{idx} (2-column index matrix) describing line segments that
are to be split into sub-segments at triangle boundaries.}

\item{output}{Names of objects to be included in the output, if different
from defaults.}

\item{keep}{When \code{TRUE}, for debugging purposes keep the \code{fmesher}
I/O files on disk.}
}
\value{
A list of generated named quantities.
}
\description{
Low level function for computing finite element matrices, spherical
harmonics, B-splines, and point mappings with barycentric triangle
coordinates.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from qreordering.R
%%
 \name{qreordering}
 \alias{inla.qreordering}
 \alias{qreordering}

 \title{Compute the reordering using the GMRFLib implementation}

 \description{This function compute the reordering (or find the best reordering)
              using the GMRFLib implementation}
 \usage{
     inla.qreordering(graph, reordering = inla.reorderings())
 }

 \arguments{
   \item{graph}{A \code{(inla-)graph} object}
   \item{reordering}{The name of the reordering algorithm to be used; either
                     one of the names listed in \code{inla.reorderings()}.
                     The default is "auto" which try several reordering algorithm
                     and use the best one for this particular matrix.}
}
\value{
  \code{inla.qreordering} returns a list with the name of the reordering algorithm used or
  found, the reordering code for the reordering algorithm,
  the actual reordering and its inverse.
}
\author{Havard Rue \email{hrue@r-inla.org}}

\examples{
 g = system.file("demodata/germany.graph", package="INLA")
 r = inla.qreordering(g)
 m = inla.graph2matrix(g)
 r = inla.qreordering(m)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.segment}
\alias{inla.mesh.segment}
\alias{inla.mesh.segment.default}
\alias{inla.mesh.segment.inla.mesh.segment}
\alias{inla.contour.segment}
\title{Constraint segments for inla.mesh

Constructs \code{inla.mesh.segment} objects that can be used to specify
boundary and interior constraint edges in calls to \code{\link[=inla.mesh]{inla.mesh()}}.}
\usage{
inla.mesh.segment(...)

\method{inla.mesh.segment}{default}(
  loc = NULL,
  idx = NULL,
  grp = NULL,
  is.bnd = TRUE,
  crs = NULL,
  ...
)

\method{inla.mesh.segment}{inla.mesh.segment}(..., grp.default = 0)

inla.contour.segment(
  x = seq(0, 1, length.out = nrow(z)),
  y = seq(0, 1, length.out = ncol(z)),
  z,
  nlevels = 10,
  levels = pretty(range(z, na.rm = TRUE), nlevels),
  groups = seq_len(length(levels)),
  positive = TRUE,
  eps = NULL,
  crs = NULL
)
}
\arguments{
\item{...}{Additional parameters.  When joining segments, a list of
\code{inla.mesh.segment} objects.}

\item{loc}{Matrix of point locations.}

\item{idx}{Segment index sequence vector or index pair matrix.  The indices
refer to the rows of \code{loc}.  If \code{loc==NULL}, the indices will be
interpreted as indices into the point specification supplied to
\code{\link[=inla.mesh.create]{inla.mesh.create()}}.  If \code{is.bnd==TRUE}, defaults to linking
all the points in \code{loc}, as \code{c(1:nrow(loc),1L)}, otherwise
\code{1:nrow(loc)}.}

\item{grp}{Vector of group labels for each segment.  Set to \code{NULL} to
let the labels be chosen automatically in a call to
\code{\link[=inla.mesh.create]{inla.mesh.create()}}.}

\item{is.bnd}{\code{TRUE} if the segments are boundary segments, otherwise
\code{FALSE}.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}

\item{grp.default}{When joining segments, use this group label for segments
that have \code{grp=NULL}.}

\item{x, y, z, nlevels, levels}{Parameters specifying a set of surface contours,
with syntax described in \code{\link[=contour]{contour()}}.}

\item{groups}{Vector of group ID:s, one for each contour level.}

\item{positive}{\code{TRUE} if the contours should encircle positive level
excursions in a counter clockwise direction.}

\item{eps}{Tolerance for \code{\link[=inla.simplify.curve]{inla.simplify.curve()}}.}
}
\value{
An \code{inla.mesh.segment} object.
}
\description{
Constraint segments for inla.mesh

Constructs \code{inla.mesh.segment} objects that can be used to specify
boundary and interior constraint edges in calls to \code{\link[=inla.mesh]{inla.mesh()}}.
}
\examples{

## Create a square boundary and a diagonal interior segment
loc.bnd <- matrix(c(0, 0, 1, 0, 1, 1, 0, 1), 4, 2, byrow = TRUE)
loc.int <- matrix(c(0.9, 0.1, 0.1, 0.6), 2, 2, byrow = TRUE)
segm.bnd <- inla.mesh.segment(loc.bnd)
segm.int <- inla.mesh.segment(loc.int, is.bnd = FALSE)

## Points to be meshed
loc <- matrix(runif(10 * 2), 10, 2) * 0.9 + 0.05
mesh <- inla.mesh.create(loc,
    boundary = segm.bnd,
    interior = segm.int,
    refine = list()
)
plot(mesh)
\dontrun{
mesh <- inla.mesh.create(loc, interior = list(segm.bnd, segm.int))
plot(mesh)
}

}
\seealso{
\code{\link[=inla.mesh.create]{inla.mesh.create()}}, \code{\link[=inla.mesh.2d]{inla.mesh.2d()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.expert}
\alias{control.expert}
\alias{inla.control.expert}
\alias{inla.set.control.expert.default}
\alias{set.control.expert.default}
\alias{control.expert.default}
\title{Control variables in control.expert}
\description{Control variables in \code{control.expert} for use in \code{inla}}
\usage{
inla.set.control.expert.default(...)
control.expert(cpo.idx, cpo.manual, disable.gaussian.check, jp)
}
\arguments{
\item{...}{Possible arguments}
\item{cpo.manual}{   A boolean variable to decide if the inla-program is to be runned in a manual-cpo-mode. (EXPERT OPTION: DO NOT USE)}
\item{cpo.idx}{   The index/indices of the data point(s) to remove. (EXPERT OPTION: DO NOT USE)}
\item{disable.gaussian.check}{   Disable the check for fast computations with a Gaussian likelihood and identity link (default \code{FALSE})}
\item{jp}{   An object of class \code{inla.jp} defining a joint prior}
}
\value{
  
The function \code{control.expert} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.expert.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from compare.results.R
%%
\name{inla.compare.results}
\alias{inla.compare.results}
\alias{compare.results}

\title{Compare INLA and MCMC results}

\description{A small utility to compare INLA and MCMC results (OBSOLETE)}
\usage{
 inla.compare.results(dir.inla = NULL, dir.mcmc = NULL)
}

\arguments{
  \item{dir.inla}{The directory with the INLA results}
  \item{dir.mcmc}{The directory with the MCMC results}
}

\value{%%
  Return nothing. This is an interactive function.

  This function is OBSOLETE
}
%%
\author{Havard Rue \email{hrue@r-inla.org}}
\examples{
## See demo("Tokyo-compare")
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from jp.R
%%
\name{jp.define}
\alias{jp}
\alias{jp.define}
\alias{inla.jp.define}
\title{Joint-prior models}
\description{A framework for defining joint priors in R}

\usage{
    inla.jp.define(jp = NULL, ...)
}

\arguments{
  \item{jp}{The \code{jp}-function which returns the joint log-prior as a function of
            argument \code{theta}. There is an optional second argument that is a
            vector of \code{theta}-names. If
            second argument is not present, argument \code{.theta.desc}
            will be added.}
  \item{...}{Named list of variables that defines the environment of \code{jp}}
}

\value{%%
  This allows joint priors to be defined in \code{R}.

  This function is for internal use only.}
\author{Havard Rue \email{hrue@r-inla.org}}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from residuals.R
%%
\name{inla.ks.plot}
\alias{inla.ks.plot}
\alias{ks.plot}
\title{
  Kolmogorov-Smirnov Test Plots
}
\description{
  Illustrate a one-sample Kolmogorov-Smirnov test by plotting the
  empirical distribution deviation.
}
\usage{
inla.ks.plot(x, y, diff=TRUE, ...)
}
\arguments{
  \item{x}{a numeric vector of data values.}
  \item{y}{a cumulative distribution function such as 'pnorm'.}
  \item{diff}{logical, indicating if the normalised difference should be
    plotted.  If \code{FALSE}, the absolute distribution functions are plotted.}
  \item{...}{additional arguments for \code{\link{ks.test}}, ignored in
    the plotting.  In particular, only two-sided tests are illustrated.}
}
\details{
  In addition to the (normalised) empirical distribution deviation,
  lines for the K-S test statistic are drawn, as well as \eqn{\pm}{+/-}
  two standard deviations around the expectation under the null hypothesis.
}
\value{
  A list with class \code{"htest"}, as generated by
  \code{\link{ks.test}}
}
%%\references{
%%}
\author{
  Finn Lindgren \email{finn.lindgren@gmail.com}
}
%%\note{
%%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
  \code{\link{ks.test}}
}
\examples{
## Check for N(0,1) data
data = rowSums(matrix(runif(100*12)*2-1,100,12))/2
inla.ks.plot(data, pnorm)

\dontrun{
## Check the goodness-of-fit of cross-validated predictions
result = inla(..., control.predictor=list(cpo=TRUE))
inla.ks.plot(result$pit, punif)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.spde.make.block.A}
\alias{inla.spde.make.block.A}
\title{Observation matrices for mesh models.}
\usage{
inla.spde.make.block.A(
  A,
  block,
  n.block = max(block),
  weights = NULL,
  rescale = c("none", "count", "weights", "sum")
)
}
\arguments{
\item{A}{A precomputed observation/prediction matrix for locations that are
to be joined.}

\item{block}{Indices specifying block groupings: Entries with the same
\code{block} value are joined into a single row in the resulting matrix, and
the \code{block} values are the row indices.}

\item{n.block}{The number of blocks.}

\item{weights}{Optional scaling weights to be applied row-wise to the input
\code{A} matrix.}

\item{rescale}{Specifies what scaling method should be used when joining the
rows of the \code{A} matrix as grouped by the \code{block} specification.
\itemize{ \item \code{'none'}: Straight sum, no rescaling.  \item
\code{'count'}: Divide by the number of entries in the block.  \item
\code{'weights'}: Divide by the sum of the \code{weight} values within each
block.  \item \code{'sum'}: Divide by the resulting row sums.  }}
}
\description{
Constructs observation/prediction weight matrices for numerical integration
schemes for regional data problems.  Primarily intended for internal use by
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}.
}
\seealso{
\code{\link[=inla.spde.make.A]{inla.spde.make.A()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from upgrade.R
%%
\name{inla.upgrade}
\alias{inla.upgrade}
\alias{inla.update}

\title{Upgrade the INLA-package}

\description{Functions to upgrade the \code{INLA}-package to the
  current version.}

\usage{
inla.upgrade(lib = NULL, testing=FALSE, ask = TRUE)
inla.update(lib = NULL, testing=FALSE, ask = TRUE)
}

\arguments{
  \item{lib}{Location to install the library.}
  \item{testing}{If \code{TRUE}, then look for a test-version if the
    INLA-package.}
  \item{ask}{same argument as in \code{update.packages}}
}
\value{\code{inla.upgrade} will update the INLA package to the current version,
 and \code{inla.update} do the same
  for backward compatibility. This function is simple wrapper for
  \code{update.packages} using the INLA repository.}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{\code{update.packages}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Salm}
\alias{Salm}
\title{Extra-Poisson variation in dose-response study}
\format{
A data frame with 18 observations on the following 3 variables.
\describe{ \item{y}{number of salmonella bacteria}
\item{dose}{dose of quinoline (mg per plate)}
\item{rand}{indicator} }
}
\source{
WinBUGS/OpenBUGS manual Examples VOl.I
}
\description{
Breslow (1984) analyses some mutagenicity assay data (shown below) on
salmonella in which three plates have been processed at each dose i of
quinoline and the number of revertant colonies of TA98 Salmonella measured
}
\examples{

data(Salm)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{PRborder}
\alias{PRborder}
\title{The PRborder data}
\format{
\describe{ \item{Longtiude}{The Longtiude coordinate}
\item{Latitude}{The Latitude coordinate} }
}
\description{
A data matrix with Longitude and Latitude coordinates for the boundary of
Parana State.
}
\seealso{
PRprec
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from lincomb.R
%%
\name{make.lincomb}
\alias{make.lincomb}
\alias{make.lincombs}
\alias{inla.make.lincomb}
\alias{inla.make.lincombs}
\title{Create linear combinations}
\description{Create a linear combination or several
linear combinations, as input to \code{inla(...,  lincomb = <lincomb>)}}
\usage{
inla.make.lincomb(...)
inla.make.lincombs(...)
}
\arguments{
   \item{...}{Arguments; see examples}
}
\value{
 A structure to be passed on to \code{\link{inla}} argument \code{lincomb}
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{TODO}
\examples{
##See the worked out examples and description in the FAQ
##section on {www.r-inla.org}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde2.R
\name{param2.matern.orig}
\alias{param2.matern.orig}
\title{Parameter settings for \code{inla.spde2.matern} models.}
\usage{
param2.matern.orig(
  mesh,
  alpha = 2,
  B.tau = matrix(c(0, 1, 0), 1, 3),
  B.kappa = matrix(c(0, 0, 1), 1, 3),
  prior.variance.nominal = 1,
  prior.range.nominal = NULL,
  prior.tau = NULL,
  prior.kappa = NULL,
  theta.prior.mean = NULL,
  theta.prior.prec = 0.1
)
}
\arguments{
\item{mesh}{The mesh to build the model on, as an \code{\link[=inla.mesh]{inla.mesh()}}
object.}

\item{alpha}{Fractional operator order, \eqn{0<\alpha\leq 2} supported.
(\eqn{\nu=\alpha-d/2})}

\item{B.tau}{Matrix with specification of log-linear model for \eqn{\tau}.}

\item{B.kappa}{Matrix with specification of log-linear model for
\eqn{\kappa}.}

\item{prior.variance.nominal}{Nominal prior mean for the field variance}

\item{prior.range.nominal}{Nominal prior mean for the spatial range}

\item{prior.tau}{Prior mean for tau (overrides
\code{prior.variance.nominal})}

\item{prior.kappa}{Prior mean for kappa (overrides
\code{prior.range.nominal})}

\item{theta.prior.mean}{(overrides \verb{prior.*})}

\item{theta.prior.prec}{Scalar, vector or matrix, specifying the joint prior
precision for \eqn{theta}.}
}
\description{
Construct parameter settings for \code{inla.spde2.matern} models.
}
\seealso{
\code{\link[=inla.spde2.matern]{inla.spde2.matern()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.fixed}
\alias{control.fixed}
\alias{inla.control.fixed}
\alias{inla.set.control.fixed.default}
\alias{set.control.fixed.default}
\alias{control.fixed.default}
\title{Control variables in control.fixed}
\description{Control variables in \code{control.fixed} for use in \code{inla}}
\usage{
inla.set.control.fixed.default(...)
control.fixed(cdf, compute, correlation.matrix, expand.factor.strategy, mean, mean.intercept, prec, prec.intercept, quantiles, remove.names)
}
\arguments{
\item{...}{Possible arguments}
\item{cdf}{   A list of values to compute the CDF for, for all fixed effects}
\item{quantiles}{   A list of quantiles to compute for all fixed effects}
\item{expand.factor.strategy}{   The strategy used to expand factors into fixed effects based on their levels. The default strategy is us use the \code{model.matrix}-function for which NA's are not allowed (\code{expand.factor.strategy="model.matrix"}) and levels are possible removed. The alternative option (\code{expand.factor.strategy="inla"}) use an \code{inla}-spesific expansion which expand a factor into one fixed effects for each level, do allow for NA's and all levels are present in the model. In this case, factors MUST BE factors in the data.frame/list and NOT added as \code{.+factor(x1)+.} in the formula only.}
\item{mean}{   Prior mean for all fixed effects except the intercept. Alternatively, a named list with specific means where name=default applies to unmatched names. For example \code{control.fixed=list(mean=list(a=1, b=2, default=0))} assign 'mean=1' to fixed effect 'a' , 'mean=2' to effect 'b' and 'mean=0' to all others. (default 0.0)}
\item{mean.intercept}{   Prior mean for the intercept (default 0.0)}
\item{prec}{   Default precision for all fixed effects except the intercept. Alternatively, a named list with specific means where name=default applies to unmatched names. For example \code{control.fixed=list(prec=list(a=1, b=2, default=0.01))} assign 'prec=1' to fixed effect 'a' , 'prec=2' to effect 'b' and 'prec=0.01' to all others. (default 0.001)}
\item{prec.intercept}{   Default precision the intercept (default 0.0)}
\item{compute}{   Compute marginals for the fixed effects ? (default TRUE)}
\item{correlation.matrix}{   Compute the posterior correlation matrix for all fixed effects? (default FALSE) OOPS: This option will set up appropriate linear combinations and the results are shown as the posterior correlation matrix of the linear combinations. This option will imply \code{control.inla=list(lincomb.derived.correlation.matrix=TRUE)}.}
\item{remove.names}{   A vector of names of expanded fixed effects to remove from the model-matrix. This is an expert option, and should only be used if you know what you are doing.}
}
\value{
  
The function \code{control.fixed} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.fixed.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from doc.R
%%
\name{inla.doc}
\alias{inla.doc}
\alias{inla.doc}
\alias{inla.inla.doc}
\title{View documentation}
\description{View documentation of latent, prior and likelihood models.}
\usage{
inla.doc(what, section, verbose=FALSE)
}
\arguments{
\item{what}{What to view documentation about;
            name of latent model,  name of prior,  etc. (A regular expression.)}
\item{section}{An optional section, like \code{names(inla.models())},
               to look for the documentation. If missing, all sections are used.}
\item{verbose}{Logical If \code{TRUE} then run in verbose mode}
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{\code{www.r-inla.org}}
\examples{
\dontrun{inla.doc("rw2")}
\dontrun{inla.doc("gaussian", section = "prior")}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Germany}
\alias{Germany}
\title{Disease Mapping}
\format{
A data frame with 544 observations on the following 4 variables.
\describe{
\item{region}{Region of Germany}
\item{E}{Fixed quantity which accounts for number of people
in the district (offset)}
\item{Y}{Number of cases}
\item{x}{covariate measuring smoking consumption}
}
}
\description{
Cases of Oral cavity cancer in Germany from 1986-1990
}
\examples{

data(Germany)
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spmesh.R
\name{inla.identical.CRS}
\alias{inla.identical.CRS}
\title{Test CRS and inla.CRS for equality}
\usage{
inla.identical.CRS(crs0, crs1, crsonly = FALSE)
}
\arguments{
\item{crs0}{A \code{CRS} or \code{inla.CRS} object.}

\item{crs1}{A \code{CRS} or \code{inla.CRS} object.}

\item{crsonly}{Logical. If \code{TRUE}, only the \code{CRS} part of a
\code{inla.CRS} object is compared.}
}
\description{
Wrapper for identical, optionally testing only the CRS part of two objects
}
\examples{

crs0 <- inla.CRS("longlat")
crs1 <- inla.CRS("longlat", oblique = c(0, 90))
print(c(
    inla.identical.CRS(crs0, crs0),
    inla.identical.CRS(crs0, crs1),
    inla.identical.CRS(crs0, crs1, crsonly = TRUE)
))
}
\seealso{
\code{\link[=inla.CRS]{inla.CRS()}}
}
\author{
Finn Lindgren \href{mailto:finn.lindgren@gmail.com}{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{plot.inla.trimesh}
\alias{plot.inla.trimesh}
\title{Low level triangulation mesh plotting}
\usage{
\method{plot}{inla.trimesh}(
  x,
  S,
  color = NULL,
  color.axis = NULL,
  color.n = 512,
  color.palette = cm.colors,
  color.truncate = FALSE,
  alpha = NULL,
  lwd = 1,
  specular = "black",
  draw.vertices = TRUE,
  draw.edges = TRUE,
  edge.color = rgb(0.3, 0.3, 0.3),
  ...
)
}
\arguments{
\item{x}{A 3-column triangle-to-vertex index map matrix.}

\item{S}{A 3-column vertex coordinate matrix.}

\item{color}{Color specification.  A single named color, a vector of scalar
values, or a matrix of RGB values.}

\item{color.axis}{The min/max limit values for the color mapping.}

\item{color.n}{The number of colors to use in the color palette.}

\item{color.palette}{A color palette function.}

\item{color.truncate}{If \code{TRUE}, truncate the colors at the color axis
limits.}

\item{alpha}{Transparency/opaqueness values. See \code{rgl.material}.}

\item{lwd}{Line width for edges. See \code{rgl.material}.}

\item{specular}{Specular color. See \code{rgl.material}.}

\item{draw.vertices}{If \code{TRUE}, draw triangle vertices.}

\item{draw.edges}{If \code{TRUE}, draw triangle edges.}

\item{edge.color}{Edge color specification.}

\item{\dots}{Additional parameters passed to and from other methods.}
}
\description{
Plots a triangulation mesh using \code{rgl}.
}
\seealso{
\code{\link[=plot.inla.mesh]{plot.inla.mesh()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-gev.R
%%
 \name{pc.gevtail}
 \alias{inla.pc.gevtail}
 \alias{pc.gevtail}
 \alias{pc.rgevtail}
 \alias{inla.pc.rgevtail}
 \alias{pc.dgevtail}
 \alias{inla.pc.dgevtail}
 \alias{pc.pgevtail}
 \alias{inla.pc.pgevtail}
 \alias{pc.qgevtail}
 \alias{inla.pc.qgevtail}

 \title{Utility functions for the PC prior for the \code{tail} parameter in the GEV likelihood}

 \description{Functions to evaluate, sample, compute quantiles and
              percentiles of the PC prior for the \code{tail} parameter
              in the GEV likelihood}
 \usage{
 inla.pc.rgevtail(n, lambda = 7)
 inla.pc.dgevtail(xi, lambda = 7, log = FALSE)
 inla.pc.qgevtail(p, lambda = 7)
 inla.pc.pgevtail(q, lambda = 7)
 }
 \arguments{
   \item{n}{Number of observations}
   \item{lambda}{The rate parameter in the PC-prior}
   \item{xi}{Vector of evaluation points, where \code{1>xi>0}.}
   \item{log}{Logical. Return the density in natural or log-scale.}
   \item{p}{Vector of probabilities}
   \item{q}{Vector of quantiles}
 }
 \details{
 This gives the PC prior for the \code{tail} parameter for the GEV likelihood,
 where \code{xi=0} is the base model.
 }
\value{%%
  \code{inla.pc.dgevtail} gives the density,
  \code{inla.pc.pgevtail} gives the distribution function,
  \code{inla.pc.qgevtail} gives the quantile function, and
  \code{inla.pc.rgevtail} generates random deviates.
 }
 \seealso{inla.doc("pc.gevtail")}
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 xi = inla.pc.rgevtail(100,  lambda = 7)
 d = inla.pc.dgevtail(xi, lambda = 7)
 xi = inla.pc.qgevtail(0.5, lambda = 7)
 inla.pc.pgevtail(xi, lambda = 7)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.spde.make.A}
\alias{inla.spde.make.A}
\title{Observation/prediction matrices for mesh models.}
\usage{
inla.spde.make.A(
  mesh = NULL,
  loc = NULL,
  index = NULL,
  group = NULL,
  repl = 1L,
  n.spde = NULL,
  n.group = NULL,
  n.repl = NULL,
  group.mesh = NULL,
  weights = NULL,
  A.loc = NULL,
  A.group = NULL,
  group.index = NULL,
  block = NULL,
  n.block = NULL,
  block.rescale = c("none", "count", "weights", "sum"),
  ...
)
}
\arguments{
\item{mesh}{An \code{\link[=inla.mesh]{inla.mesh()}} or \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object
specifying a function basis on a mesh domain.  Alternatively, an
\code{inla.spde} object that includes a mesh (e.g. from
\code{\link[=inla.spde2.matern]{inla.spde2.matern()}}).}

\item{loc}{Observation/prediction coordinates.  \code{mesh} and \code{loc}
defines a matrix \code{A.loc} of mapping weights between basis function
weights and field values.  If \code{loc} is \code{NULL}, \code{A.loc} is
defined as \code{Diagonal(n.spde, 1)}.}

\item{index}{For each observation/prediction value, an index into
\code{loc}.  Default is \code{seq_len(nrow(A.loc))}.}

\item{group}{For each observation/prediction value, an index into the group
model.}

\item{repl}{For each observation/prediction value, the replicate index.}

\item{n.spde}{The number of basis functions in the mesh model. (Note: may be
different than the number of mesh vertices/nodes/knots.)}

\item{n.group}{The size of the group model.}

\item{n.repl}{The total number of replicates.}

\item{group.mesh}{An optional \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object for the
group model.}

\item{weights}{Optional scaling weights to be applied row-wise to the
resulting matrix.}

\item{A.loc}{Optional precomputed observation/prediction matrix.
\code{A.loc} can be specified instead of \code{mesh}+\code{loc}, optionally
with \code{index} supplied.}

\item{A.group}{Optional precomputed observation/prediction matrix for the
group model. \code{A.group} can be specified instead of \code{group} and/or
\code{group.mesh}, optionally with \code{group.index} supplied.}

\item{group.index}{For each observation/prediction value, an index into the
rows of \code{A.group}.}

\item{block}{Optional indices specifying block groupings: Entries with the
same \code{block} value are joined into a single row in the resulting
matrix, and the \code{block} values are the row indices.  This is intended
for construction of approximate integration schemes for regional data
problems. See \code{\link[=inla.spde.make.block.A]{inla.spde.make.block.A()}} for details.}

\item{n.block}{The number of blocks.}

\item{block.rescale}{Specifies what scaling method should be used when
joining entries as grouped by a \code{block} specification.  See
\code{\link[=inla.spde.make.block.A]{inla.spde.make.block.A()}} for details.}

\item{\dots}{Additional parameters.  Currently unused.}
}
\description{
Constructs observation/prediction weight matrices for models based on
\code{\link[=inla.mesh]{inla.mesh()}} and \code{\link[=inla.mesh.1d]{inla.mesh.1d()}} objects.
}
\examples{

loc <- matrix(runif(10000 * 2) * 1000, 10000, 2)
mesh <- inla.mesh.2d(
    loc = loc,
    cutoff = 50,
    max.edge = c(50, 500)
)
A <- inla.spde.make.A(mesh, loc = loc)
}
\seealso{
\code{\link[=inla.spde.make.index]{inla.spde.make.index()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from print.R
%%
\name{print.inla}
\alias{print.inla}
\title{Print a INLA fit }
\description{
Print a INLA fit
}
\usage{
\method{print}{inla}(x, digits = 3L, ...)
}
\arguments{
  \item{x}{An inla-object (output from an \code{\link{inla}}-call).}
  \item{digits}{Number of digits to print}
  \item{...}{ other arguments.}
}
\details{
 None
}
\value{
 None
}
\author{Havard Rue}

\seealso{ \code{\link{inla}} }
\examples{
## None
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.lattice}
\alias{inla.mesh.lattice}
\title{Lattice grids for inla.mesh}
\usage{
inla.mesh.lattice(
  x = seq(0, 1, length.out = 2),
  y = seq(0, 1, length.out = 2),
  z = NULL,
  dims = if (is.matrix(x)) {     dim(x) } else {     c(length(x), length(y)) },
  units = NULL,
  crs = NULL
)
}
\arguments{
\item{x}{vector or grid matrix of x-values}

\item{y}{vector of grid matrix of y-values}

\item{z}{if x is a matrix, a grid matrix of z-values}

\item{dims}{the size of the grid, length 2 vector}

\item{units}{One of \code{c("default", "longlat", "longsinlat")}.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}
}
\value{
An \code{inla.mesh.lattice} object.
}
\description{
Construct a lattice grid for \code{\link[=inla.mesh]{inla.mesh()}}
}
\examples{

lattice <- inla.mesh.lattice(seq(0, 1, length.out = 17), seq(0, 1, length.out = 10))

## Use the lattice "as-is", without refinement:
mesh <- inla.mesh.create(lattice = lattice, boundary = lattice$segm)
mesh <- inla.mesh.create(lattice = lattice, extend = FALSE)
plot(mesh)

## Refine the triangulation, with limits on triangle angles and edges:
mesh <- inla.mesh.create(
    lattice = lattice,
    refine = list(max.edge = 0.08),
    extend = FALSE
)
plot(mesh)

## Add an extension around the lattice, but maintain the lattice edges:
mesh <- inla.mesh.create(
    lattice = lattice,
    refine = list(max.edge = 0.08),
    interior = lattice$segm
)
plot(mesh)

## Only add extension:
mesh <- inla.mesh.create(lattice = lattice, refine = list(max.edge = 0.08))
plot(mesh)
}
\seealso{
\code{\link[=inla.mesh]{inla.mesh()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{extract.groups}
\alias{extract.groups}
\alias{extract.groups.inla.mesh.segment}
\title{Extract tagged boundary/internal segments.}
\usage{
extract.groups(...)

\method{extract.groups}{inla.mesh.segment}(segm, groups, groups.new = groups, ...)
}
\arguments{
\item{\dots}{Additional arguments, passed on to other methods.}

\item{segm}{An \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object.}

\item{groups}{The segment groups id:s to extract.}

\item{groups.new}{Optional vector of group id remapping; \code{groups[k]} in
the input will be replaced by \code{groups.new[k]} in the output.}
}
\description{
Extract boundary or internal segments tagged by group id:s.
}
\seealso{
\code{\link[=inla.mesh.segment]{inla.mesh.segment()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{SPDEtoy}
\alias{SPDEtoy}
\title{toy simulated data set for the SPDE tutorial}
\format{
A data frame with 200 observations on the following 3 variables.
\describe{ \item{s1}{First element of the coordinates}
\item{s2}{Second element of the coordinates} \item{y}{data
simulated at the locations} }
}
\source{
SPDE tutorial
}
\description{
Simulated data set on 200 location points.  The simulation process is made
at the introduction of the SPDE tutorial.
}
\examples{

data(SPDEtoy)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Drivers}
\alias{Drivers}
\title{Time series with seasonal effect}
\format{
A data frame with 204 observations on the following 4 variables.
\describe{
\item{y}{Number of deaths}
\item{belt}{Indicator of weather the belt was compulsory to use
(1) or not (0)}
\item{trend}{time (in months)}
\item{seasonal}{time (in months)}
}
}
\description{
Montly total of car drivers killed or several injuried in England from
January 1969 to December 1984
}
\details{
NB: The last 12 lines of the data set have the first column set to
\code{NULL} since these data where not observed but we want to predict them.
}
\examples{

data(Drivers)
}
\references{
Rue, H and Held, L. (2005) \emph{Gaussian Markov Random Fields -
Theory and Applications} Chapman and Hall
}
\keyword{datasets}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from debug.graph.R
%%
\name{debug.graph}
\alias{debug.graph}
\alias{inla.debug.graph}
\title{Debug a graph-file}
\description{Debug a graph specification on file (ascii-mode only), by checking the specification along the way.}
\usage{inla.debug.graph(graph.file)}
\arguments{
    \item{graph.file}{The filename of the graph (ascii-mode)}
}
\value{
 If an error is found, then an error message is shows, otherwise the graph-object returned by
 \code{inla.read.graph()} is returned.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\seealso{inla.read.graph}
\examples{
## Not run:
## cat("3\n 1 1 2n\ 2 1 1\n 3 4\n", file="g.dat")
## g = inla.debug.graph("g.dat")
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meshbuilder.R
\name{meshbuilder}
\alias{meshbuilder}
\title{Interactive mesh building and diagnostics}
\usage{
meshbuilder()
}
\description{
Interactively design and build a triangle mesh
for use with SPDE models, and assess the finite element
approximation errors. The R code needed to recreate the mesh
outside the interactive Shiny app is also generated. Spatial
objects can be imported from the global workspace.
}
\examples{
\dontrun{
meshbuilder()
}
}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}, \code{\link[=inla.mesh.create]{inla.mesh.create()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from iidkd.R
%%
\name{inla.iidkd.sample}
\alias{inla.iidkd.sample}
\alias{iidkd.sample}

\title{Provide samples from the \code{iidkd} component (experimental)}
\description{
 This function provide samples of the \code{iidkd} component 
 using more interpretable parameters
}
\usage{
inla.iidkd.sample(n = 10^4, result, name, return.cov = FALSE)
}

\arguments{
\item{n}{Integer Number of samples to use}
\item{result}{inla-object An object of class \code{inla}, ie a result of a call to \code{inla()}}
\item{name}{Character The name of the \code{iidkd} component}
\item{return.cov}{Logical Return samples of the covariance matrix instead of
      stdev/correlation matrix described below?}
}
\value{A list of sampled matrices, with (default) correlations on the off-diagonal and
standard-deviations on the diagonal}
\seealso{\code{inla.doc("iidkd")}}
\author{Havard Rue \email{hrue@r-inla.org}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde2.R
\name{inla.spde2.matern}
\alias{inla.spde2.matern}
\alias{inla.spde2.theta2phi0}
\alias{inla.spde2.theta2phi1}
\alias{inla.spde2.theta2phi2}
\title{Matern SPDE model object for INLA}
\usage{
inla.spde2.matern(
  mesh,
  alpha = 2,
  param = NULL,
  constr = FALSE,
  extraconstr.int = NULL,
  extraconstr = NULL,
  fractional.method = c("parsimonious", "null"),
  B.tau = matrix(c(0, 1, 0), 1, 3),
  B.kappa = matrix(c(0, 0, 1), 1, 3),
  prior.variance.nominal = 1,
  prior.range.nominal = NULL,
  prior.tau = NULL,
  prior.kappa = NULL,
  theta.prior.mean = NULL,
  theta.prior.prec = 0.1,
  n.iid.group = 1,
  ...
)

inla.spde2.theta2phi0(spde, theta)

inla.spde2.theta2phi1(spde, theta)

inla.spde2.theta2phi2(spde, theta)
}
\arguments{
\item{mesh}{The mesh to build the model on, as an \code{\link[=inla.mesh]{inla.mesh()}} or
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}} object.}

\item{alpha}{Fractional operator order, \eqn{0<\alpha\leq 2} supported.
(\eqn{\nu=\alpha-d/2})}

\item{param}{Parameter, e.g. generated by \code{param2.matern.orig}}

\item{constr}{If \code{TRUE}, apply an integrate-to-zero constraint.
Default \code{FALSE}.}

\item{extraconstr.int}{Field integral constraints.}

\item{extraconstr}{Direct linear combination constraints on the basis
weights.}

\item{fractional.method}{Specifies the approximation method to use for
fractional (non-integer) \code{alpha} values. \code{'parsimonious'} gives an
overall approximate minimal covariance error, \code{'null'} uses
approximates low-order properties.}

\item{B.tau}{Matrix with specification of log-linear model for \eqn{\tau}.}

\item{B.kappa}{Matrix with specification of log-linear model for
\eqn{\kappa}.}

\item{prior.variance.nominal}{Nominal prior mean for the field variance}

\item{prior.range.nominal}{Nominal prior mean for the spatial range}

\item{prior.tau}{Prior mean for tau (overrides
\code{prior.variance.nominal})}

\item{prior.kappa}{Prior mean for kappa (overrides
\code{prior.range.nominal})}

\item{theta.prior.mean}{(overrides \verb{prior.*})}

\item{theta.prior.prec}{Scalar, vector or matrix, specifying the joint prior
precision for \eqn{theta}.}

\item{n.iid.group}{If greater than 1, build an explicitly iid replicated
model, to support constraints applied to the combined replicates, for
example in a time-replicated spatial model. Constraints can either be
specified for a single mesh, in which case it's applied to the average of
the replicates (\code{ncol(A)} should be \code{mesh$n} for 2D meshes,
\code{mesh$m} for 1D), or as general constraints on the collection of
replicates (\code{ncol(A)} should be \code{mesh$n * n.iid.group} for 2D
meshes, \code{mesh$m * n.iid.group} for 1D).}

\item{\dots}{Additional parameters for special uses.}
}
\value{
An \code{inla.spde2} object.
}
\description{
Create an \code{inla.spde2} model object for a Matern model. Use
\code{inla.spde2.pcmatern} instead for a PC prior for the parameters.
}
\details{
This method constructs a Matern SPDE model, with spatial scale parameter
\eqn{\kappa(u)} and variance rescaling parameter \eqn{\tau(u)}.

\deqn{(\kappa^2(u)-\Delta)^{\alpha/2}(\tau(u) }{(kappa^2(u)-Delta)^(alpha/2)
(tau(u) x(u)) = W(u)}\deqn{ x(u))=W(u)}{(kappa^2(u)-Delta)^(alpha/2) (tau(u)
x(u)) = W(u)}

Stationary models are supported for \eqn{0 < \alpha \leq 2}, with spectral
approximation methods used for non-integer \eqn{\alpha}, with approximation
method determined by \code{fractional.method}.

Non-stationary models are supported for \eqn{\alpha=2} only, with \itemize{
\item \eqn{\log\tau(u) = B^\tau_0(u) + \sum_{k=1}^p B^\tau_k(u) }{log tau(u)
= B.tau_0(u) + sum_{k=1}^p B.tau_k(u) theta_k}\eqn{ \theta_k}{log tau(u) =
B.tau_0(u) + sum_{k=1}^p B.tau_k(u) theta_k}

\item \eqn{\log\kappa(u) = B^{\kappa}_0(u) + \sum_{k=1}^p B^{\kappa}_k(u)
}{log kappa(u) = B.kappa_0(u) + sum_{k=1}^p B.kappa_k(u) theta_k}\eqn{
\theta_k}{log kappa(u) = B.kappa_0(u) + sum_{k=1}^p B.kappa_k(u) theta_k}

}

The same parameterisation is used in the stationary cases, but with
\eqn{B^\tau_0}, \eqn{B^\tau_k}, \eqn{B^\kappa_0}, and \eqn{B^\tau_k}
constant across \eqn{u}.

Integration and other general linear constraints are supported via the
\code{constr}, \code{extraconstr.int}, and \code{extraconstr} parameters,
which also interact with \code{n.iid.group}.
}
\section{Functions}{
\itemize{
\item \code{inla.spde2.theta2phi0}: Convert from theta vector to phi0 values in
the internal spde2 model representation

\item \code{inla.spde2.theta2phi1}: Convert from theta vector to phi1 values in
the internal spde2 model representation

\item \code{inla.spde2.theta2phi2}: Convert from theta vector to phi2 values in
the internal spde2 model representation
}}

\examples{

n = 100
field.fcn = function(loc) (10*cos(2*pi*2*(loc[,1]+loc[,2])))
loc = matrix(runif(n*2),n,2)
## One field, 2 observations per location
idx.y = rep(1:n,2)
y = field.fcn(loc[idx.y,]) + rnorm(length(idx.y))

mesh = inla.mesh.create(loc, refine=list(max.edge=0.05))
spde = inla.spde2.matern(mesh)
data = list(y=y, field=mesh$idx$loc[idx.y])
formula = y ~ -1 + f(field, model=spde)
result = inla(formula, data=data, family="normal")

## Plot the mesh structure:
plot(mesh)
\donttest{
if (require(rgl)) {
  col.pal = colorRampPalette(c("blue","cyan","green","yellow","red"))
  ## Plot the posterior mean:
  plot(mesh, rgl=TRUE,
       result$summary.random$field[,"mean"],
       color.palette = col.pal)
  ## Plot residual field:
  plot(mesh, rgl=TRUE,
       result$summary.random$field[,"mean"]-field.fcn(mesh$loc),
       color.palette = col.pal)
}
}
result.field = inla.spde.result(result, "field", spde)
plot(result.field$marginals.range.nominal[[1]])

}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}, \code{\link[=inla.mesh.create]{inla.mesh.create()}},
\code{\link[=inla.mesh.1d]{inla.mesh.1d()}}, \code{\link[=inla.mesh.basis]{inla.mesh.basis()}},
\code{\link[=inla.spde2.pcmatern]{inla.spde2.pcmatern()}}, \code{\link[=inla.spde2.generic]{inla.spde2.generic()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.components.R
\name{inla.mesh.components}
\alias{inla.mesh.components}
\title{Compute connected mesh subsets}
\usage{
inla.mesh.components(mesh)
}
\value{
A list with elements \code{vertex} and \code{triangle}, vectors of
integer labels for which connected component they belong, and \code{info}, a
\code{data.frame} with columns
\item{component}{Connected component integer label.}
\item{nV}{The number of vertices in the component.}
\item{nT}{The number of triangles in the component.}
\item{area}{The surface area associated with the component.
Component lables are not comparable across
different meshes, but some ordering stability is guaranteed by initiating
each component from the lowest numbered triangle whenever a new component is
initiated.}
}
\description{
Compute subsets of vertices and triangles in an inla.mesh object that are
connected by edges.
}
\examples{

# Construct two simple meshes:
loc <- matrix(c(0, 1, 0, 1), 2, 2)
mesh1 <- inla.mesh.2d(loc = loc, max.edge = 0.1)
bnd <- inla.nonconvex.hull(loc, 0.3)
mesh2 <- inla.mesh.2d(boundary = bnd, max.edge = 0.1)

# Compute connectivity information:
conn1 <- inla.mesh.components(mesh1)
conn2 <- inla.mesh.components(mesh2)
# One component, simply connected mesh
conn1$info
# Two disconnected components
conn2$info

# Extract the subset mesh for the largest component:
# (Note: some information is lost, such as fixed segments,
# and boundary edge labels.)
maxi <- conn2$info$component[which.max(conn2$info$area)]
mesh3 <- inla.mesh.create(
    loc = mesh2$loc,
    tv = mesh2$graph$tv[conn2$triangle == maxi, , drop = FALSE]
)
}
\seealso{
\code{\link[=inla.mesh.2d]{inla.mesh.2d()}}, \code{\link[=inla.mesh.create]{inla.mesh.create()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.2d}
\alias{inla.mesh.2d}
\title{High-quality triangulations}
\usage{
inla.mesh.2d(
  loc = NULL,
  loc.domain = NULL,
  offset = NULL,
  n = NULL,
  boundary = NULL,
  interior = NULL,
  max.edge = NULL,
  min.angle = NULL,
  cutoff = 1e-12,
  max.n.strict = NULL,
  max.n = NULL,
  plot.delay = NULL,
  crs = NULL
)
}
\arguments{
\item{loc}{Matrix of point locations to be used as initial triangulation
nodes.  Can alternatively be a \code{SpatialPoints} or
\code{SpatialPointsDataFrame} object.}

\item{loc.domain}{Matrix of point locations used to determine the domain
extent.  Can alternatively be a \code{SpatialPoints} or
\code{SpatialPointsDataFrame} object.}

\item{offset}{The automatic extension distance.  One or two values, for an
inner and an optional outer extension.  If negative, interpreted as a factor
relative to the approximate data diameter (default=-0.10???)}

\item{n}{The number of initial nodes in the automatic extensions
(default=16)}

\item{boundary}{A list of one or two \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} objects
describing domain boundaries.}

\item{interior}{An \code{\link[=inla.mesh.segment]{inla.mesh.segment()}} object describing desired
interior edges.}

\item{max.edge}{The largest allowed triangle edge length.  One or two
values.}

\item{min.angle}{The smallest allowed triangle angle.  One or two values.
(Default=21)}

\item{cutoff}{The minimum allowed distance between points.  Point at most as
far apart as this are replaced by a single vertex prior to the mesh
refinement step.}

\item{max.n.strict}{The maximum number of vertices allowed, overriding
\code{min.angle} and \code{max.edge} (default=-1, meaning no limit).  One or
two values, where the second value gives the number of additional vertices
allowed for the extension.}

\item{max.n}{The maximum number of vertices allowed, overriding
\code{max.edge} only (default=-1, meaning no limit).  One or two values,
where the second value gives the number of additional vertices allowed for
the extension.}

\item{plot.delay}{On Linux (and Mac if appropriate X11 libraries are
installed), specifying a nonnegative numeric value activates a rudimentary
plotting system in the underlying \code{fmesher} program, showing the
triangulation algorithm at work, with waiting time factor \code{plot.delay}
between each step.

On all systems, specifying any negative value activates displaying the
result after each step of the multi-step domain extension algorithm.}

\item{crs}{An optional \code{CRS} or \code{inla.CRS} object}
}
\value{
An \code{inla.mesh} object.
}
\description{
Create a triangle mesh based on initial point locations, specified or
automatic boundaries, and mesh quality parameters.
}
\examples{

loc <- matrix(runif(10 * 2), 10, 2)

if (require("splancs")) {
    boundary <- list(
        inla.nonconvex.hull(loc, 0.1, 0.15),
        inla.nonconvex.hull(loc, 0.2, 0.2)
    )
    offset <- NULL
} else {
    boundary <- NULL
    offset <- c(0.1, 0.2)
}
mesh <- inla.mesh.2d(loc, boundary = boundary, offset = offset, max.edge = c(0.05, 0.1))

plot(mesh)
}
\seealso{
\code{\link[=inla.mesh.create]{inla.mesh.create()}}, \code{\link[=inla.delaunay]{inla.delaunay()}},
\code{\link[=inla.nonconvex.hull]{inla.nonconvex.hull()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde2.R
\name{inla.spde2.matern.sd.basis}
\alias{inla.spde2.matern.sd.basis}
\title{Approximate variance-compensating basis functions}
\usage{
inla.spde2.matern.sd.basis(
  mesh,
  B.sd,
  B.range,
  method = 1,
  local.offset.compensation = FALSE,
  alpha = 2,
  ...
)
}
\arguments{
\item{mesh}{An \code{\link[=inla.mesh]{inla.mesh()}} object.}

\item{B.sd}{Desired basis for log-standard deviations.}

\item{B.range}{Desired basis for spatial range.}

\item{method}{Construction method selector.  Expert option only.}

\item{local.offset.compensation}{If \code{FALSE}, only compensate in the
average for the \code{tau} offset.}

\item{alpha}{The model \code{alpha} parameter.}

\item{\dots}{Additional parameters passed on to internal
\code{inla.spde2.matern} calls.}
}
\value{
List of basis specifications \item{B.tau }{Basis for
\code{log(tau)}} \item{B.kappa }{Basis for \code{log(kappa)}} Intended for
passing on to \code{\link[=inla.spde2.matern]{inla.spde2.matern()}}.
}
\description{
Calculates an approximate basis for \code{tau} and \code{kappa} for an
\code{inla.spde2.matern} model where \code{tau} is a rescaling parameter.
}
\seealso{
\code{\link[=inla.spde2.matern]{inla.spde2.matern()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meshassessment.R
\name{inla.mesh.assessment}
\alias{inla.mesh.assessment}
\title{Interactive mesh building and diagnostics}
\usage{
inla.mesh.assessment(mesh, spatial.range, alpha = 2, dims = c(500, 500))
}
\description{
Assess the finite element approximation errors in a mesh for interactive R
sessions.  More detailed assessment tools are in \code{\link[=meshbuilder]{meshbuilder()}}.
}
\examples{

bnd <- inla.mesh.segment(cbind(
    c(0, 10, 10, 0, 0),
    c(0, 0, 10, 10, 0)
), bnd = TRUE)
mesh <- inla.mesh.2d(boundary = bnd, max.edge = 1)
out <- inla.mesh.assessment(mesh, spatial.range = 3, alpha = 2)
}
\seealso{
inla.mesh.2d, inla.mesh.create, meshbuilder
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.simplify.curve}
\alias{inla.simplify.curve}
\title{Recursive curve simplification.}
\usage{
inla.simplify.curve(loc, idx, eps)
}
\arguments{
\item{loc}{Coordinate matrix.}

\item{idx}{Index vector into \code{loc} specifying a polygonal curve.}

\item{eps}{Straightness tolerance.}
}
\value{
An index vector into \code{loc} specifying the simplified polygonal
curve.
}
\description{
Attempts to simplify a polygonal curve by joining nearly colinear segments.
}
\details{
Uses a variation of the binary splitting Ramer-Douglas-Peucker algorithm,
with a width \code{eps} ellipse instead of a rectangle, motivated by
prediction ellipse for Brownian bridge.
}
\examples{

theta <- seq(0, 2 * pi, length = 1000)
loc <- cbind(cos(theta), sin(theta))
idx <- inla.simplify.curve(loc = loc, idx = 1:nrow(loc), eps = 0.01)
print(c(nrow(loc), length(idx)))
plot(loc, type = "l")
lines(loc[idx, ], col = "red")
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from coxph.R
%%
\name{inla.coxph}
\alias{inla.coxph}
\alias{inla.rbind.data.frames}
\alias{coxph}
\alias{cbind.data.frames}
\title{Convert a Cox proportional hazard model into Poisson regression}
\description{Tools to convert a Cox proportional hazard model into Poisson regression}
\usage{
    inla.coxph(formula, data, control.hazard = list(), tag="", debug=FALSE)
    inla.rbind.data.frames(...)
}
\arguments{
  \item{formula}{The formula for the coxph model where the reponse must be a \code{inla.surv}-object.}
  \item{data}{All the data used in the formula,  as a list.}
  \item{control.hazard}{Control the model for the baseline-hazard; see \code{?control.hazard}.}
  \item{tag}{An optional tag added to the names of the new variables created (to make them
             unique when combined with several calls of \code{inla.coxph}. Note that 
             \code{E..coxph} is not included,  as its usually merged into one 
             vector over different expansions.}
  \item{debug}{Print debug-information}
  \item{...}{Data.frames to be \code{rbind}-ed,  padding with \code{NA}.}
}
\value{
      \code{inla.coxph} returns a list of new expanded variables to be used in the \code{inla}-call.
      Note that element \code{data} and \code{data.list} needs to be merged into
      a \code{list} to be passed as the \code{data} argument. See the example for details.

      \code{inla.rbind.data.frames} returns the rbinded data.frames padded with NAs.
      There is a better
      implementation in \code{dplyr::bind_rows}, which is used if package \code{dplyr} is
      installed.
}
\author{Havard Rue \email{hrue@r-inla.org}}
\examples{
## How the cbind.data.frames works:
df1 = data.frame(x=1:2, y=2:3, z=3:4)
df2 = data.frame(x=3:4, yy=4:5, zz=5:6)
inla.rbind.data.frames(df1, df2)

## Standard example of how to convert a coxph into a Poisson regression
n = 1000
x = runif(n)
lambda = exp(1+x)
y = rexp(n, rate=lambda)
event = rep(1,n)
data = list(y=y, event=event, x=x)
y.surv = inla.surv(y, event)
intercept1 = rep(1, n)
p = inla.coxph(y.surv ~ -1 + intercept1 + x,
               list(y.surv = y.surv,  x=x, intercept1 = intercept1))

r = inla(p$formula,
        family = p$family,
        data=c(as.list(p$data), p$data.list),
        E = p$E)
summary(r)

## How to use this in a joint model
intercept2 = rep(1, n)
y = 1 + x + rnorm(n, sd=0.1)
df = data.frame(intercept2, x, y)

## new need to cbind the data.frames, and then add the list-part of
## the data
df.joint = c(as.list(inla.rbind.data.frames(p$data, df)), p$data.list)
df.joint$Y = cbind(df.joint$y..coxph, df.joint$y)

## merge the formulas, recall to add '-1' and to use the new joint
## reponse 'Y'
formula = update(p$formula, Y ~ intercept2 -1 + .)

rr = inla(formula,
        family = c(p$family, "gaussian"),
        data = df.joint,
        E = df.joint$E..coxph)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R, R/spde1.R, R/spde2.R
\name{inla.spde.result}
\alias{inla.spde.result}
\alias{inla.spde.result.inla.spde1}
\alias{inla.spde.result.inla.spde2}
\alias{inla.spde1.result}
\alias{inla.spde2.result}
\title{SPDE result extraction from INLA estimation results}
\usage{
inla.spde.result(...)

inla.spde1.result(inla, name, spde, do.transform = TRUE, ...)

\method{inla.spde.result}{inla.spde1}(inla, name, spde, do.transform = TRUE, ...)

inla.spde2.result(inla, name, spde, do.transform = TRUE, ...)

\method{inla.spde.result}{inla.spde2}(inla, name, spde, do.transform = TRUE, ...)
}
\arguments{
\item{\dots}{Further arguments passed to and from other methods.}

\item{inla}{An \code{inla} object obtained from a call to \code{\link[=inla]{inla()}}}

\item{name}{A character string with the name of the SPDE effect in the inla
formula.}

\item{spde}{The \code{inla.spde} object used for the effect in the inla
formula. (Note: this could have been stored in the inla output, but isn't.)
Usually the result of a call to \code{\link[=inla.spde2.matern]{inla.spde2.matern()}}.}

\item{do.transform}{If \code{TRUE}, also calculate marginals transformed to
user-scale.  Setting to \code{FALSE} is useful for large non-stationary
models, as transforming many marginal densities is time-consuming.}
}
\value{
For \code{inla.spde2} models, a list, where the nominal range and
variance are defined as the values that would have been obtained with a
stationary model and no boundary effects: \item{marginals.kappa }{Marginal
densities for kappa} \item{marginals.log.kappa }{Marginal densities for
log(kappa)} \item{marginals.log.range.nominal }{Marginal densities for
log(range)} \item{marginals.log.tau }{Marginal densities for log(tau)}
\item{marginals.log.variance.nominal }{Marginal densities for log(variance)}
\item{marginals.range.nominal }{Marginal densities for range}
\item{marginals.tau }{Marginal densities for tau} \item{marginals.theta
}{Marginal densities for the theta parameters} \item{marginals.values
}{Marginal densities for the field values} \item{marginals.variance.nominal
}{Marginal densities for variance} \item{summary.hyperpar }{The SPDE related
part of the inla hyperpar output summary} \item{summary.log.kappa }{Summary
statistics for log(kappa)} \item{summary.log.range.nominal }{Summary
statistics for log(range)} \item{summary.log.tau }{Summary statistics for
log(tau)} \item{summary.log.variance.nominal }{Summary statistics for
log(kappa)} \item{summary.theta }{Summary statistics for the theta
parameters} \item{summary.values }{Summary statistics for the field values}
}
\description{
Exctract field and parameter values and distributions for an
\code{inla.spde} SPDE effect from an inla result object.
}
\examples{

loc <- matrix(runif(100 * 2), 100, 2)
mesh <- inla.mesh.create.helper(points.domain = loc, max.edge = c(0.1, 0.5))
spde <- inla.spde2.matern(mesh)
index <- inla.spde.make.index("spatial", mesh$n, n.repl = 2)
spatial.A <- inla.spde.make.A(mesh, loc,
    index = rep(1:nrow(loc), 2),
    repl = rep(1:2, each = nrow(loc))
)
## Toy example with no spatial correlation (range=zero)
y <- 10 + rnorm(100 * 2)
stack <- inla.stack(
    data = list(y = y),
    A = list(spatial.A),
    effects = list(c(index, list(intercept = 1))),
    tag = "tag"
)
data <- inla.stack.data(stack, spde = spde)
formula <- y ~ -1 + intercept + f(spatial,
    model = spde,
    replicate = spatial.repl
)
result <- inla(formula,
    family = "gaussian", data = data,
    control.predictor = list(A = inla.stack.A(stack))
)
spde.result <- inla.spde.result(result, "spatial", spde)
plot(spde.result$marginals.range.nominal[[1]], type = "l")
}
\seealso{
\code{\link[=inla.spde.models]{inla.spde.models()}}, \code{\link[=inla.spde2.matern]{inla.spde2.matern()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from pc-t.R
%%
 \name{pc.ddof}
 \alias{pc.ddof}
 \alias{inla.pc.ddof}
 \alias{pc.t}
 \alias{pc.dof}
 \alias{inla.pc.t}
 \alias{inla.pc.dof}

 \title{PC-prior for dof in a standarized Student-t}

 \description{A function to evaluate the PC-prior for the degrees of freedom
              in a standarized Student-t distribution}
 \usage{
     inla.pc.ddof(dof, lambda, u, alpha, log=FALSE)
 }
 \arguments{
   \item{dof}{Degrees of freedom}
   \item{log}{Logical. Return the density or the log-density}
   \item{lambda}{The optional value of \code{lambda},  instead of defining it implicitely
                 through \code{u} and \code{alpha}}
   \item{u}{The upper value of dof used to elicitate \code{lambda},  \code{Prob(dof < u) = alpha}}
   \item{alpha}{The probability \code{alpha} used to elicitate \code{lambda}}
}
 \details{
   These functions implements the PC-prior for the dof in a standarized Student-t
   distribution (ie. with unit variance and \code{dof} > 2).
   Either \code{lambda}, or \code{u} AND \code{alpha} must be given.
   Due the internal tabulation, \code{dof} must be larger than 2.0025.
 }
 \value{%%
     \code{inla.pc.ddof} returns the prior density for given \code{dof}.
 }
 \author{Havard Rue \email{hrue@r-inla.org}}
 \examples{
 }

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from set.default.arguments.R
%%
\name{control.lincomb}
\alias{control.lincomb}
\alias{inla.control.lincomb}
\alias{inla.set.control.lincomb.default}
\alias{set.control.lincomb.default}
\alias{control.lincomb.default}
\title{Control variables in control.lincomb}
\description{Control variables in \code{control.lincomb} for use in \code{inla}}
\usage{
inla.set.control.lincomb.default(...)
control.lincomb(verbose)
}
\arguments{
\item{...}{Possible arguments}
\item{verbose}{   Use verbose mode for linear combinations if verbose model is set globally. (Default TRUE)}
}
\value{
  
The function \code{control.lincomb} is used to TAB-complete arguments and returns a list of given arguments.
The function \code{inla.set.control.lincomb.default} returns a list with all the default values of all parameters within this control statement.
}
\seealso{
\code{\link{control.update}}, \code{\link{control.lincomb}}, \code{\link{control.group}}, \code{\link{control.mix}}, \code{\link{control.link}}, \code{\link{control.expert}}, \code{\link{control.compute}}, \code{\link{control.pardiso.default}}, \code{\link{control.bgev.default}}, \code{\link{control.family}}, \code{\link{control.fixed}}, \code{\link{control.inla}}, \code{\link{control.predictor}}, \code{\link{control.mode}}, \code{\link{control.hazard}}, 
\code{\link{inla}}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from posterior.sample.R
%%
 \name{inla.sample}
 \alias{inla.posterior.sample}
 \alias{posterior.sample}
 \alias{inla.posterior.sample.eval}
 \alias{posterior.sample.eval}

 \title{Generate samples, and functions thereof, from an approximated posterior of a fitted model}

 \description{This function generate samples, and functions of those,
              from an approximated posterior of a fitted model (an inla-object)}
 \usage{
     inla.posterior.sample(n = 1L, result, selection = list(),
                           intern = FALSE,
                           use.improved.mean = TRUE, skew.corr = TRUE,
                           add.names = TRUE, seed = 0L, num.threads = NULL,
                           parallel.configs = TRUE,  verbose=FALSE)
     inla.posterior.sample.eval(fun, samples, return.matrix = TRUE, ...)
 }

 \arguments{
   \item{n}{Number of samples.}
   \item{result}{The inla-object, ie the output from an \code{inla}-call.
       The \code{inla}-object must be created with
       \code{control.compute=list(config=TRUE)}.}
   \item{selection}{Select what part of the sample to return. By default, the whole sample
       is returned. \code{selection} is a named list with the name of the components of
       the sample, and what indices of them to return. Names include \code{APredictor},
       \code{Predictor}, \code{(Intercept)},  and otherwise names in the formula.
       The values of the list, is interpreted as indices. If they
       are negative, they are interpreted as 'not', a zero is interpreted as 'all',  and
       positive indices are interpreted as 'only'. The names of elements of each samples
       refer to the indices in the full sample. }
   \item{intern}{Logical. If \code{TRUE} then produce samples in the
        internal scale for the hyperparmater, if \code{FALSE} then produce
        samples in the user-scale. (For example log-precision (intern)
        and precision (user-scale))}
   \item{use.improved.mean}{Logical. If \code{TRUE} then use the
        marginal mean values when constructing samples. If \code{FALSE}
        then use the mean in the Gaussian approximations.}
   \item{skew.corr}{Logical. If \code{TRUE} then correct samples for skewness,
       if \code{FALSE},  do not correct samples for skewness (ie use the
       Gaussian).}
   \item{add.names}{Logical. If \code{TRUE} then add name for each elements of each
       sample. If \code{FALSE}, only add name for the first sample.
       (This save space.)}
   \item{seed}{See the same argument in \code{?inla.qsample} for further
               information. In order to produce reproducible results,  you
               ALSO need to make sure the RNG in R is in the same state,
               see example below.  When \code{seed} is non-zero,
               \code{num.threads} is forced to "1:1" and parallel.configs is
               set to \code{FALSE}, since parallel sampling would not produce
               a reproducible sequence of pseudo-random numbers.}
   \item{num.threads}{The number of threads to use in the format 'A:B' defining the number threads in the
                      outer (A) and inner (B) layer for nested parallelism. A '0' will be replaced
                      intelligently.
                      \code{seed!=0} requires serial comptuations.}
   \item{parallel.configs}{Logical. If \code{TRUE} and not on Windows,
                           then try to run each configuration in
                           parallel (not Windows) using \code{A} threads (see \code{num.threads}),
                           where each of them is using \code{B:0} threads.}
   \item{verbose}{Logical. Run in verbose mode or not.}
   \item{fun}{The function to evaluate for each sample. Upon entry, the variable names
              defined in the model are defined as the value of the sample.
              The list of names are defined in \code{result$misc$configs$contents} where
              \code{result} is an \code{inla}-object. This includes predefined names for
              for the linear predictor (\code{Predictor} and \code{APredictor}),  and the
              intercept (\code{(Intercept)} or \code{Intercept}).
              The hyperparameters are defined as \code{theta},  no matter if they are in the
              internal scale or not. The function \code{fun} can also return a vector.
              To simplify usage, \code{fun} can also be a vector character's. In this case
              \code{fun} it is interpreted as (strict) variable
              names, and a function is created that return these variables:
              if argument \code{fun} equals \code{c("Intercept", "a[1:2]")},  then this is equivalent to
              pass \code{function() return(c(get('Intercept'), get('a[1:2]')))}.}
   \item{samples}{\code{samples} is the output from \code{inla.posterior.sample()}}
   \item{return.matrix}{Logical. If \code{TRUE},  then return the samples of \code{fun}
                         as matrix,  otherwise,  as a list.}
   \item{...}{Additional arguments to \code{fun}}
}
\details{The hyperparameters are sampled from the configurations used to do the
       numerical integration, hence if you want a higher resolution, you need to
       to change the \code{int.stratey} variable and friends. The latent field is
       sampled from the Gaussian approximation conditioned on the hyperparameters,
       but with a correction for the mean (default),
        and optional (and by default) corrected for the estimated skewness.

       The log.density report is only correct when there is no constraints.
       With constraints, it correct the Gaussian part of the sample for the constraints.

       After the sample is (optional) skewness corrected, the log.density
       is is not exact for correcting for constraints, but the error is very
       small in most cases.
}
\value{\code{inla.posterior.sample} returns a list of the samples,
       where each sample is a list with
     names \code{hyperpar} and \code{latent}, and with their marginal
     densities in \code{logdens$hyperpar} and \code{logdens$latent}
     and the joint density is in \code{logdens$joint}.
     \code{inla.posterior.sample.eval} return a list or a matrix of
     \code{fun} applied to each sample.
}
\author{Havard Rue \email{hrue@r-inla.org} and Cristian Chiuchiolo \email{cristian.chiuchiolo@kaust.edu.sa}}

\examples{
  r = inla(y ~ 1 ,data = data.frame(y=rnorm(1)), control.compute = list(config=TRUE))
  samples = inla.posterior.sample(2,r)

  ## reproducible results:
  inla.seed = as.integer(runif(1)*.Machine$integer.max)
  set.seed(12345)
  x = inla.posterior.sample(10, r, seed = inla.seed, num.threads="1:1")
  set.seed(12345)
  xx = inla.posterior.sample(10, r, seed = inla.seed, num.threads="1.1")
  all.equal(x, xx)

 set.seed(1234)
 n = 25
 xx = rnorm(n)
 yy = rev(xx)
 z = runif(n)
 y = rnorm(n)
 r = inla(y ~ 1 + z + f(xx) + f(yy, copy="xx"),
         data = data.frame(y, z, xx, yy),
         control.compute = list(config=TRUE),
         family = "gaussian")
 r.samples = inla.posterior.sample(10, r)

 fun = function(...) {
     mean(xx) - mean(yy)
 }
 f1 = inla.posterior.sample.eval(fun, r.samples)

 fun = function(...) {
     c(exp(Intercept), exp(Intercept + z))
 }
 f2 = inla.posterior.sample.eval(fun, r.samples)

 fun = function(...) {
     return (theta[1]/(theta[1] + theta[2]))
 }
 f3 = inla.posterior.sample.eval(fun, r.samples)

 ## Predicting nz new observations, and
 ## comparing the estimated one with the true one
 set.seed(1234)
 n = 100
 alpha = beta = s = 1
 z = rnorm(n)
 y = alpha + beta * z + rnorm(n, sd = s)
 r = inla(y ~ 1 + z,
         data = data.frame(y, z),
         control.compute = list(config=TRUE),
         family = "gaussian")
 r.samples = inla.posterior.sample(10^3, r)

 ## just return samples of the intercept
 intercepts = inla.posterior.sample.eval("Intercept", r.samples)

 nz = 3
 znew = rnorm(nz)
 fun = function(zz = NA) {
     ## theta[1] is the precision
     return (Intercept + z * zz +
             rnorm(length(zz), sd = sqrt(1/theta[1])))
 }
 par(mfrow=c(1, nz))
 f1 = inla.posterior.sample.eval(fun, r.samples, zz = znew)
 for(i in 1:nz) {
     hist(f1[i, ], n = 100, prob = TRUE)
     m = alpha + beta * znew[i]
     xx = seq(m-4*s, m+4*s, by = s/100)
     lines(xx, dnorm(xx, mean=m, sd = s), lwd=2)
 }

 ## 
 ## Be aware that using non-clean variable names might be a little tricky
 ## 
 n <- 100
 X <- matrix(rnorm(n^2), n, 2)
 x <- X[, 1]
 xx <- X[, 2]
 xxx <- x*xx
 
 y <- 1 + 2*x + 3*xx + 4*xxx + rnorm(n, sd = 0.01)
 
 r <- inla(y ~ X[, 1]*X[, 2],
           data = list(y = y, X = X),
           control.compute = list(config = TRUE))
 print(round(dig = 4, r$summary.fixed[,"mean"]))
 
 sam <- inla.posterior.sample(100, r)
 sam.extract <- inla.posterior.sample.eval(
     (function(...) {
         beta.1 <- get("X[, 1]")
         beta.2 <- get("X[, 2]")
         beta.12 <- get("X[, 1]:X[, 2]")
         return(c(Intercept, beta.1, beta.2, beta.12))
     }), sam)
 print(round(dig = 4, rowMeans(sam.extract)))
 
 ## a simpler form can also be used here, and in the examples below
 sam.extract <- inla.posterior.sample.eval(
                c("Intercept", "X[, 1]", "X[, 2]", "X[, 1]:X[, 2]"), sam)
 print(round(dig = 4, rowMeans(sam.extract)))

 r <- inla(y ~ x + xx + xxx,
           data = list(y = y, x = x, xx = xx, xxx = xxx), 
           control.compute = list(config = TRUE))
 
 sam <- inla.posterior.sample(100, r)
 sam.extract <- inla.posterior.sample.eval(
     (function(...) {
         return(c(Intercept, x, xx, xxx))
     }), sam)
 print(round(dig = 4, rowMeans(sam.extract)))

 sam.extract <- inla.posterior.sample.eval(c("Intercept", "x", "xx", "xxx"), sam)
 print(round(dig = 4, rowMeans(sam.extract)))

 r <- inla(y ~ x*xx,
           data = list(y = y, x = x, xx = xx), 
           control.compute = list(config = TRUE))
 
 sam <- inla.posterior.sample(100, r)
 sam.extract <- inla.posterior.sample.eval(
     (function(...) {
         return(c(Intercept, x, xx, get("x:xx")))
     }), sam)
 print(round(dig = 4, rowMeans(sam.extract)))

 sam.extract <- inla.posterior.sample.eval(c("Intercept", "x", "xx", "x:xx"), sam)
 print(round(dig = 4, rowMeans(sam.extract)))
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{PRprec}
\alias{PRprec}
\title{The PRprec data}
\format{
A data frame .... TODO
\describe{
\item{Altitude}{TODO}
\item{Latitude}{TODO}
\item{Longitude}{TODO}
\item{d0101}{Daily rainfall at day "mmdd"}
\item{d0102}{Daily rainfall at day "mmdd"}
\item{d0103}{Daily rainfall at day "mmdd"}
\item{d0104}{Daily rainfall at day "mmdd"}
\item{d0105}{Daily rainfall at day "mmdd"}
\item{d0106}{Daily rainfall at day "mmdd"}
\item{d0107}{Daily rainfall at day "mmdd"} \item{d0108}{Daily
rainfall at day "mmdd"} \item{d0109}{Daily rainfall at day "mmdd"}
\item{d0110}{Daily rainfall at day "mmdd"} \item{d0111}{Daily rainfall at
day "mmdd"} \item{d0112}{Daily rainfall at day "mmdd"} \item{d0113}{Daily
rainfall at day "mmdd"} \item{d0114}{Daily rainfall at day "mmdd"}
\item{d0115}{Daily rainfall at day "mmdd"} \item{d0116}{Daily rainfall at
day "mmdd"} \item{d0117}{Daily rainfall at day "mmdd"} \item{d0118}{Daily
rainfall at day "mmdd"} \item{d0119}{Daily rainfall at day "mmdd"}
\item{d0120}{Daily rainfall at day "mmdd"} \item{d0121}{Daily rainfall at
day "mmdd"} \item{d0122}{Daily rainfall at day "mmdd"} \item{d0123}{Daily
rainfall at day "mmdd"} \item{d0124}{Daily rainfall at day "mmdd"}
\item{d0125}{Daily rainfall at day "mmdd"} \item{d0126}{Daily rainfall at
day "mmdd"} \item{d0127}{Daily rainfall at day "mmdd"} \item{d0128}{Daily
rainfall at day "mmdd"} \item{d0129}{Daily rainfall at day "mmdd"}
\item{d0130}{Daily rainfall at day "mmdd"} \item{d0131}{Daily rainfall at
day "mmdd"} \item{d0201}{Daily rainfall at day "mmdd"} \item{d0202}{Daily
rainfall at day "mmdd"} \item{d0203}{Daily rainfall at day "mmdd"}
\item{d0204}{Daily rainfall at day "mmdd"} \item{d0205}{Daily rainfall at
day "mmdd"} \item{d0206}{Daily rainfall at day "mmdd"} \item{d0207}{Daily
rainfall at day "mmdd"} \item{d0208}{Daily rainfall at day "mmdd"}
\item{d0209}{Daily rainfall at day "mmdd"} \item{d0210}{Daily rainfall at
day "mmdd"} \item{d0211}{Daily rainfall at day "mmdd"} \item{d0212}{Daily
rainfall at day "mmdd"} \item{d0213}{Daily rainfall at day "mmdd"}
\item{d0214}{Daily rainfall at day "mmdd"} \item{d0215}{Daily rainfall at
day "mmdd"} \item{d0216}{Daily rainfall at day "mmdd"} \item{d0217}{Daily
rainfall at day "mmdd"} \item{d0218}{Daily rainfall at day "mmdd"}
\item{d0219}{Daily rainfall at day "mmdd"} \item{d0220}{Daily rainfall at
day "mmdd"} \item{d0221}{Daily rainfall at day "mmdd"} \item{d0222}{Daily
rainfall at day "mmdd"} \item{d0223}{Daily rainfall at day "mmdd"}
\item{d0224}{Daily rainfall at day "mmdd"} \item{d0225}{Daily rainfall at
day "mmdd"} \item{d0226}{Daily rainfall at day "mmdd"} \item{d0227}{Daily
rainfall at day "mmdd"} \item{d0228}{Daily rainfall at day "mmdd"}
\item{d0301}{Daily rainfall at day "mmdd"} \item{d0302}{Daily rainfall at
day "mmdd"} \item{d0303}{Daily rainfall at day "mmdd"} \item{d0304}{Daily
rainfall at day "mmdd"} \item{d0305}{Daily rainfall at day "mmdd"}
\item{d0306}{Daily rainfall at day "mmdd"} \item{d0307}{Daily rainfall at
day "mmdd"} \item{d0308}{Daily rainfall at day "mmdd"} \item{d0309}{Daily
rainfall at day "mmdd"} \item{d0310}{Daily rainfall at day "mmdd"}
\item{d0311}{Daily rainfall at day "mmdd"} \item{d0312}{Daily rainfall at
day "mmdd"} \item{d0313}{Daily rainfall at day "mmdd"} \item{d0314}{Daily
rainfall at day "mmdd"} \item{d0315}{Daily rainfall at day "mmdd"}
\item{d0316}{Daily rainfall at day "mmdd"} \item{d0317}{Daily rainfall at
day "mmdd"} \item{d0318}{Daily rainfall at day "mmdd"} \item{d0319}{Daily
rainfall at day "mmdd"} \item{d0320}{Daily rainfall at day "mmdd"}
\item{d0321}{Daily rainfall at day "mmdd"} \item{d0322}{Daily rainfall at
day "mmdd"} \item{d0323}{Daily rainfall at day "mmdd"} \item{d0324}{Daily
rainfall at day "mmdd"} \item{d0325}{Daily rainfall at day "mmdd"}
\item{d0326}{Daily rainfall at day "mmdd"} \item{d0327}{Daily rainfall at
day "mmdd"} \item{d0328}{Daily rainfall at day "mmdd"} \item{d0329}{Daily
rainfall at day "mmdd"} \item{d0330}{Daily rainfall at day "mmdd"}
\item{d0331}{Daily rainfall at day "mmdd"} \item{d0401}{Daily rainfall at
day "mmdd"} \item{d0402}{Daily rainfall at day "mmdd"} \item{d0403}{Daily
rainfall at day "mmdd"} \item{d0404}{Daily rainfall at day "mmdd"}
\item{d0405}{Daily rainfall at day "mmdd"} \item{d0406}{Daily rainfall at
day "mmdd"} \item{d0407}{Daily rainfall at day "mmdd"} \item{d0408}{Daily
rainfall at day "mmdd"} \item{d0409}{Daily rainfall at day "mmdd"}
\item{d0410}{Daily rainfall at day "mmdd"} \item{d0411}{Daily rainfall at
day "mmdd"} \item{d0412}{Daily rainfall at day "mmdd"} \item{d0413}{Daily
rainfall at day "mmdd"} \item{d0414}{Daily rainfall at day "mmdd"}
\item{d0415}{Daily rainfall at day "mmdd"} \item{d0416}{Daily rainfall at
day "mmdd"} \item{d0417}{Daily rainfall at day "mmdd"} \item{d0418}{Daily
rainfall at day "mmdd"} \item{d0419}{Daily rainfall at day "mmdd"}
\item{d0420}{Daily rainfall at day "mmdd"} \item{d0421}{Daily rainfall at
day "mmdd"} \item{d0422}{Daily rainfall at day "mmdd"} \item{d0423}{Daily
rainfall at day "mmdd"} \item{d0424}{Daily rainfall at day "mmdd"}
\item{d0425}{Daily rainfall at day "mmdd"} \item{d0426}{Daily rainfall at
day "mmdd"} \item{d0427}{Daily rainfall at day "mmdd"} \item{d0428}{Daily
rainfall at day "mmdd"} \item{d0429}{Daily rainfall at day "mmdd"}
\item{d0430}{Daily rainfall at day "mmdd"} \item{d0501}{Daily rainfall at
day "mmdd"} \item{d0502}{Daily rainfall at day "mmdd"} \item{d0503}{Daily
rainfall at day "mmdd"} \item{d0504}{Daily rainfall at day "mmdd"}
\item{d0505}{Daily rainfall at day "mmdd"} \item{d0506}{Daily rainfall at
day "mmdd"} \item{d0507}{Daily rainfall at day "mmdd"} \item{d0508}{Daily
rainfall at day "mmdd"} \item{d0509}{Daily rainfall at day "mmdd"}
\item{d0510}{Daily rainfall at day "mmdd"} \item{d0511}{Daily rainfall at
day "mmdd"} \item{d0512}{Daily rainfall at day "mmdd"} \item{d0513}{Daily
rainfall at day "mmdd"} \item{d0514}{Daily rainfall at day "mmdd"}
\item{d0515}{Daily rainfall at day "mmdd"} \item{d0516}{Daily rainfall at
day "mmdd"} \item{d0517}{Daily rainfall at day "mmdd"} \item{d0518}{Daily
rainfall at day "mmdd"} \item{d0519}{Daily rainfall at day "mmdd"}
\item{d0520}{Daily rainfall at day "mmdd"} \item{d0521}{Daily rainfall at
day "mmdd"} \item{d0522}{Daily rainfall at day "mmdd"} \item{d0523}{Daily
rainfall at day "mmdd"} \item{d0524}{Daily rainfall at day "mmdd"}
\item{d0525}{Daily rainfall at day "mmdd"} \item{d0526}{Daily rainfall at
day "mmdd"} \item{d0527}{Daily rainfall at day "mmdd"} \item{d0528}{Daily
rainfall at day "mmdd"} \item{d0529}{Daily rainfall at day "mmdd"}
\item{d0530}{Daily rainfall at day "mmdd"} \item{d0531}{Daily rainfall at
day "mmdd"} \item{d0601}{Daily rainfall at day "mmdd"} \item{d0602}{Daily
rainfall at day "mmdd"} \item{d0603}{Daily rainfall at day "mmdd"}
\item{d0604}{Daily rainfall at day "mmdd"} \item{d0605}{Daily rainfall at
day "mmdd"} \item{d0606}{Daily rainfall at day "mmdd"} \item{d0607}{Daily
rainfall at day "mmdd"} \item{d0608}{Daily rainfall at day "mmdd"}
\item{d0609}{Daily rainfall at day "mmdd"} \item{d0610}{Daily rainfall at
day "mmdd"} \item{d0611}{Daily rainfall at day "mmdd"} \item{d0612}{Daily
rainfall at day "mmdd"} \item{d0613}{Daily rainfall at day "mmdd"}
\item{d0614}{Daily rainfall at day "mmdd"} \item{d0615}{Daily rainfall at
day "mmdd"} \item{d0616}{Daily rainfall at day "mmdd"} \item{d0617}{Daily
rainfall at day "mmdd"} \item{d0618}{Daily rainfall at day "mmdd"}
\item{d0619}{Daily rainfall at day "mmdd"} \item{d0620}{Daily rainfall at
day "mmdd"} \item{d0621}{Daily rainfall at day "mmdd"} \item{d0622}{Daily
rainfall at day "mmdd"} \item{d0623}{Daily rainfall at day "mmdd"}
\item{d0624}{Daily rainfall at day "mmdd"} \item{d0625}{Daily rainfall at
day "mmdd"} \item{d0626}{Daily rainfall at day "mmdd"} \item{d0627}{Daily
rainfall at day "mmdd"} \item{d0628}{Daily rainfall at day "mmdd"}
\item{d0629}{Daily rainfall at day "mmdd"} \item{d0630}{Daily rainfall at
day "mmdd"} \item{d0701}{Daily rainfall at day "mmdd"} \item{d0702}{Daily
rainfall at day "mmdd"} \item{d0703}{Daily rainfall at day "mmdd"}
\item{d0704}{Daily rainfall at day "mmdd"} \item{d0705}{Daily rainfall at
day "mmdd"} \item{d0706}{Daily rainfall at day "mmdd"} \item{d0707}{Daily
rainfall at day "mmdd"} \item{d0708}{Daily rainfall at day "mmdd"}
\item{d0709}{Daily rainfall at day "mmdd"} \item{d0710}{Daily rainfall at
day "mmdd"} \item{d0711}{Daily rainfall at day "mmdd"} \item{d0712}{Daily
rainfall at day "mmdd"} \item{d0713}{Daily rainfall at day "mmdd"}
\item{d0714}{Daily rainfall at day "mmdd"} \item{d0715}{Daily rainfall at
day "mmdd"} \item{d0716}{Daily rainfall at day "mmdd"} \item{d0717}{Daily
rainfall at day "mmdd"} \item{d0718}{Daily rainfall at day "mmdd"}
\item{d0719}{Daily rainfall at day "mmdd"} \item{d0720}{Daily rainfall at
day "mmdd"} \item{d0721}{Daily rainfall at day "mmdd"} \item{d0722}{Daily
rainfall at day "mmdd"} \item{d0723}{Daily rainfall at day "mmdd"}
\item{d0724}{Daily rainfall at day "mmdd"} \item{d0725}{Daily rainfall at
day "mmdd"} \item{d0726}{Daily rainfall at day "mmdd"} \item{d0727}{Daily
rainfall at day "mmdd"} \item{d0728}{Daily rainfall at day "mmdd"}
\item{d0729}{Daily rainfall at day "mmdd"} \item{d0730}{Daily rainfall at
day "mmdd"} \item{d0731}{Daily rainfall at day "mmdd"} \item{d0801}{Daily
rainfall at day "mmdd"} \item{d0802}{Daily rainfall at day "mmdd"}
\item{d0803}{Daily rainfall at day "mmdd"} \item{d0804}{Daily rainfall at
day "mmdd"} \item{d0805}{Daily rainfall at day "mmdd"} \item{d0806}{Daily
rainfall at day "mmdd"} \item{d0807}{Daily rainfall at day "mmdd"}
\item{d0808}{Daily rainfall at day "mmdd"} \item{d0809}{Daily rainfall at
day "mmdd"} \item{d0810}{Daily rainfall at day "mmdd"} \item{d0811}{Daily
rainfall at day "mmdd"} \item{d0812}{Daily rainfall at day "mmdd"}
\item{d0813}{Daily rainfall at day "mmdd"} \item{d0814}{Daily rainfall at
day "mmdd"} \item{d0815}{Daily rainfall at day "mmdd"} \item{d0816}{Daily
rainfall at day "mmdd"} \item{d0817}{Daily rainfall at day "mmdd"}
\item{d0818}{Daily rainfall at day "mmdd"} \item{d0819}{Daily rainfall at
day "mmdd"} \item{d0820}{Daily rainfall at day "mmdd"} \item{d0821}{Daily
rainfall at day "mmdd"} \item{d0822}{Daily rainfall at day "mmdd"}
\item{d0823}{Daily rainfall at day "mmdd"} \item{d0824}{Daily rainfall at
day "mmdd"} \item{d0825}{Daily rainfall at day "mmdd"} \item{d0826}{Daily
rainfall at day "mmdd"} \item{d0827}{Daily rainfall at day "mmdd"}
\item{d0828}{Daily rainfall at day "mmdd"} \item{d0829}{Daily rainfall at
day "mmdd"} \item{d0830}{Daily rainfall at day "mmdd"} \item{d0831}{Daily
rainfall at day "mmdd"} \item{d0901}{Daily rainfall at day "mmdd"}
\item{d0902}{Daily rainfall at day "mmdd"} \item{d0903}{Daily rainfall at
day "mmdd"} \item{d0904}{Daily rainfall at day "mmdd"} \item{d0905}{Daily
rainfall at day "mmdd"} \item{d0906}{Daily rainfall at day "mmdd"}
\item{d0907}{Daily rainfall at day "mmdd"} \item{d0908}{Daily rainfall at
day "mmdd"} \item{d0909}{Daily rainfall at day "mmdd"} \item{d0910}{Daily
rainfall at day "mmdd"} \item{d0911}{Daily rainfall at day "mmdd"}
\item{d0912}{Daily rainfall at day "mmdd"} \item{d0913}{Daily rainfall at
day "mmdd"} \item{d0914}{Daily rainfall at day "mmdd"} \item{d0915}{Daily
rainfall at day "mmdd"} \item{d0916}{Daily rainfall at day "mmdd"}
\item{d0917}{Daily rainfall at day "mmdd"} \item{d0918}{Daily rainfall at
day "mmdd"} \item{d0919}{Daily rainfall at day "mmdd"} \item{d0920}{Daily
rainfall at day "mmdd"} \item{d0921}{Daily rainfall at day "mmdd"}
\item{d0922}{Daily rainfall at day "mmdd"} \item{d0923}{Daily rainfall at
day "mmdd"} \item{d0924}{Daily rainfall at day "mmdd"} \item{d0925}{Daily
rainfall at day "mmdd"} \item{d0926}{Daily rainfall at day "mmdd"}
\item{d0927}{Daily rainfall at day "mmdd"} \item{d0928}{Daily rainfall at
day "mmdd"} \item{d0929}{Daily rainfall at day "mmdd"} \item{d0930}{Daily
rainfall at day "mmdd"} \item{d1001}{Daily rainfall at day "mmdd"}
\item{d1002}{Daily rainfall at day "mmdd"} \item{d1003}{Daily rainfall at
day "mmdd"} \item{d1004}{Daily rainfall at day "mmdd"} \item{d1005}{Daily
rainfall at day "mmdd"} \item{d1006}{Daily rainfall at day "mmdd"}
\item{d1007}{Daily rainfall at day "mmdd"} \item{d1008}{Daily rainfall at
day "mmdd"} \item{d1009}{Daily rainfall at day "mmdd"} \item{d1010}{Daily
rainfall at day "mmdd"} \item{d1011}{Daily rainfall at day "mmdd"}
\item{d1012}{Daily rainfall at day "mmdd"} \item{d1013}{Daily rainfall at
day "mmdd"} \item{d1014}{Daily rainfall at day "mmdd"} \item{d1015}{Daily
rainfall at day "mmdd"} \item{d1016}{Daily rainfall at day "mmdd"}
\item{d1017}{Daily rainfall at day "mmdd"} \item{d1018}{Daily rainfall at
day "mmdd"} \item{d1019}{Daily rainfall at day "mmdd"} \item{d1020}{Daily
rainfall at day "mmdd"} \item{d1021}{Daily rainfall at day "mmdd"}
\item{d1022}{Daily rainfall at day "mmdd"} \item{d1023}{Daily rainfall at
day "mmdd"} \item{d1024}{Daily rainfall at day "mmdd"} \item{d1025}{Daily
rainfall at day "mmdd"} \item{d1026}{Daily rainfall at day "mmdd"}
\item{d1027}{Daily rainfall at day "mmdd"} \item{d1028}{Daily rainfall at
day "mmdd"} \item{d1029}{Daily rainfall at day "mmdd"} \item{d1030}{Daily
rainfall at day "mmdd"} \item{d1031}{Daily rainfall at day "mmdd"}
\item{d1101}{Daily rainfall at day "mmdd"} \item{d1102}{Daily rainfall at
day "mmdd"} \item{d1103}{Daily rainfall at day "mmdd"} \item{d1104}{Daily
rainfall at day "mmdd"} \item{d1105}{Daily rainfall at day "mmdd"}
\item{d1106}{Daily rainfall at day "mmdd"} \item{d1107}{Daily rainfall at
day "mmdd"} \item{d1108}{Daily rainfall at day "mmdd"} \item{d1109}{Daily
rainfall at day "mmdd"} \item{d1110}{Daily rainfall at day "mmdd"}
\item{d1111}{Daily rainfall at day "mmdd"} \item{d1112}{Daily rainfall at
day "mmdd"} \item{d1113}{Daily rainfall at day "mmdd"} \item{d1114}{Daily
rainfall at day "mmdd"} \item{d1115}{Daily rainfall at day "mmdd"}
\item{d1116}{Daily rainfall at day "mmdd"} \item{d1117}{Daily rainfall at
day "mmdd"} \item{d1118}{Daily rainfall at day "mmdd"} \item{d1119}{Daily
rainfall at day "mmdd"} \item{d1120}{Daily rainfall at day "mmdd"}
\item{d1121}{Daily rainfall at day "mmdd"} \item{d1122}{Daily rainfall at
day "mmdd"} \item{d1123}{Daily rainfall at day "mmdd"} \item{d1124}{Daily
rainfall at day "mmdd"} \item{d1125}{Daily rainfall at day "mmdd"}
\item{d1126}{Daily rainfall at day "mmdd"} \item{d1127}{Daily rainfall at
day "mmdd"} \item{d1128}{Daily rainfall at day "mmdd"} \item{d1129}{Daily
rainfall at day "mmdd"} \item{d1130}{Daily rainfall at day "mmdd"}
\item{d1201}{Daily rainfall at day "mmdd"} \item{d1202}{Daily rainfall at
day "mmdd"} \item{d1203}{Daily rainfall at day "mmdd"} \item{d1204}{Daily
rainfall at day "mmdd"} \item{d1205}{Daily rainfall at day "mmdd"}
\item{d1206}{Daily rainfall at day "mmdd"} \item{d1207}{Daily rainfall at
day "mmdd"} \item{d1208}{Daily rainfall at day "mmdd"} \item{d1209}{Daily
rainfall at day "mmdd"} \item{d1210}{Daily rainfall at day "mmdd"}
\item{d1211}{Daily rainfall at day "mmdd"} \item{d1212}{Daily rainfall at
day "mmdd"} \item{d1213}{Daily rainfall at day "mmdd"} \item{d1214}{Daily
rainfall at day "mmdd"} \item{d1215}{Daily rainfall at day "mmdd"}
\item{d1216}{Daily rainfall at day "mmdd"} \item{d1217}{Daily rainfall at
day "mmdd"} \item{d1218}{Daily rainfall at day "mmdd"} \item{d1219}{Daily
rainfall at day "mmdd"} \item{d1220}{Daily rainfall at day "mmdd"}
\item{d1221}{Daily rainfall at day "mmdd"} \item{d1222}{Daily rainfall at
day "mmdd"} \item{d1223}{Daily rainfall at day "mmdd"} \item{d1224}{Daily
rainfall at day "mmdd"} \item{d1225}{Daily rainfall at day "mmdd"}
\item{d1226}{Daily rainfall at day "mmdd"} \item{d1227}{Daily rainfall at
day "mmdd"} \item{d1228}{Daily rainfall at day "mmdd"} \item{d1229}{Daily
rainfall at day "mmdd"} \item{d1230}{Daily rainfall at day "mmdd"}
\item{d1231}{Daily rainfall at day "mmdd"} }
}
\description{
A data frame with daily rainfall in the Parana State.
}
\seealso{
PRborder
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.extract.el}
\alias{inla.extract.el}
\alias{inla.extract.el.data.frame}
\alias{inla.extract.el.list}
\alias{inla.extract.el.matrix}
\title{Extract elements by matching name from container objects.}
\usage{
inla.extract.el(M, ...)

\method{inla.extract.el}{matrix}(M, match, by.row = TRUE, ...)

\method{inla.extract.el}{data.frame}(M, match, by.row = TRUE, ...)

\method{inla.extract.el}{list}(M, match, ...)
}
\arguments{
\item{M}{A container object.}

\item{\dots}{Additional arguments, not used.}

\item{match}{A regex defining the matching criterion.}

\item{by.row}{If \code{TRUE}, extract data by row, otherwise by column.}
}
\description{
Extract elements by wildcard name matching from a \code{data.frame},
\code{list}, or \code{matrix}.
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from cpo.R
%%
\name{inla.cpo}
\alias{inla.cpo}
\alias{cpo.inla}

\title{Improved estimates for the CPO/PIT-values}
\description{
 Improve the estimates of the  CPO/PIT-values
 be recomputing the model-fit by removing data-points.
}
\usage{
inla.cpo(result,
         force = FALSE,
         mc.cores = NULL,
         verbose = TRUE,
         recompute.mode = TRUE)
}

\arguments{
\item{result}{An object of class \code{inla}, ie a result
of a call to \code{inla()}}
\item{force}{If TRUE,  then recompute all CPO/PIT values and not just
those with \code{result$cpo$failure > 0}.}
\item{mc.cores}{The number of cores to use in \code{parallel::mclapply}. If
                \code{is.null(mc.cores)}, then check \code{getOption("mc.cores")}
                and \code{inla.getOption("num.threads")} in that order.}
\item{verbose}{Run in verbose mode?}
\item{recompute.mode}{Should be mode (and the integration points) be recomputed when a data-point is removed or not?}
}
\value{
The object returned is the same as  \code{result} but the
new improved estimates of the CPO/PIT values replaced.}
\author{Havard Rue \email{hrue@r-inla.org}}
\examples{
n = 10
y = rnorm(n)
r = inla(y ~ 1, data = data.frame(y), control.compute = list(cpo=TRUE))
\donttest{
rr = inla.cpo(r, force=TRUE)
}
}
\seealso{\code{\link{inla}}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/INLA-package.R
\docType{data}
\name{Zambia}
\alias{Zambia}
\title{Semiparametric regression}
\format{
A data frame with 4847 observations on the following 10 variables.
\describe{ \item{hazstd}{standardised Z score of stunting}
\item{bmi}{body mass index of the mother} \item{agc}{age of
the child in months} \item{district}{district where the child lives}
\item{rcw}{mother employment status with categories "working" (1)
and "not working" (-1)} \item{edu1}{mother's educations status with
categories "complete primary but incomplete secondary " (\verb{edu1=1)},
"complete secondary or higher" (\code{edu2=1}) and "no education or
incomplete primary" (\code{edu1=edu2=-1})} \item{edu2}{see above}
\item{tpr}{locality of the domicile with categories "urban" (1) and
"rural" (-1)} \item{sex}{gender of the child with categories "male"
(1) and "female" (-1)} \item{edu}{DO NOT KNOW; check source} }
}
\source{
BayesX Manual
\url{http://www.stat.uni-muenchen.de/~bayesx/bayesx.html}
}
\description{
Undernutrition of children in each region of Zambia is measured through a
score computed on the basis of some anthropometric measures. The data set
contains also other infomation about each child.
}
\examples{

data(Zambia)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R
\name{inla.dBind}
\alias{inla.dBind}
\alias{inla.dBind-deprecated}
\title{Build a block-diagonal sparse matrix.}
\usage{
inla.dBind(...)
}
\arguments{
\item{\dots}{A list of square or rectangular matrices.}
}
\value{
A sparse matrix.
}
\description{
Build a block-diagonal sparse matrix.  Obsolete wrapper for \code{bdiag()}.
}
\examples{

inla.dBind(Matrix(1, 2, 1), Matrix(2, 1, 2))
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
\keyword{internal}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from agaussian.R
%%
 \name{aggregate.gaussian}
 \alias{inla.agaussian}
 \alias{aggregate.gaussian}
 \alias{agaussian}

 \title{Aggregate Gaussian into an equivalent observation}

 \description{Aggreagate Gaussians observerd with the same mean and precision,
              into an eqv triplet, for use with \code{family="agaussian"}}

 \usage{
     inla.agaussian(y, s = NULL)
 }

 \arguments{
   \item{y}{Repeated observations. If \code{y} is a matrix,  then each row represents
            repeated observations. if \code{y} is a list,  then each element of the list is
            a vector of repeated observations. If \code{y} is a vector,  then the whole vector
            represents repeated observations. The optional scaling \code{s},  must have 
            the same format as \code{y},  ie \code{matrix} or \code{vector}.
            \code{NA}'s in \code{y} (and \code{s}) are removed and not used or counted.
            If \code{s} is given, then the \code{NA}-pattern in \code{y} and \code{s} must be the same.}
   \item{s}{Optinonal fixed scaling of the precisions. Must be in the same format as \code{y},  and
            have the same \code{NA}-pattern. See the documentation for details.}
  }

 \value{
   The output is a \code{inla.mdata}-object ready for use
   with \code{family="agaussian"}. See the example in the documentation.
 }

 \author{Havard Rue \email{hrue@r-inla.org}}

 \examples{
  A = matrix(1:25,5,5)
  inla.agaussian(A)

  A[1,-1] = NA
  A[2,-(2:3)] = NA
  inla.agaussian(A)
 }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.R
\name{inla.mesh.query}
\alias{inla.mesh.query}
\title{High-quality triangulations}
\usage{
inla.mesh.query(mesh, ...)
}
\arguments{
\item{mesh}{An \code{inla.mesh} object.}

\item{\dots}{Query arguments.
\itemize{
\item tt.neighbours Compute
neighbour triangles for triangles; list of vectors: list(triangles, orders)

\item vt.neighbours Compute neighbour triangles for vertices; list of
vectors: list(vertices, orders)
}}
}
\value{
A list of query results.
}
\description{
Query information about an inla.mesh object.
}
\examples{

loc <- matrix(c(0.1, 0.15), 1, 2)
lattice <- inla.mesh.lattice(dims = c(10, 10))
mesh <- inla.mesh.create(loc = loc, lattice = lattice, extend = FALSE)

vt <- which(inla.mesh.query(mesh,
    vt.neighbours = list(
        mesh$idx$loc,
        4:6
    )
)$vt.neighbours)

mesh2 <- inla.mesh.create(mesh$loc,
    tv = mesh$graph$tv[vt, , drop = FALSE],
    refine = FALSE, extend = FALSE
)
}
\seealso{
\code{\link[=inla.mesh.create]{inla.mesh.create()}}, \code{\link[=inla.mesh.segment]{inla.mesh.segment()}},
\code{\link[=inla.mesh.lattice]{inla.mesh.lattice()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}

%%
%% WARNING! DO NOT EDIT!
%% This file is automatically generated from options.R
%%
 \name{inla.option}
 \alias{inla.option}
 \alias{inla.options}
 \alias{inla.setOption}
 \alias{inla.getOption}

 \title{Set and get global options for INLA}

 \description{Set and get global options for INLA}
 \usage{
 inla.setOption(...)
 inla.getOption(option)
 }

 \arguments{

   \item{...}{Option and value,  like \code{option=value} or \code{option, value}; see the Examples}
   \item{option}{The option to get. If \code{option = NULL} then
     \code{inla.getOption} then \code{inla.getOption} will return a named list of
     current values, otherwise, \code{option} must be one of

     inla.call: The path to the inla-program.

     inla.arg: Additional arguments to \code{inla.call}

     fmesher.call: The path to the fmesher-program

     fmesher.arg: Additional arguments to \code{fmesher.call}

     num.threads: Character string with the number of threads to use as \code{A:B}

     blas.num.threads: Number of threads to use for openblas and mklblas (see \code{inla} for details)

     smtp: Sparse matrix library to use,  one of \code{band}, \code{taucs} (\code{default}) or \code{pardiso}

     mkl: Use binaries buildt with Intel MKL?  (If possible)

     vecLib: This option applies to Mac only. If TRUE and mkl=FALSE, link with vecLib BLAS and LAPACK libs (if available)

     vecLibPath: This option applies to Mac only. Path to vecLib-libraries. If empty, use default.

     pardiso.license: The full path to the PARDISO license file

     keep: Keep temporary files?

     working.directory: The name of the working directory.

     silent: Run the inla-program in a silent mode?

     debug : Run the inla-program in a debug mode?

     cygwin : The home of the Cygwin installation (default "C:/cygwin") [Remote computing for Windows only]

     ssh.auth.sock: The ssh bind-adress (value of $SSH_AUTH_SOCK int the
     Cygwin-shell). [Remote computing for Windows only]

     show.warning.graph.file : Give a warning for using the obsolete argument
                               \code{graph.file} instead of \code{graph}

     scale.model.default : The default value of argument \code{scale.model} which
                           optionally scale intrinisic models to have generalized
                           unit average variance

     short.summary : Use a less verbose output for \code{summary}. Useful for Markdown
                     documents.

     inla.timeout : The timeout limit, in whole seconds, for calls to the
       inla binary. Default is 0, meaning no timeout limit.  Set to a
       positive integer to terminate inla calls if they run to long.
       Fractional seconds are
       rounded up to the nearest integer. This feature is EXPERIMENTAL
       and might change at a later stage. 

     fmesher.timeout : The timeout limit, in whole seconds, for calls to the
       fmesher binary. Default is 0, meaning no timeout limit.  Set to a
       positive integer to terminate fmesher calls that may enter infinite
       loops due to special geometry regularity. Fractional seconds are
       rounded up to the nearest integer.

     inla.mode : Which mode to use in INLA? Default is \code{"classic"}. Other options are
     (USE AT YOUR OWN RISK, THIS IS WORK IN PROGRESS!!!) options are \code{"twostage"} and
     \code{"experimental"}.
   }
 }

 \author{Havard Rue \email{hrue@r-inla.org}}

 \examples{
 ## set number of threads
 inla.setOption("num.threads", "4:1")
 ## alternative format
 inla.setOption(num.threads="4:1")
 ## check it
 inla.getOption("num.threads")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spde.common.R, R/spde1.R, R/spde2.R
\name{inla.spde.precision}
\alias{inla.spde.precision}
\alias{inla.spde1.precision}
\alias{inla.spde2.precision}
\alias{inla.spde.precision.inla.spde1}
\alias{inla.spde.precision.inla.spde2}
\title{Precision matrices for SPDE models

Calculates the precision matrix for given parameter values based on an
\code{inla.spde} model object.}
\usage{
inla.spde.precision(...)

inla.spde1.precision(spde, ...)

\method{inla.spde.precision}{inla.spde1}(spde, ...)

inla.spde2.precision(
  spde,
  theta = NULL,
  phi0 = inla.spde2.theta2phi0(spde, theta),
  phi1 = inla.spde2.theta2phi1(spde, theta),
  phi2 = inla.spde2.theta2phi2(spde, theta),
  ...
)

\method{inla.spde.precision}{inla.spde2}(
  spde,
  theta = NULL,
  phi0 = inla.spde2.theta2phi0(spde, theta),
  phi1 = inla.spde2.theta2phi1(spde, theta),
  phi2 = inla.spde2.theta2phi2(spde, theta),
  ...
)
}
\arguments{
\item{\dots}{Additional parameters passed on to other methods.}

\item{spde}{An \code{inla.spde} object.}

\item{theta}{The parameter vector.}

\item{phi0}{Internal parameter for a generic model.  Expert option only.}

\item{phi1}{Internal parameter for a generic model.  Expert option only.}

\item{phi2}{Internal parameter for a generic model.  Expert option only.}
}
\value{
A sparse precision matrix.
}
\description{
Precision matrices for SPDE models

Calculates the precision matrix for given parameter values based on an
\code{inla.spde} model object.
}
\seealso{
\code{\link[=inla.spde.models]{inla.spde.models()}}, \code{\link[=inla.spde2.generic]{inla.spde2.generic()}},
\code{\link[=inla.spde2.theta2phi0]{inla.spde2.theta2phi0()}}, \code{\link[=inla.spde2.theta2phi1]{inla.spde2.theta2phi1()}},
\code{\link[=inla.spde2.theta2phi2]{inla.spde2.theta2phi2()}}
}
\author{
Finn Lindgren \email{finn.lindgren@gmail.com}
}
